SystemOrganization addCategory: #'Aida-Admin'!
SystemOrganization addCategory: #'Aida-Components'!
SystemOrganization addCategory: #'Aida-Core'!
SystemOrganization addCategory: #'Aida-Elements'!
SystemOrganization addCategory: #'Aida-Examples'!
SystemOrganization addCategory: #'Aida-Imaging'!
SystemOrganization addCategory: #'Aida-Parties'!
SystemOrganization addCategory: #'Aida-Roles'!
SystemOrganization addCategory: #'Aida-Support'!
SystemOrganization addCategory: #'Aida-Templates'!
SystemOrganization addCategory: #'Aida-Tests'!

!String methodsFor: '*Aida' stamp: 'JM 4/22/2007 18:05'!
asCollectionOfHtmlHeadings
	"find all headings H1-5 and return a collection of headings, complete with tags"
	| in tag collection line |
	in := self readStream. collection := OrderedCollection new.
	[in atEnd] whileFalse:
		[in upTo: $<. in atEnd ifFalse: [tag := in upTo: $>].
		(tag size >= 2 and: [tag first asLowercase = $h and: [(tag at: 2) isDigit]] ) ifTrue:
			[tag := String with: $h with: (tag at: 2). "cleaned, without any parms!!"
			line := '<', tag, '>', (in upToAll: '</', tag, '>'), '</', tag, '>'. 
			collection add: line] ].
	^collection

" '<h1>Heading1</h1><h2>Heading2</h2>' asCollectionOfHtmlHeadings"
" '<h1><b>Heading1</b></h1>' asCollectionOfHtmlHeadings"! !

!String methodsFor: '*Aida' stamp: 'JM 4/22/2007 18:05'!
asSloveneWithoutCircumflexes
	"convert Unicode csz string with ^ (sumniki) to plain ascii csz"
	"'c^s^z^C^S^Z^' convertToSloveneChars asSloveneWithoutCircumflexes"
	^self collect: 
		[:ch | ch asInteger > 127 
			ifTrue:
				[ch = AIDASite charc ifTrue: [$c]
					ifFalse: [ ch  = AIDASite charC ifTrue: [$C]
					ifFalse: [ ch  = AIDASite chars ifTrue: [$s]
					ifFalse: [ ch  = AIDASite charS ifTrue: [$S]
					ifFalse: [ ch  = AIDASite charz ifTrue: [$z] 
					ifFalse: [ ch  = AIDASite charZ ifTrue: [$Z]
					ifFalse: [$- ] ]]]]]
				]
			ifFalse: [ch] ]! !

!String methodsFor: '*Aida' stamp: 'janko 3/16/2008 11:41'!
contentsAsMethod
	"return a body of method with a literal array with contents of that string"
	| input output |
	input := self readStream.
	output := WriteStream with: String new.
	output nextPutAll: '^#('.
	[input atEnd] whileFalse: [output nextPutAll: input next asInteger asString; nextPut: $ ].
	output nextPutAll: ')'.
	^output contents

" 'visual.cha'contentsAsMethod "! !

!String methodsFor: '*Aida' stamp: 'JM 4/22/2007 18:06'!
convertToSloveneChars
	"convert csz string with ^ as 'sumnik' to proper unicode string"
	"'Mivs^ek' convertToSloveneChars"
	| in out ch |
	in := self readStream. out := WriteStream on: String new.
	[in atEnd] whileFalse: 
		[ch := in next.
		(in peek = $^ ) ifTrue: [(#($c $C $s $S $z $Z) includes: ch) ifTrue: 
			[in next. 
			ch = $c ifTrue: [ch := AIDASite charc]. ch = $C ifTrue: [ch := AIDASite charC].
			ch = $s ifTrue: [ch := AIDASite chars]. 	ch = $S ifTrue: [ch := AIDASite charS].
			ch = $z ifTrue: [ch := AIDASite charz]. ch = $Z ifTrue: [ch := AIDASite charZ] ] ].
		out nextPut: ch].
	^out contents! !

!String methodsFor: '*Aida' stamp: 'JM 4/22/2007 18:07'!
ensureUnicodeSloveneChars
	"if string has other that unicode slovene chars, try to correct them!!"
	"'Mivek' ensureUnicodeSloveneChars"
	| in out ch uniSet |
	in := self readStream. out := WriteStream on: String new. uniSet := AIDASite charCszSet. 
	[in atEnd] whileFalse: 
		[ch := in next.
		(ch asInteger < 128 or: [uniSet includes: ch])
			ifTrue: [out nextPut: ch]
			ifFalse: [out nextPut: 
				([(AIDASite convert: (String with: ch) fromCodepage: #'iso-8859-2') first]
					on: Error do: [:ex | ch]) ] ].
	^out contents! !

!String methodsFor: '*Aida' stamp: 'JM 4/22/2007 18:07'!
isValidEMailAddress
	(self includes: $@) ifFalse: [^false].
	^(self trimBlanks contains: [:char | 
		char isAlphaNumeric not and: [(#($. $- $_  $@) includes: char) not] ]) not

" 'janko.mivsek@eranova.si' isValidEMailAddress"! !

!String methodsFor: '*Aida' stamp: 'np 3/31/2008 17:25'!
trimBlanks
	^SpStringUtilities trimBlanksFrom: self! !

!String methodsFor: '*Aida' stamp: 'np 1/21/2008 17:38'!
withHeadingAnchors
	"add anchors before all H1-5 tags, each with sequence number of that tag, eg: <a name=h-15>"
	| in out tag number |
	in := self readStream. number := 1.
	out := WriteStream on: String new.
	[in atEnd] whileFalse:
		[out nextPutAll: (in upTo: $<).
		in atEnd ifFalse: [tag := in upTo: $>] ifTrue: [^out contents].
		(tag size = 2 and: [tag first asLowercase = $h and: [tag last isDigit]] ) ifTrue:
			[out nextPutAll: '<a name="h-', number printString, '"></a>'. 
			number := number +1].
		out nextPutAll: '<', tag, '>'].
	^out contents

"'<h1>Heading1</h1><h2>Heading2</h2>' withHeadingAnchors"! !

!String methodsFor: '*Aida' stamp: 'JM 4/22/2007 18:08'!
withHtmlBreaks
	"changes all cr with <br>"
	^self copyReplaceAll: (String with: Character cr) with: '<br>'

"('aaa', (String with: Character cr), 'bbb') withHtmlBreaks"! !

!String methodsFor: '*Aida' stamp: 'JM 4/22/2007 18:08'!
withoutHtmlTags
	"strip all tags, replace <br> and <p> with cr"
	| in out tag |
	in := self readStream.
	out := WriteStream on: String new.
	[in atEnd] whileFalse:
		[out nextPutAll: (in upTo: $<).
		in atEnd ifFalse: [tag := in upTo: $>].
		(tag = 'br') | (tag = 'p') ifTrue: [out nextPut: Character cr ] ].
	^out contents

"'<span>by Michael A. <br>Prospero</span>' withoutHtmlTags"! !

!Time methodsFor: '*Aida' stamp: 'np 1/22/2008 08:47'!
printSloString
	^"(self hours < 10 ifTrue: ['0'] ifFalse: [''])," self hours printString, ':',
		(self minutes < 10 ifTrue: ['0'] ifFalse: ['']), self minutes printString

"Time now printSloString"! !

!SpFilename methodsFor: '*Aida' stamp: 'np 3/18/2008 05:58'!
contentsAsMethod
	"return a body of method with a literal array with contents of that file"
	| input output |
	input := self readStream.
	input binary.
	output := WriteStream with: String new.
	output nextPutAll: '^#('.
	[ [input atEnd] whileFalse: [output nextPutAll: input next printString; nextPut: $ ].
	output nextPutAll: ')' ]
		ensure: [input close].
	^output contents

"(SpFilename named: 'source.st') contentsAsMethod "! !

!SpFilename methodsFor: '*Aida' stamp: 'janko 3/22/2008 17:17'!
contentsOfEntireFileBinary
	| stream |
	[stream := FileStream oldFileNamed: self filename.
	^stream binary contentsOfEntireFile]
		ensure: [stream close]
! !

!SpFilename methodsFor: '*Aida' stamp: 'np 6/11/2008 05:02'!
directoryContents
	| entries |
	entries := FileDirectory new directoryContentsFor: (self asAbsoluteFilename filename).
	^entries collect: [:each | each name]! !

TestCase subclass: #PartiesTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Tests'!

TestCase subclass: #SchedulerTest
	instanceVariableNames: 'scheduler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Tests'!

TestCase subclass: #SwazooIntegrationTest
	instanceVariableNames: 'server site'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Tests'!

!SwazooIntegrationTest commentStamp: '<historical>' prior: 0!
All tests are commented out because they are DANGEROUS to live Swazoo systems!!

Instance Variables:
	server	<SwazooServer>	description of server
	site	<Object | Proxy>	description of site

!

TestCase subclass: #URLResolverTest
	instanceVariableNames: 'server site'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Tests'!

TestCase subclass: #VersionSpecTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Tests'!

TestCase subclass: #VersionedObjectTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Tests'!

TestCase subclass: #WebApplicationTest
	instanceVariableNames: 'server site session app request'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Tests'!

TestCase subclass: #WebFileInputTest
	instanceVariableNames: 'file filename'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Tests'!

TestCase subclass: #WebSecurityTest
	instanceVariableNames: 'server site mgr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Tests'!

TestCase subclass: #WebSessionMgmtTest
	instanceVariableNames: 'server site session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Tests'!

Object subclass: #AIDAProtocolAdaptor
	instanceVariableNames: 'subject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Support'!

AIDAProtocolAdaptor subclass: #AIDAAspectAdaptor
	instanceVariableNames: 'aspect'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Support'!

!AIDAAspectAdaptor class methodsFor: 'instance creation' stamp: ' 25/4/07 23:28'!
forAspect: aSymbol

	^super new aspect: aSymbol! !

!AIDAAspectAdaptor methodsFor: 'accessing' stamp: ' 25/4/07 23:28'!
aspect

	^aspect! !

!AIDAAspectAdaptor methodsFor: 'accessing' stamp: ' 25/4/07 23:28'!
aspect: aSymbol

	aspect := aSymbol! !

!AIDAAspectAdaptor methodsFor: 'accessing' stamp: ' 25/4/07 23:28'!
value

	^self subject perform: self aspect! !

!AIDAAspectAdaptor methodsFor: 'accessing' stamp: ' 25/4/07 23:28'!
value: aValue

	self subject perform: (self aspect asString, ':') asSymbol with: aValue! !

AIDAProtocolAdaptor subclass: #AIDAIndexedAdaptor
	instanceVariableNames: 'index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Support'!

!AIDAIndexedAdaptor class methodsFor: 'instance creation' stamp: ' 25/4/07 23:28'!
forIndex: aNumber

	^super new index: aNumber! !

!AIDAIndexedAdaptor methodsFor: 'accessing' stamp: ' 25/4/07 23:28'!
index

	^index! !

!AIDAIndexedAdaptor methodsFor: 'accessing' stamp: ' 25/4/07 23:28'!
index: aNumber

	index := aNumber! !

!AIDAIndexedAdaptor methodsFor: 'accessing' stamp: ' 26/4/07 10:37'!
value

	^self subject at: self index! !

!AIDAIndexedAdaptor methodsFor: 'accessing' stamp: ' 25/4/07 23:28'!
value: aValue

	self subject at: self index put: aValue! !

!AIDAProtocolAdaptor methodsFor: 'accessing' stamp: ' 25/4/07 23:28'!
subject

	^subject! !

!AIDAProtocolAdaptor methodsFor: 'accessing' stamp: ' 25/4/07 23:28'!
subject: anObject

	subject := anObject! !

!AIDAProtocolAdaptor methodsFor: 'accessing' stamp: ' 25/4/07 23:28'!
value
! !

!AIDAProtocolAdaptor methodsFor: 'accessing' stamp: ' 25/4/07 23:28'!
value: aValue! !

Object subclass: #AIDARoot
	instanceVariableNames: ''
	classVariableNames: 'Root Lock'
	poolDictionaries: ''
	category: 'Aida-Support'!

!AIDARoot class methodsFor: 'critical sections' stamp: 'mivsek 6/13/2007 17:11'!
critical: aBlock
	"For protecting critical sections in parallel execution of web requests. Use it always
	when you do things, which cannot be disturbed by another request. Example:
		WebTransactionMonitor critical: [<a block with critical section>]. "

	^self localCritical: aBlock
! !

!AIDARoot class methodsFor: 'private' stamp: 'mu 8/20/2008 21:53'!
initLock
	Lock := Monitor new! !

!AIDARoot class methodsFor: 'private' stamp: 'mivsek 6/13/2007 17:10'!
initRoot
	Root := self newRootObject.! !

!AIDARoot class methodsFor: 'critical sections' stamp: ' 21/4/07 22:07'!
localCritical: aBlock
	"For protecting critical sections in parallel execution of web requests. Use it always
	when you do things, which cannot be disturbed by another request. Example:
		WebTransactionMonitor critical: [<a block with critical section>]. "

	^self lock critical: aBlock! !

!AIDARoot class methodsFor: 'private' stamp: ' 21/4/07 22:07'!
lock
	Lock isNil ifTrue: [self initLock].
	^Lock! !

!AIDARoot class methodsFor: 'private' stamp: ' 21/4/07 22:07'!
newRoot: anObject named: aSymbol

	| createAnyway |
	(self root includesKey: aSymbol) ifTrue: 
		[createAnyway := Dialog confirm: 'Root named ', 
			aSymbol asString, ' already exist!! Do you realy want to replace it?'.
		createAnyway ifFalse: [^nil] ].
	self root at: aSymbol put: anObject.
	self inGemstone ifTrue: [GBSM currentSession commitTransaction]! !

!AIDARoot class methodsFor: 'private' stamp: ' 21/4/07 22:07'!
newRootObject
	^Dictionary new! !

!AIDARoot class methodsFor: 'private' stamp: ' 21/4/07 22:07'!
removeRootNamed: aSymbol
	| realyRemove |
	(self root includesKey: aSymbol) ifTrue: 
		[realyRemove := Dialog confirm: 'Do you realy want to remove a root named ', 
			aSymbol asString, ' ?'.
		realyRemove ifFalse: [^nil] ].
	self root removeKey: aSymbol ifAbsent: [].! !

!AIDARoot class methodsFor: 'private' stamp: ' 21/4/07 22:07'!
root
	Root isNil ifTrue: [self initRoot].
	^Root! !

Object subclass: #Address
	instanceVariableNames: 'title firstName middleName lastName company street city stateProvince postalCode country phones fax emails'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Parties'!

!Address commentStamp: '<historical>' prior: 0!
"ECML v1.1 definition of address (www.ecml.org)"

Instance Variables:
	city	<Object>	description of city
	company	<Object>	description of company
	country	<Object>	description of country
	emails	<Object>	description of emails
	firstName	<Object>	description of firstName
	lastName	<Object>	description of lastName
	middleName	<Object>	description of middleName
	phones	<Object>	description of phones
	postalCode	<Object>	description of postalCode
	stateProvince	<Object>	description of stateProvince
	street	<Object>	description of street
	title	<Object>	description of title

!

!Address class methodsFor: 'odb specific' stamp: ' 10/1/08 16:40'!
replicationSpec
	"Gemstone"

	^super replicationSpec, 
		#(	(firstName replicate)
			(lastName replicate)
			(company replicate)
			(street replicate)
			(city replicate)
			(postalCode replicate)
			(country replicate)
			(phones max 4)
			(emails max 4)
		 )! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
city
	city isNil ifTrue: [^''].
	^city! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
city: aString
	city := aString.! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
company
	company isNil ifTrue: [^''].
	^company! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
company: aString
	company := aString.! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
country
	country isNil ifTrue: [^''].
	^country! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
country: aString
	country := aString.! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
email
	self emails size > 0 ifTrue: [^self emails at: 1].
	^' '! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
email: aString 
	self emails size > 0 ifTrue: [^self emails at: 1 put: aString].
	^self emails add: aString! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
emails
	emails isNil ifTrue: [self initEMails].
	^emails! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
fax
	fax isNil ifTrue: [^''].
	^fax! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
fax: aString
	fax := aString.! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
firstName
	firstName isNil ifTrue: [^''].
	^firstName! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
firstName: aString
	firstName := aString.! !

!Address methodsFor: 'initialize - release' stamp: ' 10/1/08 16:40'!
initEMails
	emails := OrderedCollection new.! !

!Address methodsFor: 'initialize - release' stamp: ' 10/1/08 16:40'!
initPhones
	phones := OrderedCollection new.
	phones add: ''.! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
lastName
	lastName isNil ifTrue: [^''].
	^lastName! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
lastName: aString
	lastName := aString.! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
middleName
	middleName isNil ifTrue: [^''].
	^middleName! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
middleName: aString
	middleName := aString.! !

!Address methodsFor: 'private' stamp: ' 10/1/08 16:40'!
migrateToUnicode
	"from iso8859-2"
	"Address allInstances do: [:each | each migrateToUnicode]"
	title notNil ifTrue: [title := title ensureUnicodeSloveneChars].
	firstName notNil ifTrue: [firstName := firstName ensureUnicodeSloveneChars].
	middleName notNil ifTrue: [middleName := middleName ensureUnicodeSloveneChars].
	lastName notNil ifTrue: [lastName := lastName ensureUnicodeSloveneChars].
	company notNil ifTrue: [company := company ensureUnicodeSloveneChars].
	street notNil ifTrue: [street := street ensureUnicodeSloveneChars].
	city notNil ifTrue: [city := city ensureUnicodeSloveneChars].
	country notNil ifTrue: [country := country ensureUnicodeSloveneChars].! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
phone
	self phones size > 0 ifTrue: [^self phones first].
	^' '! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
phone: aString
	self phones size > 0 ifTrue: [^self phones at: 1 put: aString].
	^self phones add: aString! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
phones
	phones isNil ifTrue: [self initPhones].
	^phones! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
post
	^self city! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
post: anObject
	self city: anObject! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
postalCode
	postalCode isNil ifTrue: [^''].
	^postalCode! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
postalCode: aString
	postalCode := aString.! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
stateProvince
	stateProvince isNil ifTrue: [^''].
	^stateProvince! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
stateProvince: aString
	stateProvince := aString.! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
street
	street isNil ifTrue: [^''].
	^street! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
street: aString
	street := aString.! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
title
	title isNil ifTrue: [^''].
	^title! !

!Address methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
title: aString
	title := aString.! !

Object subclass: #DailyCollection
	instanceVariableNames: 'days'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Support'!

!DailyCollection methodsFor: 'adding - removing' stamp: ' 21/4/07 22:07'!
add: newObject

	self add: newObject onDate: newObject date! !

!DailyCollection methodsFor: 'adding - removing' stamp: ' 21/4/07 22:07'!
add: newObject onDate: aDate
	(self includes: newObject onDate: aDate) ifTrue: [^nil].
	self existCheckDate: aDate.
	(self days at: aDate asDays) add: newObject.
	^newObject! !

!DailyCollection methodsFor: 'adding - removing' stamp: ' 21/4/07 22:07'!
addAll: aCollection 

	aCollection do: [:each | self add: each].
	^aCollection! !

!DailyCollection methodsFor: 'adding - removing' stamp: ' 21/4/07 22:07'!
addAll: aCollection onDate: aDate

	self existCheckDate: aDate.
	^aCollection do: [:each | 	self add: each onDate: aDate].! !

!DailyCollection methodsFor: 'adding - removing' stamp: 'JM 4/26/2007 21:29'!
addFirst: newObject inYear: aYear
	"put in first place at 1st jan of this year"
	| date |
	date := SpDate newDay: 1 month: 1 year: aYear.
	(self includes: newObject onDate: date) ifTrue: [^nil].
	self existCheckDate: date.
	(self days at: date asDays) addFirst: newObject.
	^newObject! !

!DailyCollection methodsFor: 'adding - removing' stamp: 'JM 4/26/2007 21:29'!
addLast: newObject inYear: aYear
	"put in first place at 1st jan of this year"
	| date |
	date := SpDate newDay: 31 month: 12 year: aYear.
	(self includes: newObject onDate: date) ifTrue: [^nil].
	self existCheckDate: date.
	(self days at: date asDays) addLast: newObject.
	^newObject! !

!DailyCollection methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
all
	| dayCol |
	dayCol := SortedCollection
		withAll: self days keys
		sortBlock: [:a :b | a > b].
	^dayCol inject: OrderedCollection new into: [:col :each | col addAll: (self days at: each); yourself ].! !

!DailyCollection methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allDated: aDate

	^(self days at: aDate asDays ifAbsent: [^#()] ) copy! !

!DailyCollection methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allDates
	"all dates on which something exist"
	^self allDays collect: [:each | Date fromDays: each]! !

!DailyCollection methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
allDays
	"all day numbers on which something exist"
	^self days keys! !

!DailyCollection methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
allDaysForKindOf: aClass
	"all day numbers on which something of that class exist"
	^self days keys select: [:day | 
		(self days at: day) contains: [:each | each isKindOf: aClass] ].! !

!DailyCollection methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allFromDate: aStartDate to: anEndDate
	| startDay endDay daysInRange |
	startDay := aStartDate asDays. endDay := anEndDate asDays.
	daysInRange := self allDays select: [:each | each >= startDay and: [each <= endDay] ].
	daysInRange := daysInRange asSortedCollection. 
	^daysInRange inject: OrderedCollection new into: [:col :each | col addAll: (self days at: each). col]

"DailyCollection allInstances last allFromDate: (Date readSloFrom: '1.1.2004' readStream) 
  	to: (Date readSloFrom: '31.12.2006' readStream)"! !

!DailyCollection methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allLastMonth
	^self allMonthly: Date today - Date today day! !

!DailyCollection methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allLastWeek
	^self allWeekly: Date today - 7! !

!DailyCollection methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allLastYear
	^self allYear: Date today year - 1! !

!DailyCollection methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allMonthly: aDate
	^self 
		allFromDate: aDate firstDayOfMonth
		to: aDate firstDayInMonth + aDate daysInMonth - 1! !

!DailyCollection methodsFor: 'accessing' stamp: 'np 3/29/2008 05:18'!
allSinceDate: aDate
	^self allFromDate: aDate to: SpDate today! !

!DailyCollection methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allThisMonth
	^self allMonthly: Date today! !

!DailyCollection methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allThisWeek
	^self allWeekly: Date today! !

!DailyCollection methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allThisYear
	^self allYear: Date today year! !

!DailyCollection methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allToday
	^self allDated: Date today! !

!DailyCollection methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allWeekly: aDate
	^self 
		allFromDate: aDate - aDate weekdayIndex + 1
		to: aDate - aDate weekdayIndex + 1 + 7! !

!DailyCollection methodsFor: 'accessing' stamp: 'JM 4/26/2007 21:30'!
allYear: aNumber
	^self 
		allFromDate: (SpDate newDay: 1 month: 1 year: aNumber)
		to: (SpDate newDay: 31 month: 12 year: aNumber)! !

!DailyCollection methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allYears
	"all years on which something exist"
	^SortedCollection 
		withAll: (self allDates inject: Set new into: [:set :each | set add: each year. set])
		sortBlock: [:a :b | a < b]

"DailyCollection allInstances last allYears"! !

!DailyCollection methodsFor: 'private-accessing' stamp: 'JM 4/26/2007 21:30'!
allYearsForKindOf: aClass
	"all years on which something of that class exist"
	| allDays currentYear years skipToDay |
	allDays := self days keys asSortedCollection. allDays isEmpty ifTrue: [^#()].
	currentYear := (SpDate fromDays: allDays first) year.
	years := Set new. skipToDay := 0.
	allDays do: [:day |
		day >= skipToDay ifTrue:
			[((self days at: day) contains: [:entry | entry isKindOf: aClass]) ifTrue:
				[years add: currentYear. currentYear := currentYear + 1.
				skipToDay := (SpDate newDay: 1 month: 1 year: currentYear) asDays] ] ].
	^years asSortedCollection

"DailyCollection allInstances last allYearsForKindOf: AIDA.Invoice "! !

!DailyCollection methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allYesterday
	^self allDated: Date today - 1! !

!DailyCollection methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
contains: aBlock 
	"Evaluate aBlock with each of the receiver's elements as the argument. 
	Answer true if aBlock ever evaluates to true, otherwise answer false."

	self detect: aBlock ifNone: [^false].
	^true! !

!DailyCollection methodsFor: 'private' stamp: ' 21/4/07 22:07'!
days
	days isNil ifTrue: [self initDays].
	^days! !

!DailyCollection methodsFor: 'private' stamp: ' 21/4/07 22:07'!
existCheckDate: aDate

	(self days includesKey: aDate asDays) ifFalse:
		[self days at: aDate asDays put: OrderedCollection new].! !

!DailyCollection methodsFor: 'accessing' stamp: 'JM 4/26/2007 21:30'!
firstInYear: aYear
	"on 1st january of that year"
	| col |
	col := self allDated: (SpDate newDay: 1 month: 1 year: aYear).
	col isEmpty ifTrue: [^nil].
	^col first! !

!DailyCollection methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
includes: anObject
	"Answer whether anObject is one of the receiver's elements."

	self days values do: [:each | (each includes: anObject) ifTrue: [^true]].
	^false! !

!DailyCollection methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
includes: anObject onDate: aDate
	| values |
	values := self days at: aDate asDays ifAbsent: [^false].
	^values includes: anObject! !

!DailyCollection methodsFor: 'private' stamp: ' 21/4/07 22:07'!
initDays
	days := Dictionary new.! !

!DailyCollection methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isEmpty
	"Answer whether the receiver contains any elements."

	^self size = 0! !

!DailyCollection methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
last: aNumber
	"find last number of values, starting from today and back in past. Most recent first!!"
	| collection dayColl |
	collection := OrderedCollection new.
	dayColl := SortedCollection withAll: self days keys sortBlock: [:a :b | a > b].
	dayColl do: [:day || coll |
		coll := (self days at: day) copy reverse.
		collection addAll: (coll copyFrom: 1 to: ((aNumber - collection size) min: coll size) ).
		collection size = aNumber ifTrue: [^collection] ].
	^collection

"LogisticSystem default owner events size last: 1000"! !

!DailyCollection methodsFor: 'accessing' stamp: 'JM 4/26/2007 21:31'!
lastDayInYear: aYear
	"on 31 december of that year"
	^self allDated: (SpDate newDay: 31 month: 12 year: aYear).! !

!DailyCollection methodsFor: 'adding - removing' stamp: ' 21/4/07 22:07'!
move: anObject fromDate: aDate toDate: aNewDate
	| collection |
	collection := self days at: aDate asDays.
	collection remove: anObject.
	self add: anObject onDate: aNewDate! !

!DailyCollection methodsFor: 'adding - removing' stamp: ' 21/4/07 22:07'!
remove: oldObject 
	^self remove: oldObject ifAbsent: [self notFoundError]! !

!DailyCollection methodsFor: 'adding - removing' stamp: ' 21/4/07 22:07'!
remove: oldObject ifAbsent: anExceptionBlock
	| collection |
	collection := self days at: oldObject date asDays ifAbsent: [^anExceptionBlock value].
	collection remove: oldObject ifAbsent: [^anExceptionBlock value].! !

!DailyCollection methodsFor: 'adding - removing' stamp: ' 21/4/07 22:07'!
removeAll: aCollection 
	"Remove each element of aCollection from the receiver.  If successful for each,
	answer aCollection."
	aCollection do: [:each | self remove: each].
	^aCollection! !

!DailyCollection methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
size
	days isNil ifTrue: [^0].
	^self days values inject: 0 into: [:sum :each | sum + each size].! !

Object subclass: #DailyValues
	instanceVariableNames: 'days'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Support'!

!DailyValues methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allDates
	"all dates on which something exist"
	^self days keys collect: [:each | Date fromDays: each]! !

!DailyValues methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allFromDate: aStartDate to: anEndDate

	| collection |
	collection := OrderedCollection new: 100.
	aStartDate to: anEndDate do: [:date |
		collection add: (self at: date) ].
	^collection! !

!DailyValues methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allLastMonth
	^self allMonthly: Date today - Date today day! !

!DailyValues methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allLastWeek
	^self allWeekly: Date today - 7! !

!DailyValues methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allMonthly: aDate
	^self 
		allFromDate: aDate firstDayOfMonth
		to: aDate firstDayInMonth + aDate daysInMonth - 1! !

!DailyValues methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allThisMonth
	^self allMonthly: Date today! !

!DailyValues methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allThisWeek
	^self allWeekly: Date today! !

!DailyValues methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allWeekly: aDate
	^self 
		allFromDate: aDate - aDate weekdayIndex + 1
		to: aDate - aDate weekdayIndex + 1 + 7! !

!DailyValues methodsFor: 'adding - removing' stamp: ' 21/8/07 09:38'!
at: aDate
	^self at: aDate ifAbsent: [self defaultValue]! !

!DailyValues methodsFor: 'adding - removing' stamp: ' 21/4/07 22:07'!
at: aDate add: aValue 
	self existCheckDate: aDate.
	^self days at: aDate asDays put: (self at: aDate) + aValue! !

!DailyValues methodsFor: 'adding - removing' stamp: ' 21/8/07 09:38'!
at: aDate ifAbsent: aBlock
	^self days at: aDate asDays ifAbsent: [aBlock value]! !

!DailyValues methodsFor: 'adding - removing' stamp: ' 21/4/07 22:07'!
at: aDate put: aValue
	self existCheckDate: aDate.
	^self days at: aDate asDays put: aValue! !

!DailyValues methodsFor: 'adding - removing' stamp: ' 21/4/07 22:07'!
at: aDate subtract: aValue
	self existCheckDate: aDate.
	^self days at: aDate asDays put: (self at: aDate) - aValue! !

!DailyValues methodsFor: 'adding - removing' stamp: ' 21/8/07 09:38'!
atOrLastFrom: aDate
	"return value on that date or last date when some value exist"
	^self at: aDate ifAbsent: [self lastFrom: aDate]! !

!DailyValues methodsFor: 'private' stamp: ' 21/4/07 22:07'!
days
	days isNil ifTrue: [self initDays].
	^days! !

!DailyValues methodsFor: 'private' stamp: ' 21/4/07 22:07'!
defaultValue
	^0! !

!DailyValues methodsFor: 'private' stamp: ' 21/4/07 22:07'!
existCheckDate: aDate

	(self days includesKey: aDate asDays) ifFalse:
		[self days at: aDate asDays put: self defaultValue].! !

!DailyValues methodsFor: 'private' stamp: ' 21/4/07 22:07'!
initDays
	days := IdentityDictionary new.! !

!DailyValues methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isEmpty
	"Answer whether the receiver contains any elements."

	^self size = 0! !

!DailyValues methodsFor: 'accessing' stamp: 'mivsek 1/10/2008 18:28'!
last
	"return value on last (most future) date entered"
	| coll |
	coll := self days keys asSortedCollection.  "not very optimized way for big colections!! "
	coll isNil ifTrue: [^nil].
	^self at: (Date fromDays: coll last)! !

!DailyValues methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
last: aNumber
	"find last number of values, starting from today and back in past. Most recent first!!"
	| collection |
	collection := OrderedCollection new.
	1 to: aNumber do: [:inx | collection add: (self at: Date today - inx + 1)].
	^collection! !

!DailyValues methodsFor: 'private' stamp: ' 21/8/07 09:38'!
lastDateFrom: aDate
	"search back in time for some date on which a value exist and return that date"
	| start |
	start := aDate asDays.
	self days keys asSortedCollection reverse do:  "not very optimized way for big colections!! "
		[:day | day <= start ifTrue: [^Date fromDays: day] ].
	^nil! !

!DailyValues methodsFor: 'private' stamp: ' 21/8/07 09:38'!
lastFrom: aDate
	"search back in time for some date on which a value exist and return it"
	| start |
	start := aDate asDays.
	self days keys asSortedCollection reverse do:  "not very optimized way for big colections!! "
		[:day | day <= start ifTrue: [^self days at: day] ].
	^nil! !

!DailyValues methodsFor: 'adding - removing' stamp: ' 21/4/07 22:07'!
resetAt: aDate
	"put default value at this date"
	^self at: aDate put: self defaultValue! !

!DailyValues methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
size
	^self days values size! !

!DailyValues methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
sumFromDate: aStartDate to: anEndDate

	^(self allFromDate: aStartDate to: anEndDate)
		inject: 0
		into: [:sum :each | sum + each]! !

!DailyValues methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
sumLast: aNumber
	"sum values of aNumber of days, starting from today and back"

	^(self last: aNumber)
		inject: 0
		into: [:sum :each | sum + each]! !

!DailyValues methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
sumMonthly: aDate

	^(self allMonthly: aDate)
		inject: 0
		into: [:sum :each | sum + each]! !

!DailyValues methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
sumWeekly: aDate

	^(self allWeekly: aDate)
		inject: 0
		into: [:sum :each | sum + each]! !

Object subclass: #DelimitedFile
	instanceVariableNames: 'filename delimiter codepage records'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Support'!

!DelimitedFile class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
fileToWrite: aFilenameString

	"create a new instance on a specified (empty) file to write. After you fill records attribute with all
	data, call writeToFile method to do actual exporting to this file"

	| instance |
	instance := self new.
	instance filename: aFilenameString asFilename.
	^instance.! !

!DelimitedFile class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new initialize! !

!DelimitedFile class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
readFrom: aFilenameString

	"read a specified delimited file and store records in a records attribute. Use default delimiter
	for field separator (character ; )"

	| instance |
	instance := self new.
	instance readFrom: aFilenameString.
	^instance.

"DelimitedFile readFrom: 'i:\projekti\dare\proizvod.txt' "! !

!DelimitedFile methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
codepage
	codepage isNil ifTrue: [self codepage: self defaultCodepage].
	^codepage! !

!DelimitedFile methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
codepage: aSymbol
	" #cp1250  #iso2 #'7bit'"
	codepage := aSymbol.! !

!DelimitedFile methodsFor: 'private' stamp: ' 21/4/07 22:07'!
crlf
	"return an cr-lf combination"

	^(Character cr) asSymbol ", (Character lf) asSymbol".! !

!DelimitedFile methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
defaultCodepage
	^#cp1250 "Windows"! !

!DelimitedFile methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
defaultDelimiter
	"default delimiter is ; "

	^$;! !

!DelimitedFile methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
delimiter
	"delimiter is a character, which separate fields in a delimited file"
	delimiter isNil ifTrue: [self delimiter: self defaultDelimiter].
	^delimiter! !

!DelimitedFile methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
delimiter: aCharacter
	"delimiter is a character, which separate fields in a delimited file"
	delimiter := aCharacter.! !

!DelimitedFile methodsFor: 'private' stamp: ' 21/4/07 22:07'!
endOfLine: aStream
	"return true if we are at the end of line (on cr or lf characters)"
	aStream peek isNil ifTrue: [^true].
	^(aStream peek = Character cr or: [aStream peek = Character lf]).! !

!DelimitedFile methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
filename
	^filename! !

!DelimitedFile methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
filename: aFilename
	filename := aFilename.! !

!DelimitedFile methodsFor: 'initialize - release' stamp: ' 21/4/07 22:07'!
initialize! !

!DelimitedFile methodsFor: 'private' stamp: ' 21/4/07 22:07'!
newRecordFrom: aStream

	"read one line from delimited file, detect and separate fields and convert to
	appropriate objects into an ordered collection. return that collection"

	| collection field |
	collection := OrderedCollection new.
	[self endOfLine: aStream] whileFalse:
		[field := self readFieldFrom: aStream.
		collection add: field].
	self skipEndOfLine: aStream.
	^collection! !

!DelimitedFile methodsFor: 'private' stamp: 'mivsek 6/13/2007 16:03'!
readFieldFrom: aStream
	"read one field from delimited file, and convert to appropriate object. Return that object"
	| object stream |
	aStream peek = $" ifTrue:  "start of a text field"
		[object := aStream upTo: $"; upTo: $".
		(self endOfLine: aStream) ifFalse: [aStream upTo: self delimiter].
		^AIDASite convert: object fromCodepage: self codepage].
	stream := WriteStream on: String new.
	[aStream peek = self delimiter or: [self endOfLine: aStream]] 
		whileFalse: [aStream peek isNil ifTrue: [^stream contents]. stream nextPut: aStream next].
	aStream peek = self delimiter ifTrue: [aStream skip: 1].
	object := stream contents. stream close.
	^object
"
	(object includes: $. ) false
		ifTrue:
			[(Date readSloFrom: object readStream) notNil 
				ifTrue: [^Date readSloFrom: object readStream]
				ifFalse: [^object asInteger]]
		ifFalse: [^object asInteger].
"! !

!DelimitedFile methodsFor: 'file reading' stamp: ' 21/4/07 22:07'!
readFrom: aFilenameString
	"read a delimited file and convert content to object representation in 'records' collection"
	| stream | 
	self filename: aFilenameString asFilename.
	[stream :=  self filename readStream.
	self readFromStream: stream] 
		ensure: [stream close].

"DelimitedFile readFrom: 'm:\geoplin\tarifni\exports\ddnevnik.txt' "! !

!DelimitedFile methodsFor: 'file reading' stamp: ' 21/4/07 22:07'!
readFromStream: aStream
	"read a delimited file and convert content to object representation in 'records' collection"
	[aStream atEnd] whileFalse: [self records add: (self newRecordFrom: aStream)]! !

!DelimitedFile methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
records
	"when aDelimitedFile is created on a existing file, all records (lines in a file) are 
	converted and stored in this ordered collection. Fields in each record are stored in 
	another ordered collection as objects of appropriate class (texts as Strings, 
	numbers as Integers or Floats, dates as Dates)"

	records isNil ifTrue: [self records:OrderedCollection new].
	^records! !

!DelimitedFile methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
records: aCollection
	records := aCollection.! !

!DelimitedFile methodsFor: 'private' stamp: ' 21/4/07 22:07'!
skipEndOfLine: aStream

	"skip all cr and lf characters)"

	[aStream atEnd] whileFalse: 
		[(aStream peek = Character cr or: [aStream peek = Character lf]) 
			ifTrue:	[aStream skip: 1]
			ifFalse: [^self]
		].! !

!DelimitedFile methodsFor: 'file writing' stamp: ' 21/4/07 22:07'!
writeRecord: anArray to: aStream

	| dm data1 |
	dm := false.
	anArray do: [:data |
		dm ifTrue: [aStream nextPut: self delimiter].
		(data isKindOf: Integer) ifTrue: [aStream nextPutAll: data printString].
		(data isKindOf: String) ifTrue: 
			[data1 := AIDASite returnCSZFromCP852String:  data.
			data1 := data1 select: [:ch | ch asInteger <=255].
			aStream nextPutAll: '"',data1,'"'].
		(data isKindOf: FixedPoint) ifTrue: 
			[data1 := data printDotString select: [:ch | ch ~= $. ].
			aStream nextPutAll: data1].
		dm := true].
	aStream nextPutAll: self crlf.! !

!DelimitedFile methodsFor: 'file writing' stamp: ' 21/4/07 22:07'!
writeToFile
	self writeToFile: self filename.
"
|dmFile|
dmFile := DelimitedFile fileToWrite: 'test.dat'.
dmFile records 
	add: (Array with: 123 with: 'abcd' with: 345);
	add: (Array with: 234 with: 'CDE' with: 567).
dmFile writeToFile
"! !

!DelimitedFile methodsFor: 'file writing' stamp: ' 21/4/07 22:07'!
writeToFile: aFilenameString
	"export all records to delimited file"
	| stream | 
	self filename: aFilenameString asFilename.
	[stream :=  self filename writeStream.
	records do: [:record | self writeRecord: record to: stream] ]
		ensure: [stream close].
"
|dmFile|
dmFile := DelimitedFile new.
dmFile records 
	add: (Array with: 123 with: 'abcd' with: 345);
	add: (Array with: 234 with: 'CDE' with: 567).
dmFile writeToFile: 'test.dat'
"! !

Object subclass: #DirectoryProxy
	instanceVariableNames: 'server filename fileDates origContent origCodepage contentType elements bodyTagIndex imgTagIndexes linkTagIndexes servletTagIndexes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Core'!

Object subclass: #DocLink
	instanceVariableNames: 'title url object'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Support'!

!DocLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	"Answer a newly created and initialized instance."

	^super new initialize! !

!DocLink methodsFor: 'converting' stamp: ' 21/4/07 22:07'!
asWebLink
	^WebLink text: self title linkTo: self url! !

!DocLink methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	^self! !

!DocLink methodsFor: 'private' stamp: ' 21/4/07 22:07'!
migrateToUnicode
	"DocLink allInstances do: [:each | each migrateToUnicode]"
	title notNil ifTrue: [title := title ensureUnicodeSloveneChars]! !

!DocLink methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
object
	^object! !

!DocLink methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
object: anObject
	object := anObject! !

!DocLink methodsFor: 'private' stamp: ' 21/4/07 22:07'!
printString
	^'a Doclink
		title:  ', (self title notNil ifTrue: [self title] ifFalse: ['']), '
		url: ', (self url notNil ifTrue: [self url] ifFalse: ['']), '
		object: ', self object printString! !

!DocLink methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
title
	^title! !

!DocLink methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
title: aString
	title := aString! !

!DocLink methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
url
	^url! !

!DocLink methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
url: aString
	url := aString! !

!DocLink methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
uuid
	^self hash printString! !

Object subclass: #FileProxy
	instanceVariableNames: 'site filename timestamps content contentType size codepage elements bodyTagIndex imgTagIndexes linkTagIndexes servletTagIndexes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Core'!

!FileProxy class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
from: aFilenameString onSite: aSite
	"read a file and make a HTML proxy of it. Return nil, if file does not exist or cannot be opened. 
	Also record a server for which you make a proxy"
	| proxy |
	proxy := self new site: aSite; from: aFilenameString.
	proxy indexContent.
	^proxy! !

!FileProxy methodsFor: 'parsing' stamp: ' 21/4/07 22:07'!
absoluteTagFrom: aTagString urlStart: anURLString

	""

	| tag url |
	tag := (aTagString copyReplaceAll: '\' with: '/').
	((tag copyFrom: 1 to: 3) = '../')
		ifFalse:  "not relative to parent directory"
			[^((tag copyUpTo: $"), '"', anURLString, 
				(tag copyFrom: ((tag indexOf: $") + 1) to: tag size))]
		ifTrue:
			[tag := tag readStream upTo: $/; upToEnd.
			url := anURLString copyFrom: 1 to: (anURLString lastIndexOf: $/).
			^self absoluteTagFrom: tag urlStart: url].! !

!FileProxy methodsFor: 'accessing-timestamps' stamp: 'mivsek 8/22/2007 16:02'!
accessedTimestamp
	^self timestamps at: #accessed ifAbsent: [nil]! !

!FileProxy methodsFor: 'printing-servlets' stamp: ' 21/4/07 22:07'!
argumentsFrom: aMessageString

	^Array new: 0 "for now!! "! !

!FileProxy methodsFor: 'accessing-tags' stamp: ' 21/4/07 22:07'!
bodyTagIndex
	"index of element, which represent body tag in original html file"
	^bodyTagIndex! !

!FileProxy methodsFor: 'accessing-tags' stamp: ' 21/4/07 22:07'!
bodyTagIndex: anInteger
	"set the index of element, which represent body tag in original html file"
	bodyTagIndex := anInteger.! !

!FileProxy methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
clearElements
	elements := OrderedCollection new.! !

!FileProxy methodsFor: 'accessing-tags' stamp: ' 21/4/07 22:07'!
clearImgTagIndexes
	imgTagIndexes := OrderedCollection new.! !

!FileProxy methodsFor: 'accessing-tags' stamp: ' 21/4/07 22:07'!
clearLinkTagIndexes
	linkTagIndexes := OrderedCollection new.! !

!FileProxy methodsFor: 'accessing-tags' stamp: ' 21/4/07 22:07'!
clearServletTagIndexes
	servletTagIndexes := OrderedCollection new.! !

!FileProxy methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
codepage
	" #iso2 by default"
	codepage isNil ifTrue: [self codepage: #'iso-8859-2'].
	^codepage.! !

!FileProxy methodsFor: 'private' stamp: ' 21/4/07 22:07'!
codepage: aSymbol
	codepage := aSymbol.! !

!FileProxy methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
content
	content isNil ifTrue: 
		[(self contentType ~= 'text/html') 
			ifTrue: [self refreshContent] 
			ifFalse: [self content: ''] ].
	^content! !

!FileProxy methodsFor: 'private' stamp: ' 21/4/07 22:07'!
content: aString
	content := aString.! !

!FileProxy methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
contentType
	"MIME type of original content. 'unknown' if not known"
	contentType isNil ifTrue: [self contentType: 'unknown'].
	^contentType.! !

!FileProxy methodsFor: 'private' stamp: ' 21/4/07 22:07'!
contentType: aMIMEString
	"MIME type of original content"
	contentType := aMIMEString.! !

!FileProxy methodsFor: 'accessing-timestamps' stamp: 'mivsek 8/22/2007 16:02'!
createdTimestamp
	^self timestamps at: #created ifAbsent: [nil]! !

!FileProxy methodsFor: 'private' stamp: ' 21/4/07 22:07'!
detectCodepage: aString
	" "
	| win1250 iso2 sevenBit above127 |
	win1250 := 0.
	aString do: [:ch | (#(200 154 138 158 142) includes: ch asInteger) ifTrue: [win1250 := win1250+1] ].
	iso2 := 0.
	aString do: [:ch | (#(200 185 169 190 174) includes: ch asInteger) ifTrue: [iso2 := iso2+1] ].
	sevenBit := 0.
"	aString do: [:ch | (#(94 123 91 96 64) includes: ch asInteger) ifTrue: [sevenBit := sevenBit+1] ]."
	above127 := 0.
	aString do: [:ch | ch asInteger > 127 ifTrue: [above127 := above127+1] ].
	above127 = 0 
		ifTrue: [sevenBit ~= 0 ifTrue: [^#'7bit'] ifFalse: [^#csz] ]
		ifFalse: [iso2 > win1250 ifTrue: [^#'iso-8859-2'] ifFalse: [^#'win-1250'] ].! !

!FileProxy methodsFor: 'private' stamp: ' 21/4/07 22:07'!
elements

	"all elements (texts and tags) of a gtml page"

	elements isNil ifTrue: 
		[self clearElements.
		(self contentType = 'text/html') ifTrue: [self refreshContent] ].
	^elements! !

!FileProxy methodsFor: 'accessing-timestamps' stamp: 'mivsek 8/22/2007 17:10'!
expiresTimestamp
	"for broswer: after 6 hours by default. It is good to reload from time to time 
	if changes were made "
	^SpTimestamp fromSeconds: (SpTimestamp now asSeconds + (6 * 3600))! !

!FileProxy methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
fileStillExist
	^self filename exists! !

!FileProxy methodsFor: 'accessing' stamp: 'mivsek 8/22/2007 15:51'!
filename
	| homeDir |
	(filename isKindOf: SpFilename) ifTrue: [self filename: filename].
	filename isNil ifTrue: [^''].
	(filename includes: $: ) ifTrue:   " TEMPORARY !!!!!!"
		[filename := filename copyReplaceAll: (filename copyUpTo: ':') with: ''.
		homeDir := self site homeDirectory copyReplaceAll: (filename copyUpTo: ':') with: ''.
		filename := filename copyReplaceAll: homeDir with: ''].
	^SpFilename named:
		(self site homeDirectory, (filename copyReplaceAll: self site homeDirectory with: ''))! !

!FileProxy methodsFor: 'private' stamp: 'janko 3/22/2008 13:20'!
filename: aFilename
	filename := aFilename asString.
	self site notNil ifTrue: 
		[filename := filename copyReplaceAll: self site homeDirectory with: ''].! !

!FileProxy methodsFor: 'initialize-release' stamp: 'mivsek 8/22/2007 15:48'!
from: aString
	| file relFilename |
	file := SpFilename named: aString. file exists ifFalse: [self releaseContent. ^nil].
	relFilename := (file asString copyReplaceAll: self site homeDirectory with: '').
	self filename: (relFilename isEmpty ifTrue: ['\'] ifFalse: [relFilename]).
	self refreshTimestamps.
	self contentType: (self site mimeMap typeForExtension: file extension).
	self size: file fileSize.
	self contentType = 'text/html' 
		ifTrue:
			[self content: file contentsOfEntireFile.
			self codepage: (self detectCodepage: self content).
			self content: (AIDASite convert: self content fromCodepage: self codepage).	
			"self prepareHTMLPage. self makeAbsoluteImgTags. self content: nil"
			self setSize]
		ifFalse: 
			[self sizeAboveMark ifFalse:  "cache small files"
				[| stream | 
				[stream := file readStream binary. self content: stream contents] ensure: [stream close] ] ]! !

!FileProxy methodsFor: 'testing' stamp: 'mivsek 8/22/2007 17:07'!
hasFileChanged
	"check if original file changed since last visit"
	^self modifiedTimestamp ~= self filename modifiedTimestamp.! !

!FileProxy methodsFor: 'accessing-tags' stamp: ' 21/4/07 22:07'!
imgTagIndexes
	"indexes of elements, which represent image tags in original html file"
	imgTagIndexes isNil ifTrue: [self clearImgTagIndexes].
	^imgTagIndexes! !

!FileProxy methodsFor: 'indexing' stamp: ' 21/4/07 22:07'!
indexContent
	"index text in server global index"
	self contentType = 'text/html' ifTrue: [self site index indexObject: self]! !

!FileProxy methodsFor: 'indexing' stamp: ' 21/4/07 22:07'!
indexText
	"pure text, without tags, for indexer"
	| stream bodyStart element text |
	bodyStart := (1 to: self elements size) 
		detect: [:index | '<BODY*' match: (self elements at: index)] ifNone: [1].
	stream := WriteStream on: String new.
	bodyStart + 1 to: self elements size do:  [:index | 
		element := self elements at: index.
		(element notEmpty and: [element first ~= $<]) ifTrue: [stream nextPutAll: element] ].
	text := stream contents "copyReplaceAll: WebElement new eol with: ' '". 
	text := text copyReplaceAll: '&nbsp;' with: ' '.
	^text! !

!FileProxy methodsFor: 'indexing' stamp: ' 21/4/07 22:07'!
indexTitle
	"find title, if noone, then return url"
	1 to: self elements size do: [:index |
		('<TITLE*' match: (self elements at: index)) ifTrue: [^self elements at: index+1] ].
	^self site urlResolver halfUrlFor: self! !

!FileProxy methodsFor: 'testing' stamp: 'mivsek 8/22/2007 20:33'!
isRespondingStreamed
	"should we stream that file directly to TCP socket or not?"
	self content notNil ifTrue: [^false].  "cached content is sent faster without streaming"
	self contentType = 'text/html' ifTrue: [^false]. "pure HTML files also not"
	^true! !

!FileProxy methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isWebElement
	^false! !

!FileProxy methodsFor: 'testing' stamp: ' 21/8/07 09:38'!
isWebPage
	^true "not actually, but let pretend it is"! !

!FileProxy methodsFor: 'accessing-tags' stamp: ' 21/4/07 22:07'!
linkTagIndexes
	"indexes of elements, which represent link tags in original html file"
	linkTagIndexes isNil ifTrue: [self clearLinkTagIndexes].
	^linkTagIndexes! !

!FileProxy methodsFor: 'parsing' stamp: 'mivsek 1/10/2008 19:16'!
makeAbsoluteImgTags
	"if image url's are relative, make them absolute. Also do it for a body tag (background image). 
	Url is constructed from a imageServer address from WebServer settings, and from filename with 	homeDirectory substracted. "
	| fname urlStart index tag bodyTag |
	fname := (self filename asString) copyReplaceAll: (self site homeDirectory) with: ''.
	fname := fname copyFrom: 1 to: (fname lastIndexOf: (AIDASite slash at: 1)).
	urlStart := 'http://', self site host, (fname copyReplaceAll: '\' with: '/').
	self imgTagIndexes do: [:inx  | tag := self elements at: inx.
		('*http://*' match: tag) ifFalse:    "not an absolute source url"
			[self elements at: inx put: (self absoluteTagFrom: tag urlStart: urlStart)] ].
	self bodyTagIndex notNil ifTrue:
		[bodyTag := self elements at: bodyTagIndex.
		(('*background*' match:  bodyTag) and: [('*http://*' match:  bodyTag) not]) 
			ifTrue: 
				[index := bodyTag asLowercase findString: 'background' startingAt: 1.
				index := bodyTag findString: '"' startingAt: index.
				(( bodyTag at: index+1) = $#) ifTrue: [^self]. "only colored background"
				self elements at: bodyTagIndex put:
					((bodyTag copyFrom: 1 to: index), urlStart, 
						(bodyTag copyFrom: index + 1 to: bodyTag size)) ] ].! !

!FileProxy methodsFor: 'printing-servlets' stamp: ' 21/4/07 22:07'!
methodFrom: aMessageString

	^aMessageString asSymbol "for now!! "! !

!FileProxy methodsFor: 'accessing-timestamps' stamp: 'mivsek 8/22/2007 16:01'!
modifiedTimestamp
	^self timestamps at: #modified ifAbsent: [nil]! !

!FileProxy methodsFor: 'parsing' stamp: ' 21/8/07 09:38'!
prepareHTMLPage
	"decompose content into elements (texts and tags)"
	| stream tag index |
	self clearElements. self clearImgTagIndexes. 
	self clearLinkTagIndexes. self clearServletTagIndexes.
	stream := ReadStream on: self content.
	index := 1.
	[stream atEnd] whileFalse:
		[self elements add: (stream upTo: $<). index := index + 1.
		tag := ('<', (stream upTo: $>), '>').
		('<aida*' match: tag) ifTrue: [self servletTagIndexes add: index].
		('*img*' match: tag) ifTrue: [self imgTagIndexes add: index].
		('*href*' match: tag) 
			ifTrue: [self linkTagIndexes add: index]
			ifFalse: 
				[(('*body*' match: tag) and: [('*/body*' match: tag) not])
					ifTrue: [self bodyTagIndex: index]].
		self elements add: tag. index := index + 1.
		].! !

!FileProxy methodsFor: 'printing' stamp: 'janko 7/18/2008 20:16'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	"convert elements into a HTML stream"
	| response |
	self isRespondingStreamed 
		ifTrue: 
			[ response := aRequest streamedResponse.
			response length: self size.	"to stream on HTTP/1.0 too, because chunking 
				is not allowed"
			self site 
				addResponseHeadersTo: response
				forPage: self
				on: aSession.
			self streamFileToResponse: response.
			response close ]
		ifFalse: [ aStream nextPutAll: self content asString ].	"temporary text/html too!! "
	self sizeAboveMark ifTrue: [ self releaseContent ]	"for large files"

	" 
	self contentType = 'text/html'
		ifTrue:
			[index := 1.
			self elements do: [:each | 
				each == self elements last ifTrue: [^self]. 
				(self servletTagIndexes includes: index) 
					ifTrue: [self printServlet: index on: aStream forSession: aSession]
					ifFalse: [aStream nextPutAll: 
						(AIDASite convertToWeb: each on: aSession)].
				index := index + 1 ] ]
		ifFalse: 
			[aStream nextPutAll: self content asByteString].
"! !

!FileProxy methodsFor: 'printing-servlets' stamp: 'janko 7/18/2008 20:48'!
printServlet: index on: aStream for: aRequest on: aSession 
	| tag appClass message app element |
	tag := (self elements at: index) copy readStream.
	appClass := (tag
		upTo: $  ;
		upTo: $ ) asSymbol.
	message := tag upTo: $>.
	app := aSession servletAppFor: appClass.
	app isNil ifTrue: [ ^ self ].
	element := self 
		servletPerform: message
		on: app.
	element notNil ifTrue: 
		[ element 
			printHTMLPageOn: aStream for: aRequest
			on: aSession ]! !

!FileProxy methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
printString
	^('aFileProxy for: ', self filename asString)! !

!FileProxy methodsFor: 'printing' stamp: 'janko 7/18/2008 19:10'!
printWebPageFor: aRequest on: aSession 
	self fileStillExist ifFalse: [self removeYourself. ^WebPage new].
	(self content isNil or: [self content isEmpty]) ifTrue: [self refreshContent]. "not cached"
	self hasFileChanged ifTrue: 
		[self refreshContent.
		self indexContent].
	^self! !

!FileProxy methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
put: aString
	"replace self with contents in aString from PUT request"
	self content: aString.
	self writeToFile! !

!FileProxy methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
refreshContent
	self from: self filename asString.! !

!FileProxy methodsFor: 'private' stamp: 'mivsek 8/22/2007 16:34'!
refreshTimestamps
	self timestamps at: #created put: self filename createdTimestamp.
	self timestamps at: #modified put: self filename modifiedTimestamp.! !

!FileProxy methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
releaseContent
	"release origContent and elements to save memory"
	content := nil.
	elements := nil.! !

!FileProxy methodsFor: 'private' stamp: ' 21/8/07 09:38'!
removeYourself
	"remove from url resolver and elsewhere"
	self releaseContent.
	self site urlResolver removeObject: self.! !

!FileProxy methodsFor: 'printing-servlets' stamp: ' 21/4/07 22:07'!
servletPerform: aMessageString on: aWebApp

	| method |
	method := (self methodFrom: aMessageString).
	(aWebApp class canUnderstand: method) ifFalse: [^nil].
	^aWebApp
		perform: method
		withArguments: (self argumentsFrom: aMessageString).! !

!FileProxy methodsFor: 'accessing-tags' stamp: ' 21/4/07 22:07'!
servletTagIndexes
	"indexes of elements, which represent servlet in original html file"
	servletTagIndexes isNil ifTrue: [self clearServletTagIndexes].
	^servletTagIndexes! !

!FileProxy methodsFor: 'private' stamp: ' 21/8/07 09:38'!
setSize
	"size of content in bytes"
	(self elements notEmpty)
		ifTrue: [self size: (self elements inject: 0 into: [:sum : e | sum + e size]) ]
		ifFalse: [self size: self content size]! !

!FileProxy methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
site
	^site! !

!FileProxy methodsFor: 'private' stamp: ' 21/4/07 22:07'!
site: anAIDASite
	site := anAIDASite.! !

!FileProxy methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
size
	"size of content"
	size isNil ifTrue: [self setSize].
	^size! !

!FileProxy methodsFor: 'private' stamp: ' 21/8/07 09:38'!
size: anInteger
	size := anInteger! !

!FileProxy methodsFor: 'testing' stamp: 'mivsek 2/10/2008 15:53'!
sizeAboveMark

	"test if content is above specified size"
	^self size > (5 * 1024) "bytes" "5KB"! !

!FileProxy methodsFor: 'printing' stamp: ' 21/8/07 09:38'!
streamFileToResponse: aHTTPStreamedResponse
	| stream | 
	[stream := self filename asFilename readStream binary. 
	[stream atEnd] whileFalse: [aHTTPStreamedResponse nextPut: stream next] ] 
		ensure: [stream close]! !

!FileProxy methodsFor: 'private' stamp: ' 21/4/07 22:07'!
timestamps
	timestamps isNil ifTrue: [self timestamps: Dictionary new.].
	^timestamps! !

!FileProxy methodsFor: 'private' stamp: ' 21/4/07 22:07'!
timestamps: aDictionary
	" a dictionary with a DateAndTime at: 
	#accessed #modified #statusChanged #created"
	timestamps := aDictionary.! !

!FileProxy methodsFor: 'private' stamp: ' 21/4/07 22:07'!
writeToFile
	| stream |
	[stream := self filename writeStream. stream binary.
	stream nextPutAll: self content]
		ensure: [stream close].
	self refreshTimestamps.! !

FileProxy subclass: #WebCaptcha
	instanceVariableNames: 'text random'
	classVariableNames: 'Cache'
	poolDictionaries: ''
	category: 'Aida-Imaging'!

!WebCaptcha commentStamp: 'janko 3/22/2008 15:26' prior: 0!
A WebCaptcha provides a CAPTCHA image from input text, to be used for separating humans from bots, for instance when submiting blog comments (http://en.wikipedia.org/wiki/Captcha)

Our capthca is using ImageMagick (http://www.imagemagick.org) which must be installed first. It combines a background image (by default captcha-background.png, must be on current directory) with generated one from provided text in default font (see method fontName, currently Andy), which also must be installed and known to ImageMagick. 

To collect all installed fonts in your system for IMagics, run script
	imagick_type_gen > ~/.magics/type.xml

See http://www.imagemagick.org/Usage/scripts/imagick_type_gen

To see and change parameters of building captcha, look into method #prepareCaptchaImage!

!WebCaptcha class methodsFor: 'private-cache' stamp: 'janko 3/22/2008 16:29'!
addToCache: aCaptcha
	self cache add: aCaptcha! !

!WebCaptcha class methodsFor: 'private-cache' stamp: 'janko 3/22/2008 16:28'!
cache
	Cache isNil ifTrue: [self initCache].
	^Cache! !

!WebCaptcha class methodsFor: 'private-cache' stamp: 'janko 3/22/2008 16:28'!
initCache
	Cache := Set new.! !

!WebCaptcha class methodsFor: 'instance creation' stamp: 'janko 3/22/2008 18:38'!
new
	"prepare a CAPTCHA with random string"
	| session |
	session := self firstSessionFromStack. "on which we are building this captcha"
	^super new 
		site: (session notNil ifTrue: [session site] ifFalse: [nil]);
		setRandomText! !

!WebCaptcha class methodsFor: 'instance creation' stamp: 'janko 3/22/2008 18:39'!
newFor: aString
	"prepare a CAPTCHA image for that string"
	^self new 
		text: aString! !

!WebCaptcha class methodsFor: 'private-cache' stamp: 'janko 3/22/2008 16:30'!
releaseExpiredCaptchas
	self cache do: [:each | 
		each isExpired ifTrue: [each removeYourself "from url resolver"]]! !

!WebCaptcha methodsFor: 'defaults' stamp: 'janko 3/22/2008 01:16'!
backgroundImageFilename
	^'captcha-background.png'! !

!WebCaptcha methodsFor: 'private' stamp: 'janko 3/22/2008 15:27'!
contentType
	^'image/png'.! !

!WebCaptcha methodsFor: 'printing' stamp: 'janko 3/22/2008 17:20'!
contentsOfCaptcha
	^self filename contentsOfEntireFileBinary asString
	! !

!WebCaptcha methodsFor: 'defaults' stamp: 'janko 3/22/2008 15:11'!
fontName
	^'Andy'! !

!WebCaptcha methodsFor: 'defaults' stamp: 'janko 3/22/2008 23:23'!
imageSize
	^150@30	! !

!WebCaptcha methodsFor: 'private' stamp: 'janko 3/22/2008 16:26'!
isExpired
	"it exists more than few hours"
	^(SpTimestamp now asSeconds - self preparedTimestamp asSeconds) > (10*3600)! !

!WebCaptcha methodsFor: 'private' stamp: 'janko 3/22/2008 17:09'!
isRespondingStreamed
	^false! !

!WebCaptcha methodsFor: 'converting' stamp: 'janko 7/18/2008 17:52'!
prepareCaptchaImage
	| textFName captchaFName |
	textFName := 'captchatext', self random, '.png'. captchaFName := 'captcha', self random, '.png'.
	OSProcess waitForCommand: 
		'convert -size 150x30 xc:transparent -font ', self fontName, ' -pointsize 25 -draw "text 40,23 ''', 
		self text, ''' " ', textFName.
	OSProcess waitForCommand: 'composite -gravity center ', 
		textFName, ' ', self backgroundImageFilename, ' ',  captchaFName.
	(SpFilename named: textFName) delete.
	self filename: captchaFName.
	self setPreparedTimestamp.! !

!WebCaptcha methodsFor: 'private' stamp: 'janko 3/22/2008 16:07'!
preparedTimestamp
	^self timestamps at: #prepared ifAbsent: [nil]! !

!WebCaptcha methodsFor: 'printing' stamp: 'janko 7/18/2008 20:17'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	self prepareCaptchaImage.
	aStream nextPutAll: self contentsOfCaptcha.
	self filename delete.
	self class
		addToCache: self ;
		releaseExpiredCaptchas! !

!WebCaptcha methodsFor: 'printing' stamp: 'janko 7/18/2008 19:10'!
printWebPageFor: aRequest on: aSession 
	^self! !

!WebCaptcha methodsFor: 'converting' stamp: 'janko 7/18/2008 17:53'!
random
	random isNil ifTrue: 
		[random := (AIDASite random next * 1000) truncated printString].
	^random
! !

!WebCaptcha methodsFor: 'accessing' stamp: 'janko 3/22/2008 18:35'!
randomCharacter
	"from 25 lowercase, 25 upercase, 10 numbers"
	| rnum |
	rnum := (Random new next * 60) truncated.
	rnum <= 25 ifTrue: [^($a asInteger + rnum) asCharacter].
	rnum <= 50 ifTrue: [^($A asInteger + rnum-25) asCharacter].
	^($0 asInteger + rnum-50) asCharacter
! !

!WebCaptcha methodsFor: 'accessing' stamp: 'janko 3/22/2008 18:40'!
randomCharacterOnGen: aRandom
	"from 25 lowercase, 25 upercase, 10 numbers"
	| rnum |
	rnum := (aRandom next * 60) truncated.
	rnum <= 25 ifTrue: [^($a asInteger + rnum) asCharacter].
	rnum <= 50 ifTrue: [^($A asInteger + rnum-25) asCharacter].
	^($0 asInteger + rnum-50) asCharacter
! !

!WebCaptcha methodsFor: 'private' stamp: 'janko 3/22/2008 16:08'!
setPreparedTimestamp
	^self timestamps at: #prepared put: SpTimestamp now.! !

!WebCaptcha methodsFor: 'accessing' stamp: 'janko 3/22/2008 18:42'!
setRandomText
	"5 characters or numbers"
	| rnd rgen |
	rnd := ByteString new: 5. rgen := Random new.
	1 to: 5 do: [:inx | rnd at: inx put: (self randomCharacterOnGen: rgen)].
	self text: rnd.
! !

!WebCaptcha methodsFor: 'private' stamp: 'janko 3/22/2008 15:59'!
size
	^self filename fileSize! !

!WebCaptcha methodsFor: 'accessing' stamp: 'janko 3/22/2008 18:27'!
text
	"string to be converted into captcha image"
	text isNil ifTrue: [^''].
	^text! !

!WebCaptcha methodsFor: 'accessing' stamp: 'janko 3/22/2008 18:27'!
text: aString
	"string to be converted into captcha image"
	text := aString! !

Object subclass: #History
	instanceVariableNames: 'dates values changedDates authors comments historyCollection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Support'!

!History class methodsFor: 'odb specific' stamp: ' 21/4/07 22:07'!
instancesAreForwarders
	"Gemstone"

	^false! !

!History class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new initialize! !

!History class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
tests
	"some test of histories. Use it directly from code with doIt/printIt/inspectIt "

	| history |
	history := History new.
	history value: 0 dated: (Date today ) author: '' comment: ''.
	history value: -2000 dated: (Date today - (365*100) -2) author: '' comment: ''.
	history value: -3 dated: (Date today -3) author: '' comment: ''.
	history value: -1 dated: (Date today -1) author: '' comment: ''.
	history value: 2 dated: (Date today +2) author: '' comment: ''.
	history value: 1 dated: (Date today +1) author: '' comment: ''.! !

!History methodsFor: 'private - history arrays' stamp: ' 21/4/07 22:07'!
activeFromArray: anArray

	"return a active flag from entries in history array"

	^anArray at: 5! !

!History methodsFor: 'private - adding-removing' stamp: ' 21/4/07 22:07'!
addValue: aValue dated: aDate author: aString comment: aCommentString
	"adds data, which is newer than any existing one"
	self dates add: aDate asDays.
	self values add: aValue.
	self changedDates add: Date today asDays.
	self authors add: aString.
	self comments add: aString.! !

!History methodsFor: 'private - history arrays' stamp: ' 21/4/07 22:07'!
arrayWithDate: aDate time: aTime value: aValue active: aBoolean

	"return a array with argument vaules for entry to history collection"

	| array |
	array := Array new: 5.
	array 
		at: 1 put: aDate year;
		at: 2 put: aDate day;
		at: 3 put: (aTime notNil ifTrue: [aTime asSeconds] ifFalse: [0]);
		at: 4 put: aValue;
		at: 5 put: aBoolean.
	^array! !

!History methodsFor: 'private - accessing' stamp: ' 21/4/07 22:07'!
authors
	" a collection of authors, which made changes, (as a String or reference to a WebUser)"
	^authors! !

!History methodsFor: 'private - adding-removing' stamp: ' 21/4/07 22:07'!
changeValue: aValue author: aString comment: aCommentString onIndex: anIndexNumber
	"change data in a specificied index"
	self values at: anIndexNumber put: aValue.
	self changedDates at: anIndexNumber put: Date today asDays.
	self authors at: anIndexNumber put: aString.
	self comments at: anIndexNumber put: aString.! !

!History methodsFor: 'private - accessing' stamp: ' 21/4/07 22:07'!
changedDates
	" a collection of dates, when value was changed in asDays format"
	^changedDates! !

!History methodsFor: 'private - accessing' stamp: ' 21/4/07 22:07'!
comments
	" a collection of comments about changes"
	^comments! !

!History methodsFor: 'private - history arrays' stamp: ' 21/4/07 22:07'!
dateFromArray: anArray

	"return a date from entries in history array"

	^Date
		newDay: (anArray at: 2)
		year: (anArray at: 1)! !

!History methodsFor: 'private - accessing' stamp: ' 21/4/07 22:07'!
dates
	" a collection of dates of changes in asDays format"
	^dates! !

!History methodsFor: 'private - adding-removing' stamp: ' 21/4/07 22:07'!
hasDuplicateDates

	"maintenance - check if more than one entry with the same date exist"

	| unique |
	unique := Set new.
	self dates do: [: date | (unique includes: date)
		ifTrue: [^true]
		ifFalse: [unique add: date] ].
	^false! !

!History methodsFor: 'private - accessing' stamp: ' 21/4/07 22:07'!
historyCollection
	"this is an ordered collection of array with year, day in year, time, value, active flag. 
	Collection is ordered from oldest to newest version of value. Active flag is used to 
	'delete' some version from history, but you can still have a trace, who/when some 
	change occured.

	3.11.98 NOT USED anymore. Here just for migration!!"


	historyCollection isNil ifTrue: [self initHistoryCollection].
	^historyCollection! !

!History methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
historyOfChanges

	"return a history of all changes of value. It is returned as ordered collection of collections, each with:
	valid from date
	valid to date (for last change: nil)
	new value
	date of change
	author of change
	comment of change	

Returned collection has the oldest change (by valied from date) as first, the newest as last. "! !

!History methodsFor: 'private - accessing' stamp: ' 21/4/07 22:07'!
indexForDate: aDate

	"return index of entry in dates collection, which fits most to the specified date. If no direct 
	entry on a specified date exist, then entry for previous date is used. If aDate is older than
	any date in collection, nil is returned!!"


	| first last days mdays index |

	first := 1. last := self dates size.
	last = 0 ifTrue: [^nil].
	days := aDate asDays.
	days >= self dates last ifTrue: [^last].
	days < self dates first ifTrue: [^nil].
	[last >= first] whileTrue: 	
		[ "(first = last and: [days = (self dates at: first)]) ifTrue: [^first].  " " not needed!! "
		index := (first + last) // 2.
		mdays := self dates at: index.
		days = mdays ifTrue: [^index].
		days < mdays
			ifTrue: [last := index - 1.]
			ifFalse: [first := index + 1.]].
"2.11.98 Sivec prej :
		^1 max: index - 1.
"
	index := (first + last) // 2.
	^1 max: index.




"
| h |
h := History new.
h dates
	add: (Date today) asDays;
	add: (Date today + 1) asDays;
	add: (Date today + 3) asDays;
	add: (Date today + 4) asDays.
Transcript cr;show: (h indexForDate: Date today - 1) printString.
Transcript cr;show: (h indexForDate: Date today + 1) printString.
Transcript cr;show: (h indexForDate: Date today + 2) printString.
Transcript cr;show: (h indexForDate: Date today + 3) printString.
h inspect.
"





"
	self historyCollection keysAndValuesDo: [:index :array |
		(self activeFromArray: array) 
			ifTrue:
				[(self dateFromArray: array) < (aDate + 1)
					ifTrue: [prevIndex := index]
					ifFalse: [^prevIndex]	] ].
	^prevIndex
"! !

!History methodsFor: 'initialize - release' stamp: ' 21/4/07 22:07'!
initAuthors
	authors := OrderedCollection new.! !

!History methodsFor: 'initialize - release' stamp: ' 21/4/07 22:07'!
initChangedDates
	changedDates := OrderedCollection new.! !

!History methodsFor: 'initialize - release' stamp: ' 21/4/07 22:07'!
initComments
	comments := OrderedCollection new.! !

!History methodsFor: 'initialize - release' stamp: ' 21/4/07 22:07'!
initDates
	"adds default entry: 1.1.1901"
	dates := OrderedCollection new.! !

!History methodsFor: 'private - accessing' stamp: ' 21/4/07 22:07'!
initHistoryCollection
	historyCollection := OrderedCollection new.! !

!History methodsFor: 'initialize - release' stamp: ' 21/4/07 22:07'!
initValues
	values := OrderedCollection new.! !

!History methodsFor: 'initialize - release' stamp: ' 21/4/07 22:07'!
initialize
	"2.11.1998 all inits do not add default values anymore"

	self initDates.
	self initValues.
	self initChangedDates.
	self initAuthors.
	self initComments.! !

!History methodsFor: 'private - adding-removing' stamp: ' 21/4/07 22:07'!
insertValue: aValue dated: aDate author: aString comment: aCommentString beforeIndex: anIndex
	"adds data in position before specified index"
	self dates add: aDate asDays beforeIndex: anIndex.
	self values add: aValue beforeIndex: anIndex.
	self changedDates add: Date today asDays beforeIndex: anIndex.
	self authors add: aString beforeIndex: anIndex.
	self comments add: aString beforeIndex: anIndex.! !

!History methodsFor: 'private - history arrays' stamp: ' 21/4/07 22:07'!
migrateHistoryCollection

	"migrate from historyCollection to bunch of collections. Remove duplicates"

	| date |
	date := nil.
	self initialize.
	self historyCollection reverseDo: [:array |
		date = (self dateFromArray: array)
			ifFalse: 
				[self 
					value: (self valueFromArray: array)
					dated: (self dateFromArray: array)
					author: ('')
					comment: ('').
				date := self dateFromArray: array] ].


"
Janko := History selectFromOdb select: [:h | h historyCollection size > 10].
(Janko at: 3) migrateHistoryCollection
Janko do: [:each | each migrateHistoryCollection].
Janko  select: [:h | h dates size > 3].
"! !

!History methodsFor: 'private - accessing' stamp: ' 21/4/07 22:07'!
newestDate

	"return a date of newest version of value"

	self dates isEmpty
		ifTrue: [^nil]
		ifFalse: 	[^Date fromDays: self dates last]

"History new newestDate"! !

!History methodsFor: 'private - accessing' stamp: ' 21/4/07 22:07'!
newestValue

	"return a newest version of value in historyCollection."

	self values isEmpty
		ifTrue: [^nil]
		ifFalse: 	[^self values last]
	

"History new newestValue"! !

!History methodsFor: 'private - accessing' stamp: ' 21/4/07 22:07'!
oldestDate

	"return a date of oldest version of value"

	self dates isEmpty
		ifTrue: [^nil]
		ifFalse: 	[^self dates first]
	

"History new oldestDate"! !

!History methodsFor: 'private - accessing' stamp: ' 21/4/07 22:07'!
oldestValue

	"return a oldest version of value"

	self values isEmpty
		ifTrue: [^nil]
		ifFalse: 	[^self values last]
	

"History new oldestValue"! !

!History methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
printString

	^('aHistory: ', self value printString)! !

!History methodsFor: 'private - adding-removing' stamp: ' 21/4/07 22:07'!
removeAllNils
	"REPARING BAD HISTORIES - remove all nil entries. Initialize if no more entries"

	2 to: self values size do: [:index |
		(self values at: index) isNil ifTrue: 
			[self removeIndex: index. ^self removeAllNils] ]! !

!History methodsFor: 'private - adding-removing' stamp: ' 21/4/07 22:07'!
removeDuplicateDates

	"maintenance - remove all duplicate entries with same dates. Live last one"

	| date index |
	self hasDuplicateDates ifTrue: 
		[date := 0. index := nil.
		1 to: self dates size do: 
			[:inx | 
				(self dates at: inx) = date ifTrue: [index := inx].
				date := self dates at: inx].
		index notNil 
			ifTrue: 
				[self removeIndex: index-1.
				self removeDuplicateDates].
		 ].! !

!History methodsFor: 'private - adding-removing' stamp: ' 21/4/07 22:07'!
removeIndex: aNumber
	"remove all entries on specified index"
	self dates removeIndex: aNumber.
	self values removeIndex: aNumber.
	self changedDates removeIndex: aNumber.
	self authors removeIndex: aNumber.
	self comments removeIndex: aNumber.! !

!History methodsFor: 'private - adding-removing' stamp: ' 21/4/07 22:07'!
removeLastNil
	"REPARING BAD HISTORIES - remove last entry if value nil. Initialize if no more entries"
	self values isEmpty 
		ifTrue: 
			[self initialize. self values removeLast; addLast: true.
			Transcript cr; show: 'init, true']
		ifFalse: [self values last isNil ifTrue: 
			[self removeIndex: (self dates size).
			self removeLastNil] ]! !

!History methodsFor: 'private - history arrays' stamp: ' 21/4/07 22:07'!
timeFromArray: anArray

	"return a time from entries in history array"

	^Time fromSeconds: (anArray at: 3)! !

!History methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
value
	"get the curently valid value"

	^self valueDated: Date today

"History new value"! !

!History methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
value: aValue author: aString comment: aCommentString 
	"change a value of time series, which will be valid immediately - today. You should state author and a short comment about changes. see other methods for detailed explanation"

	self value: aValue
		dated: Date today
		author: aString
		comment: aCommentString! !

!History methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
value: aValue dated: aDate author: aString comment: aCommentString 
	""

	| index |

	index := self indexForDate: aDate. 
	index isNil   "older than any existing or first" 
		ifTrue: 
			[self 
				insertValue: aValue 
				dated: aDate
				author: aString 
				comment: aCommentString
				beforeIndex: 1.
			^self].

"Sprememba 18.11.1998 Sivec, prej 'index >= self dates  size'."
	index > self dates  size  "newer than any existing"
		ifTrue: 
			[self addValue: aValue dated: aDate author: aString comment: aCommentString. ^self].


	(index ~= 0 and: [(self dates at: index) = aDate asDays]) "already exists"
		ifTrue: 
			[self 
				changeValue: aValue 
				author: aString 
				comment: aCommentString 
				onIndex: index.
			^self].

		self    "insert new somewhere in the middle"
			insertValue: aValue 
			dated: aDate
			author: aString 
			comment: aCommentString
			beforeIndex: index+1.



"History new value: 1234 dated: Date today author: 'Janko' comment: 'test'"! !

!History methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
valueDated: aDate 
	"get the value from history, which was valid on specified date. Returns nil if aDate is older from 
	oldest entry in history"

	
	| index |
	index := self indexForDate: aDate.
	index = 0 ifTrue: [^nil].
	index = nil ifTrue: [^nil].
	^self values at: index.! !

!History methodsFor: 'private - history arrays' stamp: ' 21/4/07 22:07'!
valueFromArray: anArray

	"return a value from entries in history array"

	^anArray at: 4! !

!History methodsFor: 'private - accessing' stamp: ' 21/4/07 22:07'!
values
	" a collection of values, each valid from date in dates at the same index"
	^values! !

Object subclass: #ImageStream
	instanceVariableNames: 'imageStream progressValue'
	classVariableNames: 'GrayPalette256 ImageKindTable ColorPalette256'
	poolDictionaries: ''
	category: 'Aida-Imaging'!

!ImageStream commentStamp: '<historical>' prior: 0!
ImageStream 

Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.
!

ImageStream subclass: #BmpImageStream
	instanceVariableNames: 'bfType bfSize bfReserved1 bfReserved2 bfOffBits biSize biWidth biHeight biPlanes biBitCount biCompression biSizeImage biXPelsPerMeter biYPelsPerMeter biClrUsed biClrImportant imagePalette imageObject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Imaging'!

!BmpImageStream commentStamp: '<historical>' prior: 0!
BmpImageStream 

Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.
!

!BmpImageStream class methodsFor: 'copyright' stamp: ' 21/4/07 22:07'!
copyright
	^'Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.'! !

!BmpImageStream class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
example1
	"BmpImageStream example1."

	| image filename stream |
	image := Image fromUser.
	filename := 'zzz.bmp' asFilename.
	stream := BmpImageStream on: filename writeStream.
	[Cursor write showWhile: [stream nextPutImage: image]]
		valueNowOrOnUnwindDo: [stream close].
	^image! !

!BmpImageStream class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
example2
	"BmpImageStream example2."

	| filename stream image |
	filename := 'zzz.bmp' asFilename.
	stream := BmpImageStream on: filename readStream.
	[Cursor read showWhile: [image := stream nextImage]]
		valueNowOrOnUnwindDo: [stream close].
	stream show: image.
	^image! !

!BmpImageStream class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
example3
	"BmpImageStream example3."

	| image filename stream |
	image := Image fromUser.
	filename := 'zzz.bmp' asFilename.
	stream := BmpImageStream on: filename writeStream.
	[Cursor write
		showWhile: 
			[stream compute: [:value | Transcript cr; show: value printString].
			stream nextPutImage: image]]
		valueNowOrOnUnwindDo: [stream close].
	^image! !

!BmpImageStream class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
example4
	"BmpImageStream example4."

	| filename stream image |
	filename := 'zzz.bmp' asFilename.
	stream := BmpImageStream on: filename readStream.
	[Cursor read
		showWhile: 
			[stream compute: [:value | Transcript cr; show: value printString].
			image := stream nextImage]]
		valueNowOrOnUnwindDo: [stream close].
	stream show: image.
	^image! !

!BmpImageStream class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
example5
	"BmpImageStream example5."

	| image filename stream progress |
	image := Image fromUser.
	filename := 'zzz.bmp' asFilename.
	stream := BmpImageStream on: filename writeStream.
	[Cursor write
		showWhile: 
			[progress := Progress new.
			stream compute: [:value | progress value: value].
			progress message: 'writing bmp...'.
			progress do: [stream nextPutImage: image]]]
		valueNowOrOnUnwindDo: [stream close].
	^image! !

!BmpImageStream class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
example6
	"BmpImageStream example6."

	| filename stream progress image |
	filename := 'zzz.bmp' asFilename.
	stream := BmpImageStream on: filename readStream.
	[Cursor read
		showWhile: 
			[progress := Progress new.
			stream compute: [:value | progress value: value].
			progress message: 'reading bmp...'.
			progress do: [image := stream nextImage]]]
		valueNowOrOnUnwindDo: [stream close].
	stream show: image.
	^image! !

!BmpImageStream class methodsFor: 'copyright' stamp: ' 21/4/07 22:07'!
system
	^'Goodies'! !

!BmpImageStream class methodsFor: 'copyright' stamp: ' 21/4/07 22:07'!
version
	^'003'! !

!BmpImageStream methodsFor: 'private' stamp: ' 21/4/07 22:07'!
colorValueFrom: rgbInteger 
	| scalingValue |
	scalingValue := ColorValue scalingValue.
	^ColorValue
		scaledRed: (self
				convertValue: ((rgbInteger bitShift: -16)
						bitAnd: 255)
				from: 255
				to: scalingValue)
		scaledGreen: (self
				convertValue: ((rgbInteger bitShift: -8)
						bitAnd: 255)
				from: 255
				to: scalingValue)
		scaledBlue: (self
				convertValue: (rgbInteger bitAnd: 255)
				from: 255
				to: scalingValue)! !

!BmpImageStream methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
nextImage
	self readBitmapFileHeader.
	self readBitmapInfoHeader.
	self readImagePalette.
	self readImageData.
	^imageObject! !

!BmpImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
nextLSBLong
	^self next + (self next bitShift: 8) + (self next bitShift: 16) + (self next bitShift: 24)! !

!BmpImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
nextLSBLongPut: a32BitW 
	self nextPut: (a32BitW bitAnd: 255).
	self nextPut: ((a32BitW bitShift: -8)
			bitAnd: 255).
	self nextPut: ((a32BitW bitShift: -16)
			bitAnd: 255).
	self nextPut: ((a32BitW bitShift: -24)
			bitAnd: 255)! !

!BmpImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
nextLSBWord
	^self next + (self next bitShift: 8)! !

!BmpImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
nextLSBWordPut: a16BitW 
	self nextPut: (a16BitW bitAnd: 255).
	self nextPut: ((a16BitW bitShift: -8)
			bitAnd: 255)! !

!BmpImageStream methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
nextPutImage: anImage 
	(anImage isKindOf: Image)
		ifFalse: [^self errorCanNotWrite].
	(anImage bitsPerPixel = 8 or: [anImage bitsPerPixel = 24])
		ifTrue: [imageObject := anImage]
		ifFalse: [anImage bitsPerPixel > 8
				ifTrue: [imageObject := anImage convertToPalette: (FixedPalette
									redShift: 16
									redMask: 255
									greenShift: 8
									greenMask: 255
									blueShift: 0
									blueMask: 255)]
				ifFalse: [Cursor wait showWhile: [imageObject := anImage convertToPalette: self class colorPalette256 renderedBy: ErrorDiffusion new]]].
	self writeBitmapFileHeader.
	self writeBitmapInfoHeader.
	self writeImagePalette.
	self writeImageData.
	bfSize := self position.
	^anImage! !

!BmpImageStream methodsFor: 'decoding' stamp: ' 21/4/07 22:07'!
readBitmapFileHeader
	| position |
	position := self position.
	(self hasMagicNumber: 'BM' asByteArray)
		ifFalse: [^self errorCanNotRead].
	self position: position.
	bfType := self nextWord.
	bfSize := self nextLSBLong.
	bfReserved1 := self nextLSBWord.
	bfReserved2 := self nextLSBWord.
	bfOffBits := self nextLSBLong! !

!BmpImageStream methodsFor: 'decoding' stamp: ' 21/4/07 22:07'!
readBitmapInfoHeader
	biSize := self nextLSBLong.
	biWidth := self nextLSBLong.
	biHeight := self nextLSBLong.
	biPlanes := self nextLSBWord.
	biBitCount := self nextLSBWord.
	biCompression := self nextLSBLong.
	biSizeImage := self nextLSBLong.
	biXPelsPerMeter := self nextLSBLong.
	biYPelsPerMeter := self nextLSBLong.
	biClrUsed := self nextLSBLong.
	biClrImportant := self nextLSBLong! !

!BmpImageStream methodsFor: 'decoding' stamp: ' 21/4/07 22:07'!
readDepth1Data
	| image total progress count index |
	self position: bfOffBits.
	image := Image
				extent: biWidth @ biHeight
				depth: biBitCount
				palette: imagePalette.
	total := biWidth * biHeight.
	progress := 0.
	self progress: progress / total.
	biHeight - 1
		to: 0
		by: -1
		do: 
			[:y | 
			count := 128.
			0 to: biWidth - 1
				do: 
					[:x | 
					count = 128 ifTrue: [index := self next].
					image
						atX: x
						y: y
						put: (index bitAnd: count).
					count := count / 2.
					count < 1 ifTrue: [count := 128].
					progress := progress + 1.
					self progress: progress / total]].
	^image! !

!BmpImageStream methodsFor: 'decoding' stamp: ' 21/4/07 22:07'!
readDepth1Palette
	^MappedPalette blackWhite! !

!BmpImageStream methodsFor: 'decoding' stamp: ' 21/4/07 22:07'!
readDepth24Data
	| image total progress count rgb color index |
	self position: bfOffBits.
	image := Image
				extent: biWidth @ biHeight
				depth: biBitCount
				palette: imagePalette.
	total := biWidth * biHeight.
	progress := 0.
	self progress: progress / total.
	biHeight - 1
		to: 0
		by: -1
		do: 
			[:y | 
			count := 0.
			0 to: biWidth - 1
				do: 
					[:x | 
					rgb := self next + (self next bitShift: 8) + (self next bitShift: 16).
					color := self colorValueFrom: rgb.
					index := image palette indexOfPaintNearest: color.
					image
						atX: x
						y: y
						put: index.
					count := count + 3.
					progress := progress + 1.
					self progress: progress / total].
			[count \\ 4 = 0]
				whileFalse: 
					[self next.
					count := count + 1]].
	^image! !

!BmpImageStream methodsFor: 'decoding' stamp: ' 21/4/07 22:07'!
readDepth24Palette
	^FixedPalette
		redShift: 16
		redMask: 255
		greenShift: 8
		greenMask: 255
		blueShift: 0
		blueMask: 255! !

!BmpImageStream methodsFor: 'decoding' stamp: ' 21/4/07 22:07'!
readDepth8Data
	| image total progress count index |
	self position: bfOffBits.
	image := Image
				extent: biWidth @ biHeight
				depth: biBitCount
				palette: imagePalette.
	total := biWidth * biHeight.
	progress := 0.
	self progress: progress / total.
	biHeight - 1
		to: 0
		by: -1
		do: 
			[:y | 
			count := 0.
			0 to: biWidth - 1
				do: 
					[:x | 
					index := self next.
					image
						atX: x
						y: y
						put: index.
					count := count + 1.
					progress := progress + 1.
					self progress: progress / total].
			[count \\ 4 = 0]
				whileFalse: 
					[self next.
					count := count + 1]].
	^image! !

!BmpImageStream methodsFor: 'decoding' stamp: ' 21/4/07 22:07'!
readDepth8Palette
	| size colors rgb color |
	size := 1 bitShift: biBitCount.
	biClrUsed = 0 ifFalse: [size := biClrUsed].
	colors := Array new: size.
	1 to: size
		do: 
			[:index | 
			rgb := self next + (self next bitShift: 8) + (self next bitShift: 16).
			self next.
			color := self colorValueFrom: rgb.
			colors at: index put: color].
	^MappedPalette withColors: colors! !

!BmpImageStream methodsFor: 'decoding' stamp: ' 21/4/07 22:07'!
readImageData
	biCompression = 0 ifFalse: [^self errorCanNotRead].
	imageObject := nil.
	biBitCount = 24 ifTrue: [imageObject := self readDepth24Data].
	biBitCount = 8 ifTrue: [imageObject := self readDepth8Data].
	biBitCount = 1 ifTrue: [imageObject := self readDepth1Data].
	imageObject isNil ifTrue: [^self errorCanNotRead].
	^imageObject! !

!BmpImageStream methodsFor: 'decoding' stamp: ' 21/4/07 22:07'!
readImagePalette
	imagePalette := nil.
	biBitCount = 24 ifTrue: [imagePalette := self readDepth24Palette].
	biBitCount = 8 ifTrue: [imagePalette := self readDepth8Palette].
	biBitCount = 1 ifTrue: [imagePalette := self readDepth1Palette].
	imagePalette isNil ifTrue: [^self errorCanNotRead]! !

!BmpImageStream methodsFor: 'private' stamp: ' 21/4/07 22:07'!
rgbIntegerArrayFor: aPalette 
	| colors |
	colors := Array new: aPalette maxIndex + 1.
	1 to: colors size do: [:i | colors at: i put: (self rgbIntegerFrom: (aPalette at: i - 1 ifAbsent: [ColorValue white]))].
	^colors! !

!BmpImageStream methodsFor: 'private' stamp: ' 21/4/07 22:07'!
rgbIntegerFrom: aColorValue 
	| scalingValue |
	scalingValue := ColorValue scalingValue.
	^((self
		convertValue: aColorValue scaledRed
		from: scalingValue
		to: 255)
		bitShift: 16)
		+ ((self
				convertValue: aColorValue scaledGreen
				from: scalingValue
				to: 255)
				bitShift: 8) + (self
			convertValue: aColorValue scaledBlue
			from: scalingValue
			to: 255)! !

!BmpImageStream methodsFor: 'encoding' stamp: ' 21/4/07 22:07'!
writeBitmapFileHeader
	bfType := ($B asInteger bitShift: 8)
				+ $M asInteger.
	bfSize := 0.
	bfReserved1 := 0.
	bfReserved2 := 0.
	bfOffBits := nil.
	imageObject bitsPerPixel = 8 ifTrue: [bfOffBits := 54 + (imageObject palette size * 4)].
	imageObject bitsPerPixel = 24 ifTrue: [bfOffBits := 54].
	bfOffBits isNil ifTrue: [^self errorCanNotWrite].
	self nextWordPut: bfType.
	self nextLSBLongPut: bfSize.
	self nextLSBWordPut: bfReserved1.
	self nextLSBWordPut: bfReserved2.
	self nextLSBLongPut: bfOffBits! !

!BmpImageStream methodsFor: 'encoding' stamp: ' 21/4/07 22:07'!
writeBitmapInfoHeader
	biSize := 40.
	biWidth := imageObject width.
	biHeight := imageObject height.
	biPlanes := 1.
	biBitCount := imageObject bitsPerPixel.
	biCompression := 0.
	biSizeImage := 0.
	biXPelsPerMeter := 0.
	biYPelsPerMeter := 0.
	biClrUsed := 0.
	imageObject bitsPerPixel = 8 ifTrue: [biClrUsed := (imageObject palette size)].
	biClrImportant := 0.
	self nextLSBLongPut: biSize.
	self nextLSBLongPut: biWidth.
	self nextLSBLongPut: biHeight.
	self nextLSBWordPut: biPlanes.
	self nextLSBWordPut: biBitCount.
	self nextLSBLongPut: biCompression.
	self nextLSBLongPut: biSizeImage.
	self nextLSBLongPut: biXPelsPerMeter.
	self nextLSBLongPut: biYPelsPerMeter.
	self nextLSBLongPut: biClrUsed.
	self nextLSBLongPut: biClrImportant! !

!BmpImageStream methodsFor: 'encoding' stamp: ' 21/4/07 22:07'!
writeDepth24Data
	| total progress count index color rgb |
	self position = bfOffBits ifFalse: [^self errorCanNotWrite].
	total := biWidth * biHeight.
	progress := 0.
	self progress: progress / total.
	biHeight - 1
		to: 0
		by: -1
		do: 
			[:y | 
			count := 0.
			0 to: biWidth - 1
				do: 
					[:x | 
					index := imageObject atX: x y: y.
					color := imageObject palette at: index.
					rgb := self rgbIntegerFrom: color.
					self nextPut: (rgb bitAnd: 255).
					self nextPut: ((rgb bitShift: -8)
							bitAnd: 255).
					self nextPut: ((rgb bitShift: -16)
							bitAnd: 255).
					count := count + 3.
					progress := progress + 1.
					self progress: progress / total].
			[count \\ 4 = 0]
				whileFalse: 
					[self nextPut: 0.
					count := count + 1]].
	^imageObject! !

!BmpImageStream methodsFor: 'encoding' stamp: ' 21/4/07 22:07'!
writeDepth24Palette
	^imageObject palette! !

!BmpImageStream methodsFor: 'encoding' stamp: ' 21/4/07 22:07'!
writeDepth8Data
	| count index |
	self position = bfOffBits ifFalse: [^self errorCanNotWrite].
"
	total := biWidth * biHeight.
	progress := 0.
	self progress: progress / total. 
"
	biHeight - 1
		to: 0
		by: -1
		do: 
			[:y | 
			count := 0.
			0 to: biWidth - 1
				do: 
					[:x | 
					index := imageObject atX: x y: y.
					self nextPut: index.
					count := count + 1.
"					progress := progress + 1.
					self progress: progress / total
"
					].
			[count \\ 4 = 0]
				whileFalse: 
					[self nextPut: 0.
					count := count + 1]].
	^imageObject! !

!BmpImageStream methodsFor: 'encoding' stamp: ' 21/4/07 22:07'!
writeDepth8Palette
	| size palette array |
	size := 1 bitShift: biBitCount.
	palette := imageObject palette.
	palette size = size ifFalse: [self errorCanNotWrite].
	array := self rgbIntegerArrayFor: palette.
	array
		do: 
			[:rgb | 
			self nextPut: (rgb bitAnd: 255).
			self nextPut: ((rgb bitShift: -8)
					bitAnd: 255).
			self nextPut: ((rgb bitShift: -16)
					bitAnd: 255).
			self nextPut: 0].
	^imageObject palette! !

!BmpImageStream methodsFor: 'encoding' stamp: ' 21/4/07 22:07'!
writeImageData
	biCompression = 0 ifFalse: [^self errorCanNotWrite].
	biBitCount = 24 ifTrue: [^self writeDepth24Data].
	biBitCount = 8 ifTrue: [^self writeDepth8Data].
	^self errorCanNotWrite! !

!BmpImageStream methodsFor: 'encoding' stamp: ' 21/4/07 22:07'!
writeImagePalette
	imagePalette := nil.
	biBitCount = 24 ifTrue: [imagePalette := self writeDepth24Palette].
	biBitCount = 8 ifTrue: [imagePalette := self writeDepth8Palette].
	imagePalette isNil ifTrue: [^self errorCanNotWrite]! !

ImageStream subclass: #BosImageStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Imaging'!

!BosImageStream commentStamp: '<historical>' prior: 0!
BosImageStream 

Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.

!

!BosImageStream class methodsFor: 'copyright' stamp: ' 21/4/07 22:07'!
copyright
	^'Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.'! !

!BosImageStream class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
example1
	"BosImageStream example1."

	| image filename stream |
	image := Image fromUser.
	filename := 'zzz.bos' asFilename.
	stream := BosImageStream on: filename writeStream.
	[Cursor write showWhile: [stream nextPutImage: image]]
		valueNowOrOnUnwindDo: [stream close].
	^image! !

!BosImageStream class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
example2
	"BosImageStream example2."

	| filename stream image |
	filename := 'zzz.bos' asFilename.
	stream := BosImageStream on: filename readStream.
	[Cursor read showWhile: [image := stream nextImage]]
		valueNowOrOnUnwindDo: [stream close].
	stream show: image.
	^image! !

!BosImageStream class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
example3
	"BosImageStream example3."

	| image filename stream |
	image := Image fromUser.
	filename := 'zzz.bos' asFilename.
	stream := BosImageStream on: filename writeStream.
	[Cursor write
		showWhile: 
			[stream compute: [:value | Transcript cr; show: value printString].
			stream nextPutImage: image]]
		valueNowOrOnUnwindDo: [stream close].
	^image! !

!BosImageStream class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
example4
	"BosImageStream example4."

	| filename stream image |
	filename := 'zzz.bos' asFilename.
	stream := BosImageStream on: filename readStream.
	[Cursor read
		showWhile: 
			[stream compute: [:value | Transcript cr; show: value printString].
			image := stream nextImage]]
		valueNowOrOnUnwindDo: [stream close].
	stream show: image.
	^image! !

!BosImageStream class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
example5
	"BosImageStream example5."

	| image filename stream progress |
	image := Image fromUser.
	filename := 'zzz.bos' asFilename.
	stream := BosImageStream on: filename writeStream.
	[Cursor write
		showWhile: 
			[progress := Progress new.
			stream compute: [:value | progress value: value].
			progress message: 'writing bos...'.
			progress do: [stream nextPutImage: image]]]
		valueNowOrOnUnwindDo: [stream close].
	^image! !

!BosImageStream class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
example6
	"BosImageStream example6."

	| filename stream image progress |
	filename := 'zzz.bos' asFilename.
	stream := BosImageStream on: filename readStream.
	[Cursor read
		showWhile: 
			[progress := Progress new.
			stream compute: [:value | progress value: value].
			progress message: 'reading bos...'.
			progress do: [image := stream nextImage]]]
		valueNowOrOnUnwindDo: [stream close].
	stream show: image.
	^image! !

!BosImageStream class methodsFor: 'copyright' stamp: ' 21/4/07 22:07'!
system
	^'Goodies'! !

!BosImageStream class methodsFor: 'copyright' stamp: ' 21/4/07 22:07'!
version
	^'003'! !

!BosImageStream methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
nextImage
	| size process bos image |
	self readHeader isNil ifTrue: [^nil].
	size := imageStream size max: 1.0e-6.
	process := [[imageStream position < size]
				whileTrue: 
					[self progress: imageStream position / size.
					(Delay forMilliseconds: 3) wait]] newProcess.
	process priority: Processor activeProcess priority + 5.
	process resume.
	
	[bos := BinaryObjectStorage onOldNoScan: imageStream.
	self progress: 0.
	image := bos next.
	self progress: 1]
		valueNowOrOnUnwindDo: 
			[process terminate.
			process := nil].
	^image! !

!BosImageStream methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
nextPutImage: anImage 
	| size process bos bytes |
	(anImage isKindOf: Image) not ifTrue: [^self errorCanNotWrite].
	self writeHeader.
	bytes := WriteStream on: (bytes := ByteArray new: 1024).
	self size timesRepeat: [bytes nextPut: 0].
	bos := BinaryObjectStorage onNew: bytes.
	bos nextPut: anImage asImage.
	size := bytes contents size max: 1.0e-6.
	process := [[imageStream position < size]
				whileTrue: 
					[self progress: imageStream position / size.
					(Delay forMilliseconds: 3) wait]] newProcess.
	process priority: Processor activeProcess priority + 5.
	process resume.
	
	[bos := BinaryObjectStorage onNew: imageStream.
	self progress: 0.
	bos nextPut: anImage asImage.
	self progress: 1]
		valueNowOrOnUnwindDo: 
			[process terminate.
			process := nil].
	^anImage! !

!BosImageStream methodsFor: 'decoding' stamp: ' 21/4/07 22:07'!
readHeader
	(self hasMagicNumber: 'BosImage' asByteArray)
		ifFalse: [^self errorCanNotRead]! !

!BosImageStream methodsFor: 'encoding' stamp: ' 21/4/07 22:07'!
writeHeader
	self nextPutAll: 'BosImage' asByteArray.! !

ImageStream subclass: #GifImageStream
	instanceVariableNames: 'width height bitsPerPixel colorPalette rowByteSize xpos ypos pass interlace codeSize clearCode eoiCode freeCode maxCode prefixTable suffixTable remainBitCount bufByte bufStream transparentPixel'
	classVariableNames: 'Extension Terminator ImageSeparator GraphicControlLabel'
	poolDictionaries: ''
	category: 'Aida-Imaging'!

!GifImageStream commentStamp: '<historical>' prior: 0!
GifImageStream 

Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.
!

!GifImageStream class methodsFor: 'copyright' stamp: ' 21/4/07 22:07'!
copyright
	^'Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.'! !

!GifImageStream class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
example1
	"GifImageStream example1."

	| image filename stream |
	image := Image fromUser.
	filename := 'zzz.gif' asFilename.
	stream := GifImageStream on: filename writeStream.
	[Cursor write showWhile: [stream nextPutImage: image]]
		valueNowOrOnUnwindDo: [stream close].
	^image! !

!GifImageStream class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
example2
	"GifImageStream example2."

	| filename stream image |
	filename := 'zzz.gif' asFilename.
	stream := GifImageStream on: filename readStream.
	[Cursor read showWhile: [image := stream nextImage]]
		valueNowOrOnUnwindDo: [stream close].
	stream show: image.
	^image! !

!GifImageStream class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
example3
	"GifImageStream example3."

	| image filename stream |
	image := Image fromUser.
	filename := 'zzz.gif' asFilename.
	stream := GifImageStream on: filename writeStream.
	[Cursor write
		showWhile: 
			[stream compute: [:value | Transcript cr; show: value printString].
			stream nextPutImage: image]]
		valueNowOrOnUnwindDo: [stream close].
	^image! !

!GifImageStream class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
example4
	"GifImageStream example4."

	| filename stream image |
	filename := 'zzz.gif' asFilename.
	stream := GifImageStream on: filename readStream.
	[Cursor read
		showWhile: 
			[stream compute: [:value | Transcript cr; show: value printString].
			image := stream nextImage]]
		valueNowOrOnUnwindDo: [stream close].
	stream show: image.
	^image! !

!GifImageStream class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
example5
	"GifImageStream example5."

	| image filename stream progress |
	image := Image fromUser.
	filename := 'zzz.gif' asFilename.
	stream := GifImageStream on: filename writeStream.
	[Cursor write
		showWhile: 
			[progress := Progress new.
			stream compute: [:value | progress value: value].
			progress message: 'writing gif...'.
			progress do: [stream nextPutImage: image]]]
		valueNowOrOnUnwindDo: [stream close].
	^image! !

!GifImageStream class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
example6
	"GifImageStream example6."

	| filename stream image progress |
	filename := 'zzz.gif' asFilename.
	stream := GifImageStream on: filename readStream.
	[Cursor read
		showWhile: 
			[progress := Progress new.
			stream compute: [:value | progress value: value].
			progress message: 'reading gif...'.
			progress do: [image := stream nextImage]]]
		valueNowOrOnUnwindDo: [stream close].
	stream show: image.
	^image! !

!GifImageStream class methodsFor: 'class initialization' stamp: ' 21/4/07 22:07'!
initialize
	"GifImageStream initialize."

	ImageSeparator := $, asInteger.
	Extension := $!! asInteger.
	Terminator := $; asInteger.
	GraphicControlLabel := 249! !

!GifImageStream class methodsFor: 'copyright' stamp: ' 21/4/07 22:07'!
system
	^'Goodies'! !

!GifImageStream class methodsFor: 'copyright' stamp: ' 21/4/07 22:07'!
version
	^'003'! !

!GifImageStream methodsFor: 'private' stamp: ' 21/4/07 22:07'!
checkCodeSize
	(freeCode > maxCode and: [codeSize < 12])
		ifTrue:
			[codeSize := codeSize + 1.
			maxCode := (1 bitShift: codeSize) - 1]! !

!GifImageStream methodsFor: 'private' stamp: ' 21/4/07 22:07'!
colorValueFrom: rgbInteger 
	| scalingValue |
	scalingValue := ColorValue scalingValue.
	^ColorValue
		scaledRed: (self
				convertValue: ((rgbInteger bitShift: -16)
						bitAnd: 255)
				from: 255
				to: scalingValue)
		scaledGreen: (self
				convertValue: ((rgbInteger bitShift: -8)
						bitAnd: 255)
				from: 255
				to: scalingValue)
		scaledBlue: (self
				convertValue: (rgbInteger bitAnd: 255)
				from: 255
				to: scalingValue)! !

!GifImageStream methodsFor: 'private' stamp: ' 21/4/07 22:07'!
convertToCoverageWithTransparentPixel: pixelValue with: anImage 
	| image row |
	image := Image
				extent: anImage extent
				depth: 1
				palette: CoveragePalette monoMaskPalette.
	0 to: anImage height - 1
		do: 
			[:i | 
			row := anImage rowAt: i.
			row := row collect: [:p | p = pixelValue
							ifTrue: [0]
							ifFalse: [1]].
			image rowAt: i putAll: row].
	^image! !

!GifImageStream methodsFor: 'private' stamp: ' 21/4/07 22:07'!
convertToGray: coverageValue 
	coverageValue = CoverageValue transparent ifTrue: [^ColorValue white].
	coverageValue = CoverageValue opaque ifTrue: [^ColorValue black].
	^ColorValue brightness: coverageValue coverage! !

!GifImageStream methodsFor: 'private' stamp: ' 21/4/07 22:07'!
convertToMappedPalette: coveragePalette 
	coveragePalette size = 2 ifTrue: [^MappedPalette whiteBlack].
	^MappedPalette withColors: (coveragePalette collect: [:coverage | self convertToGray: coverage])! !

!GifImageStream methodsFor: 'packing' stamp: ' 21/4/07 22:07'!
fillBuffer
	| packSize |
	packSize := self next.
	bufStream := ReadStream on: (self next: packSize)! !

!GifImageStream methodsFor: 'bits access' stamp: ' 21/4/07 22:07'!
flushBits
	remainBitCount = 0
		ifFalse: 
			[self nextBytePut: bufByte.
			remainBitCount := 0].
	self flushBuffer! !

!GifImageStream methodsFor: 'packing' stamp: ' 21/4/07 22:07'!
flushBuffer
	bufStream isEmpty ifTrue: [^self].
	self nextPut: bufStream size.
	self nextPutAll: bufStream contents.
	bufStream := WriteStream on: (ByteArray new: 256)! !

!GifImageStream methodsFor: 'encoding' stamp: ' 21/4/07 22:07'!
flushCode
	self flushBits! !

!GifImageStream methodsFor: 'bits access' stamp: ' 21/4/07 22:07'!
nextBits
	| integer readBitCount shiftCount |
	integer := 0.
	remainBitCount = 0
		ifTrue: 
			[readBitCount := 8.
			shiftCount := 0]
		ifFalse: 
			[readBitCount := remainBitCount.
			shiftCount := remainBitCount - 8].
	[readBitCount < codeSize]
		whileTrue: 
			[self peekByte isNil ifTrue: [^eoiCode].
			integer := integer + (self nextByte bitShift: shiftCount).
			shiftCount := shiftCount + 8.
			readBitCount := readBitCount + 8].
	self peekByte isNil ifTrue: [^eoiCode].
	(remainBitCount := readBitCount - codeSize) = 0
		ifTrue: [integer := integer + (self nextByte bitShift: shiftCount)]
		ifFalse: [integer := integer + (self peekByte bitShift: shiftCount)].
	^integer bitAnd: maxCode! !

!GifImageStream methodsFor: 'bits access' stamp: ' 21/4/07 22:07'!
nextBitsPut: anInteger 
	| integer writeBitCount shiftCount |
	shiftCount := 0.
	remainBitCount = 0
		ifTrue: 
			[writeBitCount := 8.
			integer := anInteger]
		ifFalse: 
			[writeBitCount := remainBitCount.
			integer := bufByte + (anInteger bitShift: 8 - remainBitCount)].
	[writeBitCount < codeSize]
		whileTrue: 
			[self nextBytePut: ((integer bitShift: shiftCount)
					bitAnd: 255).
			shiftCount := shiftCount - 8.
			writeBitCount := writeBitCount + 8].
	(remainBitCount := writeBitCount - codeSize) = 0
		ifTrue: [self nextBytePut: (integer bitShift: shiftCount)]
		ifFalse: [bufByte := integer bitShift: shiftCount].
	^anInteger! !

!GifImageStream methodsFor: 'packing' stamp: ' 21/4/07 22:07'!
nextByte
	bufStream atEnd
		ifTrue:
			[self atEnd ifTrue: [^nil].
			self fillBuffer].
	^bufStream next! !

!GifImageStream methodsFor: 'packing' stamp: ' 21/4/07 22:07'!
nextBytePut: aByte
	bufStream nextPut: aByte.
	bufStream size >= 254 ifTrue: [self flushBuffer]! !

!GifImageStream methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
nextImage
	| bits depth image |
	self readHeader isNil ifTrue: [^nil].
	bits := self readBitData.
	depth := bitsPerPixel > 8
				ifTrue: [^self error: 'unexpected bits per pixel']
				ifFalse: [bitsPerPixel = 1
						ifTrue: [1]
						ifFalse: [bitsPerPixel = 2
								ifTrue: [2]
								ifFalse: [bitsPerPixel <= 4
										ifTrue: [4]
										ifFalse: [8]]]].
	depth < 8 ifTrue: [bits := self
					packBits: bits
					depthFrom8To: depth
					width: width
					height: height
					pad: 8].
	image := Image
				extent: width @ height
				depth: depth
				palette: colorPalette
				bits: bits
				pad: 8.
	transparentPixel notNil ifTrue: [image := OpaqueImage figure: image shape: (CachedImage on: (self convertToCoverageWithTransparentPixel: transparentPixel with: image) asRetainedMedium)].
	^image! !

!GifImageStream methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
nextPutImage: anImage 
	| theImage trueBitsPerPixel achievableBitsPerPixel bits |
	(anImage isKindOf: Image)
		ifFalse: [^self errorCanNotWrite].
	anImage bitsPerPixel > 8
		ifTrue: [theImage := anImage convertToPalette: self class colorPalette256 "renderedBy: ErrorDiffusion new"]
		ifFalse: [theImage := anImage].
	width := theImage width.
	height := theImage height.
	colorPalette := theImage palette.
	trueBitsPerPixel := theImage bitsPerPixel.
	achievableBitsPerPixel := 2 raisedTo: ((colorPalette size - 1) highBit - 1) highBit.
	bitsPerPixel := achievableBitsPerPixel.
	colorPalette := theImage palette.
	colorPalette paintBasis == CoverageValue
		ifTrue: 
			[transparentPixel := 0.
			colorPalette := self convertToMappedPalette: colorPalette].
	bits := theImage bits.
	trueBitsPerPixel < 8 ifTrue: [bits := self
					unpackBits: bits
					depthTo8From: bitsPerPixel
					width: theImage width
					height: theImage height
					pad: 32].
	interlace := true.
	
	[self writeHeader.
	self writeBitData: bits]
		valueNowOrOnUnwindDo: [self close].
	^theImage! !

!GifImageStream methodsFor: 'private' stamp: ' 21/4/07 22:07'!
packBits: bits depthFrom8To: depth width: xSize height: ySize pad: pad 
	| maxPixelVal pixelInByte bitsWidth pBitsWidth pBits |
	(#(1 2 4) includes: depth)
		ifFalse: [^self error: 'depth must be 1, 2, or 4'].
	(#(8 16 32) includes: pad)
		ifFalse: [^self error: 'pad must be 8, 16, or 32'].
	maxPixelVal := (1 bitShift: depth)
				- 1.
	pixelInByte := 8 / depth.
	bitsWidth := xSize * 8 + pad - 1 // pad * (pad / 8).
	pBitsWidth := xSize * depth + pad - 1 // pad * (pad / 8).
	pBits := ByteArray new: pBitsWidth * ySize.
	1 to: ySize
		do: 
			[:i | 
			| bitIndex pBitIndex pixelVal count |
			bitIndex := i - 1 * bitsWidth.
			pBitIndex := i - 1 * pBitsWidth.
			pixelVal := 0.
			count := 0.
			1 to: xSize
				do: 
					[:j | 
					| val |
					val := bits at: (bitIndex := bitIndex + 1).
					val > maxPixelVal ifTrue: [^self error: 'can''t pack bits'].
					pixelVal := (pixelVal bitShift: depth)
								+ val.
					(count := count + 1) >= pixelInByte
						ifTrue: 
							[pBits at: (pBitIndex := pBitIndex + 1) put: pixelVal.
							pixelVal := 0.
							count := 0]].
			count > 0 ifTrue: [pBits at: (pBitIndex := pBitIndex + 1) put: (pixelVal bitShift: depth * (pixelInByte - count))]].
	^pBits! !

!GifImageStream methodsFor: 'packing' stamp: ' 21/4/07 22:07'!
peekByte
	bufStream atEnd
		ifTrue:
			[self atEnd ifTrue: [^nil].
			self fillBuffer].
	^bufStream peek! !

!GifImageStream methodsFor: 'decoding' stamp: ' 21/4/07 22:07'!
readBitData
	| bits outCodes outCount bitMask initCodeSize code curCode oldCode inCode finChar set |
	pass := 0.
	xpos := 0.
	ypos := 0.
	rowByteSize := width * 8 + 7 // 8.
	remainBitCount := 0.
	bufByte := 0.
	bufStream := ReadStream on: ByteArray new.
	bits := ByteArray new: width * height.
	outCodes := ByteArray new: 1025.
	outCount := 0.
	bitMask := (1 bitShift: bitsPerPixel)
				- 1.
	prefixTable := Array new: 4096.
	suffixTable := Array new: 4096.
	initCodeSize := self next.
	self setParameters: initCodeSize.
	set := Set new: height.
	self progress: set size / height.
	[(code := self readCode) = eoiCode]
		whileFalse: [code = clearCode
				ifTrue: 
					[self setParameters: initCodeSize.
					curCode := oldCode := code := self readCode.
					finChar := curCode bitAnd: bitMask.
					set add: ypos.
					self progress: set size / height.
					self writePixel: finChar to: bits]
				ifFalse: 
					[curCode := inCode := code.
					curCode >= freeCode
						ifTrue: 
							[curCode := oldCode.
							outCodes at: (outCount := outCount + 1) put: finChar].
					[curCode > bitMask]
						whileTrue: 
							[outCount > 1024 ifTrue: [^self error: 'corrupt GIF file (OutCount)'].
							outCodes at: (outCount := outCount + 1) put: (suffixTable at: curCode + 1).
							curCode := prefixTable at: curCode + 1].
					finChar := curCode bitAnd: bitMask.
					outCodes at: (outCount := outCount + 1) put: finChar.
					outCount
						to: 1
						by: -1
						do: 
							[:i | 
							set add: ypos.
							self progress: set size / height.
							self writePixel: (outCodes at: i)
								to: bits].
					outCount := 0.
					prefixTable at: freeCode + 1 put: oldCode.
					suffixTable at: freeCode + 1 put: finChar.
					oldCode := inCode.
					freeCode := freeCode + 1.
					self checkCodeSize]].
	prefixTable := suffixTable := nil.
	^bits! !

!GifImageStream methodsFor: 'decoding' stamp: ' 21/4/07 22:07'!
readCode
	^self nextBits! !

!GifImageStream methodsFor: 'GIF89a' stamp: ' 21/4/07 22:07'!
readExtension
	| label |
	label := self next.
	label == GraphicControlLabel
		ifTrue: [self readGraphicControlExtensionBlock]
		ifFalse: [self error: 'unsupported extension block']! !

!GifImageStream methodsFor: 'GIF89a' stamp: ' 21/4/07 22:07'!
readGraphicControlExtensionBlock
	| isTransparent |
	self next.
	isTransparent := (self next bitAnd: 1)
				= 1.
	self next: 2.
	isTransparent
		ifTrue: [transparentPixel := self next]
		ifFalse: [self next].
	self next! !

!GifImageStream methodsFor: 'decoding' stamp: ' 21/4/07 22:07'!
readHeader
	| byte hasColorMap |
	((self hasMagicNumber: 'GIF89a' asByteArray)
		or: [self hasMagicNumber: 'GIF87a' asByteArray])
		ifFalse: [^self errorCanNotRead].
	self readWord.
	self readWord.
	byte := self next.
	hasColorMap := (byte bitAnd: 128)
				~= 0.
	bitsPerPixel := (byte bitAnd: 7)
				+ 1.
	byte := self next.
	byte := self next.
	hasColorMap
		ifTrue: 
			[| array |
			array := Array new: (1 bitShift: bitsPerPixel).
			1 to: array size do: [:i | array at: i put: (self colorValueFrom: (self next bitShift: 16)
							+ (self next bitShift: 8) + self next)].
			colorPalette := MappedPalette withColors: array]
		ifFalse: 
			[Transcript cr; show: 'GIF file does not have a color map.'.
			colorPalette := MappedPalette monochromeDefault].
	
	[byte := self next.
	byte == Extension]
		whileTrue: [self readExtension].
	byte = ImageSeparator ifFalse: [^self error: 'corrupt GIF file (nosep)'].
	self readWord.
	self readWord.
	width := self readWord.
	height := self readWord.
	interlace := (self next bitAnd: 64)
				~= 0! !

!GifImageStream methodsFor: 'encoding' stamp: ' 21/4/07 22:07'!
readPixelFrom: bits
	| pixel |
	ypos >= height ifTrue: [^nil].
	pixel := bits at: (ypos * rowByteSize + xpos + 1).
	self updatePixelPosition.
	^pixel! !

!GifImageStream methodsFor: 'decoding' stamp: ' 21/4/07 22:07'!
readWord
	^self next + (self next bitShift: 8)! !

!GifImageStream methodsFor: 'private' stamp: ' 21/4/07 22:07'!
rgbIntegerArrayFor: aPalette 
	| colors |
	colors := Array new: aPalette maxIndex + 1.
	1 to: colors size do: [:i | colors at: i put: (self rgbIntegerFrom: (aPalette at: i - 1 ifAbsent: [ColorValue white]))].
	^colors! !

!GifImageStream methodsFor: 'private' stamp: ' 21/4/07 22:07'!
rgbIntegerFrom: aColorValue 
	| scalingValue |
	scalingValue := ColorValue scalingValue.
	^((self
		convertValue: aColorValue scaledRed
		from: scalingValue
		to: 255)
		bitShift: 16)
		+ ((self
				convertValue: aColorValue scaledGreen
				from: scalingValue
				to: 255)
				bitShift: 8) + (self
			convertValue: aColorValue scaledBlue
			from: scalingValue
			to: 255)! !

!GifImageStream methodsFor: 'private' stamp: ' 21/4/07 22:07'!
setParameters: initCodeSize
	clearCode := 1 bitShift: initCodeSize.
	eoiCode := clearCode + 1.
	freeCode := clearCode + 2.
	codeSize := initCodeSize + 1.
	maxCode := (1 bitShift: codeSize) - 1! !

!GifImageStream methodsFor: 'GIF89a' stamp: ' 21/4/07 22:07'!
transparentPixel: aPixelValueOrNil 
	transparentPixel := aPixelValueOrNil! !

!GifImageStream methodsFor: 'private' stamp: ' 21/4/07 22:07'!
unpackBits: bits depthTo8From: depth width: xSize height: ySize pad: pad 
	| bitMask pixelInByte bitsWidth upBitsWidth stopWidth trailingSize upBits |
	(#(1 2 4) includes: depth)
		ifFalse: [^self error: 'depth must be 1, 2, or 4'].
	(#(8 16 32) includes: pad)
		ifFalse: [^self error: 'pad must be 8, 16, or 32'].
	bitMask := (1 bitShift: depth)
				- 1.
	pixelInByte := 8 / depth.
	bitsWidth := xSize * depth + pad - 1 // pad * (pad / 8).
	upBitsWidth := xSize * 8 + pad - 1 // pad * (pad / 8).
	stopWidth := xSize * depth + 7 // 8.
	trailingSize := xSize - (stopWidth - 1 * pixelInByte).
	upBits := ByteArray new: upBitsWidth * ySize.
	1 to: ySize
		do: 
			[:i | 
			| bitIndex upBitIndex val |
			bitIndex := i - 1 * bitsWidth.
			upBitIndex := i - 1 * upBitsWidth.
			1 to: stopWidth - 1
				do: 
					[:j | 
					val := bits at: (bitIndex := bitIndex + 1).
					upBitIndex := upBitIndex + pixelInByte.
					1 to: pixelInByte
						do: 
							[:k | 
							upBits at: upBitIndex - k + 1 put: (val bitAnd: bitMask).
							val := val bitShift: depth negated]].
			val := (bits at: (bitIndex := bitIndex + 1))
						bitShift: depth negated * (pixelInByte - trailingSize).
			upBitIndex := upBitIndex + trailingSize.
			1 to: trailingSize
				do: 
					[:k | 
					upBits at: upBitIndex - k + 1 put: (val bitAnd: bitMask).
					val := val bitShift: depth negated]].
	^upBits! !

!GifImageStream methodsFor: 'private' stamp: ' 21/4/07 22:07'!
updatePixelPosition
	(xpos := xpos + 1) >= width ifFalse: [^self].
	xpos := 0.
	interlace
		ifFalse: 
			[ypos := ypos + 1.
			^self].
	pass = 0
		ifTrue: 
			[(ypos := ypos + 8) >= height
				ifTrue: 
					[pass := pass + 1.
					ypos := 4].
			^self].
	pass = 1
		ifTrue: 
			[(ypos := ypos + 8) >= height
				ifTrue: 
					[pass := pass + 1.
					ypos := 2].
			^self].
	pass = 2
		ifTrue: 
			[(ypos := ypos + 4) >= height
				ifTrue: 
					[pass := pass + 1.
					ypos := 1].
			^self].
	pass = 3
		ifTrue: 
			[ypos := ypos + 2.
			^self].
	^self error: 'can''t happen'! !

!GifImageStream methodsFor: 'encoding' stamp: ' 21/4/07 22:07'!
writeBitData: bits 
	| maxBits maxMaxCode tSize initCodeSize ent tShift fCode pixel set |

	pass := 0.
	xpos := 0.
	ypos := 0.
	rowByteSize := width * 8 + 31 // 32 * 4.
	remainBitCount := 0.
	bufByte := 0.
	bufStream := WriteStream on: (ByteArray new: 256).
	maxBits := 12.
	maxMaxCode := 1 bitShift: maxBits.
	tSize := 5003.
	prefixTable := Array new: tSize.
	suffixTable := Array new: tSize.
	initCodeSize := bitsPerPixel <= 1
				ifTrue: [2]
				ifFalse: [bitsPerPixel].
	self nextPut: initCodeSize.
	self setParameters: initCodeSize.
	tShift := 0.
	fCode := tSize.
	[fCode < 65536]
		whileTrue: 
			[tShift := tShift + 1.
			fCode := fCode * 2].
	tShift := 8 - tShift.
	1 to: tSize do: [:i | suffixTable at: i put: -1].

"	set := Set new: height.
	self progress: set size / height.
	set add: ypos. 
	self progress: set size / height."

	self writeCodeAndCheckCodeSize: clearCode.
	ent := self readPixelFrom: bits.
	[(pixel := self readPixelFrom: bits) == nil]
		whileFalse: 
			[| index disp nomatch |

"			set add: ypos. 
			self progress: set size / height."

			fCode := (pixel bitShift: maxBits)
						+ ent.
			index := ((pixel bitShift: tShift)
						bitXor: ent)
						+ 1.
			(suffixTable at: index)
				= fCode
				ifTrue: [ent := prefixTable at: index]
				ifFalse: 
					[nomatch := true.
					(suffixTable at: index)
						>= 0
						ifTrue: 
							[disp := tSize - index + 1.
							index = 1 ifTrue: [disp := 1].
							
							[(index := index - disp) < 1 ifTrue: [index := index + tSize].
							(suffixTable at: index)
								= fCode
								ifTrue: 
									[ent := prefixTable at: index.
									nomatch := false].
							nomatch and: [(suffixTable at: index)
									> 0]] whileTrue].
					nomatch
						ifTrue: 
							[self writeCodeAndCheckCodeSize: ent.
							ent := pixel.
							freeCode < maxMaxCode
								ifTrue: 
									[prefixTable at: index put: freeCode.
									suffixTable at: index put: fCode.
									freeCode := freeCode + 1]
								ifFalse: 
									[self writeCodeAndCheckCodeSize: clearCode.
									1 to: tSize do: [:i | suffixTable at: i put: -1].
									self setParameters: initCodeSize]]].
"			set add: ypos. 
			self progress: set size / height"].

	prefixTable := suffixTable := nil.
	self writeCodeAndCheckCodeSize: ent.
	self writeCodeAndCheckCodeSize: eoiCode.
	self flushCode.
	self nextPut: 0.
	self nextPut: Terminator! !

!GifImageStream methodsFor: 'encoding' stamp: ' 21/4/07 22:07'!
writeCode: aCode
	self nextBitsPut: aCode! !

!GifImageStream methodsFor: 'encoding' stamp: ' 21/4/07 22:07'!
writeCodeAndCheckCodeSize: aCode
	self writeCode: aCode.
	self checkCodeSize! !

!GifImageStream methodsFor: 'GIF89a' stamp: ' 21/4/07 22:07'!
writeGraphicControlExtensionBlock
	self nextPut: Extension.
	self nextPut: GraphicControlLabel.
	self nextPut: 4.
	transparentPixel notNil
		ifTrue: [self nextPut: 1]
		ifFalse: [self nextPut: 0].
	self nextPut: 0.
	self nextPut: 0.
	transparentPixel notNil
		ifTrue: [self nextPut: transparentPixel]
		ifFalse: [self nextPut: 0].
	self nextPut: 0! !

!GifImageStream methodsFor: 'encoding' stamp: ' 21/4/07 22:07'!
writeHeader
	| byte array |
	self nextPutAll: 'GIF89a' asByteArray.
	self writeWord: width.
	self writeWord: height.
	byte := 128.
	byte := byte bitOr: (bitsPerPixel - 1 bitShift: 5).
	byte := byte bitOr: bitsPerPixel - 1.
	self nextPut: byte.
	self nextPut: 0.
	self nextPut: 0.
	array := self rgbIntegerArrayFor: colorPalette.
	array do: [:rgb | self nextPut: ((rgb bitShift: -16)
				bitAnd: 255); nextPut: ((rgb bitShift: -8)
				bitAnd: 255); nextPut: (rgb bitAnd: 255)].
	array size + 1 to: (1 bitShift: bitsPerPixel)
		do: [:i | self nextPut: 0; nextPut: 0; nextPut: 0].
	transparentPixel notNil ifTrue: [self writeGraphicControlExtensionBlock].
	self nextPut: ImageSeparator.
	self writeWord: 0.
	self writeWord: 0.
	self writeWord: width.
	self writeWord: height.
	byte := interlace
				ifTrue: [64]
				ifFalse: [0].
	self nextPut: byte! !

!GifImageStream methodsFor: 'decoding' stamp: ' 21/4/07 22:07'!
writePixel: pixel to: bits 
	bits at: ypos * rowByteSize + xpos + 1 put: pixel.
	self updatePixelPosition! !

!GifImageStream methodsFor: 'encoding' stamp: ' 21/4/07 22:07'!
writeWord: aWord 
	self nextPut: (aWord bitAnd: 255).
	self nextPut: ((aWord bitShift: -8)
			bitAnd: 255).
	^aWord! !

!ImageStream class methodsFor: 'controls' stamp: ' 21/4/07 22:07'!
assert: assertBlock do: doBlock ensure: ensureBlock 
	assertBlock value.
	[doBlock value]
		valueNowOrOnUnwindDo: [ensureBlock value]! !

!ImageStream class methodsFor: 'constants of palette' stamp: ' 21/8/07 09:38'!
colorPalette256
	"ImageStream colorPalette256."

	| anArray |
	ColorPalette256 notNil ifTrue: [^ColorPalette256].
	anArray := self colorPalette256Array
				collect: 
					[:array | 
					| r g b color |
					r := array at: 1.
					g := array at: 2.
					b := array at: 3.
					ColorValue scalingValue = 8191
						ifTrue: [color := ColorValue
										scaledRed: r
										scaledGreen: g
										scaledBlue: b]
						ifFalse: 
							[r := r / 8191.
							g := g / 8191.
							b := b / 8191.
							color := ColorValue
										red: r
										green: g
										blue: b].
					color yourself].
	ColorPalette256 := MappedPalette withColors: anArray reverse.
	^ColorPalette256! !

!ImageStream class methodsFor: 'constants of palette' stamp: ' 21/8/07 09:38'!
colorPalette256Array
	"ImageStream colorPalette256Array."
	^#(	#(8191 8191 8191)
		#(8191 8191 6553)
		#(8191 8191 4915)
		#(8191 8191 3276)
		#(8191 8191 1638)
		#(8191 8191 0)
		#(8191 6553 8191)
		#(8191 6553 6553)
		#(8191 6553 4915)
		#(8191 6553 3276)
		#(8191 6553 1638)
		#(8191 6553 0)
		#(8191 4915 8191)
		#(8191 4915 6553)
		#(8191 4915 4915)
		#(8191 4915 3276)
		#(8191 4915 1638)
		#(8191 4915 0)
		#(8191 3276 8191)
		#(8191 3276 6553)
		#(8191 3276 4915)
		#(8191 3276 3276)
		#(8191 3276 1638)
		#(8191 3276 0)
		#(8191 1638 8191)
		#(8191 1638 6553)
		#(8191 1638 4915)
		#(8191 1638 3276)
		#(8191 1638 1638)
		#(8191 1638 0)
		#(8191 0 8191)
		#(8191 0 6553)
		#(8191 0 4915)
		#(8191 0 3276)
		#(8191 0 1638)
		#(8191 0 0)
		#(6553 8191 8191)
		#(6553 8191 6553)
		#(6553 8191 4915)
		#(6553 8191 3276)
		#(6553 8191 1638)
		#(6553 8191 0)
		#(6553 6553 8191)
		#(6553 6553 6553)
		#(6553 6553 4915)
		#(6553 6553 3276)
		#(6553 6553 1638)
		#(6553 6553 0)
		#(6553 4915 8191)
		#(6553 4915 6553)
		#(6553 4915 4915)
		#(6553 4915 3276)
		#(6553 4915 1638)
		#(6553 4915 0)
		#(6553 3276 8191)
		#(6553 3276 6553)
		#(6553 3276 4915)
		#(6553 3276 3276)
		#(6553 3276 1638)
		#(6553 3276 0)
		#(6553 1638 8191)
		#(6553 1638 6553)
		#(6553 1638 4915)
		#(6553 1638 3276)
		#(6553 1638 1638)
		#(6553 1638 0)
		#(6553 0 8191)
		#(6553 0 6553)
		#(6553 0 4915)
		#(6553 0 3276)
		#(6553 0 1638)
		#(6553 0 0)
		#(4915 8191 8191)
		#(4915 8191 6553)
		#(4915 8191 4915)
		#(4915 8191 3276)
		#(4915 8191 1638)
		#(4915 8191 0)
		#(4915 6553 8191)
		#(4915 6553 6553)
		#(4915 6553 4915)
		#(4915 6553 3276)
		#(4915 6553 1638)
		#(4915 6553 0)
		#(4915 4915 8191)
		#(4915 4915 6553)
		#(4915 4915 4915)
		#(4915 4915 3276)
		#(4915 4915 1638)
		#(4915 4915 0)
		#(4915 3276 8191)
		#(4915 3276 6553)
		#(4915 3276 4915)
		#(4915 3276 3276)
		#(4915 3276 1638)
		#(4915 3276 0)
		#(4915 1638 8191)
		#(4915 1638 6553)
		#(4915 1638 4915)
		#(4915 1638 3276)
		#(4915 1638 1638)
		#(4915 1638 0)
		#(4915 0 8191)
		#(4915 0 6553)
		#(4915 0 4915)
		#(4915 0 3276)
		#(4915 0 1638)
		#(4915 0 0)
		#(3276 8191 8191)
		#(3276 8191 6553)
		#(3276 8191 4915)
		#(3276 8191 3276)
		#(3276 8191 1638)
		#(3276 8191 0)
		#(3276 6553 8191)
		#(3276 6553 6553)
		#(3276 6553 4915)
		#(3276 6553 3276)
		#(3276 6553 1638)
		#(3276 6553 0)
		#(3276 4915 8191)
		#(3276 4915 6553)
		#(3276 4915 4915)
		#(3276 4915 3276)
		#(3276 4915 1638)
		#(3276 4915 0)
		#(3276 3276 8191)
		#(3276 3276 6553)
		#(3276 3276 4915)
		#(3276 3276 3276)
		#(3276 3276 1638)
		#(3276 3276 0)
		#(3276 1638 8191)
		#(3276 1638 6553)
		#(3276 1638 4915)
		#(3276 1638 3276)
		#(3276 1638 1638)
		#(3276 1638 0)
		#(3276 0 8191)
		#(3276 0 6553)
		#(3276 0 4915)
		#(3276 0 3276)
		#(3276 0 1638)
		#(3276 0 0)
		#(1638 8191 8191)
		#(1638 8191 6553)
		#(1638 8191 4915)
		#(1638 8191 3276)
		#(1638 8191 1638)
		#(1638 8191 0)
		#(1638 6553 8191)
		#(1638 6553 6553)
		#(1638 6553 4915)
		#(1638 6553 3276)
		#(1638 6553 1638)
		#(1638 6553 0)
		#(1638 4915 8191)
		#(1638 4915 6553)
		#(1638 4915 4915)
		#(1638 4915 3276)
		#(1638 4915 1638)
		#(1638 4915 0)
		#(1638 3276 8191)
		#(1638 3276 6553)
		#(1638 3276 4915)
		#(1638 3276 3276)
		#(1638 3276 1638)
		#(1638 3276 0)
		#(1638 1638 8191)
		#(1638 1638 6553)
		#(1638 1638 4915)
		#(1638 1638 3276)
		#(1638 1638 1638)
		#(1638 1638 0)
		#(1638 0 8191)
		#(1638 0 6553)
		#(1638 0 4915)
		#(1638 0 3276)
		#(1638 0 1638)
		#(1638 0 0)
		#(0 8191 8191)
		#(0 8191 6553)
		#(0 8191 4915)
		#(0 8191 3276)
		#(0 8191 1638)
		#(0 8191 0)
		#(0 6553 8191)
		#(0 6553 6553)
		#(0 6553 4915)
		#(0 6553 3276)
		#(0 6553 1638)
		#(0 6553 0)
		#(0 4915 8191)
		#(0 4915 6553)
		#(0 4915 4915)
		#(0 4915 3276)
		#(0 4915 1638)
		#(0 4915 0)
		#(0 3276 8191)
		#(0 3276 6553)
		#(0 3276 4915)
		#(0 3276 3276)
		#(0 3276 1638)
		#(0 3276 0)
		#(0 1638 8191)
		#(0 1638 6553)
		#(0 1638 4915)
		#(0 1638 3276)
		#(0 1638 1638)
		#(0 1638 0)
		#(0 0 8191)
		#(0 0 6553)
		#(0 0 4915)
		#(0 0 3276)
		#(0 0 1638)
		#(7645 0 0)
		#(7099 0 0)
		#(6007 0 0)
		#(5461 0 0)
		#(4369 0 0)
		#(3822 0 0)
		#(2730 0 0)
		#(2184 0 0)
		#(1092 0 0)
		#(546 0 0)
		#(0 7645 0)
		#(0 7099 0)
		#(0 6007 0)
		#(0 5461 0)
		#(0 4369 0)
		#(0 3822 0)
		#(0 2730 0)
		#(0 2184 0)
		#(0 1092 0)
		#(0 546 0)
		#(0 0 7645)
		#(0 0 7099)
		#(0 0 6007)
		#(0 0 5461)
		#(0 0 4369)
		#(0 0 3822)
		#(0 0 2730)
		#(0 0 2184)
		#(0 0 1092)
		#(0 0 546)
		#(7645 7645 7645)
		#(7099 7099 7099)
		#(6007 6007 6007)
		#(5461 5461 5461)
		#(4369 4369 4369)
		#(3822 3822 3822)
		#(2730 2730 2730)
		#(2184 2184 2184)
		#(1092 1092 1092)
		#(546 546 546)
		#(0 0 0)
	)! !

!ImageStream class methodsFor: 'copyright' stamp: ' 21/4/07 22:07'!
copyright
	^'Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.'! !

!ImageStream class methodsFor: 'constants of palette' stamp: ' 21/4/07 22:07'!
flushPalettes
	"ImageStream flushPalettes."

	ColorPalette256 := nil.
	GrayPalette256 := nil! !

!ImageStream class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
fromClipboard
	"ImageStream show: ImageStream fromClipboard."

	| pixmap image |
	pixmap := Pixmap fromClipboard.
	image := pixmap asImage.
	^image! !

!ImageStream class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
fromDisplay
	"ImageStream show: ImageStream fromDisplay."

	^self fromDisplay: Screen default bounds! !

!ImageStream class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
fromDisplay: aRectangle 
	"ImageStream show: (ImageStream fromDisplay: (0 @ 0 extent: 100 @ 100))."

	^Screen default completeContentsOfArea: aRectangle! !

!ImageStream class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
fromUser
	"ImageStream show: ImageStream fromUser."

	^self fromDisplay: Rectangle fromUser! !

!ImageStream class methodsFor: 'constants of palette' stamp: ' 21/4/07 22:07'!
grayPalette256
	"ImageStream grayPalette256."

	| paletteSize anArray |
	GrayPalette256 notNil ifTrue: [^GrayPalette256].
	paletteSize := 256.
	anArray := Array new: paletteSize.
	1 to: paletteSize
		do: 
			[:index | 
			| color |
			color := ColorValue brightness: 1 - (index - 1 / (paletteSize - 1)).
			anArray at: index put: color].
	GrayPalette256 := MappedPalette withColors: anArray reverse.
	^GrayPalette256! !

!ImageStream class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
imageKindTable
	"ImageStream imageKindTable."

	ImageKindTable isNil
		ifTrue: 
			[| aDictionary |
			aDictionary := Dictionary new.
			aDictionary at: 'bos' put: #BosImageStream.
			aDictionary at: 'boss' put: #BosImageStream.
			aDictionary at: 'bmp' put: #BmpImageStream.
			aDictionary at: 'bmpf' put: #BmpImageStream.
			aDictionary at: 'gif' put: #GifImageStream.
			aDictionary at: 'giff' put: #GifImageStream.
			aDictionary at: 'jpg' put: #JpegImageStream.
			aDictionary at: 'jpeg' put: #JpegImageStream.
			aDictionary at: 'pic' put: #PictImageStream.
			aDictionary at: 'pict' put: #PictImageStream.
			ImageKindTable := aDictionary].
	^ImageKindTable! !

!ImageStream class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
imageStreamClassForFileName: aFilename 
	"ImageStream imageStreamClassForFileName: 'zzz.gif' asFilename."

	| aString aSymbol aClass |
	aString := (aFilename asString reverse copyUpTo: $.) reverse.
	aString := aString asLowercase.
	aSymbol := self imageKindTable at: aString ifAbsent: [nil].
	aSymbol isNil ifTrue: [^nil].
	aClass := Smalltalk at: aSymbol ifAbsent: [nil].
	^aClass! !

!ImageStream class methodsFor: 'class initialization' stamp: ' 21/4/07 22:07'!
initialize
	"ImageStream initialize."

	ImageKindTable := nil.
	self flushPalettes! !

!ImageStream class methodsFor: 'class initialization' stamp: ' 21/4/07 22:07'!
install
	"ImageStream install."

	self colorPalette256.
	self grayPalette256! !

!ImageStream class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
on: aStream 
	^self new on: aStream! !

!ImageStream class methodsFor: 'saving' stamp: ' 21/4/07 22:07'!
save
	"ImageStream save."

	| encodingName fileName classCollection aStream |
	encodingName := #default.
	fileName := 'ImgStrm.st'.
	classCollection := self saveClasses.
	aStream := (fileName asFilename withEncoding: encodingName) writeStream.
	[Cursor write
		showWhile: 
			[| timeStamp |
			timeStamp := Date today shortPrintString , ' ' , Time now shortPrintString.
			aStream cr.
			aStream nextChunkPut: timeStamp printString.
			aStream cr; cr.
			(self comment isNil or: [self comment isEmpty])
				ifFalse: 
					[aStream nextChunkPut: self comment printString.
					aStream cr; cr].
			classCollection
				do: 
					[:aClass | 
					aStream nextChunkPut: aClass definition.
					aStream cr; cr].
			classCollection
				do: 
					[:aClass | 
					| sourceCodeStream |
					aStream nextPut: Character newPage.
					aStream cr.
					sourceCodeStream := SourceCodeStream on: aStream.
					aClass fileOutSourceOn: sourceCodeStream.
					aStream cr]]]
		valueNowOrOnUnwindDo: [aStream close].
	^classCollection! !

!ImageStream class methodsFor: 'saving' stamp: ' 21/4/07 22:07'!
saveClasses
	"ImageStream saveClasses."

	| patternCollection classCollection |
	patternCollection := #('*ImageStream*' '*Progress*').
	classCollection := Smalltalk organization superclassOrder: self category.
	classCollection := classCollection
				select: 
					[:aClass | 
					| string something |
					string := aClass name asString.
					something := patternCollection detect: [:it | it match: string]
								ifNone: [nil].
					something notNil].
	^classCollection , Progress saveClasses! !

!ImageStream class methodsFor: 'viewing' stamp: ' 21/4/07 22:07'!
show: anImage 
	"ImageStream show: Image fromUser."

	| image extent window |
	(anImage isKindOf: OpaqueImage)
		ifTrue: [image := anImage]
		ifFalse: [image := CachedImage on: anImage].
	extent := 64 @ 64 max: (image extent min: Screen default bounds extent - 60 asPoint).
	window := ScheduledWindow new.
	window label: 'Image'.
	window minimumSize: extent.
	window component: image.
	window open.
	^anImage! !

!ImageStream class methodsFor: 'copyright' stamp: ' 21/4/07 22:07'!
system
	^'Goodies'! !

!ImageStream class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
toClipboard: anImage 
	"ImageStream toClipboard: Image fromUser."

	| pixmap |
	pixmap := anImage asRetainedMedium.
	pixmap toClipboard.
	^anImage! !

!ImageStream class methodsFor: 'copyright' stamp: ' 21/4/07 22:07'!
version
	^'003'! !

!ImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
atEnd
	^imageStream atEnd! !

!ImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
close
	imageStream close! !

!ImageStream methodsFor: 'progress' stamp: ' 21/4/07 22:07'!
compute: aBlock 
	(self progressValue isKindOf: ValueHolder)
		ifTrue: [self progressValue compute: aBlock]! !

!ImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
contents
	^imageStream contents! !

!ImageStream methodsFor: 'private' stamp: ' 21/4/07 22:07'!
convertValue: value from: fromScale to: toScale 
	^value = 0
		ifTrue: [0]
		ifFalse: [value = fromScale
				ifTrue: [toScale]
				ifFalse: [(value + 1 * (toScale + 1) / (fromScale + 1)) rounded - 1 max: 0]]! !

!ImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
cr
	^imageStream nextPut: Character cr asInteger! !

!ImageStream methodsFor: 'private' stamp: ' 21/4/07 22:07'!
errorCanNotRead
	self error: 'can''t read the image'.
	^nil! !

!ImageStream methodsFor: 'private' stamp: ' 21/4/07 22:07'!
errorCanNotWrite
	self error: 'can''t write the image'.
	^nil! !

!ImageStream methodsFor: 'private' stamp: ' 21/4/07 22:07'!
hasMagicNumber: aByteArray 
	| position array |
	position := imageStream position.
	imageStream size - position >= aByteArray size
		ifTrue: 
			[array := (imageStream next: aByteArray size) asByteArray.
			array = aByteArray ifTrue: [^true]].
	imageStream position: position.
	^false! !

!ImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
lf
	^imageStream nextPut: Character lf asInteger! !

!ImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
next
	^imageStream next! !

!ImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
next: size 
	^imageStream next: size! !

!ImageStream methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
nextImage
	^self subclassResponsibility! !

!ImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
nextLong
	^(imageStream next bitShift: 24)
		+ (imageStream next bitShift: 16) + (imageStream next bitShift: 8) + imageStream next! !

!ImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
nextLongPut: a32BitW 
	imageStream nextPut: ((a32BitW bitShift: -24)
			bitAnd: 255).
	imageStream nextPut: ((a32BitW bitShift: -16)
			bitAnd: 255).
	imageStream nextPut: ((a32BitW bitShift: -8)
			bitAnd: 255).
	imageStream nextPut: (a32BitW bitAnd: 255).
	^a32BitW! !

!ImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
nextPut: aByte 
	^imageStream nextPut: aByte! !

!ImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
nextPutAll: aByteArray 
	^imageStream nextPutAll: aByteArray! !

!ImageStream methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
nextPutImage: anImage 
	^self subclassResponsibility! !

!ImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
nextWord
	^(imageStream next bitShift: 8)
		+ imageStream next! !

!ImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
nextWordPut: a16BitW 
	imageStream nextPut: ((a16BitW bitShift: -8)
			bitAnd: 255).
	imageStream nextPut: (a16BitW bitAnd: 255).
	^a16BitW! !

!ImageStream methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
on: aStream 
	imageStream := aStream.
	(imageStream respondsTo: #binary)
		ifTrue: [imageStream binary].
	self progressValue: (ValueHolder with: nil)! !

!ImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
position
	^imageStream position! !

!ImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
position: anInteger 
	^imageStream position: anInteger! !

!ImageStream methodsFor: 'progress' stamp: ' 21/4/07 22:07'!
progress
	^self progressValue value! !

!ImageStream methodsFor: 'progress' stamp: ' 21/4/07 22:07'!
progress: normalizedNumber 
	(0 <= normalizedNumber and: [normalizedNumber <= 1])
		ifTrue: 
			[| truncatedValue |
			truncatedValue := normalizedNumber roundTo: 0.005.
			self progressValue value = truncatedValue ifFalse: [self progressValue value: truncatedValue]]! !

!ImageStream methodsFor: 'progress' stamp: ' 21/4/07 22:07'!
progressValue
	^progressValue! !

!ImageStream methodsFor: 'progress' stamp: ' 21/4/07 22:07'!
progressValue: aValueHolder 
	progressValue := aValueHolder! !

!ImageStream methodsFor: 'viewing' stamp: ' 21/4/07 22:07'!
show: anImage 
	^self class show: anImage! !

!ImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
size
	^imageStream size! !

!ImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
skip: anInteger 
	^imageStream skip: anInteger! !

!ImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
space
	^imageStream nextPut: Character space asInteger! !

!ImageStream methodsFor: 'stream access' stamp: ' 21/4/07 22:07'!
tab
	^imageStream nextPut: Character tab asInteger! !

Object subclass: #MIMEMap
	instanceVariableNames: 'mimeTypes fileExtensions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Core'!

!MIMEMap class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
apacheMimeTypes
	"from /etc/httpd/mime.types after Apache v1.3 is installed"

^'application/EDI-Consent
application/EDI-X12
application/EDIFACT
application/activemessage
application/andrew-inset	ez
application/applefile
application/atomicmail
application/cals-1840
application/commonground
application/cybercash
application/cu-seeme		csm cu
application/dca-rft
application/dec-dx
application/eshop
application/excel		xls
application/ghostview
application/hyperstudio
application/iges
application/mac-binhex40	hqx
application/mac-compactpro	cpt
application/macwriteii
application/marc
application/mathematica
application/msword		doc dot wrd
application/news-message-id
application/news-transmission
application/octet-stream	bin dms lha lzh exe class
application/oda			oda
application/pdf			pdf
application/pgp			pgp
application/pgp-encrypted
application/pgp-keys
application/pgp-signature	pgp
application/postscript		ai eps ps
application/powerpoint		ppt
application/remote-printing
application/rtf			rtf
application/slate
application/wita
application/wordperfect5.1	wp5
application/x-123		wk
application/x-Wingz		wz
application/x-bcpio		bcpio
application/x-bzip2		bz2
application/x-cdlink		vcd
application/x-chess-pgn		pgn
application/x-compress		z Z
application/x-cpio		cpio
application/x-csh		csh
application/x-debian-package	deb
application/x-director		dcr dir dxr
application/x-dvi		dvi
application/x-gtar		gtar tgz
application/x-gunzip		gz
application/x-gzip		gz
application/x-hdf		hdf
application/x-httpd-php		phtml pht php
application/x-javascript	js
application/x-kword		kwd kwt
application/x-kspread		ksp
application/x-kpresenter	kpr kpt
application/x-kchart		chrt
application/x-koan		skp skd skt skm
application/x-latex		latex
application/x-maker		frm maker frame fm fb book fbdoc
application/x-mif		mif
application/x-msdos-program	com exe bat
application/x-netcdf		nc cdf
application/x-ns-proxy-autoconfig	pac
application/x-perl		pl pm
application/x-rad		rad
application/x-rpm		rpm spm
application/x-sh		sh
application/x-shar		shar
application/x-shockwave-flash	swf
application/x-stuffit		sit
application/x-sv4cpio		sv4cpio
application/x-sv4crc		sv4crc
application/x-tar		tar
application/x-tcl		tcl
application/x-tex		tex
application/x-texinfo		texinfo texi
application/x-troff		t tr roff
application/x-troff-man		man
application/x-troff-me		me
application/x-troff-ms		ms
application/x-ustar		ustar
application/x-wais-source	src
application/zip			zip
audio/basic			au snd
audio/midi			mid midi kar
audio/mpeg			mpga mp2 mp3
audio/x-aiff			aif aifc aiff
audio/x-realaudio		ra
audio/x-wav			wav
chemical/x-pdb			pdb xyz
image/gif			gif
image/ief			ief
image/jpeg			jpeg jpg jpe
image/png			png
image/tiff			tiff tif
image/x-cmu-raster		ras
image/x-portable-anymap		pnm
image/x-portable-bitmap		pbm
image/x-portable-graymap	pgm
image/x-portable-pixmap		ppm
image/x-rgb			rgb
image/x-xbitmap			xbm
image/x-xpixmap			xpm
image/x-xwindowdump		xwd
message/external-body
message/news
message/partial
message/rfc822
model/iges			igs iges
model/mesh			msh mesh silo
model/vrml			wrl vrml
multipart/alternative
multipart/appledouble
multipart/digest
multipart/mixed
multipart/parallel
text/css			css
text/html			html htm
text/plain			asc txt c cc h hh cpp hpp
text/richtext			rtx
text/rtf			rtf
text/sgml			sgml sgm
text/tab-separated-values	tsv
text/x-setext			etx
text/x-vCalendar		vcs
text/x-vCard			vcf
text/xml			xml dtd
video/dl			dl
video/fli			fli
video/gl			gl
video/mpeg			mp2 mpe mpeg mpg
video/quicktime			qt mov
video/x-msvideo			avi
video/x-sgi-movie		movie
x-conference/x-cooltalk		ice
x-world/x-vrml			wrl vrml
audio/x-pn-realaudio rmm ram
audio/vnd.rn-realaudio ra
application/smil smi smil
text/vnd.rn-realtext rt
video/vnd.rn-realvideo rv
image/vnd.rn-realflash rf swf
application/x-shockwave-flash2-preview rf swf
application/sdp sdp
application/x-sdp sdp
application/vnd.rn-realmedia rm
image/vnd.rn-realpix rp
'! !

!MIMEMap class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
arrayFromApacheMimeTypes
	| collection stream |
	collection:= OrderedCollection new.
	stream := ReadStream on: self apacheMimeTypes.
	[stream atEnd] whileFalse:
		[ | col line word |
		col := OrderedCollection new.
		line := (stream upTo: Character cr) readStream.
		[line atEnd] whileFalse: [word := line upToSeparator. word notEmpty ifTrue: [col add: word] ].
		collection add: col].
	^collection

"MIMEMap arrayFromApacheMimeTypes"! !

!MIMEMap class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
default
	^AIDASite default mimeMap! !

!MIMEMap class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new initialize! !

!MIMEMap methodsFor: 'adding-removing' stamp: ' 21/4/07 22:07'!
addType: aTypeString andExtension: anExtensionString
	| exts |
	exts := self mimeTypes at: aTypeString ifAbsentPut: [OrderedCollection new].
	(exts includes: anExtensionString) ifFalse: [exts add: anExtensionString].
	self fileExtensions at: anExtensionString ifAbsentPut: [aTypeString].! !

!MIMEMap methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
extensionForType: aString
	"only first one if more than one ext exist for this mime type"
	| exts string |
	string := aString = 'image/pjpeg' ifTrue: ['image/jpeg'] ifFalse: [aString]. "wierd IE problem"
	string := aString = 'image/x-png' ifTrue: ['image/png'] ifFalse: [string]. "wierd IE problem"
	exts := self mimeTypes at: string asLowercase ifAbsent: [#()].
	^exts notEmpty ifTrue: [exts first] ifFalse: [nil]

"MIMEMap new extensionForType: 'image/jpeg'"! !

!MIMEMap methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
extensionsForType: aString
	^self mimeTypes at: aString asLowercase ifAbsent: [#()].

"MIMEMap new extensionsForType: 'image/jpeg'"! !

!MIMEMap methodsFor: 'private' stamp: ' 21/4/07 22:07'!
fileExtensions
	fileExtensions isNil ifTrue: [self initFileExtensions].
	^fileExtensions! !

!MIMEMap methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
iconForType: aString
	"a name of icon to represent this content type, from WebStyle imgs-filetype icons"
	aString isNil ifTrue: [^#unknownSmallPng].
	'text/plain' = aString ifTrue: [^#txtSmallPng].
	'text/xml' = aString ifTrue: [^#xmlSmallPng].
	'application/pdf' = aString ifTrue: [^#pdfSmallPng].
	'application/msword' = aString ifTrue: [^#wordSmallPng].
	'application/rtf' = aString ifTrue: [^#wordSmallPng].
	'application/excel' = aString ifTrue: [^#excelSmallPng].
	'application/powerpoint' = aString ifTrue: [^#powerpointSmallPng].
	'application/zip' = aString ifTrue: [^#zipSmallPng].
	('image/*' match: aString) ifTrue: [^#imageSmallPng].
	('audio/*' match: aString) ifTrue: [^#multimediaSmallPng].
	('video/*' match: aString) ifTrue: [^#multimediaSmallPng].
	^#unknownSmallPng

"MIMEMap new iconForType: 'image/jpeg'"! !

!MIMEMap methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initFileExtensions
	fileExtensions := Dictionary new.! !

!MIMEMap methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initFromApacheMimeTypes
	"see class method. Only mime types, which have some extension defined, are set!! "
	| collection |
	collection := self class arrayFromApacheMimeTypes.
	collection do: [:col | 
		2 to: col size do: [:inx | self addType: col first andExtension: (col at: inx)] ]

"MIMEMap new"! !

!MIMEMap methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initMimeTypes
	mimeTypes := Dictionary new.! !

!MIMEMap methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	self initFromApacheMimeTypes! !

!MIMEMap methodsFor: 'private' stamp: ' 21/4/07 22:07'!
mimeTypes
	mimeTypes isNil ifTrue: [self initMimeTypes].
	^mimeTypes! !

!MIMEMap methodsFor: 'adding-removing' stamp: ' 21/4/07 22:07'!
removeType: aTypeString andExtension: anExtensionString
	| exts |
	exts := self mimeTypes at: aTypeString ifAbsent: [OrderedCollection new].
	(exts includes: anExtensionString) ifTrue: [exts remove: anExtensionString].
	exts isEmpty ifTrue: [self mimeTypes removeKey: aTypeString ifAbsent: [] ].
	self fileExtensions removeKey: anExtensionString ifAbsent: [].! !

!MIMEMap methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
typeForExtension: aString
	| ext |
	ext := aString asLowercase copyWithout: $. .
	^self fileExtensions at: ext ifAbsent: [nil].

"MIMEMap new typeForExtension: 'jpg'"! !

Object subclass: #Numberer
	instanceVariableNames: 'counters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Support'!

!Numberer commentStamp: '<historical>' prior: 0!
Numberer is used to provide counters for things like invoice numbers

Instance Variables:
	counters	<Dictionary>	 counters for different purposes

!

!Numberer methodsFor: 'private' stamp: ' 21/4/07 22:07'!
counters
	counters isNil ifTrue: [self initCounters].
	^counters! !

!Numberer methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
currentCounter: aSymbol
	"get curent (last nextCounter) number"
	(self counters includesKey: aSymbol) ifFalse: [self resetCounter: aSymbol].
	^self counters at: aSymbol! !

!Numberer methodsFor: 'private' stamp: ' 21/4/07 22:07'!
decrementCounter: aSymbol
	"get next number and increment this counter"
	^self counters at: aSymbol put: (((self currentCounter: aSymbol) - 1) max: 0)! !

!Numberer methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initCounters
	counters := Dictionary new! !

!Numberer methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
nextCounter: aSymbol
	"get next number and increment this counter"
	^self counters at: aSymbol put: (self peekCounter: aSymbol)! !

!Numberer methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
peekCounter: aSymbol
	"get next number but not increment it"
	(self counters includesKey: aSymbol) ifFalse: [self resetCounter: aSymbol].
	^(self counters at: aSymbol) + 1! !

!Numberer methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
resetCounter: aSymbol
	"put counter to 0, peekCounter will then return 1"
	self counters at: aSymbol put: 0! !

!Numberer methodsFor: 'private' stamp: ' 21/4/07 22:07'!
setCounter: aSymbol to: aNumber
	"use it sparingly!!"
	^self counters at: aSymbol put: aNumber! !

!Object methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:23'!
aidaCanBeLocked
	"can we get an exclusive lock on that object (not already locked)?"
	^false! !

!Object methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:23'!
aidaDontCache
	"don't cache web content in a browser. Appropriate header is added to http response"
	^false! !

!Object methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:24'!
aidaIsLocked
	"is object locked exclusively?"
	^false! !

!Object methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:24'!
aidaLock
	"get an exclusive lock on that object. Until unlocked, noon else can get that lock. Return false if already locked, true if successfull"
	^false! !

!Object methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:24'!
aidaUnlock
	"release an exclusive lock if any"
	^true! !

!Object methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:24'!
app
	"fastest and most convinient way to find a web app for that object"
	^self webAppFor: self firstSessionFromStack! !

!Object methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:25'!
contentType

	"Janko Mivsek, apr98"
	"return 'text/html' as content type for web pages"

	^'text/html'! !

!Object methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:25'!
deepSearchOfClass: aClassName
	"finf all objects of that class down in object hierarchy"
	| objectDictionary class |
	objectDictionary := IdentityDictionary new.
	self deepCopyNotIn: objectDictionary.
	class := aClassName asSymbol.
	^objectDictionary keys select: [:each | each class name = class].! !

!Object methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:26'!
deepSearchOfObsoleteClasses
	"find all objects of obsolete classes down in object hierarchy"
	| objectDictionary |
	objectDictionary := IdentityDictionary new.
	self deepCopyNotIn: objectDictionary.
	^objectDictionary keys select: [:each | each class isObsolete].! !

!Object methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:26'!
expiresTimestamp
	"until when content of this object wont be changed"
	"used in http response, override if you like to be included"
	^self modifiedTimestamp  "to reload pages immediately"! !

!Object methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:26'!
firstAppFromStack
	"try to find a first sender up in calling stack, who is  WebApplication"
	| context |
	context := thisContext.
	[context notNil] whileTrue: [
		(context receiver isKindOf: WebApplication) ifTrue: [^context receiver].
		context := context sender].
	^self firstSessionFromStack lastApp! !

!Object methodsFor: '*Aida' stamp: 'JM 4/25/2007 21:34'!
firstSessionFromStack
	"try to find a first sender up in calling stack, who is  AIDASite and get session if that call"
	| context |
	context := thisContext.
	[context notNil] whileTrue: [
		(context receiver isKindOf: AIDASite) ifTrue: 	[^(context at: 3) "always?"].
		context := context sender].
	^nil! !

!Object methodsFor: '*Aida' stamp: 'mivsek 1/10/2008 18:14'!
forLanguage: aLanguageCodeSymbol
	"for multilingual support: returns an apropriate instance of itself for that language. 
	Langage is defined by ISO 639 2-letter language code, see 
	http://en.wikipedia.org/wiki/List_of_ISO_639-1_codes"
	^false! !

!Object methodsFor: '*Aida' stamp: 'mivsek 1/10/2008 18:14'!
isMultilingual
	"for multilingual support: override this if your domain object responds 
	to #forLanguage: and returns an apropriate instance of itself for that language"
	^false! !

!Object methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:27'!
isVersionedObject
	^false! !

!Object methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:27'!
isWebApplication
	^false! !

!Object methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:27'!
isWebStyle
	^false! !

!Object methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:27'!
modifiedTimestamp
	"when this object was last modified"
	"used in http response, override if you like to be included"
	^nil! !

!Object methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:28'!
preferedUrl
	"override with a suggestion for url of this method!! If not already used, 
	it will be considered by URLResolver during automatic url generation"
	^nil! !

!Object methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:28'!
printWebAppNotFoundFor: aSession 
	| page |
	page := WebPage new.
	page addText: 'Cannot find aWebApplication for object a', self class name.
	^page! !

!Object methodsFor: '*Aida' stamp: 'janko 7/18/2008 22:30'!
printWebPageFor: aRequest on: aSession 
	"find appropriate web application to represent self as web page"

	| webApp |
	webApp := self webAppFor: aSession.
	^webApp notNil 
		ifTrue: [webApp printWebPageFor: aRequest]
		ifFalse: [self printWebAppNotFoundFor: aSession]! !

!Object methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:29'!
sendOver: aStream 
	"from Wiki rendering"
	self printOn: aStream! !

!Object methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:29'!
webAppFor: aSession
	| webApp |
	aSession isNil ifTrue: [^nil].
	webApp := aSession webAppFor: self.
	webApp notNil ifTrue: [^webApp].
	webApp := WebApplication newFor: self on: aSession.
	webApp notNil ifTrue: [aSession addWebApp: webApp for: self].
	^webApp! !

Object subclass: #Party
	instanceVariableNames: 'id name description addresses roles relatedPartyRoles events relatedObjects otherValues'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Parties'!

Party subclass: #OrganizationUnit
	instanceVariableNames: 'type parent units specialRoles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Parties'!

OrganizationUnit subclass: #Company
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Parties'!

!Company class methodsFor: 'odb specific' stamp: ' 10/1/08 16:40'!
replicationSpecaa
	"Gemstone"
	^#( 	(id 	replicate)
			(name replicate)
			(description replicate)
			(addresses max 4)
			(roles max 2)
			(relatedPartyRoles max 2)
			(events forwarder)
			(relatedObjects max 1) )! !

!Company methodsFor: 'adding-removing' stamp: ' 10/1/08 16:40'!
addEmployee: aPerson
	aPerson becomeEmployeeOf: self! !

!Company methodsFor: 'accessing-events' stamp: ' 10/1/08 16:40'!
addInvoice: anInvoice
	anInvoice parent: self.
	^self events add: anInvoice onDate: anInvoice dateForStoring! !

!Company methodsFor: 'accessing-events' stamp: ' 10/1/08 16:40'!
addLetter: aLetter
	aLetter parent: self.
	^self events add: aLetter onDate: aLetter dateForStoring! !

!Company methodsFor: 'accessing-events' stamp: ' 10/1/08 16:40'!
allEvents
	^self events all! !

!Company methodsFor: 'accessing-events' stamp: ' 10/1/08 16:40'!
allInvoices
	^self events all select: [:each | each isInvoice]! !

!Company methodsFor: 'accessing-events' stamp: ' 10/1/08 16:40'!
allLetters
	^self events all select: [:each | each isLetter]! !

!Company methodsFor: 'import-export' stamp: ' 10/1/08 16:40'!
autoEMailAddressesForPersons
	"Repository default company autoEMailAddressesForPersons"
	self allEmployees do: [:person |
		person email isValidEMailAddress ifFalse:  	
			[person email: (person name asSloveneWithoutCircumflexes asLowercase , '.',
			person surname asSloveneWithoutCircumflexes asLowercase, 
			'@salus.si')] ]! !

!Company methodsFor: 'private' stamp: ' 10/1/08 16:40'!
cleanupEmployeeToOrgUnitsConnections
	"(AIDASite named: 'biart') repository company cleanupEmployeeToOrgUnitsConnections"
	| set employees |
	set := self allEmployees asSet.
	employees := self allOrgUnits "collect them through orgunits hierarchy"
		inject: Set new into: [:sett :each | sett addAll: each allMembers; yourself].
	(employees reject: [:each | set includes: each]) "those dangling"
		do: [:each | each becomeFormerEmployeeOf: self. each removeJobRole.
			self allOrgUnits do: [:unit | unit removeMember: each] ].
	employees := self allOrgUnits inject: Set new into: [:sett :each | 
		sett addAll: each allPersonsWithJobRoles; yourself].
	(employees reject: [:each | set includes: each])
		do: [: each | each removeJobRole].! !

!Company methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
company
	^self! !

!Company methodsFor: 'import-export' stamp: ' 10/1/08 16:40'!
connectEmployeesToOrgUnits
	"salus"	
	"preimenuj zacasno org.enote brez sumnikov!! "
	"Repository default company connectEmployeesToOrgUnits"
	| orgUnit |
	self allEmployees do: [:each |
		orgUnit := self salusOrgUnitFor: each.
		orgUnit notNil ifTrue: [each allOrgUnits isEmpty ifTrue: [each becomeMemberOf: orgUnit] ]]! !

!Company methodsFor: 'import-export' stamp: ' 10/1/08 16:40'!
connectPersonsToWebUsers
	"Repository default company connectPersonsToWebUsers"
	self allEmployees do: [:each | 	AIDASite default securityManager addPerson: each]! !

!Company methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
employeeNamed: aString
	^self allEmployees detect: [:each | aString = each name] ifNone: [nil]! !

!Company methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
employeeNamedSurnamed: aString
	^self allEmployees detect: [:each | aString = each nameSurname] ifNone: [nil]! !

!Company methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
employeeSurnamedNamed: aString
	^self allEmployees detect: [:each | aString = each surnameName] ifNone: [nil]! !

!Company methodsFor: 'testing' stamp: ' 10/1/08 16:40'!
existEmployeeWithId: aString
	^self allEmployees contains: [:each | each id = aString].! !

!Company methodsFor: 'testing' stamp: ' 10/1/08 16:40'!
existEmployeeWithName: aNameString surname: aSurnameString
	^self allEmployees contains: [:each | 
		(each name = aNameString) and: [each surname = aSurnameString]].! !

!Company methodsFor: 'import-export' stamp: ' 10/1/08 16:40'!
importEmployeesFrom: aFileString
	"Repository default company 
		importEmployeesFrom: '/home/mivsek/projekti/salus/evidenca-popr1.csv' "
	"common delimited file with: id name surname org.unit "
	| delimited |
	delimited := DelimitedFile new readFrom: aFileString.
	delimited records do: [:each || idp nme sname orgUnit person |
		idp := each at: 1. nme := each at: 4. sname := each at: 3. orgUnit := each at: 5.
		(self existEmployeeWithId: idp) ifFalse: 
			[person := Person new id: idp; name: nme; surname: sname;
				otherValuesAt: #orgUnit put: orgUnit; yourself.
			self addEmployee: person ] ].
	^delimited! !

!Company methodsFor: 'accessing-other' stamp: ' 10/1/08 16:40'!
invoiceFooter
	^self otherValuesAt: #InvoiceFooter ifAbsent: ''! !

!Company methodsFor: 'accessing-other' stamp: ' 10/1/08 16:40'!
invoiceFooter: aString
	"AIDASite default repository company invoiceFooter: 
		 'Osnovni kapital: <b>2.100.000 SIT</b> S^t.vlozka: <b>1/30613/00</b> SRG: <b>98/02155</b> Ident.s^t.: <b>SI76352226</b> TRR: <b>05100-8000073227</b>'"
	self otherValuesAt: #InvoiceFooter put: aString! !

!Company methodsFor: 'testing' stamp: ' 10/1/08 16:40'!
isCompany
	^true! !

!Company methodsFor: 'private' stamp: ' 10/1/08 16:40'!
migrateToUnicode
	"from iso8859-2"
	"Company allInstances do: [:each | each migrateToUnicode]"
	super migrateToUnicode.! !

!Company methodsFor: 'accessing-events' stamp: ' 10/1/08 16:40'!
paidInvoices
	^self allInvoices select: [:each | each isPaid]! !

!Company methodsFor: 'private' stamp: ' 10/1/08 16:40'!
preferedUrl
	| nme |
	nme := (self name copyUpTo: $ ) trimBlanks asSloveneWithoutCircumflexes.
	nme := (nme copyReplaceAll: ' ' with: '-') copyReplaceAll: '/' with: '-'.
	^'/podjetje/', nme, '.html'.! !

!Company methodsFor: 'adding-removing' stamp: ' 10/1/08 16:40'!
removeEmployee: aPerson
	aPerson noMoreEmployeeOf: self.
	self allOrgUnits do: [:each | each removeMember: aPerson].
	aPerson removeJobRole! !

!Company methodsFor: 'import-export' stamp: ' 10/1/08 16:40'!
salusOrgUnitFor: aPerson
	"preimenuj zacasno org.enote brez sumnikov!! "
	| map |
	map := Dictionary new
		at: 'GFS' put: 'Gospodarsko-financni sektor';
		at: 'IC' put: 'Informacijski center';
		at: 'Nabava' put: 'Nabavni sektor';
		at: 'Prodaja' put: 'Prodajni sektor';
		at: 'Sektor kakovosti' put: 'Sektor kakovosti';
		at: 'Skladisce' put: 'Skladisce';
		at: 'Tehnicni' put: 'Tehnicni sektor';
		at: 'ZT' put: 'ZT sektor zastopstva, konsignacije'; yourself.
	^self orgUnitWithName: (map at: (aPerson otherValuesAt: #orgUnit ifAbsent: [^nil]) ifAbsent: [^nil]).! !

!Company methodsFor: 'accessing-other' stamp: ' 10/1/08 16:40'!
taxIdentificationCode
	^self otherValuesAt: #TaxIdentificationCode! !

!Company methodsFor: 'accessing-other' stamp: ' 10/1/08 16:40'!
taxIdentificationCode: aString
	self otherValuesAt: #TaxIdentificationCode put: aString! !

!Company methodsFor: 'accessing-events' stamp: ' 10/1/08 16:40'!
unpaidInvoices
	^self allInvoices select: [:each | each isPaid not]! !

!OrganizationUnit class methodsFor: 'instance creation' stamp: ' 10/1/08 16:40'!
newNamed: aString
	^self new name: aString! !

!OrganizationUnit class methodsFor: 'odb specific' stamp: ' 10/1/08 16:40'!
replicationSpecaa
	"Gemstone"
	^super replicationSpec! !

!OrganizationUnit methodsFor: 'job roles' stamp: ' 10/1/08 16:40'!
addJobRoleNamed: aString
	(self existJobRoleNamed: aString) ifTrue: [^nil].
	self otherValuesAt: #jobRoles ifAbsent: 
		[self otherValuesAt: #jobRoles put: OrderedCollection new].
	(self otherValuesAt: #jobRoles) add: (JobRole new name: aString)! !

!OrganizationUnit methodsFor: 'adding-removing' stamp: ' 10/1/08 16:40'!
addMember: aPerson
	aPerson becomeMemberOf: self! !

!OrganizationUnit methodsFor: 'adding-removing' stamp: ' 10/1/08 16:40'!
addOrganizationUnit: anOrganizationUnit
	units isNil ifTrue: [self initUnits].
	(self units includes: anOrganizationUnit) ifFalse:
		[self units add: anOrganizationUnit. anOrganizationUnit parent: self]! !

!OrganizationUnit methodsFor: 'adding-removing' stamp: ' 10/1/08 16:40'!
addOrganizationUnitNamed: aString
	self addOrganizationUnit: (OrganizationUnit newNamed: aString)! !

!OrganizationUnit methodsFor: 'roles - special' stamp: ' 10/1/08 16:40'!
addSpecialRoleNamed: aName
	(self existSpecialRoleNamed: aName) ifFalse:
		[self specialRoles add: (Role new name: aName)]! !

!OrganizationUnit methodsFor: 'job roles' stamp: ' 10/1/08 16:40'!
allJobRoles
	^self otherValuesAt: #jobRoles ifAbsent: [^#()]! !

!OrganizationUnit methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
allMembers
	"all members of that org.unit"
	| parties |
	parties :=  (self roles select: [:role | role isMemberRole]) 
		collect: [:each | each relatedParty].
	parties := parties addAll:
		((self relatedPartyRoles select: [:role | role isMemberRole]) 
			collect: [:each | each party]);
		yourself.
	^parties asSet asOrderedCollection! !

!OrganizationUnit methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
allOrgUnits
	"all units down in hierarchy, together with self"
	| collection |
	collection := OrderedCollection new.
	collection add: self.
	self units do: [:each | collection addAll: each allOrgUnits].
	^collection! !

!OrganizationUnit methodsFor: 'job roles' stamp: ' 10/1/08 16:40'!
allPersonsWithJobRoles
	"collect all persons holding job roles in this org unit"
	^self allJobRoles inject: Set new into: [:set :role | set addAll: role allPersons; yourself].! !

!OrganizationUnit methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
company
	"return a company whom belongs that org.unit"
	^self parent company! !

!OrganizationUnit methodsFor: 'job roles' stamp: ' 10/1/08 16:40'!
existJobRoleNamed: aString
	^(self otherValuesAt: #jobRoles ifAbsent: [^false])
		contains: [:each | each name = aString]! !

!OrganizationUnit methodsFor: 'roles - special' stamp: ' 10/1/08 16:40'!
existSpecialRoleNamed: aName
	^(self specialRoleNamed: aName) notNil! !

!OrganizationUnit methodsFor: 'initialize-release' stamp: ' 10/1/08 16:40'!
initSpecialRoles
	specialRoles := OrderedCollection new! !

!OrganizationUnit methodsFor: 'initialize-release' stamp: ' 10/1/08 16:40'!
initUnits
	units := OrderedCollection new! !

!OrganizationUnit methodsFor: 'testing' stamp: ' 10/1/08 16:40'!
isOrganizationUnit
	^true! !

!OrganizationUnit methodsFor: 'job roles' stamp: ' 10/1/08 16:40'!
jobRoleNamed: aString
	^(self otherValuesAt: #jobRoles ifAbsent: [^nil])
		detect: [:each | each name = aString] ifNone: [nil]! !

!OrganizationUnit methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
jobRoleWithUuid: aString
	"search for this job role through hierarchy starting from self"
	| found |
	^self allJobRoles detect: [:each | each uuid = aString] ifNone:
		[self units do: [:each | found := each jobRoleWithUuid: aString. found notNil ifTrue: [^found] ].
		nil]! !

!OrganizationUnit methodsFor: 'private' stamp: ' 10/1/08 16:40'!
migrateToUnicode
	"from iso8859-2"
	"OrganizationUnit allInstances do: [:each | each migrateToUnicode]"
	super migrateToUnicode.! !

!OrganizationUnit methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
orgUnitWithName: aString
	"search for this org.unit through hierarchy starting from self"
	| found |
	self name = aString ifTrue: [^self].
	self units do: [:each | each name = aString ifTrue: [^each] ].
	self units do: [:each | 
		found := each orgUnitWithName: aString. found notNil ifTrue: [^found] ].
	^nil! !

!OrganizationUnit methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
orgUnitWithUuid: aString
	"search for this org.unit through hierarchy starting from self"
	| found |
	self uuid = aString ifTrue: [^self].
	self units do: [:each | each uuid = aString ifTrue: [^each] ].
	self units do: [:each | found := each orgUnitWithUuid: aString. found notNil ifTrue: [^found] ].
	^nil! !

!OrganizationUnit methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
parent
	^parent! !

!OrganizationUnit methodsFor: 'private' stamp: ' 10/1/08 16:40'!
parent: anOrganizationUnit
	parent := anOrganizationUnit! !

!OrganizationUnit methodsFor: 'private' stamp: ' 10/1/08 16:40'!
preferedUrl
	| nme |
	nme := self name trimBlanks asSloveneWithoutCircumflexes.
	nme := (nme copyReplaceAll: ' ' with: '-') copyReplaceAll: '/' with: '-'.
	^'/org-enota/', nme, '.html'.

"OrganizationUnit allInstances do: [:each | 
	each repository notNil ifTrue: [each repository site urlResolver changeToPreferedURL: each] ]"! !

!OrganizationUnit methodsFor: 'job roles' stamp: ' 10/1/08 16:40'!
removeJobRole: aJobRole
	self otherValuesAt: #jobRoles ifAbsent: [^nil].
	(self otherValuesAt: #jobRoles) remove: aJobRole ifAbsent: [^nil]! !

!OrganizationUnit methodsFor: 'job roles' stamp: ' 10/1/08 16:40'!
removeJobRoleNamed: aString
	(self existJobRoleNamed: aString) ifFalse: [^nil].
	(self otherValuesAt: #jobRoles) remove: (self jobRoleNamed: aString)! !

!OrganizationUnit methodsFor: 'adding-removing' stamp: ' 10/1/08 16:40'!
removeMember: aPerson
	aPerson noMoreMemberOf: self! !

!OrganizationUnit methodsFor: 'adding-removing' stamp: ' 10/1/08 16:40'!
removeOrganizationUnit: anOrganizationUnit
	units isNil ifTrue: [^nil].
	(self units includes: anOrganizationUnit) ifTrue:
		[self units remove: anOrganizationUnit. anOrganizationUnit parent: nil]! !

!OrganizationUnit methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
repository
	self parent isNil ifTrue: [^nil].
	self parent class == Repository ifTrue: [^self parent].
	^self parent repository! !

!OrganizationUnit methodsFor: 'roles - special' stamp: ' 10/1/08 16:40'!
specialRoleCollection
	"all specail roles for that and parent org.units up in to the top of hierarchy.
	sorted by the top first!!"
	^(self parent notNil and: [self parent isKindOf: Party])
		ifTrue: [self parent specialRoleCollection addAll: self specialRoles; yourself]
		ifFalse: [self specialRoles]! !

!OrganizationUnit methodsFor: 'roles - special' stamp: ' 10/1/08 16:40'!
specialRoleNamed: aName
	^self specialRoles detect: [:each | each name = aName] ifNone: [nil]! !

!OrganizationUnit methodsFor: 'roles - special' stamp: ' 10/1/08 16:40'!
specialRoles
	"a set of roles, specific for member of that org.unit, like aprover, tester etc"
	specialRoles isNil ifTrue: [self initSpecialRoles].
	^specialRoles! !

!OrganizationUnit methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
type
	^type! !

!OrganizationUnit methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
type: aSymbol
	"kind of org.unit like #division #sector #group etc."
	type := aSymbol! !

!OrganizationUnit methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
units
	units isNil ifTrue: [^#()].
	^units! !

!Party class methodsFor: 'instance creation' stamp: ' 10/1/08 16:40'!
newCustomerOf: aParty

	^super new
		becomeCustomerOf: aParty! !

!Party class methodsFor: 'instance creation' stamp: ' 10/1/08 16:40'!
newSupplierTo: aParty

	^super new
		becomeSupplierTo: aParty! !

!Party class methodsFor: 'odb specific' stamp: ' 10/1/08 16:40'!
replicationSpec
	"Gemstone"
	^super replicationSpec , 
		#( 	(id 	replicate)
			(name replicate)
			(description replicate)
			(addresses max 4)
			(roles max 2)
			(relatedPartyRoles max 2)
			(events forwarder)
			(relatedObjects max 1) )! !

!Party methodsFor: 'related objects' stamp: ' 10/1/08 16:40'!
addRelatedObject: anObject
	(self relatedObjects includes: anObject) ifFalse:
		[self relatedObjects add: anObject].! !

!Party methodsFor: 'private' stamp: ' 10/1/08 16:40'!
addRelatedPartyRole: aRole
	(self relatedPartyRoles includes: aRole) ifTrue: [^nil].
	self relatedPartyRoles add: aRole.! !

!Party methodsFor: 'private' stamp: ' 10/1/08 16:40'!
addRole: aRole
	(self roles includes: aRole) ifTrue: [^nil].
	self roles add: aRole.! !

!Party methodsFor: 'role groups' stamp: ' 10/1/08 16:40'!
addRoleGroup: aRoleGroup
	(self roleGroups includes: aRoleGroup) ifFalse: [self roleGroups add: aRoleGroup]! !

!Party methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
addresses
	addresses isNil ifTrue: [self initAddresses].
	^addresses! !

!Party methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
allCustomers
	"return all parties which are my customers, therefore I am a supplier to them"
	| parties |
	parties :=  (self roles select: [:role | role isSupplierRole]) 
		collect: [:each | each relatedParty].
	parties := parties addAll:
		((self relatedPartyRoles select: [:role | role isCustomerRole]) 
			collect: [:each | each party]);
		yourself.
	^parties asSet asOrderedCollection! !

!Party methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
allEmployees
	"all employees of that party"
	| parties |
	parties :=  (self roles select: [:role | role isEmployeeRole]) 
		collect: [:each | each relatedParty].
	parties := parties addAll:
		((self relatedPartyRoles select: [:role | role isEmployeeRole]) 
			collect: [:each | each party]);
		yourself.
	^parties asSet asOrderedCollection! !

!Party methodsFor: 'roles - basic' stamp: ' 10/1/08 16:40'!
allRelatedRolesFrom: aParty
	"return all roles this party has to me"
	^self relatedPartyRoles select: [:each | each party == aParty]! !

!Party methodsFor: 'role groups' stamp: ' 10/1/08 16:40'!
allRoleGroups
	^self roleGroups copy! !

!Party methodsFor: 'roles - basic' stamp: ' 10/1/08 16:40'!
allRoles
	^self roles copy! !

!Party methodsFor: 'roles - basic' stamp: ' 10/1/08 16:40'!
allRolesTo: aParty
	"return all roles I have to this party"
	^self roles select: [:each | each relatedParty == aParty]! !

!Party methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
allSuppliers
	"return all parties which supply goods to me, therefore I am a customer of them"
	| parties |
	parties :=  (self roles select: [:role | role isCustomerRole]) 
		collect: [:each | each relatedParty].
	parties := parties addAll:
		((self relatedPartyRoles select: [:role | role isSupplierRole]) 
			collect: [:each | each party]); 
		yourself.
	^parties asSet asOrderedCollection! !

!Party methodsFor: 'accessing-other' stamp: ' 10/1/08 16:40'!
bankAccount
	^self bankAccounts notEmpty ifTrue: [self bankAccounts first] ifFalse: [''].! !

!Party methodsFor: 'accessing-other' stamp: ' 10/1/08 16:40'!
bankAccount: aString
	self bankAccounts isEmpty ifTrue: 
		[self otherValuesAt: #BankAccounts put: (OrderedCollection new add: ''; yourself) ].
	self bankAccounts at: 1 put: aString.! !

!Party methodsFor: 'accessing-other' stamp: ' 10/1/08 16:40'!
bankAccounts
	^self otherValuesAt: #BankAccounts ifAbsent: [OrderedCollection new].! !

!Party methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
becomeCustomerOf: aParty
	"I become a customer of related party, I buy goods from this party"
	| role |
	(self isCustomerOf: aParty) ifTrue: [^false].
	role := CustomerRole new.
	role name: 'customer role'.
	self newRole: role relatedTo: aParty! !

!Party methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
becomeSiteOwner
	"usual for Companies, SiteOwnerRole has a reference to his WebSite"! !

!Party methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
becomeSupplierTo: aParty 
	"I become a supplier to related party, I supply goods to this party"
	| role |
	(self isSupplierTo: aParty) ifTrue: [^false].
	role := SupplierRole new.
	role name: 'supplier role'.
	self newRole: role relatedTo: aParty! !

!Party methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
becomeWebUserTo: aWebSiteOwnerPary! !

!Party methodsFor: 'roles - basic' stamp: ' 10/1/08 16:40'!
changeRole: anOldRole to: aNewRole! !

!Party methodsFor: 'roles - basic' stamp: ' 10/1/08 16:40'!
changeRole: anOldRole to: aNewRole relatedTo: aParty! !

!Party methodsFor: 'addresses' stamp: ' 10/1/08 16:40'!
city
	^self mainAddress city! !

!Party methodsFor: 'addresses' stamp: ' 10/1/08 16:40'!
city: aString
	self mainAddress city: aString! !

!Party methodsFor: 'private' stamp: ' 10/1/08 16:40'!
combineOrdersWithPartyDocumentNumber: anIdString toOrder: anOrder
	| orders sumOrder |
	sumOrder := anOrder.
	anOrder isShippingOrder
		ifTrue: 
			[orders := self allShippingOrders select: [: each | 
				(each partyDocumentId = anIdString) & (each isCompleted)]]
		ifFalse:
			[orders := self allReceivingOrders select: [: each | 
				(each partyDocumentId = anIdString) & (each isCompleted)]].
	orders notEmpty ifTrue:
		[orders do: [:order | sumOrder copyFromOrder: order].
		sumOrder state: #completed.
		sumOrder partyDocumentId: sumOrder id.
		MailInput new sendOrder: sumOrder]! !

!Party methodsFor: 'addresses' stamp: ' 10/1/08 16:40'!
country
	^self mainAddress country! !

!Party methodsFor: 'addresses' stamp: ' 10/1/08 16:40'!
country: aString
	self mainAddress country: aString! !

!Party methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
description
	description isNil ifTrue: [self description: ''].
	^description! !

!Party methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
description: aString
	description := aString! !

!Party methodsFor: 'roles - basic' stamp: ' 10/1/08 16:40'!
discontinueAllRoles
	self roles copy do: [:each | self discontinueRole: each].
	self relatedPartyRoles copy do: [:each | self discontinueRole: each]! !

!Party methodsFor: 'roles - basic' stamp: ' 10/1/08 16:40'!
discontinueAllRolesTo: aParty
	(self roles select: [:role | role relatedParty == aParty])
		do: [:each | self discontinueRole: each].! !

!Party methodsFor: 'roles - basic' stamp: ' 10/1/08 16:40'!
discontinueRole: aRole
	self roles remove: aRole ifAbsent: [].
	aRole relatedParty relatedPartyRoles remove: aRole ifAbsent: [].! !

!Party methodsFor: 'addresses' stamp: ' 10/1/08 16:40'!
email
	^self mainAddress email! !

!Party methodsFor: 'addresses' stamp: ' 10/1/08 16:40'!
email: aString
	self mainAddress email: aString! !

!Party methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
events
	events isNil ifTrue: [self initEvents].
	^events! !

!Party methodsFor: 'addresses' stamp: ' 10/1/08 16:40'!
fax
	^self mainAddress fax! !

!Party methodsFor: 'addresses' stamp: ' 10/1/08 16:40'!
fax: aString
	self mainAddress fax: aString! !

!Party methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
id
	id isNil ifTrue: [self id: ''].
	^id! !

!Party methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
id: aString
	id := aString.! !

!Party methodsFor: 'initialize - release' stamp: ' 10/1/08 16:40'!
initAddresses
	addresses := Dictionary new.! !

!Party methodsFor: 'initialize - release' stamp: ' 10/1/08 16:40'!
initEvents
	events := DailyCollection new.! !

!Party methodsFor: 'initialize - release' stamp: ' 10/1/08 16:40'!
initOtherValues
	otherValues := Dictionary new.! !

!Party methodsFor: 'initialize - release' stamp: ' 10/1/08 16:40'!
initRelatedObjects
	relatedObjects := OrderedCollection new.! !

!Party methodsFor: 'initialize - release' stamp: ' 10/1/08 16:40'!
initRelatedPartyRoles
	relatedPartyRoles := OrderedCollection new.! !

!Party methodsFor: 'initialize - release' stamp: ' 10/1/08 16:40'!
initRoles
	roles := OrderedCollection new.! !

!Party methodsFor: 'testing' stamp: ' 10/1/08 16:40'!
isCompany
	^false! !

!Party methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
isCustomerOf: aParty

	| isCustomer |
	isCustomer := self roles contains: [:role | role relatedParty == aParty and: [role isCustomerRole] ].
	isCustomer ifTrue: [^true].
	"check also a complementary role"
	^self relatedPartyRoles contains: [:role | role party == aParty and: [role isSupplierRole] ].! !

!Party methodsFor: 'testing' stamp: ' 10/1/08 16:40'!
isOrganizationUnit
	^false! !

!Party methodsFor: 'testing' stamp: ' 10/1/08 16:40'!
isPerson
	^false! !

!Party methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
isSupplierTo: aParty

	| isSupplier |
	isSupplier := self roles contains: [:role | role relatedParty == aParty and: [role isSupplierRole] ].
	isSupplier ifTrue: [^true].
	"check also a complementary role"
	^self relatedPartyRoles contains: [:role | role party == aParty and: [role isCustomerRole] ].! !

!Party methodsFor: 'addresses' stamp: ' 10/1/08 16:40'!
mainAddress
	
	^self addresses 
		at: #MainAddress 
		ifAbsent: [self addresses at: #MainAddress put: Address new. self mainAddress].! !

!Party methodsFor: 'private' stamp: ' 10/1/08 16:40'!
migrateToUnicode
	"from iso8859-2"
	id notNil ifTrue: [id := id ensureUnicodeSloveneChars].
	name notNil ifTrue: [name := name ensureUnicodeSloveneChars].! !

!Party methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
name
	name isNil ifTrue: [self name: ''].
	^name! !

!Party methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
name: aString
	name := aString trimBlanks.! !

!Party methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
nameToDisplay
	"on web pages for instance"
	^self name! !

!Party methodsFor: 'roles - basic' stamp: ' 10/1/08 16:40'!
newRole: aRole
	"new role, but not in relationship with anyone"! !

!Party methodsFor: 'roles - basic' stamp: ' 10/1/08 16:40'!
newRole: aRole relatedTo: aParty
	"new role in relationship with other party"

	aRole party: self.
	aRole relatedParty: aParty.
	self addRole: aRole.
	aParty addRelatedPartyRole: aRole.! !

!Party methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
noMoreCustomerOf: aParty 
	"no more a customer of related party"
	(self isCustomerOf: aParty) ifFalse: [^nil].
	self discontinueRole:
		(self roles detect: [:role | role relatedParty == aParty and: [role isCustomerRole] ] ifNone: [^nil])! !

!Party methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
noMoreSupplierTo: aParty 
	"no more a supplier to related party"
	(self isSupplierTo: aParty) ifFalse: [^nil].
	self discontinueRole:
		(self roles detect: [:role | role relatedParty == aParty and: [role isSupplierRole] ] ifNone: [^nil])! !

!Party methodsFor: 'private' stamp: ' 10/1/08 16:40'!
otherValues
	^otherValues! !

!Party methodsFor: 'private' stamp: ' 10/1/08 16:40'!
otherValuesAt: aString
	^self otherValuesAt: aString ifAbsent: [nil]! !

!Party methodsFor: 'private' stamp: ' 10/1/08 16:40'!
otherValuesAt: aString ifAbsent: aBlock
	self otherValues isNil ifTrue: [^aBlock value].
	^self otherValues at: aString ifAbsent: [aBlock value]! !

!Party methodsFor: 'private' stamp: ' 10/1/08 16:40'!
otherValuesAt: aString put: anObject
	self otherValues isNil ifTrue: [self initOtherValues].
	^self otherValues at: aString put: anObject! !

!Party methodsFor: 'private' stamp: ' 10/1/08 16:40'!
partialyConnectedRoles
	"roles, which are not correctly connected to related party and back"
	"not working well yet !!!!!! "
	| col |
	col := self roles reject: [:each | each relatedParty roles contains: [:role | role relatedParty == self] ].
	col addAll: (self relatedPartyRoles 
		reject: [:each | each relatedParty roles contains: [:role | role party == self] ]).
	^col

"Company allInstances select: [:each | each partialyConnectedRoles notEmpty]"! !

!Party methodsFor: 'addresses' stamp: ' 10/1/08 16:40'!
phone
	^self mainAddress phone! !

!Party methodsFor: 'addresses' stamp: ' 10/1/08 16:40'!
phone: aString
	self mainAddress phone: aString! !

!Party methodsFor: 'addresses' stamp: ' 10/1/08 16:40'!
post
	^self mainAddress post! !

!Party methodsFor: 'addresses' stamp: ' 10/1/08 16:40'!
post: anObject
	^self mainAddress post: anObject! !

!Party methodsFor: 'addresses' stamp: ' 10/1/08 16:40'!
postalCode
	^self mainAddress postalCode! !

!Party methodsFor: 'addresses' stamp: ' 10/1/08 16:40'!
postalCode: aString
	self mainAddress postalCode: aString! !

!Party methodsFor: 'printing' stamp: ' 10/1/08 16:40'!
printString
	^'party: ', self name! !

!Party methodsFor: 'private' stamp: ' 10/1/08 16:40'!
reconnectEmployees
	"for debugging"
	self allEmployees do: [:each | self removeEmployee: each; addEmployee: each]! !

!Party methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
relatedObjects
	relatedObjects isNil ifTrue: [self initRelatedObjects].
	^relatedObjects! !

!Party methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
relatedParty
	"which party is the 'owner', e.g. doing business in elogis terms with that party"

	| thirdPLCustomers parties |
	thirdPLCustomers := LogisticSystem default owner all3PLCustomers.
	parties := thirdPLCustomers select: [:party | party isSupplierTo: self].
	^parties isEmpty 
		ifTrue: 
			[parties := thirdPLCustomers select: [:party | party isCustomerOf: self].
			parties isEmpty ifTrue: [^nil] ifFalse: [parties first] ]
		ifFalse: [parties first]! !

!Party methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
relatedPartyRoles
	relatedPartyRoles isNil ifTrue: [self initRelatedPartyRoles].
	^relatedPartyRoles! !

!Party methodsFor: 'related objects' stamp: ' 10/1/08 16:40'!
removeRelatedObject: anObject
	self relatedObjects remove: anObject ifAbsent: []! !

!Party methodsFor: 'role groups' stamp: ' 10/1/08 16:40'!
removeRoleGroup: aRoleGroup
	(self roleGroups includes: aRoleGroup) ifTrue: [self roleGroups remove: aRoleGroup]! !

!Party methodsFor: 'role groups' stamp: ' 10/1/08 16:40'!
roleGroupNamed: aString
	^self roleGroups detect: [:each | each name = aString] ifNone: [nil]! !

!Party methodsFor: 'role groups' stamp: ' 10/1/08 16:40'!
roleGroupWithUuid: aString
	^self roleGroups detect: [:each | each uuid = aString] ifNone: [nil]! !

!Party methodsFor: 'role groups' stamp: ' 10/1/08 16:40'!
roleGroups
	(self otherValuesAt: #roleGroups) isNil ifTrue: 
		[self otherValuesAt: #roleGroups put: OrderedCollection new].
	^self otherValuesAt: #roleGroups! !

!Party methodsFor: 'role groups' stamp: ' 10/1/08 16:40'!
roleGroupsIncludeRole: aRole
	^self roleGroups select: [:each | each includes: aRole]! !

!Party methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
roles
	roles isNil ifTrue: [self initRoles].
	^roles! !

!Party methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
searchEmployeesWithSurname: aSurname name: aName
	| surname nme |
	surname := aSurname asLowercase, '*'. nme := aName asLowercase, '*'.
	^self allEmployees select: [:each | 
		(surname match: each surname asLowercase) and: [nme match: each name asLowercase] ]! !

!Party methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
searchEmployeesWithSurnameName: aString
	| string nm |
	string := aString asLowercase, '*'. 
	^self allEmployees select: [:each | 
		nm := each surnameName asLowercase.
		(string match: nm)].! !

!Party methodsFor: 'addresses' stamp: ' 10/1/08 16:40'!
street
	^self mainAddress street! !

!Party methodsFor: 'addresses' stamp: ' 10/1/08 16:40'!
street: aString
	self mainAddress street: aString! !

!Party methodsFor: 'changing' stamp: ' 10/1/08 16:40'!
updateFrom: aParty
	self name: aParty name.
	self description: aParty description.
	self city: aParty city.
	self street: aParty street.
	self postalCode: aParty postalCode.! !

!Party methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
uuid
	"some unique identifier. Hash for now, probably unique enough!! "
	^self hash printString! !

Party subclass: #Person
	instanceVariableNames: 'surname'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Parties'!

!Person methodsFor: 'record flows' stamp: ' 10/1/08 16:40'!
activitiesNotified
	"all process activities on which this person is notified"
	| activities set |
	activities := self allRecordFlows collect: [:each | each activity].
	set := Set new: activities size. "remove duplicates, preserve order"
	^activities select: [:each || exist | exist := set includes: each. set add: each. exist not]! !

!Person methodsFor: 'responsibilities' stamp: ' 10/1/08 16:40'!
activitiesResponsible
	"all process activities where this person have some kind of responsibility"
	| activities set |
	activities := (self allResponsibilities select: [:each | each isTaskResponsibility])
		collect: [:each | each task activity].
	set := Set new: activities size. "remove duplicates, preserve order"
	^activities select: [:each || exist | exist := set includes: each. set add: each. exist not]! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
allDeputies
	"all deputies of that person"
	^(self relatedPartyRoles select: [:role | role isDeputyRole]) collect: [:each | each party]! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
allDeputyOf
	"all persons to whom that person is deputy"
	^(self roles select: [:role | role isDeputyRole]) collect: [:each | each relatedParty]! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
allOrgUnits
	"all org.units where person is member"
	^((self roles select: [:role | role isMemberRole]) collect: [:role | role relatedParty]) asSet.! !

!Person methodsFor: 'responsibilities' stamp: ' 10/1/08 16:40'!
allPurposeResponsibilities
	^self allResponsibilities select: [:each | each isPurposeResponsibility]! !

!Person methodsFor: 'record flows' stamp: ' 10/1/08 16:40'!
allRecordFlows
	"all record flows, concerned to that person"
	| flows set |
	flows := OrderedCollection new.
	self repository root allProcesses do: [:process | 
		process allTasks do: [:task | flows addAll: task allRecordFlows] ].
	set := Set new. self jobRole isNil ifTrue: [^#()].
	set add: self jobRole parentRole.
	set addAll: (self repository company roleGroupsIncludeRole: self jobRole parentRole).
	^flows select: [:flow | set includes: flow destination]! !

!Person methodsFor: 'responsibilities' stamp: ' 10/1/08 16:40'!
allResponsibilities
	"search for all responsibilities through document repository, specially processes"
	| resps set |
	resps := OrderedCollection new. self repository isNil ifTrue: [^resps].
	self repository root allProcesses do: [:process |
		process allPurposes do: [:purpose | resps addAll: purpose responsible].
		process allTasks do: [:task | resps addAll: task responsible] ].
	set := Set new. self jobRole isNil ifTrue: [^#()].
	set add: self jobRole parentRole.
	set addAll: (self repository company roleGroupsIncludeRole: self jobRole parentRole).
	^resps select: [:resp | set includes: resp role]! !

!Person methodsFor: 'responsibilities' stamp: ' 10/1/08 16:40'!
allTaskResponsibilities
	^self allResponsibilities select: [:each | each isTaskResponsibility]! !

!Person methodsFor: 'accessing' stamp: 'mivsek 1/10/2008 16:46'!
asICalUser
	^ICUser new
		commonName: self nameSurname  asSloveneWithoutCircumflexes "Outlook!!" ;
		emailAddress: self email! !

!Person methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
asPerson
	^self! !

!Person methodsFor: 'accessing' stamp: 'np 1/19/2008 11:36'!
asWebUser
	"parallel WebUser object"
	^self otherValuesAt: #WebUser ifAbsent: [^nil]! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
becomeDeputyOf: aPerson
	| role |
	(self isDeputyOf: aPerson) ifTrue: [^false].
	role := DeputyRole new.
	role name: 'deputy role'.
	self newRole: role relatedTo: aPerson.! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
becomeEmployeeOf: aParty 
	"I become an employee of that party"
	| role |
	(self isEmployeeOf: aParty) ifTrue: [^false].
	role := EmployeeRole new.
	role name: 'employee role'.
	self newRole: role relatedTo: aParty.

	self noMoreFormerEmployeeOf: aParty "if it was former before"! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
becomeFormerEmployeeOf: aParty 
	"I become an employee of that party"
	| role |
	(self isFormerEmployeeOf: aParty) ifTrue: [^false].
	role := FormerEmployeeRole new.
	role name: 'former employee role'.
	self newRole: role relatedTo: aParty! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
becomeMemberOf: anOrgUnit 
	"I become a member  of that org.unit"
	| role |
	(self isMemberOf: anOrgUnit) ifTrue: [^false].
	role := MemberRole new.
	role name: 'member role'.
	self newRole: role relatedTo: anOrgUnit! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
company
	"in which he is employeed"
	| parties |
	parties :=  (self roles select: [:role | role isEmployeeRole]) 
		collect: [:each | each relatedParty].
	parties := parties addAll:
		((self relatedPartyRoles select: [:role | role isEmployeeRole]) 
			collect: [:each | each party]);
		yourself.
	^parties isEmpty ifTrue: [nil] ifFalse: [parties first]! !

!Person methodsFor: 'documents' stamp: ' 10/1/08 16:40'!
documentsToKnow
	"all documents with which this person should know about (be notified)"
	^self repository root allDocuments select: [:each | each isReleased and: [each shouldKnow: self]].! !

!Person methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
email
	^self asWebUser notNil 
		ifTrue: [self asWebUser email]
		ifFalse: [super email].! !

!Person methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
email: aString 
	self asWebUser notNil 
		ifTrue: [self asWebUser email: aString]
		ifFalse: [super email: aString].! !

!Person methodsFor: 'roles - testing' stamp: ' 10/1/08 16:40'!
haveDeputies
	^self relatedPartyRoles contains: [:role | role isDeputyRole].! !

!Person methodsFor: 'accessing-other' stamp: ' 10/1/08 16:40'!
integratedWithOffice
	"WebDAV integration on MS IE"
	^self otherValuesAt: #integratedWithOffice ifAbsent: [false]! !

!Person methodsFor: 'accessing-other' stamp: ' 10/1/08 16:40'!
integratedWithOffice: aBoolean
	"WebDAV integration on MS IE"
	self otherValuesAt: #integratedWithOffice put: aBoolean.! !

!Person methodsFor: 'roles - testing' stamp: ' 10/1/08 16:40'!
isDeputy
	^self roles contains: [:role | role isDeputyRole].! !

!Person methodsFor: 'roles - testing' stamp: ' 10/1/08 16:40'!
isDeputyOf: aPerson
	^self roles contains: [:role | role relatedParty == aPerson and: [role isDeputyRole] ].! !

!Person methodsFor: 'roles - testing' stamp: ' 10/1/08 16:40'!
isEmployeeOf: aParty
	| employee |
	employee := self roles contains: [:role | role relatedParty == aParty and: [role isEmployeeRole] ].
	employee ifTrue: [^true].
	"check also a complementary role"
	^self relatedPartyRoles contains: [:role | role party == aParty and: [role isEmployeeRole] ].! !

!Person methodsFor: 'roles - testing' stamp: ' 10/1/08 16:40'!
isFormerEmployeeOf: aParty
	| formerEmployee |
	formerEmployee := self roles contains: [:role | 
		role relatedParty == aParty and: [role isFormerEmployeeRole]].
	formerEmployee ifTrue: [^true].
	"check also a complementary role"
	^self relatedPartyRoles contains: [:role | role party == aParty and: [role isFormerEmployeeRole] ].! !

!Person methodsFor: 'roles - testing' stamp: ' 10/1/08 16:40'!
isMemberOf: anOrgUnit
	| isMember |
	isMember := self roles contains: [:role | role relatedParty == anOrgUnit and: [role isMemberRole] ].
	isMember ifTrue: [^true].
	"check also a complementary role"
	^self relatedPartyRoles contains: [:role | role party == anOrgUnit and: [role isMemberRole] ].! !

!Person methodsFor: 'testing' stamp: ' 10/1/08 16:40'!
isPerson
	^true! !

!Person methodsFor: 'testing' stamp: ' 10/1/08 16:40'!
isWebUser
	^false! !

!Person methodsFor: 'testing' stamp: ' 10/1/08 16:40'!
isWebUserGroup
	^false! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
jobParentRole
	self jobRole isNil ifTrue: [^nil].
	^self jobRole parentRole.! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
jobRole
	^self roles detect: [:role | role isJobRole] ifNone: [^nil].! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
jobRoleName
	^self jobRole notNil ifTrue: [self jobRole name] ifFalse: [''].! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
jobRoleNamed: aString
	"org.unit should have this job role already defined!!"
	| parentRole |
	self organizationUnit isNil ifTrue: [^nil].
	parentRole := self organizationUnit jobRoleNamed: aString. parentRole isNil ifTrue: [^nil].
	self jobRole notNil ifTrue: [self removeJobRole].
	self newRole: parentRole newChildRole relatedTo: self company! !

!Person methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
menuName
	"in dropdown menus"
	^self surnameName! !

!Person methodsFor: 'private' stamp: ' 10/1/08 16:40'!
migrateToUnicode
	"from iso8859-2"
	"Person allInstances do: [:each | each migrateToUnicode]"
	super migrateToUnicode.
	surname notNil ifTrue: [surname := surname ensureUnicodeSloveneChars].! !

!Person methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
nameSurname
	^self name, ' ', self surname! !

!Person methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
nameToDisplay
	"on web pages for instance"
	^self surnameName! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
noMoreDeputyOf: aPerson
	| role |
	role := self roles 
		detect: [:each | each relatedParty == aPerson and: [each isDeputyRole] ] ifNone: [nil].
	role notNil ifTrue: [self discontinueRole: role].
	role := aPerson relatedPartyRoles 
		detect: [:each | each relatedParty == self and: [each isDeputyRole] ] ifNone: [nil].
	role notNil ifTrue: [aPerson discontinueRole: role].! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
noMoreEmployeeOf: aParty 
	"no more an employee of related party"
	| role |
	role := self roles 
		detect: [:each | each relatedParty == aParty and: [each isEmployeeRole] ] ifNone: [nil].
	role notNil ifTrue: [self discontinueRole: role].
	role := aParty relatedPartyRoles 
		detect: [:each | each relatedParty == self and: [each isEmployeeRole] ] ifNone: [nil].
	role notNil ifTrue: [aParty discontinueRole: role].

	self becomeFormerEmployeeOf: aParty.! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
noMoreFormerEmployeeOf: aParty 
	| role |
	role := self roles 
		detect: [:each | each relatedParty == aParty and: [each isFormerEmployeeRole] ] ifNone: [nil].
	role notNil ifTrue: [self discontinueRole: role].
	role := aParty relatedPartyRoles 
		detect: [:each | each relatedParty == self and: [each isFormerEmployeeRole] ] ifNone: [nil].
	role notNil ifTrue: [aParty discontinueRole: role].! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
noMoreMemberOf: anOrgUnit 
	"no more a memebr of related party"
	(self isMemberOf: anOrgUnit) ifFalse: [^nil].
	self discontinueRole:
		(self roles detect: [:role | role relatedParty == anOrgUnit and: [role isMemberRole] ] ifNone: [^nil])! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
organizationUnit
	"in which he is a member"
	| parties |
	parties :=  (self roles select: [:role | role isMemberRole]) 
		collect: [:each | each relatedParty].
	parties := parties addAll:
		((self relatedPartyRoles select: [:role | role isMemberRole]) 
			collect: [:each | each party]);
		yourself.
	^parties isEmpty ifTrue: [nil] ifFalse: [parties first]! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
organizationUnit: anOrganizationUnit
	self allOrgUnits do: [:each | self noMoreMemberOf: each].
	anOrganizationUnit notNil ifTrue: [self becomeMemberOf: anOrganizationUnit]! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
organizationUnitName
	^self organizationUnit notNil ifTrue: [self organizationUnit name] ifFalse: ['']! !

!Person methodsFor: 'accessing' stamp: 'janko 3/16/2008 17:00'!
password
	self asWebUser notNil ifTrue: [^self asWebUser password].
	^nil! !

!Person methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
password: aString 
	self asWebUser notNil ifTrue: [^self asWebUser password: aString trimBlanks].! !

!Person methodsFor: 'private' stamp: ' 10/1/08 16:40'!
preferedUrl
	| nme |
	nme := self surnameName trimBlanks asSloveneWithoutCircumflexes.
	nme := nme copyReplaceAll: ' ' with: '-'.
	^'/oseba/', nme, '.html'.

"Person allInstances do: [:each | 
	each repository notNil ifTrue: [each repository site urlResolver changeToPreferedURL: each] ]"! !

!Person methodsFor: 'private' stamp: ' 10/1/08 16:40'!
printString
	^'Person: ', self name, ' ', self surname! !

!Person methodsFor: 'record flows' stamp: ' 10/1/08 16:40'!
processesNotified
	"all processes where on which this person is notified"
	| processes set |
	processes := self allRecordFlows collect: [:each | each process].
	set := Set new: processes size. "remove duplicates, preserve order"
	^processes select: [:each || exist | exist := set includes: each. set add: each. exist not]! !

!Person methodsFor: 'responsibilities' stamp: ' 10/1/08 16:40'!
processesResponsible
	"all processes where this person have some kind of responsibility"
	| processes set |
	processes := self allResponsibilities collect: [:each | each process].
	set := Set new: processes size. "remove duplicates, preserve order"
	^processes select: [:each || exist | exist := set includes: each. set add: each. exist not]! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
removeJobRole
	self jobRole notNil ifTrue: [self discontinueRole: self jobRole].! !

!Person methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
repository
	self organizationUnit notNil ifTrue: [^self organizationUnit repository].
	^self company notNil ifTrue: [self company repository] ifFalse: [nil].! !

!Person methodsFor: 'roles - specific' stamp: ' 10/1/08 16:40'!
specialRoleCollection
	"a set of additional roles by person's org.unit and org.hierarch up to the top"
	^self organizationUnit notNil
		ifTrue: [self organizationUnit specialRoleCollection]
		ifFalse: [self company notNil
			ifTrue: [self company specialRoleCollection]
			ifFalse: [OrderedCollection new] ].! !

!Person methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
surname
	surname isNil ifTrue: [self surname: ''].
	^surname! !

!Person methodsFor: 'accessing' stamp: 'janko 3/16/2008 17:00'!
surname: aString
	surname := aString trimBlanks! !

!Person methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
surnameName
	^self surname, ' ', self name! !

!Person methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
surnameName2
	^self surnameName trimBlanks notEmpty 
		ifTrue: [self surnameName]
		ifFalse: ['--brez priimka in imena---']! !

!Person methodsFor: 'record flows' stamp: ' 10/1/08 16:40'!
tasksNotified
	"all process tasks  on which this person is notified"
	| tasks set |
	tasks := self allRecordFlows collect: [:each | each task].
	set := Set new: tasks size. "remove duplicates, preserve order"
	^tasks select: [:each || exist | exist := set includes: each. set add: each. exist not]! !

!Person methodsFor: 'responsibilities' stamp: ' 10/1/08 16:40'!
tasksResponsible
	"all process tasks where this person have some kind of responsibility"
	| tasks set |
	tasks := (self allResponsibilities select: [:each | each isTaskResponsibility]) 
		collect: [:each | each task].
	set := Set new: tasks size. "remove duplicates, preserve order"
	^tasks select: [:each || exist | exist := set includes: each. set add: each. exist not]! !

!Person methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
username
	self asWebUser notNil ifTrue: [^self asWebUser username].
	^nil! !

!Person methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
username: aString 
	self asWebUser notNil ifTrue: [^self asWebUser username: aString trimBlanks].! !

!Person methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
webUser: aWebUser
	"a parallel WebUser for web server security etc."
	self otherValuesAt: #WebUser put: aWebUser.
	(aWebUser otherValuesAt: #Person) ~= self ifTrue: [aWebUser person: self].! !

Object subclass: #PartyCollection
	instanceVariableNames: 'parent parties'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Parties'!

!PartyCollection commentStamp: '<historical>' prior: 0!
PartyCollection is used for doing many thing on mix of org.units, roles, rolegroups and persons. For instance, geting out all persons from above mix ...

Instance Variables:
	parent	<aParty>	a root party, conatining that mix of parties (eg. a company)
	parties	<anOrderedCollection>	mix of parties (orgUniits, roles, roleGroups, persons)

!

!PartyCollection class methodsFor: 'instance creation' stamp: ' 10/1/08 16:40'!
newOn: aCompany
	^super new parent: aCompany! !

!PartyCollection class methodsFor: 'instance creation' stamp: ' 10/1/08 16:40'!
newOn: aCompany from: aPartyCollection
	^(self newOn: aCompany) from: aPartyCollection! !

!PartyCollection methodsFor: 'adding-removing' stamp: ' 10/1/08 16:40'!
add: aPartyOrRole
	"anOrganizationalUnit, aRole, aRoleGroup or aPerson"
	self parties add: aPartyOrRole! !

!PartyCollection methodsFor: 'adding-removing' stamp: ' 10/1/08 16:40'!
addAll: aPartyCollection
	aPartyCollection do: [:each | self add: each]! !

!PartyCollection methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
all
	^self parties copy! !

!PartyCollection methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
allJobRoles
	^self parties select: [:each | each isKindOf: JobRole].! !

!PartyCollection methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
allOrgUnits
	^self parties select: [:each | each isKindOf: OrganizationUnit].! !

!PartyCollection methodsFor: 'accessing-parent' stamp: ' 10/1/08 16:40'!
allParentJobRoles
	^self parent allOrgUnits
		inject: OrderedCollection new into: [:col :each | col addAll: each allJobRoles; yourself]! !

!PartyCollection methodsFor: 'accessing-parent' stamp: ' 10/1/08 16:40'!
allParentOrgUnits
	^self parent allOrgUnits! !

!PartyCollection methodsFor: 'accessing-parent' stamp: ' 10/1/08 16:40'!
allParentOrgUnitsRolesPersons
	"sorted in that order.."
	| all |
	all := (SortedCollection withAll: self allParentOrgUnits sortBlock: [:a :b | a name < b name]) 
		asOrderedCollection.
	all addAll: (SortedCollection withAll: self allParentRoleGroups sortBlock: [:a :b | a name < b name]).
	all addAll: (SortedCollection withAll: self allParentJobRoles sortBlock: [:a :b | a name < b name]).
	all addAll: (SortedCollection withAll: self allParentPersons 
		sortBlock: [:a :b | a surnameName < b surnameName]).
	^all! !

!PartyCollection methodsFor: 'accessing-parent' stamp: ' 10/1/08 16:40'!
allParentPersons
	^self parent allEmployees! !

!PartyCollection methodsFor: 'accessing-parent' stamp: ' 10/1/08 16:40'!
allParentRoleGroups
	^self parent allRoleGroups! !

!PartyCollection methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
allPersons
	"both individual, from job roles and from org.units"
	| persons |
	persons := self onlyPersons asSet.
	persons := self allOrgUnits inject: persons into: [:set :each | set addAll: each allMembers; yourself].
	persons addAll: self allPersonsFromJobRoles.
	persons addAll: self allPersonsFromRoleGroups.
	^persons asOrderedCollection! !

!PartyCollection methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
allPersonsFromJobRoles
	| jobRoles  |
	jobRoles := self allJobRoles asSet.
	^self allParentPersons select: [:each |
		each jobRole notNil and: [jobRoles includes: each jobRole parentRole] ].

"	^self allJobRoles inject: Set new into: [:set :each | set addAll: each allPersons; yourself]."! !

!PartyCollection methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
allPersonsFromOrgUnit: anOrgUnit
	| members |
	members := anOrgUnit allMembers asSet.
	^self allPersons select: [:each | members includes: each]! !

!PartyCollection methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
allPersonsFromRoleGroups
	| jobRoles |
	jobRoles := self allRoleGroups inject: Set new into: [:set :each | set addAll: each allRoles; yourself].
	^self allParentPersons select: [:each |
		each jobRole notNil and: [jobRoles includes: each jobRole parentRole] ]! !

!PartyCollection methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
allRoleGroups
	^self parties select: [:each | each isKindOf: RoleGroup].! !

!PartyCollection methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
allSorted
	"org.unitss first, then role groups, roles, persons"
	| all |
	all := (SortedCollection withAll: self allOrgUnits sortBlock: [:a :b | a name < b name]) 
		asOrderedCollection.
	all addAll: (SortedCollection withAll: self allRoleGroups sortBlock: [:a :b | a name < b name]).
	all addAll: (SortedCollection withAll: self allJobRoles sortBlock: [:a :b | a name < b name]).
	all addAll: (SortedCollection withAll: self onlyPersons 
		sortBlock: [:a :b | a surnameName < b surnameName]).
	^all! !

!PartyCollection methodsFor: 'adding-removing' stamp: ' 10/1/08 16:40'!
from: aPartyCollection
	self initParties.
	self addAll: aPartyCollection! !

!PartyCollection methodsFor: 'testing' stamp: ' 10/1/08 16:40'!
includes: aPartyOrRole
	^self parties includes: aPartyOrRole! !

!PartyCollection methodsFor: 'initialize-release' stamp: ' 10/1/08 16:40'!
initParties
	parties := OrderedCollection new.! !

!PartyCollection methodsFor: 'testing' stamp: ' 10/1/08 16:40'!
notEmpty
	^self parties notEmpty! !

!PartyCollection methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
onlyPersons
	"just individual persons"
	^self parties select: [:each | each isKindOf: Person]! !

!PartyCollection methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
parent
	^parent! !

!PartyCollection methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
parent: aCompany
	"a root company, containing those mix of parties"
	parent := aCompany! !

!PartyCollection methodsFor: 'accessing-parent' stamp: ' 10/1/08 16:40'!
parentPartyOrRoleNamed: aString
	(aString isNil or: [aString isEmpty]) ifTrue: [^nil].
	^self allParentOrgUnitsRolesPersons detect: [:each | each nameToDisplay = aString] ifNone: [nil].! !

!PartyCollection methodsFor: 'private' stamp: ' 10/1/08 16:40'!
parties
	parties isNil ifTrue: [self initParties].
	^parties! !

!PartyCollection methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
partyOrRoleNamed: aString
	^self parties detect: [:each | each nameToDisplay = aString] ifNone: [nil].! !

!PartyCollection methodsFor: 'adding-removing' stamp: ' 10/1/08 16:40'!
remove: aPartyOrRole
	"anOrganizationalUnit, aRole, aRoleGroup or aPerson"
	self parties remove: aPartyOrRole! !

!PartyCollection methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
size
	^self parties size! !

!PartyCollection methodsFor: 'accessing' stamp: ' 10/1/08 16:40'!
sort
	"permanently sort a parties collection"
	self from: self allSorted! !

Object subclass: #RDBTable
	instanceVariableNames: 'environment username password codepage table columns records'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Support'!

!RDBTable class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
fromDemoTable
	"RDBTable fromDemoTable"
	^self newFromTable: 'ks801art'
		environment: 'skladisca'
		username:''
		password: ''
		codepage: #'7bit'.! !

!RDBTable class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
joinTwoTables
	"RDBTable joinTwoTables"
	| first merged |
	first := self newFromTable: 'ks801art'
		environment: 'skladisca'
		username:''
		password: ''
		codepage: #'7bit'.
	merged := first joinWithTable: 'ks810kar' myKey: 'SIFRA801' itsKey: 'SIFRA801'.
	^merged.! !

!RDBTable class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newFromTable: aTableString environment: anEnvString username: anUserString password: aPswString

	^super new
		table: aTableString;
		environment: anEnvString;
		username: anUserString;
		password: aPswString;
		readRecords.! !

!RDBTable class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newFromTable: aTableString environment: anEnvString username: anUserString password: aPswString codepage: aSymbol

	^super new
		table: aTableString;
		environment: anEnvString;
		username: anUserString;
		password: aPswString;
		codepage: aSymbol;
		readRecords.! !

!RDBTable methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
codepage
	"return a codepage of a external RDB source"
	codepage isNil ifTrue: [self codepage: self defaultCodepage].
	^codepage! !

!RDBTable methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
codepage: aSymbol
	" #cp1250 #cp852 #'7bit' #iso2 #czs    (slovenian codepages) "
	codepage := aSymbol.! !

!RDBTable methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
columns
	columns isNil ifTrue: [self initColumns].
	^columns! !

!RDBTable methodsFor: 'private' stamp: ' 21/4/07 22:07'!
columns: anObject
	columns := anObject! !

!RDBTable methodsFor: 'private' stamp: ' 21/4/07 22:07'!
convertRecords
	"convert db records to appropriate smalltalk format"
	| colType |
	1 to: columns size do: [:inx |
		colType := (self columns at: inx) at: 2.
		self records do: [:record |
			record 
				at: inx
				put: (self readFieldFrom: (record at: inx) odbcType: colType) ]
		].! !

!RDBTable methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
defaultCodepage
	"default codepage is #7bit "

	^#'7bit'! !

!RDBTable methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
environment
	environment isNil ifTrue: [self environment: ''].
	^environment! !

!RDBTable methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
environment: aString
	environment := aString! !

!RDBTable methodsFor: 'table reading' stamp: ' 21/4/07 22:07'!
getTableColumnsFrom: anODBCSession
	| answerStream |
	self initColumns.
	anODBCSession 
		getSQLColumns: nil
		tableOwner: nil
		tableName: self table 
		columnName: nil.
	answerStream := anODBCSession answer.
	self columns: (answerStream upToEnd
		collect: [:each | Array with: (each at: 4) with: (each at: 6)]).! !

!RDBTable methodsFor: 'private' stamp: ' 21/4/07 22:07'!
indexOfColumn: aString

	1 to: self columns size do: [:inx | 
		(self columns at: inx) first asLowercase = aString asLowercase ifTrue: [^inx] ].
	^0! !

!RDBTable methodsFor: 'initialize - release' stamp: ' 21/4/07 22:07'!
initColumns
	columns := OrderedCollection new.! !

!RDBTable methodsFor: 'table join' stamp: ' 21/4/07 22:07'!
joinWithTable: aTableName myKey: aMyKey itsKey: anItsKey
	"return an outer join of both tables, columns combined"
	| secondTable myKeyInx myKeyPos newRecords itsKeyPos mergedRecord |
	secondTable := 	RDBTable newFromTable: aTableName
		environment: self environment
		username: self username
		password: self password
		codepage: self codepage.
	myKeyPos := self indexOfColumn: aMyKey.
	myKeyInx := Dictionary new.
	newRecords := OrderedCollection new.
	self records do: [:record |
		myKeyInx at: (record at: myKeyPos) put: record ].
	itsKeyPos := secondTable indexOfColumn: anItsKey.
	secondTable records do: [:record |
		(myKeyInx includesKey: (record at: itsKeyPos)) ifTrue:
			[mergedRecord := (myKeyInx at: (record at: itsKeyPos)).
			mergedRecord := mergedRecord, record.
			newRecords add: mergedRecord].
		].
	self records: newRecords.
	self columns: self columns, secondTable columns.! !

!RDBTable methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
password
	password isNil ifTrue: [self password: ''].
	^password! !

!RDBTable methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
password: aString
	password := aString! !

!RDBTable methodsFor: 'private' stamp: ' 21/4/07 22:07'!
readFieldFrom: aString odbcType: aTypeString

	"read one field from string, and convert to
	appropriate object. Do a codepage converting also. Return that object"

	aString isNil ifTrue: [^nil].
	aTypeString = 'Char' ifTrue:
		[self codepage = #cp1250 ifTrue: [^WebServer returnCP852FromCP1250String: aString].
		self codepage = #iso2 ifTrue: [^WebServer returnCP852FromISO2String: aString].
		self codepage = #csz   ifTrue: [^aString].  "you cannot convert!!"
		self codepage = #'7bit' ifTrue: [^WebServer returnCP852From7BitString: aString]. 
		].
	aTypeString = 'Date' ifTrue: [^aString]. "already converted"
	^aString! !

!RDBTable methodsFor: 'table reading' stamp: ' 21/4/07 22:07'!
readRecords
	| connection session answer |
	connection := ODBCConnection new.
	[connection
		environment: self environment;
		username: self username;
		password: self password;
		connect.
	session := connection getSession.
	self getTableColumnsFrom: session.
	session 
		prepare: 'select * from "', self table, '" ';
		bindOutput: (Array new: self columns size);
		execute.
	answer := session answer.
	self records: answer upToEnd.
	self convertRecords.
	] ensure: [session disconnect. connection disconnect].! !

!RDBTable methodsFor: 'table accessing' stamp: ' 21/4/07 22:07'!
recordNr: aNumber column: aString 
	^(self records at: aNumber) at: (self indexOfColumn: aString)! !

!RDBTable methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
records
	"when aRDBTable is created from existing table, all records (rows in a table) are 
	converted and stored in this ordered collection. Fields in each record are stored in 
	another ordered collection as objects of appropriate class (texts as Strings, 
	numbers as Integers or Floats, dates as Dates)"

	records isNil ifTrue: [self records:OrderedCollection new].
	^records! !

!RDBTable methodsFor: 'private' stamp: ' 21/4/07 22:07'!
records: aCollection
	records := aCollection.! !

!RDBTable methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
table
	table isNil ifTrue: [self table: ''].
	^table! !

!RDBTable methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
table: aString
	table := aString! !

!RDBTable methodsFor: 'private' stamp: ' 21/4/07 22:07'!
typeOfColumn: aString

	1 to: self columns size do: [:inx | 
		(self columns at: inx) first asLowercase = aString asLowercase 
			ifTrue: [^(self columns at: inx) at: 2] ].
	^nil! !

!RDBTable methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
username
	username isNil ifTrue: [self username: ''].
	^username! !

!RDBTable methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
username: aString
	username := aString! !

Object subclass: #Role
	instanceVariableNames: 'name party relatedParty'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Roles'!

Role subclass: #MemberRole
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Roles'!

!MemberRole methodsFor: 'testing' stamp: ' 10/1/08 16:41'!
isEmployeeRole
	^true! !

!MemberRole methodsFor: 'testing' stamp: ' 10/1/08 16:41'!
isMemberRole
	^true! !

!MemberRole methodsFor: 'private' stamp: ' 10/1/08 16:41'!
printString
	^'aMemberRole ', self name! !

!Role methodsFor: 'testing' stamp: ' 10/1/08 16:41'!
isCustomerRole
	^false! !

!Role methodsFor: 'testing' stamp: ' 10/1/08 16:41'!
isDeputyRole
	^false! !

!Role methodsFor: 'testing' stamp: ' 10/1/08 16:41'!
isEmployeeRole
	^false! !

!Role methodsFor: 'testing' stamp: ' 10/1/08 16:41'!
isFormerEmployeeRole
	^false! !

!Role methodsFor: 'testing' stamp: ' 10/1/08 16:41'!
isJobRole
	^false! !

!Role methodsFor: 'testing' stamp: ' 10/1/08 16:41'!
isMemberRole
	^false! !

!Role methodsFor: 'testing' stamp: ' 10/1/08 16:41'!
isRole
	^true! !

!Role methodsFor: 'testing' stamp: ' 10/1/08 16:41'!
isRoleGroup
	^false! !

!Role methodsFor: 'testing' stamp: ' 10/1/08 16:41'!
isSupplierRole
	^false! !

!Role methodsFor: 'private' stamp: ' 10/1/08 16:41'!
migrateToUnicode
	"from iso8859-2"
	name notNil ifTrue: [name := name ensureUnicodeSloveneChars].! !

!Role methodsFor: 'accessing' stamp: ' 10/1/08 16:41'!
name
	^name! !

!Role methodsFor: 'accessing' stamp: ' 10/1/08 16:41'!
name: aString
	name := aString.! !

!Role methodsFor: 'accessing' stamp: ' 10/1/08 16:41'!
nameToDisplay
	^self name! !

!Role methodsFor: 'accessing' stamp: ' 10/1/08 16:41'!
party
	^party! !

!Role methodsFor: 'accessing' stamp: ' 10/1/08 16:41'!
party: aParty
	party := aParty.! !

!Role methodsFor: 'private' stamp: ' 10/1/08 16:41'!
printString
	^'aRole ', self name! !

!Role methodsFor: 'accessing' stamp: ' 10/1/08 16:41'!
relatedParty
	^relatedParty! !

!Role methodsFor: 'accessing' stamp: ' 10/1/08 16:41'!
relatedParty: aParty
	relatedParty := aParty.! !

!Role methodsFor: 'accessing' stamp: ' 10/1/08 16:41'!
uuid
	"some unique identifier. Hash for now, probably unique enough!! "
	^self hash printString! !

Role subclass: #WebSiteOwnerRole
	instanceVariableNames: 'webSite'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Roles'!

Role subclass: #WebUserRole
	instanceVariableNames: 'username password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Roles'!

!WebUserRole methodsFor: 'private' stamp: ' 10/1/08 16:41'!
printString
	^'aWebUserRole ', self name! !

Object subclass: #RoleGroup
	instanceVariableNames: 'name subroles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Roles'!

!RoleGroup commentStamp: '<historical>' prior: 0!
To group more roles or other groups together. to build a hierarchy of roles. If you use somewhere RoleGroup instead of Role, then you access all Roles together.

Instance Variables:
	name	<String> name of role group
	subroles	<Role | RoleGroup>	group of roles or other role groups

!

!RoleGroup methodsFor: 'adding-removing' stamp: ' 10/1/08 16:41'!
add: aRoleOrGroup
	(self subroles includes: aRoleOrGroup) ifFalse: [self subroles add: aRoleOrGroup]! !

!RoleGroup methodsFor: 'accessing' stamp: ' 10/1/08 16:41'!
allPersonsWithJobRoles
	"collect all persons holding job roles as part of that role group"
	^self allRoles inject: Set new into: [:set :role | set addAll: role allPersons; yourself].! !

!RoleGroup methodsFor: 'accessing' stamp: ' 10/1/08 16:41'!
allRoleGroups
	^self subroles select: [:each | each isRoleGroup]! !

!RoleGroup methodsFor: 'accessing' stamp: ' 10/1/08 16:41'!
allRoles
	^self subroles select: [:each | each isRole]! !

!RoleGroup methodsFor: 'adding-removing' stamp: ' 10/1/08 16:41'!
includes: aRoleOrGroup
	^self subroles includes: aRoleOrGroup! !

!RoleGroup methodsFor: 'private' stamp: ' 10/1/08 16:41'!
initSubroles
	subroles := OrderedCollection new.! !

!RoleGroup methodsFor: 'testing' stamp: ' 10/1/08 16:41'!
isPerson
	^false! !

!RoleGroup methodsFor: 'testing' stamp: ' 10/1/08 16:41'!
isRole
	^false! !

!RoleGroup methodsFor: 'testing' stamp: ' 10/1/08 16:41'!
isRoleGroup
	^true! !

!RoleGroup methodsFor: 'private' stamp: ' 10/1/08 16:41'!
migrateToUnicode
	"from iso8859-2"
	name notNil ifTrue: [name := name ensureUnicodeSloveneChars].! !

!RoleGroup methodsFor: 'accessing' stamp: ' 10/1/08 16:41'!
name
	^name! !

!RoleGroup methodsFor: 'accessing' stamp: ' 10/1/08 16:41'!
name: aString
	name := aString.! !

!RoleGroup methodsFor: 'accessing' stamp: ' 10/1/08 16:41'!
nameToDisplay
	^name! !

!RoleGroup methodsFor: 'private' stamp: ' 10/1/08 16:41'!
printString
	^'aRoleGroup ', self name! !

!RoleGroup methodsFor: 'adding-removing' stamp: ' 10/1/08 16:41'!
remove: aRoleOrGroup
	(self subroles includes: aRoleOrGroup) ifTrue: [self subroles remove: aRoleOrGroup]! !

!RoleGroup methodsFor: 'private' stamp: ' 10/1/08 16:41'!
subroles
	subroles isNil ifTrue: [self initSubroles].
	^subroles! !

!RoleGroup methodsFor: 'accessing' stamp: ' 10/1/08 16:41'!
uuid
	"some unique identifier. Hash for now, probably unique enough!! "
	^self hash printString! !

Object subclass: #ScheduledEvent
	instanceVariableNames: 'parent timestamp period method object block'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Core'!

!ScheduledEvent class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newOn: aScheduler
	^super new parent: aScheduler! !

!ScheduledEvent methodsFor: 'setup' stamp: ' 21/4/07 22:07'!
at: aTimestamp runBlock: aBlockClosure
	self timestamp: aTimestamp.
	self block: aBlockClosure.
	self setPeriodSingle! !

!ScheduledEvent methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
block
	"a block to be run at event"
	^block! !

!ScheduledEvent methodsFor: 'private' stamp: ' 21/4/07 22:07'!
block: anObject
	block := anObject! !

!ScheduledEvent methodsFor: 'private' stamp: ' 21/4/07 22:07'!
deepCopyNotIn: aDictionary
	^nil! !

!ScheduledEvent methodsFor: 'setup' stamp: ' 21/4/07 22:07'!
everyDayAt: aTimeOrHour  runBlock: aBlock
	| time |
	time := aTimeOrHour class == Time 
		ifTrue: [aTimeOrHour] ifFalse: [Time fromSeconds: aTimeOrHour*60*60].
	self timestamp: (Timestamp fromDate: Date today andTime: time).
	self block: aBlock.
	self periodType: #day value: time! !

!ScheduledEvent methodsFor: 'setup' stamp: ' 21/4/07 22:07'!
everyHourAt: aMinuteNumber runBlock: aBlock
	self timestamp: (Timestamp fromDate: Date today andTime: 
		(Time fromSeconds: (Time now hours * 3600) + (aMinuteNumber * 60))).
	self block: aBlock.
	self periodType: #hour value: aMinuteNumber! !

!ScheduledEvent methodsFor: 'setup' stamp: 'mivsek 8/22/2007 20:55'!
everyMinuteAt: aSecondNumber runBlock: aBlock
	self timestamp: (SpTimestamp fromDate: Date today andTime: 
		(Time fromSeconds: (Time now hours * 3600) + (Time now minutes * 60) + aSecondNumber)).
	self block: aBlock.
	self periodType: #minute value: aSecondNumber! !

!ScheduledEvent methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isDayPeriod
	^self period notNil and: [self period key = #day]! !

!ScheduledEvent methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isHourPeriod
	^self period notNil and: [self period key = #hour]! !

!ScheduledEvent methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isMinutePeriod
	^self period notNil and: [self period key = #minute]! !

!ScheduledEvent methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isPeriodic
	^(self period isNil or: [self period key = #single]) not! !

!ScheduledEvent methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
method
	"a method to be called on an object at a scheduled time"
	^method! !

!ScheduledEvent methodsFor: 'private' stamp: ' 21/4/07 22:07'!
method: aSymbol
	"a method to be called on an object at a scheduled time"
	method := aSymbol! !

!ScheduledEvent methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
object
	"object on which a method wil be called at scheduled time"
	^object! !

!ScheduledEvent methodsFor: 'private' stamp: ' 21/4/07 22:07'!
object: anObject
	object := anObject! !

!ScheduledEvent methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
parent
	^parent! !

!ScheduledEvent methodsFor: 'private' stamp: ' 21/4/07 22:07'!
parent: anObject
	parent := anObject! !

!ScheduledEvent methodsFor: 'private-periods' stamp: ' 21/4/07 22:07'!
period
	"asociation with type and parameter: #single>nil #day>aTime, ... "
	^period! !

!ScheduledEvent methodsFor: 'private-periods' stamp: ' 21/4/07 22:07'!
periodType: aSymbol value: anObject
	period := Association key: aSymbol value: anObject! !

!ScheduledEvent methodsFor: 'private' stamp: ' 21/4/07 22:07'!
printString
	^'aScheduledEvent 
	timestamp: ', self timestamp printSloString, ':', self timestamp second printString, '
	period: ', self period key printString, ' ', self period value printString, '
	method: ', self method printString, '
	object: ', self object printString, '
	block: ', self block printString! !

!ScheduledEvent methodsFor: 'private' stamp: ' 21/4/07 22:07'!
representBinaryOn: binWriter
	"for BOSS out. Blocks are not BOSSed out correctly, therefore we cannot export evens that way!!"
	"TEMPORARY, find a better solution!!"
	^0! !

!ScheduledEvent methodsFor: 'private' stamp: 'mivsek 8/22/2007 20:55'!
reschedule
	"calculate and set a new time to run, according to a period"
	self isDayPeriod ifTrue: 
		[self timestamp: (SpTimestamp fromSeconds: (self timestamp asSeconds + (60*60*24)))].
	self isHourPeriod ifTrue: 
		[self timestamp: (SpTimestamp fromSeconds: (self timestamp asSeconds + (60*60)))].
	self isMinutePeriod ifTrue: 
		[self timestamp: (SpTimestamp fromSeconds: (self timestamp asSeconds + 60))].
	self parent scheduleEvent: self.! !

!ScheduledEvent methodsFor: 'running' stamp: ' 21/4/07 22:07'!
run
	"run this event"
	self block notNil 
		ifTrue: [ [self block value] forkAt: self runPriority]
		ifFalse: [ [self object perform: self method] forkAt: self runPriority].
	self isPeriodic ifTrue: [self reschedule]! !

!ScheduledEvent methodsFor: 'running' stamp: ' 21/4/07 22:07'!
runPriority
	^Processor userBackgroundPriority! !

!ScheduledEvent methodsFor: 'private-periods' stamp: ' 21/4/07 22:07'!
setPeriodSingle
	self periodType: #single value: nil! !

!ScheduledEvent methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
timestamp
	"when this event must occur"
	^timestamp! !

!ScheduledEvent methodsFor: 'private' stamp: ' 21/4/07 22:07'!
timestamp: aTimestamp
	timestamp := aTimestamp! !

Object subclass: #Scheduler
	instanceVariableNames: 'site queue loop lock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Core'!

!Scheduler commentStamp: '<historical>' prior: 0!
Scheduler is a service for running scheduled events. Events can be single or periodic (daily, hourly, ...).
When event is triggered, its block is executed in a separate low priority process. Scheduler time resolution is 1 second. 

Example of periodic event:
	self site scheduler everyHourAt: 30 "minutes" runBlock: [Trascript show: 'half a hour!!].

Instance Variables:
	site		<anAIDASite>	
	queue	<OrderedCollection>	queue of events, waiting for execution
	loop	<Process>		loop process, every second looks into queue to run an event
	lock		<RecursionLock> to protect queue operations

!

!Scheduler class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newOn: anAIDASite
	^super new initialize; site: anAIDASite! !

!Scheduler methodsFor: 'events-single' stamp: ' 21/4/07 22:07'!
at: aTimestamp callMethod: aSymbol of: anObject
	| event |
	event := ScheduledEvent newOn: self.
	event at: aTimestamp callMethod: aSymbol of: anObject.
	self scheduleEvent: event.
	^event! !

!Scheduler methodsFor: 'events-single' stamp: ' 21/4/07 22:07'!
at: aTimestamp runBlock: aBlock
	| event |
	event := ScheduledEvent newOn: self.
	event at: aTimestamp runBlock: aBlock.
	self scheduleEvent: event.
	^event! !

!Scheduler methodsFor: 'private' stamp: ' 21/4/07 22:07'!
considerMissedEvents
	"events which we miss to run, eg. if scheduler didn't run for a while"
	self removeMissedEvents "and nothing more, for now"! !

!Scheduler methodsFor: 'events-periodic' stamp: ' 21/4/07 22:07'!
everyDayAt: aTimeOrHour runBlock: aBlock
	| event |
	event := ScheduledEvent newOn: self.
	event everyDayAt: aTimeOrHour runBlock: aBlock.
	self scheduleEvent: event.
	^event! !

!Scheduler methodsFor: 'events-periodic' stamp: ' 21/4/07 22:07'!
everyHourAt: aMinuteNumber  runBlock: aBlock
	| event |
	event := ScheduledEvent newOn: self.
	event everyHourAt: aMinuteNumber  runBlock: aBlock.
	self scheduleEvent: event.
	^event! !

!Scheduler methodsFor: 'events-periodic' stamp: ' 21/4/07 22:07'!
everyMinuteAt: aSecondNumber callMethod: aSymbol of: anObject
	| event |
	event := ScheduledEvent newOn: self.
	event everyMinuteAt: aSecondNumber callMethod: aSymbol of: anObject.
	self scheduleEvent: event.
	^event! !

!Scheduler methodsFor: 'events-periodic' stamp: ' 21/4/07 22:07'!
everyMinuteAt: aSecondNumber  runBlock: aBlock
	| event |
	event := ScheduledEvent newOn: self.
	event everyMinuteAt: aSecondNumber  runBlock: aBlock.
	self scheduleEvent: event.
	^event! !

!Scheduler methodsFor: 'events-periodic' stamp: ' 21/4/07 22:07'!
everyMonthday: aDayNumber at: aTime  runBlock: aBlock! !

!Scheduler methodsFor: 'events-periodic' stamp: ' 21/4/07 22:07'!
everyWeekday: aDaySymbol at: aTime  runBlock: aBlock! !

!Scheduler methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initQueue
	queue := OrderedCollection new.! !

!Scheduler methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize! !

!Scheduler methodsFor: 'private' stamp: 'mivsek 8/22/2007 20:55'!
insertToQueueEvent: aScheduledEvent
"	self lock critical:
		["self queue isEmpty ifTrue: [^self queue add: aScheduledEvent].
		aScheduledEvent timestamp >= self queue last timestamp 
			ifTrue: [^self queue add: aScheduledEvent].
		aScheduledEvent timestamp < self queue first timestamp 
			ifTrue: [^self queue addFirst: aScheduledEvent].
		self queue size to: 1 by: -1 do: [:inx |
			aScheduledEvent timestamp >= (queue at: inx) timestamp ifTrue: 
				[^self queue add: aScheduledEvent beforeIndex: inx+1] ]
		"]"! !

!Scheduler methodsFor: 'private' stamp: ' 21/4/07 22:07'!
isLoopRunning
	^self loop notNil! !

!Scheduler methodsFor: 'private' stamp: 'mivsek 8/22/2007 20:49'!
lock
"	lock isNil ifTrue: [lock := RecursionLock new]."
	^lock! !

!Scheduler methodsFor: 'private' stamp: ' 21/4/07 22:07'!
loop
	^loop! !

!Scheduler methodsFor: 'private' stamp: ' 21/4/07 22:07'!
loop: aProcess
	loop := aProcess! !

!Scheduler methodsFor: 'private' stamp: ' 21/4/07 22:07'!
printString
	^'aScheduler on site: ', (self site notNil ifTrue: [self site name] ifFalse: [''])! !

!Scheduler methodsFor: 'private' stamp: ' 21/4/07 22:07'!
queue
	"queue of events next one to run on first place"
	queue isNil ifTrue: [self initQueue].
	^queue! !

!Scheduler methodsFor: 'private' stamp: 'mivsek 8/22/2007 20:51'!
removeMissedEvents
	| now event |
"	self lock critical:
		["self queue isEmpty ifTrue: [^nil].
		now := SpTimestamp now asSeconds.
		[now > self queue first timestamp asSeconds] whileTrue:
			[event := self queue first.
			self queue removeFirst.
			event isPeriodic ifTrue: [event reschedule].
			self queue isEmpty ifTrue: [^nil] ]
		"]"! !

!Scheduler methodsFor: 'private' stamp: ' 21/4/07 22:07'!
scheduleEvent: aScheduledEvent
	self insertToQueueEvent: aScheduledEvent.! !

!Scheduler methodsFor: 'private' stamp: ' 21/4/07 22:07'!
schedulerPriority
	^Processor userInterruptPriority! !

!Scheduler methodsFor: 'private' stamp: ' 21/4/07 22:07'!
site
	^site! !

!Scheduler methodsFor: 'private' stamp: ' 21/4/07 22:07'!
site: anAIDASite
	site := anAIDASite! !

!Scheduler methodsFor: 'start/stop' stamp: ' 21/4/07 22:07'!
start
	self considerMissedEvents.
	self startLoop! !

!Scheduler methodsFor: 'private' stamp: 'mivsek 8/22/2007 20:48'!
startLoop
	self isLoopRunning ifTrue: [self stopLoop].
	self loop: 
		([ [true] whileTrue: 
			[| event now |
"			self lock critical: 
				["self queue notEmpty ifTrue:
					[event := self queue first. now := SpTimestamp now asSeconds.
				 	now = event timestamp asSeconds
						ifTrue:  [event run. self queue removeFirst]
						ifFalse: [now > event timestamp asSeconds  
							ifTrue: [self removeMissedEvents]] 
				"]" ].
			(Delay forSeconds: 1) wait]
		] forkAt: self schedulerPriority)! !

!Scheduler methodsFor: 'start/stop' stamp: ' 21/4/07 22:07'!
stop
	self stopLoop! !

!Scheduler methodsFor: 'private' stamp: ' 21/4/07 22:07'!
stopLoop
	self loop notNil ifTrue: [self loop terminate. self loop: nil].! !

Object subclass: #SwazooAida
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Core'!

!SwazooAida commentStamp: 'mivsek 1/10/2008 22:03' prior: 0!
SwazooAida is here just for easier start and stop your Aida/Web application server

	- SwazooAida demoStart             "will create and start a demo site http://localhost:8888"
	- SwazooAida startOn: portNum   "will start a site on that port on all IP interfaces, any host"
								      "sie will be created if doesn't exist yet"
	- SwazooAida start    "will start all existing sites"
	- SwazooAida stop     "will stop all running sites"!

!SwazooAida class methodsFor: 'start/stop' stamp: ' 10/1/08 22:01'!
demoStart             
	"will create (if not yet) and start a demo site on http://localhost:8888"
	SwazooServer aidaStart! !

!SwazooAida class methodsFor: 'start/stop' stamp: ' 10/1/08 22:01'!
start    
	"will start all existing sites, both Aida ones and others on  Swazoo web server"
	SwazooServer start! !

!SwazooAida class methodsFor: 'start/stop' stamp: ' 10/1/08 22:01'!
startOn: portNum   
	"will start a site on that port on all IP interfaces, any host"
      "site will be created if doesn't exist yet"
	SwazooServer aidaStartOn: portNum! !

!SwazooAida class methodsFor: 'start/stop' stamp: ' 10/1/08 22:01'!
stop    
	"will stop all running sites, both Aida ones and other on Swazoo web server"
	SwazooServer stop! !

Object subclass: #URLResolver
	instanceVariableNames: 'site allWebPages allURLLinks counters totalCounter randomGen'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Core'!

!URLResolver class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
default
	^AIDASite default urlResolver! !

!URLResolver class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newOn: anAIDASite
	^super new 
		initialize; 
		site: anAIDASite! !

!URLResolver methodsFor: 'request counting' stamp: ' 21/4/07 22:07'!
addCountsFromDictionary: aCountDictionary

| object origCounter stopUrls |
	stopUrls := Col collect: [:assoc | assoc notNil ifTrue: [assoc key] ].
	aCountDictionary keysAndValuesDo: [:url :counter |
	(stopUrls includes: url asLowercase) ifFalse:
		[object := self ooRefFromURL: url.
		object notNil ifTrue: 
			[origCounter := self counterFor: object.
			origCounter addCounter: counter ] ]
		ifTrue: ["self halt"] ]

"URLResolver default addCountsFromDictionary: Janko"
"
| boss |
boss := (BinaryObjectStorage onOld: '..\images\janko\LogDict.boss' asFilename readStream).
Janko := boss next.
boss close
"! !

!URLResolver methodsFor: 'adding-removing' stamp: 'janko 8/2/2008 14:32'!
addObject: anObject withURL: aString
	"add an object if not already exist and add url to the collection of urls for that object.
	If aString url point to some other object already, then remove url from that other object"
	"warning, if web elements (such as web apps) will be registered"
	self isWebElement: anObject.
	anObject isNil ifTrue: [^nil]. "do not add nil objects!!"
	self site critical: 
		[(self allWebPages at: anObject ifAbsentPut: [OrderedCollection new])
				add: aString asLowercase.
		(self allURLLinks includesKey: aString asLowercase) ifTrue:
			[(self allWebPages at: (self allURLLinks at: aString asLowercase)	
				ifAbsentPut: [OrderedCollection new])
					remove: aString asLowercase ifAbsent: [] ].
		self allURLLinks at: aString asLowercase put: anObject ].! !

!URLResolver methodsFor: 'static pages' stamp: ' 21/4/07 22:07'!
allHTMLPageUrls
	"find all .htm and .html pages in directory hierarchy starting at aWebServer homeDirectory.
	return as collection of relative url links"

	^self allHTMLPageUrlsIn: self site homeDirectory.

"URLResolver default allHTMLPageUrls"! !

!URLResolver methodsFor: 'static pages' stamp: ' 21/4/07 22:07'!
allHTMLPageUrlsIn: aDirectoryString
	"find all .htm and .html pages in directory hierarchy starting at a specified directory and all 	subdirectories.	Return as collection of relative url links"

	| collection dir |
	dir := aDirectoryString.
	dir last ~= $\ ifTrue: [dir := dir, '\'].
	dir asFilename exists ifFalse: [^#()].
	collection := OrderedCollection new.
	aDirectoryString asFilename directoryContents do: [:each | 
		(dir, each) asFilename isDirectory
			ifTrue: [collection addAll: (self allHTMLPageUrlsIn: (dir, each))]
			ifFalse: 
				[('.htm' match: each asFilename extension) ifTrue:
					[collection add: (self urlForFile: each asString directory: aDirectoryString)] ]
		].
	^collection.

"URLResolver default allHTMLPageUrlsIn: 'h:\wwwroot\planid' "! !

!URLResolver methodsFor: 'static pages' stamp: ' 21/4/07 22:07'!
allHTMLPagesSize
	"sums sizes of all static pages"

	| fname sum |
	sum := 0.
	self allHTMLPageUrls do: [:url | 
		fname := self site homeDirectory, url.
		fname := fname copyReplaceAll: '/' with: '\'.
		fname asFilename exists 
			ifTrue: [sum := sum + fname asFilename fileSize] ].
	^sum

"URLResolver default allHTMLPagesSize"! !

!URLResolver methodsFor: 'request counting' stamp: ' 21/4/07 22:07'!
allPagesAndCounts

	"return total counts for all objects that have counters, most counted first"

	| collection |
	collection := OrderedCollection new.
	self counters keysAndValuesDo: [:object :counter |
		collection add: (Array
			with: object
			with: counter total)].
	^SortedCollection
		withAll: collection
		sortBlock: [:a :b | (a at: 2) > (b at: 2)]

"URLResolver default allPagesAndCounts"! !

!URLResolver methodsFor: 'private' stamp: ' 21/4/07 22:07'!
allURLLinks
	"return a dictionary with url as key and reference to an object with that url as value"
	^allURLLinks! !

!URLResolver methodsFor: 'request counting' stamp: ' 21/4/07 22:07'!
allUrlsAndCounters
	"return copies of all counters as dictionary with url as key"
	| dictionary |
	dictionary := Dictionary new.
	self counters keysAndValuesDo: [:object :counter |
		dictionary 
			at: (self halfUrlFor: object)
			put: counter deepCopy].
	^dictionary

"URLResolver default allUrlsAndCounters"
"
(BinaryObjectStorage onNew: 'counters.boss' asFilename writeStream)
	nextPut: URLResolver default allUrlsAndCounters;
	close
"! !

!URLResolver methodsFor: 'private' stamp: ' 21/4/07 22:07'!
allWebPages
	" return a dictionary with object reference as key and one on more urls for that object as value.
	first  is default one, when urls are dinamicaly generated"
	^allWebPages! !

!URLResolver methodsFor: 'private' stamp: ' 21/4/07 22:07'!
autoUrlFor: anObject
	"use prefered url if possible. If that url already exist, add -a, -b, ... to url and repeat"
	| url extension lastPoint |
	url := anObject preferedUrl. url isNil ifTrue: [^self randomUrlFor: anObject].
	[(self ooRefFromURL: url) notNil] whileTrue: "already exist"
		[lastPoint := url lastIndexOf: $. . lastPoint = 0 ifTrue: [lastPoint := url size+1].
		extension := url copyFrom: lastPoint to: url size.
		url := url copyReplaceAll: extension with: ''. 
		url isEmpty ifTrue:  [^self randomUrlFor: anObject].
		(url size > 1 and: [(url at: url size-1) = $-]) "already have an extension char!!"
			ifTrue: 
				[url last = $z ifTrue: [^self randomUrlFor: anObject]. "only to $z, then back to random!!"
				url := (url copyFrom: 1 to: url size-1), (String with: (url last asInteger + 1) asCharacter)]
			ifFalse: [url := url, '-a'].
		url := url, extension].
	^url

"URLResolver default halfUrlFor: (Document new id: 'SOP-0111-02').
URLResolver default autoUrlFor: (Document new id: 'SOP-0111-02')"! !

!URLResolver methodsFor: 'adding-removing' stamp: ' 21/4/07 22:07'!
changeToPreferedURL: anObject
 	"remove all existing urls and make this as only one"
	"BEWARE, all ols urls will be erased, url references can be breaked!! "
	anObject preferedUrl isNil ifTrue: [^nil].
	self removeObject: anObject.
	self defaultURL: anObject preferedUrl forObject: anObject.! !

!URLResolver methodsFor: 'adding-removing' stamp: ' 21/4/07 22:07'!
changeToURL: aString forObject: anObject
 	"remove all existing urls and make this as only one"
	"BEWARE, all ols urls will be erased, url references can be breaked!! "
	self removeObject: anObject.
	self defaultURL: aString forObject: anObject.! !

!URLResolver methodsFor: 'private' stamp: ' 21/4/07 22:07'!
correctUrlEncoding
	"make sure all urls are decodedHTTP, without %20 etc"
	self allURLLinks keys do: [:key | (key decodedHTTP ~= key) 
		ifTrue: [self allURLLinks at: key decodedHTTP put: (self allURLLinks at: key). 
			self allURLLinks removeKey: key] ].
	self allWebPages values do: [:coll |
	coll copy do: [:each | (each decodedHTTP ~= each) 
		ifTrue: [coll at: (coll indexOf: each) put: each decodedHTTP] ] ].

"
self allURLLinks keys select: [:key | (key decodedHTTP ~= key)].
self allWebPages associations select: [:assoc |
	assoc value contains: [:each | (each decodedHTTP ~= each)] ].
"! !

!URLResolver methodsFor: 'request counting' stamp: ' 21/4/07 22:07'!
counterFor: anObject
	anObject isNil ifTrue: [^nil].
	(self counters includesKey: anObject)
		ifFalse: [self resetCounterFor: anObject].
	^self counters at: anObject! !

!URLResolver methodsFor: 'request counting' stamp: ' 21/4/07 22:07'!
counterForUrl: anUrlString

	"return a counter for object with specified url. Nil if not exist"

	| object  |
	object := self ooRefFromURL: anUrlString.
	object isNil ifTrue: [^nil].
	^self counterFor: object.

"URLResolver default counterForUrl: '/osn/g_l-vse.htm'"
"WebServer default totalCounter"
"WebServer default pagesCounter"! !

!URLResolver methodsFor: 'private' stamp: ' 21/4/07 22:07'!
counters

	"dictionary of web counters for every object, registered in URLResolver"

	counters isNil ifTrue: [self initCounters].
	^counters.! !

!URLResolver methodsFor: 'adding-removing' stamp: 'janko 8/2/2008 14:20'!
defaultURL: aString forObject: anObject
	"set url as default url for that object. This url is then used in dynamic url creation. If an object 
	does not already exist, then create a new entry first"
	| urls first defInx |
	self addObject: anObject withURL: aString.  "just in case object is new one"
	"swap old default with new default url on a first place in collection"
	self site critical: 
		[urls := allWebPages at: anObject.
		first := urls first.
		defInx := urls indexOf: aString asLowercase.
		urls at: 1 put: (urls at: defInx).
		urls at: defInx put: first].! !

!URLResolver methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
existURL: aString
	"return true if URL has connection to some object"
	^self allURLLinks includesKey: aString asLowercase! !

!URLResolver methodsFor: 'accessing' stamp: 'mivsek 8/22/2007 17:21'!
fileProxyForURL: aString
	" Try to find a html file for a specified URL and make aHTMLFileProxy. 
	Returns nil if such a file does not exist. Index content in default WebIndex"
	| add home fname proxy |
	aString isEmpty ifTrue: [^nil].
	(aString last = $/) ifTrue: [add := 'index.htm'] ifFalse: [add := ''].
	home := self site homeDirectory.
	(home last = $/ ) | (home last = $\ ) ifTrue: [home := home copyFrom: 1 to: home size-1].
	fname := home, aString, add.
	SpEnvironment onWindows ifTrue: [fname := fname copyReplaceAll: '/' with: '\'].
	(SpFilename named: fname) exists ifFalse:
		[fname := fname, 'l'.
		(SpFilename named: fname) exists ifFalse: [^nil] ].
	proxy := FileProxy from: fname onSite: self site.
	proxy indexContent.
	^proxy! !

!URLResolver methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
fullUrlEncodedSpacesFor: anObject on: aSession
	"change spaces to %20"
	^(self fullUrlFor: anObject on: aSession) copyReplaceAll: ' ' with: '%20'! !

!URLResolver methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
fullUrlFor: anObject on: aSession
	"find or make a path for anObject and compose full url together with server address and 
	query string, eg: http://www.eranova.si/welcome.html?view=brief"
	^self 
		fullUrlFromPath: (self halfUrlFor: anObject)
		andParms: (aSession notNil ifTrue: [aSession parms] ifFalse: [#()])
		on: aSession! !

!URLResolver methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
fullUrlFromPath: aPathString andParms: aParmsDictionary on: aSession
	"compose full url: server address, document path and additional parameters in query string, 
	for instance: http://www.eranova.si/welcome.html?view=brief"
	| url |
	url := self urlFromHalfUrl: aPathString on: aSession.
	^self halfUrlFromPath: url andParms: aParmsDictionary on: aSession. "not very nice, i know ..."
	
"
| parms |
parms := Dictionary new.
parms at: 'view' put: 'brief'. parms at: 'id' put: '12345'. 
URLResolver default fullUrlFromPath: '/dzs/panorama.html' andParms: parms on: WebSession new
"! !

!URLResolver methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
halfUrlFor: anObject
	"finds or composes an URL reference to that object . If the URL is new one, then it is written to both dictionaries for later lookup. If resolution is unsuccessfull then returns nil. If anObject dont have an url yet, url part for them is automatically generated, for example:  /objecto1638948.html"
	| newUrl urls |
	urls := (self allWebPages at: anObject ifAbsent: [
	      newUrl := ((anObject isKindOf: WebPage)  and: [anObject name notNil] )
			ifTrue: ['/', anObject name asLowercase,'.html']			
			ifFalse: [self autoUrlFor: anObject].
		self addObject: anObject withURL: newUrl.
		OrderedCollection with: newUrl]).
	"just in case object has an empty collection of urls"
	(urls isNil or: [urls isEmpty]) ifTrue: 
		[self removeObject: anObject. ^self halfUrlFor: anObject].
	^urls at: 1.! !

!URLResolver methodsFor: 'accessing' stamp: 'janko 3/21/2008 16:51'!
halfUrlFromPath: aPathString andParms: aParmsDictionary on: aSession
	"compose half url: /document path and additional parameters in query string, for instance: 	/welcome.html?view=brief"
	| url parmsString newValue |
	url := aPathString.
	aParmsDictionary size > 0 ifTrue:
		[parmsString := ''.
		aParmsDictionary keysAndValuesDo: [:name :value |
			(value isKindOf: Set) ifTrue:[newValue := value asArray at: 1]ifFalse:[newValue := value].
			parmsString := parmsString, 
				(AIDASite convertToWebString: name asString), '=', 
				(AIDASite convertToWebString: newValue asString), '&'].  "value"
		url := url, '?', (parmsString copyFrom: 1 to: (parmsString size - 1))].
	^url

"
| parms |
parms := Dictionary new.
parms at: 'view' put: 'brief'. parms at: 'id' put: '12345'. 
URLResolver default halfUrlFromPath: '/dzs/panorama.html' andParms: parms on: WebSession new
"! !

!URLResolver methodsFor: 'request counting' stamp: ' 21/4/07 22:07'!
incCounterFor: anObject
	"increment a web counter for that object. Only text/html object are counted
	Don't count temporary objects"
	(anObject contentType = 'text/html') ifFalse: [^nil].
	self site critical: 
		[(self counters includesKey: anObject) ifFalse: [self resetCounterFor: anObject].
		(self counters at: anObject) incCounter.
		"self totalCounter incCounter   **aWebServer already counts!! "]! !

!URLResolver methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initCounters
	"init a counters dictionary and open a new counter for each already registered object"
	counters := Dictionary new. 
	self allWebPages keys do: [:object | self resetCounterFor: object].! !

!URLResolver methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initURLLinks
	allURLLinks := Dictionary new.! !

!URLResolver methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initWebPages
	allWebPages := IdentityDictionary new.! !

!URLResolver methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	self initWebPages.
	self initURLLinks.
	self initCounters.! !

!URLResolver methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isRootURL: aString

	^aString = '/'! !

!URLResolver methodsFor: 'private' stamp: ' 21/4/07 22:07'!
isWebElement: anObject


	" test if anObject is a web element, such as web app and write warning to the transcript"
	
	(anObject isKindOf: WebElement) ifTrue:
		[Transcript cr; show: '*** WARNING: URLResolver detected a ', 
		anObject class printString, '  will be registered ***'; cr.

		"to detect from where this request comes"
		"self halt"
		].! !

!URLResolver methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
methodImageForURL: aString 
	"maybe a method image can be done from current WebStyle?"
	|  name selector |
	('/img/*' match: aString) ifFalse: [^nil].
	name := (aString readStream upTo: $/ ; upTo: $/; upTo: $. ) asLowercase.
	selector := self site style class allSelectors 
		detect: [:each | each asString asLowercase = name] ifNone: [^nil].
	^WebMethodImage fromMethod: selector on: self site style contentType: 'image/gif' site: self site

"URLResolver default methodImageForURL: '/img/eranovaLogoGif.gif' "! !

!URLResolver methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
objectsOfClass: aClass
	^self allWebPages keys select: [:object | object class = aClass ]

"URLResolver default objectsOfClass: WebMethodImage"! !

!URLResolver methodsFor: 'accessing' stamp: 'janko 8/2/2008 18:04'!
ooRefFromURL: aString
	" finds a reference to WebPage, specified in a URL string. If not yet known, try to 
	find it as a html file. if not found, search methods in current WebStyle too.."
	| object |
	^self allURLLinks at: aString asLowercase ifAbsent: 
		[(self isRootURL: aString) ifTrue: [^self site defaultUserService].
		object := self fileProxyForURL: aString.
		object isNil ifTrue: [object := self methodImageForURL: aString].
		object notNil ifTrue: [self addObject: object withURL: aString].
		^object].
	

"URLResolver default ooRefFromURL: '/osn/g_l-vse.htm' "! !

!URLResolver methodsFor: 'private' stamp: ' 21/4/07 22:07'!
randomGen
	randomGen isNil ifTrue: [self randomGen: Random new].
	^randomGen! !

!URLResolver methodsFor: 'private' stamp: ' 21/4/07 22:07'!
randomGen: anObject
	randomGen := anObject! !

!URLResolver methodsFor: 'private' stamp: ' 21/4/07 22:07'!
randomUrlFor: anObject
	| number ext url |
	number := (self randomGen next * 10000000) asInteger.
	ext := self site mimeMap extensionForType: anObject contentType.
	ext isNil ifTrue: [ext := 'html'].
	url := '/object/o', number printString, '.', ext.
	^(self ooRefFromURL: url) isNil  "avoid duplicates!!"
		ifTrue: [url]
		ifFalse: [self randomUrlFor: anObject]! !

!URLResolver methodsFor: 'static pages' stamp: ' 21/4/07 22:07'!
refreshAllHTMLPages
	""

	| count |
	count := 1.
	self allWebPages keys do: [:object |
		(object isKindOf: FileProxy) ifTrue:	 
			[count \\ 10 = 0 ifTrue: [Transcript cr; show: count printString].
			object refreshContent.
			count := count + 1] ].

"URLResolver default refreshAllHTMLPages"! !

!URLResolver methodsFor: 'static pages' stamp: ' 21/4/07 22:07'!
registerAllHTMLPages
	"if not already, then register all static html pages in directory hierarchy starting
	at aWebServer homeDirectory. Opan also counters for them"

	| count proxy |
	count := 1.
	self allHTMLPageUrls do: [:url | 
		count \\ 10 = 0 ifTrue: [Transcript cr; show: count printString].
		proxy := self ooRefFromURL: url.
		self counterFor: proxy.
		count := count + 1].

"URLResolver default registerAllHTMLPages"! !

!URLResolver methodsFor: 'indexing' stamp: ' 21/4/07 22:07'!
reindexAllObjects

	"all text/html objects are reindexed"
	self allWebPages keysDo: [:object |
		(object contentType = 'text/html') ifTrue:
			[self site index indexObject: object] ]

"URLResolver default reindexAllObjects"! !

!URLResolver methodsFor: 'indexing' stamp: ' 21/4/07 22:07'!
reindexNonIndexedObjects
	"all text/html objects are reindexed"
	self allWebPages keysDo: [:object |
		(object contentType = 'text/html') ifTrue:
			[(self site index indexedObjects includesKey: object) ifFalse:
				[self site index indexObject: object] ] ]

"URLResolver default reindexNonIndexedObjects"! !

!URLResolver methodsFor: 'static pages' stamp: ' 21/4/07 22:07'!
releaseAllHTMLPages

	self allWebPages keys do: [:obj |
		(obj isKindOf: FileProxy) ifTrue:
	 		[obj releaseContent] ]

"URLResolver default releaseAllHTMLPages"! !

!URLResolver methodsFor: 'static pages' stamp: ' 21/4/07 22:07'!
releaseAllImages

	self allWebPages keys do: [:obj |
		((obj isKindOf: FileProxy) and: ['*image*' match: obj contentType])
			ifTrue: [obj releaseContent] ]

"URLResolver default releaseAllImages"! !

!URLResolver methodsFor: 'static pages' stamp: ' 21/4/07 22:07'!
releaseAllStaticPages

	self allWebPages keysDo: [:object | 
		(object isKindOf: FileProxy) ifTrue: [object releaseContent] ]

"URLResolver default releaseAllStaticPages"! !

!URLResolver methodsFor: 'request counting' stamp: ' 21/4/07 22:07'!
removeCounterFor: anObject


	(self counters includesKey: anObject)
		ifTrue: [self counters removeKey: anObject]! !

!URLResolver methodsFor: 'request counting' stamp: ' 21/4/07 22:07'!
removeNonHTMLCounters

	self counters keys do: [:object |
		object contentType ~= 'text/html' ifTrue:
			[self removeCounterFor: object] ]

"URLResolver default removeNonHTMLCounters"! !

!URLResolver methodsFor: 'adding-removing' stamp: ' 21/4/07 22:07'!
removeObject: anObject
	"remove object and all its URLs. Returns true if object existed and false if not. 
	Remove also its web counter"
	| urls |
	self site critical:
		[urls := (self allWebPages at: anObject ifAbsent: [^nil]).
		self allWebPages removeKey: anObject.
		urls notNil ifTrue: [urls do: [: each | self allURLLinks removeKey: each ifAbsent: [] ] ].
		self counters removeKey: anObject ifAbsent: [] ].
	^true! !

!URLResolver methodsFor: 'adding-removing' stamp: ' 21/4/07 22:07'!
removeObjectWithURL: aString

"remove object and all its URLs. Returns true if object existed and false if not"

	| object |
	object := allURLLinks at: aString asLowercase ifAbsent: [^false].
	^self removeObject: object.! !

!URLResolver methodsFor: 'adding-removing' stamp: ' 21/8/07 09:38'!
removeObjectsOfClass: aClass
	AIDARoot critical:
		[(self objectsOfClass: aClass) do: [:object | self removeObject: object] ].

"URLResolver default removeObjectsOfClass: WebMethodImage"
"URLResolver default allWebPages size 10934"! !

!URLResolver methodsFor: 'adding-removing' stamp: ' 21/4/07 22:07'!
removeObjectsOfClassNamed: aString
	"good for removing Obsolete classes!!"
	AIDARoot critical:
		[self allWebPages keys do: [:object | 
			object class name= aString ifTrue: [self removeObject: object]] ].

"URLResolver default removeObjectsOfClassNamed: WebMethodImage"
"URLResolver default allWebPages size 10934"! !

!URLResolver methodsFor: 'adding-removing' stamp: ' 21/4/07 22:07'!
removeURL: aString
	"remove this URL and also object, if this url was the last one pointed to object Returns 
	true if url existed and false if not. Also remove web counter"
	| object |
	self site critical:
		[object := (self allURLLinks at: aString asLowercase ifAbsent: [^nil]).
		(self allWebPages at: object) remove: aString asLowercase ifAbsent: [].
		(self allWebPages at: object) isEmpty ifTrue: [self allWebPages removeKey: object]. 
		self allURLLinks removeKey: aString asLowercase.
		self counters removeKey: object ifAbsent: [] ].
	^true! !

!URLResolver methodsFor: 'request counting' stamp: ' 21/4/07 22:07'!
resetAllCounters

	self counters values do: [:each | each reset].

"URLResolver default resetAllCounters"! !

!URLResolver methodsFor: 'request counting' stamp: ' 21/4/07 22:07'!
resetCounterFor: anObject
	"reset counters for specified object. If not yet exist, initialize it first"
	anObject isNil ifTrue: [^nil].
	self site critical:
		[(self counters includesKey: anObject) ifFalse: 
			[self counters at: anObject put: WebCounter new].
		(self counters at: anObject) reset]! !

!URLResolver methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
scanForURLsFrom: aWebPage

"start looking for all WebLinks in pages starting from aWebPage and converting them to the URL links in instance variables allWebPages and allURLLinks."! !

!URLResolver methodsFor: 'private' stamp: ' 21/4/07 22:07'!
site
	"a parent site with this url resolver"
	^site! !

!URLResolver methodsFor: 'private' stamp: ' 21/4/07 22:07'!
site: anAIDASite
	site := anAIDASite.! !

!URLResolver methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
start! !

!URLResolver methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
stop
	self randomGen: nil! !

!URLResolver methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
urlFor: anObject on: aSession

	"finds or composes an URL reference to the given WebPage or any object capable to represent itself as a WebPage. If the URL is new one, then it is written to both dictionaries for later lookup. If resolution is unsuccessfull then returns nil. The second argument is a WebPage where an OO link to resolve exist. If this argument is nil, then an URL must already exist, eg. automatic composition is not possible.
If anObject is not WebPge or has no name, then the url part for them is automatically generated, for example: http://pu.tris-a.si/cgi-bin/stweb?/demo/overview/o1638948.htm"

	^self urlFromHalfUrl: (self halfUrlFor: anObject on: aSession) on: aSession! !

!URLResolver methodsFor: 'static pages' stamp: ' 21/4/07 22:07'!
urlForFile: aFilenameString directory: aDirectoryString

	| path dir |
	dir := aDirectoryString.
	dir last ~= $\ ifTrue: [dir := dir, '\'].
	path := dir, aFilenameString.
	path := path copyReplaceAll: self site homeDirectory with: ''.
	^path copyReplaceAll: '\' with: '/'

"URLResolver default urlForFile: 'index.htm' directory: 'h:\wwwroot\planid\osn' "! !

!URLResolver methodsFor: 'accessing' stamp: 'mivsek 1/10/2008 19:18'!
urlFromHalfUrl: aString on: aSession
	| portString protocolString |
	portString := (aSession notNil and: [aSession isEncrypted]) 
		ifFalse: [(self site port = 80) ifTrue: [''] ifFalse: [':', self site port printString] ]
		ifTrue: [(self site sslPort = 443) ifTrue: [''] ifFalse: [':', self site sslPort printString] ].
	protocolString := (aSession notNil and: [aSession isEncrypted])  
		ifFalse: ['http://'] ifTrue: ['https://'].
	^protocolString, self site host asLowercase, portString, aString.! !

Object subclass: #VersionSpec
	instanceVariableNames: 'object number current parent next'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Support'!

!VersionSpec commentStamp: '<historical>' prior: 0!
VersionSpec defines a versioned object by its number and position in version chain.

Instance Variables:
	number	<String>	 number of that version. Integer by default, but it can be any string
	current	<Boolean> true, if this version is current, that is, most important, released, etc.
	parent	<Object> parent, that is, previous version of that object, nil if noone
	next		<Object> next version of that object, nil if noone

!

!VersionSpec class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
firstFor: anObject
	"for a first version of an object"
	^super new
		object: anObject;
		setDefaultNumber;
		setCurrent! !

!VersionSpec class methodsFor: 'private' stamp: ' 21/4/07 22:07'!
new
	^self shouldNotImplement! !

!VersionSpec class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newFromParent: anOldObject for: aNewObject
	| newSpec |
	newSpec := super new
		object: aNewObject;
		parent: anOldObject;
		setIncrementedNumberFromParent;
		clearCurrent.
	aNewObject version: newSpec.
	anOldObject version next: aNewObject.
	^newSpec! !

!VersionSpec methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allVersionSpecs
	^self olderVersionSpecs, (Array with: self), self newerVersionSpecs! !

!VersionSpec methodsFor: 'private' stamp: ' 21/4/07 22:07'!
clearCurrent
	current := false! !

!VersionSpec methodsFor: 'private' stamp: ' 21/4/07 22:07'!
current
	"this version current one? Current version object have a direct url, without 'version=' in query part"
	^current! !

!VersionSpec methodsFor: 'private' stamp: ' 21/4/07 22:07'!
current: aBoolean
	current := aBoolean! !

!VersionSpec methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
currentVersionSpec
	"find a version spec of current object in a chain"
	self isCurrent ifTrue: [^self].
	^self olderVersionSpecs detect: [:each | each isCurrent] ifNone: 
		[^self newerVersionSpecs detect: [:each | each isCurrent] 
			ifNone: [self error: 'no current version!!'] ].! !

!VersionSpec methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isCurrent
	^self current! !

!VersionSpec methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isNewest
	^self next isNil! !

!VersionSpec methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isOldest
	^self parent isNil! !

!VersionSpec methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
newerVersionSpecs
	"return all version specs of that object, newer than this one"
	self isNewest ifTrue: [^#()].
	^OrderedCollection new 
		add: self next version; addAll: self next version newerVersionSpecs;
		yourself! !

!VersionSpec methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
next
	"next version of an object in version chain, if any"
	^next! !

!VersionSpec methodsFor: 'private' stamp: ' 21/4/07 22:07'!
next: anObject
	next := anObject! !

!VersionSpec methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
number
	"version number, integer by default, but it can be any string"
	^number! !

!VersionSpec methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
number: aString
	number := aString! !

!VersionSpec methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
object
	"object for which is this version spec"
	^object! !

!VersionSpec methodsFor: 'private' stamp: ' 21/4/07 22:07'!
object: anObject
	object := anObject! !

!VersionSpec methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
olderVersionSpecs
	"return all version specs of that object, older than this one"
	self isOldest ifTrue: [^#()].
	^OrderedCollection new 
		addAll: self parent version olderVersionSpecs; add: self parent version;
		yourself! !

!VersionSpec methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
otherVersionSpecs
	"older and newer versions, if any"
	^self olderVersionSpecs, self newerVersionSpecs! !

!VersionSpec methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
parent
	"older version of an object in version chain, if any. It is a parent of that object, 
	because it is derived from it"
	^parent! !

!VersionSpec methodsFor: 'private' stamp: ' 21/4/07 22:07'!
parent: anObject
	parent := anObject! !

!VersionSpec methodsFor: 'private' stamp: ' 21/4/07 22:07'!
printString
	^'aVersionSpec version: ', self number! !

!VersionSpec methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
setCurrent
	self current: true.
	self otherVersionSpecs do: [:each | each clearCurrent].! !

!VersionSpec methodsFor: 'initalize-release' stamp: ' 21/4/07 22:07'!
setDefaultNumber
	self isOldest 
		ifTrue: [self number: '1'] 
		ifFalse: [self setIncrementedNumberFromParent]! !

!VersionSpec methodsFor: 'initalize-release' stamp: ' 21/4/07 22:07'!
setIncrementedNumberFromParent
	"for now, later it should increment last number in string"
	self number: (self parent version number asInteger + 1) printString! !

!VersionSpec methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
versionSpecWithNumber: aString
	"find spec  with that version number"
	self number = aString ifTrue: [^self].
	^self olderVersionSpecs detect: [:each | each number = aString] ifNone: 
		[^self newerVersionSpecs detect: [:each | each number = aString] ifNone: [nil] ].! !

Object subclass: #VersionedObject
	instanceVariableNames: 'version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Support'!

VersionedObject subclass: #VersionedExample
	instanceVariableNames: 'title body'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Tests'!

!VersionedExample methodsFor: 'accessing' stamp: ' 21/4/07 22:08'!
body
	^body! !

!VersionedExample methodsFor: 'accessing' stamp: ' 21/4/07 22:08'!
body: anObject
	body := anObject! !

!VersionedExample methodsFor: 'copying' stamp: ' 21/4/07 22:08'!
copyContentsTo: anObject
	anObject title: self title copy.
	anObject body: self body copy.! !

!VersionedExample methodsFor: 'printing' stamp: ' 21/4/07 22:08'!
printString
	^'aVersionedExample ', 
		(self title notNil ifTrue: [self title] ifFalse: ['']), ' ', 
		(self body notNil ifTrue: [self body] ifFalse: [''])! !

!VersionedExample methodsFor: 'accessing' stamp: ' 21/4/07 22:08'!
title
	^title! !

!VersionedExample methodsFor: 'accessing' stamp: ' 21/4/07 22:08'!
title: anObject
	title := anObject! !

!VersionedObject methodsFor: 'accessing-versions' stamp: ' 21/4/07 22:07'!
allVersions
	^self version allVersionSpecs collect: [:each | each object]! !

!VersionedObject methodsFor: 'copying' stamp: ' 21/4/07 22:07'!
asNewVersion
	"return a new version of that object with all its current contents"
	| new |
	new := self class new.
	new version: (VersionSpec newFromParent: self for: new).
	self copyContentsTo: new.
	^new! !

!VersionedObject methodsFor: 'copying' stamp: ' 21/4/07 22:07'!
copyContentsTo: anObject
	"override in subclases to copy contents to a new object"! !

!VersionedObject methodsFor: 'accessing-current' stamp: ' 21/4/07 22:07'!
currentVersion
	"find a current version of object in a version chain"
	^self version currentVersionSpec object! !

!VersionedObject methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initVersion
	self version: (VersionSpec firstFor: self)! !

!VersionedObject methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isCurrentVersion
	^self version isCurrent! !

!VersionedObject methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isNewestVersion
	^self version isNewest! !

!VersionedObject methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isOldestVersion
	^self version isOldest! !

!VersionedObject methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isOnlyVersion
	^self version isOldest & self version isNewest! !

!VersionedObject methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isVersionedObject
	^true! !

!VersionedObject methodsFor: 'accessing-versions' stamp: ' 21/4/07 22:07'!
newerVersions
	^self version newerVersionSpecs collect: [:each | each object]! !

!VersionedObject methodsFor: 'accessing-versions' stamp: ' 21/4/07 22:07'!
newestVersion
	self isNewestVersion ifTrue: [^self].
	^self newerVersions last! !

!VersionedObject methodsFor: 'accessing-versions' stamp: ' 21/4/07 22:07'!
nextVersion
	"next version of an object in version chain, if any"
	^self version next! !

!VersionedObject methodsFor: 'accessing-versions' stamp: ' 21/4/07 22:07'!
olderVersions
	^self version olderVersionSpecs collect: [:each | each object]! !

!VersionedObject methodsFor: 'accessing-versions' stamp: ' 21/4/07 22:07'!
oldestVersion
	self isOldestVersion ifTrue: [^self].
	^self olderVersions first! !

!VersionedObject methodsFor: 'accessing-versions' stamp: ' 21/4/07 22:07'!
parentVersion
	"older version of an object in version chain, if any. It is a parent of that object, 
	because it is derived from it"
	^self version parent! !

!VersionedObject methodsFor: 'accessing-current' stamp: ' 21/4/07 22:07'!
setCurrentVersion
	"set this object as current version, which has always the same identity "
	| other |
	self isCurrentVersion ifTrue: [^nil].
	other := self currentVersion.
	self swapIdentityWith: other.
	^other version setCurrent "because other is now me!! "! !

!VersionedObject methodsFor: 'private' stamp: ' 21/4/07 22:07'!
swapIdentityWith: anObject
	"carefully adjust object references (object, parent, next) in version specs too!!"
	|  o1 o2 s1Spec s2Spec |
	o1 := self. o2 := anObject.  s1Spec := o1 version. s2Spec := o2 version.
	o1 become: o2.
	s1Spec object: o2. s2Spec object: o1.
	s1Spec isNewest ifFalse:
		[s1Spec next == o2 ifTrue: [s1Spec next: o1] ifFalse: [s1Spec next version parent: o2] ].
	s1Spec isOldest ifFalse:
		[s1Spec parent == o2 ifTrue: [s1Spec parent: o1] ifFalse: [s1Spec parent version next: o2]].
	s2Spec isNewest ifFalse:
		[s2Spec next == o1 ifTrue: [s2Spec next: o2] ifFalse: [s2Spec next version parent: o1] ].
	s2Spec isOldest ifFalse:
		[s2Spec parent == o1 ifTrue: [s2Spec parent: o2] ifFalse: [s2Spec parent version next: o1]].! !

!VersionedObject methodsFor: 'private' stamp: ' 21/4/07 22:07'!
version
	version isNil ifTrue: [self initVersion].
	^version! !

!VersionedObject methodsFor: 'private' stamp: ' 21/4/07 22:07'!
version: aVersionSpec
	version := aVersionSpec! !

!VersionedObject methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
versionNumber
	"version number, integer by default, but it can be any string"
	^self version number! !

!VersionedObject methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
versionNumber: aString
	"set version number other than default"
	^self version number: aString! !

!VersionedObject methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
versionWithNumber: aString
	"find version with that number"
	| spec |
	spec := self version versionSpecWithNumber: aString.
	^spec notNil ifTrue: [spec object] ifFalse: [nil]! !

Object subclass: #WebAdmin
	instanceVariableNames: 'host ip port'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Admin'!

!WebAdmin methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
host
	^host! !

!WebAdmin methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
host: aString
	host := aString! !

!WebAdmin methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
ip
	^ip! !

!WebAdmin methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
ip: aString
	ip := aString! !

!WebAdmin methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
port
	^port! !

!WebAdmin methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
port: aNumber
	port := aNumber! !

Object subclass: #WebClipboard
	instanceVariableNames: 'title url object'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Core'!

!WebClipboard commentStamp: '<historical>' prior: 0!
Used for cut/copy/paste page references (not page contents!!) for easier hyperlinking of pages. Every page should have copy action. Then you can paste url to related links section in some other page.

Instance Variables:
	title	<String> title of a page
	url	<String>	url link to a page

!

!WebClipboard methodsFor: 'converting' stamp: ' 21/4/07 22:07'!
asDocLink
	"return clipboard contents as a new DocLink"
	^DocLink new
		title: self title;
		url: self url;
		object: self object! !

!WebClipboard methodsFor: 'converting' stamp: ' 21/4/07 22:07'!
copyFromDocLink: aDocLink
	self title: aDocLink title.
	self url: aDocLink url.
	self object: aDocLink object! !

!WebClipboard methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isEmpty
	^(self url isNil or: [self url isEmpty]) and: [self object isNil].! !

!WebClipboard methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
object
	^object! !

!WebClipboard methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
object: anObject
	object := anObject! !

!WebClipboard methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
title
	^title! !

!WebClipboard methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
title: anObject
	title := anObject! !

!WebClipboard methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
url
	^url! !

!WebClipboard methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
url: anObject
	url := anObject! !

Object subclass: #WebCounter
	instanceVariableNames: 'started day year dailyCounts hourlyCounts todayHourlyCounts total yearlyHistory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Core'!

!WebCounter class methodsFor: 'odb specific' stamp: ' 21/4/07 22:07'!
instVarMap
	"Gemstone odb"

	^super instVarMap,
		#( 	(todayHourlyCounts nil)	)! !

!WebCounter class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new reset! !

!WebCounter class methodsFor: 'odb specific' stamp: ' 21/4/07 22:07'!
replicationSpec
	"Gemstone"
	^super replicationSpec, 
	 	#( 	(started max 2)
			(day replicate)
			(year replicate)
			(dailyCounts max 2)
			(hourlyCounts max 2)
			(total replicate)
			(yearlyHistory stub)	)! !

!WebCounter class methodsFor: 'odb specific' stamp: ' 21/4/07 22:07'!
setupOdbStorage
	"Versant"

	self 
		store: #(started dailyCounts hourlyCounts total )
		as: #(Integer IntegerArray IntegerArray Integer).! !

!WebCounter class methodsFor: 'summing' stamp: ' 21/4/07 22:07'!
sumCounters: aCounterCollection

	"return a new counter with sum of all specified counters"
	| sumCounter |
	sumCounter := WebCounter new.
	aCounterCollection do: [:counter |
		sumCounter addCounter: counter].
	^sumCounter! !

!WebCounter class methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
throughMidnightTest

	Janko := WebCounter new.
	[100 timesRepeat: 
		[Janko incCounter.
		(Delay forSeconds: 1) wait]
	] fork.
	^Janko inspect

"WebCounter throughMidnightTest"! !

!WebCounter class methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
throughNewYearTest

	Janko := WebCounter new.
	[100 timesRepeat: 
		[Janko incCounter.
		(Delay forSeconds: 1) wait]
	] fork.
	^Janko inspect

"WebCounter throughNewYearTest"! !

!WebCounter methodsFor: 'private' stamp: ' 21/4/07 22:07'!
addArray: aFirstArray to: aSecondArray

	1 to: aFirstArray size do: [:index |
		aSecondArray
			at: index
			put: (aFirstArray at: index) + (aSecondArray at: index)]! !

!WebCounter methodsFor: 'adding' stamp: ' 21/4/07 22:07'!
addCounter: aWebCounter
	"add counts from specified counter"
	aWebCounter year = self year ifTrue: 
		[self addArray: aWebCounter dailyCounts to: self dailyCounts].
	self addArray: aWebCounter hourlyCounts to: self hourlyCounts.
	self total: self total + aWebCounter total.
	aWebCounter yearlyHistory keysAndValuesDo: [:yr :array |
		self addArray: array to: (self yearlyHistory at: yr ifAbsentPut: [Array new: 366 withAll: 0]) ].

"WebCounter new addCounter: WebCounter new"! !

!WebCounter methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allCountsInYear: aYearNumber

	aYearNumber = self year 
		ifTrue: [^self dailyCounts copy]
		ifFalse: 
			[^(self yearlyHistory 
				at: aYearNumber 
				ifAbsent: [^Array new: 366 withAll: 0]) copy]! !

!WebCounter methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allCountsThatYear
	
	"return array of counts for each day in current year"

	^self allCountsInYear: Date today year! !

!WebCounter methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allHourlyCounts

	^self hourlyCounts copy! !

!WebCounter methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allTodayHourlyCounts

	^self todayHourlyCounts copy! !

!WebCounter methodsFor: 'private' stamp: ' 21/4/07 22:07'!
checkIfNewYear

	(Date today year = (self year + 1)) ifTrue:
		[self yearlyHistory
			at: self year
			put: self dailyCounts.
		self initDailyCounts.
		self year: Date today year].! !

!WebCounter methodsFor: 'counting' stamp: ' 21/4/07 22:07'!
countRequest: aWebRequest

	"register a request by incrementing daily, hourly and total counter."

	self incCounterOnTimestamp: aWebRequest timestamp! !

!WebCounter methodsFor: 'accessing' stamp: 'mivsek 1/10/2008 19:19'!
countsFrom: aStartDate to: anEndDate

	| collection |
	collection := OrderedCollection new.
	aStartDate asDays to: anEndDate asDays do: [:days | 
		collection add: (self countsOnDate: (SpDate fromDays: days))].
	^collection! !

!WebCounter methodsFor: 'accessing' stamp: 'JM 4/26/2007 21:38'!
countsMonthlyFromDate: aDate
	| date collection |
	date := SpDate newDay: 1
		month: aDate monthIndex
		year: aDate year.
	collection := OrderedCollection new.
	[date < SpDate today] whileTrue:
		[collection add: (self countsOnMonth: date monthIndex year: date year).
		date := SpDate newDay: 1
			month: (date + 31) monthIndex
			year: (date + 31) year].
	^collection

"AIDASite default totalCounter countsMonthlyFromDate: Date today - 6"! !

!WebCounter methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
countsMonthlyOnYear: aYear

	^(1 to: 12) collect: [:month | self countsOnMonth: month year: aYear]

"WebServer default totalCounter countsMonthlyOnYear: 1999"! !

!WebCounter methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
countsOnDate: aDate

	| counts |
	counts := self allCountsInYear: aDate year.
	^counts at: aDate day! !

!WebCounter methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
countsOnHour: aNumber

	^self hourlyCounts at: aNumber! !

!WebCounter methodsFor: 'accessing' stamp: 'JM 4/26/2007 21:32'!
countsOnMonth: aMonth year: aYear

	| counts date |
	date := SpDate newDay: 1 month: aMonth year: aYear.
	counts := self allCountsInYear: date year.
	^(date day to: date day + date daysInMonth - 1)
		inject: 0 into: [:sum :dy | sum + (counts at: dy)]

"WebServer default totalCounter countsOnMonth: 10 year: 1999"! !

!WebCounter methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
countsTodayOnHour: aNumber

	^self todayHourlyCounts at: aNumber! !

!WebCounter methodsFor: 'accessing' stamp: 'mivsek 1/10/2008 19:19'!
currentWeekCounts
	| firstDay collection |
	firstDay := SpDate today subtractDays: SpDate today weekdayIndex + 1.
	collection := OrderedCollection new.
	firstDay asDays to: firstDay asDays + 6 do: [:days |
		collection add: (self countsOnDate: (SpDate fromDays: days))].
	^collection! !

!WebCounter methodsFor: 'private' stamp: ' 21/4/07 22:07'!
dailyCounts
	"array of counters for each day in current year. When new year arrives, counters are 
	copied in a yearlyHistory"

	dailyCounts isNil ifTrue: [self initDailyCounts].
	^dailyCounts! !

!WebCounter methodsFor: 'private' stamp: ' 21/4/07 22:07'!
day
	day isNil ifTrue: [self day: Date today dayOfYear].
	^day! !

!WebCounter methodsFor: 'private' stamp: ' 21/4/07 22:07'!
day: aNumber
	day := aNumber! !

!WebCounter methodsFor: 'private' stamp: ' 21/4/07 22:07'!
hourlyCounts
	"array of counters for each hour in a day."

	hourlyCounts isNil ifTrue: [self initHourlyCounts].
	^hourlyCounts! !

!WebCounter methodsFor: 'counting' stamp: 'JM 4/25/2007 21:22'!
incCounter

	"increment daily, hourly and total counter with date and time now"

	self incCounterOnTimestamp: SpTimestamp now! !

!WebCounter methodsFor: 'counting' stamp: ' 21/4/07 22:07'!
incCounterOnTimestamp: aTimestamp

	"increment daily, hourly and total counter with date and time defined"

	
	self incDailyCounterOnDate: aTimestamp asDate.
	self incHourlyCounterOnTimestamp: aTimestamp.
	self incTotal.! !

!WebCounter methodsFor: 'private' stamp: ' 21/4/07 22:07'!
incDailyCounterOnDate: aDate
	aDate year = self year 
		ifTrue: 
			[self dailyCounts at: aDate day put: (self  dailyCounts at: aDate day) + 1]
		ifFalse:
			[self checkIfNewYear.
			self incHistoryCounterOnDate: aDate].! !

!WebCounter methodsFor: 'private' stamp: ' 21/4/07 22:07'!
incHistoryCounterOnDate: aDate
	| array |
	aDate year < self year ifTrue: [^self error: 'future years not allowed'].
	array := (self yearlyHistory at: aDate year ifAbsentPut: [Array new: 366 withAll: 0] ).
	array at: aDate day put: (array at: aDate day)+1! !

!WebCounter methodsFor: 'private' stamp: ' 21/4/07 22:07'!
incHourlyCounterOnTime: aTime
	| hours |
	hours := aTime hours.
	self hourlyCounts at: hours+1 	put: (self  hourlyCounts at: hours+1) + 1.! !

!WebCounter methodsFor: 'private' stamp: ' 21/4/07 22:07'!
incHourlyCounterOnTimestamp: aTimestamp
	| hours |
	hours := aTimestamp asTime hours.
	self hourlyCounts at: hours+1 put: (self  hourlyCounts at: hours+1) + 1.
	self incTodayHourlyCounterOnTimestamp: aTimestamp! !

!WebCounter methodsFor: 'private' stamp: ' 21/4/07 22:07'!
incTodayHourlyCounterOnTimestamp: aTimestamp

	| hours dayOfYear |
	dayOfYear := aTimestamp asDate day.
	self day ~= dayOfYear 	ifTrue: 
		[self initTodayHourlyCounts.
		self day: dayOfYear].
	hours := aTimestamp asTime hours.
	self todayHourlyCounts 
		at: hours+1
		put: (self  todayHourlyCounts at: hours+1) + 1.! !

!WebCounter methodsFor: 'private' stamp: ' 21/4/07 22:07'!
incTotal

	self total: self total + 1.! !

!WebCounter methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initDailyCounts
	dailyCounts := Array new: 366 withAll: 0.
	self year: Date today year.! !

!WebCounter methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initHourlyCounts
	hourlyCounts := Array new: 24 withAll: 0.! !

!WebCounter methodsFor: 'initialize-release' stamp: 'JM 4/25/2007 21:22'!
initStarted
	"set a timestamp to a current time"
	started := SpTimestamp now asSeconds.! !

!WebCounter methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initTodayHourlyCounts

	todayHourlyCounts := Array new: 24 withAll: 0.
	self day: Date today day.! !

!WebCounter methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initTotal

	self total: 0.! !

!WebCounter methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initYearlyHistory
	yearlyHistory := Dictionary new.! !

!WebCounter methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
printString

	^'aWebCounter total: ', self total printDotString! !

!WebCounter methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
reset

	"set all counters to zero and set started timestamp to now. YOU WILL LOOSE ALL HISTORY
	OF COUNTS !! "

	self initDailyCounts.
	self initHourlyCounts.
	self initTotal.
	self initStarted.! !

!WebCounter methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
started
	"return a timestamp, from when web visits are counted"

	started isNil ifTrue: [self initStarted].
	^Timestamp fromSeconds: started! !

!WebCounter methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
today
	
	"return number of requests today"

	^self countsOnDate: Date today! !

!WebCounter methodsFor: 'private' stamp: ' 21/4/07 22:07'!
todayHourlyCounts
	"array of counters for each hour today."

	todayHourlyCounts isNil ifTrue: [self initTodayHourlyCounts].
	^todayHourlyCounts! !

!WebCounter methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
todayHourlyCounts: anObject
	todayHourlyCounts := anObject! !

!WebCounter methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
total
	"total number of visits, registered by this counter"

	total isNil ifTrue: [self initTotal].
	^total.! !

!WebCounter methodsFor: 'private' stamp: ' 21/4/07 22:07'!
total: aNumber
	total := aNumber.! !

!WebCounter methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
weekly
	"total count in current week"
	^self currentWeekCounts inject: 0 into: [:sum :each | sum + each]! !

!WebCounter methodsFor: 'private' stamp: ' 21/4/07 22:07'!
year
	year isNil ifTrue: [self year: Date today year].
	^year! !

!WebCounter methodsFor: 'private' stamp: ' 21/4/07 22:07'!
year: aNumber
	year := aNumber.! !

!WebCounter methodsFor: 'private' stamp: ' 21/4/07 22:07'!
yearlyHistory
	yearlyHistory isNil ifTrue: [self initYearlyHistory].
	^yearlyHistory! !

!WebCounter methodsFor: 'accessing' stamp: 'mivsek 1/10/2008 19:19'!
yesterday
	"return number of requests yesterday"
	^self countsOnDate: (SpDate today subtractDays: 1)! !

Object subclass: #WebDemo
	instanceVariableNames: 'date input delayedInput field editField'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Examples'!

!WebDemo class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
default
	^AIDASite default demo! !

!WebDemo methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
date
	date isNil ifTrue: [^''].
	^date! !

!WebDemo methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
date: aString
	date := aString! !

!WebDemo methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
delayedInput
	delayedInput isNil ifTrue: [^''].
	^delayedInput! !

!WebDemo methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
delayedInput: aString
	delayedInput := aString! !

!WebDemo methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
editField
	"for in-place editor demo"
	editField isNil ifTrue: [^'Click me to edit!!'].
	^editField! !

!WebDemo methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
editField: anObject
	editField := anObject! !

!WebDemo methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
field
	^field! !

!WebDemo methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
field: anObject
	field := anObject! !

!WebDemo methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
input
	input isNil ifTrue: [^''].
	^input! !

!WebDemo methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
input: aString
	input := aString! !

!WebDemo methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
preferedUrl
	^'/demos.html'! !

Object subclass: #WebElement
	instanceVariableNames: 'parent elements attributes other'
	classVariableNames: 'Colors'
	poolDictionaries: ''
	category: 'Aida-Elements'!

WebElement subclass: #JoomlaModule
	instanceVariableNames: 'header body'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Templates'!

!JoomlaModule commentStamp: '<historical>' prior: 0!
JoomlaModule 

Add elements to #body, not to module itself!! 
!

JoomlaModule subclass: #JoomlaMenu
	instanceVariableNames: 'pages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Templates'!

!JoomlaMenu class methodsFor: 'instance creation' stamp: ' 10/1/08 16:19'!
new
	^super new initialize! !

!JoomlaMenu methodsFor: 'accessing' stamp: ' 10/1/08 16:19'!
addFirstPageWithDescription: aString
	| firstPage assoc firstPageTitle |
	firstPage := self app observee parent defaultPage.
	assoc := self app observee parent pages associations detect: [:each | each value == firstPage].
	firstPageTitle := assoc key.
	self addPage: firstPageTitle description: aString! !

!JoomlaMenu methodsFor: 'accessing' stamp: ' 10/1/08 16:19'!
addPage: aNameString description: aTabNameString
	"aNameString is original/first title of page, not changed one!!"
	| name pageTitle |
	name := aNameString convertToSloveneChars.
	(self pages contains: [:each | each key == name]) ifTrue: [^nil]. 
	pageTitle := (self app observee parent pageTitled: name).
	pageTitle := pageTitle notNil ifTrue: [pageTitle title] ifFalse: [''].
	self pages add:
		(Association key: name value: (Array with: aTabNameString with: pageTitle ))! !

!JoomlaMenu methodsFor: 'initialize-release' stamp: ' 10/1/08 16:19'!
initPages
	pages := OrderedCollection new.! !

!JoomlaMenu methodsFor: 'initialize-release' stamp: ' 10/1/08 16:19'!
initialize
	self initPages.
	^self! !

!JoomlaMenu methodsFor: 'printing' stamp: ' 10/1/08 16:19'!
linkToWikiPage: aTitleString text: aString
	| page titleString |
	titleString := aTitleString "AIDASite convertFromWeb: aTitleString on: self app session".
	page := self app observee parent pageTitled: titleString.
	page notNil ifTrue: [^WebLink text: aString linkTo: page ].
	^(WebLink text: aString linkTo: self app observee parent view: #add )  "new page"
		parameter: 'title' value: (AIDASite convertToWebString: aTitleString)! !

!JoomlaMenu methodsFor: 'private' stamp: ' 10/1/08 16:19'!
pages
	^pages! !

!JoomlaMenu methodsFor: 'printing' stamp: ' 10/1/08 16:19'!
prepareModule
	| inx tabTitle |
	super prepareModule.
	inx := 1.
	self pages do: [:each | 
		tabTitle := each key ensureUnicodeSloveneChars.
		self cell add: 
			((self linkToWikiPage: each key text: each value first) class: #mainlevel).
		each ~= self pages last ifTrue: [self newRow]. 
		inx := inx + 1]! !

!JoomlaModule methodsFor: 'accessing' stamp: ' 10/1/08 16:19'!
body
	"body of the module, add elements to body, not to module directly!!"
	body isNil ifTrue: [body := WebElement new].
	^body! !

!JoomlaModule methodsFor: 'accessing' stamp: ' 10/1/08 16:19'!
header
	header isNil ifTrue: [^''].
	^header! !

!JoomlaModule methodsFor: 'accessing' stamp: ' 10/1/08 16:19'!
header: aString 
	"text on top of menu"
	header := aString! !

!JoomlaModule methodsFor: 'printing' stamp: ' 10/1/08 16:19'!
prepareModule
	self class: #moduletable.
	self addTextH3: self header.
	self table width: 1; cellPadding: 0; cellSpacing: 0.
	self body elements notEmpty ifTrue: [self add: self body].! !

!JoomlaModule methodsFor: 'printing' stamp: 'janko 7/18/2008 20:19'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	self prepareModule.
	self prepareToHTMLPrintOn: aSession.
	self scriptBefore notNil ifTrue: 
		[ self scriptBefore 
			printHTMLPageOn: aStream for: aRequest on: aSession ].
	aStream nextPutAll: self ident , '<div'.
	self 
		printHTMLPageOn: aStream for: aRequest on: aSession.
	aStream nextPutAll: '>' , self eol.
	elements notNil ifTrue: 
		[ elements do: 
			[ : element | 
			element notNil ifTrue: 
				[ element 
					printHTMLPageOn: aStream for: aRequest on: aSession ] ] ].
	aStream nextPutAll: self ident , '</div>' , self eol.
	self scriptAfter notNil ifTrue: 
		[ self scriptAfter 
			printHTMLPageOn: aStream for: aRequest on: aSession ]! !

WebElement subclass: #WebComment
	instanceVariableNames: 'text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebComment class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new initialize! !

!WebComment class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
text: aString
	^self new text: aString

"WebText text: 'test'"! !

!WebComment methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	text := ''.! !

!WebComment methodsFor: 'printing' stamp: 'janko 7/18/2008 20:21'!
printHTMLPageOn: aStream for: aRequest on: aSession
	aStream nextPutAll: self ident , '<!!-- '.
	aStream nextPutAll: self text.
	aStream nextPutAll: ' -->' , self eol! !

!WebComment methodsFor: 'private' stamp: ' 21/4/07 22:07'!
shouldIdent
	^false! !

!WebComment methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
text
	^text! !

!WebComment methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
text: aString
	text := aString.! !

WebElement subclass: #WebComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Components'!

!WebComponent commentStamp: 'mivsek 1/10/2008 22:35' prior: 0!
WebComponent is abstract class for Aida components - standalone parts of web page, many times also long living, that is, they live longer than just one page view. 
!

!WebComponent class methodsFor: 'instance creation' stamp: 'np 6/24/2008 06:21'!
new
	^super new
		initialize;
		yourself! !

!WebComponent methodsFor: 'printing' stamp: 'np 4/19/2008 20:50'!
build
	"Build the component, override this method"
	self clear! !

!WebComponent methodsFor: 'printing' stamp: 'janko 7/18/2008 20:21'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	self build.
	super 
		printHTMLPageOn: aStream for: aRequest on: aSession! !

WebComponent subclass: #WebLightbox
	instanceVariableNames: 'component'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Components'!

!WebLightbox class methodsFor: 'instance creation' stamp: 'np 6/24/2008 02:32'!
newFor: aComponent
	^self new 
		component: aComponent;
		yourself! !

!WebLightbox methodsFor: 'printing' stamp: 'np 6/24/2008 00:52'!
build
	| e f |
	super build.
	e := (WebElement newId: 'overlay').
	f := (WebElement newId: 'lightbox')
		style: 'display: none;'.
	f add: self component.
	self scriptAfter: 
'/*<!![CDATA[*/updateLightbox();Event.observe(window,"resize",function(){updateLightbox();})/*]]>*/'.
	self add: e; add: f.! !

!WebLightbox methodsFor: 'accessing' stamp: 'np 6/23/2008 23:36'!
component
	^component! !

!WebLightbox methodsFor: 'accessing' stamp: 'np 6/23/2008 23:37'!
component: aWebComponent
	"aWebComponent can be a WebComponent or WebElement"
	component := aWebComponent! !

!WebLightbox methodsFor: 'initialize-release' stamp: 'np 6/24/2008 01:06'!
initialize
	super initialize.
	self app ensureJsResourceForScriptaculous.
	self app ensureJavascriptForScriptaculousInHeader.
	self app ensureJsResourceForLightbox.
	self app ensureJavascriptForLightboxInHeader.
	self app ensureCSSForLightboxInHeader! !

WebComponent subclass: #WebRichEditorWakeup
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Components'!

!WebRichEditorWakeup commentStamp: '<historical>' prior: 0!
WebRichEditorWakeup is a component which speedup editing by loading all JS and other libraries for rich text editor in background. 

Add it to the end of your pages but only if user has editing rights.!

!WebRichEditorWakeup class methodsFor: 'instance creation' stamp: ' 18/7/08 16:34'!
new
	^super new
		method: #ajaxUpdate;
		build! !

!WebRichEditorWakeup methodsFor: 'printing' stamp: ' 18/7/08 16:34'!
ajaxUpdate
	self build! !

!WebRichEditorWakeup methodsFor: 'printing' stamp: 'janko 7/31/2008 18:27'!
build
	| iframe |
	self clear. self setDiv. self hide.
	self isWakeupTimeout ifFalse: [^self]. 
	iframe := WebIFrame new src: '/admin.html?view=wakeupRichEditor'.
	self add: iframe.
	self setWakeupTimestamp.
	
"	self setDiv. self registerId.
	self session lastRequest isAjaxRequest 
		ifFalse: 
			[self hide.
			self update ] ""to call iframe below in backgrund""
		ifTrue: 
			[iframe := WebIFrame new src: '/admin.html?view=wakeupRichEditor'.
			self add: iframe.
			self setWakeupTimestamp]
"! !

!WebRichEditorWakeup methodsFor: 'timeout' stamp: 'janko 7/18/2008 17:13'!
isWakeupTimeout
	"more that one hour since last prefech?"
	"why refresh every hour?"
	| last |
	last := self session userValues at: #wakeupRichEditorTimestamp ifAbsent: [^true].
	^SpTimestamp now asSeconds - last asSeconds > 3600! !

!WebRichEditorWakeup methodsFor: 'timeout' stamp: 'janko 7/18/2008 17:11'!
setWakeupTimestamp
	"store in user's session when last wakeup was done"
	self session userValues at: #wakeupRichEditorTimestamp put: SpTimestamp now.
! !

!WebElement class methodsFor: 'color values' stamp: ' 21/4/07 22:07'!
colorDictionary
	"return a dictionary of color names as keys and hex values for colors in 
	some web elements such as page and table background, text, links etc."
	Colors isNil ifTrue: [self initColorDictionary].
	^Colors! !

!WebElement class methodsFor: 'color values' stamp: ' 21/4/07 22:07'!
initColorDictionary
	"WebElement initColorDictionary"
	"WebElement colorDictionary"
	Colors := IdentityDictionary new.
	Colors 
		at: #white				put: #'ffffff';
		at: #red					put: #'ff0000' ;
		at: #green				put: #'00ff00' ;
		at: #blue				put: #'0000ff' ;
		at: #magenda			put: #'ff00ff';
		at: #cyan				put: #'00ffff' ;
		at: #yellow				put: #'ffff00' ;
		at: #black				put: #'000000';

		at: #aquamarine			put: #'70db93';

		at: #bakerschocolade	put: #'5c3317';
		at: #blueviolet	put: #'9f5f9f';
		at: #brass		put: #'b5a642';
		at: #brightgold	put: #'d9d919';
		at: #brown		put: #'a62a2a';
		at: #bronze		put: #'8c7853';
		at: #bronzeii		put: #'a67d3d';

		at: #cadetblue		put: #'5f9f9f';
		at: #coolcooper		put: #'d98719';
		at: #cooper			put: #'b87333';
		at: #coral			put: #'ff7f00';
		at: #cornflowerblue	put: #'42426f';

		at: #darkbrown		put: #'5c4033';
		at: #darkgreen		put: #'2f4f2f';
		at: #darkgreencooper put: #'4a766e';
		at: #darkolivegreen	put: #'4f4f2f';
		at: #darkorchid		put: #'9932cd';
		at: #darkpurple		put: #'871f78';
		at: #darkslateblue	put: #'6b238e';
		at: #darkslategrey	put: #'2f4f4f';
		at: #darktan			put: #'97694f';
		at: #darkturquoise	put: #'7093db';
		at: #darkwood		put: #'855e42';
		at: #dimgrey		put: #'545454';
		at: #dustyrose		put: #'856363';

		at: #feldspar			put: #'d19275';
		at: #firebrick			put: #'8e2323';
		at: #forestgreen		put: #'238e23';

		at: #gold			put: #'cd7f32';
		at: #goldenrod		put: #'dbdb70';
		at: #grey			put: #'c0c0c0';
		at: #greencooper	put: #'527f76';
		at: #greenyellow		put: #'93db70';

		at: #huntergreen		put: #'215e21';

		at: #indianred		put: #'4e2f2f';

		at: #khaki		 	put: #'9f9f5f';

		at: #lightblue		put: #'c0d9d9';
		at: #lightgrey		put: #'a8a8a8';
		at: #lightsteelblue	put: #'8f8fbd';
		at: #lightwood		put: #'e9c2a6';
		at: #limegreen		put: #'32cd32';

		at: #mandarianorange	put: #'e47833';
		at: #maroon			put: #'8e236b';
		at: #mediumaquamarine	put: #'32cd99';
		at: #mediumblue	put: #'3232cd';
		at: #mediumforestgreen	put: #'6b8e23';
		at: #mediumgoldenrod	put: #'eaeaae';
		at: #mediumorchid	put: #'9370db';
		at: #mediumseagreen	put: #'426f42';
		at: #mediumslateblue	put: #'7f00ff';
		at: #mediumspringgreen	put: #'7fff00';
		at: #mediumturquoise	put: #'70dbdb';
		at: #mediumvioletred		put: #'db7093';
		at: #mediumwood	put: #'a68064';
		at: #midnightblue	put: #'2f2f4f';
		
		at: #navyblue		put: #'23238e ';
		at: #neonblue		put: #'4d4dff';
		at: #neonpink		put: #'ff6ec7';
		at: #newmidnightblue	put: #'00009c';
		at: #newtan			put: #'ebc79e ';
		
		at: #oldgold			put: #'cfb53b ';
		at: #orange			put: #'ff7f00';
		at: #orangered		put: #'ff2400';
		at: #orchid			put: #'db70db';

		at: #palegreen		put: #'8fbc8f';
		at: #pink			put: #'bc8f8f';
		at: #plum			put: #'eaadea';

		at: #quartz			put: #'d9d9f3';

		at: #richblue			put: #'5959ab';

		at: #salmon			put: #'6f4242';
		at: #scarlet			put: #'8c1717';
		at: #seagreen		put: #'238e68';
		at: #semiswetchocolate		put: #'6b4226';
		at: #sienna			put: #'8e6b23';
		at: #silver			put: #'e6e8fa';
		at: #skyblue			put: #'3299cc';
		at: #slateblue		put: #'007fff';
		at: #spicypink		put: #'ff1cae';
		at: #springgreen		put: #'00ff7f';
		at: #steelblue		put: #'236b8e';
		at: #summersky		put: #'38b0de';

		at: #tan				put: #'db9370';
		at: #thistle			put: #'d8bfd8';
		at: #turquoise		put: #'adeaea';

		at: #verydarkbrown	put: #'5c4033';
		at: #verylightgrey	put: #'cdcdcd';
		at: #violet			put: #'4f2f4f';
		at: #violetred		put: #'cc3299';

		at: #wheat			put: #'d8d8bf';
		at: #yellowgreen		put: #'99cc32'.! !

!WebElement class methodsFor: 'instance creation' stamp: 'JM 4/25/2007 22:23'!
new
	| instance |
	instance := super basicNew.
	self == WebElement ifTrue: [instance setCreationMethod].
	^instance! !

!WebElement class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newClass: aSymbol
	"new with CSS class as specified"
	^self new class: aSymbol! !

!WebElement class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newDiv
	"element enclosed in div tag"
	^self new setDiv! !

!WebElement class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newId: aSymbol
	"new with id as specified"
	^self new id: aSymbol! !

!WebElement class methodsFor: 'printing' stamp: 'janko 7/18/2008 19:11'!
printWebPageFor: aRequest on: aSession 

	"do a page of all colors in a color dicionary"

	| colors page table col |
	colors := self colorDictionary keys.
	page := WebPage new.
	page title: 'Web Color Table'.
	table := WebTable new.
	col := 0. 
	table add: WebTableRow new.
	colors do: [:color |
		(col \\ 5 = 0) ifTrue: [table add: WebTableRow new]. col := col + 1.
		table add: (WebTableCell new bgColor: (self valueForColor: color); 
			addBreak; 
			addText: '<font face=helvetica size=-1>';
			addText: (color printString); addBreak;
			addText: ((self valueForColor: color) printString); addBreak;
			addText: (color printString) color: #white; addBreak;
			addText: ((self valueForColor: color) printString) color: #white; addBreak;
			addText: '</font>').
		].
	page add: table.
	^page

"WebElement printWebPageFor: (WebSession new)"! !

!WebElement class methodsFor: 'color values' stamp: ' 21/4/07 22:07'!
valueForColor: aColorSymbolOrString
	"return a hex value for specified color. Case of color string is not important. 
	return red if color is unknown"
	| color |
	color := aColorSymbolOrString asString asLowercase asSymbol.
	^self colorDictionary at: color ifAbsent: [^self colorDictionary at: #red].

"
WebElement valueForColor: 'blue' 
"! !

!WebElement methodsFor: 'events-ajax' stamp: 'np 4/1/2008 21:44'!
acceptDropsTo: aCollectionOfElements
	| script |
	self registerId.
	self app ensureJsResourceForScriptaculous.
	self app ensureJavascriptForScriptaculousInHeader.
	script := '// <!![CDATA[ 
		Sortable.create("', self id asString,'"'.
	(self isKindOf: WebList) ifFalse: [
		self setDiv.
		script := script,  'tag: "div", '].
	script := script, ',{dropOnEmpty: true,constraint: false, containment: ['.
	aCollectionOfElements do: [:each |
		each registerId.
		script := script, each id, ','].
	script := script,']});
	// ]]>'.
	self parent scriptAfter: script! !

!WebElement methodsFor: 'private' stamp: ' 21/4/07 22:07'!
adaptFormElements
	"in case of AJAX request, otherwise WebForm should do that!! "
	self isFormElement ifTrue: [self adapt].
	self elements do: [:each | each adaptFormElements]! !

!WebElement methodsFor: 'subelements' stamp: 'JM 4/26/2007 13:48'!
add: aWebElement
	"Include new element as one of the receiver's elements.  Answer aWebElement.
	If you nest a web page into another web page, then add only elements of it"
	self checkAndInitElements.  "if not composite element, then error"
	"if nested web pages then add elements without header"
	(aWebElement isKindOf: WebApplication) ifTrue: [^self halt].
	((aWebElement isKindOf: WebPage) and: [self isKindOf: WebPage])
		ifTrue: [^elements add: aWebElement elements].
	aWebElement parent: self.
	^elements add: aWebElement.! !

!WebElement methodsFor: 'adding links' stamp: ' 21/8/07 09:38'!
addAnchorName: aString
	^self add: (WebAnchor name: aString)! !

!WebElement methodsFor: 'adding form elements' stamp: ' 21/8/07 09:38'!
addAspect: aSymbol for: anObject input: aBoolean size: aSizeNumberOrPoint

	"adds an auto converted text of aspect for that object. If imput argument is true, then add 
	an input field or text area, depending in size (eg. size: 5 or size: 5@5)"

	aBoolean
		ifFalse: 
			[^self addText: (WebFormElement autoConvertToString: (anObject perform: aSymbol))]
		ifTrue:
			[(aSizeNumberOrPoint isKindOf: Number)
				ifTrue: 
					[^self add: 
						(WebInputField new
							size: aSizeNumberOrPoint;
							aspect: aSymbol for: anObject)].
			(aSizeNumberOrPoint isKindOf: Point)
				ifTrue: 
					[^self add: 
						(WebTextArea new
							size: aSizeNumberOrPoint;
							aspect: aSymbol for: anObject)] 
			].! !

!WebElement methodsFor: 'adding other elements' stamp: ' 21/8/07 09:38'!
addBreak
	^self add: (WebSeparator break).! !

!WebElement methodsFor: 'adding form elements' stamp: ' 21/4/07 22:07'!
addButtonText: aString
	^self add: (WebButton text: aString)! !

!WebElement methodsFor: 'adding form elements' stamp: ' 21/4/07 22:07'!
addButtonText: aString action: aSymbol
	"for more than one buttons. Action method is a composed name from 'action', aView, Symbol.
	example: for view #main, action #addFolder is action method #actionMainAddFolder"
	^self add: (WebButton text: aString action: aSymbol)! !

!WebElement methodsFor: 'adding form elements' stamp: ' 21/4/07 22:07'!
addCheckboxAspect: aSymbol for: anObject
	"aspect method will be called and true/false will be set/clear on an object"
	^self add: (WebCheckBox aspect:  aSymbol for: anObject)! !

!WebElement methodsFor: 'adding form elements' stamp: ' 21/4/07 22:07'!
addCheckboxObject: anObject from: aCollection
	"if checked, this object will be put in collection. If object is initially in collection, 
	checkbox will be checked. Usefull for easy selection among many objects"
	^self add: (WebCheckBox object: anObject from: aCollection)! !

!WebElement methodsFor: 'adding other elements' stamp: ' 21/8/07 09:38'!
addComment: aString
	^self add: (WebComment text: aString).! !

!WebElement methodsFor: 'adding ajax components' stamp: ' 21/8/07 09:38'!
addDateInputFieldAspect: aSymbol for: anObject
	"with popup calendar"
	^self add: (WebDateInputField new aspect:  aSymbol for: anObject)! !

!WebElement methodsFor: 'adding ajax components' stamp: ' 21/4/07 22:07'!
addDelayedFieldAspect: aSymbol for: anObject
	"for Ajax instant posting, posting is done with a delay. Usefull for live-search fields"
	^self add: (WebDelayedField new aspect:  aSymbol for: anObject)! !

!WebElement methodsFor: 'adding ajax components' stamp: ' 21/4/07 22:07'!
addDelayedFieldAspect: aSymbol for: anObject size: aNumber
	"for Ajax instant posting, posting is done with a delay. Usefull for live-search fields"
	^self add: ((WebDelayedField new aspect:  aSymbol for: anObject) size: aNumber)! !

!WebElement methodsFor: 'adding other elements' stamp: ' 21/8/07 09:38'!
addErrorReport
	^self add: self app errorReport! !

!WebElement methodsFor: 'adding images' stamp: ' 21/8/07 09:38'!
addGif: aSymbol
	"this method will be called in your webStyle and this method 
	should return a gif in byte array format"
	^self add: (WebImage gif: aSymbol)! !

!WebElement methodsFor: 'adding images' stamp: ' 21/8/07 09:38'!
addGif: aSymbol size: aPoint
	"this method will be called in your webStyle and this method 
	should return a gif in byte array format"
	^self add: (WebImage gif: aSymbol size: aPoint)! !

!WebElement methodsFor: 'adding other elements' stamp: ' 21/8/07 09:38'!
addHelpLink
	"link to help page for current view and App. If not exist, then nothing, exept for 
	admin - link to creation of new help page"
	^self add: self helpLink! !

!WebElement methodsFor: 'adding images' stamp: ' 21/8/07 09:38'!
addImage: anImageOrURL
	^self add: (WebImage image: anImageOrURL)! !

!WebElement methodsFor: 'adding images' stamp: ' 21/8/07 09:38'!
addImage: anImageOrURL imageMap: aWebImageMap
	^self add: 
		((WebImage image: anImageOrURL)
			imageMap: aWebImageMap)! !

!WebElement methodsFor: 'adding images' stamp: ' 21/8/07 09:38'!
addImage: anImageOrURL size: aPoint
	^self add: (WebImage image: anImageOrURL size: aPoint)! !

!WebElement methodsFor: 'adding images' stamp: ' 21/8/07 09:38'!
addImage: anImageOrURL size: aPoint align: aSymbol
	^self add: ((WebImage image: anImageOrURL size: aPoint) align: aSymbol)! !

!WebElement methodsFor: 'adding images' stamp: ' 21/8/07 09:38'!
addImage: anImageOrURL size: aPoint valign: aSymbol
	^self add: ((WebImage image: anImageOrURL size: aPoint) valign: aSymbol)! !

!WebElement methodsFor: 'adding ajax components' stamp: ' 21/8/07 09:38'!
addInPlaceEditableTextAspect: aSymbol for: anObject
	^self add: (WebInPlaceEditableText aspect:  aSymbol for: anObject)! !

!WebElement methodsFor: 'adding ajax components' stamp: ' 21/8/07 09:38'!
addInPlaceEditableTextAspect: aSymbol for: anObject size: aNumberOrPoint
	^self add: (WebInPlaceEditableText aspect:  aSymbol for: anObject size: aNumberOrPoint)! !

!WebElement methodsFor: 'adding ajax components' stamp: ' 21/8/07 09:38'!
addInPlaceEditableTextAspect: aSymbol for: anObject size: aNumberOrPoint allow: aBoolean
	^self add: 
		(WebInPlaceEditableText aspect:  aSymbol for: anObject size: aNumberOrPoint allow: aBoolean)! !

!WebElement methodsFor: 'adding form elements' stamp: ' 21/4/07 22:07'!
addInputFieldAspect: aSymbol for: anObject
	^self add: (WebInputField new aspect:  aSymbol for: anObject)! !

!WebElement methodsFor: 'adding form elements' stamp: ' 21/4/07 22:07'!
addInputFieldAspect: aSymbol for: anObject size: aNumber
	^self add: ((WebInputField new aspect:  aSymbol for: anObject) size: aNumber)! !

!WebElement methodsFor: 'adding images' stamp: ' 21/8/07 09:38'!
addJpeg: aSymbol
	"this method will be called in your webStyle and this method 
	should return a gif in byte array format"
	^self add: (WebImage jpeg: aSymbol)! !

!WebElement methodsFor: 'adding images' stamp: ' 21/8/07 09:38'!
addJpeg: aSymbol size: aPoint
	"this method will be called in your webStyle and this method 
	should return a gif in byte array format"
	^self add: (WebImage jpeg: aSymbol size: aPoint)! !

!WebElement methodsFor: 'adding links (image)' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject gif: aGifSymbol title: aString
	^self add: ((WebLink linkTo: anObject) title: aString; addGif: aGifSymbol; yourself)! !

!WebElement methodsFor: 'adding links (image)' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject gif: aGifSymbol title: aString size: aPoint
	^self add: ((WebLink linkTo: anObject) title: aString; addGif: aGifSymbol size: aPoint; yourself)! !

!WebElement methodsFor: 'adding links (image)' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject gif: aGifSymbol title: aString view: aViewSymbol
	^self add: (((WebLink linkTo: anObject) title: aString; view: aViewSymbol) 
		addGif: aGifSymbol; yourself)! !

!WebElement methodsFor: 'adding links (image)' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject gif: aGifSymbol title: aString view: aViewSymbol
	 parameter: aParmString value: aValueString

	^self add: (((WebLink linkTo: anObject) 
		title: aString; view: aViewSymbol;
		parameter: aParmString value: aValueString) 
			addGif: aGifSymbol; yourself)! !

!WebElement methodsFor: 'adding links (image)' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject image: anImageOrURL
	^self add: ((WebLink linkTo: anObject) addImage: anImageOrURL; yourself)! !

!WebElement methodsFor: 'adding links (image)' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject image: anImageOrURL parameter: aParmString value: aValueString
	^self add: (((WebLink linkTo: anObject)
		parameter: aParmString value: aValueString) addImage: anImageOrURL; yourself)! !

!WebElement methodsFor: 'adding links (image)' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject image: anImageOrURL size: aPoint
	^self add: ((WebLink linkTo: anObject) addImage: anImageOrURL size: aPoint; yourself)! !

!WebElement methodsFor: 'adding links (image)' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject image: anImageOrURL view: aViewSymbol
	^self add: (((WebLink linkTo: anObject) view: aViewSymbol) 
		addImage: anImageOrURL; yourself)! !

!WebElement methodsFor: 'adding links (image)' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject jpeg: aSymbol title: aString
	^self add: ((WebLink linkTo: anObject) title: aString; addJpeg: aSymbol; yourself)! !

!WebElement methodsFor: 'adding links (image)' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject jpeg: aSymbol title: aString view: aViewSymbol
	^self add: (((WebLink linkTo: anObject) title: aString; view: aViewSymbol) 
		addJpeg: aSymbol; yourself)! !

!WebElement methodsFor: 'adding links' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject message: aSymbol
	^self add: (WebLink message: aSymbol  linkTo: anObject)! !

!WebElement methodsFor: 'adding links (image)' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject png: aPngSymbol title: aString
	^self add: ((WebLink linkTo: anObject) title: aString; addPng: aPngSymbol; yourself)! !

!WebElement methodsFor: 'adding links (image)' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject png: aPngSymbol title: aString view: aViewSymbol
	^self add: (((WebLink linkTo: anObject) title: aString; view: aViewSymbol) 
		addPng: aPngSymbol; yourself)! !

!WebElement methodsFor: 'adding links (image)' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject png: aPngSymbol title: aString view: aViewSymbol
	 parameter: aParmString value: aValueString

	^self add: (((WebLink linkTo: anObject) 
		title: aString; view: aViewSymbol;
		parameter: aParmString value: aValueString) 
			addPng: aPngSymbol; yourself)! !

!WebElement methodsFor: 'adding links' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject text: aString
	^self add: (WebLink text: aString  linkTo: anObject)! !

!WebElement methodsFor: 'adding links' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject text: aString attributes: anArray
	^self add: (WebLink text: aString  attributes: anArray linkTo: anObject)! !

!WebElement methodsFor: 'adding links' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject text: aString header: aNumber
	^self add: (WebLink text: aString  header: aNumber linkTo: anObject)! !

!WebElement methodsFor: 'adding links' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject text: aString parameter: aParmString value: aValueString
	^self add: (WebLink text: aString  linkTo: anObject 
		parameter: aParmString value: aValueString)! !

!WebElement methodsFor: 'adding links' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject text: aString 
	parameter: a1ParmString value: a1ValueString
	parameter: a2ParmString value: a2ValueString

	^self add: (WebLink text: aString  linkTo: anObject 
		parameter: a1ParmString value: a1ValueString
		parameter: a2ParmString value: a2ValueString)! !

!WebElement methodsFor: 'adding links' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject text: aString view: aViewString
	^self add: ((WebLink text: aString  linkTo: anObject)
		view: aViewString; yourself)! !

!WebElement methodsFor: 'adding links' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject text: aString view: aViewString  parameter: aParmString value: aValueString
	^self add: ((WebLink text: aString  linkTo: anObject)
		view: aViewString;
		parameter: aParmString value: aValueString; yourself)! !

!WebElement methodsFor: 'adding links' stamp: ' 21/8/07 09:38'!
addLinkTo: anObject text: aString view: aViewString  
		parameter: aParm1String value: aValue1String 
		parameter: aParm2String value: aValue2String

	^self add: ((WebLink text: aString  linkTo: anObject)
		view: aViewString;
		parameter: aParm1String value: aValue1String;
		parameter: aParm2String value: aValue2String; yourself)! !

!WebElement methodsFor: 'adding images' stamp: ' 21/8/07 09:38'!
addLiveImage: aWebLiveImage
	^self add: 
		((WebImage image: aWebLiveImage)
			size: (aWebLiveImage width)@(aWebLiveImage height))! !

!WebElement methodsFor: 'adding images' stamp: ' 21/8/07 09:38'!
addLiveImage: aWebLiveImage imageMap: aWebImageMap
	^self add: 
		((WebImage image: aWebLiveImage)
			size: (aWebLiveImage width)@(aWebLiveImage height);
			imageMap: aWebImageMap)! !

!WebElement methodsFor: 'adding form elements' stamp: ' 21/4/07 22:07'!
addMenuAspect: aSymbol collection: aCollection selected: aSelectedCollection
	"Multiple selection menu.. Aspect of every element in aColection will be shown in menu. 
	Selection will be put in aSelectedCollection. Here also an initial selection can be set"
	^self add: (WebMenu aspect: aSymbol collection: aCollection selected: aSelectedCollection)! !

!WebElement methodsFor: 'adding form elements' stamp: ' 21/4/07 22:07'!
addMenuAspect: aSymbol collection: aCollection selectedToAspect: aSymbol2 of: anObject
	"Single selection menu.. Aspect of every element in aColection will be shown in menu. 
	Single selection will be put in an seelcted aspect of object. Here also initial selection can be set"
	^self add: 
		(WebMenu aspect: aSymbol collection: aCollection selectedToAspect: aSymbol2 of: anObject)! !

!WebElement methodsFor: 'adding form elements' stamp: ' 21/4/07 22:07'!
addMenuCollection: aCollection selected: aSelectedCollection
	"Multiple selection menu..Every element (text!!) in aColection will be shown in menu. 
	Selection will be put 	in aSelectedCollection. Here also an initial selection can be set"
	^self add: (WebMenu collection: aCollection selected: aSelectedCollection)! !

!WebElement methodsFor: 'adding form elements' stamp: ' 21/4/07 22:07'!
addMenuCollection: aCollection selectedToAspect: aSymbol2 of: anObject
	"Single selection menu. Every element (text!!) in aColection will be shown in menu. 
	Single selection will be put in an seelcted aspect of object. Here also initial selection can be set"
	^self add: (WebMenu collection: aCollection selectedToAspect: aSymbol2 of: anObject)! !

!WebElement methodsFor: 'adding images' stamp: ' 21/8/07 09:38'!
addMethodImage: aWebMethodImage
	^self add: 
		(WebImage image: aWebMethodImage)! !

!WebElement methodsFor: 'adding images' stamp: ' 21/8/07 09:38'!
addMethodImage: aWebMethodImage size: aPoint
	^self add: 
		((WebImage image: aWebMethodImage) size: aPoint)! !

!WebElement methodsFor: 'adding other elements' stamp: ' 21/8/07 09:38'!
addNbSp
	^self addText: '&nbsp;'.! !

!WebElement methodsFor: 'adding other elements' stamp: ' 21/8/07 09:38'!
addNbSp: aNumber
	aNumber timesRepeat: [self addNbSp].! !

!WebElement methodsFor: 'adding links' stamp: 'janko 6/28/2008 18:08'!
addNilLinkText: aString
	^self addLinkTo: 'javascript:nic()' text: aString! !

!WebElement methodsFor: 'adding links' stamp: ' 21/8/07 09:38'!
addPDFLinkTo: anObject
	^self add: ((WebLink linkTo: anObject) addGif: #pdfBigGif)! !

!WebElement methodsFor: 'adding other elements' stamp: ' 21/8/07 09:38'!
addPageBreak
	"for printing to printer, to break into a new page!!"
	^self add: ((WebElement newClass: #pageBreak) 
		addComment: 'Page break'; yourself).! !

!WebElement methodsFor: 'adding other elements' stamp: ' 21/8/07 09:38'!
addParagraph
	^self add: (WebSeparator paragraph).! !

!WebElement methodsFor: 'adding form elements' stamp: ' 21/4/07 22:07'!
addPasswordFieldAspect: aSymbol for: anObject
	^self add: (WebInputField new type: #password; aspect:  aSymbol for: anObject)! !

!WebElement methodsFor: 'adding form elements' stamp: ' 21/4/07 22:07'!
addPasswordFieldAspect: aSymbol for: anObject size: aNumber
	^self add: ((WebInputField new type: #password; aspect:  aSymbol for: anObject) size: aNumber)! !

!WebElement methodsFor: 'adding images' stamp: ' 21/8/07 09:38'!
addPng: aSymbol
	"this method will be called in your webStyle and this method 
	should return a png in byte array format"
	^self add: (WebImage png: aSymbol)! !

!WebElement methodsFor: 'adding images' stamp: ' 21/8/07 09:38'!
addPng: aSymbol size: aPoint
	"this method will be called in your webStyle and this method 
	should return a png in byte array format"
	^self add: (WebImage png: aSymbol size: aPoint)! !

!WebElement methodsFor: 'adding text' stamp: 'janko 7/30/2008 15:15'!
addRedStar
	"red * for mandatory fields in a form"
	^self addText: '<sup>*</sup>'! !

!WebElement methodsFor: 'adding ajax components' stamp: ' 21/8/07 09:38'!
addRichEditorAspect: aSymbol for: anObject
	"JavaScript HTML WYSIWYG editor instead of textarea"
	^self add: (WebRichEditor new aspect:  aSymbol for: anObject)! !

!WebElement methodsFor: 'adding ajax components' stamp: ' 21/8/07 09:38'!
addRichEditorAspect: aSymbol for: anObject size: aPoint
	"JavaScript HTML WYSIWYG editor instead of textarea"
	^self add: ((WebRichEditor new aspect:  aSymbol for: anObject) size: aPoint)! !

!WebElement methodsFor: 'adding other elements' stamp: ' 21/8/07 09:38'!
addRulerSize: aNumber
	^self add: (WebSeparator rulerSize: aNumber).! !

!WebElement methodsFor: 'adding links' stamp: ' 21/8/07 09:38'!
addSecureLinkTo: anObject text: aString
	^self add: ((WebLink text: aString  linkTo: anObject) security: #grayed)! !

!WebElement methodsFor: 'adding links' stamp: ' 21/8/07 09:38'!
addSecureLinkTo: anObject 
	text: aString parameter: aParmString value: aValueString

	^self add: ((WebLink text: aString  linkTo: anObject 
		parameter: aParmString value: aValueString) security: #grayed)! !

!WebElement methodsFor: 'adding links' stamp: ' 21/8/07 09:38'!
addSecureLinkTo: anObject text: aString view: aViewString
	^self add: ((WebLink text: aString  linkTo: anObject 
		view: aViewString) security: #grayed)! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addSpace
	"just a simple space, nothing more"
	^self addText: ' '! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addText: aString
	^self add: (WebText text: aString)! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addText: aString attributes:  aSymbolArray
	^self add: ((WebText text: aString) textAttributes: aSymbolArray)! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addText: aString attributes:  aSymbolArray color: aColorSymbol
	^self add: ((WebText text: aString) attributes: aSymbolArray; color: aColorSymbol)! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addText: aString attributes:  aSymbolArray font: aFontString
	^self add: ((WebText text: aString) attributes: aSymbolArray; font: aFontString)! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addText: aString attributes:  aSymbolArray font: aFontString color: aColorSymbol
	^self add: ((WebText text: aString) 
		textAttributes: aSymbolArray; font: aFontString; color: aColorSymbol)! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addText: aString  color: aColorSymbol
	^self add: ((WebText text: aString)  color: aColorSymbol)! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addText: aString font: aFontString
	^self add: ((WebText text: aString) font: aFontString)! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addText: aString  font: aFontString color: aColorSymbol
	^self add: ((WebText text: aString)  font: aFontString; color: aColorSymbol)! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addText: aString header: aNumber
	^self add: ((WebText text: aString) header: aNumber)! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addText: aString header: aNumber color:  aColorSymbol
	^self add: ((WebText text: aString) header: aNumber; color:  aColorSymbol)! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addText: aString header: aNumber font: aFontString
	^self add: ((WebText text: aString) header: aNumber; font: aFontString)! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addText: aString header: aNumber font: aFontString color: aColorSymbol
	^self add: ((WebText text: aString) 
		header: aNumber; font: aFontString; color: aColorSymbol)! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addText: aString style: aStyleString
	^self add: ((WebText text: aString) style: aStyleString)! !

!WebElement methodsFor: 'adding form elements' stamp: ' 21/4/07 22:07'!
addTextAreaAspect: aSymbol for: anObject
	^self add: (WebTextArea new aspect:  aSymbol for: anObject)! !

!WebElement methodsFor: 'adding form elements' stamp: ' 21/4/07 22:07'!
addTextAreaAspect: aSymbol for: anObject size: aPoint
	^self add: ((WebTextArea new aspect:  aSymbol for: anObject) size: aPoint)! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addTextAspect: aSymbol for: anObject
	"depening on portlet mode (#view or #edit) return only text or make an input field"
	^self app inEditMode
		ifTrue: [self addInputFieldAspect: aSymbol for: anObject]
		ifFalse: [self addText: (anObject perform: aSymbol)]! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addTextAspect: aSymbol for: anObject attributes: aSymbolOrArray
	"depening on portlet mode (#view or #edit) return only text or make an input field"
	^self app inEditMode
		ifTrue: [self addInputFieldAspect: aSymbol for: anObject]
		ifFalse: [self addText: (anObject perform: aSymbol) attributes: aSymbolOrArray]! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addTextBig: aString
	^self addText: '<big>', aString, '</big>'! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addTextBold: aString
	^self addText: aString attributes: #bold! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addTextBoldAspect: aSymbol for: anObject
	"depening on portlet mode (#view or #edit) return only text or make an input field"
	^self addTextAspect: aSymbol for: anObject attributes: #bold! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addTextCode: aString
	^self addText: '<code>', aString, '</code>'.! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addTextH1: aString
	^self addText: aString header: 1! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addTextH2: aString
	^self addText: aString header: 2! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addTextH3: aString
	^self addText: aString header: 3! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addTextH4: aString
	^self addText: aString header: 4! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addTextH5: aString
	^self addText: aString header: 5! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addTextItalic: aString
	^self addText: aString attributes: #italic! !

!WebElement methodsFor: 'adding text' stamp: ' 21/8/07 09:38'!
addTextPreformated: aString
	^self addText: '<pre>', aString, '</pre>'.! !

!WebElement methodsFor: 'adding text' stamp: 'mivsek 1/10/2008 22:19'!
addTextSmall: aString
	aString isNil ifTrue: [^self].
	^self addText: '<small>', aString, '</small>'! !

!WebElement methodsFor: 'private-ajax' stamp: ' 21/4/07 22:07'!
ajaxCallUrl
	"relative url of an app observee object. This url is used for AJAX calls back to the server"
	| app object |
	app := self app.
	object := app observee isVersionedObject 
		ifTrue: [app observee currentVersion] ifFalse: [app observee].
	^app site urlResolver halfUrlFor: object! !

!WebElement methodsFor: 'private-ajax' stamp: ' 21/4/07 22:07'!
ajaxCallUrlExtended
	"relative url of an app observee object. This url is used for AJAX calls back to the server"
	"also includes 'ajaxRequest' in query part, to distinguish ajax request from others"
	| viewParm versionParm |
	viewParm := 'view=', self form view asString.
	versionParm := ''.
	self app observee isVersionedObject ifTrue:
		[self app observee isCurrentVersion ifFalse: 
			[versionParm := 'version=' self app observee versionNumber, '&'] ].
	^self ajaxCallUrl, '?', viewParm, '&', versionParm, 'ajaxRequest'! !

!WebElement methodsFor: 'private-ajax' stamp: ' 21/4/07 22:07'!
ajaxCallUrlParametersFor: anElementOrId
	"ajax url must have 'ajaxRequest' parm in query part!! Deal also with versioned objects"
	| idSymbol ajaxForm viewParm versionParm |
	idSymbol := anElementOrId isSymbol 
		ifTrue: [ajaxForm := self form. anElementOrId] 
		ifFalse: [anElementOrId isNil
			ifTrue: [ajaxForm := self app form. #nil] 
			ifFalse: [ajaxForm := anElementOrId form. 
				anElementOrId registerId. anElementOrId id] ].
	viewParm := 'view=', ajaxForm view asString.
	versionParm := ''.
	self app observee isVersionedObject ifTrue:
		[self app observee isCurrentVersion 
			ifFalse: [versionParm := 'version=', self app observee versionNumber, '&'] ].
	^viewParm, '&', versionParm, 'ajaxRequest&ajaxGetElementId=', idSymbol asString! !

!WebElement methodsFor: 'attributes' stamp: 'mivsek 6/13/2007 16:12'!
align: aSymbol
	"element alignment #left #center #right"
	self attributesAt: #align put: aSymbol asString! !

!WebElement methodsFor: 'subelements' stamp: ' 21/4/07 22:07'!
allElements
	"get a collection of all subelements of elements down in a hierarcy of this element"
	| collection |
	collection := OrderedCollection new.
	self elements notNil ifTrue:
		[collection addAll: self elements.
		self elements do: [:each | collection addAll: each allElements] ].
	^collection

"
| el |
el := WebElement new.
el add: (WebLink image: nil  linkTo: nil).
el add: (WebText text: 'aaa').
el allElements.
"! !

!WebElement methodsFor: 'subelements' stamp: ' 21/4/07 22:07'!
allElementsOfClass: aClass

"returns a collection of subelements of given class, actualy type of an element. Broswe entire hierarchy of that element"

	| found |
	found := OrderedCollection new.
	(self class = aClass) ifTrue: [found add: self].
      (self elements isKindOf: Collection) 
		ifTrue: [self elements do: 
			[:element | found addAll: (element allElementsOfClass: aClass)]]
		ifFalse: 
			[(self elements isKindOf: WebElement)   "nil or other excluded"
				ifTrue: [found addAll:       "only one element"
					(self elements allElementsOfClass: aClass)]].
	^found! !

!WebElement methodsFor: 'subelements' stamp: ' 21/4/07 22:07'!
allOOLinks
	"return collection of all objects to which this page has a link"
	| ooLinks |
	ooLinks := self allElements select: [:element | 
		(element isKindOf: WebLink) and: [element ooReference notNil] ].
	^ooLinks collect: [:each | each ooReference].

"
| el |
el := WebElement new.
el add: (WebLink linkTo: Date today); addText: 'aaaa'.
el allOOLinks
"! !

!WebElement methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
app
	"return anApplication on which we are composing a web page"
	| object |
	self parent isNil ifTrue: 
		[object := self firstAppFromStack.
		^object isNil ifTrue: [nil] ifFalse: [object ] ].
	^parent app! !

!WebElement methodsFor: 'private-attributes' stamp: ' 21/4/07 22:07'!
attributes
	^attributes! !

!WebElement methodsFor: 'private-attributes' stamp: ' 21/4/07 22:07'!
attributesAt: aSymbol
	self attributes isNil ifTrue: [^nil].
	^self attributes at: aSymbol ifAbsent: [nil]! !

!WebElement methodsFor: 'private-attributes' stamp: ' 21/4/07 22:07'!
attributesAt: aSymbol add: aString
	"some attributes can have more than one value (like javascript for events)"
	self attributes isNil ifTrue: [self initAttributes].
	(self attributes at: aSymbol ifAbsentPut: [OrderedCollection new]) add: aString! !

!WebElement methodsFor: 'private-attributes' stamp: ' 21/4/07 22:07'!
attributesAt: aSymbol put: aString
	self attributes isNil ifTrue: [self initAttributes].
	self attributes at: aSymbol put: aString! !

!WebElement methodsFor: 'events-ajax' stamp: 'np 3/31/2008 01:39'!
beDraggable
	self beDraggableAndRevert: false! !

!WebElement methodsFor: 'events-ajax' stamp: 'np 3/31/2008 17:21'!
beDraggableAndRevert: aBoolean
	self registerId.
	self app ensureJsResourceForScriptaculous.
	self app ensureJavascriptForScriptaculousInHeader.
	self scriptAfter: '// <!![CDATA[ 
	new Draggable("', self id asString,'",{revert:', aBoolean asString, '});
	// ]]> 
	'! !

!WebElement methodsFor: 'events-ajax' stamp: 'np 4/1/2008 17:50'!
beSortable
	| script |
	self registerId.
	self app ensureJsResourceForScriptaculous.
	self app ensureJavascriptForScriptaculousInHeader.
	
	script := '// <!![CDATA[ 
		Sortable.create("', self id asString,'",{'.
	(self isKindOf: WebList) ifFalse: [
		self setDiv.
		script := script,  'tag: "div", '].
	script := script, 'dropOnEmpty: true, constraint: false});
		// ]]>'.
	self scriptAfter: script! !

!WebElement methodsFor: 'tables' stamp: ' 21/4/07 22:07'!
cell
	"return a current table cell"
	self initTableIfNessesary.
	^self otherAt: #currentCell! !

!WebElement methodsFor: 'private' stamp: ' 21/4/07 22:07'!
cell: aWebTableCell
	self initTableIfNessesary.
	self otherAt: #currentCell put: aWebTableCell.! !

!WebElement methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
checkAndInitElements

	"lazy initialization od elements, when needed. Only for composite elements, else error"
	
	self isComposite 
		ifTrue:[(elements = nil) ifTrue: [elements := OrderedCollection new.       ]]
		ifFalse: [Smalltalk error: 'This is not a composite WebElement'].! !

!WebElement methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
class: aSymbol
	"style class of that element. Used for Cascaded Style Sheets"
	self attributesAt: #class put: aSymbol! !

!WebElement methodsFor: 'subelements' stamp: ' 21/4/07 22:07'!
clear
	"remove all subelements"
	self initElements.! !

!WebElement methodsFor: 'tables' stamp: ' 21/4/07 22:07'!
clearTableInfo
	"clear all table information"
	self initTable! !

!WebElement methodsFor: 'private' stamp: ' 21/4/07 22:07'!
colorValue: aColorSymbol
	"Hex number format is: #rrbbgg if not in that format, then it can be a color name as defined in 
	class variable Colors (class method initColorDictionary). If unknown color, then attribute is set to nil"
	| attribute |
	attribute := aColorSymbol asString asLowercase asSymbol.
	attribute := WebElement colorDictionary at: attribute ifAbsent:
		[attribute asString do: [:chr | 
			(chr isDigit or: [chr asInteger between: $a asInteger and: $f asInteger]) ifFalse: [^self] ] ].
	^'#', (attribute asString copyWithout: $# )

"WebElement new colorValue: #red"! !

!WebElement methodsFor: 'private' stamp: ' 21/4/07 22:07'!
div
	^self otherAt: #divElement ifAbsent: [false]! !

!WebElement methodsFor: 'private' stamp: ' 21/4/07 22:07'!
div: aBoolean
	"if an element is DIV tag"
	self otherAt: #divElement put: aBoolean! !

!WebElement methodsFor: 'private' stamp: ' 21/4/07 22:07'!
elements
	elements isNil ifTrue: [self initElements].
	^elements! !

!WebElement methodsFor: 'private-identation' stamp: ' 21/4/07 22:07'!
eol
	self setNewline.
	^String with: Character cr with: Character lf.! !

!WebElement methodsFor: 'subelements' stamp: ' 21/4/07 22:07'!
first
	"Answer the first element.  If the receiver is empty, provide an error 
	notification."

	self checkAndInitElements.  "if not composite element, then error"
	^elements first! !

!WebElement methodsFor: 'private' stamp: ' 21/4/07 22:07'!
form
	"aWebForm on which that element was created. Used for form element model adapting. 
	Usually an app form for view, on which this element was created.
	Use it in conjunction with #method for Ajax"
	^self otherAt: #form ifAbsent: 
		[self parent notNil 
			ifTrue: [self parent form]
			ifFalse: [nil]  ] "maybe app form?"! !

!WebElement methodsFor: 'private' stamp: ' 21/4/07 22:07'!
form: aSymbol
	"form on which that element was created. Used for form element model adapting"
	"Usually an app form for view, on which this element was created."
	self otherAt: #form put: aSymbol! !

!WebElement methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
helpLink
	^self app helpLink! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
hide
	"hide me from web page"
	self registerId.
	self addText: '<script>Element.hide(''', self id, ''')</script>'! !

!WebElement methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
id
	^self attributesAt: #id! !

!WebElement methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
id: aSymbol
	"set a unique id of that element on a page. Used in javascript, urls, style sheets"
	| app |
	app := self app.
	self id notNil ifTrue: [(app form isRegistered: self) ifTrue: [app form removeId: self id]].
	self attributesAt: #id put: aSymbol asSymbol.
	app form registerIdFor: self.! !

!WebElement methodsFor: 'private-identation' stamp: ' 21/4/07 22:07'!
ident
	"ident this tag to its level"
	| depth eol|
	eol := ''.
	self shouldIdent & self isNewline not ifTrue: [eol := self eol]. 
	depth := (self identationLevel - self identDepth) max: 0.
	self identDepth: self identationLevel.
	^eol, (String new: depth withAll: Character tab )! !

!WebElement methodsFor: 'private-identation' stamp: ' 21/4/07 22:07'!
identDepth
	"how idented we are"
	| app |
	app := self app.
	^app notNil ifTrue: [app identDepth] ifFalse: [0].! !

!WebElement methodsFor: 'private-identation' stamp: ' 21/4/07 22:07'!
identDepth: aNumber
	"how idented we are"
	| app |
	app := self app.
	^app isNil ifTrue: [0] ifFalse: [app identDepth: aNumber]! !

!WebElement methodsFor: 'private-identation' stamp: ' 21/4/07 22:07'!
identMore
	"ident inside tag content to its level and one deepier"
	self isNewline ifFalse: [self eol]. 
	self identDepth: self identationLevel+1.
	^String new: self identationLevel+1 withAll: $ ! !

!WebElement methodsFor: 'private-identation' stamp: ' 21/4/07 22:07'!
identationLevel	
	"How much we need to ident. Top element has level 0"
	^self parent notNil
		ifTrue: [self shouldIdent ifTrue: [self parent identationLevel + 1] ifFalse: [self parent identationLevel]]
		ifFalse: [0]! !

!WebElement methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initAttributes
	attributes := Dictionary new.! !

!WebElement methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initElements
	elements := OrderedCollection new.! !

!WebElement methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initOther
	other := Dictionary new.! !

!WebElement methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initScriptBefore
	self other removeKey: #scriptBefore ifAbsent: []! !

!WebElement methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initTable
	"just remove from other, later will be initalized lazily"
	self other isNil ifTrue: [^nil].
	self other 
		removeKey: #currentTable ifAbsent: [];
		removeKey: #currentRow ifAbsent: [];
		removeKey: #currentCell ifAbsent: [].! !

!WebElement methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initTableIfNessesary
	"if not already exist"
	self otherAt: #currentTable ifAbsent: [self newTable]! !

!WebElement methodsFor: 'private' stamp: ' 21/4/07 22:07'!
insideDivTag
	"to enclose or not in div tag. Only if element have any attribute!!"
	^self div | (self class == WebElement and: [self attributes notNil]).! !

!WebElement methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isComposite

	"true, if this element is allowed to be composed by subelements.
	This method should be overriden by subclasess if a new element is not a 	composite."

	^true! !

!WebElement methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isFormElement
	^false! !

!WebElement methodsFor: 'private-identation' stamp: ' 21/4/07 22:07'!
isNewline
	"are we at start of new line?"
	| app |
	app := self app.
	^app notNil ifTrue: [app isNewline] ifFalse: [false]! !

!WebElement methodsFor: 'testing' stamp: ' 21/8/07 09:38'!
isRespondingStreamed
	^false! !

!WebElement methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isWebApplication
	^false! !

!WebElement methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isWebElement
	^true! !

!WebElement methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isWebPage
	^false! !

!WebElement methodsFor: 'subelements' stamp: ' 21/4/07 22:07'!
last
	"Answer the last element.  If the receiver is empty, create an error notification."

	self checkAndInitElements.  "if not composite element, then error"
	^elements last! !

!WebElement methodsFor: 'private-identation' stamp: ' 21/4/07 22:07'!
level	
	"How deep we are in element hierarchy. Top element has level 0"
	^self parent notNil
		ifTrue: [self parent level + 1]
		ifFalse: [0]! !

!WebElement methodsFor: 'private' stamp: ' 21/4/07 22:07'!
method
	"a method which created this element, always from some App. 
	Valid only for aWebElement, never for its subclasses!!"
	^self otherAt: #method ifAbsent: [nil]! !

!WebElement methodsFor: 'private' stamp: ' 21/4/07 22:07'!
method: aSymbol
	"a method which created this element, always from some App. 
	Valid only for aWebElement, never for its subclasses!!"
	self otherAt: #method put: aSymbol.
	self form: self app form! !

!WebElement methodsFor: 'tables' stamp: ' 21/4/07 22:07'!
newCell
	"add a new cell to current table row. Return a new WebTableCell so that you can send messages
	to it immediatelly - color, width, addText etc."
	self cell: WebTableCell new.
	self row add: self cell.
	^self cell! !

!WebElement methodsFor: 'tables' stamp: ' 21/4/07 22:07'!
newRow
	"add a new row to current table. Return a new WebTableRow so that you can send messages
	to it immediatelly - color, width etc. Also reset current row and current cell to a new ones. "
	self row: WebTableRow new.
	self table add: self row.
	self newCell.
	^self row! !

!WebElement methodsFor: 'tables' stamp: ' 21/4/07 22:07'!
newTable
	"add a new table to this element. Return a new WebTable so that you can send messages
	to it immediatelly - color, width etc. Also reset current row and current cell to a new ones. "
	self table: WebTable new.
	self add: self table.
	self newRow.
	^self table! !

!WebElement methodsFor: 'events' stamp: ' 21/4/07 22:07'!
onClick: aJavascriptCode
	self attributesAt: #onClick add: aJavascriptCode! !

!WebElement methodsFor: 'events-ajax' stamp: 'np 3/31/2008 00:06'!
onClickDoEffect: aSymbol
	self onClickDoEffect: aSymbol for: self! !

!WebElement methodsFor: 'events-ajax' stamp: 'np 3/30/2008 23:48'!
onClickDoEffect: aSymbol for: anElementOrId
	| idSymbol |
	self app ensureJsResourceForScriptaculous.
	self app ensureJavascriptForScriptaculousInHeader.
	idSymbol := anElementOrId isSymbol 
		ifTrue: [anElementOrId] ifFalse: [anElementOrId registerId. anElementOrId id].
	self onClick: 'new Effect.', aSymbol asString, '(''', idSymbol asString, ''', 1);'.! !

!WebElement methodsFor: 'events-ajax' stamp: 'np 3/31/2008 17:02'!
onClickDoToggleEffect: aSymbol for: anElementOrId
	| idSymbol |
	self app ensureJsResourceForScriptaculous.
	self app ensureJavascriptForScriptaculousInHeader.
	idSymbol := anElementOrId isSymbol 
		ifTrue: [anElementOrId] ifFalse: [anElementOrId registerId. anElementOrId id].
	self onClick: 'new Effect.toggle(''', idSymbol asString, ''', ''', aSymbol asString, ''');'.! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onClickHide: anElementOrId
	"hide specified element on mouse click on me"
	| idSymbol |
	idSymbol := anElementOrId isSymbol 
		ifTrue: [anElementOrId] ifFalse: [anElementOrId registerId. anElementOrId id].
	self onClick: 'Element.hide(''', idSymbol asString, ''')'.! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onClickShow: anElementOrId
	"show specified element on mouse click on me"
	| idSymbol |
	idSymbol := anElementOrId isSymbol 
		ifTrue: [anElementOrId] ifFalse: [anElementOrId registerId. anElementOrId id].
	self onClick: 'Element.show(''', idSymbol asString, ''')'.! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onClickToggle: anElementOrId
	"toggle visibility of specified element on mouse click on me"
	| idSymbol |
	idSymbol := anElementOrId isSymbol 
		ifTrue: [anElementOrId] ifFalse: [anElementOrId registerId. anElementOrId id].
	self onClick: 'Element.toggle(''', idSymbol asString, ''')'.! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onClickUpdate: anElementOrId
	"update (and show if not yet) a specified element from server (AJAX)"
	self onClickUpdate: anElementOrId with: nil! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onClickUpdate: anElementOrId with: aParmString
	"update a specified element from server (AJAX) wtih calling an element method with a parameter"
	| idSymbol url parms  |
	idSymbol := anElementOrId isSymbol 
		ifTrue: [anElementOrId] ifFalse: [anElementOrId registerId. anElementOrId id].
	url := self ajaxCallUrl. 
	parms:= self ajaxCallUrlParametersFor: anElementOrId.
	aParmString notNil ifTrue: [parms := parms, '&parm=', aParmString].
	self onClick: 'new Ajax.Updater(''', idSymbol asString, ''', ''', url, 
		''', {method: ''post'', postBody: ''', parms, ''', evalScripts: true})'! !

!WebElement methodsFor: 'events' stamp: ' 21/4/07 22:07'!
onDblClick: aJavascriptCode
	self attributesAt: #onDblClick add: aJavascriptCode! !

!WebElement methodsFor: 'events' stamp: ' 21/4/07 22:07'!
onKeyDown: aJavascriptCode
	self attributesAt: #onKeyDown add: aJavascriptCode! !

!WebElement methodsFor: 'events' stamp: ' 21/4/07 22:07'!
onKeyPress: aJavascriptCode
	self attributesAt: #onKeyPress add: aJavascriptCode! !

!WebElement methodsFor: 'events' stamp: ' 21/4/07 22:07'!
onKeyUp: aJavascriptCode
	self attributesAt: #onKeyUp add: aJavascriptCode! !

!WebElement methodsFor: 'events' stamp: ' 21/4/07 22:07'!
onMouseDown: aJavascriptCode
	self attributesAt: #onMouseDown add: aJavascriptCode! !

!WebElement methodsFor: 'events' stamp: ' 21/4/07 22:07'!
onMouseMove: aJavascriptCode
	self attributesAt: #onMouseMove add: aJavascriptCode! !

!WebElement methodsFor: 'events' stamp: ' 21/4/07 22:07'!
onMouseOut: aJavascriptCode
	self attributesAt: #onMouseOut add: aJavascriptCode! !

!WebElement methodsFor: 'events-ajax' stamp: 'np 3/31/2008 01:15'!
onMouseOutDoEffect: aSymbol
	self onMouseOutDoEffect: aSymbol for: self! !

!WebElement methodsFor: 'events-ajax' stamp: 'np 3/31/2008 01:15'!
onMouseOutDoEffect: aSymbol for: anElementOrId
	| idSymbol |
	self app ensureJsResourceForScriptaculous.
	self app ensureJavascriptForScriptaculousInHeader.
	idSymbol := anElementOrId isSymbol 
		ifTrue: [anElementOrId] ifFalse: [anElementOrId registerId. anElementOrId id].
	self onMouseOut: 'new Effect.', aSymbol asString, '(''', idSymbol asString, ''', 1);'.! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onMouseOutHide: anElementOrId
	"hide specified element when mouse get out of me"
	| idSymbol |
	idSymbol := anElementOrId isSymbol 
		ifTrue: [anElementOrId] ifFalse: [anElementOrId registerId. anElementOrId id].
	self onMouseOut: 'Element.hide(''', idSymbol asString, ''')'.! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onMouseOutShow: anElementOrId
	"show specified element when mouse get out of me"
	| idSymbol |
	idSymbol := anElementOrId isSymbol 
		ifTrue: [anElementOrId] ifFalse: [anElementOrId registerId. anElementOrId id].
	self onMouseOut: 'Element.show(''', idSymbol asString, ''')'.! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onMouseOutUpdate: anElementOrId
	"update (and show if not yet) a specified element from server (AJAX)"
	self onMouseOutUpdate: anElementOrId with: nil! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onMouseOutUpdate: anElementOrId with: aParmString
	"update a specified element from server (AJAX) wtih calling an element method with a parameter"
	| idSymbol url parms  |
	idSymbol := anElementOrId isSymbol 
		ifTrue: [anElementOrId] ifFalse: [anElementOrId registerId. anElementOrId id].
	url := self ajaxCallUrl. 
	parms:= self ajaxCallUrlParametersFor: anElementOrId.
	aParmString notNil ifTrue: [parms := parms, '&parm=', aParmString].
	self onMouseOut: 'new Ajax.Updater(''', idSymbol asString, ''', ''', url, 
		''', {method: ''post'', postBody: ''', parms, ''', evalScripts: true})'! !

!WebElement methodsFor: 'events' stamp: ' 21/4/07 22:07'!
onMouseOver: aJavascriptCode
	self attributesAt: #onMouseOver add: aJavascriptCode! !

!WebElement methodsFor: 'events-ajax' stamp: 'np 3/31/2008 01:15'!
onMouseOverDoEffect: aSymbol
	self onMouseOverDoEffect: aSymbol for: self! !

!WebElement methodsFor: 'events-ajax' stamp: 'np 3/31/2008 01:15'!
onMouseOverDoEffect: aSymbol for: anElementOrId
	| idSymbol |
	self app ensureJsResourceForScriptaculous.
	self app ensureJavascriptForScriptaculousInHeader.
	idSymbol := anElementOrId isSymbol 
		ifTrue: [anElementOrId] ifFalse: [anElementOrId registerId. anElementOrId id].
	self onMouseOver: 'new Effect.', aSymbol asString, '(''', idSymbol asString, ''', 1);'.! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onMouseOverHide: anElementOrId
	"hide specified element when mouse come over me"
	| idSymbol |
	idSymbol := anElementOrId isSymbol 
		ifTrue: [anElementOrId] ifFalse: [anElementOrId registerId. anElementOrId id].
	self onMouseOver: 'Element.hide(''', idSymbol asString, ''')'.! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onMouseOverShow: anElementOrId
	"show specified element when mouse come over me"
	| idSymbol |
	idSymbol := anElementOrId isSymbol 
		ifTrue: [anElementOrId] ifFalse: [anElementOrId registerId. anElementOrId id].
	self onMouseOver: 'Element.show(''', idSymbol asString, ''')'.! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onMouseOverSyncUpdate: anElementOrId
	"update (and show if not yet) a specified element from server (AJAX)"
	"wait until AJAX response returns (synchronous update)"
	self onMouseOverSyncUpdate: anElementOrId with: nil! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onMouseOverSyncUpdate: anElementOrId with: aParmString
	"update a specified element from server (AJAX) wtih calling an element method with a parameter"
	"wait until AJAX response returns (synchronous update)"
	| idSymbol url parms  |
	idSymbol := anElementOrId isSymbol 
		ifTrue: [anElementOrId] ifFalse: [anElementOrId registerId. anElementOrId id].
	url := self ajaxCallUrl. 
	parms:= self ajaxCallUrlParametersFor: anElementOrId.
	aParmString notNil ifTrue: [parms := parms, '&parm=', aParmString].
	self onMouseOver: 'new Ajax.Updater(''', idSymbol asString, ''', ''', url, 
		''', {method: ''post'', postBody: ''', parms, ''', evalScripts: true, asynchronous: false})'! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onMouseOverUpdate: anElementOrId
	"update (and show if not yet) a specified element from server (AJAX)"
	self onMouseOverUpdate: anElementOrId with: nil! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onMouseOverUpdate: anElementOrId with: aParmString
	"update a specified element from server (AJAX) wtih calling an element method with a parameter"
	| idSymbol url parms  |
	idSymbol := anElementOrId isSymbol 
		ifTrue: [anElementOrId] ifFalse: [anElementOrId registerId. anElementOrId id].
	url := self ajaxCallUrl. 
	parms:= self ajaxCallUrlParametersFor: anElementOrId.
	aParmString notNil ifTrue: [parms := parms, '&parm=', aParmString].
	self onMouseOver: 'new Ajax.Updater(''', idSymbol asString, ''', ''', url, 
		''', {method: ''post'', postBody: ''', parms, ''', evalScripts: true})'! !

!WebElement methodsFor: 'events' stamp: ' 21/4/07 22:07'!
onMouseUp: aJavascriptCode
	self attributesAt: #onMouseUp add: aJavascriptCode! !

!WebElement methodsFor: 'private-other' stamp: ' 21/4/07 22:07'!
other
	^other! !

!WebElement methodsFor: 'private-other' stamp: ' 21/4/07 22:07'!
otherAt: aSymbol
	^self otherAt: aSymbol ifAbsent: [nil]! !

!WebElement methodsFor: 'private-other' stamp: ' 21/4/07 22:07'!
otherAt: aSymbol ifAbsent: aBlock
	self other isNil ifTrue: [^aBlock value].
	^self other at: aSymbol ifAbsent: aBlock! !

!WebElement methodsFor: 'private-other' stamp: ' 21/4/07 22:07'!
otherAt: aSymbol ifAbsentPut: aBlock
	self other isNil ifTrue: [self initOther].
	^self other at: aSymbol ifAbsent: [self other at: aSymbol put: aBlock value]! !

!WebElement methodsFor: 'private-other' stamp: ' 21/4/07 22:07'!
otherAt: aSymbol put: anObject
	self other isNil ifTrue: [self initOther].
	^self other at: aSymbol put: anObject! !

!WebElement methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
parent
	^parent! !

!WebElement methodsFor: 'private' stamp: ' 21/4/07 22:07'!
parent: aWebElement
	parent := aWebElement! !

!WebElement methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
prepareAttributesToPrintOn: aSession
	"override if you like to change or otherwise prepare attribute values"! !

!WebElement methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
prepareToHTMLPrintOn: aSession
	"all preparations just before html rendering"
	self prepareAttributesToPrintOn: aSession.  "some pre-html preparation of values"! !

!WebElement methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
printAttribute: aNameSymbol value: aValue on: aStream for: aSession
	"if value is true/false, then if true, just print an atribute name, otherwise ommit both"
	"multivalue attributes are supposed to be javascript only!! "
	| multiValue |
	aStream nextPut:  $ . 
	((aValue ~= false) | (aValue == true)) ifTrue: [aStream nextPutAll: aNameSymbol asString].
	(aValue isKindOf: Boolean) ifTrue: [^self].
	multiValue := aValue class == OrderedCollection.    aStream nextPutAll: '="'.
	multiValue not ifTrue: [aStream 
		nextPutAll: (AIDASite convertToWeb: aValue asString on: aSession); nextPutAll: '"'. ^self].
	aValue do: [:value | 
		aStream nextPutAll:  (AIDASite convertToWeb: value asString on: aSession). 
		aStream nextPutAll: '; ']. "javascript sentence separator"
	aStream nextPutAll: '"'.! !

!WebElement methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
printAttributesOn: aStream for: aSession
	"start and end of tag is not my responsibility!!"
	self attributes isNil ifTrue: [^self].
	self attributes keysAndValuesDo: [:name :value |
		self printAttribute: name value: value on: aStream for: aSession]! !

!WebElement methodsFor: 'printing' stamp: 'janko 7/18/2008 21:29'!
printHTMLPageOn: aStream for: aRequest on: aSession
	"WebElement with some attributes is enclosed inside <div> tag, for tricks with CSS"
	self prepareToHTMLPrintOn: aSession.
	self scriptBefore notNil ifTrue: 
		[ self scriptBefore 
			printHTMLPageOn: aStream for: aRequest on: aSession ].
	self insideDivTag ifTrue: 
		[ aStream nextPutAll: self ident , '<div'.
		self printAttributesOn: aStream for: aSession.
		aStream nextPutAll: '>' , self eol ].
	elements notNil ifTrue: 
		[ elements do: 
			[ : element | 
			element notNil ifTrue: 
				[ element 
					printHTMLPageOn: aStream for: aRequest on: aSession ] ] ].
	self insideDivTag ifTrue: [ aStream nextPutAll: self ident , '</div>' , self eol ].
	self scriptAfter notNil ifTrue: 
		[ self scriptAfter 
			printHTMLPageOn: aStream for: aRequest on: aSession ]! !

!WebElement methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
registerId
	"put into current form ids dictionary for faster search from AJAX requests"
	self id notNil ifTrue: [^nil]. "must be already registered!! "
	self app notNil ifTrue: [self app form registerIdFor: self]! !

!WebElement methodsFor: 'tables' stamp: ' 21/4/07 22:07'!
row
	"return a current table row"
	self initTableIfNessesary.
	^self otherAt: #currentRow! !

!WebElement methodsFor: 'private' stamp: ' 21/4/07 22:07'!
row: aWebTableRow
	self initTableIfNessesary.
	self otherAt: #currentRow put: aWebTableRow.! !

!WebElement methodsFor: 'scripts' stamp: ' 21/4/07 22:07'!
script: aString
	"add this JavaScript  to scripts executed after this element"
	self scriptAfter: aString! !

!WebElement methodsFor: 'private' stamp: ' 21/4/07 22:07'!
scriptAfter
	^self otherAt: #scriptAfter ifAbsent: [nil]! !

!WebElement methodsFor: 'scripts' stamp: ' 21/4/07 22:07'!
scriptAfter: aString
	"add this JavaScript  to scripts executed after this element"
	(self otherAt: #scriptAfter ifAbsentPut: [WebScript new]) script: aString! !

!WebElement methodsFor: 'scripts' stamp: 'mivsek 1/10/2008 22:19'!
scriptAfterExternal: anUrlOrObject
	"add this externalJavaScript  to scripts executed after this element"
	| url |
	url := anUrlOrObject isString
		ifTrue: [anUrlOrObject]
		ifFalse: [self site urlResolver halfUrlFor: anUrlOrObject].
	(self otherAt: #scriptAfter ifAbsentPut: [WebScript new]) source: url! !

!WebElement methodsFor: 'private' stamp: ' 21/4/07 22:07'!
scriptBefore
	^self otherAt: #scriptBefore ifAbsent: [nil]! !

!WebElement methodsFor: 'scripts' stamp: ' 21/4/07 22:07'!
scriptBefore: aString
	"add this JavaScript  to scripts executed before this element"
	(self otherAt: #scriptBefore ifAbsentPut: [WebScript new]) script: aString! !

!WebElement methodsFor: 'scripts' stamp: 'mivsek 1/10/2008 22:20'!
scriptBeforeExternal: anUrlOrObject
	"add this external JavaScript  to scripts executed before this element"
	| url |
	url := anUrlOrObject isString
		ifTrue: [anUrlOrObject]
		ifFalse: [self site urlResolver halfUrlFor: anUrlOrObject].
	(self otherAt: #scriptBefore ifAbsentPut: [WebScript new]) source: url! !

!WebElement methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
session
	"return a Session for which we are composing a web page"
	self parent isNil ifTrue: [^self app session].
	^parent session! !

!WebElement methodsFor: 'private' stamp: ' 21/4/07 22:07'!
setCreationMethod
	"find a name of a method which created me"
	| context |
	context := thisContext.
	[context notNil] whileTrue: [
		(context receiver isKindOf: WebApplication) ifTrue: [^self method: context selector].
		context := context sender].! !

!WebElement methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
setDiv
	self div: true! !

!WebElement methodsFor: 'private-identation' stamp: ' 21/4/07 22:07'!
setNewline
	"we are no longer at start of line"
	| app |
	app := self app.
	app notNil ifTrue: [app setNewline].! !

!WebElement methodsFor: 'private-identation' stamp: ' 21/4/07 22:07'!
shouldIdent
	"true, if this element should be idented in html page"
	^self insideDivTag! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
show
	"show me (if not already) on web page"
	self registerId.
	self addText: '<script>Element.show(''', self id, ''')</script>'! !

!WebElement methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
site
	"return a Site on which we are composing a web page"
	| app |
	self parent isNil ifTrue: 
		[app := self firstAppFromStack.
		^app isNil ifTrue: [nil] ifFalse: [app site] ].
	^parent site! !

!WebElement methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
style
	"return a WebStyle on which we are composing a web page"
	self parent isNil ifTrue: [^self firstSessionFromStack site style].
	^parent style! !

!WebElement methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
style: aString
	"redefine a style of that element with CSS syntax, example:
	style: '{color: blue; font-style: italic}'   "
	self attributesAt: #style put: aString! !

!WebElement methodsFor: 'tables' stamp: ' 21/4/07 22:07'!
table
	"return a current web table. If not yet exist, create it"
	self initTableIfNessesary.
	^self otherAt: #currentTable! !

!WebElement methodsFor: 'private' stamp: ' 21/4/07 22:07'!
table: aWebTable
	self otherAt: #currentTable put: aWebTable.! !

!WebElement methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
title: aString
	"title of an element. it will be shown when mouse pause over it"
	self attributesAt: #title put: 
		"slovene csz are converted if char ^ is after such a char"
		((aString includes: $^ ) ifTrue: [aString convertToSloveneChars] ifFalse: [aString])! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
toogle
	"toogle element visibility on  web page"
	self registerId.
	self addText: '<script>Element.toggle(''', self id, ''')</script>'! !

!WebElement methodsFor: 'events-ajax' stamp: 'janko 7/18/2008 16:33'!
update
	"update self from server (AJAX) wtih calling an element method"
	^self updateWith: nil! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
updateEverySeconds: aNumber
	"periodically update itself from server (AJAX) with calling an element method with a parameter"
	self updateEverySeconds: aNumber with: nil! !

!WebElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
updateEverySeconds: aNumber with: aParmString
	"periodically update itself from server (AJAX) with calling an element method with a parameter"
	| url parms  |
	self registerId. self id.
	url := self ajaxCallUrl. 
	parms:= self ajaxCallUrlParametersFor: self id.
	aParmString notNil ifTrue: [parms := parms, '&parm=', aParmString].
	self scriptAfter: 'new Ajax.PeriodicalUpdater(''', self id asString, ''', ''', url, 
		''', {method: ''post'', postBody: ''', parms, 
		''', evalScripts: true, frequency: ', aNumber printString, '})'! !

!WebElement methodsFor: 'events-ajax' stamp: 'janko 7/18/2008 16:33'!
updateWith: aParmString
	"update self from server (AJAX) wtih calling an element method with a parameter"
	| url parms  |
	self registerId. self id.
	url := self ajaxCallUrl. 
	parms:= self ajaxCallUrlParametersFor: self id.
	aParmString notNil ifTrue: [parms := parms, '&parm=', aParmString].
	self scriptAfter: 'new Ajax.Updater(''', self id asString, ''', ''', url, 
		''', {method: ''post'', postBody: ''', parms, ''', evalScripts: true})'! !

WebElement subclass: #WebFieldSet
	instanceVariableNames: 'legend'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebFieldSet commentStamp: '<historical>' prior: 0!
FieldSet element groups one or more input fields together. It usually (depends on style) also enclose them with border. Legend is for writing a name of the filedset on the corner!

!WebFieldSet class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	"Answer a newly created and initialized instance."
	^super new initialize! !

!WebFieldSet class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newLegend: aString
	^self new legend: aString! !

!WebFieldSet methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	"Initialize a newly created instance. This method must answer the receiver."

	" *** Replace this comment with the appropriate initialization code *** "
	^self! !

!WebFieldSet methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
legend
	^legend! !

!WebFieldSet methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
legend: aString
	"a label to be shown on left top corner of fieldset"
	legend := aString! !

!WebFieldSet methodsFor: 'printing' stamp: 'np 8/18/2008 01:33'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	self prepareToHTMLPrintOn: aSession.
	aStream nextPutAll: self ident , '<fieldset'.
	self 
		printAttributesOn: aStream
		for: aSession.
	aStream nextPutAll: '>' , self eol.
	self legend notNil ifTrue: [
		aStream nextPutAll: self identMore , '<legend>' , 
		(AIDASite convertToWeb: self legend on: aSession) , 
		'</legend>' , self eol].
	super 
		printHTMLPageOn: aStream for: aRequest on: aSession.
	aStream nextPutAll: self ident , '</fieldset>' , self eol! !

WebElement subclass: #WebForm
	instanceVariableNames: 'view ids fields tabOrder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebForm class methodsFor: 'constants' stamp: ' 21/4/07 22:07'!
idFieldName
	"this name is used to name a hidden field which is included in every form to identify 
	it from other forms. The name is so strange so that you probably won't name your field the same"

	^'id5273'! !

!WebForm methodsFor: 'model adapting' stamp: 'mivsek 8/21/2007 11:09'!
acceptFormInputFrom: aRequest
 	"read a post data from request and write values to the valueModels of fields in a form.
	For checkboxes and radiobuttons, a set of values is expected under the same name"
	| postData |
	postData := HTTPPostDataArray new. "like Dictionary, but multiple values per key"
	self uncheckAllChecboxes. self uncheckAllRadioButtons.
	aRequest  postKeysAndValuesDo: [:key :value | postData at: key put: value].
	self fields keysAndValuesDo: [:key :field | 
		(postData includesKey: key) ifTrue:
			[field isCheckBox ifTrue: [field acceptFormInputFrom: postData].
			field isRadioButton ifTrue: [field acceptFormInputFrom: postData].
			field isMenu ifTrue: [field acceptFormInputFrom: postData].
			field isButton ifTrue: ["do nothing"].
			field isFileInputField ifTrue: 
				[field isStreamed ifFalse: [field value: (postData at: key)] "if streamed then already read". 
				field acceptFileAttributesFrom: aRequest field: key].
			(field isInputField | field isTextArea)  
				ifTrue: [field saveThroughAdapterValue: (postData at: key)]
				]]! !

!WebForm methodsFor: 'model adapting' stamp: ' 21/4/07 22:07'!
adaptFormElement: aWebFormElement
	"set appropriate aspect adaptor for that element"
	aWebFormElement isFormElement ifFalse: [^nil].
	(aWebFormElement isRadioButton | aWebFormElement isCheckBox)
		ifTrue: [aWebFormElement joinToForm: self]
		ifFalse:	[aWebFormElement adapt]! !

!WebForm methodsFor: 'private-fields' stamp: ' 21/8/07 09:38'!
addToFields: aWebFormElement
	"and to tab order"
	(self alreadyRegistered: aWebFormElement) ifTrue: [^self error: 'already exist!!'].
	self fields at: aWebFormElement name put: aWebFormElement.
	self tabOrder add: aWebFormElement.
	aWebFormElement isStreamed ifTrue: [self streamedFieldSet add: aWebFormElement].
	^self fieldSet add: aWebFormElement! !

!WebForm methodsFor: 'model adapting' stamp: ' 21/8/07 09:38'!
ajaxAcceptFormInputFrom: aRequest
 	"Ajax have only one input per request!!"
	| k postData |
	aRequest isPost not ifTrue:
			[k :=  aRequest queryData keys detect: [:each | 'field*' match: each] ifNone: nil.
			postData := Dictionary new at: k put: (aRequest queryAt: k); yourself]
		ifFalse: 
			[k :=  aRequest postDataKeys detect: [:each | 'field*' match: each] ifNone: nil.
			postData := Dictionary new at: k put: (aRequest postDataAt: k) value; yourself].
	self fields keysAndValuesDo: [:key :field | 
		(postData includesKey: key) ifTrue:
			[field isCheckBox ifTrue: [field acceptFormInputFrom: postData].
			field isRadioButton ifTrue: [field acceptFormInputFrom: postData].
			field isMenu ifTrue: [field acceptFormInputFrom: postData].
			field isButton ifTrue: ["do nothing"].
			field isFileInputField ifTrue: 
				[field value: (postData at: key). 
				field acceptFileAttributesFrom: aRequest field: key].
			(field isInputField | field isTextArea) 
				ifTrue: [field saveThroughAdapterValue: (postData at: key)]
				]]! !

!WebForm methodsFor: 'private-fields' stamp: ' 21/4/07 22:07'!
alreadyRegistered: aWebFormElement
	^self fieldSet includes: aWebFormElement! !

!WebForm methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
app
	^self parent! !

!WebForm methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
clear
	"delete all elements on the form. Also reset instance variables fields and fieldNum"
	super clear.
	self initFields. self initTabOrder.! !

!WebForm methodsFor: 'private-ids' stamp: ' 21/4/07 22:07'!
elementForMethod: aSymbol
	^self registeredSet detect: [:each | each method = aSymbol] ifNone: [nil]! !

!WebForm methodsFor: 'private-ids' stamp: ' 21/4/07 22:07'!
elementId: aSymbol
	^self ids at: aSymbol ifAbsent: [nil].! !

!WebForm methodsFor: 'private-fields' stamp: ' 21/4/07 22:07'!
fieldSet
	^self fields at: #set ifAbsent: [self initFieldSet].! !

!WebForm methodsFor: 'tabulation' stamp: ' 21/4/07 22:07'!
fieldWithTabIndex: aNumber	
	"return a form element with that tab index"
	^self tabOrder detect: [:field | field tabIndex = aNumber] ifNone: [nil].! !

!WebForm methodsFor: 'private-fields' stamp: ' 21/4/07 22:07'!
fields
	"return a dictionary of all fields in this form. a field name is used as a key"
	fields isNil ifTrue: [self initFields].
	^fields! !

!WebForm methodsFor: 'private-ids' stamp: ' 21/4/07 22:07'!
ids
	"dictionary of elements which have id defined, for fast access to them from AJAX requests"
	ids isNil ifTrue: [self initIds].
	^ids! !

!WebForm methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initFieldSet
	"with all fields for fast check, if field is already registered"
	^self fields at: #set put: Set new.! !

!WebForm methodsFor: 'initialize-release' stamp: ' 21/8/07 09:38'!
initFields
	fields := Dictionary new.
	self initFieldSet. "with all fields for fast check, if field is already registered"
	self initStreamedFieldSet! !

!WebForm methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initIdField
	"hidden id field in a form will provide  view identification of posted form"
	self add: (WebInputField new 
		name: (self class idFieldName); 
		type: #hidden;
		aspect: #view 
		for: self app).! !

!WebForm methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initIds
	ids := Dictionary new.
	ids at: #setOfRegisteredElements put: Set new.! !

!WebForm methodsFor: 'initialize-release' stamp: ' 21/8/07 09:38'!
initStreamedFieldSet
	"all fields which demands request post data streaming directly to output stream"
	^self fields at: #streamedFields put: Set new.! !

!WebForm methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initTabOrder
	tabOrder := OrderedCollection new.! !

!WebForm methodsFor: 'private-ids' stamp: ' 21/4/07 22:07'!
isRegistered: anElement
	^self registeredSet includes: anElement! !

!WebForm methodsFor: 'private-ids' stamp: ' 21/4/07 22:07'!
isRegisteredMethod: aSymbol
	aSymbol = #'doesNotUnderstand:' ifTrue: [^false]. "WebStyle methods are somehow such"
	^self registeredSet contains: [:each | each method = aSymbol].! !

!WebForm methodsFor: 'private-ids' stamp: ' 21/4/07 22:07'!
nextId
	" example: #id9 "
	^('id', (self ids size+1) printString) asSymbol! !

!WebForm methodsFor: 'private-tabulation' stamp: ' 21/4/07 22:07'!
nextTabOrderFrom: aFormElement
	"next element to tab into"
	| inx |
	self tabOrder isEmpty ifTrue: [^nil].
	inx := self tabOrder indexOf: aFormElement.
	inx = 0 ifTrue: [^self tabOrder first].
	inx = self tabOrder size ifTrue: [^self tabOrder last].
	^self tabOrder at: inx+1! !

!WebForm methodsFor: 'events' stamp: ' 21/4/07 22:07'!
onReset: aJavascriptCode
	self attributesAt: #onReset put: aJavascriptCode! !

!WebForm methodsFor: 'events' stamp: ' 21/4/07 22:07'!
onSubmit: aJavascriptCode
	self attributesAt: #onSubmit put: aJavascriptCode! !

!WebForm methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
prepareToHTMLPrintOn: aSession
	super prepareToHTMLPrintOn: aSession.
	self reorderTabulation. "late reordering to ensure that all form elements are present"! !

!WebForm methodsFor: 'printing' stamp: 'janko 7/18/2008 20:23'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	| action |
	self prepareToHTMLPrintOn: aSession.
	action := aSession fullUrlForCurrentPage.
	aStream nextPutAll: self ident , '<form  method=post action="' , action , '"' , ' enctype="multipart/form-data"'.
	self 
		printAttributesOn: aStream
		for: aSession.
	aStream nextPutAll: '>' , self eol.
	super 
		printHTMLPageOn: aStream for: aRequest on: aSession.
	aStream nextPutAll: self ident , '</form>' , self eol! !

!WebForm methodsFor: 'model adapting' stamp: ' 21/4/07 22:07'!
registerFormElementsIn: aWebElement
	"find and register (add to fields ans set aspect adapters) all form elements in this and subelements"
	aWebElement isNil ifTrue: [^nil]. 
	aWebElement isFormElement ifTrue:
		[(aWebElement name isNil or: [aWebElement name isEmpty]) 
			ifTrue:
				[aWebElement name: 'field', (self fields size + 1 - 1"set") printString.
				self addToFields: aWebElement]
			ifFalse: 
				[(self alreadyRegistered: aWebElement) ifFalse: [self addToFields: aWebElement]].
		self adaptFormElement: aWebElement].
	aWebElement isComposite ifTrue: 
		[aWebElement elements do: [:each | self registerFormElementsIn: each] ]! !

!WebForm methodsFor: 'private-ids' stamp: ' 21/4/07 22:07'!
registerIdFor: anElement
	(anElement method notNil and: [self app session lastRequest isAjaxRequest
		and: [self isRegisteredMethod: anElement method]])
			ifTrue: [self registerIdFor: anElement instead: (self elementForMethod: anElement method)].
	anElement id isNil ifTrue: [^anElement id: self nextId]. "auto id if not manually defined!! "
	(self isRegistered: anElement) ifTrue: [^nil].
	(self ids includesKey: anElement id) ifTrue: [^nil"self error: 'element with that id already exist!!'"].
	self ids at: anElement id put: anElement.
	self registeredSet add: anElement.! !

!WebForm methodsFor: 'private-ids' stamp: ' 21/4/07 22:07'!
registerIdFor: anElement instead: anOldElement
	"new anElement should have same id as an old one"
	anOldElement isNil ifTrue: [^nil].
	self registeredSet remove: anOldElement ifAbsent: []; add: anElement.
	self ids removeKey: anOldElement id ifAbsent: [].
	anElement attributesAt: #id put: anOldElement id. "anElement id: would deadlock!! "
	self ids at: anElement id put: anElement.! !

!WebForm methodsFor: 'model adapting' stamp: ' 21/8/07 09:38'!
registerStreamedFieldsInto: aHTTPPost
	"for streams it needs to be done before you use any post data, otherwise it is too late!!"
	self streamedFieldSet do: [:field | 
		field writeBlock notNil 
			ifTrue: [aHTTPPost postDataAt: field name beforeStreamingDo: field writeBlock].
		field writeStream notNil 
			ifTrue: [aHTTPPost postDataAt: field name streamTo: field writeStream] ]! !

!WebForm methodsFor: 'private-ids' stamp: ' 21/4/07 22:07'!
registeredSet
	^self ids at: #setOfRegisteredElements! !

!WebForm methodsFor: 'private-ids' stamp: ' 21/4/07 22:07'!
removeId: aSymbol
	| element |
	element := self ids at: aSymbol ifAbsent: [^nil].
	self ids removeKey: aSymbol.
	self registeredSet remove: element ifAbsent: [].! !

!WebForm methodsFor: 'tabulation' stamp: ' 21/4/07 22:07'!
reorderTabulation
	"reorders tabOrder to match tabIndex order of form elements"
	| old withIndex |
	old := self tabOrder copy.
	self initTabOrder.
	withIndex := old select: [:each | each tabIndex notNil and: [each tabIndex ~= 0] ].
	withIndex := SortedCollection withAll: withIndex sortBlock: [:a :b | a tabIndex < b tabIndex].
	self tabOrder addAll: withIndex.
	withIndex := withIndex asSet.
	 "those without tab index at the end"
	self tabOrder addAll: (old reject: [:each | withIndex includes: each  ]).! !

!WebForm methodsFor: 'private' stamp: ' 21/4/07 22:07'!
shouldIdent
	^true! !

!WebForm methodsFor: 'private-fields' stamp: ' 21/8/07 09:38'!
streamedFieldSet
	^self fields at: #streamedFields ifAbsent: [self initStreamedFieldSet].! !

!WebForm methodsFor: 'private' stamp: ' 21/4/07 22:07'!
tabOrder
	"return a collection of all fields in this form, ordered by tab order"
	tabOrder isNil ifTrue: [self initTabOrder].
	^tabOrder! !

!WebForm methodsFor: 'model adapting' stamp: ' 21/4/07 22:07'!
uncheckAllChecboxes
	self fieldSet do: [:each | each isCheckBox ifTrue: [each uncheckValue] ]! !

!WebForm methodsFor: 'model adapting' stamp: ' 21/4/07 22:07'!
uncheckAllRadioButtons! !

!WebForm methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
view
	"for which app view is that form"
	^view! !

!WebForm methodsFor: 'private' stamp: ' 21/4/07 22:07'!
view: anObject
	view := anObject! !

WebElement subclass: #WebFormElement
	instanceVariableNames: 'aspect object adaptor enterTab'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

WebFormElement subclass: #WebButton
	instanceVariableNames: 'text type size'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebButton class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
text: aString

	^self type: #submit text: aString! !

!WebButton class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
text: aString action: aSymbol

	^self type: #submit text: aString action: aSymbol! !

!WebButton class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
type: aSymbol message: aMsgSymbol

	^self new type: aSymbol; message: aMsgSymbol! !

!WebButton class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
type: aSymbol text: aString

	^self new type: aSymbol; text: aString! !

!WebButton class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
type: aSymbol text: aString action: anActionSymbol

	^self new type: aSymbol; text: aString; action: anActionSymbol! !

!WebButton methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
action: aSymbol
	self name: aSymbol asString! !

!WebButton methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	self type: 'submit'.! !

!WebButton methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isButton
	^true! !

!WebButton methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isEmpty
	^true! !

!WebButton methodsFor: 'printing' stamp: 'janko 7/18/2008 21:32'!
printHTMLPageOn: aStream for: aRequest on: aSession
	self prepareToHTMLPrintOn: aSession.
	aStream nextPutAll: self ident , '<input'.
	self printAttributesOn: aStream for: aSession.
	aStream nextPutAll: '>'
	"
	super printHTMLPageOn: aStream for: aRequest on: aSession.
	aStream nextPutAll:  '</button>'.
"! !

!WebButton methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
shouldIdent
	^false! !

!WebButton methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
text
	^self attributesAt: #value! !

!WebButton methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
text: aString
	"set the text to be shown in a button, It is actually a value attribute in input tag"
	| txt |
	"slovene csz are converted if char ^ is after such a char"
	txt := (aString includes: $^ ) ifTrue: [aString convertToSloveneChars] ifFalse: [aString].
	self attributesAt: #value put: txt.
	self addText: txt! !

WebFormElement subclass: #WebCheckBox
	instanceVariableNames: 'checked selected'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebCheckBox class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
aspect: aSymbol for: anObject
	"this method will be called and true/false will be set/clear on an object"
	^self new aspect: aSymbol for: anObject! !

!WebCheckBox class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
object: anObject from: aCollection
	"if checked, this object will be put in collection. If object is initially in collection, 
	checkbox will be checked"
	"Usefull for easy selection among many objects"
	^self new object: anObject from: aCollection! !

!WebCheckBox methodsFor: 'private' stamp: ' 21/4/07 22:07'!
acceptFormInputFrom: aPostDataDictionary
	(aPostDataDictionary at: self name) = 'on' ifTrue: [self checkValue]! !

!WebCheckBox methodsFor: 'private' stamp: ' 21/4/07 22:07'!
addObjectToSelected
	(self selected includes: self object) ifFalse: [self selected add: self object]! !

!WebCheckBox methodsFor: 'private' stamp: ' 21/4/07 22:07'!
checkValue
	self aspect notNil ifTrue: [self setAspectTrue].
	self selected notNil ifTrue: [self addObjectToSelected]! !

!WebCheckBox methodsFor: 'private' stamp: ' 21/4/07 22:07'!
checked
	^checked! !

!WebCheckBox methodsFor: 'private' stamp: ' 21/4/07 22:07'!
checked: aBoolean
	checked := aBoolean! !

!WebCheckBox methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	super initialize.
	self type: 'checkbox'.
	self checked: false.! !

!WebCheckBox methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isCheckBox
	^true! !

!WebCheckBox methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isChecked
	| value |
	self aspect notNil ifTrue:
		[value := self object perform: self aspect.
		^value == true].  "so that false will be returned even if value is not so"
	self selected notNil ifTrue: [^self selected includes: self object]. 
	^self checked! !

!WebCheckBox methodsFor: 'private' stamp: ' 21/4/07 22:07'!
joinToForm: aWebForm
	aWebForm fields keysAndValuesDo: [:fname :element | 
		element == self ifTrue: [self name: fname]. ^self]! !

!WebCheckBox methodsFor: 'private' stamp: ' 21/4/07 22:07'!
object: anObject from: aCollection
	"if checked, this object will be put in collection. If object is initially in collection, 
	checkbox will be checked"
	"Usefull for easy selection among many objects"
	self object: anObject.
	self selected: aCollection.! !

!WebCheckBox methodsFor: 'printing' stamp: 'janko 7/18/2008 20:20'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	self prepareToHTMLPrintOn: aSession.
	aStream nextPutAll: self ident , '<input'.
	self 
		printAttributesOn: aStream
		for: aSession.
	self isChecked ifTrue: [ aStream nextPutAll: ' checked' ].
	aStream nextPutAll: '>'! !

!WebCheckBox methodsFor: 'private' stamp: ' 21/4/07 22:07'!
removeObjectFromSelected
	(self selected includes: self object) ifTrue: [self selected remove: self object]! !

!WebCheckBox methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
selected
	^selected! !

!WebCheckBox methodsFor: 'private' stamp: ' 21/4/07 22:07'!
selected: aCollection
	"checked objects"
	selected := aCollection! !

!WebCheckBox methodsFor: 'private' stamp: ' 21/4/07 22:07'!
setAspectFalse
	self object perform: (self aspect asString, ':') asSymbol with: false! !

!WebCheckBox methodsFor: 'private' stamp: ' 21/4/07 22:07'!
setAspectTrue
	self object perform: (self aspect asString, ':') asSymbol with: true! !

!WebCheckBox methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
setChecked
	self checked: true! !

!WebCheckBox methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
setUnchecked
	self checked: false! !

!WebCheckBox methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
shouldIdent
	^false! !

!WebCheckBox methodsFor: 'private' stamp: ' 21/4/07 22:07'!
uncheckValue
	self aspect notNil ifTrue: [self setAspectFalse].
	self selected notNil ifTrue: [self removeObjectFromSelected]! !

!WebCheckBox methodsFor: 'private' stamp: ' 21/4/07 22:07'!
value
	^self attributesAt: 'value'! !

!WebCheckBox methodsFor: 'private' stamp: ' 21/4/07 22:07'!
value: aString
	"set the value name  of the  check box"
	^self attributesAt: 'value' put: aString! !

!WebFormElement class methodsFor: 'auto type converting' stamp: ' 21/4/07 22:07'!
autoConvertString: aString toObject: anObject
	"try to convert string depending on object type"
	(anObject isKindOf: String) ifTrue: [^aString].
	(anObject isKindOf: Integer) ifTrue: [^aString asInteger].
	(anObject isKindOf: FixedPoint) ifTrue: [^aString asFixedPoint: anObject scale].
	(anObject isKindOf: Date) ifTrue: [^Date readSloFrom: aString readStream].
	anObject isNil ifTrue: [^aString].
	^aString! !

!WebFormElement class methodsFor: 'auto type converting' stamp: ' 21/4/07 22:07'!
autoConvertToString: anObject
	"try to convert object to string depending on object type"
	(anObject isKindOf: String) ifTrue: [^anObject].
	(anObject isKindOf: Integer) ifTrue: [^anObject printDotString].
	(anObject isKindOf: FixedPoint) ifTrue: [^anObject printDotString].
	(anObject isKindOf: Date) ifTrue: [^anObject shorterPrintSloString].
	^anObject printString! !

!WebFormElement class methodsFor: 'instance creation' stamp: 'JM 4/25/2007 22:23'!
new
	^super basicNew initialize! !

!WebFormElement methodsFor: 'events' stamp: ' 21/4/07 22:07'!
activate
	"move focus and select the text in that form field. (focus+select)"
	self registerId.
	self scriptAfter: 'Field.activate(''', self id, ''')'! !

!WebFormElement methodsFor: 'model adapting' stamp: ' 26/4/07 10:37'!
adapt
	"set appropriate aspect adaptor for that element"
	(self aspect isKindOf: Symbol) ifTrue:
		[self adaptor: ((AIDAAspectAdaptor forAspect: self aspect) subject: self object)].
	(self aspect  isKindOf: Number) ifTrue:  
		"adapt to an element of collection"
		[self adaptor: ((AIDAIndexedAdaptor forIndex: self aspect) subject: self object)]! !

!WebFormElement methodsFor: 'accessing' stamp: ' 26/4/07 10:37'!
adaptor
	^adaptor! !

!WebFormElement methodsFor: 'accessing' stamp: ' 26/4/07 10:37'!
adaptor: aProtocolAdaptor
	adaptor := aProtocolAdaptor.! !

!WebFormElement methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
addEnterTabScriptIfNessesary
	self enterTab ifFalse: [^nil].
	self onEnterTabulateTo: (self app form nextTabOrderFrom: self)! !

!WebFormElement methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
aspect
	^aspect! !

!WebFormElement methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
aspect: aSymbol
	aspect := aSymbol.! !

!WebFormElement methodsFor: 'model adapting' stamp: ' 21/4/07 22:07'!
aspect: aSymbol for: anObject
	"set the link between this form element and an aspect (name of instance variable) 
	of an object , which value is presented in this webFormElement. The appropriate 
	aspect adaptor is generated after page composition in aWebApplication printWebView 
	method to check also elements which were added to subelements later!!. Name of a form 
	element is set to an aspect name, if not already set differently"
	self aspect: aSymbol.
	self object: anObject! !

!WebFormElement methodsFor: 'private' stamp: ' 21/4/07 22:07'!
enterTab
	"is tabulation witn Enter key enabled?"
	enterTab isNil ifTrue: [^false].
	^enterTab! !

!WebFormElement methodsFor: 'private' stamp: ' 21/4/07 22:07'!
enterTab: aBoolean
	enterTab := aBoolean! !

!WebFormElement methodsFor: 'events' stamp: ' 21/4/07 22:07'!
focus
	"move the input focus to that form field"
	self registerId.
	self scriptAfter: 'Field.focus(''', self id, ''')'! !

!WebFormElement methodsFor: 'events' stamp: ' 21/4/07 22:07'!
ignoreEnterKey
	"don't allow ENTER/RETURN key to submit a form unintentionally, for instance"
	self scriptAfter: 'Event.observe(''', self id, ''', ''keypress'', function(event) {
  		if (event.keyCode == Event.KEY_RETURN) { Event.stop(event); } } )'! !

!WebFormElement methodsFor: 'initialize-release' stamp: ' 26/4/07 10:37'!
initialize
	self name: ''.
"	self app form registerFormElementsIn: self "! !

!WebFormElement methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isAutocompleteField
	^false! !

!WebFormElement methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isButton
	^false! !

!WebFormElement methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isCheckBox
	^false! !

!WebFormElement methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isDateInputField
	^false! !

!WebFormElement methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isEmpty
	"input field is empty or nil"
	| vlue |
	vlue := self object perform: self aspect.
	^vlue isNil 
		or: [((vlue isKindOf: String) and: [vlue isEmpty])
			or: [(vlue isKindOf: Number) and: [vlue = 0] ]]! !

!WebFormElement methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isFileInputField
	^false! !

!WebFormElement methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isFormElement
	^true! !

!WebFormElement methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isInputField
	^false! !

!WebFormElement methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isListBox
	^false! !

!WebFormElement methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isMenu
	^false! !

!WebFormElement methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isRadioButton
	^false! !

!WebFormElement methodsFor: 'testing' stamp: ' 21/8/07 09:38'!
isStreamed
	"Post data from web request is streamed directly into output stream. 
	Only WebFileInputField can do that for now"
	^false! !

!WebFormElement methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isTextArea
	^false! !

!WebFormElement methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
name
	^self attributesAt: #name! !

!WebFormElement methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
name: aString
	self attributesAt: #name put: aString! !

!WebFormElement methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
noTab
	"skip this form element when tabulating with TAB"
	"This attribute specifies the position of the current element in the tabbing order of that form"
	self tabIndex: 0.! !

!WebFormElement methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
object
	^object! !

!WebFormElement methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
object: anObject

	"set the object, which instance variable is presented as aspect in this fom element"

	object := anObject.! !

!WebFormElement methodsFor: 'events' stamp: ' 21/4/07 22:07'!
onChange: aJavascriptCode
	self attributesAt: #onChange add: aJavascriptCode! !

!WebFormElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onChangePost
	"after field change immediatelly post contents to server (AJAX)"
	self onChangePostAndUpdate: nil! !

!WebFormElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onChangePostAndUpdate: anElementOrId
	"after field change, post contents to server and AJAX update anElementOrId  "
	self onChange: (self scriptForPostAndUpdate: anElementOrId with: nil)! !

!WebFormElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onChangePostAndUpdate: anElementOrId with: aParmString
	"after field change, post contents to server and AJAX update anElementOrId  with a parameter"
	self onChange: (self scriptForPostAndUpdate: anElementOrId with: aParmString)! !

!WebFormElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onChangePostAndUpdateSelf
	"DONT WORK YET!!"
	"after field change, post contents to server and AJAX update that form element  "
	self registerId.
	self onChangePostAndUpdate: self! !

!WebFormElement methodsFor: 'events' stamp: ' 21/4/07 22:07'!
onEnterTabulate
	"It goes to the next element in tabulation order"
	"This allows ENTER key to be used for tabulation, like a TAB. "
	self enterTab: true. "to make tabulation script late, just before htmlPrint"! !

!WebFormElement methodsFor: 'events' stamp: ' 21/4/07 22:07'!
onEnterTabulateTo: aFormElementOrId
	"allow ENTER key to be used for tabulation, like a TAB"
	| nextId |
	nextId := (aFormElementOrId isKindOf: WebElement) 
		ifTrue: [aFormElementOrId id] ifFalse: [aFormElementOrId asString].
	nextId isNil ifTrue: [nextId := ''].
	self scriptAfter: 'Event.observe(''', self id, ''', ''keypress'', function(event) {
  		if (event.keyCode == Event.KEY_RETURN) {  
			Event.stop(event); $(''', nextId, ''').activate(); } } )'! !

!WebFormElement methodsFor: 'events' stamp: ' 21/4/07 22:07'!
onFocus: aJavascriptCode
	self attributesAt: #onFocus add: aJavascriptCode! !

!WebFormElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onKeyPressPostAndUpdate: anElementOrId
	"after any key press in a field, post contents to server (AJAX)"
	self onKeyPress: (self scriptForPostAndUpdate: anElementOrId  with: nil)! !

!WebFormElement methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
onKeyUpPostAndUpdate: anElementOrId
	"after any key  release in a field, post contents to server (AJAX)"
	self onKeyUp: (self scriptForPostAndUpdate: anElementOrId with: nil)! !

!WebFormElement methodsFor: 'events' stamp: ' 21/4/07 22:07'!
onSelect: aJavascriptCode
	self attributesAt: #onSelect add: aJavascriptCode! !

!WebFormElement methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
prepareToHTMLPrintOn: aSession
	super prepareToHTMLPrintOn: aSession.
	self addEnterTabScriptIfNessesary.! !

!WebFormElement methodsFor: 'private-ajax' stamp: ' 21/4/07 22:07'!
scriptForPostAndUpdate: anElementOrId with: aParmString
	| url idSymbol parms |
	idSymbol := anElementOrId isSymbol 
		ifTrue: [anElementOrId] 
		ifFalse: [anElementOrId isNil 
			ifTrue: [#nil] ifFalse: [anElementOrId registerId. anElementOrId id] ].
	self registerId.
	url := self ajaxCallUrl. 
	parms := self ajaxCallUrlParametersFor: anElementOrId.
	aParmString notNil ifTrue: [parms := parms, '&parm=', aParmString].
	^'var field = Form.Element.serialize(''', self id asString, '''); new Ajax.Updater(''', idSymbol asString, ''', ''', "url, ''', {method: ''get'', parameters: ''', parms, ''' + field, evalScripts: true})'"
url, ''', {method: ''post'', postBody: field + ''', ('&', parms), ''', evalScripts: true})'! !

!WebFormElement methodsFor: 'events' stamp: ' 21/4/07 22:07'!
select
	"select a whole input. Be sure to focut it first!!"
	self registerId.
	self scriptAfter: 'Field.select(''', self id, ''')'! !

!WebFormElement methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
tabIndex
	"This attribute specifies the position of the current element in the tabbing order of that form"
	^(self attributes at: #tabindex ifAbsent: [^nil]) asInteger! !

!WebFormElement methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
tabIndex: aNumber
	"This attribute specifies the position of the current element in the tabbing order 
	for the current document"
	self attributes at: #tabindex put: aNumber printString! !

!WebFormElement methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
type: aString
	"set the type of  this form element"
	self attributesAt: #type put: aString! !

!WebFormElement methodsFor: 'accessing' stamp: ' 26/4/07 10:37'!
value
	^self adaptor value! !

!WebFormElement methodsFor: 'accessing' stamp: ' 26/4/07 10:37'!
value: aValue
	"set the value of this form element"
	self adaptor value:  aValue.! !

WebFormElement subclass: #WebInPlaceEditableText
	instanceVariableNames: 'size attribute formated allow triggerElement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Components'!

!WebInPlaceEditableText class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
aspect: aSymbol for: anObject
	^self new aspect: aSymbol for: anObject! !

!WebInPlaceEditableText class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
aspect: aSymbol for: anObject size: aNumberOrPoint
	^self new aspect: aSymbol for: anObject; size: aNumberOrPoint! !

!WebInPlaceEditableText class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
aspect: aSymbol for: anObject size: aNumberOrPoint allow: aBoolean
	^self new aspect: aSymbol for: anObject; 
		size: aNumberOrPoint;
		allow: aBoolean	"is edit allowed at all?"! !

!WebInPlaceEditableText methodsFor: 'private' stamp: ' 21/4/07 22:07'!
addSetupScript
	| url urlFormated |
	url := self ajaxCallUrl, '?', (self ajaxCallUrlParametersFor: self id), '&ajaxInPlaceEditing'.
	urlFormated := self isFormated ifTrue: [url, '&wikiFormated'] ifFalse: [url].
	self scriptAfter: 'new Ajax.InPlaceEditor(''', self id asString, 
		''', ''', urlFormated, ''', {', self rowsCols, self textOptions, self externalControl, 
		(self isFormated ifTrue: [',loadTextURL:''', url, ''''] ifFalse: ['']), '})'! !

!WebInPlaceEditableText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allow
	"is edit allowed at all?"
	allow isNil ifTrue: [^true].
	^allow! !

!WebInPlaceEditableText methodsFor: 'private' stamp: ' 21/4/07 22:07'!
allow: aBoolean
	allow := aBoolean! !

!WebInPlaceEditableText methodsFor: 'private' stamp: ' 21/4/07 22:07'!
attribute
	"<b> etc"
	attribute isNil ifTrue: [^''].
	^attribute! !

!WebInPlaceEditableText methodsFor: 'private' stamp: ' 21/4/07 22:07'!
attribute: aString
	attribute := aString! !

!WebInPlaceEditableText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
bold
	self attribute: '<b>'! !

!WebInPlaceEditableText methodsFor: 'settings' stamp: 'mivsek 1/10/2008 22:37'!
cancelText
	| app |
	app := self app.
	^app notNil 
		ifTrue: [self app style inPlaceCancelText]
		ifFalse: ['Cancel']! !

!WebInPlaceEditableText methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
columns
	"length of edit field or textarea"
	self size isNil ifTrue: [^20].
	self size class == Point ifTrue: [^self size x].
	^self size! !

!WebInPlaceEditableText methodsFor: 'private' stamp: ' 21/4/07 22:07'!
endAttributeTag
	self attribute isEmpty ifTrue: [^''].
	^'</', self attribute copyFrom: 3 to: 5! !

!WebInPlaceEditableText methodsFor: 'private' stamp: ' 21/4/07 22:07'!
externalControl
	| idSymbol |
	self triggerElement isNil ifTrue: [^''].
	idSymbol := self triggerElement isSymbol 
		ifTrue: [self triggerElement] ifFalse: [self triggerElement registerId. self triggerElement id].
	^', externalControl:''', idSymbol asString, ''''! !

!WebInPlaceEditableText methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	super initialize.
	self app style ensureJavascriptForScriptaculousInHeader.
	self app style ensureJsResourceForScriptaculous.! !

!WebInPlaceEditableText methodsFor: 'private' stamp: ' 21/4/07 22:07'!
isFormated
	^formated notNil and: [formated]! !

!WebInPlaceEditableText methodsFor: 'settings' stamp: 'mivsek 1/10/2008 22:37'!
okText
	| app |
	app := self app.
	^app notNil 
		ifTrue: [self app style inPlaceOkText]
		ifFalse: ['Ok']! !

!WebInPlaceEditableText methodsFor: 'printing' stamp: 'janko 7/18/2008 20:26'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	self registerId.
	self prepareToHTMLPrintOn: aSession.
	self allow ifTrue: [ self addSetupScript ].
	self scriptBefore notNil ifTrue: 
		[ self scriptBefore 
			printHTMLPageOn: aStream for: aRequest on: aSession ].
	aStream nextPutAll: self startAttributeTag.
	aStream nextPutAll: '<span '.
	self 
		printAttributesOn: aStream
		for: aSession.
	aStream nextPut: $>.
	aStream nextPutAll: (self properlyFormatedValueOn: aSession).
	aStream nextPutAll: '</span>' , self endAttributeTag.
	(self allow and: [ self triggerElement isKindOf: WebElement ]) ifTrue: 
		[ self triggerElement 
			printHTMLPageOn: aStream for: aRequest on: aSession ].
	self scriptAfter notNil ifTrue: 
		[ self scriptAfter 
			printHTMLPageOn: aStream for: aRequest on: aSession ]! !

!WebInPlaceEditableText methodsFor: 'private' stamp: ' 26/4/07 10:37'!
properlyFormatedValueOn: aSession
	| value |
	value := self value.
	self isFormated ifTrue: [value := value asWikiHtml].
	^AIDASite convertToWeb:  (WebFormElement autoConvertToString: value) on: aSession! !

!WebInPlaceEditableText methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
rows
	"number of lines in edit area"
	self size isNil ifTrue: [^1].
	self size class == Point ifTrue: [^self size y].
	^1! !

!WebInPlaceEditableText methodsFor: 'private' stamp: ' 21/4/07 22:07'!
rowsCols
	^'rows:', self rows printString, ',cols:', self columns printString! !

!WebInPlaceEditableText methodsFor: 'settings' stamp: 'mivsek 1/10/2008 22:37'!
savingText
	^'...'! !

!WebInPlaceEditableText methodsFor: 'private' stamp: ' 21/4/07 22:07'!
size
	"if number, then editor is one line, if point, it is text area!!"
	^size! !

!WebInPlaceEditableText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
size: aNumberOrPoint
	"if number, then editor is one line, if point, it is text area!!"
	size := aNumberOrPoint! !

!WebInPlaceEditableText methodsFor: 'private' stamp: ' 21/4/07 22:07'!
startAttributeTag
	^self attribute! !

!WebInPlaceEditableText methodsFor: 'private' stamp: ' 21/4/07 22:07'!
textOptions
	| text |
	text := ', okText:''', self okText, ''', cancelText:''', self cancelText, ''', savingText:''', self savingText, ''''.
	^AIDASite convertToWeb: text on: self app session! !

!WebInPlaceEditableText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
triggerElement
	"element which triggers edit mode. If it is real element and not only id, it will be 
       added after editable text, "
	^triggerElement! !

!WebInPlaceEditableText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
triggerElement: aWebElementOrId
	triggerElement := aWebElementOrId! !

!WebInPlaceEditableText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
wikiFormated
	"source text is formated in wiki format"
	formated := true! !

WebFormElement subclass: #WebInputField
	instanceVariableNames: 'size maxLength type'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

WebInputField subclass: #WebAutocompleteField
	instanceVariableNames: 'choicesAspect choicesObject choiceAspect elementToUpdate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Components'!

!WebAutocompleteField class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
aspect: aSymbol for: anObject choicesAspect: aChoicesSymbol for: aChoicesObject
	"choicesAspect method is called with our field as argument on aChoicesObject. 
	It must return a collection of strings"
	"example: myObject searchFor: aString (choicesAspect is #searchFor: )"
	^self new 
		aspect: aSymbol for: anObject;
		choicesAspect: aChoicesSymbol for: aChoicesObject! !

!WebAutocompleteField class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
aspect: aSymbol for: anObject choicesAspect: aChoicesSymbol for: aChoicesObject 
	afterPostAndUpdate: aWebElementOrId
	"immediately post and update element, after user has selected one of choices"
	^(self aspect: aSymbol for: anObject choicesAspect: aChoicesSymbol for: aChoicesObject)
		elementToUpdate: aWebElementOrId! !

!WebAutocompleteField class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
aspect: aSymbol for: anObject choicesAspect: aChoicesSymbol for: aChoicesObject choiceAspect: aTextSymbol
	"choicesAspect method is called with our field as argument on aChoicesObject. 
	It returs a collection of objects, an choice's text 
	is retrieved with a call of choiceAspect method on each"
	^self new 
		aspect: aSymbol for: anObject;
		choicesAspect: aChoicesSymbol for: aChoicesObject;
		choiceAspect: aTextSymbol! !

!WebAutocompleteField class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
aspect: aSymbol for: anObject choicesAspect: aChoicesSymbol for: aChoicesObject choiceAspect: aTextSymbol afterPostAndUpdate: aWebElementOrId
	"choicesAspect method is called with our field as argument on aChoicesObject. 
	It returs a collection of objects, an choice's text 
	is retrieved with a call of choiceAspect method on each"
	^self new 
		aspect: aSymbol for: anObject;
		choicesAspect: aChoicesSymbol for: aChoicesObject;
		choiceAspect: aTextSymbol;
		elementToUpdate: aWebElementOrId! !

!WebAutocompleteField methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
addSetupScriptOn: aStream choicesElement: aChoicesElement
	| url afterUpdateElement idSymbol updateUrl parms |
	url := self ajaxCallUrlExtended, '&ajaxAutocompleteField'.
	afterUpdateElement := self elementToUpdate isNil ifTrue: [''] ifFalse:
		[idSymbol := self elementToUpdate isSymbol 
			ifTrue: [self elementToUpdate] ifFalse: [self elementToUpdate registerId. self elementToUpdate id].
		updateUrl := self ajaxCallUrl. parms := self ajaxCallUrlParametersFor: self elementToUpdate.
		'afterUpdateElement: function(element, selectedElement){var field = Form.Element.serialize(''', 
			self id asString, '''); new Ajax.Updater(''', idSymbol asString, ''', ''', updateUrl, 
			''', {method: ''post'', postBody: field + ''', ('&', parms), ''', evalScripts: true})} '].

	aStream nextPutAll: '<script type="text/javascript">new Ajax.Autocompleter(''', 
		self id asString, ''', ''', aChoicesElement id asString, ''', ''', url, 
		''', {', afterUpdateElement, '});</script>', self eol.! !

!WebAutocompleteField methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
choiceAspect
	"this method is called on each choice to display choice's text"
	^choiceAspect! !

!WebAutocompleteField methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
choiceAspect: aSymbol
	choiceAspect := aSymbol! !

!WebAutocompleteField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
choicesAspect
	^choicesAspect! !

!WebAutocompleteField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
choicesAspect: aSymbol
	choicesAspect := aSymbol! !

!WebAutocompleteField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
choicesAspect: aSymbol for: anObject
	"this method is called with our field as argument on that object. It must return a collection of strings"
	"example: myObject searchFor: aString (choicesAspect is #searchFor: )"
	self choicesAspect: aSymbol.
	self choicesObject: anObject! !

!WebAutocompleteField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
choicesObject
	^choicesObject! !

!WebAutocompleteField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
choicesObject: anObject
	choicesObject := anObject! !

!WebAutocompleteField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
elementToUpdate
	^elementToUpdate! !

!WebAutocompleteField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
elementToUpdate: aWebElementOrId
	(aWebElementOrId isKindOf: WebElement) ifTrue: [aWebElementOrId registerId].
	elementToUpdate := aWebElementOrId! !

!WebAutocompleteField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
getChoicesForEntry: aString
	"return list of choices, selected already entered characters in  aString. "
	"always return text strings, not choice objects!!"
	| choices matchString |
	self choiceAspect isNil "one possiblity, direct question to choicesObject"
		ifTrue: [^self choicesObject perform: self choicesAspect with: aString].
	"other possibility, we select internaly from all choices"
	choices := self choicesObject perform: self choicesAspect.
	matchString := aString, '*'.
	choices := choices select: [:each | matchString match: (each perform: self choiceAspect)].
	^choices collect: [:each | each perform: self choiceAspect].! !

!WebAutocompleteField methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	super initialize.
	self app style ensureJavascriptForScriptaculousInHeader.
	self app style ensureJsResourceForScriptaculous.! !

!WebAutocompleteField methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isAutocompleteField
	^true! !

!WebAutocompleteField methodsFor: 'printing' stamp: ' 26/4/07 10:37'!
prepareAttributesToPrintOn: aSession
	"if choiceAspect is set, use it!! "
	self choiceAspect isNil ifTrue: [^super prepareAttributesToPrintOn: aSession].
	self value isNil ifTrue: [^nil].
	(self value class canUnderstand: self choiceAspect) ifFalse: [^nil].
	self attributesAt: #value put: 
		(WebFormElement autoConvertToString: (self value perform: self choiceAspect) )! !

!WebAutocompleteField methodsFor: 'printing' stamp: 'janko 7/18/2008 20:27'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	| choicesElement |
	choicesElement := (WebElement newClass: #'auto_complete') parent: self.
	self registerId.
	choicesElement registerId.
	super 
		printHTMLPageOn: aStream for: aRequest on: aSession.
	choicesElement 
		printHTMLPageOn: aStream for: aRequest on: aSession.
	self 
		addSetupScriptOn: aStream
		choicesElement: choicesElement! !

!WebAutocompleteField methodsFor: 'private' stamp: ' 26/4/07 10:37'!
saveThroughAdapterValue: aValueString
	| vlue choices |
	vlue :=  AIDASite convertFromWeb: aValueString on: self session. "ensure unicode!!"
	vlue := (WebFormElement autoConvertString:  vlue toObject: self value).
	self choiceAspect notNil ifTrue: "save a real object, not only text!! "
		[choices := self choicesObject perform: self choicesAspect.
		vlue := choices detect: [:each | (each perform: self choiceAspect) = vlue] ifNone: [nil] ].
	self value: vlue.! !

WebInputField subclass: #WebDateInputField
	instanceVariableNames: 'button'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Components'!

!WebDateInputField commentStamp: '<historical>' prior: 0!
WebDateInputField is used for entry of dates with help od JavaScript calendar (http://www.dynarch.com/projects/calendar/), fro Romanian author Mihai Bazon. 
See also WebStyle calendar* methods, specially calendarCSS and calendarLang*

Usage is similar as usual input field. Example: 
	element addDateInputFieldAspect: #methodName for: self observee
 
Instance Variables:
	button	<WebImage>	
!

!WebDateInputField methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
addCalendarSetupScriptOn: aStream
	aStream nextPutAll: '<script type="text/javascript">Calendar.setup({ inputField : "', self inputFieldId, 		'", ifFormat : "', self inputFieldFormat, 
		'", button : "', self buttonId, '"});</script>', self eol! !

!WebDateInputField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
button
	^button! !

!WebDateInputField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
button: aWebButton
	button := aWebButton! !

!WebDateInputField methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
buttonId
	^self button id! !

!WebDateInputField methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
buttonImage
	"from WebStyle calendarButtonGif !! "
	| methodImage |
	methodImage := WebMethodImage
		fromMethod: #calendarButtonGif
		on: self style 
		contentType: 'image/gif' 
		site: self style site.
	^(WebImage image: methodImage)
		title: 'Izberite datum';
		imageAlign: #middle;
		onMouseOver: 'this.style.cursor=''pointer'' ';
		onMouseOut: 'this.style.cursor=''default'' '! !

!WebDateInputField methodsFor: 'initialize-release' stamp: ' 21/8/07 09:38'!
initialize
	super initialize.
	self size: 11. 
	self app style ensureJavascriptAndCssForCalendarInHeader.
	self app style ensureJsResourceForCalendarSetup.! !

!WebDateInputField methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
inputFieldFormat
	"the date format"
	^'%e.%m.%Y'  "17.01.1965"! !

!WebDateInputField methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
inputFieldId
	^self id! !

!WebDateInputField methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isDateInputField
	^true! !

!WebDateInputField methodsFor: 'printing' stamp: 'janko 7/18/2008 20:28'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	self registerId.
	self button: self buttonImage.
	self button registerId.
	super 
		printHTMLPageOn: aStream for: aRequest on: aSession.
	self button 
		printHTMLPageOn: aStream for: aRequest on: aSession.
	self addCalendarSetupScriptOn: aStream! !

WebInputField subclass: #WebDelayedField
	instanceVariableNames: 'update parameter delay'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Components'!

!WebDelayedField commentStamp: '<historical>' prior: 0!
WebDelayedField is used to delayed Ajax posting to server. It waits for delay after last key input, before it posts its contents. Ideal for live-search fields.

Use it as a normal input field, sending it #onClickPost, #onClickPostAndUdate: etc.
!

!WebDelayedField methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
addSetupScript
	self scriptAfter: (self scriptForPostAndUpdate: self update with: self parameter)! !

!WebDelayedField methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
afterPostUpdate: anElementOrId
	"update that element after delayed post"
	self update: anElementOrId! !

!WebDelayedField methodsFor: 'events-ajax' stamp: ' 21/4/07 22:07'!
afterPostUpdate: anElementOrId with: aParmString
	"update that element after delayed post with a parameter"
	self update: anElementOrId.
	self parameter: aParmString.! !

!WebDelayedField methodsFor: 'accessing' stamp: 'mivsek 6/13/2007 16:16'!
defaultDelay
	"delay before posting, in seconds"
	^1! !

!WebDelayedField methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
delay
	"delay before posting, in seconds"
	delay isNil ifTrue: [self delay: self defaultDelay].
	^delay! !

!WebDelayedField methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
delay: aNumber
	"delay before posting, in seconds"
	delay := aNumber! !

!WebDelayedField methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isDelayedField
	^true! !

!WebDelayedField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
onChangePost
	"that's done by default!! "! !

!WebDelayedField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
onChangePostAndUpdate: anElementOrId
	self error: 'use afterPostUpdate: instead!!'! !

!WebDelayedField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
onChangePostAndUpdate: anElementOrId with: aParmString
	self error: 'use afterPostUpdate:with: instead!!'! !

!WebDelayedField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
onChangePostAndUpdateSelf
	self error: 'use afterPostUpdate: instead!!'! !

!WebDelayedField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
onKeyPressPostAndUpdate: anElementOrId
	self error: 'use afterPostUpdate: instead!!'! !

!WebDelayedField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
onKeyUpPostAndUpdate: anElementOrId
	self error: 'use afterPostUpdate: instead!!'! !

!WebDelayedField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
parameter
	^parameter! !

!WebDelayedField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
parameter: anObject
	parameter := anObject! !

!WebDelayedField methodsFor: 'printing' stamp: 'janko 7/18/2008 20:28'!
printHTMLPageOn: aStream for: aRequest on: aSession
	self registerId.
	self addSetupScript.
	super 
		printHTMLPageOn: aStream for: aRequest on: aSession! !

!WebDelayedField methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
scriptForPostAndUpdate: anElementOrId with: aParmString
	| url idSymbol parms |
	idSymbol := anElementOrId isSymbol 
		ifTrue: [anElementOrId] 
		ifFalse: [anElementOrId isNil 	ifTrue: [#nil] ifFalse: [anElementOrId registerId. anElementOrId id] ].
	self registerId. 
      url := self ajaxCallUrl. parms := self ajaxCallUrlParametersFor: anElementOrId.
	aParmString notNil ifTrue: [parms := parms, '&parm=', aParmString].
	^'
new Form.Element.AIDADelayedObserver ("', self id asString, '", ', self delay printString, ', function () 
	{new Ajax.Updater(''', idSymbol asString, ''', ''', url, ''', {method: ''post'', 
		postBody: ''', self name, '=''+$F (''', self id asString, ''') + ''', ('&', parms), ''', evalScripts: true}); 
      });
'! !

!WebDelayedField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
update
	^update! !

!WebDelayedField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
update: anElementOrId
	update := anElementOrId! !

WebInputField subclass: #WebFileInputField
	instanceVariableNames: 'filenameAspect contentTypeAspect writeStream writeBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebFileInputField class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
aspect: aSymbol filenameAspect: aSymbol2 contentTypeAspect: aSymbol3 forObject: anObject

	^self new aspect: aSymbol for: anObject; 
		filenameAspect: aSymbol2;
		contentTypeAspect: aSymbol3! !

!WebFileInputField class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
aspect: aSymbol filenameAspect: aSymbol2 forObject: anObject

	^self new aspect: aSymbol for: anObject; 
		filenameAspect: aSymbol2! !

!WebFileInputField class methodsFor: 'instance creation' stamp: ' 21/8/07 09:38'!
beforeStreamingDo: aBlock filenameAspect: aSymbol2 contentTypeAspect: aSymbol3 forObject: anObject
	"A block will be called just before streaming begins, with a HTTPPostDatum as parameter. Block
	  must set a write stream in datum (for instance open an output file and stream on it)"
	"example block: [:datum | datum writeStream: 'myfile' writeStream binary] "
	^self new 
		writeBlock: aBlock; 
		object: anObject; 
		filenameAspect: aSymbol2;
		contentTypeAspect: aSymbol3! !

!WebFileInputField class methodsFor: 'instance creation' stamp: ' 21/8/07 09:38'!
beforeStreamingDo: aBlock filenameAspect: aSymbol2 forObject: anObject
	"A block will be called just before streaming begins, with a HTTPPostDatum as parameter. Block
	 must set a write stream in datum (for instance open an output file and stream on it)"
	"example block: [:datum | datum writeStream: 'myfile' writeStream binary] "
	"Don't forget to close such a stream after!! This can be done in App action method"
	^self new 
		writeBlock: aBlock; 
		object: anObject; 
		filenameAspect: aSymbol2! !

!WebFileInputField class methodsFor: 'instance creation' stamp: ' 21/8/07 09:38'!
toStream: aStream filenameAspect: aSymbol2 contentTypeAspect: aSymbol3 forObject: anObject
	"wrrite uploaded file directly to aStream, which must be binary"
	^self new 
		writeStream: aStream; 
		object: anObject; 
		filenameAspect: aSymbol2;
		contentTypeAspect: aSymbol3! !

!WebFileInputField class methodsFor: 'instance creation' stamp: ' 21/8/07 09:38'!
toStream: aStream filenameAspect: aSymbol2 forObject: anObject
	"wrrite uploaded file directly to aStream, which must be binary"
	^self new 
		writeStream: aStream; 
		object: anObject; 
		filenameAspect: aSymbol2! !

!WebFileInputField methodsFor: 'aspect adapting' stamp: 'mu 5/5/2008 10:48'!
acceptFileAttributesFrom: aRequest field: aString
	| mutator |
	self filenameAspect notNil ifTrue:
		[| filename |
		mutator := (self filenameAspect asString, ':') asSymbol.
		filename := AIDASite convert: (aRequest postDataAt: aString) filenameWithoutPath fromCodepage: self session codePage.
		self object perform: mutator with: filename].
	self contentTypeAspect notNil ifTrue:
		[mutator := (self contentTypeAspect asString, ':') asSymbol.
		self object perform: mutator with: (aRequest postDataAt: aString) contentType]! !

!WebFileInputField methodsFor: 'aspect adapting' stamp: ' 21/4/07 22:07'!
aspect: aSymbol filenameAspect: aSymbol2 contentTypeAspect: aSymbol3 forObject: anObject
	^self aspect: aSymbol for: anObject; 
		filenameAspect: aSymbol2; 
		contentTypeAspect: aSymbol3! !

!WebFileInputField methodsFor: 'aspect adapting' stamp: ' 21/4/07 22:07'!
aspect: aSymbol filenameAspect: aSymbol2 forObject: anObject
	^self aspect: aSymbol for: anObject; filenameAspect: aSymbol2! !

!WebFileInputField methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
contentTypeAspect
	^contentTypeAspect! !

!WebFileInputField methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
contentTypeAspect: aSymbol
	contentTypeAspect := aSymbol! !

!WebFileInputField methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
filenameAspect
	^filenameAspect! !

!WebFileInputField methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
filenameAspect: aSymbol
	filenameAspect := aSymbol! !

!WebFileInputField methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	super initialize.
	self type: 'file'.! !

!WebFileInputField methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isFileInputField
	^true! !

!WebFileInputField methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isInputField
	^false! !

!WebFileInputField methodsFor: 'testing' stamp: ' 21/8/07 09:38'!
isStreamed
	"file is received directly from request into output stream"
	^self writeStream notNil or: [self writeBlock notNil]! !

!WebFileInputField methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
prepareAttributesToPrintOn: aSession
	"just override value preparation to nothing"! !

!WebFileInputField methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
writeBlock
	^writeBlock! !

!WebFileInputField methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
writeBlock: aBlockClosure
	"this block will be called just before start of streaming to writeStream"
	writeBlock := aBlockClosure! !

!WebFileInputField methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
writeStream
	^writeStream! !

!WebFileInputField methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
writeStream: aStream
	"output binary stream where uploaded file will be directy writen"
	writeStream := aStream! !

!WebInputField class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
adaptIndex: aNumber ofCollection: aCollection

	^self new aspect: aNumber for: aCollection! !

!WebInputField class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
aspect: aSymbol for: anObject
	^self new aspect: aSymbol for: anObject! !

!WebInputField class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
aspect: aSymbol for: anObject size: aNumber
	^self new aspect: aSymbol for: anObject size: aNumber! !

!WebInputField class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
size: aNumber aspect: aSymbol forObject: anObject
	"OBSOLETE!!"
	^self new size: aNumber; aspect: aSymbol for: anObject! !

!WebInputField methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
aspect: aSymbol for: anObject size: aNumber
	^self aspect: aSymbol for: anObject; size: aNumber! !

!WebInputField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
hidden
	^self type = 'hidden'! !

!WebInputField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
hidden: aBoolean
	"make this input field hidden one"
	aBoolean ifTrue: [self type: 'hidden']! !

!WebInputField methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	super initialize.
	self type: 'text'.! !

!WebInputField methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isHidden
	^self type = 'hidden'! !

!WebInputField methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isInputField
	^true! !

!WebInputField methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isPassword
	^self type = 'password'! !

!WebInputField methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
maxLength: aNumber
	"maximum number of characters allowed in this input field"
	self attributesAt: #maxlength put: aNumber printString! !

!WebInputField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
password
	^self type = 'password'! !

!WebInputField methodsFor: 'private' stamp: ' 21/4/07 22:07'!
password: aBoolean
	"if true, then value will not be shown in input field"
	self type: 'password'.! !

!WebInputField methodsFor: 'printing' stamp: ' 26/4/07 10:37'!
prepareAttributesToPrintOn: aSession
	self value notNil ifTrue:
		[self attributesAt: #value put: 
			(WebFormElement autoConvertToString: self value)]! !

!WebInputField methodsFor: 'printing' stamp: 'janko 7/18/2008 20:27'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	self prepareToHTMLPrintOn: aSession.
	self scriptBefore notNil ifTrue: 
		[ self scriptBefore 
			printHTMLPageOn: aStream for: aRequest on: aSession ].
	aStream nextPutAll: self ident , '<input'.
	self 
		printAttributesOn: aStream
		for: aSession.
	aStream nextPutAll: '>'.
	self scriptAfter notNil ifTrue: 
		[ self scriptAfter 
			printHTMLPageOn: aStream for: aRequest on: aSession ]! !

!WebInputField methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
resetHidden
	self hidden: false! !

!WebInputField methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
resetPassword
	self password: false! !

!WebInputField methodsFor: 'private' stamp: ' 26/4/07 10:37'!
saveThroughAdapterValue: aValueString
	| vlue |
	vlue :=  AIDASite convertFromWeb: aValueString on: self session. "ensure unicode!!"
	vlue := (WebFormElement autoConvertString:  vlue toObject: self value).
	self value: vlue.! !

!WebInputField methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
setHidden
	self hidden: true! !

!WebInputField methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
setPassword
	self password: true! !

!WebInputField methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
shouldIdent
	^false! !

!WebInputField methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
size: aNumber
	"number of characters displayed in this input field"
	self attributesAt: #size put: aNumber printString! !

WebFormElement subclass: #WebMenu
	instanceVariableNames: 'multiple collection selected aspectToStore objectToStore sort'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebMenu commentStamp: '<historical>' prior: 0!
WebMenu is a dropdown menu, which can select single or multiple options. Options are read from a collection, selected options are put in selected. You can set aspect of each option if options are not plain text. 

Example:

	WebMenu aspect: #name collection: self persons selected: self selectedPersons

will show dropdown menu of all persons, shown by name. Selected person will be stored in selectedPersons collection. Above menu is  single selection, it can be multiple selection too if you do:

	aWebMenu setMultiple.!

!WebMenu class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
aspect: aSymbol collection: aCollection selected: aSelectedCollection
	"Multiple selection menu. Aspect of every element in aColection will be shown in menu. 
	Multiple selections will be put in aSelectedCollection. Here also initial selections can be set"
	^self new aspect: aSymbol collection: aCollection selected: aSelectedCollection! !

!WebMenu class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
aspect: aSymbol collection: aCollection selectedToAspect: aSymbol2 of: anObject
	"Single selection menu.. Aspect of every element in aColection will be shown in menu. 
	Single selection will be put in an seelcted aspect of object. Here also initial selection can be set"
	^self new aspect: aSymbol collection: aCollection  selectedToAspect: aSymbol2 of: anObject! !

!WebMenu class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
collection: aCollection selected: aSelectedCollection
	"Multiple selection menu.Every element (text!!) in aColection will be shown in menu. 
	Multiple selection will be put in aSelectedCollection. Here also initial selections can be set"
	^self new collection: aCollection selected: aSelectedCollection! !

!WebMenu class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
collection: aCollection selectedToAspect: aSymbol2 of: anObject
	"Single selection menu.. Every element (text!!) in aColection will be shown in menu. 
	Single selection will be put in an seelcted aspect of object. Here also initial selection can be set"
	^self new collection: aCollection selectedToAspect: aSymbol2 of: anObject! !

!WebMenu methodsFor: 'private' stamp: 'mivsek 1/10/2008 22:24'!
acceptFormInputFrom: aPostDataArray
	| option result values |
	self selected isNil ifTrue: [self selected: OrderedCollection new]. "we need a collection!! "
	self selected copy do: [:each | self selected remove: each].
	values := aPostDataArray class == Dictionary
		ifTrue:  [Set new add: (aPostDataArray at: self name); yourself]
		ifFalse: [(aPostDataArray allAt: self name) asSet].
	values := values collect: [:each | AIDASite convert: each fromCodepage: #'UTF-8'].
	self selected addAll: 
		(self collection select:  [:each |
			option := self aspect isNil ifTrue: [each] ifFalse: [each perform: self aspect].
			option := option trimBlanks. "it should also eliminate more than one spaces together!!"
			values contains: [:value | value match: option] ] ).
	self aspectToStore notNil ifTrue: 
		[result := self selected notEmpty 
			ifTrue: [self selected asOrderedCollection first] ifFalse: [nil].
		self objectToStore perform: (self aspectToStore asString, ':') asSymbol with: result]! !

!WebMenu methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
aspect: aSymbol collection: aCollection selected: aSelectedCollection
	self aspect: aSymbol.
	self collection: aCollection.
	self selected: aSelectedCollection.
	self setMultiple.
	self size: 4.! !

!WebMenu methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
aspect: aSymbol collection: aCollection selectedToAspect: aSymbol2 of: anObject
	self multiple: false.
	self aspect: aSymbol.
	self collection: aCollection.
	self aspectToStore: aSymbol2.
	self objectToStore: anObject! !

!WebMenu methodsFor: 'private' stamp: ' 21/4/07 22:07'!
aspectToStore
	^aspectToStore! !

!WebMenu methodsFor: 'private' stamp: ' 21/4/07 22:07'!
aspectToStore: aSymbol
	"a name of a method to call on objectToStore, to write down a result from selection"
	aspectToStore := aSymbol! !

!WebMenu methodsFor: 'private' stamp: ' 21/4/07 22:07'!
collection
	^collection! !

!WebMenu methodsFor: 'private' stamp: ' 21/4/07 22:07'!
collection: aCollection
	collection := aCollection! !

!WebMenu methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
collection: aCollection selected: aSelectedCollection
	self collection: aCollection.
	self selected: aSelectedCollection.
	self setMultiple.
	self size: 4.! !

!WebMenu methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
collection: aCollection selectedToAspect: aSymbol of: anObject
	self multiple: false.
	self collection: aCollection.
	self aspectToStore: aSymbol.
	self objectToStore: anObject! !

!WebMenu methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	super initialize.
	self resetMultiple.! !

!WebMenu methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isEmpty
	"input field is empty or nil"
	| vlue |
	self objectToStore notNil ifTrue: [vlue := self objectToStore perform: self aspectToStore].
	^vlue isNil 
		or: [((vlue isKindOf: String) and: [vlue isEmpty])
			or: [(vlue isKindOf: Number) and: [vlue = 0] ]]! !

!WebMenu methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isMenu
	^true! !

!WebMenu methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isMultiple
	"multiple selection?"
	^self multiple! !

!WebMenu methodsFor: 'private' stamp: ' 21/4/07 22:07'!
joinToForm: aWebForm
	aWebForm fields keysAndValuesDo: [:fname :element | 
		element == self ifTrue: [self name: fname]. ^self]! !

!WebMenu methodsFor: 'private' stamp: ' 21/4/07 22:07'!
multiple
	^multiple! !

!WebMenu methodsFor: 'private' stamp: ' 21/4/07 22:07'!
multiple: aBoolean
	multiple := aBoolean! !

!WebMenu methodsFor: 'private' stamp: ' 21/4/07 22:07'!
objectToStore
	^objectToStore! !

!WebMenu methodsFor: 'private' stamp: ' 21/4/07 22:07'!
objectToStore: anObject
	"an object in which to write down a result from selection"
	objectToStore := anObject! !

!WebMenu methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
prepareSelected
	"add aspect of objectToStore selected result. There a preselection can reside"
	self aspectToStore isNil ifTrue: [^nil]. "multiple selections used, selected already set"
	self selected: (Set with: (self objectToStore perform: self aspectToStore))! !

!WebMenu methodsFor: 'printing' stamp: 'janko 7/18/2008 20:32'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	self prepareToHTMLPrintOn: aSession.
	self scriptBefore notNil ifTrue: 
		[ self scriptBefore 
			printHTMLPageOn: aStream for: aRequest on: aSession ].
	aStream nextPutAll: self ident , '<select'.
	self 
		printAttributesOn: aStream
		for: aSession.
	self isMultiple ifTrue: [ aStream nextPutAll: ' multiple' ].
	aStream nextPutAll: '>'.
	self printOptionsOn: aStream.
	aStream nextPutAll: '</select>'.
	self scriptAfter notNil ifTrue: 
		[ self scriptAfter 
			printHTMLPageOn: aStream for: aRequest on: aSession ]! !

!WebMenu methodsFor: 'printing' stamp: 'mivsek 1/10/2008 22:25'!
printOptionsOn: aStream
	| option preselected session |
	self collection isEmpty ifTrue: [^nil].
	self prepareSelected.
	preselected := self selected notNil ifTrue: [self selected asSet] ifFalse: [#()].
	aStream nextPutAll: '<option></option>'. "empty choice"
	session := self firstSessionFromStack.
	self collection do: [:each |
		option := self aspect isNil ifTrue: [each] ifFalse: [each perform: self aspect]. 
		option := AIDASite convertToWeb: option on: session.
		aStream nextPutAll: '<option'.
		aStream nextPutAll: ((preselected includes: each) ifTrue: [' selected>'] ifFalse: ['>']).
 		aStream nextPutAll: option; nextPutAll: '</option> ' ]! !

!WebMenu methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
resetMultiple
	self multiple: false! !

!WebMenu methodsFor: 'private' stamp: ' 21/4/07 22:07'!
selected
	"selected options wiil be stored here"
	^selected! !

!WebMenu methodsFor: 'private' stamp: ' 21/4/07 22:07'!
selected: aCollection
	selected := aCollection! !

!WebMenu methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
setMultiple
	self multiple: true! !

!WebMenu methodsFor: 'private' stamp: ' 21/4/07 22:07'!
shouldIdent
	^false! !

!WebMenu methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
size: aRowNumber
	"number of rows visible in multiple selection menu"
	self attributesAt: #size put: aRowNumber printString! !

WebFormElement subclass: #WebRadioButton
	instanceVariableNames: 'checked'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebRadioButton class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
name: aNameString value: aValueString 

	^(self new) name: aNameString; value: aValueString! !

!WebRadioButton methodsFor: 'private' stamp: ' 21/4/07 22:07'!
checked
	^checked! !

!WebRadioButton methodsFor: 'private' stamp: ' 21/4/07 22:07'!
checked: aBoolean
	checked := aBoolean! !

!WebRadioButton methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	super initialize.
	self type: 'radio'.
	self checked: false.! !

!WebRadioButton methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isChecked
	^self checked! !

!WebRadioButton methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isRadioButton
	^true! !

!WebRadioButton methodsFor: 'private' stamp: ' 26/4/07 10:37'!
joinToForm: aWebForm
	"join  this checkbox to the group of them with the same name in aWebForm. 
	This method is called by aWebForm!!"
	| existing |
	existing := aWebForm fieldSet 
		detect: [:each | each isRadioButton and: [each name = self name]]
		ifNone: [self value: Dictionary new. self].
	(existing value = '') ifTrue: [existing value: Dictionary new].
	self adaptor: existing adaptor.
	(self value includesKey: self value) ifFalse:
		[self value at: self value put: false] "????"! !

!WebRadioButton methodsFor: 'printing' stamp: 'janko 7/18/2008 20:33'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	self prepareToHTMLPrintOn: aSession.
	aStream nextPutAll: self ident , '<input'.
	self 
		printAttributesOn: aStream
		for: aSession.
	self isChecked ifTrue: [ aStream nextPutAll: ' checked' ].
	aStream nextPutAll: '>'! !

!WebRadioButton methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
setChecked
	self checked: true! !

!WebRadioButton methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
setUnchecked
	self checked: false! !

!WebRadioButton methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
shouldIdent
	^false! !

!WebRadioButton methodsFor: 'private' stamp: ' 21/4/07 22:07'!
value
	^self attributesAt: 'value'! !

!WebRadioButton methodsFor: 'private' stamp: ' 21/4/07 22:07'!
value: aString
	"set the value name  of the  radio button"
	^self attributesAt: 'value' put: aString! !

WebFormElement subclass: #WebTextArea
	instanceVariableNames: 'rows columns'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

WebTextArea subclass: #WebRichEditor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Components'!

!WebRichEditor commentStamp: '<historical>' prior: 0!
WebRichEditor is a JavaScript HTML WYSIWYG editor. Use it instead of text areas and you can rich edit your text!!

Usage:
	anElement addRichEditorAspect: #body for: myDocument!

!WebRichEditor methodsFor: 'setup' stamp: 'janko 7/21/2008 22:12'!
contentStyleUrl
	"where we have a CSS for content"
	^'/tinymce/jscripts/tiny_mce/themes/advanced/skins/default/content.css, /screen-a.css'! !

!WebRichEditor methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
ensureCssResourceForEditor
	"CSS for content of editor at /rich-editor.css
	see WebStyle richEditorCssResource and richEditorStyleSheet"
	"Be carefull to add the same url in initRichEditorScript at content_css !!"
	| site |
	site := self app site.
	site urlResolver halfUrlFor: site style richEditorCssResource. "just to register if not yet"! !

!WebRichEditor methodsFor: 'initialize-release' stamp: ' 21/8/07 09:38'!
ensureJavascriptForEditorInHeader
	| page url headerValue |
	page := self app.
	url := '/tinymce/jscripts/tiny_mce/tiny_mce.js'.
	headerValue := ' src="', url, '" language="JavaScript" type="text/javascript"'.
	(page headers contains: [:each | each key = 'script' and: [each value = headerValue] ]) ifFalse: 
		[page addHeader: 'script' value: headerValue.
		page addJavascript: self richEditorInitScript].! !

!WebRichEditor methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	super initialize.
	self attributesAt: 'mce_editable' put: 'true'.
	self ensureJavascriptForEditorInHeader.
	self ensureCssResourceForEditor.! !

!WebRichEditor methodsFor: 'setup' stamp: 'janko 7/21/2008 21:54'!
language
	^'en'! !

!WebRichEditor methodsFor: 'setup' stamp: 'janko 7/21/2008 22:00'!
richEditorInitScript
	"See TinyMCE docs for explanation: http://tinymce.moxiecode.com/tinymce/docs/index.html"
	^'
tinyMCE.init({
	mode : "specific_textareas",
	textarea_trigger : "mce_editable",
	language : "', self language, '",
	content_css : "', self contentStyleUrl, '",
	theme : "advanced",
	plugins : "paste, table, inlinepopups, save, autosave, searchreplace, advimage",
	theme_advanced_buttons1 : "save, , | , undo, redo, | , bold, italic, underline, strikethrough, sub, sup, | , bullist, numlist, outdent, indent, | , formatselect, fontsizeselect, | , justifyleft, justifycenter, justifyright,  justifyfull",
	theme_advanced_buttons2 : "cut, copy, pastetext, pasteword, | , search, replace, hr, | , tablecontrols, | , forecolor,backcolor, | , charmap, image,  link, unlink",
	theme_advanced_buttons3 : "",
	theme_advanced_toolbar_location : "top",
	theme_advanced_toolbar_align : "left",
	extended_valid_elements : "a[name|href|target|title|onclick],img[class|src|border=0|alt|title|hspace|vspace|width|height|align|onmouseover|onmouseout|name],hr[class|width|size|noshade],font[face|size|color|style],span[class|align|style]",
	save_enablewhendirty: "true",
	paste_use_dialog : "false",
	button_tile_map : "true",
	gecko_spellcheck : "true"
});
'! !

!WebTextArea class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
aspect: aSymbol for: anObject
	^self new aspect: aSymbol for: anObject! !

!WebTextArea class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
aspect: aSymbol for: anObject size: aNumber
	^self new aspect: aSymbol for: anObject size: aNumber! !

!WebTextArea methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
aspect: aSymbol for: anObject size: aNumber
	^self aspect: aSymbol for: anObject; size: aNumber! !

!WebTextArea methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
columns: aNumber
	^self attributesAt: #cols put: aNumber printString! !

!WebTextArea methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	super initialize.
	self columns: 5; rows: 5; wrap: 'soft'.! !

!WebTextArea methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isTextArea
	^true! !

!WebTextArea methodsFor: 'printing' stamp: 'janko 7/18/2008 20:39'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	self prepareToHTMLPrintOn: aSession.
	self scriptBefore notNil ifTrue: 
		[ self scriptBefore 
			printHTMLPageOn: aStream for: aRequest on: aSession ].
	aStream nextPutAll: self ident , '<textarea'.
	self 
		printAttributesOn: aStream
		for: aSession.
	aStream nextPutAll: '>'.
	self value notNil ifTrue: 
		[ aStream nextPutAll: (AIDASite 
				convertToWeb: (WebFormElement autoConvertToString: self value)
				on: aSession) ].
	aStream nextPutAll: '</textarea>'.
	self scriptAfter notNil ifTrue: 
		[ self scriptAfter 
			printHTMLPageOn: aStream for: aRequest on: aSession ]! !

!WebTextArea methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
rows: aNumber
	^self attributesAt: #rows put: aNumber printString! !

!WebTextArea methodsFor: 'private' stamp: ' 26/4/07 10:37'!
saveThroughAdapterValue: aValueString
	| vlue |
	vlue :=  AIDASite convertFromWeb: aValueString on: self session. "ensure unicode!!"
	vlue := (WebFormElement autoConvertString:  vlue toObject: self value).
	self value: vlue.! !

!WebTextArea methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
shouldIdent
	^false! !

!WebTextArea methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
size: aPoint
	"set the size of area in format colums@rows"
	self columns: aPoint x.
	self rows: aPoint y.! !

!WebTextArea methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
wrap: aString
	"how to wrap text in area"
	^self attributesAt: #wrap put: aString! !

WebElement subclass: #WebGrid
	instanceVariableNames: 'collection filtered columns page settings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Components'!

!WebGrid class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new initialize! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
addNewColumn
	self columns add: ((WebGridColumn newOn: self) id: (self columns size + 1))! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
ajaxUpdateWith: aParmString
	| parm |
	(aParmString notNil and: ['sort-*' match: aParmString]) ifTrue: 
		[parm := aParmString readStream upTo: $-; upToEnd.
		(self columnWithId: parm asInteger) sort. "or toogle sort order"
		self page: 1]. "always back to first page after sort change"
	(aParmString notNil and: ['page-*' match: aParmString]) ifTrue: 
		[parm := aParmString readStream upTo: $-; upToEnd.
		self page: parm asInteger]. 
	^self! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
buildPageSelectionRow
	| fullLength |
	self newRow. self row class: #tablePageSelection.
	self cell colspan: self columns size; align: #center.
	fullLength := self filtered size.
	1 to: (fullLength // self rowsOnPage) + 1 do: [:inx |
		inx = self page 
			ifTrue: [self cell addTextBold: inx printString]
			ifFalse: [self cell add: (self linkForPage: inx)].
		self cell addText: ' | '].! !

!WebGrid methodsFor: 'private-printing' stamp: 'np 5/7/2008 17:37'!
buildTable
	self initElements; initTable.
	self table class: self cssClass. 
	self width notNil ifTrue: [self table width: self width].
	self buildTableHeader.
	self sortColumnsIfNessesary.
	self hasFilters 
		ifTrue: [self buildTableFilterRow. self filterRows]
		ifFalse: [self filtered removeAll; addAll: self collection]. "filtered is what is shown, not collection!!"
	self buildTableRows.
	self needsSummary ifTrue: [self buildTableSummaryRow].
	self hasPages ifTrue: [self buildPageSelectionRow].
	self app form registerFormElementsIn: self.  "this will add registrations to grid form elements"! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
buildTableFilterRow
	| field |
	self newRow.
	self columns do: [:each | 
		each align notNil ifTrue: [self cell align: each align].
		each filter notNil ifTrue: 
			[field := WebDelayedField aspect: #filter for: each size: 10.
			each filterWidth notNil ifTrue: [field size: each filterWidth].
			field adapt. "because we are not part of a form"
			(self wasFilterChangedFor: each on: field) ifTrue: [field focus].
			field afterPostUpdate: self.
			self cell add: field].
		each ~= self columns last ifTrue: [self newCell] ].! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
buildTableHeader
	self columns do: [:each | 
		self cell setHeader.
		each align notNil ifTrue: [self cell align: each align].
		each name notNil ifTrue: [self cell add: (self sortLinkColumn: each)].
		each isSortedAscending ifTrue: [self cell addGif: #arrowOrangeUpGif].
		each isSortedDescending ifTrue: [self cell addGif: #arrowOrangeDownGif].
		each ~= self columns last ifTrue: [self newCell] ].! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/8/07 09:38'!
buildTableRows
	| inx objects pageLength |
	self collection notEmpty ifTrue: [self newRow].
	objects := self filtered.
	self resetSummary. inx := 0. objects do: [:object | inx := inx + 1. "summary in advance!!"
		self columns do: [:column | self summaryForColumn: column index: inx object: object]].
	inx := 1. pageLength := self hasPages ifTrue: [self rowsOnPage] ifFalse: [objects size]. 
	(self page-1) * pageLength > objects size ifTrue: [self page: 1]. 
	objects := objects 
		copyFrom: ((self page-1) * pageLength)+1 to: (self page * pageLength min: objects size).
	objects do: [:object |
		self setRowAttributesFor: self row object: object.
		self columns do: [:column |
			(object == objects first and: [column width notNil]) 
				ifTrue: [self cell width: column width].
			self printColumn: column index: inx object: object in: self cell.
			column ~= self columns last ifTrue: [self newCell] ].
		object ~= objects last ifTrue: [self newRow].
		inx := inx + 1 ]! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
buildTableSummaryRow
	self newRow. self row class: #summary.
	self columns do: [:each | 
		each align notNil ifTrue: [self cell align: each align].
		each name notNil ifTrue: [self cell addText: each summaryValue].
		each ~= self columns last ifTrue: [self newCell] ].! !

!WebGrid methodsFor: 'private-printing' stamp: 'mivsek 1/10/2008 17:42'!
canFilterObject: anObject onColumns: aColumnsToFilter
	aColumnsToFilter do: [:column || value |
		column aspect notNil ifTrue: 
			[value := anObject perform: column aspect.
			value notNil ifTrue:
				[((column filter, '*') match: value) ifFalse: [^false]]  ] ].
	^true! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
checkboxes
	"nil or column"
	^self settingsAt: #checkboxesColumn ifAbsent: [nil]! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
checkboxes: aColumnId
	^self settingsAt: #checkboxesColumn put: aColumnId! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
checkboxesCollection
	"collection of checked row objects"
	^self settingsAt: #checkboxesCollection ifAbsent: [nil]! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
checkboxesCollection: aCollection
	^self settingsAt: #checkboxesCollection put: aCollection! !

!WebGrid methodsFor: 'settings-column' stamp: ' 21/4/07 22:07'!
checkboxesColumn: aColumnId collection: aCollection
	"this column will have checboxes. Checked row objects will be in aCollection.
	preselected rows are done from initial aCollection"
	self checkboxes: aColumnId.
	self checkboxesCollection: aCollection! !

!WebGrid methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
collection
	collection isNil ifTrue: [^#()].
	^collection! !

!WebGrid methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
collection: aCollection
	collection := aCollection! !

!WebGrid methodsFor: 'settings-column' stamp: ' 21/4/07 22:07'!
column: aColumnId addBlock: aBlock
	"this block will be called with current row object as an argument. 
	It must return kind of WebElement which will be added to the table cell"
	self columns size < aColumnId ifTrue: [self addNewColumn].
	(self columns at: aColumnId) addBlock: aBlock! !

!WebGrid methodsFor: 'settings-column' stamp: ' 21/4/07 22:07'!
column: aColumnId viewBlock: aBlock
	"this block will be called with current row object as an argument. It must return plain string!!"
	self columns size < aColumnId ifTrue: [self addNewColumn]. "?"
	(self columns at: aColumnId) viewBlock: aBlock! !

!WebGrid methodsFor: 'settings-column' stamp: ' 21/4/07 22:07'!
columnAlign: anArrayOfSymbols
	"#left #right #center, nil if default left is used"
	| inx |
	inx := 1.
	anArrayOfSymbols do: [:each |
		self columns size < inx ifTrue: [self addNewColumn].
		(self columns at: inx) align: each.
		inx := inx + 1]! !

!WebGrid methodsFor: 'settings-column' stamp: ' 21/4/07 22:07'!
columnAspects: anArrayOfSymbols
	"methods to be called for each column on every object in collection"
	| inx |
	inx := 1.
	anArrayOfSymbols do: [:each |
		self columns size < inx ifTrue: [self addNewColumn].
		(self columns at: inx) aspect: each.
		inx := inx + 1]! !

!WebGrid methodsFor: 'settings-column' stamp: ' 21/4/07 22:07'!
columnFilters: anArrayOfBooleanOrNumbers
	"row with filter input fields, put true or field width on a column to add filter"
	| inx |
	inx := 1.
	anArrayOfBooleanOrNumbers do: [:each |
		self columns size < inx ifTrue: [self addNewColumn].
		each notNil ifTrue: [(self columns at: inx) setFilter].
		(each notNil and: [each class ~= True "but number"])  
			ifTrue: [(self columns at: inx) filterWidth: each].
		inx := inx + 1]! !

!WebGrid methodsFor: 'settings-column' stamp: ' 21/4/07 22:07'!
columnLinkAspects: anArrayOfSymbols
	"url link to an aspects of an object. Specify views with separate columnLinkViews: method"
	| inx |
	inx := 1.
	anArrayOfSymbols do: [:each |
		self columns size < inx ifTrue: [self addNewColumn].
		(self columns at: inx) linkAspect:  each.
		inx := inx + 1]! !

!WebGrid methodsFor: 'settings-column' stamp: ' 21/4/07 22:07'!
columnLinkViews: anArrayOfSymbols
	"link to an aspect of object(if set with columnLinks:, otherwise to object itself) with a view 
	will be shown in thi column"
	| inx |
	inx := 1.
	anArrayOfSymbols do: [:each |
		self columns size < inx ifTrue: [self addNewColumn].
		(self columns at: inx) linkView:  each.
		inx := inx + 1]! !

!WebGrid methodsFor: 'settings-column' stamp: ' 21/4/07 22:07'!
columnNames: anArrayOfStrings
	"set names of columns to be shown in header of table"
	| inx |
	inx := 1.
	anArrayOfStrings do: [:each |
		self columns size < inx ifTrue: [self addNewColumn].
		(self columns at: inx) name: each.
		inx := inx + 1]! !

!WebGrid methodsFor: 'settings-column' stamp: ' 21/4/07 22:07'!
columnWidth: anArrayOfNumbers
	"width of column or nil."
	| inx |
	inx := 1.
	anArrayOfNumbers do: [:each |
		self columns size < inx ifTrue: [self addNewColumn].
		(self columns at: inx) width: each.
		inx := inx + 1]! !

!WebGrid methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
columnWithId: aNumber
	aNumber > self columns size ifTrue: [^nil].
	^self columns at: aNumber! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
columns
	columns isNil ifTrue: [self initColumns].
	^columns! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
cssClass
	^self settingsAt: #cssClass ifAbsent: [self defaultCssClass]! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
deepCopyNotIn: aDict
	^nil! !

!WebGrid methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
defaultCssClass
	^#webGrid! !

!WebGrid methodsFor: 'accessing' stamp: 'np 5/7/2008 17:37'!
defaultWidth
	^1 "100%"! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
endTagFor: anObject
	(self rowBoldBlock notNil and: [self rowBoldBlock value: anObject]) ifTrue: [^'</b>'].
	^''! !

!WebGrid methodsFor: 'settings-row' stamp: ' 21/4/07 22:07'!
filterRow: anArrayOfBooleanOrNumbers
	"OBSOLETE!!"
	"row with filter input fields, put true or field width on a column to add filter"
	self columnFilters: anArrayOfBooleanOrNumbers! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
filterRows
	| columnsToFilter | 
	columnsToFilter := self columns select: [:each | each filter notNil]. 
	self filtered removeAll; addAll: 
		(columnsToFilter notEmpty
			ifTrue: [(self collection select: [:object | 
				self canFilterObject: object onColumns: columnsToFilter])] 
			ifFalse: [self collection])! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
filtered
	"objects which pass filter criteria. if nil, filters are not set"
	filtered isNil ifTrue: [self initFiltered].
	^filtered! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
filtered: aCollection
	filtered := aCollection! !

!WebGrid methodsFor: 'settings-column' stamp: ' 21/4/07 22:07'!
filteredCollection: aCollection
	"for external access to filtered rows only"
	filtered := aCollection! !

!WebGrid methodsFor: 'private-testing' stamp: ' 21/4/07 22:07'!
hasFilters
	"any column has a filter"
	^self columns contains: [:each | each filter notNil].! !

!WebGrid methodsFor: 'private-testing' stamp: ' 21/4/07 22:07'!
hasNoZeros
	"don't show zeros"
	^self settingsAt: #noZeros ifAbsent: [false]! !

!WebGrid methodsFor: 'private-testing' stamp: ' 21/4/07 22:07'!
hasPages
	^self rowsOnPage notNil and: [self collection size > self rowsOnPage].! !

!WebGrid methodsFor: 'private-testing' stamp: ' 21/4/07 22:07'!
hasShortDates
	"year in two digits only"
	^self settingsAt: #shortDates ifAbsent: [false]! !

!WebGrid methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initColumns
	columns := OrderedCollection new! !

!WebGrid methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initFiltered
	filtered isNil 
		ifTrue: [filtered := OrderedCollection new]
		ifFalse: [filtered removeAll]! !

!WebGrid methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initSettings
	settings := Dictionary new.! !

!WebGrid methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	self method: #ajaxUpdateWith: "this method will be called after ajax update"! !

!WebGrid methodsFor: 'private-testing' stamp: ' 21/4/07 22:07'!
insideDivTag
	"to enclose or not in div tag. Only if element have any attribute!!"
	^self div | self attributes notNil.! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
lastColumn
	^self columns notEmpty ifTrue: [self columns last] ifFalse: [nil].! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
linkForPage: aNumber
	| dummyLink |
	dummyLink := WebLink text: aNumber printString linkTo: (Array with: self app observee with: '-').
	self app inDefaultView ifFalse: [dummyLink view: self app view].
	dummyLink onClickUpdate: self with: 'page-', aNumber printString.
	^dummyLink! !

!WebGrid methodsFor: 'private-testing' stamp: ' 21/4/07 22:07'!
needsSummary
	"summary row needed or not"
	^self columns contains: [:each | each needsSummary].! !

!WebGrid methodsFor: 'settings-column' stamp: ' 21/4/07 22:07'!
noZeros
	"don't show if value is zero"
	^self settingsAt: #noZeros put: true! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
numbering
	"false or column"
	^self settingsAt: #numberingColumn ifAbsent: [nil]! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
numbering: aColumnId
	"false or column"
	^self settingsAt: #numberingColumn put: aColumnId! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
page
	"number of a page currently shown"
	page isNil ifTrue: [self page: 1].
	^page! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
page: aNumber
	page := aNumber! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
parent: aWebElement
	"self buildTable." "build just before rendering!!"
	^super parent: aWebElement! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
prepareToHTMLPrintOn: aSession
	super prepareToHTMLPrintOn: aSession.
	self buildTable! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
printCheckboxIndex: aNumber object: anObject in: anElement
	| field |
	field := WebCheckBox object: anObject from: self checkboxesCollection.
"	self app form registerFormElementsIn: field.  "
	anElement add: field.! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
printColumn: aColumn index: aNumber object: anObject in: anElement
	| start |
	aColumn align notNil ifTrue: [anElement align: aColumn align].
	aColumn isNumbered  ifTrue: 
		[start := self hasPages ifTrue: [(self page-1) * self rowsOnPage] ifFalse: [0].
		^self printNumber: start+aNumber object: anObject in: anElement].
	aColumn isCheckboxed  ifTrue: [^self printCheckboxIndex: aNumber object: anObject in: anElement].
	aColumn hasLink ifTrue: [^self printLinkTo: anObject column: aColumn in: anElement]. 
	aColumn hasAddBlock ifTrue: [^anElement add: (aColumn addBlock value: anObject)].
	anElement addText: 
		(self startTagFor: anObject), (aColumn viewBlock value: anObject), (self endTagFor: anObject)! !

!WebGrid methodsFor: 'private-printing' stamp: 'janko 7/18/2008 20:23'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	"self buildTable."
	"not so late, build imediately after adding to some parent!!"
	self prepareToHTMLPrintOn: aSession.	"buildTable"
	aStream nextPutAll: self ident , '<div'.
	self 
		printAttributesOn: aStream
		for: aSession.
	aStream nextPutAll: '>' , self eol.
	elements notNil ifTrue: 
		[ elements do: 
			[ : element | 
			element notNil ifTrue: 
				[ element 
					printHTMLPageOn: aStream for: aRequest on: aSession ] ] ].
	aStream nextPutAll: self ident , '</div>' , self eol! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
printLinkTo: anObject column: aColumn in: anElement
	| view object linkText |
	view := aColumn linkView notNil ifTrue: [aColumn linkView] ifFalse: [#main].
	object := aColumn linkAspect notNil 
		ifTrue: [anObject perform: aColumn linkAspect] ifFalse: [anObject].
	linkText := (self startTagFor: anObject), (aColumn viewBlock value: anObject), 
		(self endTagFor: anObject).
	object notNil
		ifTrue: [anElement addLinkTo: object text: linkText view: view]
		ifFalse: [anElement addText: linkText]! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
printNumber: aNumber object: anObject in: anElement
	anElement addText: 
		(self startTagFor: anObject), aNumber printDotString, '.', (self endTagFor: anObject)! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
removeLastColumn
	self columns notEmpty ifTrue: [self columns remove: self lastColumn].! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
representBinaryOn: writer
	^0! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
resetSummary
	self columns do: [:each | each resetSummary].! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
rowBlueBlock
	^self settingsAt: #rowBlueBlock! !

!WebGrid methodsFor: 'settings-row' stamp: ' 21/4/07 22:07'!
rowBlueIfTrue: aBlock
	^self settingsAt: #rowBlueBlock put: aBlock! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
rowBoldBlock
	^self settingsAt: #rowBoldBlock! !

!WebGrid methodsFor: 'settings-row' stamp: ' 21/4/07 22:07'!
rowBoldIfTrue: aBlock
	^self settingsAt: #rowBoldBlock put: aBlock! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
rowGrayBlock
	^self settingsAt: #rowGrayBlock! !

!WebGrid methodsFor: 'settings-row' stamp: ' 21/4/07 22:07'!
rowGrayIfTrue: aBlock
	^self settingsAt: #rowGrayBlock put: aBlock! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
rowGreenBlock
	^self settingsAt: #rowGreenBlock! !

!WebGrid methodsFor: 'settings-row' stamp: ' 21/4/07 22:07'!
rowGreenIfTrue: aBlock
	^self settingsAt: #rowGreenBlock put: aBlock! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
rowRedBlock
	^self settingsAt: #rowRedBlock! !

!WebGrid methodsFor: 'settings-row' stamp: ' 21/4/07 22:07'!
rowRedIfTrue: aBlock
	^self settingsAt: #rowRedBlock put: aBlock! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
rowYellowBlock
	^self settingsAt: #rowYellowBlock! !

!WebGrid methodsFor: 'settings-row' stamp: ' 21/4/07 22:07'!
rowYellowIfTrue: aBlock
	^self settingsAt: #rowYellowBlock put: aBlock! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
rowsOnPage
	^self settingsAt: #rowsOnPage! !

!WebGrid methodsFor: 'settings-row' stamp: ' 21/4/07 22:07'!
rowsOnPage: anInteger
	^self settingsAt: #rowsOnPage put: anInteger! !

!WebGrid methodsFor: 'settings-column' stamp: ' 21/4/07 22:07'!
setNumbering
	"in first column a sequential nr. of this row will be shown"
	self setNumberingOn: 1! !

!WebGrid methodsFor: 'settings-column' stamp: ' 21/4/07 22:07'!
setNumberingOn: aColumnId
	"in this column a sequential nr. of this row will be shown"
	self numbering: aColumnId! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
setRowAttributesFor: aWebRow object: anObject
	(self rowRedBlock notNil and: [self rowRedBlock value: anObject]) 
		ifTrue: [^aWebRow class: #red].
	(self rowGreenBlock notNil and: [self rowGreenBlock value: anObject]) 
		ifTrue: [^aWebRow class: #green].
	(self rowBlueBlock notNil and: [self rowBlueBlock value: anObject]) 
		ifTrue: [^aWebRow class: #blue].
	(self rowYellowBlock notNil and: [self rowYellowBlock value: anObject]) 
		ifTrue: [^aWebRow class: #yellow].
	(self rowGrayBlock notNil and: [self rowGrayBlock value: anObject]) 
		ifTrue: [^aWebRow class: #gray].! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
settings
	settings isNil ifTrue: [self initSettings].
	^settings! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
settingsAt: aSymbol
	^self settingsAt: aSymbol ifAbsent: [nil]! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
settingsAt: aSymbol ifAbsent: aBlock
	^self settings at: aSymbol ifAbsent: aBlock! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
settingsAt: aSymbol put: aValue
	self settings at: aSymbol put: aValue! !

!WebGrid methodsFor: 'settings-column' stamp: ' 21/4/07 22:07'!
shortDates
	"year in two digits only"
	^self settingsAt: #shortDates put: true! !

!WebGrid methodsFor: 'settings-column' stamp: ' 21/4/07 22:07'!
sortAscendingOn: aColumnId
	"this column will be sorted ascending"
	self sortColumn: aColumnId.
	self sortOrder: #ascending! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
sortColumn
	"false or column"
	^self settingsAt: #sortColumn ifAbsent: [nil]! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
sortColumn: aColumnId
	"false or column"
	^self settingsAt: #sortColumn put: aColumnId! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
sortColumnsIfNessesary
	| column sortBlock | 
	 self sortColumn isNil ifTrue: [^nil].
	column := self columns at: self sortColumn.
	sortBlock := (self sortOrder = #ascending)
		ifTrue: [ [:a :b | (a perform: column aspect) < (b perform: column aspect)] ]
		ifFalse: [ [:a :b | (a perform: column aspect) > (b perform: column aspect)] ].
	self collection: (SortedCollection withAll: self collection sortBlock: sortBlock).! !

!WebGrid methodsFor: 'settings-column' stamp: ' 21/4/07 22:07'!
sortDescendingOn: aColumnId
	"this column will be sorted ascending"
	self sortColumn: aColumnId.
	self sortOrder: #descending! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
sortLinkColumn: aWebGridColumn
	| dummyLink |
	dummyLink := WebLink text: aWebGridColumn name linkTo: (Array with: self app observee with: '-').
	self app inDefaultView ifFalse: [dummyLink view: self app view].
	dummyLink onClickUpdate: self with: 'sort-', aWebGridColumn id printString.
	^dummyLink! !

!WebGrid methodsFor: 'settings-column' stamp: ' 21/4/07 22:07'!
sortOn: aColumnId
	"this column will be sorted ascending"
	self sortAscendingOn: aColumnId! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
sortOrder
	^self settingsAt: #sortOrder ifAbsent: [#ascending]! !

!WebGrid methodsFor: 'private' stamp: ' 21/4/07 22:07'!
sortOrder: aSymbol
	"#ascending or #descending"
	^self settingsAt: #sortOrder put: aSymbol! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
startTagFor: anObject
	(self rowBoldBlock notNil and: [self rowBoldBlock value: anObject]) ifTrue: [^'<b>'].
	^''! !

!WebGrid methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
summaryForColumn: aColumn index: aNumber object: anObject
	aColumn needsSummary ifFalse: [^nil].
	aColumn addToSummary: anObject.! !

!WebGrid methodsFor: 'settings-row' stamp: ' 21/4/07 22:07'!
summaryRow: anArrayOfSymbolsOrStrings
	"if you want a summary row, define for desired column: #sum, #count or 'some text' "
	| inx |
	inx := 1.
	anArrayOfSymbolsOrStrings do: [:each |
		self columns size < inx ifTrue: [self addNewColumn].
		(self columns at: inx) summaryType:  each.
		inx := inx + 1]! !

!WebGrid methodsFor: 'private-testing' stamp: ' 21/4/07 22:07'!
wasFilterChangedFor: aWebGridColumn on: anInputField
	| request vlue |
	request := self app session lastRequest.
	(request isAjaxRequest and: [request isPost]) ifFalse: [^false].
	request postKeysAndValuesDo: [:key :value |
		(('field', '*') match: key) ifTrue: [vlue := value] ].
	^vlue = aWebGridColumn filter! !

!WebGrid methodsFor: 'accessing' stamp: 'np 5/7/2008 17:37'!
width
	^self settingsAt: #width ifAbsent: [self defaultWidth]! !

!WebGrid methodsFor: 'accessing' stamp: 'np 5/7/2008 17:38'!
width: anInteger
	self settingsAt: #width put: anInteger! !

WebElement subclass: #WebIFrame
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebIFrame commentStamp: '<historical>' prior: 0!
WebIFrame is frame, inserted within a block of text. See IFRAME tag in HTML 4.01 spec
!

!WebIFrame methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
frameBorder: aNumber
	"1 yes (default), 0 no"
	self attributesAt: #frameborder put: aNumber! !

!WebIFrame methodsFor: 'attributes' stamp: 'mivsek 1/10/2008 22:20'!
height: aNumber
	self attributesAt: #height put: aNumber printString! !

!WebIFrame methodsFor: 'printing' stamp: 'janko 7/18/2008 20:24'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	self prepareToHTMLPrintOn: aSession.
	self scriptBefore notNil ifTrue: 
		[ self scriptBefore 
			printHTMLPageOn: aStream for: aRequest on: aSession ].
	aStream nextPutAll: self ident , '<iframe'.
	self 
		printAttributesOn: aStream
		for: aSession.
	aStream nextPutAll: '>'.
	aStream nextPutAll: '</iframe>'.
	self scriptAfter notNil ifTrue: 
		[ self scriptAfter 
			printHTMLPageOn: aStream for: aRequest on: aSession ]! !

!WebIFrame methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
scrolling: aSymbol
	"#yes #no #auto (default)"
	self attributesAt: #scrolling put: aSymbol! !

!WebIFrame methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
size: aPoint
	"set the size of frame area in format width@height pixels"
	self width: aPoint x.
	self height: aPoint y.! !

!WebIFrame methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
src: anUrlString
	self attributesAt: #src put: anUrlString! !

!WebIFrame methodsFor: 'attributes' stamp: 'mivsek 1/10/2008 22:21'!
width: aNumber
	self attributesAt: #width put: aNumber printString! !

WebElement subclass: #WebImage
	instanceVariableNames: 'image imageMap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebImage class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
gif: aSymbol
	"this method will be called in your webStyle and this method 
	should return a gif in byte array format"
	| methodImage image |
	image := self new.
	methodImage := WebMethodImage
		fromMethod: aSymbol 
		on: image style 
		contentType: 'image/gif' 
		site: image site.
	^image image: methodImage! !

!WebImage class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
gif: aSymbol  size: aPoint
	"this method will be called in your webStyle and this method 
	should return a gif in byte array format"
	| methodImage image |
	image := self new.
	methodImage := WebMethodImage
		fromMethod: aSymbol 
		on: image style 
		contentType: 'image/gif' 
		site: image site.
	^image image: methodImage;  size: aPoint! !

!WebImage class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
image: anImage
	^self new image: anImage! !

!WebImage class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
image: anImage align: aSymbol
	^self new image: anImage align: aSymbol! !

!WebImage class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
image: anImage size: aPoint
	^self new image: anImage; size: aPoint! !

!WebImage class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
jpeg: aSymbol
	"this method will be called in your webStyle and this method 
	should return a jpeg in byte array format"
	| methodImage image |
	image := self new.
	methodImage := WebMethodImage
		fromMethod: aSymbol 
		on: image style 
		contentType: 'image/jpeg' 
		site: image site.
	^image image: methodImage! !

!WebImage class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
jpeg: aSymbol  size: aPoint
	"this method will be called in your webStyle and this method 
	should return a jpeg in byte array format"
	| methodImage image |
	image := self new.
	methodImage := WebMethodImage
		fromMethod: aSymbol 
		on: image style 
		contentType: 'image/jpeg' 
		site: image site.
	^image image: methodImage;  size: aPoint! !

!WebImage class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new initialize! !

!WebImage class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
png: aSymbol
	"this method will be called in your webStyle and this method 
	should return a png in byte array format"
	| methodImage image |
	image := self new.
	methodImage := WebMethodImage
		fromMethod: aSymbol 
		on: image style 
		contentType: 'image/png' 
		site: image site.
	^image image: methodImage! !

!WebImage class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
png: aSymbol  size: aPoint
	"this method will be called in your webStyle and this method 
	should return a jpeg in byte array format"
	| methodImage image |
	image := self new.
	methodImage := WebMethodImage
		fromMethod: aSymbol 
		on: image style 
		contentType: 'image/png' 
		site: image site.
	^image image: methodImage;  size: aPoint! !

!WebImage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
alt: aString
	self attributes at: #alt put: aString! !

!WebImage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
border: aNumber
	self attributesAt: #border put: aNumber printString! !

!WebImage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
height: aNumber
	self attributesAt: #height put: aNumber printString! !

!WebImage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
hspace: aNumber
	self attributesAt: #hspace put: aNumber printString! !

!WebImage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
image
	^image! !

!WebImage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
image: anImage
	"set the image object of WebImage. 
	If nil, then urlreference should exist for image somewhere else"
	image := anImage.! !

!WebImage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
image: anImage align: aSymbol
	"set the image object of WebImage. and aligment"
	self image: anImage.
	self align: aSymbol! !

!WebImage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
imageAlign: aSymbol
	"#left #center #right"
	self attributesAt: #align put: aSymbol asString! !

!WebImage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
imageMap
	^imageMap! !

!WebImage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
imageMap: anImageMap
	imageMap := anImageMap! !

!WebImage methodsFor: 'initialize-release' stamp: 'janko 3/21/2008 16:23'!
initialize
	self border: 0.
	self alt: ''.! !

!WebImage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
name: aString
	self attributesAt: #name put: aString! !

!WebImage methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
prepareAttributesToPrintOn: aSession
	self src: (self urlOnSession: aSession).
	self imageMap notNil ifTrue: [self usemap: self imageMap name].! !

!WebImage methodsFor: 'printing' stamp: 'janko 7/18/2008 20:25'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	self prepareToHTMLPrintOn: aSession.
	self scriptBefore notNil ifTrue: 
		[ self scriptBefore 
			printHTMLPageOn: aStream for: aRequest on: aSession ].
	aStream nextPutAll: self ident , '<img'.
	self 
		printAttributesOn: aStream
		for: aSession.
	aStream nextPutAll: '>'.
	self imageMap notNil ifTrue: 
		[ imageMap 
			printHTMLPageOn: aStream for: aRequest on: aSession ].
	self scriptAfter notNil ifTrue: 
		[ self scriptAfter 
			printHTMLPageOn: aStream for: aRequest on: aSession ]! !

!WebImage methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
shouldIdent
	^false! !

!WebImage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
size: aPoint
	"set the size of an image in format width@height"
	self width: aPoint x.
	self height: aPoint y.! !

!WebImage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
src: aString
	self attributesAt: #src put: aString! !

!WebImage methodsFor: 'urls' stamp: ' 21/4/07 22:07'!
urlForLiveImageOn: aSession
	 ^aSession site urlResolver halfUrlFor: self image! !

!WebImage methodsFor: 'urls' stamp: ' 21/4/07 22:07'!
urlForMethodImageOn: aSession
	 ^aSession site urlResolver halfUrlFor: self image! !

!WebImage methodsFor: 'urls' stamp: ' 21/4/07 22:07'!
urlOnSession: aSession
	self image isNil ifTrue: [^''].
	self image isString ifTrue: [^self image].
	(self image isKindOf: WebLiveImage) ifTrue: [^self urlForLiveImageOn: aSession].
	(self image isKindOf: WebMethodImage) ifTrue: [^self urlForMethodImageOn: aSession].
	^aSession site urlResolver halfUrlFor: self image.! !

!WebImage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
usemap: aString
	self attributesAt: #usemap put: aString! !

!WebImage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
width: aNumber
	self attributesAt: #width put: aNumber printString! !

WebElement subclass: #WebImageMap
	instanceVariableNames: 'areas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebImageMap class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new initialize! !

!WebImageMap methodsFor: 'adding areas' stamp: ' 21/4/07 22:07'!
addCirclePos: aPositionPoint diameter: aNumber  link: aWebLink

	| coords |
	coords := Array
		with: aPositionPoint x
		with: aPositionPoint y
		with: aNumber.
	self addShape: #circle coordinates: coords link: aWebLink! !

!WebImageMap methodsFor: 'adding areas' stamp: ' 21/4/07 22:07'!
addRectanglePos: aPositionPoint size: aSizePoint  link: aWebLink

	| coords |
	coords := Array
		with: aPositionPoint x
		with: aPositionPoint y
		with: (aPositionPoint + aSizePoint) x
		with: (aPositionPoint + aSizePoint) y.
	self addShape: #rect coordinates: coords link: aWebLink! !

!WebImageMap methodsFor: 'adding areas' stamp: ' 21/4/07 22:07'!
addShape: aShapeSymbol coordinates: anArray link: aWebLink

	self areas add: 
		(Array with: aShapeSymbol with: anArray with: aWebLink)! !

!WebImageMap methodsFor: 'private' stamp: ' 21/4/07 22:07'!
areas
	areas isNil ifTrue: [self initAreas].
	^areas! !

!WebImageMap methodsFor: 'initialize - release' stamp: ' 21/4/07 22:07'!
initAreas
	areas := OrderedCollection new.! !

!WebImageMap methodsFor: 'initialize - release' stamp: ' 21/4/07 22:07'!
initialize
	self name: self randomName.! !

!WebImageMap methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
name
	^self attributesAt: #name! !

!WebImageMap methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
name: aString
	self attributesAt: #name put: aString.! !

!WebImageMap methodsFor: 'printing' stamp: 'janko 7/18/2008 20:26'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	| link |
	self prepareToHTMLPrintOn: aSession.
	aStream nextPutAll: self ident , '<map'.
	self 
		printAttributesOn: aStream
		for: aSession.
	aStream nextPutAll: '"> ' , self eol.
	self areas do: 
		[ : area | 
		aStream
			nextPutAll: self identMore , '<area shape="' ;
			nextPutAll: (area at: 1) asString ;
			nextPutAll: '" coords="'.
		1 
			to: (area at: 2) size
			do: 
				[ : inx | 
				inx > 1 ifTrue: [ aStream nextPutAll: ',' ].
				aStream nextPutAll: ((area at: 2) at: inx) printString ].
		link := area at: 3.
		link prepareAttributesToPrintOn: aSession.
		link 
			printAttributesOn: aStream
			for: aSession.
		aStream nextPutAll: '"> ' , self eol ].
	aStream nextPutAll: self ident , '</map>' , self eol! !

!WebImageMap methodsFor: 'private' stamp: ' 21/4/07 22:07'!
randomName

	^'map', ((Random new next * 100) truncated) printString! !

!WebImageMap methodsFor: 'private' stamp: ' 21/4/07 22:07'!
shouldIdent
	^true! !

WebElement subclass: #WebList
	instanceVariableNames: 'name kind type'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebList class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
kind: aSymbol
	"should be #ordered, #unordered, #definition"

	^self new initialize kind: aSymbol! !

!WebList class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new initialize! !

!WebList class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newDefinition
	^self kind: #definition! !

!WebList class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newOrdered
	^self kind: #ordered! !

!WebList class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newUnordered
	^self kind: #unordered! !

!WebList methodsFor: 'adding/removing' stamp: ' 21/4/07 22:07'!
add: anElement
	"ensure always WebListItem as subelement of list"
	| element |
	element := (anElement class == WebListItem)
		ifTrue: [anElement] 
		ifFalse: [WebListItem newListItem add: anElement; yourself].
	^super add: element! !

!WebList methodsFor: 'adding/removing' stamp: ' 21/4/07 22:07'!
addDefinition: anElement
	"ensure always WebListItem as subelement of list"
	| element |
	element := (anElement class == WebListItem)
		ifTrue: [anElement setDefinition] 
		ifFalse: [WebListItem newDefinition add: anElement; yourself].
	^super add: element! !

!WebList methodsFor: 'adding/removing' stamp: ' 21/4/07 22:07'!
addDefinitionTerm: anElement
	"ensure always WebListItem as subelement of list"
	| element |
	element := (anElement class == WebListItem)
		ifTrue: [anElement setDefinitionTerm] 
		ifFalse: [WebListItem newDefinitionTerm add: anElement; yourself].
	^super add: element! !

!WebList methodsFor: 'adding/removing' stamp: ' 21/4/07 22:07'!
addLinkTo: anObject text: aString 
	"add new line to the list with text and link to some object" 
	| line |
	line := WebListItem new.
	line	addLinkTo: anObject text: aString.
	self add: line! !

!WebList methodsFor: 'adding/removing' stamp: ' 21/4/07 22:07'!
addLinkTo: anObject text: aString class: aSymbol
	"add new line to the list with text and link to some object. Set class for CSS too" 
	| line |
	line := WebListItem new class: aSymbol.
	line	addLinkTo: anObject text: aString.
	self add: line! !

!WebList methodsFor: 'adding/removing' stamp: ' 21/4/07 22:07'!
addLinkTo: anObject text: aString id: aSymbol
	"add new line to the list with text and link to some object. set id too" 
	| line |
	line := WebListItem new id: aSymbol.
	line	addLinkTo: anObject text: aString.
	self add: line! !

!WebList methodsFor: 'adding/removing' stamp: ' 21/4/07 22:07'!
addLinkTo: anObject text: aString withBullet: anImageOrURL
	"add new line to the list with bullet image and text and link to some object" 
	| line |
	line := WebListItem new.
	line add: (WebImage image: anImageOrURL); add: (WebLink text: aString linkTo: anObject).
	self  add: line! !

!WebList methodsFor: 'adding/removing' stamp: ' 21/4/07 22:07'!
addText: aString
	"make a new element with aString. Attributes are same as previous text." 
	| line |
	line := WebListItem new.
	line addText: aString.
	self  add: line! !

!WebList methodsFor: 'adding/removing' stamp: ' 21/4/07 22:07'!
addText: aString class: aSymbol
	"add list item with specified text. set class for CSS to that item too" 
	| line |
	line := WebListItem new class: aSymbol.
	line addText: aString.
	self  add: line! !

!WebList methodsFor: 'adding/removing' stamp: ' 21/4/07 22:07'!
addText: aString id: aSymbol
	"add list item with specified text. set id to that item too" 
	| line |
	line := WebListItem new id: aSymbol.
	line addText: aString.
	self  add: line! !

!WebList methodsFor: 'adding/removing' stamp: ' 21/4/07 22:07'!
addText: aString withBullet: anImageOrURL
	"add new line to the list with bullet image and text" 
	| line |
	line := WebListItem new.
	line add: (WebImage image: anImageOrURL); addText: aString.
	self  add: line! !

!WebList methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	kind := #definition.! !

!WebList methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
kind
	^kind! !

!WebList methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
kind: aSymbol
	"should be #ordered, #unordered, #definition. By default is #definition"

	kind := aSymbol.! !

!WebList methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
name
	^name! !

!WebList methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
name:  aString
	"set the name of this list. Used for accessing the selections from WebForm or WebApplication"

	name := aString.! !

!WebList methodsFor: 'printing' stamp: 'janko 7/18/2008 20:29'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	| tag |
	self prepareToHTMLPrintOn: aSession.
	self kind == #ordered ifTrue: [ tag := 'ol' ].
	self kind == #unordered ifTrue: [ tag := 'ul' ].
	self kind == #definition ifTrue: [ tag := 'dl' ].
	aStream nextPutAll: self ident , '<' , tag.
	self 
		printAttributesOn: aStream
		for: aSession.
	aStream nextPutAll: '>' , self eol.
	super 
		printHTMLPageOn: aStream for: aRequest on: aSession.
	aStream nextPutAll: self ident , '</' , tag , '>' , self eol! !

!WebList methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
shouldIdent
	^true! !

WebList subclass: #WebTabs
	instanceVariableNames: 'selected'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Components'!

!WebTabs commentStamp: '<historical>' prior: 0!
WebTabs are visual selection tool. selected tab is in front, others are back and have url links.
By default an instance have CSS class 'webtabs' and selected tab 'webtabselected"

Usage:
	self addTextBold: 'Selected without link. 
	self addLinkTo: someObject text: 'not-selected with link' view: #view.
	self selected: 1.


Instance Variables:
	selected	<Integer>	which tab is selected. By default is first.

!

WebTabs subclass: #ViewTabs
	instanceVariableNames: 'views'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Components'!

!ViewTabs commentStamp: '<historical>' prior: 0!
For quick selection of App views. Just fill with view names and descriptions, all other will be automatic. Tab for curently shown will be colored as shown, other tabls will have links to self observee and appropriate view!

!ViewTabs class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new initialize! !

!ViewTabs methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
addView: aSymbol description: aString
	(self views contains: [:each | each key == aSymbol asSymbol]) 
		ifFalse: [self views add: (Association key: aSymbol asSymbol value: (Array with: aString with: #()))]! !

!ViewTabs methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
addView: aSymbol description: aString also: aSymbolArray
	"if view is in aSymbolArray, then it is also considered selected"
	(self views contains: [:each | each key == aSymbol asSymbol]) 
		ifFalse: [self views add: (Association key: aSymbol asSymbol 
			value: (Array with: aString with: aSymbolArray))]! !

!ViewTabs methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initViews
	views := OrderedCollection new.! !

!ViewTabs methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	super initialize.
	self initViews.
	^self! !

!ViewTabs methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
prepareTabsList
	| inx view |
	inx := 1. view := self app view. view = #'' ifTrue: [view := #main].
	self views do: [:each | 
		view = each key | ((each value at: 2) includes: view)
		 	ifTrue: [self addTextBold: (each value at: 1). self selected: inx]
			ifFalse: [self addLinkTo: self app observee text: (each value at: 1) view: each key].
		inx := inx + 1]! !

!ViewTabs methodsFor: 'printing' stamp: 'janko 7/18/2008 20:30'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	self prepareTabsList.
	^ super 
		printHTMLPageOn: aStream for: aRequest on: aSession! !

!ViewTabs methodsFor: 'private' stamp: ' 21/4/07 22:07'!
views
	^views! !

!WebTabs class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new initialize! !

!WebTabs methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
defaultClass
	"default CSS class for tabs, see WebStyle cssTabs1 for more info"
	^#tabs1! !

!WebTabs methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	super initialize.
	self kind: #unordered.
	self class: self defaultClass.
	self selected: 1! !

!WebTabs methodsFor: 'printing' stamp: 'janko 7/18/2008 20:29'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	(self selected notNil and: [ self selected <= self elements size ]) 
		ifTrue: [ (self elements at: self selected) class: self selectedClass ].
	super 
		printHTMLPageOn: aStream for: aRequest on: aSession! !

!WebTabs methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
selected
	^selected! !

!WebTabs methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
selected: aNumber
	"whivh tab is selected, that is in front"
	selected := aNumber! !

!WebTabs methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
selectedClass
	"CSS class for selected tab, auto coposed!! "
	^(self attributesAt: #class), 'selected'! !

WebElement subclass: #WebListItem
	instanceVariableNames: 'tag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebListItem class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new initialize! !

!WebListItem class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newDefinition
	^self new setDefinition "dd"! !

!WebListItem class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newDefinitionTerm
	^self new setDefinitionTerm "dt"! !

!WebListItem class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newListItem
	^self new setListItem "li"! !

!WebListItem methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	self setListItem! !

!WebListItem methodsFor: 'printing' stamp: 'janko 7/18/2008 20:31'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	self prepareToHTMLPrintOn: aSession.
	aStream nextPutAll: self ident , '<' , self tag.
	self 
		printAttributesOn: aStream
		for: aSession.
	aStream nextPutAll: '>'.
	super 
		printHTMLPageOn: aStream for: aRequest on: aSession.
	aStream nextPutAll: self ident , '</' , self tag , '>' , self eol! !

!WebListItem methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
setDefinition
	self tag: 'dd'! !

!WebListItem methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
setDefinitionTerm
	self tag: 'dt'! !

!WebListItem methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
setListItem
	self tag: 'li'! !

!WebListItem methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
shouldIdent
	^false! !

!WebListItem methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
tag
	^tag! !

!WebListItem methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
tag: aString
	tag := aString! !

WebElement subclass: #WebLiveImage
	instanceVariableNames: 'gif refreshed width height painter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Imaging'!

!WebLiveImage commentStamp: '<historical>' prior: 0!
WebLiveImage is used for live, dynamic GIF image construction.

Instance Variables:
	gif			<ByteArray>	content of image in GIF format
	refreshed	<Timestamp>	timestamp of last gif creation, used for caching algoritms

!

WebLiveImage subclass: #WebGraph
	instanceVariableNames: 'data margin title graphics'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Imaging'!

!WebGraph methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
data
	data isNil ifTrue: [data := #(120 200 500 230 90 80 200 100 40)].
	^data! !

!WebGraph methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
data: anArray 
	data := anArray! !

!WebGraph methodsFor: 'drawing' stamp: ' 21/4/07 22:07'!
displayOn: aGraphicsContext
	| rectangle |
	self initGraphics.
	rectangle := Rectangle origin: 2 @ 2 extent: (self width - 4) @ (self height - 4).
	self graphics add: rectangle asStroker.
	self drawAxle.
	self drawYPoints.
	self graph.
	self drawTitle.
	self graphics displayOn: aGraphicsContext.! !

!WebGraph methodsFor: 'constants' stamp: ' 21/4/07 22:07'!
distance
	^(self xRightPosition - self xLeftPosition) // self data size! !

!WebGraph methodsFor: 'drawing' stamp: ' 21/4/07 22:07'!
drawAxle
	| points yAxis xAxis z c a grayLine blackLine b |
	z := self xLeftPosition.
	c := self yBottomPosition.
	b := self yTopPosition.
	a := self xRightPosition.
	points := List new: 2.
	points add: z @ (self yTopPosition - 10);
		add: z @ (c + 5).
	yAxis := Spline controlPoints: points.
	points := List new: 2.
	points add: (z - 5) @ c;
		add: a @ c.
	xAxis := Spline controlPoints: points.
	graphics add: yAxis asStroker;
		add: xAxis asStroker.
	b to: c - ((c - b) // 5)
		by: (c - b) // 5
		do: 
			[:x | 
			points := List new: 2.
			points add: (z + 1) @ x;
				add: a @ x.
			grayLine := Spline controlPoints: points.
			graphics add: ((GraphicsAttributesWrapper on: grayLine asStroker)
						attributes: (GraphicsAttributes new paint: ColorValue veryLightGray)).
			points := List new: 2.
			points add: (z - 5) @ x;
				add: z @ x.
			blackLine := Spline controlPoints: points.
			graphics add: blackLine asStroker]! !

!WebGraph methodsFor: 'drawing' stamp: ' 21/4/07 22:07'!
drawGraphWindow
	"WebGraph new drawGraphWindow"
	| win rectangle |
	win := ScheduledWindow new.
	win label: 'Graf obiskov na spletno stran'.
	win minimumSize: self width @ self height.
	graphics := CompositePart new.
	rectangle := Rectangle origin: 2 @ 2 extent: (self width - 4) @ (self height - 4).
	graphics add: rectangle asStroker.
	self drawAxle.
	self drawYPoints.
	self graph.
	self drawTitle.
	win component: (BoundedWrapper on: graphics).
	win open! !

!WebGraph methodsFor: 'drawing' stamp: ' 21/4/07 22:07'!
drawTitle
	| aTitle |
	aTitle:= (self title asComposedText compositionWidth: (self width-40)) centered.
	self graphics add: aTitle at: 20@10! !

!WebGraph methodsFor: 'drawing' stamp: ' 21/4/07 22:07'!
drawYPoints
	| string c stepOne step mark b |
	c := self yBottomPosition.
	b := self yTopPosition.
	stepOne := 0.
	step :=self stepFor:(self getMax at: 1).
	c to: b
		by: (0-((c - b) // 5))
		do: 
			[:x | 
			string := stepOne printString.
			mark := (string asText emphasizeAllWith: #small) asComposedText.
			mark compositionWidth: 30.
			mark rightFlush.
			graphics add: mark at: (self xLeftPosition - 40) @ ( x -13).
			stepOne := stepOne + step]! !

!WebGraph methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
getMax
	| b max1 maxim maxAt |
	max1 := 0.
	b := self data size.
	1 to: b
		do: [:x | (data at: x) > max1 ifTrue: 
					[max1 := data at: x.
					maxAt := x]].
	maxim := max1.
	^Array with: maxim with: maxAt.! !

!WebGraph methodsFor: 'drawing' stamp: ' 21/4/07 22:07'!
graph
	"WebGraph new drawGraph"

	| aData aRectangle z c |
	z := self xLeftPosition + 1.
	c := self yBottomPosition.
	1 to: self data size
		do: 
			[:x | 
			aData := data at: x.
			aRectangle := Rectangle left: z
						right: z + self distance - 2
						top: c - (self heightInPixelsFor: aData)
						bottom: c.
			z := z + self distance.
			graphics add: ((GraphicsAttributesWrapper on: aRectangle asFiller)
						attributes: (GraphicsAttributes new paint: ColorValue blue))]! !

!WebGraph methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
graphics
	graphics isNil ifTrue: [self initGraphics].
	^graphics! !

!WebGraph methodsFor: 'constants' stamp: ' 21/4/07 22:07'!
height	
	^300! !

!WebGraph methodsFor: 'transformations' stamp: ' 21/4/07 22:07'!
heightInPixelsFor:  aNumber

	^(((self yBottomPosition - self yTopPosition) / self margin) * aNumber) truncated

"
|graf|
graf := WebGraf new.
graf data: #(10 20 30 40 50).
graf heightInPixelsFor: 15.
"! !

!WebGraph methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initGraphics
	graphics := CompositePart new! !

!WebGraph methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
margin
	margin isNil ifTrue: [self margin: (self marginFor: (self getMax at: 1))].
	^margin! !

!WebGraph methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
margin: aNumber

	margin := aNumber.! !

!WebGraph methodsFor: 'transformations' stamp: ' 21/4/07 22:07'!
marginFor:  aNumber

	| margin |
	aNumber < 10 ifTrue: [^10].
	margin := 10 ** ((aNumber log: 10) truncated + 1).
	margin = (aNumber * 10) ifTrue: [^margin / 10].
	aNumber > (margin / 2) ifTrue: [^margin].
	aNumber > (margin / 4) ifTrue: [^margin / 2].
	^margin / 4.

"WebGraf new marginFor: 1001"! !

!WebGraph methodsFor: 'transformations' stamp: ' 21/4/07 22:07'!
stepFor:  aNumber

	^(self marginFor: aNumber) / 5

"WebGraf new stepFor: 1001"! !

!WebGraph methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
title 
	title isNil ifTrue: [title := 'Here comes the title'].
	^title! !

!WebGraph methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
title: aString
	title:= aString! !

!WebGraph methodsFor: 'constants' stamp: ' 21/4/07 22:07'!
width
	^500! !

!WebGraph methodsFor: 'constants' stamp: ' 21/4/07 22:07'!
xLeftPosition
	^50! !

!WebGraph methodsFor: 'constants' stamp: ' 21/4/07 22:07'!
xRightPosition
	^self width - 20! !

!WebGraph methodsFor: 'constants' stamp: ' 21/4/07 22:07'!
yBottomPosition
	^self height - 30! !

!WebGraph methodsFor: 'constants' stamp: ' 21/4/07 22:07'!
yTopPosition
	^40! !

!WebLiveImage class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
cache
	Cache isNil ifTrue: [self initCache].
	^Cache! !

!WebLiveImage class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
initCache
	Cache := Set new.! !

!WebLiveImage methodsFor: 'private-caching' stamp: ' 21/4/07 22:07'!
addToCache
	self makeRoomInCache.
	self class cache add: self.! !

!WebLiveImage methodsFor: 'private-caching' stamp: ' 21/4/07 22:07'!
cacheHysteresis
	"when removing from cache, make room for that number of entries"
	^10! !

!WebLiveImage methodsFor: 'private' stamp: ' 21/4/07 22:07'!
contentType
	"MIME type for our image"
	^'image/gif'! !

!WebLiveImage methodsFor: 'drawing' stamp: ' 21/4/07 22:07'!
displayOn: aGraphicsContext

	"default painting - some hello message. Subclasses should override for some more paintings"

	| composite text |
	composite := CompositePart new.
	text := 'This is a live GIF image' asText.
	text emphasizeAllWith: (Array with: #bold with: #color -> ColorValue blue).
	text := text asComposedText.
	composite add: text at: 100 @ (height/2).
	composite displayOn: aGraphicsContext.! !

!WebLiveImage methodsFor: 'drawing' stamp: ' 21/4/07 22:07'!
drawImage
	"it paints a drawing with a painter and convert it to GIF"
	| pixmap graphicsContext image stream |
	pixmap := Pixmap extent: self width @ self height.
	graphicsContext := pixmap graphicsContext.
	self painter displayOn: graphicsContext.
	image := pixmap asImage.
	stream := GifImageStream on: ByteArray new writeStream.
	[stream nextPutImage: image. self gif: stream contents] 
		valueNowOrOnUnwindDo: [stream close].! !

!WebLiveImage methodsFor: 'private' stamp: ' 21/4/07 22:07'!
gif
	"returns a representation of a image in GIF format"
	^gif! !

!WebLiveImage methodsFor: 'private' stamp: ' 21/4/07 22:07'!
gif: aByteArray

	"also set a refreshed timestamp"

	gif := aByteArray.
	self refreshed: Timestamp now.! !

!WebLiveImage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
height
	"height of image, default is 300 pixels"
	height isNil ifTrue: [self height: 300].
	^height! !

!WebLiveImage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
height: anInteger
	height := anInteger.! !

!WebLiveImage methodsFor: 'private' stamp: ' 21/4/07 22:07'!
isComposite
	^false! !

!WebLiveImage methodsFor: 'testing' stamp: ' 21/8/07 09:38'!
isRespondingStreamed
	"is this page able to stream HTML directly into response?"
	^false! !

!WebLiveImage methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isWebElement
	^false! !

!WebLiveImage methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isWebPage
	^true! !

!WebLiveImage methodsFor: 'private' stamp: ' 21/4/07 22:07'!
lastUsed
	"timestamp of last usage. Used for caching algorithms. Set at HTML generation"

	lastUsed isNil ifTrue: [self lastUsed: Timestamp now].
	^lastUsed! !

!WebLiveImage methodsFor: 'private' stamp: ' 21/4/07 22:07'!
lastUsed: aTimestamp
	"timestamp of last usage. Used for caching algorithms"

	lastUsed := aTimestamp! !

!WebLiveImage methodsFor: 'private-caching' stamp: ' 21/4/07 22:07'!
makeRoomInCache
	"if cache is full, remove least used images"
	| cached  toRemove |
	self class cache size < self maxCacheSize ifTrue: [^self].
	cached := (SortedCollection 
		withAll: self class cache
		sortBlock: [:a :b | a lastUsed < b lastUsed]) asOrderedCollection.
	self cacheHysteresis timesRepeat:
		[toRemove := cached first.
		toRemove removeFromCache.
		cached removeFirst]! !

!WebLiveImage methodsFor: 'private-caching' stamp: ' 21/4/07 22:07'!
maxCacheSize
	^50! !

!WebLiveImage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
painter
	"painter is an object, which actually paints an image. It must respond to message displayOn: anGraphicsContect. By default, painter is self and paints some hello message. Subclasses should override displayOn: appropriately"

	painter isNil ifTrue: [self painter: self].
	^painter! !

!WebLiveImage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
painter: anObject
	painter := anObject.! !

!WebLiveImage methodsFor: 'private' stamp: ' 21/4/07 22:07'!
preferedUrl
	| random |
	random := (Random new next * 10000) truncated printString.
	^'/image/live-', random, '.gif'! !

!WebLiveImage methodsFor: 'printing' stamp: 'janko 7/18/2008 20:31'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	"refresh gif if nesessary and put it into a HTML stream"
	Processor activeProcess priority: Processor activeProcess priority - 3.
	self refreshNeeded ifTrue: 
		[ self drawImage.
		self addToCache ].
	aStream nextPutAll: self gif asByteString.
	Processor activeProcess priority: Processor activeProcess priority + 3.
	self lastUsed: SpTimestamp now

	"	aSession server urlResolver removeObject: self "! !

!WebLiveImage methodsFor: 'printing' stamp: 'janko 7/18/2008 19:11'!
printWebPageFor: aRequest on: aSession 
	^self! !

!WebLiveImage methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
refreshNeeded
	^self gif isNil! !

!WebLiveImage methodsFor: 'private' stamp: ' 21/4/07 22:07'!
refreshed
	"timestamp of last gif creation. Used for caching algorithms. Set,when calling gif: method"

	refreshed isNil ifTrue: [self refreshed: Timestamp now].
	^refreshed! !

!WebLiveImage methodsFor: 'private' stamp: ' 21/4/07 22:07'!
refreshed: aTimestamp
	"timestamp of last gif creation. Used for caching algorithms"

	refreshed := aTimestamp! !

!WebLiveImage methodsFor: 'private-caching' stamp: ' 21/4/07 22:07'!
removeFromCache
	self class cache remove: self ifAbsent: [].
	self removeFromURLResolver.
	self gif: nil.! !

!WebLiveImage methodsFor: 'private-caching' stamp: ' 21/4/07 22:07'!
removeFromURLResolver
	self resolver notNil
		ifTrue: [self resolver removeObject: self]! !

!WebLiveImage methodsFor: 'private' stamp: ' 21/4/07 22:07'!
resolver
	^resolver! !

!WebLiveImage methodsFor: 'private' stamp: ' 21/4/07 22:07'!
resolver: anObject
	resolver := anObject! !

!WebLiveImage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
width
	"width of image, default is 500 pixels"
	width isNil ifTrue: [self width: 500].
	^width! !

!WebLiveImage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
width: anInteger
	width := anInteger.! !

WebElement subclass: #WebPage
	instanceVariableNames: 'session'
	classVariableNames: 'WellKnownPages'
	poolDictionaries: ''
	category: 'Aida-Elements'!

WebPage subclass: #WebApplication
	instanceVariableNames: 'observee view forms'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Components'!

WebApplication subclass: #VersionedExampleApp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Tests'!

!WebApplication class methodsFor: 'view, action methods' stamp: 'JM 4/26/2007 17:10'!
actionMethodForView: aViewSymbol
	"obsolete, only for WebSecurityManagerApp !!"
	| method viewSymbol |
	viewSymbol := aViewSymbol asString isEmpty 
		ifTrue: [self defaultView] ifFalse: [aViewSymbol asSymbol].
	method := self composedActionMethodNameForView: viewSymbol.
	(self canUnderstand: method) ifFalse: [^nil].
	^method! !

!WebApplication class methodsFor: 'view, action methods' stamp: 'JM 4/26/2007 17:10'!
actionMethodForView: aViewSymbol buttonName: aString
	"if no method for that button name, try to call action method without button name"
	| method viewSymbol |
	viewSymbol := aViewSymbol asString isEmpty 
		ifTrue: [self defaultView] ifFalse: [aViewSymbol asSymbol].
	method := self composedActionMethodNameForView: viewSymbol buttonName: aString.
	(self canUnderstand: method) ifTrue: [^method].
	'search' = aString ifTrue: [^#actionSearch].
	method := self composedActionMethodNameForView: viewSymbol.
	(self canUnderstand: method) ifTrue: [^method].
	^nil! !

!WebApplication class methodsFor: 'private' stamp: 'mivsek 6/13/2007 16:44'!
allComposedViewMethods
	| methods |
	methods := self selectors select: [:each | 'view*' match:each asString].
	methods := methods reject: [:each | 
		#(view view: viewAllowed viewMethodForView: ) includes: each].
	^self superclass isWebApplication 
		ifTrue: [methods addAll: self superclass allComposedViewMethods; yourself]
		ifFalse: [methods]

"WebDemoApp new class allComposedViewMethods"! !

!WebApplication class methodsFor: 'private' stamp: ' 21/4/07 22:07'!
allComposedViews
	"remove 'view' and lowercase start of remining part of method to get a view name from method"
	^self allComposedViewMethods collect: [:each | 
		((String with: (each at: 5) asLowercase), 
			(each copyReplaceFrom: 1 to: 5 with: String new)) asSymbol].

"WebDemoApp new class allComposedViews"! !

!WebApplication class methodsFor: 'accessing' stamp: 'JM 4/26/2007 17:06'!
allViews
	^self allComposedViews! !

!WebApplication class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allWebAppClasses
	"return an ordered collection with a WebApplication class all their subclasses ordered
	by levels of inheritance and by class name on the same level"
	| classes |
	classes := OrderedCollection new.
	self allWebAppClassesTo: classes.
	^classes

"WebApplication allWebAppClasses"! !

!WebApplication class methodsFor: 'private' stamp: ' 21/4/07 22:07'!
allWebAppClassesTo: aCollection 
	"return an ordered collection with a WebApplication class all their subclasses ordered
	by levels of inheritance and by class name on the same level"
	| subcls |
	aCollection add: self.
	subcls := SortedCollection withAll: self subclasses
				sortBlock: [:a :b | a name < b name].
	subcls do: [:subclass | subclass allWebAppClassesTo: aCollection]! !

!WebApplication class methodsFor: 'security' stamp: ' 21/4/07 22:07'!
allowAllUpdatesFor: aWebUserOrGroup on: anAIDASite
	"set access rigths to allow all updates on that web app for specified user or group 
	on specified server"
	self allWebAppClasses do: [:appClass |
		appClass allViews do: [:appView |
			anAIDASite securityManager
				setAccessRights: (appClass updateRightSymbolFor: appView)
				for: aWebUserOrGroup
				on: appClass name] ].

"
WebSecurityManagerApp allowAllUpdatesFor: 
	(WebSecurityManager default adminUser)
		on: AIDASite default
"! !

!WebApplication class methodsFor: 'security' stamp: ' 21/4/07 22:07'!
allowAllViewsFor: aWebUserOrGroup on: anAIDASite
	"set access rigths to allow all views on all web apps for specified user or group on specified site"
	self allWebAppClasses do: [:appClass |
		appClass allViews do: [:appView |
			anAIDASite securityManager
				setAccessRights: (appClass viewRightSymbolFor: appView)
				for: aWebUserOrGroup
				on: appClass name] ].
"
WebApplication allowAllViewsFor: 
	(WebSecurityManager default adminUser)
	on: AIDASite default
"! !

!WebApplication class methodsFor: 'private' stamp: ' 21/4/07 22:07'!
appClassLevel
	"return the level of inheritance of a class, relative to a WebApplication class"
	| level cls |
	level := 1.
	cls := self.
	[cls == WebApplication] whileFalse: 
			[level := level + 1.
			cls := cls superclass].
	^level! !

!WebApplication class methodsFor: 'private' stamp: ' 21/4/07 22:07'!
capitalizeName: aString
	"make first char uppercase"
	aString isEmpty ifTrue: [^''].
	^(String with: aString first asUppercase), (aString copyFrom: 2 to: aString size)

"WebApplication capitalizeName: 'main' "! !

!WebApplication class methodsFor: 'private' stamp: ' 21/4/07 22:07'!
composedActionMethodNameForView: aViewName
	^('action', (self capitalizeName: aViewName asString)) asSymbol

"WebApplication composedActionMethodNameForView: 'main'"! !

!WebApplication class methodsFor: 'private' stamp: ' 21/4/07 22:07'!
composedActionMethodNameForView: aViewName buttonName: aString
	^((self composedActionMethodNameForView: aViewName) asString,
		(self capitalizeName: aString)) asSymbol.

"WebApplication composedActionMethodNameForView: 'main' buttonName: 'addFolder'"! !

!WebApplication class methodsFor: 'private' stamp: ' 21/4/07 22:07'!
composedMethodNameForView: aViewName
	^('view', (self capitalizeName: aViewName asString)) asSymbol

"WebApplication new class composedMethodNameForView: 'main'"! !

!WebApplication class methodsFor: 'accessing' stamp: 'JM 4/26/2007 17:16'!
defaultView
	^'main'! !

!WebApplication class methodsFor: 'security' stamp: ' 21/4/07 22:07'!
encryptSymbolFor: aViewSymbol
	"if view does not exist, then use a default (first) view. Be sure to use a
	sublclass of WebApplication when calling that method to find the right default view"
	| view allViews|
	allViews := self allViews.
	allViews isEmpty ifTrue: [^nil].
	view := (allViews includes: aViewSymbol asSymbol)
		ifTrue: [aViewSymbol]
		ifFalse: [self defaultView].
	view := (String with: view asString first asUppercase), 
		(view asString copyFrom: 2 to: view asString size).
	^('encrypt', view asString) asSymbol! !

!WebApplication class methodsFor: 'private' stamp: ' 21/4/07 22:07'!
isWebApplication
	^true! !

!WebApplication class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new initialize! !

!WebApplication class methodsFor: 'instance creation' stamp: 'JM 4/26/2007 17:05'!
newFor: anObject on: aSession
	"guess a class from anObject class name (class name + 'Views') and create instance of it"
	| instance |
	instance := self newFromNameFor: anObject on: aSession.
	instance isNil ifTrue: [^nil].
	^instance session: aSession; postInitialize! !

!WebApplication class methodsFor: 'private' stamp: 'JM 4/25/2007 21:45'!
newFromNameFor: anObject on: aSession
	"Try to create of AnObjectApp instance if exist."
	| className class |
	className := (anObject class name asString, 'App') asSymbol.
	class := [Smalltalk at: className] on: Error do: [:ex | ^nil].
	(class allSuperclasses includes: WebApplication) ifFalse: [^nil].
	^class new observee: anObject; session: aSession; postInitialize.

"WebApplication newFromNameFor: WebDiscussions new "! !

!WebApplication class methodsFor: 'security' stamp: ' 21/4/07 22:07'!
updateRightSymbolFor: aViewSymbol
	"return a right symbol, used for setting/testing rights to update a page with specified view.
	A symbol is composed as follows: '#upd',<aViewSymbol> for expample for view #brief
	result is #updbrief"
	"if view does not exist, then use a default (first) view. Be sure to use a
	sublclass of WebApplication when calling that method to find the right defult view"
	| view allViews|
	allViews := self allViews.
	allViews isEmpty ifTrue: [^nil].
	view := (allViews includes: aViewSymbol asSymbol)
		ifTrue: [aViewSymbol]
		ifFalse: [self defaultView].
	view := (String with: view asString first asUppercase), 
		(view asString copyFrom: 2 to: view asString size).
	^('upd', view) asSymbol! !

!WebApplication class methodsFor: 'private' stamp: 'JM 4/26/2007 17:14'!
viewMethodDict
	"obsolete, just for compatibility"
	^Dictionary new.! !

!WebApplication class methodsFor: 'view, action methods' stamp: ' 21/4/07 22:07'!
viewMethodForView: aViewSymbol
	"registered views have a precedence over composed method names"
	| method viewSymbol |
	viewSymbol := aViewSymbol asString isEmpty 
		ifTrue: [self defaultView] ifFalse: [aViewSymbol asSymbol].
	method := self viewMethodDict at: viewSymbol ifAbsent: [nil]. 
	method isNil ifTrue: [method := self composedMethodNameForView: viewSymbol].
	(self canUnderstand: method) ifFalse: [^nil].
	^method

"WebDemoApp new class viewMethodForView: #imageGallery"! !

!WebApplication class methodsFor: 'security' stamp: ' 21/4/07 22:07'!
viewRightSymbolFor: aViewSymbol
	"return a right symbol, used for setting/testing rights to view a page with specified view.
	a symbol is composed as follows: '#view',<aViewSymbol> for expample for view #brief
	result is #viewbrief"
	"if view does not exist, then use a default (first) view. Be sure to use a
	sublclass of WebApplication when calling that method to find the right defult view"
	| view allViews|
	allViews := self allViews. allViews isEmpty ifTrue: [^nil].
	view := (allViews includes: aViewSymbol asSymbol)
		ifTrue: [aViewSymbol]
		ifFalse: [self defaultView].
	view := (String with: view asString first asUppercase), 
		(view asString copyFrom: 2 to: view asString size).
	^('view', view) asSymbol! !

!WebApplication methodsFor: 'private' stamp: 'janko 7/18/2008 22:19'!
acceptInputsFrom: aRequest
	"this method read form inputs (if any) and write them to the appropriate aspects of model 
	objects. You must use this method in your printWebPage method, if you want form processing 
	before printing a result web page. If a page for posted view is not current, then it is 
	regenerated before input reading"
	| postView |
	postView := aRequest postDataStringAt: WebForm idFieldName.
	postView isNil ifTrue: [^nil].
	self view asSymbol ~= postView asSymbol
		ifTrue: [self printWebView: postView].  "refresh view"
	self form acceptFormInputFrom: aRequest.
	self callActionMethodForButton: (self buttonNameFrom: aRequest)! !

!WebApplication methodsFor: 'searching' stamp: ' 21/4/07 22:07'!
actionMainSearch
	self actionSearch! !

!WebApplication methodsFor: 'searching' stamp: ' 21/4/07 22:07'!
actionSearch
	"a generic action when search button is pressed. It can be in almost every page."
	self indexApp searchString: self search. self search: ''.
	self indexApp actionMain.
	self redirectTo: self site index view: #results! !

!WebApplication methodsFor: 'private' stamp: ' 21/4/07 22:07'!
add: aWebElement
	"add to a form (for current view) actually !!"
	^self form add: aWebElement! !

!WebApplication methodsFor: 'private-ajax' stamp: 'janko 7/18/2008 22:21'!
ajaxAcceptInputFrom: aRequest on: aWebForm
	"AJAX request can post input field data too!! accept it!!"
	aWebForm ajaxAcceptFormInputFrom: aRequest! !

!WebApplication methodsFor: 'private-ajax' stamp: 'janko 7/18/2008 22:21'!
ajaxUpdateOfOld: anOldElement for: aRequest
	"update (recreate or refresh) that element"
	| parm |
	anOldElement isNil ifTrue:  [^WebElement new].
	anOldElement method isNil ifTrue: [^anOldElement]. "in case of form elements it will return new value!!"
	parm := aRequest isPost
		ifTrue: [aRequest postDataStringAt: 'parm']
		ifFalse: [aRequest queryAt: 'parm'].
	(self class canUnderstand: anOldElement method) "for elements created with App methods"
		ifTrue: [parm isNil 
			ifTrue: [^self perform: anOldElement method] "this really recreate an element"
			ifFalse: [^self perform: anOldElement method with: parm ]]. "object method: parm"
	(anOldElement class canUnderstand: anOldElement method) "for standalone components"
		ifTrue: [(anOldElement method asString last = $: )
			ifTrue: [^anOldElement perform: anOldElement method with: parm ]
			ifFalse: [^anOldElement perform: anOldElement method] ].
	^anOldElement "similar case as above"! !

!WebApplication methodsFor: 'private' stamp: 'JM 4/26/2007 14:24'!
announceAction: anActionSymbol onView: aSymbol
"	self announce: 
		(AppActionTrigered for: self observee on: self session 
			view: aSymbol action: anActionSymbol)"! !

!WebApplication methodsFor: 'private' stamp: 'JM 4/26/2007 14:05'!
announceView: aSymbol
"	self  announce: 
		(AppViewShowed for: self observee on: self session view: aSymbol)"! !

!WebApplication methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
app
	^self! !

!WebApplication methodsFor: 'private' stamp: ' 21/4/07 22:07'!
buttonNameFrom: aRequest
	"return a name of button pressed. Name defines an action to be called"
	| buttonNames |
	buttonNames := (self form fieldSet select: [:each | each isButton]) collect: [:each | each name].
	^buttonNames detect: [:each | aRequest postDataKeys includes: each] ifNone: ['']! !

!WebApplication methodsFor: 'private' stamp: ' 21/4/07 22:07'!
callActionMethodForButton: aString
	"name of button pressed is in argument"
	| methd |
	methd := self class actionMethodForView: self view buttonName: aString. 
	methd notNil ifTrue:
		[(methd asString last = $: )  "obsolete methods have session for argument"
			ifTrue: [self perform: methd asSymbol with: self session]
			ifFalse: [self perform: methd asSymbol] ].
	self announceAction: aString asSymbol onView: self view! !

!WebApplication methodsFor: 'private-versions' stamp: 'janko 7/18/2008 22:22'!
changeObserveeToVersionFor: aRequest
	| object |
	object := self observee versionWithNumber: (self versionFromRequest: aRequest).
	object notNil ifTrue: [self observee: object]! !

!WebApplication methodsFor: 'private-multilingual' stamp: 'janko 7/18/2008 22:24'!
checkObserveeLanguageFor: aRequest
	"change observee to point to domain object in right language if necessary 
	(from session language or explicit language request in query part of Url)"
	| language new |
	language := (self isLanguageRequest: aRequest) "language= in query part of url"
		ifTrue: [(aRequest queryAt: 'language') asSymbol]
		ifFalse: [self session language].
	self observee language = language ifFalse: 
		[new := self observee forLanguage: language.
		new notNil ifTrue: [self observee: new] ].! !

!WebApplication methodsFor: 'private-versions' stamp: 'janko 7/18/2008 22:25'!
checkObserveeVersionFor: aRequest
	"change observee to point to correct version if necessary"
	(self isVersionRequest: aRequest) ifTrue: [^self changeObserveeToVersionFor: aRequest].
	self observee isCurrentVersion ifFalse: [self observee: self observee currentVersion].! !

!WebApplication methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
cleanup
	"clear all elements and web table info, when page is not needed anymore"
	self initScript. self initAttributes.
	self clearTableInfo.
	self form 
		initElements; 
		clearTableInfo.
	self initPageHeaderLinks.! !

!WebApplication methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
clear
	"delete all elements of a WebPage for this WebApplication. It clears all in a form for current view."
	self form clear.
	self form initIdField.! !

!WebApplication methodsFor: 'clipboard' stamp: ' 21/4/07 22:07'!
clipboard
	^self session clipboard! !

!WebApplication methodsFor: 'clipboard' stamp: ' 21/4/07 22:07'!
copyToClipboard
	"copy an url and title of current view of observee object to web clipboard"
	| object |
	object := self observee isVersionedObject
		ifTrue: [self observee currentVersion] "so that link will be aways same regardles of version!!"
		ifFalse: [self observee].
	title := (object class canUnderstand: #id) ifTrue: [object id, ' '] ifFalse: [''].
	title := (object class canUnderstand: #title) ifTrue: [title, object title] ifFalse: [nil].
	title isNil ifTrue: 
		[title := (object class canUnderstand: #indexTitle) ifTrue: [object indexTitle] ].
	title isNil ifTrue: [title := self title].
	self clipboard 
		title: title;
		url: (self site urlResolver halfUrlFor: object);
		object: object! !

!WebApplication methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
defaultView
	"default view is first view in method registerViews. If none, then it is 'main' "
	^self class defaultView! !

!WebApplication methodsFor: 'private' stamp: ' 21/4/07 22:07'!
doesNotUnderstand: aMessage
	"forward a message to a style for processing"
	^self style
		perform: aMessage selector
		withArguments: aMessage arguments! !

!WebApplication methodsFor: 'accessing-other' stamp: ' 21/4/07 22:07'!
error
	^self otherAt: #Error! !

!WebApplication methodsFor: 'accessing-other' stamp: ' 21/4/07 22:07'!
error: aString
	"error message to be written on web page. "
	self otherAt: #Error put: aString! !

!WebApplication methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
errorReport
	"error report, if any"
	| e |
	e := WebElement newId: #error.
	self error notNil ifTrue: [e addText: self error style: '{color: red}'].
	self error: nil.
	^e! !

!WebApplication methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
form
	"a form for current view"
	^self formForView: self view.! !

!WebApplication methodsFor: 'private-forms' stamp: ' 21/4/07 22:07'!
formForView: aViewSymbol
	| viewName |
	viewName := aViewSymbol asSymbol.
	viewName = #'' ifTrue: [viewName := #main].
	^(self forms at: viewName ifAbsentPut: 
		[AIDA.WebForm new parent: self; view: viewName])! !

!WebApplication methodsFor: 'private-forms' stamp: ' 21/4/07 22:07'!
forms
	"a dictionary of forms for each view"
	forms isNil ifTrue: [self initForms].
	^forms! !

!WebApplication methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
helpLink
	"link to help page for current view and App. If not exist, then nothing, exept for 
	admin - link to creation of new help page"
	| link viewName |
	self class == WebHelpPageApp ifTrue: [^WebElement new].
	viewName := self inDefaultView ifTrue: ['main'] ifFalse: [self view asString].
	link := WebLink newClass: #helpLink.
	(self site help existApp: self view: viewName)
		ifTrue: [link text: ('<b>', self style helpText, '</b>') 
			linkTo: (self site help forApp: self view: viewName)]
		ifFalse: [self user isAdmin
			ifTrue: [link text: '<b>', self style addHelpText, '</b>' linkTo: self site help; view: 'newPage'; 
				parameter: 'app' value: self class name asString;
				parameter: 'hview' value: viewName]
			ifFalse: [^WebElement new] ].
	^link! !

!WebApplication methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
inDefaultView
	^(self view = self defaultView asSymbol) | (self view = #'')! !

!WebApplication methodsFor: 'accessing-modes' stamp: ' 21/4/07 22:07'!
inEditMode
	^self mode = #edit! !

!WebApplication methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
inError
	^self error notNil and: [self error notEmpty]! !

!WebApplication methodsFor: 'accessing-modes' stamp: ' 21/4/07 22:07'!
inViewMode
	^self mode = #view! !

!WebApplication methodsFor: 'searching' stamp: ' 21/4/07 22:07'!
indexApp
	^self site index webAppFor: self session! !

!WebApplication methodsFor: 'searching' stamp: ' 21/4/07 22:07'!
indexObservee
	"index or reindex observee of that app"
	^self site index indexObject: self observee! !

!WebApplication methodsFor: 'security' stamp: 'mivsek 6/13/2007 16:15'!
initAdminAccess
	"admin group has rights to all views and updates in all Apps!! "
	self site securityManager initAdminAccess.! !

!WebApplication methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initForms
	forms := Dictionary new.! !

!WebApplication methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	super clear.  "clear all in case of cascading inits"
	self initAttributes.
	self bgColor: #ffffff.
	self initForms.
	"self setViewMode."! !

!WebApplication methodsFor: 'private-bpm' stamp: ' 21/4/07 22:07'!
isControledByBPMProcess
	"by a BPM process instance, which control a workflow of this app too"
	^self process notNil! !

!WebApplication methodsFor: 'security' stamp: ' 21/4/07 22:07'!
isEncryptionSatisfied
	^(self mustBeEncrypted: self session newView) and: [self session lastRequest isEncrypted not]! !

!WebApplication methodsFor: 'private-multilingual' stamp: 'janko 7/18/2008 22:26'!
isLanguageRequest: aRequest
	^(aRequest queryAt: 'language') notNil! !

!WebApplication methodsFor: 'private-versions' stamp: ' 21/4/07 22:07'!
isObserveeVersioned
	^self observee isVersionedObject! !

!WebApplication methodsFor: 'private-versions' stamp: 'janko 7/18/2008 22:26'!
isVersionRequest: aRequest
	^(self versionFromRequest: aRequest) notNil! !

!WebApplication methodsFor: 'private' stamp: ' 21/4/07 22:07'!
isWebApplication
	^true! !

!WebApplication methodsFor: 'accessing-modes' stamp: ' 21/4/07 22:07'!
mode
	| mode |
	mode := self otherAt: #Mode.
	^mode isNil 
		ifTrue: [self setViewMode. self mode] 
		ifFalse: [mode]! !

!WebApplication methodsFor: 'private' stamp: ' 21/4/07 22:07'!
mode: aSymbol
	"portlet modes #view #edit "
	self otherAt: #Mode put: aSymbol! !

!WebApplication methodsFor: 'security' stamp: ' 21/4/07 22:07'!
mustBeEncrypted
	"check  if current view must be sent encrypted"
	^self mustBeEncrypted: self view! !

!WebApplication methodsFor: 'security' stamp: ' 21/4/07 22:07'!
mustBeEncrypted: aViewSymbol
	"check  if this view must be sent encrypted"
	| right |
	(self observee = self site admin) & (aViewSymbol = #login) ifTrue: [^true].
	 right := self site securityManager 
		isUser: self user
		allowedTo: (self class encryptSymbolFor: aViewSymbol)
		on: self class name.
	^right! !

!WebApplication methodsFor: 'redirection' stamp: ' 21/4/07 22:07'!
newView: aSymbol
	"in action methods, to direct to a specified view after button pressed"
	self session newView: aSymbol! !

!WebApplication methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
observee
	"return reference to an object, for which this app acts as an observer - to make an user interface
	of them"
	^observee! !

!WebApplication methodsFor: 'private' stamp: ' 21/4/07 22:07'!
observee: anObject
	"set the reference to an object, for which this app acts as an observer - to make an user interface
	of them"
	observee := anObject! !

!WebApplication methodsFor: 'private-printing' stamp: 'np 4/1/2008 21:45'!
pageFrameWith: anElement title: aString
	^self style pageFrameWith: anElement title: aString! !

!WebApplication methodsFor: 'clipboard' stamp: ' 21/4/07 22:07'!
pasteFromClipboard
	"get a DocLink with title and url. Nil if clipboard empty"
	self clipboard isEmpty ifTrue: [^nil].
	^self clipboard asDocLink! !

!WebApplication methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
performViewMethod: aSymbol
 	"obsolete method calls have session for argument!!"
	^(aSymbol asString last = $: )
		ifTrue: [self perform: aSymbol asSymbol with: self session]
		ifFalse: [self perform: aSymbol asSymbol]! !

!WebApplication methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
printCurrentWebView
	"build a web page for a view in a view attributem, that is a current view"
	| printView |
	printView := (self class viewMethodForView: self view) notNil
		ifTrue: [self view asSymbol]
		ifFalse: [self class defaultView asSymbol].
	self session lastView: printView.
	^self printWebView: printView.! !

!WebApplication methodsFor: 'private-printing' stamp: 'janko 7/18/2008 22:29'!
printWebPageFor: aRequest 
	| toLogin |
	self observee isMultilingual 
		ifTrue: [self checkObserveeLanguageFor: aRequest].  "and change observee if needed"
	self isObserveeVersioned 
		ifTrue: [self checkObserveeVersionFor: aRequest]. "and change observee if needed"
	aRequest isPost 
		ifTrue: [self form registerStreamedFieldsInto: aRequest].
	toLogin := false. "self isEncryptionSatisfied ifTrue: [toLogin := true]."
	(aRequest isPost and: [aRequest isAjaxRequest not]) ifTrue: 
		[self updateAllowed ifTrue: [self acceptInputsFrom: aRequest] ifFalse: [toLogin := true] ].
	toLogin ifTrue:
		[(self observee == self site admin and: [self session newView = 'login']) 
			ifFalse: [self redirectTo: self site admin view: #login].
		^WebPage new].
	self session lastApp: self.
 	aRequest isAjaxRequest ifTrue: [^self respondToAjaxRequest: aRequest].
	self view: self session newView.
	self session redirectLink notNil ifTrue: [^WebPage new].
	self setProcessFromTaskId. "for BPM engine, if present"
	^self printCurrentWebView.! !

!WebApplication methodsFor: 'private-printing' stamp: 'janko 7/18/2008 22:29'!
printWebPageFor: aRequest on: aSession 
	"OBSOLETE!!"
	^self printWebPageFor: aRequest! !

!WebApplication methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
printWebView: aViewName
 	"build a web page for that view"
	| method page |
	self initPageHeaderLinks.
	method := self class viewMethodForView: aViewName. 
	method isNil ifTrue: [^self clear; addText: 'ERROR: view named ', aViewName printString, 
		' does not exist' header: 3].
	self clear. self view: aViewName.
	^self viewAllowed 
		ifTrue: [page := self performViewMethod: method.
			page isWebApplication ifTrue: [page form registerFormElementsIn: page form].
			self storeThisUrl.
			self announceView: aViewName.
			page]
		ifFalse: [self redirectTo: self site admin view: #login.
			WebPage new].! !

!WebApplication methodsFor: 'accessing-other' stamp: ' 21/4/07 22:07'!
process
	"a BPM.Process instance, on which this app is activated by some task"
	^self otherAt: #Process! !

!WebApplication methodsFor: 'accessing-other' stamp: ' 21/4/07 22:07'!
process: aBPMProcess
	self other at: #Process put: aBPMProcess! !

!WebApplication methodsFor: 'redirection' stamp: ' 21/4/07 22:07'!
redirectTo: anObjectOrUrlString
	"redirect browser to a default view for specified object or url link"
	self session redirectLink: (WebLink text:'' linkTo: anObjectOrUrlString).! !

!WebApplication methodsFor: 'redirection' stamp: ' 21/4/07 22:07'!
redirectTo: anObjectOrUrlString view: aSymbol
	"redirect browser to a specified view for specified object or url link"
	self session redirectLink: (WebLink text:'' linkTo: anObjectOrUrlString view: aSymbol).! !

!WebApplication methodsFor: 'redirection' stamp: ' 21/4/07 22:07'!
redirectTo: anObjectOrUrlString view: aSymbol parameter: aParmString value: aValueString
	"redirect browser to a specified view for specified object or url link"
	self session redirectLink: ((WebLink text:'' linkTo: anObjectOrUrlString view: aSymbol)
		 parameter: aParmString value: aValueString).! !

!WebApplication methodsFor: 'redirection' stamp: ' 21/4/07 22:07'!
redirectToView: aSymbol
	"redirect browser to a different view for same observee"
	self session redirectLink: (WebLink text:'' linkTo: self observee view: aSymbol).! !

!WebApplication methodsFor: 'private-ajax' stamp: 'janko 7/18/2008 22:31'!
respondToAjaxAutocomplete: aRequest on: aWebForm
	| fieldName fieldValue field choices e |
	fieldName := aRequest postDataKeys detect: [:each | 'field*' match: each] ifNone: nil.
	fieldValue := (aRequest postDataAt: fieldName) value.
	fieldValue := AIDASite convert: fieldValue fromCodepage: #utf8.
	field := aWebForm fields at: fieldName. 
	choices := field getChoicesForEntry: fieldValue.
	e := WebList newUnordered.
	choices do: [:each || string |
		string := each convertToSloveneChars.
		e add: (WebRawText text: (AIDASite convert: string toCodepage: #utf8))].
	^e! !

!WebApplication methodsFor: 'private-ajax' stamp: 'janko 7/18/2008 22:32'!
respondToAjaxInPlaceEditing: aRequest on: aWebForm
	| fieldValue field response |
	field := aWebForm elementId: aRequest ajaxElementId.
	field isFormated & aRequest isAjaxWikiFormatedRequest not  ifFalse:
		[fieldValue := (aRequest postDataAt: 'value') value.
		fieldValue := AIDASite convert: fieldValue fromCodepage: #utf8.
		field allow ifTrue: [field object perform: (field aspect asString, ':') asSymbol with: fieldValue] ].
	response := (field object perform: field aspect).
	aRequest isAjaxWikiFormatedRequest 
		ifTrue: [response := response asWikiHtml].
	response := AIDASite convert: response toCodepage: #utf8.
	^WebElement new add: (WebRawText text: response); yourself! !

!WebApplication methodsFor: 'private-ajax' stamp: 'janko 7/18/2008 22:32'!
respondToAjaxRequest: aRequest
	| oldElement ajaxForm e |
	ajaxForm := self formForView: aRequest view asSymbol.
	oldElement := ajaxForm elementId: aRequest ajaxElementId.
	aRequest isAjaxInPlaceEditingRequest 
		ifTrue: [^self respondToAjaxInPlaceEditing: aRequest on: ajaxForm].
	aRequest isAjaxPostWithInput ifTrue: [self ajaxAcceptInputFrom: aRequest on: ajaxForm].
	aRequest isAjaxAutocompleteRequest 
		ifTrue: [^self respondToAjaxAutocomplete: aRequest on: ajaxForm].
	e := self ajaxUpdateOfOld: oldElement for: aRequest. "update (recreate or refresh) that element"
	e adaptFormElements.
	ajaxForm registerFormElementsIn: e.
	e prepareToHTMLPrintOn: self session. "to prepare again"
	^e elements "because you need to send inner html of element only, not its tags"! !

!WebApplication methodsFor: 'accessing-other' stamp: ' 21/4/07 22:07'!
search
	^self otherAt: #Search! !

!WebApplication methodsFor: 'accessing-other' stamp: ' 21/4/07 22:07'!
search: aString
	"search input field"
	self otherAt: #Search put: aString! !

!WebApplication methodsFor: 'accessing-modes' stamp: ' 21/4/07 22:07'!
setEditMode
	self mode: #edit! !

!WebApplication methodsFor: 'private-bpm' stamp: ' 21/4/07 22:07'!
setProcessFromTaskId
	"set a reference to a BPM process instance, which control a workflow of this app too"
	| uuid workitem |
	uuid := self session lastRequest queryAt: 'taskId' ifAbsent: [^nil].
	workitem := self site repository bpm worklist itemWithTaskUuid: uuid for: self user.
	workitem isNil ifTrue: [^nil].
	self process: workitem task process.
	workitem subscribeToApp: self.! !

!WebApplication methodsFor: 'accessing-modes' stamp: ' 21/4/07 22:07'!
setViewMode
	self mode: #view! !

!WebApplication methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
site
	"reference to a site on which this app shows some object. "
	^self session site! !

!WebApplication methodsFor: 'statistics' stamp: ' 21/4/07 22:07'!
statisticsPageLink
	"returns a link to a page with day/hour statistic graphs of visits"
	| url link |
	url := self site urlResolver halfUrlFor: self observee.
	link := WebLink linkTo: self site statistics.
	link 
		view: 'page';
		parameter: 'url' value: url.
	^link! !

!WebApplication methodsFor: 'private-printing' stamp: ' 21/4/07 22:07'!
storeThisUrl
	"to a User. It is helpull for return after logon, for instance"
	self user lastAppUrl: self session lastRequest uriString! !

!WebApplication methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
style
	^self site style! !

!WebApplication methodsFor: 'security' stamp: 'JM 4/26/2007 17:44'!
updateAllowed
	"check if update is allowed to this observee object with specified view"
	"update for admin login page is always allowed"
	"reregister rights for admin if denial is encountered "
	| right |
	(self observee = self site admin) & (self view = #login) ifTrue: [^true].
	 right := self site securityManager 
		isUser: self user allowedTo: (self class updateRightSymbolFor: self view)
		on: self class name.
	(right not and: [self session adminAllowed]) 
		ifTrue: [self initAdminAccess. ^true].
	^right! !

!WebApplication methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
user
	"reference to a user of current session "
	^self session user! !

!WebApplication methodsFor: 'private-versions' stamp: 'janko 7/18/2008 22:33'!
versionFromRequest: aRequest
	^aRequest queryAt: 'version'! !

!WebApplication methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
view
	"name of a view to the object, currently/last generated. This name is included into 
	hidden field named 'id5273', which is used to identify correct form when it is posted 
	by the web user"
	^view isNil ifTrue: [''] ifFalse: [view]! !

!WebApplication methodsFor: 'private' stamp: ' 21/4/07 22:07'!
view: aString
	view := aString asSymbol.
	view = #'' ifTrue: [view := #main].! !

!WebApplication methodsFor: 'security' stamp: 'JM 4/26/2007 17:44'!
viewAllowed
	"check if page view is allowed to this observee object with specified view"
	"admin logon page always allowed"
	"reregister rights for admin if denial is encountered "
	| right |
	(self observee = self site admin) & (self view = #login) ifTrue: [^true].
	 right := self site securityManager 
		isUser: self user allowedTo: (self class viewRightSymbolFor: self view)
		on: self class name.
	(right not and: [self session adminAllowed]) 
		ifTrue: [self initAdminAccess. ^true].
	^right! !

!WebApplication methodsFor: 'statistics' stamp: ' 21/4/07 22:07'!
visitsCount
	"returns number of visits of this page"
	^self webCounter total! !

!WebApplication methodsFor: 'statistics' stamp: ' 21/4/07 22:07'!
webCounter
	"returns a counter which counts visits to this page"
	^self site urlResolver
		counterFor: self observee! !

WebApplication subclass: #WebFrameApp
	instanceVariableNames: 'date'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Components'!

WebFrameApp subclass: #WebAdminApp
	instanceVariableNames: 'username password newUser newSite'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Admin'!

!WebAdminApp methodsFor: 'actions' stamp: 'mivsek 6/13/2007 16:33'!
actionAddSite
	SwazooServer singleton addSite: self newSite.
	self newSite start.
	self redirectToView: #sites.! !

!WebAdminApp methodsFor: 'actions' stamp: ' 21/4/07 22:07'!
actionForgoten
	self redirectTo: self site admin view: #passwordSent.! !

!WebAdminApp methodsFor: 'actions' stamp: 'janko 3/16/2008 13:23'!
actionLogin
	| user |
	user := self findLoggingUser.
	self password: ''.
	user notNil 
		ifTrue: [self session user: user. self reportLogin] 	
		ifFalse: [self error: 'error message from WebStyle!!'. self session logout. ^self].
	self site afterLogin = #lastPage ifFalse: [^self redirectTo: self site afterLogin]. 
	(user logoutFromUrl notNil and: [self site urlResolver existURL: user logoutFromUrl])
		ifTrue: [self redirectTo: user logoutFromUrl] "back from where we were logout"
		ifFalse: [self redirectTo: '/']
"		ifFalse: [self redirectTo: (user asPerson notNil ifTrue: [user asPerson] ifFalse: ['/']) ] "! !

!WebAdminApp methodsFor: 'actions' stamp: ' 21/4/07 22:07'!
actionRegistration
	self isRegistrationValid 
		ifTrue: 
			[self site securityManager addActivatingUser: self newUser.
			self newView: #waitingConfirmation]
		ifFalse: ["just reload registration form again"]! !

!WebAdminApp methodsFor: 'actions' stamp: 'mivsek 6/13/2007 16:34'!
actionSites
	self newSite: AIDASite new.
	self newSite uriPattern first 
		ip: self observee ip;
		port: self observee port.
	self redirectToView: #addSite.! !

!WebAdminApp methodsFor: 'actions' stamp: ' 21/4/07 22:07'!
actionUpdateSettings
	[(Delay forMilliseconds: 500) wait.
	self site stop.
	self updateSettings.
	self site start] fork.
	self redirectToView: #redirect! !

!WebAdminApp methodsFor: 'printing-elements' stamp: 'mivsek 6/13/2007 16:36'!
addSiteElement
	| e |
	e := WebElement new.
	e cell addText: 'Site name: '. 
	e newCell addInputFieldAspect: #name for: self newSite. e newRow.
	e cell colspan: 2; addRulerSize: 1. e newRow.
	e cell addText: 'Hostname: '. 
	e newCell addInputFieldAspect: #host for: self newSite uriPattern first. e newRow.
	e cell addText: 'IP: '. 
	e newCell addInputFieldAspect: #ip for: self newSite uriPattern first. e newRow.
	e cell addText: 'Port: '. 
	e newCell addInputFieldAspect: #port for: self newSite uriPattern first. e newRow.
	e cell colspan: 2; addRulerSize: 1. e newRow.
	e cell addText: 'Style class: '. 
	e newCell addInputFieldAspect: #styleClass for: self newSite. e newRow.
	e cell colspan: 2; addRulerSize: 1. e newRow.
	e cell addButtonText: 'Add and start a new site'.
	e cell colspan: 2; addRulerSize: 1. e newRow.
	^e.! !

!WebAdminApp methodsFor: 'private' stamp: ' 21/4/07 22:07'!
aidaDontCache
	^true! !

!WebAdminApp methodsFor: 'private' stamp: 'janko 7/18/2008 16:37'!
dummy
	"aspect, just for nothing"
	^''! !

!WebAdminApp methodsFor: 'private' stamp: 'janko 7/18/2008 16:38'!
dummy: aString
	"aspect, just for nothing"! !

!WebAdminApp methodsFor: 'actions' stamp: 'mivsek 1/10/2008 22:48'!
findLoggingUser
	^self site securityManager userNamed: self username withPassword: self password.
! !

!WebAdminApp methodsFor: 'testing' stamp: 'np 4/6/2008 15:48'!
isRegistrationValid
	"check entry fields and set error report if not"
	| text |
	text := ''.
	(self site securityManager 
		existUserNamed: self newUser username withPassword: self newUser username)
			ifTrue: [text := text, ' User with that username and password already exist!! ' ].
	text isEmpty ifTrue: 
		[self newUser username isEmpty ifTrue: [text := text, ' Username missing, '].
		self password isEmpty ifTrue: [text := text, ' Password missing, '].
		self newUser password isEmpty ifTrue: [text := text, ' Confirm password missing, '].
		((WebSecurityManager hashPassword: self password) = self newUser password) ifFalse: [text := text, ' Passwords not equal!! '] ].
	text notEmpty ifTrue: 
		[self error: 'Registration failed!! ', text, ' Please correct errors and try again!!'. ^false].
	^true! !

!WebAdminApp methodsFor: 'accessing' stamp: 'mivsek 6/13/2007 16:35'!
newSite
	^newSite! !

!WebAdminApp methodsFor: 'accessing' stamp: 'mivsek 6/13/2007 16:35'!
newSite: anObject
	newSite := anObject! !

!WebAdminApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
newUser
	^newUser! !

!WebAdminApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
newUser: anUser
	newUser := anUser! !

!WebAdminApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
password
	password isNil ifTrue: [self password: ''].
	^password! !

!WebAdminApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
password: aString
	password := aString.! !

!WebAdminApp methodsFor: 'testing' stamp: 'mivsek 1/10/2008 22:48'!
preloginCheck
	"this is called at the start of login page. Override if you want to redirect for some reason
	to another login page (#viewLoginSecondary)"
	^true! !

!WebAdminApp methodsFor: 'actions' stamp: ' 21/4/07 22:07'!
prepareSettings
	self observee host: self site host.
	self observee ip: self site ip.
	self observee port: self site port.! !

!WebAdminApp methodsFor: 'private' stamp: ' 21/4/07 22:07'!
reportLogin
	Transcript cr; show: 'login: ', self session user nameSurname asSloveneWithoutCircumflexes, 
	' on ', self site name,
	' at ', 
 	Date today dayOfMonth printString, '.',
	Date today monthIndex printString, ' ',
	Time now hours printString, ':', 
	Time now minutes printString,
	' from ', self session lastRequest peer! !

!WebAdminApp methodsFor: 'private' stamp: ' 21/4/07 22:07'!
reportLogout
	Transcript cr; show: 'logout: ', self session user nameSurname asSloveneWithoutCircumflexes, 
	' on ', self site name,
	' at ', 
 	Date today dayOfMonth printString, '.',
	Date today monthIndex printString, ' ',
	Time now hours printString, ':', 
	Time now minutes printString,
	' from ', self session lastRequest peer! !

!WebAdminApp methodsFor: 'printing-elements' stamp: ' 21/4/07 22:07'!
settingsElementForm: aBoolean
	| element |
	self prepareSettings.
	element := WebElement new.
	element table width: 500.
	element newRow.
	element cell addText: 'hostname: '.
	element newCell addAspect: #host for: self observee input: aBoolean size: 30.
	element newRow.
	element cell addText: 'ip: '.
	element newCell addAspect: #ip for: self observee input: aBoolean size: 30.
	element newRow.
	element cell addText: 'port: '.
	element newCell addAspect: #port for: self observee input: aBoolean size: 5.
	element newRow.
	element cell addText: 'home directory: '.
	element newCell addAspect: #homeDirectory for: self site input: aBoolean size: 30.
	element newRow.
"
	element cell addText: 'web server e-mail address: '.
	element newCell addAspect: #webServerEMail for: self site input: aBoolean size: 30.
	element newRow.
	element cell addText: 'e-mail SMTP server: '.
	element newCell addAspect: #smtpServer for: self site input: aBoolean size: 30.
	element newRow.
	element cell addText: 'urgent notification e-mail to: '.
	element newCell addAspect: #urgentNotificationEMail for: self site input: aBoolean size: 30.
	element newRow.
"
	element cell colspan: 2; addRulerSize: 1.

	^element.! !

!WebAdminApp methodsFor: 'printing-elements' stamp: 'mivsek 6/13/2007 16:36'!
sitesElement
	| e |
	e := (WebGrid new
		columnFilters: #(nil 5 );
		columnWidth: #(0.03 0.15);
		columnNames: #(nil 'name' 'created' 'started' 'status');
		column: 2 addBlock: [:site | WebLink text: site name linkTo: 'http://', site host, '/admin.html'];
		collection: SwazooServer singleton sites;
		setNumbering; 
		yourself).
	^e.! !

!WebAdminApp methodsFor: 'actions' stamp: ' 21/4/07 22:07'!
updateSettings
	self site host: self observee host.
	self site ip: self observee ip.
	self site port: self observee port.! !

!WebAdminApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
username
	username isNil ifTrue: [self username: ''].
	^username! !

!WebAdminApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
username: aString
	username := aString.! !

!WebAdminApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewActivation
	| e id user |
	(self session lastRequest includesQuery: 'userid') ifTrue:
		[id := (self session lastRequest queryAt: 'userid') asInteger.
		(self site securityManager existUserWithId: id) ifTrue:
			[user := self site securityManager userWithId: id.
			 self site securityManager addRegisteredUser: user.
			self session loginUser: user.
			self clear. self title: 'Activation successfull!!'.
			e := WebElement new.
			e addTextH1: 'Activation successfull!!'.
			e addText: 'Congratulations!! You are now a registered user of our portal. You are 
			already logged in and your name is shown on the page'. 
 			^self pageFrameWith: e title: self title] ].

	self clear. self title: 'Activation failed!!'.
	e := WebElement new.
	e addTextH1: 'Activation failed!!'.
	e addText: ' You tried to activate an user account but activation failed. Please send an email to '. 
	e addLinkTo: 'info@eranova.si' text: 'Portal Administrator'.
 	^self pageFrameWith: e title: self title .! !

!WebAdminApp methodsFor: 'printing' stamp: 'mivsek 6/13/2007 16:36'!
viewAddSite
	| e |
	self	title: 'Web sites'.
	e := WebElement new.
	e addTextH1: 'Adding a new web site'. e addBreak. 
	e add: self addSiteElement.
	^self pageFrameWith: e title: self title .! !

!WebAdminApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewForgoten
	| e f |
	self clear. self title: 'Lost Password'.
	e := WebElement new.
	e addTextH1: 'Lost Password'.
	e addText: ' Enter your username below, click Send me my password, and your password 
	will be mailed to you if you gave a valid email address when you registered. If this will not 
	work for you (for example, if you forgot your member name or didn''t enter your email address) 
	send an email to '. 
	e addLinkTo: 'info@eranova.si' text: 'Portal Administrator'.
	f := WebFieldSet newLegend: 'User details'.
	f cell addText: 'My username is: '.
	f newCell add: (WebInputField new aspect: #username for: self).
	f newRow. f newCell addButtonText: 'Send me my password'. 
	e add: f.
 	^self pageFrameWith: e title: self title .! !

!WebAdminApp methodsFor: 'printing' stamp: 'mivsek 1/11/2008 12:32'!
viewLogin
	| e l |
	self preloginCheck ifFalse: [^self viewLoginSecondary].  "if some error etc."
	self clear. self title: self style loginTitle.
	e := WebElement new.
	e scriptBefore: 'Element.hide(''', self searchButton id, ''')'. "remove search button on navbar!!"
	e addText: self style loginWelcomeMessage. 
	l := (WebFieldSet newId: #login) legend: self style loginText.
	self inError ifTrue: 
		[l cell colspan: 2; addText: self style loginErrorText. self error: nil.
		l newRow. l cell addNbSp. l newRow].
	l cell addText: self style usernameText. 
	l newCell add: (WebInputField new aspect: #username for: self;
		focus). "let text cursor be there at the start!! "
	l newRow. l cell addText: self style passwordText. 
	l newCell add: (WebInputField new type: #password; aspect: #password for: self).
	l newRow. l newCell addButtonText: self style loginButton.
	e add: l.
	e addText: self style loginBelowMessage.
" 
	e newRow. e newCell addGif: #bulletSquareLeftWhiteGif ; 
		addLinkTo: self observee text: ' Forgot your password ?' view: 'forgoten'.
	e newRow. e newCell addGif: #bulletSquareLeftWhiteGif ; 
		addLinkTo: self observee text: ' New user ?' view: 'registration'.
"
 	^self pageFrameWith: e title: self title .! !

!WebAdminApp methodsFor: 'printing' stamp: 'mivsek 1/10/2008 22:50'!
viewLoginSecondary
	"this one is called if preloginCheck returns false. Usefull for prelogin error reporting etc.
	Byy thefault it returns error report in red. Override that by your wishes"
	| e |
	self clear. self title: self style loginTitle.
	e := WebElement new.
	self error: self style loginErrorTextSecondary.
	e add: self errorReport.
 	^self pageFrameWith: e title: self title .! !

!WebAdminApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewLogout
	self reportLogout.
	self session logout.
	self username: ''. self password: ''.
	self site securityManager hasFormAuthenticationScheme ifTrue: [^self redirectToView: #login].
	self site securityManager hasHttpAuthenticationScheme ifTrue: [^self redirectTo: '/'].! !

!WebAdminApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewMain
	| element |
	self clear.
	self	title: 'Admin'.
	element := WebElement new.
	^self pageFrameWith: element title: self title .! !

!WebAdminApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewPasswordSent
	| e |
	self clear. self title: 'Lost Password sent'.
	e := WebElement new.
	e addTextH1: 'Password sent'.
	e addText: ' Your password was sent to email address from your user profile. If you have 
      any more troble, please send an email to '. 
	e addLinkTo: 'info@eranova.si' text: 'Portal Administrator'.
 	^self pageFrameWith: e title: self title .! !

!WebAdminApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewRedirect
	| e url |
	self title: 'Restarting a site'.
	e := WebElement new.
	e addText: 'Restarting a site with new settings ...'.
	url := 'http://', self observee host, ':', self observee port printString, '/admin.html?view=settings'.
" this don't work on mozilla!!
	self redirectAfter: 2 toUrl: url.
"
self script: ('<SCRIPT LANGUAGE="JavaScript">
<!!-- Begin
redirTime = "2000";
redirURL = "', url, '";
function redirTimer() { self.setTimeout("self.location.href = redirURL;",redirTime); }
//  End -->
</script>').

	self attributesAt: #onLoad put: 'redirTimer()'.
	^self pageFrameWith: e title: self title .! !

!WebAdminApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewRegistration
	| e f |
	self inError ifFalse: [self newUser: WebUser new. self password: ''].
	self clear. self title: 'Registration Form'.
	e := WebElement new.
	e addTextH1: 'Registration Form'.
	e addErrorReport.
	f := WebFieldSet newLegend: 'Personal details'.
	f cell addText: 'Name: '. f newCell addInputFieldAspect: #name for: self newUser. f newRow.
	f cell addText: 'Surname: '. f newCell addInputFieldAspect: #surname for: self newUser. f newRow.
	f cell addText: 'E-mail: '. f newCell addInputFieldAspect: #email for: self newUser. f newRow.
	f cell addText: 'Username: '. f newCell addInputFieldAspect: #username for: self newUser.
	f newRow.
	f cell addText: 'Password: '. f newCell addPasswordFieldAspect: #password for: self. f newRow.
	f cell addText: 'Confirm Password: '. f newCell addPasswordFieldAspect: #password for: self newUser.
	f newRow. f newCell addButtonText: 'Register'. 
	e add: f.
 	^self pageFrameWith: e title: self title .! !

!WebAdminApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewServerRuns
	| element inx runs |
	self clear.
	self site setLastTimeAliveTimestamp.
	self	title: 'Runs for site: ', self site name.
	element := WebElement new.
	element table width: self pageContentWidth.
      element cell color: (self navigatorColor).
	element cell colspan: 6; addText: self title header: 3.
	element newRow color: self tableHeaderColor.
	element cell align: #center; addText: 'nr '.
	element newCell align: #center; addText: 'started '.
	element newCell align: #center; addText: 'last time alive '.
	element newCell align: #center; addText: 'uptime'.
	element newCell align: #center; addText: 'downtime'.
	element newCell align: #center; addText: 'termination'.
	element newRow.
	inx := 1.
	runs := self site runningHistory.
	runs do: [:array |
		inx even ifTrue: [element row color: self reportHeaderColor].
		element cell align: #center; addText: inx printString.
		element newCell align: #right; addText: (self printStamp: (array at: 1)).
		element newCell align: #right; addText: (self printStamp: (array at: 2)).
		element newCell align: #right; addText: (self printSeconds: (array at: 3)).
		element newCell align: #right; addText: (self printSeconds: (array at: 4)).
		element newCell align: #center; addText: 
			(array == runs last
				ifTrue: ['running']
				ifFalse: [(array at: 5) ifTrue: ['crashed'] ifFalse: ['normal']]).
		inx := inx + 1.
		element newRow].
	element cell colspan: 6; addRulerSize: 1.
	^self pageFrameWith: element title: self title .! !

!WebAdminApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewServerStatistics
	| element |
	self clear.
	self site setLastTimeAliveTimestamp.
	self	title: 'Statistics for site: ', self site name.
	element := WebElement new.
	element table width: self pageContentWidth.
      element cell color: (self navigatorColor).
	element cell colspan: 3; addText: self title header: 3.
	element newRow.
	element cell addText: 'created: '.
	element newCell align: #right; addText: (self printStamp: self session site createdTimestamp).
	element newRow.
	element cell addText: 'started: '.
	element newCell align: #right; addText: (self printStamp: self session site startedTimestamp).
	element newCell align: #right; addLinkTo: self observee text: 'History of runs' view: 'serverRuns'.
	element newRow.
"
	element cell addText: 'last request: '.
	element newCell align: #right; addText: (self printStamp: self session site lastRequestTimestamp).
	element newRow.
	element cell addText: 'last db commit: '.
	element newCell align: #right; addText: (self printStamp: self session site lastCommitTimestamp).
	element newCell align: #right; addLinkTo: self observee text: 'History of runs' view: 'serverRuns'.
	element newRow.
"
	element cell colspan: 3; addRulerSize: 1.
	element newRow.
	element cell addText: 'uptime: '.
	element newCell align: #right; addText: (self printSeconds: self session site uptime).
	element newRow.
	element cell addText: 'total uptime: '.
	element newCell align: #right; addText: (self printSeconds: self session site totalUptime ).
	element newRow.
	element cell addText: 'total downtime: '.
	element newCell align: #right; addText: (self printSeconds: self session site totalDowntime ).
	element newRow.
	element cell addText: 'availability [%]: '.
	element newCell align: #right; addText: (self session site availability printDotString).
	element newRow.
	element cell colspan: 3; addRulerSize: 1.
	element newRow.
	element cell addText: 'today requests: '.
	element newCell align: #right; addText: ((self session site requestsOnDate: Date today) printDotString).
	element newRow.
	element cell addText: 'yesterday requests: '.
	element newCell align: #right; 
		addText: ((self session site requestsOnDate: Date today - 1) printDotString).
	element newRow.
	element cell addText: 'all requests: '.
	element newCell align: #right; addText: ((self session site totalRequests) printDotString).
	element newRow.
	element cell colspan: 3; addRulerSize: 1.
	^self pageFrameWith: element title: self title .! !

!WebAdminApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewSettings
	| element |
	self clear.
	self	title: 'Settings for site: ', self site name.
	element := WebElement new.
	element table width: 500.
      element cell color: (self navigatorColor).
	element cell addText: self title header: 3.
	element newRow.
	element cell add: (self settingsElementForm: false).
	element newRow.
	element cell addLinkTo: self observee text: '<b>Update settings</b>' view: #updateSettings.
	^self pageFrameWith: element title: self title .! !

!WebAdminApp methodsFor: 'printing' stamp: 'mivsek 6/13/2007 16:37'!
viewSites
	| e |
	self	title: 'Web sites'.
	e := WebElement new.
	e addTextH1: 'Virtual web sites on Swazoo web server'. e addBreak. 
	e addButtonText: 'Add new web site'. e addBreak.
	e add: self sitesElement.
	^self pageFrameWith: e title: self title .! !

!WebAdminApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewTests
	| e inx bundle testClasses versionId |
	self title: 'Test results'.
	e := WebElement new. inx := 1.
	bundle := Store.BundleModel allInstances detect: [:each | each name = 'BiArt'].
	testClasses := bundle definedClasses
		 select: [:each | each inheritsFrom: XProgramming.SUnit.TestCase].
	testClasses := SortedCollection withAll: testClasses sortBlock: [:a :b | a name < b name].
	e addTextH2: 'Unit testing report'. 
	e table width: 1. e cell colspan: 3; addRulerSize: 1. e newRow.
	e cell addText: 'Bundle: '. e newCell addTextBold: bundle name. e newCell width: 0.7. 
	e newRow.
	e cell addText: 'Version:'. e newCell addText: bundle traceVersionString. e newRow.
	e cell addText: 'Testing date: '. e newCell addText: Date today shortPrintSloString.  e newRow.
	e cell addText: 'Time: '. e newCell addText: Time now printSloString.  e newRow.
	e cell colspan: 3; addText: 'Testing environment: ', 'Cincom VisualWorks Smalltalk v'.  
	versionId := ObjectMemory versionId at: 5.
	e cell addText: (versionId // 10) printString, '.', (versionId \\ 10) printString. e newRow.
	e cell colspan: 3; addRulerSize: 1. e newRow.
	e newTable width: 1.
	e newCell width: 0.05; addText: 'Test class/method'. e newCell width: 0.4; align: #right; addText: 'Result'. 
	e newCell width: 0.3. e newRow.
	e cell colspan: 4; addRulerSize: 1. e newRow.
	testClasses do: [:cls || suite result |
		e cell addText: inx printString, '.'.  e newCell addText: cls name.
		suite := TestSuite new. 
		cls allTestSelectors do: [:each | suite addTest: (cls selector: each)]. 
		result := [suite run] ifCurtailed: []. 
		e newCell align: #right; addTextSmall: result runCount printString, ' tests '.
		result hasPassed 
			ifTrue: [e cell addText: 'Passed' color: #green] 
			ifFalse: [e cell addText: '<b>Failed</b>' color: #red].
		cls allTestSelectors asSortedCollection do: [:each |
			e newRow. e newCell addNbSp: 4; addTextSmall: each asString. 
			e newCell align: #right; addText: '<small>Passed</small>' color: #green].
		e newCell.
		e newRow. inx := inx + 1].
	e cell colspan: 4; addRulerSize: 1. e newRow.
	^self pageFrameWith: e title: self title .! !

!WebAdminApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewUpdateSettings
	| element |
	self clear.
	self	title: 'Update Settings for site: ', self site name.
	element := WebElement new.
	element table width: 500.
      element cell color: (self navigatorColor).
	element cell addText: self title header: 3.
	element newRow.
	element cell add: (self settingsElementForm: true).
	element newRow.
	element cell addButtonText: 'Update'.
	^self pageFrameWith: element title: self title .! !

!WebAdminApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewWaitingConfirmation
	| e |
	self clear. self title: 'Waiting Confirmation'.
	e := WebElement new.
	e addTextH1: 'Waiting Confirmation'.
	e addText: 'And e-mail was sent to address you entered in a registration form. Please read it
	and click to the activation link to confirm that your e-mail is valid.
	If you have any problems send an email to '. 
	e addLinkTo: 'info@eranova.si' text: 'Portal Administrator'.
 	^self pageFrameWith: e title: self title .! !

!WebAdminApp methodsFor: 'printing' stamp: 'janko 7/18/2008 16:38'!
viewWakeupRichEditor
	"to prefetch all JS libraries"
	"call that from background Ajax call"
	self addText: 'Just to prefetch all TinyMCE JavaScript libraries'; addBreak.
	self addRichEditorAspect: #dummy for: self! !

!WebAdminApp methodsFor: 'printing' stamp: 'janko 7/18/2008 16:38'!
viewWakeupScriptaculous
	"to prefetch all JS libraries"
	"call that from background Ajax call"
	self addText: 'Just to prefetch all Scriptaculous JavaScript libraries'; addBreak.
	self add: (WebAutocompleteField new)! !

WebFrameApp subclass: #WebDemoApp
	instanceVariableNames: 'liveImage selectedClasses filename fileContentType fileStream startStamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Examples'!

!WebDemoApp methodsFor: 'actions' stamp: 'mivsek 3/7/2008 19:17'!
actionEditMembers
	"just for security to work!!"! !

!WebDemoApp methodsFor: 'actions' stamp: ' 21/4/07 22:07'!
actionEditMembersSave
	self redirectToView: #gridSelected! !

!WebDemoApp methodsFor: 'actions' stamp: 'janko 8/1/2008 20:44'!
actionFileUpload
	self fileStream notNil ifTrue:
		[self fileStream close]. "while content were already streamed to the file before"! !

!WebDemoApp methodsFor: 'actions' stamp: 'mivsek 3/7/2008 19:16'!
actionGridSelected
	"just for security to work!!"! !

!WebDemoApp methodsFor: 'actions' stamp: ' 21/4/07 22:07'!
actionGridSelectedEditMembers
	self redirectToView: #editMembers! !

!WebDemoApp methodsFor: 'actions' stamp: ' 21/4/07 22:07'!
actionWebdav
	| stream file |
	file := self webdavExampleFilename asFilename. file exists ifTrue: [file delete].
	[stream :=  file appendStream. stream binary. 
	stream nextPutAll: self observee file asByteArray] 
		ensure: [self observee file: nil. stream close].! !

!WebDemoApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
addWebDAVVBScript
	"to open directly from web and not copy a document in local cache"
	self script: '
 <SCRIPT LANGUAGE=VBScript>
   Function OpenDoc(strLocation)
   Dim objWord
      Set objWord = CreateObject("Word.Application")
      objWord.Visible = true
      objWord.Documents.Open strLocation
   End Function
 </SCRIPT>
'! !

!WebDemoApp methodsFor: 'printing-ajax' stamp: 'np 3/21/2008 20:41'!
ajaxAutocompleteExample
	| e field |
	e := WebElement new.
	e addTextH4: 'Example 4: Autocomplete field with data from server'.
	e addText: 'Type first chars of month names to select: '. e addBreak.
	field := WebAutocompleteField 
		aspect: #field for: self observee choicesAspect: #monthsStarting: for: self.
	e add: field.
	^e! !

!WebDemoApp methodsFor: 'printing-ajax' stamp: ' 21/4/07 22:07'!
ajaxDelayedFieldElement
	| e |
	e := WebElement new.
	e style: '{background-color: #eee }'.
	e addTextBold: self observee delayedInput.
	^e! !

!WebDemoApp methodsFor: 'printing-ajax' stamp: 'mivsek 1/10/2008 23:03'!
ajaxDelayedFieldExample
	| e element input |
	e := WebElement new.
	e addTextH4: 'Example 6: Delayed send input to server'.
	element := self ajaxDelayedFieldElement.
	input := WebDelayedField aspect: #delayedInput for: self observee.
	input afterPostUpdate: element.
	e add: input; addBreak; add: element.
	^e! !

!WebDemoApp methodsFor: 'printing-ajax' stamp: 'mivsek 1/10/2008 23:03'!
ajaxInPlaceEditorExample
	| e |
	e := WebElement new.
	e addTextH4: 'Example 5: In place editor example'.
	e addText: 'Click text below to edit it'. e addBreak.
"	e addInPlaceEditableTextAspect: #editField for: self observee." "that's shorter way"
	e add: ((WebInPlaceEditableText aspect: #editField for: self observee)
		triggerElement: ((WebImage gif: #buttonEditGif) title: 'Edit this text') ). "image to enter edit"
	^e! !

!WebDemoApp methodsFor: 'printing-ajax' stamp: ' 21/4/07 22:07'!
ajaxInputFieldElement
	| e |
	e := WebElement new.
	e style: '{background-color: #eee }'.
	e addTextBold: self observee input.
	^e! !

!WebDemoApp methodsFor: 'printing-ajax' stamp: 'mivsek 1/10/2008 23:04'!
ajaxInputFieldExample
	| e element input |
	e := WebElement new.
	e addTextH4: 'Example 3: Live send input to server'.
	element := self ajaxInputFieldElement.
	input := WebInputField aspect: #input for: self observee.
	input onKeyUpPostAndUpdate: element.
	e add: input; addBreak; add: element.
	^e! !

!WebDemoApp methodsFor: 'printing-ajax' stamp: ' 21/4/07 22:07'!
ajaxPeriodicUpdaterExample
	| e element |
	e := WebElement new.
	e addTextH4: 'Example 2: Periodicaly update element from server'.
	element := self ajaxTimeElement.
	element updateEverySeconds: 1.
	e add: element.
	^e! !

!WebDemoApp methodsFor: 'printing-ajax' stamp: 'JM 4/26/2007 19:59'!
ajaxTimeElement
	| e |
	e := WebElement new.
	e style: '{background-color: #eee }'.
	e addTextBold: SpDate today shortPrintSloString, ' ', Time now printString.
	^e! !

!WebDemoApp methodsFor: 'printing-ajax' stamp: 'mivsek 1/10/2008 23:04'!
ajaxUpdateTimeExample
	| e element |
	e := WebElement new.
	e addTextH4: 'Example 1: Update element from server'.
	element := self ajaxTimeElement.
	e 
	 	add: element; addBreak;
		add: ((WebElement new addText: 'click here to update') onClickUpdate: element).
	^e! !

!WebDemoApp methodsFor: 'printing-grid' stamp: ' 21/4/07 22:07'!
classesEditElement
	| e |
	e := WebElement new.
	e addButtonText: 'Save' action: #save.
	e add: (WebGrid new
		columnNames: #('' 'Class name');
		columnAspects: #(nil #name);
		sortOn: 2;
		collection: WebApplication allWebAppClasses;
		checkboxesColumn: 1 collection: self selectedClasses;
		yourself).
	^e! !

!WebDemoApp methodsFor: 'printing-grid' stamp: ' 21/4/07 22:07'!
classesSelectedElement
	| e |
	e := WebElement new.
	e addButtonText: 'Edit' action: #editMembers.
	e add: (WebGrid new
		columnNames: #('nr.' 'Class name');
		columnAspects: #(nil #name);
		setNumbering; sortOn: 2;
		collection: self selectedClasses;
		yourself).
	^e! !

!WebDemoApp methodsFor: 'printing-grid' stamp: ' 21/4/07 22:07'!
classesTableElement
	| e |
	e := WebElement new.
	e add: (WebGrid new
		columnNames: #('nr.' 'Class name');
		columnFilters: #(nil true);
		columnAspects: #(nil #name);
		setNumbering; sortOn: 2;
		collection: WebApplication allWebAppClasses;
		yourself).
	^e! !

!WebDemoApp methodsFor: 'accessing' stamp: 'janko 8/1/2008 21:13'!
fileContentType
	^fileContentType! !

!WebDemoApp methodsFor: 'accessing' stamp: 'janko 8/1/2008 21:14'!
fileContentType: aString
	fileContentType := aString! !

!WebDemoApp methodsFor: 'accessing' stamp: 'janko 8/1/2008 20:09'!
fileStream
	"for file upload"
	^fileStream! !

!WebDemoApp methodsFor: 'accessing' stamp: 'janko 8/1/2008 20:09'!
fileStream: aFileStream
	"for file upload"
	fileStream := aFileStream! !

!WebDemoApp methodsFor: 'accessing' stamp: 'janko 8/1/2008 21:12'!
filename
	filename isNil ifTrue: [^''].
	^filename! !

!WebDemoApp methodsFor: 'accessing' stamp: 'janko 8/1/2008 21:13'!
filename: aString
	filename := aString! !

!WebDemoApp methodsFor: 'printing-grid' stamp: ' 21/4/07 22:07'!
gridTabsElement
 	^ViewTabs new
		addView: #grid description: 'All App classes';
		addView: #gridSelected description: 'Selected App classes';
		yourself.! !

!WebDemoApp methodsFor: 'printing' stamp: 'mivsek 1/11/2008 15:20'!
introductionElement
	| e |
	e := WebElement new.
	e addText: '
<b>Congratulations!!</b><br><br>
You just started your AIDA/Web Smalltalk Web Application Server. Here you can look at few simple <a href=/demo.html?view=grid><b>demos</b></a> (see navigation bar at the left) and on top left there are links to main website, where you can find more information. <br>
<br>
You are encougared to:
<ul>
<li>look at <a href=http://www.aidaweb.si/screenshots.html><b>Screenshots</b></a> to see, what kind of complex web applications you can build with AIDA/Web,</li>
<li>see <a href=http://www.aidaweb.si/tutorial.html><b>Tutorial</b></a> to build your first web application,</li>
<li>read other <a href=http://www.aidaweb.si/documentation.html><b>Documentation</b></a> to learn more (Architecture, Administrator''s guide, Programer''s guide, Ajax, Advanced features, Articles and presentations and more),</li> 
<li>join the <a href=http://www.aidaweb.si/community.html><b>Community</b></a>.</li>
</ul>
You can also <a href=/admin.html?view=login><b>login</b></a> with username: <b>admin</b> and password: <b>password</b>. <br><br>

Development team wish you a lot of great moments with Aida/Web !!
'.
	^e! !

!WebDemoApp methodsFor: 'printing-javascript' stamp: ' 21/4/07 22:07'!
jsShowAlertExample
	| e |
	e := WebElement new.
	e addTextH4: 'Example 1: onClick event on some text'. e addBreak.
	e add: 
		((WebText text: 'click on me to show alert!!') onClick: 'alert(''This is an alert!!'')'). 
	^e.! !

!WebDemoApp methodsFor: 'printing-javascript' stamp: ' 21/4/07 22:07'!
jsShowHideToggleExample
	| e element |
	e := WebElement new.
	e addTextH4: 'Example 2: show/hide an element with a click'.
	element := WebElement new addText: 'This element is now shown'; style: '{background-color: #ccc }'.
	element hide.  "hide from the start"
	e 
		add: ((WebText text: 'show') onClickShow: element); addSpace;
		add: ((WebText text: 'hide') onClickHide: element); addSpace;
		add: ((WebText text: 'toggle') onClickToggle: element); addSpace;
	 	add: element.
	^e! !

!WebDemoApp methodsFor: 'printing-javascript' stamp: ' 21/4/07 22:07'!
jsShowWithMouseOverExample
	| e element |
	e := WebElement new.
	e addTextH4: 'Example 3: show/hide an element with mouse over'.
	element := WebElement new addText: 'This element is now shown'; style: '{background-color: #ccc }'.
	element hide.  "hide from the start"
	e 
		add: ((WebText text: 'mouse over to show') 
			onMouseOverShow: element; onMouseOutHide: element); addSpace;
	 	add: element.
	^e! !

!WebDemoApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
liveImage
	^liveImage! !

!WebDemoApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
liveImage: aWebLiveImage
	liveImage := aWebLiveImage! !

!WebDemoApp methodsFor: 'printing-ajax' stamp: 'mivsek 1/10/2008 23:06'!
monthsStarting: aString
	"as choicesAspect for autocompletion demo"
	"WebDemoApp new monthsStarting: 'j' "
	| months |
	months := 
		#('januar' 'prosinec' 'sijec^anj' 'January' 
		   'februar', 'svec^an' 'veljac^a' 'February'
		   'marec', 'sus^ec' 'oz^ujak' 'Mart'
		   'april', 'mail traven' 'travanj' 'April'
		   'maj', 'veliki traven' 'svibanj' 'May'
		   'junij', 'roz^nik' 'lipanj' 'June'
		   'julij', 'mali srpen' 'srpanj' 'July'
		   'avgust', 'veliki srpen' 'kolovoz' 'August'
		   'september', 'kimavec' 'rujan' 'September'
		   'oktober', 'vinotok' 'listopad' 'October'
		   'november', 'listopad' 'studeni' 'November'
		   'december', 'gruden' 'prosinac' 'December').
	^months select: [:each | aString, '*' match: each]! !

!WebDemoApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
selectedClasses
	selectedClasses isNil ifTrue: [self selectedClasses: OrderedCollection new].
	^selectedClasses! !

!WebDemoApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
selectedClasses: anObject
	selectedClasses := anObject! !

!WebDemoApp methodsFor: 'accessing' stamp: 'janko 8/1/2008 20:12'!
setStartStamp
	"start timestamp of some action"
	^startStamp := SpTimestamp now! !

!WebDemoApp methodsFor: 'accessing' stamp: 'janko 8/1/2008 20:12'!
startStamp
	"start timestamp of some action"
	^startStamp! !

!WebDemoApp methodsFor: 'printing-upload' stamp: 'janko 8/1/2008 21:17'!
uploadReportAndResetElement
	"report if anything then delete file and reset variables"
	| e elapsed fileSize |
	e := WebElement new.
	self filename isEmpty ifTrue: [^e].
	elapsed := SpTimestamp now asSeconds - self startStamp asSeconds.
	fileSize := (SpFilename named: self filename) fileSize.
	e addText: 'uploaded file: ', self filename, ' size: ', fileSize printDotString, ' bytes in ', 
		elapsed printDotString, ' seconds (', ((fileSize/(elapsed max: 1)) rounded) printDotString, 'bytes/s)'.
	(SpFilename named: self filename) delete.
	self filename: ''. self fileStream: nil.
	^e! !

!WebDemoApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewAjax
	| e |
	e := WebElement new.
	e cell addTextH1: 'Simple AJAX examples'. e newRow. e newRow.
	e cell add: self ajaxUpdateTimeExample. e newRow.
	e cell add: self ajaxPeriodicUpdaterExample. e newRow.
	e cell add: self ajaxInputFieldExample. e newRow.
	e cell add: self ajaxAutocompleteExample. e newRow.
	e cell add: self ajaxInPlaceEditorExample. e newRow.
	e cell add: self ajaxDelayedFieldExample. e newRow.
	self pageFrameWith: e title: 'AJAX examples'! !

!WebDemoApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewCalendar
	| e |
	e := WebElement new.
	e addTextH1: 'Date input with Javascript Calendar example'.	 e addBreak.
	e addDateInputFieldAspect: #date for: self observee. e addBreak.
	self pageFrameWith: e title: 'Calendar example'! !

!WebDemoApp methodsFor: 'printing-grid' stamp: 'mivsek 1/11/2008 14:34'!
viewEditMembers
	| e |
	e := WebElement new.
	e cell addTextH1: 'WebGrid, tabs, live filtering, Ajax sorting, checkboxes...'. e newRow.
	e cell add: self classesEditElement.
	self pageFrameWith: e title: 'Grid example'! !

!WebDemoApp methodsFor: 'printing-upload' stamp: 'janko 8/1/2008 21:38'!
viewFileUpload
	| e field fname |
	e := WebElement new.
	e cell addTextH1: 'File upload example'. e newRow. e newRow.
	field := WebFileInputField 
		beforeStreamingDo: [:datum |  "datum is actually MIME part"
			fname := datum filename trimBlanks. 
			fname notEmpty ifTrue:
				[self fileStream: (SpFilename named: fname) writeStream. "will be closed in action method"
				datum writeStream: self fileStream.
				self setStartStamp ] ]    "for measuring upload time"
		filenameAspect: #filename 
		contentTypeAspect: #fileContentType 
		forObject: self.
	e cell add: field; addButtonText: 'Upload'. e newRow.
	e cell add: self uploadReportAndResetElement. "also deletes uploaded file!!"
	self pageFrameWith: e title: 'File upload example'! !

!WebDemoApp methodsFor: 'printing-grid' stamp: 'mivsek 1/11/2008 14:34'!
viewGrid
	| e |
	e := WebElement new.
	e cell addTextH1: 'WebGrid, tabs, live filtering, Ajax sorting, checkboxes...'. e newRow.
	e cell add: self gridTabsElement. e newRow.
	e cell add: self classesTableElement.
	self pageFrameWith: e title: 'Grid example'! !

!WebDemoApp methodsFor: 'printing-grid' stamp: 'mivsek 1/11/2008 14:34'!
viewGridSelected
	| e |
	e := WebElement new.
	e cell addTextH1: 'WebGrid, tabs, live filtering, Ajax sorting, checkboxes...'. e newRow.
	e cell add: self gridTabsElement. e newRow.
	e cell add: self classesSelectedElement.
	self pageFrameWith: e title: 'Grid example'! !

!WebDemoApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewImageGallery
	| e |
	e := WebElement new.
	e addText: 'This is a collection of so called <b>method images</b>, that is: images embeded in methods in class <b>WebStyle</b> or its subclasses.'.

	"filetype icons"
	e cell cell addGif: #pdfSmallGif. e cell newCell addText: 'pdfSmallGif'. e cell newRow.
	e cell cell addGif: #pdfMediumGif. e cell newCell addText: 'pdfMediumGif'. e cell newRow.
	e cell cell addGif: #pdfBigGif. e cell newCell addText: 'pdfBigGif'. e cell newRow.
	e cell cell addGif: #wordBigGif. e cell newCell addText: 'wordBigGif'. e cell newRow.
	e cell cell addGif: #excelBigGif. e cell newCell addText: 'excelBigGif'. e cell newRow.
	e cell cell addGif: #xmlBigGif. e cell newCell addText: 'xmlBigGif'. e cell newRow.
	e newCell.

	"filetype icons"
	e cell cell addGif: #arrowBigBlueGif. e cell newCell addText: 'arrowBigBlueGif'. e cell newRow.
	e cell cell addGif: #arrowBigRedGif. e cell newCell addText: 'arrowBigRedGif'. e cell newRow.
	e cell cell addGif: #arrowBigYellowGif. e cell newCell addText: 'arrowBigYellowGif'. e cell newRow.
	e cell cell addGif: #arrowDownBigRedGif. e cell newCell addText: 'arrowDownBigRedGif'. e cell newRow.
	e cell cell addGif: #arrowSmallBlackGif. e cell newCell addText: 'arrowSmallBlackGif'. e cell newRow.
	e cell cell addGif: #arrowTriangleRedGif. e cell newCell addText: 'arrowTriangleRedGif'. e cell newRow.
	e cell cell addGif: #arrowTriangleYellowGif. e cell newCell addText: 'arrowTriangleYellowGif'. e cell newRow.
	e cell cell addGif: #arrowCircledRedGif. e cell newCell addText: 'arrowCircledRedGif'. e cell newRow.
	e cell cell addGif: #arrowCircledGreyGif. e cell newCell addText: 'arrowCircledGreyGif'. e cell newRow.
	e newRow.

	e cell cell addPDFLinkTo: 'http://wiki.eranova.si/e-logis/DOWNLOAD/EAN-standards-roadmap.pdf'.
	e cell newCell addText: 'addPDFLinkTo: '.


	self pageFrameWith: e title: 'Gallery of embedded images'! !

!WebDemoApp methodsFor: 'printing-javascript' stamp: ' 21/4/07 22:07'!
viewJavascript
	| e |
	e := WebElement new.
	e cell addTextH1: 'Simple Javascript examples'. e newRow. e newRow.
	e cell add: self jsShowAlertExample. e newRow.
	e cell add: self jsShowHideToggleExample. e newRow.
	e cell add: self jsShowWithMouseOverExample. e newRow. 
	self pageFrameWith: e title: 'Javascript examples'! !

!WebDemoApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewLiveImage
	| e |
	self liveImage isNil ifTrue: [self liveImage: WebGraph new].  "subclass WebLiveImage"
	e := WebElement new.
	e addTextH1: 'Live image example'. e addBreak.
	e addImage: self liveImage.
	self pageFrameWith: e title: 'Live image example'! !

!WebDemoApp methodsFor: 'printing' stamp: 'mivsek 1/10/2008 23:06'!
viewMain
	| e |
	e := WebElement new.
	e addTextH1: 'Introduction'. e addBreak.
	e add: self introductionElement.
	self pageFrameWith: e title: 'Aida/Web - Introduction'! !

!WebDemoApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewRichEditor
	| e |
	e := WebElement new.
	e addTextH1: 'JavaScript HTML WYSIWYG Editor example'.	 e addBreak.
	e addRichEditorAspect: #content for: self observee size: 50@10. e addBreak.
	e addButtonText: 'Save'. e addBreak.
	e addTextH2: 'Result: '. e addBreak.
	e addText: self observee content.
	self pageFrameWith: e title: 'Rich Editor example'! !

!WebDemoApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewTabs
	| e |
	e := WebElement new.
	e add: (WebTabs new
		addLinkTo: self observee text: 'first'; 
		addText: 'second' attributes: #bold; selected: 2;
		addLinkTo: self observee text: 'third'; 
		addLinkTo: self observee text: 'forth').
	e addParagraph; addNbSp.
	e add: ((WebTabs newClass: #tabs2)
		addLinkTo: self observee text: 'first'; 
		addText: 'second' attributes: #bold; selected: 2;
		addLinkTo: self observee text: 'third'; 
		addLinkTo: self observee text: 'forth').
"
	e addParagraph; addNbSp.
	e add: ((WebTabs newClass: #tabs3)
		addLinkTo: self observee text: 'first'; 
		addText: 'second' attributes: #bold; selected: 2;
		addLinkTo: self observee text: 'third'; 
		addLinkTo: self observee text: 'forth').
	e addParagraph; addNbSp.
	e add: ((WebTabs newClass: #tabs4)
		addLinkTo: self observee text: 'first'; 
		addText: 'second' attributes: #bold; selected: 2;
		addLinkTo: self observee text: 'third'; 
		addLinkTo: self observee text: 'forth').
	e addParagraph; addNbSp.
"
	e add: ((WebTabs newClass: #tabs5)
		addLinkTo: self observee text: 'first'; 
		addText: 'second' attributes: #bold; selected: 2;
		addLinkTo: self observee text: 'third'; 
		addLinkTo: self observee text: 'forth').
	self pageFrameWith: e title: 'Tabs example'! !

!WebDemoApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewWebdav
	| e link |
	"link := '/', self webdavExampleFilename."
	link := self site uriPattern first currentUrl, '/', self webdavExampleFilename.
	self addWebDAVVBScript. "IE only!!"
	link := 'javascript:OpenDoc(''', link, ''')'. "for IE browser only!!"
	e := WebElement new. self title: 'WebDAV publishing example'.
	e cell addTextH2: self title. e newRow. e cell addNbSp. e newRow.
	e cell addText: '1. Upload  MS Word document: '.
	e cell add: (WebFileInputField aspect: #file filenameAspect: #filename 
		contentTypeAspect: #fileContentType forObject: self observee). 
	e cell addSpace; addButtonText: 'Upload'. e newRow.
	e cell addText: '2. Now open document from: '. e cell addLinkTo: link text: 'here'. e newRow.
	e cell addText: '3. Save document'. e newRow.
	e cell addText: '4. Repeat from 2 to see if changes got saved'. e newRow.

"	
	e cell addBreak; addText: self observee filename; addBreak; addText: self observee fileContentType.
"
	self pageFrameWith: e title: self title.! !

!WebDemoApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
webdavExampleFilename
	^'aida-webdav-example.doc'.! !

!WebFrameApp methodsFor: 'date calculations' stamp: 'JM 4/26/2007 21:33'!
adjustViewingDate
	"read url parameters and adjust a viewing date accordingly"
	| adjusted |
	adjusted := false.
	(self session lastRequest includesQuery: 'day') ifTrue: 
		[self date: (SpDate 
			newDay: (self session lastRequest queryAt: 'day') asInteger
			month: self date monthIndex
			year: self date year). adjusted := true].
	(self session lastRequest includesQuery: 'month') ifTrue: 
		[self date: (SpDate 
			newDay: 1
			month: (self session lastRequest queryAt: 'month') asInteger
			year: self date year). adjusted := true].
	(self session lastRequest includesQuery: 'year') ifTrue: 
		[self date: (SpDate 
			newDay: 1
			month: self date monthIndex
			year: (self session lastRequest queryAt: 'year') asInteger).
			adjusted := true].
	adjusted ifFalse: [date := Date today].! !

!WebFrameApp methodsFor: 'date calculations' stamp: ' 21/4/07 22:07'!
currentMonth
	^self currentYear and: [self date monthIndex = Date today monthIndex]! !

!WebFrameApp methodsFor: 'date calculations' stamp: ' 21/4/07 22:07'!
currentYear
	^self date year = Date today year! !

!WebFrameApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
date
	date isNil ifTrue: [self date: Date today].
	^date! !

!WebFrameApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
date: anObject
	date := anObject! !

!WebFrameApp methodsFor: 'date calculations' stamp: 'JM 4/26/2007 21:07'!
firstDayInMonth
	^SpDate newDay: 1 month: self date monthIndex year: self date year! !

!WebFrameApp methodsFor: 'date calculations' stamp: 'mivsek 1/13/2008 20:18'!
firstDayInNextMonth
	^SpDate 
		newDay: 1 
		month: (self lastDayInMonth addDays: 1) monthIndex
		year: (self lastDayInMonth addDays: 1) year! !

!WebFrameApp methodsFor: 'date calculations' stamp: 'JM 4/26/2007 21:33'!
firstDayInNextYear
	^SpDate 
		newDay: 1
		month: self firstDayInMonth monthIndex
		year: self firstDayInMonth year + 1! !

!WebFrameApp methodsFor: 'date calculations' stamp: 'mivsek 3/7/2008 19:39'!
firstDayInPreviousMonth
	^SpDate 
		newDay: 1
		month: (self firstDayInMonth subtractDays: 1) monthIndex
		year: (self firstDayInMonth subtractDays: 1) year! !

!WebFrameApp methodsFor: 'date calculations' stamp: 'JM 4/26/2007 21:34'!
firstDayInPreviousYear
	^SpDate 
		newDay: 1
		month: self firstDayInMonth monthIndex
		year: self firstDayInMonth year + 1! !

!WebFrameApp methodsFor: 'date calculations' stamp: 'JM 4/26/2007 21:07'!
lastDayInMonth
	^SpDate newDay: self date daysInMonth month: self date monthIndex year: self date year! !

WebFrameApp subclass: #WebIndexApp
	instanceVariableNames: 'searchString results page'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Admin'!

!WebIndexApp methodsFor: 'printing-elements' stamp: ' 21/4/07 22:07'!
abstractTextSize

	^400! !

!WebIndexApp methodsFor: 'actions' stamp: ' 21/4/07 22:07'!
actionMain
	"search default web index for objects containing words from searchString, store results
   	in attribute results and st the next view as 'results' in order to print result web page"
	self results: (self observee objectsForWords: self searchString).
	self page: 1.
	self newView: #results! !

!WebIndexApp methodsFor: 'actions' stamp: ' 21/4/07 22:07'!
actionResults
	self actionMain! !

!WebIndexApp methodsFor: 'printing-elements' stamp: ' 21/4/07 22:07'!
chopText: aString toSize: aNumber 
	| inStream outStream |
	inStream := aString readStream.
	outStream := WriteStream on: String new.
	1 to: aString size
		do: 
			[:count | 
			inStream atEnd ifTrue: [^outStream contents].
			(count >= aNumber and: [inStream peek = $ ]) 
				ifTrue: [^outStream contents , ' ...'].
			outStream nextPut: inStream next].
	^outStream contents

	"WebIndexApp new chopText: 'hojladri hojladra' toSize: 5"! !

!WebIndexApp methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initResults
	results := OrderedCollection new.! !

!WebIndexApp methodsFor: 'printing-elements' stamp: ' 21/4/07 22:07'!
miniSearchFormElement
	| form actionLink |
	self cleanup.
	form := self defaultForm.
	actionLink := (WebLink text: '' linkTo: self observee)  view: 'results'.
	form 
		actionUrl: (actionLink composeURLOn: self session);
		target: 'Vsebina'.
	form 
		addText: '<center>';
		add: (WebInputField new size: 15;
				aspect: #searchString for: self);
		addBreak;
		add: (WebButton new text: '&nbsp;&nbsp;&nbsp;Najdi&nbsp;&nbsp;&nbsp;');
		addText: '</center>'.
	form registrateWebFormElements.
	^form! !

!WebIndexApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
page
	"return current result page number to show. Each page shows 10 hits"
	page isNil ifTrue: [self page: 1].
	^page! !

!WebIndexApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
page: aNumber

	page := aNumber.! !

!WebIndexApp methodsFor: 'printing-elements' stamp: ' 21/4/07 22:07'!
pageSelectionElement

	| element pages |
	element := WebElement new.
	pages := self results size = 0
		ifTrue: [0]
		ifFalse: [(self results size // self resultsOnPage) + 1].
	pages = 1 ifTrue: [^element].
	element addText: '<font face="helvetica, arial" size="-1">'.
	1 to: pages do: [:index |
		index = self page
			ifTrue: [element addText: index printString attributes: #bold]
			ifFalse: 
				[element add: 
					((WebLink text: index printString linkTo: self observee )
						view: #results;
						parameter: 'page' value: index printString)].
		element addNbSp ].
	element addText: '</font>'.
	^element! !

!WebIndexApp methodsFor: 'printing-elements' stamp: ' 21/4/07 22:07'!
resultNumberElement

	
	| element start end |
	element := WebElement new.
	(self results size = 0) ifTrue: 
		[element add: (self smallText: '0 hits' attributes: #bold). ^element].
	(self results size = 1) ifTrue: 
		[element add: (self smallText: '1 hit' attributes: #bold). ^element].

	start := (self page-1)*self resultsOnPage+1.
	(start+self resultsOnPage > self results size)
		ifTrue: [end := self results size]
		ifFalse: [end := start+self resultsOnPage-1].
	element cell add: 
		(self smallText:  (start printString, ' - ', end printString,
			' from ', self results size printString, ' hits') attributes: #bold).
	^element! !

!WebIndexApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
resultWithHash: aString
	"find an object in results with that hash value"
	^(self results detect: [:each | each key hash asString = aString] ifNone: [Association new]) key! !

!WebIndexApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
results
	"here the results of a search are stored."
	results isNil ifTrue: [self initResults].
	^results! !

!WebIndexApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
results: aCollection
	results := aCollection! !

!WebIndexApp methodsFor: 'printing-elements' stamp: 'np 3/21/2008 21:14'!
resultsElement
	"return a web element for search results"
	| e abstract objTitle modifiedText modifiedDate first last assoc |
	first := (self page - 1) * self resultsOnPage + 1.
	last := (first + self resultsOnPage - 1) min: self results size.
	e := WebList  newDefinition id: #searchRslts.
	first to: last do: [:index |
		assoc := self results at: index.
		objTitle := (assoc key class canUnderstand: #indexTitle) 
			ifTrue: [assoc key indexTitle] ifFalse: [self chopText: assoc key indexText toSize: 50].
		abstract := (assoc key class canUnderstand: #indexAbstract) 
			ifTrue: [assoc key indexAbstract]
			ifFalse: [self chopText: assoc key indexText "trimNewlines" toSize: self abstractTextSize].
		modifiedText := ''.
		(assoc key class canUnderstand: #modified) ifTrue: 
			[modifiedDate := assoc key modified. modifiedDate notNil ifTrue: 
				[modifiedText := 'changed: <b> ', modifiedDate shortPrintSloString, '</b>,  '] ].
		e addDefinitionTerm: (WebElement new addText: (index printDotString, '.  ');
			addLinkTo: assoc key text: objTitle;
			addLinkTo: self gif: #copyIconGif title: 'Copy to web clipboard' 
				view: #copyToClipboard parameter: 'uuid' value: assoc key hash asString; yourself).
		e addDefinition: (WebElement new addText: abstract).
"
		e newCell color: self tableRowColor; addText: 
			('modifiedText, 'visits: <b>', 
			(self site urlResolver counterFor: assoc key) total printDotString, '</b>')
"
		].
	^e! !

!WebIndexApp methodsFor: 'printing-elements' stamp: ' 21/4/07 22:07'!
resultsOnPage
	^10! !

!WebIndexApp methodsFor: 'printing-elements' stamp: ' 21/4/07 22:07'!
searchFormServlet
	"servlet for using in static pages such as frames for search"
	| indexApp |
	self session cookie ifTrue: 
		[indexApp := self site index webAppFor: self session.
		^indexApp miniSearchFormElement]! !

!WebIndexApp methodsFor: 'printing-elements' stamp: ' 21/4/07 22:07'!
searchLinkServlet
	"servlet for using in static pages such as frames for search"
	| element |
	element := WebElement new.
	element add: ((WebLink text: 'Search' linkTo: self site index)).
	^element! !

!WebIndexApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
searchString
	searchString isNil ifTrue: [self searchString: ''].
	^searchString! !

!WebIndexApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
searchString: aString

	searchString := aString.! !

!WebIndexApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewCopyToClipboard
	"put this refrence to web clipboard"
	| result |
	result := self resultWithHash: (self session lastRequest queryAt: 'uuid').
	result notNil ifTrue: [(result webAppFor: self session) copyToClipboard].
	^self redirectToView: #results "to have an url without view=paste"! !

!WebIndexApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewMain
	| element |
	self clear. self title: 'Search our web site'.
	element := WebElement new.
      element table width: 480. 
	element cell 	addText: ('<font face= "arial, helvetica" size=+2><b>', self title).
	element newRow.
	element cell color: self tableHeaderColor;
		add: (self smallText: 'Find the following words: ');
		add: (WebInputField new size: 30; aspect: #searchString for: self);
		add: (WebButton new text: 'Search').
	^self pageFrameWith: element title: self title! !

!WebIndexApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewResults
	| e |
	self page: (self session lastRequest queryAt: 'page' ifAbsent: [self page]) asInteger.
	self clear. self title: 'Search results'.
	e := WebElement new.
      e table width: 1. "100%"
	e cell addTextH1: self title. e newRow.
	e cell color: self tableHeaderColor;
		addInputFieldAspect: #searchString for: self size: 40; addButtonText: 'Search'.
	e newRow. e cell add: self resultNumberElement.
	e newRow. e add: self resultsElement.
	e newRow.
	e cell align: #center; add: self pageSelectionElement.
	^self pageFrameWith: e title: self title.! !

WebFrameApp subclass: #WebSecurityManagerApp
	instanceVariableNames: 'usr group aclObject aclUserOrGroup aclForWho'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Admin'!

!WebSecurityManagerApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
aclForWho
	^aclForWho! !

!WebSecurityManagerApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
aclForWho: anObject
	aclForWho := anObject! !

!WebSecurityManagerApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
aclObject
	^aclObject! !

!WebSecurityManagerApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
aclObject: anObject
	aclObject := anObject! !

!WebSecurityManagerApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
aclUserOrGroup
	^aclUserOrGroup! !

!WebSecurityManagerApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
aclUserOrGroup: anObject
	aclUserOrGroup := anObject! !

!WebSecurityManagerApp methodsFor: 'actions' stamp: 'mivsek 6/12/2007 23:20'!
actionAddGroup
	self group ~= '' ifTrue: [self observee addGroup: self group].
	self newView: #groups.! !

!WebSecurityManagerApp methodsFor: 'actions' stamp: 'mivsek 3/7/2008 19:22'!
actionAddUser
	self usr ~= '' ifTrue: [self observee addUser: self usr].
	self newView: #users.! !

!WebSecurityManagerApp methodsFor: 'actions' stamp: 'mivsek 6/12/2007 23:20'!
actionUpdateUser
	self newView: #user.! !

!WebSecurityManagerApp methodsFor: 'printing - access control' stamp: ' 21/4/07 22:07'!
allYesNoLinks
	| element |
	element := WebElement new.
	element
		addText: 'all views: ';
		addLinkTo: self observee text: ' YES ' view: #aclObject parameter: 'allViews' value: 'YES';
		add: ((WebLink text: ' NO ' linkTo: self observee)
			view: #aclObject;
			parameter: 'allViews' value: 'NO');
		addText: 'all updates: ';
		add: ((WebLink text: ' YES ' linkTo: self observee)
			view: #aclObject;
			parameter: 'allUpdates' value: 'YES');
		add: ((WebLink text: ' NO ' linkTo: self observee)
			view: #aclObject;
			parameter: 'allUpdates' value: 'NO');
		addText: 'all encrypted: ';
		add: ((WebLink text: ' YES ' linkTo: self observee)
			view: #aclObject;
			parameter: 'allEncrypted' value: 'YES');
		add: ((WebLink text: ' NO ' linkTo: self observee)
			view:  #aclObject;
			parameter: 'allEncrypted' value: 'NO').
	^element! !

!WebSecurityManagerApp methodsFor: 'printing - access control' stamp: ' 21/4/07 22:07'!
checkAccessRight: aSymbol

	"if YES or NO is pressed for a #view or #update method in aclObjectPage, then set or reset a view
	access rights for current user or group, web app and view symbol"

	| right userOrGroup command |
	(self session lastRequest includesQuery: 'right') 
		ifTrue: 
			[right := (self session lastRequest queryAt: 'right')].
	(self aclForWho = 'user') 
		ifTrue: [userOrGroup := self usr] 
		ifFalse: [userOrGroup := self group].
	aSymbol = #view
		ifTrue:
			[(self session lastRequest includesQuery: 'viewRight') 
				ifTrue: 
					[command := (self session lastRequest queryAt: 'viewRight')] ]
		ifFalse:
			[(self session lastRequest includesQuery: 'updateRight') 
				ifTrue: 
					[command := (self session lastRequest queryAt: 'updateRight')] ].
	
	command = 'YES'
		ifTrue: 
			[self observee 
				setAccessRights: 
					(aSymbol = #view
						ifTrue: [aclObject viewRightSymbolFor: right]
						ifFalse:  [aclObject updateRightSymbolFor: right])
				for: userOrGroup 
				on: self aclObject name].
	command = 'NO'
		ifTrue: 
			[self observee 
				removeAccessRights: 
					(aSymbol = #view
						ifTrue: [aclObject viewRightSymbolFor: right]
						ifFalse:  [aclObject updateRightSymbolFor: right])
				for: userOrGroup 
				on: self aclObject name].! !

!WebSecurityManagerApp methodsFor: 'printing - access control' stamp: ' 21/4/07 22:07'!
checkAllYesNo
	"if YES or NO is pressed for a #view or #update method in aclObjectPage, then set or reset a view
	access rights for current user or group, web app and view symbol"
	| userOrGroup yesNo |
	(self aclForWho = 'user') 
		ifTrue: [userOrGroup := self usr] 
		ifFalse: [userOrGroup := self group].
	(self session lastRequest includesQuery: 'allViews') 
		ifTrue: 
			[yesNo := (self session lastRequest queryAt: 'allViews').
			self aclObject allViews do: [:each |
				yesNo = 'YES'
					ifTrue: 
						[self observee
							setAccessRights: (aclObject viewRightSymbolFor: each)
							for: userOrGroup 
							on: self aclObject name]
					ifFalse: 
						[self observee
							removeAccessRights: (aclObject viewRightSymbolFor: each)
							for: userOrGroup 
							on: self aclObject name] ] ].
	(self session lastRequest includesQuery: 'allUpdates') 
		ifTrue: 
			[yesNo := (self session lastRequest queryAt: 'allUpdates').
			self aclObject allViews do: [:each |
				(aclObject actionMethodForView: each) notNil ifTrue: 
					[yesNo = 'YES'
						ifTrue: 
							[self observee
								setAccessRights: (aclObject updateRightSymbolFor: each)
								for: userOrGroup 
								on: self aclObject name]
						ifFalse: 
							[self observee
								removeAccessRights: (aclObject updateRightSymbolFor: each)
								for: userOrGroup 
								on: self aclObject name] ] ] ].
	(self session lastRequest includesQuery: 'allEncrypted') 
		ifTrue: 
			[yesNo := (self session lastRequest queryAt: 'allEncrypted').
			self aclObject allViews do: [:each |
				yesNo = 'YES'
					ifTrue: [self observee
						setAccessRights: (aclObject encryptSymbolFor: each)
						for: userOrGroup 
						on: self aclObject name]
					ifFalse: 	[self observee
						removeAccessRights: (aclObject encryptSymbolFor: each)
						for: userOrGroup 
						on: self aclObject name]  ] ].! !

!WebSecurityManagerApp methodsFor: 'printing - access control' stamp: ' 21/4/07 22:07'!
checkIfEncrypted
	| right userOrGroup command |
	(self session lastRequest includesQuery: 'right') 
		ifTrue: [right := (self session lastRequest queryAt: 'right')].
	(self aclForWho = 'user') 
		ifTrue: [userOrGroup := self usr] 
		ifFalse: [userOrGroup := self group].
	(self session lastRequest includesQuery: 'encrypt') 
 		ifTrue: [command := (self session lastRequest queryAt: 'encrypt')].
	command = 'YES' ifTrue: 
		[self observee 
			setAccessRights: (aclObject encryptSymbolFor: right)
			for: userOrGroup 
			on: self aclObject name].
	command = 'NO' ifTrue: 
		[self observee 
			removeAccessRights: (aclObject encryptSymbolFor: right)
			for: userOrGroup 
			on: self aclObject name].! !

!WebSecurityManagerApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
group
	^group! !

!WebSecurityManagerApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
group: anObject
	group := anObject! !

!WebSecurityManagerApp methodsFor: 'printing - access control' stamp: ' 21/4/07 22:07'!
needsEncryption: aViewSymbol
	^self observee 
		has: self aclUserOrGroup
		rightTo: (aclObject encryptSymbolFor: aViewSymbol)
		on: self aclObject name! !

!WebSecurityManagerApp methodsFor: 'printing - access control' stamp: ' 21/4/07 22:07'!
negate: yesNoString

		yesNoString = 'YES' ifTrue: [^'NO'] ifFalse: [^'YES'].! !

!WebSecurityManagerApp methodsFor: 'printing - access control' stamp: ' 21/4/07 22:07'!
updateAllowedFor: aViewSymbol
	^self observee 
		has: self aclUserOrGroup
		rightTo: (aclObject updateRightSymbolFor: aViewSymbol)
		on: self aclObject name! !

!WebSecurityManagerApp methodsFor: 'printing - access control' stamp: ' 21/4/07 22:07'!
updateInOneGroupAllowedFor: aViewSymbol

	^self observee 
		hasUser: self aclUserOrGroup
		rightInAnyGroupTo: (aclObject class updateRightSymbolFor: aViewSymbol)
		on: self aclObject name! !

!WebSecurityManagerApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
user
	^self session user! !

!WebSecurityManagerApp methodsFor: 'printing - users' stamp: ' 21/4/07 22:07'!
userElementForm: aBoolean

	| element   |
	element := WebElement new.
	element cell addText: 'Name:'.
	aBoolean
		ifFalse: 
			[element newCell addText: self usr name attributes: #bold]
		ifTrue: 
			[element newCell add: (WebInputField new aspect: #name for: self usr)].
	element newRow.

	element cell addText: 'Surname:'.
	aBoolean
		ifFalse: 
			[element newCell addText: self usr surname attributes: #bold]
		ifTrue: 
			[element newCell add: (WebInputField new aspect: #surname for: self usr)].
	element newRow.

	element cell addText: 'Company:'.
	aBoolean
		ifFalse: 
			[element newCell addText: self usr company attributes: #bold]
		ifTrue: 
			[element newCell add: (WebInputField new aspect: #company for: self usr)].
	element newRow.

	element cell addText: 'Address:'.
	aBoolean
		ifFalse: 
			[element newCell addText: self usr address attributes: #bold]
		ifTrue: 
			[element newCell add: (WebInputField new aspect: #address for: self usr)].
	element newRow.

	element cell addText: 'City:'.
	aBoolean
		ifFalse: 
			[element newCell addText: self usr city attributes: #bold]
		ifTrue: 
			[element newCell add: (WebInputField new aspect: #city for: self usr)].
	element newRow.

	element cell addText: 'Zip:'.
	aBoolean
		ifFalse: 
			[element newCell addText: self usr zip attributes: #bold]
		ifTrue: 
			[element newCell add: (WebInputField new aspect: #zip for: self usr)].
	element newRow.

	element cell addText: 'Phone:'.
	aBoolean
		ifFalse: 
			[element newCell addText: self usr phone attributes: #bold]
		ifTrue: 
			[element newCell add: (WebInputField new aspect: #phone for: self usr)].
	element newRow.

	element cell addText: 'Fax:'.
	aBoolean
		ifFalse: 
			[element newCell addText: self usr fax attributes: #bold]
		ifTrue: 
			[element newCell add: (WebInputField new aspect: #fax for: self usr)].
	element newRow.

	element cell addText: 'E-mail:'.
	aBoolean
		ifFalse: 
			[element newCell addText: self usr email attributes: #bold]
		ifTrue: 
			[element newCell add: (WebInputField new aspect: #email for: self usr)].
	element newRow.

	element newRow.
	element cell addText: 'Username:'.
	aBoolean
		ifFalse: 
			[element newCell addText: self usr username attributes: #bold]
		ifTrue: 
			[element newCell add: (WebInputField new aspect: #username for: self usr).
			element newRow.
			element cell addText: 'Password:'.
			element newCell add: 
				(WebInputField new type: #password; aspect: #password for: self usr)].
	element newRow.
	aBoolean 
		ifTrue: 
			[element cell add: (WebButton new text: 'Save').
			element cell add: self usersGroupsLink].

	element addParagraph.
	^element! !

!WebSecurityManagerApp methodsFor: 'printing - users' stamp: 'np 1/22/2008 09:17'!
userGroupElement

	| element  groups |
	element := WebElement new.
	groups := SortedCollection 
		withAll: self usr allGroups
		sortBlock: [:a :b | a name < b name].
	element addText: 'Member of groups:' header: 5.
	groups do: [:grp |
		element
			add: ((WebLink  text: ('<dd>',grp  name) linkTo: self observee)
				view: #group;
				parameter: 'guuid' value: grp uuid);
			addBreak].
	^element! !

!WebSecurityManagerApp methodsFor: 'printing - users' stamp: ' 21/4/07 22:07'!
usersElementUsers: aUserCollection command: aCommandString newView: aViewString 
	| element users |
	element := WebElement new.
	users := SortedCollection withAll: aUserCollection sortBlock: [:a :b | a surname < b surname].
	users do: [:each | 
		element cell addLinkTo: self observee text: each someId view: aViewString
				parameter: 'uuid' value: each uuid parameter: 'command' value: aCommandString;
			addBreak]. element newCell.
	^element! !

!WebSecurityManagerApp methodsFor: 'printing - users' stamp: ' 21/4/07 22:07'!
usersGroupsLink
	| element  |
	element := WebElement new.
	element
		addLinkTo: self observee  text: (' Users ') view: 'users';
		addLinkTo: self observee  text: (' Groups ') view: 'groups'.
	^element! !

!WebSecurityManagerApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
usr
	"temporary user"
	^usr! !

!WebSecurityManagerApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
usr: aWebUser
	usr := aWebUser! !

!WebSecurityManagerApp methodsFor: 'printing - access control' stamp: 'mivsek 6/12/2007 23:12'!
viewAclObject
	| e index aclRights actionMethod yesNo |
	(self session lastRequest includesQuery: 'index') 
		ifTrue: 
			[index := (self session lastRequest queryAt: 'index').
			self aclObject: (WebApplication allWebAppClasses at: index asInteger)].
	self checkAccessRight: #view.
	self checkAccessRight: #update.
	self checkIfEncrypted.
	self checkAllYesNo.
	self	title: ('Access Rigths for: ', self aclObject name, ' on site: ', self site name).

	e := WebElement new.
	e addText: self title header: 3; addParagraph.
	self aclForWho = 'user'
		ifTrue: 
			[e addText: ('User: ', self usr someId) header: 5.
			self aclUserOrGroup: self usr]
		ifFalse: 
			[e addText: ('Group: ', self group name) header: 5.
			self aclUserOrGroup: self group].
		
	e add: self allYesNoLinks.
	aclRights := SortedCollection withAll: self aclObject allViews sortBlock: [:a : b | a < b]. 
	e table border: 1.
	e cell addText: 'view'.
	e newCell colspan: 2; addText: 'view right'.
	e newCell colspan: 2; addText: 'update right'.
	e newCell addText: 'encrypt'. e newRow.
	aclRights do: [:right |
		e cell addText: right asString attributes: #bold.
		e newCell addText: (self aclObject viewMethodForView: right asSymbol) asString.
		(self viewAllowedFor: right) ifTrue: [yesNo := 'YES'] ifFalse: [yesNo := 'NO'].
		e newCell
			add: ((WebLink  text: ('<b>', yesNo) linkTo: self observee)
				view: #aclObject;
				parameter: 'right' value: right asString;
				parameter: 'viewRight' value: (self negate: yesNo)).
		(self aclForWho = 'user') ifTrue: 
			[(self viewInOneGroupAllowedFor: right) ifTrue: [yesNo := 'YES'] ifFalse: [yesNo := 'NO'].
			e newCell addText: ('<font size=-1>', yesNo)].

		actionMethod := (self aclObject actionMethodForView: right asSymbol).
		e newCell
			addText: (actionMethod notNil ifTrue: [actionMethod asString] ifFalse: ['<br>']).
		(self updateAllowedFor: right) ifTrue: [yesNo := 'YES'] ifFalse: [yesNo := 'NO'].
		actionMethod notNil
			ifTrue: 
				[e newCell
					add: ((WebLink  text: ('<b>', yesNo) linkTo: self observee)
						view:  #aclObject;
						parameter: 'right' value: right asString;
						parameter: 'updateRight' value: (self negate: yesNo)).
				(self aclForWho = 'user') ifTrue: 
					[(self viewInOneGroupAllowedFor: right) 
						ifTrue: [yesNo := 'YES'] ifFalse: [yesNo := 'NO'].
					e newCell addText: ('<font size=-1>', yesNo)] ]
			ifFalse:
				[e newCell addText: '<br>'].
			(self needsEncryption: right) ifTrue: [yesNo := 'YES'] ifFalse: [yesNo := 'NO'].
		e newCell align: #center;
			add: ((WebLink  text: ('<b>', yesNo) linkTo: self observee)
				view: #aclObject;
				parameter: 'right' value: right asString;
				parameter: 'encrypt' value: (self negate: yesNo)).
		e newRow].
	e addParagraph; add: self usersGroupsLink.

	self pageFrameWith: e title: self title! !

!WebSecurityManagerApp methodsFor: 'printing - access control' stamp: 'mivsek 6/12/2007 23:33'!
viewAclObjects
	| e aclObjects ident |
	(self session lastRequest includesQuery: 'who') 
		ifTrue: [self aclForWho: (self session lastRequest queryAt: 'who')].
	self	title: ('Access Rigths on site: ', self site name).
	e := WebElement new.
	e addText: self title header: 3; addParagraph.
	self aclForWho = 'user'
		ifTrue: 
			[e addText: ('User: ', self usr someId) header: 5.
			self aclUserOrGroup: self usr]
		ifFalse: 
			[e addText: ('Group: ', self group name) header: 5.
			self aclUserOrGroup: self group].
	aclObjects := WebApplication allWebAppClasses.
	aclObjects do: [:app |
		ident := '<dd>'.
		app appClassLevel timesRepeat: [ident := ident, '&nbsp;&nbsp;&nbsp;&nbsp;'].
		e addText: ident.
		app allViews isEmpty
			ifFalse:
				[e add: ((WebLink  text: app name asString linkTo: self observee)
					view: #aclObject; parameter: 'index' value: (aclObjects indexOf: app) printString)]
			ifTrue: [e addText: app name asString].
		e addBreak].
	e addParagraph;	add: self usersGroupsLink.
	self pageFrameWith: e title: self title! !

!WebSecurityManagerApp methodsFor: 'printing - groups' stamp: 'mivsek 6/12/2007 23:15'!
viewAddGroup
	| e |
	self group: WebUserGroup new.
	self title: ('Adding a new group on site: ', self site name).
	e := WebElement new.
	e
		addText: self title header: 3;
		addText: 'New group name: ';
		add: (WebInputField new aspect: #name for: self group);
		addParagraph;
		add: (WebButton new text: 'Add');
		add: self usersGroupsLink.
	self pageFrameWith: e title: self title! !

!WebSecurityManagerApp methodsFor: 'printing - users' stamp: 'mivsek 6/12/2007 23:19'!
viewAddUser
	| e |
	self usr: WebUser new.
	self title: ('Adding a new user on site: ', self site name).
	e := WebElement new.
	e addText: self title header: 3; add: (self userElementForm: true).
	self pageFrameWith: e title: self title! !

!WebSecurityManagerApp methodsFor: 'printing - users' stamp: 'mivsek 6/12/2007 23:16'!
viewAddUserToGroup
	| e |
	self title: ('Adding a user to group: ', self group name, ' on site: ', self site name).
	e := WebElement new.
	e
		addText: self title header: 3;
		addText: 'Select an user to add:' attributes: #bold; 
		addParagraph;
		add: (self usersElementUsers: self observee allUsers  command: #add newView: 'group');
		addParagraph;
		add: self usersGroupsLink.
	self pageFrameWith: e title: self title! !

!WebSecurityManagerApp methodsFor: 'printing - access control' stamp: ' 21/4/07 22:07'!
viewAllowedFor: aViewSymbol
	^self observee 
		has: self aclUserOrGroup
		rightTo: (aclObject viewRightSymbolFor: aViewSymbol)
		on: self aclObject name! !

!WebSecurityManagerApp methodsFor: 'printing - groups' stamp: 'mivsek 6/12/2007 23:15'!
viewGroup
	| e users command |
	(self session lastRequest includesQuery: 'guuid') 
		ifTrue: [group := self observee groupWithUuid: (self session lastRequest queryAt: 'guuid')].
	(self session lastRequest includesQuery: 'uuid') 
		ifTrue: [self usr: (self observee userWithUuid: (self session lastRequest queryAt: 'uuid'))].
	(self session lastRequest includesQuery: 'command') 
		ifTrue: 
			[command := (self session lastRequest queryAt: 'command').
			command = 'add'  ifTrue: [self group addUser: self usr].
			command = 'remove' ifTrue: [self group removeUser: self usr] ].
	self	title: ('Group ', self group name, ' on site: ', self site name).

	e := WebElement new.
	e
		addText: ('Group ', self group name, ' on site: ', self site name) header: 3;
		addParagraph.
	users := SortedCollection 
		withAll: self group allUsers
		sortBlock: [:a :b | a surname < b surname].
	users do: [:each | 
		e cell addLinkTo: self observee text: each someId 
			view: 'user' parameter: 'uuid' value: each uuid; addBreak]. e newCell.
	e
		addParagraph;
		addLinkTo: self observee text: ' Add user ' view: 'addUserToGroup';
		addLinkTo: self observee text: ' Remove user ' view: 'removeUserFromGroup';
		add: ((WebLink text: ' Access rights ' linkTo: self observee)
			view: #aclObjects;
			parameter: 'who' value: 'group');
		add: self usersGroupsLink.

	self pageFrameWith: e title: self title! !

!WebSecurityManagerApp methodsFor: 'printing - groups' stamp: 'mivsek 6/12/2007 23:15'!
viewGroups
	| e groups |
	self title: ('Groups of users on site: ', self site name).
	e := WebElement new.
	e addText: ('Groups of users on site: ', self site name) header: 3; addParagraph.
	groups := SortedCollection 
		withAll: self observee allGroups sortBlock: [:a :b | a name < b name].
	groups do: [:grp |
		e 
			addLinkTo: self observee  text: grp name view: #group
				parameter: 'guuid' value: grp uuid;
			addText: (' (',grp allUsers size printDotString,') '); addBreak].
	e 
		addParagraph;
		addLinkTo: self observee text: ' Add group ' view: 'addGroup';
		addLinkTo: self observee text: ' Remove group ' view: 'removeGroup';
		add: self usersGroupsLink.
	self pageFrameWith: e title: self title! !

!WebSecurityManagerApp methodsFor: 'printing - access control' stamp: ' 21/4/07 22:07'!
viewInOneGroupAllowedFor: aViewSymbol
	^self observee 
		hasUser: self aclUserOrGroup
		rightInAnyGroupTo: (aclObject viewRightSymbolFor: aViewSymbol)
		on: self aclObject name! !

!WebSecurityManagerApp methodsFor: 'printing' stamp: 'mivsek 6/12/2007 23:12'!
viewMain
	| e |
	self title: ('Security management for site: ', self site name).
	e := WebElement new.
	e
		addText: ('Security management for site: ', self site name) header: 3;
		addParagraph;
		addLinkTo: self observee text: ' Users ' view: 'users';
		addText: (' (',self observee allUsers size printDotString,') ');
		addBreak;
		addLinkTo: self observee text: ' Groups ' view: 'groups';
		addText: (' (',self observee allGroups size printDotString,') ');
		addBreak.
	self pageFrameWith: e title: self title! !

!WebSecurityManagerApp methodsFor: 'printing - users' stamp: 'mivsek 6/12/2007 23:16'!
viewRemoveUser
	| e |
	self title: ('Removing a user on site: ', self site name).
	e := WebElement new.
	e
		addText: self title header: 3;
		addText: 'Select an user to remove:' attributes: #bold; 
		addParagraph;
		add: (self usersElementUsers: self observee allUsers  command: #remove newView: 'users');
		addParagraph;
		add: self usersGroupsLink.
	self pageFrameWith: e title: self title! !

!WebSecurityManagerApp methodsFor: 'printing - users' stamp: 'mivsek 6/12/2007 23:17'!
viewRemoveUserFromGroup
	| e |
	self title: ('Removing a user from group: ', self group name, ' on site: ', self site name).
	e := WebElement new.
	e
		addText: self title header: 3;
		addText: 'Select an user to remove:' attributes: #bold; 
		addParagraph;
		add: (self usersElementUsers: self group allUsers  command: #remove newView: 'group');
		addParagraph;
		add: self usersGroupsLink.
	self pageFrameWith: e title: self title! !

!WebSecurityManagerApp methodsFor: 'printing - users' stamp: 'mivsek 6/12/2007 23:17'!
viewUpdateUser
	| e |
	self title: ('Updating user: ', self usr email, ' on site: ', self site name).
	e := WebElement new.
	e addText: self title header: 3; add: (self userElementForm: true).
	self pageFrameWith: e title: self title! !

!WebSecurityManagerApp methodsFor: 'printing - users' stamp: 'mivsek 6/12/2007 23:17'!
viewUser
	| e |
	(self session lastRequest includesQuery: 'uuid') 
		ifTrue: [self usr: (self observee userWithUuid: (self session  lastRequest queryAt: 'uuid'))].
	(self session  lastRequest includesQuery: 'uuid') ifTrue: 
		[self usr: (self observee userWithUuid: (self session  lastRequest queryAt: 'uuid'))].
	self clear. self title: ('User ', self usr someId, ' on server: ', self session  site name).
	e := WebElement new.
	self usr isNil ifTrue: 
		[e addText: 'User with that e-mail address does not exist!!' header: 5. ^self].
	e
		addText: self title header: 3;
		add: (self userElementForm: false);
		add: (self userGroupElement);
		addParagraph;
		addLinkTo: self observee text: ' Update information ' view: 'updateUser';
		add: ((WebLink text: ' Access rights ' linkTo: self observee)
			view: #aclObjects;
			parameter: 'who' value: 'user');
		add: self usersGroupsLink.
	self pageFrameWith: e title: self title! !

!WebSecurityManagerApp methodsFor: 'printing - users' stamp: 'mivsek 6/12/2007 23:17'!
viewUsers
	| e users command |
	(self session lastRequest includesQuery: 'command') ifTrue: 
		[command := (self session lastRequest queryAt: 'command').
		command = 'remove' ifTrue: [self observee removeUser: self usr] ].
	self title: ('Users on server: ', self site name).
	e := WebElement new.
	e addText: ('Users on server: ', self site name) header: 3; addParagraph.
	users := SortedCollection 
		withAll: self observee allUsers sortBlock: [:a :b | a surname < b surname].
	users do: [:each | e cell addLinkTo: self observee text: each someId 
		view: 'user' parameter: 'uuid' value: each uuid; addBreak]. e newCell.
	e addParagraph;
		addLinkTo: self observee text: ' Add user ' view: 'addUser';
		addLinkTo: self observee text: ' Remove user ' view: 'removeUser';
		add: self usersGroupsLink.
	self pageFrameWith: e title: self title! !

WebFrameApp subclass: #WebStatisticsApp
	instanceVariableNames: 'domainObject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Admin'!

!WebStatisticsApp methodsFor: 'accessing' stamp: 'mivsek 1/10/2008 22:54'!
counterForUrlOrFirstPage
	"if url query part contains and url for target domain object to show counter, return that
  	otherwise first page counter only"
	"Also sets instvar domainObject!!"
	| url |
	url := (self session lastRequest includesQuery: 'url')
		ifTrue: [self session lastRequest queryAt: 'url'] ifFalse: [nil].
	self domainObject: (url notNil 
			ifTrue: [self site urlResolver ooRefFromURL: url]
			ifFalse: [self firstPageObject notNil ifTrue: [self firstPageObject] ifFalse: [nil] ]).
	^self domainObject notNil 
		ifTrue: [self site urlResolver counterFor: self domainObject]
		ifFalse: [nil]! !

!WebStatisticsApp methodsFor: 'printing-elements' stamp: 'JM 4/21/2007 22:35'!
currentWeekStats
	| firstDay collection |
	firstDay := Date today - Date today weekdayIndex + 1.
	collection := OrderedCollection new.
	firstDay to: firstDay + 6 do: [:dte |
		collection add: (self site requestsOnDate: dte)].
	^collection! !

!WebStatisticsApp methodsFor: 'accessing' stamp: 'mivsek 1/10/2008 22:52'!
domainObject
	"for which we show statistics"
	^domainObject! !

!WebStatisticsApp methodsFor: 'accessing' stamp: 'mivsek 1/10/2008 22:52'!
domainObject: anObject
	domainObject := anObject! !

!WebStatisticsApp methodsFor: 'printing-elements' stamp: ' 21/4/07 22:07'!
firstPageCountOnDate: aDate
	| countedObject |
	countedObject := self firstPageObject.
	countedObject isNil ifTrue: [^0].
	^(self site urlResolver counterFor: countedObject) countsOnDate: aDate.! !

!WebStatisticsApp methodsFor: 'printing-elements' stamp: ' 21/4/07 22:07'!
firstPageCountTotal
	| countedObject |
	countedObject := self firstPageObject.
	countedObject isNil ifTrue: [^0].
	^(self site urlResolver counterFor: countedObject) total.! !

!WebStatisticsApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
firstPageObject
	^self site urlResolver ooRefFromURL: '/'! !

!WebStatisticsApp methodsFor: 'printing-graphs' stamp: ' 21/8/07 09:38'!
graphElementCounter: aFirstCounter and: aSecondCounter
	| e |
	e := WebElement new. e table width: self pageWidth.
	(self session lastRequest includesQuery: 'yearly') 
		ifTrue: [e cell valign: #top; add: (self yearlyStatsCounter: aFirstCounter and: aSecondCounter)]
		ifFalse: 
			[e cell valign: #top; add: (self monthlyStatsCounter: aFirstCounter and: aSecondCounter).
			e newCell addNbSp.
			e newCell valign: #top; add: (self hourlyStatsCounter: aFirstCounter and: aSecondCounter)].
	^e! !

!WebStatisticsApp methodsFor: 'printing-graphs' stamp: ' 21/8/07 09:38'!
graphElementCounter: aFirstCounter total: aTotalCounter
	| e |
	e := WebElement new.
	e table width: self pageWidth.
	e cell valign: #top; add: (self monthlyStatsCounter: aFirstCounter total: aTotalCounter).
	e newCell addNbSp.
	e newCell valign: #top; add: (self hourlyStatsCounter: aFirstCounter total: aTotalCounter).
	^e! !

!WebStatisticsApp methodsFor: 'printing-graphs' stamp: ' 21/8/07 09:38'!
graphLineCount: firstCount and: secondCount max: maxCount
	| e width maxLength |
	width := 6. maxLength := 148.
	e := WebElement new.
	e addGif: #redPixelGif size: ((maxLength * firstCount // (maxCount max: 1)) max: 2)@width.
	(secondCount notNil and: [secondCount > 0]) ifTrue:
		[e addGif: #bluePixelGif size: ((maxLength * secondCount // maxCount) max: 2)@width].
	^e! !

!WebStatisticsApp methodsFor: 'printing-graphs' stamp: ' 21/4/07 22:07'!
graphLineCount: count max: maxCount

	^self graphLineCount: count and: nil max: maxCount! !

!WebStatisticsApp methodsFor: 'printing-graphs' stamp: ' 21/4/07 22:07'!
hourlyStatsCounter: aFirstCounter and: aTotalCounter

	| firstArray secondArray |
	firstArray := aFirstCounter allHourlyCounts.
	secondArray := aTotalCounter notNil
		ifTrue: [aTotalCounter allHourlyCounts]
		ifFalse: [Array new: firstArray size withAll: 0].
	^self hourlyStatsFrom: firstArray and: secondArray! !

!WebStatisticsApp methodsFor: 'printing-graphs' stamp: ' 21/4/07 22:07'!
hourlyStatsCounter: aFirstCounter total: aTotalCounter

	| firstArray secondArray |
	firstArray := aFirstCounter allHourlyCounts.
	secondArray := self subtractArray: firstArray
		from: aTotalCounter allHourlyCounts.
	^self hourlyStatsFrom: firstArray and: secondArray! !

!WebStatisticsApp methodsFor: 'printing-graphs' stamp: ' 21/8/07 09:38'!
hourlyStatsFrom: aFirstArray and: aSecondArray
	| e maxCount turn sumArray |
	sumArray := aSecondArray notNil 
		ifTrue: [self sumArray: aFirstArray and: aSecondArray]
		ifFalse: [aFirstArray].
	maxCount := sumArray inject: 0 into: [:max :each | each max: max].
	e := WebElement new.
	e table width: 250; cellSpacing: 0; cellPadding: 0.
	e newRow color: self 	tableHeaderColor.
	e cell width: 50; add: (self smallText: 'hour').
	e newCell width: 150; addBreak.
	e newCell width: 50; align: #right; add: (self smallText: 'hits').
	e newRow.
	turn := true.
	1 to: 24 do: [:hour |
		e newRow.
		(hour-1) \\ 6 = 0 ifTrue: [turn := turn not].
		turn ifTrue: [e row color: self tableRowColor].
		e cell add: (self smallText: ( (hour-1) printString, ' - ', (hour) printString)).
		e newCell add: 
			(self graphLineCount: (aFirstArray at: hour) and: (aSecondArray at: hour) max: maxCount).
		e newCell align: #right; add: (self smallText: (sumArray at: hour) printDotString).
		].
	^e! !

!WebStatisticsApp methodsFor: 'testing' stamp: 'mivsek 1/10/2008 22:54'!
isFirstPage
	^self domainObject == self firstPageObject! !

!WebStatisticsApp methodsFor: 'printing-elements' stamp: 'mivsek 8/21/2007 11:41'!
mainStatsElement
	| element |
	self title: 'Site  access statistics'.
	element := WebElement new.
	element table width: self pageWidth; cellSpacing: 0; cellPadding: 0.
      element cell color: self tableHeaderColor.
	element cell colspan: 4; addText: self title header: 3.
	element newRow.
	element cell colspan: 4; addRulerSize: 1.
	element newRow.
	element cell colspan: 4;
		addBreak;
		add: (self smallText: 'statistics from: ');
		add: (self smallText: (self printStamp: self session site createdTimestamp) attributes: #bold);
		add: (self smallText: ' to: ');
		add: (self smallText: (self printStamp: SpTimestamp now) attributes: #bold);
		addBreak; addNbSp.
	element newRow.
	element cell colspan: 4; addRulerSize: 1.
	element newRow color: self tableHeaderColor.
	element cell addBreak.
	element newCell align: #right; add: (self smallText: 'today').
	element newCell align: #right; add: (self smallText: 'yesterday').
	element newCell align: #right; add: (self smallText: 'until now').
	element newRow.
	element cell add: (self smallText: 'first page').
	element newCell align: #right; 
		add: (self smallText: ((self firstPageCountOnDate: Date today) printDotString)).
	element newCell align: #right; 
		add: (self smallText: ((self firstPageCountOnDate: Date today - 1) printDotString)).
	element newCell align: #right; 
		add: (self smallText: ((self firstPageCountTotal) printDotString)).
	element newRow color: self tableRowColor.
	element cell add: (self smallText: 'all pages').
	element newCell align: #right; 
		add: (self smallText: ((self site pagesCounter countsOnDate: Date today) printDotString)).
	element newCell align: #right; 
		add: (self smallText: ((self site pagesCounter countsOnDate: Date today - 1) printDotString)).
	element newCell align: #right; 
		add: (self smallText: ((self site pagesCounter total) printDotString)).
	element newRow.
	element cell add: (self smallText: 'all pages+images').
	element newCell align: #right; 
		add: (self smallText: ((self site totalCounter countsOnDate: Date today) printDotString)).
	element newCell align: #right; 
		add: (self smallText: ((self site totalCounter countsOnDate: Date today - 1) printDotString)).
	element newCell align: #right; 
		add: (self smallText: ((self site totalCounter total ) printDotString)).
	element newRow.
	element cell colspan: 4; addRulerSize: 1.
	^element! !

!WebStatisticsApp methodsFor: 'printing-graphs' stamp: ' 21/4/07 22:07'!
monthlyStatsCounter: aFirstCounter and: aSecondCounter

	| firstArray secondArray |
	firstArray := aFirstCounter
		countsFrom: self firstDayInMonth to: self lastDayInMonth.
	secondArray := aSecondCounter notNil 
		ifTrue: [aSecondCounter countsFrom: self firstDayInMonth to: self lastDayInMonth]
		ifFalse: [Array new: firstArray size withAll: 0].
	^self monthlyStatsDate: self firstDayInMonth from: firstArray and: secondArray! !

!WebStatisticsApp methodsFor: 'printing-graphs' stamp: ' 21/4/07 22:07'!
monthlyStatsCounter: aFirstCounter total: aTotalCounter

	| firstArray secondArray |
	firstArray := aFirstCounter
		countsFrom: self firstDayInMonth to: self lastDayInMonth.
	secondArray := self subtractArray: firstArray
		from: (aTotalCounter 
			countsFrom: self firstDayInMonth to: self lastDayInMonth).
	^self monthlyStatsDate: self firstDayInMonth from: firstArray and: secondArray! !

!WebStatisticsApp methodsFor: 'printing-graphs' stamp: 'mivsek 1/13/2008 20:28'!
monthlyStatsDate: aDate from: aFirstArray and: aSecondArray
	| e maxCount turn sumArray cDate weekday monthText yearLink |
	sumArray := aSecondArray notNil 
		ifTrue: [self sumArray: aFirstArray and: aSecondArray]
		ifFalse: [aFirstArray].
	maxCount := sumArray inject: 0 into: [:max :each | each max: max].
	e := WebElement new.
	e table width: 300; cellSpacing: 0; cellPadding: 0.
	e newRow color: self 	tableHeaderColor.
	monthText := (Date nameOfMonth: aDate monthIndex), ' ', aDate year printString.
	yearLink := (WebLink text: '<font face="helvetica">yearly</font>' linkTo: self observee)
		parameter: 'view' value: self view; parameter: 'yearly' value: 'true'.
	e cell  colspan: 3;
		add: self previousMonthLink;
		add: (self smallText: ' ', monthText, ' ');
		add: self nextMonthLink;
		addNbSp: 3; add: yearLink.
	"e newCell width: 150; addBreak."
	e newCell width: 50; align: #right; add: (self smallText: 'hits').
	e newRow.
	turn := false.
	1 to: aFirstArray size do: [:day |
		e newRow.
		cDate := aDate addDays: day - 1.
		weekday := cDate weekdayIndex.
		weekday = 1 ifTrue: [turn := turn not].
		turn ifTrue: [e row color: self tableRowColor].
		e cell add: (self smallText: 
			((Date nameOfDay: weekday) copyFrom: 1 to: 3) ).
		e newCell align: #right; add: (self smallText: 
			(day printString, '.', aDate monthIndex printString, ' ' ) ).
		e newCell add: 
			(self graphLineCount: (aFirstArray at: day) and: (aSecondArray at: day) max: maxCount).
		e newCell align: #right; add: (self smallText: (sumArray at: day) printDotString).
		].
	^e! !

!WebStatisticsApp methodsFor: 'printing-graphs' stamp: ' 21/4/07 22:07'!
monthlyStatsFor: aFirstCounter total: aTotalCounter

	| firstArray secondArray |
	firstArray := aFirstCounter
		countsFrom: self firstDayInMonth to: self lastDayInMonth.
	secondArray := self subtractArray: firstArray
		from: (aTotalCounter 
			countsFrom: self firstDayInMonth to: self lastDayInMonth).
	^self monthlyStatsDate: self firstDayInMonth from: firstArray and: secondArray! !

!WebStatisticsApp methodsFor: 'printing-elements' stamp: ' 21/4/07 22:07'!
nextMonthLink
	self currentMonth ifTrue: [^WebText text: '<font face="helvetica">></font>' ].
	^(WebLink text: '<font face="helvetica">></font>' linkTo: self observee)
		parameter: 'view' value: self view;
		parameter: 'month' value: self firstDayInNextMonth monthIndex printString;
		parameter: 'year' value: self firstDayInNextMonth year printString.! !

!WebStatisticsApp methodsFor: 'printing-elements' stamp: ' 21/4/07 22:07'!
nextYearInLink: aWebLink

	aWebLink
		parameter: 'month' value: self firstDayInNextYear monthIndex printString;
		parameter: 'year' value: self firstDayInNextYear year printString.! !

!WebStatisticsApp methodsFor: 'printing-elements' stamp: 'mivsek 1/10/2008 22:55'!
pageDescription
	| e url |
	e := WebElement new.
	url := self site urlResolver halfUrlFor: self domainObject.
	e addText: 'For page: '; addLinkTo: url text: url.
	^e! !

!WebStatisticsApp methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
pageWidth
	^super pageWidth - 150 "actually from WebStyle"! !

!WebStatisticsApp methodsFor: 'printing-elements' stamp: ' 21/4/07 22:07'!
previousMonthLink
	^(WebLink text: '<font face="helvetica"><</font>' linkTo: self observee)
		parameter: 'view' value: self view;
		parameter: 'month' value: self firstDayInPreviousMonth monthIndex printString;
		parameter: 'year' value: self firstDayInPreviousMonth year printString.! !

!WebStatisticsApp methodsFor: 'printing-elements' stamp: ' 21/4/07 22:07'!
previousYearInLink: aWebLink

	aWebLink
		parameter: 'month' value: self firstDayInPreviousYear monthIndex printString;
		parameter: 'year' value: self firstDayInPreviousYear year printString.! !

!WebStatisticsApp methodsFor: 'printing-elements' stamp: ' 21/4/07 22:07'!
printCounter
	| element count |
	element := WebElement new.
	count := self firstPageCountTotal.
	element 
		addText: '<font face="arial, helvetica"> <b>';
		addText: count printDotString;
		addText: '</b></font>'.
	^element.! !

!WebStatisticsApp methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
refererLinesOnPage
	^50! !

!WebStatisticsApp methodsFor: 'printing-elements' stamp: ' 21/8/07 09:38'!
referersElementFor: aCollection
	| e referers nr max |
	e := WebElement new.
	e table width: self pageWidth; cellSpacing: 0; cellPadding: 0.
	referers := aCollection copyFrom: 1 to: (self refererLinesOnPage min: aCollection size).
	max := referers inject: 0 into: [:maxi :each | maxi max: (each at: 2)].
	nr := 1.
	referers do: [:array |
		e cell align: #right; addTextSmall: nr printString, '. '.
		e newCell add: 
			(WebLink new 
				text: '<font size=-1>', (array at: 1), '</font>';
				urlReference: (array at: 1)).
"		e cell add:
			(WebLink new text: '	stat';
				ooReference: self observee;
				parameter: 'view' value: 'referers';
				parameter: 'url' value: (array at: 1))."
		e newCell add: (self graphLineCount: (array at: 2) max: max "length: 80").
		e newCell align: #right; addTextSmall: ' ', (array at: 2) printDotString. 	e newRow.
		nr even ifTrue: [e row color: self tableRowColor].
		nr := nr + 1].
	e cell colspan: 4; addRulerSize: 1.
	^e! !

!WebStatisticsApp methodsFor: 'accessing' stamp: 'JM 4/26/2007 21:35'!
statsStartingDate
	^SpDate newDay: 1 month: 1 year: 2005.! !

!WebStatisticsApp methodsFor: 'printing-graphs' stamp: ' 21/4/07 22:07'!
subtractArray: aFirstArray from: aSecondArray

	| result |
	result := OrderedCollection new.
	aFirstArray with: aSecondArray do: [:first :second | result add: second - first].
	^result.

"WebStatisticsApp new subtractArray: #(1 2 3 4) from: #(1 2 3 4)"! !

!WebStatisticsApp methodsFor: 'printing-graphs' stamp: ' 21/4/07 22:07'!
sumArray: aFirstArray and: aSecondArray

	| result |
	result := OrderedCollection new.
	aFirstArray with: aSecondArray do: [:first :second | result add: first + second].
	^result.

"WebStatisticsApp new sumArray: #(1 2 3 4) and: #(1 2 3 4)"! !

!WebStatisticsApp methodsFor: 'printing-elements' stamp: 'mivsek 1/10/2008 22:55'!
tabsElement
	^ViewTabs new
		addView: #main description: 
			(self isFirstPage ifTrue: ['First page'] ifFalse: ['Selected page']);
"		addView: #hits description: 'All hits';"
		addView: #referers description: 'Referers';
		yourself! !

!WebStatisticsApp methodsFor: 'printing-elements' stamp: ' 21/8/07 09:38'!
tabsReferersElement
	^ViewTabs new
		addView: #referers description: 'Today';
		addView: #referersYesterday description: 'Yesterday';
		addView: #referersWeekly description: 'That week';
		addView: #referersAll description: 'All';
		yourself! !

!WebStatisticsApp methodsFor: 'printing' stamp: ' 21/8/07 09:38'!
viewHits
	| e |
	self title: 'Site  access statistics'.
	self adjustViewingDate.
	e := WebElement new.
	e cell add: self mainStatsElement. e newRow.
	e cell add: self tabsElement. e newRow.
	e cell add: (self graphElementCounter: self site pagesCounter total: self site totalCounter).
	^self pageFrameWith: e title: self title .! !

!WebStatisticsApp methodsFor: 'printing' stamp: 'mivsek 1/10/2008 22:56'!
viewMain
	| e counter |
	self title: 'Site statistics - page'.
	self adjustViewingDate.
	counter := self counterForUrlOrFirstPage. "from query part of url, otherwise first page"
	e := WebElement new.
	e cell add: self mainStatsElement. e newRow.
	e cell add: self tabsElement. e newRow.
	self isFirstPage ifFalse: [e cell add: self pageDescription. e newRow].
	counter notNil ifTrue: 
		[e cell add: (self graphElementCounter: counter and:nil)].
	^self pageFrameWith: e title: self title .! !

!WebStatisticsApp methodsFor: 'printing' stamp: ' 21/8/07 09:38'!
viewReferers "Today"
	| e |
	self title: 'Site statistics - referers'.
	self adjustViewingDate.
	e := WebElement new.
	e cell add: self mainStatsElement. e newRow.
	e cell add: self tabsElement. e newRow.
	e cell add: self tabsReferersElement. e newRow.
	e cell addTextH1: 'Today referers - visitors comming from: '. e newRow.
	e cell add: (self referersElementFor: self observee allRefererCountsToday).
	^self pageFrameWith: e title: self title .! !

!WebStatisticsApp methodsFor: 'printing' stamp: ' 21/8/07 09:38'!
viewReferersAll
	| e |
	self title: 'Site statistics - referers'.
	self adjustViewingDate.
	e := WebElement new.
	e cell add: self mainStatsElement. e newRow.
	e cell add: self tabsElement. e newRow.
	e cell add: self tabsReferersElement. e newRow.
	e cell addTextH1: 'Referers - visitors comming mostly from: '. e newRow.
	e cell add: (self referersElementFor: self observee allRefererCounts).
	^self pageFrameWith: e title: self title .! !

!WebStatisticsApp methodsFor: 'printing' stamp: ' 21/8/07 09:38'!
viewReferersWeekly
	| e |
	self title: 'Site statistics - referers'.
	self adjustViewingDate.
	e := WebElement new.
	e cell add: self mainStatsElement. e newRow.
	e cell add: self tabsElement. e newRow.
	e cell add: self tabsReferersElement. e newRow.
	e cell addTextH1: 'That week referers - visitors comming from: '. e newRow.
	e cell add: (self referersElementFor: self observee allRefererCountsWeekly).
	^self pageFrameWith: e title: self title .! !

!WebStatisticsApp methodsFor: 'printing' stamp: ' 21/8/07 09:38'!
viewReferersYesterday
	| e |
	self title: 'Site statistics - referers'.
	self adjustViewingDate.
	e := WebElement new.
	e cell add: self mainStatsElement. e newRow.
	e cell add: self tabsElement. e newRow.
	e cell add: self tabsReferersElement. e newRow.
	e cell addTextH1: 'Yesterday referers - visitors comming from: '. e newRow.
	e cell add: (self referersElementFor: self observee allRefererCountsYesterday).
	^self pageFrameWith: e title: self title .! !

!WebStatisticsApp methodsFor: 'printing' stamp: ' 21/8/07 09:38'!
viewVisitors
	| e |
	self adjustViewingDate.
	self title: 'Site statistics - returning visitors'.
	e := WebElement new.
	e cell add: self mainStatsElement. e newRow.
	e cell add: (self graphSelectionForView: self view). e newRow.
	e cell add: (self graphElementCounter: self site returningVisitorsCounter 
		and: self site newVisitorsCounter).
	^self pageFrameWith: e title: self title .! !

!WebStatisticsApp methodsFor: 'printing-graphs' stamp: ' 21/4/07 22:07'!
weeklyStatsElement

	| firstArray secondArray |
	firstArray := self site pagesCounter currentWeekCounts.
	secondArray := self subtractArray: firstArray
		from: self site totalCounter currentWeekCounts.
	^self weeklyStatsFrom: firstArray and: secondArray! !

!WebStatisticsApp methodsFor: 'printing-graphs' stamp: ' 21/8/07 09:38'!
weeklyStatsFrom: aFirstArray and: aSecondArray

	| e maxCount turn sumArray |
	sumArray := aSecondArray notNil 
		ifTrue: [self sumArray: aFirstArray and: aSecondArray]
		ifFalse: [aFirstArray].
	maxCount := sumArray inject: 0 into: [:max :each | each max: max].
	e := WebElement new.
	e table width: 400; cellSpacing: 0; cellPadding: 0.
	e newRow color: self 	tableHeaderColor.
	e cell width: 50; add: (self smallText: 'this week').
	e newCell width: 300; addBreak.
	e newCell width: 50; align: #right; add: (self smallText: 'hits').
	e newRow.
	turn := false.
	1 to: 7 do: [:day |
		e newRow.
		turn ifTrue: [e row color: self tableRowColor].
		e cell add: (self smallText: ((Date nameOfDay: day) copyFrom: 1 to: 3) ).
		e newCell add: 
			(self graphLineCount: (aFirstArray at: day) and: (aSecondArray at: day) max: maxCount).
		e newCell align: #right; add: (self smallText: (sumArray at: day) printDotString).
		turn := turn not].
	^e! !

!WebStatisticsApp methodsFor: 'printing-graphs' stamp: ' 21/4/07 22:07'!
yearlyStatsCounter: aFirstCounter and: aSecondCounter

	| firstArray secondArray |
	firstArray := aFirstCounter countsMonthlyFromDate: self statsStartingDate.
	secondArray := aSecondCounter notNil 
		ifTrue: [aSecondCounter  countsMonthlyFromDate: self statsStartingDate.]
		ifFalse: [Array new: firstArray size withAll: 0].
	^self yearlyStatsStarting: self statsStartingDate from: firstArray and: secondArray! !

!WebStatisticsApp methodsFor: 'printing-graphs' stamp: ' 21/4/07 22:07'!
yearlyStatsCounter: aFirstCounter total: aTotalCounter

	| firstArray secondArray |
	firstArray := aFirstCounter countsMonthlyFromDate: self statsStartingDate.
	secondArray := self subtractArray: firstArray
		from: (aTotalCounter  countsMonthlyFromDate: self statsStartingDate).
	^self yearlyStatsStarting: self statsStartingDate from: firstArray and: secondArray! !

!WebStatisticsApp methodsFor: 'printing-graphs' stamp: ' 21/4/07 22:07'!
yearlyStatsFor: aFirstCounter total: aTotalCounter

	| firstArray secondArray |
	firstArray := aFirstCounter countsMonthlyFromDate: self statsStartingDate.
	secondArray := self subtractArray: firstArray
		from: (aTotalCounter  countsMonthlyFromDate: self statsStartingDate).
	^self yearlyStatsStarting: self statsStartingDate from: firstArray and: secondArray! !

!WebStatisticsApp methodsFor: 'printing-graphs' stamp: 'mivsek 3/7/2008 19:15'!
yearlyStatsStarting: aDate from: aFirstArray and: aSecondArray

	| e maxCount turn sumArray cdate |
	sumArray := aSecondArray notNil 
		ifTrue: [self sumArray: aFirstArray and: aSecondArray]
		ifFalse: [aFirstArray].
	maxCount := sumArray inject: 0 into: [:max :each | each max: max].
	e := WebElement new.
	e table width: 300; cellSpacing: 0; cellPadding: 0.
	e newRow color: self 	tableHeaderColor.
	e cell addNbSp. 	e newCell addNbSp.
	e newCell width: 150; addBreak.
	e newCell width: 50; align: #right; add: (self smallText: 'hits').
	e newRow.
	turn := false.
	cdate := SpDate newDay: 1
		monthNumber: aDate monthIndex
		year: aDate year.
	1 to: aFirstArray size do: [:inx |
		e newRow.
		cdate monthIndex \\ 3 = 1 ifTrue: [turn := turn not].
		turn ifTrue: [e row color: self tableRowColor].
		e cell add: (self smallText: 
			((Date nameOfMonth: cdate monthIndex) asString copyFrom: 1 to: 3) ).
		e newCell add: (self smallText: ((cdate year ) printString  ) ).
		e newCell add: 
			(self graphLineCount: (aFirstArray at: inx) 
				and: (aSecondArray at: inx) max: maxCount).
		e newCell align: #right; add: (self smallText: (sumArray at: inx) printDotString).
		cdate := SpDate
			newDay: 1
			monthNumber: (cdate addDays: 31) monthIndex
			year: (cdate addDays: 31) year
		].
	^e! !

WebApplication subclass: #WebHelpApp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Admin'!

!WebHelpApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewNewPage
	| appName viewName helpPage |
	appName := (self session lastRequest queryAt: 'app') asSymbol.
	viewName := (self session lastRequest queryAt: 'hview') asSymbol.
	helpPage := WebHelpPage newForApp: appName view: viewName parent: self observee.
	self redirectTo: helpPage view: #new.! !

WebApplication subclass: #WebHelpPageApp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Admin'!

!WebHelpPageApp methodsFor: 'actions' stamp: ' 21/4/07 22:07'!
actionEdit
	self newView: #main! !

!WebHelpPageApp methodsFor: 'actions' stamp: ' 21/4/07 22:07'!
actionMainEdit
	self redirectToView: #edit! !

!WebHelpPageApp methodsFor: 'actions' stamp: ' 21/4/07 22:07'!
actionNew
	self observee parent add: self observee.
	self newView: #main! !

!WebHelpPageApp methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
canBeEdited
	^self user isAdmin! !

!WebHelpPageApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewEdit
	| e |
	e := WebElement new.
	e cell addTextH4: 'Help page for ', self observee app asString, ', view #', self observee view. 
	e newRow.
	e cell addText: 'Title'; addBreak; 
		addInputFieldAspect: #title for: self observee size: 79. e newRow.
	e cell addText: 'Contents'; addBreak; 
		addRichEditorAspect: #body for: self observee size: 80@20. e newRow.
	e cell addButtonText: 'Save help page'.
	self pageFrameWith: e title: self title! !

!WebHelpPageApp methodsFor: 'printing' stamp: 'np 3/22/2008 03:59'!
viewMain
	| e |
	e := WebElement new.
	e table width: 1 "100%".
	e cell addTextH1: 'Help: ', self observee title. e newRow.
	self canBeEdited ifTrue: [e cell addButtonText: 'Edit' action: #edit. e newRow].
	e cell addText: self observee body asWikiLinksOnlyHtml.
	self pageFrameWith: e title: self title! !

!WebHelpPageApp methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
viewNew
	self viewEdit! !

WebApplication subclass: #WebMsgsApp
	instanceVariableNames: 'webMsgs assocs texts'
	classVariableNames: 'Instance'
	poolDictionaries: ''
	category: 'Aida-Admin'!

!WebMsgsApp methodsFor: 'actions' stamp: 'mivsek 6/13/2007 16:38'!
actionMessages
	| i |
	i := 1.
	self assocs do: [:each |
		(self observee messagesForCurrentLanguage at: (each key)
			put: (self texts at: i) ).
		i := i +1].! !

!WebMsgsApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
assocs


	^assocs.! !

!WebMsgsApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
assocs: aCollection

	"used in ordering of classifcation dictionary"
	assocs := aCollection.! !

!WebMsgsApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
texts
	^texts! !

!WebMsgsApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
texts: anOrderedCollection

	texts := anOrderedCollection.! !

!WebMsgsApp methodsFor: 'printing' stamp: 'mivsek 6/13/2007 16:39'!
viewMessages
	"show/update all web messages"
	| table row cell i |
	self title: (WebMsgs at: #msgTitle).
	self  title: self title
.	self 	addText: self title header: 2; addRulerSize: 2.
	self assocs: (SortedCollection 
		withAll: (self observee messagesForCurrentLanguage associations)
		sortBlock: [:elem1 :elem2 | (elem1) < (elem2)]).
	self texts: (self assocs collect: [:each | each value]).
	table := WebTable new width: self pageWidth; border: 1.
	row := WebTableRow new.
	cell := WebTableCell new.
	cell addMessage: #msgTbSymbol. row add: cell.
	cell := WebTableCell new.
	cell addText: #msgTbText. row add: cell. 
	i := 1.
	self assocs do: 
		[:each |
			row := WebTableRow new.
			cell := WebTableCell new.
			cell addText: each key asString attributes: #bold. row add:cell.
			cell := WebTableCell new.
			cell add: 
				(WebInputField new size: 60; aspect: i for: self texts). 
			row add:cell. table add: row. 
			i := i + 1.].
		self add: table; addRulerSize: 2.
		self add: (WebButton new text: 'Zapisi').! !

!WebMsgsApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
webMsgs

	^webMsgs! !

!WebMsgsApp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
webMsgs: aWebMsgs

	webMsgs := aWebMsgs.! !

!WebPage class methodsFor: 'initialize' stamp: ' 21/4/07 22:07'!
buildNotFoundPage

	"this page report an URL not found error"

	| page |
	page := WebPage new.
	page
		name: 'URLNotFound';
		title: 'URL not found error'.
	page
		add: 
			((WebText
				text: 'Error')
				header: 1);
		add: (WebSeparator rulerSize: 5);
		add:
			((WebText 
				text: 'Document with this URL not found on our server, 
						Please correct and try again. Thank you.')
				attributes: #bold);
		add: (WebSeparator rulerSize: 5);
		add:
			((WebText
				text: 'AIDA/Web, Smalltalk Web Server, (C) EraNova d.o.o., Ljubljana, Slovenia')).

	^page! !

!WebPage class methodsFor: 'initialize' stamp: ' 21/4/07 22:07'!
initWellKnownPages

	"make a dictionary of well known pages, such as error reporting etc."

	WellKnownPages := Dictionary new.
	WellKnownPages at: #URLNotFound put: self buildNotFoundPage.

"WebPage initWellKnownPages"! !

!WebPage class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newOn: aSession
	^self new session: aSession! !

!WebPage class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
wellKnownPages

	"return a dictionary of well known WebPages, such as error reporting 	pages etc."

"	(WellKnownPages notNil)
		ifTrue: [^WellKnownPages]
		ifFalse: [self initWellKnownPages. #WellKnownPages]
"
	self initWellKnownPages.
	^WellKnownPages! !

!WebPage methodsFor: 'private' stamp: ' 21/4/07 22:07'!
addHeader: aTagString value: aString
	self headers isNil ifTrue: [self initHeaders].
	self headers add: (aTagString -> aString)! !

!WebPage methodsFor: 'header elements' stamp: ' 21/4/07 22:07'!
addJavascript: aScriptString
	self addScriptWithParms: 'language="JavaScript" type="text/javascript"' script: aScriptString! !

!WebPage methodsFor: 'header elements' stamp: ' 21/4/07 22:07'!
addLinkToAlternateScreenStyleSheet: anObjectOrUrl
	| url |
	url := anObjectOrUrl isString
		ifTrue: [anObjectOrUrl]
		ifFalse: [self site urlResolver halfUrlFor: anObjectOrUrl].
	self addHeader: 'link' value: 'rel="alternate stylesheet" type="text/css" media="screen" href="', url, '" '! !

!WebPage methodsFor: 'header elements' stamp: ' 21/4/07 22:07'!
addLinkToJavascript: anObjectOrUrl
	| url |
	url := anObjectOrUrl isString
		ifTrue: [anObjectOrUrl]
		ifFalse: [self site urlResolver halfUrlFor: anObjectOrUrl].
	self addScriptWithParms: ' src="', url, '" language="JavaScript" type="text/javascript"' script: ''.! !

!WebPage methodsFor: 'header elements' stamp: ' 21/4/07 22:07'!
addLinkToPrintStyleSheet: anObjectOrUrl
	| url |
	url := anObjectOrUrl isString
		ifTrue: [anObjectOrUrl]
		ifFalse: [self site urlResolver halfUrlFor: anObjectOrUrl].
	self addHeader: 'link' value: 'rel="stylesheet" type="text/css" media="print" href="', url, '" '! !

!WebPage methodsFor: 'header elements' stamp: ' 21/4/07 22:07'!
addLinkToScreenStyleSheet: anObjectOrUrl
	| url |
	url := anObjectOrUrl isString
		ifTrue: [anObjectOrUrl]
		ifFalse: [self site urlResolver halfUrlFor: anObjectOrUrl].
	self addHeader: 'link' value: 'rel="stylesheet" type="text/css" media="screen" href="', url, '" '! !

!WebPage methodsFor: 'header elements' stamp: ' 21/4/07 22:07'!
addMetaContentType: aString
	"example: 'text/html; charset=iso8859-2'"
	self addHeader: 'meta' value: 'http-equiv="Content-Type"  content="', aString, '" '! !

!WebPage methodsFor: 'header elements' stamp: ' 21/4/07 22:07'!
addMetaDescription: aString
	self addHeader: 'meta' value: 'name="description" content="', aString, '"'! !

!WebPage methodsFor: 'header elements' stamp: ' 21/4/07 22:07'!
addMetaKeywords: aString
	"keywords separated with , "
	self addHeader: 'meta' value: 'name="keywords" content="', aString, '"'! !

!WebPage methodsFor: 'private' stamp: ' 21/4/07 22:07'!
addScriptWithParms: aString script: aScriptString
	self headers isNil ifTrue: [self initHeaders].
	self headers add: ('script' -> (Array with: aString with: aScriptString) )! !

!WebPage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
alink: aSymbol
	"set the color of active link text. It can be in hex format (for example #00FF00) 
	or name of color (for example #White)"
	self attributesAt: #alink put: (self colorValue: aSymbol)! !

!WebPage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
alinkColor: aSymbol
	self alink: aSymbol.! !

!WebPage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
app
	"return anApplication on which we are composing this web page"
	self session isNil ifTrue: [^nil].
	^self session lastApp! !

!WebPage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
background: anImageOrURL
	"set the URL for the background image"
	self attributesAt: #background put:
		(anImageOrURL isString 
			ifTrue: [anImageOrURL]
			ifFalse: [self site urlResolver halfUrlFor: anImageOrURL]).! !

!WebPage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
bgColor: aSymbol
	"set the color of page background)"
	self attributesAt: #bgcolor put: (self colorValue: aSymbol)! !

!WebPage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
bgImage: anImageOrURL
	self background: anImageOrURL.! !

!WebPage methodsFor: 'initialize - release' stamp: ' 21/4/07 22:07'!
cleanup
	"clear all elements and web table info, when page is not needed anymore"
	self initElements.
	self initAttributes.
	self initScript.
	self initPageHeaderLinks.
	self clearTableInfo.! !

!WebPage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
contentType
	"return 'text/html' as content type for web pages"
	^'text/html'! !

!WebPage methodsFor: 'private' stamp: ' 21/4/07 22:07'!
headers
	^self otherAt: #pageHeaders ifAbsent: [nil]! !

!WebPage methodsFor: 'private' stamp: ' 21/4/07 22:07'!
identDepth
	^self otherAt: #identDepth ifAbsent: [0]! !

!WebPage methodsFor: 'private' stamp: ' 21/4/07 22:07'!
identDepth: aNumber
	"current depth of identation. If 0, the we are at the start of the line"
	self otherAt: #identDepth put: aNumber! !

!WebPage methodsFor: 'initialize - release' stamp: ' 21/4/07 22:07'!
initHeaders
	self otherAt: #pageHeaders put: OrderedCollection new.! !

!WebPage methodsFor: 'initialize - release' stamp: ' 21/4/07 22:07'!
initPageHeaderLinks
	self initHeaders.
	self site isNil ifTrue: [^nil].
	self addMetaContentType: 'text/html; charset=UTF-8'.
	self addLinkToScreenStyleSheet: self site style screenCssResource.
	self addLinkToPrintStyleSheet: self site style printCssResource.
	self addLinkToJavascript: self site style javascriptResource.! !

!WebPage methodsFor: 'initialize - release' stamp: ' 21/4/07 22:07'!
initScript
	self initScriptBefore! !

!WebPage methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isComposite

	"a WebPage is a composite element"
	^true! !

!WebPage methodsFor: 'private' stamp: ' 21/4/07 22:07'!
isNewline
	^self identDepth = 0! !

!WebPage methodsFor: 'testing' stamp: ' 21/8/07 09:38'!
isRespondingStreamed
	"is this page able to stream HTML directly into response?"
	^false! !

!WebPage methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isWapDocument
	^false! !

!WebPage methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isWebApplication
	^false! !

!WebPage methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isWebPage
	^true! !

!WebPage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
link: aSymbol
	"set the color of link text."
	self attributesAt: #link put: (self colorValue: aSymbol).! !

!WebPage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
linkColor: aSymbol
	self vlink: aSymbol.! !

!WebPage methodsFor: 'initialize - release' stamp: ' 21/4/07 22:07'!
postInitialize
	self initPageHeaderLinks! !

!WebPage methodsFor: 'header elements' stamp: ' 21/4/07 22:07'!
printDummyJavascriptSnippetOn: aStream
| snippet |
	snippet :=
'<script language="JavaScript">
...
</script>'.
	aStream nextPutAll: snippet! !

!WebPage methodsFor: 'printing' stamp: 'janko 7/18/2008 20:33'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	aStream nextPutAll: '<!!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">' , self eol.
	aStream
		nextPutAll: '<html>' , self eol , ' <head>' , self eol ;
		nextPutAll: '  <title>' , (AIDASite 
					convertToWeb: self title
					on: aSession) , '</title>' , self eol ;
		nextPutAll: '  <meta name="generator" content="AIDA/Web, Smalltalk Web Application Server (www.aidaweb.si)">' , self eol.
	self printHeaderElementsOn: aStream.
	aStream nextPutAll: ' </head>' , self eol.
	aStream nextPutAll: ' <body'.
	self 
		printAttributesOn: aStream
		for: aSession.
	aStream nextPutAll: '>' , self eol.
	self scriptBefore notNil ifTrue: 
		[ self scriptBefore 
			printHTMLPageOn: aStream for: aRequest on: aSession.
		self eol ].
	self isWebApplication 
		ifTrue: 
			[ self form 
				printHTMLPageOn: aStream for: aRequest on: aSession	"form for cur.view" ]
		ifFalse: 
			[ super 
				printHTMLPageOn: aStream for: aRequest on: aSession ].
	aStream nextPutAll: self ident , ' </body>' , self eol , '</html>' , self eol.
	self cleanup! !

!WebPage methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
printHeaderElementsOn: aStream
	self headers isNil ifTrue: [^self].
	self headers do: [:assoc | 
		aStream nextPutAll: '  <', assoc key, ' '.
		assoc key = 'script' 
			ifTrue: [| array | array := assoc value class == Array 
				ifTrue: [assoc value] ifFalse: [Array with: assoc value with: ''].
				aStream nextPutAll: (array at: 1), '>', (array at: 2), '</script>']
			ifFalse: [aStream nextPutAll: assoc value, '>'].
		aStream nextPutAll: self eol].! !

!WebPage methodsFor: 'printing' stamp: 'janko 7/18/2008 19:12'!
printWebPageFor: aRequest on: aSession 
	^self! !

!WebPage methodsFor: 'header elements' stamp: ' 21/4/07 22:07'!
redirectAfter: aSecondsNumber toUrl: aString
	"redirection or refresh of specified url, does not work on Mozilla!! "
	self addHeader: 'meta' value: 'http-equiv="REFRESH"  content="', 
		aSecondsNumber printString, '; url=', aString, '" '! !

!WebPage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
script: aString
	"add this JavaScript  to scripts executed at the opening of page"
	self scriptBefore: aString.! !

!WebPage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
session
	"reference to a session on which this page is created"
	^session! !

!WebPage methodsFor: 'private' stamp: ' 21/4/07 22:07'!
session: aSession
	session := aSession! !

!WebPage methodsFor: 'private' stamp: ' 21/4/07 22:07'!
setNewline
	self identDepth: 0! !

!WebPage methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
shouldIdent
	^false! !

!WebPage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
text: aSymbol
	"set the color of page text."
	self attributesAt: #text put: (self colorValue: aSymbol).! !

!WebPage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
textColor: aSymbol
	self text: aSymbol.! !

!WebPage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
title
	^self otherAt: #pageTitle ifAbsent: ['']! !

!WebPage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
title: aString
	self otherAt: #pageTitle put: aString convertToSloveneChars! !

!WebPage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
vlink: aSymbol
	"set the color of visited link text."
	self attributesAt: #vlink put: (self colorValue: aSymbol).! !

!WebPage methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
vlinkColor: aSymbol
	self vlink: aSymbol.! !

WebElement subclass: #WebScript
	instanceVariableNames: 'script source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebScript class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
script: aString
	^super new script: aString! !

!WebScript class methodsFor: 'instance creation' stamp: 'mivsek 1/10/2008 19:38'!
source: anURLString
	"and external script, with an Url to it"
	^super new source: anURLString! !

!WebScript methodsFor: 'printing' stamp: 'janko 7/18/2008 20:33'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	aStream nextPutAll: '<script type="text/javascript">'.
	script notNil ifTrue: [ aStream nextPutAll: script ].
	aStream nextPutAll: '</script>' , self eol! !

!WebScript methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
script
	^script! !

!WebScript methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
script: aString
	"add to others if already exists, separate with JavaScript separator ; "
	script isNil ifTrue: [script := ''].
	(script notEmpty and: [script last = $; ]) ifFalse: [script := script, '; '].
	script := script, aString trimBlanks.
	"always end with ; "
	(script notEmpty and: [script last ~= $; ]) ifTrue: [script := script , ';' ]! !

WebElement subclass: #WebSeparator
	instanceVariableNames: 'type size length'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebSeparator class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
break
	
	^ self new type: #break! !

!WebSeparator class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
paragraph
	
	^ self new type: #paragraph! !

!WebSeparator class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
rulerSize: aNumber
	
	^(self new) type: #ruler; size: aNumber! !

!WebSeparator methodsFor: 'printing' stamp: 'janko 7/18/2008 20:34'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	| tag |
	self prepareToHTMLPrintOn: aSession.
	type = #paragraph ifTrue: [ tag := 'p' ].
	type = #break ifTrue: [ tag := 'br' ].
	type = #ruler ifTrue: [ tag := 'hr' ].
	aStream nextPutAll: self ident , '<' , tag.
	self 
		printAttributesOn: aStream
		for: aSession.
	aStream nextPutAll: '>' , self eol! !

!WebSeparator methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
shouldIdent
	^self type ~= #break! !

!WebSeparator methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
size: aNumber
	"size of a ruler"
	self attributesAt: #size put: aNumber printString.! !

!WebSeparator methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
type
	^type! !

!WebSeparator methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
type: aSymbol
	"#ruler #paragraph #break"
	type := aSymbol.! !

WebElement subclass: #WebTable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebTable class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new initialize! !

!WebTable methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
bgColor: aColorSymbol
	self attributesAt: #bgcolor put: (self colorValue: aColorSymbol)! !

!WebTable methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
border: aNumber
	"set the width of table border. default is 0"
	self attributesAt: #border put: aNumber printString.! !

!WebTable methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
cellPadding: aNumber
	"define spacing between vertically adjacent cells"
	self attributesAt: #cellpadding put: aNumber printString.! !

!WebTable methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
cellSpacing: aNumber
	"define spacing between adjacent cells horizontally"
	self attributesAt: #cellspacing put: aNumber printString.! !

!WebTable methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
color: aColorSymbol
	self	bgColor: aColorSymbol! !

!WebTable methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
height: aNumber
	| text |
	text := aNumber <= 1 
			ifTrue: [(aNumber * 100) asInteger printString, '%']
			ifFalse: [aNumber asInteger printString].
	self attributesAt: #height put: text! !

!WebTable methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	"self border: 0."! !

!WebTable methodsFor: 'printing' stamp: 'janko 7/18/2008 20:34'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	self prepareToHTMLPrintOn: aSession.
	self scriptBefore notNil ifTrue: 
		[ self scriptBefore 
			printHTMLPageOn: aStream for: aRequest on: aSession ].
	aStream nextPutAll: self ident , '<table'.
	self 
		printAttributesOn: aStream
		for: aSession.
	aStream nextPutAll: '>' , self eol.
	super 
		printHTMLPageOn: aStream for: aRequest on: aSession.
	aStream nextPutAll: self ident , '</table>' , self eol.
	self scriptAfter notNil ifTrue: 
		[ self scriptAfter 
			printHTMLPageOn: aStream for: aRequest on: aSession ]! !

!WebTable methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
shouldIdent
	^true! !

!WebTable methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
width: aNumber
	"set the width of entire table If nil, then table automaticaly addjust itself. 
	If number is between 0 and 1 then width is percent of document width.
	If number above 1 then width in pixels"
	| text |
	text := aNumber <= 1 
			ifTrue: [(aNumber * 100) asInteger printString, '%']
			ifFalse: [aNumber asInteger printString].
	self attributesAt: #width put: text! !

WebElement subclass: #WebTableCell
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebTableCell class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new initialize! !

!WebTableCell methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
align: aSymbol
	"set a horizontal aligment if cell content. It can be #left, #center or #right"
	self attributesAt: #align put: aSymbol asString.! !

!WebTableCell methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
background: aWebImage
	"bachground image"
	self attributesAt: #background put:
		(((aWebImage isKindOf: WebImage) ifTrue: [aWebImage] ifFalse: [WebImage image: aWebImage]) 
			urlOnSession: self session)! !

!WebTableCell methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
bgColor: aColorSymbol
	"set the color of table cell background. It can be in hex format (for example #00FF00) 
	or name of color (for example #White)"
	self attributesAt: #bgcolor put: (self colorValue: aColorSymbol)! !

!WebTableCell methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
color: aColorSymbol
	self	bgColor: aColorSymbol! !

!WebTableCell methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
colspan: aNumber
	"set the number of columns this cell will span"
	self attributesAt: #colspan put: aNumber printString.! !

!WebTableCell methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
height: aNumber
	| text |
	text := aNumber <= 1 
			ifTrue: [(aNumber * 100) asInteger printString, '%']
			ifFalse: [aNumber asInteger printString].
	self attributesAt: #height put: text! !

!WebTableCell methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize! !

!WebTableCell methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isTableHeader
	^false! !

!WebTableCell methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
nowrap: aBoolean
	"if true then text is not wrapped in new lines of a cell but a cell is expanded to carry all the text"
	self attributesAt: #nowrap put: aBoolean.! !

!WebTableCell methodsFor: 'printing' stamp: 'janko 7/18/2008 20:36'!
printHTMLPageOn: aStream for: aRequest on: aSession
	self prepareToHTMLPrintOn: aSession.
	aStream nextPutAll: self ident , '<td'.
	self 
		printAttributesOn: aStream
		for: aSession.
	aStream nextPutAll: '>'.
	super 
		printHTMLPageOn: aStream for: aRequest on: aSession.
	aStream nextPutAll: self ident , '</td>' , self eol! !

!WebTableCell methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
rowspan: aNumber
	"set the number of rows this cell will span"
	self attributesAt: #rowspan put: aNumber printString.! !

!WebTableCell methodsFor: 'attributes' stamp: 'JM 4/26/2007 19:34'!
setHeader
"	self changeClassTo: WebTableHeader"
	self become: (self as: WebTableHeader)! !

!WebTableCell methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
shouldIdent
	^false! !

!WebTableCell methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
valign: aSymbol
	"set a verttical aligment if cell content. It can be #top, #middle, #bottom or #centerline"
	self attributesAt: #valign put: aSymbol asString.! !

!WebTableCell methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
width: aNumber
	"set the width of a cell. If nil, then cell automaticaly addjust itself. 
	If number is between 0 and 1 then width is percent of table width. 
	If number above 1 then width in pixels"
	| text |
	text := aNumber <= 1 
			ifTrue: [(aNumber * 100) asInteger printString, '%']
			ifFalse: [aNumber asInteger printString].
	self attributesAt: #width put: text! !

WebTableCell subclass: #WebTableHeader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebTableHeader methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isTableHeader
	^true! !

!WebTableHeader methodsFor: 'printing' stamp: 'janko 7/18/2008 20:36'!
printHTMLPageOn: aStream for: aRequest on: aSession
	self prepareToHTMLPrintOn: aSession.
	aStream nextPutAll: self ident , '<th'.
	self 
		printAttributesOn: aStream
		for: aSession.
	aStream nextPutAll: '>'.
	elements notNil ifTrue: 
		[ elements do: 
			[ : element | 
			element notNil ifTrue: 
				[ element 
					printHTMLPageOn: aStream for: aRequest on: aSession ] ] ].
	aStream nextPutAll: self ident , '</th>' , self eol! !

WebElement subclass: #WebTableRow
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebTableRow class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new initialize! !

!WebTableRow methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
align: aSymbol
	"set a horizontal aligment for all cells in this row. It can be #left, #center or #right"
	self attributesAt: #align put: aSymbol asString.! !

!WebTableRow methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
bgColor: aColorSymbol
	"set the color of table row background. It can be in hex format (for example #00FF00) 
	or name of color (for example #White)"
	self attributesAt: #bgcolor put: (self colorValue: aColorSymbol)! !

!WebTableRow methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
color: aColorSymbol
	self	bgColor: aColorSymbol! !

!WebTableRow methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
height: aNumber
	| text |
	text := aNumber <= 1 
			ifTrue: [(aNumber * 100) asInteger printString, '%']
			ifFalse: [aNumber asInteger printString].
	self attributesAt: #height put: text! !

!WebTableRow methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize! !

!WebTableRow methodsFor: 'printing' stamp: 'janko 7/18/2008 20:36'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	self prepareToHTMLPrintOn: aSession.
	aStream nextPutAll: self ident , '<tr'.
	self 
		printAttributesOn: aStream
		for: aSession.
	aStream nextPutAll: '>' , self eol.
	super 
		printHTMLPageOn: aStream for: aRequest on: aSession.
	aStream nextPutAll: self ident , '</tr>' , self eol! !

!WebTableRow methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
shouldIdent
	^true! !

!WebTableRow methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
valign: aSymbol
	"set a vertical aligment for all cells in this row. It can be #top, #middle, #bottom or #baseline"
	self attributesAt: #valign put: aSymbol asString.! !

WebElement subclass: #WebText
	instanceVariableNames: 'text size textAttributes header paragraph font'
	classVariableNames: 'AttributeMarkup'
	poolDictionaries: ''
	category: 'Aida-Elements'!

WebText subclass: #WebAnchor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebAnchor class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
name: aString
	"make an anchor with specified name"
	^self new name: aString! !

!WebAnchor class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
name: aString text: anotherString
	"make an anchor with specified name and text"
	^self new name: aString; text: anotherString.! !

!WebAnchor class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
nr: aNumber
	"make a web anchor in format 'anchorx', for example 'anchor6'"
	^self new name: ('anchor', aNumber printString).! !

!WebAnchor class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
nr: aNumber text: aString
	"make an anchor with specified numberName ('anchor5') and text"
	^(self nr: aNumber) text: aString.! !

!WebAnchor methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
name: aString
	"name of the anchor. Used for referencing in web links"
	self attributesAt: #name put: aString.! !

!WebAnchor methodsFor: 'printing' stamp: 'janko 7/18/2008 20:37'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	self prepareToHTMLPrintOn: aSession.
	aStream nextPutAll: self ident , '<a'.
	self 
		printAttributesOn: aStream
		for: aSession.
	aStream nextPutAll: '>'.
	"print a text with attributes and composite elements"
	super 
		printHTMLPageOn: aStream for: aRequest on: aSession.
	aStream nextPutAll: '</a>' , self eol! !

!WebAnchor methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
shouldIndent
	^true! !

WebText subclass: #WebLink
	instanceVariableNames: 'ooReference urlReference parms anchor security'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
image: anImage linkTo: aLink

	"set an image as hypertext link to some other object or page. If aLink is a String, then this is an url reference, else this is object reference, which is resolved to url with help of URLResolver"

	| webLink |
	webLink := self linkTo: aLink.
	webLink add: (WebImage image: anImage).
	^webLink! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
image: anObjectOrURL linkTo: aLink parameter: aParmString value: aValueString

	"set hypertext link and text to some other object or page. If argumet is a String, then this is an url reference, else this is object reference, which is resolved to url with help of URLResolver. Add also one  parameter in query part of an URL"

	| webLink |
	webLink := self linkTo: aLink.
	webLink addImage: anObjectOrURL.
	webLink parameter: aParmString value: aValueString.
	^webLink

"WebLink text: 'text' linkTo: 'http://www.tris-a.si'"
"WebLink text: 'text' linkTo: Smalltalk"! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
image: anObjectOrURL linkTo: aLink view: aString
	^self image: anObjectOrURL linkTo: aLink parameter: 'view' value: aString! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
image: anImageOrURL linkToAnchor: aString

	"set hypertext link anchor in the same page in format '#anchorx', for example '#anchor4'. Use WebAnchor nr: method to add an anchor in a web page."

	^(self linkToAnchor: aString) addImage: anImageOrURL

"WebLink text: 'hello' linkToAnchorNr: 22"! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
image: anObjectOrURL linkToAnchor: aString parameter: aParmString value: aValueString

	"set hypertext link and text to some other object or page. If argumet is a String, then this is an url reference, else this is object reference, which is resolved to url with help of URLResolver. Add also one  parameter in query part of an URL"

	| webLink |
	webLink := self linkToAnchor: aString.
	webLink addImage: anObjectOrURL.
	webLink parameter: aParmString value: aValueString.
	^webLink

"WebLink text: 'text' linkTo: 'http://www.tris-a.si'"
"WebLink text: 'text' linkTo: Smalltalk"! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
image: anImageOrURL linkToAnchorNr: aNumber

	"set hypertext link anchor in the same page in format '#anchorx', for example '#anchor4'. Use WebAnchor nr: method to add an anchor in a web page."

	^(self linkToAnchorNr: aNumber) addImage: anImageOrURL

"WebLink text: 'hello' linkToAnchorNr: 22"! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
image: anObjectOrURL linkToAnchorNr: aNumber parameter: aParmString value: aValueString

	"set hypertext link and text to some other object or page. If argumet is a String, then this is an url reference, else this is object reference, which is resolved to url with help of URLResolver. Add also one  parameter in query part of an URL"

	| webLink |
	webLink := self linkToAnchor: aNumber.
	webLink addImage: anObjectOrURL.
	webLink parameter: aParmString value: aValueString.
	^webLink

"WebLink text: 'text' linkTo: 'http://www.tris-a.si'"
"WebLink text: 'text' linkTo: Smalltalk"! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
linkTo: aLink
	"set hypertext link to some other object or page. If argumet is a String, then this is an url reference, 	else this is object reference, which is resolved to url with help of URLResolver"
	"to add anchor: #(object anchor) !! "
	^self new linkTo: aLink! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
linkToAnchor: aString

	"set hypertext link anchor in the same page in format '#anchorx', for example '#anchor4'. Use WebAnchor nr: method to add an anchor in a web page."

	^self new anchor: aString.

"WebLink linkToAnchorNr: 22"! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
linkToAnchorNr: aNumber

	"set hypertext link anchor in the same page in format '#anchorx', for example '#anchor4'. Use WebAnchor nr: method to add an anchor in a web page."

	^self new anchorNr: aNumber.

"WebLink linkToAnchorNr: 22"! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
message: aSymbol linkTo: aLink

	"set hypertext link and a message to some other object or page. If argumet is a String, then this is an url reference, else this is object reference, which is resolved to url with help of URLResolver"
	| webLink |
	webLink := self linkTo: aLink.
	webLink text: (WebMsgs at: aSymbol).
	^webLink

"WebLink text: 'text' linkTo: 'http://www.tris-a.si'"
"WebLink text: 'text' linkTo: Smalltalk"! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
message: aSymbol linkTo: aLink parameter: aParmString value: aValueString

	"set hypertext link and text to some other object or page. If argumet is a String, then this is an url reference, else this is object reference, which is resolved to url with help of URLResolver. Add also one  parameter in query part of an URL"

	| webLink |
	webLink := self message: aSymbol linkTo: aLink.
	webLink parameter: aParmString value: aValueString.
	^webLink

"WebLink message: #test linkTo: 'http://www.tris-a.si'"! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new initialize! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
text: aString attributes: anArray linkTo: aLink

	"set hypertext link and text to some other object or page. If argumet is a String, then this is an url reference, else this is object reference, which is resolved to url with help of URLResolver"
	| webLink |
	webLink := self linkTo: aLink.
	webLink text: aString attributes: anArray.
	^webLink

"WebLink text: 'text' linkTo: 'http://www.tris-a.si'"
"WebLink text: 'text' linkTo: Smalltalk"! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
text: aString header: aNumber linkTo: aLink

	"set hypertext link and text to some other object or page. If argumet is a String, then this is an url reference, else this is object reference, which is resolved to url with help of URLResolver"
	| webLink |
	webLink := self linkTo: aLink.
	webLink text: aString;  header: aNumber.
	^webLink

"WebLink text: 'text' linkTo: 'http://www.tris-a.si'"
"WebLink text: 'text' linkTo: Smalltalk"! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
text: aString linkTo: aLink

	"set hypertext link and text to some other object or page. If argumet is a String, then this is an url reference, else this is object reference, which is resolved to url with help of URLResolver"
	| webLink |
	webLink := self linkTo: aLink.
	webLink text: aString.
	^webLink

"WebLink text: 'text' linkTo: 'http://www.tris-a.si'"
"WebLink text: 'text' linkTo: Smalltalk"! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
text: aString linkTo: aLink parameter: aParmString value: aValueString
	"set hypertext link and text to some other object or page. If argumet is a String, then this is an url reference, else this is object reference, which is resolved to url with help of URLResolver. Add also one  parameter in query part of an URL"
	| webLink |
	webLink := self text: aString linkTo: aLink.
	webLink parameter: aParmString value: aValueString.
	^webLink

"WebLink text: 'text' linkTo: 'http://www.tris-a.si'"
"WebLink text: 'text' linkTo: Smalltalk"! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
text: aString linkTo: aLink 
	parameter: a1ParmString value: a1ValueString 
	parameter: a2ParmString value: a2ValueString
	"set hypertext link and text to some other object or page. If argumet is a String, then this is an url reference, else this is object reference, which is resolved to url with help of URLResolver. Add also one  parameter in query part of an URL"
	| webLink |
	webLink := self text: aString linkTo: aLink.
	webLink parameter: a1ParmString value: a1ValueString.
	webLink parameter: a2ParmString value: a2ValueString.
	^webLink! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
text: aString linkTo: aLink parameter: a1ParmString value: a1ValueString parameter: a2ParmString value: a2ValueString parameter: a3ParmString value: a3ValueString

	"set hypertext link and text to some other object or page. If argumet is a String, then this is an url reference, else this is object reference, which is resolved to url with help of URLResolver. Add also one  parameter in query part of an URL"

	| webLink |
	webLink := self text: aString linkTo: aLink.
	webLink parameter: a1ParmString value: a1ValueString.
	webLink parameter: a2ParmString value: a2ValueString.
	webLink parameter: a3ParmString value: a3ValueString.
	^webLink! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
text: aString linkTo: aLink view: aViewString
	^(self text: aString linkTo: aLink) view: aViewString! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
text: aString linkToAnchor: anAnchorString

	"set hypertext link anchor in the same page in format '#anchorx', for example '#anchor4'. Use WebAnchor nr: method to add an anchor in a web page."

	^(self linkToAnchor: anAnchorString) text: aString

"WebLink text: 'hello' linkToAnchorNr: 22"! !

!WebLink class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
text: aString linkToAnchorNr: aNumber

	"set hypertext link anchor in the same page in format '#anchorx', for example '#anchor4'. Use WebAnchor nr: method to add an anchor in a web page."

	^(self linkToAnchorNr: aNumber) text: aString

"WebLink text: 'hello' linkToAnchorNr: 22"! !

!WebLink methodsFor: 'security' stamp: 'JM 4/26/2007 17:46'!
accessAllowedFor: aSession
	"true if for object of that link a view is allowed. Only for object links, for plain urls access is always 	allowed"
	| app view right |
	self ooReference isNil ifTrue: [^true].
	app := self ooReference webAppFor: aSession.
	view := self parms at: 'view' ifAbsent: [app defaultView].
	right := aSession site securityManager 
		isUser: aSession user allowedTo: (app class viewRightSymbolFor: view) on: app class name.
	(right not and: [aSession adminAllowed]) ifTrue: 
		[app initAdminAccess. ^true].
	^right! !

!WebLink methodsFor: 'private' stamp: ' 21/4/07 22:07'!
addAnchorTo: anUrlString
	| in out anch | 
	anch := (self anchor includes: $#) ifTrue: [''] ifFalse: ['#']. 
	anch := anch, self anchor.
	in := anUrlString readStream. out := WriteStream on: String new.
"
	out nextPutAll: (in upTo: $? ). out nextPutAll: anch. 
	in atEnd ifFalse: [out nextPut: $? . out nextPutAll: in upToEnd].
"
	out nextPutAll: in upToEnd. out nextPutAll: anch.
	^out contents

"(WebLink new anchor: 'activity222') addAnchorTo:	
	'http://biart.eranova.si/object/o7665965.html?view=editActivityName&uuid=1518'
"! !

!WebLink methodsFor: 'private' stamp: ' 21/4/07 22:07'!
addSessionIdTo: aString on: aSession
	"for connection of secure seesion to nonsecure one"
	| url |
	aSession hasSecureCookie ifTrue: [^aString].
	url := (aString includes: $? ) ifTrue: [aString, '&'] ifFalse: [aString, '?'].
	^url, 'sessionId=', aSession id printString.! !

!WebLink methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
anchor

	^anchor! !

!WebLink methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
anchor: aString

	anchor := aString! !

!WebLink methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
anchorNr: aNumber

	"set hypertext link anchor in the same page in format '#anchorx', for example '#anchor4'. Use WebAnchor nr: method to add an anchor in a web page."

	^self anchor: '#anchor', aNumber printString.! !

!WebLink methodsFor: 'private' stamp: ' 21/4/07 22:07'!
checkVersionedReference
	"if ooReference is not a current version, then point it to current and add 
	version parameter to url. Version objects have always same identity, that
	is those of current object in version chain!! "
	self ooReference isCurrentVersion ifTrue: [^self].
	self parameter: 'version' value: self ooReference versionNumber.
	ooReference := self ooReference currentVersion.! !

!WebLink methodsFor: 'private' stamp: 'mivsek 2/10/2008 20:25'!
composeURLOn: aSession
	"do url link only if ooReference or urlReference exists, return nil otherwise"
	| stream url |
	stream := WriteStream on: String new.
	(self ooReference notNil or: [self urlReference ~= '']) ifTrue:
		["copy parameters to aSession parms, remove all other except session id"
		parms keysAndValuesDo: [:key :value | aSession parms at: key put: value].
		aSession parms copy keysAndValuesDo: [:key :value | 
			((parms includesKey: key) or: [key = 'id']) 
				ifFalse: [aSession parms removeKey: key ifAbsent: [nil]] ].
		"make an url reference"
		self ooReference notNil
			ifTrue:	
				[url := (aSession site urlResolver
					halfUrlFromPath:	(aSession site urlResolver halfUrlFor: self ooReference)
					andParms: aSession parms on: aSession).
"				(self isEncryptedViewFor: self ooReference on: aSession) 
					ifTrue: [url := self encryptedUrl: url on: aSession]" ]
			ifFalse: [url := self urlReference].
		anchor notNil ifTrue: [url := self addAnchorTo: url].
		stream nextPutAll: url.
		^stream contents]
	ifFalse: [^nil].! !

!WebLink methodsFor: 'private' stamp: ' 21/4/07 22:07'!
encryptedUrl: aString on: aSession
	"https and port if not default"
	| url port sslPort stream |
	url := aSession site urlResolver urlFromHalfUrl: aString on: aSession.
	('https*' match: url) ifFalse: [url := url copyReplaceAll: 'http' with: 'https'].
	port := aSession site port.
	sslPort := aSession site sslPort.
	port = 80 & sslPort = 443 ifTrue: [^url].
	stream := url readStream.
	url := (stream upTo: $/ ), '//'.
	stream upTo: $/.
	url := url, (port = 80 ifTrue: [stream upTo: $/] ifFalse: [stream upTo: $: ]).
	port  ~= 80 ifTrue: [stream upTo: $/].
	sslPort ~= 443 ifTrue: [url := url, ':', sslPort printString].
	url := url, '/', stream upToEnd.
	aSession hasSecureCookie ifFalse: [url := self addSessionIdTo: url on: aSession].
	^url! !

!WebLink methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
href: aString
	"url of that link"
	self attributesAt: #href put: aString! !

!WebLink methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	self urlReference: ''.     
	parms := Dictionary new.! !

!WebLink methodsFor: 'private' stamp: ' 21/4/07 22:07'!
isEncryptedViewFor: anObject on: aSession
	"check if we need to make an https link"
	| app view |
	app := anObject webAppFor: aSession.
	app isNil ifTrue: [^false].
	view := self parms at: 'view' ifAbsent: [nil].
	view isNil ifTrue: [view := app class defaultView].
	^app mustBeEncrypted: view! !

!WebLink methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isLink
	^true! !

!WebLink methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
linkTo: aLink
	"set hypertext link to some other object or page. If argumet is a String, then this is an url reference, 	else this is object reference, which is resolved to url with help of URLResolver"
	"to add anchor: #(object anchor) !! "
	(aLink isKindOf: String ) ifTrue: [self urlReference: aLink. ^self].
	(aLink isKindOf: Array)
		ifTrue: [self ooReference: (aLink at: 1). self anchor: (aLink at: 2)]
		ifFalse: [self ooReference: aLink].! !

!WebLink methodsFor: 'events' stamp: ' 21/4/07 22:07'!
onBlur: aJavascriptCode
	"link lost a mouse focus (when in image map)"
	self attributesAt: #onBlur add: aJavascriptCode! !

!WebLink methodsFor: 'events' stamp: ' 21/4/07 22:07'!
onFocus: aJavascriptCode
	"link got maouse focus (when in image map)"
	self attributesAt: #onFocus add: aJavascriptCode! !

!WebLink methodsFor: 'private' stamp: ' 21/4/07 22:07'!
ooReference
	^ooReference! !

!WebLink methodsFor: 'private' stamp: ' 21/4/07 22:07'!
ooReference: anObject
	"Set the hypertext link to other object. This reference is at HTML generation 
	resolved to urlReference by URLResolver.  ooReference has precedence over 
	urlReference, if both are set."
	ooReference := anObject.
	anObject isVersionedObject ifTrue: [self checkVersionedReference]! !

!WebLink methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
parameter: aParmString value: aValueString

	"set an optional parameter for query part of an URL string (e.g in http://www.eranova.si/demo.html?view=brief 'view' is a parameter and 'brief' is its value).
You can set more than one parameters by repeating this method"

	self parms at: aParmString asString put: aValueString asString.! !

!WebLink methodsFor: 'private' stamp: ' 21/4/07 22:07'!
parms
	"return a dictionary of parameter:value pairs for a query part of an URL"
	^parms! !

!WebLink methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
prepareAttributesToPrintOn: aSession
	self href: (self composeURLOn: aSession).! !

!WebLink methodsFor: 'printing' stamp: 'janko 7/18/2008 20:38'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	| url allowed |
	self prepareToHTMLPrintOn: aSession.
	allowed := self security notNil 
		ifTrue: [ self accessAllowedFor: aSession ]
		ifFalse: [ true ].
	allowed 
		ifTrue: 
			[ url := self composeURLOn: aSession.
			url notNil ifTrue: 
				[ aStream nextPutAll: self ident , '<a'.
				self 
					printAttributesOn: aStream
					for: aSession.
				aStream nextPutAll: '>'.
				super 
					printHTMLPageOn: aStream for: aRequest on: aSession.
				aStream nextPutAll: '</a>' ] ]
		ifFalse: 
			[ self security = #grayed ifTrue: 
				[ "print a grayed text with attributes and composite elements"
				aStream nextPutAll: self ident , '<font'.
				self 
					printAttributesOn: aStream
					for: aSession.
				aStream nextPutAll: 'color="#c0c0c0">'.
				super 
					printHTMLPageOn: aStream for: aRequest on: aSession.
				aStream nextPutAll: '</font>' , self eol ] ]! !

!WebLink methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
printString
	^'aWebLink to: ', (self ooReference notNil 
		ifTrue: ['a', self ooReference printString] ifFalse: [self urlReference]).! !

!WebLink methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
security
	"you can gray or not show a link at all if this object is not accessible - a view 
	on that aWebApplication has no access rights in SecurityManager for current user.
	possible values:
	nil - no access restrictions, always show the link,
	#grayed - show only text in gray - no link if access denied
	#invisible - don't show anything - invisible link if access denied
	"

	^security! !

!WebLink methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
security: aSymbol

	security := aSymbol! !

!WebLink methodsFor: 'attributes' stamp: ' 21/4/07 22:07'!
target: aString
	self attributesAt: #target put: aString! !

!WebLink methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
text: aString linkTo: aLink
	self linkTo: aLink.
	self text: aString.! !

!WebLink methodsFor: 'private' stamp: ' 21/4/07 22:07'!
urlReference
	^urlReference! !

!WebLink methodsFor: 'private' stamp: ' 21/4/07 22:07'!
urlReference: aString
	"Set the URL hypertext link to the other Web page.  It is used if there is no ooReference 
	to resolve. ooReference has precedence over urlReference when both are set."
	urlReference := aString.! !

!WebLink methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
view: aViewStringOrSymbol
	"for speedy additions of a view parameters to urls"
	"ignore #main, because this is default view anyway!!"
	((aViewStringOrSymbol = 'main') | (aViewStringOrSymbol = #main)) ifTrue: [^self].
	self parameter: 'view' value: aViewStringOrSymbol! !

WebText subclass: #WebRawText
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Elements'!

!WebRawText commentStamp: '<historical>' prior: 0!
WebRawText for plain ascii text, without any encodings!!!

!WebRawText methodsFor: 'printing' stamp: 'janko 7/18/2008 20:38'!
printHTMLPageOn: aStream for: aRequest on: aSession
	text notNil ifTrue: [ aStream nextPutAll: self text ]! !

!WebRawText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
text: aString
	text := aString! !

!WebText class methodsFor: 'accessing' stamp: 'mivsek 1/10/2008 19:40'!
attributeMarkup
	^AttributeMarkup! !

!WebText class methodsFor: 'examples' stamp: ' 21/4/07 22:07'!
example1
	"examples of use a WebText."

	| text stream |
	text := WebText header: 1 text: 'Hello' .
	text addText: 'String Hello' attributes: #bold.
	text addText: 'String and Italic Hello' attributes: #(#bold #italic).
"	text addText: 'Strong Hello again' prevAttributes."

	stream := WriteStream on: String new.
	text printHTMLPageOn: stream.
	^stream contents.! !

!WebText class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
header: aNumber

	"create an empty header. You can add elements such as images and text later"

	^(self new) header: aNumber.

"| a | 
a := (WebText header: 1).
a add: (WebImage urlReference: 'http://pu/stefan.gif').
a inspect"! !

!WebText class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
header: aNumber text: aString

	"create a header text"

	^(self new) header: aNumber; text: aString.

"WebText header: 1 text: 'test'"! !

!WebText class methodsFor: 'initialization' stamp: ' 21/4/07 22:07'!
initialize
	"set the AttributeMarkup dictionary with the allowed atributes and their HTML markup symbols"
	AttributeMarkup := IdentityDictionary new.
	AttributeMarkup
		at: #bold		put: 'b';
		at: #italic		put: 'I';
		at: #teletype 	put: 'tt';
		at: #address 	put: 'address';
		at: #cite			put: 'cite';
		at: #code		put: 'code';
		at: #sequence 	put: 'samp';
		at: #blockquote 	put: 'blockquote';
		at: #definition 	put: 'dfn';
		at: #emphasized put: 'em';
		at: #keyboard 	put: 'kbd';
		at: #strong 		put: 'strong';
		at: #variable 	put: 'var'.

"WebText initialize"! !

!WebText class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new initialize! !

!WebText class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newParagraph

	"To open a new paragraph. Use addText: for adding a paragraph text."

	^self new paragraph: true.! !

!WebText class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
text: aString
	^self new text: aString

"WebText text: 'test'"! !

!WebText class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
text: aString attributes: anArray

	^self new text: aString attributes: anArray

"WebText text: 'test' attributes: #bold"! !

!WebText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
addText: aString
	"make a new element with aString. Attributes are same as previous text." 

	self add: (WebText text: aString attributes: self attributes)! !

!WebText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
addText: aString attributes: anArray
	"make a new element with aString. Attributes are same as previous text." 

	self add: (WebText text: aString attributes: anArray)! !

!WebText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
color: aString
	self style: '{color: ', aString asString, '}'.! !

!WebText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
font
	^font! !

!WebText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
font: aString
	font := aString! !

!WebText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
header
	^header! !

!WebText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
header: aNumber
	
	"Assigns this WebText as a header text. Parameter can be 1 .. 5. If this parameter is nil then WebText is not a header"

	aNumber < 1 
		ifTrue: [header:= 1.   ]
		ifFalse:
			[aNumber > 5 ifTrue: [header := 5.] ifFalse: [header := aNumber.]].
	^self! !

!WebText methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	text := ''.   
	paragraph := false.! !

!WebText methodsFor: 'private' stamp: ' 21/4/07 22:07'!
insideSpanTag
	"to enclose or not in span tag. Only if element have any attribute!!"
	^self attributes notNil and: [self isLink not]! !

!WebText methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isLink
	^false! !

!WebText methodsFor: 'private' stamp: ' 21/8/07 09:38'!
onChangePost
	"just if addAspect:for:editing: false returns text instead of input field!!"! !

!WebText methodsFor: 'private' stamp: ' 21/8/07 09:38'!
onChangePostAndUpdate: e
	"just if addAspect:for:editing: false returns text instead of input field!!"! !

!WebText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
paragraph
	^paragraph! !

!WebText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
paragraph: aBoolean
	
	"call this method if you want this text in a new paragraph"

	paragraph := aBoolean.! !

!WebText methodsFor: 'printing' stamp: 'janko 7/18/2008 20:37'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	"make a header"
	self prepareToHTMLPrintOn: aSession.
	aStream nextPutAll: self ident.
	self insideSpanTag ifTrue: 
		[ aStream nextPutAll: '<span'.
		self 
			printAttributesOn: aStream
			for: aSession.
		aStream nextPutAll: '>' ].
	header notNil ifTrue: [ aStream nextPutAll: '<h' , header printString , '>' ].
	"font size, color, face"
	size notNil | font notNil ifTrue: 
		[ aStream nextPutAll: '<font '.
		size notNil ifTrue: 
			[ aStream nextPutAll: 'size=' , (size > 0 
						ifTrue: [ '+' ]
						ifFalse: [ '' ]) , size printString , ' ' ].
		font notNil ifTrue: [ aStream nextPutAll: 'face=' , font , ' ' ].
		aStream nextPutAll: '> ' ].
	"set text attributes, if any"
	textAttributes notNil ifTrue: 
		[ textAttributes do: 
			[ : attribute | 
			aStream nextPutAll: '<' , (self class attributeMarkup at: attribute) , '>' ] ].
	"print a text"
	text notNil ifTrue: 
		[ aStream nextPutAll: (AIDASite 
				convertToWeb: text
				on: aSession) ].
	"print composite elements"
	super 
		printHTMLPageOn: aStream for: aRequest on: aSession.
	"reset text attributes"
	textAttributes notNil ifTrue: 
		[ textAttributes do: [ : attribute | aStream nextPutAll: '</' , (AttributeMarkup at: attribute) , '>' ] ].
	"end of font adjustment"
	size notNil | font notNil ifTrue: [ aStream nextPutAll: '</font>' ].
	header notNil ifTrue: [ aStream nextPutAll: '</h' , header printString , '>' , self eol ].
	self insideSpanTag ifTrue: [ aStream nextPutAll: '</span>' , self eol ]! !

!WebText methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
printString

	^'''', [self text isNil ifTrue: [''] ifFalse: [self text] ] value,''''! !

!WebText methodsFor: 'private' stamp: ' 21/4/07 22:07'!
shouldIdent
	^false! !

!WebText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
size
	^size! !

!WebText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
size: aNumber
	
	"larger or smaller text for aNumber factor. Range -7..+7"

	aNumber < -7 
		ifTrue: [size := -7.  ]
		ifFalse:
			[aNumber > 7 ifTrue: [size := 7.  ] ifFalse: [size := aNumber.]].
	^self! !

!WebText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
text
	^text! !

!WebText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
text: aString
	text := WebFormElement autoConvertToString: aString.
	"slovene csz are converted if char ^ is after such a char"
	(text includes: $^ ) ifTrue: [text := text convertToSloveneChars].! !

!WebText methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
text: aString attributes: anArray
	self text: aString.
	self textAttributes: anArray.! !

!WebText methodsFor: 'private' stamp: ' 21/4/07 22:07'!
textAttributes
	"get the attributes of a text" 
	^textAttributes! !

!WebText methodsFor: 'private' stamp: 'mivsek 1/13/2008 20:37'!
textAttributes: anArray
	"check and set the attributes of a text. Atribute can be one or a set of them. Attribute is ignored if not correct. See class variable AttributeMarkup or method WebText initialize for list of allowed attributes" 

	(anArray isKindOf: Symbol)		"if only one attribute, convert to an Array"
		ifTrue: [textAttributes := Array with: anArray.       ]
		ifFalse: [(anArray isKindOf: Array)
			ifTrue: [textAttributes := anArray.      ]
			ifFalse: [anArray isNil
				ifTrue: [textAttributes := nil.     ]
				ifFalse: 
					[self error: 'Text attributes should be Array of Symbols']]].

	"attribute correctness check. Ignore if attribute not correct"
	textAttributes isNil ifFalse:
		[textAttributes := textAttributes select: [:attribute | 
			(self class attributeMarkup at: attribute ifAbsent: [false])  ~~ false ]]

"WebText text: 'test' attributes: #bold"
"WebText text: 'test' attributes: #( #bold #italic)"
"WebText text: 'test' attributes: nil"! !

Object subclass: #WebGridColumn
	instanceVariableNames: 'parent id name width align aspect linkAspect linkView viewBlock addBlock sorted filter filterWidth summaryType summary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Components'!

!WebGridColumn class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newOn: aWebGrid
	^super new initialize parent: aWebGrid! !

!WebGridColumn methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
addBlock
	^addBlock! !

!WebGridColumn methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
addBlock: aBlock
	"result must be kind of WebElement which will be added to table cell"
	"block needs a row object as an argument"
	addBlock := aBlock! !

!WebGridColumn methodsFor: 'private' stamp: ' 21/4/07 22:07'!
addToSummary: anObject
	self summaryType = #count ifTrue: [^self summary: self summary + 1].
	self summaryType = #sum ifTrue:
		[^self summary: self summary + (anObject perform: self aspect)]! !

!WebGridColumn methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
align
	^align! !

!WebGridColumn methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
align: aSymbol
	align := aSymbol! !

!WebGridColumn methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
aspect
	^aspect! !

!WebGridColumn methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
aspect: aSymbol
	"a method to be called for this column on objects of current row in a collection"
	aspect := aSymbol! !

!WebGridColumn methodsFor: 'private' stamp: ' 21/4/07 22:07'!
autoConvert: anObject
	(anObject class == Date and: [self parent hasShortDates]) ifTrue: [^anObject shorterPrintSloString].
	(anObject = 0 and: [self parent hasNoZeros]) ifTrue: [^''].
	^WebFormElement autoConvertToString: anObject! !

!WebGridColumn methodsFor: 'private' stamp: ' 21/4/07 22:07'!
deepCopyNotIn: aDict
	^nil! !

!WebGridColumn methodsFor: 'private' stamp: ' 21/4/07 22:07'!
defaultViewBlock
	^[:object | 
		self aspect isNil ifTrue: [''] ifFalse: [self autoConvert: (object perform: self aspect) ] ]! !

!WebGridColumn methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
filter
	^filter! !

!WebGridColumn methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
filter: aString
	"show only rows with value starting with aString"
	filter := aString.
	self parent page: 1 "always to the first page after filter change!!"! !

!WebGridColumn methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
filterWidth
	"filter input field width"
	^filterWidth! !

!WebGridColumn methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
filterWidth: aNumber
	"filter input field width"
	filterWidth := aNumber! !

!WebGridColumn methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
hasAddBlock
	^self addBlock notNil! !

!WebGridColumn methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
hasLink
	^self linkAspect notNil | self linkView notNil! !

!WebGridColumn methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
id
	^id! !

!WebGridColumn methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
id: aSymbolOrNumber
	"by default it is a position (1st, 2nd etc.)"
	id := aSymbolOrNumber! !

!WebGridColumn methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize! !

!WebGridColumn methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isCheckboxed
	self parent checkboxes isNil ifTrue: [^false].
	^self parent checkboxes = self id! !

!WebGridColumn methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isNumbered
	self parent numbering isNil ifTrue: [^false].
	^self parent numbering = self id! !

!WebGridColumn methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isSortedAscending
	^(self parent sortColumn = self id) and: [self parent sortOrder = #ascending]! !

!WebGridColumn methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isSortedDescending
	^(self parent sortColumn = self id) and: [self parent sortOrder = #descending]! !

!WebGridColumn methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
linkAspect
	^linkAspect! !

!WebGridColumn methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
linkAspect: aSymbol
	"if set, a link to aspect of  row object will be made"
	linkAspect := aSymbol.! !

!WebGridColumn methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
linkView
	^linkView! !

!WebGridColumn methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
linkView: aSymbol
	"if set, a link to aspect of  row object (spesified with linkAspect:) with that view will be shown"
	linkView := aSymbol.! !

!WebGridColumn methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
name
	^name! !

!WebGridColumn methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
name: aString
	"header name for that column"
	name := aString! !

!WebGridColumn methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
needsSummary
	"summary row is needed"
	^self summaryType notNil! !

!WebGridColumn methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
needsSummaryCount
	^self summaryType notNil and: [self summaryType = #count]! !

!WebGridColumn methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
needsSummarySum
	^self summaryType notNil and: [self summaryType = #sum]! !

!WebGridColumn methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
parent
	^parent! !

!WebGridColumn methodsFor: 'private' stamp: ' 21/4/07 22:07'!
parent: anObject
	parent := anObject! !

!WebGridColumn methodsFor: 'private' stamp: ' 21/4/07 22:07'!
printString
	^'aWebGridColumn id: ', self id printString! !

!WebGridColumn methodsFor: 'private' stamp: ' 21/4/07 22:07'!
representBinaryOn: writer
	^0! !

!WebGridColumn methodsFor: 'private' stamp: ' 21/4/07 22:07'!
resetSummary
	(self summary notNil and: [self summary isKindOf: Number]) 
		ifTrue: [self summary: 0].! !

!WebGridColumn methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
setFilter
	"this column will have a filter input field in a row below header"
	self filter: ''! !

!WebGridColumn methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
sort
	self parent sortColumn == self id
		ifTrue: [self toggleSortOrder]
		ifFalse: [self sortAscending]! !

!WebGridColumn methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
sortAscending
	self parent sortAscendingOn: self id! !

!WebGridColumn methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
sortDescending
	self parent sortDescendingOn: self id! !

!WebGridColumn methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
sorted
	^sorted! !

!WebGridColumn methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
sorted: aSymbol
	"nil #ascending #descending"
	sorted := aSymbol! !

!WebGridColumn methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
summary
	"here summary of all rows is acumulated, depending on summaryType"
	^summary! !

!WebGridColumn methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
summary: aNumber
	"see comment in method #summary"
	summary := aNumber! !

!WebGridColumn methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
summaryType
	"summary row is added with appropriate value, which can be:
	- #sum : sum of all cels in that column
	- #count : number of all rows, without heading and summary
	- 'some text' to be shown in this summary cell"
	^summaryType! !

!WebGridColumn methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
summaryType: aSymbolOrString
	"see comment in method #summary"
	summaryType := aSymbolOrString.
	(aSymbolOrString = #count) | (aSymbolOrString = #sum)
		ifTrue: [self summary: 0]
		ifFalse: [self summary: aSymbolOrString].! !

!WebGridColumn methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
summaryValue
	"for adding to a table"
	self summary isNil ifTrue: [^''].
	(self summary isKindOf: Number) ifTrue: [^self summary printDotString].
	^self summary "as text"! !

!WebGridColumn methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
toggleSortOrder
	self parent sortColumn == self id
		ifTrue: [self parent sortOrder == #descending
			ifTrue: [self sortAscending]
			ifFalse: [self sortDescending]]
		ifFalse: [self sortAscending]! !

!WebGridColumn methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
viewBlock
	viewBlock isNil ifTrue: [self viewBlock: self defaultViewBlock].
	^viewBlock! !

!WebGridColumn methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
viewBlock: aBlock
	"how to show this column from collection. There is a default"
	"block needs a row object as an argument"
	viewBlock := aBlock! !

!WebGridColumn methodsFor: 'private-accessing' stamp: ' 21/4/07 22:07'!
width
	^width! !

!WebGridColumn methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
width: aNumber
	width := aNumber! !

Object subclass: #WebHelp
	instanceVariableNames: 'pages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Admin'!

!WebHelp commentStamp: '<historical>' prior: 0!
WebHelp for easy made of help system. each view of each App can have its own help page. Each aWebHelpPage has url like http://www.server.com/help/AdminApp/main.html for view #main in AdminApp.


Instance Variables:
	pages	<Dictionary of Dictionary> key=App class name, 
											value=dictionary key: view, value aWebHelpPage

!

!WebHelp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
add: aWebHelpPage
	self forApp: aWebHelpPage app view: aWebHelpPage view put: aWebHelpPage! !

!WebHelp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
existApp: anApp view: aSymbol
	^(self forApp:anApp view: aSymbol) notNil! !

!WebHelp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
forApp: anApp view: aSymbol
	| appName |
	appName := (anApp isKindOf: WebApplication) ifTrue: [anApp class name] ifFalse: [anApp].
	^(self pages at: appName ifAbsent: [^nil]) at: aSymbol asSymbol ifAbsent: [nil]! !

!WebHelp methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
forApp: anApp view: aSymbol put: aWebHelpPage
	| appName |
	appName := (anApp isKindOf: WebApplication) ifTrue: [anApp class name] ifFalse: [anApp].
	(self pages at: appName ifAbsentPut: [Dictionary new]) at: aSymbol asSymbol put: aWebHelpPage! !

!WebHelp methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initPages
	pages := Dictionary new.! !

!WebHelp methodsFor: 'private' stamp: ' 21/4/07 22:07'!
pages
	pages isNil ifTrue: [self initPages].
	^pages! !

Object subclass: #WebHelpPage
	instanceVariableNames: 'parent app view title body'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Admin'!

!WebHelpPage commentStamp: '<historical>' prior: 0!
WebHelpPage contains help for particulal view on particular WebApp

Instance Variables:
	parent	<WebHelp>	back link to WebHelp
	app		<Symbol>	a name of WebApplication subclass
	view	<Symbol>	view of app for which is this help page
	title		<String>	 	title of that help page
	body	<String>	   	help content for that App and view in Wiki format

!

!WebHelpPage class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newForApp: anAppNameSymbol view: aSymbol parent: aWebHelp
	^super new
		app: anAppNameSymbol;
		view: aSymbol;
		parent: aWebHelp! !

!WebHelpPage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
app
	^app! !

!WebHelpPage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
app: aSymbol
	app := aSymbol! !

!WebHelpPage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
body
	body isNil ifTrue: [^''].
	^body! !

!WebHelpPage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
body: aString
	body := aString! !

!WebHelpPage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
parent
	^parent! !

!WebHelpPage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
parent: aWebHelp
	parent := aWebHelp! !

!WebHelpPage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
preferedUrl
	^'/help/',  self app asString, '/', self view asString, '.html'! !

!WebHelpPage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
title
	title isNil ifTrue: [^''].
	^title! !

!WebHelpPage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
title: anObject
	title := anObject! !

!WebHelpPage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
view
	^view! !

!WebHelpPage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
view: aSymbol
	view := aSymbol! !

Object subclass: #WebIndex
	instanceVariableNames: 'index indexedObjects popularWords workQueue indexer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Core'!

!WebIndex class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
default
	^AIDASite default index! !

!WebIndex class methodsFor: 'odb specific' stamp: ' 21/4/07 22:07'!
instVarMap
	"Gemstone"

	^super instVarMap,
		#( 	(workQueue nil) )! !

!WebIndex class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new initialize.

"WebIndex new"! !

!WebIndex class methodsFor: 'odb specific' stamp: ' 21/4/07 22:07'!
replicationSpec

	^super replicationSpec, 
		#(  	(index forwarder)
			(indexedObjects forwarder) 
			(popularWords forwarder)  	)! !

!WebIndex methodsFor: 'searching' stamp: ' 21/8/07 09:38'!
addFreshnessRelevance: aDictionary
	"objects with newer modified date are more relevant"
	| weight |
	aDictionary keysDo: [:object |
		(object class selectors includes: #modified) ifTrue:
			[weight := 1000 - (Date today subtractDate: object modified) max: 0.
			aDictionary at: object put:  (aDictionary at: object) + weight] ]! !

!WebIndex methodsFor: 'searching' stamp: ' 21/8/07 09:38'!
addVisitsRelevance: aDictionary
	"objects with more visits are more relevant"
	| weight |
	aDictionary keysDo: [:object |
		(object class selectors includes: #modified) ifTrue:
			[weight := 1000 - (Date today subtractDate: object modified) max: 0.
			aDictionary at: object put:  (aDictionary at: object) + weight] ]! !

!WebIndex methodsFor: 'popular words' stamp: ' 21/8/07 09:38'!
allPopularWordsAndCounts
	"return a collection of words and their total counts, most popular first"
	| collection |
	collection := OrderedCollection new.
	self popularWords keysAndValuesDo: [:word :counter |  
		collection add: (Array with: word with: counter total)].
	^SortedCollection
		withAll: collection
		sortBlock: [:a :b | (a at: 2) > (b at: 2)].! !

!WebIndex methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
allWordsAndWeights
	"return sorted collection of all words and sum of weights, biggeest weight first"
	| collection weight |
	collection := OrderedCollection new.
	self index keysDo: [:halfWord |
		(self index at: halfWord) keysDo: [:restWord |
			weight := ((self index at: halfWord) at: restWord) inject: 0 into: 
				[:sum :thirdDictWeight | sum + (thirdDictWeight)].
			collection add: (Array with: halfWord, restWord with: weight) ] ].
	^SortedCollection 
		withAll: collection
		sortBlock: [:a :b | (a at: 2) > (b at: 2)].

" WebIndex default allWordsAndWeights "! !

!WebIndex methodsFor: 'popular words' stamp: ' 21/8/07 09:38'!
countPopularWords: anArray
	anArray do: [:word |
		(self popularWords includesKey: word) ifFalse: [self openPopularWord: word].
		(self popularWords at: word) incCounter].! !

!WebIndex methodsFor: 'private' stamp: ' 21/8/07 09:38'!
halfWordFrom: aString
	"prepare first three letters, lowercase (also slovene chars)"
	^AIDASite sloveneLowercase: (aString copyFrom: 1 to: 3).! !

!WebIndex methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
index
	"index is a dictionary with first three lowercase letters of each indexed world as key and
	another dictionary for remaining letters as value. The second dictionary has remaining letters for key
      and dictionary with objects, where this word occured, as key and number of occurences as value"
	index isNil ifTrue: [self initIndex].
	^index! !

!WebIndex methodsFor: 'indexing' stamp: ' 21/8/07 09:38'!
indexObject: anObject
	"if anObject responds to a message indexText, then index all its world in index.
	 However, first 	remove indexing for that object if already exist (so, for reindexing  
	objects, use this method also). For performance reasons indexing is done in background"
	self workQueue nextPut: anObject.

"WebIndex default indexObject: WebIndex default"! !

!WebIndex methodsFor: 'indexing' stamp: ' 21/4/07 22:07'!
indexObjects: aCollection
	aCollection do: [:each | self indexObject: each].! !

!WebIndex methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
indexText
	^'Search engine iskanje rezultati iskanja iskalnik'! !

!WebIndex methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
indexTitle
	^'Iskalnik'! !

!WebIndex methodsFor: 'indexing' stamp: ' 21/8/07 09:38'!
indexWord: aString inObject: anObject
	"put a word in that object in index if word is valid one for indexing"
	self indexWord: aString occurences: 1 inObject: anObject! !

!WebIndex methodsFor: 'private' stamp: ' 21/8/07 09:38'!
indexWord: aString occurences: aNumber inObject: anObject
	"put a aNumber occurences of a word in that object in index if word is valid one for indexing"
	| halfWord restWord secDic thirdDic num |
	(self isValidWord: aString) ifFalse: [^self].
	"put an object in indexed object, if not already exist"
	(self indexedObjects includesKey: anObject) ifFalse:
		[self indexedObjects at: anObject put: IdentitySet new].
	"prepare two parts of a words, first three letters and rest"
	halfWord := self halfWordFrom: aString.
	restWord := self restWordFrom: aString.
	"find or create a second level dictionary from first level one"
	secDic := self index at: halfWord 
		ifAbsent: [self index at: halfWord put: Dictionary new].
	"find or create last level dictionary from second level one"
	thirdDic := secDic at: restWord 
		ifAbsent: [secDic at: restWord put: Dictionary new].
	"put an object to third level dictionary and increment nr of worlds if not already exist"
	num := thirdDic at: anObject ifAbsent: [thirdDic at: anObject put: 0].
	thirdDic at: anObject put: num + aNumber.
	"make reference to the third level dictionary for a indexed word"
	(self indexedObjects at: anObject) add: thirdDic.! !

!WebIndex methodsFor: 'private' stamp: ' 21/8/07 09:38'!
indexWords: aWordDictionary inObject: anObject
	self removeObject: anObject.  "if exist"
	aWordDictionary keysAndValuesDo: [:key :value | 
		self indexWord: key occurences: value inObject: anObject].! !

!WebIndex methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
indexedObjects
	"return a dictionary with objects which worlds are in index as keys and a set of pointers to third 
       level dictionary in index as values"
	indexedObjects isNil ifTrue: [self initIndexedObjects].
	^indexedObjects! !

!WebIndex methodsFor: 'private' stamp: 'mivsek 6/13/2007 16:25'!
indexer
	"indexing process"
	^indexer! !

!WebIndex methodsFor: 'private' stamp: 'mivsek 6/13/2007 16:26'!
indexerProcess
	"this background process wait for objects to be indexed in workQueue and
	index them without disturbing others."
	| object |
	[true] whileTrue: 
		[object := self workQueue next.
		self privIndexObject: object.
		self reportIndexingOf: object].! !

!WebIndex methodsFor: 'initialize-release' stamp: ' 21/8/07 09:38'!
initIndex
	index := Dictionary new.! !

!WebIndex methodsFor: 'initialize-release' stamp: ' 21/8/07 09:38'!
initIndexedObjects
	indexedObjects := IdentityDictionary new.! !

!WebIndex methodsFor: 'initialize-release' stamp: ' 21/8/07 09:38'!
initPopularWords
	popularWords := Dictionary new.! !

!WebIndex methodsFor: 'initialize-release' stamp: 'mivsek 6/13/2007 16:26'!
initialize
	self initIndex.
	self initIndexedObjects.
	self initPopularWords.
	workQueue := nil.
	self start.

"WebIndex default initialize"! !

!WebIndex methodsFor: 'testing' stamp: ' 21/8/07 09:38'!
isValidWord: aString
	"a valid word for indexing has more than three letters, without numbers, "
	(aString size > 3) ifFalse: [^false].
	aString do:[:chr | chr isDigit ifTrue: [^false] ].
	^true! !

!WebIndex methodsFor: 'searching' stamp: ' 21/8/07 09:38'!
objectsForWord: aString
	"find all objects, which include this word. Return more important objects 
	first (more occurences of worlds etc.)"
	^self privObjectsForWord: aString.

"(WebIndex default objectsForWord: 'knjiga') keys "! !

!WebIndex methodsFor: 'searching' stamp: ' 21/8/07 09:38'!
objectsForWords: aString
	"find all objects, which include one or more specified words. return a sorted collection 
	of associations with found objects as keys an relevance of them as values. Most relevant 
	objects are first. Relevance means: 
		nr. of words found * 1000 + occurences of each word"
	| words hits newHits foundWords |
	Transcript show: ' search.'.
	words := (self wordsInText: aString) keys.
	words := words select: [:word | self isValidWord: word].
	hits := IdentityDictionary new.
	foundWords := OrderedCollection new. 
	words do: [:word | 
		newHits := self objectsForWord: word.
		newHits notEmpty ifTrue: [foundWords add: word].
		newHits keysAndValuesDo: [:key :value |
			(hits includesKey: key)
				ifTrue: [hits at: key put: (hits at: key)+1000]
				ifFalse: [hits at: key put: 1000+value] ] ].
	self countPopularWords: foundWords.  "count as popular only words, which exist in objects"
	self addFreshnessRelevance: hits.
	Transcript show: '.ok '.
	^SortedCollection withAll: hits associations sortBlock: [:first : second | first value > second value].

"
WebIndex instance objectsForWords: 'ljub'
"! !

!WebIndex methodsFor: 'popular words' stamp: ' 21/4/07 22:07'!
openPopularWord: aString
	self popularWords at: aString put: WebCounter new.! !

!WebIndex methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
popularWords
	"return a dictionary of words as keys and aWebCounters as values"
	popularWords isNil ifTrue: [self initPopularWords].
	^popularWords! !

!WebIndex methodsFor: 'private' stamp: ' 21/8/07 09:38'!
prepareWord: aString
	"cut last char if vowel. Used for simple declension of slovene words"
	aString last isVowel 
		ifTrue: [aString size > 3
			ifTrue: [^aString copyFrom: 1 to: (aString size - 1)] ].
	^aString.

"
WebIndex new prepareWord: 'knjiga'
"! !

!WebIndex methodsFor: 'private' stamp: 'np 3/21/2008 23:11'!
privIndexObject: anObject
	"if anObject responds to a message indexText, then index all its world in index. However, 
	first remove indexing for that object if already exist (so, for reindexing  objects, 
	use this method also)"
	| words |
	WebTransactionMonitor critical: 
		[(anObject class canUnderstand: #indexText) ifTrue:
			[words := self wordsInText: anObject indexText.
			(anObject class canUnderstand: #indexTitle) ifTrue:
				[words addAll: (self wordsInText: anObject indexTitle)].
			self indexWords: words inObject: anObject] ]! !

!WebIndex methodsFor: 'private' stamp: ' 21/8/07 09:38'!
privObjectsForWord: aString
	"find all objects, which include this word. Return more important objects first (more 
	occurences of worlds etc.)"
	| hits secDic searchWord keys word |
	(self isValidWord: aString) ifTrue:
		[hits := IdentityDictionary new.
		word := self prepareWord: aString.
		searchWord := (self restWordFrom: word), '*'.
		secDic := self index at: (self halfWordFrom: word) ifAbsent: [^hits].
		keys := secDic keys select: [:fkey | searchWord match: fkey].
		keys do: [:skey | 
       		(secDic at: skey) keysAndValuesDo: [:tkey :tvalue | 
				(hits includesKey: tkey) ifFalse: [hits at: tkey put: 0].
				hits at: tkey put: (hits at: tkey)+tvalue] ].
		^hits.
		] ifFalse: [^nil].

"(WebIndex default objectsForWord: 'knjiga') keys "! !

!WebIndex methodsFor: 'indexing' stamp: ' 21/8/07 09:38'!
removeObject: anObject
	"remove object from all word pointers in index"
	(self indexedObjects at: anObject ifAbsent: [^nil]) do:
		[:each | each removeKey: anObject ifAbsent: [] ].
	self indexedObjects removeKey: anObject.! !

!WebIndex methodsFor: 'private' stamp: 'mivsek 6/13/2007 16:29'!
reportIndexingOf: anObject
	| text |
	[text := anObject class name.
"	(anObject isKindOf: Document) ifTrue: 
		[text := text, '  ', anObject title asSloveneWithoutCircumflexes, 
			' on ', anObject repository site name.
		anObject modifiedBy notNil ifTrue:
			[text := text, ' by ', anObject modifiedBy nameSurname 
			            asSloveneWithoutCircumflexes] ].
"	  (anObject isKindOf: FileProxy) ifTrue: 
		[text := text, ' "', anObject filename asString, '" on ', anObject site name].
	Transcript cr; show: '(indexing  ', text, ')']
		fork. "to prevent stoping indexer in case of error"! !

!WebIndex methodsFor: 'private' stamp: ' 21/8/07 09:38'!
restWordFrom: aString
	"preparerest of the word, lowercase (also slovene chars)"
	^AIDASite sloveneLowercase: (aString copyFrom: 4 to: aString size)! !

!WebIndex methodsFor: 'initialize-release' stamp: 'mivsek 6/13/2007 16:30'!
start
	self stop.
	indexer := [self indexerProcess] forkAt: Processor userBackgroundPriority.! !

!WebIndex methodsFor: 'initialize-release' stamp: 'mivsek 6/13/2007 16:30'!
stop
	self indexer notNil ifTrue: [self indexer terminate. indexer := nil].! !

!WebIndex methodsFor: 'indexing' stamp: ' 21/8/07 09:38'!
wordsInText: aString
	"return a dictionary with all words as keys and number of occurences as values"
	| lastInx currInx word words |
	lastInx := 1. currInx := 1. words := Dictionary new. 
	aString do: [:chr | 
		(chr isAlphaNumeric or: [AIDASite isSloveneCharacter: chr]) ifFalse: 
			[word := aString copyFrom: lastInx to: currInx-1.
			lastInx := currInx+1.
			word := AIDASite sloveneLowercase: word.
			(word = '') ifFalse: 
				[(words includesKey: word) ifFalse: [words at: word put: 0].
				words at: word put: (words at: word) + 1] ].
		currInx := currInx + 1].
		word := aString copyFrom: lastInx to: currInx-1.
		word := AIDASite sloveneLowercase: word.
		(word = '') ifFalse: 
			[(words includesKey: word) ifFalse: [words at: word put: 0].
			words at: word put: (words at: word) + 1].
	^words

"
WebIndex instance wordsInText: 'erot ljub'
"! !

!WebIndex methodsFor: 'private' stamp: 'mivsek 6/13/2007 16:31'!
workQueue
	"all object to be indexed goes in this queue. An indexer process (which is started first time
	someone requests indexing) then index an object in background"

	workQueue isNil ifTrue: 
		[workQueue := SharedQueue new.
		self indexer isNil ifTrue: [self start] ].
	^workQueue.! !

Object subclass: #WebMethodResource
	instanceVariableNames: 'site object method contentType preferedUrl'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Components'!

WebMethodResource subclass: #WebLivePDFCreator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Components'!

!WebLivePDFCreator commentStamp: '<historical>' prior: 0!
WebLivePDFCreator created PDF from HTML source together with Table of contents!! It uses external HTMLDOC converter (http://www.htmldoc.org/)
!

!WebLivePDFCreator class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
fromHtmlMethod: aMethodSymbol on: anObject site: anAIDASite
	"this method should return a complete html page!! "
	| docname |
	docname := (anObject class canUnderstand: #preferedUrlName) 
		ifTrue: [anObject preferedUrlName] ifFalse: ['live'].
	^self fromMethod: aMethodSymbol on: anObject 
		contentType: 'application/pdf' preferedUrl: '/generated/', docname, '.pdf' site: anAIDASite! !

!WebLivePDFCreator methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
aidaDontCache
	"refresh content always!! "
	^true! !

!WebLivePDFCreator methodsFor: 'converting' stamp: 'np 4/2/2008 22:13'!
convertToPdf: aHtmlString
	| fname pdfname stream |
	fname := (Random new next * 1000) truncated printString, '.html'.
	pdfname := (Random new next * 1000) truncated printString, '.pdf'.
	[stream := (SpFilename named: fname) writeStream. stream nextPutAll: aHtmlString] 
		ensure: [stream close].
	OSProcess exec: 'htmldoc --batch htmldoc.book --outfile ', pdfname, ' ', fname.
	(SpFilename named: fname) delete.
	[stream := (SpFilename named: pdfname) readStream binary. ^stream contents] 
		ensure: [stream close. (SpFilename named: pdfname) delete].! !

!WebLivePDFCreator methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
expiresTimestamp
	^nil! !

!WebLivePDFCreator methodsFor: 'printing' stamp: 'janko 7/18/2008 20:40'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	| html pdf |
	html := AIDASite 
		convert: (self object perform: self method)
		toCodepage: #'iso-8859-2'.
	pdf := self convertToPdf: html.
	aStream nextPutAll: pdf asByteString! !

WebMethodResource subclass: #WebMethodImage
	instanceVariableNames: 'lastUsed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Components'!

WebMethodImage class
	instanceVariableNames: 'Cache'!

!WebMethodImage commentStamp: '<historical>' prior: 0!
This image is actually in-lined in a method, which is called on defined object and must return a ByteArray of image contents. 

Instance Variables:
	contentType	<String>	 content type of an image eg. 'image/gif', which is default.
	method	<Symbol>	symbol of a method to be called on object
	object	<Object>	object on which a method will be called

!

WebMethodImage class
	instanceVariableNames: 'Cache'!

!WebMethodImage class methodsFor: 'caching' stamp: ' 21/4/07 22:07'!
cache
	Cache isNil ifTrue: [self initCache].
	^Cache! !

!WebMethodImage class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
fromMethod: aSymbol on: anObject contentType: aString site: anAIDASite
	"cached one only preservers its url, but method is still called everytime image is shown!!"
	| cached | 
	cached := self imageForObject: anObject andMethod: aSymbol.
	^cached notNil 
		ifTrue: [cached] 
		ifFalse: [super new
			object: anObject;
			method: aSymbol;
			contentType: aString;
			site: anAIDASite;
			addToCache]! !

!WebMethodImage class methodsFor: 'caching' stamp: ' 21/4/07 22:07'!
imageForObject: anObject andMethod: aSymbol

	^self cache detect: [:each | each object == anObject and: [each method = aSymbol] ] ifNone: [nil]! !

!WebMethodImage class methodsFor: 'caching' stamp: ' 21/4/07 22:07'!
initCache
	Cache := Set new.! !

!WebMethodImage methodsFor: 'caching' stamp: ' 21/4/07 22:07'!
addToCache
	self makeRoomInCache.
	self class cache add: self.! !

!WebMethodImage methodsFor: 'caching' stamp: ' 21/4/07 22:07'!
cacheHysteresis
	"when removing from cache, make room for that number of entries"
	^10! !

!WebMethodImage methodsFor: 'accessing' stamp: 'JM 4/26/2007 17:48'!
lastUsed
	"timestamp of last usage. Used for caching algorithms. Set at HTML generation"

	lastUsed isNil ifTrue: [self lastUsed: SpTimestamp now].
	^lastUsed! !

!WebMethodImage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
lastUsed: aTimestamp
	"timestamp of last usage. Used for caching algorithms"

	lastUsed := aTimestamp! !

!WebMethodImage methodsFor: 'caching' stamp: ' 21/4/07 22:07'!
makeRoomInCache
	"if cache is full, remove least used images"
	| cached  toRemove |
	self class cache size < self maxCacheSize ifTrue: [^self].
	cached := (SortedCollection 
		withAll: self class cache
		sortBlock: [:a :b | a lastUsed < b lastUsed]) asOrderedCollection.
	self cacheHysteresis timesRepeat:
		[toRemove := cached first.
		toRemove removeFromCache.
		cached removeFirst]! !

!WebMethodImage methodsFor: 'caching' stamp: ' 21/4/07 22:07'!
maxCacheSize
	^50! !

!WebMethodImage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
preferedUrl
	| ext |
	ext := self site mimeMap extensionForType: self contentType.
	ext isNil ifTrue: [ext := '.html'].
	^'/img/' , self method asString asLowercase , ext! !

!WebMethodImage methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
printString
	^'WebMethodImage #', self method asString, ' on ', self object class name! !

!WebMethodImage methodsFor: 'caching' stamp: ' 21/4/07 22:07'!
removeFromCache
	self class cache remove: self ifAbsent: [].
	self removeFromURLResolver.! !

!WebMethodImage methodsFor: 'caching' stamp: ' 21/4/07 22:07'!
removeFromURLResolver
	self resolver notNil
		ifTrue: [self resolver removeObject: self]! !

!WebMethodImage methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
resolver
	^self site urlResolver! !

!WebMethodResource class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
fromMethod: aSymbol on: anObject contentType: aString preferedUrl: aString2 site: anAIDASite
	^super new
		object: anObject;
		method: aSymbol;
		contentType: aString;
		preferedUrl: aString2;
		site: anAIDASite! !

!WebMethodResource class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
fromMethod: aSymbol on: anObject contentType: aString site: anAIDASite
	^super new
		object: anObject;
		method: aSymbol;
		contentType: aString;
		site: anAIDASite! !

!WebMethodResource methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
contentType
	"MIME type for our resource"	
	contentType isNil ifTrue: [self contentType: 'image/gif'].
	^contentType! !

!WebMethodResource methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
contentType: aString
	contentType := aString! !

!WebMethodResource methodsFor: 'accessing' stamp: 'mivsek 1/10/2008 17:41'!
expiresTimestamp
	"24 hours from now, to avoid reloading of images" 
	^SpTimestamp fromSeconds: SpTimestamp now asSeconds + (24*3600)! !

!WebMethodResource methodsFor: 'testing' stamp: ' 21/8/07 09:38'!
isRespondingStreamed
	"is this page able to stream HTML directly into response?"
	^false! !

!WebMethodResource methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isWebElement
	^false! !

!WebMethodResource methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isWebPage
	^true! !

!WebMethodResource methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
method
	^method! !

!WebMethodResource methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
method: aSymbol
	method := aSymbol! !

!WebMethodResource methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
object
	^object! !

!WebMethodResource methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
object: anObject
	object := anObject! !

!WebMethodResource methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
preferedUrl
	^preferedUrl! !

!WebMethodResource methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
preferedUrl: aString
	preferedUrl := aString! !

!WebMethodResource methodsFor: 'printing' stamp: 'janko 7/18/2008 20:39'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	aStream nextPutAll: (self object perform: self method) asByteArray asString! !

!WebMethodResource methodsFor: 'printing' stamp: 'janko 7/18/2008 19:11'!
printWebPageFor: aRequest on: aSession 
	^self! !

!WebMethodResource methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
site
	"a Site on which this resource will be shown"
	^site! !

!WebMethodResource methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
site: anAIDASite
	site := anAIDASite! !

Object subclass: #WebMsgs
	instanceVariableNames: 'messagesByLanguage language'
	classVariableNames: 'Instance'
	poolDictionaries: ''
	category: 'Aida-Core'!

!WebMsgs class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
at: aSymbol

	"return a message text at specificied symbol in a language, which is previosly set by language: method. If not set, then default language is used"

	^self default at: aSymbol! !

!WebMsgs class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
at: aSymbol put: aString

	"store  a message text at specificied symbol in a language, which is previosly set by language: method. If not set, then default language is used"

	self default at: aSymbol put: aString! !

!WebMsgs class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
default

	"find or create an instance of WebMsgs."

	^AIDASite default webMsgs! !

!WebMsgs class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
language

	"return a current language, for which messages are retrieved"
	
	^self default language.! !

!WebMsgs class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
language: aSymbol

	"set a current language, for which messages are retrieved"

	self default language: aSymbol! !

!WebMsgs class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new

	^super new initialize! !

!WebMsgs methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
at: aSymbol
	"return a message text at specificied symbol in a language, which is previosly set 
	by language: method. If not set, then default language is used"
	^(self messagesByLanguage at: self language) 
		at: aSymbol asSymbol 
			ifAbsent: 
				[self at: aSymbol put: aSymbol asString. 
				^self at: aSymbol]! !

!WebMsgs methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
at: aSymbol put: aString
	"store  a message text at specificied symbol in a language, which is previosly set by language: 	method. If not set, then default language is used. Also opens new associations in all other
	languages, if not yet exist"
	(self messagesByLanguage at: self language) at: aSymbol put: aString.
	((self messagesByLanguage at: self language) associationAt: aSymbol).
	self messagesByLanguage keysAndValuesDo: [:key :value |
		(value includesKey: aSymbol asSymbol) 
			ifFalse: [value at: aSymbol asSymbol put: ''] ].! !

!WebMsgs methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initNewLanguage: aSymbol
	"in a messagesByLanguage dictionary open a new key and make an empty message dictionary 
	as value, but all symbol keys, which already exist in other languages"
	(self messagesByLanguage includesKey: aSymbol) ifFalse:
		[self messagesByLanguage at: aSymbol put: Dictionary new].
	(self messagesByLanguage at: #en) keysDo: [:key | 
		(self messagesByLanguage at: aSymbol) at: key put: ''].! !

!WebMsgs methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize

	self initNewLanguage: #en.


"
WebMsgs  at: #test put: 'This is my first test'.
WebMsgs language: #hr
WebMsgs at: #test
WebMsgs instance
"! !

!WebMsgs methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
language

	"return a current language, for which messages are retrieved"
	
	language isNil ifTrue: [self language: #en].
	^language! !

!WebMsgs methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
language: aSymbol
	"set a current language, for which messages are retrieved"
	language := aSymbol asSymbol.
	(self messagesByLanguage includesKey: aSymbol asSymbol) 
		ifFalse: [self initNewLanguage: aSymbol asSymbol].! !

!WebMsgs methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
messagesByLanguage
	"return a dictionary which language symbol as key and value asanother dictionary 
	with message symbol as key and message text as value"
	messagesByLanguage isNil ifTrue:
		[messagesByLanguage := Dictionary new. 
		self initNewLanguage: #en].
	^messagesByLanguage.! !

!WebMsgs methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
messagesForCurrentLanguage

	"return a dictionary of all messages for a current language"

	^(self messagesByLanguage at: self language)! !

Object subclass: #WebSecurityManager
	instanceVariableNames: 'site users groups authenticationScheme accessByObject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Core'!

!WebSecurityManager class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
default 
	^AIDASite default securityManager! !

!WebSecurityManager class methodsFor: 'hashing' stamp: 'mivsek 3/7/2008 23:09'!
hashPassword: aString
	"Returns a hashed string, used to store and compare passwords in a secure way"
	^(SecureHashAlgorithm new 
			hashMessage: (AIDASite convert: aString toCodepage: #UTF8) )
				asString! !

!WebSecurityManager class methodsFor: 'instance creation' stamp: 'mivsek 1/10/2008 19:01'!
newOn: anAIDASite
	^super basicNew 
		site: anAIDASite.
		"initialize"  "do that separately, otherwise it drops in indefinitive recursion"! !

!WebSecurityManager methodsFor: 'private' stamp: ' 21/4/07 22:07'!
accessByObject 
	"this is a dictionary of user - access level pairs for access to the object, 
	which reference is a key in dictionary"
	accessByObject isNil ifTrue: [self initAccessByObject].
	^accessByObject! !

!WebSecurityManager methodsFor: 'private' stamp: ' 21/4/07 22:07'!
accessRightsArray: anAccessSymbols for: aWebUserOrGroup on: anObject
	| objectRights rigthsArray |
	self site critical:
		[objectRights := self accessByObject at: anObject 
			ifAbsent: 
				[self accessByObject at: anObject put: (Dictionary new).
				self accessByObject at: anObject].
		rigthsArray := objectRights at: aWebUserOrGroup 
			ifAbsent:
				[objectRights at: aWebUserOrGroup put: IdentitySet new.
				objectRights at: aWebUserOrGroup].
		rigthsArray addAll: anAccessSymbols].! !

!WebSecurityManager methodsFor: 'access control' stamp: ' 21/4/07 22:07'!
accessRightsFor: aWebUserOrGroup on: anObject
	"get the access rights for specified user or group on object  as literal array of 
	rights (e.g. #(#read #write). " 
	^(self accessByObject at: anObject 
		ifAbsent: [^self defaultAccessRights])
			at: aWebUserOrGroup ifAbsent:  [^self defaultAccessRights]! !

!WebSecurityManager methodsFor: 'group management' stamp: ' 21/4/07 22:07'!
activatingGroup
	"group for users to confirm registration"
	^self groups detect: [:each | each isActivatingGroup] ifNone: [nil].! !

!WebSecurityManager methodsFor: 'user management' stamp: ' 21/4/07 22:07'!
addActivatingUser: aWebUser
	"add this user also to group waiting for confirmation"
	self addUser: aWebUser.
	self activatingGroup addUser: aWebUser.! !

!WebSecurityManager methodsFor: 'group management' stamp: ' 21/4/07 22:07'!
addGroup: aWebUserGroup
	self addGroup: aWebUserGroup ifExist: [^nil].! !

!WebSecurityManager methodsFor: 'private' stamp: ' 21/4/07 22:07'!
addGroup: aWebUserGroup ifExist: aBlock
	"add new group. If already exist ( as object or as group with the same name) do aBlock"
 	(aWebUserGroup isKindOf: WebUserGroup) ifFalse: [^self error: 'This is not aWebUserGroup'].
	(self existGroupNamed: aWebUserGroup name) ifTrue: [aBlock value].
	self groups add: aWebUserGroup! !

!WebSecurityManager methodsFor: 'group management' stamp: ' 21/4/07 22:07'!
addGroupNamed: aString
 	"add new group with a specified name. If already exist, error"
	self addGroup: (WebUserGroup new name: aString).! !

!WebSecurityManager methodsFor: 'user management' stamp: ' 21/4/07 22:07'!
addPerson: aPerson
	"make parallel WebUser, cross-connect both and put into Registered group"
	"by default username and password are aPerson surname!!"
	| user |
	aPerson asWebUser notNil ifTrue: [^self error: 'already added!!'].
	user := WebUser new.
	user username: aPerson surname.
	user password: aPerson surname.
	(self addRegisteredUser: user) notNil "no duplicates in username"
		ifTrue: [ aPerson webUser: user. user person: aPerson]
		ifFalse: [^nil]! !

!WebSecurityManager methodsFor: 'user management' stamp: ' 21/4/07 22:07'!
addRegisteredUser: aWebUser
	"add this user also to registered users group. Remove him from activating group"
	self addUser: aWebUser.
	self registeredGroup addUser: aWebUser.
	self activatingGroup removeUser: aWebUser.! !

!WebSecurityManager methodsFor: 'user management' stamp: ' 21/4/07 22:07'!
addUser: aWebUser
	"add new user also in All Users group"
	(aWebUser isKindOf: WebUser) ifFalse: [^self error: 'This is not aWebUser'].
	(self users includes: aWebUser) ifTrue: [^nil].
	(self existUserNamed: aWebUser username withPassword: aWebUser password) ifTrue: [^nil]. 
	self users add: aWebUser.
	self allUsersGroup addUser: aWebUser.
	aWebUser parent: self.! !

!WebSecurityManager methodsFor: 'group management' stamp: ' 21/4/07 22:07'!
adminGroup
	"group for admins, those who have all acess rights !! "
	^self groups detect: [:each | each isAdminGroup] ifNone: [nil].! !

!WebSecurityManager methodsFor: 'user management' stamp: ' 21/4/07 22:07'!
adminUser
	"first admin user, initialy it is with username 'admin'"
	^self users detect: [:each | each isAdmin] ifNone: [nil]! !

!WebSecurityManager methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allGroups
	^self groups copy! !

!WebSecurityManager methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allUsers
	^self users copy! !

!WebSecurityManager methodsFor: 'group management' stamp: ' 21/4/07 22:07'!
allUsersGroup
	"group for all users in system"
	^self groups detect: [:each | each isAllUsersGroup] ifNone: [nil].! !

!WebSecurityManager methodsFor: 'authentication' stamp: ' 21/4/07 22:07'!
authenticationScheme
	"#Form - with WebAdminApp login form - default!!
	#HttpBasic - rfc2617 Basic authentication - passwords NOT encrypted!!
	#HttpDigest - rfc2617 Digest authentication - encrypted passwords
	#SSLClientCertificate - most secure, user needs a valid PKI certificate"
	authenticationScheme isNil ifTrue: [self setFormAuthenticationScheme].
	^authenticationScheme! !

!WebSecurityManager methodsFor: 'private' stamp: ' 21/4/07 22:07'!
authenticationScheme: aSymbol
	authenticationScheme := aSymbol! !

!WebSecurityManager methodsFor: 'private' stamp: ' 21/4/07 22:07'!
copyAccessByObject
 
|dict|
Janko := Dictionary new.
WebSecurityManager default accessByObject 
keysAndValuesDo:
	[:key :value | 
		dict := Dictionary new.
		value keysAndValuesDo: [:key1 :value1 |
			dict at: key1 put: (IdentitySet withAll: (value1 collect: [:el | el asString asSymbol] )) ].
		Janko at: key put: dict].! !

!WebSecurityManager methodsFor: 'private' stamp: ' 21/4/07 22:07'!
defaultAccessRights
	
	^#()! !

!WebSecurityManager methodsFor: 'group management' stamp: ' 21/4/07 22:07'!
existGroupNamed: aString
	^(self groupNamed: aString) notNil! !

!WebSecurityManager methodsFor: 'user management' stamp: ' 21/4/07 22:07'!
existUserNamed: anUsernameString withPassword: aPasswordString
	^(self userNamed: anUsernameString withPassword: aPasswordString) notNil! !

!WebSecurityManager methodsFor: 'user management' stamp: ' 21/4/07 22:07'!
existUserWithId: aNumber
	^self users contains: [:each | each id = aNumber].! !

!WebSecurityManager methodsFor: 'user management' stamp: 'mivsek 1/10/2008 19:21'!
extranetUser
	"a common extranet user, for easier setup of access rights"
	^self users detect: [:each | each isExtranet] ifNone: [nil]! !

!WebSecurityManager methodsFor: 'group management' stamp: ' 21/4/07 22:07'!
groupNamed: aString
	aString isEmpty ifTrue: [^nil].
	^self groups detect: [:group | group name =  aString] ifNone: [nil]! !

!WebSecurityManager methodsFor: 'group management' stamp: ' 21/4/07 22:07'!
groupWithUuid: aString
	" find and return a group with specified uuid. Return nil if not found" 
	^self groups detect: [:each | each uuid = aString] ifNone: [nil]! !

!WebSecurityManager methodsFor: 'private' stamp: ' 21/4/07 22:07'!
groups 
	groups isNil ifTrue: [self initGroups].
	^groups! !

!WebSecurityManager methodsFor: 'user management' stamp: ' 21/4/07 22:07'!
guestUser
	"actually only one is guest user"
	^self users detect: [:each | each isGuest] ifNone: [nil]! !

!WebSecurityManager methodsFor: 'private' stamp: ' 21/4/07 22:07'!
has: aWebUserOrGroup rightTo: anAccessSymbol on: anObject 
	"test the access right for that object and that user or group "
	| usersAndGroups |
	usersAndGroups := self accessByObject at: anObject ifAbsent: [^false].
	(usersAndGroups includesKey: aWebUserOrGroup)	ifTrue:
		[((usersAndGroups at: aWebUserOrGroup) includes: anAccessSymbol) ifTrue: [^true] ].
	^false! !

!WebSecurityManager methodsFor: 'authentication' stamp: ' 21/4/07 22:07'!
hasFormAuthenticationScheme
	^self authenticationScheme = #Form! !

!WebSecurityManager methodsFor: 'authentication' stamp: ' 21/4/07 22:07'!
hasHttpAuthenticationScheme
	^self hasHttpBasicAuthenticationScheme | self hasHttpDigestAuthenticationScheme! !

!WebSecurityManager methodsFor: 'authentication' stamp: ' 21/4/07 22:07'!
hasHttpBasicAuthenticationScheme
	^self authenticationScheme = #HttpBasic! !

!WebSecurityManager methodsFor: 'authentication' stamp: ' 21/4/07 22:07'!
hasHttpDigestAuthenticationScheme
	^self authenticationScheme = #HttpDigest! !

!WebSecurityManager methodsFor: 'authentication' stamp: ' 21/4/07 22:07'!
hasSSLClientCertificateAuthenticationScheme
	^self authenticationScheme = #SSLClientCertificate! !

!WebSecurityManager methodsFor: 'access control' stamp: ' 21/4/07 22:07'!
hasUser: aWebUser rightInAnyGroupTo: anAccessSymbol on: anObject 
	"test the access right for that object and in groups for that user"
	aWebUser groups do: [:group |
		(self has: group rightTo: anAccessSymbol on: anObject) ifTrue: [^true] ].
	^(self has: self allUsersGroup rightTo: anAccessSymbol on: anObject)! !

!WebSecurityManager methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initAccessByObject
	accessByObject := Dictionary new.! !

!WebSecurityManager methodsFor: 'initialize-release' stamp: 'mivsek 6/13/2007 16:00'!
initAdminAccess
	"admin group has rights to all views and updates in all Apps!! "
	WebApplication allowAllViewsFor: self adminGroup on: self site.
	WebApplication allowAllUpdatesFor: self adminGroup  on: self site.! !

!WebSecurityManager methodsFor: 'initialize-release' stamp: 'mivsek 1/10/2008 19:21'!
initAdminUser
	"add default admin user"
	| user |
	(self users detect: [:each | each isAdmin and: [each name = WebUser adminName]] 
		ifNone: [nil] )
			notNil ifTrue: [^nil].
	user := WebUser newAdmin.
	self addUser: user.
	self adminGroup addUser: user.! !

!WebSecurityManager methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initDefaultGroups
	self addGroup: WebUserGroup newAllUsers.
	self users do: [:user | 	self allUsersGroup addUser: user].
	self addGroup: WebUserGroup newAdmin.
	self addGroup: WebUserGroup newRegistered.
	self addGroup: WebUserGroup newActivating.
	self postInitDefaultGroups.  "override this method for your own default groups"! !

!WebSecurityManager methodsFor: 'initialize-release' stamp: 'mivsek 1/11/2008 15:14'!
initDemoAccessRights
	"access to all for demos in WebDemoApp"
	"(AIDASite named: 'aidademo') securityManager initDemoAccessRights"
	| views updates |
	views := WebDemoApp allViews asArray
		collect: [:each | WebDemoApp viewRightSymbolFor: each].
	self setAccessRights: views for: self allUsersGroup on: WebDemoApp name.
	updates := #(editMembers fileUpload gridSelected webdav) 
		collect: [:each | WebDemoApp updateRightSymbolFor: each].
	self setAccessRights: updates for: self allUsersGroup on: WebDemoApp name.
! !

!WebSecurityManager methodsFor: 'initialize-release' stamp: 'mivsek 1/10/2008 19:22'!
initExtranetAccessRights
	"for extranet user"
! !

!WebSecurityManager methodsFor: 'initialize-release' stamp: 'mivsek 1/10/2008 19:22'!
initExtranetUser
	"add default extranet user"
	| user |
	(self users detect: [:each | each isExtranet and: [each name = WebUser extranetName]] 
		ifNone: [nil] )
			notNil ifTrue: [^nil].
	user := WebUser newExtranet.
	self addUser: user.
	self registeredGroup addUser: user.! !

!WebSecurityManager methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initGroups
	groups := Set new.! !

!WebSecurityManager methodsFor: 'initialize-release' stamp: 'mivsek 1/10/2008 19:22'!
initGuestAccessRights
	"for guest user"! !

!WebSecurityManager methodsFor: 'initialize-release' stamp: 'mivsek 1/10/2008 19:22'!
initGuestUser
	"add new user,with name 'Guest'"
	| user |
	(self users detect: [:each | each isGuest] ifNone: [nil] ) notNil ifTrue: [^nil].
	user := WebUser newGuest.
	self addUser: user.
	self allUsersGroup addUser: user.! !

!WebSecurityManager methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initUsers
	users := Set new. 
	self groups do: [:each | each initUsers].! !

!WebSecurityManager methodsFor: 'initialize-release' stamp: 'mivsek 1/11/2008 15:10'!
initialize
	self initUsers.
	self initGroups.
	self initAccessByObject.
	self initDefaultGroups.
	self initAdminUser. self initGuestUser. self initExtranetUser.
	self initDefaultAccessRights. self initDemoAccessRights.
	self initAdminAccess. self initGuestAccessRights. self initExtranetAccessRights.! !

!WebSecurityManager methodsFor: 'private' stamp: ' 21/4/07 22:07'!
isDefaultAllowedTo: anAccessSymbol 
	"default access right for all objects and all users"
	^self defaultAccessRights includes: anAccessSymbol! !

!WebSecurityManager methodsFor: 'access control' stamp: ' 21/4/07 22:07'!
isUser: aWebUser allowedTo: anAccessSymbol on: anObject 
	"test the access right for that object and that user or groups for that user"
	(self has: aWebUser rightTo: anAccessSymbol on: anObject) ifTrue: [^true].
	^(self hasUser: aWebUser rightInAnyGroupTo: anAccessSymbol on: anObject)! !

!WebSecurityManager methodsFor: 'user management' stamp: ' 21/4/07 22:07'!
isUser: aWebUser inGroup: aWebGroup
	"test if user is member of this group"
	^aWebGroup includes: aWebUser! !

!WebSecurityManager methodsFor: 'user management' stamp: ' 21/4/07 22:07'!
isUser: aWebUser inGroupNamed: aString
	| group |
	group := self groupNamed: aString.
	group isNil ifTrue: [^false].
	^group includes: aWebUser! !

!WebSecurityManager methodsFor: 'private' stamp: ' 21/4/07 22:07'!
migrateAppClassessToSymbols
	"do not use app classes but their names as symbols"

	self accessByObject keys do: [:obj |
		((obj isKindOf: Behavior) and: [obj includesBehavior: WebApplication]) ifTrue: [
			self accessByObject 
				at: obj name 
				put: (self accessByObject at: obj).
			self accessByObject removeKey: obj] ].

"WebSecurityManager default migrateAppClassessToSymbols"! !

!WebSecurityManager methodsFor: 'private' stamp: 'mivsek 3/7/2008 23:03'!
migrateToEncryptedPasswords
	"if not already"
	self allUsers do: [:each | each password size ~= 48 ifTrue: [each password: each password] ]

"WebSecurityManager allInstances do: [:each | each migrateToEncryptedPasswords] "
"WebSecurityManager hashPassword: 'aaaaaaaaa' "! !

!WebSecurityManager methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
postInitDefaultGroups
	"override this method for your own default groups"! !

!WebSecurityManager methodsFor: 'private' stamp: ' 21/4/07 22:07'!
reconnectUsersToGroups
	"if user groups don't match group users"
	"(AIDASite named: 'biart') securityManager reconnectUsersToGroups"
	self users do: [:user |
		user groups do: [:group | (group includes: user) ifFalse: [group addUser: user] ] ].
	self groups do: [:group |
		group users do: [:user | (user groups includes: group) ifFalse: [user addToGroup: group] ] ]! !

!WebSecurityManager methodsFor: 'group management' stamp: ' 21/4/07 22:07'!
registeredGroup
	"group for registered users"
	^self groups detect: [:each | each isRegisteredGroup] ifNone: [nil].! !

!WebSecurityManager methodsFor: 'private' stamp: ' 21/4/07 22:07'!
removeAccessRightArray: anAccessSymbols for: aWebUserOrGroup and: anObject 
	"remove all specified access rigths for specified user or group on specified object"
	self site critical:
		[((self accessByObject at: anObject ifAbsent: [^self])
			at: aWebUserOrGroup ifAbsent: [^self])	removeAll: anAccessSymbols].! !

!WebSecurityManager methodsFor: 'private' stamp: ' 21/4/07 22:07'!
removeAccessRightArray: anAccessSymbols for: aWebUserOrGroup on: anObject 
	"remove all specified access rigths for specified user or group on specified object"
	| usrGrps rights |
	self site critical:
		[usrGrps := self accessByObject at: anObject ifAbsent: [^self].
		rights := usrGrps at: aWebUserOrGroup ifAbsent: [^self].
		anAccessSymbols do: [:each | rights remove: each ifAbsent: [] ] ].! !

!WebSecurityManager methodsFor: 'access control' stamp: ' 21/4/07 22:07'!
removeAccessRights: anAccessSymbol for: aWebUserOrGroup on: anObject 
	"remove all specified access rigths for specified user or group on specified object"
	| access |
	(anAccessSymbol isKindOf: Array)
		ifTrue: [access := anAccessSymbol ]
		ifFalse: [access := Array with: anAccessSymbol].
	self removeAccessRightArray: access for: aWebUserOrGroup on: anObject! !

!WebSecurityManager methodsFor: 'access control' stamp: ' 21/4/07 22:07'!
removeAllAccessRightsFor: aWebUserOrGroup 
	self accessByObject keys do: [:object |
		self 
			removeAllAccessRightsFor: aWebUserOrGroup 
			on: object]! !

!WebSecurityManager methodsFor: 'access control' stamp: ' 21/4/07 22:07'!
removeAllAccessRightsFor: aWebUserOrGroup on: anObject 
	self site critical:
		[(self accessByObject at: anObject ifAbsent: [^self])
			removeKey: aWebUserOrGroup ifAbsent: [^self] ].! !

!WebSecurityManager methodsFor: 'access control' stamp: ' 21/4/07 22:07'!
removeAllAccessRightsForObject: anObject 
	self site critical:
		[self accessByObject removeKey: anObject ifAbsent: []	].! !

!WebSecurityManager methodsFor: 'group management' stamp: ' 21/4/07 22:07'!
removeGroup: aWebUserGroup
	"remove group, if exist. If not exist do nothing" 
	self removeGroup: aWebUserGroup ifAbsent: [].! !

!WebSecurityManager methodsFor: 'private' stamp: ' 21/4/07 22:07'!
removeGroup: aWebUserGroup ifAbsent: aBlock
	"remove group, if exist. If not exist do aBlock. You cannot remove group AllUsers!!" 
	aWebUserGroup isAllUsersGroup ifTrue: [^self error: 'group AllUsers cannot be removed!!'].
	(self groups includes: aWebUserGroup) ifFalse: [^aBlock value].
	self groups remove: aWebUserGroup.
	aWebUserGroup allUsers do: [:each | aWebUserGroup removeUser: each].
	self removeAllAccessRightsFor: aWebUserGroup! !

!WebSecurityManager methodsFor: 'group management' stamp: ' 21/4/07 22:07'!
removeGroupNamed:  aString 
	| group |
	group := self groupNamed: aString.
	group notNil ifTrue: [self removeGroup: group]! !

!WebSecurityManager methodsFor: 'user management' stamp: ' 21/4/07 22:07'!
removeUser: aWebUser
	"remove user, if exist" 
	(self users includes: aWebUser) ifFalse: [^nil].
	self users remove: aWebUser.
	self allGroups do: [:group | group removeUser: aWebUser].
	self removeAllAccessRightsFor: aWebUser! !

!WebSecurityManager methodsFor: 'user management' stamp: ' 21/4/07 22:07'!
removeUserNamed:  anUsername withPassword: aPassword
	| user |
	user := self userNamed: anUsername withPassword: aPassword.
	self removeUser: user! !

!WebSecurityManager methodsFor: 'access control' stamp: ' 21/4/07 22:07'!
setAccessRights: anAccessSymbols for: aWebUserOrGroup on: anObject
	"set the access rights for specified user or group on object. it can be only one right 
	or an literal array of rights (e.g.#(#read #write) ) " 
	| access |
	aWebUserOrGroup isNil ifTrue: [^self error: 'User is nil'].
	access := (anAccessSymbols isKindOf: Array) 
		ifTrue: [anAccessSymbols ] 
		ifFalse: [Array with: anAccessSymbols].
	self accessRightsArray: access for: aWebUserOrGroup on: anObject.! !

!WebSecurityManager methodsFor: 'authentication' stamp: ' 21/4/07 22:07'!
setFormAuthenticationScheme
	"AIDASite default securityManager setFormAuthenticationScheme"
	self authenticationScheme: #Form! !

!WebSecurityManager methodsFor: 'authentication' stamp: ' 21/4/07 22:07'!
setHttpBasicAuthenticationScheme
	"AIDASite default securityManager setHttpBasicAuthenticationScheme"
	self authenticationScheme: #HttpBasic! !

!WebSecurityManager methodsFor: 'authentication' stamp: ' 21/4/07 22:07'!
setHttpDigestAuthenticationScheme
	"AIDASite default securityManager setHttpDigestAuthenticationScheme"
	self authenticationScheme: #HttpDigest! !

!WebSecurityManager methodsFor: 'authentication' stamp: ' 21/4/07 22:07'!
setSSLClientCertificateAuthenticationScheme
	"AIDASite default securityManager setSSLClientCertificateAuthenticationScheme"
	self authenticationScheme: #SSLClientCertificate! !

!WebSecurityManager methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
site
	"a parent site using that security manager"
	^site! !

!WebSecurityManager methodsFor: 'private' stamp: ' 21/4/07 22:07'!
site: anAIDASite
	site := anAIDASite.! !

!WebSecurityManager methodsFor: 'user management' stamp: ' 21/4/07 22:07'!
userNamed: anUsernameString
	" find and return a WebUser with username . Return nil if not found"
	"WebSecurityManager default userNamed: 'mivsek'"
 	(anUsernameString ~= '') ifFalse: [^nil].
	^self users 
		detect: [:user | (user username asLowercase = anUsernameString asLowercase)] ifNone: [nil]! !

!WebSecurityManager methodsFor: 'user management' stamp: 'np 4/6/2008 18:46'!
userNamed: anUsernameString withPassword: aPasswordString
	" find and return a WebUser with username and password. Return nil if not found"
 	((anUsernameString = '') | (aPasswordString = '')) ifTrue: [^nil].
	^self users detect: [:user | 
		(user username asLowercase = anUsernameString asLowercase) and: 
			[user password = (WebSecurityManager hashPassword: aPasswordString)]] 
				ifNone: [nil]
! !

!WebSecurityManager methodsFor: 'user management' stamp: ' 21/4/07 22:07'!
userWithEMail: aString
	" find and return a WebUser with specified email. Return nil if not found" 
	(aString ~= '') ifFalse: [^nil]. 
	^self users detect: [:user | user email asLowercase = aString asLowercase ] ifNone: [^nil]! !

!WebSecurityManager methodsFor: 'user management' stamp: ' 21/4/07 22:07'!
userWithId: aNumber
	^self users detect: [:each | each id = aNumber] ifNone: [nil]! !

!WebSecurityManager methodsFor: 'user management' stamp: ' 21/4/07 22:07'!
userWithUuid: aString
	" find and return a WebUser with specified uuid. Return nil if not found" 
	^self users detect: [:each | each uuid = aString] ifNone: [nil]! !

!WebSecurityManager methodsFor: 'private' stamp: ' 21/4/07 22:07'!
users 
	users isNil ifTrue: [self initUsers].
	^users! !

Object subclass: #WebSession
	instanceVariableNames: 'ids created cookies parent user parms requests lastRequest language country appsForObjects lastApp lastView clipboard redirectLink redirectOrigin userValues'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Core'!

!WebSession class methodsFor: 'instance creation' stamp: 'mivsek 8/22/2007 21:17'!
new
	^super basicNew.
! !

!WebSession class methodsFor: 'instance creation' stamp: 'JM 4/25/2007 21:21'!
newOn: aWebSessionManager
	"new web session on a specified session manager"
	| session |
	session := super basicNew.
	session parent: aWebSessionManager.
	session initialize.
	^session! !

!WebSession methodsFor: 'private' stamp: ' 21/4/07 22:07'!
addApp: anApplication for: anObject type: aSymbol
	"web or wap apps, type can be #web or #wap "
	| objDict |
	objDict := self appsForObjects at: anObject ifAbsentPut: [Dictionary new].
	objDict at: aSymbol put: anApplication! !

!WebSession methodsFor: 'private' stamp: ' 21/4/07 22:07'!
addSessionID
	"add session id to  parms dictionary. Url with session id looks like: 	http://www.tris-a.si/welcome.html?id=523453. This should be done for every automaticaly 
	generated url (WebLink printHTMLPage) to distinguish sesions among themselves."
	self parms at: 'id' put: self id printString.! !

!WebSession methodsFor: 'web/wap apps' stamp: ' 21/4/07 22:07'!
addWebApp: aWebApplication for: anObject
	^self addApp: aWebApplication for: anObject type: #web! !

!WebSession methodsFor: 'security' stamp: ' 21/4/07 22:07'!
adminAllowed
	"check if general administration is allowed for user on our session"
	^self site securityManager
		isUser: self user
		inGroupNamed: self site securityManager adminGroup name! !

!WebSession methodsFor: 'private' stamp: ' 21/4/07 22:07'!
appFor: anObject type: aSymbol 
	^(self appsForObjects at: anObject ifAbsent: [^nil]) 
		at: aSymbol ifAbsent: [^nil]! !

!WebSession methodsFor: 'private' stamp: ' 21/4/07 22:07'!
appsForObjects
	
	appsForObjects isNil ifTrue: [self initAppsForObjects].
	^appsForObjects! !

!WebSession methodsFor: 'security' stamp: ' 21/4/07 22:07'!
authenticateFrom: aRequest
	"try to login with username and password from request (rfc2617)"
	| usr |
	aRequest username isNil ifTrue: [^nil].
	usr := self site securityManager userNamed: aRequest username.
	usr isNil ifTrue: [^nil].
	(aRequest matchUserWithPassword: usr password) 
		ifTrue: [self loginUser: usr] ifFalse: [self logout].! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
browser
	
	"try to find out, which browser a web request originator is using. It can be #Netscape, #IBM, #Microsoft, #lynx, #Mosaic.  Returns #Unknown if not possible to find out the browser type"

	| partOfString |
	self browserString = '' ifTrue: [^#Unknown].
	partOfString := self browserString copyFrom: 1 to: 6.
	partOfString = 'Mozill' ifTrue: [^#Netscape].
	partOfString = '' ifTrue: [^#Microsoft].
	partOfString = 'IBM-We' ifTrue: [^#IBM].
	partOfString = '' ifTrue: [^#Lynx].
	partOfString = '' ifTrue: [^#Mosaic].! !

!WebSession methodsFor: 'accessing' stamp: 'JM 4/21/2007 22:26'!
browserString
	
	"return the string, whick browser sends as identification in a web request"

	self lastRequest isNil ifTrue: [^''].
	^self lastRequest envVariables at: #'http_user_agent'.! !

!WebSession methodsFor: 'security' stamp: 'mivsek 1/10/2008 19:29'!
checkExpirationAndPossiblyLogoutFor: aRequest
	"don't logout if request is POST on the same object (eg. after long editing)"
	self isLoggedIn ifFalse: [^self].
	(self site autoLogout or: [self user autoLogout]) ifFalse: [^self].
	(self isExpired and: [aRequest isPost not]) ifTrue: 
		[self logout.
		(self site admin webAppFor: self) error: self site style sessionTimeoutText].! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
clipboard
	"for cut/copy/paste references to web pages to easier hyperlink them"
	clipboard isNil ifTrue: [self initClipboard].
	^clipboard! !

!WebSession methodsFor: 'locale support' stamp: ' 21/4/07 22:07'!
codePage
	^#'UTF_8' "always!!"! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
cookie
	"true if web request in this nonsecure session uses cookies, false otherwise"
	^self cookies at: 1.! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
cookie: aBoolean
	self cookies 	at: 1 put: aBoolean! !

!WebSession methodsFor: 'private' stamp: ' 21/4/07 22:07'!
cookies
	cookies isNil ifTrue: [self initCookies].
	^cookies! !

!WebSession methodsFor: 'locale support' stamp: ' 21/4/07 22:07'!
country
	^country! !

!WebSession methodsFor: 'locale support' stamp: ' 21/4/07 22:07'!
country: aSymbol
	"set the country of a web request originator in ISO 2letter format"
	country := aSymbol.! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
created

	"return the timestamp of a session creation."

	^created! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
fullUrlForCurrentPage
	"composes and returns a full url for a curent requested page. If parms dictionary is changed, 
	then a query string with changed parameters is generated. Usefull for presenting the same 
	page in a different way by parameteres in query string"
	^self lastRequest urlString! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
fullUrlForPreviousPage
	""
	(self requests isEmpty or: [self requests last isNil]) ifTrue: [^nil].
	^self requests last urlString! !

!WebSession methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
hasSecureCookie
	"check if secure cookie is already set on secure part of session"
	^self secureCookie! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
id

	"return the unique identification of a nonsecure session to browser. This is a random generated number at session generation. It can be used for session tracking in URLs, forms, etc."

	^self ids at: 1! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
id: aNumber
	"return the unique identification of a nonsecure session. This is a random generated number 
	at session generation. It can be used for session tracking in URLs, forms, etc."
	self ids at: 1 put: aNumber.! !

!WebSession methodsFor: 'private' stamp: ' 21/4/07 22:07'!
ids
	ids isNil ifTrue: [self initIds].
	^ids! !

!WebSession methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initAppsForObjects
	appsForObjects := IdentityDictionary new.! !

!WebSession methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initClipboard
	clipboard := WebClipboard new! !

!WebSession methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initCookies
	cookies := Array with: false with: false.! !

!WebSession methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initIds
	ids := Array new: 2.! !

!WebSession methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initParms
	parms := Dictionary new.! !

!WebSession methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initRequests
	requests := OrderedCollection new.! !

!WebSession methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initUser
	self user: self site securityManager guestUser! !

!WebSession methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	self setRandomIds.
	self setCreatedTimestamp.
	self initCookies.
	self initParms.
	self initRequests.
	self initUser.
	self initAppsForObjects.! !

!WebSession methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isActive

	"active if at least one request received last hour"

	self lastRequest isNil ifTrue: [^false].
	^(Timestamp now asSeconds - self lastRequest timestamp asSeconds) < 3600! !

!WebSession methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isEncrypted
	^self lastRequest isEncrypted "for now!! "! !

!WebSession methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isExpired
	"more than 15min of inactivity"
	self lastRequest isNil ifTrue: [^false].
	^(Timestamp now asSeconds - self lastRequest timestamp asSeconds) > (15*60)! !

!WebSession methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isFromLinux
	^self lastRequest isFromLinux! !

!WebSession methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isFromMSIE
	^self lastRequest isFromMSIE! !

!WebSession methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isFromNetscape
	".. or Moziila or Firefox"
	^self lastRequest isFromNetscape! !

!WebSession methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isFromWindows
	^self lastRequest isFromWindows! !

!WebSession methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isHttpAuthenticationNeeded
	"Usually we authenticate by WebAdminApp logon form. But if site is configured for HTTP 	authentication (to open a separate username/password dialog window to user), and 
	a session is not yet logged in, this method will return true"
	^self isLoggedIn not and: [self site securityManager hasHttpAuthenticationScheme]! !

!WebSession methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isLoggedIn
	^self user notNil and: [self user isGuest not]! !

!WebSession methodsFor: 'locale support' stamp: 'mivsek 1/10/2008 19:30'!
language
	"preferred language of a web user. Initialy set from the last request (lazily, 
	when first needed)"
	(language isNil and: [self lastRequest notNil]) ifTrue:
		[self language: (self originatorLanguageFrom: self lastRequest)].
	^language! !

!WebSession methodsFor: 'locale support' stamp: 'mivsek 1/10/2008 19:30'!
language: aLanguageCodeSymbol
	"preferred language of a web user. Initialy set from the last request (lazily, 
	when first needed)"
	language := aLanguageCodeSymbol.! !

!WebSession methodsFor: 'web/wap apps' stamp: ' 21/4/07 22:07'!
lastApp
	"return WebApplication subclass which was called with last request"
	^lastApp! !

!WebSession methodsFor: 'web/wap apps' stamp: ' 21/4/07 22:07'!
lastApp: anObject
	lastApp := anObject! !

!WebSession methodsFor: 'accessing' stamp: 'mivsek 1/10/2008 19:30'!
lastRequest
	"the last, actualy current request pending. also add request to the history in a request 
	if logging is on."
	"BE CAREFULL!! more than one request can be processed concurrently, so don't count 
	that last reques is actually a current one too!! It mostly is, but not always!!"
	^lastRequest! !

!WebSession methodsFor: 'accessing' stamp: 'mivsek 1/10/2008 19:32'!
lastRequest: aRequest
	"remember the last, actualy current request pending. also add request to the history in 
	a request if logging is on. If first request then also try to find country, 
	language and codePage 
	of the originator"
	self lastRequest isNil ifTrue:   "first request on a session"
		[lastRequest := aRequest.  
"		self country: (self originatorCountryFrom: aRequest).
		self language: (self originatorLanguageFrom: aRequest)"].  "this is set lazily"
	self requests isEmpty 
		ifTrue: [self requests add: self lastRequest] 
			ifFalse: [self requests at: 1 put: self lastRequest]. 
	lastRequest := aRequest.  
"	self site logging ifTrue: [self requests add: aRequest]. "
	parms := Dictionary new.       "clear old paramters"
	self newView: self viewFromLastRequest.   "default new view should be view 
		from last request"! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
lastView
	"return a view of WebApllication, which is created in response to last request"
	^lastView! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
lastView: aSymbol

	lastView := aSymbol! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
lastViewOld

	"return a value of a 'view' parameter in query part of last request's URL. If no such a parameter, then returns '' "

	^self lastRequest queryAt: 'view' ifAbsent: [^''].! !

!WebSession methodsFor: 'security' stamp: ' 21/4/07 22:07'!
loginUser: aWebUser
	self user: aWebUser.! !

!WebSession methodsFor: 'security' stamp: ' 21/4/07 22:07'!
logout
	"Replace logged user with Guest. Remember last page to return after login again"
	| origin usr |
	self redirectLink: nil. 	self redirectOrigin: nil.
	self isLoggedIn ifTrue: [usr := self user. self initUser] ifFalse: [^nil].
	origin := usr lastAppUrl. 
	(origin isNil or: [origin = self lastRequest uriString])  ifFalse: 
		[usr logoutFromUrl: origin]. "to jump back after login"! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
newView

	"return a value of a 'view' parameter in query part of a URL to be generated. Also used for changing views in a state machine for web applications (in method actionFormForm (here is set) and method printWebPage (here is used to genererate appropriate page))"

	^self parms at: 'view' ifAbsent: [^''].! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
newView: aString
	"set a value of a 'view' parameter in query part of a URL to be generated. Also used for changing 
	views in a state machine for web applications (in method actionFormForm (here is set) and method 	printWebPage (here is used to genererate appropriate page))"
	((aString = '') or: [aString isNil]) ifFalse: [self parms at: 'view' put: aString ].! !

!WebSession methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
nilLastApp
	lastApp := nil! !

!WebSession methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
nilLastRequest
	lastRequest := nil! !

!WebSession methodsFor: 'private-locale' stamp: ' 21/4/07 22:07'!
originatorCountryFrom: aRequest
	"find an ISO country code from a web request"
	"not yet implemented!!"! !

!WebSession methodsFor: 'private-locale' stamp: 'mu 4/6/2008 10:55'!
originatorLanguageFrom: aRequest
	"find a language from a web request"
	| defaultLang header |
	defaultLang := self site defaultLanguage.
	header := aRequest headerAt: 'Accept-Language' ifAbsent:[^defaultLang].
	header values isEmpty ifFalse: [^(header values at: 1) asSymbol].
	^defaultLang! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
parent
	"a session manager !! "
	^parent! !

!WebSession methodsFor: 'private' stamp: ' 21/4/07 22:07'!
parent: aWebSessionManager
	parent := aWebSessionManager.! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
parms
	"This is a dictionary of parameters in query part of url. When web request arrives, paramteres from query string are written here. When any url is dynamically generated, those parms are appended to it as a query string. You can add, change or delete any parameter to better suit your needs (eg. view=brief to instruct brief view of an object) "


	^parms! !

!WebSession methodsFor: 'private' stamp: ' 21/4/07 22:07'!
printString
^'aWebSession
	ip: ', (self lastRequest notNil ifTrue: [self lastRequest ip] ifFalse: ['']), '
	user: ', self user username, ' (', self user nameSurname, ')
	created: ', self created printSloString, '
	last:      ', (self lastRequest notNil ifTrue: [self lastRequest timestamp printSloString] ifFalse: ['']), ''! !

!WebSession methodsFor: 'accessing' stamp: 'mu 4/10/2008 19:59'!
queryAt: aKey
	"Shortcut of HTTPRequest>>queryAt:"
	| value |
	value := self lastRequest queryAt: aKey.
	value isNil ifTrue: [^nil].
	^value! !

!WebSession methodsFor: 'accessing' stamp: 'mu 4/10/2008 19:58'!
queryStringAt: aKey
	"If the query value could include multibyte string, use this"
	| value |
	value := self queryAt: aKey.
	value isNil ifTrue: [^''].
	^AIDASite convert: value fromCodepage: self codePage! !

!WebSession methodsFor: 'redirection' stamp: ' 21/4/07 22:07'!
redirectLink
	"if this aWebLink is set, then web browser will recreate another request with url from that link. Used, if you like, after form action in one page to show page for some other object, e.g. if you have search field in page in one object, and search result is exactly one, then you want to show result page immediately. You can do this with this method.
Atribute redirectLink is reset to nil immediately after printWebPage and before printHTMLPage in WebMediator method dicpatchClient (see WebMediator sendResponseHeaderOn:)"


	^redirectLink! !

!WebSession methodsFor: 'redirection' stamp: ' 21/4/07 22:07'!
redirectLink: aWebLink
	(aWebLink isKindOf: WebLink) | aWebLink isNil ifTrue: [redirectLink:= aWebLink].! !

!WebSession methodsFor: 'redirection' stamp: ' 21/4/07 22:07'!
redirectOrigin

	"this is an URL from where the redirection request originates. Used for returning back from
	exception page, for example when you want to see page without access rights, then you are 	redirected to login page. after successfull login, you are redirected back to intended page.
	Redirect origin is set up automatically by redirectLink: request."

	^redirectOrigin! !

!WebSession methodsFor: 'redirection' stamp: ' 21/4/07 22:07'!
redirectOrigin: anUrlString
	redirectOrigin := anUrlString.! !

!WebSession methodsFor: 'redirection' stamp: ' 21/4/07 22:07'!
redirectToOrigin

	"Used for returning back from
	exception page, for example when you want to see page without access rights, then you are 	redirected to login page. after successfull login, you are redirected back to intended page.
	Redirect origin is set up automatically by redirectLink: request."

	self redirectOrigin notNil
		ifTrue:
			[self redirectLink: (WebLink text: '' linkTo: self redirectOrigin).
			self redirectOrigin: nil].! !

!WebSession methodsFor: 'private' stamp: ' 21/4/07 22:07'!
removeApp: anApplication for: anObject type: aSymbol
	(self appsForObjects at: anObject ifAbsent: [^nil])
		removeKey: aSymbol ifAbsent: [^nil]! !

!WebSession methodsFor: 'web/wap apps' stamp: ' 21/4/07 22:07'!
removeObject: anObject
	self appsForObjects 	removeKey: anObject ifAbsent: [].! !

!WebSession methodsFor: 'web/wap apps' stamp: ' 21/4/07 22:07'!
removeWebApp: aWebApplication for: anObject
	^self removeApp: aWebApplication for: anObject type: #web! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
removeYourself
	" .. from session manager"
	self parent notNil ifTrue: [self parent removeSession: self].
	self initAppsForObjects.
	self parent: nil.! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
requestNum

	"return number of all requests up to now on this session"

	^requests size! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
requests
	"return the history of all requests on this session as ordered collection with the oldest as 
	first and newest as last"
	requests isNil ifTrue: [requests := OrderedCollection new].
	^requests! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
secureCookie
	"true if web request in this secure session uses cookies, false otherwise"
	^self cookies at: 2! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
secureCookie: aBoolean
	self cookies at: 2 put: aBoolean! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
secureId

	"return the unique identification of a secure session to browser. This is a random generated number at session generation. It can be used for session tracking in URLs, forms, etc."

	^self ids at: 2! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
secureId: aNumber
	"return the unique identification of a secure session. This is a random generated number 
	at session generation. It can be used for session tracking in URLs, forms, etc."
	self ids at: 2 put: aNumber.! !

!WebSession methodsFor: 'servlets' stamp: ' 21/4/07 22:07'!
servletAppFor: aWebAppSymbol
	| app class |
	(self servlets includesKey: aWebAppSymbol) ifFalse:
		[class := WebApplication allSubclasses detect:  
			[:each | each name = aWebAppSymbol] ifNone: [^nil].
		app := class new.
		app session: self.
		self servlets at: aWebAppSymbol put: app].
	^self servlets at: aWebAppSymbol.! !

!WebSession methodsFor: 'servlets' stamp: ' 21/4/07 22:07'!
servlets
	(self userValues includesKey: #ServletApps) ifFalse:
		[self userValues at: #ServletApps put: IdentityDictionary new].
	^self userValues at: #ServletApps! !

!WebSession methodsFor: 'initialize-release' stamp: 'JM 4/25/2007 21:10'!
setCreatedTimestamp
	created := SpTimestamp now.! !

!WebSession methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
setRandomIds
	self id: (Random new next * 1000000000) asInteger.
	self secureId: (Random new next * 1000000000) asInteger.! !

!WebSession methodsFor: 'testing' stamp: 'mivsek 1/10/2008 19:34'!
shouldCountRequests
	| policy |
	policy := self site countingPolicy.
	policy = #all ifTrue: [^true].
	policy = #none ifTrue: [^false].
	policy = #onlyGuests ifTrue: [^self user isGuest].
	policy = #excludeAdmins ifTrue: [^self user isAdmin not].
	^true! !

!WebSession methodsFor: 'testing' stamp: 'mivsek 1/10/2008 19:34'!
shouldLogRequests
	| policy |
	policy := self site loggingPolicy.
	policy = #all ifTrue: [^true].
	policy = #none ifTrue: [^false].
	policy = #onlyGuests ifTrue: [^self user isGuest].
	policy = #excludeAdmins ifTrue: [^self user isAdmin not].
	^true! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
site
	"a session manager !! "
	^self parent site! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
user
	user isNil ifTrue: [self initUser].
	^user! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
user: aWebUser
	"set a reference to aWebUser, who is logged into this session"
	user := aWebUser.! !

!WebSession methodsFor: 'user values' stamp: ' 21/4/07 22:07'!
userValueAt: aSymbol
	"anything you need to share among Apps in that seesion, like last selected stuff, etc."
	^self userValueAt: aSymbol ifAbsent: [nil]! !

!WebSession methodsFor: 'user values' stamp: ' 21/4/07 22:07'!
userValueAt: aSymbol ifAbsent: aBlock
	^self userValues at: aSymbol ifAbsent: aBlock! !

!WebSession methodsFor: 'user values' stamp: ' 21/4/07 22:07'!
userValueAt: aSymbol put: anObject
	^self userValues at: aSymbol put: anObject! !

!WebSession methodsFor: 'private' stamp: ' 21/4/07 22:07'!
userValues
	"return a dictionary with user defined values, which scope is this session. Used for such things
	as global navigation, remembering views etc."

	userValues isNil ifTrue: [userValues := Dictionary new].
	^userValues! !

!WebSession methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
viewFromLastRequest

	"return a value of a 'view' parameter in query part of last request's URL. If no such a parameter, then returns '' "

	^self lastRequest queryAt: 'view' ifAbsent: [''].! !

!WebSession methodsFor: 'web/wap apps' stamp: ' 21/4/07 22:07'!
webAppFor: anObject
	^self appFor: anObject type: #web! !

Object subclass: #WebSessionManager
	instanceVariableNames: 'site sessions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Core'!

!WebSessionManager class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
default
	^AIDASite default sessionManager! !

!WebSessionManager class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newOn: aSite
	^super new 
		initialize; 
		site: aSite! !

!WebSessionManager methodsFor: 'adding-removing' stamp: ' 7/6/08 18:41'!
addSession: aWebSession
	self site critical:
		[self sessions 
			at: aWebSession id put: aWebSession;
			at: aWebSession secureId put: aWebSession]! !

!WebSessionManager methodsFor: 'accessing' stamp: ' 7/6/08 18:41'!
allActiveSessions
	^self allSessions select: [:each | each user isActive].! !

!WebSessionManager methodsFor: 'accessing' stamp: ' 7/6/08 18:41'!
allGuestSessions
	^self allSessions select: [:each | each user isGuest].! !

!WebSessionManager methodsFor: 'accessing' stamp: ' 7/6/08 18:41'!
allSessions
	^self sessions values asSet  "to remove duplicates because the session id can be double, 
							     one for ssl too!!"! !

!WebSessionManager methodsFor: 'private' stamp: ' 7/6/08 18:41'!
bindSSLSessionFor: aRequest
	"Try to find a session among currently live sessions. It helps with a session id in a query 
	part of url. If not found, or id not in query string, then make a new session"
	| session id |
	id := aRequest queryAt: 'sessionId' ifAbsent: [self error: 'no session id!!'].
	session := (self existSessionWithID: id asInteger)
		ifTrue: [self findSessionWithID: id asInteger]
		ifFalse: [self error: 'no session with that id'].
	session requests size = 1
		ifTrue: [self countNewVisitor]
		ifFalse: 	[self checkAndCountReturningVisitor: session].
	session lastRequest: aRequest.
	(aRequest includesQuery:  'setLanguage') ifTrue:
		[session language: (aRequest queryAt: 'setLanguage') asSymbol].
	^session.! !

!WebSessionManager methodsFor: 'private' stamp: ' 7/6/08 18:41'!
checkAndCountReturningVisitor: aSession
	"returning visitor is those who is not active more than 1h"
	(aSession isActive not and: [aSession requests size > 1]) ifTrue:
		[self site critical: [self site returningVisitorsCounter incCounter] ]! !

!WebSessionManager methodsFor: 'private' stamp: ' 7/6/08 18:41'!
countNewVisitor
	self site critical:
		[self site newVisitorsCounter incCounter]! !

!WebSessionManager methodsFor: 'testing' stamp: ' 7/6/08 18:41'!
existSessionWithID: anIDNumber
	^self sessions includesKey: anIDNumber.
! !

!WebSessionManager methodsFor: 'adding-removing' stamp: ' 7/6/08 18:41'!
findOrMakeSessionFor: aRequest
	"Try to find a session among currently live sessions. It helps with a session id in a query 
	part of url. If not found, or id not in query string, then make a new session"
	| session id cookie |
	(self hasSSLBinding: aRequest) ifTrue: [^self bindSSLSessionFor: aRequest].
	(aRequest hasCookie and: [aRequest idFromCookie notNil])
		ifTrue: [id := aRequest idFromCookie. cookie := true. self site log: ' cookie ' ]
		ifFalse: 
			[cookie := false. id := aRequest queryAt: 'id' ifAbsent: [(self newSessionCookie: cookie) id] ].
	session := (self existSessionWithID: id asInteger)
		ifTrue: [self findSessionWithID: id asInteger] 
		ifFalse: [self newSessionCookie: cookie id: id asInteger].
	session requests size = 1 
		ifTrue: [self countNewVisitor] ifFalse: 	[self checkAndCountReturningVisitor: session].
	session checkExpirationAndPossiblyLogoutFor: aRequest.
	session lastRequest: aRequest.
	session cookie: cookie. 
	(cookie not and: [self site urlsWithSessionId]) ifTrue: [session addSessionID].

	(aRequest includesQuery:  'setLanguage') ifTrue:
		[session language: (aRequest queryAt: 'setLanguage') asSymbol].
	^session.! !

!WebSessionManager methodsFor: 'accessing' stamp: ' 7/6/08 18:41'!
findSessionWithID: anIDNumber
	^self sessions at: anIDNumber ifAbsent: [nil].
! !

!WebSessionManager methodsFor: 'private' stamp: ' 7/6/08 18:41'!
hasSSLBinding: aRequest
	"if sessionId=456546464 exist in query part of request. This means that this is 
	a SSL session with its own cookie and must be bind to a normal session with sessionId"

	^aRequest isEncrypted and: [aRequest includesQuery: 'sessionId']! !

!WebSessionManager methodsFor: 'initialize-release' stamp: ' 7/6/08 18:41'!
initSessions
	sessions := Dictionary new.! !

!WebSessionManager methodsFor: 'initialize-release' stamp: ' 7/6/08 18:41'!
initialize
	self initSessions.

"WebSessionManager default initialize"! !

!WebSessionManager methodsFor: 'private' stamp: ' 7/6/08 18:41'!
newSessionCookie: aBoolean
	"open and return a fresh new session"
	^self newSessionCookie: aBoolean id: nil! !

!WebSessionManager methodsFor: 'private' stamp: ' 7/6/08 18:41'!
newSessionCookie: aBoolean id: aSessionID
	"open and return a fresh new session"
	| session |
	session := WebSession newOn: self.
	aSessionID notNil ifTrue: [session id: aSessionID].
	self site log: ' new session '.
	self addSession: session.
	session cookie: aBoolean.
	^session! !

!WebSessionManager methodsFor: 'adding-removing' stamp: ' 7/6/08 18:41'!
removeGuestSessions
	^self allGuestSessions do: [:each | 
		each parent == self 
			ifTrue: [each removeYourself] 
			ifFalse: [self removeSession: each] ]. "in which case?"! !

!WebSessionManager methodsFor: 'adding-removing' stamp: ' 7/6/08 18:41'!
removeSession: aWebSession
	"remove from sessions"
	self site critical:
		[self sessions 
			removeKey: aWebSession id ifAbsent: [];
			removeKey: aWebSession secureId ifAbsent: [] ].! !

!WebSessionManager methodsFor: 'private' stamp: ' 7/6/08 18:41'!
sessions
	"dictionary of sessions by session id as the key. Not that session can have two ids, for ssl too!!"
	sessions isNil ifTrue: [self initSessions].
	^sessions! !

!WebSessionManager methodsFor: 'accessing' stamp: ' 7/6/08 18:41'!
site
	^site! !

!WebSessionManager methodsFor: 'private' stamp: ' 7/6/08 18:41'!
site: anAIDASite
	site := anAIDASite.! !

Object subclass: #WebStatistics
	instanceVariableNames: 'site referers refererStopList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Core'!

!WebStatistics class methodsFor: 'log analysis' stamp: ' 21/4/07 22:07'!
analyzeAndStoreLine: aLineAsArray into: aDictionary forDate:aDate

	| datum |
	((aLineAsArray at: 2) > ' ' ) ifTrue:
		[(aLineAsArray at: 2) = '#Date:'
			ifTrue: [datum := self parseDatum: (aLineAsArray at: 3).]
			ifFalse: [datum:=aDate.].
		((aLineAsArray at: 2) copyFrom: 1 to: 1) = '#' ifFalse: 
			[aLineAsArray at: 2 put: 
				(Time readFrom: (ReadStream on: (aLineAsArray at: 2))).
			aLineAsArray at: 1 put: 
				(Timestamp new fromDate: datum andTime: (aLineAsArray at: 2)).
			(aDictionary at:  (aLineAsArray at: 3) ifAbsentPut: [WebCounter new  ] )
				 incCounterOnTimestamp: (aLineAsArray at: 1).
			].
		].
	^datum! !

!WebStatistics class methodsFor: 'log analysis' stamp: ' 21/4/07 22:07'!
bossCountersFromIISLogsIn: aDirectoryString

	" WebStatistics bossCountersFromIISLogsIn: 'h:\winnt\system32\logfiles\w3svc3\temp'  "
	| file boss |
	file := 'LogDict.boss'.
	boss := BinaryObjectStorage onNew: file asFilename writeStream.
	boss nextPut: 
		(self countersFromIISLogsIn: aDirectoryString).
	boss close.! !

!WebStatistics class methodsFor: 'log analysis' stamp: ' 21/4/07 22:07'!
convertFromGMT: aTimestamp

	"IIS logs have time in GMT timezone. Return local time"
	TimeZone reference
		convertGMT: aTimestamp asSeconds
		do: [:date :sec | 	^Timestamp fromDate: date andTime: (Time fromSeconds: sec)]! !

!WebStatistics class methodsFor: 'log analysis' stamp: ' 21/4/07 22:07'!
countUrlsIn: aFileString to: anUrlDictionary

self parseFile: aFileString into: anUrlDictionary.! !

!WebStatistics class methodsFor: 'log analysis' stamp: ' 21/4/07 22:07'!
countersFromIISLogsIn: aDirectoryString

	"return a dictionary (url as key) of counters for urls in all IIS log files in specified directory"
	" WebStatistics countersFromIISLogsIn: 'h:\winnt\system32\logfiles\w3svc8'  "

	| logFiles urlCounters |
	logFiles := aDirectoryString asFilename directoryContents.
	logFiles := SortedCollection
		withAll: logFiles 	sortBlock: [:a :b | a < b].
	urlCounters := Dictionary new.
	logFiles do: [:logFile | 
		" self halt. "
		self countUrlsIn: (aDirectoryString, '\', logFile) to: urlCounters].
	^urlCounters! !

!WebStatistics class methodsFor: 'log analysis' stamp: 'JM 4/26/2007 21:34'!
dateFromFilename: aFilename

	^SpDate 
		newDay: (aFilename tail copyFrom: 7 to: 8) asInteger
		month:(aFilename tail copyFrom: 5 to: 6) asInteger
		year: ((aFilename tail copyFrom: 3 to: 4) asInteger + 2000)! !

!WebStatistics class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
default

	^AIDASite default statistics! !

!WebStatistics class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newOn: anAIDASite
	^super new site: anAIDASite! !

!WebStatistics class methodsFor: 'log analysis' stamp: 'JM 4/26/2007 21:35'!
parseDatum: stringDatum 
	"pretvorimo datum iz formata LLLL-MM-DD v date format
	| tmp | 
	 tmp := DelimitedFile new. 
	Transcript show: (tmp parseDatum: '1999-12-11') printString ; cr  
	"

	| datumStream leto mesec dan |
	datumStream := (stringDatum copyReplaceAll: '-' with: ' ') readStream.
	leto := (self  parseWords: datumStream) asNumber.
	mesec := (self  parseWords: datumStream) asNumber.
	dan := (self  parseWords: datumStream) asNumber.
	^SpDate newDay: dan
		month: mesec
		year: leto! !

!WebStatistics class methodsFor: 'log analysis' stamp: ' 21/4/07 22:07'!
parseFile: aFile 
	"
 	Trenutno opusceno, osnova za parseFile: into:
	WebStatistics parseFile: 'h:\winnt\system32\logfiles\w3svc8\ex990506.log' 
	"

	| logFile stream cr besedeVrstice vrstica stevecBesed |
	cr := Character cr.	" self halt. "
	logFile := aFile asFilename.
	stream := logFile readStream.
	[stream atEnd] whileFalse: 
			[besedeVrstice := Array new: 15.
			stevecBesed := 1.
			vrstica := (stream throughAll: (String with: cr)) readStream.
			[vrstica atEnd] whileFalse: 
					[besedeVrstice at: stevecBesed put: vrstica parseWordsIntoArray.
					stevecBesed := stevecBesed + 1].
			vrstica close].
	stream close! !

!WebStatistics class methodsFor: 'log analysis' stamp: ' 21/4/07 22:07'!
parseFile: aFile into: URLCountDictionary 
	| stream besedeVrstice vrstica stevecBesed datum |
	datum := Date today.
	stream := aFile asFilename readStream.
	[stream atEnd] whileFalse: 
		[besedeVrstice := Array new: 15.
		stevecBesed := 2.
		vrstica := (stream throughAll: (String with: Character cr)) readStream.
		[vrstica atEnd] whileFalse: 
			[besedeVrstice at: stevecBesed put: (self parseWords: vrstica) .
			stevecBesed := stevecBesed + 1].
 		datum:= self analyzeAndStoreLine: besedeVrstice 
			into: URLCountDictionary forDate:datum.
		vrstica close.].
	stream close

	" test lines : 
	Xx := Dictionary new.
	WebStatistics parseFile: 'h:\winnt\system32\logfiles\w3svc8\ex990506.log' into: Xx.
	Transcript show: Xx printString
	"! !

!WebStatistics class methodsFor: 'log analysis' stamp: ' 21/4/07 22:07'!
parseWords: inputStream 
	"Answer the contents of the receiver, up to the next separator  character."

	| aStream |
	aStream := (String new: 200) writeStream.
	inputStream skipSeparators.
	inputStream class endOfStreamSignal handle: [:ex | ex return]
		do: 
			[
			[| char |
			char := inputStream next.
			char isSeparator ifTrue: [^aStream contents] ifFalse: [aStream nextPut: char]]
					repeat].
	^aStream contents! !

!WebStatistics methodsFor: 'private-referers' stamp: ' 21/8/07 09:38'!
addNewReferer: anUrlString

	"add new counter to referers dictionary"

	self referers 
		at: anUrlString
		put: WebCounter new.! !

!WebStatistics methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
allRefererCounts
	"return a collection of all referers and their total counts, most counted first"
	| collection |
	collection := OrderedCollection new.
	self referers keysAndValuesDo: [:referer :counter |  
		collection add: (Array 
			with: referer
			with: counter total)].
	^SortedCollection
		withAll: collection
		sortBlock: [:a :b | (a at: 2) > (b at: 2)].

"WebStatistics default allReferersAndCounts"! !

!WebStatistics methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
allRefererCountsToday
	"return a collection of today referers and their total counts, most counted first"
	| collection |
	collection := OrderedCollection new.
	self referers keysAndValuesDo: [:referer :counter |  
		collection add: (Array 
			with: referer
			with: counter today)].
	^SortedCollection
		withAll: (collection select: [:each | each value last > 0])
		sortBlock: [:a :b | (a at: 2) > (b at: 2)].

"WebStatistics default allReferersAndCounts"! !

!WebStatistics methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
allRefererCountsWeekly
	"return a collection of current week referers and their total counts, most counted first"
	| collection |
	collection := OrderedCollection new.
	self referers keysAndValuesDo: [:referer :counter |  
		collection add: (Array 
			with: referer
			with: counter weekly)].
	^SortedCollection
		withAll: (collection select: [:each | each value last > 0])
		sortBlock: [:a :b | (a at: 2) > (b at: 2)].

"WebStatistics default allReferersAndCounts"! !

!WebStatistics methodsFor: 'accessing' stamp: ' 21/8/07 09:38'!
allRefererCountsYesterday
	"return a collection of yesterday referers and their total counts, most counted first"
	| collection |
	collection := OrderedCollection new.
	self referers keysAndValuesDo: [:referer :counter |  
		collection add: (Array 
			with: referer
			with: counter yesterday)].
	^SortedCollection
		withAll: (collection select: [:each | each value last > 0])
		sortBlock: [:a :b | (a at: 2) > (b at: 2)].

"WebStatistics default allReferersAndCounts"! !

!WebStatistics methodsFor: 'collecting' stamp: 'mu 4/6/2008 10:16'!
collectStatsFrom: aWebRequest
	| referer |
	referer := AIDASite convert: aWebRequest referer fromCodepage: #UTF8.
	(referer notNil and: [(self isLocalUrl: referer) not]) 
		ifTrue: [self countReferer: referer]! !

!WebStatistics methodsFor: 'private-referers' stamp: ' 21/8/07 09:38'!
countReferer: anUrlString
	| url |
	url := self prepareReferer: anUrlString.
	url isEmpty ifTrue: [^self].
	(self refererOnStopList: url) ifTrue: [^self].
	(self referers includesKey: url)
		ifFalse: [self addNewReferer: url].
	(self referers at: url) incCounter! !

!WebStatistics methodsFor: 'initialize-release' stamp: ' 21/8/07 09:38'!
initRefererStopList
	refererStopList := Set new.
	refererStopList
		add: '[unknown origin]';
		add: 'bookmarks';
		add: 'http://lw2fd.hotmail.msn.com/cgi-bin/getmsg';
		add: 'http://mobitel.sux.nu/cgi-bin/tabla/showpost.pl'.
"AIDASite default statistics initRefererStopList"! !

!WebStatistics methodsFor: 'initialize-release' stamp: ' 21/8/07 09:38'!
initReferers
	referers := Dictionary new.

"AIDASite default statistics initReferers"! !

!WebStatistics methodsFor: 'private' stamp: ' 21/8/07 09:38'!
isLocalUrl: anUrlString
	"true, if this url points to page from this server"
	^self site uriPattern contains: [:siteIdentifier | 
		('http://', siteIdentifier host, '*') match: anUrlString].! !

!WebStatistics methodsFor: 'private-referers' stamp: ' 21/8/07 09:38'!
prepareReferer: anUrlString

	"ingnore all parameters in url"

	^(anUrlString copyUpTo: $?)! !

!WebStatistics methodsFor: 'private-referers' stamp: ' 21/8/07 09:38'!
refererCounterFor: anUrlString

	^self referers at: anUrlString ifAbsent: [^nil]! !

!WebStatistics methodsFor: 'private-referers' stamp: ' 21/8/07 09:38'!
refererOnStopList: anUrlString

	| |
	^self refererStopList includes: anUrlString! !

!WebStatistics methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
refererStopList
	refererStopList isNil ifTrue: [self initRefererStopList].
	^refererStopList! !

!WebStatistics methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
referers
	referers isNil ifTrue: [self initReferers].
	^referers! !

!WebStatistics methodsFor: 'private-referers' stamp: ' 21/8/07 09:38'!
removeReferer: anUrlString

	self referers removeKey: anUrlString ifAbsent: [^self].! !

!WebStatistics methodsFor: 'private-referers' stamp: ' 21/8/07 09:38'!
removeReferersOnStopList

	self refererStopList do: [:referer |
		self removeReferer: referer]

"WebStatistics default  removeReferersOnStopList"! !

!WebStatistics methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
site
	^site! !

!WebStatistics methodsFor: 'private' stamp: ' 21/8/07 09:38'!
site: anAIDASite
	site := anAIDASite.! !

Object subclass: #WebStyle
	instanceVariableNames: 'site resources searchButton'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Templates'!

WebStyle class
	instanceVariableNames: 'each'!

WebStyle class
	instanceVariableNames: 'each'!

WebStyle subclass: #DefaultWebStyle
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Templates'!

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
adobeLeafGif
	"'imgs/adobe-leaf.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 50 0 50 0 247 0 0 202 44 13 255 230 0 175 117 98 255 175 74 173 171 166 114 35 9 213 205 201 215 54 41 171 68 11 69 84 18 207 184 182 254 218 1 247 181 170 137 45 11 203 25 9 255 200 89 109 135 58 168 152 82 231 89 70 231 53 38 252 244 231 47 13 9 150 140 140 242 161 148 255 181 1 220 59 51 194 148 78 69 22 12 255 243 243 254 171 1 108 114 46 251 218 215 43 54 14 230 227 227 252 153 0 251 156 152 59 74 14 250 167 135 255 72 6 244 229 150 250 169 163 255 197 0 254 139 0 220 64 54 18 4 3 205 111 3 137 141 78 178 184 159 223 26 11 249 150 137 253 101 6 142 5 6 167 155 155 149 72 65 217 221 208 234 68 67 255 88 19 253 246 188 253 236 234 142 162 89 234 220 218 248 199 197 113 11 2 255 117 39 252 198 136 226 60 46 226 65 59 79 96 20 235 67 59 252 136 87 255 135 57 244 119 120 255 108 37 254 191 133 228 34 26 255 229 166 134 102 101 247 162 113 181 48 7 149 156 124 216 43 35 254 250 249 251 76 16 217 100 41 166 3 7 230 43 36 255 204 166 185 6 5 248 176 126 247 120 107 245 210 128 254 137 134 91 35 15 255 107 80 234 165 1 230 74 60 255 241 0 104 114 81 229 65 52 200 200 114 255 217 136 235 102 88 90 109 34 229 73 68 255 255 241 255 129 39 195 85 3 246 104 58 243 58 40 246 136 118 252 195 188 229 217 204 182 80 39 251 215 178 255 248 215 246 80 51 145 116 51 255 102 31 251 199 152 255 255 0 243 105 98 235 230 221 255 151 39 252 156 117 194 88 18 242 137 23 253 38 1 255 83 11 250 230 221 204 84 67 252 112 5 236 68 52 181 19 7 212 89 20 238 234 149 238 89 83 234 62 59 245 233 230 232 121 43 231 52 49 161 22 4 255 220 27 255 57 0 245 187 149 213 68 47 113 88 87 238 238 235 230 180 3 253 224 225 255 153 60 236 75 59 252 253 251 213 172 91 245 157 167 226 48 45 233 226 137 229 59 17 232 58 52 240 113 92 235 73 73 240 71 70 85 82 73 226 69 63 255 101 66 225 128 57 27 36 2 244 150 20 220 69 5 236 102 74 242 87 64 251 209 207 247 121 78 231 76 51 157 72 27 233 79 71 236 84 63 255 251 251 249 248 247 255 255 255 234 64 64 255 255 251 221 96 93 223 185 167 249 231 205 242 166 109 234 149 94 255 255 254 221 123 114 237 94 75 255 194 60 255 253 253 214 210 132 230 68 48 232 129 1 255 255 207 248 189 196 188 100 0 232 171 161 222 187 180 127 52 21 236 86 2 60 50 51 237 94 20 192 147 130 246 247 247 231 120 57 219 37 18 254 247 247 234 191 128 241 142 136 255 145 146 255 230 94 255 248 125 191 188 184 68 74 42 255 172 52 225 104 0 237 116 12 250 122 95 255 127 21 255 224 181 214 199 192 249 193 175 238 80 80 225 175 104 177 64 53 225 177 112 243 186 105 230 199 182 248 208 167 244 228 134 156 181 115 234 116 100 239 127 115 240 63 61 239 64 51 210 69 29 255 93 28 239 128 75 238 48 30 230 153 78 202 151 136 253 150 64 251 160 81 232 55 55 203 166 153 229 63 56 231 208 200 223 84 0 254 254 254 252 239 239 255 254 254 99 96 90 206 78 43 198 76 49 132 132 125 33 249 4 0 0 0 0 0 44 0 0 0 0 50 0 50 0 0 8 255 0 101 9 28 72 144 224 135 130 8 19 42 92 200 144 32 186 50 28 26 74 156 216 208 147 24 49 187 18 81 220 184 241 25 176 47 98 28 80 9 167 145 35 65 85 176 76 10 188 16 228 75 16 40 34 103 24 51 169 131 193 23 86 42 101 121 252 226 74 76 144 9 0 24 205 80 48 241 67 52 151 65 196 72 200 137 2 152 43 87 103 190 88 178 20 132 145 15 73 12 245 149 233 212 201 158 168 51 103 36 68 81 89 230 213 183 79 55 210 126 2 5 10 10 164 133 31 22 117 181 39 228 107 212 131 38 81 73 56 132 22 209 172 27 108 213 41 81 149 240 67 149 185 116 235 70 189 160 18 21 175 67 103 111 204 154 60 11 81 149 67 8 159 113 37 98 239 34 93 187 101 84 122 122 28 89 50 87 122 139 148 232 40 136 103 2 17 34 98 236 101 88 161 88 212 82 147 109 94 185 74 75 25 17 106 77 48 24 15 164 84 133 136 165 65 178 103 215 21 101 59 179 62 125 148 122 48 184 192 224 195 88 89 12 94 89 234 180 104 17 189 239 191 149 16 255 33 88 102 2 200 32 25 210 43 103 126 155 32 172 76 240 51 141 24 113 161 190 183 103 170 214 173 155 80 5 10 20 77 0 2 88 133 18 215 69 81 69 16 43 168 167 224 10 162 172 144 78 66 195 28 33 97 22 20 182 209 134 39 68 172 193 134 18 206 56 224 161 3 255 1 152 26 94 110 40 129 222 1 40 46 184 194 1 12 36 164 195 28 169 12 210 225 135 14 112 146 134 20 87 16 162 35 33 87 92 1 5 43 171 236 81 2 32 2 225 161 4 138 72 30 144 222 10 76 30 176 26 66 186 224 224 135 3 61 246 72 5 33 70 24 81 7 33 51 48 66 5 21 0 144 34 79 150 15 76 147 131 64 118 88 2 69 146 74 50 185 98 32 11 213 225 7 12 85 126 73 197 40 163 32 193 201 12 124 78 49 64 37 117 252 33 5 18 215 44 33 11 45 194 228 80 4 36 254 165 184 164 19 247 44 132 132 35 130 82 209 194 29 147 180 96 74 35 24 72 193 39 41 149 164 129 68 29 237 180 131 3 18 73 200 130 198 18 75 144 49 14 41 253 196 255 10 9 36 254 180 226 1 67 69 248 97 194 31 48 76 114 71 0 119 4 123 71 7 84 176 147 198 15 63 212 129 67 59 127 8 90 132 44 113 0 97 133 21 64 96 162 193 181 116 208 225 65 2 88 45 228 134 174 126 168 0 198 2 1 4 176 192 2 96 96 42 131 20 117 140 186 172 9 82 152 240 236 60 243 120 195 192 35 208 88 235 130 7 102 144 240 66 67 251 56 98 130 184 96 20 108 238 2 41 164 123 135 32 127 200 240 7 188 142 212 145 6 16 178 40 114 138 54 49 148 240 136 56 17 64 208 239 19 19 49 32 131 193 96 4 128 65 7 1 164 80 238 175 56 152 32 240 15 3 60 208 11 55 178 156 2 0 20 120 100 140 69 4 31 83 228 203 54 232 142 43 130 10 29 96 128 65 201 11 220 161 130 9 105 60 64 205 3 3 204 67 129 44 0 120 72 68 27 37 148 160 65 2 255 110 36 7 6 228 98 32 66 10 24 168 192 137 23 11 120 49 110 35 15 84 162 135 30 3 96 145 18 12 14 56 163 4 30 123 52 161 129 13 42 45 255 209 129 8 34 4 0 70 10 34 32 19 140 23 99 167 112 71 47 70 8 34 67 26 123 200 2 139 18 48 216 157 10 46 229 12 20 75 53 255 240 19 202 63 6 52 148 195 208 41 32 124 114 7 168 171 160 248 2 219 200 128 143 50 37 232 196 139 51 149 187 51 181 44 54 132 65 2 8 165 244 222 187 53 221 42 68 65 31 130 0 174 130 8 106 183 224 68 194 41 100 163 6 2 78 160 32 139 14 233 40 82 72 46 2 161 241 203 16 32 144 224 61 8 224 243 94 10 223 12 249 125 188 8 193 4 227 131 15 24 16 222 192 251 140 20 35 139 42 71 228 114 139 64 57 152 51 70 191 9 244 191 123 248 189 235 26 67 228 240 0 21 96 195 42 196 40 155 26 142 81 128 247 197 67 22 40 136 134 47 100 65 129 77 108 194 16 251 235 159 255 254 7 190 82 176 128 0 20 1 132 45 200 208 11 83 16 131 11 92 40 64 1 124 80 15 89 140 96 31 208 130 134 33 102 56 6 15 12 225 134 67 72 0 7 65 240 193 156 8 36 22 33 48 192 50 255 26 32 0 29 60 3 22 40 192 130 29 78 96 136 19 128 99 95 56 204 225 255 88 0 50 31 22 36 22 112 144 134 27 164 49 2 44 36 65 11 90 24 199 60 52 176 47 51 152 225 134 58 44 69 24 84 18 133 68 240 224 13 6 48 64 55 186 177 12 1 48 128 18 148 144 198 22 246 176 135 85 172 33 21 107 128 71 4 92 64 72 143 37 0 4 46 216 72 30 108 161 1 62 60 175 1 5 216 64 5 42 208 64 125 124 32 34 81 64 65 23 230 192 134 78 178 97 13 26 216 129 40 119 0 1 8 8 144 33 177 176 64 5 184 208 0 4 168 225 149 8 56 6 23 54 176 129 6 144 195 133 41 17 8 7 80 48 136 9 188 227 29 108 104 70 4 68 121 14 27 92 98 34 247 168 0 11 88 192 5 56 240 161 16 133 224 67 44 103 89 0 56 8 100 11 9 209 71 15 80 16 3 111 188 33 15 177 216 136 49 26 192 5 101 110 160 21 173 64 128 58 143 33 201 10 108 160 36 120 176 162 66 120 224 131 6 28 67 153 21 56 134 62 245 201 153 133 101 178 32 18 3 169 133 60 19 162 66 89 46 115 3 42 84 97 57 253 201 140 129 192 105 160 4 81 192 42 103 201 130 137 162 112 161 203 4 225 64 194 241 12 136 10 36 31 180 68 33 45 71 74 75 101 46 147 31 5 169 1 15 60 42 11 38 176 96 164 147 140 105 76 151 25 10 132 8 128 6 30 13 129 73 101 42 83 127 226 20 33 10 0 40 75 37 161 74 127 26 149 5 22 8 103 66 36 81 1 150 18 36 4 213 160 1 13 170 97 128 16 72 132 5 74 117 106 78 146 65 62 173 170 132 31 26 245 170 73 254 241 15 177 170 132 6 22 144 103 64 0 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: 'np 8/21/2008 17:52'!
aidaBannerGif
^#(71 73 70 56 57 97 7 3 79 0 231 255 0 38 46 62 42 49 65 46 53 70 53 60 77 127 129 138 128 130 133 135 136 140 134 136 146 120 137 193 114 140 193 137 139 136 128 137 194 117 140 201 118 139 208 124 138 201 122 141 183 127 139 189 122 140 195 116 142 196 121 142 190 124 141 196 140 141 150 132 140 185 118 144 197 125 142 197 120 143 204 115 144 210 123 144 192 117 146 193 119 145 198 126 143 199 114 146 205 128 142 205 123 143 211 122 144 205 122 146 188 129 141 217 125 145 194 120 146 200 127 144 200 129 145 188 123 145 206 116 147 207 131 144 194 119 148 195 121 147 201 128 145 201 140 147 153 126 147 195 122 148 202 129 146 202 145 146 155 124 147 208 136 145 202 132 148 185 129 149 198 124 150 204 131 148 204 126 151 193 133 149 193 136 148 198 127 150 211 145 152 158 132 152 201 127 153 207 150 151 160 134 151 208 147 149 199 143 153 172 148 153 168 140 152 203 129 155 209 138 154 197 131 156 198 134 155 204 136 156 192 131 156 211 144 154 193 136 156 205 138 155 211 142 157 187 151 158 165 156 157 166 134 159 214 158 154 196 138 159 208 143 159 203 146 158 209 142 159 216 140 161 210 148 160 198 143 162 199 139 163 205 138 163 218 142 163 212 149 161 213 151 160 218 147 163 207 155 161 206 151 165 196 164 164 174 145 166 216 142 167 223 160 166 181 160 167 174 151 166 211 149 166 223 158 167 188 149 168 205 145 169 211 154 166 217 148 168 218 156 167 206 154 169 214 149 170 220 169 169 179 158 172 203 152 172 222 148 173 229 159 171 223 157 173 218 151 175 218 155 175 212 162 173 212 155 175 225 173 174 183 171 176 179 164 176 215 171 174 215 170 178 186 168 175 222 162 176 230 155 179 222 162 177 222 163 178 210 157 178 228 159 179 216 165 179 205 170 177 211 178 178 188 168 179 219 171 180 201 161 181 232 175 181 196 179 178 214 163 183 220 166 182 227 160 184 227 169 184 217 164 184 235 182 183 193 176 183 217 172 185 211 179 186 193 168 188 239 166 190 234 172 189 228 187 188 197 175 190 222 174 190 235 182 189 223 180 191 218 186 190 218 186 189 231 191 191 214 181 195 208 190 193 209 181 196 216 193 194 204 184 195 222 187 195 216 178 196 234 173 197 241 176 196 248 189 197 203 179 198 223 181 197 229 188 195 229 191 195 224 187 198 225 182 201 249 190 202 229 184 203 241 194 201 236 195 202 224 185 205 230 202 202 212 196 204 212 188 204 236 189 205 225 199 203 219 199 202 231 201 201 237 194 206 220 195 206 233 198 206 227 205 207 211 202 210 231 198 211 238 205 209 238 197 212 232 200 212 226 209 209 232 204 212 221 207 211 227 210 211 221 205 213 235 212 216 232 215 216 226 210 217 239 207 219 233 206 218 246 211 220 228 204 221 241 216 220 236 217 219 245 219 220 230 226 219 231 224 221 226 224 220 238 204 226 238 221 223 220 218 224 226 220 224 240 214 226 240 223 224 234 217 226 234 217 225 247 225 230 247 223 232 241 230 231 241 221 234 248 226 234 236 236 233 251 228 238 246 235 240 236 241 238 243 238 239 249 235 241 243 234 243 252 234 247 248 242 248 250 249 246 250 248 250 247 255 249 242 255 249 248 247 252 255 251 253 250 255 252 250 245 255 250 254 253 244 248 255 243 255 253 255 252 255 251 254 255 252 255 255 255 33 254 17 67 114 101 97 116 101 100 32 119 105 116 104 32 71 73 77 80 0 33 249 4 1 10 0 255 0 44 0 0 0 0 7 3 79 0 0 8 254 0 253 9 28 72 176 160 193 131 8 19 42 92 200 176 161 195 135 16 35 74 156 72 177 162 197 139 24 51 106 220 200 177 163 199 143 31 249 137 172 7 82 33 191 146 40 83 170 92 201 178 165 203 151 48 99 202 156 73 179 166 205 155 28 235 221 187 39 146 159 206 150 250 252 145 180 119 15 167 209 163 72 147 42 93 202 180 169 211 167 80 153 234 188 55 175 158 85 151 243 118 222 179 23 181 171 215 175 96 195 138 29 75 182 108 83 171 250 138 198 115 7 47 158 219 183 112 227 202 157 75 247 45 209 170 36 205 234 221 203 183 175 223 191 128 3 175 172 231 115 107 185 80 77 16 40 94 204 160 177 227 199 144 35 67 94 188 24 2 28 99 238 238 233 11 42 184 179 231 207 160 67 139 30 61 179 94 218 120 197 180 76 232 192 186 53 107 21 176 99 203 158 77 91 182 235 214 20 80 80 34 167 153 180 239 223 192 131 11 31 14 88 159 189 104 86 48 132 72 81 226 182 243 231 208 163 179 190 144 226 68 137 80 237 136 107 223 206 189 187 247 239 30 53 254 191 251 195 218 133 8 12 30 60 156 112 33 195 58 12 6 32 48 88 128 99 229 66 140 11 34 78 176 62 193 95 122 235 11 29 184 64 193 15 189 128 103 224 129 8 38 168 224 111 250 124 243 67 107 39 160 231 66 132 232 149 240 94 8 50 128 50 142 49 73 112 0 67 10 50 120 128 158 136 234 241 103 226 137 253 117 176 94 7 152 200 179 224 139 48 198 40 227 140 73 233 243 76 115 172 133 72 193 122 234 121 0 67 9 240 93 193 203 41 200 120 50 66 9 30 8 72 193 132 232 161 232 228 9 49 116 224 65 10 29 208 193 14 141 88 102 169 229 150 92 74 164 15 54 48 180 230 66 7 246 117 64 129 7 34 116 208 28 3 105 240 162 5 35 220 60 64 162 132 234 181 247 228 137 100 158 144 166 149 93 246 233 231 159 128 38 184 211 51 43 116 16 67 132 100 94 0 224 9 14 168 25 97 28 171 8 81 3 60 72 132 112 194 5 19 122 64 1 5 24 220 137 103 12 230 85 121 101 160 164 150 106 234 169 130 13 10 193 126 24 196 0 68 7 254 18 176 230 0 6 37 68 184 5 44 94 172 80 141 33 14 0 56 166 11 66 184 208 169 167 253 197 192 192 158 163 162 170 236 178 204 54 187 148 62 210 192 112 2 5 82 202 97 11 47 148 52 177 130 3 20 148 112 159 21 175 176 81 66 41 189 128 224 66 122 86 120 194 11 39 57 56 153 222 122 237 5 72 65 154 108 36 187 80 81 11 113 37 16 63 250 250 35 18 190 248 250 99 207 73 247 146 84 148 72 50 33 60 16 193 206 54 236 240 195 53 74 163 162 8 181 22 50 77 41 204 20 163 72 13 30 24 90 130 19 178 48 114 65 34 210 56 0 194 9 105 228 18 140 42 219 236 225 105 143 29 135 202 103 67 247 212 147 213 78 6 113 22 84 97 155 161 85 20 206 53 107 37 80 192 4 213 92 207 62 60 49 252 146 85 90 205 35 143 86 92 225 12 241 212 84 87 13 18 180 19 123 187 197 57 68 104 193 137 49 179 104 225 1 3 23 236 128 203 37 23 104 1 142 176 123 32 147 136 19 16 12 19 138 138 40 50 217 100 128 105 234 97 175 254 66 246 200 147 21 66 250 252 196 143 60 58 109 198 213 62 87 89 69 15 61 58 253 68 52 65 76 243 148 87 76 105 237 20 143 60 241 108 53 244 227 86 119 238 249 231 12 97 173 103 115 35 84 243 10 15 39 112 209 75 53 140 132 89 66 42 179 116 176 3 56 40 232 129 12 34 34 80 192 70 53 136 56 121 65 143 82 198 156 247 222 128 243 19 78 59 55 231 204 142 57 229 124 99 205 211 247 184 83 78 57 232 152 99 78 59 216 203 67 120 228 156 251 171 189 56 232 216 116 79 57 213 8 83 11 54 241 232 219 61 232 236 183 239 185 232 20 51 80 37 55 91 156 192 192 14 134 156 131 73 18 24 120 34 12 12 70 192 6 29 170 145 136 19 128 96 9 220 224 196 132 80 116 27 69 225 173 3 122 107 200 60 218 1 9 92 100 142 115 241 24 134 13 70 48 1 45 148 227 30 241 80 133 5 32 128 2 20 236 64 11 108 160 4 47 162 209 14 123 16 197 31 156 33 136 79 234 33 143 68 168 34 30 51 17 137 60 20 129 2 8 172 64 20 254 56 212 138 210 220 71 196 34 62 12 126 222 74 1 4 94 209 139 29 44 103 2 99 24 199 44 120 112 137 90 232 96 8 223 56 135 39 96 64 1 36 84 35 24 15 216 17 138 40 32 1 7 138 105 120 13 177 71 53 32 160 5 117 248 196 32 241 224 133 183 68 128 132 115 128 80 22 32 8 129 107 46 80 198 18 108 97 24 231 32 74 247 138 18 141 27 132 129 28 253 90 26 73 20 129 1 17 136 224 18 46 170 220 16 141 72 201 74 154 10 107 41 112 64 9 56 80 2 18 24 161 26 152 160 128 10 92 176 130 33 28 3 23 161 224 69 18 238 96 15 119 40 162 3 34 16 68 59 208 17 139 18 157 64 83 12 136 193 20 186 48 5 28 224 40 84 17 100 8 60 232 160 201 94 196 99 146 242 248 197 4 26 217 4 115 232 35 30 169 192 192 5 116 16 38 21 164 32 5 34 32 27 10 198 96 141 55 26 164 40 107 208 0 3 66 161 175 68 130 196 133 3 185 199 62 252 145 8 20 24 234 18 233 235 217 36 45 73 207 122 106 201 42 214 254 232 0 11 126 180 73 53 69 194 24 80 152 128 6 26 112 130 53 96 163 29 210 56 197 57 168 177 150 80 236 161 26 132 240 160 19 68 224 2 35 136 65 22 225 32 31 56 162 161 133 20 104 224 129 193 44 8 97 212 233 52 88 216 96 2 20 176 194 7 249 49 15 127 240 68 31 243 16 6 181 64 128 4 116 60 115 22 23 80 65 9 68 112 131 29 240 32 5 13 32 1 54 25 16 134 98 224 11 157 46 173 135 48 110 208 49 39 128 131 43 155 241 215 224 118 162 62 194 76 206 31 249 56 24 63 180 34 143 193 141 100 39 244 184 7 61 228 249 19 64 80 9 6 55 220 106 205 210 66 67 126 173 207 158 112 141 171 119 182 106 13 253 168 8 3 1 194 192 13 96 225 9 27 116 128 6 97 48 134 50 148 97 13 115 168 2 23 207 104 135 59 162 145 138 19 160 0 28 137 48 196 43 144 145 12 88 8 2 10 107 40 134 48 108 16 3 21 128 148 120 66 217 234 62 244 97 142 48 52 71 4 43 72 197 229 130 226 66 152 202 180 3 41 112 254 130 77 227 17 59 21 120 224 6 181 24 198 44 60 177 133 21 80 32 5 242 27 67 56 56 243 66 125 180 131 13 23 184 84 12 200 9 67 122 188 80 32 89 221 71 86 107 198 143 181 52 205 165 233 212 76 80 96 42 176 120 180 3 135 216 29 8 35 168 180 130 70 184 208 30 110 137 154 85 255 38 215 246 186 151 59 208 58 129 53 167 229 128 250 66 0 10 194 96 4 5 22 80 13 202 250 2 26 239 16 133 34 124 113 60 81 80 193 8 152 128 135 52 144 113 10 58 252 64 6 34 176 193 44 166 1 7 20 192 192 87 104 252 166 75 247 97 15 92 200 0 64 226 84 155 118 177 123 143 215 198 118 182 181 245 192 18 50 51 15 120 160 67 21 75 40 129 109 81 112 137 117 142 20 166 195 224 128 161 88 211 132 111 248 99 180 213 37 31 50 122 129 141 180 12 205 31 214 240 197 205 254 6 142 97 32 67 26 229 24 45 210 220 145 80 79 88 89 21 210 112 71 62 70 235 143 87 6 40 20 231 144 6 46 84 129 137 87 36 227 28 65 140 225 254 123 215 204 230 209 104 198 26 32 0 174 7 190 80 8 70 196 98 22 167 168 198 56 24 65 5 112 72 99 22 204 240 134 58 200 17 12 107 148 195 27 190 32 44 50 98 193 8 43 80 192 1 185 123 2 44 140 1 135 213 156 233 179 7 249 153 63 206 1 135 243 236 20 4 29 120 69 60 70 203 19 127 196 148 90 39 190 105 7 84 112 130 38 100 70 157 207 236 197 18 96 233 2 39 172 99 39 63 81 7 30 236 83 130 230 172 128 18 163 78 75 57 48 97 132 28 80 0 16 237 80 235 61 222 65 135 49 152 195 165 56 43 4 6 100 128 4 76 188 195 31 238 72 134 33 144 48 129 11 48 128 83 86 192 132 52 94 136 137 28 137 161 16 59 128 192 4 18 112 1 24 108 225 20 31 52 114 155 231 77 111 193 232 3 206 185 131 131 52 208 129 140 75 236 1 14 116 8 197 45 162 65 14 253 49 99 29 210 192 4 29 114 161 142 181 180 227 30 143 200 163 35 77 134 7 110 240 98 11 171 17 1 8 210 4 76 208 242 100 43 181 112 65 46 254 117 32 131 248 112 65 28 48 180 74 62 78 13 91 217 170 218 182 59 112 199 84 120 18 143 80 236 180 58 193 224 138 205 236 129 12 251 76 96 7 72 128 229 236 180 18 143 87 76 0 155 70 176 134 169 183 130 12 24 240 0 23 68 9 74 57 158 224 0 6 192 32 20 110 129 5 18 230 69 37 17 164 32 4 12 96 163 48 184 130 137 48 157 160 6 11 168 47 124 28 73 129 17 36 66 28 45 173 183 220 231 110 150 157 148 108 154 189 80 135 52 222 161 14 112 96 162 9 59 16 131 40 228 177 14 183 128 67 11 211 46 129 39 222 161 12 42 28 122 2 44 216 233 23 120 1 142 70 64 160 215 49 64 0 8 26 213 113 132 16 14 29 99 104 204 14 60 145 134 142 193 96 241 86 57 26 203 83 77 219 85 171 24 30 91 161 138 85 200 97 133 20 116 44 17 80 189 135 58 232 64 173 17 112 98 15 243 42 129 181 111 102 141 160 127 96 2 169 248 155 58 216 192 0 13 196 225 28 134 11 197 13 78 144 0 40 128 35 30 190 192 192 218 97 254 176 4 40 36 193 5 52 72 65 74 159 225 15 74 196 32 74 29 59 255 15 120 80 130 9 136 124 2 116 40 7 221 231 79 127 167 244 107 39 208 26 211 14 18 97 217 33 136 33 20 234 160 14 198 192 11 191 208 14 235 16 14 228 128 11 60 16 2 13 192 0 63 128 14 212 0 8 228 240 14 87 80 3 98 176 58 188 96 5 15 48 109 172 129 3 12 160 31 122 82 37 217 241 77 247 32 15 170 128 2 199 178 4 239 128 11 23 38 1 135 84 24 84 97 98 98 128 98 176 68 1 75 0 123 247 144 85 86 17 15 112 144 30 37 96 5 246 176 25 243 48 12 19 224 72 90 64 14 181 96 63 46 128 4 228 112 51 237 96 8 55 240 129 99 96 83 242 48 12 40 224 81 39 48 11 243 160 15 239 0 7 163 131 9 241 96 13 54 224 117 32 16 6 121 7 15 224 160 7 141 164 38 99 224 14 148 192 2 44 224 2 46 48 2 99 240 10 211 80 12 161 48 6 37 160 1 61 48 1 168 247 86 245 23 136 130 184 18 34 193 21 130 4 45 254 35 224 1 189 242 1 20 192 0 136 16 12 141 144 6 86 144 6 96 0 7 166 128 61 214 208 4 19 16 3 19 16 7 210 80 10 195 160 14 229 80 12 214 240 13 177 32 6 43 112 1 56 64 55 38 162 38 181 82 37 153 1 109 251 98 26 230 144 6 214 129 1 161 224 15 230 160 5 171 118 117 65 8 83 241 32 83 28 64 71 84 152 10 37 144 2 24 128 4 175 134 52 58 97 15 138 0 97 191 243 139 229 48 6 162 180 0 170 48 15 229 32 6 243 114 2 158 16 15 86 33 15 204 128 2 203 193 2 203 112 15 237 160 8 9 32 37 24 0 7 31 20 13 53 224 72 72 240 12 241 144 8 9 160 2 12 16 7 223 128 51 190 16 6 34 240 81 48 96 8 234 128 9 212 4 2 32 176 7 31 228 82 216 72 7 50 64 37 58 240 13 44 229 82 255 162 21 59 49 79 53 33 145 22 121 145 24 153 145 26 185 145 28 217 145 30 249 145 32 25 146 34 57 146 36 89 146 38 121 146 40 153 146 42 185 146 44 217 146 46 249 146 48 233 254 146 51 132 48 247 32 13 44 16 130 31 32 3 14 80 8 179 208 4 20 128 3 40 101 5 12 182 6 193 32 12 107 0 5 152 64 14 234 192 12 214 48 13 77 160 5 200 48 14 113 176 2 205 209 0 182 212 31 201 117 41 108 144 108 61 49 18 65 129 11 43 160 31 98 64 14 250 32 15 162 224 78 12 176 4 118 36 132 194 128 1 44 32 2 70 224 14 101 153 10 176 228 1 202 184 19 204 104 28 134 16 42 40 160 25 247 16 12 59 144 2 28 128 2 223 32 93 169 0 3 26 215 4 31 180 19 238 16 122 34 48 1 141 240 14 197 128 2 157 37 37 40 96 76 161 128 1 213 65 9 237 80 14 77 192 0 37 96 3 200 224 66 226 128 8 60 208 24 46 160 5 179 144 29 148 96 118 86 192 27 85 209 82 247 16 13 72 32 2 156 136 11 242 64 20 132 49 73 49 121 146 131 248 155 150 196 25 169 87 15 210 80 43 29 147 35 78 0 11 176 240 5 48 192 65 158 192 13 224 192 12 204 16 138 226 64 14 233 240 13 229 0 14 105 254 80 95 59 208 11 178 230 0 33 144 38 12 148 92 244 50 130 66 49 16 250 112 14 90 176 141 120 112 14 232 192 14 191 128 4 49 64 49 228 164 25 243 240 11 221 194 0 70 240 65 242 64 151 31 24 115 120 41 20 207 68 7 117 9 5 2 209 14 188 23 2 37 144 6 223 224 14 252 38 4 155 7 1 161 144 25 250 144 15 200 0 1 84 178 4 194 0 7 245 24 3 78 208 49 108 192 13 75 144 77 59 192 12 246 16 142 200 72 7 234 128 14 189 176 5 246 40 3 248 243 13 236 176 72 81 226 2 96 216 51 242 160 15 244 32 15 112 128 0 250 113 10 65 132 107 87 21 94 71 209 149 70 122 164 72 154 164 74 186 164 76 218 164 78 250 164 80 26 165 82 58 165 84 90 165 86 122 165 88 154 165 90 186 165 92 218 165 94 250 165 96 218 165 34 69 24 210 112 40 20 0 32 29 144 0 58 0 5 167 96 12 158 160 8 98 48 13 212 96 13 99 48 12 96 168 10 207 64 10 190 64 13 234 160 10 145 64 7 137 80 12 213 192 11 254 140 32 4 32 136 39 19 3 65 230 73 16 246 144 10 97 169 39 64 192 6 99 48 6 105 192 26 100 131 4 227 96 136 191 128 87 250 41 127 242 80 91 16 240 3 177 152 15 254 32 15 249 48 14 92 112 1 232 129 123 252 240 11 59 64 1 33 0 2 66 0 7 147 170 5 34 135 140 94 180 133 249 240 14 98 211 1 48 224 5 136 249 1 117 214 1 57 240 3 132 48 2 249 97 8 239 96 15 195 0 3 155 183 7 184 144 8 59 32 63 44 192 6 200 240 14 154 65 18 144 32 3 172 49 118 129 131 48 245 224 140 43 128 1 42 0 164 230 84 16 97 122 165 254 128 11 236 218 174 238 250 174 240 26 175 242 58 175 244 90 175 246 122 175 248 154 175 250 186 175 252 218 175 254 250 175 0 27 176 2 59 176 4 91 176 6 123 176 8 155 176 1 235 157 201 192 148 225 192 14 197 144 87 205 65 45 37 48 2 54 176 7 201 240 14 199 64 14 222 0 13 164 48 12 197 224 14 153 21 10 204 128 13 239 192 22 46 22 13 160 16 10 212 128 254 12 98 192 138 173 168 39 42 192 6 216 41 14 54 107 179 234 240 13 72 0 32 135 34 2 237 230 126 41 16 37 190 106 8 230 192 14 236 16 11 120 165 1 70 32 14 238 96 14 158 160 99 37 192 3 229 192 14 6 232 14 237 160 69 23 6 32 189 224 14 219 16 7 0 2 31 156 178 41 104 34 1 81 72 1 140 112 14 238 176 14 238 16 11 97 18 3 232 17 3 28 96 12 195 240 3 142 212 1 28 160 2 55 48 11 238 192 14 181 0 3 230 193 3 60 0 1 141 180 4 36 227 14 123 187 183 236 240 74 0 2 10 234 176 14 84 107 184 238 112 14 116 192 69 12 96 8 223 224 103 201 144 12 10 139 176 155 210 185 158 251 185 160 27 186 162 187 41 138 82 186 23 48 186 168 75 186 158 123 186 103 138 186 138 226 185 18 208 186 157 203 186 175 203 186 162 251 186 178 187 186 169 123 166 166 251 185 180 139 187 188 171 186 179 187 41 177 107 187 185 235 187 193 43 188 100 212 186 182 123 186 206 75 187 179 219 187 164 91 186 201 91 189 202 254 43 187 207 203 188 210 59 186 198 107 189 194 11 189 199 203 187 172 27 187 160 75 189 220 11 188 190 11 190 211 171 186 217 11 189 221 59 188 229 107 188 205 123 166 228 11 191 187 187 190 186 171 190 240 219 189 191 235 186 248 107 191 209 123 189 247 59 192 161 219 24 50 96 4 97 96 103 217 16 59 200 216 28 16 166 2 249 81 3 167 0 14 98 208 103 122 134 14 234 32 14 193 144 177 241 240 12 204 16 14 204 112 12 192 240 11 210 176 12 190 96 13 224 240 3 182 116 156 122 34 2 90 192 12 96 19 10 161 144 10 194 96 12 27 163 2 28 192 2 218 7 130 12 144 2 23 192 1 167 187 3 179 16 14 218 0 10 103 154 2 60 144 12 218 96 13 148 0 181 50 32 13 225 240 13 222 16 14 219 144 10 167 217 49 63 16 13 216 32 10 60 80 30 106 146 83 142 100 117 35 160 28 60 16 11 225 48 197 199 224 4 172 209 41 29 192 5 214 0 13 113 0 158 172 225 1 92 224 12 218 16 14 177 0 1 63 178 76 32 32 3 105 80 11 254 223 128 13 217 144 13 215 112 13 225 16 14 138 16 37 39 0 7 214 32 14 165 40 14 222 32 14 213 16 99 122 34 8 192 0 10 141 230 4 24 144 186 239 75 192 158 188 187 196 18 202 162 124 34 114 104 34 11 116 34 36 82 34 233 33 135 231 114 2 50 32 135 75 114 75 231 130 1 153 242 36 73 210 30 87 89 203 180 76 38 190 122 1 16 134 1 120 245 31 191 179 30 235 145 3 34 114 46 121 162 34 98 196 31 239 194 203 66 219 26 247 225 49 189 230 49 188 124 27 81 130 166 23 144 0 34 64 3 12 160 30 20 229 72 41 0 2 154 210 41 216 116 77 223 182 99 29 211 107 48 160 30 102 52 199 50 160 35 194 146 38 97 18 42 165 187 71 174 17 43 183 161 30 100 98 33 50 112 44 211 50 33 249 193 26 231 199 204 108 172 125 199 146 30 134 130 166 2 61 199 252 225 72 52 32 1 48 16 38 176 37 3 173 18 37 49 112 166 252 225 2 23 48 205 208 33 2 30 80 186 38 130 87 19 0 3 135 242 129 40 130 38 50 192 254 41 214 108 205 27 45 208 10 61 29 206 113 31 47 237 26 34 242 204 29 16 34 211 178 31 156 210 202 239 178 64 167 92 203 236 177 30 200 156 207 239 114 55 47 221 72 180 12 32 36 13 32 167 91 202 38 146 3 193 236 26 236 33 44 235 65 203 183 172 30 84 125 75 163 188 213 79 98 40 64 80 5 105 64 9 179 0 13 183 192 173 82 34 2 205 25 3 33 64 1 241 183 6 37 208 0 73 32 12 239 32 14 120 144 3 234 112 14 215 48 9 49 128 8 200 128 2 80 18 209 110 112 11 233 224 15 101 96 38 233 113 156 250 204 6 30 252 11 163 48 11 193 176 12 201 160 26 56 112 1 16 96 1 38 20 209 40 160 3 27 196 0 107 77 7 115 26 10 224 153 2 63 224 12 219 192 196 205 193 1 68 32 13 144 44 201 173 217 89 148 45 8 219 160 12 86 176 106 106 2 184 128 187 3 58 96 217 15 32 1 200 8 7 134 220 13 226 192 8 86 23 3 32 0 1 194 128 128 92 236 117 29 192 0 160 96 13 225 32 14 209 176 3 231 254 226 117 63 128 9 206 141 13 81 108 13 201 0 201 214 0 12 93 28 3 63 16 11 227 160 13 222 80 222 225 224 9 8 160 1 39 48 1 170 208 11 158 176 7 94 96 4 49 144 203 66 45 135 175 204 213 248 157 223 250 189 223 157 219 41 167 108 34 135 253 46 25 253 223 172 156 41 231 146 36 90 253 36 192 220 36 48 147 30 192 220 49 18 144 0 150 230 202 99 114 27 156 194 31 180 28 210 100 162 41 174 28 34 234 113 55 255 33 29 54 253 204 231 231 28 167 117 44 200 172 41 243 226 209 55 189 30 217 52 109 210 20 37 20 245 35 235 76 45 206 193 179 107 156 38 237 183 36 134 242 204 170 58 33 1 98 87 33 254 46 241 226 24 30 240 202 151 118 27 183 52 44 154 249 202 8 189 208 215 172 40 11 110 208 235 221 28 180 124 30 208 252 46 8 46 213 199 233 26 210 116 186 232 177 201 106 178 110 191 229 46 84 77 45 22 125 27 185 3 229 29 136 166 174 124 204 30 144 230 208 49 212 61 174 34 34 210 26 71 222 202 83 157 209 254 124 142 213 118 162 229 99 162 40 215 188 31 220 154 36 206 161 125 234 17 229 190 98 39 65 125 228 81 237 229 35 242 223 1 78 203 33 242 223 251 77 44 94 13 214 98 77 214 102 157 30 112 187 220 78 32 12 177 96 63 50 176 0 138 48 12 90 224 0 53 32 13 90 96 1 92 148 6 213 128 2 34 7 31 14 176 0 90 144 10 76 192 225 183 196 208 29 160 216 210 240 11 183 128 11 201 32 13 158 176 26 49 80 2 156 112 13 228 48 14 204 126 185 200 192 6 29 208 0 14 128 4 128 156 153 182 185 3 195 96 13 76 220 136 23 240 3 171 32 234 123 240 170 141 113 236 90 112 11 219 0 10 59 112 31 48 16 9 203 222 236 224 112 13 211 144 8 59 101 151 160 32 197 225 112 11 87 112 211 101 24 13 135 236 12 105 32 2 42 48 1 78 112 12 223 112 200 227 240 7 19 80 2 27 157 7 211 0 14 204 206 88 99 0 5 179 32 14 223 112 13 90 208 28 23 96 5 177 16 13 223 240 13 206 16 10 72 160 2 61 144 0 127 140 254 11 158 128 7 94 240 3 243 141 39 117 115 233 46 255 242 48 255 206 153 34 224 117 227 224 35 50 109 66 96 168 20 125 228 57 31 210 77 226 36 192 28 212 157 50 34 156 146 30 20 16 3 76 144 5 78 0 3 157 114 186 56 80 5 101 224 5 147 237 223 155 252 224 24 128 3 94 192 4 211 210 41 34 178 41 232 81 205 237 252 210 21 254 209 66 238 204 48 13 32 19 0 180 223 214 126 105 111 38 188 59 45 60 210 49 10 45 3 93 80 7 85 240 138 54 205 212 177 219 28 9 112 2 83 32 7 114 192 4 143 222 26 12 96 30 219 232 209 3 158 77 40 110 224 183 20 3 177 146 126 138 82 2 103 114 216 183 68 81 187 14 211 174 241 3 114 48 5 55 189 41 57 96 247 19 128 38 82 50 250 82 66 204 195 178 31 142 164 227 115 124 211 111 80 7 94 16 37 155 124 1 89 32 7 117 128 5 150 110 34 170 232 5 117 112 97 208 220 1 154 233 209 120 229 64 56 142 30 216 52 37 250 129 227 229 241 206 75 126 41 173 225 45 80 254 114 166 221 54 199 20 192 173 167 27 33 151 198 204 69 13 204 252 193 227 131 142 206 107 252 204 101 180 225 2 210 129 94 208 5 76 128 41 30 157 36 154 242 228 33 222 225 179 124 38 4 141 30 237 177 204 48 127 39 153 30 214 99 93 214 115 156 31 16 70 1 158 128 12 0 241 196 131 8 7 110 174 173 210 0 66 76 185 38 34 46 120 64 194 109 7 5 17 41 64 132 72 1 163 4 6 12 30 92 156 56 209 161 131 135 144 108 158 73 27 54 44 88 50 94 87 24 168 232 192 69 216 55 108 219 108 90 11 247 45 20 10 6 33 16 212 137 22 10 66 7 24 72 138 229 164 132 33 4 3 6 39 74 60 61 33 130 134 139 9 108 102 78 67 34 178 3 146 99 57 183 89 179 134 237 219 183 83 40 68 136 232 144 38 154 184 112 227 246 12 148 225 9 220 183 112 224 60 197 136 49 130 210 56 176 207 198 221 178 82 98 66 9 29 91 18 25 66 196 102 71 9 15 117 248 146 11 53 81 4 131 27 112 16 33 10 35 227 229 133 29 151 132 165 254 98 148 166 10 144 24 31 65 122 64 13 242 52 73 213 173 93 191 134 29 91 246 108 218 181 101 167 62 129 154 181 106 15 20 40 112 84 221 193 55 16 38 56 58 92 184 208 129 201 148 14 185 127 239 110 173 85 248 133 24 210 69 122 192 208 1 71 23 47 48 40 32 239 224 196 75 21 38 213 81 163 213 154 28 135 23 39 215 61 36 7 47 50 249 113 234 224 171 75 232 72 210 133 71 233 224 225 207 79 163 4 10 100 112 225 135 58 240 112 228 141 58 170 128 1 6 32 166 136 1 187 240 218 59 238 4 25 166 168 67 14 24 72 66 110 190 24 156 56 196 145 58 248 224 195 9 12 110 240 162 14 38 48 152 15 188 143 48 248 8 53 24 115 3 201 133 20 68 248 237 4 12 40 112 129 69 47 110 40 225 56 25 60 32 236 130 142 120 59 97 63 162 226 211 106 2 24 234 120 227 130 141 40 88 161 142 58 140 100 64 43 29 183 76 205 183 239 60 58 139 40 24 180 250 144 143 58 34 204 237 6 71 226 240 226 9 211 90 219 239 130 9 156 208 254 228 134 244 168 171 40 5 29 229 155 175 194 222 146 196 32 164 244 70 114 46 80 9 47 184 129 193 11 40 80 238 13 62 222 136 1 62 250 156 236 224 67 32 190 243 176 11 38 146 171 142 62 79 133 19 169 186 14 69 194 97 138 55 222 144 195 139 46 78 248 208 140 41 2 77 18 212 78 229 251 244 56 70 171 243 141 73 28 34 21 201 54 95 93 171 20 136 42 210 160 100 22 104 110 145 225 58 6 42 162 32 12 99 12 17 65 135 17 190 24 167 28 74 52 88 225 24 117 18 169 14 132 18 144 201 138 197 14 28 186 128 133 145 116 19 148 164 14 76 178 166 152 96 130 65 166 16 7 52 80 161 132 189 176 41 70 153 95 126 81 38 154 118 181 160 0 4 6 132 88 133 147 101 49 224 225 40 112 40 161 32 4 7 4 166 85 4 12 86 128 99 166 112 34 9 178 131 27 20 225 203 153 97 132 73 137 25 113 164 217 226 4 21 16 133 133 174 111 126 49 194 129 43 140 17 7 44 112 132 177 98 130 37 114 41 38 37 92 134 185 6 20 35 92 254 208 74 176 32 97 100 224 137 95 172 25 217 144 21 24 72 129 211 14 24 208 192 161 18 244 16 166 22 79 246 248 194 9 227 114 211 13 53 235 154 251 53 108 177 199 38 219 5 2 185 118 205 35 25 8 196 238 214 24 112 48 238 2 184 229 102 162 57 15 214 142 141 62 28 128 128 187 3 9 180 43 174 3 22 47 0 162 59 173 126 24 239 134 78 101 164 53 6 32 16 119 162 70 33 128 144 14 7 38 96 192 129 186 31 164 188 65 163 27 128 224 123 239 145 38 7 2 134 9 52 6 226 134 31 110 112 210 137 66 126 16 34 7 10 170 40 177 18 39 82 45 225 6 241 122 189 161 141 55 96 167 145 2 47 222 216 16 3 24 126 96 226 7 211 97 144 227 13 241 234 240 130 2 39 157 79 110 2 213 181 238 96 5 35 132 48 130 7 30 110 144 242 7 39 180 182 144 123 241 115 32 146 7 71 156 16 31 3 17 78 128 33 9 28 176 91 45 73 70 197 119 34 6 39 75 128 193 137 29 82 44 1 4 110 240 132 31 164 225 74 12 40 16 248 254 62 242 62 236 188 40 6 200 251 65 117 78 52 62 59 121 160 3 37 24 222 152 68 226 4 73 32 225 7 111 74 219 15 148 39 137 31 240 79 124 63 104 14 5 78 112 66 59 177 175 4 143 115 1 15 182 247 3 239 221 64 2 142 3 221 5 143 199 55 210 108 176 14 78 40 207 5 96 80 5 17 81 136 2 123 91 157 233 126 240 134 42 8 33 110 29 0 66 23 120 165 30 213 137 4 116 124 219 212 113 30 136 3 229 149 64 60 78 200 66 21 212 208 35 9 76 225 85 29 121 91 113 34 197 4 197 197 224 6 157 99 20 220 118 197 41 184 137 42 6 83 40 206 214 200 86 155 96 13 171 88 199 74 214 72 42 226 128 18 204 34 23 59 184 192 8 18 49 141 88 224 226 20 58 160 195 57 48 129 140 37 120 32 4 43 200 69 28 70 242 20 254 177 128 3 30 233 77 71 174 83 146 103 60 35 37 201 56 197 14 28 0 147 36 212 34 26 190 72 69 40 40 65 137 80 228 2 25 206 160 132 128 100 64 129 52 48 130 89 217 170 11 38 32 254 176 128 21 128 64 4 22 236 128 11 110 96 133 75 40 99 44 201 176 65 144 46 128 132 90 88 35 25 189 8 133 39 94 241 10 92 36 67 26 151 224 1 114 74 192 134 104 132 195 26 209 136 195 10 240 128 140 101 48 67 25 197 48 6 34 70 80 49 89 164 194 19 158 8 197 47 142 177 138 47 8 136 41 200 132 64 238 226 32 12 103 36 35 26 198 160 132 19 158 194 1 44 81 160 40 134 168 197 47 64 17 154 237 25 7 93 46 112 1 94 102 5 182 61 118 212 163 123 204 65 72 115 176 31 232 128 68 8 39 21 2 129 148 195 4 150 214 173 3 85 192 35 30 5 119 2 51 158 13 88 202 241 66 78 205 80 41 38 120 161 12 94 152 66 72 46 128 56 24 32 103 136 86 2 170 112 238 150 44 228 48 161 12 111 40 195 15 39 96 4 44 168 225 13 94 136 65 2 152 96 134 230 213 65 124 114 208 29 244 12 36 34 43 121 213 9 142 74 213 234 222 80 137 58 28 162 14 48 184 129 28 54 129 135 65 96 65 6 226 185 128 35 74 192 254 4 55 101 97 68 153 11 207 36 38 97 37 33 96 244 2 195 43 129 12 48 144 162 67 60 74 35 85 200 80 78 171 32 38 57 56 129 81 79 226 67 130 178 240 3 73 124 162 16 142 40 4 30 188 88 136 207 122 245 6 133 248 132 36 36 81 7 213 189 193 179 146 192 67 14 28 192 0 47 172 34 168 129 34 41 50 157 80 7 71 136 168 132 99 45 196 106 39 0 89 71 224 97 181 227 122 194 38 248 0 131 24 252 115 36 46 224 145 29 38 65 8 73 16 111 172 142 168 172 72 48 168 161 49 249 213 22 159 21 130 10 93 227 130 55 132 168 14 159 208 8 99 249 80 133 167 120 33 15 132 32 196 36 110 80 135 50 252 160 12 135 112 66 21 84 139 135 66 208 23 131 35 130 94 9 126 48 9 73 20 226 16 93 160 192 18 63 97 165 41 104 228 139 93 168 144 29 236 32 7 40 37 138 173 124 240 2 229 128 240 6 71 80 238 56 91 205 16 131 156 192 135 67 40 168 61 135 189 106 29 178 144 59 47 164 225 13 55 192 2 72 58 48 133 254 46 236 8 3 83 240 130 25 52 21 3 47 60 234 170 94 80 158 120 114 90 183 173 150 129 59 68 54 3 80 167 128 131 143 202 166 143 196 50 22 178 220 3 93 56 32 3 14 14 144 129 30 198 161 180 42 191 162 26 152 88 129 49 20 113 130 75 230 162 16 37 8 83 133 246 179 31 88 141 114 93 165 252 133 48 122 129 9 43 36 15 5 134 48 134 48 84 65 9 68 232 65 15 136 240 68 42 132 193 11 36 236 0 9 72 160 67 34 126 160 132 36 192 161 24 214 144 198 43 172 32 6 45 136 65 12 73 216 2 27 138 53 13 127 125 195 26 189 24 131 21 86 167 135 98 248 226 20 178 100 4 35 40 1 138 89 20 227 22 98 184 30 28 134 193 140 100 32 195 19 90 224 132 48 76 129 139 88 200 34 23 158 88 2 35 64 225 9 62 35 34 18 161 80 197 49 132 65 137 45 56 97 164 60 248 129 22 232 32 104 100 244 98 22 239 68 70 40 232 176 133 36 192 192 123 86 152 118 49 106 49 209 56 124 225 164 91 195 168 12 66 186 183 208 254 237 170 201 239 134 247 107 82 179 54 117 143 20 55 188 9 86 15 247 86 29 226 28 79 121 253 107 207 221 240 40 3 216 16 229 7 85 24 95 119 16 135 112 48 186 244 224 37 16 103 255 128 116 28 187 101 7 174 236 225 159 243 96 240 5 30 80 160 4 85 200 2 243 74 136 162 167 252 46 1 48 197 64 23 204 80 133 60 56 33 13 59 8 67 26 242 235 223 31 240 65 121 206 123 74 28 76 199 168 193 192 224 16 140 225 144 125 183 86 2 46 172 184 168 70 66 49 12 214 6 165 42 84 1 119 89 112 68 37 222 16 64 11 194 128 122 94 116 68 211 229 80 136 29 196 1 15 86 200 3 18 242 160 131 54 16 130 11 92 232 239 156 62 81 232 74 252 0 2 79 226 36 69 24 224 4 66 48 161 71 188 193 192 144 96 144 133 52 120 161 18 94 152 192 85 75 176 131 208 50 207 11 252 235 47 4 60 32 132 18 3 253 152 177 194 0 14 228 208 6 46 228 33 12 129 111 67 78 51 196 4 233 121 65 14 199 193 14 12 244 42 160 146 158 254 96 183 37 88 129 19 62 113 112 55 8 231 6 191 219 173 19 80 176 3 60 80 64 60 26 41 131 19 96 16 135 52 196 33 14 59 176 57 148 102 254 131 9 176 105 136 124 144 18 12 14 191 145 244 86 225 6 101 0 210 89 59 247 240 18 252 78 2 125 186 128 87 255 134 131 205 58 9 227 57 29 98 119 148 159 95 231 37 248 13 120 224 195 4 118 20 146 41 100 65 36 46 240 66 22 200 175 208 50 24 129 61 236 67 81 21 196 167 255 143 7 217 9 201 97 2 15 195 15 174 185 183 143 18 137 28 240 163 40 11 36 15 96 154 9 200 133 84 2 1 65 152 134 80 128 145 14 208 3 88 136 4 20 88 128 75 48 134 44 112 128 19 224 133 66 152 128 27 73 1 61 241 0 16 32 9 212 248 8 55 99 3 105 176 134 97 168 133 94 200 5 94 176 5 98 131 5 97 144 165 61 224 189 222 211 3 70 128 5 94 152 165 89 48 134 106 168 134 92 48 194 104 240 52 105 136 6 94 56 6 99 80 6 103 64 134 106 0 7 112 136 153 103 96 254 6 102 112 134 105 152 134 89 128 5 78 128 133 97 80 133 137 210 65 65 192 3 70 8 180 99 184 132 72 240 4 91 16 134 98 192 5 25 76 5 67 136 5 79 192 132 80 0 5 80 8 5 109 227 132 75 96 4 61 192 3 60 208 3 90 242 133 100 32 194 89 240 4 78 0 5 94 48 134 104 24 134 83 56 53 80 16 5 85 8 134 98 56 168 87 88 181 88 48 134 113 112 67 81 80 132 61 72 131 43 168 168 24 96 46 118 195 159 241 89 157 93 249 154 141 250 26 140 138 183 86 148 55 143 200 129 181 137 69 130 43 189 228 17 161 210 193 1 2 201 59 38 144 129 31 136 27 226 80 13 223 200 27 141 17 31 159 114 1 33 112 131 251 107 143 167 32 42 237 82 62 32 201 17 29 241 13 40 122 177 193 248 130 39 224 129 47 88 129 223 48 2 109 228 3 15 232 185 6 129 62 12 192 2 44 192 128 39 192 2 251 67 145 31 104 30 43 8 35 133 50 144 6 193 184 249 18 12 141 16 19 71 208 136 228 128 129 28 120 3 28 200 145 156 254 226 31 211 193 142 42 120 131 209 169 3 54 201 41 39 240 0 44 184 2 117 67 13 97 52 3 133 194 160 191 243 130 48 8 3 20 1 50 226 137 3 177 203 3 46 104 131 56 120 130 21 224 1 77 216 145 67 184 1 143 24 4 193 16 129 179 137 70 215 192 50 15 240 130 66 240 61 73 184 188 67 32 190 9 24 30 3 97 29 12 18 193 143 88 148 11 160 145 24 97 145 4 112 185 52 40 132 169 227 131 142 116 177 42 136 1 12 192 160 55 128 174 222 232 128 74 32 148 215 120 169 50 80 60 8 168 132 108 116 3 42 161 0 55 48 130 39 112 3 146 240 13 237 41 131 167 200 61 12 66 156 31 41 132 17 171 200 156 42 161 248 243 162 58 80 60 30 168 131 109 228 8 23 32 145 204 2 188 156 90 129 158 123 138 202 162 0 146 232 8 12 248 161 193 20 130 47 8 40 156 116 62 140 195 32 37 170 130 225 249 130 19 16 2 231 35 190 140 249 184 195 235 159 55 144 63 39 48 2 217 225 184 47 48 130 209 236 70 80 28 77 15 24 162 254 145 115 191 19 96 130 240 74 18 231 64 151 119 59 192 4 4 164 235 112 24 27 56 7 61 64 129 63 48 6 58 128 128 138 232 0 134 98 128 15 180 1 25 68 1 8 224 5 61 24 19 62 34 148 55 179 6 95 192 133 89 152 133 95 24 134 98 112 134 207 240 4 67 200 131 52 8 3 46 72 131 54 40 4 97 171 133 71 195 6 113 24 139 177 176 6 43 44 134 245 92 207 103 0 139 190 176 66 157 169 5 97 88 137 90 131 193 46 188 12 63 211 3 67 168 195 89 104 67 100 8 134 94 56 181 123 2 133 75 184 4 98 35 54 3 229 195 203 240 189 52 128 3 56 216 3 53 156 5 103 64 26 113 176 80 105 168 6 100 200 133 80 72 53 3 5 5 88 216 25 247 12 135 112 192 6 83 178 26 70 40 4 238 180 2 246 89 52 29 16 33 39 64 2 46 176 130 48 112 71 39 0 2 22 85 157 213 121 163 29 205 209 29 21 161 148 162 183 234 120 205 168 120 31 87 36 27 21 164 55 140 98 197 215 112 41 209 233 0 2 113 129 28 40 254 143 58 130 162 37 227 136 154 2 33 144 56 162 41 80 168 215 139 210 43 40 33 136 187 190 227 97 144 172 74 128 33 26 19 197 210 15 188 153 177 46 173 3 109 4 69 239 185 130 43 168 146 43 56 30 246 88 71 118 124 3 8 80 199 31 97 15 159 18 154 9 24 174 58 224 159 180 132 1 155 99 199 44 184 128 4 40 1 175 42 42 29 202 189 205 145 0 39 168 175 44 112 30 162 24 22 24 144 0 15 72 21 211 121 138 9 200 2 163 27 30 10 120 32 246 24 140 216 51 29 65 133 203 23 139 131 2 194 131 29 64 1 245 163 128 43 168 4 4 64 0 155 83 161 65 96 31 71 144 162 234 211 132 24 120 141 20 48 155 3 65 1 43 144 132 195 227 3 109 252 1 17 177 58 172 250 129 56 56 132 9 160 0 44 208 4 59 152 31 24 97 51 145 96 130 74 208 8 43 217 184 175 19 12 245 139 129 18 200 130 250 26 147 227 89 174 14 56 157 215 56 171 213 241 2 77 248 129 43 112 3 30 208 158 58 144 161 67 48 2 10 48 130 173 254 188 130 5 241 130 245 193 201 68 65 214 184 234 14 65 77 128 192 3 60 3 217 70 30 152 87 121 229 74 30 240 0 74 205 175 202 18 88 32 201 169 252 107 144 11 200 189 30 145 1 44 168 2 239 211 189 238 139 171 55 34 30 12 186 2 12 16 130 238 16 176 201 186 160 200 212 8 156 172 47 30 216 37 12 224 184 43 240 158 238 177 217 21 240 128 21 128 128 225 98 29 221 1 202 229 16 2 150 34 41 51 186 187 38 59 64 151 251 35 41 27 9 10 120 0 74 136 6 11 248 131 106 208 3 8 192 128 20 200 14 14 224 0 23 96 0 16 128 0 43 224 134 94 32 2 106 48 132 32 97 78 117 49 137 104 136 78 233 252 5 92 192 5 97 224 194 236 252 72 139 236 189 50 12 180 157 97 134 247 244 180 244 76 137 190 109 195 90 43 165 43 212 25 233 156 5 92 64 134 148 168 26 67 156 165 203 48 4 74 192 167 89 168 133 90 112 91 85 200 166 89 178 220 203 189 92 6 245 189 56 96 131 66 80 67 65 43 6 247 140 25 102 24 134 254 89 176 167 84 51 4 69 112 220 89 72 137 208 141 207 210 245 4 70 240 196 132 28 159 241 201 130 47 120 131 56 208 3 8 213 193 222 179 2 134 27 31 33 168 221 225 37 94 207 244 204 31 224 27 188 48 141 179 48 210 177 185 168 28 56 169 145 2 142 214 240 0 203 89 142 242 144 1 33 192 35 237 21 130 24 88 14 51 154 130 28 8 148 154 130 13 12 184 35 51 184 170 44 232 136 158 66 149 42 144 1 32 40 3 10 171 131 37 43 28 40 121 131 46 16 95 88 217 17 10 0 177 83 49 131 247 85 57 59 240 130 19 224 95 9 171 131 46 88 212 46 176 3 40 161 175 17 241 171 248 5 178 20 49 19 253 147 131 3 235 2 71 176 131 242 248 87 71 40 131 24 160 128 41 112 4 77 152 223 145 168 131 73 168 4 182 218 28 38 176 3 71 224 131 41 152 212 18 222 132 73 152 58 3 169 132 34 34 162 26 238 45 57 168 190 74 208 132 77 144 132 113 117 148 19 134 158 222 42 131 4 73 16 26 214 4 71 152 2 32 56 132 79 80 254 96 77 224 3 33 32 199 26 254 161 220 169 4 66 224 40 56 137 88 71 144 4 62 232 226 213 145 131 26 230 131 74 64 184 204 234 45 219 113 129 39 168 4 59 56 23 174 217 17 234 56 97 182 98 130 9 184 0 14 59 97 72 129 129 77 216 132 79 168 132 50 80 62 66 224 99 155 204 98 144 128 129 52 32 132 66 104 131 234 154 49 243 18 2 225 240 130 67 88 99 47 208 14 57 200 44 43 225 48 5 225 131 44 176 157 44 200 44 57 102 2 22 102 130 49 54 131 249 89 225 3 254 100 71 240 2 33 224 173 238 128 44 62 200 224 227 152 2 59 224 3 57 80 158 30 35 4 87 118 147 158 114 101 8 185 96 59 152 2 51 200 96 16 131 18 94 196 130 178 170 3 59 128 222 55 144 176 46 96 14 248 61 21 53 120 2 53 144 131 41 80 3 102 30 101 38 80 57 47 168 155 95 198 35 149 243 199 28 0 95 244 241 8 163 21 47 3 236 128 28 80 90 5 116 15 20 152 192 75 152 6 223 236 90 16 8 9 141 208 12 16 0 1 10 254 24 131 106 56 135 115 208 2 157 61 219 146 144 134 107 248 5 46 236 53 101 168 78 184 189 39 217 221 3 8 253 3 85 11 52 214 77 134 103 24 139 17 53 167 104 56 220 94 136 193 94 64 134 141 6 11 102 88 207 232 148 133 194 237 219 143 121 219 84 136 165 60 12 180 84 128 220 143 169 133 89 168 92 85 155 165 123 202 38 98 155 165 72 64 4 210 234 51 99 163 37 255 140 6 127 177 6 44 44 93 123 138 132 203 125 232 98 64 134 98 160 181 94 192 133 153 70 132 61 192 131 29 236 187 52 120 170 56 216 3 67 104 92 74 64 140 63 208 65 168 46 131 174 254 41 159 42 131 52 16 235 177 22 235 48 24 15 246 217 149 229 109 94 177 185 183 20 36 192 180 241 136 56 177 160 188 251 8 130 187 0 179 161 145 252 120 145 216 144 74 81 156 0 250 17 137 49 249 13 25 208 136 196 154 70 81 44 141 35 209 17 143 224 8 188 8 21 228 208 40 80 233 19 235 0 146 194 214 8 28 248 155 162 194 28 162 248 14 196 238 143 220 65 14 254 144 16 197 244 64 205 6 121 35 50 193 11 228 48 30 211 246 224 175 9 72 184 210 136 35 234 156 6 33 96 209 206 71 141 161 14 145 40 161 11 66 197 74 225 148 18 40 13 6 240 128 213 129 184 137 156 172 155 114 1 78 105 16 13 98 148 140 226 159 227 128 1 160 201 29 193 169 94 204 94 155 174 177 160 4 152 109 243 136 107 233 88 110 204 169 142 215 6 159 65 118 138 128 12 200 80 49 14 90 169 148 72 33 224 128 164 20 28 24 147 234 24 215 215 174 20 82 241 144 14 64 0 249 14 149 80 233 148 249 24 19 38 105 144 167 248 110 25 184 17 65 137 20 82 149 15 199 118 198 8 241 8 55 82 186 93 202 40 50 129 238 230 128 174 28 104 237 247 192 108 78 81 112 245 182 155 188 126 200 20 44 61 178 73 218 52 48 4 117 30 9 7 240 128 106 16 135 105 72 39 166 89 150 247 176 71 18 124 31 43 144 93 8 24 146 127 126 51 125 33 92 92 80 133 71 48 5 85 8 234 123 178 220 3 125 92 92 8 93 107 208 5 83 8 254 7 83 232 132 247 44 134 94 24 134 141 70 6 247 52 5 83 176 134 70 0 114 150 158 92 74 124 6 95 240 133 234 84 5 85 136 78 144 201 153 42 135 65 123 242 132 75 112 92 124 194 132 70 48 221 60 172 92 204 197 167 84 48 114 85 176 114 102 240 133 118 169 133 84 24 54 153 14 5 149 30 134 104 40 6 105 72 134 73 4 67 52 231 195 84 219 131 69 255 195 240 244 4 85 104 132 72 255 243 151 78 181 84 251 3 173 94 116 173 174 244 74 55 132 221 229 61 132 19 33 155 90 235 176 73 141 6 194 14 197 134 235 252 224 26 223 208 95 142 112 117 174 49 141 2 60 141 215 212 10 37 13 236 26 145 14 144 224 136 249 64 27 212 248 167 193 164 21 14 121 8 235 64 38 30 97 20 253 16 198 142 152 108 80 25 148 233 24 9 145 216 84 27 163 184 27 127 82 96 117 159 100 193 20 225 144 24 41 49 18 17 192 149 250 254 154 145 112 117 11 170 107 225 96 238 112 167 56 91 145 149 228 176 17 161 10 20 10 16 150 13 185 41 165 254 250 13 162 16 137 64 201 142 116 39 148 167 112 159 20 16 149 24 161 31 248 208 32 113 119 159 21 164 143 228 88 108 180 192 155 129 200 210 61 17 149 254 16 14 85 4 189 250 110 16 245 78 142 105 188 21 21 188 27 175 121 15 70 17 137 1 57 142 193 249 27 38 153 14 85 212 199 49 57 139 247 232 141 184 22 165 123 175 186 69 65 23 210 227 154 230 224 147 37 129 145 10 148 142 3 111 17 70 49 146 143 103 13 236 240 8 11 33 76 88 65 218 115 118 57 67 72 133 218 28 9 129 121 133 94 96 3 10 96 128 249 240 245 124 159 149 16 16 1 136 69 38 28 55 9 131 150 206 30 47 130 25 8 130 51 240 232 97 136 206 48 87 105 49 223 153 35 135 134 69 8 2 104 144 130 10 192 134 159 78 79 102 112 79 107 240 180 51 56 131 111 40 130 71 176 66 103 88 79 83 40 2 95 8 135 93 136 130 71 0 139 57 120 4 102 136 134 82 74 79 50 56 3 70 196 133 88 26 182 80 152 5 85 32 130 192 215 25 183 53 93 149 150 206 254 94 232 5 111 178 134 71 40 130 103 32 131 57 112 79 55 220 113 183 173 206 87 2 11 84 32 3 83 32 93 178 63 5 147 206 166 153 166 132 75 192 195 83 56 131 34 224 253 71 200 153 232 140 165 147 134 243 225 135 115 218 207 67 62 243 68 80 95 206 81 39 245 160 39 122 25 41 41 95 31 210 6 23 28 67 137 138 55 81 42 180 161 145 160 204 168 64 98 32 119 223 126 66 129 46 113 153 17 214 112 129 201 232 13 160 217 147 83 87 141 30 201 129 247 131 174 187 59 143 60 233 8 160 185 27 21 42 117 124 103 141 100 178 240 230 224 8 11 2 136 24 56 100 96 56 113 2 131 8 17 12 82 8 113 225 193 5 134 135 10 61 28 100 32 130 130 135 140 25 79 228 200 1 209 131 8 16 10 69 80 20 113 194 67 68 15 23 14 186 56 104 80 99 70 12 29 10 10 241 8 163 195 137 132 20 58 164 112 97 146 66 196 150 6 79 118 40 170 81 38 133 151 24 46 184 192 152 148 100 137 18 30 44 22 237 144 81 134 144 19 56 98 116 160 254 80 162 170 199 132 36 101 118 96 122 194 5 131 180 22 79 164 21 73 148 226 137 20 52 114 208 16 145 194 36 203 147 25 87 158 136 145 35 99 81 24 48 36 148 45 138 161 96 196 160 40 101 80 236 136 195 33 69 178 139 247 150 37 140 243 50 82 12 64 135 118 40 225 147 193 97 25 162 29 38 181 26 243 108 209 179 38 53 19 76 136 146 162 198 162 14 135 66 60 12 27 104 193 152 17 79 96 60 121 66 134 239 217 27 113 2 143 33 116 40 242 228 201 59 184 200 97 36 141 161 84 208 110 185 48 202 160 131 14 24 20 68 116 184 121 19 33 201 238 37 56 148 224 110 53 69 103 229 202 141 122 232 192 230 89 180 97 189 122 149 138 50 67 151 46 84 212 164 73 75 150 140 25 51 193 248 194 140 51 207 0 8 205 55 216 88 18 4 54 82 28 16 14 42 115 52 19 14 54 207 60 131 13 133 100 72 17 142 15 150 136 99 141 133 207 248 82 129 43 225 184 66 128 20 216 96 19 196 34 225 88 35 13 53 226 136 131 6 26 253 249 50 204 44 254 57 226 130 203 35 7 28 128 10 136 0 14 179 99 49 206 20 3 96 50 212 196 24 72 16 208 248 225 71 140 214 20 131 203 48 197 12 51 12 51 214 124 195 76 17 186 96 163 139 37 204 124 99 13 128 204 20 99 37 46 170 168 146 74 42 58 62 162 64 38 187 160 226 75 144 193 32 115 230 149 59 246 34 95 47 84 90 121 229 48 200 204 146 10 37 136 196 145 134 19 63 112 133 147 122 141 58 250 40 164 200 101 228 2 112 27 197 244 154 65 135 157 5 217 9 64 76 161 210 78 64 61 148 88 85 20 92 80 148 64 64 24 231 16 16 94 188 241 152 88 188 61 100 81 90 108 157 164 169 166 36 181 20 153 169 64 84 85 84 82 14 65 84 85 12 59 149 117 106 97 200 190 246 43 6 187 178 164 153 65 34 96 49 136 35 66 0 219 193 15 124 240 113 72 21 56 105 74 129 16 142 164 1 65 75 198 118 85 212 5 154 33 4 209 9 29 216 225 136 17 16 236 228 211 67 25 133 7 195 5 15 205 148 19 3 168 37 154 232 15 55 116 0 68 254 37 154 88 209 85 73 128 253 218 85 93 98 93 199 40 99 24 180 197 64 12 55 21 6 87 110 205 250 214 46 5 93 124 18 71 37 142 224 176 83 84 121 72 114 150 110 35 201 192 131 94 20 48 32 195 76 46 220 165 210 15 48 228 36 67 23 142 56 34 240 75 27 123 16 149 196 117 68 178 220 89 23 92 128 131 192 12 128 68 65 75 22 201 112 193 13 120 16 194 199 20 105 121 245 85 87 50 244 214 46 92 237 85 37 152 90 246 82 244 153 7 50 228 81 71 29 79 48 144 212 65 32 137 37 83 92 32 164 192 175 8 52 40 157 17 219 112 53 197 174 12 14 191 44 43 108 65 217 134 178 11 236 190 4 219 5 49 116 241 131 19 78 48 26 105 114 205 61 1 93 40 203 204 2 67 10 63 91 188 92 106 156 253 202 181 65 157 63 42 130 11 44 112 0 3 27 202 72 131 75 142 170 20 145 98 56 177 163 210 9 26 81 100 66 123 20 186 124 211 204 28 82 144 225 229 34 12 74 81 129 46 195 207 129 159 134 125 120 57 71 16 225 176 136 254 77 39 150 64 3 226 12 126 104 51 199 1 51 64 3 77 5 168 64 19 8 141 203 251 224 135 239 190 40 35 204 47 71 146 129 198 11 125 168 8 230 34 82 72 161 75 56 96 206 17 5 25 203 132 179 32 52 157 116 18 206 50 50 65 6 50 88 130 119 3 4 210 28 10 64 6 83 128 9 26 216 64 133 134 22 177 12 111 88 98 17 100 8 194 28 110 116 166 76 40 192 75 24 170 223 253 2 177 12 106 100 34 16 81 112 133 242 168 177 140 69 184 130 26 125 144 2 26 118 1 13 75 244 33 10 107 216 67 28 178 224 4 174 244 12 114 62 252 97 114 38 149 3 73 161 100 112 30 160 128 168 142 18 3 38 32 113 39 94 139 1 199 98 0 132 173 148 5 103 102 232 2 22 133 224 21 24 236 48 39 17 169 212 173 68 21 58 83 161 106 82 47 65 150 226 128 80 130 83 65 177 92 14 57 149 4 36 96 44 28 76 1 8 141 115 2 14 58 192 149 194 45 202 98 93 49 90 7 38 80 2 130 132 198 13 70 48 10 14 170 240 6 65 174 254 209 42 76 67 162 23 210 96 51 4 196 96 2 200 42 203 64 110 133 18 99 201 193 9 95 25 153 30 1 121 144 42 120 65 59 36 105 207 5 162 114 2 39 84 130 21 147 208 132 38 222 112 129 157 200 225 96 50 192 65 42 163 210 199 9 104 167 42 200 74 74 17 101 34 3 180 120 192 11 94 40 74 2 78 146 20 117 81 10 56 100 185 128 23 62 97 5 73 220 161 3 50 72 64 84 146 64 8 208 104 230 53 5 241 194 21 98 112 1 209 180 39 42 56 17 1 6 178 85 5 151 201 196 9 117 184 193 74 216 104 170 124 221 160 4 49 248 25 39 142 99 144 210 116 192 11 89 192 73 225 48 112 147 179 200 36 11 105 216 129 103 64 130 129 168 16 38 160 112 65 14 178 78 53 24 74 249 70 93 27 105 78 30 240 117 196 159 197 160 163 40 49 200 169 102 137 24 189 225 229 136 50 232 137 102 54 179 27 131 12 19 3 195 20 205 111 244 34 211 125 22 68 153 189 217 8 206 24 231 56 32 210 38 7 66 240 2 34 66 17 140 89 148 71 254 4 188 228 41 82 135 162 175 206 104 225 76 172 155 133 42 206 80 129 223 197 142 12 4 152 131 85 103 208 135 25 4 161 25 186 232 131 37 164 208 213 224 53 136 120 51 56 0 25 102 183 136 78 76 21 27 189 115 222 34 44 81 1 75 132 99 76 3 36 134 239 94 160 139 78 4 65 23 204 11 107 87 231 64 128 40 44 162 2 69 32 80 49 162 161 11 31 16 35 19 93 13 199 34 80 180 8 174 54 163 15 4 32 195 100 103 208 12 254 201 15 27 100 168 192 28 214 250 213 176 222 167 15 10 52 133 37 124 208 140 78 28 32 180 21 64 3 94 175 138 189 69 60 35 177 186 120 65 16 58 1 141 112 52 35 8 82 88 208 241 102 64 0 52 184 162 2 45 210 197 1 44 113 65 192 22 143 0 62 88 3 35 240 224 5 39 240 108 99 73 189 174 122 132 40 154 237 14 243 42 206 252 141 79 133 80 28 38 44 113 138 3 99 66 29 185 226 169 41 76 129 9 93 1 2 19 182 194 132 134 148 128 139 76 112 129 209 126 64 74 47 212 140 148 254 58 172 130 39 185 104 204 42 8 198 9 198 244 130 123 59 48 133 58 152 225 13 94 224 10 70 154 217 148 4 180 138 137 10 54 131 25 202 80 6 4 87 69 47 69 201 66 29 28 252 131 9 220 192 85 114 200 66 9 36 112 1 30 184 225 9 178 241 130 206 228 240 224 155 32 241 44 20 240 66 30 188 96 7 53 224 160 4 14 238 64 22 222 192 4 94 86 1 109 111 80 20 12 100 92 6 62 188 161 4 238 60 68 39 39 224 132 73 76 226 108 76 88 155 34 243 160 7 39 120 224 7 7 117 66 144 113 130 128 55 132 225 48 216 106 3 34 8 65 221 50 224 225 80 117 144 67 61 179 192 7 71 212 129 7 37 240 2 218 152 92 22 47 108 98 18 135 176 3 14 100 122 149 194 69 70 38 23 112 130 36 178 112 8 89 42 152 15 118 104 3 33 78 178 228 67 148 18 6 94 8 25 31 234 224 222 31 188 129 16 121 240 164 11 132 80 7 62 12 210 39 24 184 65 29 126 176 18 39 215 1 202 63 56 1 12 170 96 6 53 120 129 19 143 254 99 41 5 114 96 6 57 212 172 4 63 40 131 28 222 16 7 38 96 32 6 85 208 153 131 71 150 3 44 212 161 12 55 32 76 123 212 243 43 38 24 243 9 14 46 65 137 155 237 73 86 19 226 13 102 248 203 20 222 80 6 234 146 216 11 92 32 101 21 234 233 4 7 59 1 6 50 152 194 23 220 208 5 38 108 146 9 93 240 194 20 242 248 131 44 76 55 143 20 64 47 123 125 149 196 77 30 110 40 135 161 232 67 222 141 131 31 252 224 214 62 44 10 86 188 176 7 79 224 130 23 60 192 64 10 224 134 93 32 206 50 5 41 160 192 30 226 51 11 88 192 226 72 124 61 64 31 194 209 7 6 65 99 6 150 48 7 42 238 99 142 189 6 225 0 186 32 235 240 246 151 34 115 204 124 178 82 168 236 134 196 58 131 22 213 213 26 139 240 1 95 117 17 5 84 248 78 23 51 32 67 39 4 171 11 242 89 163 126 4 64 133 52 158 97 13 87 24 192 21 92 71 5 100 43 0 141 153 207 0 21 133 29 59 114 81 193 191 11 234 194 184 225 48 254 199 219 87 43 5 3 152 130 26 6 208 5 57 80 145 59 13 241 220 18 47 128 96 147 156 231 7 113 68 35 26 223 208 197 103 251 202 90 255 205 160 123 83 141 81 239 120 119 159 214 70 189 2 39 15 6 46 24 17 135 126 67 209 186 27 255 188 16 59 210 17 209 228 96 152 163 15 181 79 231 11 179 14 48 161 245 91 193 1 16 100 144 170 14 208 177 245 9 182 183 25 166 128 19 15 8 38 11 76 172 111 227 126 176 131 127 25 243 13 78 144 3 12 110 208 184 27 48 46 214 117 104 156 29 220 107 239 246 194 115 37 15 105 202 78 46 144 5 71 116 1 144 72 188 192 4 136 101 205 150 12 172 14 172 38 48 41 25 55 221 18 76 192 3 110 112 92 81 152 176 97 39 84 129 43 245 18 149 23 38 97 252 46 188 193 249 48 248 129 28 84 193 157 25 95 21 112 11 182 188 1 30 20 130 241 89 90 186 113 90 187 181 65 30 32 129 19 200 0 3 228 128 35 180 193 18 24 194 131 57 65 159 105 66 162 88 133 11 200 65 24 52 69 7 254 84 1 23 44 193 18 212 65 21 88 193 199 72 2 23 228 65 18 36 65 27 112 193 14 40 82 172 85 2 165 53 91 242 229 65 30 160 87 190 20 142 47 25 77 166 160 82 12 180 219 13 212 211 20 56 2 22 96 65 30 76 130 12 120 129 21 32 1 18 120 65 25 36 223 27 188 1 243 1 1 212 112 65 18 188 32 60 173 82 37 112 153 213 252 64 170 21 132 19 140 27 253 121 193 194 229 193 18 214 129 173 41 199 82 8 129 140 177 31 12 212 65 33 164 65 27 120 65 61 221 128 131 85 1 16 72 0 170 121 129 29 97 161 108 60 212 80 116 84 89 148 24 157 17 24 12 148 129 21 40 95 162 36 95 21 180 193 15 12 17 181 5 12 41 193 128 147 17 66 27 100 1 16 192 64 18 180 155 126 121 146 59 121 65 21 200 65 87 76 1 130 217 155 123 133 65 192 52 14 190 116 65 123 181 30 115 20 142 45 234 13 109 236 74 77 21 142 181 116 84 82 65 156 16 84 65 28 68 66 42 220 66 28 64 64 77 125 222 15 21 133 221 240 64 254 44 28 195 44 120 2 201 213 194 51 196 206 34 28 0 239 72 193 246 148 221 3 221 71 38 160 21 25 220 29 206 85 128 130 252 28 52 108 213 202 253 156 134 132 131 20 16 192 1 116 2 134 140 195 55 160 194 11 116 22 86 37 151 37 28 128 239 12 80 51 160 1 25 140 73 56 16 64 11 89 131 147 20 64 20 4 193 85 97 195 100 153 131 138 148 221 130 192 93 51 204 213 100 53 67 211 17 15 133 64 200 84 205 193 12 152 2 52 20 128 238 232 93 241 164 220 151 204 64 39 136 21 134 144 129 31 64 67 225 89 3 220 57 29 102 129 22 86 29 79 5 4 66 57 136 67 204 149 100 39 52 3 56 14 80 241 84 0 51 132 130 230 77 87 117 25 162 50 94 151 16 113 202 65 148 158 75 1 199 9 48 129 172 148 139 226 48 1 16 236 196 15 152 98 21 0 24 87 196 87 226 168 24 124 249 96 70 212 87 21 88 24 167 137 91 136 49 28 117 9 160 179 225 217 27 160 77 21 150 88 81 204 219 78 204 27 32 237 222 109 84 133 175 232 254 209 20 216 129 182 100 154 238 149 5 70 180 7 14 188 129 182 188 211 13 28 130 35 104 139 35 188 65 89 184 128 27 88 75 87 80 64 21 60 129 28 53 139 224 32 81 21 176 193 13 132 31 97 54 217 4 24 83 29 102 129 32 9 6 182 21 194 199 8 18 43 85 66 200 132 225 5 84 65 24 172 81 185 216 161 36 68 66 183 252 64 30 164 65 147 49 7 58 125 129 23 160 4 19 56 130 36 72 194 39 84 194 116 21 2 18 20 194 13 196 193 186 125 194 111 162 38 195 61 95 9 108 88 42 113 1 23 132 84 179 28 209 175 156 138 111 20 6 186 164 210 21 242 70 18 76 66 2 212 193 111 58 66 37 188 19 12 240 91 84 68 69 21 20 194 36 112 194 36 124 225 4 56 130 19 172 139 12 0 129 166 1 135 23 104 2 121 234 140 16 108 216 118 192 64 62 37 7 211 116 133 49 69 69 103 54 89 137 213 83 2 148 129 17 160 12 167 73 69 63 121 82 72 41 71 117 160 11 25 86 18 182 85 66 166 101 154 39 117 192 186 105 7 6 254 116 65 166 25 153 181 252 128 30 252 0 90 196 64 26 116 155 28 48 217 13 4 102 125 41 102 12 32 152 4 212 151 175 133 88 25 132 24 19 148 192 126 189 65 21 76 0 33 65 11 99 12 92 224 236 70 73 137 14 16 129 197 115 232 1 38 240 130 39 104 129 188 244 75 81 70 10 176 148 0 34 8 67 45 120 2 40 112 130 39 168 66 48 76 8 42 216 220 11 109 35 216 233 66 95 69 129 20 160 195 34 136 163 240 84 192 254 116 85 68 74 207 152 190 80 56 104 213 28 116 79 56 24 222 50 248 192 1 60 194 254 28 0 1 120 213 208 97 3 52 76 72 239 84 99 39 16 128 47 88 200 98 5 2 53 52 67 240 76 221 1 120 67 253 220 71 97 129 105 247 144 149 134 56 221 34 168 8 54 28 100 255 208 93 51 20 0 42 204 163 106 93 16 54 152 67 31 116 207 5 81 136 20 204 1 124 68 195 51 56 80 111 97 22 54 110 79 161 130 150 140 244 22 87 65 195 46 252 72 51 108 143 54 56 72 49 120 66 36 228 80 187 49 10 254 81 78 41 79 105 151 119 121 128 71 64 235 16 121 0 189 97 196 48 181 139 237 21 5 227 212 147 104 178 30 19 28 205 200 192 151 30 117 192 95 116 135 31 70 132 16 116 129 227 220 132 7 8 65 77 116 129 12 124 129 11 124 193 23 240 192 10 180 148 12 96 193 48 181 151 169 116 129 175 252 0 19 1 69 66 188 81 226 224 17 170 224 0 210 192 64 30 141 107 166 28 77 21 96 128 168 93 1 99 122 1 233 177 5 233 52 166 76 168 196 21 92 1 2 204 196 38 233 70 22 32 2 18 76 0 14 164 90 29 194 0 15 212 65 22 212 225 49 149 128 29 97 27 30 196 172 39 9 193 29 72 44 165 104 70 21 148 129 95 60 129 162 120 1 165 220 129 26 196 192 15 112 65 125 253 223 84 156 64 25 8 32 5 164 108 189 174 192 13 252 95 29 212 97 9 116 146 23 28 66 187 37 223 5 144 161 39 121 1 22 180 199 27 164 129 4 160 23 74 184 192 20 116 75 226 164 6 101 44 76 63 213 193 84 88 193 36 76 64 28 96 161 61 221 254 192 78 20 232 242 29 66 22 160 64 26 28 2 15 60 68 9 132 161 197 138 64 96 226 128 210 48 155 17 240 128 11 32 174 49 185 140 19 188 33 28 222 24 138 29 70 129 150 24 67 169 193 19 76 69 7 160 218 76 196 192 145 213 104 123 57 220 194 28 223 70 197 192 243 157 197 119 9 193 27 216 134 48 254 0 112 20 142 8 228 0 27 192 0 3 128 192 9 132 65 41 217 211 216 122 65 84 200 193 74 140 91 84 28 97 85 214 76 12 104 13 5 80 23 23 17 24 111 8 193 125 85 103 187 54 68 74 112 132 120 229 196 41 181 203 47 126 69 14 232 87 28 96 66 42 12 131 39 28 204 117 192 4 248 134 175 248 142 175 248 118 64 2 152 104 46 12 67 44 92 2 35 48 2 37 52 194 25 160 193 202 145 129 91 105 35 247 128 93 204 237 2 26 84 128 31 4 129 71 226 220 131 176 213 28 160 130 15 72 129 31 188 0 99 181 106 16 20 144 20 220 135 51 48 131 52 68 193 213 133 3 49 100 143 55 120 195 252 250 65 38 64 3 246 254 68 129 31 160 21 51 92 73 143 0 201 55 152 130 1 60 73 1 160 193 1 147 193 57 94 150 70 146 1 164 106 143 252 120 195 10 247 1 237 160 156 20 144 42 54 224 150 41 116 66 20 52 67 204 145 65 31 248 64 31 80 100 17 16 170 20 156 65 8 51 67 38 120 48 251 184 2 49 4 65 245 248 129 43 180 221 28 200 8 54 88 22 25 100 8 6 249 65 39 68 241 1 100 41 34 164 65 21 52 28 64 49 107 82 89 74 217 200 64 233 177 49 99 136 158 187 14 83 77 184 171 120 145 107 124 181 151 242 218 30 19 152 49 192 181 87 34 149 1 123 113 152 126 149 129 44 222 18 19 84 65 22 100 129 242 98 209 19 168 1 234 50 78 22 92 101 21 184 107 23 152 193 19 48 129 131 197 0 143 150 1 134 233 94 77 193 133 209 84 1 31 116 159 53 1 83 68 148 17 104 232 17 158 81 97 29 8 1 6 96 129 29 108 152 26 178 30 140 213 1 127 149 96 166 249 83 177 96 95 81 224 128 28 76 2 30 188 193 28 162 154 28 212 65 254 121 102 91 49 171 224 4 176 25 18 224 129 112 222 64 139 162 141 26 22 239 134 166 26 43 213 129 27 80 218 15 196 65 149 197 65 33 88 179 120 42 224 14 213 178 184 29 159 112 22 130 23 48 89 145 101 154 143 90 218 36 4 160 35 28 194 15 220 25 157 57 130 25 220 83 14 160 102 128 125 69 76 92 210 177 92 0 19 132 152 28 16 194 38 84 193 14 140 104 59 59 25 157 49 25 25 214 65 28 16 130 36 80 172 181 109 66 48 11 193 140 138 167 45 39 95 49 59 88 22 220 128 18 228 129 131 185 33 22 24 142 74 28 90 58 51 89 162 160 141 129 201 103 172 29 66 136 77 129 38 139 91 37 175 172 148 85 194 27 232 83 187 160 75 64 143 242 20 212 83 241 225 27 112 88 27 31 76 23 182 200 50 130 73 17 142 113 109 15 156 64 85 14 216 20 117 193 243 145 97 183 248 151 49 57 65 103 86 1 123 145 215 175 253 151 39 25 196 20 116 129 16 60 196 111 176 87 14 48 205 67 176 87 86 56 11 118 17 11 227 64 7 37 204 254 130 48 120 2 34 88 129 17 212 181 93 223 53 94 231 181 94 239 117 93 35 1 27 120 130 48 8 3 80 238 1 97 35 2 37 172 1 17 20 193 25 4 3 56 152 194 35 24 200 35 208 137 47 60 194 50 232 47 26 84 80 51 248 130 37 64 143 163 46 3 248 236 130 254 6 66 38 100 194 50 184 66 38 132 195 35 152 194 51 152 194 25 148 194 142 12 195 35 156 193 162 50 195 35 60 2 52 136 67 51 128 15 26 116 2 53 184 66 32 4 194 25 200 54 142 168 66 35 172 193 141 68 3 51 4 130 111 31 128 0 45 2 253 28 93 32 96 214 132 232 194 99 155 66 38 96 195 50 196 54 26 196 73 31 200 111 32 48 3 54 100 194 25 68 118 39 104 29 107 111 119 50 72 67 109 51 131 47 52 66 34 12 202 44 192 175 98 75 15 53 236 66 252 182 54 109 63 66 50 12 8 107 171 118 161 234 182 37 4 195 35 172 1 40 32 194 30 132 129 19 220 229 178 162 49 228 108 132 233 110 87 233 201 10 27 119 132 67 176 113 77 212 6 16 254 252 133 79 13 145 12 180 94 244 254 134 16 76 65 88 47 209 20 56 65 235 105 34 227 176 23 30 193 0 137 251 49 19 36 184 237 169 145 242 93 165 175 208 223 30 55 78 30 221 0 90 243 70 76 105 12 38 249 161 28 177 68 66 204 77 219 216 69 123 220 192 126 57 206 169 80 155 49 169 155 116 114 129 30 150 224 134 181 115 7 184 70 103 220 25 128 57 152 38 58 153 143 26 88 129 149 51 12 76 64 36 43 223 21 106 34 21 106 185 32 5 153 30 66 77 206 106 121 137 225 97 27 196 129 0 58 193 161 92 225 142 90 90 27 180 193 65 133 1 162 112 129 14 149 82 137 93 33 250 117 121 25 200 109 35 186 193 23 84 5 217 154 237 162 68 27 76 12 97 12 8 65 26 32 138 41 98 155 23 4 91 187 213 151 171 16 152 175 21 186 129 17 216 21 196 1 23 180 65 25 168 17 159 235 121 27 160 184 171 36 173 186 109 24 167 77 1 80 8 133 170 155 91 27 180 91 85 43 18 41 26 83 25 252 83 226 168 98 23 180 203 5 188 129 254 35 188 154 128 90 232 129 113 186 147 217 110 22 60 141 129 241 155 0 58 108 85 99 91 34 27 147 253 1 1 34 243 23 143 122 162 167 116 11 182 100 65 29 21 197 13 28 114 137 119 148 19 148 237 14 173 4 196 46 175 176 184 235 71 80 68 28 51 107 85 64 98 26 224 1 35 0 182 160 196 66 44 220 2 47 212 2 191 247 187 191 255 59 192 7 60 191 223 194 45 224 251 44 4 67 49 204 2 40 68 215 164 167 1 28 252 1 151 170 2 149 92 201 159 228 73 149 148 73 150 128 136 133 48 195 198 99 60 153 132 8 199 215 150 149 212 66 142 132 130 39 120 66 40 180 137 142 0 10 197 95 137 145 156 9 150 14 74 40 132 2 37 120 2 155 164 66 45 20 195 139 4 194 11 148 131 150 124 3 106 23 129 53 12 61 153 56 131 51 220 137 149 224 9 150 148 137 214 69 195 203 159 137 51 84 9 160 84 188 48 12 74 42 156 60 214 195 66 40 192 181 48 84 201 209 91 73 96 215 194 205 231 200 202 87 60 201 123 2 195 95 193 133 59 254 56 131 35 149 62 57 56 108 24 68 75 104 198 220 119 211 65 104 76 176 188 132 181 150 139 168 32 75 28 93 146 209 132 148 63 31 139 47 229 6 186 88 12 224 95 231 44 69 216 108 12 75 81 96 205 76 212 20 57 145 133 231 136 199 47 165 132 198 252 12 228 207 82 84 128 202 178 232 81 248 45 62 70 212 98 83 60 4 110 24 77 123 124 132 67 20 68 224 3 18 42 125 159 65 128 134 185 60 190 32 93 192 66 216 5 104 240 198 82 176 95 122 242 254 87 100 95 85 144 133 95 94 18 132 81 192 10 32 147 240 163 11 115 28 68 18 69 14 104 140 4 58 157 192 44 25 13 204 124 31 96 252 189 169 232 133 38 242 132 11 148 64 2 132 57 139 87 74 111 0 11 28 61 102 64 101 10 5 16 198 96 116 128 138 41 63 170 84 69 2 132 82 28 1 133 56 121 166 195 141 254 169 176 31 83 32 172 85 164 71 173 72 5 64 120 144 113 194 67 65 10 20 60 16 244 80 162 68 135 14 5 49 80 64 128 193 5 66 15 23 36 72 184 224 254 240 66 71 135 30 56 110 116 56 114 99 12 23 17 61 80 56 113 18 67 75 12 5 21 190 60 49 147 102 77 155 55 113 58 140 241 227 135 21 49 123 16 121 154 213 43 88 50 163 210 144 38 85 186 148 105 211 164 70 147 245 154 21 138 210 158 56 105 172 100 13 147 6 15 35 74 158 64 121 18 59 150 108 168 80 169 82 205 66 139 214 172 89 81 169 204 202 90 219 214 172 216 75 138 240 238 209 203 200 144 33 74 95 203 210 173 235 233 47 163 63 122 17 121 189 4 42 213 169 51 69 158 21 43 198 140 89 163 53 190 134 213 82 219 214 83 93 193 103 215 166 18 21 74 212 232 207 99 41 41 50 164 218 16 157 190 134 46 45 14 27 202 19 92 187 119 95 83 186 244 87 48 38 74 134 224 132 113 98 68 200 64 152 56 145 39 87 190 124 166 139 153 5 33 190 76 232 66 134 11 235 206 155 87 119 225 161 165 117 151 39 40 38 124 137 125 229 72 157 28 205 167 31 121 240 96 7 6 12 68 176 36 153 94 2 133 146 216 97 82 199 208 33 70 254 135 134 29 46 192 96 32 130 34 146 96 164 234 236 243 111 164 132 68 104 240 61 17 24 128 8 161 237 28 18 240 61 144 0 204 144 129 20 26 228 240 189 13 83 120 48 133 17 61 132 15 66 248 160 163 65 6 15 26 236 240 196 17 69 224 46 194 24 49 184 128 2 248 24 192 224 132 14 100 232 49 161 148 184 3 47 63 25 40 160 136 186 27 11 18 225 4 149 16 74 193 133 21 236 219 104 71 135 162 236 128 160 149 172 83 72 60 151 92 248 80 73 154 88 164 1 198 22 93 72 97 160 248 34 218 14 194 34 41 144 161 165 27 77 92 243 165 54 69 80 9 65 149 26 84 17 188 137 140 227 78 4 151 122 156 50 67 28 114 32 48 7 135 110 178 174 32 5 79 52 241 132 247 64 80 242 33 132 28 36 145 76 24 16 37 168 69 17 72 140 161 191 254 212 187 32 1 145 212 3 245 130 149 154 19 146 186 130 174 35 144 185 87 109 2 41 134 27 140 112 34 13 56 246 240 43 54 179 224 10 205 215 95 129 13 237 44 179 64 185 132 48 58 172 254 178 194 9 39 126 112 2 9 43 210 136 67 175 105 169 173 246 15 70 176 245 42 146 109 35 73 236 218 108 19 203 54 91 189 226 40 247 170 52 208 53 215 218 105 15 163 214 220 52 188 72 55 142 66 24 233 44 21 76 212 154 101 150 90 122 13 229 18 108 217 101 87 92 112 7 110 247 143 131 171 197 163 220 91 225 104 184 225 56 240 136 248 91 195 246 136 88 97 115 205 197 245 90 189 244 96 35 13 225 140 0 162 58 232 96 45 217 100 154 240 131 137 187 227 92 130 174 72 232 90 74 232 4 25 114 168 136 72 25 132 200 129 61 84 91 206 52 37 132 210 196 48 134 139 40 136 129 137 41 152 232 64 204 136 100 56 154 9 25 58 112 110 187 233 118 142 181 186 12 157 240 98 10 28 100 38 72 167 18 58 234 8 75 41 185 115 161 82 78 61 212 239 160 141 44 146 33 107 47 50 52 219 193 13 55 117 145 211 186 29 84 178 34 33 79 224 112 83 19 115 244 0 7 191 23 237 15 6 24 218 72 130 11 47 60 16 2 139 21 103 210 177 111 12 254 98 152 41 6 34 55 170 174 37 130 218 164 73 8 55 220 232 162 191 231 60 200 210 37 201 129 46 146 160 11 126 240 226 141 50 126 40 193 185 139 116 196 224 189 49 9 180 47 134 39 188 144 1 71 153 25 0 239 4 188 83 184 146 187 1 79 146 185 67 130 152 48 67 8 227 75 159 122 71 201 49 84 73 71 168 165 76 238 162 14 240 30 190 65 15 110 200 194 241 39 124 127 48 239 29 43 39 200 108 28 113 100 239 35 153 22 189 154 100 231 30 74 168 3 201 87 10 210 56 158 75 223 255 100 228 102 39 43 35 220 32 107 91 129 24 30 38 54 48 5 46 144 129 216 58 88 33 34 118 21 225 52 139 39 60 113 66 21 178 224 5 13 150 65 131 29 236 32 186 162 245 46 16 142 48 93 32 92 216 8 59 248 133 43 172 176 10 87 200 96 11 51 152 133 12 198 139 132 29 204 66 21 170 112 65 28 126 225 11 16 211 67 95 34 49 176 72 232 69 97 32 124 3 9 49 150 68 18 46 17 93 30 84 97 86 160 104 5 47 124 65 131 75 212 254 224 23 100 152 69 30 202 208 131 97 8 67 14 127 112 131 203 77 173 107 255 51 227 114 72 166 37 232 116 77 81 36 91 153 64 158 230 156 151 8 129 9 212 73 21 202 166 230 157 235 141 44 122 6 233 0 19 186 32 186 253 56 36 107 83 16 2 119 54 7 158 151 200 224 9 79 192 223 65 242 231 144 12 58 97 10 49 40 210 64 246 115 131 42 200 129 15 111 112 66 212 254 132 52 133 140 232 83 11 74 65 141 58 80 5 62 56 194 17 124 168 3 16 46 112 3 47 212 225 6 27 1 212 118 44 7 42 86 33 82 71 49 240 136 167 64 101 134 184 105 14 33 224 121 72 134 250 230 2 79 217 82 2 55 96 69 30 16 177 9 24 220 192 17 83 160 0 122 66 98 30 38 216 193 60 158 50 79 209 222 80 135 47 56 161 63 214 235 14 206 186 240 60 153 40 146 34 51 137 86 24 184 0 131 134 144 202 33 43 162 222 72 64 229 4 62 60 1 62 20 128 1 168 240 41 131 82 142 228 104 69 58 85 144 34 34 157 26 253 64 14 102 88 27 254 128 74 2 208 118 94 105 68 21 81 137 149 108 210 160 237 20 72 71 206 201 193 227 48 32 130 234 56 97 18 85 168 131 23 112 208 129 107 242 103 36 25 9 82 223 100 224 17 150 10 100 73 214 19 8 75 32 178 146 132 174 147 116 47 145 206 74 32 201 185 98 190 244 140 53 81 210 162 58 144 3 25 224 160 89 87 240 130 23 211 208 6 169 78 149 170 85 181 234 85 219 192 134 54 164 129 11 94 204 194 178 156 32 50 28 140 21 7 55 176 32 88 209 154 86 176 226 240 130 106 117 235 91 153 245 3 33 0 129 174 66 168 224 89 221 154 195 180 242 132 174 21 188 160 6 179 26 7 55 136 16 90 86 184 225 91 113 200 214 101 233 21 174 96 109 214 178 228 122 215 198 162 245 174 146 85 43 79 196 232 179 227 20 149 179 55 89 227 103 191 243 28 43 185 160 66 139 154 89 28 49 144 82 28 76 1 60 212 193 165 212 182 163 168 153 245 8 151 61 234 145 37 1 132 131 46 156 128 33 37 144 165 19 96 48 144 44 65 135 58 56 123 94 65 254 98 48 214 21 97 72 131 222 76 136 64 83 26 75 215 57 225 6 15 41 19 19 128 176 18 17 168 40 185 41 173 167 16 58 112 56 37 180 33 14 23 148 131 19 24 242 131 55 252 224 2 164 237 0 14 188 217 129 27 252 19 6 97 188 64 119 1 122 3 253 102 182 4 48 168 68 24 49 165 30 1 155 231 84 145 202 129 38 188 80 8 66 232 0 5 39 173 239 167 60 53 205 216 249 86 150 12 145 167 132 47 12 3 39 212 225 10 60 176 104 78 83 130 179 74 174 180 187 64 0 21 14 28 12 42 12 247 247 63 233 209 239 225 166 9 175 19 8 33 6 12 121 111 114 29 50 77 254 220 192 194 179 132 49 202 94 50 18 9 187 174 191 211 156 175 141 15 247 3 165 186 32 7 211 25 8 205 114 144 92 80 37 151 182 2 129 175 55 47 208 60 38 224 0 106 48 72 66 36 54 156 5 22 243 199 151 35 153 147 12 26 132 79 51 11 183 116 168 106 99 144 166 70 91 148 93 105 118 169 146 45 233 58 91 19 22 197 200 126 39 136 129 82 115 254 32 4 58 50 97 178 133 158 44 19 152 64 199 177 42 117 74 17 62 220 126 231 251 98 24 239 183 190 97 172 32 164 37 172 95 75 95 26 210 158 170 78 132 41 13 227 73 239 23 192 243 53 114 12 96 80 86 77 35 186 173 85 248 2 14 111 152 67 48 134 17 210 144 230 244 126 147 171 223 178 26 217 212 188 6 166 64 109 155 131 90 155 245 197 98 148 244 163 81 93 235 203 1 147 180 107 204 115 158 121 186 146 28 76 155 143 177 138 1 16 176 123 98 68 31 13 7 46 0 2 210 142 38 164 64 226 210 38 254 105 93 7 9 217 65 39 20 228 2 56 240 130 236 40 128 131 77 134 110 10 50 131 8 72 128 160 6 55 56 210 63 114 200 67 185 224 22 131 89 214 225 13 112 235 128 19 222 144 112 39 92 96 154 111 232 79 71 128 96 134 66 192 77 4 49 136 131 36 234 144 113 121 110 146 224 89 144 167 23 174 114 193 31 248 7 6 114 72 41 6 126 192 149 56 172 215 9 136 136 132 30 242 80 136 56 76 32 11 243 194 131 23 96 176 254 3 60 72 162 16 133 72 3 10 120 128 135 77 16 194 231 54 219 15 72 102 50 146 154 44 136 32 50 112 158 25 112 62 130 54 16 189 231 63 224 193 27 28 145 7 66 16 2 189 97 32 196 36 14 71 129 31 196 1 17 132 168 131 19 32 176 225 79 176 210 9 62 77 94 69 152 64 8 68 88 193 33 63 136 4 39 190 158 134 134 212 65 18 147 240 121 127 167 78 136 54 32 129 33 122 246 64 14 238 80 135 155 123 128 7 243 234 57 30 8 111 82 87 86 1 6 105 200 67 18 144 48 137 54 148 192 9 146 216 68 30 240 176 238 50 134 215 9 114 200 120 29 218 128 130 36 180 33 15 121 168 66 9 38 80 250 211 47 235 16 102 173 67 26 118 240 248 56 56 248 6 111 48 125 25 120 32 3 172 231 129 224 135 147 131 35 50 14 183 118 155 193 3 93 152 194 5 64 34 179 76 61 103 233 73 199 232 179 177 159 125 84 161 111 218 221 247 254 247 193 31 126 241 119 255 182 215 185 126 185 49 58 253 41 89 199 83 202 220 145 77 148 201 126 254 96 182 223 189 231 167 137 210 17 213 77 147 120 106 74 75 215 137 167 198 202 172 44 8 8 126 128 0 249 234 202 62 66 80 16 101 254 174 67 254 232 167 255 238 207 254 252 47 253 212 79 2 173 79 251 48 208 38 98 6 38 170 131 102 138 227 74 244 44 185 176 205 196 112 192 196 58 0 219 174 205 196 140 134 9 166 3 209 162 45 233 56 207 11 150 229 221 206 237 175 132 96 63 128 0 110 60 160 63 100 160 10 120 96 102 160 103 144 46 64 6 174 192 154 58 128 230 112 15 9 202 0 8 0 100 89 88 76 228 234 203 11 152 96 2 96 160 14 112 32 222 174 240 2 22 39 164 46 0 9 10 225 7 170 144 186 78 10 135 248 224 245 222 64 18 36 1 231 54 130 225 204 128 9 61 192 11 240 32 43 86 14 6 142 200 11 238 192 8 52 97 5 238 32 14 172 32 12 28 225 11 40 192 8 6 65 208 6 1 3 64 224 4 248 224 6 220 196 232 20 130 57 182 3 33 114 128 34 56 132 2 56 73 8 74 224 13 220 64 6 120 128 239 126 254 160 219 34 228 7 36 1 6 108 36 193 216 128 11 10 225 16 120 128 243 226 224 200 160 175 116 22 165 35 198 110 10 54 132 2 240 224 11 245 41 7 24 160 7 220 192 8 172 131 2 186 160 12 28 98 155 184 70 75 102 66 8 62 102 94 126 192 5 238 96 16 102 66 13 176 224 7 104 233 112 248 160 19 157 7 3 178 166 65 176 224 13 160 164 107 96 66 2 74 14 184 110 128 15 212 192 233 152 0 250 96 231 27 75 160 117 120 76 14 192 208 8 190 96 5 190 192 13 190 224 14 170 96 189 102 169 10 194 32 14 188 96 5 54 209 10 190 113 5 96 192 11 192 240 63 40 162 157 74 231 5 51 48 33 21 50 81 26 176 33 29 242 33 33 18 34 23 242 85 26 114 33 143 131 253 200 12 191 230 207 211 134 171 140 38 242 35 65 178 100 96 134 34 122 68 169 170 3 4 193 4 187 70 240 189 128 32 1 18 192 104 140 6 8 14 2 219 14 178 116 84 226 38 106 140 128 170 160 185 156 128 131 118 210 11 34 241 2 128 160 10 84 234 4 254 238 171 195 84 130 60 102 98 13 49 224 10 186 96 35 98 239 221 2 50 6 246 227 130 214 48 107 46 128 33 156 15 3 122 175 63 2 73 165 144 224 13 218 173 3 126 0 15 62 174 89 92 41 225 214 171 3 178 64 89 70 110 36 212 139 7 235 160 16 194 169 224 2 146 7 190 64 13 78 96 16 100 128 15 248 64 97 222 128 23 159 64 13 48 32 7 6 129 2 64 128 1 6 129 37 212 167 157 148 18 128 88 165 60 64 192 73 10 110 34 170 192 13 10 229 13 232 142 69 64 128 2 114 160 18 228 41 6 248 174 245 94 199 220 86 17 2 38 224 33 158 3 149 194 171 14 176 96 67 16 224 164 58 64 8 212 160 7 34 69 13 158 128 2 182 67 13 132 96 66 120 243 168 88 69 248 10 1 154 8 225 7 100 192 13 176 32 33 6 19 11 188 0 2 32 224 18 133 192 5 112 147 2 166 192 11 28 192 1 132 160 14 210 228 36 97 130 2 122 15 246 74 192 11 120 115 28 35 164 11 150 5 118 38 32 61 229 169 14 126 96 2 28 39 254 208 176 96 48 179 6 26 43 161 12 182 10 110 26 15 246 202 0 184 92 64 32 251 235 125 150 164 157 16 50 36 9 148 179 62 235 64 17 52 65 21 244 179 10 180 65 99 101 179 210 35 190 210 3 76 54 203 65 45 148 64 71 114 182 92 5 57 62 133 174 0 144 63 80 16 7 152 192 83 16 237 126 92 96 10 166 128 220 106 226 190 166 160 61 25 170 4 115 200 198 116 211 104 40 143 72 36 64 6 190 32 248 220 36 114 34 199 33 120 224 10 62 169 4 18 192 9 60 78 150 114 0 1 98 96 89 248 67 40 183 166 190 210 233 224 188 192 43 153 32 222 118 50 6 234 3 26 25 66 6 123 15 189 24 34 1 58 192 11 178 128 10 145 6 36 16 110 68 41 0 11 228 128 199 228 169 3 210 224 89 226 160 4 28 97 5 220 224 221 38 160 4 48 96 2 54 137 10 249 160 33 74 64 18 146 192 164 82 148 2 204 160 18 234 141 57 70 50 37 42 98 2 242 96 18 128 139 15 190 224 240 202 192 10 186 212 220 8 97 190 124 177 14 144 254 224 1 82 147 2 18 160 247 230 107 2 138 100 184 40 192 64 82 46 11 110 32 61 39 78 195 60 137 10 219 148 33 240 244 13 228 41 11 214 235 160 88 5 197 144 96 4 244 96 18 192 176 22 109 79 32 235 0 6 168 240 16 192 176 14 178 224 7 8 33 14 28 162 10 242 64 7 188 128 15 184 51 33 138 198 12 196 236 6 14 1 86 139 181 191 162 241 236 210 241 236 38 160 14 170 128 9 234 32 77 219 10 118 100 176 30 129 149 33 36 160 244 118 96 2 192 212 183 188 245 6 202 160 146 70 11 147 218 236 66 237 245 94 149 3 65 205 40 65 31 244 64 159 77 95 251 181 107 172 100 36 234 79 233 40 212 35 213 104 65 241 117 97 209 40 58 102 70 208 4 205 127 192 4 69 183 13 8 74 16 69 41 118 181 48 118 10 34 209 5 2 9 114 110 66 6 166 192 14 234 192 13 214 13 3 152 96 224 186 128 102 186 96 149 248 192 12 142 198 12 14 33 227 118 139 64 156 99 35 186 192 17 6 193 47 189 160 10 116 22 77 29 193 14 254 132 192 14 28 161 18 28 193 12 44 41 103 43 193 53 101 64 105 43 225 105 205 32 103 7 225 105 223 160 11 6 65 19 204 192 14 42 193 14 176 205 17 52 65 19 170 201 12 190 182 18 90 65 19 30 199 107 191 214 17 162 211 11 42 225 107 3 105 19 34 33 15 54 161 13 88 1 40 15 225 19 190 22 11 146 96 18 230 150 16 88 1 236 30 160 13 86 97 110 129 160 34 236 64 19 186 160 80 150 195 72 242 103 71 38 128 11 56 1 17 38 225 19 14 33 209 42 129 22 88 225 153 56 79 18 86 193 22 88 65 19 158 224 4 234 160 21 72 215 14 114 160 11 52 129 22 104 1 193 238 167 77 170 227 190 236 160 21 48 119 19 184 128 11 62 129 21 210 96 18 158 73 7 88 193 115 63 79 7 56 64 110 89 65 18 8 15 6 180 132 58 222 224 118 191 174 149 56 193 243 230 54 9 96 128 9 52 129 116 159 175 4 184 96 19 62 143 19 188 96 2 116 128 16 6 151 11 76 162 17 185 3 7 188 214 104 55 65 113 248 118 19 188 192 254 62 166 224 16 90 161 18 132 9 1 204 160 104 15 161 245 254 118 234 252 114 189 234 160 18 54 161 18 210 233 112 235 160 11 140 214 196 166 160 18 14 65 14 98 32 1 218 100 78 58 10 127 24 246 129 45 20 96 79 134 95 13 207 95 243 76 130 15 22 97 91 69 123 42 148 116 22 180 131 33 56 132 167 198 167 242 227 49 21 48 67 48 2 61 108 36 61 160 239 122 16 118 38 16 226 33 86 96 5 150 132 180 250 203 162 250 67 70 29 2 74 156 35 41 131 4 64 40 160 4 86 128 197 116 211 77 250 235 81 114 96 2 126 36 34 138 132 33 60 32 120 26 98 104 108 100 63 86 132 199 36 224 37 126 9 64 88 133 42 65 162 4 60 0 6 92 194 63 238 235 10 253 73 39 238 171 198 152 4 8 179 178 33 234 228 80 54 130 65 80 237 136 27 209 146 94 88 3 243 67 55 57 101 33 38 192 57 74 224 32 22 2 88 99 0 66 18 162 4 70 192 120 9 98 136 193 230 154 206 181 150 138 184 166 56 162 143 191 120 2 218 107 100 220 254 203 3 36 224 63 169 196 36 22 233 38 237 237 4 158 243 63 11 2 2 204 24 162 0 68 35 16 34 165 168 144 1 104 192 63 192 198 141 126 100 59 222 88 37 82 66 58 168 228 161 84 234 154 250 3 67 72 185 200 168 176 183 238 39 188 36 249 225 0 228 194 32 234 116 86 2 100 67 88 132 35 114 153 153 249 33 147 185 65 179 164 38 42 242 153 169 57 251 16 41 154 211 136 66 35 231 101 28 194 5 160 47 61 136 203 132 107 34 34 150 4 250 20 34 84 121 116 73 64 162 115 42 196 59 28 86 55 243 207 74 44 130 183 16 34 120 234 244 126 192 195 159 142 174 60 86 130 42 85 165 70 78 197 152 30 174 189 210 100 165 68 66 120 166 100 81 64 162 70 48 37 164 190 36 106 224 3 147 2 196 5 34 165 135 239 38 154 29 100 17 47 196 189 238 75 122 184 35 188 26 98 63 30 5 62 200 163 4 116 170 35 130 179 139 251 203 135 51 132 72 88 10 246 60 224 148 98 58 71 88 74 72 218 100 5 186 100 67 24 0 205 48 164 70 254 64 34 115 90 228 154 2 116 184 20 196 74 108 4 205 98 228 33 90 66 53 3 7 151 78 233 154 104 202 126 124 201 70 222 24 2 134 135 1 40 37 67 243 103 58 134 103 32 16 130 2 254 228 160 194 139 3 2 204 107 230 227 152 62 66 53 65 184 154 217 186 173 221 154 131 179 249 173 229 250 127 14 213 217 12 175 1 233 245 185 188 230 146 240 35 102 112 130 138 205 217 78 184 90 184 172 3 100 15 15 64 176 163 206 16 21 104 88 101 129 23 226 71 78 224 112 134 198 76 252 99 162 90 165 37 52 39 59 174 166 155 184 135 1 42 226 225 12 53 178 125 235 137 99 228 112 30 66 73 250 196 65 188 179 219 150 75 101 32 196 155 229 102 73 222 171 35 32 36 62 96 176 33 84 7 62 138 71 151 249 3 151 126 26 164 127 249 154 108 36 122 156 67 6 82 224 139 97 96 67 238 7 66 140 146 35 12 130 7 80 32 143 96 88 205 88 106 166 92 128 107 62 2 90 225 249 155 184 70 160 10 162 120 68 186 220 150 9 153 150 18 153 44 201 254 5 60 44 121 238 141 140 228 72 71 200 36 72 74 39 188 176 163 3 56 192 2 231 250 190 241 59 191 245 123 191 57 171 35 65 184 141 90 102 37 232 136 180 74 2 187 172 228 167 114 130 219 68 2 206 248 70 125 88 66 64 186 128 242 70 235 103 116 57 222 238 167 118 114 134 7 10 208 58 158 64 14 102 53 169 49 101 163 84 199 32 59 234 250 182 39 61 116 230 36 12 86 57 98 192 244 210 64 18 214 141 32 244 107 185 127 196 32 34 36 32 253 242 41 173 4 67 168 3 178 48 169 3 36 96 39 96 0 59 246 3 160 80 101 58 218 251 33 72 229 120 100 194 64 78 162 206 116 164 61 8 66 8 8 225 13 148 155 115 62 26 64 180 35 193 178 139 156 193 67 152 33 170 4 0 74 8 28 129 40 149 156 85 2 168 4 96 233 74 88 88 2 59 224 13 236 32 65 58 219 180 72 162 12 238 224 101 85 74 36 108 74 71 162 7 85 40 32 68 4 36 249 242 192 9 172 207 190 249 251 208 17 61 209 21 61 153 47 50 58 64 120 250 18 254 112 73 154 198 151 70 69 2 176 77 169 163 249 38 138 102 107 214 16 81 214 89 74 22 36 33 178 236 245 214 240 155 75 253 143 82 234 154 192 245 244 92 179 37 90 103 228 6 130 225 90 76 165 128 56 67 228 60 64 72 130 84 126 85 166 12 253 254 234 128 16 184 128 21 62 105 63 118 82 20 11 118 36 38 192 10 46 143 33 34 98 62 58 111 19 166 209 160 178 12 231 240 249 32 250 120 120 100 249 167 15 42 202 49 36 43 213 199 32 108 50 159 139 164 204 138 166 37 200 43 68 224 77 74 220 235 81 2 241 11 175 9 7 58 2 160 12 164 198 254 179 114 60 192 46 193 38 106 216 228 123 232 144 5 249 60 43 215 20 97 135 180 14 122 171 4 72 250 32 90 98 35 128 192 147 90 44 35 244 60 67 84 74 197 1 228 81 184 35 227 184 0 204 192 154 207 22 125 227 57 190 227 61 254 35 135 196 36 251 164 220 200 42 190 114 128 9 2 109 68 201 146 9 154 37 105 230 40 58 115 162 3 192 109 203 248 99 68 87 48 183 182 237 6 254 61 64 228 100 240 112 136 180 10 192 235 224 204 0 11 12 169 35 114 168 190 156 224 9 182 3 94 135 93 236 38 105 190 118 146 173 94 79 195 38 105 154 10 169 103 29 194 11 202 92 10 181 152 57 46 224 13 18 71 87 41 34 101 157 32 53 99 224 39 253 115 231 241 64 240 4 178 48 187 192 12 202 0 188 74 0 9 218 160 61 143 169 12 246 243 63 178 64 14 52 104 10 90 34 7 100 213 147 214 148 38 230 40 225 196 83 103 78 0 225 52 232 7 138 198 117 52 232 84 75 160 10 240 126 171 102 230 39 97 231 38 3 100 32 130 174 131 98 39 107 78 32 101 203 145 117 234 81 10 119 4 76 77 74 32 127 224 244 67 183 63 29 161 106 167 192 77 128 32 12 182 202 8 6 100 233 156 224 16 126 242 7 4 162 58 243 94 190 222 160 90 171 0 222 83 95 131 208 43 42 47 8 11 116 230 109 12 233 34 8 206 9 242 128 241 31 250 227 165 127 250 169 191 250 243 213 201 74 146 209 60 82 86 198 10 84 232 40 229 229 11 178 148 244 254 240 2 169 201 110 34 188 94 231 130 202 0 187 186 32 105 166 160 11 28 98 99 153 17 3 140 192 117 248 192 8 144 222 234 190 192 8 248 209 13 174 224 91 1 226 194 4 47 117 14 121 249 145 131 1 133 27 94 110 116 112 129 163 138 147 31 63 36 190 113 244 134 143 151 55 48 156 148 249 225 36 203 15 134 24 223 212 249 81 194 9 33 39 94 156 196 160 128 225 132 204 153 52 101 94 200 113 35 165 12 23 39 170 212 225 24 67 134 28 71 117 234 200 185 129 65 200 155 60 94 170 120 240 224 198 205 151 43 110 132 72 112 130 231 6 5 15 20 58 120 172 131 116 69 25 77 63 249 48 233 224 37 141 149 36 109 156 148 232 112 130 129 8 15 39 132 56 170 148 198 17 159 18 63 210 164 65 146 38 206 14 43 113 172 84 241 226 165 68 153 55 136 57 17 194 240 164 80 152 52 133 170 116 160 123 249 132 7 30 146 42 189 185 88 229 71 165 27 78 126 194 16 221 180 32 147 9 129 29 57 202 2 163 205 22 43 72 222 24 225 81 69 99 150 254 39 60 221 224 89 146 167 142 140 154 39 38 252 32 155 197 139 35 24 89 254 90 73 227 4 198 13 147 89 68 194 240 82 166 233 71 190 174 235 96 145 1 227 77 21 36 86 18 151 192 33 68 198 143 14 91 59 188 37 14 63 190 252 249 244 235 219 191 143 63 191 254 253 252 251 251 255 15 96 128 2 14 72 96 129 247 61 117 130 12 50 8 33 4 19 66 228 192 83 77 30 128 228 196 68 48 0 33 132 11 30 228 192 132 11 55 252 48 193 105 78 176 71 129 130 241 185 39 81 9 19 220 144 69 9 18 73 192 80 74 94 192 80 2 12 50 80 192 131 19 26 193 16 98 29 122 241 193 135 27 37 64 240 5 15 20 92 80 99 133 85 100 33 129 7 12 184 0 68 67 18 80 128 69 37 174 233 5 210 65 111 252 208 146 143 86 146 197 87 29 19 40 230 22 12 121 140 48 193 4 112 197 84 31 6 29 192 112 89 10 30 160 213 70 9 37 96 80 133 28 167 157 148 2 5 136 149 192 147 17 110 240 128 129 7 95 96 129 1 19 117 224 224 2 254 22 94 164 233 132 158 48 64 160 227 13 19 212 225 196 13 155 172 194 201 42 172 180 1 39 6 12 140 234 129 12 111 196 129 2 10 147 236 16 7 43 171 188 186 74 18 122 188 250 137 38 159 192 240 90 141 108 228 65 129 16 133 172 242 137 36 48 80 224 130 155 46 204 197 67 33 78 164 249 39 31 32 30 86 194 70 53 194 48 108 12 97 4 11 162 19 156 120 242 42 43 28 253 176 37 6 226 186 240 69 167 156 120 193 38 77 20 252 176 87 141 111 132 65 8 167 156 112 82 7 143 224 230 52 129 71 57 93 231 86 9 124 180 88 104 21 193 110 50 239 14 48 120 32 2 3 23 96 86 194 8 112 25 8 113 196 18 79 76 113 197 22 95 140 113 198 5 62 133 160 11 59 157 160 33 199 18 102 122 218 15 48 196 144 33 5 57 164 55 146 157 32 117 128 129 202 25 194 247 166 68 60 86 81 5 12 41 178 212 129 16 94 188 212 65 14 29 120 133 88 98 215 229 108 39 141 44 174 135 67 182 253 150 112 195 5 210 121 129 131 138 89 112 68 254 45 181 75 114 201 16 75 57 211 72 163 19 111 184 215 82 7 50 128 101 92 12 67 147 182 222 208 116 213 244 240 76 49 123 225 198 86 16 84 193 209 116 40 117 0 109 9 30 8 106 36 84 85 228 11 22 4 139 65 202 17 164 245 42 54 69 12 142 76 193 147 11 110 123 80 168 155 29 152 241 6 4 19 44 71 80 116 73 47 102 50 181 254 14 14 67 25 124 24 23 157 19 149 60 250 208 92 238 193 48 73 21 118 126 230 175 201 101 148 49 80 189 33 142 120 129 21 120 20 242 6 212 117 180 72 45 92 60 12 74 193 109 41 81 123 17 156 53 229 187 92 141 114 228 105 52 212 29 36 192 146 165 235 190 97 41 191 42 214 33 123 9 18 224 29 157 218 113 35 168 217 208 113 107 204 126 251 238 191 15 127 252 242 207 79 19 199 116 121 156 67 14 10 186 160 33 113 64 252 207 4 38 224 0 101 12 98 194 20 132 128 131 0 6 176 11 56 216 80 23 166 144 131 183 209 164 3 224 42 67 22 14 131 146 42 100 103 35 255 235 130 7 61 216 254 1 134 72 164 14 31 209 96 83 166 0 132 4 116 96 10 102 48 67 23 50 245 25 196 144 208 69 114 224 131 120 106 180 145 236 84 72 14 117 32 200 70 90 194 24 198 76 164 12 142 120 130 142 222 144 144 159 20 197 50 18 200 130 103 46 144 46 250 8 225 11 124 112 132 84 120 224 163 228 96 196 69 135 32 202 140 48 240 133 162 68 5 36 124 168 196 79 48 2 174 187 104 4 92 154 64 76 37 242 160 3 46 228 129 16 121 232 66 135 222 22 179 21 90 137 7 111 248 132 72 228 112 8 147 36 70 71 54 148 67 22 114 87 148 74 104 162 52 68 241 145 91 186 146 2 25 188 201 11 155 32 74 21 151 85 144 42 58 34 83 142 144 3 15 223 192 168 213 121 161 18 124 56 76 81 122 200 132 19 172 224 148 117 48 194 13 244 242 134 64 194 128 56 29 120 3 89 120 88 7 190 25 133 49 64 160 160 143 126 98 50 236 196 240 6 204 49 227 13 167 243 19 51 76 65 6 133 58 1 5 232 7 205 104 74 115 154 212 172 166 252 68 198 191 254 252 237 132 127 18 148 73 7 112 0 78 32 12 208 108 249 203 31 14 190 153 64 32 8 48 38 12 26 14 45 113 48 133 0 58 1 8 47 193 65 133 128 16 207 111 78 97 159 57 160 64 12 44 180 36 122 2 161 10 15 4 130 4 46 128 207 3 206 9 103 201 105 201 5 14 227 133 7 118 64 2 3 69 12 10 17 195 37 39 224 172 35 69 19 224 97 132 192 18 47 228 64 4 44 41 195 136 134 198 165 179 80 174 155 241 201 1 230 90 248 29 196 84 8 49 49 192 91 12 113 160 50 44 188 225 35 94 89 204 103 76 122 157 207 48 166 34 152 18 91 27 116 16 199 54 180 161 11 66 160 156 134 40 112 36 38 200 193 54 157 75 73 16 189 48 180 41 248 180 10 18 224 91 12 223 48 83 198 100 65 109 207 20 193 141 136 150 133 197 44 203 170 69 67 9 75 122 8 132 18 229 237 51 38 203 211 27 236 56 180 28 232 212 9 46 136 22 118 168 182 62 111 38 39 57 193 187 192 5 152 240 87 38 120 128 9 62 45 3 14 46 128 131 44 254 116 193 11 76 56 18 222 34 58 133 14 28 22 59 118 116 193 51 79 16 69 107 138 118 180 164 45 173 105 73 203 63 205 216 15 125 51 185 204 101 160 138 77 113 97 102 78 234 163 220 124 212 103 88 205 14 237 2 234 235 173 110 99 130 1 129 168 137 183 154 61 146 111 97 194 149 55 213 40 183 188 149 192 65 115 123 61 227 18 55 1 20 160 174 102 13 155 0 232 62 83 69 19 16 213 152 114 75 219 38 169 214 62 93 129 234 110 181 203 92 231 78 148 182 188 109 239 148 206 219 149 11 24 87 183 134 125 166 108 97 43 46 153 196 236 4 245 165 192 10 156 41 223 243 18 119 192 237 189 192 65 39 58 52 10 216 73 183 77 26 90 51 187 162 94 232 234 22 102 118 122 139 124 133 203 221 164 193 224 189 115 146 111 121 135 134 0 246 240 150 56 93 73 48 125 53 219 85 246 4 119 76 111 193 109 137 47 160 182 250 94 215 195 205 148 9 75 79 139 227 28 235 120 199 60 238 79 106 53 195 63 143 237 79 66 91 57 1 204 196 133 228 66 37 254 151 43 159 5 173 200 104 169 217 24 15 205 124 196 157 239 101 144 44 131 24 168 45 6 236 125 74 146 131 252 20 36 149 160 202 196 85 223 150 205 55 97 15 240 4 3 204 68 115 7 94 66 1 26 149 224 70 42 60 103 111 75 181 19 17 144 247 197 93 233 166 155 120 171 101 59 227 22 46 80 69 46 108 219 179 19 152 101 38 46 9 75 88 10 42 7 213 4 25 249 181 16 0 153 106 197 181 229 41 71 217 206 49 136 217 112 74 12 167 221 246 246 4 34 16 129 63 111 196 102 205 32 121 43 196 53 31 6 98 144 226 9 191 89 179 175 78 177 10 47 51 167 152 221 152 114 226 146 65 114 205 12 51 144 25 150 70 19 5 116 148 69 93 102 245 197 132 201 106 238 177 179 159 13 237 104 67 172 156 212 174 182 181 175 141 109 253 41 104 219 219 182 182 12 82 64 131 30 244 128 6 52 216 182 11 2 152 131 30 164 64 221 226 150 129 254 248 231 110 111 43 72 219 57 208 178 189 7 24 131 132 50 33 6 228 78 129 254 158 0 240 114 126 123 254 221 226 30 183 8 252 157 63 126 238 68 155 49 24 231 57 113 160 109 119 167 39 5 224 30 55 13 196 41 206 134 11 225 9 136 97 130 12 46 62 64 28 0 161 220 10 18 55 185 227 157 109 109 210 251 227 228 110 121 195 241 29 3 33 11 153 229 45 111 121 15 24 160 110 112 83 220 226 243 206 129 29 204 208 33 119 115 59 203 50 96 130 27 176 48 110 130 171 187 151 224 108 248 153 241 45 238 157 211 192 222 13 143 164 200 105 158 243 130 87 156 6 165 238 129 200 191 25 131 30 236 164 219 89 182 183 11 114 64 114 110 239 220 226 52 168 246 130 214 77 131 20 224 156 234 226 28 183 187 113 160 32 145 239 27 236 33 23 57 196 135 206 237 136 167 124 240 132 47 188 225 15 143 248 196 43 126 241 140 111 188 227 31 15 249 200 75 126 242 148 87 60 0 46 143 249 204 107 126 243 156 239 188 231 63 175 249 0 4 64 243 2 24 61 232 79 79 250 1 148 62 243 1 16 128 234 77 127 122 215 15 0 246 168 199 60 237 59 47 123 1 212 126 247 43 188 223 252 237 123 127 249 0 12 64 245 159 151 253 239 129 143 252 228 43 127 249 204 111 190 243 159 15 253 232 75 127 250 212 175 190 245 175 143 125 235 7 4 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
arrowBigBlueGif
	"'imgs/sun-ar_lg_blue_r.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 15 0 13 0 128 1 0 89 79 191 255 255 255 33 249 4 1 0 0 1 0 44 0 0 0 0 15 0 13 0 0 2 26 140 143 169 0 234 220 92 130 114 50 4 179 62 186 223 224 109 21 248 85 212 120 162 209 56 22 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
arrowBigRedGif
	"'imgs/sun-ar_lg_red_r.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 15 0 13 0 128 1 0 205 43 64 0 0 0 33 249 4 1 0 0 1 0 44 0 0 0 0 15 0 13 0 64 2 27 140 143 169 6 189 217 192 155 47 170 136 179 92 150 6 231 49 224 161 105 100 137 81 221 52 34 5 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
arrowBigYellowGif
	"'imgs/sun-ar_lg_yell_r_pad.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 24 0 21 0 128 1 0 251 226 73 0 0 0 33 249 4 1 0 0 1 0 44 0 0 0 0 24 0 21 0 64 2 37 140 143 169 123 224 15 29 155 180 34 105 233 203 188 251 186 101 209 8 124 1 102 166 234 202 133 171 171 145 81 10 119 53 139 231 74 1 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
arrowCircledGreyGif
	"'imgs/adobe-grey_arrow.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 10 0 10 0 145 0 0 102 102 102 255 255 255 153 153 153 204 204 204 33 249 4 0 0 0 0 0 44 0 0 0 0 10 0 10 0 0 2 29 76 38 0 50 22 169 162 56 241 69 84 237 138 192 65 238 41 159 1 49 213 215 124 146 131 45 77 80 0 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
arrowCircledRedGif
	"'imgs/adobe-red_arrow.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 10 0 10 0 162 0 0 255 0 0 255 255 255 255 153 155 255 204 204 254 242 243 0 0 0 0 0 0 0 0 0 33 249 4 0 0 0 0 0 44 0 0 0 0 10 0 10 0 0 3 33 24 49 2 32 131 168 230 170 32 163 6 10 152 222 14 7 40 96 229 144 20 55 133 217 201 13 165 41 144 222 3 7 9 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: 'mivsek 8/21/2007 11:24'!
arrowCircledRedLeftGif
	"'imgs/adobe-red_arrow-l.gif' asFilename contentsAsMethod"
^#(71 73 70 56 55 97 10 0 10 0 194 1 0 255 0 0 255 255 255 255 153 155 255 204 204 254 242 243 255 255 255 255 255 255 255 255 255 44 0 0 0 0 10 0 10 0 0 3 31 24 49 2 32 67 133 230 170 88 54 84 230 174 238 157 84 101 202 72 121 214 144 109 4 101 42 4 247 68 65 2 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
arrowDownBigRedGif
	"'imgs/sun-ar_lg_red_d_pad.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 24 0 21 0 128 1 0 204 51 51 0 0 0 33 249 4 1 0 0 1 0 44 0 0 0 0 24 0 21 0 64 2 39 140 143 169 203 224 12 163 92 14 76 90 235 221 188 67 221 105 226 19 146 222 137 94 32 186 158 173 106 5 160 41 101 182 119 167 113 202 115 5 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
arrowNextBigGif
	"'imgs/nav_arrow_right.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 20 0 19 0 213 0 0 237 237 237 245 245 245 242 242 242 234 234 234 246 246 246 253 253 253 235 235 235 251 251 251 249 249 249 254 254 254 241 241 241 236 236 236 247 247 247 239 239 239 248 248 248 250 250 250 228 228 228 229 229 229 224 224 224 243 243 243 222 222 222 252 252 252 233 233 233 217 217 217 225 225 225 219 219 219 237 237 238 233 234 234 225 225 224 218 218 218 227 227 227 226 226 226 230 231 231 221 222 222 232 232 231 231 231 231 237 238 238 229 229 228 234 234 233 249 249 248 220 219 220 234 234 235 240 241 241 251 250 250 220 220 221 246 247 247 221 222 221 227 226 227 221 221 221 239 238 238 227 227 228 241 241 242 230 230 230 245 245 246 232 232 232 238 238 238 244 244 244 240 240 240 186 186 186 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 9 0 0 59 0 44 0 0 0 0 20 0 19 0 0 8 230 0 119 232 208 145 99 135 193 131 8 19 10 12 128 99 96 65 133 16 117 48 192 193 208 33 196 132 58 90 40 192 193 177 33 193 139 6 117 4 184 33 128 0 2 134 30 31 42 212 49 97 65 14 28 39 30 20 72 192 209 34 70 1 27 110 204 32 240 160 66 2 6 52 83 34 212 161 0 132 1 5 1 28 28 40 192 180 2 195 0 54 117 52 24 49 128 100 210 7 43 14 32 96 64 128 35 84 130 58 110 120 176 1 32 199 132 0 4 184 214 192 33 32 199 13 142 5 6 222 248 64 195 4 128 27 57 114 40 80 113 163 129 6 3 21 193 146 144 16 65 132 133 1 41 6 40 30 96 161 230 71 129 0 36 188 136 80 34 2 4 8 50 32 8 16 96 51 36 0 10 24 48 112 144 224 2 70 140 6 157 15 234 0 16 130 2 11 20 29 22 24 72 61 20 64 134 12 23 12 12 160 141 113 193 133 5 95 85 130 20 57 251 49 200 161 198 143 31 12 8 0 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
arrowNextRedPng
	"'imgs/suse/suse-arrow-next.png' asFilename contentsAsMethod"
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 15 0 0 0 14 8 3 0 0 0 199 84 182 221 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 1 23 80 76 84 69 117 160 37 139 191 44 143 197 45 114 156 36 146 200 46 147 202 46 154 209 53 115 158 36 143 196 45 120 165 38 105 145 33 103 141 47 101 91 89 77 108 90 137 192 75 7 9 5 50 53 62 195 184 201 77 83 74 100 105 78 152 165 184 73 75 74 2 10 0 30 50 39 154 184 174 104 143 56 20 27 35 73 81 92 94 87 78 116 158 53 102 140 43 159 165 165 96 127 54 157 163 179 132 149 165 56 53 46 138 195 89 134 193 101 166 189 205 151 207 47 105 135 187 24 19 13 17 13 27 131 159 144 107 130 148 9 20 24 140 156 171 26 28 40 112 146 113 110 149 144 111 152 35 152 208 50 17 24 26 11 15 26 21 21 49 87 94 87 105 143 60 31 26 20 17 4 33 154 173 171 68 64 65 136 187 43 32 40 27 5 16 4 59 53 53 108 97 91 171 175 213 145 199 46 82 113 63 120 146 145 152 208 48 10 14 12 111 153 35 21 21 29 137 189 43 70 81 67 84 86 88 154 155 157 134 184 42 32 31 47 80 78 79 156 167 185 73 72 80 108 149 58 119 163 38 6 6 7 81 85 70 127 127 119 115 145 173 137 135 122 100 102 99 9 11 27 255 255 255 159 1 143 8 0 0 0 93 116 82 78 83 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 225 175 157 144 0 0 0 193 73 68 65 84 120 218 98 136 129 130 112 8 5 16 64 12 48 190 185 56 152 2 8 32 56 95 148 207 10 68 1 4 16 156 47 160 202 107 15 164 0 2 136 33 210 49 192 91 58 204 70 217 95 133 73 206 61 38 6 32 128 24 132 89 57 212 253 56 24 153 88 25 217 217 184 45 99 0 2 136 65 136 153 141 193 153 147 133 193 150 203 152 221 75 86 6 32 128 24 124 20 24 140 56 25 24 60 152 185 24 88 44 76 20 1 2 136 33 202 206 76 95 83 215 211 52 58 132 81 146 223 80 16 32 128 24 98 212 68 172 229 37 2 245 164 220 130 67 117 180 99 0 2 8 110 159 131 139 152 134 107 76 12 64 0 193 249 65 90 17 74 64 10 32 128 224 124 30 95 3 16 5 16 64 112 126 140 19 152 4 8 48 0 50 218 53 59 137 15 17 99 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
arrowOrangeDownGif
	"'imgs/dars-arrow-down.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 8 0 5 0 161 2 0 251 171 61 50 138 66 255 255 255 255 255 255 33 249 4 1 0 0 2 0 44 0 0 0 0 8 0 5 0 0 2 12 140 127 160 27 18 11 154 112 74 78 103 11 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
arrowOrangeLeftGif
	"'imgs/dars-arrow-left.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 5 0 8 0 161 2 0 251 171 61 50 138 66 255 255 255 255 255 255 33 249 4 1 0 0 2 0 44 0 0 0 0 5 0 8 0 0 2 13 148 19 7 25 176 237 144 91 2 166 74 79 40 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
arrowOrangeRightGif
	"'imgs/dars-arrow2.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 5 0 8 0 145 2 0 251 171 61 50 138 66 255 255 255 0 0 0 33 249 4 1 0 0 2 0 44 0 0 0 0 5 0 8 0 0 2 14 76 132 1 39 186 13 162 137 203 29 197 12 18 5 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
arrowOrangeUpGif
	"'imgs/dars-arrow-up.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 8 0 5 0 161 2 0 251 171 61 50 138 66 255 255 255 255 255 255 33 249 4 1 0 0 2 0 44 0 0 0 0 8 0 5 0 0 2 12 148 19 166 1 144 215 90 98 81 162 123 11 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
arrowPreviousBigGif
	"'imgs/nav_arrow_left.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 20 0 19 0 213 0 0 237 237 237 245 245 245 242 242 242 234 234 234 246 246 246 253 253 253 235 235 235 251 251 251 249 249 249 254 254 254 241 241 241 236 236 236 247 247 247 239 239 239 248 248 248 250 250 250 228 228 228 229 229 229 224 224 224 243 243 243 222 222 222 252 252 252 233 233 233 217 217 217 225 225 225 219 219 219 237 237 238 233 234 234 225 225 224 218 218 218 227 227 227 226 226 226 230 231 231 221 222 222 232 232 231 231 231 231 237 238 238 229 229 228 234 234 233 249 249 248 220 219 220 234 234 235 240 241 241 251 250 250 220 220 221 246 247 247 221 222 221 227 226 227 221 221 221 239 238 238 227 227 228 241 241 242 230 230 230 245 245 246 232 232 232 238 238 238 244 244 244 240 240 240 186 186 186 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 9 0 0 59 0 44 0 0 0 0 20 0 19 0 0 8 222 0 119 8 28 72 176 224 142 28 58 116 24 92 40 16 161 14 28 56 20 50 28 232 48 0 68 6 18 25 86 132 8 177 69 198 130 21 45 34 192 33 224 70 128 143 13 19 114 12 240 224 4 142 28 11 38 124 12 25 32 64 130 10 15 8 204 184 177 65 128 68 135 12 32 86 40 64 244 128 131 0 10 12 128 80 160 99 227 69 4 7 86 60 56 90 114 192 136 6 9 45 114 20 128 163 6 1 6 4 2 76 200 1 192 134 135 27 77 85 6 48 160 161 193 13 21 10 114 228 184 1 192 4 141 15 104 83 62 196 97 97 128 223 1 41 6 88 16 17 65 2 137 153 9 5 8 128 32 3 2 132 8 37 34 188 144 0 0 229 193 132 13 98 192 112 33 129 3 6 12 20 42 107 76 104 96 65 7 20 44 40 132 16 61 209 225 0 3 23 50 100 96 61 241 178 142 0 11 46 44 176 60 90 135 129 147 181 13 58 220 17 16 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
arrowPreviousRedPng
	"'imgs/suse/suse-arrow-prev.png' asFilename contentsAsMethod"
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 14 8 3 0 0 0 17 241 108 184 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 1 50 80 76 84 69 114 156 36 143 196 45 109 150 35 115 158 36 146 200 46 151 207 47 117 160 37 119 163 38 152 208 50 27 36 39 1 11 0 86 78 67 33 32 64 19 30 16 0 7 30 74 100 46 18 26 15 39 45 41 104 142 33 71 67 68 113 154 45 17 16 21 106 177 120 13 18 14 104 173 115 78 76 63 15 24 39 162 173 179 113 155 36 98 135 31 137 193 67 154 173 179 30 43 26 162 169 179 162 167 160 153 170 180 42 59 51 75 94 99 130 144 147 51 56 38 8 11 0 131 134 175 164 167 182 126 151 158 156 150 184 110 151 46 39 47 49 155 172 180 202 203 207 14 19 22 110 125 168 14 13 8 153 155 170 19 23 60 102 139 68 1 1 11 96 99 92 34 39 45 94 95 81 126 188 91 17 21 10 79 83 74 147 202 46 20 27 35 162 177 182 149 204 47 15 12 36 167 174 192 81 72 75 134 184 42 131 135 121 120 165 38 88 84 72 124 170 39 148 204 61 127 144 162 87 78 71 4 0 19 7 5 18 100 137 32 161 170 177 85 75 61 141 159 173 71 79 54 154 209 52 140 198 71 155 169 178 95 128 81 158 211 60 187 196 175 11 24 15 5 6 11 160 169 168 95 132 50 18 16 27 151 205 83 143 197 45 137 189 43 132 181 42 127 149 137 176 181 174 255 255 255 56 147 122 14 0 0 0 102 116 82 78 83 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 53 142 168 183 0 0 0 202 73 68 65 84 120 218 98 72 5 3 183 84 24 0 8 32 6 48 169 96 5 23 0 8 32 176 128 176 88 32 92 0 32 128 64 2 190 114 33 42 112 1 128 0 2 10 112 74 120 50 241 168 7 219 74 122 184 248 88 164 2 4 16 67 170 120 104 18 51 171 127 132 35 99 2 99 188 151 96 10 64 0 49 112 91 187 50 176 8 217 201 176 186 51 50 115 196 74 197 0 4 16 67 152 13 59 11 3 7 131 44 19 27 3 59 27 191 169 22 64 0 49 164 106 251 233 38 138 56 105 24 71 9 24 198 217 235 41 1 4 16 208 80 111 46 179 112 62 181 32 101 125 105 7 121 19 3 128 0 2 89 107 100 174 42 170 3 179 22 32 128 192 14 211 140 230 13 128 9 0 4 16 196 233 201 150 138 48 1 128 0 130 8 164 58 71 194 4 0 2 12 0 195 211 63 171 113 236 31 0 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
arrowSmallBlackGif
	"'imgs/ibm-arrow-right-black.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 7 0 7 0 128 1 0 0 0 0 0 0 0 33 249 4 1 0 0 1 0 44 0 0 0 0 7 0 7 0 0 2 11 140 143 1 105 11 15 23 146 135 178 2 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
arrowTriangleRedGif
	"'imgs/filo-redarr.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 14 0 13 0 128 0 0 255 255 255 255 0 0 33 249 4 1 0 0 0 0 44 0 0 0 0 14 0 13 0 0 2 23 132 143 169 123 193 25 226 51 177 190 138 23 222 112 59 197 49 214 244 77 230 121 20 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
arrowTriangleYellowGif
	"'imgs/dars-arrow2.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 5 0 8 0 145 2 0 251 171 61 50 138 66 255 255 255 0 0 0 33 249 4 1 0 0 2 0 44 0 0 0 0 5 0 8 0 0 2 14 76 132 1 39 186 13 162 137 203 29 197 12 18 5 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
bannerLeftGif
	"'imgs/fast/fast-banner_left.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 3 0 3 0 128 0 0 127 160 177 255 255 255 33 249 4 1 0 0 0 0 44 0 0 0 0 3 0 3 0 0 2 4 12 96 24 81 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
bucketTopGif
	"'imgs/fast/fast-bucket_top.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 80 1 5 0 128 1 0 255 255 255 229 236 243 33 249 4 1 0 0 1 0 44 0 0 0 0 80 1 5 0 0 2 50 132 17 169 203 237 15 163 156 180 218 139 179 222 188 251 15 42 199 17 150 230 137 166 234 202 182 37 233 198 242 76 215 246 141 25 248 206 247 254 15 100 32 130 196 162 241 136 172 20 0 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletBookTranspWhiteGif
	"'imgs/fast/fast-book.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 153 204 102 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 29 68 142 25 166 141 0 95 148 17 206 99 77 78 123 83 109 113 161 135 141 225 183 156 138 230 180 143 139 21 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletBubbleTranspWhiteGif
	"'imgs/fast/fast-soundoff.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 0 152 248 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 26 68 142 25 166 157 0 35 67 178 190 26 47 158 107 243 110 53 153 115 64 36 117 166 165 10 20 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletCalendarTranspWhiteGif
	"'imgs/fast/fast-calendar.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 251 176 51 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 29 68 142 25 166 141 0 99 76 178 210 58 159 62 140 239 117 97 29 232 149 102 55 138 144 195 142 14 80 0 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletCheckTranspWhiteGif
	"'imgs/fast/fast-register.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 0 0 0 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 22 68 142 25 166 237 15 89 148 135 2 90 217 125 119 79 8 134 34 130 53 64 1 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletConnectTranspWhiteGif
	"'imgs/fast/fast-connect.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 0 0 0 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 23 68 142 25 166 237 15 29 91 114 130 121 174 222 234 34 223 88 209 72 70 152 84 0 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletFindTranspWhiteGif
	"'imgs/fast/fast-find.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 0 0 0 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 24 68 142 25 166 157 0 89 139 209 209 101 171 124 114 243 234 40 94 56 90 161 121 46 5 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletFolderTranspWhiteGif
	"'imgs/fast/fast-archives.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 153 204 102 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 24 68 142 25 166 237 127 128 132 178 218 100 51 206 117 115 22 125 224 34 66 38 117 2 5 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletGraphTranspWhiteGif
	"'imgs/fast/fast-bargraph.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 0 0 0 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 27 68 142 25 166 221 192 220 129 18 209 186 204 125 58 42 186 77 93 246 141 33 9 122 216 180 2 5 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletGuidesTranspWhiteGif
	"'imgs/fast/fast-guides.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 0 0 0 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 25 68 142 25 166 237 236 14 152 17 193 198 178 154 188 111 9 70 215 67 85 139 89 141 88 1 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletIssueTranspWhiteGif
	"'imgs/fast/fast-issue.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 0 84 166 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 26 68 142 25 166 141 0 99 76 178 82 246 240 170 51 43 120 113 26 120 133 156 105 57 142 182 22 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletLetterTranspWhiteGif
	"'imgs/fast/fast-newsletters.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 0 0 0 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 29 68 142 25 166 237 127 128 156 12 25 122 147 92 123 219 206 85 225 53 105 88 249 157 34 196 42 107 3 20 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletManTranspWhiteGif
	"'imgs/fast/fast-cof.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 0 0 0 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 22 68 142 25 166 221 0 29 130 76 82 137 21 141 184 246 124 120 22 56 102 64 1 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletNoteTranspWhiteGif
	"'imgs/fast/fast-riskfree.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 0 0 0 168 236 255 255 255 33 249 4 1 0 0 0 0 44 0 0 0 0 13 0 13 0 0 2 23 12 142 9 166 221 1 29 130 76 82 121 46 214 54 226 63 121 93 5 38 164 19 20 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletPencilTranspWhiteGif
	"'imgs/fast/fast-pencil.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 153 102 102 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 26 68 142 25 166 173 236 142 129 13 210 135 110 178 113 229 62 1 90 54 117 139 104 98 38 80 0 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletQuoteTranspWhiteGif
	"'imgs/fast/fast-first_imp.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 1 113 165 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 26 68 142 25 166 237 191 128 140 19 201 121 217 201 84 247 111 37 158 72 58 222 8 165 10 186 22 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletRelatedTranspWhiteGif
	"'imgs/fast/fast-related.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 0 84 166 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 26 68 142 25 166 157 0 153 139 70 190 232 14 197 106 195 230 89 154 7 146 157 153 129 169 86 0 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletSquareDownTranspWhiteGif
	"'imgs/fast/fast-down_arrow.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 0 84 166 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 23 68 142 25 166 237 159 128 132 107 66 201 30 214 120 183 158 125 214 69 145 20 80 0 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletSquareLeftWhiteGif
	"'imgs/fast/fast-left_bull.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 11 0 11 0 162 2 0 242 242 236 166 165 165 245 245 236 232 232 222 196 196 181 255 255 255 0 0 0 0 0 0 33 249 4 1 0 0 2 0 44 0 0 0 0 11 0 11 0 0 3 38 40 50 220 163 171 200 89 72 164 148 12 26 242 158 65 39 105 82 104 142 95 41 2 36 56 177 41 5 176 2 33 207 150 66 236 124 158 0 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletSquareRightTranspWhiteGif
	"'imgs/fast/fast-arrow.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 0 152 248 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 22 68 142 25 166 237 223 24 92 50 129 139 235 201 89 241 59 129 225 68 142 97 1 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletStarTranspWhiteGif
	"'imgs/fast/fast-fast50.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 0 0 0 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 23 68 142 25 166 237 175 24 4 212 209 139 35 174 109 191 202 69 34 68 78 37 80 0 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletToolsTranspWhiteGif
	"'imgs/fast/fast-tools_ico.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 0 0 0 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 24 68 142 25 166 157 0 87 108 176 58 52 111 190 146 119 255 105 140 55 130 74 233 0 5 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
bulletTriangleRightTranspWhiteGif
	"'imgs/fast/fast-interactive.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 128 1 0 255 255 255 0 0 0 33 249 4 1 0 0 1 0 44 0 0 0 0 13 0 13 0 0 2 24 68 142 25 166 173 236 16 128 113 210 102 111 202 250 228 184 76 96 56 46 229 216 109 5 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-buttons' stamp: ' 21/4/07 10:33'!
buttonBackWhiteGif
	"'imgs/awful-button-back.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 30 0 22 0 230 0 0 229 229 229 232 232 232 239 239 239 207 207 207 143 143 143 223 223 223 64 64 64 252 252 252 226 226 226 228 228 228 159 159 159 140 140 140 212 212 212 221 221 221 254 254 254 214 214 214 216 216 216 199 199 199 0 0 0 16 16 16 189 189 189 175 175 175 253 253 253 225 225 225 128 128 128 222 222 222 201 201 201 148 148 148 172 172 172 209 209 209 191 191 191 244 244 244 240 240 240 151 151 151 48 48 48 235 235 235 179 179 179 177 177 177 180 180 180 186 186 186 205 205 205 163 163 163 142 142 142 247 247 247 218 218 218 156 156 156 80 80 80 174 174 174 194 194 194 224 224 224 155 155 155 166 166 166 150 150 150 145 145 145 202 202 202 96 96 96 220 220 220 203 203 203 242 242 242 211 211 211 195 195 195 165 165 165 137 137 137 215 215 215 249 249 249 141 141 141 164 164 164 250 250 250 130 130 130 231 231 231 204 204 204 112 112 112 32 32 32 208 208 208 152 152 152 248 248 248 135 135 135 134 134 134 178 178 178 146 146 146 246 246 246 219 219 219 173 173 173 176 176 176 251 251 251 132 132 132 138 138 138 217 217 217 236 236 236 241 241 241 181 181 181 147 147 147 133 133 133 153 153 153 158 158 158 245 245 245 183 183 183 136 136 136 210 210 210 144 144 144 230 230 230 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 0 0 0 0 0 44 0 0 0 0 30 0 22 0 0 7 255 128 100 130 131 132 133 134 135 136 25 17 21 10 27 99 143 144 99 53 74 51 20 12 8 136 130 23 17 33 27 48 5 101 161 162 162 31 12 10 11 47 16 0 135 13 36 11 8 163 177 178 101 38 79 3 9 133 49 78 33 84 179 190 162 81 4 3 171 130 0 60 11 191 201 161 12 93 44 131 16 42 32 202 161 30 179 22 50 39 196 20 41 202 2 24 19 6 178 7 75 35 99 13 100 9 45 176 190 3 55 18 72 24 160 177 7 31 89 11 54 100 13 4 190 21 6 18 6 10 126 29 0 49 66 8 24 50 15 246 201 114 49 225 136 60 129 2 0 76 225 128 80 97 44 12 18 68 40 16 144 108 8 150 11 37 40 234 19 229 96 148 183 9 19 110 60 140 5 133 12 14 25 90 208 169 11 85 82 150 2 17 255 42 204 19 144 225 7 23 13 130 182 89 24 90 83 150 7 23 225 72 126 72 240 32 71 16 28 130 32 44 16 176 98 5 144 162 178 56 138 250 2 224 65 135 48 217 138 193 176 162 67 0 8 29 211 66 93 96 128 66 10 13 103 131 80 98 148 80 81 132 12 128 0 120 243 234 13 0 160 0 132 36 57 76 248 24 86 168 85 19 13 5 174 60 216 33 166 177 99 49 29 6 160 48 162 97 139 48 92 134 54 209 96 50 195 3 227 199 141 59 216 224 80 133 72 42 98 137 22 121 113 20 233 81 141 16 61 44 97 202 68 187 118 161 64 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-buttons' stamp: ' 21/4/07 10:33'!
buttonBeginWhiteGif
	"'imgs/awful-button-begin.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 22 0 22 0 230 0 0 239 239 239 143 143 143 207 207 207 159 159 159 223 223 223 128 128 128 221 221 221 254 254 254 212 212 212 228 228 228 214 214 214 191 191 191 226 226 226 32 32 32 140 140 140 216 216 216 225 225 225 252 252 252 253 253 253 199 199 199 189 189 189 64 64 64 222 222 222 201 201 201 16 16 16 232 232 232 172 172 172 96 96 96 209 209 209 151 151 151 240 240 240 175 175 175 244 244 244 156 156 156 141 141 141 242 242 242 247 247 247 235 235 235 174 174 174 163 163 163 155 155 155 194 194 194 220 220 220 180 180 180 142 142 142 166 166 166 224 224 224 202 202 202 150 150 150 218 218 218 205 205 205 177 177 177 179 179 179 186 186 186 203 203 203 145 145 145 219 219 219 138 138 138 144 144 144 173 173 173 135 135 135 134 134 134 0 0 0 248 248 248 246 246 246 195 195 195 217 217 217 80 80 80 165 165 165 236 236 236 133 133 133 215 215 215 181 181 181 250 250 250 176 176 176 146 146 146 158 158 158 152 152 152 153 153 153 249 249 249 245 245 245 251 251 251 164 164 164 231 231 231 183 183 183 132 132 132 130 130 130 208 208 208 204 204 204 147 147 147 241 241 241 137 137 137 178 178 178 136 136 136 148 148 148 229 229 229 255 255 255 230 230 230 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 0 0 0 0 0 44 0 0 0 0 22 0 22 0 0 7 255 128 97 130 131 132 133 134 135 22 19 31 3 55 77 45 20 8 12 135 131 16 19 29 94 41 4 96 32 8 3 14 38 15 95 135 6 52 14 12 96 168 169 168 43 75 2 9 133 46 92 29 81 170 181 96 56 1 2 162 130 95 65 14 182 192 8 78 49 131 15 44 30 192 182 18 40 53 187 20 39 181 11 182 11 17 63 37 58 6 97 9 33 167 168 11 21 24 170 3 13 67 17 32 90 14 47 97 6 1 168 4 27 13 3 0 222 21 21 31 96 17 30 37 82 84 97 10 237 2 48 12 80 21 160 129 128 84 17 0 124 81 162 193 95 59 48 227 54 104 2 3 160 64 131 2 243 146 20 129 48 163 33 187 84 7 10 96 168 144 10 192 6 31 5 128 132 81 129 2 137 54 110 168 14 80 44 80 139 128 8 0 22 142 24 185 32 232 153 132 159 50 109 29 0 145 64 129 13 17 42 4 61 112 0 128 4 137 39 65 85 65 249 162 128 67 151 102 188 82 228 24 1 192 195 8 91 16 16 200 216 1 131 216 32 23 51 88 76 9 243 37 131 219 47 4 52 30 92 177 177 98 139 174 66 164 122 92 32 32 164 170 0 25 88 46 100 201 245 202 80 37 24 60 90 44 224 240 66 67 21 43 160 118 77 74 244 129 201 141 14 68 32 73 154 196 185 115 32 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-buttons' stamp: 'mivsek 8/21/2007 11:30'!
buttonCloseGif
	" '/home/mivsek/vw7.2/image/imgs/yuiForum/tab-close.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 13 0 135 0 0 77 87 90 115 116 115 64 255 64 133 143 143 138 145 144 175 181 179 158 190 245 192 218 249 198 221 250 204 225 250 211 229 251 217 232 252 223 236 252 228 230 224 230 240 253 236 244 253 242 247 254 249 251 254 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 3 0 0 2 0 44 0 0 0 0 13 0 13 0 0 8 105 0 5 24 24 72 176 160 1 129 13 34 40 92 184 176 193 64 8 16 35 74 132 48 240 193 131 0 5 44 62 40 48 192 226 64 7 14 10 0 40 16 114 36 200 129 12 82 138 36 48 50 37 131 129 11 98 46 96 73 64 230 130 129 10 114 174 28 153 83 193 192 4 9 68 22 8 58 18 232 64 4 8 6 20 64 138 128 35 210 129 7 162 74 157 122 0 42 85 170 7 13 106 21 16 16 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-buttons' stamp: 'mivsek 8/21/2007 11:30'!
buttonCommentPng
	"'imgs/wikimatrix-comment.png' asFilename contentsAsMethod"
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 1 166 73 68 65 84 120 218 98 252 255 255 63 3 37 0 32 128 152 24 40 4 0 1 196 2 99 68 118 236 20 2 82 147 129 56 4 136 217 240 232 217 13 196 89 203 43 220 239 128 56 0 1 196 0 242 2 8 71 180 239 216 184 242 224 173 255 191 255 252 253 143 15 108 57 121 255 63 80 237 109 32 102 3 233 3 8 32 22 168 237 60 64 202 59 196 86 133 225 243 15 6 134 159 127 254 51 252 5 6 205 191 127 12 96 250 47 136 254 247 159 129 17 164 200 76 129 225 248 245 23 42 119 159 127 52 0 114 79 1 4 16 204 11 66 34 124 28 204 76 140 140 12 63 126 255 103 248 247 31 73 35 216 160 255 12 127 254 65 12 4 1 62 46 176 15 69 65 4 64 0 177 160 123 16 166 241 31 146 205 8 87 96 6 8 64 0 193 98 225 221 155 79 63 128 106 255 51 176 48 65 20 254 1 219 10 177 25 134 25 24 32 81 254 229 199 111 176 30 16 1 16 64 96 3 128 33 250 5 72 109 93 115 248 14 131 8 48 52 20 132 25 25 148 69 152 24 212 196 152 24 120 216 25 25 254 254 101 96 144 228 99 100 208 147 102 102 216 122 234 1 195 237 167 31 64 49 112 1 164 23 32 128 24 97 9 9 41 26 195 129 152 25 36 230 106 40 203 144 228 174 5 150 255 240 229 39 195 186 163 119 25 118 159 127 124 5 200 13 132 69 35 64 0 193 163 17 27 6 70 213 157 231 239 190 254 159 187 227 42 40 234 190 3 113 45 186 26 128 0 98 196 149 148 129 46 234 117 212 151 41 218 127 241 201 87 32 183 22 104 99 63 54 117 0 1 132 213 0 160 102 37 32 181 7 136 219 128 26 231 224 75 202 0 1 196 72 105 102 2 8 32 138 51 19 64 0 81 108 0 64 128 1 0 173 158 253 27 90 6 162 174 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!DefaultWebStyle methodsFor: 'imgs-buttons' stamp: ' 21/4/07 10:33'!
buttonEditGif
	"'/home/mivsek/projekti/viart/edit.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 20 0 20 0 247 0 0 0 0 0 255 255 255 0 0 8 0 12 35 5 25 57 9 23 45 0 9 22 21 35 54 27 45 68 27 42 61 37 56 77 37 52 68 44 61 80 40 61 82 38 56 75 52 73 95 53 74 95 59 79 99 33 44 55 84 110 136 58 79 99 60 82 102 61 82 101 67 89 110 46 60 73 64 83 101 74 95 114 31 44 55 36 47 56 83 107 128 81 103 121 63 80 94 80 101 119 117 143 164 62 75 86 30 39 46 35 45 53 88 112 131 81 103 120 96 121 140 99 124 143 92 115 132 50 62 71 97 120 137 105 129 147 70 85 96 60 78 91 114 143 164 88 111 127 99 124 141 105 131 149 104 128 145 97 119 135 95 117 132 118 145 163 110 134 150 107 128 142 105 131 148 56 70 79 45 56 63 116 144 161 60 74 83 112 138 154 62 76 85 79 97 108 108 132 147 125 152 169 101 122 135 118 142 157 65 80 89 111 136 150 121 147 162 139 168 184 36 43 47 124 146 158 114 139 152 60 73 80 120 146 159 126 152 166 93 112 122 155 184 199 129 153 165 63 76 82 142 169 181 131 156 167 93 111 119 131 155 166 152 178 190 142 167 178 145 170 181 143 171 183 139 166 177 164 194 207 145 172 183 149 176 187 146 172 183 147 172 182 177 203 214 149 176 186 135 158 166 149 172 180 108 124 130 148 174 183 158 184 193 162 188 197 157 183 191 177 205 214 147 174 182 172 201 209 168 196 204 161 188 196 173 201 209 176 204 212 144 166 172 163 190 197 185 215 223 168 195 202 193 223 231 178 206 213 52 60 62 171 199 205 161 187 193 208 240 247 96 111 114 181 208 214 183 212 217 193 222 227 145 166 170 177 203 207 183 209 212 207 235 238 200 229 232 156 179 181 216 244 246 162 185 186 37 39 38 41 46 42 83 88 82 48 50 39 110 111 95 97 97 76 72 71 53 255 219 13 95 87 53 255 204 11 255 201 12 255 203 13 255 195 11 255 197 11 255 199 11 255 195 12 255 197 12 255 199 12 107 78 0 218 159 1 184 134 1 255 191 4 239 175 4 228 167 4 241 178 5 230 169 5 255 189 10 255 192 10 255 193 10 125 93 5 109 81 5 255 190 12 255 191 12 255 192 12 255 193 12 175 131 10 125 94 7 120 90 8 130 99 11 116 92 23 61 51 23 211 152 0 148 105 0 131 94 0 123 87 0 227 165 1 209 152 1 190 138 1 227 166 4 226 165 4 230 168 5 255 188 10 160 118 7 119 90 13 143 100 0 116 79 0 111 78 0 110 76 0 63 44 0 36 24 0 41 39 35 41 23 0 7 3 0 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 1 0 0 198 0 44 0 0 0 0 20 0 20 0 0 8 255 0 141 9 28 72 144 32 160 130 8 19 254 249 226 199 80 194 135 198 198 24 105 242 4 98 66 47 65 106 88 36 56 40 80 158 37 102 86 108 52 70 136 75 4 49 103 168 240 24 233 36 1 177 98 100 220 100 1 242 80 24 169 85 172 126 101 226 181 8 75 154 27 16 27 113 146 52 169 210 167 67 97 206 200 49 17 99 8 66 52 35 0 64 50 149 202 211 48 61 133 96 60 0 65 3 7 193 41 107 230 32 128 101 9 213 45 69 80 250 192 73 1 193 194 12 21 2 219 68 17 244 102 193 46 93 169 64 33 154 16 34 209 43 7 21 52 164 64 98 108 79 14 58 124 172 144 176 117 41 84 43 70 143 84 105 114 165 160 129 7 34 27 140 233 240 129 167 206 157 94 167 80 137 170 133 139 146 163 96 2 118 108 65 49 176 8 11 59 113 70 200 138 148 10 211 168 77 177 102 1 51 240 162 96 143 11 106 202 248 234 148 139 86 41 2 17 50 48 72 146 240 71 137 42 18 10 12 56 128 65 137 16 23 45 32 138 144 113 133 77 23 45 54 62 112 14 24 201 228 4 152 35 20 70 18 148 210 97 99 64 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-buttons' stamp: ' 21/4/07 10:33'!
buttonEndWhiteGif
	"'imgs/awful-button-end.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 22 0 22 0 230 0 0 143 143 143 254 254 254 207 207 207 239 239 239 253 253 253 223 223 223 159 159 159 226 226 226 128 128 128 221 221 221 191 191 191 228 228 228 216 216 216 212 212 212 214 214 214 32 32 32 140 140 140 64 64 64 189 189 189 199 199 199 251 251 251 96 96 96 16 16 16 201 201 201 175 175 175 151 151 151 209 209 209 232 232 232 172 172 172 247 247 247 225 225 225 177 177 177 145 145 145 156 156 156 166 166 166 179 179 179 224 224 224 203 203 203 163 163 163 235 235 235 222 222 222 186 186 186 180 180 180 250 250 250 155 155 155 150 150 150 142 142 142 248 248 248 249 249 249 242 242 242 240 240 240 220 220 220 141 141 141 202 202 202 174 174 174 194 194 194 205 205 205 218 218 218 231 231 231 134 134 134 217 217 217 130 130 130 173 173 173 195 195 195 148 148 148 153 153 153 204 204 204 181 181 181 183 183 183 219 219 219 138 138 138 215 215 215 236 236 236 152 152 152 246 246 246 252 252 252 132 132 132 147 147 147 133 133 133 241 241 241 144 144 144 244 244 244 176 176 176 136 136 136 80 80 80 178 178 178 146 146 146 164 164 164 0 0 0 245 245 245 165 165 165 137 137 137 158 158 158 135 135 135 208 208 208 229 229 229 255 255 255 230 230 230 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 0 0 0 0 0 44 0 0 0 0 22 0 22 0 0 7 255 128 97 130 131 132 133 134 135 40 19 24 6 32 73 34 18 13 7 135 131 30 19 25 64 55 5 96 81 13 6 16 54 12 95 135 9 35 16 7 96 168 169 168 42 86 2 11 133 36 85 25 20 170 181 96 69 0 2 162 130 95 63 16 182 192 13 65 57 131 12 46 50 192 182 4 44 41 187 18 38 96 4 10 182 211 168 1 47 39 80 9 97 11 33 167 4 84 15 6 170 22 17 213 29 79 16 53 97 9 0 168 4 96 24 17 230 168 3 6 15 21 5 1 72 87 68 97 14 238 192 44 73 37 224 65 64 84 6 44 8 56 33 133 195 63 119 20 86 128 25 128 224 1 130 1 168 10 84 16 7 70 199 7 135 237 80 41 65 128 165 2 70 84 17 44 32 8 0 6 198 12 22 67 184 121 3 243 130 134 38 85 23 193 4 32 112 224 136 147 11 130 158 17 32 64 129 165 173 0 43 22 56 40 65 99 134 32 6 16 6 116 232 0 195 168 170 44 95 28 104 152 210 140 215 13 35 49 6 200 136 97 203 67 3 28 62 90 16 27 68 226 131 11 29 97 58 190 108 152 251 165 0 3 47 37 84 108 209 85 136 212 142 11 5 120 104 21 128 67 200 133 38 185 94 25 170 212 162 139 8 5 26 106 112 96 210 3 212 174 73 137 48 112 1 145 65 11 36 73 147 66 139 14 4 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-buttons' stamp: ' 21/4/07 10:33'!
buttonNextWhiteGif
	"'imgs/awful-button-next.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 30 0 22 0 230 0 0 229 229 229 232 232 232 239 239 239 207 207 207 143 143 143 223 223 223 64 64 64 252 252 252 226 226 226 228 228 228 159 159 159 140 140 140 212 212 212 221 221 221 214 214 214 216 216 216 0 0 0 16 16 16 225 225 225 189 189 189 253 253 253 128 128 128 254 254 254 199 199 199 175 175 175 222 222 222 201 201 201 191 191 191 148 148 148 172 172 172 209 209 209 240 240 240 244 244 244 151 151 151 48 48 48 177 177 177 179 179 179 145 145 145 174 174 174 194 194 194 163 163 163 180 180 180 247 247 247 156 156 156 220 220 220 166 166 166 203 203 203 142 142 142 218 218 218 155 155 155 80 80 80 235 235 235 224 224 224 186 186 186 205 205 205 96 96 96 150 150 150 202 202 202 242 242 242 211 211 211 195 195 195 217 217 217 137 137 137 178 178 178 181 181 181 138 138 138 246 246 246 133 133 133 146 146 146 134 134 134 153 153 153 32 32 32 236 236 236 250 250 250 219 219 219 251 251 251 112 112 112 241 241 241 208 208 208 173 173 173 147 147 147 158 158 158 249 249 249 215 215 215 176 176 176 132 132 132 135 135 135 231 231 231 141 141 141 164 164 164 136 136 136 245 245 245 152 152 152 165 165 165 130 130 130 248 248 248 183 183 183 204 204 204 210 210 210 144 144 144 230 230 230 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 0 0 0 0 0 44 0 0 0 0 30 0 22 0 0 7 255 128 100 130 131 132 133 134 135 136 25 23 24 10 28 99 143 144 99 37 92 45 19 12 8 136 130 18 23 33 28 39 5 101 161 162 162 32 12 10 11 38 15 0 135 13 36 11 8 163 177 178 101 41 68 3 9 133 52 63 33 75 179 190 162 74 4 3 171 130 0 60 11 191 201 161 12 70 48 131 15 47 31 179 27 202 161 20 49 53 196 19 40 190 6 17 21 2 191 7 95 51 99 13 100 9 43 176 179 5 21 71 16 55 3 179 7 32 77 11 57 100 13 4 213 10 6 16 6 24 98 29 248 48 35 11 24 50 14 246 85 43 83 128 73 4 25 163 14 8 0 64 165 3 66 133 202 4 40 16 1 161 194 168 36 72 36 140 176 168 79 148 5 118 55 34 124 11 55 74 8 25 22 49 128 160 83 23 234 228 40 12 254 68 40 152 39 32 195 148 33 26 4 109 163 64 212 102 40 3 50 168 205 178 0 34 129 3 23 88 88 8 122 176 64 128 10 21 82 70 177 244 181 5 128 3 15 90 178 21 59 17 68 135 128 15 58 22 150 145 192 192 198 19 28 206 82 6 209 24 241 226 10 25 0 1 242 234 221 27 0 64 129 7 78 92 164 240 49 172 80 171 34 26 10 244 112 176 67 140 227 199 98 60 12 176 17 70 3 20 97 184 12 109 194 97 165 197 134 198 144 29 123 200 209 161 138 151 84 196 18 45 138 226 40 210 163 18 33 186 88 194 148 169 182 237 66 129 0 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
buttonSquareYellowGif
	"'imgs/dars-kvadrat.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 9 0 9 0 128 0 0 50 138 66 251 171 61 33 249 4 0 0 0 0 0 44 0 0 0 0 9 0 9 0 0 2 15 132 143 161 27 198 235 94 3 114 214 248 48 67 28 20 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-buttons' stamp: ' 21/4/07 10:33'!
buttonUpWhiteGif
	"'imgs/awful-button-up.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 22 0 22 0 198 96 0 239 239 239 143 143 143 207 207 207 159 159 159 223 223 223 128 128 128 221 221 221 254 254 254 212 212 212 228 228 228 214 214 214 191 191 191 226 226 226 32 32 32 140 140 140 216 216 216 225 225 225 252 252 252 253 253 253 199 199 199 189 189 189 64 64 64 222 222 222 201 201 201 16 16 16 232 232 232 172 172 172 96 96 96 209 209 209 151 151 151 240 240 240 175 175 175 244 244 244 156 156 156 141 141 141 242 242 242 247 247 247 235 235 235 174 174 174 163 163 163 155 155 155 194 194 194 220 220 220 180 180 180 142 142 142 166 166 166 224 224 224 202 202 202 150 150 150 218 218 218 205 205 205 177 177 177 179 179 179 186 186 186 203 203 203 145 145 145 219 219 219 138 138 138 144 144 144 173 173 173 135 135 135 134 134 134 0 0 0 248 248 248 246 246 246 195 195 195 217 217 217 80 80 80 165 165 165 236 236 236 133 133 133 215 215 215 181 181 181 250 250 250 176 176 176 146 146 146 158 158 158 152 152 152 153 153 153 249 249 249 245 245 245 251 251 251 164 164 164 231 231 231 183 183 183 132 132 132 130 130 130 208 208 208 204 204 204 147 147 147 241 241 241 137 137 137 178 178 178 136 136 136 148 148 148 229 229 229 255 255 255 230 230 230 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 44 0 0 0 0 22 0 22 0 0 7 254 128 97 130 131 130 9 6 10 10 6 9 132 140 132 95 15 20 33 1 147 1 33 20 15 95 141 131 46 41 14 39 12 96 161 96 12 39 44 65 46 154 6 51 57 0 18 162 2 162 30 14 92 6 140 16 52 44 35 36 174 96 3 24 4 162 96 29 52 16 132 19 61 83 0 187 7 7 13 3 27 193 81 14 19 131 22 48 23 97 30 79 7 96 5 208 13 11 193 12 29 22 130 19 60 4 95 35 220 0 191 189 21 193 96 94 212 97 31 45 66 25 162 5 240 96 237 225 162 41 62 8 98 178 64 65 190 120 8 67 17 24 32 232 6 7 14 7 69 1 8 144 16 204 13 65 29 94 8 248 18 175 0 56 132 32 154 8 34 162 65 6 48 137 13 4 240 11 134 160 133 32 10 85 176 60 8 182 161 0 152 10 3 226 13 160 32 8 129 149 11 87 32 64 57 64 192 7 0 48 31 26 196 115 128 64 16 3 19 89 108 32 248 2 66 132 205 80 67 254 129 89 97 130 193 160 7 1 86 200 80 144 0 0 144 36 17 210 70 8 133 99 201 3 71 71 2 182 236 224 160 160 92 17 0 30 64 252 144 128 32 192 70 70 9 4 192 232 98 227 136 10 8 95 74 104 41 129 194 137 128 69 141 190 196 168 33 194 8 138 25 74 164 56 208 81 35 70 38 77 130 190 168 184 128 68 131 6 42 47 12 124 6 205 154 117 32 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: ' 21/4/07 10:33'!
calendar1Gif
	"'imgs/calendar1.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 19 0 20 0 179 0 0 0 0 0 255 255 255 2 100 199 2 97 194 4 120 237 57 153 251 121 188 253 204 229 254 225 240 255 70 162 251 252 2 2 250 250 250 255 255 255 0 0 0 0 0 0 0 0 0 33 249 4 1 0 0 12 0 44 0 0 0 0 19 0 20 0 0 4 93 144 201 73 107 77 56 235 141 89 26 96 40 142 32 54 20 104 170 174 3 38 144 112 136 25 72 109 223 248 49 35 4 193 251 189 223 79 151 160 17 22 60 228 49 153 36 210 112 80 155 83 24 84 40 168 132 233 114 169 80 122 167 209 168 22 72 198 106 189 204 237 2 28 206 237 130 240 114 239 156 70 31 217 109 105 194 195 233 103 44 128 128 17 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: ' 21/4/07 10:33'!
calendar2Gif
	"'imgs/calendar2.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 20 0 20 0 230 0 0 0 0 0 255 255 255 73 72 73 58 57 59 62 62 63 58 58 59 84 84 85 78 78 79 72 72 73 98 98 99 91 91 92 16 49 249 92 135 223 137 178 244 116 184 250 145 198 250 57 58 59 115 184 249 115 185 249 116 184 249 116 185 249 121 187 249 128 191 249 136 195 249 137 195 249 145 199 250 154 203 250 154 204 250 164 208 249 174 213 250 174 213 249 183 218 250 192 222 250 201 226 250 201 227 250 208 230 250 215 233 249 208 231 250 215 234 250 222 237 250 221 236 249 226 239 250 208 220 223 66 67 67 105 106 106 98 99 98 32 117 2 47 167 5 207 207 205 197 197 195 182 182 180 182 182 181 178 178 177 197 196 195 63 62 62 85 84 84 114 113 113 146 145 145 164 164 164 145 145 145 129 129 129 127 127 127 121 121 121 106 106 106 91 91 91 67 67 67 58 58 58 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 1 0 0 67 0 44 0 0 0 0 20 0 20 0 0 7 189 128 67 130 131 132 133 134 48 136 137 138 139 137 67 49 41 144 145 146 147 145 53 50 40 46 47 154 155 156 156 46 39 51 58 36 157 164 157 38 58 57 37 42 171 172 173 174 42 35 59 60 34 61 181 182 183 184 61 33 60 62 32 61 52 1 191 193 192 194 197 32 62 56 31 61 1 193 204 203 205 208 61 31 56 44 30 185 182 11 217 11 182 29 63 45 28 197 196 192 11 12 13 11 196 28 9 10 26 207 237 204 11 13 230 206 27 64 55 15 215 181 218 219 181 25 6 7 23 225 134 9 44 134 225 128 0 11 238 162 57 115 102 1 193 138 10 248 240 85 8 98 35 66 4 10 23 39 104 140 224 64 227 4 139 31 37 16 24 82 160 100 1 33 5 6 160 52 201 18 66 1 67 48 99 202 156 57 36 16 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: ' 21/4/07 10:33'!
calendar3Gif
	"'imgs/calendar3.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 20 0 20 0 179 0 0 0 0 0 255 255 255 4 111 231 57 148 251 178 178 177 255 0 0 253 125 125 252 215 215 127 127 127 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 1 0 0 9 0 44 0 0 0 0 20 0 20 0 0 4 97 48 137 73 171 165 50 232 205 251 158 193 32 142 100 57 4 160 169 142 168 224 189 29 136 204 116 109 35 109 128 16 58 191 247 64 156 76 131 211 17 143 70 217 109 86 104 22 104 185 159 148 87 48 28 10 190 40 178 88 56 92 143 202 165 243 57 139 250 206 193 236 208 200 46 186 163 203 184 57 77 23 186 222 237 45 60 126 203 193 254 2 18 23 131 21 9 17 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: ' 21/4/07 10:33'!
calendar4Gif
	"'imgs/calendar4.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 20 0 20 0 230 0 0 0 0 0 255 255 255 116 184 250 145 198 250 9 134 252 9 135 252 14 137 252 20 141 252 27 143 252 27 144 252 34 147 252 34 148 252 39 149 252 39 150 252 40 149 252 40 150 252 42 151 252 50 154 252 50 155 252 59 159 252 67 163 252 68 163 252 76 167 252 76 168 252 84 171 252 86 172 252 86 173 252 87 172 252 87 173 252 92 175 252 99 178 252 99 179 252 106 182 252 112 185 252 117 187 252 115 184 249 115 185 249 116 184 249 116 185 249 121 189 252 121 187 249 128 191 249 136 195 249 137 195 249 145 199 250 154 203 250 154 204 250 164 208 249 174 213 250 174 213 249 183 218 250 192 222 250 201 226 250 201 227 250 208 230 250 215 233 249 208 231 250 215 234 250 222 237 250 221 236 249 226 239 250 208 220 223 252 132 14 251 172 95 251 226 201 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 1 0 0 65 0 44 0 0 0 0 20 0 20 0 0 7 202 128 65 130 131 132 133 134 39 136 137 138 139 137 65 34 60 144 145 146 147 145 34 33 59 25 28 28 25 153 27 25 26 28 27 158 27 154 58 33 32 55 15 14 13 170 15 13 175 15 12 177 178 12 57 32 31 56 61 186 187 188 189 61 54 30 29 53 6 196 197 198 199 6 52 29 24 51 6 49 1 206 208 207 209 212 51 24 23 50 6 1 208 219 218 220 223 6 50 22 20 49 200 197 62 232 62 197 48 21 19 47 212 211 207 62 63 64 62 211 47 19 18 45 222 252 219 62 64 245 186 185 136 0 97 128 57 98 233 212 17 99 1 97 129 10 120 210 34 82 91 161 0 65 138 126 224 186 117 75 145 224 0 138 131 7 81 28 48 48 98 132 9 147 37 82 142 16 144 178 68 73 151 36 12 4 41 64 179 38 77 2 54 115 26 218 201 179 167 207 32 129 0 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: ' 21/4/07 10:33'!
calendar5Gif
	"'imgs/calendar5.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 19 0 20 0 179 0 0 0 0 0 255 255 255 1 130 248 1 104 199 2 76 145 5 128 241 107 183 253 164 209 251 206 207 207 252 1 1 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 1 0 0 10 0 44 0 0 0 0 19 0 20 0 0 4 100 80 169 51 43 189 85 202 114 184 239 160 167 21 195 81 158 102 122 22 219 224 190 112 92 32 10 25 223 239 92 227 188 94 24 192 160 112 232 11 24 143 200 100 17 41 8 52 159 1 66 96 153 172 26 139 9 167 22 218 148 82 173 74 90 33 144 229 110 163 83 49 216 138 61 155 189 234 117 184 70 118 107 145 95 121 122 83 232 251 255 127 52 26 8 132 133 134 134 26 17 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: ' 21/4/07 10:33'!
calendarButtonGif
	^self calendar2Gif! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: ' 21/4/07 10:33'!
commentsSmallGif
	"a small bubble"
	"'imgs/itp-comments.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 11 0 11 0 162 5 0 171 171 171 246 246 246 212 212 212 255 255 255 148 148 148 255 255 255 0 0 0 0 0 0 33 249 4 1 0 0 5 0 44 0 0 0 0 11 0 11 0 0 3 38 88 186 188 244 144 184 65 71 8 66 18 42 122 199 143 117 141 32 33 96 222 153 21 38 9 58 80 7 72 13 43 215 138 73 227 79 145 0 0 59 )! !

!DefaultWebStyle methodsFor: 'frame printing' stamp: ' 21/4/07 10:33'!
contactElement

	| element |
	element := WebElement new.
	element 
		addText: '<font face=helvetica size=-2>Send comments to: <br>'; 
		addText: '<b><a href="mailto:info@eranova.si">';
		addText: '<font face=helvetica size=-1>info@eranova.si</font></a></b></font>'.
	^element! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
copyIconGif
	"'imgs/jure-copy.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 11 0 12 0 196 0 0 0 0 0 255 255 255 255 0 0 255 16 16 255 32 32 255 48 48 255 64 64 255 80 80 255 96 96 255 112 112 255 128 128 255 143 143 255 159 159 255 175 175 255 191 191 255 207 207 255 223 223 255 239 239 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 1 0 0 18 0 44 0 0 0 0 11 0 12 0 64 5 61 96 32 142 164 152 8 2 226 44 40 34 66 104 17 5 144 161 60 163 130 238 133 136 160 142 210 232 113 32 8 108 179 18 100 161 80 52 72 58 1 65 97 20 48 2 44 171 48 96 0 138 20 135 43 109 176 139 37 183 163 16 0 59 )! !

!DefaultWebStyle methodsFor: 'frame printing' stamp: ' 21/4/07 10:33'!
copyrightElement
	| element |
	element := WebElement new.
	element
		addLinkTo: 'http://www.eranova.si/' text: '<font face=helvetica size=-1>(C) EraNova d.o.o</font>'.
	^element! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
cornerDashboardLeftGif
	"'imgs/fast/fast-dashboard_left.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 7 0 24 0 128 0 0 127 160 177 255 255 255 33 249 4 1 0 0 0 0 44 0 0 0 0 7 0 24 0 0 2 29 140 143 6 144 139 203 14 124 78 86 117 195 180 17 119 157 109 30 245 137 96 25 166 232 7 54 172 81 0 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
cornerDashboardRightGif
	"'imgs/fast/fast-dashboard_right.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 7 0 24 0 145 0 0 0 0 0 255 255 255 255 255 255 0 0 0 33 249 4 1 0 0 2 0 44 0 0 0 0 7 0 24 0 0 2 28 140 31 34 152 139 234 12 124 138 198 118 231 209 178 110 223 101 32 102 149 159 136 154 29 51 50 71 1 0 59 )! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
css1Body
^'
body {
	margin: 0;
	padding: 0;
	font-family: verdana, sans-serif;
	font-color: #666;
	font-size: 90%;
	line-height: 1.4em;
	background: #fff url("/img/leftstripegif.gif") repeat-y;
	}
#container {
	position: relative;
	width: 800px;
	margin: 0 0 0 8px;
	padding: 0;
	}
#containerPadded {    /* for printing header, has no efect here */
	position: relative;
	width: 800px;
	margin: 0 0 0 8px;
	padding: 0;
	}
#content {
	margin: 10px 15px 10px 10px;
	padding-top: 20px;
	font-size: 90%;
	text-align: left;
	margin-top/* */: 0; /* IE5.0 Win wont see this - fixes weird float bug for hp */
	}
html>body #content  {
	margin-top: 0;
	}
#content {
	margin-left: 135px;
	padding-left: 0;
	}

/* through all width of page, in navigation space too */
#contentWide {
	margin: 10px 15px 10px 10px;
	padding-top: 20px;
	font-size: 90%;
	text-align: left;
	margin-top/* */: 0; /* IE5.0 Win wont see this - fixes weird float bug for hp */
	}
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
css21Text
^'
h1, h2, h3, h4, h5, h6 {
	color: Black;
	background-color: transparent;
	font-family: Helvetica, Arial, sans-serif;
	font-size: 100%;
	font-weight: normal;
	margin: 0;
	padding-top: 0.5em;
      border-bottom: 1px solid #8cacbb;
}
h1 {
	margin-top: 10px;
	padding-top: 10px;
	font-size: 150%;
    }
h1 span {
	color: #999;
	}
h2 {
	font-size: 130%;
	font-weight: bold;
	border-bottom: none;
    }
h3 {
	margin: .7em 0 .7em 0;
	font-size: 120%;
	font-weight: bold;
	border-bottom: none;
     }
h4 {
	margin: .5em 0 .5em 0;
	font-size: 100%;
	border-bottom: none;
	font-weight: bold;
    }
h5 {
	font-size: 90%;
	border-bottom: none;
	font-weight: bold;
    }
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
css22Links
^'
a:link { color: #369; text-decoration: none}
a:visited {  color: #369; text-decoration: none}
a:active {  color:  #f90; text-decoration: none}

a.helpLink { }
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: 'np 6/24/2008 02:27'!
css23Inputs
^'
fieldset {
	border: 1px solid #ccc;
	margin: 1em 0em 1em 0em;
	padding: 0em 1em 1em 1em;
	line-height: 1.5em;
	color: #666;
}
fieldset td {
	color: #666;
}
legend {
	background: White;
	padding: 0.5em;
	font-size: 90%;
	color: #666;
}
form {
	border: none;
	margin: 0;
	height:100%;
}
input, button {
	font-family: "Lucida Grande", Verdana, Lucida, Helvetica, Arial, sans-serif;
	font-size: 80%;
	visibility: visible;
	border: 1px solid #ccc;
	background: #f5f5f5;
	color: #666;
	vertical-align: middle;
	padding: 1px; spacing: 2px;
	padding-left: 0px; padding-right: 0px;
	spacing-left: 0px; spacing-right: 0px;
}
textarea {
	font-family: "Lucida Grande", Verdana, Lucida, Helvetica, Arial, sans-serif;
	font-size: 80%;
	visibility: visible;
	border: 1px solid #ccc;
	background: #f5f5f5;
	color: #666;
	vertical-align: middle;
	padding: 2px;
	width: 98%;
}
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: 'np 5/7/2008 23:25'!
css24WebGrid
	^'table.webGrid {
	border: 1px solid #E4E4E4;
	border-collapse: collapse;
	margin-bottom: 4px;
	font-size: 12px;
	color: #484849;
	}
table.webGrid tr {}
table.webGrid td {border-top: 1px solid #F8F8F8;}
table.webGrid td input {font-size: 9pt;}

table.webgrid tbody tr:hover {background-color:#ffffdd;}

table.webGrid th {
	padding: 4px; 
	background: #EEEEEE;
	}
table.webGrid th a {color: #2A5685;}
table.webGrid th a:hover {color: #C61A1A;}

table.webGrid tr.red td, tr.red a { color: red; }
table.webGrid tr.green td, tr.green a { color: forestgreen; }
table.webGrid tr.blue td, tr.blue a { color: blue; }
table.webGrid tr.yellow td, tr.yellow a { color: yellow; }
table.webGrid tr.gray td, tr.gray a { color: darkgray; }
table.webGrid tr.summary td, tr.summary a { color: darkgray; font-weight: bold }
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
css25AutocompleteField
^'
div.auto_complete {
      position:absolute;
      width:350px;
	background-color: #fff
}
div.auto_complete ul {
	border:1px solid #888;
	background-color: #fff;
	margin:0;
	padding:0;
	width:100%;
	list-style-type:none;
}
div.auto_complete ul li {
	margin:0;
	padding:3px;
}
div.auto_complete ul li.selected { 
	background-color: #ffb; 
}
div.auto_complete ul strong.highlight { 
	color: #800; 
	margin:0;
	padding:0;
}
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
css26Popup
^'
div.popup {
    position: absolute;
    float: left;
    z-index: 10;
    background-color: #d5e4f2;
    font-family: arial;
    border: 1px solid #6599cd;
    padding: 15px;
    margin: 0px 6px 6px -6px !!important;
    margin: 0;
}
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
css27Menus
	"AnyLink dropdown and popit menus"
	"http://www.dynamicdrive.com/dynamicindex1/dropmenuindex.htm"
^'
#dropmenudiv{
	position:absolute;
	background-color: white;
	border:1px solid black;
	border-bottom-width: 0;
	font:normal 12px Verdana;
	line-height:18px;
	z-index:100;
}
#dropmenudiv a{
	width: 100%;
	display: block;
	text-indent: 3px;
	border-bottom: 1px solid black;
	padding: 1px 0;
	text-decoration: none;
}

#dropmenudiv a:hover{ /*hover background color*/
	background-color: #CCFF9D;
}
#popitmenu {
	position: absolute;
	background-color: white;
	border:1px solid black;
	font: normal 12px Verdana;
	line-height: 18px;
	z-index: 100;
	visibility: hidden;
}
#popitmenu a {
	text-decoration: none;
	padding-left: 6px;
	color: black;
	display: block;
}
#popitmenu a:hover{ /*hover background color*/
	background-color: #CCFF9D;
}
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: 'np 8/21/2008 17:54'!
css3Header
^'
#banner {
	height: 0px;
	width: 780px;
	margin: 0 0px 4px 8px; 
	padding: 6px 0; 
	background: #fff;
	text-align: center;
	white-space: nowrap;
	}
#banner div {
	padding: 0;
	border-bottom: none;
	}
#header {
	height: 82px;
	width: 780px;
	border: none;
	}
#logo {
	height: 50px;
	position: absolute;
	top: 5px;
	left: 5px;
	right: auto;
	color: #fff;
	white-space: nowrap;
	z-index: 1000;
	}
#dashboard {
	height: 24px;
	width: 780px;
	margin: 4px 0;
	line-height: 23px;
	font-size: 10px;
	background: #F0F0E7 url(/img/cornerdashboardleftGif.gif) no-repeat;
	color: #666;
	}
#switcher {
	position: absolute; 
	vertical-align: middle;
	top: 90px;
	right: 45px;
	margin: 0;
/*	z-index: 1001; */
	}
#switcher a b { position: relative; bottom: 4px; right: 10px}
#switcher a img {
	background: #999;
	}
#switcher a:hover img {
	background: #000;
	}

'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
css40NavigationBar
	^'
#navigation {
	position: absolute;
	width: 120px;
	top: 124px;
	margin-right: 0;
	font-size: 80%;
	color: #333;
	left: 8px; /* fixes weird IE5.0/win absolute position within relative */
	left/* */: 0;
	background: #F0F0E7 url("/img/leftcornergif.gif") no-repeat top right;
	}
html>body #navigation  {
	left: 0;
	}
#navigation div {
	padding: 10px;
	font-size: 100%;
	border-bottom: 1px solid #fff;
	}
#navigation div div {
	padding: 0;
	}
#navigation .boxGrey {
	background: #fff;
	}
#navigation h3 {
	padding: 0;
	margin: 0;
	margin-bottom: 6px;
	font-family: verdana, sans-serif;
	font-size: 100%;
	font-weight: bold;
	letter-spacing: 0;
	color: #AAA799;
      padding-top: 0;
      border-bottom: 0;
	}
#navigation h3 img {
	vertical-align: middle;
	display: inline;
	background: #AAA799;
	}
#navigation p {
	margin: 3px 0 0 0;
	line-height: 1.2em;
	}
#navigation ul, .styledBox ul {
	margin: 0;
	padding: 0;
	list-style: none;
	}
#navigation li, .styledBox li {
	background: url("/img/bulletsquareleftwhitegif.gif") no-repeat 0 1px;
	padding: 0 0 6px 16px;
	line-height: 1em;
	}
button {
	visibility: visible;
	border: 1px solid #ccc;
	background: #f5f5f5;
	color: #666;
	vertical-align: middle;
	padding: 2px;
}
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
css41Login
^'
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
css42ErrorReport
^'
#error {
	color: red;
	font-size: 120%;
}
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
css43Buckets
	^'
/* column of actions and aditional info to the right */
/* one #buckets of many #wrapper divs */

#buckets {
	float: right;
	margin: 0 0 6px 14px;
	width: 336px;
	font-size: 80%;
	}
/* to fix weird IE/Win border issue */
#buckets .wrapper {
	width: 336px;
	}
#buckets h3 {
	margin: 0;
	padding: 3px 0 3px 4px;
	font-family: verdana, sans-serif;
	font-size: 100%;
	font-weight: bold;
	letter-spacing: 0;
	padding-left: 6px;
	color: #AAA799;
	background: #F0F0E7 url("/img/buckettopgif.gif") no-repeat top;
	}
#buckets h3 img {
	vertical-align: middle;
	display: inline;
	background: #AAA799;
	}
#buckets div div {
	margin: 0 0 10px 0;
	padding: 6px 12px 6px 12px;
	color: #333;
	border: 1px solid #F0F0E7;
	border-top: none;
	}
#buckets div div td {
	vertical-align: top;
	font-family: verdana, sans-serif;
	font-size: 70%;
	}
#buckets div div input {
	margin-top: 4px;
	vertical-align: middle;
	font-family: verdana, sans-serif;
	font-size: 90%;
	}
#buckets img.posp {
	margin-right: 4px;
	}
#imu {
	margin: 0 0 10px 0;
	}
#buckets li {
	background: url("/img/bulletsquareleftwhitegif.gif") no-repeat 0 1px;
	padding: 0 0 6px 15px;
	line-height: 1em;
	}
#buckets ul {
	margin: 0;
	padding: 0;
	list-style: none;
	}
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
css44ContentHierarchy
	^'
/* open/closed folders and content */
#contentHierarchy, #contentHierarchy td { font-size: 90%; }
#contentHierarchy input { font-size: 80%; }
#contentHierarchyTable { font-size: 90%; }
#contentHierarchyTable tr.red td, tr.red a { color: red; }
#contentHierarchyTable tr.green td, tr.green a { color: forestgreen; }
#contentHierarchyTable tr.blue td, tr.blue a { color: blue; }
#contentHierarchyTable tr.yellow td, tr.yellow a { color: yellow; }
#contentHierarchyTable tr.gray td, tr.gray a { color: darkgray; }

'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
css451Document
	^'
#documentMain { }
#documentEdit td { font-size: 10px; }
#documentEdit input { font-size: 12px; }
#documentEdit textarea { font-size: 12px; }
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
css452DocumentActions
	^'
/* img links to common actions on this document like print ... */
.documentActions {
	float: right;
	margin: -1.5em 0 0 0;
	padding-bottom: 5px;
}
.documentActions ul,
.documentActions li {
	display: inline;
	list-style: none;
	list-style-image: none;
}
.documentActions a {
	border: 0;
	text-decoration: none;
}
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
css453ClassificationLevel
	^'
/* level of classified docs */
#classificationLevel  {
	font-size: 10pt; font-weight: bold;
	color: #800000; background-color: #FAFAD2;
	width: 290px; padding: 5px;
}
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
css46Search
	^'
dl#searchRslts dt a, dl#searchRslts dd span {
	font-weight: bold;
	}
dl#searchRslts dd span {
	color: #666;
	}
dl#searchRslts dt strong, dl#searchRslts dd strong {
	background: #FFFFCC;
	}
dl#searchRslts dd {
	margin: 0 0 1em 0;
	border-bottom: 1px dashed #999;
	color: #000;
	}
#searchTop {
	margin: 0;
	font-size: 90%;
	}
table.searchTable {
	border-top: 3px solid #ccc;
	border-bottom: 2px solid #ccc;
	background: #f3f3f3;
	}
table.searchTable td, table.searchTable th {
	border-bottom: 1px solid #ccc;
	padding: 8px;
	}
table.searchTable th {
	width: 75px;
	text-align: left;
	}
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
css47Comments
	^'
table.commentsTable  { font-size: 100%; }
table.commentsTable tr {
	border-right: 1px solid rgb(215,215,215);
	border-left: 1px solid rgb(215,215,215);
	border-top: 1px solid rgb(215,215,215);
	border-bottom: 1px solid rgb(215,215,215);
    }
table.commentsTable td {
	padding: 0px;
	spacing: 2px;
	vertical-align: top;
	border-right: 1px solid rgb(215,215,215);
	border-left: 1px solid rgb(215,215,215);
	border-top: 1px solid rgb(215,215,215);
	border-bottom: 1px solid rgb(215,215,215);
	}
tr.commentHeaderRow { background: #F0F0E7; ; }
tr.commentBodyRow { height: 50px; }
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
css51Wiki
	^'
#wikiPage {
	margin: 10px 15px 10px 10px;
	padding-top: 20px;
	font-size: 90%;
	text-align: left;
	margin-top/* */: 0; /* IE5.0 Win wont see this - fixes weird float bug for hp */
	}
html>body #wikiPage  { margin-top: 0; 	}
#wikiPage { margin-left: 15px; padding-left: 0; }
#wikiPage td { font-size: 80% }
#wikiPage input, textarea { font-size: 100% }
#wikiPage table.webGrid { font-size: 120% }
table.wikiTable td { border: thin solid black; padding: 1px; margin: 0px }
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
css9Hide
 ^'
.print { display: none; }
#printHeader { display: none; }
#documentTitlePrint { display: none; }
#printFooter { display: none; }
.printOnly { display: none; }
.screenOnly { display: show; }


'! !

!DefaultWebStyle methodsFor: 'styles-print' stamp: ' 21/4/07 10:33'!
cssPrintBody
	^'
body {
	font-family: Times, serif;
	font-size: 11pt;
	background-color: #fff;
      margin: 0em;
      margin-top: 0em;
      }
td { font-size: 10pt }

h1,h2,h3,h4,h5 { font-family: Arial, Helvetica, sans-serif; }
h1 { font-size: 14pt; }
h2 { font-size: 12pt; margin-top: 25pt; }
h3 { font-size: 11pt; margin-top: 20pt; }
h4 { font-size: 11pt; }
h5 { font-size: 10pt; }

a:link, a:visited, a:active, a:hover {
	color: black;
	background: transparent;
	font-weight: normal;
	text-decoration: none;
	}

#content a:link:after, #content a:visited:after  {
	content: " (" attr(href) ") ";
	}
'! !

!DefaultWebStyle methodsFor: 'styles-print' stamp: ' 21/4/07 10:33'!
cssPrintDocFooter
	^'
#printFooter table {
	width: 100%;
  	margin-bottom: 2em;
	border-top: 1px solid #999;
	border-left: 1px solid #999;
	border-collapse: collapse;
	}
#printFooter td { 
	font-family: Arial, Helvetica, sans-serif;
	font-size: 8pt;
	padding: 5px;
	border-right: 1px solid #999;
	border-bottom: 1px solid #999;
	}
#printFooter td table { border-top: 0; border-left: 0; }
#printFooter td table td { border-right: 0; border-bottom: 0; }
'! !

!DefaultWebStyle methodsFor: 'styles-print' stamp: ' 21/4/07 10:33'!
cssPrintDocHeader
	^'
body { position: relative; }
#containerPadded { position: relative;  top: 80pt; padding-bottom: 80pt; }   /*place for header, Moz.only*/
#printHeader { position: fixed; top: 0; }
#printHeader table { 
  	margin-bottom: 2em;
	border-top: 1px solid #999;
	border-left: 1px solid #999;
	border-collapse: collapse;
	}
#printHeader td { 
	font-family: Arial, Helvetica, sans-serif;
	font-size: 8pt;
	padding: 5px;
	border-right: 1px solid #999;
	border-bottom: 1px solid #999;
	}
#printHeader td b { font-weight: bold; font-size: 10pt }

', (self isBrowserMSIE ifTrue: ['#containerPadded { padding-top: 0 }'] ifFalse: ['']) "reset place for header"! !

!DefaultWebStyle methodsFor: 'styles-print' stamp: ' 21/4/07 10:33'!
cssPrintDocTitle
	^'
#documentTitlePrint{ 
	width: 100%; 
	margin-top: 120pt;
}
#documentTitlePrint table { 
	width: 100%; 
	font-family: Arial, Helvetica, sans-serif;
	}
#documentTitlePrint td { text-align: center; }
#documentTitlePrint h1 {font-size: 13pt;}
#documentTitlePrint h2 {font-size: 11pt}
'! !

!DefaultWebStyle methodsFor: 'styles-print' stamp: ' 21/4/07 10:33'!
cssPrintHide
	^'
#banner, #header, #logo, #dashboard, #switcher { display: none; }
#navigation, #hierarchyLinks { display: none; }
#buckets { display: none; }
.screenOnly { display: none; }
.documentActions { display: none; }
#documentTitleScreen { display: none; }
#classificationLevel  { display: none; }
#chapterNavigation { display: none; }
#processDescription { display: none; }
'! !

!DefaultWebStyle methodsFor: 'styles-print' stamp: ' 21/4/07 10:33'!
cssPrintShow
	^'
body { display: inline-table; }
div.pageBreak { page-break-after: always; }
.print { display: show; }
.printOnly { display: show; }

'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
cssTabs1
	^'
/*from A List Apart article: http://www.alistapart.com/articles/slidingdoors/ */
/*WebTabs newClass: #tabs1 but dont forget to define class tabs1selected too !! */

ul.tabs1 {
  float:left; 
  width:100%; 
  background:#DAE0D2 url("/img/tab1bggif.gif") repeat-x bottom; 
  font-size:10px;
  line-height:normal;
  margin:0;
  padding:10px 0 0 0;
  list-style:none;
  }
ul.tabs1 li {
  float:left; 
  background:url("/img/tab1leftgif.gif") no-repeat left top;
  margin:0;
  padding:0 0 0 9px;
  }
ul.tabs1 a, ul.tabs1 b {
  float:left; 
  display:block;
  background:url("/img/tab1rightgif.gif") no-repeat right top;
  padding:5px 15px 4px 6px;
  text-decoration:none;
  font-weight:bold;
  color:#765;
  }
/* Commented Backslash Hack
   hides rule from IE5-Mac \*/
ul.tabs1 a {float:none;}
/* End IE5-Mac hack */
ul.tabs1 a:hover {
  color:#333;
  }
ul.tabs1 li.tabs1selected {
  background-image:url("/img/tab1leftongif.gif");
  }
ul.tabs1 li.tabs1selected b {
  background-image:url("/img/tab1rightongif.gif");
  color:#333;
  padding-bottom:5px;
  }
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
cssTabs2
	^'
/*WebTabs newClass: #tabs2 but dont forget to define here class tabs2selected too !! */

ul.tabs2 {
	height: 20px;
	margin: 0;
	padding-left: 10px;
	background: url("/img/tab2bottomgif.gif") repeat-x bottom;
	}
ul.tabs2, ul.tabs2 li {
	margin: 0;
	padding: 0;
	display: inline;
	list-style-type: none;
	}
ul.tabs2 a:link, ul.tabs2 a:visited, ul.tabs2 b {
	float: left;
	background: #f3f3f3;
	font-size: 10px;
	line-height: 14px;
	font-weight: bold;
	padding: 2px 10px 2px 10px;
	margin-right: 4px;
	border: 1px solid #ccc;
	text-decoration: none;
	color: #666;
	}
ul.tabs2 a:hover {
	background: #fff;
	}
ul.tabs2 li.tabs2selected b {
	border-bottom: 1px solid #fff;
	background: #fff;
	color: #000;
	}
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
cssTabs3
	^'
/*WebTabs newClass: #tabs3 but dont forget to define here class tabs3selected too !! */

ul.tabs3 {
	border-top-width: 2px;
	border-right-width: 2px;
	border-bottom-width: 2px;
	border-left-width: 2px;
	border-bottom-style: solid;
	border-top-color: #CCCCCC;
	border-right-color: #CCCCCC;
	border-bottom-color: #CCCCCC;
	border-left-color: #CCCCCC;
}
ul.tabs3 li {
	background-color: #f0f0e7;
	background-repeat: repeat-x;
	padding: 0px;
	border-top: 2px solid #CCCCCC;
	border-right: 2px none #FFFFFF;
	border-bottom: 0px none #CCCCCC;
	border-left: 2px none #FFFFFF;
	background-position: left center;
	color: #000000;
	display: inline;
	white-space: pre;
	text-align: center;
	font-family: Arial, Helvetica, sans-serif;
	font-size: 12px;
	letter-spacing: normal;
	word-spacing: normal;
	margin: 0px;
	clear: none;
	float: none;
	height: 70px;
	width: 60px;
	text-decoration: none;
	background-image: url(/img/tab3backgif.gif);
	vertical-align: baseline;
	}
ul.tabs3 a:link {
	background-color: #f0f0e7;
	background-repeat: repeat-x;
	padding: 4px;
	border-top: 0px solid #CCCCCC;
	border-right: 1px solid #CCCCCC;
	border-bottom: 0px none #FFFFFF;
	border-left: 1px solid #CCCCCC;
	background-position: left center;
	color: #336699;
	display: inline;
	white-space: pre;
	text-align: center;
	font-family: Arial, Helvetica, sans-serif;
	font-size: 12px;
	letter-spacing: normal;
	word-spacing: normal;
	margin: 0px;
	clear: none;
	float: none;
	height: 70px;
	width: auto;
	text-decoration: none;
	background-image: url(/img/tab3backgif.gif);
}
ul.tabs3 a:visited {
	background-color: #F0F0E7;
	background-repeat: repeat;
	padding: 4px;
	border-top: 2px solid #CCCCCC;
	border-right: 1px solid #CCCCCC;
	border-bottom: 0px none #CCCCCC;
	border-left: 1px solid #CCCCCC;
	background-position: left center;
	color: #336699;
	display: inline;
	white-space: pre;
	text-align: center;
	font-family: Arial, Helvetica, sans-serif;
	font-size: 12px;
	letter-spacing: normal;
	word-spacing: normal;
	margin: 0px;
	clear: none;
	float: none;
	height: 60px;
	width: auto;
	text-decoration: none;
	background-image: url(/img/tab3backgif.gif);
	vertical-align: baseline;
	list-style-image: url(back1.gif);
}
ul.tabs3 a:hover {
	background-color: #F0F0E7;
	background-repeat: repeat;
	padding: 4px;
	border-top: 2px solid #999999;
	border-right: 1px solid #CCCCCC;
	border-bottom: 2px none #FFFFFF;
	border-left: 1px solid #CCCCCC;
	background-position: left center;
	color: #FF9900;
	display: inline;
	white-space: pre;
	text-align: center;
	font-family: Arial, Helvetica, sans-serif;
	font-size: 12px;
	letter-spacing: normal;
	word-spacing: normal;
	margin: 0px;
	clear: none;
	float: none;
	height: 70px;
	width: auto;
	text-decoration: none;
	background-image: url(/img/tab3backgif.gif);
}
ul.tabs3 li.tabs3selected {
	background-color: #F0F0E7;
	background-repeat: repeat;
	padding: 4px;
	border-top: 0px solid #CCCCCC;
	border-right: 0px none #FFFFFF;
	border-bottom: 2px solid #CCCCCC;
	border-left: 0px none #FFFFFF;
	background-position: left center;
	color: #FF9900;
	display: inline;
	white-space: pre;
	text-align: center;
	font-family: Arial, Helvetica, sans-serif;
	font-size: 12px;
	letter-spacing: normal;
	word-spacing: normal;
	margin: 0px;
	clear: none;
	float: none;
	height: 70px;
	width: auto;
	text-decoration: none;
}
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
cssTabs4
	^'
/*WebTabs newClass: #tabs4 but dont forget to define here class tabs4selected too !! */

ul.tabs4 {
	border-top-width: 1px;
	border-right-width: 1px;
	border-bottom-width: 1px;
	border-left-width: 1px;
	border-bottom-style: solid;
	border-top-color: #CCCCCC;
	border-right-color: #CCCCCC;
	border-bottom-color: #CCCCCC;
	border-left-color: #CCCCCC;
}
ul.tabs4 li {
	background-color: #f0f0e7;
	background-repeat: repeat-x;
	padding: 0px;
	border-top: 2px solid #CCCCCC;
	border-right: 2px none #FFFFFF;
	border-bottom: 0px none #CCCCCC;
	border-left: 2px none #FFFFFF;
	background-position: left center;
	color: #000000;
	display: inline;
	white-space: pre;
	text-align: center;
	font-family: Arial, Helvetica, sans-serif;
	font-size: 12px;
	letter-spacing: normal;
	word-spacing: normal;
	margin: 0px;
	clear: none;
	float: none;
	height: 70px;
	width: 60px;
	text-decoration: none;
	background-image: url(/img/tab3backgif.gif);
	vertical-align: baseline;
}
ul.tabs4 a:link {
	background-color: #f0f0e7;
	background-repeat: repeat-x;
	padding: 4px;
	border-top: 0px solid #CCCCCC;
	border-right: 1px solid #CCCCCC;
	border-bottom: 0px none #FFFFFF;
	border-left: 1px solid #CCCCCC;
	background-position: left center;
	color: #336699;
	display: inline;
	white-space: pre;
	text-align: center;
	font-family: Arial, Helvetica, sans-serif;
	font-size: 12px;
	letter-spacing: normal;
	word-spacing: normal;
	margin: 0px;
	clear: none;
	float: none;
	height: 70px;
	width: auto;
	text-decoration: none;
	background-image: url(/img/tab3backgif.gif);
}
ul.tabs4 a:visited {
	background-color: #F0F0E7;
	background-repeat: repeat;
	padding: 4px;
	border-top: 2px solid #CCCCCC;
	border-right: 1px solid #CCCCCC;
	border-bottom: 2px solid #CCCCCC;
	border-left: 1px solid #CCCCCC;
	background-position: left center;
	color: #336699;
	display: inline;
	white-space: pre;
	text-align: center;
	font-family: Arial, Helvetica, sans-serif;
	font-size: 12px;
	letter-spacing: normal;
	word-spacing: normal;
	margin: 0px;
	clear: none;
	float: none;
	height: 60px;
	width: auto;
	text-decoration: none;
	background-image: url(/img/tab3backgif.gif);
	vertical-align: baseline;
	list-style-image: url(back1.gif);
}
ul.tabs4 a:hover {
	background-color: #F0F0E7;
	background-repeat: repeat;
	padding: 4px;
	border-top: 2px solid #CCCCCC;
	border-right: 1px solid #CCCCCC;
	border-bottom: 2px none #FFFFFF;
	border-left: 1px solid #CCCCCC;
	background-position: left center;
	color: #FF9900;
	display: inline;
	white-space: pre;
	text-align: center;
	font-family: Arial, Helvetica, sans-serif;
	font-size: 12px;
	letter-spacing: normal;
	word-spacing: normal;
	margin: 0px;
	clear: none;
	float: none;
	height: 70px;
	width: auto;
	text-decoration: none;
	background-image: url(/img/tab3backgif.gif);
}
ul.tabs4 li.tabs4selected {
	background-color: #F0F0E7;
	background-repeat: repeat;
	padding: 4px;
	border-top: 0px solid #CCCCCC;
	border-right: 0px none #FFFFFF;
	border-bottom: 2px solid #CCCCCC;
	border-left: 0px none #FFFFFF;
	background-position: left center;
	color: #FF9900;
	display: inline;
	white-space: pre;
	text-align: center;
	font-family: Arial, Helvetica, sans-serif;
	font-size: 12px;
	letter-spacing: normal;
	word-spacing: normal;
	margin: 0px;
	clear: none;
	float: none;
	height: 70px;
	width: auto;
	text-decoration: none;
}
'! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
cssTabs5
	^'
ul.tabs5 {
    float:left;
    width:600px;
    background:#DAE0D2 url("/img/tab5bggif.gif") repeat-x bottom;
    font-size:93%;
    line-height:normal;
    text-decoration: none;
    color: #333333;
    margin:0;
    padding:10px 10px 0;
    list-style:none;
    }
ul.tabs5 li {
    float:left;
    background:url("/img/tab5rightgif.gif") no-repeat right top;
    margin:0;
    padding:0 0 0 ;
    }
ul.tabs5 a, ul.tabs5 b {
   display:block;
   background:url("/img/tab5leftgif.gif") no-repeat left top;
   padding:5px 15px 4px 6px;
   text-decoration: none;
   color: #006699;
   font-family: Arial, Helvetica, sans-serif;
   font-weight: bold;
   font-size: 12px;
   }

/* Commented Backslash Hack
   hides rule from IE5-Mac \*/
ul.tabs5 a {float:none;}
/* End IE5-Mac hack */
ul.tabs5 a:hover {
  color:#333;
  }
ul.tabs5 li.tabs5selected {
  background-image:url("/img/tab5rightongif.gif");
  }
ul.tabs5 li.tabs5selected b {
  background-image:url("/img/tab5leftongif.gif");
  padding-bottom:5px;
  color: #666666;
  font-family: Arial, Helvetica, sans-serif;
  }
ul.tabs5 a:hover {
  color: #EFEFE6;
  }
'! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
cutIconGif
	"'imgs/jure-cut.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 11 0 12 0 196 0 0 0 0 0 255 255 255 242 244 248 42 86 143 69 107 157 95 128 171 136 160 192 175 191 213 215 223 234 55 97 150 82 118 164 122 150 185 149 171 199 161 181 206 202 213 227 228 234 241 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 1 0 0 16 0 44 0 0 0 0 11 0 12 0 64 5 60 96 32 142 100 128 12 232 178 160 131 35 10 4 203 30 129 144 176 79 160 160 137 216 200 45 17 138 32 16 197 80 14 4 99 217 120 45 25 1 6 203 176 66 138 14 192 230 21 56 40 136 110 3 133 192 193 242 150 206 33 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
docYellowPng
	"'imgs/suse/suse-doc.png' asFilename contentsAsMethod"
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 3 0 0 0 40 45 15 83 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 0 84 80 76 84 69 60 83 19 99 99 99 196 228 137 161 212 68 182 222 110 211 235 166 167 215 79 193 227 132 202 231 148 198 229 141 180 221 106 189 225 122 209 234 164 191 226 127 186 224 117 156 210 58 154 209 52 143 196 45 147 202 46 196 228 138 204 232 153 169 216 84 151 207 47 132 181 42 128 176 40 199 229 143 172 217 89 255 255 255 99 155 214 1 0 0 0 28 116 82 78 83 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 255 0 23 178 226 215 0 0 0 214 73 68 65 84 120 218 98 144 102 134 1 70 105 16 0 8 32 6 105 102 86 86 86 30 32 100 230 7 139 0 4 16 72 128 1 8 120 56 132 132 196 65 34 0 1 4 22 96 101 101 224 100 98 98 98 97 1 138 0 4 16 76 64 88 132 131 131 137 11 40 0 16 64 48 45 156 64 12 22 0 8 32 160 0 15 7 7 39 19 39 80 15 59 72 0 32 128 64 2 12 80 192 203 11 20 0 8 32 160 128 36 80 55 39 59 19 59 59 11 72 5 64 0 49 72 243 11 195 84 176 128 84 0 4 16 131 180 24 19 19 39 80 154 137 155 27 108 45 64 0 49 72 11 48 65 228 185 89 120 65 2 0 1 196 32 45 8 116 18 80 63 72 1 200 12 128 0 2 9 128 180 179 240 241 241 129 5 0 2 136 65 90 66 74 148 13 10 4 128 2 0 1 196 32 45 205 136 4 164 165 1 2 12 0 95 63 9 245 228 242 236 115 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: ' 21/4/07 10:33'!
emailJpg
	"'imgs/toledo-email.jpg' asFilename contentsAsMethod"
^#(255 216 255 224 0 16 74 70 73 70 0 1 1 1 0 72 0 72 0 0 255 219 0 67 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 255 219 0 67 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 255 192 0 17 8 0 14 0 22 3 1 34 0 2 17 1 3 17 1 255 196 0 31 0 0 1 5 1 1 1 1 1 1 0 0 0 0 0 0 0 0 1 2 3 4 5 6 7 8 9 10 11 255 196 0 181 16 0 2 1 3 3 2 4 3 5 5 4 4 0 0 1 125 1 2 3 0 4 17 5 18 33 49 65 6 19 81 97 7 34 113 20 50 129 145 161 8 35 66 177 193 21 82 209 240 36 51 98 114 130 9 10 22 23 24 25 26 37 38 39 40 41 42 52 53 54 55 56 57 58 67 68 69 70 71 72 73 74 83 84 85 86 87 88 89 90 99 100 101 102 103 104 105 106 115 116 117 118 119 120 121 122 131 132 133 134 135 136 137 138 146 147 148 149 150 151 152 153 154 162 163 164 165 166 167 168 169 170 178 179 180 181 182 183 184 185 186 194 195 196 197 198 199 200 201 202 210 211 212 213 214 215 216 217 218 225 226 227 228 229 230 231 232 233 234 241 242 243 244 245 246 247 248 249 250 255 196 0 31 1 0 3 1 1 1 1 1 1 1 1 1 0 0 0 0 0 0 1 2 3 4 5 6 7 8 9 10 11 255 196 0 181 17 0 2 1 2 4 4 3 4 7 5 4 4 0 1 2 119 0 1 2 3 17 4 5 33 49 6 18 65 81 7 97 113 19 34 50 129 8 20 66 145 161 177 193 9 35 51 82 240 21 98 114 209 10 22 36 52 225 37 241 23 24 25 26 38 39 40 41 42 53 54 55 56 57 58 67 68 69 70 71 72 73 74 83 84 85 86 87 88 89 90 99 100 101 102 103 104 105 106 115 116 117 118 119 120 121 122 130 131 132 133 134 135 136 137 138 146 147 148 149 150 151 152 153 154 162 163 164 165 166 167 168 169 170 178 179 180 181 182 183 184 185 186 194 195 196 197 198 199 200 201 202 210 211 212 213 214 215 216 217 218 226 227 228 229 230 231 232 233 234 242 243 244 245 246 247 248 249 250 255 218 0 12 3 1 0 2 17 3 17 0 63 0 254 213 30 211 196 102 79 17 248 239 84 190 241 127 140 124 60 124 77 227 155 93 71 195 126 29 241 103 140 124 61 226 93 2 203 195 62 43 214 124 51 167 63 131 180 207 12 120 143 68 208 60 65 167 91 233 90 37 140 247 222 28 185 210 237 188 87 121 121 54 185 172 216 248 147 196 186 149 198 147 224 217 124 143 197 255 0 26 62 20 91 121 182 190 13 241 199 140 188 83 174 147 115 28 30 22 240 215 197 31 31 106 62 36 51 90 222 71 97 113 13 230 155 125 227 24 63 177 69 189 204 233 246 155 191 17 75 164 105 246 112 44 183 151 119 73 105 11 200 223 22 254 210 255 0 240 80 207 134 191 2 254 50 124 71 253 156 126 42 248 103 226 13 246 139 224 221 126 243 91 212 173 252 7 99 160 106 182 30 62 63 18 89 254 37 105 218 111 136 175 53 191 21 248 78 238 195 195 90 70 141 227 29 58 207 91 240 205 165 141 244 126 44 213 160 184 179 213 53 111 248 68 226 190 209 124 79 243 39 196 15 248 42 151 192 31 29 216 221 105 183 190 11 248 195 37 149 211 70 236 146 232 190 16 183 154 41 109 231 142 238 210 234 210 234 215 226 31 159 99 168 217 94 91 218 222 88 234 86 143 21 229 141 229 188 55 118 50 219 77 111 3 39 101 8 115 43 201 90 246 106 214 178 139 73 197 91 85 170 215 68 154 91 251 199 93 42 112 122 206 221 58 54 186 93 53 102 157 189 45 109 53 110 231 234 7 236 223 241 31 226 149 255 0 237 41 103 224 207 19 248 222 239 84 240 207 136 62 11 124 80 241 101 207 132 62 221 168 107 250 94 147 172 248 59 199 159 5 116 175 14 222 90 248 135 196 242 95 248 146 239 81 131 77 241 247 137 236 117 59 187 41 252 59 161 107 144 201 166 207 47 132 109 46 244 203 107 201 10 249 103 254 9 129 241 57 191 104 159 142 190 55 248 139 162 45 228 126 27 248 61 240 171 87 240 13 245 207 137 162 181 211 188 83 174 95 124 98 241 119 129 252 65 225 203 193 165 232 242 234 218 49 26 109 167 193 95 19 65 226 59 229 213 180 245 158 247 80 209 46 52 189 18 56 47 53 8 52 146 176 196 90 53 90 140 82 86 142 137 36 175 104 173 180 232 215 225 231 108 107 242 123 89 114 43 71 221 73 90 218 168 171 233 167 91 179 255 217 )! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: ' 21/4/07 10:33'!
emailSmall1Gif
	"'imgs/intelicom-email.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 15 0 11 0 194 0 0 0 0 0 255 255 255 204 204 204 204 0 0 102 51 204 153 153 153 153 0 102 0 0 0 33 249 4 1 0 0 7 0 44 0 0 0 0 15 0 11 0 0 3 42 8 186 220 16 48 74 40 84 16 56 227 65 70 125 66 81 104 2 97 16 223 69 110 93 138 137 48 105 173 235 28 194 163 102 211 217 206 23 142 224 34 1 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: ' 21/4/07 10:33'!
emailSmall2Gif
	"'imgs/jta-email.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 13 0 14 0 128 0 0 255 255 255 42 42 161 33 249 4 0 0 0 0 0 44 0 0 0 0 13 0 14 0 0 2 31 132 143 23 203 237 137 24 8 112 78 106 163 89 170 111 124 133 223 70 102 28 168 73 213 229 180 220 10 199 114 1 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: ' 21/4/07 10:33'!
emailSmall3Gif
	"'imgs/las-email.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 15 0 11 0 162 255 0 255 255 255 204 204 204 192 192 192 51 102 153 0 51 102 0 0 0 0 0 0 0 0 0 33 249 4 1 0 0 2 0 44 0 0 0 0 15 0 11 0 64 3 45 40 186 220 66 36 200 57 161 138 97 128 189 49 185 26 39 2 214 35 69 90 228 93 80 235 182 236 43 95 217 216 5 159 41 133 100 69 99 184 10 142 70 41 14 19 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: ' 21/4/07 10:33'!
eranovaHeaderGif
	"'imgs/head-en.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 88 2 100 0 179 0 0 255 255 255 231 231 237 223 223 223 204 204 204 180 180 180 161 159 183 153 153 153 116 131 158 153 102 102 111 111 111 100 108 144 57 93 121 69 69 92 24 83 125 51 51 51 15 15 15 33 249 4 4 20 0 255 0 44 0 0 0 0 88 2 100 0 0 4 255 16 200 73 171 189 56 235 205 187 255 96 40 142 100 105 158 104 170 174 108 235 190 112 44 207 116 109 223 120 174 239 124 239 255 192 160 112 72 44 26 143 200 164 114 201 108 58 159 208 168 116 74 173 90 175 216 172 118 203 237 122 191 224 176 120 76 46 155 207 232 52 53 192 86 187 217 112 183 124 78 199 193 219 117 242 61 144 239 251 255 35 123 120 128 92 130 124 91 123 132 138 139 37 134 140 90 142 136 119 143 148 149 26 137 150 87 130 146 152 153 158 148 157 159 82 161 88 145 162 52 164 167 71 131 170 163 172 144 169 173 45 177 178 181 182 97 180 183 39 166 186 189 190 89 113 191 43 188 194 197 198 199 104 155 200 203 204 205 156 175 206 209 99 135 94 208 210 36 193 215 218 107 212 23 214 77 223 18 225 219 228 229 184 2 2 214 227 74 232 232 221 19 233 230 242 243 231 2 3 247 239 235 73 1 3 246 248 20 1 4 208 27 72 16 146 191 127 19 222 69 65 119 175 31 181 128 5 35 74 28 213 112 192 67 133 11 43 82 139 55 177 163 199 125 1 255 57 2 208 183 36 164 187 67 34 63 170 92 249 35 17 73 38 108 210 181 121 201 178 166 205 89 0 49 114 131 167 243 166 207 159 47 122 106 18 8 180 168 209 97 185 166 40 59 202 180 105 135 152 73 161 152 156 228 180 170 51 126 21 179 54 60 132 85 171 69 161 226 218 45 189 84 145 40 133 131 102 19 10 32 80 160 45 129 148 0 14 14 176 208 181 95 5 172 39 169 90 221 43 44 128 1 4 128 3 11 14 76 64 2 129 4 136 19 43 78 96 96 46 6 67 196 42 8 88 64 121 129 130 180 0 14 84 62 208 77 128 230 6 160 67 31 112 44 225 64 232 5 239 10 132 110 112 0 160 88 200 96 249 202 206 20 96 176 109 194 134 23 235 70 76 218 53 236 92 3 42 83 46 156 80 51 229 214 19 10 44 88 205 188 65 129 228 171 49 43 88 77 154 159 204 188 119 224 206 222 158 76 175 135 1 183 111 59 54 176 123 119 190 56 144 219 105 55 44 124 1 231 132 10 42 63 151 160 124 121 115 234 244 241 195 91 173 128 43 58 9 215 169 199 144 255 69 220 21 216 157 119 28 128 23 222 96 115 249 165 152 1 16 146 183 88 111 1 113 5 155 88 24 212 39 92 117 241 13 39 65 112 149 53 96 217 1 159 133 214 31 0 3 232 151 95 104 196 133 36 78 118 1 182 3 12 130 6 214 24 153 6 10 2 70 128 87 255 212 150 24 1 217 73 136 24 113 97 241 81 225 111 21 94 96 156 112 243 197 181 225 72 37 54 160 0 129 252 76 199 98 92 171 53 25 128 149 160 25 105 143 151 175 201 100 210 122 81 68 85 163 108 99 113 64 128 96 96 129 247 99 5 3 40 70 100 146 80 33 233 206 93 75 86 118 34 138 194 9 4 98 101 189 161 184 90 107 1 216 231 220 126 161 205 151 151 128 98 198 164 158 38 105 158 89 160 153 21 172 25 88 155 138 5 42 128 156 240 8 132 36 28 7 229 211 97 123 196 17 32 28 31 26 46 208 36 124 38 178 193 37 114 0 168 22 154 167 118 141 217 232 29 26 89 113 163 164 219 101 227 129 95 130 17 32 236 176 194 146 118 88 98 152 29 203 91 145 159 134 148 85 255 103 194 45 137 156 134 253 5 144 103 160 165 157 198 135 105 160 41 32 206 171 0 198 248 26 168 247 236 216 16 164 52 242 170 46 7 192 46 8 152 1 19 40 155 128 185 229 62 232 165 145 144 221 195 208 107 4 78 240 231 148 29 94 22 235 102 35 141 186 103 5 220 138 136 106 171 130 134 214 96 60 160 138 41 151 190 237 244 203 141 175 235 78 113 207 46 216 214 81 155 187 58 78 32 100 121 137 57 150 100 66 112 52 116 171 163 105 153 74 89 127 26 62 183 228 115 147 17 172 164 182 0 16 128 179 172 82 242 145 87 88 3 58 180 199 190 232 102 92 69 112 12 36 141 25 8 5 48 64 72 142 238 182 56 114 121 64 162 60 18 156 66 27 242 104 114 155 241 211 245 204 78 218 108 65 194 168 53 12 90 58 92 62 231 98 55 120 29 153 93 69 70 203 64 41 21 229 198 102 129 2 222 2 160 0 114 118 249 235 88 58 4 218 67 159 211 29 203 1 53 177 136 155 76 178 189 47 98 132 157 32 234 81 35 45 148 149 17 48 106 97 53 31 231 13 255 151 39 10 16 93 138 179 142 100 141 67 39 137 37 87 220 49 204 45 69 185 16 18 233 65 211 7 84 173 55 222 222 22 26 159 183 241 49 112 0 237 168 193 30 159 31 150 2 6 150 0 8 36 102 192 176 83 183 168 206 111 114 113 5 118 206 155 61 185 165 158 10 129 14 26 114 133 58 156 176 162 235 29 41 113 197 91 161 238 194 174 86 12 32 172 144 34 20 160 64 210 194 50 192 6 3 64 206 117 128 211 191 19 224 126 0 64 54 141 34 3 133 167 17 60 2 152 122 83 188 30 84 36 12 172 140 92 42 251 150 124 0 52 170 62 149 134 122 120 202 210 183 18 197 165 191 117 239 43 48 146 203 91 196 55 139 72 97 97 0 16 74 64 210 18 16 130 141 101 134 1 77 163 221 91 104 71 63 111 233 47 57 132 227 95 30 218 133 0 3 128 69 94 20 18 18 188 70 242 21 111 64 78 61 230 219 17 3 43 55 129 60 189 236 16 169 90 85 172 86 179 0 179 36 236 85 40 113 200 5 196 165 193 182 44 141 131 187 80 29 20 14 147 52 248 255 133 32 62 247 192 219 0 100 8 36 188 157 112 118 251 235 199 2 6 160 191 49 246 239 12 52 68 0 226 136 229 24 121 189 67 135 70 218 81 56 134 86 46 2 64 200 49 127 90 163 191 218 227 158 67 4 82 85 112 80 206 160 40 192 51 21 9 160 0 82 244 97 72 248 33 172 2 144 232 0 87 196 98 35 210 5 9 33 138 192 51 73 19 152 114 24 224 45 151 205 15 141 3 211 93 172 98 248 70 51 64 45 106 18 24 217 29 141 119 136 97 97 80 33 123 48 95 132 12 64 20 151 81 198 44 211 139 86 113 8 25 31 230 8 12 58 198 124 72 91 32 217 168 156 128 138 45 187 179 140 221 52 249 1 140 81 147 6 255 115 87 131 240 88 1 110 162 200 143 199 203 74 163 26 114 190 16 242 146 61 71 100 100 123 86 37 128 6 222 71 144 20 176 94 162 224 180 187 216 245 145 116 12 25 150 37 59 244 158 107 250 115 10 217 92 80 131 138 135 152 29 82 192 155 248 139 80 226 178 130 60 115 10 139 15 6 232 90 60 73 5 167 81 53 103 255 74 22 240 28 115 48 19 0 229 236 173 45 7 112 11 57 161 185 187 128 21 64 118 255 76 105 19 192 185 203 150 70 200 72 187 116 93 110 10 250 16 63 26 239 120 115 100 41 121 254 40 147 2 36 0 111 6 21 71 73 197 152 81 245 25 202 50 5 184 98 48 41 163 68 0 25 81 1 110 97 11 72 73 52 170 216 145 73 165 88 29 66 179 6 145 10 82 240 67 135 173 67 92 76 139 101 33 107 230 162 46 22 147 164 53 197 97 84 17 141 136 68 203 28 234 37 33 57 205 172 218 117 26 186 116 169 66 201 138 144 37 60 114 159 150 193 219 37 225 186 35 127 212 245 174 204 48 97 92 50 249 33 147 245 237 24 108 8 98 177 40 118 145 39 164 108 153 244 138 228 90 17 187 141 245 177 47 86 207 105 203 5 62 59 191 188 113 246 180 242 192 74 9 154 182 173 5 44 86 0 186 51 18 105 72 249 28 218 1 168 95 115 73 43 106 119 235 11 252 33 175 149 24 24 163 2 54 56 187 20 182 111 125 115 65 161 183 148 27 43 188 201 48 148 94 255 84 69 95 211 170 219 167 20 14 127 140 29 129 76 193 1 92 222 50 129 88 17 34 193 0 60 219 26 51 162 80 111 173 81 31 0 248 199 63 168 150 114 36 231 85 165 102 78 49 128 7 56 192 1 15 32 33 0 28 224 186 4 0 119 83 20 32 128 3 48 112 223 160 150 96 192 251 141 193 188 94 32 96 239 94 193 166 34 100 128 126 67 176 150 193 141 119 185 207 33 165 109 219 11 85 245 170 111 126 25 14 173 211 62 17 128 7 236 48 0 14 128 23 127 31 235 183 179 84 173 193 241 66 112 92 136 244 128 22 195 227 111 165 50 11 66 106 12 16 148 206 152 46 62 238 241 33 248 43 14 31 111 12 127 92 9 242 89 238 33 227 34 99 132 184 14 214 42 98 238 43 195 244 193 111 0 243 219 18 134 209 184 49 254 41 135 141 83 226 31 108 51 140 59 211 102 194 0 35 254 80 141 239 139 95 18 18 57 1 108 150 64 2 236 251 0 1 60 32 191 49 6 0 156 241 235 52 252 174 249 206 8 190 115 126 239 59 104 252 22 134 205 15 64 177 255 125 1 80 99 2 208 185 48 123 126 64 154 81 244 104 196 72 128 1 141 169 244 157 41 109 232 253 18 58 210 252 120 244 5 230 28 231 120 137 26 30 116 158 112 148 133 16 68 19 104 134 148 133 49 227 252 86 248 89 254 141 177 185 122 211 48 153 127 41 10 75 83 96 205 36 44 49 138 219 119 105 32 37 90 207 194 106 114 131 225 60 18 30 55 26 193 41 102 180 64 152 93 226 185 32 6 192 251 45 140 179 179 157 179 26 83 155 199 114 38 225 91 236 220 236 0 88 122 131 68 118 64 110 7 236 0 113 31 155 191 231 94 154 176 245 220 100 34 59 154 2 150 46 113 118 87 237 135 243 170 194 215 226 88 51 113 248 11 111 65 231 23 198 134 81 246 128 125 189 109 95 91 154 199 12 15 55 0 208 220 233 109 131 219 219 250 5 119 92 238 156 0 129 240 23 205 27 207 175 199 181 109 112 110 195 152 191 118 22 185 5 96 140 112 70 79 64 227 68 230 54 191 41 225 239 83 216 153 26 32 143 185 186 63 174 95 63 33 120 45 45 95 118 198 37 255 80 99 242 72 28 226 67 215 51 99 86 76 100 139 27 9 227 68 207 201 24 157 102 0 7 120 49 178 73 227 246 177 227 98 114 104 3 137 31 89 135 19 130 91 126 236 18 83 32 186 49 159 185 218 49 0 103 97 205 217 227 240 102 247 91 76 236 232 67 207 171 221 2 54 139 208 163 238 114 59 67 168 206 46 87 58 223 11 90 112 109 207 197 219 240 147 176 224 3 47 103 248 49 28 226 142 119 51 188 68 120 24 185 231 204 235 148 183 116 90 241 126 223 197 234 57 241 36 252 79 213 15 83 99 126 172 253 244 221 28 33 81 24 35 97 62 216 251 179 113 129 115 176 21 255 33 18 66 72 2 208 54 12 236 17 124 251 4 79 220 134 178 95 176 8 19 108 238 118 67 116 135 77 46 126 2 14 225 107 229 67 212 1 233 144 61 31 252 139 34 253 250 215 249 200 6 136 8 141 126 123 236 247 190 234 29 175 126 128 115 74 254 242 155 255 252 232 79 191 250 215 207 254 246 187 255 253 240 143 191 252 231 79 255 250 219 255 254 248 71 156 17 210 254 133 255 18 111 87 5 141 129 122 2 184 3 139 22 6 118 166 106 45 240 127 3 184 128 12 216 128 14 248 128 16 24 129 18 152 7 44 214 65 255 113 1 21 56 129 26 88 52 106 178 0 161 244 58 77 101 1 88 86 26 23 118 1 41 180 129 40 168 43 35 37 20 1 160 74 147 193 55 164 145 91 62 19 91 113 97 49 181 130 55 2 225 90 102 52 23 255 161 101 68 145 129 41 24 132 72 96 46 224 164 128 186 7 15 5 99 91 194 133 92 198 165 62 200 53 25 183 227 50 73 85 102 235 229 92 39 197 84 193 17 31 176 34 132 92 104 4 231 131 24 180 151 33 147 150 25 121 179 55 110 180 94 33 150 51 240 181 79 63 102 70 4 80 94 91 102 70 114 216 26 176 101 132 93 120 135 59 0 66 123 230 62 26 96 63 72 84 130 179 115 134 241 21 90 123 163 92 102 38 107 177 22 135 84 232 111 53 135 135 142 232 3 248 179 96 28 96 25 108 132 66 202 129 14 170 34 136 33 230 70 99 166 28 57 211 31 114 200 7 230 85 91 84 232 134 85 246 250 136 168 200 4 237 148 52 243 49 63 231 165 137 39 164 62 186 118 66 94 100 63 102 132 70 231 37 135 222 178 138 209 149 138 190 248 139 192 24 140 194 56 140 196 88 140 198 120 140 200 152 140 202 184 140 204 88 11 52 33 25 68 97 122 205 56 141 68 128 21 195 34 2 183 248 66 218 192 73 212 248 19 59 242 133 118 120 1 217 56 98 252 128 18 145 245 33 55 102 11 30 212 141 62 241 133 34 132 128 28 128 55 109 177 62 57 67 74 94 196 59 188 147 25 150 81 54 173 64 62 236 200 18 122 216 69 135 165 55 129 181 62 45 88 24 126 104 70 172 53 70 246 19 107 91 40 10 254 248 143 43 225 23 16 50 144 168 212 52 103 232 70 10 25 67 41 36 143 178 176 142 18 121 38 227 8 0 238 161 143 104 212 70 236 245 28 80 166 10 90 20 146 123 49 146 227 69 74 2 177 145 105 100 57 50 121 11 22 233 146 58 185 147 60 217 147 62 249 147 64 25 148 66 57 148 68 89 148 70 121 148 72 153 148 74 185 148 230 16 1 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: ' 21/4/07 10:33'!
eranovaLogoGif
	"'imgs/color-slo-small.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 161 0 40 0 135 0 0 35 38 39 39 42 44 39 44 46 42 44 45 41 45 48 41 48 51 45 50 53 46 53 57 48 52 54 49 54 57 49 57 61 54 60 63 54 61 65 56 63 67 57 65 69 61 68 71 56 67 72 60 68 73 61 71 76 61 72 78 62 76 84 65 70 73 64 74 78 66 76 81 65 78 85 68 77 82 66 80 87 70 80 85 67 81 88 69 82 89 74 83 87 72 83 89 73 85 90 74 86 92 76 86 92 74 88 95 77 88 93 75 92 101 76 90 97 78 92 99 77 93 101 77 95 105 80 92 99 81 94 101 80 96 104 82 101 110 85 98 105 85 100 107 86 101 109 88 100 107 89 103 110 93 104 110 85 103 113 86 104 114 87 106 116 88 104 112 88 107 117 91 109 117 94 105 112 92 108 116 89 109 120 93 111 120 93 113 122 93 114 125 95 117 127 96 110 117 101 112 118 96 112 120 97 115 124 97 116 126 100 115 122 101 118 126 105 117 124 118 118 118 119 119 120 118 120 122 121 122 123 98 119 129 100 118 128 100 120 130 101 121 132 102 124 134 106 121 129 106 123 132 104 124 134 104 126 138 108 127 136 119 126 129 123 126 128 103 128 141 106 128 139 105 129 141 108 130 141 106 132 145 109 133 145 109 135 149 111 137 151 119 128 132 124 129 131 124 133 137 126 136 142 113 134 145 113 137 149 113 138 152 114 140 154 114 142 156 117 141 153 116 142 156 125 140 147 126 142 152 118 144 157 122 145 156 126 144 153 125 146 157 117 146 161 119 149 164 120 146 160 121 148 162 121 150 165 124 147 160 126 148 160 124 150 164 124 152 166 122 153 169 124 154 169 125 156 171 126 157 173 126 159 176 127 160 176 190 157 92 163 157 127 182 155 101 182 156 106 188 158 97 186 158 104 181 159 112 174 160 118 173 162 125 187 162 109 189 161 105 191 163 108 189 164 110 179 161 117 179 163 123 186 164 116 186 168 125 218 155 54 216 156 59 223 160 60 236 153 26 254 152 0 252 155 11 245 155 19 242 156 25 251 157 17 226 153 39 228 156 44 238 158 38 229 158 49 241 158 32 255 161 23 253 162 27 236 160 43 229 160 52 226 161 59 232 161 50 232 164 58 243 161 38 242 162 41 250 163 34 240 164 48 212 159 70 206 160 79 198 160 91 204 161 86 202 163 91 201 163 93 204 163 89 206 164 90 205 165 94 214 160 71 212 161 76 219 161 68 217 162 73 218 164 75 217 164 77 211 164 84 208 164 89 217 166 81 217 168 86 216 169 92 195 162 98 194 165 108 198 168 109 203 168 102 192 166 114 194 168 115 205 175 118 210 172 105 225 164 67 130 132 133 128 134 137 129 137 140 136 139 141 129 140 147 132 144 150 130 147 155 138 146 150 136 147 152 142 159 155 146 159 153 159 159 146 129 150 161 129 153 165 133 152 162 132 154 165 129 154 168 129 156 170 129 158 173 132 156 169 132 158 172 138 155 164 136 157 168 128 159 176 143 160 158 147 161 156 152 160 147 153 162 152 155 164 154 157 165 153 130 160 174 133 160 170 132 161 174 141 162 165 138 162 171 129 160 176 132 162 177 137 163 176 138 166 180 142 168 178 146 164 164 162 156 132 163 160 133 164 162 138 171 161 129 169 163 132 172 163 128 170 166 139 172 168 142 163 165 146 169 168 144 178 165 129 178 169 138 180 176 151 166 175 166 171 175 161 33 249 4 3 0 0 128 0 44 0 0 0 0 161 0 40 0 0 8 255 0 1 9 28 56 80 93 34 93 158 64 89 90 200 16 148 167 88 195 126 117 98 72 177 34 67 95 2 195 237 58 37 239 15 193 116 144 106 113 74 5 42 212 169 81 186 18 169 251 227 81 224 31 107 212 98 202 140 185 141 229 53 102 98 144 137 73 102 141 160 79 64 127 252 80 35 147 19 25 178 43 202 218 112 107 57 144 221 204 153 126 6 254 121 42 147 219 207 106 108 146 93 49 122 148 12 28 109 76 127 226 163 116 201 162 89 75 154 92 69 18 117 182 98 32 64 138 202 90 18 212 18 222 173 182 11 47 197 58 55 80 91 146 191 128 1 179 201 150 44 48 224 49 97 93 102 35 163 196 176 225 100 217 8 110 115 252 183 218 64 107 148 149 44 21 232 167 25 22 202 129 201 104 251 73 206 20 222 182 154 130 193 58 109 169 81 35 138 130 4 218 195 196 122 97 39 112 2 241 128 78 194 134 201 110 103 62 255 56 107 188 27 48 147 56 3 117 83 222 12 136 25 101 49 30 95 126 46 14 248 74 228 129 246 228 214 54 139 139 216 105 99 218 231 254 255 217 23 158 117 39 190 206 65 251 222 141 88 42 25 234 134 153 100 99 25 39 115 84 64 235 166 27 118 198 18 14 124 195 100 12 132 200 118 167 173 242 11 94 154 84 36 136 58 9 18 184 208 47 127 136 241 31 101 203 248 17 93 27 19 6 198 140 71 221 80 134 133 57 2 93 147 25 136 212 100 24 152 64 249 56 120 218 48 164 16 40 200 107 102 221 178 72 37 103 117 98 14 113 148 53 3 7 142 134 93 49 16 54 60 26 7 7 27 234 173 3 136 51 148 49 193 18 32 68 58 166 204 31 218 172 103 24 22 113 140 177 27 32 230 124 82 91 38 149 148 103 209 37 247 120 121 150 32 45 126 233 13 32 143 180 133 25 104 108 176 132 12 104 1 10 84 24 101 112 0 226 87 102 124 48 73 89 50 45 233 23 152 53 127 96 72 89 77 122 92 9 12 107 164 132 195 206 31 240 224 114 218 46 181 108 39 72 38 102 129 194 14 32 227 180 53 205 110 212 252 193 206 110 205 176 164 77 144 127 41 49 218 100 160 245 241 71 147 134 133 193 18 55 73 70 255 229 39 96 88 120 84 205 149 90 226 149 73 58 4 161 51 17 106 250 72 74 155 69 161 24 25 78 91 241 20 9 8 54 187 225 33 144 127 207 121 132 100 146 139 66 67 25 51 2 77 11 224 31 204 82 70 134 71 245 101 150 226 105 178 252 116 32 94 251 228 122 218 62 13 86 84 73 70 109 209 3 26 98 83 237 86 19 32 111 122 203 18 171 83 122 148 158 97 109 254 49 167 97 150 105 107 88 157 128 52 3 154 24 231 226 133 200 79 145 156 230 10 43 8 170 34 73 62 98 190 11 72 57 109 61 2 218 134 205 129 134 197 55 128 124 10 26 127 128 72 168 47 32 86 58 86 231 141 142 213 10 200 123 131 10 164 178 99 100 148 137 23 45 133 244 236 115 46 167 149 226 200 89 154 60 130 142 71 239 180 75 145 198 13 91 116 8 104 157 254 145 175 99 99 132 184 219 53 118 146 250 87 27 2 181 108 24 30 47 81 198 6 80 55 7 166 132 71 37 130 6 71 42 42 110 135 201 60 103 221 179 164 64 224 68 99 247 221 209 128 51 207 40 120 17 255 2 218 82 235 104 157 4 215 128 8 234 152 18 70 162 74 89 53 30 45 67 217 29 33 59 86 211 31 124 84 110 185 229 212 36 35 120 18 213 136 217 246 151 255 152 133 201 162 63 125 180 8 42 172 13 146 164 71 138 59 6 232 31 94 27 6 29 32 208 58 38 31 217 148 157 243 205 172 73 136 81 58 80 217 176 193 187 99 230 124 190 157 56 102 169 242 59 126 243 216 226 185 69 134 236 217 223 110 163 253 33 165 97 27 254 65 179 99 208 177 116 61 96 218 220 234 24 127 76 65 9 71 217 197 213 106 124 109 200 91 68 74 98 2 221 195 150 131 209 59 70 248 246 253 2 2 43 212 2 13 159 68 156 127 88 130 237 180 161 48 195 104 70 42 216 32 195 247 168 243 164 231 173 111 33 237 171 136 36 230 38 144 117 244 162 45 182 104 75 253 8 198 18 244 1 70 25 2 73 27 101 34 99 142 223 8 132 29 188 99 194 58 6 6 152 100 88 8 63 205 16 92 1 41 195 181 80 176 166 18 155 200 161 14 119 200 67 29 114 34 83 22 153 196 79 255 134 118 22 71 28 235 44 134 32 149 57 254 176 142 221 32 236 95 241 49 146 8 29 195 184 63 124 99 129 88 200 6 169 128 3 148 218 25 144 26 226 163 12 214 60 193 154 123 44 239 140 227 170 136 16 125 194 55 179 204 2 16 71 52 203 34 98 102 171 221 88 6 118 11 203 22 104 148 0 34 64 244 129 50 202 240 226 95 150 0 22 44 45 48 9 74 216 70 225 246 120 52 93 176 38 17 247 57 227 79 236 97 150 53 18 100 88 22 129 7 32 164 209 150 57 110 107 145 160 49 146 245 64 19 42 60 70 75 32 127 164 26 254 254 242 36 129 24 14 96 44 89 37 96 102 71 201 211 252 2 98 163 200 165 46 119 57 10 96 180 162 146 62 113 135 210 24 162 137 149 196 209 34 158 12 12 194 100 249 151 217 181 206 48 212 232 223 199 90 114 39 195 196 208 49 209 148 19 104 176 129 59 111 69 165 29 14 180 132 40 22 197 20 157 157 101 17 156 0 38 65 146 102 150 80 92 234 152 21 73 38 96 176 246 7 255 197 105 138 134 137 76 53 169 56 255 144 118 80 102 106 198 153 219 33 149 4 8 116 8 142 139 128 184 203 105 130 49 55 72 176 134 31 103 177 164 64 216 105 145 84 188 179 147 60 66 92 65 77 120 36 145 69 5 159 128 81 66 36 195 104 54 199 100 15 16 207 60 12 218 118 195 205 140 132 211 18 157 168 69 45 126 133 151 99 200 34 162 62 161 104 69 64 129 142 77 182 133 17 178 11 225 110 174 195 175 192 204 174 168 45 156 219 29 38 100 25 129 132 203 164 30 153 97 20 9 242 203 7 82 228 18 236 194 41 65 132 121 22 120 252 33 141 22 1 106 104 252 165 44 150 129 6 26 29 4 13 227 146 243 31 153 9 4 138 202 236 38 247 194 130 14 50 90 117 33 192 56 70 91 38 145 24 76 86 228 24 136 144 196 79 247 211 181 133 45 9 174 180 122 3 98 255 210 158 129 116 8 62 32 19 72 24 22 166 216 221 124 43 44 233 72 167 85 93 33 12 188 72 84 32 243 107 155 88 1 19 153 122 126 108 32 6 155 80 22 41 40 72 208 220 75 178 38 10 76 54 125 162 142 85 255 172 239 18 195 232 172 103 205 101 188 209 38 129 160 251 163 140 179 4 18 37 19 97 161 169 4 65 170 99 194 224 147 216 153 232 58 165 171 199 41 84 116 9 91 232 131 18 172 249 44 32 224 225 215 138 152 130 109 102 241 237 24 62 106 47 169 164 182 56 98 152 79 98 94 121 178 176 156 55 164 82 13 12 65 151 135 142 121 228 162 187 95 18 5 48 248 193 139 151 90 66 187 128 200 135 186 40 82 10 117 128 55 172 129 9 152 114 147 128 133 66 186 4 15 135 12 12 22 156 97 164 210 89 171 56 74 128 110 5 153 137 5 107 176 55 169 146 60 33 57 232 177 136 87 244 34 23 189 136 69 33 30 161 15 127 244 163 24 39 230 133 140 103 76 99 26 15 131 130 5 121 196 49 50 161 9 76 144 2 18 32 130 196 34 134 76 228 33 231 195 25 72 118 198 117 174 145 228 38 59 99 182 4 17 10 51 146 129 133 37 88 121 39 109 184 6 201 150 199 100 39 59 249 117 63 185 6 81 150 192 132 49 56 227 27 127 240 134 151 145 124 13 248 133 248 255 205 112 14 177 59 220 28 231 58 19 132 37 120 182 179 158 93 146 231 61 251 249 207 128 14 180 160 7 77 232 66 27 250 208 62 241 66 9 126 16 20 40 12 196 209 3 73 131 27 232 44 201 52 156 225 140 118 240 194 239 206 224 134 42 32 250 211 160 254 201 19 50 96 7 22 216 192 11 35 168 176 28 74 6 20 12 184 65 32 62 216 67 5 103 61 144 42 156 129 137 2 49 129 26 8 82 225 10 211 0 10 158 106 9 57 231 208 7 57 80 58 212 200 254 243 31 14 16 149 46 80 64 3 27 200 192 23 124 144 5 24 196 192 3 127 88 64 20 76 80 134 0 148 97 2 72 120 128 30 118 224 2 11 176 196 13 3 152 194 13 92 176 129 111 32 96 7 14 232 195 5 140 80 129 62 160 224 5 25 248 67 4 248 176 129 25 188 128 223 51 80 128 30 56 112 6 20 80 97 5 18 152 67 178 23 30 232 60 64 64 32 44 120 66 3 252 224 132 30 56 192 11 5 72 65 2 212 48 128 19 252 193 12 29 224 131 2 254 96 129 50 8 32 5 5 255 200 19 31 34 176 6 2 164 192 0 102 184 0 32 22 64 7 9 0 66 2 92 0 193 31 46 128 135 8 156 192 11 127 96 0 10 170 176 142 4 112 161 4 37 224 2 0 82 160 0 48 48 252 233 123 214 195 200 211 112 128 53 48 128 228 111 112 0 13 136 112 13 42 152 32 10 13 248 195 9 182 240 132 26 248 33 2 78 24 2 54 166 224 145 31 180 160 8 68 168 134 21 80 224 133 60 76 160 5 79 176 131 4 122 80 132 62 52 160 7 57 216 183 23 58 96 129 57 60 129 5 27 64 195 3 186 16 130 118 72 1 234 144 183 51 15 46 16 130 57 112 65 6 36 0 66 19 108 160 7 16 132 160 14 65 248 6 17 220 0 5 25 28 65 15 117 120 2 31 68 16 2 52 8 228 11 36 176 131 8 64 128 134 33 248 65 13 92 216 65 31 206 16 133 61 124 128 4 104 56 2 31 204 240 129 21 236 65 11 42 48 0 29 132 96 247 63 140 0 4 91 136 188 244 253 108 134 7 240 225 216 160 238 195 3 116 96 130 63 204 193 1 95 88 94 2 64 0 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-filetype icons' stamp: ' 21/4/07 10:33'!
excelBigGif
	"'imgs/excel_icon.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 34 0 32 0 135 0 0 0 0 0 0 128 0 128 128 128 192 192 192 192 220 192 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 3 0 0 4 0 44 0 0 0 0 34 0 32 0 0 8 255 0 9 8 28 72 176 160 193 131 8 13 10 88 200 176 161 67 134 9 15 10 40 64 177 162 197 139 5 4 0 136 88 112 34 198 143 20 23 110 84 24 64 64 201 147 38 83 158 4 41 96 128 70 131 40 99 170 76 57 160 102 205 138 19 93 142 28 104 18 164 197 158 31 61 234 36 88 178 0 74 156 69 139 6 173 56 84 96 207 153 25 87 154 180 73 85 39 128 171 88 5 38 53 58 51 105 75 139 3 192 50 29 9 52 0 215 163 81 195 50 21 75 113 192 72 165 70 207 122 228 90 181 238 88 167 102 207 202 53 59 149 109 219 187 4 188 174 60 90 82 237 223 181 109 201 242 141 169 119 106 93 170 128 5 171 60 91 216 111 1 195 110 241 6 69 217 23 241 225 203 111 231 250 44 252 216 38 224 174 168 79 26 190 12 50 115 224 212 168 87 51 189 153 56 225 220 199 151 105 155 174 141 240 54 219 221 187 65 219 246 204 58 183 93 222 18 103 87 53 222 150 182 240 222 196 213 2 119 238 58 249 231 226 211 49 239 84 168 188 244 241 234 220 125 138 23 71 30 126 188 79 240 29 189 171 167 186 189 32 214 247 240 227 203 231 72 159 126 64 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-filetype icons' stamp: ' 21/4/07 10:33'!
excelSmallPng
	"'imgs/ikone-dok/corendal-excel.png' asFilename contentsAsMethod"
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 2 0 0 0 144 145 104 54 0 0 0 4 103 65 77 65 0 0 88 199 252 71 224 2 0 0 0 32 99 72 82 77 0 0 122 37 0 0 128 131 0 0 249 255 0 0 128 230 0 0 117 46 0 0 234 95 0 0 58 151 0 0 23 111 105 228 196 43 0 0 2 130 73 68 65 84 120 156 98 248 15 3 155 14 221 216 120 240 250 134 3 215 214 237 187 178 118 239 165 213 187 46 174 218 121 126 197 182 179 203 182 157 94 186 229 212 191 127 80 101 0 1 196 144 187 33 61 115 109 82 234 242 232 248 133 193 81 179 60 131 39 59 248 246 152 185 183 234 58 53 168 218 85 203 125 254 254 239 237 151 191 115 86 31 48 247 78 253 7 214 4 16 64 32 13 255 113 0 227 2 222 47 223 255 189 255 242 119 234 146 157 187 142 94 129 232 1 8 32 144 134 201 135 250 82 86 36 4 77 247 0 42 58 115 235 116 220 140 80 32 154 179 97 182 113 46 223 151 31 127 223 127 253 59 121 209 118 160 106 8 2 8 32 134 244 85 241 79 223 60 157 124 164 55 107 77 170 83 167 105 201 210 130 196 5 17 238 29 214 215 30 93 51 206 228 159 176 238 26 4 45 221 115 103 201 238 187 64 13 0 1 196 144 184 40 4 104 240 249 187 231 103 29 155 22 189 40 194 170 86 221 40 67 227 192 197 3 64 65 147 100 193 207 63 64 78 2 106 120 248 230 207 194 93 119 205 188 82 0 2 136 33 98 134 59 80 110 194 222 238 136 121 65 211 142 76 4 234 177 46 84 42 152 157 123 255 229 125 147 56 17 184 13 247 95 253 158 187 227 30 80 3 64 0 49 248 247 89 2 85 219 148 203 91 229 203 153 37 75 3 217 30 147 236 173 51 228 22 110 158 111 18 33 10 52 254 213 71 144 13 119 95 254 158 185 237 1 80 3 64 0 49 184 181 232 32 135 76 251 202 86 203 2 57 211 56 169 132 186 24 147 32 113 184 13 183 158 253 158 186 249 17 80 3 64 0 49 216 215 42 226 10 86 83 95 137 23 239 255 60 122 243 231 206 139 223 215 158 252 234 223 248 24 168 1 32 128 24 172 202 37 129 225 109 146 203 7 12 19 147 20 33 147 56 97 160 75 76 130 197 77 124 37 77 61 165 128 170 239 190 248 13 180 225 210 195 95 93 235 158 0 53 0 4 16 3 220 188 153 43 246 189 248 240 247 229 135 191 64 242 249 251 63 79 223 254 121 248 26 164 244 250 211 95 64 242 220 189 159 173 107 158 1 53 0 4 16 66 195 180 101 187 31 189 249 13 119 52 200 221 207 127 193 217 167 110 255 104 92 254 28 168 1 32 128 16 26 128 241 15 244 217 173 231 191 111 62 251 125 227 233 175 107 79 126 95 126 244 251 194 131 159 103 238 254 56 121 251 231 241 91 63 106 150 188 4 106 0 8 32 132 6 96 252 95 122 244 107 222 142 251 192 224 155 186 229 209 196 141 143 122 214 63 233 88 251 180 101 245 179 134 229 207 171 151 190 168 88 252 26 168 1 32 128 16 26 128 28 98 16 64 128 1 0 75 134 252 209 86 255 47 249 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
favicon
	"'imgs/favicon.ico' asFilename contentsAsMethod"
^#(0 0 1 0 2 0 16 16 0 0 0 0 0 0 104 5 0 0 38 0 0 0 16 16 0 0 0 0 0 0 104 3 0 0 142 5 0 0 40 0 0 0 16 0 0 0 32 0 0 0 1 0 8 0 0 0 0 0 64 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 255 255 255 0 0 0 0 0 153 204 255 0 102 153 204 0 191 191 191 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 255 255 255 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 4 4 4 4 4 1 1 1 1 1 1 2 2 1 3 3 1 1 4 4 4 4 1 1 1 2 2 2 2 1 3 3 3 3 1 4 4 4 1 1 2 2 2 2 2 1 3 3 3 3 3 1 4 1 1 1 2 2 2 2 2 1 3 3 3 3 3 1 1 1 1 1 2 2 2 2 2 1 3 3 3 3 3 1 1 1 1 1 2 2 2 2 2 1 3 3 3 3 3 1 1 1 1 1 2 2 2 2 2 1 3 3 3 3 3 1 1 1 1 1 2 2 2 1 1 2 1 1 3 3 3 1 1 1 1 1 2 1 1 2 2 2 2 2 1 1 3 1 1 1 1 1 1 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 255 255 19 156 248 1 16 160 224 0 19 92 192 0 16 160 128 1 19 100 128 3 16 160 128 3 19 108 128 3 16 160 128 3 19 116 128 3 16 160 128 3 19 124 192 7 16 160 224 15 19 132 248 63 16 160 255 255 19 140 255 255 16 160 40 0 0 0 16 0 0 0 32 0 0 0 1 0 24 0 0 0 0 0 64 3 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 191 191 191 191 191 191 191 191 191 191 191 191 191 191 191 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 153 204 255 153 204 255 0 0 0 102 153 204 102 153 204 0 0 0 0 0 0 191 191 191 191 191 191 191 191 191 191 191 191 0 0 0 0 0 0 0 0 0 153 204 255 153 204 255 153 204 255 153 204 255 0 0 0 102 153 204 102 153 204 102 153 204 102 153 204 0 0 0 191 191 191 191 191 191 191 191 191 0 0 0 0 0 0 153 204 255 153 204 255 153 204 255 153 204 255 153 204 255 0 0 0 102 153 204 102 153 204 102 153 204 102 153 204 102 153 204 0 0 0 191 191 191 0 0 0 0 0 0 0 0 0 153 204 255 153 204 255 153 204 255 153 204 255 153 204 255 0 0 0 102 153 204 102 153 204 102 153 204 102 153 204 102 153 204 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 153 204 255 153 204 255 153 204 255 153 204 255 153 204 255 0 0 0 102 153 204 102 153 204 102 153 204 102 153 204 102 153 204 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 153 204 255 153 204 255 153 204 255 153 204 255 153 204 255 0 0 0 102 153 204 102 153 204 102 153 204 102 153 204 102 153 204 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 153 204 255 153 204 255 153 204 255 153 204 255 153 204 255 0 0 0 102 153 204 102 153 204 102 153 204 102 153 204 102 153 204 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 153 204 255 153 204 255 153 204 255 0 0 0 0 0 0 153 204 255 0 0 0 0 0 0 102 153 204 102 153 204 102 153 204 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 153 204 255 0 0 0 0 0 0 153 204 255 153 204 255 153 204 255 153 204 255 153 204 255 0 0 0 0 0 0 102 153 204 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 153 204 255 153 204 255 153 204 255 153 204 255 153 204 255 153 204 255 153 204 255 153 204 255 153 204 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 153 204 255 153 204 255 153 204 255 153 204 255 153 204 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 255 255 19 156 248 1 16 160 224 0 19 92 192 0 16 160 128 1 19 100 128 3 16 160 128 3 19 108 128 3 16 160 128 3 19 116 128 3 16 160 128 3 19 124 192 7 16 160 224 15 19 132 248 63 16 160 255 255 19 140 255 255 16 160 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
folderYellowClosedPng
	"'imgs/suse/suse-folder_closed.png' asFilename contentsAsMethod"
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 3 0 0 0 40 45 15 83 0 0 0 129 80 76 84 69 255 255 255 204 153 51 203 152 50 201 150 48 199 148 46 255 255 255 194 143 41 255 255 153 189 138 36 186 135 33 183 132 30 181 130 28 179 128 26 176 125 23 255 247 145 255 244 142 174 123 21 102 102 102 255 235 133 255 230 128 197 146 44 192 141 39 188 137 35 184 133 31 180 129 27 255 224 122 163 112 10 255 212 110 248 197 95 160 109 7 255 204 102 239 188 86 158 107 5 230 179 77 156 105 3 191 140 38 220 169 67 154 103 1 211 160 58 153 102 0 171 120 18 168 117 15 165 114 12 116 56 156 204 0 0 0 1 116 82 78 83 0 64 230 216 102 0 0 0 1 98 75 71 68 0 136 5 29 72 0 0 0 9 112 72 89 115 0 0 14 194 0 0 14 194 1 21 40 74 128 0 0 0 7 116 73 77 69 7 210 9 10 13 46 39 87 90 186 133 0 0 0 135 73 68 65 84 120 156 101 207 235 14 130 48 12 5 224 110 48 10 58 145 194 16 1 153 136 92 212 247 127 64 55 50 146 37 251 254 245 164 73 79 1 2 140 241 40 246 103 97 37 126 128 134 72 179 211 89 94 108 192 243 171 181 239 21 100 130 168 172 212 174 190 53 119 41 1 226 86 137 67 37 58 2 213 51 60 60 112 32 72 52 199 220 121 226 72 80 107 133 165 243 194 137 32 213 111 108 157 25 23 130 76 55 216 59 43 110 230 176 233 83 124 190 191 110 24 167 101 35 219 132 124 225 175 129 63 253 91 11 53 214 159 30 119 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
folderYellowOpenedPng
	"'imgs/suse/suse-folder_opened.png' asFilename contentsAsMethod"
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 3 0 0 0 40 45 15 83 0 0 0 117 80 76 84 69 255 255 255 204 153 51 203 152 50 201 150 48 199 148 46 255 255 255 194 143 41 255 255 153 189 138 36 186 135 33 183 132 30 181 130 28 179 128 26 176 125 23 255 247 145 255 244 142 255 224 122 102 102 102 255 235 133 255 230 128 197 146 44 174 123 21 192 141 39 184 133 31 180 129 27 220 169 67 158 107 5 255 204 102 248 197 95 156 105 3 154 103 1 255 212 110 211 160 58 153 102 0 171 120 18 168 117 15 165 114 12 163 112 10 160 109 7 41 224 143 149 0 0 0 1 116 82 78 83 0 64 230 216 102 0 0 0 1 98 75 71 68 0 136 5 29 72 0 0 0 9 112 72 89 115 0 0 14 194 0 0 14 194 1 21 40 74 128 0 0 0 7 116 73 77 69 7 210 9 10 13 46 54 61 234 154 119 0 0 0 135 73 68 65 84 120 156 101 143 235 18 130 32 16 70 215 11 162 133 193 162 80 136 90 118 123 255 71 140 221 154 198 25 206 191 239 204 94 1 50 138 162 172 234 125 22 68 179 23 50 33 218 238 112 84 61 137 242 164 181 209 154 235 122 76 162 178 131 37 134 4 139 218 252 243 87 140 134 112 222 251 241 124 81 10 160 225 204 35 68 144 19 130 143 49 186 217 74 234 11 114 65 104 57 91 110 12 114 69 232 102 23 175 63 110 114 75 99 211 61 238 254 120 190 222 211 178 110 72 123 112 79 254 107 198 7 32 212 11 87 225 11 89 100 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!DefaultWebStyle methodsFor: 'frame printing' stamp: ' 21/4/07 10:33'!
headerBanner
	^WebElement newId: #banner! !

!DefaultWebStyle methodsFor: 'frame printing' stamp: 'JM 4/26/2007 13:44'!
headerDashboard
	"login, font size selection, etc"
	^(WebElement newId: #dashboard)
		add: self headerLogin;
		add: ((WebElement newId: #switcher)
			addHelpLink; addNbSp;
			addGif: #textFontSizeGif size: 54@15; 
			addLinkTo: '/' gif: #textSizeSmallGif title: 'Small font size' size: 16@15; 
			addLinkTo: '/' gif: #textSizeMediumGif title: 'Medium font size' size: 16@15; 
			addLinkTo: '/' gif: #textSizeLargeGif title: 'Large font size' size: 16@15; 
			yourself);
"		add: (WebElement new style: '{flow: right}'; addGif: #cornerDashboardRightGif; yourself);"
		yourself! !

!DefaultWebStyle methodsFor: 'frame printing' stamp: 'np 8/21/2008 17:53'!
headerElement
	^(WebElement newId: #header)
		add: ((WebElement newId: #logo)
			addImage: '/img/aidabannergif.gif'; yourself ); yourself! !

!DefaultWebStyle methodsFor: 'frame printing' stamp: 'JM 4/26/2007 13:44'!
headerLogin
	| e name |
	e := WebElement new.
	name := self app user isGuest 
		ifTrue: [self guestUserText] 
		ifFalse: [self app user name, ' ', self app user surname].
	e addNbSp: 3; addGif: #keyGif; addTextBold: ' ', name.
	self app user isGuest 
		ifTrue: [e addLinkTo: self site admin text: ' | ', self loginText view: 'login'] 
		ifFalse: [e addLinkTo: self site admin text: ' | ', self logoutText view: 'logout'].
	^e! !

!DefaultWebStyle methodsFor: 'frame printing' stamp: ' 21/4/07 10:33'!
htmlHeaderElements
	self app addMetaDescription: 'Demo of AIDA/Web Smalltalk Web Application server'.
	self app addMetaKeywords: 'Smalltalk, Web, Web Server, Web Applications, HTML'.! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: ' 21/4/07 10:33'!
ieBigGif
	"'imgs/ie_icon.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 35 0 34 0 135 0 0 27 67 106 45 70 89 45 74 94 55 77 92 56 78 95 44 75 98 32 79 115 42 80 115 44 86 126 46 90 126 52 81 101 59 85 97 48 85 114 48 90 125 59 90 114 60 94 122 61 98 122 66 92 102 69 96 106 72 101 111 68 98 114 80 111 126 89 115 121 29 85 136 29 97 155 36 90 141 37 91 145 37 98 143 35 104 153 54 102 143 63 105 138 55 106 150 56 117 154 39 120 170 34 120 177 49 111 163 48 114 163 57 120 167 50 124 190 43 126 197 71 106 131 66 110 143 76 113 135 68 114 146 73 122 145 74 122 154 82 113 128 90 119 128 91 124 131 84 123 144 66 127 183 37 129 182 49 132 188 57 129 180 58 132 189 38 138 205 45 133 198 45 133 201 37 134 210 37 137 212 37 142 220 42 134 213 45 134 219 42 138 212 42 143 221 39 148 219 51 132 195 53 137 204 56 130 198 58 134 202 49 134 216 54 157 219 46 139 225 37 147 225 38 153 226 43 148 229 46 150 232 41 155 229 45 154 232 54 142 225 49 143 232 51 148 228 50 148 234 50 159 224 51 155 236 56 145 229 59 155 234 57 157 241 57 167 220 44 166 233 38 164 243 37 171 249 40 172 247 45 177 238 51 164 233 57 166 242 57 179 237 55 182 245 59 193 245 95 132 138 87 131 157 91 132 153 93 136 144 92 136 155 69 134 180 67 130 185 69 139 189 78 133 177 79 139 183 73 146 179 84 131 162 87 142 167 86 142 170 91 142 161 87 143 180 93 147 171 83 151 178 101 132 139 108 138 142 98 133 145 99 139 144 108 141 147 107 146 150 117 147 148 115 155 157 122 150 148 126 155 149 124 155 153 102 152 165 104 157 187 114 159 179 115 162 172 123 165 166 120 163 175 117 166 191 64 134 193 66 138 196 69 151 198 66 152 223 93 159 198 64 149 224 72 161 200 69 162 219 91 170 201 82 175 213 90 184 216 70 164 234 78 171 233 64 162 245 68 172 247 67 171 248 73 175 248 73 186 231 66 177 246 64 191 247 72 180 246 73 182 248 74 185 247 88 177 233 83 180 243 81 190 248 95 184 243 97 158 196 102 168 197 105 181 202 106 186 216 112 172 195 113 181 202 112 191 204 120 178 207 126 188 195 120 185 203 119 184 211 98 179 227 68 197 245 75 211 247 83 201 235 88 195 249 85 214 245 89 227 249 116 194 206 122 197 212 96 197 225 98 198 250 105 215 245 118 205 238 120 214 249 100 233 252 105 241 251 120 237 254 118 247 254 129 156 151 132 163 156 137 167 157 140 171 159 130 164 163 133 168 162 130 168 171 137 171 164 141 173 169 142 178 172 140 186 185 144 171 161 150 180 165 149 182 173 150 184 175 153 180 167 152 182 169 156 186 171 158 189 176 161 190 173 161 191 176 140 189 194 154 198 189 164 194 175 164 195 178 165 203 188 169 198 178 172 202 181 171 204 185 175 209 190 177 207 184 180 209 186 185 214 189 188 217 190 137 198 203 135 198 212 141 211 217 146 194 195 149 202 207 157 205 201 148 206 217 146 210 209 133 204 243 134 213 231 128 216 250 152 215 230 149 216 243 141 224 232 137 226 253 135 240 254 149 226 232 157 233 253 152 240 254 165 204 193 166 208 198 172 209 194 162 214 213 177 212 193 182 216 195 184 215 192 189 219 192 165 231 252 166 248 254 180 254 255 192 220 192 33 249 4 3 0 0 255 0 44 0 0 0 0 35 0 34 0 0 8 255 0 255 9 28 72 176 160 193 131 8 19 42 36 136 143 156 169 81 163 200 45 156 56 240 91 181 71 144 34 141 162 211 6 75 155 102 221 192 81 60 8 238 98 36 90 128 198 224 225 195 7 80 34 61 215 190 237 155 104 207 92 57 113 226 106 209 34 133 201 150 25 24 122 140 117 19 152 141 212 27 102 67 19 238 171 119 174 221 188 121 240 122 237 154 245 234 21 172 70 115 136 73 155 249 239 27 74 100 221 184 26 196 119 142 159 191 126 80 125 241 218 245 202 149 24 49 154 192 36 34 164 109 32 171 65 200 184 33 180 199 206 159 191 121 239 220 185 227 69 85 76 152 46 136 195 132 57 2 8 90 183 124 166 134 53 243 38 118 32 190 190 254 222 169 187 119 15 87 97 196 89 186 100 201 242 5 204 148 51 207 178 37 51 150 77 95 229 127 250 250 246 91 167 46 95 57 92 175 192 96 81 162 100 116 150 38 77 72 127 57 146 7 218 181 107 220 94 255 35 215 175 159 174 113 251 106 201 122 133 101 6 7 14 34 130 40 1 222 196 137 151 47 153 74 252 255 186 230 77 100 65 110 234 250 229 66 183 15 149 44 89 88 72 192 248 229 39 15 139 27 220 153 120 255 98 69 133 51 202 6 81 243 78 46 225 252 227 158 44 96 132 96 65 54 3 45 243 70 16 77 44 225 4 19 84 124 81 201 7 125 100 99 30 65 226 228 82 14 56 170 192 34 139 43 93 24 208 71 82 2 5 131 131 19 75 48 65 225 21 150 216 240 130 52 223 20 4 78 43 225 148 4 75 91 135 113 0 194 27 115 4 57 7 28 57 48 209 226 139 150 56 66 193 127 5 97 115 202 55 250 68 210 22 24 162 41 17 68 16 60 4 1 196 150 75 28 41 197 21 87 84 242 137 3 202 132 68 16 54 216 252 115 138 43 36 138 198 93 119 78 196 233 162 139 95 130 121 201 45 14 248 81 151 65 213 96 162 73 24 163 53 161 196 14 58 232 176 67 15 63 244 160 104 15 62 24 129 196 19 85 72 18 202 3 123 104 104 16 36 127 186 105 37 6 23 100 144 129 6 26 124 234 233 167 160 114 240 193 7 14 236 161 205 134 2 37 3 134 38 154 42 255 145 4 0 126 36 165 220 72 143 252 249 197 111 178 46 145 129 48 222 16 116 204 26 105 200 144 134 26 137 136 146 138 33 198 88 35 211 153 175 134 225 69 22 113 46 145 68 15 119 64 19 236 63 221 8 226 3 20 95 126 17 203 59 242 4 114 135 179 5 153 162 88 24 187 82 193 98 151 39 144 193 140 54 219 8 210 3 184 95 124 1 202 58 253 180 243 0 12 232 14 164 143 35 97 76 225 68 190 94 76 104 36 20 72 16 113 72 17 223 86 168 175 46 253 240 227 198 0 170 178 74 143 23 97 100 176 193 119 95 92 49 161 20 224 66 1 174 20 21 110 18 139 46 243 240 147 74 0 46 48 73 16 57 91 76 81 0 5 53 88 145 239 21 84 80 33 197 207 40 231 219 201 45 235 180 204 74 1 18 4 179 106 65 171 108 65 131 2 118 252 129 70 20 148 80 2 230 213 148 124 193 73 44 184 144 219 78 32 10 68 208 71 192 4 157 178 5 9 20 108 5 141 27 38 60 97 69 213 148 84 194 9 40 183 232 178 78 58 169 180 16 192 4 126 56 150 203 170 64 102 143 80 193 182 208 228 177 194 8 68 48 162 136 36 147 120 210 74 43 139 164 80 192 2 47 8 147 205 55 127 11 68 141 22 52 184 176 237 63 210 0 211 66 7 8 28 128 64 2 13 48 160 192 0 18 192 0 12 52 218 212 136 16 62 63 104 81 67 49 250 216 67 142 33 139 200 17 3 12 99 212 33 188 29 121 0 179 140 52 218 148 183 80 33 57 240 192 195 15 67 32 194 70 25 121 28 19 141 52 211 92 99 205 53 217 36 143 57 69 222 28 67 70 10 30 160 240 66 29 192 100 227 141 55 223 180 143 185 72 183 142 36 255 66 1 1 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-filetype icons' stamp: ' 21/4/07 10:33'!
imageSmallPng
	"'imgs/ikone-dok/corendal-jpeg.png' asFilename contentsAsMethod"
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 2 0 0 0 144 145 104 54 0 0 0 4 103 65 77 65 0 0 88 199 252 71 224 2 0 0 0 32 99 72 82 77 0 0 122 37 0 0 128 131 0 0 249 255 0 0 128 230 0 0 117 46 0 0 234 95 0 0 58 151 0 0 23 111 105 228 196 43 0 0 2 149 73 68 65 84 120 156 98 248 15 6 237 43 239 160 161 142 213 247 90 22 95 3 50 94 127 252 249 31 9 0 4 16 3 92 195 127 84 240 239 223 255 159 127 64 226 229 91 94 165 148 246 174 218 176 251 193 227 231 63 126 254 2 8 32 236 26 32 170 191 252 250 239 178 236 59 16 57 77 126 80 215 62 253 224 209 115 111 222 125 4 8 32 132 134 95 191 126 161 169 126 251 237 255 153 171 32 180 106 243 215 250 206 45 211 231 173 186 126 243 62 64 0 33 52 252 248 241 227 255 252 138 239 185 166 255 75 205 255 247 187 253 95 19 250 255 96 226 163 143 255 239 188 251 127 237 245 255 93 39 191 20 86 117 29 58 118 22 32 128 16 26 190 125 251 246 191 61 108 103 130 225 255 114 203 255 211 60 255 111 141 250 127 119 230 255 243 101 127 87 133 76 92 123 110 239 217 207 9 25 149 219 119 31 6 8 32 132 134 207 159 191 108 183 50 202 55 209 216 100 42 124 194 90 230 170 147 252 171 72 173 119 177 58 183 61 149 218 231 30 223 117 230 115 84 114 201 150 29 7 0 2 8 161 225 221 135 79 15 38 198 205 138 54 90 144 164 243 166 217 244 115 143 249 247 125 169 95 187 205 95 181 219 119 172 60 183 253 228 39 136 6 128 0 130 106 104 94 116 245 229 155 15 215 23 214 111 175 13 78 118 214 187 61 33 252 217 204 144 111 87 170 190 223 174 191 57 57 161 121 229 149 173 39 160 26 0 2 8 170 161 113 206 249 39 47 223 29 154 51 117 87 71 85 132 163 203 246 230 130 203 243 75 158 30 107 122 123 170 230 224 132 218 154 229 183 54 30 253 8 209 0 16 64 80 13 181 83 79 220 127 246 102 205 244 53 253 181 75 115 50 119 44 107 155 242 248 72 211 175 231 109 255 159 53 108 157 62 185 116 233 189 245 135 63 64 52 0 4 16 84 67 85 255 161 219 143 94 205 152 116 168 171 113 219 132 198 245 147 155 214 172 154 48 251 230 174 182 39 7 234 230 246 172 40 89 241 108 245 65 168 6 128 0 130 106 40 235 216 125 245 254 139 139 47 254 174 187 244 109 229 133 111 75 206 126 91 112 250 219 156 147 223 166 31 255 54 249 200 183 165 23 255 174 216 255 30 162 1 32 128 160 26 138 155 182 94 188 253 236 226 139 255 103 158 253 63 250 232 255 193 7 255 119 223 253 191 237 214 255 141 55 254 175 186 242 127 233 165 255 75 247 190 139 78 41 221 186 243 32 64 0 65 53 228 215 172 155 187 246 230 172 21 215 102 44 187 60 125 201 165 153 11 207 76 159 119 114 218 156 99 83 103 30 153 60 237 96 247 196 109 221 125 59 210 11 234 119 237 61 10 16 64 80 13 103 206 95 233 155 178 32 45 191 14 104 12 208 106 52 4 20 4 74 1 21 0 149 1 4 24 0 49 85 52 237 131 13 12 50 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: ' 21/4/07 10:33'!
infoBigGif
	"'imgs/info.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 35 0 34 0 135 0 0 1 1 61 1 1 73 9 27 70 13 30 74 1 18 80 3 30 88 1 1 100 1 8 101 1 11 110 1 12 125 1 25 127 14 35 81 19 38 77 20 41 83 25 45 84 27 49 87 28 50 88 20 52 110 27 54 100 31 56 112 33 52 90 37 56 93 35 59 98 33 60 106 40 63 103 35 64 110 43 67 102 45 66 101 38 68 115 41 76 124 51 69 100 50 71 105 55 73 102 57 77 109 50 75 117 54 81 127 62 83 121 68 86 111 67 90 121 76 99 126 82 100 122 1 28 131 1 28 139 1 33 141 5 45 129 1 39 146 1 43 151 1 43 154 3 49 143 1 48 156 25 63 137 1 48 163 11 63 166 29 68 141 27 74 170 27 79 189 28 81 190 44 74 132 32 72 147 50 80 133 62 95 135 61 100 149 63 98 155 41 94 204 44 96 204 64 99 142 75 101 135 66 99 155 89 108 129 82 110 149 95 112 131 93 117 141 78 109 162 87 125 167 99 119 137 103 123 138 105 124 138 97 118 150 96 127 163 87 128 172 93 138 185 109 128 142 110 132 147 113 133 147 124 142 150 100 136 172 106 129 167 97 138 180 104 140 177 102 145 190 109 146 185 113 138 180 114 144 179 112 149 190 123 155 188 79 134 223 82 133 227 100 135 201 106 141 193 117 150 201 116 154 192 123 149 192 120 151 200 124 154 201 111 161 214 121 160 198 113 161 212 115 170 223 125 168 215 128 146 154 136 154 159 137 155 160 128 153 177 128 159 191 136 156 189 143 161 164 132 162 191 145 162 165 150 168 170 155 173 171 150 174 180 170 175 189 165 182 178 174 191 183 173 190 184 133 157 199 131 157 200 130 163 196 129 165 206 132 169 202 139 170 196 139 172 203 129 168 218 138 175 212 140 178 211 148 173 194 146 174 204 153 174 194 153 174 212 147 177 205 154 176 195 154 178 203 146 180 212 147 182 216 150 184 219 152 182 211 158 185 213 155 187 216 146 188 227 162 183 204 166 185 202 171 187 195 169 187 201 163 189 212 178 191 205 177 195 188 178 196 188 180 197 188 158 193 229 156 196 236 157 193 241 173 195 195 167 192 214 172 197 219 180 195 205 185 195 205 187 204 194 177 196 213 176 198 222 180 201 223 184 198 211 188 201 214 163 196 229 164 198 232 170 198 226 170 199 232 172 201 231 163 203 244 165 207 249 167 209 253 172 210 250 180 204 227 180 207 235 184 206 228 188 210 229 178 212 245 176 212 248 182 216 241 180 217 252 185 215 246 185 216 245 187 221 253 189 226 255 195 205 215 193 209 197 198 216 206 201 218 203 197 210 210 192 210 218 202 211 221 210 214 220 210 216 221 220 222 222 198 217 235 203 220 238 196 218 243 194 222 250 220 222 224 207 224 208 208 224 207 208 225 208 212 228 209 215 231 212 214 233 223 219 235 214 223 239 217 196 226 254 202 228 254 205 232 255 212 227 228 221 227 235 223 236 229 219 234 236 210 226 242 209 230 253 211 234 254 218 230 243 221 233 244 219 237 254 211 243 255 220 241 255 223 249 255 224 240 217 226 242 220 227 244 221 230 246 221 231 248 223 228 228 226 225 229 233 230 232 235 232 231 231 236 237 237 227 235 243 225 239 255 230 246 226 227 242 254 227 250 255 234 245 254 236 250 255 240 240 238 245 245 243 241 246 251 243 251 254 249 248 246 253 254 254 0 0 0 33 249 4 3 0 0 236 0 44 0 0 0 0 35 0 34 0 0 8 255 0 217 9 28 72 176 160 193 131 8 19 178 163 183 205 93 188 135 249 30 202 243 54 44 154 66 133 197 144 233 219 232 175 163 199 126 253 252 241 187 100 241 34 193 110 238 64 246 227 231 177 84 30 125 30 249 133 28 105 82 224 54 120 249 96 122 244 71 2 0 128 2 33 61 238 219 231 111 149 169 139 219 218 189 35 186 243 88 128 20 42 12 108 217 41 82 30 63 102 124 18 38 93 26 147 165 188 3 49 190 204 80 36 242 30 75 127 251 228 221 27 181 231 96 53 101 238 152 242 219 199 175 174 191 62 54 192 132 41 123 175 47 203 123 227 238 193 57 90 240 216 51 121 29 231 210 197 119 86 36 223 123 245 34 179 28 55 110 217 148 130 213 144 113 187 135 246 30 62 207 68 207 242 235 107 47 242 188 211 230 248 225 251 102 174 201 29 130 198 146 113 171 219 183 54 62 43 19 114 236 136 64 64 14 191 122 243 204 145 27 94 15 31 184 111 163 148 148 100 87 234 216 184 207 145 33 123 238 195 34 129 138 21 8 202 240 19 62 156 92 184 112 246 192 45 255 195 21 34 171 192 82 193 196 65 54 61 79 50 63 29 45 98 164 240 195 207 187 247 239 225 204 125 195 133 139 194 156 129 165 172 242 205 61 168 205 67 142 57 230 212 195 15 18 41 200 119 6 62 247 129 19 14 56 18 102 115 11 46 16 184 49 16 41 2 34 216 221 112 249 225 51 68 131 41 60 24 142 54 218 128 131 162 54 225 56 115 203 45 15 104 40 144 37 170 48 195 221 125 223 145 99 207 136 242 153 97 207 137 217 100 163 77 144 218 52 83 139 43 49 14 148 137 38 198 128 131 227 132 223 157 195 99 10 62 14 153 13 54 87 94 105 11 42 143 100 56 80 40 152 156 210 204 119 43 170 24 206 57 62 144 56 198 57 87 98 227 166 155 189 184 50 74 17 14 212 49 16 52 162 96 146 74 54 42 90 41 164 57 105 202 55 134 57 111 2 99 232 47 175 100 226 200 5 32 152 39 208 33 150 88 82 11 54 67 186 41 164 148 41 184 160 128 24 246 96 3 204 47 191 96 195 203 43 146 76 146 196 0 75 72 67 144 53 116 36 82 73 45 206 96 255 233 166 54 190 200 16 3 13 46 212 176 139 167 191 0 51 106 36 139 144 177 128 7 119 168 83 80 40 130 28 210 136 43 190 184 89 14 32 7 188 112 3 16 63 224 0 3 33 162 234 194 138 35 139 164 145 129 3 83 44 71 16 30 94 48 210 136 36 172 228 2 140 46 160 240 34 75 44 177 0 51 139 44 186 180 2 201 34 139 116 193 1 3 70 152 178 206 65 235 216 81 69 178 136 68 50 73 186 188 248 98 232 186 158 56 130 200 32 129 60 209 64 3 68 108 98 44 66 233 232 113 4 22 116 8 34 72 32 129 12 98 136 39 180 196 2 201 195 127 92 193 193 2 21 68 209 201 197 9 173 35 12 21 66 240 224 132 23 94 196 17 199 32 159 124 82 136 22 65 116 32 0 5 40 212 65 76 77 2 89 211 9 21 68 152 128 129 8 92 240 28 11 32 18 48 224 1 17 111 8 99 13 210 171 90 99 141 41 37 140 80 8 37 179 176 81 1 19 124 16 131 14 215 9 17 227 65 22 148 196 162 198 6 175 177 125 17 31 26 64 17 11 26 22 216 30 97 183 73 166 156 160 198 26 22 252 247 183 73 82 244 176 129 157 135 155 164 71 27 156 52 46 185 65 1 1 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: ' 21/4/07 10:33'!
javaBigGif
	"'imgs/sunjava-java_logo.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 38 0 66 0 196 31 0 254 35 13 52 71 158 255 145 134 154 163 207 205 210 231 254 49 28 255 228 226 254 76 59 254 115 101 103 117 182 255 173 165 255 242 241 255 201 195 230 233 243 255 214 210 243 244 249 254 90 74 254 63 43 128 140 195 192 198 225 116 129 188 77 94 170 254 132 119 255 187 180 167 175 213 179 186 219 90 106 176 255 159 150 218 221 237 141 152 201 65 83 164 255 255 255 33 249 4 1 0 0 31 0 44 0 0 0 0 38 0 66 0 0 5 255 224 39 142 100 105 138 203 169 174 236 136 180 112 252 33 151 108 159 136 112 239 227 161 243 55 3 96 3 188 65 0 142 162 108 3 128 40 99 10 0 242 217 18 72 21 212 213 2 33 125 161 20 8 98 214 113 144 90 68 6 75 1 145 202 50 10 87 145 160 80 168 101 63 81 64 33 185 40 31 12 119 31 14 112 123 31 125 0 7 109 119 17 82 118 22 122 138 89 66 136 40 93 129 34 14 82 63 12 150 151 31 112 155 82 5 128 151 86 94 159 157 129 11 116 109 86 82 98 129 153 94 11 140 122 158 120 0 118 131 82 183 120 5 109 143 17 189 120 94 86 118 74 13 25 3 20 24 36 12 34 7 167 35 29 9 21 1 20 15 43 15 25 26 1 18 28 42 2 88 43 18 1 26 42 13 30 1 222 224 42 15 4 4 202 20 9 18 13 36 214 4 42 3 214 235 253 254 255 21 40 12 104 71 162 91 62 22 240 148 13 88 184 48 3 60 123 43 26 172 203 214 162 193 0 32 15 186 93 108 49 193 27 139 6 4 6 80 20 145 49 192 198 22 24 250 255 85 72 192 178 101 183 126 30 154 125 152 224 33 1 65 24 19 36 240 251 183 142 30 134 118 29 58 64 4 2 18 30 129 155 119 8 72 24 70 130 67 0 15 29 6 24 77 56 32 1 133 131 200 118 242 244 183 244 198 131 9 11 39 136 200 64 65 235 186 128 24 134 146 24 89 130 0 5 111 19 216 142 40 42 247 68 7 172 35 50 172 235 144 133 67 135 10 106 69 188 237 151 96 0 134 135 238 140 98 88 150 192 28 134 108 19 240 126 8 41 33 129 186 173 152 3 172 28 48 193 222 182 4 231 90 188 131 183 152 225 66 163 67 27 128 109 236 225 228 138 9 212 36 152 142 172 112 225 188 151 53 165 238 0 9 214 180 239 195 146 153 10 47 162 0 130 147 225 174 134 127 72 142 156 23 10 6 208 73 25 128 78 194 1 3 82 11 160 95 255 96 192 1 41 57 188 12 148 145 98 6 215 20 17 154 196 147 239 114 1 192 15 240 0 12 173 17 32 224 72 174 73 103 184 55 250 148 163 126 156 18 204 145 176 138 123 31 248 33 66 123 0 124 55 2 159 40 38 4 72 194 17 58 48 145 224 114 0 8 99 2 132 13 58 103 0 2 112 144 167 67 38 67 20 8 128 23 27 214 162 73 134 241 237 18 129 5 2 48 242 3 35 16 44 176 159 1 112 172 216 34 129 0 242 194 197 113 31 96 248 193 35 0 68 81 128 8 92 28 48 130 143 36 36 119 68 126 34 134 162 199 136 34 28 113 74 25 239 141 144 28 23 5 40 160 192 120 239 117 8 192 51 51 92 177 229 137 57 166 232 101 1 84 186 32 133 133 130 156 153 102 9 101 28 103 192 6 226 44 48 221 119 211 109 55 194 156 117 222 89 66 30 85 222 34 161 145 194 57 16 65 160 183 132 0 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-bullets' stamp: ' 21/4/07 10:33'!
keyGif
	"'imgs/fast/fast-key.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 18 0 8 0 145 2 0 209 173 75 221 185 87 245 245 236 245 216 100 33 249 4 1 0 0 2 0 44 0 0 0 0 18 0 8 0 0 2 32 148 134 99 203 156 217 226 155 20 29 4 64 157 33 229 20 116 67 216 5 23 103 132 139 153 62 108 212 144 47 108 20 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'frame printing' stamp: 'mivsek 1/10/2008 17:35'!
lastChangedElement
	| e |
	e := WebElement newId: #sitePageChanged. e table width: 1.
	e cell align: #center; addRulerSize: 1; addBreak.
	e cell addText: self lastChangedText.
	self observee modifiedTimestamp notNil ifTrue: 
		[e cell addTextBold: self app observee modifiedTimestamp asDate shortPrintSloString].
	self app user isAdmin ifTrue: 
		[e cell addText: ' visits: '; addTextBold: self app visitsCount printDotString].
	^e! !

!DefaultWebStyle methodsFor: 'frame printing' stamp: 'mivsek 6/13/2007 16:50'!
lastChangedText
	^'Updated: '! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
leftCornerGif
	"'imgs/fast/fast-left_corner.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 4 0 4 0 128 1 0 255 255 255 229 236 243 33 249 4 1 0 0 1 0 44 0 0 0 0 4 0 4 0 0 2 6 132 17 97 169 12 5 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
leftStripeGif
	"'imgs/fast/fast-left_stripe.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 8 0 1 0 128 0 0 170 167 153 0 0 0 33 249 4 0 0 0 0 0 44 0 0 0 0 8 0 1 0 0 2 3 132 111 5 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-filetype icons' stamp: ' 21/4/07 10:33'!
multimediaSmallPng
	"'imgs/ikone-dok/corendal-wave.png' asFilename contentsAsMethod"
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 2 0 0 0 144 145 104 54 0 0 0 4 103 65 77 65 0 0 88 199 252 71 224 2 0 0 0 32 99 72 82 77 0 0 122 37 0 0 128 131 0 0 249 255 0 0 128 230 0 0 117 46 0 0 234 95 0 0 58 151 0 0 23 111 105 228 196 43 0 0 2 143 73 68 65 84 120 156 98 248 15 3 237 43 239 160 161 142 213 247 90 22 95 43 105 216 116 247 238 221 175 95 191 66 148 1 4 16 3 92 195 212 67 95 255 254 251 143 9 128 26 10 23 159 56 122 244 40 68 15 64 0 33 52 76 218 251 1 168 1 19 101 101 207 222 119 234 191 211 212 39 16 61 0 1 132 208 208 191 245 5 80 250 231 159 255 63 254 128 72 56 3 168 1 130 106 27 87 3 245 0 4 16 66 195 132 29 175 222 205 153 0 68 83 118 39 47 221 183 98 201 201 55 39 159 254 255 252 243 255 231 31 32 18 104 73 85 221 242 206 206 78 128 0 130 106 248 182 116 210 207 16 157 255 110 74 64 84 124 128 49 101 33 99 65 191 209 244 157 231 151 93 253 255 250 219 255 55 95 255 239 61 245 191 164 98 113 97 97 33 64 0 129 52 124 95 53 253 79 144 250 255 134 248 175 179 59 239 110 234 88 117 43 170 110 47 95 204 36 198 168 114 217 174 237 119 55 220 250 255 228 211 255 93 167 254 231 23 205 3 106 0 8 0 49 0 206 255 0 248 235 231 247 88 47 255 125 91 244 175 155 144 146 146 214 220 226 240 235 248 211 225 189 125 173 37 81 120 1 102 123 97 119 192 254 219 237 255 183 204 255 104 106 151 113 113 113 2 136 225 203 250 57 127 60 197 255 79 46 251 246 249 109 94 94 30 80 104 222 188 121 239 95 127 219 113 126 114 116 179 136 109 132 66 214 238 255 151 95 254 223 114 230 127 98 242 4 160 44 64 0 49 108 158 181 74 189 246 126 252 214 159 16 13 106 106 106 140 140 140 201 201 153 64 61 47 15 171 28 95 46 149 177 231 223 153 167 255 55 156 253 31 29 211 13 212 0 16 64 12 109 171 111 179 86 125 128 104 48 51 53 145 145 145 169 171 175 191 126 235 241 255 191 47 63 157 182 4 106 72 217 245 239 200 163 255 107 207 254 15 11 107 7 106 0 8 32 134 163 103 238 200 38 156 6 90 178 254 236 91 160 99 174 93 189 4 10 181 191 47 95 95 74 1 218 48 165 221 56 117 215 191 253 247 255 175 62 251 63 40 168 25 168 1 32 128 24 126 252 248 30 152 187 69 208 247 152 81 225 229 238 245 111 79 92 121 244 253 217 194 215 23 18 94 157 48 59 177 136 189 112 230 198 190 99 255 79 221 248 191 242 28 84 3 64 0 129 130 245 254 253 135 22 126 11 197 172 182 200 187 29 1 162 87 199 244 95 31 81 60 182 144 179 36 55 54 45 99 74 124 226 132 152 152 158 136 200 206 208 176 54 160 6 128 0 130 70 220 253 251 247 138 203 103 169 27 76 80 55 232 235 169 214 238 169 210 206 206 72 40 42 42 42 68 5 83 167 78 5 8 32 68 210 0 38 44 96 50 190 136 23 0 21 0 4 24 0 34 93 26 208 218 204 108 191 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!DefaultWebStyle methodsFor: 'frame navigation' stamp: 'mivsek 8/22/2007 20:36'!
navBarAdminElement
	| element |
	self app user isAdmin ifFalse: [^WebElement new]. 
	element := (WebElement newClass: #navdiv)
		add: (self navHeader: ' ADMIN' bullet: #bulletManTranspWhiteGif);
		add: (WebList newUnordered
			addLinkTo: self site admin text: 'Login' view: 'login'; 
			addLinkTo: self site admin text: 'Logout'  view: 'logout'; 
"			addSecureLinkTo: self site admin text: 'Sites' view: 'sites';  "
			addSecureLinkTo: self site admin text: 'Settings' view: 'settings'; 
			addSecureLinkTo: self site securityManager text: 'Security';
			addSecureLinkTo: self site admin text: 'Uptime' view: 'serverStatistics'; 
			addSecureLinkTo: self site statistics text: 'Statistics';
"			addSecureLinkTo: self site admin text: 'Tests' view: #tests;"
			yourself);
		yourself.
	^element! !

!DefaultWebStyle methodsFor: 'frame navigation' stamp: ' 21/4/07 10:33'!
navBarContactElement
	| element |
	element := (WebElement newClass: #navdiv)
		add: (self navLinkTo: 'mailto:info@eranova.si' 
			header: 'info@eranova.si' bullet: #bulletLetterTranspWhiteGif);
		yourself. 
	^element! !

!DefaultWebStyle methodsFor: 'frame navigation' stamp: 'mivsek 1/10/2008 18:02'!
navBarDemosIn: element
	element 	add: ((WebElement newClass: #navdiv)
		add: (self navHeader: ' DEMOS' bullet: #bulletStarTranspWhiteGif);
		add: (WebList newUnordered
			addLinkTo: self site demo	text: 'Introduction' view: #main; 
			addLinkTo: self site demo	text: 'Image gallery' view: #imageGallery; 
"			addLinkTo: self site demo	text: 'Live image' view: #liveImage;  "
"			addLinkTo: self site demo text: 'File upload' view: #fileUpload; "
			addLinkTo: self site demo text: 'Tabs' view: #tabs; 
			addLinkTo: self site demo text: 'Grid, tabs, menus' view: #grid; 
			addLinkTo: self site demo text: 'Javascript' view: #javascript; 
			addLinkTo: self site demo text: 'Ajax' view: #ajax; 
"			addLinkTo: self site demo text: 'Rich Editor' view: #richEditor; "
			addLinkTo: self site demo text: 'Calendar' view: #calendar; 
			yourself);
		yourself)! !

!DefaultWebStyle methodsFor: 'frame navigation' stamp: 'mivsek 8/22/2007 11:42'!
navBarMainLinksElement
	| element |
	element 	:= (WebElement newClass: #navdiv)
		add: (self navHeader: ' AIDA LINKS' bullet: #bulletSquareDownTranspWhiteGif);
		add: (WebList newUnordered
			addLinkTo: 'http://www.aidaweb.si' text: 'Website';
			addLinkTo: 'http://www.aidaweb.si/tutorial.html' text: 'Tutorial';
			addLinkTo: 'http://www.aidaweb.si/documentation.html' text: 'Documentation';
			addLinkTo: 'http://www.aidaweb.si/screenshots.html' text: 'Screenshots';
			addLinkTo: 'http://www.aidaweb.si/community.html' text: 'Community';
			yourself);
		yourself. 
	^element! !

!DefaultWebStyle methodsFor: 'frame navigation' stamp: 'JM 4/26/2007 13:45'!
navHeader: aString bullet: aSymbol
	^(WebText header: 3)
		addGif: aSymbol size: 13@13; addText: aString; yourself.! !

!DefaultWebStyle methodsFor: 'frame navigation' stamp: ' 21/4/07 10:33'!
navHeader: aString bullet: aSymbol linkTo: anObject
	^(WebText header: 3)
		addGif: aSymbol size: 13@13; addLinkTo: anObject text: aString; yourself.! !

!DefaultWebStyle methodsFor: 'frame navigation' stamp: ' 21/4/07 10:33'!
navHeader: aString bullet: aSymbol linkTo: anObject view: aView
	^(WebText header: 3)
		addGif: aSymbol size: 13@13; addLinkTo: anObject text: aString view: aView; yourself.! !

!DefaultWebStyle methodsFor: 'frame navigation' stamp: ' 21/4/07 10:33'!
navHeader: aString bullet: aSymbol linkTo: anObject view: aView parameter: aParmString value: aValueString
	^(WebText header: 3)
		addGif: aSymbol size: 13@13; addLinkTo: anObject text: aString 
		view: aView parameter: aParmString value: aValueString;
			yourself.! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
navLeftGif
	"'imgs/fast/fast-nav_left.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 7 0 82 0 128 0 0 127 160 177 255 255 255 33 249 4 1 0 0 0 0 44 0 0 0 0 7 0 82 0 0 2 64 140 143 6 144 139 203 14 124 78 86 117 195 180 17 119 157 109 30 245 137 96 25 166 232 74 182 156 59 190 114 76 159 176 61 227 181 201 171 119 207 202 1 127 62 97 113 119 212 41 135 198 32 210 185 76 50 159 68 232 20 211 248 28 10 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'frame navigation' stamp: ' 21/4/07 10:33'!
navLinkTo: anObject header: aString bullet: aSymbol
	| element |
	element := WebText header: 3.
	element 	
		addGif: aSymbol size: 13@13;
		addLinkTo: anObject text: ' ', aString.
	^element! !

!DefaultWebStyle methodsFor: 'frame navigation' stamp: 'JM 4/26/2007 13:48'!
navigationBarElement
	| element |
	element := WebElement newId: #navigation.
	element add: self navBarMainLinksElement.
	self navBarDemosIn: element.
	element add: self navBarAdminElement.
"	element add: self navBarSearchElement."
	element add: self navBarContactElement.
	^element! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: ' 21/4/07 10:33'!
noteSmallGif
	"small triangle with !! for notes/comments"
	"'imgs/simplebits-note_icon.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 12 0 12 0 162 0 0 227 203 118 250 226 142 51 51 51 204 153 102 255 255 255 0 0 0 0 0 0 0 0 0 33 249 4 0 0 0 0 0 44 0 0 0 0 12 0 12 0 0 3 42 72 170 51 251 44 194 24 220 116 33 219 231 132 168 16 230 105 28 17 140 32 117 126 1 96 97 89 236 18 86 134 182 13 1 236 60 239 52 192 160 35 1 0 59 )! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: 'np 5/7/2008 23:32'!
oldCss24WebGrid
^'
table.webGrid {
	background: #fdfae7;
	font-size: 10pt;
	}
table.webGrid tr { }
table.webGrid td {
	padding: 0px; spacing: 2px;
	vertical-align: top;
	background: #fcf4df;
	}
table.webGrid td input { font-size: 10pt }

table.webGrid th {
	padding: 0px; spacing: 2px;
	vertical-align: top;
	background: #fcf4df;
	font-size: 11pt; font-weight: normal;
	}

table.webGrid tr.red td, tr.red a { color: red; }
table.webGrid tr.green td, tr.green a { color: forestgreen; }
table.webGrid tr.blue td, tr.blue a { color: blue; }
table.webGrid tr.yellow td, tr.yellow a { color: yellow; }
table.webGrid tr.gray td, tr.gray a { color: darkgray; }
table.webGrid tr.summary td, tr.summary a { color: darkgray; font-weight: bold }
'! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: ' 21/4/07 10:33'!
outlookBigGif
	"'imgs/outlook_icon.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 30 0 30 0 135 0 0 128 128 0 153 153 51 204 204 153 192 220 192 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 3 0 0 3 0 44 0 0 0 0 30 0 30 0 0 8 206 0 7 8 28 72 176 160 193 131 3 1 40 92 200 176 161 195 135 2 31 74 156 200 48 34 197 139 14 45 2 32 192 177 163 199 143 32 9 44 212 232 81 64 128 133 1 4 132 244 56 114 192 194 142 39 29 6 88 201 177 229 75 2 49 85 114 20 160 112 230 74 155 10 9 240 220 40 146 104 81 157 32 129 110 60 169 51 232 78 0 62 147 42 36 233 180 40 75 163 31 149 90 173 201 112 171 84 0 84 141 54 44 250 144 44 73 166 33 135 150 213 58 52 164 66 164 92 205 186 116 154 179 227 80 159 47 131 106 229 24 179 97 212 188 27 247 114 69 9 23 176 220 155 113 105 118 212 59 117 46 86 198 14 175 30 174 106 85 162 100 193 94 21 67 38 185 24 99 87 204 158 63 55 70 172 152 38 230 210 110 71 135 246 172 113 53 197 214 174 37 34 156 77 155 96 64 0 59 )! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
pageContentWideWidth
	"for things in extensionElement, which is full page wide"
	^800! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
pageContentWidth
	^660! !

!DefaultWebStyle methodsFor: 'frame printing' stamp: ' 21/4/07 10:33'!
pageFrameWideNoNavigationWith: aWebElement title: aTitleString
	"set a web page just with header and wide element below"
	| element |
	self app clear;  title: aTitleString. self app htmlHeaderElements.
"	self app	add: self headerBanner. "
	element := (WebElement newId: #container)
		add: self headerElement; 	add: self headerDashboard;
		add: ((WebElement newId: #contentWide)
			add: aWebElement; yourself);
		yourself.
	^self app add: element; yourself! !

!DefaultWebStyle methodsFor: 'frame printing' stamp: ' 21/4/07 10:33'!
pageFrameWideNoNavigationWith: aWebElement title: aTitleString printHeader: aHeaderElement
	"set a web page just with header and wide element below"
	| element containerId |
	self app clear;  title: aTitleString. self app htmlHeaderElements.
"	self app	add: self headerBanner. "
	containerId := aHeaderElement elements notEmpty ifTrue: [#containerPadded] ifFalse: [#container].
	element := (WebElement newId: containerId)
		add: self headerElement; 	add: self headerDashboard;
		add: ((WebElement newId: #contentWide)
			add: aWebElement; yourself);
		yourself.
	^self app 
		add: aHeaderElement; "print header"
		add: element; yourself! !

!DefaultWebStyle methodsFor: 'frame printing' stamp: ' 21/4/07 10:33'!
pageFrameWith: aWebElement title: aTitleString
	"set a web page with standard  page look (navigation bar, header) "
	^self pageFrameWith: aWebElement wide: WebElement new title: aTitleString! !

!DefaultWebStyle methodsFor: 'frame printing' stamp: 'np 4/1/2008 22:34'!
pageFrameWith: aWebElement wide: aWideElement title: aTitleString
	"set a web page with standard  page look (navigation bar, header) "
	"Wide element comes below content besides navigation and it have full page width"
	| element |
	self app clear;  title: aTitleString. self htmlHeaderElements.
"	self app	add: self headerBanner. "
	element := (WebElement newId: #container)
		add: self headerElement; 	add: self headerDashboard;
		add: ((WebElement newId: #content) 
			add: self navigationBarElement; add: aWebElement;  yourself); 
		add: ((WebElement newId: #contentWide)
			add: aWideElement; yourself);
		yourself.
	^self app add: element; yourself! !

!DefaultWebStyle methodsFor: 'frame printing' stamp: ' 21/4/07 10:33'!
pageFrameWith: aWebElement wide: aWideElement title: aTitleString printHeader: aHeaderElement
	"set a web page with standard  page look (navigation bar, header) "
	"Wide element comes below content besides navigation and it have full page width"
	"printHeader if exist is added first, out of all other elements"
	| element containerId |
	self app clear;  title: aTitleString. self app htmlHeaderElements.
"	self app	add: self headerBanner. "
	containerId := aHeaderElement elements notEmpty ifTrue: [#containerPadded] ifFalse: [#container].
	element := (WebElement newId: containerId)
		add: self headerElement; 	add: self headerDashboard;
		add: ((WebElement newId: #content) 
			add: self navigationBarElement; add: aWebElement;  yourself); 
		add: ((WebElement newId: #contentWide)
			add: aWideElement; yourself);
		yourself.
	^self app 
		add: aHeaderElement; "print header"
		add: element; yourself! !

!DefaultWebStyle methodsFor: 'styles-screen' stamp: ' 21/4/07 10:33'!
pageWidth
	^800! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
pasteIconGif
	"'imgs/jure-paste.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 11 0 12 0 196 0 0 0 0 0 255 255 255 242 244 240 139 158 117 152 169 132 177 190 162 203 212 193 216 222 209 49 82 9 62 93 24 88 115 55 101 125 71 114 136 86 127 147 102 165 179 147 229 233 224 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 1 0 0 16 0 44 0 0 0 0 11 0 12 0 64 5 59 96 32 142 164 120 32 8 83 142 4 90 4 45 98 136 15 106 35 137 32 42 55 126 136 133 197 226 183 18 17 142 4 135 65 39 50 160 22 4 158 76 196 64 53 6 9 220 8 165 48 120 73 5 20 161 72 22 133 0 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-filetype icons' stamp: ' 21/4/07 10:33'!
pdfBigGif
	"'imgs/pdf_icon.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 35 0 35 0 135 0 0 48 49 51 56 55 58 59 59 60 62 62 65 59 67 59 60 69 60 77 1 1 66 65 67 66 66 68 69 69 69 71 71 73 73 73 76 79 79 83 79 91 79 82 82 86 84 84 86 84 84 88 90 90 93 93 93 94 95 95 97 96 96 99 98 98 101 101 101 102 103 103 104 104 104 109 108 108 110 110 110 114 114 114 117 121 121 122 122 122 126 121 121 129 185 79 82 179 88 89 201 1 4 206 3 6 208 2 6 215 3 6 214 5 8 217 4 7 221 6 9 225 5 8 230 7 10 227 8 11 234 6 9 236 7 10 236 8 12 242 8 11 241 10 13 245 9 12 247 12 15 255 7 10 248 10 13 255 10 14 247 13 16 250 14 17 254 14 17 254 18 21 249 23 26 254 22 25 254 25 29 255 28 31 255 30 33 228 63 63 234 50 50 255 33 37 255 37 41 255 40 43 255 41 45 255 46 49 255 48 50 255 53 55 255 53 57 255 56 58 255 57 60 255 61 63 255 62 64 214 109 112 222 123 124 226 78 78 232 76 76 231 87 88 230 88 88 231 91 92 230 93 94 234 87 87 233 89 89 236 89 89 255 71 74 255 81 83 255 91 93 254 93 96 232 96 97 234 100 100 232 105 106 233 108 108 230 112 112 230 115 116 231 124 125 236 117 118 233 121 122 233 123 124 254 98 101 251 109 111 252 110 112 254 114 116 255 119 122 255 122 124 222 127 129 227 127 129 254 126 128 129 129 130 133 133 137 141 141 141 143 143 145 143 143 148 137 156 137 145 145 148 150 150 154 151 151 156 153 153 154 154 154 157 159 159 159 159 159 163 159 159 168 145 166 145 156 178 156 163 163 164 165 165 168 166 166 174 167 169 175 170 167 175 169 169 173 169 169 177 173 172 178 174 174 181 176 176 178 177 176 181 181 181 182 180 180 185 181 182 189 185 185 189 188 188 191 187 187 194 189 188 194 190 190 197 173 199 173 178 204 178 182 208 182 186 213 186 189 217 189 214 140 143 217 135 138 211 157 160 221 176 179 225 129 131 230 129 130 228 131 132 232 134 135 233 135 136 235 136 137 235 139 140 238 141 142 226 145 146 231 144 146 232 153 155 232 156 157 252 132 133 253 135 136 253 138 139 253 143 144 252 146 148 252 150 152 231 158 160 230 160 162 230 165 167 231 167 170 236 162 163 233 171 172 234 173 174 234 175 176 225 181 181 232 177 178 235 182 183 236 179 180 239 182 182 234 182 184 194 194 194 193 192 198 197 197 198 194 194 201 196 195 202 197 197 202 198 198 204 196 201 196 200 199 206 200 200 200 200 201 205 202 202 208 203 204 209 205 205 210 205 206 212 192 220 192 196 218 196 201 209 201 200 218 200 209 209 213 210 212 214 214 214 214 212 212 217 215 215 220 209 216 209 216 216 219 217 217 221 219 220 223 220 220 222 222 221 225 223 224 227 229 197 200 238 193 194 235 198 200 228 221 223 236 209 210 237 215 216 235 216 217 235 218 220 236 220 221 254 219 220 228 222 224 238 223 225 224 224 225 226 226 229 228 228 230 230 230 233 229 229 236 231 232 234 233 225 228 232 231 234 238 230 232 232 232 234 234 234 236 235 236 238 239 232 233 236 235 237 236 236 238 238 238 240 234 241 246 240 228 228 255 224 224 240 240 241 243 243 244 245 245 246 247 247 248 251 251 251 251 251 252 254 254 254 0 0 0 33 249 4 3 0 0 201 0 44 0 0 0 0 35 0 35 0 0 8 255 0 147 9 28 72 176 160 193 131 8 17 82 146 36 41 18 195 135 12 41 81 74 72 81 96 165 57 132 2 17 26 100 232 16 162 142 27 7 37 26 20 172 226 193 139 128 2 5 18 84 168 144 161 150 131 98 14 186 195 107 24 179 132 168 84 165 66 117 202 84 155 54 106 210 164 65 67 244 140 209 163 102 62 216 73 84 110 152 180 74 7 87 149 209 130 229 202 18 36 68 132 104 13 194 21 72 15 30 59 116 228 192 97 192 207 32 127 214 120 45 139 90 53 73 145 123 226 218 220 16 71 55 11 13 186 226 102 184 88 177 194 192 32 63 254 252 53 93 91 80 213 21 37 69 132 136 219 33 110 174 140 187 56 196 217 168 241 162 133 10 20 6 14 229 9 140 86 23 225 129 169 150 24 17 2 132 174 154 185 58 24 71 22 103 170 69 138 19 36 12 32 250 195 89 240 48 101 4 83 29 33 18 164 199 189 29 59 122 136 227 161 163 177 184 24 47 86 156 40 49 194 64 163 68 112 162 231 209 245 140 23 65 84 137 135 12 185 199 187 52 93 45 54 196 237 255 69 97 66 68 8 3 137 70 198 252 147 231 142 174 73 3 79 25 65 130 196 72 226 32 64 194 234 184 97 67 239 10 20 36 132 112 222 49 198 16 227 203 35 140 48 226 135 31 125 12 4 130 1 16 70 40 225 132 20 94 19 141 51 199 0 179 139 35 139 252 49 7 69 23 21 115 12 52 212 88 131 205 57 230 112 227 4 45 231 160 120 13 53 208 32 35 204 46 144 64 242 97 66 23 65 51 141 53 215 84 131 141 57 41 174 216 162 57 47 198 56 163 47 192 220 136 208 69 214 80 115 77 57 39 170 243 206 60 83 212 242 206 59 234 156 131 13 140 25 10 67 140 146 39 205 81 14 143 230 156 227 206 60 244 116 227 195 24 104 206 227 206 57 22 58 83 76 49 200 128 105 208 69 112 70 41 15 62 248 132 178 137 23 218 208 67 207 60 234 152 83 77 140 199 52 99 103 65 23 169 83 142 57 232 184 67 15 62 237 84 225 141 45 92 8 74 207 59 112 78 211 12 50 138 130 56 135 163 144 190 35 40 46 83 12 10 198 39 242 204 51 15 58 229 80 255 227 76 51 161 226 56 135 60 144 186 35 15 61 246 80 113 203 160 237 120 97 69 41 179 200 226 74 38 206 24 227 204 162 4 93 132 206 179 238 192 67 79 40 80 144 163 77 41 160 108 210 197 22 63 64 193 70 19 151 16 179 172 168 238 68 122 229 55 79 192 18 139 44 219 128 227 234 59 217 136 162 201 26 150 0 83 235 146 115 96 201 142 59 235 116 66 70 59 248 12 250 46 58 68 70 131 9 19 189 220 27 38 60 238 176 83 11 43 82 128 83 15 29 135 160 147 142 4 4 255 146 1 58 197 248 2 201 49 204 14 116 209 166 164 180 242 197 43 229 30 96 1 58 190 44 144 14 48 199 56 208 12 52 139 44 242 72 200 22 205 129 79 46 97 120 194 137 59 234 92 99 193 27 138 8 48 65 28 3 216 177 0 2 127 68 0 129 33 56 39 115 81 62 180 136 49 138 58 207 242 66 129 27 23 236 66 1 5 139 40 240 134 30 1 96 128 0 212 162 238 147 15 61 242 148 139 14 30 21 88 224 0 36 17 76 96 72 2 29 212 1 128 6 132 32 90 18 245 69 253 232 147 207 60 239 148 235 198 49 135 112 48 193 6 114 48 80 135 3 16 236 241 128 7 104 219 234 15 63 106 211 3 15 167 216 196 10 77 178 51 114 168 136 33 132 56 242 55 31 13 16 80 192 234 172 183 238 58 235 4 52 192 71 69 145 240 49 199 237 184 231 174 187 238 124 68 98 210 239 192 7 47 188 64 1 1 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-filetype icons' stamp: ' 21/4/07 10:33'!
pdfMediumGif
	"'imgs/sigov-pdf.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 16 0 16 0 247 0 0 0 0 0 128 0 0 0 128 0 128 128 0 0 0 128 128 0 128 0 128 128 192 192 192 192 220 192 166 202 240 0 0 0 16 16 16 255 0 0 231 41 41 231 49 49 239 57 57 231 57 66 231 66 66 231 74 74 239 66 74 231 82 90 231 90 90 239 82 82 239 90 90 231 99 99 231 107 107 231 115 115 231 123 123 231 132 132 231 140 140 231 156 156 231 173 173 231 181 181 231 181 189 231 189 189 222 206 206 222 214 214 222 222 222 222 231 231 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 255 251 240 160 160 164 128 128 128 255 0 0 0 255 0 255 255 0 0 0 255 255 0 255 0 255 255 255 255 255 33 249 4 1 0 0 39 0 44 0 0 0 0 16 0 16 0 0 8 130 0 79 8 4 64 176 96 193 5 11 4 42 4 80 162 161 195 134 11 24 36 52 72 145 96 137 136 18 1 48 216 200 145 35 195 131 26 59 118 100 232 112 65 69 138 15 19 14 124 40 225 195 195 139 11 31 66 240 240 82 229 9 146 13 27 104 112 104 2 230 202 134 30 70 116 200 144 50 102 9 18 23 62 128 112 240 32 2 133 16 54 73 114 136 208 97 3 136 15 29 48 76 136 90 162 66 132 151 71 69 68 205 32 97 4 216 18 38 162 90 56 91 82 33 194 183 112 225 6 4 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-filetype icons' stamp: ' 21/4/07 10:33'!
pdfSmallGif
	"'imgs/ibm-pdf.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 11 0 12 0 162 0 0 0 0 0 204 204 204 153 153 153 204 0 0 255 255 255 254 1 2 0 0 0 0 0 0 33 249 4 5 20 0 5 0 44 0 0 0 0 11 0 12 0 0 3 39 72 176 204 42 48 6 48 169 157 181 181 192 129 226 193 48 128 30 197 137 228 119 142 157 138 162 101 40 206 67 137 130 45 192 226 147 162 53 9 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-filetype icons' stamp: ' 21/4/07 10:33'!
pdfSmallPng
	"'imgs/ikone-dok/corendal-pdf.png' asFilename contentsAsMethod"
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 2 0 0 0 144 145 104 54 0 0 0 4 103 65 77 65 0 0 88 199 252 71 224 2 0 0 0 32 99 72 82 77 0 0 122 37 0 0 128 131 0 0 249 255 0 0 128 230 0 0 117 46 0 0 234 95 0 0 58 151 0 0 23 111 105 228 196 43 0 0 2 131 73 68 65 84 120 156 98 248 15 3 183 111 223 222 186 117 235 198 141 27 55 108 216 0 33 33 96 243 230 205 47 94 188 128 43 3 8 32 6 56 235 200 145 35 135 15 159 222 183 239 216 158 61 135 118 237 58 176 105 211 142 21 43 214 116 116 244 222 191 127 31 168 237 209 163 71 16 101 0 1 196 240 175 182 225 127 118 238 255 144 240 255 158 190 255 157 92 255 219 57 252 183 176 254 111 100 250 91 223 248 182 148 92 99 99 235 239 223 191 129 122 214 172 89 3 209 3 16 64 12 255 75 203 255 155 91 253 87 82 253 47 38 245 159 87 232 63 59 247 127 70 214 175 140 44 47 25 89 87 51 177 22 21 149 252 6 3 160 158 245 235 215 191 121 243 6 32 128 24 254 39 165 252 215 208 254 47 45 255 95 88 226 63 143 192 127 86 206 95 140 44 239 24 89 110 51 178 116 51 50 119 117 245 150 150 86 20 23 151 214 215 55 246 244 244 221 188 121 19 32 128 24 174 75 201 2 37 176 162 70 86 246 103 207 158 157 62 125 250 232 209 163 187 119 239 62 113 226 196 161 67 135 0 2 8 225 233 227 199 143 95 185 114 229 204 153 51 23 46 92 184 119 239 222 229 160 224 59 183 111 63 120 240 224 198 141 27 151 46 93 2 202 2 141 223 179 103 15 64 0 161 104 0 74 95 189 122 245 238 221 187 95 190 124 185 239 239 247 248 209 99 160 163 159 62 125 10 12 241 115 231 206 61 124 248 112 251 246 237 0 1 132 162 225 201 147 39 183 110 221 2 42 122 113 251 246 93 27 235 23 179 230 2 117 190 123 247 14 168 7 104 249 243 231 207 55 109 218 4 16 64 40 26 128 66 64 99 190 126 253 250 170 182 230 229 219 143 207 250 39 188 90 181 250 219 183 111 192 136 3 26 244 242 229 75 160 6 128 0 66 209 0 52 251 213 171 87 175 47 95 123 233 227 251 114 213 42 32 122 238 233 245 220 195 227 109 112 240 237 150 182 143 31 63 2 53 0 4 16 138 134 79 159 62 125 254 252 249 113 116 244 171 61 251 222 223 186 243 254 195 167 15 255 255 63 121 251 225 198 182 157 151 83 51 129 102 1 53 0 4 16 138 134 247 143 158 62 138 75 120 218 219 15 212 246 250 245 107 160 11 129 174 7 134 4 208 61 103 207 158 5 6 6 80 3 64 0 161 104 120 88 80 248 180 176 16 104 53 208 247 64 207 220 185 115 7 24 166 23 47 94 4 6 244 129 3 7 46 95 190 12 212 0 16 64 8 13 192 216 1 146 223 191 127 127 251 246 45 208 118 96 148 1 19 15 68 15 48 30 142 29 59 6 212 0 76 133 0 1 132 208 112 248 240 97 71 71 71 53 28 64 94 94 30 40 187 119 239 94 128 0 3 0 11 108 48 6 214 146 182 59 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!DefaultWebStyle methodsFor: 'imgs-filetype icons' stamp: ' 21/4/07 10:33'!
powerpointSmallPng
	"'imgs/ikone-dok/corendal-powerpoint.png' asFilename contentsAsMethod"
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 2 0 0 0 144 145 104 54 0 0 0 4 103 65 77 65 0 0 88 199 252 71 224 2 0 0 0 32 99 72 82 77 0 0 122 37 0 0 128 131 0 0 249 255 0 0 128 230 0 0 117 46 0 0 234 95 0 0 58 151 0 0 23 111 105 228 196 43 0 0 2 166 73 68 65 84 120 156 98 248 15 3 155 14 221 216 120 240 250 134 3 215 214 237 187 178 118 239 165 213 187 46 174 218 121 126 197 182 179 203 182 157 94 186 229 212 191 127 80 101 0 1 196 112 187 57 9 136 110 54 38 92 173 137 185 80 30 113 166 48 228 120 182 255 225 84 239 253 137 30 123 227 92 63 127 255 247 246 203 223 57 171 15 152 123 167 254 3 107 2 8 32 144 6 32 245 239 249 237 13 241 46 19 173 140 129 104 110 172 199 183 107 199 128 130 64 13 95 190 255 123 255 229 239 212 37 59 119 29 189 2 209 3 16 64 12 55 27 226 129 170 215 187 91 254 249 242 254 255 239 239 97 60 252 64 61 109 218 186 64 61 219 35 28 191 252 248 251 254 235 223 201 139 182 3 85 67 16 64 0 49 92 173 141 1 154 253 231 243 219 255 127 126 253 255 251 27 168 225 102 164 246 135 217 233 64 193 245 1 214 19 214 93 131 160 165 123 238 44 217 125 23 168 1 32 128 24 46 150 71 76 50 213 253 90 237 244 174 41 228 69 127 214 30 127 253 15 211 179 62 44 43 1 10 46 113 55 249 252 3 228 164 151 31 254 60 124 243 103 225 174 187 102 94 41 0 1 196 112 166 48 120 162 181 201 135 147 211 63 156 93 248 97 111 247 135 137 233 32 180 32 31 168 97 166 141 14 220 134 251 175 126 207 221 113 15 168 1 32 128 24 78 100 251 3 109 255 48 39 238 195 209 73 31 182 53 126 152 89 248 97 90 246 135 137 32 119 246 26 170 2 141 127 245 241 239 147 183 127 238 190 252 61 115 219 3 160 6 128 0 98 56 156 230 13 241 244 135 185 177 31 150 102 125 152 150 241 97 82 244 6 75 125 160 96 179 134 28 220 134 91 207 126 79 221 252 8 168 1 32 128 24 14 36 122 64 130 245 96 73 244 68 115 131 73 102 122 64 6 144 11 20 172 81 144 120 241 254 207 163 55 127 238 188 248 125 237 201 175 254 141 143 129 26 0 2 136 1 24 216 59 162 156 54 5 219 174 242 177 88 228 98 12 114 183 137 122 135 142 98 131 138 76 153 148 8 220 134 75 15 127 117 173 123 2 212 0 16 64 12 240 164 49 115 197 190 23 31 128 1 242 23 72 62 127 255 231 233 219 63 15 95 255 6 154 125 253 233 175 203 143 126 157 187 247 179 117 205 51 160 6 128 0 66 104 152 182 108 247 163 55 191 129 14 128 132 9 220 108 8 58 117 251 71 227 242 231 64 13 0 1 132 208 0 140 127 160 207 110 61 255 125 243 217 239 27 79 127 93 123 242 251 242 163 223 23 30 252 60 115 247 199 201 219 63 143 223 250 81 179 228 37 80 3 64 0 33 52 0 227 255 210 163 95 243 118 220 7 6 223 212 45 143 38 110 124 212 179 254 73 199 218 167 45 171 159 53 44 127 94 189 244 69 197 226 215 64 13 0 1 132 208 0 228 16 131 0 2 12 0 18 21 0 105 149 238 239 221 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!DefaultWebStyle methodsFor: 'imgs-common symbols' stamp: ' 21/4/07 10:33'!
printerJpg
	"'imgs/toledo-printer.jpg' asFilename contentsAsMethod"
^#(255 216 255 224 0 16 74 70 73 70 0 1 1 1 0 72 0 72 0 0 255 219 0 67 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 255 219 0 67 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 255 192 0 17 8 0 22 0 20 3 1 34 0 2 17 1 3 17 1 255 196 0 31 0 0 1 5 1 1 1 1 1 1 0 0 0 0 0 0 0 0 1 2 3 4 5 6 7 8 9 10 11 255 196 0 181 16 0 2 1 3 3 2 4 3 5 5 4 4 0 0 1 125 1 2 3 0 4 17 5 18 33 49 65 6 19 81 97 7 34 113 20 50 129 145 161 8 35 66 177 193 21 82 209 240 36 51 98 114 130 9 10 22 23 24 25 26 37 38 39 40 41 42 52 53 54 55 56 57 58 67 68 69 70 71 72 73 74 83 84 85 86 87 88 89 90 99 100 101 102 103 104 105 106 115 116 117 118 119 120 121 122 131 132 133 134 135 136 137 138 146 147 148 149 150 151 152 153 154 162 163 164 165 166 167 168 169 170 178 179 180 181 182 183 184 185 186 194 195 196 197 198 199 200 201 202 210 211 212 213 214 215 216 217 218 225 226 227 228 229 230 231 232 233 234 241 242 243 244 245 246 247 248 249 250 255 196 0 31 1 0 3 1 1 1 1 1 1 1 1 1 0 0 0 0 0 0 1 2 3 4 5 6 7 8 9 10 11 255 196 0 181 17 0 2 1 2 4 4 3 4 7 5 4 4 0 1 2 119 0 1 2 3 17 4 5 33 49 6 18 65 81 7 97 113 19 34 50 129 8 20 66 145 161 177 193 9 35 51 82 240 21 98 114 209 10 22 36 52 225 37 241 23 24 25 26 38 39 40 41 42 53 54 55 56 57 58 67 68 69 70 71 72 73 74 83 84 85 86 87 88 89 90 99 100 101 102 103 104 105 106 115 116 117 118 119 120 121 122 130 131 132 133 134 135 136 137 138 146 147 148 149 150 151 152 153 154 162 163 164 165 166 167 168 169 170 178 179 180 181 182 183 184 185 186 194 195 196 197 198 199 200 201 202 210 211 212 213 214 215 216 217 218 226 227 228 229 230 231 232 233 234 242 243 244 245 246 247 248 249 250 255 218 0 12 3 1 0 2 17 3 17 0 63 0 254 205 254 42 248 150 223 225 223 197 164 93 102 105 46 244 31 136 158 17 147 94 211 18 238 254 241 97 208 252 73 224 43 237 63 68 214 173 99 34 230 24 173 237 252 71 163 120 191 68 190 210 236 98 24 55 62 26 241 93 244 155 140 200 34 229 126 19 124 105 117 253 160 238 188 7 37 196 255 0 240 175 190 32 252 59 26 247 132 181 11 155 139 137 180 93 39 226 103 131 53 147 99 226 111 6 218 106 183 114 73 109 30 173 226 239 7 235 26 87 138 244 79 14 165 201 190 189 183 240 71 143 181 123 75 87 180 210 245 57 45 190 102 241 254 183 30 181 226 47 136 94 33 248 185 55 196 63 7 90 106 255 0 16 126 34 248 58 215 197 119 26 222 181 109 240 235 89 240 87 194 223 138 94 59 208 188 45 163 105 154 239 132 245 43 221 47 193 239 6 139 100 210 106 122 30 191 31 132 124 109 172 234 218 150 179 172 120 58 227 84 212 144 220 105 158 7 175 254 208 191 0 175 109 78 145 240 235 225 245 231 142 32 136 188 95 240 144 234 22 158 34 214 244 141 57 132 13 9 212 127 225 97 252 81 215 244 123 61 46 242 202 218 121 26 41 238 175 36 241 21 128 103 146 207 66 186 12 209 201 239 75 52 203 231 147 44 12 178 185 60 198 209 143 246 146 197 168 193 66 157 104 78 31 236 107 12 249 167 236 99 236 37 47 172 71 79 222 107 38 209 229 172 22 49 102 127 92 89 132 150 13 198 207 47 116 28 147 147 167 200 218 174 235 174 85 237 57 106 164 168 189 83 141 236 238 191 160 10 43 241 175 246 28 253 178 124 109 227 31 134 190 55 182 212 116 169 47 52 191 7 252 80 191 240 207 131 207 136 245 185 53 237 106 223 194 23 94 4 248 127 227 45 34 11 205 114 218 218 198 61 86 47 51 197 183 115 233 211 165 172 22 214 218 84 246 58 118 153 20 122 69 150 159 26 21 243 240 148 103 8 206 46 241 156 99 56 187 53 120 205 41 69 217 164 213 211 78 205 38 186 164 207 77 53 40 198 73 221 74 42 81 125 212 146 105 217 234 174 154 118 122 247 62 203 248 175 251 47 248 138 254 251 198 158 45 248 17 241 22 79 135 218 255 0 143 45 117 51 227 127 135 94 50 178 159 198 223 2 188 125 171 234 54 115 91 73 226 61 71 194 82 220 65 173 120 7 198 51 23 140 223 248 179 225 158 181 225 248 252 67 177 102 241 223 134 252 111 113 109 166 201 97 249 109 225 63 248 36 79 237 11 23 139 244 255 0 29 248 255 0 226 183 192 63 21 120 147 80 182 182 131 81 130 227 195 63 23 117 63 1 252 62 22 75 50 66 124 25 240 230 247 199 22 154 95 139 238 164 136 219 43 203 174 234 190 18 209 37 187 130 107 237 95 195 26 242 94 77 96 10 43 69 38 148 146 229 247 151 43 188 99 38 149 212 189 215 36 220 29 210 247 160 227 43 94 55 229 148 147 127 215 245 223 230 126 161 124 27 253 136 62 28 124 42 240 222 167 166 234 254 40 241 175 196 143 18 248 143 94 147 197 30 41 241 126 187 54 129 225 79 237 29 102 77 23 68 240 250 71 163 120 59 225 190 135 224 239 2 248 63 195 246 58 71 135 180 171 45 43 195 158 25 240 237 134 159 99 20 12 248 158 230 121 238 37 40 162 164 15 255 217 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
redPixelGif
	"'imgs/redpix.gif' asFilename contentsAsMethod"
	 ^#(71 73 70 56 57 97 1 0 1 0 145 255 0 255 255 255 196 0 67 
		0 0 0 0 0 0 44 0 0 0 0 1 0 1 0 0 2 2 76 1 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
tab1BgGif
	"'/home/mivsek/download/tab1bg.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 1 0 45 0 132 32 0 221 227 213 219 225 211 254 254 254 242 244 238 230 233 224 218 224 210 252 252 252 245 247 242 249 249 247 234 237 228 224 228 214 229 232 222 238 241 234 237 240 233 239 242 235 226 229 218 248 248 246 247 247 245 243 245 240 233 236 227 250 250 248 251 251 251 235 238 231 119 102 85 227 230 219 228 231 221 253 253 253 236 239 232 225 228 217 241 243 238 224 227 216 232 235 226 44 0 0 0 0 1 0 45 0 0 5 33 96 81 4 36 96 42 232 131 101 11 225 78 73 98 109 13 227 12 221 32 29 71 4 33 8 74 197 96 208 8 142 151 16 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
tab1LeftGif
	"'imgs/tab1left.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 9 0 150 0 196 31 0 183 174 136 239 237 229 224 220 204 236 234 224 241 239 233 231 228 216 215 210 189 219 214 196 214 209 187 244 243 237 251 251 250 246 245 242 232 229 218 223 219 203 248 247 244 217 213 194 225 222 208 241 239 232 240 238 231 197 190 160 235 232 222 237 235 226 238 236 227 249 248 246 220 216 198 188 180 144 211 207 183 216 211 191 238 236 228 229 226 212 199 192 163 226 229 218 33 249 4 1 0 0 31 0 44 0 0 0 0 9 0 150 0 0 5 240 224 39 66 136 151 1 128 120 76 199 224 40 202 247 104 4 108 67 83 4 95 206 178 104 15 5 111 145 40 78 24 195 4 33 34 201 36 122 202 8 199 2 232 17 9 1 75 165 234 75 72 44 3 10 160 139 173 80 196 100 51 131 49 86 6 40 133 14 4 80 148 12 10 16 65 131 46 165 116 26 24 15 0 4 18 21 12 16 129 27 131 1 3 135 7 27 8 139 119 2 7 6 145 88 147 149 151 140 5 148 150 146 157 154 160 158 155 153 159 152 161 167 156 164 163 162 168 172 175 174 171 178 166 165 169 182 176 179 170 181 173 187 183 189 184 180 191 177 190 185 188 196 193 197 194 198 195 186 201 207 192 209 200 210 206 212 199 213 211 217 216 219 215 221 205 222 204 225 203 227 202 229 208 218 224 228 231 220 223 237 226 230 214 238 234 241 239 235 233 240 232 242 248 236 245 244 243 249 253 0 254 227 55 240 158 61 125 7 3 18 220 103 208 31 67 132 14 19 22 132 40 240 161 194 134 21 37 90 10 1 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
tab1LeftOnGif
	"'imgs/tab1lefton.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 9 0 150 0 196 31 0 255 255 255 119 102 85 254 254 254 253 253 252 250 250 248 249 249 247 253 253 253 252 252 250 249 248 246 248 247 245 245 244 239 246 245 241 244 242 237 242 241 235 246 245 240 247 246 242 221 217 213 187 179 170 245 244 243 129 113 97 233 230 227 152 139 126 147 133 120 147 133 119 174 164 154 214 212 204 242 241 234 214 212 203 196 189 180 185 177 167 183 174 164 226 229 218 33 249 4 1 0 0 31 0 44 0 0 0 0 9 0 150 0 0 5 223 224 39 102 93 53 5 129 200 89 17 36 1 192 199 97 20 108 111 87 109 195 94 180 219 21 200 15 54 121 13 3 67 24 18 38 104 54 3 78 131 97 64 13 76 7 7 2 161 80 8 80 179 136 68 226 17 208 22 16 15 135 130 81 62 39 212 141 70 59 236 96 52 52 229 109 98 177 102 103 207 15 124 10 1 7 127 97 11 11 94 88 91 8 97 86 83 89 91 93 82 87 133 7 80 77 148 85 76 78 79 73 0 75 71 159 161 63 164 59 166 54 168 74 163 172 73 170 160 173 162 174 177 165 180 167 182 169 184 171 179 188 178 190 181 189 192 191 183 193 196 195 185 197 200 199 187 203 176 201 204 194 209 198 210 202 212 208 211 216 213 217 215 218 221 220 223 206 205 175 227 186 225 214 230 219 232 222 234 224 228 207 236 240 238 226 229 242 231 245 233 247 235 249 237 244 253 239 251 241 252 205 251 39 208 94 65 124 7 245 37 4 23 2 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
tab1RightGif
	"'imgs/tab1right.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 144 1 150 0 196 31 0 213 208 186 183 174 136 210 204 181 217 212 193 226 222 208 233 230 219 222 218 201 243 242 236 214 209 188 235 232 222 214 208 187 236 234 224 240 238 231 247 246 243 238 236 227 231 228 216 250 250 248 253 253 252 201 194 166 212 207 184 208 202 178 205 199 173 203 196 169 192 184 151 229 226 212 220 216 198 216 211 191 207 201 176 206 200 174 202 195 167 252 252 251 226 229 218 33 249 4 1 0 0 31 0 44 0 0 0 0 144 1 150 0 0 5 255 96 32 142 100 105 158 104 170 174 108 235 190 112 44 207 116 109 223 120 174 239 176 68 101 159 96 100 72 44 26 143 200 164 114 201 108 58 159 208 168 116 74 173 90 175 216 172 118 203 117 122 32 7 130 224 34 8 66 206 232 180 122 205 110 187 223 240 184 124 78 175 219 239 248 188 126 207 239 251 255 128 115 13 7 12 9 6 18 101 13 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 150 7 7 11 15 4 3 23 25 166 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 186 12 14 9 24 6 25 29 22 12 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 221 202 169 205 99 14 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 0 3 10 28 72 176 160 193 131 8 19 42 236 151 160 0 179 12 3 16 4 88 64 177 162 197 139 24 51 106 220 200 177 163 199 143 32 67 138 28 73 178 164 201 147 40 83 255 170 92 201 178 229 199 4 9 30 60 28 160 33 0 204 155 56 115 234 220 201 179 167 207 159 64 131 10 29 74 180 168 209 163 72 147 42 93 202 180 169 211 167 67 11 184 203 160 65 98 129 171 88 179 106 221 202 181 171 215 175 96 195 138 29 75 182 172 217 179 104 211 170 93 203 182 173 219 183 112 199 202 36 0 17 129 196 7 120 243 234 221 203 183 175 223 191 128 3 11 30 76 184 176 225 195 136 19 43 94 204 184 177 227 199 144 35 19 198 64 160 153 221 9 1 8 104 222 204 185 179 231 207 160 67 139 30 77 186 180 233 211 168 83 171 94 205 186 181 235 215 176 99 203 158 125 186 153 6 0 2 4 4 48 192 187 183 239 223 192 131 11 31 78 188 184 241 227 200 147 43 95 206 188 185 243 231 208 163 75 159 78 189 250 241 12 16 53 76 160 64 97 67 128 1 224 195 139 31 79 190 188 249 243 232 211 171 95 207 190 189 251 247 240 227 203 159 79 191 190 253 251 248 243 179 175 186 157 67 133 10 1 216 37 224 128 4 22 104 224 129 8 38 168 224 255 130 12 54 232 224 131 16 70 40 225 132 20 86 104 225 133 24 102 168 225 131 19 8 176 65 5 22 116 16 224 134 36 150 104 226 137 40 166 168 226 138 44 182 168 33 0 29 82 224 95 7 34 2 96 227 141 56 230 168 227 142 60 246 232 227 143 64 6 41 228 144 68 22 105 228 145 72 38 169 228 146 76 54 233 228 147 65 78 16 35 136 29 72 16 0 148 88 102 169 229 150 92 118 233 229 151 96 134 9 165 148 2 80 64 165 149 98 166 169 230 154 108 182 233 230 155 112 226 72 166 153 33 162 25 231 157 120 230 169 231 158 124 114 57 231 153 87 246 41 232 160 132 22 106 168 155 127 214 25 232 161 140 54 234 232 163 144 242 152 104 149 139 70 106 233 165 152 102 202 230 164 118 106 234 233 167 160 134 122 36 167 149 138 106 234 169 168 122 74 106 170 172 182 234 234 161 171 190 42 235 172 180 182 25 107 173 184 230 170 235 147 183 238 234 235 175 192 250 216 107 176 196 22 187 235 176 198 38 171 108 171 200 46 235 236 179 170 78 169 40 180 212 86 139 105 255 179 214 102 171 45 159 216 110 235 237 183 182 74 75 41 184 228 150 187 169 184 157 154 171 238 186 90 118 203 238 187 240 18 233 110 188 244 214 171 227 188 246 230 75 47 190 250 246 187 46 191 254 6 12 46 192 2 23 156 45 193 6 39 252 44 194 10 55 108 44 195 14 71 252 43 196 18 87 140 43 197 22 103 252 42 198 26 119 140 42 199 30 135 12 42 200 34 151 124 45 186 165 154 172 50 179 40 175 236 178 171 36 191 44 243 160 49 207 108 179 158 53 223 172 51 156 57 239 236 243 154 61 255 44 52 152 65 15 109 244 150 69 31 173 52 175 45 47 237 116 158 73 63 45 181 145 81 79 109 117 148 77 95 173 117 152 85 111 237 181 156 89 127 45 54 150 93 143 189 117 217 102 95 141 118 218 83 175 205 246 211 110 191 189 116 220 114 31 77 119 221 67 223 141 247 207 122 239 189 115 223 126 223 12 120 224 51 15 78 248 203 134 31 190 114 226 138 155 204 120 227 34 63 14 185 199 146 79 174 113 229 150 91 140 121 230 18 111 206 185 195 158 127 174 112 254 232 162 27 76 122 233 2 159 142 186 191 170 175 174 111 235 174 219 11 123 236 251 134 77 251 215 179 223 254 110 238 186 255 107 123 239 106 255 14 124 219 194 15 15 119 241 198 207 141 124 242 118 47 207 124 222 206 63 207 119 244 210 255 77 125 245 130 95 143 125 225 218 111 143 120 247 222 47 14 126 248 142 143 79 126 228 230 159 79 121 250 234 95 206 126 251 154 191 15 127 231 242 207 15 122 253 246 143 142 127 254 166 239 207 127 234 254 251 31 235 2 40 192 215 17 176 128 178 59 32 2 107 87 38 64 45 208 101 188 123 96 174 34 40 193 90 81 176 130 179 186 32 6 55 166 192 13 154 75 131 30 100 21 8 67 248 177 14 146 240 91 35 60 161 168 82 168 194 145 153 176 133 7 123 33 12 171 197 194 25 102 170 134 54 188 20 14 115 24 169 29 242 240 81 62 252 97 163 130 40 68 88 201 176 136 15 59 34 18 137 69 196 37 210 76 137 78 156 24 20 163 120 172 41 82 113 130 86 188 162 5 179 168 197 12 114 177 139 28 108 160 162 66 0 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
tab1RightOnGif
	"'imgs/tab1righton.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 144 1 150 0 196 31 0 255 255 255 253 253 252 254 254 254 244 242 237 249 249 247 242 241 234 119 102 85 242 241 235 250 250 248 247 246 242 246 245 241 248 247 245 209 205 193 145 131 116 229 225 213 232 230 218 239 237 230 174 164 150 249 248 246 234 232 222 237 234 226 231 227 216 245 244 239 246 245 240 253 253 253 241 240 233 252 252 250 240 238 231 189 181 169 235 233 223 128 112 95 226 229 218 33 249 4 1 0 0 31 0 44 0 0 0 0 144 1 150 0 0 5 255 160 33 142 100 105 158 104 170 174 108 235 190 112 44 207 116 109 223 120 174 239 175 215 68 140 143 16 64 44 26 143 200 164 114 201 108 58 159 208 168 116 74 173 90 175 216 172 118 203 237 58 5 1 9 133 17 105 112 134 222 180 122 205 110 187 223 240 184 156 11 150 88 54 227 200 121 206 239 251 255 128 129 130 131 69 117 22 16 29 15 12 13 12 132 142 143 144 145 146 147 109 134 136 15 14 17 17 148 156 157 158 159 160 129 150 137 21 139 161 167 168 169 170 171 75 163 152 14 30 172 178 179 180 181 162 97 135 164 14 6 2 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 199 24 26 18 3 151 14 187 1 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 234 26 4 9 220 19 175 12 16 24 72 176 160 193 131 8 19 42 92 200 176 161 195 135 16 35 74 156 72 177 162 197 139 24 51 106 220 200 209 225 130 11 5 40 60 168 224 205 64 130 147 40 83 255 170 92 201 178 165 203 151 48 99 202 156 73 179 166 205 155 56 115 234 220 201 179 167 207 159 64 99 42 184 112 96 67 34 111 187 6 40 93 202 180 169 211 167 80 163 74 157 74 181 170 213 171 88 179 106 221 202 181 171 215 175 96 195 138 29 75 245 64 81 10 0 145 26 40 192 182 173 219 183 112 227 202 157 75 183 174 221 187 120 243 234 221 203 183 175 223 191 128 3 11 30 76 184 240 221 12 16 208 190 218 101 184 177 227 199 144 35 75 158 76 185 178 229 194 25 54 36 38 85 161 130 1 179 160 67 139 30 77 186 180 233 211 168 83 171 94 205 186 181 235 215 176 99 203 158 77 187 182 237 219 184 87 23 64 76 33 209 131 7 6 200 10 31 78 188 184 241 227 200 147 43 95 46 252 0 111 10 189 13 40 152 78 189 186 245 235 216 179 107 223 206 189 187 247 239 224 195 139 31 79 190 188 249 243 232 211 171 95 207 190 251 133 11 22 6 236 214 12 193 192 130 251 248 243 235 223 207 191 191 255 255 0 6 40 224 128 4 22 104 224 129 8 38 168 224 255 130 12 54 232 224 131 16 10 120 18 124 3 128 38 80 71 24 102 168 225 134 28 118 232 225 135 32 134 216 145 4 11 36 48 221 123 6 32 160 226 138 44 182 232 226 139 48 198 40 227 140 52 214 104 227 141 56 230 168 227 142 60 246 232 227 143 64 6 41 228 144 55 14 36 193 145 18 24 160 207 146 76 54 233 228 147 80 70 41 229 148 84 50 169 193 149 43 242 114 205 150 92 118 233 229 151 96 134 41 230 152 100 110 137 193 153 104 26 96 203 154 108 182 233 102 26 106 190 41 231 156 116 214 89 68 156 118 230 169 231 158 168 224 201 231 159 128 6 42 136 159 130 22 106 232 161 112 34 170 232 162 140 78 65 104 163 144 70 202 232 163 146 86 106 233 159 148 94 170 233 166 111 102 202 233 167 160 178 226 105 168 164 150 218 201 168 166 166 170 234 32 168 174 234 234 171 113 180 10 235 172 180 118 33 107 173 184 230 234 168 174 188 246 122 197 173 190 6 235 43 176 194 22 139 43 177 198 38 251 42 178 202 54 107 42 179 206 70 251 41 180 210 86 107 41 255 181 214 102 219 40 182 218 118 123 40 183 222 134 11 40 184 226 150 155 39 185 230 166 43 39 186 234 182 187 38 187 238 198 43 11 188 242 214 219 167 189 248 242 73 111 190 252 78 178 111 191 0 59 242 111 192 4 3 50 112 193 8 207 113 112 194 12 187 177 112 195 16 39 26 241 196 167 82 108 177 191 23 103 252 200 195 26 119 236 4 199 30 135 156 4 200 34 151 76 4 201 38 139 140 114 202 30 175 204 178 198 46 191 124 113 204 50 83 76 115 205 17 223 140 115 195 58 239 156 112 207 62 23 12 116 208 1 15 77 116 191 70 31 157 111 210 74 219 203 116 211 242 62 13 181 187 82 79 173 110 213 86 155 139 117 214 226 110 205 181 183 94 127 173 109 216 98 91 75 118 217 210 158 141 182 179 106 175 173 108 219 110 27 11 119 220 194 206 77 247 176 119 251 108 119 222 186 238 205 247 177 127 215 236 119 224 179 14 78 248 178 135 179 108 120 226 170 46 206 248 179 143 151 236 120 228 161 78 78 249 180 151 183 156 121 199 150 111 126 105 231 158 87 10 122 140 232 145 142 78 250 182 167 219 156 250 196 166 175 142 104 235 174 27 10 123 236 130 206 78 251 184 183 255 156 59 194 182 239 174 103 239 190 219 9 124 240 116 14 79 252 186 199 35 157 60 191 198 47 207 102 243 206 219 2 125 244 180 76 79 253 188 215 71 157 125 188 214 111 159 74 247 222 159 2 126 248 160 140 79 190 39 230 159 207 73 250 234 99 220 126 184 236 191 15 73 252 242 11 92 127 183 244 223 63 168 254 217 230 207 255 31 254 251 95 31 2 40 64 133 21 48 90 4 60 32 28 18 168 64 135 53 240 109 33 0 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
tab2BottomGif
	"'imgs/tab2bottom.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 1 0 1 0 128 0 0 204 204 204 0 0 0 33 249 4 0 0 0 0 0 44 0 0 0 0 1 0 1 0 0 2 2 68 1 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
tab3BackGif
	"'imgs/jure-tabs-back.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 7 0 24 0 213 0 0 0 0 0 255 255 255 254 255 254 254 255 253 251 252 250 251 252 249 250 251 248 250 251 247 248 249 245 247 248 242 245 246 240 244 245 238 243 244 237 242 243 236 243 244 236 242 243 235 243 243 234 243 243 235 242 242 234 244 244 237 243 243 236 242 242 235 247 247 241 246 246 240 245 245 239 244 244 238 243 243 237 248 248 243 247 247 242 246 246 241 249 249 245 247 247 243 246 246 242 252 252 249 251 251 248 250 250 247 252 252 250 250 250 248 254 254 253 253 253 252 247 246 242 251 250 247 254 253 252 252 251 250 254 254 254 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 1 0 0 45 0 44 0 0 0 0 7 0 24 0 64 6 88 192 71 37 242 128 72 32 30 15 34 153 12 56 159 1 140 116 138 97 153 88 44 1 107 0 125 106 40 140 73 216 97 65 129 56 40 139 37 180 34 21 8 132 85 119 78 127 70 40 148 72 227 190 232 103 22 127 23 10 23 29 131 29 9 28 31 31 27 139 34 6 7 41 37 35 37 39 149 39 42 42 39 117 155 156 157 80 65 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
tab5BgGif
	"'imgs/jure-tab-bg.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 20 0 45 0 196 0 0 221 227 213 219 225 211 254 254 254 242 244 238 230 233 224 218 224 210 252 252 252 245 247 242 249 249 247 234 237 228 224 228 214 229 232 222 238 241 234 237 240 233 239 242 235 226 229 218 248 248 246 247 247 245 243 245 240 233 236 227 250 250 248 251 251 251 235 238 231 119 102 85 227 230 219 228 231 221 253 253 253 236 239 232 225 228 217 241 243 238 224 227 216 232 235 226 44 0 0 0 0 20 0 45 0 0 5 40 160 32 142 100 105 158 104 170 174 108 235 190 112 44 207 116 109 223 120 174 239 124 239 255 192 160 112 72 44 26 143 200 228 204 194 108 58 159 207 16 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
tab5LeftGif
	"'imgs/jure-tab-left.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 9 0 142 0 213 0 0 0 0 0 255 255 255 214 226 233 133 176 198 132 175 197 131 174 196 131 173 195 129 171 193 128 170 191 134 177 199 130 172 193 129 170 192 128 169 190 127 168 189 135 177 199 131 172 193 125 164 184 136 178 200 136 177 199 136 178 199 134 175 197 135 176 197 131 170 191 128 167 187 127 165 185 130 169 189 135 173 193 132 168 187 144 177 194 154 184 200 185 206 217 198 215 224 209 223 230 211 224 231 235 241 244 130 173 194 132 175 196 135 178 199 134 177 198 132 175 195 134 177 197 133 174 194 129 169 188 126 165 184 140 175 192 152 183 198 172 197 209 171 196 208 253 254 254 252 253 253 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 1 0 0 50 0 44 0 0 0 0 9 0 142 0 0 6 191 192 128 80 8 19 184 90 195 128 168 131 89 173 134 159 77 211 41 244 56 87 153 20 37 16 114 94 42 145 200 36 198 90 89 36 225 201 4 4 121 168 223 165 151 10 94 42 57 56 168 122 221 193 215 240 255 14 9 9 10 130 133 9 38 38 39 136 139 136 3 4 3 144 145 146 147 148 145 4 151 152 153 154 155 156 36 152 36 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 181 5 204 205 206 205 6 206 6 211 212 213 214 213 35 217 218 35 6 217 10 223 223 222 224 227 223 7 230 231 7 10 232 235 231 11 238 232 238 241 239 238 8 245 245 241 245 12 250 247 8 250 254 251 255 2 254 107 240 47 8 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
tab5LeftOnGif
	"'imgs/jure-tab-left-on.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 9 0 142 0 213 0 0 0 0 0 255 255 255 228 228 219 225 225 216 224 224 215 223 223 214 222 222 213 240 240 231 239 239 230 238 238 229 237 237 228 236 236 227 235 235 226 234 234 225 232 232 223 231 231 222 227 227 219 226 226 218 225 225 217 224 224 216 223 223 215 222 222 214 221 221 213 220 220 212 219 219 211 218 218 210 217 217 209 216 216 208 240 240 232 239 239 231 236 236 228 234 234 226 233 233 225 232 232 224 230 230 222 220 220 213 215 215 208 214 214 207 228 228 221 226 226 219 224 224 217 236 236 230 231 231 225 229 229 223 239 239 234 242 242 238 245 245 242 245 245 243 244 244 242 250 250 249 254 254 254 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 1 0 0 51 0 44 0 0 0 0 9 0 142 0 0 6 255 192 128 112 248 74 169 134 129 152 106 82 144 12 91 17 138 193 224 12 176 8 149 138 32 145 8 184 38 134 129 130 195 149 153 10 162 14 135 156 128 77 66 235 53 55 5 137 31 14 220 149 231 206 231 158 16 124 119 92 19 129 130 9 132 133 131 133 120 135 139 138 137 9 20 8 147 148 92 146 148 147 150 152 153 145 156 92 92 21 159 162 9 161 163 160 166 167 168 165 166 171 163 173 162 175 159 177 169 172 168 164 182 179 183 170 184 188 187 190 181 191 174 189 192 196 194 193 176 195 198 197 200 199 178 201 204 203 206 205 180 202 213 208 214 210 209 212 215 220 217 216 219 222 221 224 227 186 218 229 223 231 226 233 225 236 228 185 239 207 237 235 238 241 244 211 243 248 240 247 250 230 252 232 254 234 0 202 19 104 175 95 189 124 7 9 38 176 160 160 161 67 46 12 27 46 104 200 101 196 130 139 24 43 98 204 152 96 4 131 143 32 185 92 96 240 161 100 73 46 24 76 154 68 9 162 165 75 46 25 92 190 76 16 19 68 136 155 32 96 222 220 25 130 139 134 18 16 14 130 6 245 25 244 129 81 7 92 54 216 34 177 212 86 16 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
tab5RightGif
	"'imgs/jure-tab-right.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 236 0 142 0 213 0 0 0 0 0 255 255 255 133 176 198 132 175 197 131 174 196 131 173 195 129 171 193 128 170 191 126 167 188 124 164 185 134 177 199 130 172 193 129 170 192 128 169 190 127 168 189 126 166 187 125 165 186 124 164 184 123 163 183 135 177 199 129 169 190 125 164 184 123 162 182 136 178 200 136 178 199 135 176 198 134 175 197 137 178 200 137 179 200 133 173 194 127 165 185 138 179 200 139 180 201 139 179 200 133 172 192 131 169 189 130 168 187 129 167 186 128 166 185 141 181 202 140 180 201 139 178 199 138 178 198 130 167 186 142 182 202 134 172 191 132 169 188 131 168 187 134 170 189 130 173 194 132 175 196 125 166 186 135 178 199 134 177 198 131 173 193 127 166 185 126 165 184 130 170 189 140 181 201 137 177 196 131 169 187 132 169 187 133 170 188 255 255 255 33 249 4 1 0 0 63 0 44 0 0 0 0 236 0 142 0 0 6 255 64 83 169 180 34 189 120 61 151 111 185 132 57 159 208 168 116 74 173 90 175 216 172 118 203 237 122 191 93 230 210 213 235 253 206 232 223 77 72 52 34 147 226 184 124 78 175 219 239 248 188 126 207 239 251 255 128 118 46 100 60 47 47 105 63 30 38 108 69 134 111 73 131 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 131 101 60 133 36 43 105 28 31 31 32 32 40 58 58 39 179 44 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 185 41 59 45 43 67 38 103 27 171 173 174 176 178 179 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 231 182 218 177 42 35 37 207 63 23 209 172 211 175 213 214 232 249 250 251 252 253 254 255 249 98 233 64 129 2 68 8 17 38 110 252 192 32 143 131 180 122 246 8 74 156 72 177 162 197 139 24 51 106 220 200 177 163 199 143 32 67 138 252 40 112 160 68 87 174 88 117 240 240 131 6 195 134 15 33 162 156 73 179 166 205 155 56 115 234 220 201 179 167 207 255 159 64 131 10 29 106 147 222 7 14 57 112 76 112 201 112 67 180 152 70 163 74 157 74 181 170 213 171 88 179 106 221 202 181 171 215 175 96 89 205 164 234 112 67 134 10 10 150 50 189 32 207 169 67 168 97 227 202 157 75 183 174 221 187 88 223 58 101 139 129 66 141 180 106 49 8 102 219 214 169 225 195 136 19 43 94 204 184 177 227 199 144 35 75 158 76 185 178 229 201 111 57 36 38 44 152 134 141 26 127 21 0 166 65 90 240 96 194 168 83 171 94 205 186 181 235 215 176 99 203 158 77 187 182 237 219 184 83 155 238 76 99 130 6 1 2 64 135 6 60 65 45 233 227 200 147 43 95 206 188 185 243 231 208 163 75 159 78 189 186 245 235 212 119 51 47 62 65 52 240 239 193 65 139 30 207 189 188 249 243 232 211 171 95 207 190 189 251 247 240 227 203 159 79 191 126 250 241 162 65 15 24 0 190 191 240 255 0 6 40 224 128 4 22 104 224 129 8 38 168 224 130 12 54 232 224 131 195 229 71 32 112 251 85 184 95 127 24 102 168 225 134 28 118 232 255 225 135 32 134 40 226 136 36 150 104 226 137 26 90 168 226 133 40 182 232 226 139 48 198 40 227 140 38 174 184 34 141 56 230 168 227 142 60 186 104 227 143 44 246 40 228 144 68 22 137 34 144 72 86 104 228 146 76 54 201 100 146 80 58 41 229 148 84 194 8 101 148 85 102 169 229 150 27 94 153 36 151 96 134 153 165 151 72 138 105 230 153 70 146 9 36 154 108 182 153 163 154 63 186 41 231 156 45 194 105 35 157 120 230 9 162 157 55 234 233 231 159 224 241 169 34 160 132 250 41 168 133 133 38 74 231 161 74 42 234 40 155 140 6 249 232 164 96 70 202 31 165 152 110 105 105 166 156 86 185 105 167 160 54 249 105 168 164 18 57 106 169 168 238 120 106 170 172 206 184 106 171 176 250 24 105 172 180 190 248 106 173 184 134 120 107 174 188 114 184 107 175 192 246 247 107 176 196 14 75 44 176 198 30 203 107 178 202 226 202 108 179 180 62 11 45 172 210 78 203 106 181 214 162 138 109 182 164 110 203 45 168 222 126 203 105 184 226 98 74 110 185 147 158 139 255 174 163 234 174 155 104 187 238 18 10 111 188 127 206 75 175 158 246 222 139 103 190 250 206 201 111 191 110 254 11 48 164 179 14 172 109 193 6 119 139 112 194 224 46 204 240 184 14 63 108 110 196 18 167 75 113 197 236 94 140 241 187 26 111 44 111 199 30 215 11 114 200 248 142 76 242 190 38 159 236 111 202 42 7 204 114 203 4 51 10 51 199 50 207 252 113 205 54 139 140 115 206 37 239 204 51 202 62 255 188 114 208 66 187 76 116 209 49 31 138 52 208 74 47 61 116 211 78 27 13 117 212 73 11 74 181 212 86 95 93 53 159 90 111 109 103 215 104 10 12 118 140 98 143 109 235 203 102 23 89 118 218 71 162 205 182 144 107 191 93 98 220 114 143 72 119 221 186 186 141 247 155 122 239 77 227 221 126 119 8 120 224 93 246 77 184 149 134 31 46 235 209 138 247 56 120 227 129 38 14 121 141 146 79 78 226 227 150 99 62 185 230 144 115 222 184 231 138 131 126 184 232 132 147 30 184 233 126 163 190 183 234 120 91 42 169 229 124 67 41 195 236 50 184 110 255 251 237 184 231 174 251 238 188 211 46 3 1 192 7 79 128 239 196 23 111 252 241 200 39 175 252 242 204 55 239 252 243 208 71 47 253 244 209 11 47 124 1 216 103 175 253 246 214 119 239 253 247 224 135 47 254 248 228 151 111 254 249 232 167 175 254 250 215 111 239 126 246 4 188 95 64 12 244 211 47 255 253 248 231 175 255 254 252 247 239 255 255 0 12 160 0 7 248 190 250 217 79 126 6 92 128 2 23 200 192 6 58 240 129 16 140 160 4 39 72 193 10 90 240 130 24 204 160 6 55 200 65 12 26 112 126 11 52 128 8 71 72 194 18 154 240 132 40 76 161 10 87 200 194 22 186 240 133 48 140 161 12 103 72 67 25 46 48 6 11 40 33 3 118 200 128 3 248 240 135 64 12 162 16 135 72 196 34 26 241 136 72 76 162 18 151 200 196 38 58 241 137 80 20 34 15 25 80 194 28 138 112 135 63 108 128 22 183 200 197 46 122 241 139 96 12 163 24 199 72 198 50 154 241 140 104 76 163 26 215 200 70 49 254 112 135 36 196 98 23 29 64 199 58 218 255 241 142 120 204 163 30 247 200 199 62 250 241 143 128 12 164 32 7 73 200 66 26 82 143 93 60 0 28 123 120 128 45 222 17 1 144 140 164 36 39 73 201 74 90 242 146 152 204 164 38 55 201 201 78 122 242 147 160 12 165 40 37 121 199 45 250 176 135 142 164 227 36 31 192 202 86 186 242 149 176 140 165 44 103 73 203 90 218 242 150 184 204 165 46 119 201 203 94 250 146 149 147 172 163 41 27 217 0 85 66 146 149 51 72 166 50 151 201 204 102 58 243 153 208 140 166 52 167 73 205 106 90 243 154 216 204 166 54 183 217 76 96 66 146 142 90 36 166 3 34 249 128 100 66 224 156 232 76 167 58 215 201 206 118 186 243 157 240 140 167 60 231 73 207 122 218 243 158 248 204 103 59 147 249 128 72 58 192 145 199 156 1 58 19 64 208 8 24 244 160 8 77 168 66 23 202 208 134 58 244 161 16 141 168 68 39 74 209 138 90 244 162 24 53 40 65 19 128 206 25 244 19 1 255 84 101 57 207 89 80 9 152 244 164 40 77 169 74 87 202 210 150 186 244 165 65 48 141 169 76 103 74 211 154 218 244 166 56 85 105 4 8 122 78 143 130 212 152 2 133 64 2 34 128 82 11 24 245 168 72 77 170 82 151 202 212 166 58 245 169 80 141 170 84 167 74 213 170 90 245 170 88 53 42 74 119 218 211 111 6 1 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
tab5RightOnGif
	"'imgs/jure-tab-right-on.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 236 0 142 0 213 0 0 0 0 0 255 255 255 209 209 200 205 205 196 230 230 221 229 229 220 228 228 219 227 227 218 226 226 217 225 225 216 224 224 215 223 223 214 222 222 213 221 221 212 220 220 211 219 219 210 218 218 209 217 217 208 216 216 207 215 215 206 214 214 205 211 211 202 241 241 232 240 240 231 239 239 230 238 238 229 237 237 228 236 236 227 235 235 226 232 232 223 231 231 222 226 226 218 225 225 217 224 224 216 223 223 215 222 222 214 213 213 205 212 212 204 208 208 200 207 207 199 206 206 198 205 205 197 241 241 233 240 240 232 237 237 229 234 234 226 233 233 225 232 232 224 230 230 223 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 1 0 0 49 0 44 0 0 0 0 236 0 142 0 0 6 255 64 145 98 24 42 134 18 72 144 114 201 108 58 159 208 168 116 74 173 90 175 216 172 118 203 93 34 19 197 152 120 28 19 45 206 162 180 144 104 108 187 223 240 184 124 78 175 219 239 248 188 126 207 239 191 135 10 34 97 100 11 35 103 11 106 105 128 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 145 137 10 132 22 161 162 22 42 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 172 44 45 5 102 67 98 11 163 193 184 195 196 197 198 199 200 201 195 161 44 8 11 159 49 11 43 193 212 164 202 215 216 217 218 219 184 212 23 48 35 34 209 43 211 213 230 231 232 233 234 235 236 237 238 239 240 241 242 243 241 47 11 227 228 229 244 251 252 253 254 255 0 3 238 251 112 230 66 62 114 2 19 42 92 200 176 97 194 21 27 22 48 184 64 241 96 62 135 24 51 106 220 24 208 162 68 138 32 45 138 28 73 178 164 201 147 40 83 170 92 201 178 165 203 151 48 83 142 58 121 225 35 200 155 6 99 234 220 201 179 167 207 255 159 64 123 130 180 137 179 168 209 163 72 147 42 93 202 180 169 211 167 80 163 74 157 250 244 224 82 6 19 169 106 221 202 181 171 215 175 96 195 30 101 208 0 131 216 179 104 211 170 93 203 150 34 89 12 112 227 182 157 75 183 174 221 180 111 227 234 133 123 183 175 223 191 128 111 230 221 171 55 176 225 195 136 187 234 109 80 150 112 225 196 144 35 75 6 233 24 46 227 202 114 39 107 222 236 23 51 134 203 158 57 139 30 189 214 243 231 198 152 73 171 94 237 213 52 232 212 172 99 203 126 234 26 117 229 217 184 115 31 173 109 90 183 111 223 188 67 255 30 46 59 56 108 226 200 73 27 191 157 188 57 231 229 142 157 75 159 12 157 240 244 235 137 171 239 197 206 61 176 246 199 221 195 219 253 158 89 188 249 182 228 249 158 95 175 54 189 89 246 240 197 186 143 79 255 235 252 250 248 181 222 207 207 31 234 254 254 0 46 245 95 128 4 26 53 96 129 8 82 116 96 130 5 46 200 96 128 14 62 216 95 132 18 230 71 97 133 245 93 136 97 124 26 110 200 255 94 135 30 158 7 98 136 226 141 72 98 119 38 158 136 93 138 42 78 199 98 139 206 189 8 99 114 50 206 72 92 141 54 254 134 99 142 186 237 200 35 110 62 254 88 156 103 175 49 39 164 121 65 30 185 90 146 74 42 71 164 109 209 53 25 30 147 82 62 247 100 111 85 114 71 101 150 212 93 41 28 151 46 122 121 28 152 49 138 105 36 153 205 109 137 38 98 106 174 105 88 155 110 2 6 103 156 157 153 25 37 157 195 205 137 231 120 118 90 183 167 142 125 110 247 39 112 129 130 55 40 144 133 150 119 232 108 122 46 90 90 162 234 57 58 36 102 69 222 41 41 107 141 94 122 86 166 154 134 197 105 167 246 65 250 30 168 78 82 10 165 159 164 142 246 105 170 91 173 202 42 85 174 190 42 85 172 178 250 39 106 173 155 209 138 107 83 186 238 42 224 173 190 70 214 107 176 72 13 75 172 129 192 30 123 152 177 202 222 196 108 179 10 38 11 237 95 207 66 91 109 179 215 42 155 237 177 219 18 219 109 176 223 250 26 238 174 227 226 90 110 173 231 202 154 255 238 171 235 178 218 110 170 239 146 26 47 168 243 118 90 175 166 247 94 154 175 164 251 58 218 239 162 255 30 234 94 164 211 214 101 154 101 14 100 160 240 193 12 55 236 240 195 16 71 44 241 196 20 87 108 113 196 10 103 224 192 3 26 116 236 113 198 32 135 44 242 200 36 151 108 242 201 40 167 172 242 202 44 183 236 242 203 48 175 236 177 199 15 60 176 193 205 56 231 172 243 204 60 247 236 243 207 64 7 45 244 208 68 23 109 244 209 72 39 173 244 210 58 55 157 115 205 78 115 32 181 212 78 87 109 245 213 88 103 173 245 214 92 119 237 245 215 96 135 45 246 212 84 55 93 51 217 45 164 173 246 218 108 183 237 246 219 112 199 45 247 220 116 215 109 247 221 120 231 173 247 222 119 147 253 0 4 106 187 32 248 224 132 23 110 248 225 136 39 174 248 226 140 55 238 248 227 144 71 46 249 228 148 75 174 54 4 17 20 254 194 230 47 116 224 249 231 160 135 46 250 232 164 151 110 250 233 168 167 174 250 234 172 183 238 250 235 176 139 206 249 11 133 71 32 255 129 224 155 127 238 193 238 188 247 238 251 239 192 7 47 252 240 196 23 111 252 241 200 39 175 252 242 204 55 63 252 231 155 11 46 129 4 185 251 78 192 245 216 103 175 253 246 220 119 239 253 247 224 135 47 254 248 228 151 111 254 249 232 167 223 189 239 29 108 46 193 4 158 243 126 125 1 244 27 96 255 253 248 231 175 255 254 252 247 239 255 255 0 12 160 0 7 72 192 2 26 240 128 8 196 31 253 10 112 61 222 121 110 2 20 144 95 253 240 119 128 10 90 240 130 24 204 160 6 55 200 193 14 122 240 131 32 12 161 8 71 72 194 18 154 240 132 21 204 31 253 8 192 59 10 68 208 3 4 40 192 253 42 136 128 26 218 240 134 56 204 161 14 119 200 195 30 250 240 135 64 12 162 16 135 72 196 34 26 241 136 57 76 161 253 24 184 59 10 144 192 3 50 180 223 1 106 216 133 42 90 241 138 88 204 162 22 183 88 197 26 30 224 126 5 240 0 9 72 64 0 41 34 96 9 70 72 132 26 215 200 198 54 186 241 141 112 140 163 28 231 72 199 58 218 119 241 142 120 204 99 27 141 176 4 4 124 209 0 4 32 65 9 100 56 69 37 20 65 4 35 72 164 34 23 201 200 70 58 242 145 144 140 164 36 39 73 201 74 90 242 146 152 204 164 38 55 217 72 65 132 64 9 126 52 64 1 74 80 130 10 84 64 0 38 48 193 9 80 144 130 86 166 96 0 176 140 165 44 103 73 203 90 218 242 150 184 204 165 46 119 201 203 94 250 242 151 192 12 166 48 97 233 202 20 160 224 4 169 20 128 41 75 16 4 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
textFontSizeGif
	"'imgs/fast/fast-font_size.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 54 0 14 0 128 1 0 153 153 153 238 238 238 33 249 4 1 0 0 1 0 44 0 0 0 0 54 0 14 0 0 2 75 140 143 169 203 237 15 163 76 0 204 139 143 189 251 245 252 69 97 230 85 155 169 161 129 57 170 167 186 86 200 215 213 134 189 46 33 173 245 247 236 203 9 133 163 27 110 24 131 81 130 39 95 209 249 3 42 120 169 99 109 39 139 178 170 71 146 247 11 14 139 199 228 114 163 0 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
textSizeLargeGif
	"'imgs/fast/fast-t_lg.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 16 0 15 0 196 28 0 249 249 246 250 250 248 254 254 253 252 252 250 252 252 251 246 246 242 248 248 245 247 247 244 243 243 238 241 241 235 245 245 240 255 255 254 247 247 243 242 242 237 251 251 250 244 244 239 244 244 240 249 249 247 200 200 185 253 253 253 245 245 241 240 240 233 242 242 236 250 250 247 254 254 254 253 253 252 251 251 249 245 245 236 177 176 176 232 232 222 196 196 181 255 255 255 33 249 4 1 0 0 28 0 44 0 0 0 0 16 0 15 0 0 5 96 224 182 117 100 105 146 226 248 173 108 187 122 170 43 191 221 60 123 245 202 237 252 206 226 50 159 11 232 18 178 22 196 150 17 35 200 36 89 62 102 134 160 121 234 56 147 204 64 19 0 60 23 152 221 246 2 56 20 136 82 130 35 16 49 152 21 15 92 154 11 48 48 10 10 68 35 1 147 108 187 118 20 16 8 22 9 21 48 34 30 138 139 140 138 34 33 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
textSizeMediumGif
	"'imgs/fast/fast-t_med.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 16 0 15 0 196 28 0 249 249 246 252 252 251 250 250 248 254 254 253 252 252 250 248 248 245 246 246 242 243 243 238 247 247 243 245 245 240 241 241 235 255 255 254 253 253 253 247 247 244 200 200 185 250 250 247 242 242 237 245 245 241 240 240 233 242 242 236 251 251 250 244 244 240 249 249 247 244 244 239 254 254 254 253 253 252 251 251 249 245 245 236 177 176 176 232 232 222 196 196 181 255 255 255 33 249 4 1 0 0 28 0 44 0 0 0 0 16 0 15 0 0 5 96 224 182 117 100 105 146 226 248 173 108 187 122 170 43 191 221 60 123 181 237 226 45 231 255 45 158 203 231 90 8 123 28 22 102 144 57 178 136 203 76 64 227 92 249 24 25 130 70 0 112 46 48 140 128 246 1 104 24 132 209 0 69 96 41 152 19 23 92 122 11 40 32 12 137 3 68 1 115 104 185 118 17 21 7 19 10 18 48 34 30 138 139 140 138 34 33 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
textSizeSmallGif
	"'imgs/fast/fast-t_med.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 16 0 15 0 196 28 0 249 249 246 252 252 251 250 250 248 254 254 253 252 252 250 248 248 245 246 246 242 243 243 238 247 247 243 245 245 240 241 241 235 255 255 254 253 253 253 247 247 244 200 200 185 250 250 247 242 242 237 245 245 241 240 240 233 242 242 236 251 251 250 244 244 240 249 249 247 244 244 239 254 254 254 253 253 252 251 251 249 245 245 236 177 176 176 232 232 222 196 196 181 255 255 255 33 249 4 1 0 0 28 0 44 0 0 0 0 16 0 15 0 0 5 96 224 182 117 100 105 146 226 248 173 108 187 122 170 43 191 221 60 123 181 237 226 45 231 255 45 158 203 231 90 8 123 28 22 102 144 57 178 136 203 76 64 227 92 249 24 25 130 70 0 112 46 48 140 128 246 1 104 24 132 209 0 69 96 41 152 19 23 92 122 11 40 32 12 137 3 68 1 115 104 185 118 17 21 7 19 10 18 48 34 30 138 139 140 138 34 33 0 59 13 10 )! !

!DefaultWebStyle methodsFor: 'imgs-other' stamp: ' 21/4/07 10:33'!
transparentPixelGif
	"transparent 1x1 gif"
	"'imgs/trapix.gif' asFilename contentsAsMethod"
^#(71 73 70 56 55 97 1 0 1 0 179 0 0 255 255 255 196 0 67 255 255 255 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 1 0 0 0 0 44 0 0 0 0 1 0 1 0 0 4 2 16 68 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-filetype icons' stamp: ' 21/4/07 10:33'!
txtSmallPng
	"'imgs/ikone-dok/corendal-txt.png' asFilename contentsAsMethod"
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 2 0 0 0 144 145 104 54 0 0 0 4 103 65 77 65 0 0 88 199 252 71 224 2 0 0 0 32 99 72 82 77 0 0 122 37 0 0 128 131 0 0 249 255 0 0 128 230 0 0 117 46 0 0 234 95 0 0 58 151 0 0 23 111 105 228 196 43 0 0 1 95 73 68 65 84 120 156 98 248 143 23 252 252 254 9 77 4 32 128 24 224 172 51 27 75 238 156 206 155 90 104 181 126 74 8 144 177 109 217 20 32 185 105 201 226 157 75 187 145 53 0 4 16 84 195 175 95 127 55 111 190 83 81 113 112 66 182 225 158 85 19 123 43 243 119 110 187 188 114 66 254 190 45 187 128 26 158 61 123 6 215 0 16 64 80 13 64 213 71 142 61 106 203 116 122 249 229 19 144 124 243 237 11 144 252 254 251 23 144 4 202 174 92 185 18 174 1 32 128 16 26 118 159 189 0 84 13 65 64 13 31 127 124 7 106 248 243 247 47 154 6 128 0 66 104 56 120 254 26 49 26 0 2 8 161 97 255 185 171 64 165 117 245 245 16 180 123 239 94 172 26 0 2 8 73 195 217 171 196 216 0 16 64 8 13 251 46 92 68 182 1 98 9 68 22 89 3 64 0 33 105 56 123 133 24 27 0 2 8 201 73 167 110 16 99 3 64 0 33 133 210 185 155 196 216 0 16 64 8 13 123 206 92 129 216 0 52 24 143 6 128 0 66 104 248 240 229 43 196 248 215 95 63 191 249 250 229 200 169 147 123 15 30 92 188 108 233 164 201 147 111 220 184 1 215 0 16 64 136 196 247 245 219 247 187 247 239 55 180 180 228 21 20 228 228 228 148 149 149 29 58 116 232 210 165 75 143 30 61 250 249 243 39 92 25 64 0 33 52 0 193 135 15 31 94 190 124 249 233 211 39 160 138 191 96 199 96 2 128 0 3 0 49 229 159 98 54 52 32 115 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!DefaultWebStyle methodsFor: 'imgs-filetype icons' stamp: ' 21/4/07 10:33'!
unknownSmallPng
	"'imgs/ikone-dok/corendal-unk.png' asFilename contentsAsMethod"
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 2 0 0 0 144 145 104 54 0 0 0 4 103 65 77 65 0 0 88 199 252 71 224 2 0 0 0 32 99 72 82 77 0 0 122 37 0 0 128 131 0 0 249 255 0 0 128 230 0 0 117 46 0 0 234 95 0 0 58 151 0 0 23 111 105 228 196 43 0 0 1 250 73 68 65 84 120 156 98 248 255 255 255 133 11 231 175 92 185 120 249 50 20 93 189 122 5 130 128 130 231 207 159 123 248 240 225 127 36 0 16 64 12 64 12 84 244 31 7 248 248 241 195 129 3 251 144 245 0 4 16 1 13 47 95 190 2 146 103 207 158 122 244 232 17 68 4 32 128 8 104 248 250 245 211 221 187 183 15 29 218 63 101 202 20 136 8 64 0 129 52 92 185 114 9 72 222 188 245 240 194 133 199 151 47 223 7 162 75 151 238 159 63 255 16 136 46 92 120 112 254 252 213 75 151 206 246 246 246 64 52 0 4 16 92 195 191 240 208 94 57 169 76 14 238 137 28 28 19 132 4 38 105 106 214 105 106 86 201 201 85 7 4 128 204 62 115 230 4 68 3 64 0 65 53 252 249 243 123 254 130 5 123 54 175 222 186 100 253 198 77 187 150 173 223 188 103 207 150 109 91 118 172 88 190 170 171 171 15 168 230 226 197 75 16 13 0 1 4 213 240 227 199 247 131 39 143 126 220 178 234 82 150 223 147 175 95 53 250 230 3 197 211 139 174 102 231 175 95 182 108 246 191 127 255 46 92 184 0 209 0 16 64 72 26 14 238 59 118 124 255 180 185 61 47 95 127 40 173 219 250 243 231 159 105 11 23 207 89 188 104 222 220 57 200 26 0 2 8 161 97 215 190 237 191 31 126 253 191 247 253 15 80 200 204 253 241 235 235 199 45 155 111 174 93 49 109 14 138 13 0 1 132 208 112 228 212 193 43 43 207 31 204 221 253 234 237 141 186 6 247 31 223 190 44 136 138 88 221 92 55 127 201 98 100 13 0 1 4 213 240 243 231 143 89 51 167 207 95 50 187 111 70 215 198 13 235 59 58 186 118 237 218 214 57 169 119 202 204 201 109 173 173 255 65 201 7 170 1 32 128 16 161 52 121 242 100 96 96 79 156 56 1 72 78 158 60 169 167 167 123 210 164 137 221 221 93 171 87 175 6 170 57 127 254 60 68 3 64 0 17 136 105 56 56 119 238 28 132 1 16 64 32 13 87 175 94 38 168 225 226 69 168 161 0 1 4 210 112 228 200 33 160 19 207 225 6 64 247 236 218 181 11 162 1 32 192 0 104 150 135 8 60 245 67 255 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!DefaultWebStyle methodsFor: 'imgs-filetype icons' stamp: ' 21/4/07 10:33'!
wordBigGif
	"'imgs/word_icon.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 34 0 32 0 135 0 0 0 0 0 0 0 255 128 128 128 192 192 192 192 220 192 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 3 0 0 4 0 44 0 0 0 0 34 0 32 0 0 8 255 0 9 8 28 72 176 160 193 131 8 13 10 88 200 176 161 67 134 9 15 10 40 64 177 162 197 139 5 4 0 136 88 112 34 198 143 20 23 110 52 8 32 64 201 147 38 83 162 4 41 96 128 70 131 42 99 162 60 201 178 128 203 145 3 75 130 180 168 179 226 128 159 46 41 222 36 104 114 103 197 162 62 129 122 180 249 82 224 76 153 49 45 254 12 9 160 170 213 145 81 83 22 144 57 213 230 197 1 62 71 206 44 64 179 108 0 169 93 187 218 196 10 128 236 89 173 38 39 154 76 11 214 103 88 167 111 159 150 4 219 83 232 84 160 119 9 20 213 139 116 110 93 160 136 3 235 132 123 210 99 95 163 107 5 14 158 188 151 162 97 196 152 21 231 221 220 150 226 99 163 3 216 66 222 10 0 179 105 205 132 159 142 22 202 22 170 94 211 153 89 39 92 186 26 99 232 217 73 97 195 14 44 177 54 200 219 8 151 234 214 205 91 161 239 143 192 123 143 62 45 59 248 113 219 56 141 63 151 26 189 227 116 234 179 135 107 71 92 157 224 213 239 224 195 91 5 229 72 158 124 64 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-filetype icons' stamp: ' 21/4/07 10:33'!
wordSmallPng
	"'imgs/ikone-dok/corendal-word.png' asFilename contentsAsMethod"
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 2 0 0 0 144 145 104 54 0 0 0 4 103 65 77 65 0 0 88 199 252 71 224 2 0 0 0 32 99 72 82 77 0 0 122 37 0 0 128 131 0 0 249 255 0 0 128 230 0 0 117 46 0 0 234 95 0 0 58 151 0 0 23 111 105 228 196 43 0 0 2 120 73 68 65 84 120 156 98 248 15 3 155 14 221 216 120 240 250 134 3 215 214 237 187 178 118 239 165 213 187 46 174 218 121 126 197 182 179 203 182 157 94 186 229 212 191 127 80 101 0 1 196 48 121 245 51 32 234 92 250 178 97 222 179 154 233 55 210 187 174 70 212 93 12 42 63 229 148 119 68 43 105 251 231 239 255 222 126 249 59 103 245 1 115 239 212 127 96 77 0 1 4 210 240 31 7 224 244 222 244 229 251 191 247 95 254 78 93 178 115 215 209 43 16 61 0 1 196 0 52 123 213 190 143 205 115 110 37 52 158 189 118 231 21 144 188 243 232 75 104 238 146 238 37 119 65 26 126 252 125 255 245 239 228 69 219 129 170 33 8 32 128 24 128 46 1 170 0 106 136 175 61 12 100 184 100 237 4 146 190 89 203 203 187 230 0 53 76 88 119 13 136 94 126 252 187 116 207 157 37 187 239 2 53 0 4 16 67 249 196 171 64 219 129 36 80 233 250 61 247 21 125 102 1 205 182 138 154 113 242 210 35 54 167 217 159 127 252 3 105 248 240 231 225 155 63 11 119 221 53 243 74 1 8 32 6 160 47 129 26 210 235 54 3 53 196 150 174 102 53 107 50 138 222 166 97 151 11 20 4 106 248 244 13 164 225 217 187 63 247 95 253 158 187 227 30 80 3 64 0 49 132 86 159 3 202 1 29 13 52 117 251 137 47 64 13 64 180 118 215 5 160 32 135 77 7 208 199 64 13 79 222 254 185 251 242 247 204 109 15 128 26 0 2 136 193 179 228 12 72 78 191 219 34 113 221 135 79 191 128 170 29 210 182 64 66 9 200 134 248 225 225 171 223 183 158 253 158 186 249 17 80 3 64 0 49 88 101 238 192 21 172 140 70 21 47 222 255 121 244 230 207 157 23 191 175 61 249 213 191 241 49 80 3 64 0 49 0 99 7 24 26 16 4 116 52 4 1 29 3 52 30 168 1 98 195 245 167 191 47 61 252 213 181 238 9 80 3 64 0 49 192 205 155 185 98 223 139 15 127 95 126 248 11 36 159 191 255 243 244 237 159 135 175 127 3 205 190 254 244 215 229 71 191 206 221 251 217 186 230 25 80 3 64 0 33 52 76 91 182 251 209 155 223 64 7 64 76 69 70 231 238 253 56 117 251 71 227 242 231 64 13 0 1 132 208 0 140 127 160 207 110 61 255 125 243 217 239 27 79 127 93 123 242 251 242 163 223 23 30 252 60 115 247 199 201 219 63 143 223 250 81 179 228 37 80 3 64 0 33 52 0 227 255 210 163 95 243 118 220 7 6 223 212 45 143 38 110 124 212 179 254 73 199 218 167 45 171 159 53 44 127 94 189 244 69 197 226 215 64 13 0 1 132 208 0 228 16 131 0 2 12 0 71 114 23 164 38 70 157 109 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!DefaultWebStyle methodsFor: 'imgs-filetype icons' stamp: ' 21/4/07 10:33'!
xmlBigGif
	"'imgs/xml.gif' asFilename contentsAsMethod"
^#(71 73 70 56 57 97 32 0 32 0 179 10 0 153 102 153 128 128 128 17 17 17 198 198 198 204 204 204 102 102 102 102 51 102 0 0 0 204 153 204 255 255 255 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 33 249 4 1 0 0 10 0 44 0 0 0 0 32 0 32 0 0 4 255 80 73 5 140 189 56 235 12 166 50 72 40 142 100 73 26 30 24 30 34 139 184 237 90 162 147 138 20 236 81 200 163 11 139 52 137 237 166 107 193 14 190 89 138 244 67 174 112 188 81 240 211 131 62 115 175 40 112 25 59 102 177 217 19 247 181 179 98 193 191 208 84 85 12 65 89 184 157 15 233 68 172 221 164 50 254 230 46 248 119 118 99 38 131 131 107 27 135 136 23 30 7 9 141 142 143 144 145 144 7 139 146 150 151 141 148 19 140 152 157 143 154 18 156 158 158 160 10 162 163 152 165 167 168 150 140 2 175 176 177 177 172 9 7 2 163 3 183 168 182 9 3 190 191 192 190 1 186 146 116 198 183 192 2 190 202 191 195 150 4 7 4 210 208 5 200 203 201 205 196 144 208 211 58 10 214 204 216 3 206 146 220 212 18 200 225 193 202 228 145 220 222 232 189 185 193 243 227 218 143 208 240 241 191 234 225 195 198 0 245 237 227 119 45 219 159 131 127 60 12 164 7 108 152 194 135 10 217 5 152 72 177 98 128 92 16 51 42 72 32 171 35 172 4 26 3 33 70 0 0 59 )! !

!DefaultWebStyle methodsFor: 'imgs-filetype icons' stamp: ' 21/4/07 10:33'!
xmlSmallPng
	"'imgs/ikone-dok/corendal-xml.png' asFilename contentsAsMethod"
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 2 0 0 0 144 145 104 54 0 0 0 4 103 65 77 65 0 0 88 199 252 71 224 2 0 0 0 32 99 72 82 77 0 0 122 37 0 0 128 131 0 0 249 255 0 0 128 230 0 0 117 46 0 0 234 95 0 0 58 151 0 0 23 111 105 228 196 43 0 0 0 208 73 68 65 84 120 156 98 104 35 2 252 71 2 0 1 196 128 198 199 4 7 14 28 64 86 3 16 64 56 53 124 252 248 17 66 2 53 0 25 112 101 0 1 132 208 0 87 129 70 50 32 1 32 23 32 128 160 26 128 114 203 78 125 128 170 104 104 0 34 227 180 255 232 226 96 13 0 1 4 213 0 20 101 96 152 9 81 13 36 141 129 104 38 80 69 3 88 221 76 136 30 136 6 128 0 2 105 128 171 6 137 2 205 6 170 62 3 212 0 84 208 0 51 123 38 88 13 72 3 64 0 161 219 0 116 9 208 108 176 106 236 54 0 4 16 54 63 48 52 64 16 86 63 0 4 16 225 80 130 147 16 13 0 1 132 176 193 56 109 38 46 132 108 3 64 0 145 108 3 64 0 145 108 3 64 0 145 108 3 64 0 33 108 32 72 66 52 0 4 16 3 73 0 168 1 32 192 0 122 128 24 248 27 252 237 14 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!DefaultWebStyle methodsFor: 'imgs-filetype icons' stamp: ' 21/4/07 10:33'!
zipSmallPng
	"'imgs/ikone-dok/corendal-zip.png' asFilename contentsAsMethod"
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 2 0 0 0 144 145 104 54 0 0 0 4 103 65 77 65 0 0 88 199 252 71 224 2 0 0 0 32 99 72 82 77 0 0 122 37 0 0 128 131 0 0 249 255 0 0 128 230 0 0 117 46 0 0 234 95 0 0 58 151 0 0 23 111 105 228 196 43 0 0 2 37 73 68 65 84 120 156 98 248 143 1 90 14 127 110 60 240 169 122 207 199 210 237 31 242 54 190 75 91 243 54 113 197 107 184 44 64 0 49 188 184 216 126 126 33 31 4 1 217 64 185 47 255 255 31 127 243 119 207 211 223 107 111 255 156 125 254 107 199 225 207 101 219 63 120 182 95 132 104 0 8 32 6 160 186 143 146 140 16 4 100 171 103 30 71 67 86 13 23 98 86 127 139 158 121 157 197 123 3 80 3 64 0 1 53 8 220 218 162 117 115 187 230 205 253 90 55 247 235 220 220 163 127 115 187 225 237 157 198 183 118 153 222 219 103 189 165 134 31 168 168 105 227 253 210 141 79 32 26 0 2 136 225 234 106 165 127 127 103 254 255 127 228 255 255 157 255 255 175 255 255 119 201 255 223 179 254 255 154 240 255 71 219 231 39 185 155 106 249 128 138 38 108 187 15 244 24 68 3 64 0 49 92 93 37 255 235 219 228 159 223 158 254 120 119 234 215 203 181 191 30 247 255 190 87 242 231 102 196 191 203 22 247 182 50 66 52 76 219 121 191 251 212 79 136 6 128 0 98 184 2 212 240 117 226 239 207 231 126 189 222 252 251 249 244 63 143 106 127 223 77 249 123 211 253 207 21 67 172 26 0 2 136 40 13 83 182 35 52 0 4 16 208 73 138 191 190 78 248 253 97 239 175 23 11 255 60 105 253 115 47 247 239 173 192 191 87 45 191 157 213 129 107 152 177 243 126 231 201 239 16 13 0 1 196 112 105 153 200 247 247 77 55 214 105 92 89 46 125 117 153 216 213 101 2 87 151 114 95 89 204 113 105 1 59 92 195 132 205 247 138 54 62 131 104 0 8 32 134 157 173 154 155 170 149 87 231 242 174 202 229 93 83 34 139 25 15 133 115 46 23 172 127 23 187 244 53 68 3 64 0 49 160 165 11 72 76 103 108 255 159 184 249 127 209 238 255 21 251 64 8 168 218 186 234 8 68 1 64 0 161 107 128 232 209 201 218 173 145 186 77 57 126 3 16 41 70 175 133 152 13 1 0 1 132 69 3 154 102 228 148 7 4 0 1 132 93 67 240 172 71 37 75 30 228 46 124 16 53 229 134 95 255 13 231 174 235 150 173 215 133 252 87 2 165 0 2 8 151 134 39 39 110 124 219 114 254 219 140 67 95 203 183 124 9 91 246 197 98 230 103 177 128 229 64 41 128 0 3 0 79 174 47 198 171 105 151 124 0 0 0 0 73 69 78 68 174 66 96 130 )! !

DefaultWebStyle subclass: #JoomlaStyle
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Templates'!

!JoomlaStyle methodsFor: 'styles-screen' stamp: 'np 4/3/2008 04:16'!
addSuperStyles
	^false! !

!JoomlaStyle methodsFor: 'maintenance' stamp: 'np 4/3/2008 05:21'!
adjustCSSLinks
	self adjustLinksInCSSMethod: #cssJoomlaTemplate! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:24'!
arrowPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 9 0 0 0 9 8 3 0 0 1 160 72 198 180 0 0 0 6 80 76 84 69 135 185 42 255 255 255 116 52 158 130 0 0 0 2 116 82 78 83 255 0 229 183 48 74 0 0 0 9 112 72 89 115 0 0 10 235 0 0 10 235 1 130 139 13 90 0 0 0 51 73 68 65 84 120 156 53 139 65 14 0 48 8 194 74 194 255 191 236 64 179 131 68 108 181 48 88 206 140 156 118 153 66 9 133 13 102 205 80 232 166 5 181 56 113 16 255 229 223 142 62 184 175 7 88 67 87 53 183 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:24'!
bacFooterPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 2 0 0 0 26 8 2 0 0 1 101 86 42 19 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 0 99 73 68 65 84 120 156 125 141 49 10 0 33 16 3 87 141 173 79 216 214 255 63 201 78 16 65 43 177 242 140 32 7 87 92 32 12 33 27 22 115 78 140 49 224 156 3 181 214 66 239 29 222 123 164 148 16 66 128 181 22 198 152 67 154 155 156 51 84 21 173 181 211 209 220 222 219 175 107 173 34 130 82 10 17 99 36 246 152 216 61 33 71 55 253 97 63 121 211 3 89 72 36 16 46 250 206 227 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:25'!
bacLogoJpg
^#(255 216 255 224 0 16 74 70 73 70 0 1 1 1 0 72 0 72 0 0 255 225 50 29 104 116 116 112 58 47 47 110 115 46 97 100 111 98 101 46 99 111 109 47 120 97 112 47 49 46 48 47 0 60 63 120 112 97 99 107 101 116 32 98 101 103 105 110 61 34 239 187 191 34 32 105 100 61 34 87 53 77 48 77 112 67 101 104 105 72 122 114 101 83 122 78 84 99 122 107 99 57 100 34 63 62 10 60 120 58 120 109 112 109 101 116 97 32 120 109 108 110 115 58 120 61 34 97 100 111 98 101 58 110 115 58 109 101 116 97 47 34 32 120 58 120 109 112 116 107 61 34 65 100 111 98 101 32 88 77 80 32 67 111 114 101 32 52 46 49 45 99 48 51 52 32 52 54 46 50 55 50 57 55 54 44 32 83 97 116 32 74 97 110 32 50 55 32 50 48 48 55 32 50 50 58 51 55 58 51 55 32 32 32 32 32 32 32 32 34 62 10 32 32 32 60 114 100 102 58 82 68 70 32 120 109 108 110 115 58 114 100 102 61 34 104 116 116 112 58 47 47 119 119 119 46 119 51 46 111 114 103 47 49 57 57 57 47 48 50 47 50 50 45 114 100 102 45 115 121 110 116 97 120 45 110 115 35 34 62 10 32 32 32 32 32 32 60 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 32 114 100 102 58 97 98 111 117 116 61 34 34 10 32 32 32 32 32 32 32 32 32 32 32 32 120 109 108 110 115 58 120 97 112 61 34 104 116 116 112 58 47 47 110 115 46 97 100 111 98 101 46 99 111 109 47 120 97 112 47 49 46 48 47 34 62 10 32 32 32 32 32 32 32 32 32 60 120 97 112 58 67 114 101 97 116 111 114 84 111 111 108 62 65 100 111 98 101 32 70 105 114 101 119 111 114 107 115 32 67 83 51 60 47 120 97 112 58 67 114 101 97 116 111 114 84 111 111 108 62 10 32 32 32 32 32 32 32 32 32 60 120 97 112 58 67 114 101 97 116 101 68 97 116 101 62 50 48 48 55 45 48 53 45 50 57 84 48 55 58 51 54 58 52 52 90 60 47 120 97 112 58 67 114 101 97 116 101 68 97 116 101 62 10 32 32 32 32 32 32 32 32 32 60 120 97 112 58 77 111 100 105 102 121 68 97 116 101 62 50 48 48 55 45 48 53 45 50 57 84 48 55 58 51 57 58 52 48 90 60 47 120 97 112 58 77 111 100 105 102 121 68 97 116 101 62 10 32 32 32 32 32 32 60 47 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 62 10 32 32 32 32 32 32 60 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 32 114 100 102 58 97 98 111 117 116 61 34 34 10 32 32 32 32 32 32 32 32 32 32 32 32 120 109 108 110 115 58 100 99 61 34 104 116 116 112 58 47 47 112 117 114 108 46 111 114 103 47 100 99 47 101 108 101 109 101 110 116 115 47 49 46 49 47 34 62 10 32 32 32 32 32 32 32 32 32 60 100 99 58 102 111 114 109 97 116 62 105 109 97 103 101 47 106 112 101 103 60 47 100 99 58 102 111 114 109 97 116 62 10 32 32 32 32 32 32 60 47 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 62 10 32 32 32 60 47 114 100 102 58 82 68 70 62 10 60 47 120 58 120 109 112 109 101 116 97 62 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 60 63 120 112 97 99 107 101 116 32 101 110 100 61 34 119 34 63 62 255 219 0 67 0 4 3 3 3 3 3 4 3 3 4 5 4 3 4 5 7 5 4 4 5 7 8 6 6 7 6 6 8 10 8 8 8 8 8 8 10 8 10 10 11 10 10 8 13 13 14 14 13 13 18 18 18 18 18 20 20 20 20 20 20 20 20 20 20 255 219 0 67 1 5 5 5 8 7 8 15 10 10 15 18 15 12 15 18 22 21 21 21 21 22 22 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 255 194 0 17 8 0 219 0 3 3 1 17 0 2 17 1 3 17 1 255 196 0 24 0 1 0 3 1 0 0 0 0 0 0 0 0 0 0 0 0 0 4 5 6 7 255 196 0 24 1 1 0 3 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 4 6 2 255 218 0 12 3 1 0 2 16 3 16 0 0 1 239 80 198 101 34 21 8 0 0 0 0 0 0 0 0 5 141 246 147 89 215 255 196 0 26 16 0 2 2 3 0 0 0 0 0 0 0 0 0 0 0 0 0 2 18 20 3 4 64 255 218 0 8 1 1 0 1 5 2 154 150 243 244 209 218 34 167 255 196 0 20 17 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 64 255 218 0 8 1 3 1 1 63 1 103 255 196 0 28 17 0 0 6 3 0 0 0 0 0 0 0 0 0 0 0 0 1 4 21 82 161 209 0 16 64 255 218 0 8 1 2 1 1 63 1 197 35 14 128 174 148 211 13 144 189 127 255 196 0 28 16 0 1 3 5 0 0 0 0 0 0 0 0 0 0 0 0 0 1 2 49 50 64 146 161 209 255 218 0 8 1 1 0 6 63 2 146 83 22 242 230 141 161 7 255 196 0 24 16 0 2 3 0 0 0 0 0 0 0 0 0 0 0 0 0 0 64 97 209 240 255 218 0 8 1 1 0 1 63 33 107 24 222 242 33 255 218 0 12 3 1 0 2 0 3 0 0 0 16 217 182 219 109 182 219 109 182 219 109 175 255 196 0 22 17 0 3 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 17 64 255 218 0 8 1 3 1 1 63 16 29 127 255 196 0 22 17 0 3 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 17 64 255 218 0 8 1 2 1 1 63 16 175 12 17 255 196 0 21 16 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 64 97 255 218 0 8 1 1 0 1 63 16 153 153 41 207 255 217 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:25'!
bacModPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 201 0 0 0 4 8 3 0 0 1 100 105 219 203 0 0 0 45 80 76 84 69 216 216 216 217 217 217 227 227 227 228 228 228 237 237 237 238 238 238 246 246 246 247 247 247 248 248 248 249 249 249 250 250 250 252 252 252 253 253 253 254 254 254 255 255 255 133 212 169 111 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 0 97 73 68 65 84 120 156 197 208 73 14 128 32 12 5 80 138 31 101 240 254 23 85 130 50 136 38 186 196 5 38 141 111 219 180 191 45 140 18 109 9 122 108 87 10 132 156 95 186 218 182 8 27 116 116 93 77 157 118 229 13 48 37 205 25 178 142 42 128 136 56 51 196 53 159 96 83 118 229 200 172 47 99 229 33 7 185 64 161 222 254 222 231 187 231 128 122 2 250 44 33 69 182 32 90 206 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:26'!
botLeftPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 12 0 0 0 12 8 2 0 0 1 174 16 251 38 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 1 10 73 68 65 84 120 156 109 80 201 138 131 80 16 124 106 199 221 144 131 34 46 209 147 255 224 255 223 242 11 66 12 8 130 11 232 65 112 37 58 149 152 137 115 152 130 246 65 87 117 117 181 52 207 243 233 116 34 84 154 166 180 109 27 77 211 68 235 186 222 110 183 56 142 41 73 146 40 138 24 99 100 89 22 123 131 46 151 75 93 215 109 219 210 178 44 121 158 135 97 200 126 65 128 235 186 101 89 130 243 125 255 213 66 193 223 182 237 231 243 57 12 3 86 82 211 52 154 166 137 162 120 12 66 2 75 168 142 22 120 44 66 40 142 227 14 149 105 154 247 251 61 8 130 125 156 64 194 11 131 85 85 193 30 217 95 27 121 158 215 117 93 146 36 172 67 20 194 211 247 61 236 28 199 129 156 253 7 18 4 1 166 152 43 138 226 122 189 254 205 119 136 224 12 226 124 62 67 157 101 153 44 203 158 231 125 19 126 68 40 208 224 118 203 113 28 31 143 7 46 209 223 192 21 104 210 174 133 31 98 162 165 170 170 97 24 123 222 174 235 240 183 62 183 124 193 191 1 169 162 40 48 3 141 47 250 63 196 210 127 221 158 100 237 238 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:26'!
botModPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 201 0 0 0 32 8 3 0 0 1 248 84 156 235 0 0 0 105 80 76 84 69 199 199 199 216 216 216 217 217 217 218 218 218 219 219 219 220 220 220 221 221 221 222 222 222 224 224 224 225 225 225 226 226 226 227 227 227 228 228 228 231 231 231 232 232 232 234 234 234 237 237 237 238 238 238 239 239 239 240 240 240 241 241 241 242 242 242 243 243 243 244 244 244 245 245 245 246 246 246 247 247 247 248 248 248 249 249 249 250 250 250 251 251 251 252 252 252 253 253 253 254 254 254 255 255 255 222 211 132 4 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 2 168 73 68 65 84 120 156 221 88 11 111 211 48 16 246 157 207 121 172 221 218 138 14 144 40 32 241 255 127 19 2 193 164 141 199 70 7 93 169 211 216 92 18 39 203 68 61 106 169 142 232 190 190 146 56 238 249 203 249 206 159 143 22 137 216 13 77 158 6 134 183 201 8 82 158 38 84 244 198 211 207 16 120 58 89 146 158 22 73 239 223 237 110 185 162 5 62 50 246 157 3 216 146 159 147 7 40 120 212 155 52 200 12 2 169 205 215 162 62 169 40 91 176 32 108 117 204 71 130 223 127 67 157 167 4 179 105 208 208 138 207 196 195 11 4 9 159 19 60 0 17 108 165 122 202 62 135 250 59 97 22 116 191 5 164 73 152 9 16 19 154 126 255 105 219 179 218 23 205 143 207 47 112 250 140 22 23 175 87 129 150 194 80 200 143 175 120 94 66 176 51 131 80 241 99 43 50 208 255 129 32 247 137 203 165 181 228 203 106 7 129 174 254 157 48 101 75 154 226 240 177 91 201 70 82 36 58 255 86 190 136 197 6 56 89 95 201 115 34 161 230 229 90 151 15 27 173 203 75 77 36 8 235 46 54 65 225 190 247 51 35 213 156 195 158 166 211 210 183 58 28 4 63 198 242 38 52 141 7 163 138 146 161 140 196 141 72 128 1 152 184 117 34 114 6 147 195 249 100 24 35 38 230 3 67 103 100 27 51 75 214 249 30 116 66 54 222 44 102 249 165 89 82 221 206 209 108 101 148 39 102 237 150 37 222 109 70 201 146 117 69 36 167 0 176 35 236 230 140 212 217 167 151 49 125 162 47 103 138 96 36 110 126 155 7 246 133 147 161 78 235 176 195 156 2 106 242 127 115 97 79 96 54 27 1 71 252 56 213 69 191 83 51 11 58 53 5 162 103 204 47 179 60 54 40 81 156 86 22 191 150 111 141 189 139 42 193 162 98 121 2 8 31 38 227 1 178 240 64 168 153 64 196 144 143 14 112 195 111 124 2 199 204 4 238 215 19 140 45 38 162 2 92 128 60 177 56 57 110 159 184 223 39 199 68 150 193 37 138 255 4 221 192 29 147 68 71 174 28 196 130 233 148 41 129 148 171 145 128 212 108 212 241 113 49 133 170 11 132 43 41 129 100 146 95 43 230 133 169 45 141 221 45 118 26 37 86 79 72 167 188 234 26 148 112 135 174 104 216 157 118 162 170 221 169 139 118 241 221 95 75 253 27 188 34 34 54 133 78 125 157 39 146 48 25 225 101 250 188 90 94 228 113 198 138 253 178 25 229 9 146 144 153 164 226 194 230 185 138 90 49 138 130 194 22 235 53 100 147 140 35 131 170 121 165 202 19 93 222 89 227 117 62 212 74 219 85 158 219 98 78 191 181 63 113 154 91 109 127 31 208 22 170 15 58 189 42 161 2 128 167 137 146 237 198 84 32 170 188 226 241 8 19 183 65 185 143 147 126 73 202 237 96 58 114 237 110 163 189 45 124 203 177 39 19 126 97 171 25 255 0 33 141 209 105 254 155 242 247 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:26'!
botPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 2 0 0 0 12 8 2 0 0 1 176 217 203 149 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 0 51 73 68 65 84 120 156 99 249 255 255 63 203 205 155 55 89 64 52 8 51 48 48 176 188 123 247 14 206 134 97 144 26 81 81 81 144 56 84 8 147 122 244 232 17 136 146 144 144 0 82 0 89 4 26 71 139 72 243 54 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:27'!
botRightPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 14 0 0 0 12 8 2 0 0 1 170 229 43 27 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 0 244 73 68 65 84 120 156 109 80 203 14 68 64 16 28 52 17 207 131 120 196 133 68 252 255 247 136 155 16 92 184 172 68 56 216 26 109 55 187 118 235 48 51 221 93 213 93 211 116 28 135 16 130 170 170 74 211 148 16 205 243 76 117 93 151 101 73 92 195 185 239 251 21 108 219 70 226 68 215 117 82 101 219 118 150 101 87 245 141 113 28 81 160 119 12 89 211 52 113 28 203 236 52 77 170 170 18 145 97 24 69 81 40 138 34 29 220 58 192 66 158 231 36 190 1 5 232 95 217 182 109 125 223 215 52 141 62 83 186 174 99 148 228 194 163 188 136 92 215 245 60 15 5 57 45 138 34 72 16 160 35 30 172 187 123 184 97 89 150 190 239 195 48 180 44 235 110 236 19 248 236 48 12 142 227 112 239 255 84 140 194 6 215 117 133 121 56 5 85 238 132 107 88 55 47 253 113 2 246 193 128 125 211 52 225 24 60 233 21 251 84 94 208 78 4 65 128 54 96 224 205 164 235 91 73 146 240 146 249 68 141 101 191 174 158 88 237 108 75 239 1 82 255 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:27'!
buttonBgPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 20 0 0 0 20 8 2 0 0 1 117 236 186 204 0 0 0 9 112 72 89 115 0 0 10 235 0 0 10 235 1 130 139 13 90 0 0 0 126 73 68 65 84 120 156 197 145 75 14 128 32 12 68 89 244 254 7 244 4 70 212 196 15 162 32 70 163 16 165 198 178 104 12 179 234 240 50 253 4 112 155 21 167 160 26 10 95 201 49 84 23 245 200 191 67 63 151 226 33 144 33 24 133 110 175 76 19 251 37 246 19 230 200 107 219 137 111 29 241 150 194 104 218 11 155 154 209 28 157 138 177 113 19 133 173 211 12 108 86 94 115 50 157 113 115 90 137 222 137 48 125 214 159 147 153 107 171 76 147 121 107 231 250 231 29 26 95 110 74 9 7 6 243 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css1Body
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css21Text
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css22Links
^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css23Inputs
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css24WebGrid
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css25AutocompleteField
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css26Popup
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css27Menus
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css3Header
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css40NavigationBar
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css41Login
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css41NavigationBar
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css42ErrorReport
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css43Buckets
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css44ContentHierarchy
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css451Document
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css452DocumentActions
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css453ClassificationLevel
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css46Search
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css47Comments
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css51Wiki
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css52SiteContent
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
css9Hide
	^''! !

!JoomlaStyle methodsFor: 'styles-screen' stamp: ' 10/1/08 16:19'!
cssJoomlaTemplate
	^'
/* Joomla mitra template */

html {
	height: 100%;
}

body {
	height: 100%;
	margin: 3px 3px 3px 3px;
}
.clr {
	clear: both;
}
table.main{
	width:758px;
}
.outline {
  border: 1px solid #cccccc;
  background: #ffffff;
	padding: 2px;
}

#buttons_outer {
	width: 635px;
  	margin-bottom: 2px;
	margin-right: 2px;
	float: left;

}

#buttons_inner {
	border: 1px solid #cccccc;
	height: 21px;
}

#pathway_text {
  overflow: hidden;
	display: block;
	height: 25px;
	line-height: 25px !!important;
	line-height: 22px;
	padding-left: 4px;
	border: 1px solid #008C00;
	margin-bottom: 2px;
}

#pathway_text img {
	margin-left: 5px;
	margin-right: 5px;
	margin-top: 6px;
}

#buttons {
	float: left;
	margin: 0px;
	padding: 0px;
	width: auto;
}


ul#mainlevel-nav
{
	list-style: none;
	padding: 0;
	margin: 0;
	font-size: 0.8em;
}

ul#mainlevel-nav li
{
	background-image: none;
	padding-left: 4px;
	padding-right: 0px;
	float: left;
	font-size: 11px;
	line-height: 22px;
	white-space: nowrap;
	background-image:url(/img/topLinkPng.png);
	background-repeat:no-repeat;
	background-position:right top;
	height:22px;
}

ul#mainlevel-nav li a
{
	display: block;
	padding-left: 15px;
	padding-right: 15px;
	text-decoration: none;
	color:#FFFFFF;
	background: transparent;
}

#buttons>ul#mainlevel-nav li a { width: auto; }

	ul#mainlevel-nav li a:hover
{
	color:#00FF66;
}


#search_outer {
	float: left;
	width: 165px;
}

#search_inner {
  border: 1px solid #cccccc;
	padding: 0px;
  height: 21px !!important;
  height: 23px;
  overflow: hidden;
}

#search_inner form {
  padding: 0;
  margin: 0;
}

#search_inner .inputbox {
	border: 0px;
	padding: 3px 3px 3px 5px;
	font-family: arial, helvetica, sans-serif;
	font-size: 11px;
	color: #008C00;
}

#header_outer {
	text-align: left;
	border: 0px;
	margin: 0px;
}

#header {
	float: left;
	padding: 0px;
	margin-right: 2px;
	width: 635px;
	height: 150px;
	background: url(/img/headerShortJpg.jpg) no-repeat;
}

#top_outer{
	float: left;
	width: 165px;
}

#top_inner {
  border: 1px solid #cccccc;
	padding: 2px;
  height: 144px !!important;
  height: 150px;
  overflow: hidden;
  float: none !!important;
  float: left;
}

#left_outer {
  float: left;
	margin-top: 2px;
	width: 165px;
}

#left_inner {
  border: 1px solid #cccccc;
	padding: 2px;
  float: none !!important;
  float: left;
}

#content_outer {
	padding: 0px;
	margin-top: 0px;
	margin-left: 2px;
	/** border: 1px solid #cccccc; **/
	float: left;
	width: 635px;
}

#content_inner{
  float: none !!important;
  float: left;
  padding: 0;
  padding-top: 2px;
  margin: 0;
}

table.content_table {
  width: 100%;
	padding: 0px;
	margin: 0px;
}

table.content_table td {
	padding: 0px;
	margin: 0px;
}


#banner_inner {
	float: left;
	padding: 0px;
	height: 70px;
}

#poweredby_inner {
	float: right;
	padding: 0px;
	margin-left: 0px;
	height: 70px;
}

#right_outer {
	margin-left: 2px;
	width: 165px;
}

#right_inner {
  float: none !!important;
  float: left;
  border: 1px solid #cccccc;
  padding: 2px;
}


.user1_inner {
	border: 1px solid #cccccc;
  float: none !!important;
  float: left;
	margin: 0px;
	padding: 2px;
}

.user2_inner {
	border: 1px solid #cccccc;
  float: none !!important;
  float: left;
	margin: 0px;
	padding: 2px;
}

table td.body_outer {
	padding: 2px;
	border: 1px solid #cccccc;
}

.maintitle {
	color: #ffffff;
	font-size: 40px;
	padding-left: 15px;
	padding-top: 20px;
}

.error {
  font-style: italic;
  text-transform: uppercase;
  padding: 5px;
  color: #cccccc;
  font-size: 14px;
  font-weight: bold;
}

/** old stuff **/

div.back_button {
	float: left;
  text-align: center;
  font-size: 11px;
  font-weight: bold;
  border: 1px double #cccccc;
  width: auto;
  background: url(/img/bacBtnPng.png) repeat-x;
  padding: 0px 10px;
  line-height: 20px;
  margin: 1px;
  color:#006600;
}

.pagenav {
  text-align: center;
  font-size: 11px;
  font-weight: bold;
  border: 1px double #cccccc;
  width: auto;
  padding: 0px 10px;
  line-height: 20px;
  margin: 1px;
  color:#006600;
}

.pagenavbar {
	margin-right: 10px;
	float: right;
}

#footer {
	text-align: center;
	padding: 3px;
}

ul
{
margin: 0;
padding: 0;
list-style: none;
}

li
{
line-height: 15px;
padding-left: 15px;
padding-top: 0px;
background-image: url(/img/arrowPng.png);
background-repeat: no-repeat;
background-position: 0px 2px;
}


td {
	text-align: left;
	font-size: 11px;
}



body {
	padding: 0px;
	font-family: Arial, Helvetica, Sans Serif;
	line-height: 120%;
	font-size: 11px;
	color:#666666;
}

/* Joomla core stuff */
a:link, a:visited {
	color: #008C00; text-decoration: none;
	font-weight: bold;
}

a:hover {
	color:#008C00;	text-decoration: none;
	font-weight: bold;
}

table.contentpaneopen {
  width: 100%;
	border-collapse: collapse;
	border-spacing: 0px;
	margin: 0px;
}

table.contentpaneopen td {
   padding-right: 5px;

}

table.contentpaneopen td.componentheading {
	padding-left: 4px;
	
}



table.contentpane {
  width: 100%;
	padding: 0px;
	border-collapse: collapse;
	border-spacing: 0px;
	margin: 0px;
}

table.contentpane td {
	margin: 0px;
	padding: 0px;
}

table.contentpane td.componentheading {
	padding-left: 4px;
}

table.contentpaneopen fieldset {
	border: 0px;
	border-bottom: 1px solid #eee;
}

.button {
  color:#006600;
  font-family: verdana, Hevlvetica, sans-serif;
  text-align: center;
  font-size: 11px;
  font-weight: bold;
  border: 1px double #cccccc;
  width: auto;
  background: url(/img/buttonBgPng.png) repeat-x;
  background-position:bottom;
  padding: 0px 5px;
  line-height: 18px !!important;
  line-height: 16px;
  height: 26px !!important;
  height: 24px;
  margin: 1px;
}

.inputbox {
	padding: 2px;
	border:solid 1px #cccccc;
	background-color: #ffffff;
}

.componentheading {
	color: #008C00;
	text-align: left;
	padding-top: 13px;
	padding-left:10px;
	height: 22px;
	font-size: 18px;
	font-family:"Times New Roman", Times, serif;
	background-image:url(/img/bacContentPng.png);
	background-repeat:no-repeat;
	background-position:bottom left;

}

.contentcolumn {
	padding-right: 5px;
}

.contentheading {
	text-align: left;
	padding-top: 13px;
	padding-left:10px;
	height: 25px;
	color: #008C00;
	font-weight: bold;
	font-size: 13px;
	background-image:url(/img/bacContentPng.png);
	background-repeat:no-repeat;
	background-position:bottom left;
	padding-bottom:4px !!important;
	padding-bottom:0px;
}



.contentpagetitle {
	font-size: 13px;
	font-weight: bold;
	color: #cccccc;
	text-align:left;
}

table.searchinto {
	width: 100%;
}

table.searchintro td {
	font-weight: bold;
}

table.moduletable {
	width: 100%;
	margin-bottom: 5px;
	padding: 0px;
	border-spacing: 0px;
	border-collapse: collapse;
}

div.moduletable {
	margin-bottom: 0px;
	background-image:url(/img/botModPng.png);
	background-position:bottom;
	background-repeat:no-repeat;
	padding-bottom:20px;
	padding-left:7px;
}

table.moduletable th, div.moduletable h3 {
	background: url(/img/topModPng.png) no-repeat;
	background-position:bottom;
	color: #FFFFFF;
	text-align: left;
	padding-left: 12px;
	height: 34px;
	line-height: 34px;
	font-weight: bold;
	font-size: 12px;
	font-family:Verdana, Arial, Helvetica, sans-serif;
	margin-top:0px;
	margin-left:-7px;
	margin-bottom:0px;
}
table.moduletable td {
	font-size: 11px;
	margin: 0px;
	font-weight: normal;
}

table.pollstableborder {
  padding: 2px;
  width:96%;
}
moduletable-Banners {
	text-align:center;
}
.sectiontableheader {
  font-weight: bold;
  background-color:#CCCCCC;
  padding: 4px;
}

.sectiontablefooter {

}

.sectiontableentry1 {
	background-color:#EFEFEF;
}

.sectiontableentry2 {
	background-color:#F4F4F4;
}

.small {
	color: #008C00;
	font-size: 11px;
}

.createdate {
	height: 15px;
	padding-bottom: 10px;
	color: #999999;
	font-size: 11px;
}

.modifydate {
	height: 15px;
	padding-top: 10px;
	color: #999999;
	font-size: 11px;
}

table.contenttoc {
  border: 1px solid #cccccc;
  padding: 2px;
  margin-left: 2px;
  margin-bottom: 2px;
}

table.contenttoc td {
  padding: 2px;
}

table.contenttoc th {
  color: #008C00;
	text-align: left;
	padding-top: 2px;
	padding-left: 4px;
	height: 21px;
	font-weight: bold;
	font-size: 10px;
	text-transform: uppercase;
}

a.mainlevel:link, a.mainlevel:visited {
	display: block;
	background: url(/img/menuBgPng.png) no-repeat;
	vertical-align: middle;
	font-size: 11px;
	font-weight: bold;
	color:#008C00;
	text-align: left;
	padding-top: 5px;
	padding-left: 15px;
	height: 21px !!important;
	height: 26px;
	width: 100%;
	text-decoration: none;
}

a.mainlevel:hover {
	background-position: 0px -32px;
	text-decoration: none;
	color:#999999;
	text-align:left !!important;
	padding-right:20px;
}

a.mainlevel#active_menu {
	color:#FF0000;
	font-weight: bold;
	font-size:13px;
}

a.mainlevel#active_menu:hover {
	color: #999999;
}

a.sublevel:link, a.sublevel:visited {
	padding-left: 1px;
	vertical-align: middle;
	font-size: 11px;
	font-weight: bold;
	color: #008C00;
	text-align: left;
}

a.sublevel:hover {
	color: #008C00;
	text-decoration: none;
}

a.sublevel#active_menu {
	color: #333;
}

.highlight {
	background-color: Yellow;
	color: Blue;
	padding: 0;
}
.code {
	background-color: #ddd;
	border: 1px solid #bbb;
}

form {
/* removes space below form elements */
	margin: 0;
 	padding: 0;
	width:98%;
}

div.mosimage {
  border: 1px solid #cccccc;
}

.mosimage {
  border: 1px solid #cccccc;
  margin: 5px
}

.mosimage_caption {
  margin-top: 2px;
  background: #efefef;
  padding: 1px 2px;
  color: #666;
  font-size: 10px;
  border-top: 1px solid #cccccc;
}

span.article_seperator {
	display: block;
	height: 1.5em;
}
td.dot {
	height:5px;
}
table.top_menu{
	padding-top:4px;
}
td.tab{
	height:10px;
}
a.mostread, a.mostread:visited {
	color:#9A927D;
	font-weight:normal;
}
a.mostread:hover {
	color:#008C00;
	font-weight:normal;
}
a.latestnews, a.latestnews:visited {
	color:#9A927D;
	font-weight:normal;
}
a.latestnews:hover {
	color:#008C00;
	font-weight:normal;
}
li.latestnews-user{
	padding-left:20px !!important;
	padding-left:10px;
}
li.mostread-user{
	padding-left:20px !!important;
	padding-left:10px;
}
td.site_name {
	font-size:44px;
	padding-top:16px;
	padding-bottom:10px;
	font-weight:bold;
	color:#008C00;
}
td.site_desc {
	font-size:15px;
	font-weight:bold;
	color:#008C00;
}
table.box_table{
	background-color:#F7F7F7;
}
a.readon {
	display:block;
	border: 1px solid #cccccc;
	width:100px;
	color:#99CC33;
	padding-left:15px;
	background-color:#F4F4F4;
	padding-top:3px;
}
td.bac_module {
	background-image:url(/img/bacModPng.png);
}
td.bac_module2 {
	background-image:url(/img/bacModPng.png);
}
td.ver {
	text-align: center;
	font-size:18px;
	font-weight:bold;
	color:#FFFFFF;
	height:30px;
}
td.logo{
	background-image: url(/img/leftLogoJpg.jpg);
	background-repeat:no-repeat;
	background-position:left;
}
table.logo2 {
	background-image: url(/img/bacLogoJpg.jpg);
	height:219px;
}
td.logo3 {
	background-image: url(/img/logoJpg.jpg);
	background-repeat:no-repeat;
	background-position:right;
}
'! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
cssTabs1
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
cssTabs2
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
cssTabs3
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
cssTabs4
	^''! !

!JoomlaStyle methodsFor: 'styles-screen-no' stamp: ' 10/1/08 16:19'!
cssTabs5
	^''! !

!JoomlaStyle methodsFor: 'maintenance' stamp: ' 10/1/08 16:19'!
importImages
	SiskaStyle new importImagesFromDirectory: '/home/mivsek/projekti/siska/predloga-mitra/images'! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:28'!
indent1Png
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 13 2 3 0 0 0 221 213 6 183 0 0 0 3 115 66 73 84 8 8 8 219 225 79 224 0 0 0 9 80 76 84 69 255 255 255 153 153 153 102 102 102 131 74 186 71 0 0 0 9 112 72 89 115 0 0 10 240 0 0 10 240 1 66 172 52 152 0 0 0 22 116 69 88 116 67 114 101 97 116 105 111 110 32 84 105 109 101 0 48 55 47 50 54 47 48 52 131 188 17 75 0 0 0 37 116 69 88 116 83 111 102 116 119 97 114 101 0 77 97 99 114 111 109 101 100 105 97 32 70 105 114 101 119 111 114 107 115 32 77 88 32 50 48 48 52 135 118 172 207 0 0 0 28 73 68 65 84 120 156 99 96 192 15 24 151 0 9 182 72 32 193 25 6 35 192 92 176 4 16 0 0 54 118 2 199 200 145 236 116 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:28'!
indent2Png
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 21 0 0 0 13 4 3 0 0 0 180 188 56 83 0 0 0 3 115 66 73 84 8 8 8 219 225 79 224 0 0 0 27 80 76 84 69 255 255 255 198 198 198 197 197 197 129 129 129 128 128 128 124 124 124 110 110 110 107 107 107 106 106 106 39 121 4 106 0 0 0 9 112 72 89 115 0 0 10 240 0 0 10 240 1 66 172 52 152 0 0 0 22 116 69 88 116 67 114 101 97 116 105 111 110 32 84 105 109 101 0 48 55 47 50 54 47 48 52 131 188 17 75 0 0 0 37 116 69 88 116 83 111 102 116 119 97 114 101 0 77 97 99 114 111 109 101 100 105 97 32 70 105 114 101 119 111 114 107 115 32 77 88 32 50 48 48 52 135 118 172 207 0 0 0 29 73 68 65 84 120 156 99 96 160 42 96 74 19 128 179 57 140 11 16 108 83 4 155 17 73 13 94 0 0 131 203 2 72 7 76 171 50 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:28'!
indent3Png
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 26 0 0 0 13 4 3 0 0 0 69 183 99 222 0 0 0 3 115 66 73 84 8 8 8 219 225 79 224 0 0 0 27 80 76 84 69 255 255 255 198 198 198 197 197 197 129 129 129 128 128 128 124 124 124 110 110 110 107 107 107 106 106 106 39 121 4 106 0 0 0 9 112 72 89 115 0 0 10 240 0 0 10 240 1 66 172 52 152 0 0 0 22 116 69 88 116 67 114 101 97 116 105 111 110 32 84 105 109 101 0 48 55 47 50 54 47 48 52 131 188 17 75 0 0 0 37 116 69 88 116 83 111 102 116 119 97 114 101 0 77 97 99 114 111 109 101 100 105 97 32 70 105 114 101 119 111 114 107 115 32 77 88 32 50 48 48 52 135 118 172 207 0 0 0 27 73 68 65 84 120 156 99 96 160 37 80 75 68 230 53 155 163 240 194 145 121 98 40 42 9 1 0 172 12 2 147 242 226 65 233 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:29'!
indent4Png
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 31 0 0 0 13 4 3 0 0 0 163 158 168 154 0 0 0 3 115 66 73 84 8 8 8 219 225 79 224 0 0 0 27 80 76 84 69 255 255 255 198 198 198 197 197 197 129 129 129 128 128 128 124 124 124 110 110 110 107 107 107 106 106 106 39 121 4 106 0 0 0 9 112 72 89 115 0 0 10 240 0 0 10 240 1 66 172 52 152 0 0 0 22 116 69 88 116 67 114 101 97 116 105 111 110 32 84 105 109 101 0 48 55 47 50 54 47 48 52 131 188 17 75 0 0 0 37 116 69 88 116 83 111 102 116 119 97 114 101 0 77 97 99 114 111 109 101 100 105 97 32 70 105 114 101 119 111 114 107 115 32 77 88 32 50 48 48 52 135 118 172 207 0 0 0 30 73 68 65 84 120 156 99 96 160 55 96 74 19 64 21 224 48 46 64 19 48 69 19 96 68 215 66 36 0 0 183 76 2 72 234 160 51 137 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'elements-joomla' stamp: ' 10/1/08 16:19'!
joomlaCenterWith: aWebElement
	| e |
	e := WebElement new.
	e table width: 1; cellPadding: 0; cellSpacing: 0.
	e cell width: 5; valign: #top; style: 'background-image: url(/img/leftPng.png); height: 664px'.
	e cell addPng: #topLeftPng.
	e newCell valign: #top; style: 'background-image: url(/img/topPng.png); background-repeat: repeat-x; background-position: top; height: 664px'.
	e cell add: aWebElement.
	e newCell width: 4; valign: #top; style: 'background-image: url(/img/rightPng.png); height: 664px'.
	e cell addPng: #topRightPng. 
	e newRow.
	e cell width: 5; addPng: #botLeftPng.
	e newCell style: 'background-image: url(/img/botPng.png)'; addPng: #botPng.
	e newCell width: 4; addPng: #botRightPng. 
	^e! !

!JoomlaStyle methodsFor: 'elements-joomla' stamp: ' 10/1/08 16:19'!
joomlaFooterElement
	| e |
	e := WebElement new.
	e table width: 0.99; cellPadding: 0; cellSpacing: 0; 
		style: 'background-image: url(/img/bacFooterPng.png)'.
	e cell width: 12; addPng: #leftFooterPng.
	e newCell add: 
		(WebElement new align: #center; addTextSmall: '(c)  2007 Z^upnija Sv.Franc^is^ka Asis^kega'). 
	e newCell width: 12; addPng: #rightFooterPng. 
	^e! !

!JoomlaStyle methodsFor: 'elements-joomla' stamp: ' 10/1/08 16:19'!
joomlaHeaderElement
	| e |
	e := WebElement new.
	e table class: #logo2; width: 1; cellPadding: 0; cellSpacing: 0.
	e cell class: #logo; width: 133; valign: #bottom.
	e cell table width: 130; cellPadding: 0; cellSpacing: 0.
	e cell newRow; newRow.
	e newCell valign: #top.
	e newCell class: #logo3; width: 0.7; align: #right.
	e cell table width: 705; style: 'height:: 217px'; cellPadding: 0; cellSpacing: 0.
	e cell cell table width: 1; cellPadding: 0; cellSpacing: 0.
	e cell cell cell style: 'height: 25px'; addNbSp.
	e cell cell newCell width: 197; style: 'height: 25px'; addNbSp.
	e cell cell newCell width: 8; style: 'height: 25px'; addNbSp.
	e cell cell newRow.
	e cell cell cell style: 'height: 25px'; addNbSp.
	e cell cell newCell width: 197; style: 'height: 25px'; addNbSp. "here add 'user4' !!" 
	e cell cell newCell width: 8; style: 'height: 25px'; addNbSp.
	e cell newCell valign: #top.
	e cell cell table width: 1; style: 'height: 163px'; cellPadding: 0; cellSpacing: 0.
	e cell cell cell width: 219; addNbSp.
	e cell cell newCell addNbSp.
	e cell cell newRow.
	e cell cell cell width: 219; addNbSp.
	e cell cell newCell width: 486; style: 'height: 66px'; addNbSp. "here add 'banner'"
	^e! !

!JoomlaStyle methodsFor: 'elements-joomla' stamp: ' 10/1/08 16:19'!
joomlaLeftElement
	| e |
	e := WebElement new.
	e table width: 1; cellSpacing: 0; cellPadding: 0.
	e cell class: #'bac_module'.
	e cell add: self menuElement.
	e newTable width: 1; cellSpacing: 0; cellPadding: 0.
	e cell class: #'bac_module'.
	e cell add: self leftBottomElement.
	^e! !

!JoomlaStyle methodsFor: 'elements-joomla' stamp: ' 10/1/08 16:19'!
joomlaRightElement
	| e |
	e := WebElement new.
	e table width: 1; cellSpacing: 0; cellPadding: 0.
	e cell class: #'bac_module'.
	e cell add: self rightTopElement.
	^e! !

!JoomlaStyle methodsFor: 'elements-joomla' stamp: ' 10/1/08 16:19'!
joomlaToolbarElement
	| e |
	e := WebElement new.
	e table width: 1; cellPadding: 0; cellSpacing: 0; 
		style: 'background-image: url(/img/topLinkBacPng.png)'.
	e cell width: 133; addJpeg: #leftTopLinkJpg.
	e newCell addNbSp.  "here add 'user3' "
	e newCell width: 11; addPng: #rightTopLinkPng. 
	^e! !

!JoomlaStyle methodsFor: 'elements-general' stamp: ' 10/1/08 16:19'!
leftBottomElement
	"just for example, override in your own style subclass!! "
 	| e |
	e := JoomlaModule new.
	^e! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'np 4/3/2008 00:01'!
leftFooterPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 12 0 0 0 26 8 2 0 0 1 123 159 26 160 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 1 72 73 68 65 84 120 156 181 82 201 106 133 80 12 117 184 136 232 194 157 40 184 82 255 65 255 255 75 92 9 186 113 22 103 123 242 34 87 91 94 105 161 52 32 102 60 57 73 174 56 207 243 56 14 129 79 211 52 177 109 219 178 44 66 81 20 1 65 76 144 105 24 198 203 5 187 105 154 105 154 200 235 56 14 85 40 47 209 117 157 162 208 214 117 37 176 44 203 130 32 16 101 89 134 97 216 117 157 64 161 231 121 212 234 174 96 237 70 33 141 81 144 135 54 195 48 92 108 198 113 204 243 60 138 34 203 178 136 9 186 192 142 227 24 6 82 136 18 10 247 125 103 160 121 158 85 85 189 80 37 54 21 190 113 113 71 8 35 208 208 108 163 9 24 0 155 102 98 212 186 174 251 190 199 32 182 109 11 36 183 109 139 32 120 113 5 237 5 46 182 1 4 157 92 200 231 56 22 65 188 80 136 237 201 166 223 240 122 186 80 245 87 23 95 248 118 201 177 37 15 100 236 15 185 111 197 213 24 148 87 141 61 64 225 85 93 219 194 15 79 7 49 28 10 67 154 166 233 186 46 158 141 172 167 118 0 64 184 170 170 162 40 146 36 97 30 178 59 86 77 139 1 50 158 10 206 156 166 233 147 31 194 232 112 35 33 207 247 253 183 25 87 18 79 244 36 193 115 200 190 159 22 245 69 126 78 122 210 255 143 36 121 47 150 167 254 43 164 15 27 72 43 211 254 118 143 210 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:29'!
leftLogo1Jpg
^#(255 216 255 224 0 16 74 70 73 70 0 1 1 1 0 72 0 72 0 0 255 219 0 67 0 3 2 2 3 2 2 3 3 3 3 4 3 3 4 5 8 5 5 4 4 5 10 7 7 6 8 12 10 12 12 11 10 11 11 13 14 18 16 13 14 17 14 11 11 16 22 16 17 19 20 21 21 21 12 15 23 24 22 20 24 18 20 21 20 255 219 0 67 1 3 4 4 5 4 5 9 5 5 9 20 13 11 13 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 255 194 0 17 8 0 219 0 133 3 1 34 0 2 17 1 3 17 1 255 196 0 28 0 0 2 2 3 1 1 0 0 0 0 0 0 0 0 0 0 0 6 1 4 2 3 5 7 8 255 196 0 27 1 0 1 5 1 1 0 0 0 0 0 0 0 0 0 0 0 0 1 3 4 5 6 2 7 255 218 0 12 3 1 0 2 16 3 16 0 0 1 250 167 86 197 246 142 216 154 85 163 144 154 3 144 154 3 144 154 3 144 154 3 144 154 3 144 154 3 144 152 32 240 178 204 181 48 90 36 194 115 4 129 4 129 4 129 4 129 4 129 17 34 44 0 137 233 107 76 171 91 222 150 192 194 114 0 0 0 0 0 0 0 4 76 33 137 34 30 150 178 204 179 189 233 110 98 112 156 192 1 36 72 0 1 18 0 0 64 66 0 8 122 90 203 50 206 247 165 176 48 124 128 0 0 0 0 0 0 1 140 194 0 8 122 98 203 50 206 251 165 162 12 23 50 64 132 144 41 36 8 19 0 19 140 128 16 1 2 47 167 44 51 172 111 149 100 131 3 204 144 4 144 4 144 4 144 4 144 4 226 66 18 64 135 168 43 180 43 122 7 75 17 137 231 188 230 96 6 81 83 165 44 175 151 109 113 245 177 58 230 181 50 49 3 40 128 3 17 23 213 85 90 149 125 9 85 53 214 100 200 46 141 254 101 170 116 255 0 88 76 93 185 18 108 122 58 63 66 173 230 186 180 250 238 85 243 121 207 138 242 34 80 49 43 153 147 17 15 88 83 107 70 244 158 184 20 219 148 105 102 168 221 104 179 10 122 213 222 237 216 103 19 161 208 219 87 206 187 240 67 102 206 84 246 88 54 155 179 180 186 229 109 131 18 10 122 15 13 133 23 210 230 40 216 177 111 33 111 94 229 139 176 23 77 237 183 106 90 215 123 60 170 219 198 166 85 31 93 116 246 83 183 119 161 189 89 183 83 13 72 12 149 95 167 46 49 174 122 100 181 123 188 22 124 5 198 203 155 109 199 143 174 230 113 92 220 84 202 172 133 194 174 117 45 93 215 206 174 177 34 85 223 69 243 103 233 149 220 18 12 173 79 165 43 53 46 111 231 172 116 167 165 227 86 251 122 213 53 68 143 209 173 199 231 222 175 123 159 193 231 220 189 215 224 104 165 58 73 78 41 74 235 63 105 240 239 117 185 169 81 53 24 106 95 101 231 118 53 251 39 107 245 122 220 143 28 179 213 207 207 157 148 157 143 63 46 125 251 248 208 202 141 227 184 211 202 156 243 28 232 251 61 180 94 207 33 158 253 230 121 20 122 42 219 255 196 0 36 16 0 2 2 2 2 3 1 1 0 3 1 0 0 0 0 0 0 2 3 19 1 4 5 18 16 17 21 6 20 34 35 50 33 255 218 0 8 1 1 0 1 5 2 45 92 23 41 114 151 41 114 151 41 114 151 41 114 151 41 114 151 41 114 151 41 114 151 47 141 153 191 158 15 166 199 211 99 233 177 244 216 250 108 125 54 62 155 31 77 143 166 199 211 99 233 177 244 216 250 140 125 70 62 163 120 223 207 173 91 11 11 11 11 11 11 11 11 11 11 11 11 11 14 254 55 243 235 86 194 194 194 194 194 194 194 194 194 194 194 194 194 194 207 27 249 245 171 97 97 97 97 97 97 97 97 97 97 97 97 97 97 103 141 252 250 213 176 176 176 176 176 176 176 176 176 176 176 176 176 176 239 227 127 62 181 108 44 44 44 44 44 44 44 44 44 44 44 44 44 44 241 191 159 90 182 22 22 22 22 22 22 22 22 22 22 22 22 22 22 120 223 207 173 91 11 11 11 11 11 11 11 11 11 11 11 11 11 11 60 111 231 214 173 133 133 133 133 133 133 133 133 133 133 133 133 133 133 158 55 243 235 86 194 194 194 194 194 194 194 194 194 194 194 194 194 194 207 28 134 125 106 88 88 88 88 88 88 88 88 88 88 88 88 88 88 89 227 144 207 173 75 11 11 11 11 11 11 11 11 11 11 11 11 11 14 254 57 12 250 212 176 176 176 176 176 176 176 176 176 176 176 176 176 238 89 227 145 207 173 78 231 115 185 220 238 89 232 196 216 201 220 238 119 59 157 206 231 115 191 142 75 62 181 59 157 206 231 114 93 197 140 212 226 118 119 199 225 53 166 73 115 54 147 226 95 103 115 185 220 238 119 59 248 228 179 235 79 185 172 155 91 249 199 9 200 49 143 206 236 177 78 143 2 188 151 61 179 200 154 175 38 180 154 124 218 109 71 47 231 251 103 60 30 254 12 241 92 146 145 205 150 199 115 185 223 199 39 159 90 123 51 244 143 106 108 240 31 158 207 49 200 72 95 60 134 19 216 177 137 16 145 17 118 66 57 102 193 171 59 217 203 195 252 156 135 115 177 219 199 41 159 90 124 116 95 67 147 253 102 231 244 239 164 98 198 44 98 70 34 17 167 177 35 35 67 63 226 114 48 255 0 127 27 28 189 215 177 216 108 225 113 182 178 242 171 177 181 173 249 221 109 25 219 249 106 138 69 153 117 48 242 172 8 77 86 34 133 22 88 164 107 19 61 113 35 103 10 143 152 251 233 238 81 158 75 135 204 100 115 226 76 118 39 195 236 236 114 124 180 124 106 103 180 140 177 139 24 145 137 24 145 136 130 161 156 250 25 134 97 152 213 228 91 81 185 78 58 61 141 116 127 107 55 250 247 63 65 173 211 144 88 197 65 35 18 49 99 21 5 67 57 25 134 97 178 54 73 230 196 75 178 217 214 252 238 51 235 28 130 255 0 167 244 95 251 171 30 48 248 88 197 65 80 84 21 76 228 102 27 35 100 108 147 206 177 97 114 219 155 95 162 126 167 179 145 126 186 255 0 160 205 60 102 190 90 44 193 34 200 42 10 134 23 208 217 27 35 100 108 141 147 99 125 84 150 92 190 120 189 140 67 201 254 133 114 155 254 205 191 246 108 254 137 251 237 36 98 70 67 150 81 100 199 172 204 163 76 163 78 132 155 145 169 47 34 79 178 242 140 195 49 153 61 103 150 111 236 225 125 159 243 191 203 105 73 141 132 140 72 197 94 163 184 238 59 142 227 184 238 51 12 195 57 28 114 108 203 252 255 0 55 243 9 255 0 18 196 178 174 29 181 201 248 252 41 140 117 195 184 238 59 142 227 184 238 51 12 227 57 167 167 55 37 177 198 241 16 254 127 95 99 142 228 119 165 248 219 158 36 92 50 232 103 252 166 255 0 183 31 35 143 145 198 24 97 143 194 162 227 142 215 92 74 213 169 90 159 255 196 0 38 17 0 2 2 1 3 3 3 5 0 0 0 0 0 0 0 0 0 1 2 3 17 4 16 19 18 33 49 32 81 97 34 35 51 65 80 255 218 0 8 1 3 1 1 63 1 51 190 127 191 140 122 163 23 39 132 45 27 253 178 173 52 43 239 229 150 215 11 23 212 61 39 179 44 173 214 241 232 162 46 63 115 7 55 192 175 207 132 74 236 63 7 55 193 53 203 30 203 122 34 165 60 61 178 57 25 16 141 84 18 93 91 81 249 81 40 184 142 70 68 34 21 251 154 255 0 49 91 105 113 201 220 108 113 76 85 33 82 136 194 49 219 93 142 149 189 22 57 246 98 66 91 223 103 20 58 137 73 205 230 70 15 255 196 0 51 17 0 1 4 1 0 8 2 7 9 0 0 0 0 0 0 0 1 0 2 3 4 17 5 18 21 33 49 52 65 97 48 113 6 16 20 50 209 240 241 19 22 34 81 82 129 145 161 193 255 218 0 8 1 2 1 1 63 1 146 65 19 11 221 209 109 90 253 214 213 175 221 109 90 253 214 213 175 221 109 90 253 214 213 175 221 109 90 253 214 214 175 221 93 229 223 224 221 229 223 224 221 229 223 224 221 229 223 224 222 229 159 224 222 229 159 224 223 229 158 178 178 160 175 37 131 134 5 61 25 96 26 220 71 171 43 43 42 255 0 42 255 0 37 66 187 102 46 124 158 232 69 212 153 194 60 254 234 197 231 57 191 103 24 213 106 142 252 181 15 224 59 191 46 137 218 98 137 223 36 31 193 65 213 174 214 54 42 130 53 78 8 43 43 42 227 12 144 57 141 226 83 98 108 17 123 54 119 241 63 4 250 135 25 214 10 106 122 141 113 47 27 151 176 137 152 28 217 6 255 0 147 243 213 59 66 185 224 145 40 250 45 27 0 209 115 24 229 120 44 147 113 199 78 191 234 181 85 245 157 135 112 89 87 36 124 113 101 137 229 72 229 43 147 178 226 162 174 162 174 170 157 104 95 20 187 218 6 124 150 86 144 33 181 94 74 22 216 253 221 84 142 68 23 40 224 81 192 181 131 119 53 12 182 140 174 28 78 22 86 159 50 10 71 80 121 249 41 167 76 187 56 56 105 220 161 185 39 86 133 29 137 15 68 11 221 239 20 198 40 35 204 50 180 240 199 171 138 244 171 69 69 78 70 77 6 224 252 238 248 40 98 10 24 130 141 128 38 53 85 128 74 240 210 172 80 142 203 4 101 196 55 183 209 125 223 171 250 157 253 124 23 255 196 0 59 16 0 0 5 2 2 6 8 4 5 3 5 0 0 0 0 0 0 1 50 162 226 2 3 17 49 4 16 18 33 65 81 19 34 35 113 146 161 210 225 48 82 97 129 20 51 66 145 193 32 114 177 36 67 98 131 209 255 218 0 8 1 1 0 6 63 2 25 140 198 99 49 152 204 102 51 25 140 198 99 49 152 204 102 51 212 117 145 80 103 183 135 92 190 157 225 54 60 37 234 9 177 225 47 80 77 143 9 122 130 108 120 75 212 19 99 194 94 160 155 30 18 245 4 216 240 151 168 38 199 132 189 65 54 60 37 234 9 177 225 47 80 77 143 9 122 130 108 120 75 212 19 99 194 94 160 155 30 18 245 4 216 240 151 171 86 120 118 156 240 225 222 65 111 152 91 230 22 249 133 190 97 111 152 91 230 22 249 133 190 97 111 152 91 230 22 249 133 190 97 111 152 91 230 22 233 106 207 14 211 158 28 59 200 45 243 11 124 194 223 48 183 204 45 243 11 124 194 223 48 183 204 45 243 11 124 194 223 48 183 204 45 243 11 124 194 221 61 89 225 218 115 195 135 121 5 190 97 111 152 91 230 22 249 133 190 97 111 152 91 230 22 249 133 190 97 111 152 91 230 22 249 133 190 97 111 152 91 231 171 60 59 78 120 112 239 32 183 204 45 243 11 124 194 223 48 183 204 45 243 11 124 194 223 48 183 204 45 243 11 124 194 223 48 183 204 45 243 11 124 245 103 135 105 207 14 29 228 22 249 133 190 97 111 152 91 230 22 249 133 190 97 111 152 91 230 22 249 133 190 97 111 152 91 230 22 249 133 190 97 110 150 172 240 237 57 225 195 188 130 223 48 183 204 45 243 11 124 194 223 48 183 204 45 243 11 124 194 223 48 183 204 45 243 11 124 194 223 48 183 204 45 210 213 158 29 167 60 56 119 144 91 230 22 249 133 190 97 111 152 91 230 22 249 133 190 97 111 152 91 230 22 249 133 190 97 111 152 91 230 22 233 133 186 90 179 195 180 231 135 14 242 11 124 194 223 48 183 204 45 243 11 124 194 223 48 183 204 45 243 11 124 194 223 48 183 204 45 243 11 124 194 221 32 183 75 86 120 118 156 254 157 228 22 249 133 190 97 110 152 91 230 22 249 133 190 97 111 152 91 230 22 233 133 190 97 111 152 91 230 22 233 133 186 65 110 150 172 240 237 57 253 59 200 45 210 11 116 130 221 32 183 72 45 210 11 116 130 223 32 183 72 45 210 11 116 130 221 32 183 72 45 210 11 116 130 221 45 89 225 218 243 250 119 144 91 164 22 233 5 186 65 110 144 91 164 22 233 5 186 65 110 144 91 164 22 233 5 186 65 110 144 91 164 22 233 5 186 90 179 195 181 231 244 239 11 116 130 221 32 183 72 45 210 11 116 130 221 32 183 72 45 210 11 116 130 221 32 183 72 45 210 11 116 130 221 32 167 75 86 120 118 188 254 157 225 78 144 83 164 20 233 5 58 65 78 144 91 164 55 86 233 5 58 65 78 144 83 164 20 233 5 58 65 78 144 83 164 20 233 106 207 14 215 248 10 116 130 157 32 167 72 41 210 10 196 255 0 186 67 110 245 85 88 179 247 196 252 199 250 45 35 11 148 231 218 29 69 247 222 54 52 154 78 142 85 99 184 220 22 233 5 58 65 78 144 83 164 20 233 5 58 65 78 150 174 93 175 240 20 233 10 255 0 15 78 52 210 120 25 157 94 227 125 219 116 253 234 29 109 48 139 184 143 255 0 71 73 126 231 77 127 134 59 207 236 64 233 46 198 207 203 79 30 240 85 218 168 232 171 153 14 135 77 160 183 254 174 3 107 68 210 118 104 62 21 98 127 200 221 126 221 95 122 134 84 213 221 95 184 222 120 84 91 140 177 144 83 164 20 233 5 121 203 87 253 191 192 62 183 159 184 45 147 195 72 175 143 252 140 117 180 187 191 99 192 117 175 92 171 190 175 233 221 81 151 113 143 204 171 247 5 183 86 36 54 203 117 23 183 231 199 247 10 243 247 10 243 247 25 249 251 234 229 218 255 0 2 221 56 227 110 223 90 173 254 224 172 145 245 45 110 251 252 35 58 127 50 142 177 2 61 175 63 113 159 159 184 207 207 220 98 99 160 177 186 158 147 26 171 229 184 116 54 11 111 72 62 127 228 198 148 116 95 166 198 151 93 194 171 110 186 182 113 167 232 98 138 238 221 179 93 221 139 189 41 157 226 196 207 244 247 139 133 77 22 182 58 98 160 142 147 51 61 140 55 158 98 249 209 69 154 182 75 179 44 115 222 92 171 62 29 194 171 86 186 61 154 110 226 93 109 248 96 95 95 176 232 110 214 84 21 53 109 82 103 203 245 16 174 231 251 181 25 245 79 129 97 187 245 16 168 136 169 217 218 34 42 182 183 97 226 29 90 169 174 189 146 85 120 23 31 168 164 169 50 58 79 111 26 140 247 151 33 133 105 48 122 70 135 214 164 247 157 178 63 240 51 223 223 238 51 243 247 29 22 214 205 56 98 99 240 218 41 118 188 79 229 7 85 71 181 81 230 103 241 62 107 124 133 90 118 141 86 197 69 78 209 242 168 17 227 231 238 45 85 207 112 219 225 89 99 241 113 20 82 106 172 136 191 113 159 159 184 218 44 233 222 44 105 24 98 88 224 127 113 137 111 47 135 137 139 116 124 213 17 13 26 193 127 118 163 250 139 86 120 153 144 220 57 31 193 194 142 177 243 24 153 226 52 106 170 203 108 133 154 248 85 70 26 172 219 250 226 45 209 242 211 143 244 111 25 133 16 81 12 241 29 90 127 113 188 245 226 52 93 40 184 108 213 251 234 218 172 240 221 129 10 175 42 138 188 190 45 54 173 82 117 220 171 34 33 248 109 38 178 233 54 12 190 249 145 2 24 24 216 187 214 183 243 13 187 91 233 229 240 202 205 138 118 170 63 33 141 20 254 35 75 168 179 231 236 58 91 214 247 240 167 104 176 33 249 46 45 71 136 187 79 233 44 136 85 240 175 215 135 92 238 97 143 216 85 93 125 106 185 140 134 67 255 196 0 40 16 0 1 3 3 2 5 5 1 1 0 0 0 0 0 0 0 1 0 17 33 49 65 97 81 113 16 32 64 129 161 145 177 193 225 240 48 209 255 218 0 8 1 1 0 1 63 33 68 166 41 134 176 214 26 195 88 107 13 97 172 53 134 176 214 26 195 88 107 13 97 163 66 158 4 206 3 27 131 160 199 143 30 60 120 241 227 199 143 30 52 104 208 154 20 238 119 125 0 0 0 0 1 0 1 0 44 208 167 115 187 252 14 253 2 34 34 34 34 0 32 1 26 20 238 119 127 129 223 160 204 204 204 204 196 68 0 3 66 157 206 239 240 59 244 24 153 153 153 153 153 136 144 104 83 185 221 244 0 192 196 0 0 4 0 0 51 66 159 206 239 160 2 4 36 0 68 64 68 65 154 20 254 119 127 212 25 25 16 24 0 9 135 138 17 154 20 254 119 125 0 48 2 34 34 34 34 0 204 208 167 243 187 135 64 0 8 0 2 40 32 8 204 208 167 99 185 135 64 12 204 204 196 204 204 204 204 208 167 99 224 58 0 102 102 102 102 102 102 103 102 133 63 31 1 118 29 1 153 153 153 153 153 153 219 154 21 47 224 46 195 155 255 0 255 0 203 78 83 184 191 199 255 0 255 0 253 204 52 42 95 212 53 217 28 185 153 128 123 64 16 207 84 216 150 94 196 77 106 176 209 240 33 194 45 71 96 160 133 195 185 204 204 204 131 66 175 93 198 187 33 22 132 25 226 56 161 215 222 70 249 94 74 122 135 97 170 57 220 141 209 18 201 47 78 226 2 27 113 59 142 136 203 113 100 75 179 188 30 196 39 182 129 242 169 59 243 229 12 1 140 113 73 228 0 17 161 87 6 220 215 100 40 52 140 5 233 65 234 34 66 103 64 191 42 91 35 229 194 81 201 204 156 240 240 44 73 176 2 246 56 38 102 16 241 40 238 4 45 0 47 179 144 239 205 10 159 126 227 93 144 164 214 178 82 151 93 6 29 97 169 215 129 137 99 82 33 187 66 13 19 195 1 163 20 64 33 109 41 53 21 240 134 51 139 255 0 57 35 4 232 26 120 160 36 34 87 46 114 250 159 8 2 27 64 235 129 51 41 165 211 136 102 99 54 53 16 135 65 211 137 17 220 70 180 164 41 51 71 0 136 215 6 170 246 32 2 58 176 231 142 226 233 125 131 186 106 2 27 105 123 181 25 67 50 4 17 70 152 1 240 188 161 28 240 24 133 206 8 61 109 165 16 195 0 44 74 238 111 68 60 39 245 103 60 3 61 117 18 131 186 56 41 217 153 76 61 229 225 68 26 21 28 4 109 89 184 187 97 17 140 205 143 244 80 18 196 189 65 254 4 3 140 24 114 116 212 50 87 78 201 236 128 57 0 21 239 9 204 168 35 31 159 186 120 0 193 247 83 148 67 23 13 164 223 45 121 137 86 195 85 3 29 250 110 124 58 104 8 24 222 12 76 4 199 176 4 34 87 57 111 217 9 190 81 87 155 54 26 163 146 72 145 221 5 209 128 114 24 16 62 86 255 0 63 105 132 43 4 215 209 61 2 124 83 112 128 15 72 241 225 85 202 170 48 9 37 130 121 246 0 69 174 138 151 80 23 204 35 64 152 205 193 251 91 188 253 172 104 189 176 148 213 209 230 231 141 50 7 113 161 66 56 25 99 247 95 102 67 255 0 186 248 104 38 223 214 74 166 70 150 227 230 16 22 32 184 67 153 151 7 0 199 203 32 241 95 63 104 176 152 38 154 58 61 104 36 120 158 52 0 115 252 159 211 227 177 70 171 42 212 227 49 169 146 6 209 232 140 201 86 71 67 116 118 191 0 165 183 87 129 150 91 100 30 34 215 225 213 85 57 58 45 154 237 3 82 108 17 141 72 11 2 90 7 162 74 93 147 110 130 87 225 254 252 0 64 225 18 92 163 2 128 5 28 163 40 200 202 52 162 40 204 163 70 81 148 19 131 130 188 128 13 238 130 211 166 165 142 177 215 255 218 0 12 3 1 0 2 0 3 0 0 0 16 184 195 12 32 195 0 63 176 253 228 28 121 104 26 247 207 191 255 0 254 60 127 167 210 195 76 59 241 245 29 243 207 60 223 54 147 127 52 185 235 132 14 193 4 16 65 5 208 109 120 254 81 159 254 124 228 235 227 208 146 168 3 122 38 39 111 77 108 51 187 64 95 41 142 128 6 0 79 159 178 98 114 242 233 220 89 43 142 59 165 5 36 94 149 144 223 255 196 0 35 17 1 0 2 2 2 2 1 5 1 0 0 0 0 0 0 0 1 0 17 33 49 16 48 65 113 81 32 97 129 161 240 241 255 218 0 8 1 3 1 1 63 16 148 150 75 37 37 146 201 100 184 235 165 215 75 174 151 93 46 186 93 116 186 228 204 82 167 233 117 9 238 98 244 204 195 248 251 74 80 207 207 152 195 27 3 150 27 111 79 207 159 80 55 66 127 218 150 129 115 17 7 79 226 55 106 209 221 75 34 35 79 38 76 70 188 13 224 130 29 37 43 94 248 42 39 204 212 100 224 109 4 17 28 226 68 208 213 112 201 85 97 175 124 104 234 43 204 243 153 160 34 203 242 230 255 0 92 156 248 249 160 162 49 80 22 198 141 107 41 63 255 196 0 39 17 0 2 1 2 5 4 3 0 3 0 0 0 0 0 0 0 1 17 0 33 49 16 65 97 113 161 81 129 145 225 32 177 240 193 209 241 255 218 0 8 1 2 1 1 63 16 181 149 69 77 47 15 115 75 195 220 210 240 247 52 188 61 205 47 15 115 75 195 220 253 7 185 249 15 115 141 243 56 113 190 103 14 55 204 225 198 142 56 227 143 3 135 6 56 227 142 56 225 49 206 12 113 199 28 113 194 99 156 60 74 183 107 144 142 193 170 42 35 192 113 41 184 165 149 192 151 7 114 125 77 136 161 158 231 56 88 107 46 85 46 223 200 172 48 192 57 216 236 10 150 175 0 48 197 13 29 63 101 136 63 88 16 129 3 93 148 104 40 21 12 247 140 12 67 91 217 235 177 214 29 238 6 131 36 221 125 29 169 214 38 113 6 1 161 165 166 183 5 46 130 232 165 42 18 70 117 36 65 65 51 106 32 65 235 4 221 9 200 144 64 0 54 216 237 83 210 35 140 172 69 136 211 1 145 149 81 61 4 109 76 93 176 69 33 25 113 44 82 92 121 131 205 146 57 58 195 84 40 201 7 221 211 153 80 149 246 219 0 234 19 78 8 101 8 119 15 18 247 136 7 103 248 96 106 228 49 70 65 86 116 96 75 213 149 243 232 53 135 128 231 159 238 45 66 60 193 56 5 174 153 212 217 139 66 81 80 128 8 202 216 6 200 8 69 244 2 237 150 84 160 164 148 144 100 4 51 72 94 12 34 200 128 7 118 68 249 159 236 71 255 196 0 41 16 1 0 2 1 3 4 2 2 2 3 1 1 0 0 0 0 1 0 17 33 49 65 113 16 81 97 161 129 193 32 145 48 177 209 240 241 64 225 255 218 0 8 1 1 0 1 63 16 140 2 39 254 4 68 68 68 68 68 68 68 245 162 224 64 133 100 209 220 157 251 227 115 249 224 193 130 6 8 24 48 96 193 3 4 9 14 94 180 222 200 156 174 47 238 120 220 234 136 151 235 51 30 184 25 167 164 198 231 64 143 68 191 90 110 68 78 87 23 247 56 220 233 51 88 107 13 97 175 224 102 102 189 110 203 209 17 141 235 77 216 137 202 226 254 231 27 159 200 34 102 34 34 51 114 223 164 198 172 158 180 221 136 156 174 47 213 110 55 63 144 232 204 204 136 136 137 144 152 18 201 235 77 248 137 202 226 254 231 141 207 225 12 8 204 204 195 172 102 57 200 244 195 245 166 230 68 229 113 127 115 198 225 32 198 70 33 210 16 215 214 98 106 147 71 89 136 196 109 210 31 173 55 50 39 43 139 251 158 55 32 104 252 96 34 102 16 95 164 205 50 94 255 0 0 8 230 211 250 211 119 34 114 56 191 185 227 115 71 81 153 78 145 8 148 146 146 106 147 71 72 148 145 61 99 159 173 59 89 78 79 23 246 124 110 105 144 252 51 179 53 244 152 210 78 89 105 147 87 73 142 115 48 216 126 180 236 71 57 60 95 217 241 185 166 123 78 126 46 238 234 218 115 161 251 156 227 233 221 232 103 235 77 232 65 126 94 255 0 103 199 105 124 211 230 159 55 78 249 167 94 233 243 79 154 122 117 79 154 60 211 101 176 115 131 214 155 216 130 253 183 219 241 219 241 17 17 30 79 207 17 17 25 96 55 131 9 61 105 113 177 23 237 190 223 142 211 31 60 15 60 30 153 135 128 157 216 63 66 188 67 158 25 200 244 204 231 131 25 83 243 207 214 150 59 9 126 251 239 248 237 126 233 222 59 198 203 169 45 112 162 175 108 75 76 45 21 227 23 7 151 245 20 110 216 132 108 11 46 43 136 181 51 73 205 160 191 238 15 164 123 78 241 218 59 247 77 134 249 34 249 99 235 75 199 106 95 186 251 254 37 162 226 102 13 32 134 128 139 173 119 143 236 155 231 236 180 253 73 156 183 223 137 103 152 91 74 242 163 153 249 138 89 144 109 95 181 224 162 105 197 90 86 118 77 207 12 163 0 92 95 207 168 188 158 161 199 26 135 252 38 60 55 204 23 247 81 246 81 237 127 166 39 179 173 16 105 30 81 152 244 66 245 165 251 183 47 221 125 223 17 156 142 48 106 178 30 226 6 236 0 155 210 223 66 39 146 58 138 63 84 150 153 253 126 233 151 133 109 213 204 198 37 201 21 84 14 212 194 15 80 192 41 236 182 107 91 44 112 236 220 185 86 107 1 131 161 156 58 110 203 70 244 191 245 160 58 170 185 247 95 103 196 162 52 14 131 108 26 212 166 221 224 2 109 173 140 175 212 163 225 130 213 19 25 20 16 105 40 190 211 95 88 212 222 29 104 30 32 81 137 171 168 214 179 82 149 130 110 70 190 58 201 29 182 19 91 111 188 206 66 178 109 135 162 0 184 1 181 139 243 168 146 248 92 37 185 40 255 0 176 144 53 132 81 229 7 111 250 136 74 7 89 218 88 55 81 121 42 38 211 153 4 158 18 55 49 78 72 44 195 40 165 55 97 162 136 161 145 15 216 139 161 22 215 101 97 234 238 7 39 72 86 203 61 219 80 208 226 198 1 228 185 3 0 189 195 65 151 132 99 77 10 184 170 28 182 229 136 245 202 42 34 210 124 181 91 20 202 82 84 250 135 57 34 158 230 4 105 117 48 235 73 218 52 106 3 129 101 43 229 160 53 151 51 230 174 209 45 152 176 57 113 202 255 0 142 209 82 152 10 196 119 58 109 30 49 45 86 170 242 204 173 140 67 45 29 87 187 235 212 87 48 106 209 213 89 136 196 208 196 209 196 194 98 104 98 101 49 8 202 105 52 16 84 214 151 220 204 204 222 70 119 62 93 140 43 39 82 14 182 4 163 98 247 76 247 139 152 1 96 182 86 95 133 30 234 9 30 5 220 101 253 16 49 137 167 137 163 137 128 196 196 70 158 32 228 148 66 52 96 150 12 214 204 213 154 145 3 21 141 213 17 155 0 186 208 218 155 183 77 6 81 80 160 88 9 217 51 29 3 90 114 151 4 225 61 193 184 11 18 104 98 105 226 99 49 60 16 73 74 8 70 141 38 182 102 185 115 94 93 121 140 64 173 61 87 251 188 73 41 71 64 65 68 123 12 65 172 6 241 22 157 187 90 124 195 213 141 53 197 207 186 128 19 39 46 140 12 213 255 0 113 222 104 98 104 226 25 98 160 133 105 53 243 53 166 180 52 1 149 90 8 109 125 31 245 102 59 197 213 88 65 15 102 22 203 226 238 58 19 46 104 50 3 38 193 233 22 7 32 31 145 253 75 245 104 15 47 248 9 167 137 167 136 16 124 166 92 159 28 197 21 79 12 38 252 121 50 230 255 0 85 128 57 174 204 195 0 47 103 250 34 15 140 240 62 38 188 215 206 37 29 153 13 147 36 210 44 134 192 221 133 91 184 184 43 165 173 80 211 23 182 46 103 31 0 246 0 127 146 99 49 52 177 4 194 161 204 194 230 106 230 101 115 53 179 45 28 203 239 51 95 51 91 48 153 76 235 79 241 231 105 140 168 221 199 125 165 145 126 78 144 162 210 139 207 255 0 96 179 9 134 180 148 169 195 203 30 221 199 152 219 80 174 215 78 235 114 22 93 123 74 247 152 156 251 152 178 153 117 157 244 87 39 220 194 230 107 103 220 214 206 209 35 190 90 111 182 196 122 52 222 97 180 4 239 171 189 233 26 237 111 24 71 187 171 215 16 103 86 140 29 211 116 54 90 238 166 216 2 15 152 43 204 160 229 214 101 102 111 12 189 172 181 148 107 171 196 189 172 202 203 9 154 153 100 1 192 175 220 45 110 142 118 135 1 216 240 117 0 255 217 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:30'!
leftLogoJpg
^#(255 216 255 224 0 16 74 70 73 70 0 1 1 1 0 72 0 72 0 0 255 225 50 29 104 116 116 112 58 47 47 110 115 46 97 100 111 98 101 46 99 111 109 47 120 97 112 47 49 46 48 47 0 60 63 120 112 97 99 107 101 116 32 98 101 103 105 110 61 34 239 187 191 34 32 105 100 61 34 87 53 77 48 77 112 67 101 104 105 72 122 114 101 83 122 78 84 99 122 107 99 57 100 34 63 62 10 60 120 58 120 109 112 109 101 116 97 32 120 109 108 110 115 58 120 61 34 97 100 111 98 101 58 110 115 58 109 101 116 97 47 34 32 120 58 120 109 112 116 107 61 34 65 100 111 98 101 32 88 77 80 32 67 111 114 101 32 52 46 49 45 99 48 51 52 32 52 54 46 50 55 50 57 55 54 44 32 83 97 116 32 74 97 110 32 50 55 32 50 48 48 55 32 50 50 58 51 55 58 51 55 32 32 32 32 32 32 32 32 34 62 10 32 32 32 60 114 100 102 58 82 68 70 32 120 109 108 110 115 58 114 100 102 61 34 104 116 116 112 58 47 47 119 119 119 46 119 51 46 111 114 103 47 49 57 57 57 47 48 50 47 50 50 45 114 100 102 45 115 121 110 116 97 120 45 110 115 35 34 62 10 32 32 32 32 32 32 60 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 32 114 100 102 58 97 98 111 117 116 61 34 34 10 32 32 32 32 32 32 32 32 32 32 32 32 120 109 108 110 115 58 120 97 112 61 34 104 116 116 112 58 47 47 110 115 46 97 100 111 98 101 46 99 111 109 47 120 97 112 47 49 46 48 47 34 62 10 32 32 32 32 32 32 32 32 32 60 120 97 112 58 67 114 101 97 116 111 114 84 111 111 108 62 65 100 111 98 101 32 70 105 114 101 119 111 114 107 115 32 67 83 51 60 47 120 97 112 58 67 114 101 97 116 111 114 84 111 111 108 62 10 32 32 32 32 32 32 32 32 32 60 120 97 112 58 67 114 101 97 116 101 68 97 116 101 62 50 48 48 55 45 48 53 45 50 56 84 49 50 58 52 48 58 48 50 90 60 47 120 97 112 58 67 114 101 97 116 101 68 97 116 101 62 10 32 32 32 32 32 32 32 32 32 60 120 97 112 58 77 111 100 105 102 121 68 97 116 101 62 50 48 48 55 45 48 53 45 50 57 84 49 51 58 49 54 58 51 57 90 60 47 120 97 112 58 77 111 100 105 102 121 68 97 116 101 62 10 32 32 32 32 32 32 60 47 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 62 10 32 32 32 32 32 32 60 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 32 114 100 102 58 97 98 111 117 116 61 34 34 10 32 32 32 32 32 32 32 32 32 32 32 32 120 109 108 110 115 58 100 99 61 34 104 116 116 112 58 47 47 112 117 114 108 46 111 114 103 47 100 99 47 101 108 101 109 101 110 116 115 47 49 46 49 47 34 62 10 32 32 32 32 32 32 32 32 32 60 100 99 58 102 111 114 109 97 116 62 105 109 97 103 101 47 106 112 101 103 60 47 100 99 58 102 111 114 109 97 116 62 10 32 32 32 32 32 32 60 47 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 62 10 32 32 32 60 47 114 100 102 58 82 68 70 62 10 60 47 120 58 120 109 112 109 101 116 97 62 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 60 63 120 112 97 99 107 101 116 32 101 110 100 61 34 119 34 63 62 255 219 0 67 0 4 3 3 3 3 3 4 3 3 4 5 4 3 4 5 7 5 4 4 5 7 8 6 6 7 6 6 8 10 8 8 8 8 8 8 10 8 10 10 11 10 10 8 13 13 14 14 13 13 18 18 18 18 18 20 20 20 20 20 20 20 20 20 20 255 219 0 67 1 5 5 5 8 7 8 15 10 10 15 18 15 12 15 18 22 21 21 21 21 22 22 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 255 194 0 17 8 0 219 0 133 3 1 17 0 2 17 1 3 17 1 255 196 0 27 0 1 0 3 1 1 1 1 0 0 0 0 0 0 0 0 0 0 5 6 7 1 3 2 4 255 196 0 26 1 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 1 6 2 5 3 4 255 218 0 12 3 1 0 2 16 3 16 0 0 1 223 193 242 124 156 0 0 0 0 3 212 241 33 124 185 31 231 79 15 131 128 1 40 0 0 46 250 219 15 248 37 79 41 16 0 0 0 0 3 64 221 245 85 205 72 159 38 0 0 0 0 0 26 6 239 170 158 103 152 191 40 0 0 0 0 0 104 27 190 170 153 158 98 188 160 0 0 0 0 1 160 110 250 170 102 100 87 149 0 0 0 0 0 11 254 239 170 174 102 69 121 80 0 0 0 0 0 208 55 125 85 51 50 43 202 128 0 0 0 0 6 129 187 234 169 153 145 94 84 0 0 0 0 0 52 13 223 85 76 204 138 242 160 0 0 0 0 1 160 110 250 170 102 100 87 149 0 0 0 0 0 13 3 119 213 83 51 34 188 168 0 0 0 0 0 104 27 190 170 153 153 21 229 64 0 0 0 0 3 64 221 245 84 204 200 175 42 0 0 0 0 0 26 6 239 170 166 102 69 121 80 0 0 0 0 0 208 55 125 85 51 50 43 202 128 0 0 0 0 6 129 187 234 169 153 145 94 84 0 0 0 0 0 52 13 223 85 76 204 138 242 160 0 0 0 0 1 160 110 250 170 102 100 87 149 0 0 0 0 0 13 3 119 213 83 51 34 188 168 0 0 0 0 0 104 27 190 170 153 153 21 229 64 0 0 0 0 3 64 221 245 84 204 200 175 42 0 0 0 0 0 26 6 239 170 166 102 69 121 80 0 0 0 0 0 208 55 125 85 51 50 43 202 128 0 0 0 0 6 129 187 234 169 153 145 94 84 0 0 0 0 0 52 13 223 85 207 22 71 126 23 66 14 133 28 249 144 0 0 93 181 183 240 129 10 0 15 115 234 128 8 3 255 196 0 33 16 0 1 3 5 0 3 1 1 0 0 0 0 0 0 0 0 0 2 3 4 18 20 48 50 51 1 16 19 17 49 255 218 0 8 1 1 0 1 5 2 247 90 79 162 79 162 79 162 74 210 125 18 86 146 180 149 164 173 37 105 43 73 90 74 211 233 107 252 30 150 134 213 230 114 203 183 139 183 203 183 203 183 203 183 203 183 203 183 203 183 203 183 203 183 203 183 203 183 203 183 253 72 127 226 140 235 214 126 249 215 172 254 153 220 214 127 76 238 107 63 166 117 235 63 166 119 53 159 211 59 154 207 233 157 122 207 233 157 205 103 244 206 230 179 250 103 94 179 250 103 94 179 250 103 115 89 253 51 185 172 254 153 220 214 127 76 238 107 63 166 117 235 63 166 119 53 159 211 59 154 207 233 157 205 103 244 206 230 179 250 103 94 179 250 103 94 179 250 103 115 89 140 62 226 236 230 22 114 203 57 101 156 178 202 89 101 44 179 150 90 72 45 31 45 31 45 31 45 31 45 31 45 31 45 31 60 255 0 40 73 66 74 18 80 146 132 148 36 161 37 9 60 33 63 148 36 161 37 9 40 73 66 74 18 80 147 255 196 0 20 17 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 128 255 218 0 8 1 3 1 1 63 1 110 127 255 196 0 42 17 0 0 4 6 2 0 5 5 1 0 0 0 0 0 0 0 0 5 21 64 3 4 51 82 129 177 1 97 2 18 49 81 209 16 19 33 34 35 161 255 218 0 8 1 2 1 1 63 1 103 50 103 224 133 207 151 143 219 145 201 188 94 135 38 115 30 250 10 83 23 127 156 5 41 139 180 20 166 46 208 82 152 187 65 70 98 237 5 25 139 180 20 102 46 208 81 152 187 65 70 98 237 5 25 139 180 20 102 46 208 81 152 187 95 73 249 175 179 224 252 122 243 232 196 223 159 237 134 38 245 177 242 196 222 182 24 155 214 195 19 122 216 98 111 91 12 77 235 97 137 189 108 49 55 173 134 38 245 176 196 222 182 24 155 214 199 203 19 122 216 249 98 111 91 12 77 235 97 137 189 108 49 55 173 134 38 245 176 196 222 182 24 155 214 195 19 122 216 98 111 91 31 44 77 235 97 140 252 140 72 209 60 222 31 96 149 27 160 149 27 160 149 27 160 149 27 160 149 27 160 149 27 160 149 27 161 201 108 127 96 157 49 110 130 116 197 186 9 211 22 232 39 76 91 160 157 49 110 130 116 197 186 9 211 22 233 151 255 196 0 43 16 0 2 2 1 3 3 2 4 7 0 0 0 0 0 0 0 0 1 2 49 209 64 115 146 33 51 145 16 17 3 18 19 65 32 34 67 97 114 161 193 255 218 0 8 1 1 0 6 63 2 252 22 89 101 150 89 101 150 89 101 151 233 251 159 43 252 242 251 165 209 29 33 15 98 215 20 90 227 28 22 184 199 5 174 49 193 107 140 112 90 227 28 22 184 199 5 174 49 193 107 140 112 90 227 28 22 184 199 5 174 49 193 107 140 113 232 231 250 146 233 12 232 25 5 246 249 22 129 144 219 90 6 67 109 104 25 13 181 160 100 54 214 129 144 219 90 6 67 109 104 25 13 181 160 100 54 214 129 144 219 90 6 67 109 104 25 13 181 160 100 54 214 129 144 219 90 6 67 109 104 25 13 181 160 100 54 214 129 144 219 90 6 67 109 104 25 13 181 160 100 54 214 129 144 219 90 6 67 109 104 25 9 124 47 134 231 31 166 186 251 199 253 104 236 191 48 201 217 126 99 147 178 252 199 39 101 249 142 78 211 243 28 157 175 238 57 59 79 204 114 117 143 179 254 81 201 75 148 114 82 229 28 148 185 71 37 46 81 201 75 148 114 82 229 28 148 185 71 62 148 81 69 20 81 69 20 81 69 20 81 69 31 255 196 0 35 16 0 1 4 2 2 2 2 3 0 0 0 0 0 0 0 0 1 17 64 97 240 0 33 48 49 16 65 32 81 145 161 209 255 218 0 8 1 1 0 1 63 33 240 72 29 225 7 190 116 150 91 109 182 221 17 178 232 98 104 35 238 33 11 189 226 178 143 208 16 73 252 174 19 146 64 96 127 30 82 4 8 16 32 64 129 2 4 12 16 37 2 228 138 71 246 193 166 4 119 187 73 233 73 44 117 124 177 85 242 197 87 203 15 214 202 249 98 171 229 138 175 150 58 190 88 170 249 98 171 229 142 175 150 58 190 88 170 249 98 171 229 138 175 150 42 190 88 234 249 98 171 229 138 175 150 42 190 88 170 249 99 171 229 142 175 150 41 28 33 32 13 239 91 31 35 175 255 0 239 9 35 196 238 150 131 244 21 56 4 8 16 32 64 193 1 82 94 16 0 0 4 173 50 22 66 200 89 11 33 100 44 133 159 255 218 0 12 3 1 0 2 0 3 0 0 0 16 2 0 0 0 0 36 128 112 201 37 36 146 72 34 219 109 182 219 109 192 219 109 182 219 109 177 219 109 182 219 109 182 27 109 182 219 109 182 194 109 182 219 109 182 220 77 182 219 109 182 219 1 182 219 109 182 219 96 54 219 109 182 219 108 38 219 109 182 219 109 128 219 109 182 219 109 176 155 109 182 219 109 182 19 109 182 219 109 182 194 109 182 219 109 182 216 77 182 219 109 182 219 9 182 219 109 182 219 96 54 219 109 182 219 108 38 219 109 182 219 109 132 219 109 182 219 109 176 155 109 182 219 109 182 19 109 182 219 109 182 194 109 182 219 109 182 216 77 182 219 109 182 219 9 182 219 109 182 219 97 54 219 109 182 219 108 44 159 184 100 146 73 9 124 154 69 36 146 79 255 196 0 26 17 0 3 1 1 1 1 0 0 0 0 0 0 0 0 0 0 1 17 64 0 32 16 80 255 218 0 8 1 3 1 1 63 16 141 246 243 207 60 243 207 63 9 132 253 51 9 132 194 97 48 152 76 38 19 9 132 194 97 48 152 76 38 19 9 132 194 96 24 140 178 225 101 150 89 101 150 89 101 23 255 196 0 37 17 0 1 2 6 2 2 2 3 0 0 0 0 0 0 0 0 1 0 240 17 49 64 113 177 209 33 129 65 225 16 97 32 48 193 255 218 0 8 1 2 1 1 63 16 253 80 80 80 252 225 182 4 252 1 245 30 121 235 184 240 143 60 0 29 237 16 124 122 209 57 161 48 52 78 13 19 3 68 224 209 56 52 78 13 19 131 68 224 209 56 52 78 13 19 131 79 142 157 109 214 168 77 0 241 14 77 12 171 115 66 151 110 77 12 139 114 104 101 219 147 67 34 220 154 0 164 219 147 67 46 220 154 25 54 228 208 203 183 38 134 85 185 52 50 173 205 10 85 185 161 74 183 38 134 69 185 52 50 237 201 161 151 110 77 12 139 114 104 101 91 147 67 42 220 154 25 86 228 208 202 183 52 41 118 228 208 137 64 132 3 155 159 164 241 58 79 19 164 241 244 158 62 147 199 210 120 250 77 31 72 115 1 31 99 250 66 112 236 156 59 39 14 201 195 178 112 236 156 59 39 14 212 95 255 196 0 37 16 0 1 3 4 1 4 2 3 0 0 0 0 0 0 0 0 1 0 81 97 17 64 161 240 33 16 48 49 65 32 113 129 145 177 255 218 0 8 1 1 0 1 63 16 233 229 212 94 88 84 37 1 66 81 148 37 25 70 81 148 101 25 70 81 148 110 128 22 208 29 24 152 116 246 174 164 53 16 246 0 164 213 23 200 19 73 67 208 52 43 250 66 161 146 72 164 136 21 51 196 158 235 199 14 28 60 120 241 227 199 129 30 86 24 168 33 229 3 143 192 45 244 207 96 85 80 185 217 64 181 240 170 16 38 130 195 25 104 94 193 132 180 47 96 192 91 23 177 198 193 236 56 75 98 246 28 37 180 123 14 50 216 189 135 9 109 30 195 128 182 15 97 198 91 7 176 226 45 139 216 112 22 197 236 56 75 98 246 28 37 181 123 14 18 216 189 135 25 109 94 195 128 182 15 97 192 91 87 176 224 45 171 216 112 22 197 236 56 203 98 246 28 101 181 123 14 18 15 136 9 32 79 131 198 17 234 159 11 185 209 242 120 167 160 230 71 211 118 12 167 134 7 223 96 241 227 199 143 2 60 0 92 133 20 37 9 66 80 148 37 9 66 80 151 220 175 239 96 136 136 136 191 255 217 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:30'!
leftLogoPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 133 0 0 0 219 8 6 0 0 0 67 94 82 187 0 0 0 4 115 66 73 84 8 8 8 8 124 8 100 136 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 0 22 116 69 88 116 67 114 101 97 116 105 111 110 32 84 105 109 101 0 48 53 47 50 56 47 48 55 109 245 38 235 0 0 0 24 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 70 105 114 101 119 111 114 107 115 79 179 31 78 0 0 4 144 112 114 86 87 120 156 237 91 81 118 155 48 16 148 8 52 177 27 26 238 146 51 228 42 249 238 117 124 153 30 160 143 187 244 6 117 1 219 32 237 206 10 146 29 127 180 197 142 98 192 196 122 179 179 51 187 194 228 231 239 31 191 194 91 248 126 30 31 253 240 60 247 125 63 109 245 231 83 127 126 239 207 175 253 184 51 60 78 125 255 222 247 195 110 56 157 95 79 253 107 24 254 98 31 251 248 39 6 229 177 207 255 119 207 127 28 158 135 112 136 213 39 135 115 254 186 138 95 158 31 159 31 191 126 114 60 123 241 55 213 183 174 117 60 189 243 87 113 252 152 203 207 229 249 177 109 55 254 216 190 12 31 243 52 124 224 203 240 153 79 31 221 246 206 127 136 11 150 237 200 121 248 99 125 69 52 161 217 134 188 155 70 59 30 115 243 191 224 207 225 149 145 19 241 199 4 245 86 228 215 237 49 106 44 252 11 200 2 242 52 76 215 23 14 255 221 21 253 58 231 75 174 92 182 253 248 1 108 180 43 211 132 133 127 214 255 45 2 9 250 4 185 140 198 45 90 206 249 235 220 255 0 232 60 25 210 224 80 240 55 97 230 211 122 21 25 63 31 155 34 224 157 191 138 45 128 108 224 23 199 198 95 110 252 17 107 91 120 66 238 142 201 49 63 255 134 177 33 197 43 246 201 252 75 157 231 110 151 190 119 139 6 161 254 107 160 240 21 186 31 163 254 223 144 228 89 223 137 33 121 191 109 123 231 31 252 95 226 4 78 0 241 179 252 95 240 156 58 95 155 214 1 145 11 151 72 248 241 151 165 158 112 15 35 225 198 31 115 109 119 70 14 88 142 72 224 191 19 248 45 205 235 48 113 248 159 179 59 173 120 50 7 16 247 156 250 47 25 182 75 1 58 139 197 191 228 190 232 3 137 83 58 231 175 234 104 224 182 125 32 59 195 139 255 161 78 144 103 90 55 125 128 218 255 213 81 3 44 250 128 136 144 27 127 108 179 62 175 93 169 5 82 5 206 249 15 33 199 47 125 206 202 1 90 253 59 226 250 95 240 129 92 5 222 249 131 90 255 202 253 114 95 224 198 15 251 191 82 45 200 115 131 160 127 205 118 186 123 231 254 175 106 90 209 209 108 170 5 115 36 252 250 55 42 156 81 11 100 13 244 235 95 87 56 93 229 64 143 192 243 127 75 225 150 6 178 51 120 252 43 135 3 43 33 185 239 142 191 141 95 219 0 84 192 189 248 159 115 64 123 62 211 255 131 194 95 138 196 29 252 31 240 175 107 1 240 7 146 255 95 250 159 28 116 177 222 203 172 240 243 63 34 82 215 128 243 46 200 92 253 248 215 255 151 250 15 125 79 50 158 231 62 189 254 43 253 167 93 177 228 158 229 255 85 176 253 15 231 129 136 135 27 255 22 255 55 188 129 224 255 72 255 89 10 0 71 200 142 186 243 191 150 252 119 224 74 103 139 122 68 150 255 227 14 7 102 251 29 240 71 163 195 129 217 206 191 254 111 224 55 178 93 159 67 224 191 208 229 202 108 215 231 80 248 239 96 12 108 69 36 63 126 254 147 222 206 232 124 50 221 139 245 129 159 127 165 246 12 63 132 157 104 130 196 63 200 1 224 1 224 74 8 141 127 148 245 41 90 67 9 12 253 103 26 55 175 116 224 111 67 8 250 71 178 6 254 127 55 253 219 189 158 217 245 46 142 72 224 95 167 191 205 189 210 9 133 255 194 247 59 38 247 212 250 143 149 95 224 158 169 127 244 253 206 10 247 75 52 252 252 227 204 95 225 158 168 255 85 173 99 69 48 235 127 145 239 114 71 68 224 191 188 202 41 112 79 91 255 89 125 223 10 247 227 43 145 255 13 171 221 156 251 214 175 127 185 254 43 115 175 84 66 90 255 233 222 223 238 255 197 202 216 203 127 72 250 159 82 30 88 29 129 27 63 250 254 207 224 190 227 251 127 29 244 253 31 38 247 232 44 47 254 233 251 55 251 126 143 156 123 125 125 192 237 255 65 247 191 134 211 225 243 88 252 67 196 89 166 39 10 225 245 127 117 144 254 183 221 3 40 250 31 239 63 40 119 56 173 116 70 106 255 23 34 68 41 77 65 200 159 169 127 163 239 51 87 196 249 213 112 10 255 154 111 184 131 50 133 195 191 172 247 176 14 220 73 255 185 206 205 173 59 225 63 38 235 63 168 130 242 125 0 44 253 151 28 16 200 129 174 255 130 3 118 234 202 72 234 152 52 254 49 226 149 234 79 186 254 187 177 215 203 248 167 245 127 6 247 235 49 160 240 63 244 127 90 239 184 215 131 87 199 72 252 11 84 233 54 170 122 203 97 34 255 160 191 67 247 1 228 145 224 241 175 50 28 71 69 156 206 226 31 223 237 40 35 161 243 195 207 191 52 54 136 27 30 225 240 175 174 249 41 143 47 85 72 63 255 11 221 50 233 183 236 251 249 143 77 60 138 241 208 196 102 26 224 189 236 253 198 251 255 79 67 253 57 132 170 169 228 107 61 141 120 152 246 231 99 233 246 52 194 193 49 255 62 246 177 143 125 236 99 31 251 248 255 198 31 182 16 202 11 223 151 167 216 0 0 0 72 109 107 66 70 250 222 202 254 0 0 0 4 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 41 164 51 161 0 0 55 189 109 107 84 83 120 156 237 125 89 119 219 198 178 46 78 246 118 28 217 178 29 39 103 229 60 156 23 173 117 207 93 247 41 217 152 9 60 138 162 40 41 161 36 134 164 60 228 197 11 4 137 88 39 242 176 101 89 217 217 90 252 239 183 170 186 27 67 179 1 2 224 32 41 129 100 11 36 198 198 87 213 95 13 93 13 28 255 212 190 190 121 49 24 158 223 248 179 23 131 227 243 27 99 214 29 236 165 22 131 159 79 206 111 198 126 232 249 45 125 246 170 183 23 221 232 179 215 108 241 242 168 19 221 24 150 53 59 60 26 69 55 166 225 207 6 251 195 232 166 101 206 6 195 179 115 216 161 125 0 167 136 232 103 214 239 245 174 111 218 125 248 179 183 59 250 116 163 125 165 141 181 80 123 163 181 181 64 251 164 157 107 225 236 232 228 24 214 111 195 250 247 176 254 16 214 95 106 19 109 71 235 193 214 247 218 116 54 232 156 142 241 164 187 39 116 238 93 104 179 53 245 102 237 206 209 249 141 61 107 31 67 227 167 176 128 213 193 172 61 220 167 157 134 93 188 135 246 240 144 190 181 123 108 241 19 45 246 78 248 9 246 187 244 125 48 162 125 187 109 250 214 29 208 226 132 173 28 246 97 223 112 214 30 177 141 35 118 246 209 144 93 228 152 157 143 45 142 118 177 149 39 216 42 125 214 57 53 176 57 157 83 19 79 211 57 181 104 209 133 149 38 44 76 182 176 112 49 43 129 205 179 57 108 6 218 7 237 51 172 155 104 83 109 178 44 66 198 253 71 72 104 207 16 112 137 180 171 101 180 199 136 236 218 250 99 44 137 142 177 94 253 73 208 89 82 127 100 140 170 104 208 45 99 180 205 49 218 5 124 46 1 137 54 252 253 12 104 189 229 88 125 197 177 74 48 44 66 7 219 157 130 199 181 25 62 180 125 33 62 158 157 193 71 207 224 99 153 89 132 38 75 246 49 147 33 100 50 132 108 134 144 205 16 178 103 195 254 47 76 170 195 33 124 8 199 176 226 148 221 198 112 120 74 43 170 96 248 152 99 56 2 237 250 23 232 219 103 216 190 72 207 44 83 165 104 197 64 26 62 135 210 28 135 21 160 12 61 6 37 173 95 27 152 166 95 14 204 167 28 204 61 80 184 11 248 61 215 126 5 184 2 237 163 246 86 251 131 3 186 149 82 202 119 240 249 131 246 161 16 76 131 247 90 195 42 77 251 134 110 171 187 173 94 208 109 29 157 33 73 124 128 72 70 65 233 158 107 122 165 177 180 60 147 97 105 25 147 202 216 21 119 232 214 132 65 231 133 12 185 233 138 28 10 189 200 92 154 53 201 174 28 100 117 1 250 25 250 233 57 238 149 1 200 118 24 64 198 88 210 173 136 67 164 51 140 194 162 142 138 106 177 88 183 136 16 83 56 33 155 34 78 164 84 107 0 106 94 183 226 142 90 183 131 14 96 235 152 58 232 251 66 53 3 159 121 165 122 182 90 163 186 30 61 251 146 99 244 18 108 193 149 18 157 150 164 98 146 211 81 228 151 225 161 25 124 204 91 199 103 216 111 51 246 31 182 231 72 255 113 140 215 91 242 233 67 208 24 64 78 210 166 61 178 155 72 249 127 148 162 123 142 151 97 134 74 196 36 194 111 133 213 85 202 118 25 100 1 199 204 154 216 165 109 103 253 78 9 23 52 157 128 1 137 166 166 58 146 125 34 255 43 64 243 162 18 146 145 93 194 114 70 194 114 162 63 81 18 200 152 219 56 144 228 147 172 30 71 132 111 40 188 17 36 55 230 150 136 15 136 44 243 75 196 135 65 101 136 31 102 131 139 42 240 146 205 88 24 78 144 58 87 198 119 202 61 19 60 24 1 246 221 85 3 60 0 92 205 233 148 97 38 27 141 98 204 234 56 200 106 82 52 157 66 7 57 52 171 247 241 216 61 118 76 6 29 195 176 28 118 198 180 52 53 198 138 103 70 25 232 182 98 232 208 130 252 65 182 180 78 108 150 13 205 40 68 168 153 251 40 211 145 77 143 123 41 20 148 148 68 203 50 85 104 185 12 45 151 161 229 178 174 204 8 16 63 140 35 169 43 163 141 25 28 86 192 241 97 236 173 4 218 111 11 114 36 30 67 209 103 40 146 139 150 66 81 95 22 69 155 161 200 84 76 9 163 19 48 28 205 2 28 29 157 167 1 124 158 7 240 57 146 92 241 92 174 121 174 157 7 38 91 147 2 19 156 206 106 74 57 130 158 14 42 169 253 86 165 23 151 210 202 114 222 179 90 45 169 51 163 161 30 175 156 255 200 192 80 31 22 64 14 210 150 134 217 158 197 234 217 161 206 252 150 248 47 155 174 74 109 41 221 233 149 89 132 59 144 174 50 106 166 171 242 81 250 90 137 82 135 18 123 99 112 193 139 187 245 93 197 201 220 16 78 35 232 199 231 16 158 220 87 156 172 149 227 180 21 227 244 1 60 146 171 13 38 134 139 120 204 15 237 154 65 26 55 9 58 131 70 103 208 232 12 26 157 65 163 151 132 230 169 82 133 248 104 76 117 245 201 114 189 190 249 209 151 34 50 114 24 66 14 67 200 89 138 178 15 41 122 189 94 16 189 222 213 17 6 142 18 88 108 130 201 97 48 57 12 166 128 193 20 48 152 2 165 239 223 197 112 147 134 91 70 200 53 10 101 186 4 247 107 71 59 230 159 166 218 101 25 87 172 138 235 96 56 211 50 131 13 186 50 200 175 220 229 212 142 3 119 103 61 6 149 199 156 127 54 214 144 159 24 41 198 238 144 18 76 111 121 162 233 45 177 85 22 59 244 26 41 109 137 17 16 73 83 207 56 178 44 110 154 115 100 149 157 145 116 177 108 70 60 88 43 118 229 161 18 61 178 23 67 148 175 104 114 254 45 84 245 72 204 90 231 210 86 18 163 215 72 38 173 77 207 210 126 125 146 73 66 79 149 50 73 222 180 2 158 15 51 110 254 251 213 142 185 151 28 200 146 76 227 6 186 105 12 22 226 152 6 235 75 14 214 41 14 25 112 136 30 197 225 57 250 86 152 110 91 228 93 5 117 139 19 48 154 89 4 147 37 133 150 34 20 18 169 54 82 232 185 192 210 40 145 206 192 181 228 97 113 176 76 143 197 62 20 233 236 82 192 70 202 231 186 44 242 177 113 73 233 196 133 24 62 79 97 136 25 35 44 246 24 146 159 26 112 175 172 216 70 4 202 4 91 9 72 49 147 85 48 52 19 6 12 84 147 131 26 154 82 126 109 194 115 68 204 112 47 136 48 65 117 25 172 30 199 213 99 192 134 30 3 54 228 192 134 158 28 175 227 135 129 248 144 233 224 108 83 252 65 36 152 250 3 30 147 14 6 34 197 57 156 79 215 169 68 33 146 116 27 129 223 154 148 130 95 74 209 197 232 23 142 187 230 228 74 108 238 247 217 12 123 112 144 8 123 90 90 180 36 164 25 138 74 164 201 136 35 192 164 232 3 166 248 101 225 125 20 195 123 69 57 21 44 70 249 84 198 255 17 131 218 162 216 194 8 166 37 240 45 227 255 56 18 99 100 153 21 249 118 72 149 114 249 224 186 118 53 112 5 112 70 20 231 72 8 64 115 49 83 124 171 206 33 243 114 140 114 252 219 170 201 191 165 204 20 79 237 69 129 84 35 192 225 180 199 25 250 181 85 244 43 224 148 121 194 225 113 156 195 3 57 88 102 139 46 88 22 106 192 0 29 14 56 53 243 239 8 180 17 41 41 89 68 56 125 128 242 61 124 154 87 87 67 251 30 182 158 3 228 11 199 146 178 227 228 181 179 207 69 177 95 5 31 32 171 168 94 221 196 193 60 50 255 165 66 6 212 112 23 214 94 193 247 239 225 19 58 234 24 25 46 28 228 88 45 102 171 245 155 86 136 153 48 243 123 96 228 47 224 191 232 186 233 61 85 225 12 133 45 185 46 102 80 61 177 80 193 193 44 111 98 74 1 69 253 20 249 111 222 182 160 231 137 91 134 108 89 14 82 97 174 15 226 194 131 115 52 216 235 41 31 14 203 187 233 149 170 205 36 239 147 211 95 192 128 12 24 144 222 156 163 206 93 207 116 181 89 6 208 52 128 79 56 128 47 201 163 153 242 161 73 210 65 105 64 163 92 246 70 170 219 139 227 234 114 9 28 28 98 172 168 141 177 207 67 46 211 82 195 146 74 36 227 49 243 100 160 141 114 21 226 195 224 52 101 182 117 102 77 36 37 45 143 113 185 65 35 211 83 233 41 75 0 173 16 97 49 102 100 137 113 95 167 60 192 234 152 210 82 141 25 81 207 206 235 251 213 96 172 18 148 71 220 123 140 178 222 99 20 148 0 177 76 184 185 108 2 72 173 162 166 10 65 22 149 167 163 160 120 176 77 174 144 20 124 57 224 30 246 7 237 157 196 151 104 130 222 107 17 22 101 161 185 86 97 104 56 54 195 208 205 98 24 84 128 208 85 234 161 210 169 97 67 191 117 188 26 161 133 89 194 52 185 26 210 210 162 37 235 189 46 239 190 229 65 19 10 199 6 73 138 195 21 75 213 105 165 50 73 181 117 209 203 123 215 196 183 243 88 141 237 197 131 230 113 172 34 13 154 35 193 18 94 236 131 197 62 48 255 218 155 114 255 26 63 244 5 134 67 17 250 13 4 71 178 164 71 57 76 69 93 96 23 67 24 21 162 99 59 19 92 11 237 171 18 93 23 218 107 194 80 85 140 160 142 255 56 166 150 10 211 208 83 7 44 62 207 174 249 188 168 195 119 89 166 136 194 18 49 70 222 78 194 20 49 102 46 101 199 243 65 76 162 233 127 2 140 1 57 148 197 234 233 212 245 125 42 88 20 22 243 165 114 111 21 170 176 32 62 84 170 39 174 239 242 245 93 182 62 134 146 172 117 139 27 235 22 211 80 1 41 41 232 33 11 13 171 117 246 215 132 103 177 117 201 90 232 82 117 131 85 124 73 174 153 99 229 96 168 20 73 43 173 75 94 98 34 175 66 38 91 129 201 85 145 34 230 156 185 32 103 0 213 123 50 212 159 201 1 23 110 249 3 14 162 165 117 106 148 183 181 202 204 163 41 19 210 88 89 23 39 11 32 168 138 34 21 81 46 109 102 122 138 92 240 33 179 209 85 112 122 148 224 4 113 224 1 185 57 191 23 35 166 52 48 229 166 30 9 200 140 197 58 23 42 13 140 49 85 145 161 177 81 200 182 99 15 230 45 21 73 99 218 97 58 87 122 47 195 230 214 29 115 48 202 143 2 198 177 138 229 86 47 10 20 142 224 162 168 239 80 184 206 135 243 174 243 34 232 158 199 54 247 130 102 125 84 29 102 80 143 16 46 175 122 84 79 158 26 103 8 148 222 180 80 190 50 153 88 217 18 43 149 79 120 52 131 120 176 96 222 129 41 219 129 79 97 253 21 37 188 22 77 63 93 69 7 118 202 39 96 37 12 253 197 70 67 148 167 74 238 244 138 186 175 208 65 172 205 63 167 92 225 218 117 112 174 146 210 82 42 161 239 101 7 187 34 165 213 80 143 31 74 5 190 161 154 2 85 254 160 156 198 73 134 185 184 23 147 206 99 51 117 133 27 108 57 204 44 187 213 180 245 73 28 57 127 160 80 230 45 0 207 167 72 20 235 236 234 10 226 138 236 180 167 156 34 135 121 144 20 226 102 149 17 152 60 173 101 254 205 112 62 8 92 132 224 211 20 130 255 162 36 206 14 101 42 42 99 72 137 193 10 147 93 99 28 237 197 161 116 237 241 1 140 149 75 24 33 74 66 132 45 69 65 58 21 249 208 10 241 65 12 191 244 185 51 137 183 200 6 100 83 222 165 60 114 184 72 10 223 113 41 188 160 17 175 144 170 95 63 209 224 3 50 48 230 136 119 18 130 169 78 39 108 42 85 89 197 86 178 73 86 177 37 111 138 82 109 243 92 82 201 3 181 120 142 200 226 169 117 88 18 151 112 241 164 237 91 159 11 39 147 204 36 105 165 134 202 188 236 72 25 250 251 179 110 175 115 125 211 77 196 242 80 139 226 39 13 92 66 160 41 2 166 40 158 159 129 107 207 148 107 185 16 186 12 134 46 99 132 46 187 251 238 62 233 110 119 208 161 93 6 3 182 13 155 1 139 87 180 56 136 91 241 28 218 16 198 41 255 9 23 248 85 106 84 229 83 76 119 33 111 3 90 151 80 251 13 186 170 24 32 232 30 188 128 123 59 217 163 235 31 28 193 231 131 62 62 230 164 203 158 98 162 211 207 44 181 201 16 155 248 35 78 112 219 107 220 166 47 127 30 163 230 41 196 38 252 73 201 233 41 151 211 0 144 9 225 110 49 196 253 21 186 8 162 112 25 19 89 196 103 131 168 246 57 43 177 79 61 137 6 76 162 65 35 209 58 18 101 105 219 43 34 188 75 129 151 36 81 245 62 103 37 246 169 39 81 139 73 212 48 26 145 86 17 233 87 92 164 67 66 228 60 85 145 207 68 116 66 40 189 35 171 119 150 187 165 158 200 116 38 50 61 211 160 45 222 32 254 12 157 20 87 136 11 167 183 156 229 110 169 215 36 147 53 201 108 148 168 138 18 61 142 121 225 156 6 202 178 28 191 21 243 55 67 111 94 149 210 91 150 234 253 86 35 183 42 114 123 148 225 115 12 35 210 41 133 52 71 139 109 103 5 219 234 73 206 102 146 179 27 201 85 145 220 19 46 185 125 254 196 139 143 84 237 148 150 223 19 46 35 213 30 103 11 247 168 39 75 159 201 210 111 100 89 69 150 219 92 150 248 108 146 177 214 161 59 126 75 25 40 145 106 142 226 103 151 100 183 159 45 216 94 79 138 6 183 203 184 236 24 169 134 119 59 102 230 155 149 249 102 103 190 141 198 146 93 79 28 141 128 244 44 153 194 28 241 193 67 177 254 44 103 125 189 187 113 216 205 56 141 78 214 113 11 251 228 81 133 169 39 212 69 60 111 42 214 159 229 172 175 39 173 22 147 86 171 145 86 29 105 117 9 145 73 140 135 144 74 178 254 44 103 125 61 105 121 76 90 94 35 173 42 210 18 17 206 128 108 46 155 6 46 123 202 201 22 217 83 78 182 212 147 217 132 201 108 210 200 172 78 15 123 73 85 169 211 185 30 150 172 63 203 89 95 79 90 83 38 173 105 35 173 58 113 77 63 25 205 139 253 141 71 177 165 74 111 59 43 216 86 79 114 17 147 28 222 223 1 37 176 235 8 240 235 148 0 63 195 94 3 42 213 127 77 213 6 172 72 35 17 163 165 196 206 215 245 177 153 198 78 255 193 73 4 80 36 155 144 128 149 100 188 174 139 172 64 1 106 1 252 92 2 56 5 173 88 151 7 241 15 73 35 198 186 25 4 121 247 63 137 96 115 118 163 47 31 89 8 243 138 47 116 91 80 11 93 222 167 106 25 26 71 163 210 143 3 170 23 132 163 82 64 187 170 182 153 186 21 102 219 166 255 96 122 98 171 49 246 91 198 56 187 213 142 193 49 166 118 4 223 50 91 93 177 209 13 166 161 110 100 55 182 156 252 19 27 114 139 100 241 221 191 230 223 150 82 252 149 44 84 45 128 158 114 128 112 203 152 250 204 21 205 178 148 169 73 169 114 186 110 121 216 126 181 202 233 186 233 7 173 60 149 3 74 119 229 99 221 162 67 91 5 135 26 114 139 74 245 152 59 221 252 219 82 136 103 92 33 152 43 16 147 231 34 107 149 111 142 83 38 68 215 241 127 110 179 85 130 91 215 69 110 27 94 172 233 248 64 53 157 87 218 41 159 35 254 235 98 82 50 125 61 112 252 28 50 105 41 141 197 18 231 185 109 214 142 185 58 225 239 69 0 9 74 45 106 85 25 128 202 157 231 182 0 218 74 177 54 134 235 152 140 77 128 177 85 173 241 91 182 107 72 190 157 21 223 237 120 226 133 146 253 247 226 173 110 104 78 13 87 121 35 211 104 50 14 39 243 152 222 78 19 110 75 28 143 184 56 82 79 98 134 109 11 140 39 220 148 235 186 217 27 78 172 143 167 123 186 108 95 18 235 227 186 178 118 166 172 15 30 216 146 54 182 82 241 18 30 158 215 243 225 95 73 227 121 167 155 127 219 138 192 140 231 188 201 204 161 154 252 246 240 200 183 36 101 149 57 207 109 83 214 144 222 195 112 45 81 150 82 197 124 203 135 134 231 169 152 109 226 111 158 138 141 157 177 53 54 114 84 204 117 231 117 55 81 177 201 20 127 149 32 160 90 123 101 3 178 59 221 252 219 82 131 199 177 26 124 228 229 118 248 122 157 119 139 201 114 222 227 51 211 27 231 200 38 105 176 25 20 56 234 178 175 8 70 39 77 53 185 129 155 218 149 184 103 109 95 69 90 116 191 219 185 190 217 239 166 42 7 166 164 10 71 84 212 140 137 152 54 252 189 166 39 202 48 21 248 134 148 3 85 224 93 92 160 207 246 39 86 224 143 103 221 239 15 175 111 58 123 251 248 231 39 82 156 125 45 162 169 220 168 52 199 244 0 177 35 173 51 235 236 189 128 173 255 87 187 209 90 180 213 213 12 248 213 53 83 251 30 62 135 176 6 63 225 186 9 61 197 197 131 117 45 216 162 211 175 67 123 182 224 175 1 91 240 27 220 82 234 170 91 73 235 181 145 246 7 42 45 191 226 127 104 122 102 207 71 169 61 95 210 172 35 184 47 190 239 23 216 34 105 239 228 217 133 187 228 250 95 196 103 254 27 236 109 105 142 212 142 33 117 36 236 52 114 59 204 204 158 143 233 1 146 23 220 111 142 59 32 223 251 161 246 127 8 69 254 43 29 57 164 201 102 19 202 78 170 142 212 147 95 233 110 240 72 60 234 84 27 107 255 203 100 196 143 123 0 215 193 180 194 39 104 113 246 106 109 154 76 116 69 53 62 67 170 187 189 202 69 119 75 204 26 156 195 86 222 243 89 70 79 142 224 186 172 124 254 156 63 56 154 29 245 119 78 60 217 54 61 73 201 80 212 142 94 50 159 46 150 100 75 146 204 19 184 222 4 67 55 122 80 3 77 77 38 44 46 64 135 213 45 124 146 72 60 126 107 221 123 58 238 83 124 132 149 57 98 155 158 110 249 73 251 45 119 127 249 10 137 118 117 248 227 178 2 26 55 79 235 152 44 197 103 41 29 59 166 26 187 43 254 24 233 115 178 214 226 90 134 116 20 155 20 150 233 233 74 237 153 64 203 85 218 243 60 150 45 211 130 203 220 86 203 247 249 148 158 85 240 59 207 78 32 171 76 230 174 188 149 213 91 232 89 145 132 213 33 245 149 226 51 164 250 204 220 25 158 81 202 25 223 139 4 18 209 186 20 167 206 247 60 185 29 50 10 255 15 206 241 27 156 165 75 189 119 74 209 239 37 239 197 167 208 174 11 232 245 236 129 104 239 64 38 31 72 47 47 97 93 90 219 207 96 255 19 54 93 148 95 245 113 138 135 119 82 76 76 196 93 129 179 31 114 206 238 178 25 108 13 59 151 102 103 189 97 231 134 157 27 118 110 216 121 3 236 60 160 234 184 95 27 118 46 205 206 78 195 206 13 59 55 236 220 176 243 26 217 121 43 102 231 207 116 61 236 73 13 67 151 101 104 183 97 232 134 161 27 134 110 24 122 3 12 157 202 72 55 12 93 154 161 141 134 161 27 134 110 24 186 97 232 53 50 244 35 206 208 191 144 94 253 2 215 248 85 51 27 142 46 205 209 118 195 209 13 71 55 28 221 112 244 6 188 232 20 71 55 12 93 154 161 173 134 161 27 134 110 24 186 97 232 165 25 90 33 201 63 89 229 157 113 71 56 187 169 188 107 56 187 225 236 134 179 213 156 157 72 103 25 206 190 127 149 119 119 133 157 155 202 187 134 157 27 118 110 216 121 19 236 124 127 42 239 238 10 59 55 149 119 13 59 55 236 220 176 243 58 217 249 126 86 222 221 21 134 110 42 239 26 134 110 24 186 97 232 77 48 244 253 170 188 187 43 12 221 84 222 53 12 221 48 116 195 208 235 100 232 251 90 121 119 87 56 186 169 188 107 56 186 225 232 134 163 55 225 69 223 175 202 187 187 194 208 77 229 93 195 208 13 67 55 12 189 60 67 119 96 47 148 127 138 7 164 113 66 174 127 53 216 57 128 158 227 107 54 252 78 224 46 189 149 176 115 49 99 201 136 187 82 70 120 25 166 120 154 185 178 90 91 145 245 77 5 79 138 163 22 85 114 167 247 101 143 98 77 250 160 87 128 195 188 61 177 160 37 206 70 180 77 104 208 78 70 135 170 106 219 51 174 109 201 91 15 222 100 246 186 143 218 23 72 186 176 110 237 251 15 173 181 38 221 147 35 194 251 164 123 79 185 238 165 25 91 246 70 191 226 218 135 117 197 224 181 220 139 154 53 75 146 95 190 47 42 87 253 54 21 107 141 39 186 74 79 212 148 174 208 120 162 119 221 19 53 36 221 91 15 63 111 39 124 10 12 157 106 231 18 12 141 239 69 63 167 254 124 247 25 90 102 221 134 161 27 134 110 24 186 97 232 187 147 43 120 146 240 169 54 41 228 232 103 153 222 191 67 200 179 87 21 94 100 162 184 33 221 215 57 109 75 31 241 3 254 214 224 236 49 196 112 58 108 245 137 123 167 196 217 54 233 138 224 108 140 236 2 248 141 128 167 69 220 133 123 123 240 61 130 254 56 129 253 179 156 253 223 112 165 54 160 21 17 150 76 55 223 0 106 151 164 159 168 205 191 195 247 171 24 105 100 135 127 199 152 61 160 43 239 224 223 204 89 191 210 38 37 115 2 235 145 229 34 236 211 45 120 160 5 146 85 249 2 80 114 164 125 194 5 154 78 58 90 67 75 182 51 115 67 151 139 242 29 110 169 35 178 209 104 201 91 240 107 195 254 66 63 44 248 52 38 13 153 196 125 203 35 47 32 34 77 66 203 190 92 148 239 172 204 194 52 81 126 158 23 169 210 150 58 186 247 40 115 54 177 173 186 222 153 164 83 30 232 26 244 110 210 49 228 27 100 160 250 122 87 228 197 216 27 245 98 92 73 178 139 61 12 185 125 101 173 126 158 189 171 111 245 215 163 133 106 189 201 106 224 54 244 146 9 248 183 159 169 149 59 169 251 101 186 247 101 122 222 122 13 157 155 128 126 57 160 59 168 113 62 197 42 200 121 19 144 150 28 191 120 49 55 33 39 34 27 78 224 63 122 55 254 70 108 225 122 100 144 197 175 26 246 223 193 93 94 198 94 47 215 94 237 31 114 15 168 237 167 88 128 50 162 59 166 94 206 172 143 15 216 27 25 62 192 237 19 56 139 78 190 140 206 229 136 50 116 72 102 155 144 205 115 104 195 60 18 111 8 233 15 128 223 251 152 7 230 99 167 80 66 171 204 81 235 209 133 106 242 172 166 43 91 16 31 9 206 250 84 91 35 144 225 109 216 30 145 62 48 207 196 0 28 28 69 111 53 111 181 183 126 13 251 161 12 47 41 50 121 195 121 247 13 96 43 94 67 174 150 236 115 233 184 183 92 30 243 71 254 141 152 203 41 188 106 114 244 88 99 47 33 204 63 246 105 78 139 47 225 12 227 92 237 205 187 94 250 40 213 213 182 169 207 50 125 47 115 165 39 169 253 203 95 37 255 158 166 185 185 130 162 123 74 142 42 127 79 249 87 202 187 167 226 171 60 85 92 101 145 86 61 83 94 169 140 70 201 104 136 43 102 245 169 232 206 202 29 241 84 217 194 197 90 187 5 123 34 135 189 131 191 178 6 25 133 123 166 229 34 87 208 167 247 156 199 86 174 88 74 239 45 223 229 38 108 184 138 91 171 177 243 99 216 254 153 242 130 59 233 115 45 205 208 70 204 208 118 195 208 13 67 55 12 221 48 244 95 146 161 243 248 181 170 15 189 71 114 190 38 57 172 38 170 154 198 81 149 117 231 162 170 237 68 178 116 182 15 218 57 197 33 159 102 7 125 0 237 160 63 186 190 121 213 219 195 151 213 190 102 139 89 178 206 116 28 182 22 63 204 36 125 32 45 92 249 57 49 223 191 202 115 62 22 186 186 210 179 174 203 7 153 215 205 106 218 253 40 245 154 232 29 222 186 139 90 30 200 132 50 58 1 247 39 112 36 195 164 186 181 108 22 17 117 62 200 140 72 211 56 19 85 182 77 37 253 126 144 59 170 171 202 179 61 4 28 63 210 152 20 162 250 71 204 141 170 241 191 250 57 199 173 212 248 25 230 11 38 27 207 222 169 228 85 77 230 207 181 67 210 239 127 104 67 26 139 254 76 215 70 158 192 140 195 106 24 110 18 51 156 121 231 24 14 125 201 249 251 46 147 1 250 154 70 165 170 231 155 182 225 136 11 218 55 30 137 146 90 170 174 29 120 4 199 49 38 186 173 236 84 25 93 201 106 223 3 184 26 62 219 116 154 226 152 46 177 40 27 151 101 53 4 117 162 156 136 234 89 116 208 24 100 12 140 104 34 26 31 77 70 200 116 138 114 144 73 110 55 202 89 87 223 87 225 184 8 253 175 121 38 81 60 101 118 135 123 68 187 208 138 143 56 206 84 107 204 8 101 96 82 175 13 168 135 135 176 180 105 4 59 29 111 58 52 74 84 174 254 232 62 73 98 49 166 89 169 252 61 30 161 100 50 73 190 215 233 5 6 108 139 40 238 176 227 88 95 248 148 119 41 214 95 15 246 9 118 197 24 63 165 138 16 172 100 64 102 221 17 91 151 200 130 35 238 22 141 22 219 164 243 38 157 31 117 222 166 241 17 135 188 30 68 23 165 227 192 54 159 172 34 74 34 34 228 195 141 224 254 13 33 41 238 92 196 94 151 74 43 242 5 180 48 107 71 190 205 61 250 159 176 12 180 139 140 127 245 5 234 216 6 164 94 44 205 69 154 128 242 20 54 107 121 77 112 0 51 143 120 204 133 251 116 201 14 69 124 244 210 166 190 24 145 38 24 36 113 135 234 126 176 90 19 99 42 143 246 8 230 124 221 245 104 194 127 146 188 19 15 39 145 101 126 133 143 156 119 203 59 195 124 221 141 191 33 77 40 146 102 177 38 108 107 191 104 248 164 155 119 43 208 2 159 207 199 65 185 187 177 111 107 18 15 227 92 157 49 105 1 234 137 75 113 209 152 52 32 162 72 201 36 111 24 173 225 102 114 174 255 230 119 93 149 11 158 43 143 188 77 30 200 151 96 177 228 183 226 189 119 72 154 151 181 234 174 179 182 215 252 203 217 94 21 138 197 184 127 173 29 192 25 62 83 164 116 78 156 177 10 75 156 30 237 208 227 209 14 235 206 73 225 63 41 7 154 190 251 116 63 18 118 246 115 92 125 244 13 180 241 7 138 112 242 127 221 141 248 183 139 164 150 149 250 67 94 177 118 73 53 213 239 227 121 183 217 181 213 37 29 146 252 48 198 195 42 53 22 241 177 57 13 243 17 95 235 79 216 223 100 4 203 160 190 157 93 187 20 223 97 133 159 73 255 29 94 167 229 147 205 107 205 245 52 81 27 248 231 194 63 31 203 50 146 120 6 103 124 79 179 36 216 150 157 184 246 176 46 243 101 237 143 125 135 237 207 119 228 89 36 247 254 134 198 129 62 209 108 232 171 146 243 71 190 45 56 199 42 184 179 232 252 42 239 200 217 136 135 179 72 107 228 124 207 9 141 81 96 230 80 244 255 93 242 49 119 146 45 181 245 109 74 154 163 83 157 239 152 242 108 33 233 148 69 62 110 200 109 46 254 117 104 46 151 168 28 70 207 118 74 254 49 198 194 155 241 113 3 186 235 55 128 157 184 235 114 82 252 86 121 228 103 190 148 103 14 228 101 106 215 197 63 121 178 148 249 135 205 59 204 86 136 139 231 94 28 82 139 63 220 131 121 134 217 218 235 230 237 37 183 53 211 112 211 53 250 114 149 196 226 153 134 134 116 196 162 153 134 24 159 86 157 9 32 207 127 252 179 206 53 204 62 143 165 153 107 184 56 3 173 158 53 161 102 97 241 124 184 19 58 63 122 24 117 198 215 27 38 254 107 50 241 234 102 228 213 153 45 117 55 152 120 126 222 233 159 151 139 155 121 223 171 225 226 47 161 125 23 228 251 79 64 143 196 172 181 244 186 58 209 119 68 227 55 44 211 17 242 60 191 159 170 178 102 51 184 117 224 220 219 157 193 189 174 89 107 105 252 178 124 23 198 91 230 99 38 236 27 34 2 183 164 39 62 108 81 191 255 163 224 40 108 17 44 75 200 247 57 245 42 118 7 151 164 235 168 167 59 75 74 125 66 99 58 45 178 171 62 73 221 165 81 93 63 35 245 49 141 241 249 25 169 227 255 136 246 221 204 104 223 186 42 128 22 163 122 187 186 128 182 253 35 181 159 141 14 139 170 56 49 95 178 79 54 235 138 248 235 173 198 158 178 134 246 227 154 238 35 125 247 117 244 3 179 111 83 242 44 34 242 177 48 71 58 165 35 132 126 4 196 10 56 151 223 165 218 56 157 252 50 198 33 62 108 193 241 195 172 126 252 157 48 15 83 150 89 246 177 150 241 4 214 53 143 177 10 206 248 115 60 4 17 206 250 187 163 235 155 246 94 239 252 38 138 198 173 137 99 249 179 110 246 91 63 150 244 87 148 251 124 147 60 31 34 246 177 163 57 31 251 44 119 203 160 115 58 190 209 103 221 81 251 28 23 251 93 90 12 143 207 111 76 248 54 58 191 49 102 221 65 135 118 25 12 216 182 67 182 120 133 139 217 232 85 251 250 134 93 248 1 152 31 132 231 92 123 15 183 243 211 245 205 203 62 236 227 233 179 67 190 28 13 127 129 243 233 240 225 8 238 98 116 212 57 191 105 69 19 59 162 231 77 140 94 117 87 115 162 217 254 171 254 245 77 247 152 218 190 215 27 224 162 223 163 111 253 93 2 185 119 130 77 239 211 38 56 73 127 196 191 3 18 198 108 183 223 99 139 33 222 244 238 238 30 125 219 237 208 98 8 167 153 194 158 29 60 224 0 79 170 207 126 236 255 124 126 227 184 176 28 178 175 167 108 209 199 35 14 186 71 120 154 31 135 184 79 0 203 125 246 117 132 27 127 28 182 9 216 94 159 16 61 193 198 29 12 123 184 174 55 60 195 69 135 45 122 67 146 192 222 240 24 15 219 223 27 226 205 156 188 30 226 183 222 144 190 29 142 142 241 36 135 35 70 2 29 162 39 84 201 223 105 73 197 89 179 87 93 106 226 171 99 106 255 104 64 167 131 35 113 241 170 179 75 39 239 190 130 19 104 179 147 99 251 250 6 254 156 223 184 51 90 68 108 97 176 133 46 45 96 217 197 253 65 125 156 25 45 128 142 247 79 246 112 57 218 237 209 229 250 47 233 226 216 80 216 120 124 2 7 28 159 116 232 106 179 254 193 9 25 178 190 22 144 139 189 3 116 126 116 220 199 187 238 31 245 216 2 119 253 31 160 12 131 138 102 25 129 136 199 49 248 60 13 171 115 138 241 169 196 128 149 163 4 84 130 235 81 135 243 168 211 57 16 210 245 134 199 0 247 81 143 193 253 26 176 239 237 190 134 206 247 211 1 174 56 27 144 22 244 120 191 121 9 135 141 169 7 7 208 247 17 202 94 143 110 234 120 72 251 29 239 209 105 58 71 36 146 189 30 118 210 125 60 229 222 79 184 126 191 135 215 154 205 94 28 193 61 191 96 59 205 102 115 215 211 249 245 48 194 100 79 75 197 217 7 44 103 59 201 92 81 103 87 52 138 175 120 116 124 16 175 120 117 218 165 58 125 182 72 87 237 27 186 57 163 14 102 152 172 131 185 30 235 96 94 182 127 133 190 143 129 240 104 23 111 165 220 169 77 221 98 167 246 28 118 106 163 165 60 117 107 58 241 103 252 179 239 59 147 138 151 209 179 23 153 140 149 23 25 91 118 32 46 226 78 156 72 124 30 27 134 39 62 79 244 32 185 199 242 2 123 152 40 6 105 110 117 81 205 93 107 118 48 232 92 223 28 156 190 194 219 59 56 125 77 139 33 124 179 92 88 190 102 75 102 26 92 250 129 35 58 96 174 15 58 116 205 131 206 79 184 73 215 241 63 124 59 68 106 233 188 192 11 157 14 137 234 79 135 187 212 113 251 157 61 184 236 0 136 193 159 189 24 28 51 194 223 75 45 6 63 3 61 142 253 208 243 91 115 176 31 117 64 127 44 107 118 120 52 2 105 27 254 108 176 63 140 110 90 230 108 48 60 195 179 183 15 246 176 25 248 3 172 9 247 220 78 219 175 49 217 175 54 197 187 96 225 227 49 155 113 156 169 191 164 124 63 218 77 136 130 133 173 106 239 18 95 183 119 161 205 214 212 155 181 59 71 231 55 246 172 125 124 140 228 220 62 134 213 193 172 61 220 167 157 134 68 69 109 102 180 218 237 30 91 252 68 139 189 19 126 2 102 245 218 3 50 18 237 46 225 211 238 146 185 107 159 176 149 67 224 104 43 156 181 153 157 108 143 216 217 71 67 118 145 99 118 62 182 56 218 197 86 158 96 171 128 18 78 13 108 78 231 212 196 211 116 78 45 90 116 13 180 175 157 174 201 22 22 46 102 37 176 121 54 135 205 64 240 131 134 115 213 150 68 200 184 255 8 9 237 17 217 147 37 180 199 136 236 218 250 99 44 137 142 177 94 253 73 208 89 82 127 100 140 170 104 208 45 99 180 205 49 218 5 124 216 51 245 47 105 164 243 109 252 108 82 134 85 130 97 17 58 216 238 20 60 174 205 240 161 237 11 241 241 236 12 62 122 6 31 203 204 34 52 89 178 143 153 12 33 147 33 100 51 132 108 134 144 61 27 246 127 97 82 29 162 9 13 199 176 226 148 221 198 112 120 74 43 170 96 248 152 99 136 241 211 191 52 54 99 102 145 158 89 166 74 209 138 129 52 124 14 165 57 14 43 64 25 122 12 74 90 191 54 48 77 191 28 152 79 57 152 123 20 184 179 201 85 24 6 225 36 138 63 226 224 46 81 74 150 134 252 80 8 166 193 123 173 97 149 166 125 67 183 213 221 86 47 232 182 142 206 144 36 62 64 36 163 160 116 207 53 189 210 88 90 158 201 176 180 140 73 101 236 138 59 116 107 194 160 243 66 134 220 116 69 14 133 94 100 46 205 154 100 87 14 178 186 0 253 12 253 148 101 141 210 0 217 14 3 200 24 75 186 21 113 136 116 134 81 88 212 81 81 45 22 235 22 17 98 10 39 100 83 196 137 148 106 13 64 205 235 86 220 81 235 118 208 1 101 159 176 131 190 47 84 51 240 153 87 170 103 171 53 170 235 209 179 47 57 70 24 85 95 41 209 105 73 42 38 57 29 69 126 25 30 154 193 199 188 117 124 134 253 54 99 255 97 123 142 244 31 199 120 177 26 183 144 178 170 23 146 54 237 197 233 156 63 74 209 61 199 203 48 67 37 98 18 225 183 194 234 42 101 187 12 178 128 99 102 77 236 210 182 179 126 167 132 11 154 78 192 128 68 83 83 29 201 62 27 229 197 113 203 74 72 70 118 9 203 25 9 203 137 254 68 73 32 99 110 227 64 146 79 178 122 28 17 190 161 240 70 144 220 152 91 34 62 32 178 204 47 17 31 6 149 33 126 152 13 46 170 192 75 54 99 97 56 65 234 92 25 223 41 247 76 240 96 4 216 119 87 13 240 0 112 53 167 83 134 153 108 52 138 49 171 227 32 171 73 209 116 10 29 228 208 172 222 199 99 247 216 49 25 116 12 195 114 216 25 211 210 212 24 43 158 25 101 160 219 138 161 187 164 244 218 135 184 154 188 90 108 150 13 205 40 68 168 153 251 40 211 145 77 143 123 41 20 148 148 68 203 50 85 104 185 12 45 151 161 229 178 174 204 8 16 63 140 35 169 43 163 141 25 28 86 192 241 97 236 173 4 218 111 11 114 36 30 67 209 103 40 146 139 150 66 81 95 22 69 155 161 200 84 76 9 163 19 48 28 205 2 28 29 157 167 1 124 158 7 240 57 146 92 241 92 174 121 174 157 7 38 91 147 2 19 156 206 106 74 57 162 135 83 133 218 111 85 122 113 41 173 44 231 61 171 213 146 58 51 26 234 241 202 249 143 12 12 245 97 1 228 32 109 105 152 237 89 172 158 29 234 204 111 137 255 178 233 170 212 150 210 157 94 153 69 184 3 233 42 163 102 186 42 31 165 175 149 40 117 40 177 199 6 192 239 35 78 230 134 112 26 209 112 255 199 123 139 147 181 114 156 182 98 156 62 80 217 230 230 18 195 69 60 230 135 118 205 32 141 155 4 157 65 163 51 104 116 6 141 206 160 209 75 66 243 84 169 66 124 52 166 186 250 100 185 94 223 252 232 75 17 25 57 12 33 135 33 228 44 69 217 135 188 8 161 56 122 189 171 35 12 28 37 176 216 4 147 195 96 114 24 76 1 131 41 96 48 5 74 223 191 75 101 178 56 220 50 66 174 81 40 211 37 184 95 108 114 27 126 194 2 131 18 174 88 21 215 193 112 166 101 6 27 116 101 144 95 185 203 169 29 7 238 206 122 12 42 143 57 255 108 172 33 63 49 82 140 221 33 47 125 78 30 69 38 99 135 94 35 165 45 49 2 34 105 234 25 71 150 197 77 115 142 172 178 51 146 46 150 205 136 7 107 197 174 60 84 162 71 38 85 226 249 138 38 231 223 66 85 143 196 172 117 46 109 37 49 122 141 100 210 218 244 44 237 215 39 153 36 244 84 41 147 228 77 43 224 249 48 227 230 191 95 237 152 123 201 129 44 201 52 110 160 155 198 96 33 142 105 176 190 228 96 157 226 144 65 252 144 59 17 158 95 242 210 254 69 222 85 80 183 56 1 163 153 69 48 89 82 104 41 66 33 145 106 35 133 158 11 44 141 18 233 12 92 75 30 22 7 203 244 88 236 67 145 206 46 5 108 164 124 174 203 34 31 27 151 148 78 92 136 225 243 20 134 31 168 164 118 135 74 123 47 104 250 204 60 207 21 35 26 122 165 33 197 76 86 193 208 76 24 48 80 77 14 106 104 74 249 181 9 207 17 49 195 189 32 194 4 213 101 176 122 28 87 143 1 27 122 12 216 144 3 27 122 114 188 142 31 6 226 67 166 131 179 77 241 7 145 96 234 15 120 76 138 117 172 44 197 57 156 79 215 169 68 33 146 116 27 129 223 154 148 130 95 74 209 197 232 23 142 187 230 228 74 108 238 247 217 12 123 112 144 8 123 90 90 180 36 164 25 138 74 164 201 136 35 192 164 232 3 166 248 101 225 125 20 195 203 38 97 189 167 50 250 18 254 143 24 212 22 197 22 70 48 45 129 111 25 255 199 145 24 35 203 172 200 183 67 170 148 203 7 215 181 171 129 43 128 51 162 56 71 66 0 154 139 153 226 91 117 14 153 151 99 148 227 223 86 77 254 45 101 166 120 106 47 10 164 26 1 14 167 61 206 208 175 173 162 95 1 167 204 19 14 143 227 28 30 200 193 50 91 116 193 178 80 3 6 232 112 192 169 153 127 71 160 141 72 73 201 34 194 233 243 89 30 243 234 138 115 76 251 176 254 95 139 199 146 178 227 228 181 179 207 69 177 95 5 31 32 171 168 94 221 196 193 60 50 255 165 66 134 158 4 250 158 230 239 124 79 207 4 61 167 200 112 225 32 199 106 49 91 173 223 180 66 204 132 153 79 158 100 205 186 110 122 79 85 56 67 97 75 174 139 25 84 79 44 84 112 48 203 155 152 82 64 81 63 69 254 155 183 45 232 121 226 150 33 91 150 131 84 152 235 131 184 240 128 166 243 172 167 124 56 44 239 166 87 170 54 147 188 79 78 127 1 3 50 96 64 122 115 142 58 119 61 211 213 102 25 64 211 0 62 225 0 190 100 243 177 249 208 100 250 217 232 95 197 65 118 153 236 141 84 183 23 199 213 229 18 56 56 196 88 81 27 99 159 135 92 166 165 134 37 149 72 198 99 230 201 64 27 229 42 196 135 193 105 202 108 235 204 154 72 74 90 30 227 114 131 70 166 167 210 83 150 0 90 33 194 98 204 200 18 227 190 78 121 128 213 49 165 165 26 51 162 158 157 215 247 171 193 88 37 40 143 184 247 24 101 189 199 40 40 1 98 153 112 115 217 4 144 90 69 77 21 130 44 42 79 71 65 241 96 155 92 33 41 248 114 192 61 236 15 218 59 137 47 217 131 16 112 150 20 6 56 231 42 12 13 199 102 24 186 89 12 131 10 16 186 74 61 84 58 53 108 232 183 142 87 35 180 48 75 152 38 87 67 90 90 180 100 189 215 229 221 183 60 104 66 225 216 32 73 113 184 98 169 58 173 84 38 169 182 46 122 121 239 154 248 118 30 171 177 189 120 208 60 142 85 164 65 115 36 88 194 139 125 176 216 7 230 95 123 83 238 95 227 135 190 192 112 40 66 191 129 224 72 150 244 40 135 169 168 11 236 210 51 83 21 136 142 237 76 112 45 180 175 74 116 93 104 175 9 67 85 49 130 58 254 227 152 90 42 76 67 79 29 176 248 60 187 230 243 162 14 223 101 153 34 10 75 196 24 121 59 9 83 196 152 185 148 29 207 7 49 137 166 217 163 137 233 65 37 133 234 233 212 245 125 42 88 20 22 243 165 114 111 21 170 176 32 62 84 170 39 174 239 242 245 93 182 62 134 146 172 117 139 27 235 22 211 80 1 41 41 232 33 11 13 171 117 246 215 132 103 177 117 201 90 232 82 117 131 85 124 73 174 153 99 229 96 168 20 73 43 173 75 94 98 34 175 66 38 91 129 201 85 145 34 230 156 185 32 248 196 198 247 90 242 126 20 225 150 63 224 32 90 90 167 70 121 91 171 204 60 154 50 33 141 149 117 113 178 0 130 170 40 82 17 229 210 102 166 167 200 5 31 50 27 93 5 167 71 9 78 169 119 146 23 34 166 52 48 229 166 30 9 200 140 197 58 23 42 13 140 49 85 145 161 177 81 200 182 99 15 230 45 123 12 3 61 202 65 46 189 151 97 115 235 142 57 24 229 71 1 227 88 197 114 171 23 5 10 71 112 81 212 119 40 92 231 195 121 215 121 17 116 207 99 155 123 65 179 62 170 14 51 168 71 8 151 87 61 170 39 79 141 51 4 74 111 90 40 95 153 76 172 108 137 149 202 39 60 154 65 60 88 48 239 192 148 237 192 236 81 89 23 37 166 159 174 162 3 59 229 19 176 18 134 254 98 163 33 202 83 37 119 122 69 221 87 232 96 159 61 243 131 70 92 215 172 131 115 149 148 150 82 9 125 47 59 216 21 41 173 134 122 252 80 42 240 13 213 20 168 242 7 229 52 78 50 204 197 189 152 116 30 155 169 43 220 96 203 97 102 217 173 166 173 79 226 200 249 3 133 50 111 233 197 129 159 22 102 204 140 213 21 196 21 217 105 79 57 69 14 243 32 41 196 205 42 35 48 121 90 203 252 155 225 124 16 184 8 193 167 41 4 255 69 73 156 29 202 84 84 198 144 18 131 21 38 187 198 56 218 139 67 233 218 227 3 24 43 151 48 66 148 132 8 91 138 130 116 42 242 161 21 226 131 24 126 233 115 103 18 111 145 13 200 166 188 75 121 228 112 145 20 190 227 82 96 143 126 10 169 250 245 19 127 13 217 185 198 30 88 31 19 76 117 58 97 83 169 202 42 182 146 77 178 138 45 121 83 148 106 155 231 146 74 30 168 197 115 68 22 79 173 195 146 184 132 139 39 109 223 250 92 56 153 100 38 73 43 53 84 230 101 71 202 208 223 159 117 123 157 235 155 212 83 168 30 242 167 80 181 41 52 250 20 7 76 81 60 63 3 215 158 41 215 214 123 246 84 200 158 61 5 139 131 184 21 207 161 13 97 156 242 159 112 129 95 165 70 85 62 197 116 23 242 54 92 240 103 236 188 140 7 8 186 7 47 224 222 240 161 69 112 253 131 35 248 124 208 199 199 156 116 217 83 76 116 250 153 165 54 25 98 19 127 196 9 110 123 141 219 244 229 207 99 212 60 133 216 132 63 41 57 61 229 114 26 240 167 142 177 55 86 125 38 20 146 119 131 69 124 54 136 106 159 179 18 251 212 147 104 192 36 26 52 18 173 35 81 241 252 90 246 6 118 142 151 36 81 245 62 103 37 246 169 39 81 139 73 212 48 26 145 86 17 233 221 120 164 95 55 157 18 99 13 226 207 208 73 113 133 184 112 122 203 89 238 150 122 77 50 89 147 204 70 137 170 40 209 227 152 23 206 105 160 44 203 241 91 49 127 199 111 144 148 228 150 222 178 84 239 183 26 185 85 145 219 163 12 159 99 24 145 78 41 164 57 90 108 59 43 216 86 79 114 54 147 156 221 72 174 138 228 158 112 201 237 243 39 94 124 164 106 167 180 252 158 112 25 169 246 56 91 184 71 61 89 250 76 150 126 35 203 42 178 220 230 178 100 79 252 236 208 29 167 223 32 190 205 229 52 191 253 108 193 246 122 82 52 184 93 198 101 199 72 53 188 219 49 51 223 172 204 55 59 243 109 52 150 236 122 226 104 176 247 215 39 83 152 35 62 120 40 214 159 229 172 175 119 55 14 187 25 167 209 201 58 110 97 159 60 170 48 245 132 186 136 231 77 197 250 179 156 245 245 164 213 98 210 106 53 210 170 35 173 46 123 55 72 140 135 144 74 178 254 44 103 125 61 105 121 76 90 94 35 173 42 210 18 17 206 128 108 46 155 6 46 123 202 201 22 217 83 78 182 212 147 217 132 201 108 210 200 172 78 15 123 169 177 7 155 203 61 44 89 127 150 179 190 158 180 166 76 90 211 70 90 117 226 154 126 50 154 23 251 27 143 98 75 149 222 118 86 176 173 158 228 34 38 57 188 191 3 74 96 215 17 224 215 41 1 126 134 189 6 84 170 255 154 170 13 88 145 70 34 70 75 137 157 175 235 99 51 141 157 254 131 147 8 160 72 54 33 1 43 201 120 93 23 89 129 2 212 2 248 185 4 112 10 90 177 46 15 226 31 146 70 140 117 51 8 242 238 127 18 193 230 236 70 95 62 178 16 230 21 95 232 182 160 22 186 188 79 213 50 52 142 70 165 31 7 84 47 136 175 109 78 128 118 85 109 51 117 43 204 182 77 255 193 244 196 86 99 236 183 140 113 118 171 29 131 99 76 237 8 190 101 182 186 98 163 27 76 67 221 200 110 108 57 249 39 54 228 22 201 226 187 127 205 191 45 165 248 43 89 168 90 0 61 229 0 225 150 49 245 153 43 154 101 41 83 147 82 229 116 221 242 176 253 106 149 211 117 211 15 90 121 42 7 148 238 202 199 186 69 135 182 10 14 53 228 22 149 234 49 119 186 249 183 165 16 207 184 66 48 87 32 38 207 69 214 42 223 28 167 76 8 127 119 71 94 179 85 130 91 215 69 110 27 94 172 233 248 64 53 157 87 218 41 159 35 254 235 98 82 50 125 61 112 252 28 50 105 41 141 197 18 231 185 109 214 142 185 58 225 239 69 0 9 74 45 106 85 25 128 202 157 231 182 0 218 74 177 246 71 122 57 222 101 10 24 91 213 26 191 101 187 134 228 219 89 241 221 142 39 94 40 217 127 47 222 234 134 230 212 112 149 55 50 141 38 227 112 50 143 233 237 52 225 182 196 241 136 139 35 245 36 102 216 182 192 120 178 183 28 229 89 31 79 247 116 217 190 36 214 199 117 101 237 76 89 31 60 176 37 109 108 165 226 37 60 60 175 231 195 191 146 198 243 78 55 255 182 21 129 25 207 121 147 153 67 53 249 237 225 145 111 73 202 42 115 158 219 166 172 33 189 135 225 90 162 44 165 138 249 150 15 13 207 83 49 219 196 223 60 21 27 59 99 107 108 228 168 152 120 189 152 90 197 38 83 252 85 130 128 106 237 149 13 200 238 116 243 111 75 13 30 199 106 240 145 151 219 225 235 117 222 45 38 203 121 143 207 76 111 156 35 155 164 193 102 80 224 168 203 190 34 24 157 52 213 228 6 110 106 87 226 158 181 125 21 105 209 253 110 231 250 38 245 182 222 39 36 214 55 218 17 21 53 103 222 170 30 87 210 78 227 249 57 129 118 85 227 157 188 45 218 234 210 251 119 241 93 187 248 206 102 124 107 177 25 191 147 23 159 215 226 193 186 22 189 82 19 127 29 218 179 5 127 13 216 130 223 102 210 219 137 147 183 191 143 180 63 80 61 115 222 218 254 40 181 231 75 154 95 116 165 189 205 125 147 239 163 212 83 10 119 201 201 79 222 223 139 111 177 183 52 71 106 71 242 22 251 226 118 60 166 71 69 94 112 15 89 245 198 250 212 91 223 165 35 135 52 173 108 66 121 72 213 145 121 111 121 127 68 71 226 81 167 218 88 251 95 38 163 248 189 213 17 37 16 62 73 239 35 126 12 120 125 166 124 55 86 243 12 169 194 246 42 247 174 182 196 252 192 57 108 229 61 151 121 71 242 147 148 12 69 149 232 37 243 222 98 73 182 36 201 60 129 235 77 232 85 174 248 72 6 154 132 76 88 92 104 147 156 22 62 73 36 30 191 159 238 61 29 247 41 62 34 251 142 236 109 122 142 37 190 39 59 111 127 249 10 137 118 117 248 131 177 2 26 33 79 235 152 44 197 103 41 29 59 166 106 186 43 254 192 232 115 178 203 226 90 134 116 20 155 254 149 233 211 74 237 193 119 101 171 180 231 121 44 91 166 5 151 185 173 150 239 243 41 61 149 224 119 158 135 64 254 152 204 93 121 43 171 183 208 179 34 9 171 67 234 43 197 103 72 245 153 185 51 60 163 228 50 190 1 9 36 2 252 21 16 235 44 106 135 140 194 122 222 207 253 56 197 184 59 41 206 157 101 222 165 190 152 157 183 56 59 255 66 122 245 11 92 227 215 134 161 75 51 180 213 48 116 195 208 13 67 55 12 189 70 134 126 52 207 208 154 217 112 116 105 142 182 27 142 110 56 186 225 232 134 163 215 200 209 34 199 49 160 122 229 198 131 46 207 206 78 195 206 13 59 55 236 220 176 243 6 114 28 3 144 53 94 15 123 82 195 208 101 25 218 109 24 186 97 232 134 161 27 134 94 35 67 127 195 25 122 8 231 22 15 241 98 251 83 229 128 198 94 225 216 112 118 89 206 54 27 206 110 56 187 225 236 134 179 55 224 85 167 56 187 97 232 210 12 109 52 12 221 48 116 195 208 13 67 47 205 208 10 73 222 219 202 59 227 142 176 115 83 121 215 176 115 195 206 13 59 171 217 57 145 206 50 236 124 63 43 239 238 10 67 55 149 119 13 67 55 12 221 48 244 58 25 250 190 86 222 221 21 142 110 42 239 26 142 110 56 186 225 232 117 114 244 253 171 188 187 43 236 220 84 222 53 236 220 176 115 195 206 155 200 113 220 175 202 187 187 194 208 77 229 93 195 208 13 67 55 12 189 78 134 254 179 84 222 221 21 206 110 42 239 26 206 110 56 187 225 236 77 120 213 247 171 242 238 174 48 116 83 121 215 48 116 195 208 13 67 47 207 208 29 216 11 229 159 226 129 248 177 207 140 161 147 231 208 191 201 236 85 157 173 3 232 73 190 102 195 239 4 238 218 91 9 91 23 51 152 44 129 64 242 109 151 97 142 167 153 43 231 105 111 75 193 154 226 152 69 117 221 233 125 217 131 89 147 30 41 143 75 166 247 157 183 46 22 160 231 108 68 247 132 62 237 100 116 165 170 238 137 17 17 206 125 247 82 215 92 41 247 181 110 93 67 143 195 92 147 182 121 247 88 219 158 114 109 75 51 182 236 141 126 197 245 13 115 6 224 181 220 139 58 99 75 146 95 190 47 42 71 244 77 149 113 227 137 174 210 19 53 165 43 52 158 232 93 247 68 13 73 247 214 195 207 219 9 159 2 67 167 218 185 4 67 227 123 209 207 169 63 223 125 134 150 89 183 97 232 134 161 27 134 110 24 250 238 228 10 158 36 124 170 77 10 57 250 89 166 247 239 16 242 236 85 133 23 169 184 237 81 102 214 159 216 86 157 169 77 122 177 155 7 189 9 98 26 64 20 89 217 132 95 61 102 106 92 135 251 96 139 4 98 30 113 123 68 49 31 242 181 204 212 249 28 97 111 148 35 92 137 35 22 247 95 185 125 101 251 84 158 54 213 239 83 235 209 67 181 222 212 209 192 237 204 153 150 203 87 57 220 11 136 200 254 163 151 208 130 95 27 246 175 175 133 85 243 85 206 202 52 179 201 87 229 121 168 42 109 169 163 123 207 168 255 94 16 187 100 206 166 253 128 191 53 52 112 172 249 160 5 33 252 69 207 115 74 30 171 77 150 82 104 32 102 182 80 255 34 98 71 166 51 184 55 242 37 234 237 4 246 207 106 224 127 195 149 218 128 85 68 72 50 22 121 3 152 93 18 147 160 45 255 29 190 95 197 56 163 118 253 59 70 236 1 93 121 7 255 102 206 250 149 54 41 153 149 90 143 36 23 97 159 110 193 3 45 144 52 238 11 64 201 145 246 9 23 216 121 226 212 140 150 108 131 38 79 192 191 253 76 56 236 164 122 183 120 145 208 46 241 246 7 216 214 139 121 251 83 13 189 64 102 177 97 123 68 209 138 65 204 100 0 166 246 92 36 227 113 189 8 136 193 144 187 38 240 31 253 28 127 35 122 241 53 236 247 1 153 128 188 158 55 220 234 188 1 244 197 43 142 213 108 242 92 58 238 45 247 88 231 143 252 27 220 151 35 73 79 190 106 114 244 88 99 47 56 203 63 246 105 78 139 47 225 12 227 220 168 34 239 122 233 163 84 87 219 6 121 76 73 95 202 93 233 73 106 255 242 87 201 191 167 105 110 28 82 116 79 201 81 229 239 41 255 74 121 247 84 124 149 167 138 171 44 210 170 103 202 43 149 209 40 25 13 113 197 172 62 21 221 89 185 35 158 42 91 184 88 107 183 96 79 228 195 119 240 87 214 32 163 112 207 180 92 228 186 130 244 158 243 216 202 213 16 233 189 229 187 52 54 96 3 242 248 181 26 75 127 7 44 120 25 199 199 220 19 215 254 33 123 243 181 109 186 5 44 140 236 59 38 95 145 249 146 62 72 212 200 120 149 184 29 53 65 39 187 143 220 237 83 76 62 33 219 46 103 161 214 195 221 207 161 13 243 72 188 33 204 177 47 188 143 99 154 249 126 22 74 104 149 57 106 61 90 81 77 158 213 116 229 81 234 117 129 59 188 221 23 181 236 57 202 213 38 159 37 164 40 55 160 172 163 47 69 26 168 39 65 38 51 73 249 6 26 227 156 74 58 241 32 55 187 167 138 8 31 2 194 31 41 55 129 120 255 81 152 7 170 31 131 108 165 242 40 168 13 147 141 199 153 42 121 85 147 249 22 172 65 59 112 77 154 180 26 22 152 198 44 96 221 57 22 216 78 152 159 206 246 65 59 167 126 243 105 118 208 7 208 14 250 163 235 155 87 189 61 124 13 227 107 182 152 37 235 76 199 97 107 241 195 76 178 23 100 165 86 126 78 204 100 173 242 156 143 133 45 91 233 89 215 163 223 42 221 172 166 221 207 181 67 186 215 127 64 12 133 25 247 207 116 101 212 25 100 203 213 104 251 36 214 118 243 206 105 59 198 29 243 247 93 198 122 125 77 209 103 117 91 185 13 71 92 208 190 113 196 41 181 84 61 66 242 8 142 99 90 121 91 150 181 140 174 84 211 190 47 211 179 54 106 217 81 147 236 168 193 117 231 123 202 220 77 52 87 17 23 139 108 201 237 196 197 235 145 72 22 191 170 118 45 229 51 215 238 233 217 204 132 25 103 38 156 38 51 209 100 38 154 204 68 147 153 248 75 102 38 84 220 154 101 231 7 112 118 156 183 56 141 249 248 107 30 161 138 57 140 59 60 179 177 11 231 248 136 249 237 90 227 170 62 240 171 73 30 85 64 236 28 194 210 166 81 132 52 59 59 52 134 85 174 2 230 62 217 199 197 152 46 146 202 35 106 209 123 62 90 206 42 59 234 100 240 35 170 50 210 73 30 17 161 30 209 184 77 50 182 168 147 36 48 174 191 93 59 185 174 72 92 133 99 22 253 191 199 163 136 12 251 228 123 29 188 13 216 22 17 71 218 241 136 137 136 188 239 146 95 178 30 188 19 236 138 49 222 18 223 161 101 199 116 246 58 149 118 89 180 205 191 28 218 42 20 139 113 255 90 59 128 51 124 166 168 241 156 198 39 119 226 115 172 198 23 215 99 95 220 186 115 82 248 79 178 208 233 187 23 118 250 146 60 129 128 70 151 63 199 21 49 223 64 27 127 32 246 204 255 117 55 98 79 22 73 173 88 234 79 73 110 34 98 94 94 226 14 72 204 35 75 237 194 189 186 100 85 34 30 255 218 212 3 35 202 45 27 84 85 224 80 117 1 86 196 162 247 228 209 30 129 38 231 145 215 37 241 119 169 59 79 75 59 191 142 64 142 195 242 206 48 63 186 239 111 36 31 82 44 205 69 154 48 224 90 142 185 158 85 244 125 100 96 139 42 105 24 3 155 116 126 143 52 1 71 155 28 210 4 236 225 17 233 194 132 124 17 155 216 32 162 222 31 110 68 19 190 137 251 55 222 121 86 142 114 94 235 11 104 97 86 146 223 230 30 253 79 88 6 218 69 102 60 227 11 228 185 141 104 66 145 52 139 53 97 91 251 69 195 25 241 239 86 160 5 62 159 255 134 12 224 198 57 86 147 236 48 206 141 27 19 31 32 99 184 52 250 52 38 46 136 104 60 202 164 172 44 122 254 155 201 198 252 155 223 117 85 13 120 174 60 242 54 165 159 47 193 172 228 31 242 58 165 75 170 169 126 31 207 125 204 174 173 46 245 144 172 57 70 19 88 71 203 98 11 54 167 97 62 182 104 253 9 189 47 25 193 50 168 111 103 215 46 229 253 26 84 141 23 81 46 132 101 161 125 234 133 173 57 191 75 212 141 254 185 240 207 199 178 140 36 158 193 25 223 211 44 9 182 101 39 174 142 174 203 130 217 104 196 190 195 209 200 119 196 117 201 189 191 161 156 213 39 154 123 127 85 114 254 200 183 5 231 88 133 39 93 116 126 21 95 59 27 225 220 69 90 35 231 117 78 104 36 23 199 212 68 255 223 37 171 183 147 108 169 173 111 83 210 28 172 27 71 11 27 82 150 77 39 95 235 123 218 50 230 213 28 22 96 99 144 199 206 230 54 160 173 157 146 197 70 159 108 51 86 55 160 187 126 3 216 137 187 46 39 197 111 149 71 126 230 75 185 118 54 111 12 115 93 252 147 39 203 172 22 124 169 237 81 43 63 195 190 159 226 17 200 244 186 58 92 19 81 36 197 120 61 228 126 150 159 170 204 101 21 219 58 200 250 118 43 182 215 53 2 153 198 47 155 179 15 227 45 243 26 130 181 73 130 111 44 105 126 219 22 245 227 63 10 142 138 40 123 234 151 144 239 115 154 249 192 238 224 146 172 19 90 133 157 37 165 62 33 159 186 69 62 150 79 82 119 41 150 242 51 82 31 83 180 237 103 164 142 255 35 218 119 51 113 247 186 42 1 22 163 122 187 186 128 51 78 62 82 251 89 76 38 106 191 68 205 103 159 172 42 142 6 48 251 250 145 106 72 207 105 164 96 39 115 247 117 244 195 163 12 75 64 51 55 28 242 190 125 138 179 18 15 36 32 86 192 217 69 46 213 200 232 100 15 116 62 251 35 160 248 45 171 31 127 39 204 195 20 71 203 25 146 101 42 245 214 85 139 89 5 103 217 91 100 179 196 179 115 110 196 115 228 78 168 13 232 137 212 169 191 220 244 204 240 236 108 168 230 29 129 183 53 55 124 117 179 235 234 204 251 44 51 55 220 144 142 88 52 55 28 115 28 213 102 151 206 207 208 106 230 135 55 243 195 133 174 171 231 226 170 185 88 60 181 235 144 122 237 135 134 135 27 30 46 201 195 155 158 127 47 215 66 221 5 30 150 173 209 159 151 133 189 134 133 235 179 240 172 191 59 186 190 105 239 245 206 111 162 104 220 154 56 150 63 235 102 191 245 99 158 254 138 178 158 111 146 185 209 177 215 28 205 121 205 103 185 91 6 157 211 241 141 62 235 142 218 231 184 216 239 210 98 120 124 126 99 194 183 209 249 141 49 235 14 58 180 203 96 192 182 29 178 197 43 92 204 70 175 218 215 55 236 194 15 224 86 240 166 207 181 247 179 227 225 79 215 55 47 251 176 143 167 207 14 249 114 52 252 5 206 167 195 135 35 184 139 209 81 231 252 166 21 77 236 136 230 90 143 94 117 87 115 162 217 254 171 254 245 77 247 152 218 190 215 27 224 162 223 163 111 253 93 216 29 190 156 96 211 251 180 9 78 210 31 241 239 128 132 49 219 237 247 216 98 136 55 189 187 187 71 223 118 59 180 24 194 105 166 176 103 7 15 56 192 147 234 179 31 251 63 159 223 56 46 44 135 236 235 41 91 244 241 136 131 238 17 158 230 199 33 238 19 192 114 159 125 29 225 198 31 135 109 2 182 215 39 68 79 176 113 7 195 30 174 235 13 207 112 209 97 139 222 144 36 176 55 60 198 195 246 247 134 120 51 39 175 135 248 173 55 164 111 135 163 99 60 201 225 136 5 196 29 162 58 84 180 223 105 73 69 113 179 87 93 106 226 171 99 106 255 104 64 167 131 35 113 241 170 179 75 39 239 190 130 19 104 179 147 99 251 250 6 254 156 223 184 51 90 68 108 97 176 133 46 45 96 217 197 253 65 125 156 25 45 128 132 246 79 246 112 57 218 237 209 229 250 47 233 226 216 80 216 120 124 2 7 28 159 116 232 106 179 254 193 9 37 117 250 90 64 100 189 3 36 118 116 220 199 187 238 31 245 216 2 119 253 31 160 94 143 18 128 22 13 191 96 104 141 147 2 34 74 199 216 52 216 205 30 41 131 41 217 22 13 141 155 52 60 62 166 225 114 151 220 16 7 58 75 111 120 12 112 247 94 3 232 189 221 215 208 235 126 58 192 203 156 13 24 254 188 32 176 173 177 199 227 98 25 42 75 155 194 145 61 186 159 99 38 169 227 61 210 151 206 17 73 99 175 135 253 115 31 79 186 247 19 110 222 239 193 101 142 142 15 226 21 175 78 187 52 161 136 45 210 211 139 12 221 156 145 166 27 38 211 116 215 99 154 238 101 21 61 244 125 180 169 163 93 104 255 172 220 169 77 221 98 167 246 28 118 106 163 165 60 117 107 58 241 103 252 179 239 59 147 138 151 209 179 23 153 140 149 23 25 91 118 32 46 226 78 156 72 124 30 27 134 39 62 79 244 32 185 71 248 121 113 4 218 242 130 97 60 155 229 10 236 33 216 213 0 56 151 61 243 195 168 33 170 197 215 50 248 181 182 128 255 167 160 86 59 201 53 51 215 51 216 245 244 138 215 155 29 12 58 215 55 7 167 175 16 206 131 211 215 180 24 194 55 203 133 229 107 182 100 54 193 165 31 56 162 3 190 248 65 135 46 118 208 249 9 55 233 58 254 135 111 135 200 41 157 23 120 161 211 33 113 252 233 112 151 122 236 255 7 207 157 46 151 18 18 244 39 0 0 0 190 109 107 66 83 120 156 93 78 203 14 130 48 16 236 205 223 240 19 0 131 224 17 202 195 134 173 26 168 17 188 129 177 9 87 77 154 152 205 254 187 45 32 7 231 50 147 153 157 205 200 42 53 88 212 124 68 159 90 224 26 61 234 102 186 137 76 99 16 238 233 40 148 70 63 10 168 206 27 141 142 155 235 104 15 210 210 182 244 2 130 14 12 66 210 25 76 171 114 176 241 181 6 71 112 146 111 100 27 6 172 103 31 246 100 47 182 101 62 201 102 10 57 216 18 207 93 147 87 206 200 65 26 20 178 92 141 246 92 76 155 102 250 91 120 105 237 108 207 163 163 19 253 131 84 115 31 49 246 72 9 251 81 137 108 196 72 31 130 216 167 69 135 209 208 255 116 31 236 246 171 14 227 144 40 83 137 65 90 240 5 167 166 95 89 32 237 4 88 0 0 10 181 109 107 66 84 250 206 202 254 0 127 87 186 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 120 156 237 157 141 145 219 56 12 70 83 72 26 73 33 41 36 141 164 144 20 146 70 82 72 110 144 155 119 243 238 11 72 201 89 175 29 219 120 51 158 213 234 135 164 8 18 162 0 144 250 249 115 24 134 97 24 134 97 24 134 97 24 134 97 120 73 190 127 255 254 219 239 199 143 31 255 29 187 39 85 142 123 151 225 217 249 240 225 195 111 191 175 95 191 254 170 247 218 190 103 253 87 57 170 12 195 251 129 188 207 226 246 240 150 254 185 187 182 142 213 111 39 127 174 71 87 13 123 62 126 252 216 214 213 74 254 238 255 108 127 250 244 233 215 223 111 223 190 253 252 242 229 203 127 250 162 246 23 181 175 242 41 42 47 206 229 24 231 145 78 253 92 174 218 207 177 218 111 249 147 118 157 91 105 90 95 213 177 97 207 74 206 157 254 47 58 249 127 254 252 249 215 54 245 79 255 45 185 148 12 144 57 50 98 63 231 84 254 200 20 153 215 254 108 23 244 107 206 37 63 183 19 218 2 229 27 246 236 228 95 50 242 248 175 232 228 207 49 228 66 127 181 12 145 115 181 21 247 89 228 199 53 224 62 190 58 134 62 0 202 99 61 49 236 217 201 255 172 254 79 249 119 239 12 232 121 228 94 127 171 45 32 215 218 182 44 253 204 88 201 223 122 0 232 251 149 158 117 195 208 115 77 249 35 215 146 85 93 235 62 72 95 247 152 192 121 248 89 146 199 118 186 33 219 137 245 140 229 95 109 196 105 12 255 82 117 226 254 115 180 31 93 205 88 156 109 31 167 95 103 251 41 89 89 222 41 15 250 109 230 93 215 121 44 151 178 172 109 218 32 122 134 49 201 234 154 97 24 134 97 24 134 97 120 69 24 199 223 147 71 176 215 167 255 243 111 47 239 89 242 221 250 30 92 234 107 186 54 245 174 120 132 203 232 237 106 7 207 230 103 72 157 240 150 246 190 187 150 247 243 157 252 207 216 241 143 124 134 121 158 247 117 190 196 46 189 149 252 255 134 254 243 22 92 254 244 179 97 207 195 166 142 77 38 253 119 233 219 195 23 228 107 233 35 157 207 144 250 36 63 206 93 249 3 77 231 111 164 140 92 143 253 200 190 44 219 47 237 223 218 165 151 242 183 31 226 81 219 64 202 223 126 54 31 115 63 41 125 137 206 44 185 32 27 108 190 236 231 156 244 11 216 103 88 32 35 236 118 5 231 81 231 180 15 147 246 103 183 29 218 12 247 66 158 153 22 242 238 236 217 153 94 202 223 117 244 168 227 129 174 255 3 117 104 189 224 190 109 63 142 125 123 93 140 208 46 126 200 125 208 118 95 247 255 74 59 235 120 231 111 204 103 74 246 235 244 89 93 146 222 179 235 127 64 182 233 3 46 236 171 41 153 225 111 113 127 67 150 252 191 147 63 125 223 62 64 199 251 88 159 192 206 223 152 242 39 45 252 0 228 211 201 255 40 189 87 145 63 207 246 250 155 178 177 188 139 244 197 219 23 231 250 222 233 6 244 137 219 9 186 197 186 24 118 254 198 148 191 207 235 228 79 156 202 153 244 82 254 156 215 249 195 254 118 236 91 75 63 91 65 125 165 63 45 253 119 246 237 229 181 212 109 209 213 145 253 125 60 71 200 99 229 75 132 149 191 49 125 136 188 167 117 229 241 120 230 76 122 222 166 30 30 85 254 195 48 12 195 48 12 195 80 99 91 236 155 252 60 62 191 103 92 61 246 128 25 99 191 31 182 125 241 179 77 190 139 11 189 21 127 195 252 195 103 103 103 211 234 252 224 214 9 232 14 199 132 231 220 192 244 45 66 94 235 243 107 95 103 63 204 242 160 175 70 63 156 163 179 171 32 255 149 221 215 118 48 124 124 216 142 208 27 216 127 61 199 167 72 191 80 198 238 219 103 231 115 236 183 227 47 199 104 31 216 145 241 73 13 199 116 253 220 126 23 215 123 145 242 231 250 110 94 167 125 129 182 239 90 230 248 15 186 121 68 158 235 151 243 207 236 183 243 53 232 136 233 255 231 88 201 127 165 255 87 118 240 244 165 167 79 29 27 41 237 34 251 177 99 48 206 28 203 118 89 224 147 216 197 11 12 255 231 90 242 239 124 126 224 185 95 37 47 207 5 44 206 244 127 112 92 128 203 201 185 121 206 176 231 90 242 47 208 201 233 59 47 120 134 23 214 225 206 115 245 252 95 233 6 183 57 63 255 29 171 80 204 220 175 53 221 187 92 237 91 189 227 121 124 222 217 2 24 195 231 245 196 141 56 157 212 207 221 248 159 103 0 228 251 191 203 202 177 76 123 228 63 12 195 48 12 195 48 12 231 24 95 224 107 51 190 192 231 134 62 180 146 227 248 2 159 27 219 220 198 23 248 122 228 26 171 201 248 2 95 131 157 252 199 23 248 252 92 83 254 227 11 124 60 174 41 255 98 124 129 143 197 202 231 55 190 192 97 24 134 97 24 134 97 120 110 46 181 149 116 235 134 28 97 127 18 63 108 196 247 94 67 227 149 223 17 118 107 173 172 216 189 35 174 224 253 220 126 101 251 144 238 41 255 180 51 60 51 94 215 7 219 27 182 185 194 107 240 172 100 108 249 99 127 193 38 183 106 75 43 155 156 229 111 127 34 101 242 250 60 46 227 202 158 96 155 193 234 94 40 51 122 204 242 207 246 232 117 10 159 193 166 108 219 43 54 187 250 223 118 180 238 155 60 153 134 215 16 195 175 135 13 190 107 55 93 255 47 108 211 99 27 91 96 253 79 218 246 249 208 110 125 189 215 231 242 154 132 171 239 11 145 71 202 223 235 142 177 6 154 215 196 122 116 144 17 184 94 118 223 228 50 41 127 63 59 87 109 198 114 245 53 157 252 29 111 228 99 246 31 179 159 254 73 122 236 63 250 190 152 143 145 79 238 247 122 120 247 140 123 185 38 216 199 33 251 133 101 185 122 46 254 169 252 187 253 157 252 211 23 212 197 34 209 175 209 237 164 127 244 13 185 221 49 252 20 214 243 228 193 241 71 103 215 255 237 123 43 210 143 3 183 146 127 215 30 108 251 119 60 1 253 61 125 71 221 189 236 218 70 198 168 144 94 250 41 31 245 157 129 182 236 122 73 29 112 228 71 191 149 252 11 175 227 75 236 0 172 214 156 206 123 205 123 217 201 223 101 200 239 11 251 154 71 149 127 145 49 50 233 199 219 173 175 207 113 175 167 153 105 117 109 102 23 151 147 62 191 46 191 213 254 221 125 172 174 205 235 242 156 174 126 158 97 236 63 12 195 48 12 195 48 12 176 154 91 117 180 191 155 43 240 55 248 83 118 113 172 195 239 236 230 214 173 246 121 206 133 231 10 29 189 63 222 130 145 127 143 125 24 182 161 229 156 26 88 249 231 119 237 197 182 145 218 118 204 63 223 229 176 223 169 240 247 68 240 255 129 143 249 90 219 147 240 45 113 47 150 63 223 179 241 61 191 234 60 17 203 204 126 182 140 155 135 35 253 223 217 230 173 255 217 230 187 62 182 211 217 70 107 255 140 219 34 113 254 248 122 176 1 114 173 191 81 232 111 9 57 198 196 114 166 44 248 136 238 173 167 110 205 89 91 59 28 237 167 111 230 58 1 41 127 64 15 184 45 96 215 167 45 216 55 139 252 40 123 206 27 177 223 175 155 83 210 237 167 221 250 187 64 175 194 181 229 223 245 159 75 228 239 57 31 232 238 252 214 160 191 201 184 147 127 62 15 58 159 110 193 51 233 21 231 10 95 91 255 95 42 127 235 109 116 176 99 109 236 195 67 62 228 113 164 255 83 254 180 141 76 147 248 16 199 8 60 178 63 231 18 60 110 162 143 209 175 110 33 255 194 62 61 247 231 221 60 242 188 54 199 127 59 249 227 199 171 244 115 252 247 232 254 220 97 24 134 97 24 134 97 24 134 123 224 120 203 140 153 92 197 127 222 138 221 251 234 112 29 120 231 182 63 48 215 139 188 23 35 255 53 158 223 224 248 122 230 199 209 143 153 131 99 191 93 166 179 178 161 120 14 94 250 234 108 135 242 188 160 149 15 208 199 40 19 215 102 249 177 255 58 126 31 159 166 203 211 217 163 94 1 108 160 105 15 197 110 231 53 26 57 175 179 169 23 158 87 199 207 126 183 92 59 20 219 63 118 254 110 142 151 237 187 246 231 146 54 101 204 114 165 223 209 253 191 243 85 98 83 126 118 249 167 254 163 94 185 247 85 12 79 55 79 40 233 124 59 43 249 83 207 233 71 244 252 52 124 63 216 102 237 31 180 255 208 247 182 42 63 199 114 142 151 247 87 126 207 50 207 111 197 78 254 150 217 159 202 127 165 255 83 254 57 255 215 126 100 142 97 171 231 152 231 242 238 214 135 220 201 223 109 20 72 139 54 246 236 109 192 88 255 91 255 221 74 254 133 253 144 126 206 115 46 227 18 218 3 121 156 209 255 144 109 131 52 217 182 79 220 237 239 21 124 66 30 255 173 100 252 158 242 247 248 47 231 248 217 7 152 254 193 98 53 254 219 201 191 200 121 254 221 248 239 85 228 63 12 195 48 12 195 48 12 195 48 12 195 48 12 195 48 12 195 48 12 195 48 188 46 196 80 22 151 126 15 4 31 254 42 214 206 49 64 71 92 226 159 191 228 220 75 202 240 222 16 223 114 75 88 255 152 181 115 50 127 226 177 40 223 165 242 231 186 142 74 235 236 183 22 186 56 148 142 140 39 62 226 146 88 64 226 150 188 14 206 81 172 251 81 253 26 98 163 224 218 113 244 142 157 242 119 120 137 11 236 98 247 41 147 191 235 238 184 95 202 75 124 149 143 145 118 206 23 32 222 220 49 231 142 213 77 188 54 16 107 8 101 159 101 45 50 199 155 239 250 53 249 159 57 151 243 29 63 75 94 142 69 186 164 126 137 57 242 207 177 81 239 177 174 89 174 215 70 185 56 150 235 127 17 15 78 63 229 126 59 58 61 74 60 151 239 219 223 254 161 45 236 116 0 241 186 196 146 121 78 65 222 27 199 233 103 187 254 67 254 142 9 92 225 251 78 253 226 88 202 75 234 151 115 41 39 233 94 75 255 103 92 126 226 231 50 125 155 54 237 125 212 179 191 183 225 239 41 185 30 220 110 93 79 187 251 114 222 29 94 27 204 247 179 106 135 126 134 34 183 238 123 31 112 230 185 226 249 50 57 119 128 54 148 28 213 47 208 46 28 159 122 45 118 237 58 251 43 253 199 255 163 243 178 77 123 222 87 151 30 253 143 251 74 89 185 175 90 247 173 202 234 181 162 96 37 183 140 13 237 210 164 108 57 143 133 188 18 231 149 237 158 182 153 28 213 175 207 35 79 207 85 122 43 232 193 85 255 63 59 238 253 19 249 115 142 231 220 100 154 126 30 118 241 224 78 183 155 135 68 157 231 55 42 46 89 231 171 27 43 118 58 211 121 101 59 244 248 56 203 125 166 126 233 255 221 124 133 183 178 43 195 145 222 133 78 254 57 127 163 224 221 192 125 149 123 66 198 158 35 116 6 175 233 152 215 120 238 144 227 213 173 115 143 88 201 191 107 175 140 83 172 31 86 237 155 252 47 213 231 187 117 74 61 111 234 12 71 253 191 240 60 139 213 187 76 39 255 163 178 243 76 172 242 58 15 228 68 93 178 54 220 106 172 102 157 153 242 103 76 152 235 4 122 157 47 175 43 216 189 91 123 142 153 223 29 186 114 240 221 67 244 77 174 89 215 113 166 126 225 232 91 187 28 123 143 121 139 188 163 174 242 205 121 26 9 99 5 200 122 65 191 167 222 244 187 244 145 222 59 211 238 119 239 132 171 177 128 199 57 151 172 19 122 201 115 122 87 191 148 251 72 174 156 243 236 243 86 135 97 120 51 255 0 160 130 250 32 224 191 37 163 0 0 1 187 109 107 66 84 250 206 202 254 0 127 109 95 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 120 156 237 209 49 106 195 80 20 69 65 111 203 43 247 74 178 0 85 193 69 8 65 173 28 69 165 144 83 255 95 156 121 48 188 246 194 217 247 247 187 145 241 223 205 222 196 208 254 175 171 217 155 208 159 97 253 223 110 246 38 6 246 223 182 223 195 118 254 179 55 49 176 255 186 174 135 159 243 159 189 137 129 253 159 207 207 171 217 155 24 216 127 89 150 171 217 155 24 216 127 223 191 15 95 231 63 123 19 3 251 63 62 238 87 179 55 161 63 250 163 63 250 163 63 250 163 63 250 163 63 250 163 63 250 163 63 250 163 63 250 163 63 250 163 63 250 163 63 250 163 63 250 163 63 250 163 127 157 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 233 223 166 127 155 254 109 250 183 253 1 102 195 245 54 168 72 0 206 0 0 3 30 109 107 66 84 250 206 202 254 0 127 126 217 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 120 156 237 218 137 114 154 80 20 198 241 190 86 251 226 237 139 52 221 140 75 167 213 128 226 2 184 224 5 78 207 229 162 128 154 218 106 90 38 204 255 78 126 115 8 152 68 231 227 46 64 68 106 45 207 243 114 43 211 175 92 210 125 38 225 106 35 190 63 147 225 176 175 251 23 146 203 55 121 255 241 157 122 139 14 184 150 255 114 17 203 116 26 200 104 52 208 236 231 178 151 79 228 223 33 215 242 95 45 215 199 252 233 255 221 115 45 255 40 220 138 231 77 101 48 120 44 242 207 228 43 249 119 200 181 252 227 104 215 200 159 254 223 45 127 178 254 179 227 191 93 255 49 255 119 207 121 254 154 189 24 253 38 213 252 77 49 255 207 166 75 205 127 168 71 232 255 93 83 181 76 179 221 107 181 118 186 157 136 49 137 142 255 137 76 198 115 53 149 196 204 245 216 35 249 119 72 179 255 167 103 249 219 235 63 223 91 200 247 209 88 247 199 228 223 49 98 135 252 82 158 53 199 2 155 255 58 78 52 255 185 60 60 60 136 201 3 221 223 35 255 14 57 203 63 175 214 2 198 232 252 191 138 36 8 22 242 244 100 251 255 74 245 201 191 67 46 230 111 183 149 49 153 172 215 91 25 143 199 210 123 252 172 187 67 61 248 131 252 59 164 154 252 79 242 207 92 254 81 180 22 223 247 203 251 255 27 61 52 147 15 69 254 232 2 155 169 93 239 217 107 62 183 254 171 229 175 75 193 56 222 232 216 239 23 215 127 238 250 96 35 59 227 105 157 42 91 131 210 188 147 53 47 204 158 173 237 190 191 131 229 29 34 113 235 250 93 153 111 121 10 212 242 183 207 255 38 147 73 241 12 160 223 239 201 112 212 147 159 227 161 244 7 95 138 125 104 211 232 183 108 191 253 29 151 125 92 142 1 54 255 236 56 6 88 118 254 119 107 192 160 152 7 156 89 113 77 224 61 5 197 189 97 207 243 208 154 201 93 92 238 135 236 141 84 3 128 179 79 114 217 110 183 58 14 132 18 134 75 21 22 207 132 162 112 47 225 42 113 219 81 84 172 19 168 109 84 107 117 115 189 216 242 166 44 203 196 164 155 210 86 210 52 149 84 79 149 163 52 167 182 85 11 233 205 181 145 245 51 249 187 93 137 212 215 138 199 215 209 94 119 171 93 243 53 218 217 121 225 214 254 206 254 228 197 25 94 171 218 61 63 215 175 141 156 174 1 171 99 135 181 130 235 255 213 253 226 195 207 80 255 111 125 1 23 251 247 206 237 106 220 19 174 255 93 57 57 55 178 155 171 251 53 237 213 215 253 254 239 87 101 104 115 143 74 27 57 156 27 245 103 2 135 207 125 252 55 129 75 243 198 95 182 151 250 28 55 127 254 59 91 219 239 255 78 111 4 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 224 223 250 5 53 42 42 144 67 194 135 176 0 0 14 215 109 107 66 84 250 206 202 254 0 127 146 129 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 120 156 237 157 141 145 28 41 12 133 29 136 19 113 32 14 196 137 56 16 7 226 68 28 200 94 233 234 62 215 187 103 73 64 207 207 254 88 175 106 106 103 187 105 16 18 8 208 131 158 151 151 193 96 48 24 12 6 131 193 96 48 24 12 6 131 193 224 63 252 250 245 235 229 231 207 159 127 124 226 58 247 170 231 226 115 165 172 0 101 248 247 193 255 81 233 249 138 254 51 124 255 254 253 229 211 167 79 127 124 226 58 247 50 124 249 242 229 223 207 46 190 125 251 246 59 239 176 55 101 4 244 251 71 70 212 251 180 158 149 158 79 245 191 2 54 209 254 174 246 95 245 81 250 118 149 166 147 215 237 223 249 157 93 84 242 172 228 4 93 26 189 94 165 203 174 103 237 188 123 62 112 106 231 171 122 235 236 31 229 107 223 117 185 180 111 123 30 154 15 121 5 170 254 175 121 101 245 142 251 159 63 127 254 247 123 232 45 210 253 248 241 227 247 189 120 134 186 168 47 11 68 58 189 30 233 51 68 254 158 38 242 136 188 185 23 101 171 94 72 23 215 245 121 234 160 105 35 47 79 71 29 244 122 252 141 207 170 255 199 223 175 95 191 254 126 238 10 58 251 115 77 101 209 242 35 77 148 207 51 89 159 35 61 247 50 251 187 12 81 158 219 8 155 199 223 208 153 166 137 239 140 91 92 115 61 69 26 157 243 56 180 125 69 62 164 71 23 200 169 233 84 238 248 160 11 218 178 202 29 215 84 31 170 103 45 159 241 125 215 254 250 204 21 172 252 127 86 102 214 255 233 127 157 188 129 204 254 234 111 168 83 86 119 236 28 122 86 125 209 46 226 131 47 208 62 168 126 129 60 86 114 118 186 64 62 202 33 63 247 21 85 157 253 249 144 207 203 223 25 255 239 49 23 184 197 254 232 60 250 35 245 238 228 165 254 149 253 125 29 226 192 182 216 61 254 70 91 32 127 254 247 126 134 172 244 81 218 139 66 243 161 78 153 46 240 183 46 43 237 145 185 94 101 127 100 212 231 189 252 76 111 43 91 92 197 45 246 167 29 107 187 239 228 13 100 246 87 127 135 239 205 250 40 125 221 253 143 234 54 238 113 221 253 36 190 35 179 63 122 208 54 150 233 66 211 169 205 105 23 200 239 246 199 63 242 76 252 229 25 205 55 174 33 227 51 236 79 95 81 125 208 159 1 117 245 239 234 111 25 223 28 154 30 153 233 91 250 61 158 165 15 116 107 165 200 139 251 248 77 192 255 145 198 239 33 39 125 47 3 245 198 135 100 186 208 114 50 249 201 95 199 67 250 135 255 239 178 168 252 58 151 169 244 233 186 29 12 6 131 193 96 48 24 12 58 56 247 151 221 127 205 242 111 193 61 184 132 43 56 45 243 52 125 197 219 158 194 99 229 196 165 88 207 100 113 129 123 98 85 254 234 217 21 159 214 113 152 143 194 169 206 174 232 184 226 109 239 33 171 198 182 171 103 42 206 170 242 31 85 92 122 183 252 91 249 180 71 65 229 214 61 19 94 167 43 114 117 241 252 123 181 235 76 86 229 213 252 254 9 231 85 165 63 41 255 86 62 173 210 147 243 134 206 41 250 30 5 231 120 144 133 124 156 3 89 233 172 210 129 198 21 187 103 30 105 127 244 227 156 92 197 121 105 220 94 99 230 85 250 211 242 111 225 211 42 61 17 163 11 192 179 194 243 33 243 202 254 196 237 84 22 149 179 211 89 165 3 181 63 114 102 242 103 254 255 202 30 154 19 251 87 156 151 115 214 202 189 86 28 217 110 249 212 245 42 159 86 217 31 238 88 249 35 229 20 61 255 204 70 200 220 253 127 170 3 207 171 146 127 151 43 91 161 26 171 224 70 244 126 197 121 5 72 171 92 76 151 126 183 252 91 249 180 206 79 210 135 163 140 104 15 244 53 158 221 181 191 142 25 158 246 84 7 167 246 191 21 234 111 224 26 148 119 175 124 83 102 23 174 251 220 193 211 159 148 127 43 159 214 233 41 179 183 206 59 25 23 180 236 204 70 1 229 22 179 180 59 58 120 13 251 235 216 201 199 247 226 41 47 136 79 83 206 11 121 178 235 85 250 147 242 111 225 211 50 222 206 101 3 202 41 86 101 51 39 119 174 148 231 145 103 71 103 153 142 61 239 74 254 174 94 131 193 96 48 24 12 6 131 193 10 196 190 136 249 156 130 117 32 107 152 85 44 170 75 19 115 219 29 254 231 30 216 145 53 128 110 30 89 230 174 44 247 6 231 98 88 119 123 12 127 5 226 31 172 191 119 214 38 221 250 245 153 251 25 51 174 81 215 148 129 248 78 172 240 81 101 118 215 31 9 143 93 7 168 175 198 183 186 246 175 103 99 72 175 250 203 158 231 25 238 105 189 157 247 138 123 238 151 244 204 65 166 51 158 233 244 73 127 243 56 134 199 104 179 242 119 242 202 100 204 210 117 215 119 203 200 244 184 131 85 28 73 247 228 87 231 113 148 131 240 56 173 238 213 215 231 73 163 62 7 157 43 151 163 241 64 229 236 148 15 112 153 244 28 81 197 159 225 243 200 71 99 254 90 151 170 252 157 242 92 198 170 204 234 250 142 188 196 27 85 198 19 172 236 223 157 183 4 93 204 210 249 47 248 58 237 255 232 202 207 51 100 231 59 201 171 58 103 66 57 200 82 157 141 200 244 239 124 71 87 126 149 151 238 215 119 25 171 50 171 235 59 242 102 122 60 65 199 143 145 223 138 103 236 236 95 233 127 231 108 17 249 250 167 203 23 121 208 253 142 207 210 254 158 197 226 179 242 61 175 157 179 132 85 153 213 245 29 121 51 61 158 32 27 255 149 91 244 54 95 237 251 185 218 255 51 93 85 253 95 203 238 236 239 28 244 170 255 107 222 93 255 175 230 127 93 255 215 178 171 50 171 235 59 242 222 106 255 128 142 45 62 134 225 159 116 124 233 248 43 151 105 53 254 171 220 110 255 128 158 225 171 206 159 103 186 90 205 89 124 60 245 49 78 121 197 172 124 69 55 254 107 250 170 204 234 250 142 188 247 176 63 245 174 214 255 122 47 107 155 220 215 62 228 60 154 207 255 61 141 206 145 125 78 159 205 229 187 24 129 206 133 249 158 1 25 252 62 229 117 229 87 121 173 100 172 202 172 174 175 210 100 122 28 12 6 131 193 96 48 24 12 42 232 220 92 63 87 56 192 71 163 146 203 215 31 31 1 207 226 2 89 187 251 30 204 213 58 130 253 210 207 68 21 103 171 246 99 190 103 60 139 11 204 246 50 43 178 184 128 238 217 213 253 255 39 235 87 181 21 177 54 189 158 113 110 216 159 123 158 7 114 232 117 231 34 119 184 60 210 84 107 244 206 46 153 30 244 92 160 114 164 149 190 110 229 27 79 224 251 239 213 62 85 236 143 152 19 126 99 135 35 84 63 163 241 43 174 235 158 121 246 239 123 188 81 249 194 44 143 44 94 157 113 70 29 47 88 197 251 118 56 186 46 214 169 103 19 225 215 51 125 237 200 184 226 2 79 80 249 127 236 143 12 126 110 37 227 104 85 7 222 71 52 14 167 231 248 40 95 251 133 115 110 124 87 125 84 103 38 120 94 99 235 244 147 44 70 237 253 133 182 199 249 21 176 195 209 173 184 14 242 171 244 181 43 227 138 11 60 193 202 255 35 43 125 47 227 85 119 56 66 206 82 104 95 207 202 239 228 209 124 187 51 51 33 15 109 150 114 92 55 85 57 140 109 234 223 40 123 197 209 157 112 157 153 190 118 101 220 229 2 119 208 233 91 251 172 250 120 175 235 14 71 168 239 181 204 250 191 142 223 93 255 223 177 63 103 59 245 44 231 110 223 210 189 3 218 111 119 56 186 93 174 179 210 215 149 254 175 207 223 98 127 247 255 200 238 227 63 250 199 79 86 233 188 61 249 184 90 217 95 243 206 198 255 29 251 163 35 191 118 101 252 63 225 232 118 185 206 78 95 87 198 255 157 179 142 21 170 245 191 114 222 89 76 128 235 218 111 59 142 16 249 176 159 206 49 179 103 178 57 112 86 94 149 71 181 126 62 153 91 95 225 232 118 184 78 79 183 83 247 29 25 159 21 51 24 12 6 131 193 96 48 24 188 127 192 231 232 218 239 81 184 146 183 190 91 227 30 239 190 215 216 192 9 118 101 127 79 92 20 107 94 56 0 214 148 143 168 3 118 60 1 235 90 98 122 186 175 178 219 7 92 1 142 225 20 26 79 236 112 165 142 175 5 98 22 190 110 36 126 22 240 125 172 190 14 199 111 232 122 149 51 24 241 209 152 13 109 75 227 11 232 171 227 118 85 198 206 254 93 94 187 105 180 78 186 15 222 229 184 87 29 65 246 187 35 180 247 120 142 235 148 171 233 2 145 230 106 223 202 116 196 245 236 28 11 113 11 141 179 58 7 163 191 207 69 61 208 13 122 37 214 133 223 113 249 41 67 99 241 213 89 1 181 69 197 209 237 164 113 153 180 174 202 121 82 71 253 45 161 43 117 12 40 15 75 92 15 251 104 172 80 211 17 39 165 220 42 94 216 161 178 191 94 175 236 175 241 113 218 61 109 18 61 121 124 75 243 242 248 118 197 171 101 252 137 219 92 253 152 114 200 138 157 52 1 181 159 199 31 61 142 155 241 123 87 234 184 250 221 49 210 251 251 4 157 59 61 221 183 229 242 120 28 218 235 19 112 251 59 111 64 62 250 155 148 200 152 233 6 84 92 212 142 253 43 30 67 177 147 6 61 42 239 153 201 171 124 152 191 71 240 74 29 79 184 67 202 229 3 87 93 249 150 21 120 142 49 46 179 153 239 181 160 15 57 95 69 159 208 177 169 226 13 189 253 86 253 113 199 254 1 245 127 202 227 41 118 210 40 231 168 227 160 202 145 125 207 236 191 91 71 229 4 241 249 29 119 168 156 159 238 199 186 2 29 219 244 227 123 45 226 131 111 244 189 28 220 83 29 100 215 149 95 203 158 239 206 215 131 202 254 248 50 229 212 29 59 105 84 38 159 119 42 223 206 119 250 135 183 153 147 58 6 212 6 186 167 199 199 103 215 173 182 147 83 254 79 161 227 7 115 204 236 222 106 143 221 234 186 239 37 90 141 91 126 95 203 175 100 233 214 174 183 164 241 178 43 157 156 214 241 244 185 204 119 93 217 255 55 24 12 6 131 193 96 48 248 187 160 235 24 93 207 220 107 30 217 237 73 140 117 144 190 43 101 167 76 93 171 18 239 206 214 130 239 1 111 65 94 143 231 176 238 59 61 71 82 161 179 63 156 0 233 118 236 175 177 22 214 59 60 247 22 244 121 130 183 32 111 22 143 36 78 238 49 27 231 180 136 61 58 223 165 80 238 86 227 21 1 158 117 159 163 60 180 199 32 60 38 171 252 191 234 51 75 171 121 19 67 169 184 24 47 23 249 52 150 163 28 114 199 41 105 61 181 254 42 47 124 206 142 94 144 93 203 81 185 78 208 157 49 113 142 71 185 8 141 165 119 62 195 185 123 229 149 225 11 60 70 170 245 243 243 141 206 193 86 254 159 114 178 50 189 110 217 222 108 111 255 216 141 216 167 199 124 177 109 182 111 155 182 2 39 226 113 98 236 173 237 162 211 139 203 238 114 157 96 101 127 236 234 101 102 177 255 21 143 232 54 10 48 7 168 108 122 146 159 115 65 222 95 178 188 171 24 176 238 15 210 120 55 177 214 236 183 163 42 238 85 125 141 238 173 82 174 37 243 25 43 189 184 191 67 174 19 172 252 191 219 95 231 9 183 218 159 126 73 92 253 81 246 215 126 177 107 127 252 6 121 169 207 35 95 183 255 106 111 7 126 68 121 85 229 15 192 174 94 170 250 158 96 53 255 163 158 112 140 186 7 229 22 251 107 185 238 135 111 181 63 58 164 28 213 203 174 253 201 67 199 14 181 185 238 119 217 201 7 127 174 252 159 203 235 62 63 211 75 118 207 229 58 65 182 254 243 125 69 58 183 200 206 95 171 110 28 43 123 209 254 117 255 195 61 236 175 243 191 234 220 114 160 179 155 247 113 205 147 190 204 179 93 62 62 255 203 100 209 125 24 43 189 248 252 207 229 26 124 76 188 133 245 226 224 245 48 246 255 187 49 252 254 96 48 24 12 238 133 108 253 183 138 101 128 183 244 174 249 142 103 58 133 174 215 186 241 182 138 157 158 96 87 215 247 44 51 203 79 227 63 171 189 145 1 93 151 190 5 92 57 251 80 129 248 215 238 126 205 91 176 163 107 79 255 8 251 87 249 105 236 71 99 21 126 198 41 227 8 29 196 45 60 158 14 119 166 188 46 253 143 51 113 204 125 149 15 227 158 127 39 118 170 117 240 119 36 86 177 146 172 110 26 147 209 118 225 242 40 178 114 178 186 194 255 184 174 53 142 239 50 169 189 136 83 59 87 184 139 202 255 107 140 95 207 75 209 207 148 243 173 56 66 47 7 221 193 157 104 249 232 69 207 213 160 31 151 39 139 57 234 119 63 127 75 59 210 120 58 242 120 27 240 186 5 60 150 236 124 64 182 15 219 99 179 196 207 189 174 21 135 167 249 86 250 224 121 226 198 21 7 187 99 127 247 255 64 219 150 142 83 30 119 205 56 66 7 28 149 242 113 174 195 238 255 93 251 163 23 61 143 233 50 7 244 108 143 66 211 101 54 86 125 101 246 247 185 136 158 173 171 242 82 253 40 183 216 233 195 203 225 222 9 86 254 223 101 202 246 90 84 28 161 235 132 52 221 251 31 239 97 127 100 162 173 101 50 7 222 146 253 43 110 241 89 246 207 230 127 248 66 245 151 186 143 2 159 84 113 132 174 83 252 127 215 255 3 187 254 14 223 148 233 60 155 195 239 248 127 173 91 38 207 61 252 127 102 255 142 91 124 134 255 247 15 245 247 115 176 122 238 80 231 41 25 71 232 229 232 217 193 170 125 123 90 215 133 206 51 171 179 153 110 55 197 106 254 23 112 251 87 243 191 206 119 102 229 116 246 239 184 69 191 119 207 249 223 91 199 189 215 59 131 247 133 177 255 96 108 63 24 12 6 131 193 96 48 24 12 6 131 193 96 48 24 12 6 131 193 96 240 247 225 17 123 209 217 251 244 145 193 126 19 255 172 246 189 179 199 210 223 105 17 96 111 76 246 121 132 62 117 111 169 195 223 105 92 113 137 236 75 243 124 125 255 8 239 0 210 15 251 55 158 201 83 118 117 57 205 71 247 237 241 169 236 175 191 99 194 190 35 246 217 233 62 183 44 207 106 111 240 173 242 119 123 245 213 134 250 30 16 135 214 27 221 178 135 76 251 4 249 61 171 109 87 232 234 242 200 50 117 95 164 246 141 172 255 56 216 175 150 229 233 159 106 111 158 63 187 218 27 178 107 127 222 203 68 250 248 223 223 177 161 246 127 109 188 134 253 253 119 7 212 62 244 195 14 236 35 174 160 125 169 219 203 22 215 245 183 69 58 172 236 79 219 161 79 59 60 255 172 255 115 157 252 121 151 155 238 161 212 125 155 1 218 155 190 107 137 125 126 250 140 238 177 213 124 181 77 146 134 239 39 56 25 67 84 23 254 174 161 172 111 43 168 91 87 158 250 130 204 246 140 189 248 134 85 153 228 169 239 239 81 153 245 253 114 39 246 247 189 238 92 231 127 173 39 231 146 116 156 84 89 184 207 119 244 138 204 122 46 68 243 165 61 168 220 87 124 211 201 126 113 61 95 161 190 0 121 170 190 77 187 220 241 231 217 217 1 205 199 247 150 175 228 199 143 107 254 1 116 167 239 98 83 61 50 230 251 124 165 210 177 207 121 180 78 218 95 125 79 175 238 101 215 249 148 230 145 189 219 143 246 224 253 240 116 76 224 156 83 54 159 113 96 67 255 237 204 106 28 102 253 180 51 150 35 203 233 220 176 179 63 122 213 49 11 255 233 237 72 237 175 254 93 247 154 147 231 137 253 245 92 34 237 87 203 204 222 93 229 243 172 202 254 252 79 59 190 50 39 64 190 236 227 64 126 116 71 27 206 252 58 186 171 252 170 231 123 229 221 117 129 202 254 234 55 1 109 49 139 21 232 153 81 93 223 80 87 157 31 86 231 27 50 251 83 174 142 229 122 93 207 165 122 159 210 115 31 157 253 145 201 219 215 51 208 197 93 118 231 22 145 238 170 220 221 120 119 210 158 244 221 58 218 151 62 218 121 138 193 96 240 161 241 15 136 36 106 162 235 119 189 22 0 0 1 24 109 107 66 84 250 206 202 254 0 127 153 93 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 120 156 237 209 49 10 2 81 12 69 81 183 165 27 119 37 46 96 42 177 17 101 218 209 248 203 225 107 157 128 57 129 67 218 7 55 206 151 99 76 14 17 65 15 115 123 253 123 209 191 55 253 123 211 191 55 253 123 211 191 55 253 123 211 191 55 253 123 211 191 55 253 123 27 189 79 179 234 77 36 246 143 120 12 247 253 175 222 68 98 255 101 89 102 213 155 72 236 127 187 93 103 213 155 72 236 191 174 235 240 220 255 234 77 36 246 223 182 247 176 237 127 245 38 18 251 255 184 234 77 164 246 127 205 170 55 161 63 105 253 191 174 122 19 250 3 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 240 31 62 90 61 106 52 255 190 32 236 0 0 4 121 109 107 66 84 250 206 202 254 0 127 162 54 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 120 156 237 154 137 109 235 48 16 5 93 72 26 73 33 41 36 141 164 144 20 146 70 82 136 63 54 248 99 60 108 72 217 177 100 248 154 7 12 116 144 226 177 203 67 34 181 223 43 165 148 82 74 41 165 148 82 74 253 215 247 247 247 254 235 235 107 74 133 223 131 178 30 151 76 251 18 233 95 83 85 159 221 110 55 229 243 243 115 255 250 250 250 115 126 203 117 167 30 85 214 45 85 190 127 121 121 57 216 99 235 244 175 173 244 255 251 251 251 47 74 117 172 122 63 163 255 105 251 207 224 255 153 62 62 62 126 218 0 115 1 109 163 174 43 140 118 193 117 133 213 145 248 21 206 189 89 154 117 204 56 179 180 250 179 117 78 25 210 63 164 87 80 62 202 49 74 115 164 138 87 105 86 255 231 252 145 253 95 117 75 240 69 31 255 137 207 184 136 47 114 156 204 249 163 194 136 95 226 154 103 75 149 87 93 215 177 158 153 205 69 89 30 242 99 108 202 243 44 91 166 159 84 156 89 27 72 187 84 190 207 224 255 14 182 155 249 255 237 237 237 240 142 136 125 120 166 247 199 76 35 125 65 155 32 188 210 226 156 246 71 252 158 86 29 203 55 188 151 145 94 247 125 62 147 105 210 158 186 178 45 231 28 248 232 254 159 105 230 255 236 59 125 158 76 74 248 176 236 88 237 38 199 84 202 80 247 51 253 84 222 35 175 244 223 168 29 103 255 174 240 74 31 31 214 249 108 14 32 30 101 228 253 39 239 61 138 214 248 63 133 205 152 111 233 231 249 110 128 237 177 97 246 235 236 155 244 189 124 55 24 141 21 249 62 154 253 63 199 144 244 85 150 37 203 59 171 239 18 143 162 173 252 143 47 241 109 246 181 158 78 127 47 232 227 73 142 181 217 247 102 243 81 214 131 252 122 123 200 241 60 211 100 204 233 54 233 100 153 110 249 59 232 175 218 202 255 165 178 81 190 3 150 109 251 123 123 247 53 237 36 253 192 251 4 105 209 166 120 230 20 255 103 60 198 36 242 34 205 94 190 37 61 234 252 175 148 82 74 41 165 158 83 151 218 231 205 116 239 101 47 249 214 148 235 32 151 176 97 125 27 205 190 107 250 247 212 95 181 246 121 181 63 172 57 212 154 201 108 205 122 141 210 255 125 109 35 191 243 71 223 252 75 107 33 172 155 244 243 107 40 203 57 234 71 252 227 49 187 191 148 222 232 122 150 215 44 173 81 191 38 238 108 205 105 86 230 191 138 253 178 220 251 35 95 214 60 242 156 246 194 51 185 206 158 109 36 247 94 57 103 13 151 123 57 38 176 119 179 38 188 219 58 203 153 235 137 117 143 245 102 214 154 250 186 82 133 247 58 214 189 76 43 109 87 199 244 25 144 87 94 163 30 167 196 122 37 247 73 151 189 238 165 50 159 163 238 151 108 111 163 254 207 90 27 182 102 63 190 107 52 246 87 188 220 199 45 232 35 172 243 173 9 63 86 47 236 151 99 94 255 183 33 247 33 16 123 25 117 204 181 202 180 85 166 217 243 26 217 183 255 19 65 156 209 253 145 255 71 101 62 71 125 254 63 197 255 25 127 246 254 144 118 226 156 189 25 142 248 51 255 201 88 19 126 106 189 250 156 151 123 2 248 122 84 167 244 65 183 79 94 207 242 202 176 209 62 19 247 179 63 115 157 121 143 202 124 174 182 240 127 223 71 41 159 80 166 60 47 49 214 49 126 50 150 209 158 215 134 159 82 47 194 24 55 248 143 0 27 211 190 16 115 109 247 127 31 115 114 143 242 152 255 249 127 34 243 40 245 251 221 255 179 50 159 171 99 254 207 185 29 59 244 126 146 117 45 245 241 41 125 131 191 136 203 60 182 85 248 49 187 103 24 182 228 253 58 231 97 198 112 226 98 231 180 53 123 157 248 51 247 40 143 149 131 127 107 241 33 109 39 109 202 158 217 200 255 163 50 159 243 46 200 126 91 214 41 125 71 223 206 186 98 143 209 88 69 220 76 175 231 215 253 217 251 193 154 112 194 150 234 149 97 212 131 185 164 251 55 231 242 209 55 24 207 247 231 70 121 245 48 218 64 223 139 76 27 19 63 243 158 149 185 247 181 75 104 105 189 64 61 190 244 191 82 74 41 165 148 82 74 41 165 148 82 74 93 77 59 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 145 59 228 31 135 151 142 106 217 59 142 156 0 0 1 83 109 107 66 84 250 206 202 254 0 127 165 133 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 120 156 237 214 225 105 131 96 20 134 81 7 113 17 7 113 16 23 113 16 7 113 17 7 177 188 129 11 183 166 33 255 138 9 231 129 67 155 79 251 235 77 108 206 83 146 36 73 146 36 73 146 36 73 146 36 73 146 36 73 146 36 73 31 209 113 28 231 190 239 79 231 57 203 181 119 247 191 186 79 247 111 93 215 115 28 199 115 154 166 199 207 236 24 253 44 247 212 189 117 22 203 178 60 94 199 48 12 127 190 135 116 239 178 99 149 125 179 97 118 173 205 251 61 181 127 213 55 191 254 141 238 95 182 235 123 86 57 235 159 229 122 157 125 251 198 217 191 186 94 211 253 203 115 190 239 95 255 215 231 121 126 218 63 215 236 255 125 101 219 122 238 215 239 219 182 253 250 204 247 231 191 253 191 175 250 30 215 247 171 247 64 174 213 119 251 156 69 213 159 29 215 107 146 36 73 146 36 73 146 36 73 250 247 6 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 248 64 63 10 249 172 184 82 218 42 102 0 0 18 12 109 107 66 84 250 206 202 254 0 127 177 24 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 120 156 237 157 105 144 92 213 117 199 101 108 99 176 236 176 26 97 194 14 14 18 154 85 74 21 149 114 42 159 242 33 206 23 151 63 132 196 85 169 114 170 28 59 24 44 192 17 14 96 59 132 176 56 11 49 241 2 4 48 32 36 33 36 129 4 18 66 66 251 50 154 233 233 153 222 167 151 233 153 222 247 158 222 247 238 215 175 95 255 115 238 125 221 51 18 38 17 174 122 240 170 186 110 171 126 165 81 79 75 58 53 191 119 206 93 223 125 0 100 0 10 216 171 219 197 202 171 219 70 71 150 80 169 212 176 180 148 69 52 26 166 79 229 209 133 7 31 184 255 68 48 32 0 109 168 215 0 41 87 62 236 191 137 90 173 134 84 42 133 72 36 66 111 150 233 26 112 235 30 179 224 211 243 95 44 22 145 205 102 17 143 199 33 119 69 254 15 26 23 242 223 110 183 145 78 167 17 14 135 69 254 15 32 23 242 159 207 231 169 253 95 66 44 22 67 7 5 145 255 3 198 133 252 183 90 45 36 147 73 132 66 33 158 255 194 255 96 113 33 255 213 106 21 153 76 134 250 255 81 145 255 3 200 133 252 215 235 117 222 254 247 251 255 29 184 116 143 89 240 233 249 103 227 63 214 255 103 237 127 23 37 145 255 3 198 133 252 151 203 101 36 18 9 209 255 31 80 46 228 191 82 169 160 80 40 240 62 160 104 255 7 143 143 211 254 179 185 159 64 32 32 250 255 3 200 199 169 255 108 14 128 229 191 130 162 240 63 96 124 156 252 103 125 63 145 255 131 201 199 241 159 203 229 248 53 32 242 127 240 80 253 43 43 254 187 43 254 229 78 3 213 106 153 106 127 28 225 136 159 222 44 138 241 255 128 177 188 254 223 61 223 127 23 18 100 165 130 106 173 128 116 150 252 199 22 233 83 89 122 95 248 31 36 250 181 255 119 253 55 85 255 141 60 18 233 8 247 223 69 78 248 31 48 254 223 252 239 212 80 174 22 144 205 103 16 75 132 33 117 51 162 254 15 24 231 237 255 58 167 255 215 5 107 255 155 104 181 37 36 83 75 8 69 194 244 158 232 255 15 26 170 251 15 239 255 83 232 235 14 100 89 70 161 88 69 38 91 66 44 158 132 164 20 40 255 133 255 65 226 220 87 151 95 0 202 57 254 21 26 255 41 136 199 115 8 133 227 148 251 85 145 255 3 198 133 252 87 42 109 100 50 21 68 99 9 180 187 121 250 142 83 247 152 5 26 250 239 126 148 127 153 127 221 247 159 74 21 17 141 179 245 223 60 125 199 174 123 204 2 109 253 179 126 31 83 207 156 43 138 188 60 38 108 183 59 40 150 234 72 165 115 8 71 3 189 241 159 67 247 152 5 218 251 95 30 247 81 221 239 247 7 89 254 55 91 10 249 207 32 24 97 227 255 12 178 141 147 186 199 44 248 36 253 175 52 8 204 127 181 214 66 48 28 64 52 177 136 90 59 136 76 109 66 247 152 5 218 250 255 221 62 192 138 255 82 185 138 92 97 9 14 247 52 226 25 59 162 217 211 186 199 44 208 208 255 135 94 203 250 187 189 246 191 92 130 199 235 132 221 101 128 55 48 13 135 247 3 221 99 22 104 233 95 233 181 249 231 191 20 122 171 209 104 33 153 78 97 206 101 195 209 19 239 193 100 59 137 51 134 125 186 199 44 208 210 191 204 231 122 207 157 3 236 80 247 191 94 107 35 155 41 194 187 184 128 211 19 39 112 240 240 187 48 204 30 199 156 199 168 123 204 2 45 253 183 207 243 207 220 55 27 50 242 185 50 130 161 8 166 103 140 56 126 242 24 222 222 187 11 139 62 15 36 169 169 123 204 2 13 253 43 45 117 188 79 121 47 181 186 228 190 195 215 123 194 209 16 108 46 43 142 156 60 138 3 7 223 135 115 206 139 46 155 22 160 118 65 239 152 5 26 250 103 82 169 253 103 211 62 229 82 3 133 98 13 225 72 12 14 143 29 39 38 142 225 172 113 18 78 151 7 74 91 117 223 42 75 186 199 44 208 208 63 213 125 185 221 66 163 214 68 138 242 222 31 8 193 98 179 194 56 59 141 169 233 73 170 3 145 94 135 144 149 9 42 18 74 67 247 152 5 26 250 39 165 141 90 157 223 227 17 12 6 97 178 204 226 204 217 211 176 57 236 252 190 79 213 61 243 78 5 64 46 211 231 211 186 199 44 208 14 54 244 171 85 170 72 167 147 112 185 237 148 247 83 112 121 156 252 158 127 117 18 8 170 123 133 185 143 35 104 123 83 247 152 5 218 209 33 181 181 90 3 241 68 24 102 203 52 236 78 51 26 173 234 242 28 16 175 251 109 114 47 69 224 51 111 199 254 215 190 175 123 204 2 13 253 179 177 126 189 169 250 183 25 224 246 58 72 123 111 63 160 196 252 203 232 86 226 176 79 188 138 253 175 222 131 253 47 223 173 123 204 2 237 96 243 124 44 255 19 169 40 28 46 19 230 220 86 52 165 6 58 82 127 43 72 11 243 166 131 120 111 235 102 188 253 226 221 56 244 202 183 116 143 89 160 29 44 207 217 58 207 82 54 137 5 191 27 179 230 105 164 150 210 43 219 2 165 18 242 81 3 246 109 189 23 239 191 254 109 28 121 237 155 186 199 44 208 214 127 167 211 229 107 124 129 240 2 239 255 179 241 63 111 251 217 178 64 183 138 86 222 130 51 251 30 193 225 109 223 198 241 45 34 255 7 9 230 152 205 249 179 125 254 145 120 0 38 187 25 110 143 87 221 19 192 167 132 169 47 40 45 194 53 245 44 14 191 241 29 156 216 34 218 255 65 66 205 113 160 209 172 32 185 20 133 213 105 193 140 197 140 150 212 237 237 11 104 208 117 16 71 212 181 29 71 222 248 30 78 188 254 183 186 199 44 208 208 191 172 250 151 218 117 100 242 113 204 205 219 49 53 107 64 190 92 235 237 13 97 243 195 121 20 35 199 113 114 247 67 56 185 237 187 186 199 44 208 208 127 47 255 249 89 143 165 20 230 3 14 24 44 83 8 197 146 106 250 243 57 0 26 15 20 92 48 29 122 6 167 118 220 171 123 204 2 237 253 43 157 22 202 53 234 3 198 220 48 57 141 176 123 60 104 247 251 0 236 51 173 44 188 147 219 113 122 231 3 186 199 44 248 4 252 119 91 168 54 151 16 73 249 224 240 216 96 52 91 32 73 124 105 80 253 76 167 129 144 253 0 78 237 126 80 247 152 5 26 251 231 211 125 13 212 59 57 234 3 36 225 116 207 193 98 177 32 159 45 169 253 131 142 218 15 200 132 167 48 117 240 97 221 99 22 104 235 191 219 243 223 80 242 40 148 178 88 88 88 128 205 106 70 50 146 224 215 134 186 246 47 163 154 119 194 120 244 81 221 99 22 104 232 191 183 247 191 139 22 181 247 21 84 235 37 126 214 171 221 102 129 215 229 86 107 3 255 12 53 6 157 16 76 167 31 215 61 102 129 246 254 217 62 64 185 91 71 163 85 231 103 189 57 29 115 48 207 154 208 106 246 247 6 179 125 98 113 184 140 207 234 30 179 64 67 255 114 223 191 76 77 65 11 77 169 197 159 247 51 239 246 194 104 152 230 123 64 20 190 63 180 65 36 144 246 239 210 61 102 193 39 225 95 225 235 190 236 188 143 108 174 0 159 47 0 163 113 22 161 72 16 29 170 253 10 216 124 80 10 168 24 116 143 89 160 161 255 254 248 79 81 212 37 31 106 15 138 165 26 34 225 4 102 103 205 124 47 144 172 180 248 121 64 236 252 55 116 22 116 143 89 160 161 255 222 66 79 167 211 89 190 21 176 82 109 34 17 95 130 201 100 193 140 201 216 219 15 36 171 107 65 212 7 212 59 102 129 150 254 201 107 111 179 7 115 223 150 169 210 215 37 36 19 25 216 108 14 156 157 154 68 190 88 80 239 11 237 82 31 176 19 211 61 102 129 150 254 219 203 247 128 244 253 179 123 190 99 209 20 204 102 43 38 167 166 81 40 150 249 62 49 62 17 32 252 15 20 231 251 167 158 62 121 46 149 235 8 4 66 212 255 51 194 98 181 163 209 148 123 115 0 244 57 57 160 123 204 2 45 253 203 189 9 62 213 191 212 238 34 151 47 195 235 245 192 96 152 164 223 23 249 30 65 117 63 24 141 1 27 86 221 99 22 104 232 191 211 155 0 38 255 212 11 68 179 213 225 231 189 56 93 86 24 166 207 32 20 138 172 236 5 98 247 0 20 142 234 30 179 64 75 255 43 227 127 153 234 0 235 251 177 179 222 236 118 35 141 255 38 248 217 239 203 247 1 116 115 40 135 182 232 30 179 64 67 255 253 227 127 187 29 72 114 139 143 253 216 125 223 54 219 20 172 214 9 180 91 210 138 127 37 142 136 237 73 221 99 22 104 232 191 63 255 195 86 128 218 18 239 251 249 253 126 114 63 73 53 192 64 181 95 94 217 11 44 133 176 104 16 235 255 131 4 91 215 161 94 31 215 219 104 43 252 89 143 193 197 5 204 76 158 70 50 22 228 235 66 252 76 64 54 68 40 7 96 62 240 215 186 199 44 208 210 127 141 251 231 179 123 13 25 169 68 18 11 110 23 102 39 39 145 95 74 242 117 97 182 55 8 52 46 144 211 179 48 237 251 134 238 49 11 180 244 175 158 253 34 41 108 222 183 133 88 36 14 135 197 10 139 209 140 70 189 202 215 4 21 80 191 31 57 228 124 59 48 189 231 46 221 99 22 104 233 95 189 215 147 154 126 148 138 117 4 253 33 24 13 51 112 216 92 232 116 21 238 159 185 7 22 17 54 63 73 254 215 235 30 179 64 67 255 189 185 29 169 1 228 51 85 248 60 126 76 157 53 96 97 49 200 191 37 243 250 192 238 7 156 198 252 196 15 48 187 103 173 238 49 11 62 1 255 85 5 217 68 17 30 199 60 166 167 140 136 167 151 32 47 251 143 83 223 111 31 230 142 252 21 236 239 172 211 61 102 129 198 254 89 247 190 36 35 29 89 130 195 108 135 105 214 136 74 179 74 111 179 95 18 247 95 143 188 140 185 253 127 9 239 222 97 221 99 22 104 232 95 81 187 128 141 114 19 177 96 20 150 153 89 88 45 38 190 27 172 205 198 126 220 127 8 165 249 167 225 124 231 235 8 236 17 249 63 72 240 238 95 71 65 181 92 129 223 59 15 211 244 36 252 190 121 26 238 177 190 159 220 219 247 231 65 196 248 247 240 190 55 140 228 129 91 117 143 89 160 161 127 102 184 81 67 42 29 131 219 101 131 213 100 64 58 21 227 251 192 58 93 153 198 254 37 170 15 6 4 103 238 198 252 129 27 144 56 184 70 247 152 5 218 193 142 118 171 214 43 136 167 130 176 207 77 195 56 61 129 98 33 167 238 247 224 175 50 242 233 119 48 115 244 235 112 31 89 131 212 201 171 117 143 89 160 29 45 74 244 114 163 130 104 154 252 59 141 48 83 254 43 82 91 61 235 181 231 63 17 127 27 147 199 254 28 115 39 110 69 244 204 245 186 199 44 208 14 190 223 167 86 69 40 30 132 195 101 193 188 199 181 124 246 143 76 215 65 7 21 180 100 43 98 161 167 224 157 253 38 60 199 70 116 143 89 160 29 172 157 111 52 37 68 147 41 216 93 78 204 59 93 232 52 90 189 71 64 41 104 117 107 234 252 15 140 40 166 158 195 226 212 223 232 30 179 64 59 248 249 95 29 32 91 168 195 23 140 241 249 159 192 252 2 138 153 164 122 79 88 111 103 16 248 253 31 65 52 147 91 117 143 89 160 173 127 222 6 84 219 200 231 235 136 132 226 176 206 154 176 224 178 35 151 77 114 255 77 182 74 44 247 206 2 194 162 238 49 11 180 67 233 61 251 143 181 3 252 185 15 249 34 162 97 26 11 82 59 96 54 207 242 179 0 37 185 221 123 68 148 186 22 164 119 204 2 237 56 247 197 206 1 108 82 95 160 80 96 247 128 199 248 222 95 179 217 140 120 60 202 207 135 66 239 62 80 189 99 22 104 239 159 221 223 163 210 123 238 87 145 198 125 137 20 156 78 39 28 14 27 252 1 47 42 213 60 175 1 122 199 44 208 214 191 250 220 87 229 188 103 255 181 90 109 148 203 85 100 179 89 204 207 207 243 182 128 61 255 167 84 206 234 30 179 64 75 255 203 7 125 158 247 234 246 206 5 46 149 42 252 26 240 251 23 97 179 155 224 246 136 231 255 14 18 231 250 239 215 255 115 95 172 45 96 231 195 179 251 0 130 65 63 111 11 244 142 89 160 29 231 62 239 89 109 7 86 158 1 218 255 157 181 5 213 106 29 149 74 5 177 88 12 54 155 133 95 7 81 182 63 152 253 125 254 124 16 118 78 108 158 175 21 3 31 160 211 222 140 82 229 46 52 90 215 82 29 249 2 148 206 231 168 160 92 68 23 212 42 40 173 85 144 235 159 65 187 250 121 180 202 68 233 82 52 242 151 160 150 253 34 106 233 47 162 154 252 18 202 177 63 32 46 71 41 124 25 138 161 203 81 12 92 129 130 239 42 20 188 87 33 239 185 6 89 231 26 44 57 8 219 181 72 153 191 138 196 236 87 17 55 94 143 132 225 70 196 206 222 140 200 169 155 16 58 113 35 194 199 111 70 232 232 45 8 28 190 5 190 131 183 194 247 222 237 88 220 247 71 240 190 67 236 189 19 158 183 215 193 181 107 61 156 59 214 99 238 141 33 56 183 13 97 238 245 97 56 94 29 133 237 183 35 176 255 118 35 172 47 109 132 249 133 13 48 61 191 17 179 47 108 196 204 243 27 48 253 235 13 152 250 229 24 206 62 59 134 137 95 140 227 204 127 141 225 244 51 163 56 245 31 163 56 249 239 99 56 254 243 17 28 123 122 20 199 158 26 199 209 39 199 113 232 137 81 28 124 124 12 239 255 203 56 14 60 54 134 247 254 121 28 251 126 54 134 183 31 29 195 91 63 33 30 25 197 206 31 15 97 215 67 235 241 214 67 35 120 235 71 67 216 125 255 16 118 222 191 30 111 108 26 198 214 77 227 120 121 211 31 227 249 31 254 25 118 255 252 239 16 155 220 69 63 207 52 255 185 179 253 217 236 222 29 182 102 159 46 100 224 12 184 17 72 44 34 85 10 161 33 199 233 253 40 57 97 207 113 10 16 62 62 134 103 180 225 249 63 235 127 223 191 162 172 124 205 174 3 246 156 32 118 13 120 189 94 216 237 86 44 250 125 252 204 32 126 7 1 219 39 206 14 13 68 140 216 75 95 254 3 234 210 58 72 157 213 220 127 183 253 153 158 251 85 104 215 46 226 254 155 197 139 185 251 122 110 53 170 75 171 85 247 241 47 161 20 37 239 145 203 80 8 94 134 188 255 114 228 22 174 64 214 123 37 114 238 175 32 227 252 10 210 246 107 144 178 173 225 238 147 166 235 184 251 168 225 6 238 62 58 113 11 194 39 111 66 240 24 113 228 102 238 222 127 232 54 44 190 255 181 101 247 243 123 214 17 119 170 238 119 14 113 247 246 173 235 85 247 91 70 184 123 235 75 163 203 238 103 159 219 128 153 223 140 115 239 134 95 141 99 242 191 71 201 251 232 71 186 63 241 111 163 220 253 209 39 199 112 228 137 49 124 240 248 8 121 31 237 121 31 197 126 242 254 238 79 70 176 247 209 17 238 125 247 163 196 195 99 216 181 121 24 187 254 113 72 117 255 224 122 238 254 205 7 70 176 229 190 81 188 248 131 141 248 205 189 127 138 61 207 220 131 130 235 8 185 207 144 144 28 63 151 75 234 178 121 122 133 187 119 45 122 225 139 68 144 88 202 161 214 146 249 58 158 210 63 227 133 211 129 220 105 242 103 62 174 60 247 241 163 253 127 212 245 192 218 131 74 165 134 100 50 13 159 63 8 235 156 27 46 175 15 133 74 89 125 110 8 155 49 234 228 123 215 218 78 250 191 254 2 213 230 151 233 239 125 14 138 180 10 82 229 179 80 170 23 163 93 94 133 86 113 21 234 249 139 41 239 47 233 229 253 106 84 18 95 238 229 253 21 40 6 175 68 209 127 21 119 159 159 191 146 242 254 106 100 93 215 112 247 73 203 181 4 229 252 204 117 72 76 255 33 229 3 249 63 123 3 194 167 41 239 79 49 247 148 243 71 111 37 255 183 113 247 60 239 247 127 13 11 239 222 65 254 215 194 179 123 45 92 59 215 170 121 191 157 156 83 222 219 94 39 94 37 239 61 247 150 23 199 150 221 27 127 61 134 105 242 110 248 213 70 202 251 13 228 127 156 187 63 245 159 35 220 187 234 126 100 217 253 17 202 247 195 255 58 66 140 146 127 53 239 251 238 247 81 190 191 75 206 185 255 71 135 201 253 8 118 83 206 239 218 60 138 93 63 162 175 31 164 235 224 129 33 236 184 127 24 219 55 141 226 149 251 54 226 151 247 220 133 29 79 127 7 74 194 66 63 226 156 90 111 187 234 205 91 204 191 63 18 224 207 106 60 126 230 24 108 108 29 103 209 137 112 196 135 72 116 17 209 62 17 63 253 217 143 112 52 176 204 239 227 159 189 88 61 144 36 153 247 11 83 233 28 253 63 17 88 157 11 240 80 255 48 153 77 244 246 11 19 93 54 87 56 131 142 242 93 84 27 55 161 37 93 2 185 185 138 215 251 78 249 82 242 127 17 154 249 85 228 254 98 84 51 151 246 220 175 230 238 89 222 51 247 121 255 149 220 125 102 158 112 50 174 226 53 63 105 189 6 113 211 26 94 243 153 251 248 84 223 253 13 170 251 19 55 115 247 254 195 183 242 154 191 112 128 216 119 59 121 191 29 243 123 239 128 251 173 59 184 251 185 29 107 121 222 59 182 81 157 223 58 12 235 150 97 238 222 252 210 136 234 254 127 198 169 230 143 171 121 207 220 83 189 63 215 61 203 251 143 114 127 244 169 17 238 158 229 61 115 127 168 151 251 125 247 239 60 50 194 217 251 48 185 255 241 157 188 230 239 164 188 223 201 188 223 63 130 157 84 239 223 220 116 39 182 253 112 8 175 223 63 142 231 190 191 1 47 110 254 6 28 135 94 162 31 109 154 255 108 89 91 205 236 179 125 251 139 190 0 166 12 70 254 220 62 54 87 147 33 15 177 184 159 250 237 145 243 96 239 179 231 124 44 101 51 72 101 115 156 223 215 127 255 197 234 64 173 42 33 157 170 195 237 141 96 198 102 130 211 103 71 163 83 225 247 140 240 179 2 48 71 60 134 90 253 46 52 27 151 67 162 186 223 40 126 22 114 233 18 94 247 107 185 207 163 190 244 5 84 83 151 114 247 165 216 234 229 246 158 215 124 31 171 249 151 35 237 190 12 75 115 87 18 87 35 109 37 239 166 107 16 51 174 33 40 255 13 215 145 251 235 16 153 184 158 220 223 136 208 113 230 254 102 248 63 80 219 123 255 1 170 251 251 111 131 247 221 219 224 217 115 59 60 228 222 189 235 14 238 222 177 125 29 230 122 53 223 254 26 185 127 101 24 150 151 134 200 251 48 76 47 140 96 246 249 81 114 63 74 238 71 49 69 245 126 242 23 253 246 126 244 67 53 191 239 126 88 117 255 196 16 121 31 230 254 15 62 166 178 255 103 35 216 247 211 209 101 239 123 254 105 132 160 58 191 121 29 213 252 181 84 231 213 124 223 185 105 4 59 238 27 194 182 251 238 196 107 196 43 247 141 225 217 239 141 227 213 159 222 141 118 116 150 106 107 149 251 98 238 43 212 212 46 248 82 176 89 188 112 59 131 72 196 51 40 228 202 40 230 75 124 15 71 91 170 163 90 41 112 42 149 18 239 191 149 43 13 26 215 75 40 150 21 20 43 242 42 250 103 4 2 129 64 32 16 8 4 2 129 64 32 16 8 4 2 129 64 32 16 8 4 2 129 64 32 16 8 4 2 129 64 32 16 8 4 2 129 64 32 16 8 4 2 129 64 32 16 8 4 2 129 64 32 16 8 4 2 129 64 32 16 8 4 159 30 255 11 35 86 51 79 80 240 225 92 0 0 7 92 109 107 66 84 250 206 202 254 0 127 179 74 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 120 156 237 157 135 114 226 72 16 134 159 235 158 252 158 227 194 250 54 24 16 96 19 4 34 8 17 21 250 166 103 36 33 28 182 103 29 80 109 243 83 245 237 96 160 182 198 254 233 233 254 39 72 68 5 149 143 157 97 83 194 207 115 226 247 10 211 156 63 147 219 127 139 162 124 90 52 223 123 219 163 104 153 247 62 218 238 255 187 127 255 250 143 112 34 167 59 115 168 127 185 179 254 252 36 165 234 59 112 249 7 204 223 220 186 255 166 189 246 247 238 255 7 232 127 17 223 41 189 174 49 191 126 40 73 221 219 121 243 189 28 237 213 219 15 224 181 113 252 217 32 217 28 31 78 79 62 252 65 125 1 215 231 165 100 248 66 146 44 232 72 231 220 144 158 63 135 135 190 199 19 253 243 60 167 52 219 149 236 41 203 50 202 204 87 160 38 43 208 182 213 90 178 55 183 231 156 206 99 122 250 236 59 112 58 22 180 223 239 41 73 98 138 227 149 33 166 77 188 55 156 40 94 31 221 243 205 198 176 69 219 74 203 172 223 220 18 37 37 213 119 32 191 208 127 187 221 211 122 189 161 40 138 40 12 195 146 57 133 211 5 77 39 17 77 167 51 195 20 180 198 248 93 56 191 239 244 47 26 117 29 215 246 169 249 49 73 118 86 239 241 120 76 253 126 143 186 221 123 10 250 247 244 56 10 168 219 251 102 95 3 109 210 255 41 65 16 252 148 102 77 87 20 89 29 251 77 253 39 147 208 126 182 40 61 192 33 157 152 54 52 112 59 47 137 84 182 133 101 246 106 219 110 255 42 150 239 160 10 248 198 92 79 165 127 154 219 60 195 99 126 16 116 173 246 252 59 255 249 229 15 160 132 102 157 127 161 127 225 244 231 252 63 26 141 232 190 243 213 188 28 155 55 135 173 247 25 124 178 254 220 152 47 64 154 166 101 237 183 52 57 96 100 94 229 122 177 211 122 159 193 39 234 111 31 185 205 245 105 122 164 109 114 180 181 254 221 221 29 165 69 100 94 255 209 122 159 193 7 234 223 120 184 250 143 107 188 131 157 239 99 253 87 203 196 232 191 164 65 159 227 159 125 194 125 235 125 6 159 161 127 94 214 247 151 250 39 155 35 141 71 11 195 140 142 233 2 250 43 227 50 254 11 114 243 63 169 249 33 163 236 100 242 255 106 75 243 217 202 250 191 220 248 133 130 190 183 222 103 240 153 250 187 177 128 242 194 232 159 83 188 222 209 108 22 89 255 87 208 194 140 14 119 173 247 25 92 79 255 100 115 176 115 188 189 94 199 206 23 32 254 117 33 233 207 235 59 77 253 115 250 214 122 159 193 245 244 231 252 207 227 63 207 53 35 254 245 33 233 207 254 175 210 31 249 95 31 62 245 31 175 255 185 249 127 196 191 54 124 243 127 165 63 226 95 23 62 241 63 159 47 172 254 153 93 243 68 252 107 194 39 255 243 248 95 213 127 25 125 109 189 207 224 122 250 31 246 153 173 255 58 157 31 182 254 67 252 235 66 210 127 185 216 216 253 63 240 127 58 145 244 223 109 79 118 253 159 243 127 90 204 80 255 41 67 210 63 154 175 109 254 31 14 251 132 249 63 125 72 250 243 254 15 206 255 152 255 215 137 164 255 83 255 159 210 127 173 247 25 92 79 127 246 255 85 254 55 217 0 241 175 12 73 255 167 254 31 249 95 23 146 254 149 255 231 115 63 240 255 250 144 244 135 255 215 141 164 63 252 191 110 36 253 121 239 39 252 191 94 36 253 121 255 95 229 255 145 255 245 225 227 255 220 249 95 248 127 141 72 250 243 254 191 197 98 69 131 65 96 243 63 226 95 23 146 254 60 255 55 30 79 173 255 67 253 175 15 73 127 246 255 188 255 135 245 199 252 159 62 126 181 254 135 254 186 240 137 255 106 255 15 199 63 246 127 233 66 210 159 215 255 121 253 143 235 63 196 191 62 124 252 95 85 255 243 254 95 204 255 232 66 210 159 231 127 154 245 63 244 215 133 164 191 61 255 111 234 127 94 255 65 253 175 15 73 127 246 255 56 255 175 23 73 127 222 255 135 243 255 122 241 137 127 156 255 215 139 79 253 143 243 255 122 145 244 111 158 255 231 250 239 72 95 90 239 51 184 158 254 56 255 175 155 95 205 255 168 255 117 33 233 15 255 175 27 223 250 15 254 95 39 146 254 184 254 159 110 124 242 63 215 127 240 255 58 241 25 255 57 255 99 253 87 39 146 254 236 255 43 253 145 255 245 225 19 255 205 243 159 136 127 93 60 215 63 111 232 159 214 245 31 223 243 13 254 95 31 175 234 223 188 254 63 251 255 65 167 244 255 216 255 167 9 73 255 56 118 243 255 28 255 46 255 255 219 122 159 193 245 244 231 251 63 90 255 31 252 40 243 63 226 95 19 146 254 238 254 143 243 90 255 28 231 255 84 225 51 254 91 255 55 236 34 254 21 34 233 191 90 197 181 254 200 255 250 240 137 127 235 255 7 29 196 191 66 124 235 191 179 255 71 252 107 66 244 255 235 141 187 254 127 255 30 254 95 33 62 241 95 205 255 184 235 127 96 255 159 38 224 255 111 27 159 250 143 245 239 246 190 195 255 43 4 254 255 182 129 255 191 109 124 234 127 228 127 189 136 245 95 181 254 95 223 255 15 241 175 137 151 245 79 107 253 249 250 207 124 13 40 222 255 5 255 175 143 75 253 179 103 250 111 147 19 205 194 37 245 122 238 254 191 216 255 171 11 73 255 229 34 161 201 152 247 255 225 250 207 26 145 244 63 29 201 198 127 167 131 243 31 26 145 244 159 133 43 10 167 11 122 120 120 32 236 255 212 135 164 255 110 155 218 107 0 114 254 199 249 79 125 72 250 175 150 91 27 255 85 254 135 254 186 240 169 255 167 19 190 255 147 243 255 184 254 171 46 124 226 159 199 255 193 96 128 251 127 40 68 210 63 217 28 107 255 15 255 167 15 73 255 69 180 129 255 87 12 252 255 109 3 255 127 219 72 250 239 119 153 173 255 130 32 64 253 167 16 248 255 219 198 199 255 115 253 119 246 255 24 255 53 33 233 191 94 237 108 253 87 233 143 245 127 93 248 228 255 202 255 185 251 63 33 254 53 33 233 31 205 99 90 68 113 29 255 216 255 173 11 73 255 204 60 29 143 102 118 254 23 251 191 244 33 233 31 78 151 214 255 33 254 117 34 233 127 60 20 52 122 12 27 254 15 241 175 9 73 127 222 255 199 251 127 81 255 235 196 199 255 243 250 127 21 255 216 255 175 11 73 255 77 124 168 231 255 220 252 47 226 95 19 62 241 255 248 48 181 250 167 197 12 250 43 67 210 159 231 255 248 26 208 236 255 144 255 245 33 233 127 216 231 52 232 143 108 252 159 242 16 250 43 67 210 159 247 255 240 25 64 196 191 78 36 253 121 255 127 21 255 124 255 119 232 175 11 73 255 120 189 175 247 255 34 254 245 113 169 255 243 243 223 188 255 215 205 255 187 253 63 25 252 191 42 94 214 191 121 255 63 183 254 239 244 231 241 31 250 107 66 210 159 227 159 253 255 112 200 254 127 106 222 249 167 245 62 131 235 233 207 251 255 216 255 179 254 68 17 244 87 134 164 63 239 255 225 250 159 199 255 83 62 193 248 175 12 73 255 179 255 15 16 255 10 145 244 231 249 255 126 240 104 239 255 150 81 136 248 87 134 79 253 207 235 255 238 254 127 17 244 87 134 79 253 127 246 255 115 51 6 252 221 122 159 193 245 244 231 249 63 183 254 223 131 255 87 136 79 253 207 241 255 248 56 132 254 10 241 169 255 221 249 207 174 205 255 41 253 213 122 159 193 245 244 231 243 255 195 193 216 206 255 160 254 215 135 164 63 159 255 105 198 63 234 63 93 72 250 243 254 255 135 225 164 158 255 69 252 235 66 210 159 231 255 221 245 223 238 161 191 66 158 235 79 212 188 254 255 54 57 251 127 204 255 170 227 127 237 222 137 215 197 63 142 139 0 0 5 119 109 107 66 84 250 206 202 254 0 127 201 148 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 120 156 237 157 235 114 226 70 16 133 243 90 121 242 60 71 46 155 221 236 46 70 128 49 32 16 247 59 130 14 61 99 201 12 139 124 252 43 85 57 123 166 252 85 185 236 159 95 181 166 207 168 37 153 221 172 203 229 242 250 219 249 250 115 177 242 120 182 197 124 99 227 113 97 157 78 251 250 247 153 93 236 171 253 246 233 215 123 126 185 254 83 252 255 72 214 35 255 243 217 186 246 127 177 169 29 237 111 249 231 1 250 95 46 182 150 231 19 203 178 39 83 253 211 145 172 71 254 87 203 157 141 70 227 218 191 234 159 138 100 53 213 255 100 50 13 254 75 155 168 254 185 128 254 125 255 247 235 127 213 255 149 246 69 254 121 72 214 35 255 251 93 25 250 191 86 235 91 232 255 84 255 84 64 255 179 233 202 134 195 92 249 143 147 100 61 242 191 221 28 173 40 102 97 255 63 93 198 234 255 184 128 254 139 201 34 236 255 189 94 199 188 254 207 246 143 252 243 144 172 71 254 55 235 67 216 255 219 237 150 233 250 79 71 178 62 146 255 79 246 89 254 121 72 86 83 254 175 246 255 235 110 160 250 231 2 250 191 207 255 218 255 169 72 214 123 249 255 233 233 187 242 63 31 208 191 242 63 53 201 82 254 255 233 128 254 39 227 185 242 63 47 201 122 228 127 189 218 215 249 95 251 63 29 208 191 231 63 223 255 149 255 41 73 86 211 252 223 116 58 183 110 55 11 251 191 234 159 10 232 223 207 255 6 131 81 200 127 234 255 233 72 86 83 254 247 249 31 247 175 243 63 58 160 255 251 254 95 254 169 72 22 154 255 241 250 215 252 23 21 208 191 223 255 247 251 127 222 255 169 254 233 72 86 83 254 171 250 127 159 255 213 249 15 21 208 191 159 255 220 246 255 242 79 69 178 154 242 191 247 255 126 255 71 253 63 29 208 191 231 127 239 255 252 252 79 207 255 209 1 253 251 252 159 247 127 154 255 163 36 89 239 205 255 85 254 181 255 83 145 172 166 254 95 207 255 211 146 44 244 252 191 247 127 7 251 36 255 60 64 255 122 254 159 154 100 125 100 255 87 255 79 69 178 148 255 127 58 160 255 170 255 83 254 167 4 250 247 243 95 229 127 90 146 213 180 255 123 255 167 252 79 73 178 222 123 255 143 238 255 82 146 172 166 252 95 249 215 254 79 7 244 95 229 127 157 255 81 146 172 232 255 124 227 255 84 247 127 89 39 222 255 87 253 83 209 236 255 82 6 255 117 254 239 182 94 243 191 230 255 136 128 254 151 203 120 254 239 245 31 247 255 191 228 159 7 232 127 189 222 198 252 159 125 179 184 255 171 254 137 128 254 87 171 77 204 255 175 254 207 122 254 143 9 232 223 175 255 33 255 245 170 251 127 170 127 34 160 255 249 124 89 251 215 254 79 7 244 239 245 31 242 127 183 58 255 87 253 19 1 253 87 253 223 91 254 87 253 19 1 253 47 22 171 248 254 255 206 119 229 127 62 160 127 175 255 234 252 39 190 255 67 243 127 68 124 200 191 242 63 45 208 191 247 127 238 255 169 253 213 148 255 233 248 144 127 229 127 90 160 127 229 127 106 160 127 239 255 181 255 211 2 253 215 247 255 235 239 255 169 254 137 104 240 127 170 253 251 251 159 253 29 80 62 255 165 252 79 199 157 255 242 7 255 155 245 209 198 249 204 218 237 248 253 95 205 255 82 1 253 207 166 107 27 14 124 254 79 239 127 38 4 250 63 30 44 212 127 171 165 231 63 8 129 254 199 249 220 242 209 212 158 159 159 77 243 159 116 64 255 219 205 41 188 3 208 247 127 61 255 73 7 244 63 159 109 66 253 87 251 191 252 83 1 253 123 255 63 26 250 247 159 98 254 215 251 95 169 128 254 189 254 253 250 223 237 118 245 253 15 62 160 255 245 234 80 231 127 229 63 58 160 255 105 177 82 254 231 5 250 87 254 167 6 250 87 254 167 6 250 223 109 203 208 255 101 89 166 254 143 15 232 95 249 159 26 232 223 243 191 247 127 111 249 95 215 127 34 160 255 197 124 27 250 191 202 191 238 255 83 1 253 251 254 95 229 191 248 253 39 213 63 17 208 127 49 89 218 180 88 214 245 175 249 111 42 160 255 242 250 235 224 101 28 206 127 53 255 69 7 244 159 143 102 33 255 169 254 41 129 254 15 251 139 189 244 243 155 252 167 250 39 2 250 247 249 63 159 255 85 255 79 9 244 95 221 255 175 234 95 243 255 84 64 255 171 229 190 62 255 139 231 191 170 127 34 160 127 175 255 254 243 40 248 63 93 198 242 207 5 244 239 231 127 254 14 104 207 127 218 255 233 128 254 247 187 179 117 59 47 161 254 143 231 92 254 185 128 254 125 254 199 159 1 84 253 83 2 253 251 252 127 85 255 254 253 119 249 167 2 250 95 46 118 245 252 175 234 159 142 59 255 63 62 255 237 243 191 241 252 63 206 255 148 202 255 76 52 248 191 253 254 95 188 255 31 253 251 245 95 254 137 128 254 189 254 61 255 247 122 158 255 71 215 255 252 41 255 60 64 255 62 255 231 249 223 253 155 21 242 207 5 244 239 243 63 222 255 251 245 255 120 30 234 250 207 5 244 255 150 255 51 83 253 211 1 253 251 249 127 39 235 135 239 191 149 150 171 254 185 128 254 189 255 247 251 255 241 251 127 133 252 115 1 253 167 249 127 114 189 6 252 33 255 60 64 255 126 254 23 239 255 183 149 255 249 128 254 189 255 247 250 239 247 123 242 207 7 244 239 253 127 124 254 211 231 255 10 59 217 239 242 207 3 244 239 207 255 247 186 131 112 254 163 254 95 8 33 132 16 66 8 33 132 16 66 8 33 132 16 66 8 33 132 16 66 8 33 132 16 66 8 33 132 16 226 63 231 95 226 42 173 19 45 238 180 190 0 0 1 138 109 107 66 84 250 206 202 254 0 127 208 164 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 120 156 237 209 49 74 196 80 0 69 81 183 229 202 93 201 148 162 153 74 210 25 196 110 252 198 98 96 200 2 242 193 123 138 195 107 31 220 241 114 121 30 7 79 99 12 26 142 237 245 111 209 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 211 191 77 255 54 253 219 244 111 27 63 227 245 104 246 39 78 237 127 219 141 71 179 63 113 98 255 247 235 219 88 150 63 203 125 103 127 226 196 254 235 186 238 62 30 119 246 39 78 236 191 125 126 143 109 219 118 95 247 157 253 9 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 254 171 95 190 177 255 211 247 165 9 185 0 0 42 23 109 107 66 84 250 206 202 254 0 127 212 240 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 120 156 237 125 43 184 236 40 214 246 146 72 44 18 137 196 34 145 72 36 22 25 137 196 34 35 145 88 36 18 25 137 141 140 140 140 141 44 153 127 81 251 156 238 158 158 233 249 212 255 212 136 90 115 233 62 85 123 215 9 176 46 239 187 46 84 230 231 125 54 158 0 218 179 145 45 198 215 70 96 135 163 195 2 112 93 197 5 107 0 223 133 126 195 243 98 29 224 129 10 16 1 182 10 27 208 158 36 193 119 221 147 217 177 132 193 241 205 124 115 252 241 67 111 0 149 155 221 65 43 213 113 51 182 190 108 79 21 20 224 120 157 64 225 40 233 48 240 97 185 175 43 63 247 9 18 160 220 84 44 164 95 141 3 175 55 129 184 115 92 157 219 222 207 152 94 66 108 49 41 252 67 17 243 149 210 15 15 43 212 107 219 40 238 70 121 78 34 56 245 150 100 80 67 204 95 57 22 200 210 62 79 6 48 38 108 171 52 184 143 73 109 43 254 238 235 110 144 119 238 219 71 23 143 114 197 176 248 141 153 224 1 60 81 170 131 113 9 15 209 20 9 43 175 151 178 133 72 147 125 12 250 161 174 193 101 161 183 29 127 113 143 185 174 126 126 130 91 41 46 163 53 99 131 137 54 44 214 250 184 197 192 193 130 109 14 158 29 4 117 79 160 248 239 168 57 112 92 248 211 253 60 47 27 15 47 63 188 252 183 184 78 196 58 18 158 147 77 194 155 5 112 41 182 21 32 21 246 196 156 45 28 213 30 50 112 192 39 15 135 117 75 113 96 2 190 180 205 95 183 224 218 230 192 31 213 47 184 153 251 54 252 52 29 60 126 0 133 11 180 30 55 55 83 234 27 129 199 225 78 221 213 66 81 215 225 214 237 243 235 167 168 148 117 77 224 247 123 143 142 251 53 200 37 46 66 166 78 201 138 11 24 107 58 24 9 107 155 78 192 188 109 129 57 216 15 252 231 2 42 167 117 129 85 197 124 50 168 134 217 37 59 209 239 17 129 155 171 29 184 93 70 225 97 215 146 41 207 11 170 82 69 37 72 87 99 48 77 103 3 198 62 187 248 41 47 116 105 104 147 102 152 245 213 9 209 184 88 134 62 202 202 28 168 18 203 69 203 197 242 227 41 21 150 212 60 15 44 233 54 246 115 52 197 225 192 127 214 53 155 122 224 98 240 63 164 162 74 31 240 224 246 92 27 60 25 79 28 77 37 79 35 40 55 228 54 224 58 167 22 61 9 157 170 234 253 211 139 7 89 65 163 247 194 131 72 13 182 171 8 144 76 160 115 54 155 206 3 77 131 88 234 200 249 66 99 88 157 38 199 152 74 189 19 116 4 101 187 46 202 246 18 252 168 207 9 51 46 24 106 101 40 148 207 63 212 60 63 157 45 49 233 36 112 157 79 1 61 0 253 127 120 20 26 18 140 11 255 218 253 9 31 93 60 74 116 194 161 183 86 103 155 154 96 124 73 44 60 228 184 149 125 224 70 243 14 81 106 93 208 111 37 91 80 97 209 253 1 136 54 253 24 88 18 89 191 205 60 225 23 170 63 16 168 89 111 104 0 128 70 158 23 252 48 54 55 225 129 173 31 54 74 70 241 151 71 91 240 133 5 119 118 41 154 207 55 63 44 28 228 64 119 142 10 141 6 110 77 224 199 166 221 19 109 204 5 202 107 62 63 245 232 201 76 218 106 239 15 179 21 61 128 221 15 218 37 119 131 90 136 105 122 70 84 120 6 129 16 244 132 36 17 129 175 107 80 246 56 155 69 183 144 109 242 248 9 106 65 79 153 241 212 215 250 222 128 171 158 28 62 126 254 146 134 229 220 224 216 134 145 186 6 233 66 57 250 181 226 9 134 11 214 164 2 56 85 173 171 75 28 14 67 195 118 180 106 98 235 15 76 11 232 229 231 67 188 2 239 128 121 175 5 161 11 231 30 141 59 164 109 23 106 11 80 46 212 5 32 151 248 227 239 68 107 241 21 194 234 119 254 161 85 255 69 194 194 128 51 244 220 168 193 148 145 149 146 56 120 235 85 192 133 74 202 252 185 115 18 143 163 18 137 158 92 132 179 8 14 232 201 159 142 43 59 6 125 7 4 115 173 24 70 16 227 29 81 40 30 75 73 173 238 247 166 88 221 155 132 198 168 10 185 49 11 203 250 254 43 75 143 250 168 232 100 88 140 31 93 59 74 196 163 142 239 99 8 136 120 36 157 167 68 209 215 183 222 15 16 158 162 19 88 96 105 215 30 32 236 168 235 209 16 64 108 204 143 251 114 110 176 109 36 129 168 198 94 215 57 206 132 239 122 66 71 207 158 160 81 61 21 198 110 102 16 228 107 132 68 234 246 172 101 59 10 60 168 253 248 22 97 211 252 62 44 181 226 162 158 155 106 198 239 107 135 48 66 159 91 112 40 245 226 36 191 29 0 186 135 199 160 145 219 246 204 112 253 52 12 110 203 25 203 113 96 148 88 198 147 249 9 250 118 207 181 246 232 24 217 46 120 245 72 110 140 251 141 111 114 159 74 190 128 207 53 172 241 193 224 255 72 224 28 117 169 137 241 234 235 135 151 143 11 102 224 167 22 12 237 215 215 97 236 24 149 134 137 91 90 23 25 58 62 60 199 77 64 74 163 57 243 2 253 2 172 226 36 211 113 93 21 1 161 0 125 61 72 133 22 12 107 4 208 254 51 198 202 24 186 227 175 249 233 178 186 116 191 81 147 61 127 130 1 44 9 55 198 199 6 181 223 187 253 236 234 231 115 13 196 62 144 227 162 51 248 182 239 222 220 54 91 166 209 103 149 168 151 12 82 76 141 216 141 170 168 249 17 173 63 23 183 40 12 141 38 119 221 46 55 67 35 126 66 123 8 130 93 175 211 10 215 226 151 85 212 87 2 178 12 55 153 210 49 127 106 107 126 130 101 148 99 71 187 191 114 22 203 199 249 31 208 227 68 216 46 27 18 61 244 203 75 180 220 238 235 8 197 64 221 87 221 199 177 77 18 3 251 48 229 128 144 3 227 251 237 232 228 188 48 206 92 6 218 242 120 127 144 243 174 118 207 113 223 78 90 162 11 62 35 4 198 13 131 132 140 224 66 69 151 9 153 165 41 149 130 186 38 121 216 65 125 116 229 63 66 251 234 11 89 236 109 40 194 87 73 112 225 129 201 177 242 128 241 11 184 124 189 50 151 43 232 253 224 92 50 183 246 32 237 222 41 144 108 220 56 183 214 116 108 24 5 28 193 64 90 46 66 189 128 163 239 24 4 149 101 241 133 139 141 131 82 83 223 244 129 198 131 109 62 100 152 73 129 233 254 108 254 244 242 39 78 13 97 134 100 196 162 71 51 185 37 141 134 188 35 171 41 63 232 188 207 36 5 1 115 31 28 223 9 3 95 53 61 19 178 237 89 28 199 66 82 35 45 107 34 113 71 80 45 208 101 18 254 34 231 102 173 193 8 194 225 37 214 169 45 207 147 51 30 55 234 208 228 204 200 56 212 77 57 153 25 129 207 138 25 138 44 27 172 232 228 95 42 110 59 72 130 193 250 198 69 22 66 198 177 134 29 214 99 108 191 126 248 206 238 177 231 13 203 157 91 131 142 47 115 97 159 103 184 73 21 23 69 8 50 229 44 122 3 210 49 218 23 116 129 27 58 146 107 76 198 211 231 206 193 6 161 205 237 28 200 153 8 15 229 163 139 71 41 230 28 212 123 18 55 202 111 240 203 197 110 100 168 248 164 232 183 240 225 149 123 64 114 245 80 152 62 16 198 107 194 119 140 107 215 189 190 161 11 35 184 15 107 88 23 102 121 145 18 214 69 199 247 154 200 65 169 205 66 194 57 194 117 77 16 176 219 222 52 80 61 185 95 4 211 108 146 144 198 103 87 143 174 216 135 221 78 149 20 213 35 150 95 110 153 135 71 239 164 112 3 112 11 44 90 133 30 18 255 85 117 155 54 200 147 247 195 86 166 155 211 176 184 229 48 69 75 145 55 42 124 183 253 24 93 175 123 157 252 55 53 70 92 233 212 182 187 250 122 81 216 21 160 122 33 225 130 199 7 9 117 21 72 128 62 238 255 243 166 117 112 154 178 218 246 245 84 217 163 187 111 51 80 41 136 248 180 91 224 94 213 54 227 214 218 243 227 96 32 45 154 100 38 18 225 42 61 182 37 196 192 202 102 89 146 60 191 94 146 131 216 183 96 95 54 157 168 223 200 124 104 29 184 204 235 5 51 216 161 62 128 132 50 32 25 80 186 16 250 113 255 215 55 7 21 207 142 179 11 253 213 14 252 44 78 115 190 106 187 70 153 61 66 181 138 252 96 11 168 237 129 179 149 67 167 242 246 4 105 142 85 146 41 1 82 233 144 143 64 76 197 223 6 210 174 132 139 255 137 29 2 183 199 167 109 98 60 50 19 70 72 147 82 113 176 178 28 195 185 70 189 30 229 227 248 247 88 28 186 105 141 228 142 178 245 79 109 172 71 65 253 221 16 183 244 210 125 58 182 42 26 190 203 19 117 12 196 102 5 216 58 29 64 202 171 82 72 153 46 3 242 217 239 138 176 54 194 241 188 177 195 54 142 106 144 229 1 167 99 230 71 79 112 79 45 13 16 250 173 54 29 72 23 210 253 250 220 202 127 75 157 201 74 230 219 85 192 206 236 22 58 186 74 199 131 190 157 138 253 201 118 60 103 122 142 227 140 232 4 241 32 241 255 8 49 70 42 113 109 153 45 59 147 13 73 242 141 20 0 55 13 63 3 222 30 226 79 236 40 89 114 13 217 100 101 199 10 207 201 70 6 202 23 13 229 153 9 210 250 209 165 255 150 114 197 216 72 150 16 31 203 193 97 236 199 199 219 161 232 165 62 24 198 16 0 44 161 149 184 175 51 255 139 27 68 236 90 8 139 185 69 241 202 198 174 1 210 113 113 240 236 216 55 112 27 129 63 184 131 200 140 222 75 172 6 144 247 197 37 14 200 167 36 129 59 15 226 188 63 30 253 81 172 114 24 141 54 179 112 80 3 55 96 188 97 153 134 184 233 94 61 132 7 132 253 201 82 231 182 95 23 134 245 153 179 41 109 163 132 199 62 192 68 51 35 30 194 163 12 188 95 39 13 231 73 254 194 29 201 173 117 15 205 139 67 193 202 45 82 141 110 16 101 227 175 132 179 178 207 243 31 115 115 149 192 238 76 207 204 60 200 173 47 19 181 224 27 82 41 124 76 116 95 49 187 233 242 193 60 117 245 125 69 10 68 200 122 108 194 241 36 20 72 211 17 14 43 239 78 8 29 95 21 50 236 191 115 7 99 1 219 175 45 242 114 72 145 86 46 160 25 179 165 136 244 26 255 206 212 214 143 39 128 145 239 140 153 133 40 125 11 224 111 9 178 237 58 93 232 172 167 219 54 167 70 104 108 119 177 134 188 64 140 210 122 198 123 54 186 80 95 153 63 12 1 182 13 69 235 27 210 221 24 208 104 179 162 246 223 185 163 131 195 118 162 230 75 171 12 47 251 66 233 34 77 210 254 178 104 59 192 215 241 241 250 23 231 4 208 177 161 111 191 235 41 103 82 2 109 32 195 36 172 53 191 44 32 148 69 131 120 252 40 63 170 58 19 182 103 192 85 235 181 133 23 70 188 52 158 153 234 192 32 9 232 224 22 8 243 181 223 185 195 69 52 140 129 6 33 150 179 113 63 16 108 228 152 7 144 245 65 174 245 14 136 159 22 89 118 180 238 115 58 109 82 30 170 40 71 7 255 176 90 223 121 218 71 113 132 43 119 48 17 41 78 122 251 246 231 225 117 136 89 244 45 197 38 149 167 143 232 192 34 162 89 16 9 61 39 73 134 253 206 29 131 56 240 183 232 24 123 187 153 109 136 27 76 47 126 132 185 143 25 131 194 33 254 143 199 251 255 46 227 66 239 174 224 216 209 204 65 61 168 158 248 231 188 177 89 153 203 12 124 122 4 244 240 58 102 137 27 35 194 172 127 111 104 185 50 88 106 129 248 199 188 73 124 49 214 216 105 51 19 65 40 124 249 167 118 16 102 49 197 3 19 165 214 13 78 75 128 136 154 76 192 205 171 26 218 46 159 235 211 235 63 210 114 217 93 195 113 66 83 160 94 35 111 22 242 254 46 108 128 37 4 122 2 226 226 129 139 13 175 196 247 89 210 227 126 105 136 228 148 131 80 174 0 136 113 16 30 235 117 38 55 40 132 5 28 249 85 59 66 2 161 146 108 32 122 70 235 120 215 142 6 179 129 162 139 77 155 247 203 242 11 44 124 80 102 12 86 245 214 160 246 107 130 145 39 155 78 186 244 98 204 148 47 161 36 69 78 149 185 170 10 72 3 65 215 105 222 251 81 11 61 130 117 45 31 47 84 230 81 65 68 57 147 163 168 210 103 213 236 87 237 240 224 37 89 90 239 174 92 128 159 218 97 212 165 164 227 53 144 79 173 106 114 168 15 203 209 241 169 248 117 34 57 227 214 66 216 122 112 131 204 172 214 118 108 165 173 108 61 11 131 16 143 30 21 201 184 89 236 232 199 121 159 21 220 190 53 116 3 253 133 166 210 17 11 215 109 69 131 129 122 158 43 252 170 29 239 77 27 177 190 139 62 168 76 191 107 199 61 139 114 114 12 52 141 3 249 236 234 167 252 20 47 103 253 15 154 69 192 114 18 254 192 126 80 134 24 166 110 66 223 91 160 174 92 103 91 21 123 161 171 135 146 103 26 163 227 89 219 3 118 82 159 87 39 27 134 135 10 123 70 101 176 109 49 239 222 1 140 123 11 2 221 119 190 7 3 76 255 213 59 144 29 55 168 38 36 145 231 254 120 250 99 218 248 195 208 48 159 11 129 142 158 110 221 246 137 243 237 38 17 250 218 117 197 64 53 115 67 232 20 67 213 177 109 156 241 31 56 72 24 131 227 101 102 241 239 185 198 116 12 146 206 60 80 74 90 255 244 142 176 64 131 143 239 144 249 200 220 255 232 29 161 187 22 52 157 75 124 174 143 35 96 14 23 125 23 97 159 107 17 82 78 226 255 160 39 151 126 73 85 33 194 107 242 87 1 128 129 76 140 155 90 215 245 68 152 4 78 75 149 193 165 5 247 204 161 85 227 6 12 163 156 207 196 247 253 221 59 180 154 103 204 18 239 76 126 97 40 93 229 175 222 33 66 106 118 232 40 209 102 242 199 245 31 163 191 185 244 92 183 243 66 229 5 125 146 65 117 72 247 82 161 164 160 17 6 206 2 0 213 103 235 247 152 238 26 77 31 238 11 168 60 55 135 12 79 139 183 58 200 237 202 91 247 109 54 117 237 118 246 142 221 199 59 51 142 230 181 61 200 252 238 119 29 233 167 119 140 110 30 232 251 175 255 248 250 143 151 4 164 164 51 112 75 254 206 77 162 164 182 165 131 168 171 123 243 46 0 116 11 174 34 161 3 243 110 116 154 26 63 19 224 235 141 100 105 241 176 98 204 67 54 179 243 61 176 202 69 9 179 25 160 30 173 30 213 14 133 44 128 165 27 209 174 79 200 10 178 156 245 48 185 104 254 118 135 203 71 23 143 146 59 30 61 186 239 245 77 86 37 165 19 246 247 155 202 170 136 114 252 167 0 192 228 194 52 157 72 24 13 29 56 114 228 217 212 6 75 236 141 128 201 120 186 237 65 28 244 58 202 50 13 218 17 101 192 46 169 172 1 215 221 159 48 78 60 103 157 235 228 131 248 153 39 136 166 202 238 205 54 163 233 135 37 193 195 71 204 49 214 214 183 109 125 238 161 94 106 174 110 237 210 189 211 83 253 48 12 67 122 1 139 199 237 76 88 134 51 253 156 200 85 131 246 132 77 59 94 5 226 254 94 21 126 88 99 176 189 142 128 1 97 1 26 232 118 3 71 72 5 117 34 225 231 13 49 138 243 107 78 215 75 253 84 130 63 44 136 217 231 114 131 61 238 220 142 134 88 125 54 67 66 16 87 54 219 187 0 144 70 223 23 168 92 238 51 150 109 123 52 33 93 57 255 225 187 173 215 220 74 52 110 233 20 216 19 53 74 211 115 132 119 130 220 94 12 170 68 159 247 104 168 15 130 141 244 140 122 246 130 187 182 163 223 253 56 255 149 224 28 80 235 148 93 151 122 157 23 240 243 186 49 182 27 120 165 226 126 23 0 252 181 74 56 35 99 190 37 129 207 190 19 174 183 99 70 123 100 243 151 27 221 223 3 29 186 189 111 29 247 136 64 119 197 48 137 81 115 80 175 227 185 18 154 95 21 204 129 206 32 63 8 51 121 66 50 196 144 123 124 190 255 5 53 179 167 184 135 125 203 62 120 123 113 40 97 191 178 219 209 130 97 161 63 5 128 142 208 254 81 147 37 242 167 204 108 193 172 246 52 244 130 164 207 130 120 109 87 73 215 134 128 71 237 175 166 67 232 26 210 244 236 49 18 234 18 146 8 219 18 107 215 81 214 253 140 51 105 74 104 27 220 44 199 255 128 250 227 250 137 26 75 186 154 82 136 247 197 196 240 162 21 196 194 200 246 200 79 1 96 250 200 231 133 202 178 187 134 143 60 33 243 141 91 16 158 184 78 119 217 244 182 97 196 220 248 182 162 90 128 142 192 120 137 92 82 116 118 154 1 210 94 90 74 237 221 97 92 40 1 202 30 200 90 153 80 69 150 143 235 255 238 32 49 138 154 222 163 5 161 104 142 224 247 157 236 121 6 48 247 83 0 144 76 33 200 121 14 205 81 3 80 15 204 158 16 237 35 82 64 211 177 38 52 175 97 159 57 16 2 39 50 201 1 158 196 139 224 182 73 21 172 69 199 56 88 201 248 73 114 213 196 10 52 168 19 150 42 121 158 253 98 31 207 127 139 203 188 211 26 47 35 137 13 120 176 221 209 220 59 198 50 82 126 23 0 170 139 1 245 226 25 214 225 79 55 207 237 162 104 239 147 213 152 85 142 243 94 8 167 107 4 244 13 187 154 165 227 167 40 228 190 160 44 165 136 159 50 24 135 252 215 41 207 67 7 247 58 40 217 144 59 151 28 81 51 62 189 254 52 225 41 250 37 136 249 212 98 54 238 23 66 56 1 253 92 160 20 31 179 0 112 181 245 154 201 236 171 101 60 234 186 237 83 233 185 16 144 130 216 236 218 43 211 106 115 240 196 112 140 132 184 8 223 37 60 38 25 116 123 51 63 140 163 8 172 124 78 6 41 112 55 17 98 153 212 2 127 238 227 233 63 212 97 164 105 228 68 19 181 87 108 8 127 146 22 1 161 189 240 136 88 151 159 2 192 90 218 62 59 117 248 157 41 164 107 111 3 41 35 87 225 200 205 83 154 110 174 99 82 189 120 244 148 123 91 192 115 88 118 49 164 172 57 53 137 17 95 172 26 172 48 75 233 24 245 193 24 138 17 182 213 153 55 218 62 158 0 84 112 161 53 135 139 184 217 180 164 108 26 51 165 83 252 34 216 157 234 76 238 189 136 88 144 234 133 231 157 171 91 225 53 109 246 161 248 14 12 186 191 81 0 61 34 117 125 246 145 198 112 207 152 42 120 141 98 246 213 137 186 238 227 151 227 35 105 77 43 179 249 64 187 90 33 144 9 254 183 207 174 126 106 224 210 89 148 221 172 21 24 145 161 180 210 36 30 63 53 138 149 160 109 6 116 234 117 136 93 32 135 37 142 64 142 247 221 133 58 14 52 104 56 219 131 184 152 161 187 116 117 236 23 51 192 59 250 10 206 145 79 164 49 65 47 114 139 191 4 190 153 5 10 27 198 17 207 82 42 53 192 199 251 255 203 236 105 220 38 106 35 4 89 30 50 58 186 36 90 8 40 97 173 100 230 64 223 62 39 122 10 158 20 6 132 215 76 16 172 175 179 172 219 2 23 235 135 182 169 189 54 90 144 209 192 56 124 96 27 198 54 221 34 2 229 140 88 1 49 223 95 128 15 122 5 39 16 13 238 70 45 169 247 187 209 143 251 63 176 250 88 94 2 242 11 65 58 63 49 8 162 7 59 216 204 104 47 75 24 158 86 66 39 245 7 12 252 118 79 110 18 31 25 207 4 70 83 242 130 188 201 12 245 6 198 164 234 176 239 204 222 81 123 61 107 104 55 153 192 23 77 229 175 192 119 88 81 249 112 252 92 118 239 205 147 206 79 47 255 182 46 197 246 218 241 185 10 244 141 78 51 183 205 72 75 82 240 201 108 138 75 150 34 155 113 29 15 242 148 30 137 94 87 220 4 179 104 129 188 49 119 116 249 30 227 160 11 104 64 7 2 167 172 209 51 240 101 18 31 54 254 78 124 150 146 23 182 193 169 139 73 59 121 238 143 243 63 31 3 56 116 91 6 131 91 233 33 145 11 250 165 181 36 44 31 206 174 76 173 183 101 150 34 143 5 122 166 229 37 73 222 134 65 234 107 220 204 23 17 82 228 108 33 51 117 250 56 224 218 129 121 19 223 235 63 19 95 158 87 197 233 189 41 131 65 98 67 210 245 233 245 79 249 33 114 122 191 132 97 162 164 53 83 177 205 110 214 151 163 179 136 17 35 194 20 60 253 52 51 121 143 220 54 34 170 21 160 210 82 253 234 209 244 223 131 67 129 130 81 4 229 191 38 62 184 91 143 188 35 10 66 0 72 205 199 189 223 123 118 161 101 224 5 215 107 79 195 215 84 201 108 113 40 143 85 72 3 240 205 181 104 143 15 12 187 221 148 56 16 186 44 64 116 176 181 246 200 195 213 130 76 219 123 112 8 180 25 149 252 31 137 47 216 42 203 76 34 141 20 100 95 121 12 31 239 127 159 107 44 52 238 11 71 204 150 253 209 98 68 62 44 193 207 46 111 107 18 34 253 68 59 134 124 55 150 91 46 21 28 181 68 214 67 237 23 65 156 159 17 228 35 137 105 199 230 233 108 255 252 175 137 207 159 161 73 214 172 208 22 242 8 226 243 222 31 68 191 113 189 93 43 222 24 101 69 9 149 242 150 95 171 45 45 160 153 131 166 11 218 176 99 169 28 155 179 232 231 128 94 248 191 165 76 244 18 152 113 177 49 126 17 235 16 67 144 223 67 147 255 148 248 158 67 147 57 161 142 8 103 78 141 17 228 243 253 255 251 26 147 245 72 200 4 56 66 22 107 104 234 231 171 74 193 142 35 90 164 45 96 250 86 129 111 167 153 77 203 97 32 57 114 24 17 23 36 0 19 238 213 186 20 3 90 45 188 104 243 107 104 242 31 10 31 63 67 147 36 183 9 251 94 150 241 116 196 143 247 191 57 100 40 145 56 22 80 221 133 93 218 182 154 163 215 119 91 18 162 1 119 108 24 233 207 59 181 140 100 149 212 110 151 215 134 244 111 206 75 171 232 141 100 249 12 72 26 166 222 132 40 68 250 175 133 175 159 161 73 110 252 73 204 32 77 239 227 252 95 40 0 249 129 188 14 25 240 26 156 146 24 238 240 53 41 54 18 72 47 246 245 204 89 205 49 135 184 32 177 81 82 230 140 107 31 44 110 203 214 251 88 72 189 173 6 202 137 240 14 145 131 202 203 63 20 62 255 24 154 100 102 147 4 1 231 38 54 179 127 252 252 189 94 69 127 21 74 150 109 139 18 20 170 233 116 222 200 123 165 67 151 181 155 189 204 67 172 2 99 96 189 238 217 48 202 133 118 30 53 29 195 194 120 60 228 131 243 92 57 89 2 183 221 255 99 225 251 215 208 228 125 49 218 48 54 34 215 180 33 214 143 243 159 235 57 234 100 108 58 39 150 12 251 49 72 34 193 60 72 78 182 2 38 93 11 196 53 207 246 143 54 182 57 199 33 22 73 237 49 59 34 144 201 30 248 238 89 120 65 207 215 75 45 151 121 245 127 109 124 104 127 31 154 12 100 28 13 162 34 93 240 207 215 255 219 235 185 8 85 185 46 97 107 171 103 98 98 119 50 52 172 141 166 253 152 131 192 106 45 185 32 81 14 170 204 194 0 79 100 150 45 214 4 214 193 58 78 138 20 9 100 71 167 151 203 165 77 217 254 165 241 229 248 247 161 73 9 86 248 71 61 179 87 248 127 33 1 114 54 233 155 119 209 72 51 243 177 24 187 22 251 210 112 34 241 29 215 219 139 23 12 1 185 161 130 163 35 103 54 42 120 165 57 31 240 174 132 239 107 123 72 144 174 60 87 144 168 246 8 237 220 66 22 81 223 141 79 231 191 15 77 86 4 17 249 133 123 75 104 245 159 94 60 204 70 127 21 137 87 13 29 60 228 181 76 192 134 199 253 58 119 36 124 176 111 89 128 156 45 81 93 60 19 202 171 205 218 33 225 33 229 158 185 251 240 70 207 30 113 64 16 125 51 102 161 82 248 37 29 187 152 141 111 21 196 191 15 77 142 163 208 6 114 175 68 245 242 241 254 15 43 203 78 124 236 89 205 89 252 153 142 54 253 106 27 151 114 3 49 209 49 21 107 52 248 250 105 39 88 201 25 252 102 131 92 58 220 101 174 222 188 247 32 220 126 185 80 57 154 229 210 49 237 223 141 143 179 10 246 31 134 38 55 142 56 81 161 97 112 101 245 199 241 239 134 232 246 136 53 32 57 111 115 166 77 218 121 151 9 245 75 4 45 209 219 117 138 164 117 22 57 230 220 7 66 100 120 55 72 135 86 153 190 214 216 38 166 95 15 168 175 157 33 67 242 163 108 1 24 103 76 14 32 255 48 52 137 177 127 183 121 83 247 2 165 125 124 254 113 217 179 222 107 208 240 236 1 217 250 204 98 174 115 58 1 102 186 39 47 208 252 130 192 14 29 227 188 179 64 33 254 93 37 185 131 220 122 35 215 61 196 74 164 115 199 96 51 151 127 87 32 101 122 60 141 80 73 71 242 15 67 147 130 174 251 48 168 64 232 52 237 199 251 191 11 3 213 222 30 251 53 184 2 53 103 181 149 120 255 99 142 237 84 191 71 244 220 251 85 25 244 35 175 82 212 27 67 67 198 63 116 100 120 199 28 244 134 248 76 96 140 198 129 56 167 245 235 233 100 103 115 84 226 63 15 77 54 125 44 12 2 95 157 154 147 32 31 150 92 130 107 217 172 4 222 248 158 108 116 206 132 108 175 115 160 139 58 109 34 50 215 227 53 183 98 1 101 92 93 8 92 242 12 203 155 220 228 123 120 97 223 76 111 185 139 81 211 127 108 80 119 75 55 141 236 233 63 15 77 238 165 137 0 94 97 44 213 31 95 254 108 98 40 212 192 49 135 63 241 191 235 131 6 237 243 14 207 51 147 29 132 221 140 94 55 162 219 246 58 240 16 133 203 222 131 195 195 94 186 117 115 172 207 154 113 66 185 56 42 193 141 232 104 128 71 120 140 31 176 89 254 15 67 147 109 53 2 131 77 126 237 123 83 31 103 192 57 110 232 145 82 88 150 29 67 125 91 249 171 139 23 159 104 7 79 206 116 67 155 151 49 35 87 51 232 7 42 154 251 58 59 189 38 95 24 135 71 20 219 219 130 180 38 156 72 124 137 221 208 224 193 232 53 130 76 145 137 255 60 52 201 131 239 242 92 98 5 166 251 199 251 191 134 17 151 68 63 93 102 77 123 86 54 42 130 83 228 60 135 155 217 219 101 153 141 48 44 160 33 171 216 13 65 180 203 216 190 188 231 246 133 228 161 246 140 75 184 8 159 109 164 156 109 157 193 198 223 141 195 114 139 157 189 79 247 111 67 147 173 164 154 252 188 50 137 10 8 31 247 127 214 225 35 131 195 112 39 36 180 70 144 203 191 64 33 210 123 48 158 23 86 132 17 166 89 87 169 216 194 111 101 173 44 169 138 39 170 139 72 39 186 191 247 96 231 114 253 49 206 226 218 232 21 131 35 154 205 191 15 77 82 83 150 26 221 16 19 89 186 143 223 127 146 90 66 29 208 104 196 124 150 120 178 183 72 243 64 13 54 219 89 122 105 87 31 108 132 119 81 19 193 12 17 28 225 125 130 121 113 15 23 115 224 173 236 179 33 96 171 179 31 126 121 35 162 45 79 24 221 11 213 145 170 246 183 161 73 162 88 149 247 144 109 214 203 17 20 127 188 254 7 241 168 204 121 207 241 129 209 245 159 124 239 252 42 48 111 64 162 129 27 93 219 147 137 94 40 96 80 15 17 202 113 237 171 194 131 118 182 173 101 102 183 1 233 207 118 238 39 154 79 65 61 66 128 251 30 140 131 221 130 112 228 239 67 147 74 35 10 188 168 16 12 214 158 158 143 167 255 113 149 230 53 177 13 82 209 195 238 23 186 128 235 129 119 86 38 237 133 41 12 212 169 131 217 110 64 112 111 239 141 231 178 93 221 204 123 51 172 118 182 58 11 250 14 24 199 250 181 134 222 195 89 145 232 205 102 42 97 87 239 210 95 135 38 39 10 74 107 48 110 165 137 162 175 68 184 120 232 143 251 127 11 148 92 47 72 147 220 204 201 68 12 225 117 3 206 113 15 90 150 92 218 140 71 43 229 149 181 88 11 127 236 90 165 84 62 122 39 17 55 249 81 135 129 113 17 155 132 185 142 190 185 201 140 237 2 35 75 226 82 46 242 47 67 147 187 7 86 214 220 103 199 81 215 200 172 168 229 233 227 243 31 92 11 226 16 236 96 184 183 63 100 220 219 23 49 121 179 148 229 117 77 219 54 198 181 56 90 88 93 56 195 248 94 112 133 170 119 81 235 22 69 190 231 13 38 8 239 140 135 173 49 22 102 114 19 82 154 3 193 75 105 251 241 231 208 36 16 71 164 22 206 19 211 240 212 221 156 157 143 219 199 1 208 149 24 30 140 175 240 104 51 39 129 192 174 194 248 148 183 147 6 199 123 188 28 20 1 177 154 30 30 59 59 214 126 70 160 246 75 238 51 55 208 107 154 20 150 204 117 60 8 112 246 181 100 202 142 229 232 136 43 67 168 127 25 154 164 8 153 12 141 82 225 15 145 247 77 122 185 198 143 231 191 55 27 41 110 0 210 128 8 169 230 4 246 212 32 108 69 3 14 71 29 27 121 205 108 189 135 58 18 204 145 255 73 141 222 237 146 111 31 177 66 236 60 65 92 142 140 154 92 54 143 100 240 232 253 70 166 224 113 31 224 207 161 201 133 81 137 91 226 181 49 60 107 168 238 221 78 195 62 222 255 151 83 37 124 235 130 206 208 149 239 115 1 84 117 198 108 101 193 245 146 98 154 65 125 65 184 235 230 237 22 158 205 180 254 49 227 253 27 48 192 65 123 31 201 75 249 147 202 152 27 20 211 186 188 116 106 209 7 242 199 208 100 22 76 73 61 114 158 9 17 15 162 80 82 103 243 251 199 243 95 76 98 184 82 179 163 233 13 197 158 235 16 108 63 171 226 144 148 158 41 33 176 214 91 171 137 70 194 230 8 111 214 231 11 119 105 38 2 107 94 148 67 86 40 116 64 112 252 87 50 211 247 178 14 243 123 104 210 120 129 72 151 71 82 110 178 4 205 137 140 101 17 4 67 98 250 120 5 212 137 176 129 170 233 54 71 193 179 169 247 29 145 172 141 81 133 214 100 50 55 92 204 216 171 100 156 83 145 27 201 61 92 187 70 2 167 102 42 188 48 233 145 29 3 219 163 0 220 14 79 80 3 144 250 199 53 40 114 132 90 223 153 19 120 81 145 16 90 22 62 8 165 126 71 160 65 101 30 78 45 245 227 249 255 106 89 244 220 55 151 210 190 110 59 250 110 63 211 185 34 80 183 175 120 210 125 22 135 183 149 26 22 16 29 47 224 78 228 246 24 0 222 87 58 221 138 38 180 151 215 190 134 217 58 120 34 12 234 173 165 209 131 16 59 134 196 82 231 20 237 142 148 185 23 12 129 174 99 228 155 133 209 153 245 177 241 220 21 253 252 253 23 69 108 109 71 194 167 97 61 143 11 23 104 8 242 22 194 168 66 71 95 230 117 133 179 215 89 110 90 236 171 173 70 89 115 32 244 85 233 181 34 21 122 77 27 38 203 58 203 221 71 110 184 1 207 117 237 46 245 68 88 137 26 131 201 53 88 110 20 28 154 193 161 246 2 155 59 222 125 182 212 171 37 88 79 63 189 126 50 38 249 221 216 28 238 184 7 70 114 106 106 162 17 24 162 29 133 56 136 155 11 121 65 202 198 228 23 42 177 87 16 9 17 161 179 19 137 128 169 210 16 73 144 210 17 152 247 4 147 57 47 185 167 117 130 98 41 245 129 90 179 108 24 58 235 12 115 225 188 14 185 32 56 178 8 53 192 192 74 245 154 62 126 255 105 180 129 156 129 144 208 243 152 73 60 144 107 157 237 14 194 182 13 136 217 164 227 94 162 83 215 120 154 91 139 232 7 178 214 115 14 128 173 71 68 23 184 225 22 192 227 40 134 0 43 137 151 34 69 163 185 179 72 110 214 134 65 95 155 225 92 70 176 11 90 249 189 217 25 37 210 204 27 139 146 196 210 63 222 255 52 96 181 4 172 172 119 157 215 30 95 123 217 55 161 28 28 18 185 16 34 244 221 48 171 89 237 6 241 174 64 147 102 125 152 165 182 201 255 231 197 80 69 106 175 109 200 218 118 58 239 81 200 30 29 136 191 4 239 210 222 114 78 190 175 91 119 204 244 157 122 224 42 246 177 138 53 28 8 139 99 63 162 106 31 159 255 229 24 148 172 148 102 157 55 181 181 151 63 75 95 209 26 80 157 209 39 44 32 115 90 40 111 84 13 124 145 234 195 147 92 45 37 234 232 148 146 251 104 132 146 236 121 131 193 248 197 243 202 92 9 99 75 47 23 9 203 167 89 71 212 187 218 144 254 134 141 18 209 122 38 99 34 12 42 199 40 44 223 209 127 220 255 183 121 237 69 159 113 40 154 121 7 152 91 245 188 178 28 113 96 229 97 160 55 88 196 188 221 224 126 16 255 193 254 164 190 82 174 165 208 47 230 201 123 104 146 104 133 241 204 34 166 103 248 195 26 17 61 119 8 152 142 247 221 160 251 213 151 136 138 132 63 250 211 233 255 204 182 145 117 18 78 29 104 123 220 103 23 15 243 250 63 116 113 170 158 136 109 237 44 229 162 117 162 91 247 89 194 243 218 111 116 255 6 161 43 244 128 20 33 171 104 102 218 231 46 50 93 236 221 185 63 135 38 149 156 119 65 225 82 136 180 190 70 195 196 190 235 182 190 17 85 101 173 130 171 7 37 253 157 100 207 203 108 2 27 179 105 248 129 139 218 254 113 254 151 77 115 38 231 54 153 88 155 119 22 45 38 206 30 22 89 237 190 133 51 40 165 173 153 71 57 145 172 123 99 215 129 204 214 172 199 191 15 77 230 221 59 163 162 173 233 28 118 222 240 250 128 110 129 34 151 206 8 1 237 98 181 29 185 41 4 189 60 131 215 190 205 220 241 135 101 95 240 49 182 21 29 252 156 112 232 232 247 231 125 247 171 49 49 244 221 70 35 145 223 89 88 131 129 112 21 164 113 20 26 93 151 220 115 142 127 31 154 44 91 194 37 42 38 28 161 6 177 64 131 29 68 188 102 243 195 210 185 65 135 217 153 102 115 148 12 189 203 44 168 131 222 62 191 254 51 89 207 28 212 149 23 8 59 231 58 35 104 69 76 172 215 157 91 1 82 79 100 139 202 17 94 71 233 154 65 141 203 169 102 223 126 89 33 253 203 208 228 69 229 234 24 241 8 110 23 183 14 244 252 48 175 126 197 224 47 171 65 14 221 242 129 10 150 75 229 109 247 130 1 177 16 62 94 255 221 87 89 162 113 134 34 181 1 18 19 165 226 60 24 8 209 134 70 138 178 206 235 42 99 7 185 189 58 26 120 219 119 132 124 164 205 158 119 173 151 37 253 101 104 82 103 176 229 216 250 1 230 220 230 100 191 57 231 180 252 204 149 170 203 195 118 158 51 192 118 13 68 103 153 141 104 231 62 62 63 255 242 51 233 137 104 159 89 68 107 144 103 67 227 40 3 202 185 16 194 131 170 212 149 141 244 215 83 201 1 183 134 172 212 156 124 232 13 17 50 81 193 252 57 52 169 40 241 189 63 79 71 20 149 81 51 52 133 13 102 235 99 99 179 182 80 111 112 3 84 128 89 97 131 150 87 40 163 62 31 199 255 64 173 116 227 88 52 136 96 240 76 71 166 217 158 112 149 8 139 201 132 227 97 239 227 154 197 176 108 30 92 164 91 57 130 99 251 50 54 243 226 85 14 127 12 77 54 102 44 186 138 242 188 203 0 39 155 23 164 17 133 6 158 67 208 214 241 52 248 105 60 68 247 110 4 178 239 161 35 248 120 252 151 208 179 13 72 122 53 231 190 15 91 170 10 178 95 169 207 193 183 232 27 180 81 116 156 193 109 137 118 78 203 67 95 204 241 96 160 135 120 232 209 252 226 126 13 77 98 8 13 115 0 22 215 20 80 69 120 187 134 199 216 135 59 226 192 247 99 87 193 23 140 42 110 92 2 152 214 63 153 255 143 215 191 118 131 58 79 8 245 177 51 64 102 190 31 131 194 210 82 207 38 198 101 219 242 245 243 29 21 239 199 109 5 150 91 109 217 156 67 7 132 135 234 53 102 23 40 252 12 77 214 228 214 217 78 105 3 191 142 89 88 208 120 208 168 19 165 230 213 61 167 52 10 121 196 72 24 79 145 60 228 158 48 124 154 143 231 127 28 19 51 244 165 204 16 8 204 91 255 246 66 9 58 127 100 49 145 34 90 217 79 36 43 75 120 247 105 204 146 61 239 115 152 229 74 184 254 128 212 103 166 136 200 12 5 126 223 105 91 82 122 39 134 24 188 154 7 198 71 21 158 35 175 118 145 151 125 218 62 165 63 87 126 112 31 175 115 207 160 63 158 255 107 179 222 65 247 215 75 195 114 197 181 196 197 206 107 64 150 73 240 124 62 94 120 240 115 184 63 253 92 213 240 96 44 68 27 232 228 245 204 146 1 169 140 176 142 251 53 175 7 233 157 87 3 23 143 219 252 201 94 163 19 232 248 231 119 11 128 68 211 25 181 77 7 136 225 22 248 88 102 95 56 60 37 124 30 0 19 56 95 236 153 137 112 18 80 243 49 134 133 227 238 132 87 254 192 108 109 223 131 152 147 138 102 63 52 158 253 188 187 58 189 242 204 129 15 147 95 163 212 148 118 32 251 143 29 251 77 224 59 107 158 58 112 205 214 127 178 31 14 226 28 141 95 214 249 225 115 106 216 142 113 119 93 36 70 135 163 125 28 255 162 21 187 121 171 13 215 215 251 44 98 24 39 213 232 176 78 135 206 254 61 212 111 185 246 214 244 212 48 44 10 245 168 199 255 186 187 226 126 150 198 77 139 30 131 196 251 10 89 149 82 180 137 165 52 226 54 43 33 143 148 125 64 126 117 135 106 176 99 116 157 67 212 250 224 245 80 46 197 89 40 253 120 255 215 142 143 160 189 215 122 181 63 55 18 48 198 87 88 70 220 163 190 111 27 3 140 27 189 183 175 139 51 122 14 139 129 138 128 48 8 127 176 22 174 16 193 99 56 168 82 163 234 71 177 162 103 31 155 48 224 246 19 12 2 166 84 151 85 235 26 248 242 132 189 187 4 139 157 119 173 34 47 52 179 222 214 143 143 247 191 190 238 67 190 81 96 91 5 123 79 179 147 4 186 99 200 110 175 93 149 3 182 43 149 123 219 123 173 231 188 222 213 206 11 78 17 33 129 51 149 129 60 174 222 199 86 237 130 236 121 120 233 44 33 67 10 230 153 98 178 46 160 234 8 108 102 214 215 229 193 248 39 14 132 210 193 92 32 215 159 206 207 237 227 235 103 117 44 16 206 188 183 184 154 119 45 47 22 244 223 51 219 165 249 186 241 237 78 176 237 226 88 40 155 99 13 131 142 211 226 111 245 77 17 68 240 145 115 202 25 162 97 183 35 252 35 48 123 99 13 193 200 192 240 37 181 117 68 87 67 66 1 19 165 90 128 65 233 205 90 54 187 44 216 143 231 251 120 253 11 40 90 37 135 177 181 216 231 119 128 209 192 225 138 72 98 6 104 84 98 50 249 254 60 36 247 198 42 99 192 188 230 8 110 107 247 179 148 95 247 215 144 149 240 171 208 247 191 35 123 68 43 66 16 60 153 33 226 199 252 83 47 188 103 255 3 195 143 96 248 57 148 173 46 94 7 93 159 143 195 159 153 198 67 237 204 43 62 153 236 51 239 131 43 149 5 253 53 198 235 93 164 6 113 157 135 132 196 112 166 106 39 10 114 175 57 181 188 70 68 200 172 41 126 58 12 227 232 19 186 159 151 57 71 109 220 236 147 120 50 146 165 249 45 63 115 6 150 114 97 71 34 121 118 85 179 129 112 97 59 82 15 15 2 171 231 227 248 247 196 131 156 23 32 204 65 159 92 38 23 250 185 147 10 255 63 141 35 196 7 236 110 9 219 237 48 218 101 101 141 189 180 147 100 126 111 146 113 229 182 173 175 33 0 33 199 178 80 94 72 41 62 190 111 181 235 200 145 46 174 196 188 8 145 212 182 61 72 121 187 25 55 180 83 180 45 77 2 189 10 225 63 222 255 244 56 121 163 99 182 206 17 153 223 167 113 19 132 124 220 35 8 222 53 34 249 50 208 145 11 135 108 181 109 35 151 253 85 137 209 202 14 187 236 253 101 252 206 164 206 86 134 206 200 98 23 77 144 20 131 164 205 152 5 247 106 65 204 99 55 6 190 90 31 10 228 93 62 11 52 189 103 98 246 10 196 21 253 241 245 115 9 50 87 82 115 21 75 103 3 237 54 222 8 230 231 13 39 115 56 113 31 27 122 241 84 84 91 148 82 155 91 119 196 0 205 41 73 139 151 57 248 53 120 165 142 220 87 106 253 218 13 35 148 33 110 78 43 122 80 146 196 218 148 19 226 9 193 75 16 103 226 178 193 222 84 69 140 44 7 194 1 193 249 199 235 63 179 165 123 233 94 82 127 168 68 221 165 61 180 4 82 165 117 94 122 195 174 99 175 146 167 179 204 7 165 38 68 14 188 4 137 39 105 55 52 248 83 21 203 74 189 28 223 148 143 237 38 72 165 230 85 169 27 71 224 187 28 91 17 251 130 225 99 114 8 195 205 166 129 138 60 250 146 139 161 30 215 191 126 30 254 137 52 125 218 133 24 158 237 104 162 176 206 59 175 252 242 108 112 65 90 240 180 37 146 88 212 90 22 59 116 174 81 63 132 19 177 247 247 29 14 158 243 121 153 239 107 49 43 198 179 117 54 194 29 91 171 169 204 214 232 134 12 225 68 245 153 99 52 191 198 156 218 243 208 221 42 100 132 149 66 35 33 125 222 255 101 62 199 115 97 109 104 161 185 255 234 71 51 99 94 231 30 230 151 56 117 57 243 188 181 149 27 226 184 136 223 130 207 85 233 121 135 135 66 59 102 11 34 89 144 105 61 214 68 167 11 61 202 226 28 130 193 52 145 237 138 38 161 142 146 222 215 124 190 67 253 147 210 51 187 196 247 154 103 93 216 126 252 254 155 87 103 106 6 104 83 73 88 85 14 147 145 34 49 65 53 70 30 127 114 52 6 215 18 123 252 225 220 235 65 228 151 233 108 106 159 119 184 84 240 206 225 150 221 116 54 60 29 239 47 78 193 160 32 141 92 251 164 245 243 171 82 165 116 199 251 14 152 97 124 12 198 105 168 204 205 62 137 84 46 154 87 111 62 62 255 120 140 149 207 175 188 89 188 230 200 123 231 183 183 0 109 44 7 74 178 123 103 103 125 201 118 126 41 234 93 15 23 209 230 115 33 63 119 248 232 247 88 155 137 129 230 232 152 30 71 151 70 176 165 108 33 55 142 134 85 156 214 124 67 110 225 22 102 239 179 133 58 46 64 109 113 37 174 193 232 135 148 177 127 220 255 211 82 6 23 117 63 7 46 150 143 58 175 232 97 66 235 188 214 186 204 204 15 69 35 71 28 131 103 39 121 88 171 68 229 255 117 135 83 87 186 228 153 199 138 68 41 140 152 50 49 215 209 181 25 137 86 181 136 97 103 87 172 224 80 142 179 178 113 141 189 231 171 207 200 146 207 11 3 205 22 115 148 31 63 127 194 182 149 64 229 103 34 41 170 237 170 231 169 115 92 214 84 242 226 244 123 222 1 234 171 102 51 103 247 137 111 2 209 254 207 29 94 16 119 58 94 235 188 34 0 149 221 123 185 24 170 100 35 179 21 24 33 207 134 116 224 125 169 44 110 198 121 87 70 75 118 8 143 194 132 88 148 18 52 150 131 124 60 251 141 66 43 4 160 232 174 194 56 136 105 61 143 45 59 180 8 179 88 9 53 28 108 61 47 120 108 234 243 142 119 149 150 229 247 29 110 179 93 236 32 10 25 224 43 83 219 211 188 44 123 213 158 0 175 47 208 148 226 86 0 161 251 187 43 60 60 13 216 204 8 164 174 192 111 32 30 159 226 222 231 232 212 135 101 78 112 249 94 79 255 254 22 188 217 249 142 79 118 11 113 82 182 32 163 19 119 72 1 70 51 251 221 129 223 61 111 253 143 59 252 96 126 107 146 82 207 3 119 67 223 207 237 178 148 117 194 166 227 25 107 161 106 69 152 19 124 251 137 113 207 169 142 212 20 4 130 75 86 44 207 11 224 207 42 63 222 255 254 120 107 225 220 125 81 144 120 193 85 185 233 19 6 156 47 77 219 252 222 3 196 194 225 129 61 157 11 164 213 176 112 253 190 202 230 61 231 138 144 8 30 206 94 120 250 66 22 203 143 247 23 222 218 75 120 171 167 211 64 240 91 144 242 206 73 153 190 212 89 66 93 22 84 170 166 226 158 144 7 153 143 231 255 250 209 75 113 5 201 46 2 225 189 128 80 214 219 119 27 255 188 195 211 65 71 92 244 154 87 194 207 139 129 216 226 57 249 227 14 207 50 83 30 179 151 229 65 92 232 147 89 92 156 223 121 61 199 168 240 71 106 247 190 204 233 199 245 213 162 194 77 238 64 26 226 140 116 212 130 182 132 118 3 68 127 190 252 131 14 122 206 43 178 217 204 66 14 196 186 92 183 159 172 252 188 195 213 172 133 247 217 235 54 203 219 107 153 54 76 254 184 202 166 218 82 223 95 114 253 110 157 108 124 85 65 207 175 16 60 17 10 162 185 203 124 166 178 33 130 184 59 186 18 193 56 153 95 49 220 103 233 115 222 141 249 198 67 159 135 255 191 228 32 11 62 234 86 66 137 126 230 44 182 185 174 25 160 203 152 95 255 38 102 153 228 213 106 112 241 47 87 217 212 141 202 119 97 0 249 79 32 188 154 176 72 12 18 51 201 219 251 254 190 1 73 96 152 200 242 117 216 249 21 187 245 184 49 234 205 164 201 251 183 8 200 207 247 255 127 229 43 95 249 202 87 190 242 149 175 124 229 43 95 249 202 87 190 242 149 175 124 229 43 95 249 202 87 190 242 149 175 124 229 43 95 249 202 87 190 242 149 175 124 229 43 95 249 202 87 190 242 149 175 124 229 43 95 249 202 87 190 242 149 175 124 229 43 95 249 202 87 190 242 149 175 124 229 43 95 249 202 87 190 242 149 175 124 229 43 95 249 202 87 190 242 149 175 124 229 43 95 249 202 87 190 242 149 175 124 229 43 95 249 202 87 190 242 149 175 124 229 43 95 249 202 87 190 242 149 175 124 229 43 95 249 202 87 190 242 149 175 124 229 43 95 249 202 87 190 242 149 175 124 229 43 95 249 202 87 190 242 149 175 124 229 43 95 249 202 87 190 242 21 128 255 7 221 154 25 153 119 41 166 247 0 0 50 19 105 84 88 116 88 77 76 58 99 111 109 46 97 100 111 98 101 46 120 109 112 0 0 0 0 0 60 63 120 112 97 99 107 101 116 32 98 101 103 105 110 61 34 239 187 191 34 32 105 100 61 34 87 53 77 48 77 112 67 101 104 105 72 122 114 101 83 122 78 84 99 122 107 99 57 100 34 63 62 10 60 120 58 120 109 112 109 101 116 97 32 120 109 108 110 115 58 120 61 34 97 100 111 98 101 58 110 115 58 109 101 116 97 47 34 32 120 58 120 109 112 116 107 61 34 65 100 111 98 101 32 88 77 80 32 67 111 114 101 32 52 46 49 45 99 48 51 52 32 52 54 46 50 55 50 57 55 54 44 32 83 97 116 32 74 97 110 32 50 55 32 50 48 48 55 32 50 50 58 51 55 58 51 55 32 32 32 32 32 32 32 32 34 62 10 32 32 32 60 114 100 102 58 82 68 70 32 120 109 108 110 115 58 114 100 102 61 34 104 116 116 112 58 47 47 119 119 119 46 119 51 46 111 114 103 47 49 57 57 57 47 48 50 47 50 50 45 114 100 102 45 115 121 110 116 97 120 45 110 115 35 34 62 10 32 32 32 32 32 32 60 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 32 114 100 102 58 97 98 111 117 116 61 34 34 10 32 32 32 32 32 32 32 32 32 32 32 32 120 109 108 110 115 58 120 97 112 61 34 104 116 116 112 58 47 47 110 115 46 97 100 111 98 101 46 99 111 109 47 120 97 112 47 49 46 48 47 34 62 10 32 32 32 32 32 32 32 32 32 60 120 97 112 58 67 114 101 97 116 111 114 84 111 111 108 62 65 100 111 98 101 32 70 105 114 101 119 111 114 107 115 32 67 83 51 60 47 120 97 112 58 67 114 101 97 116 111 114 84 111 111 108 62 10 32 32 32 32 32 32 32 32 32 60 120 97 112 58 67 114 101 97 116 101 68 97 116 101 62 50 48 48 55 45 48 53 45 50 56 84 49 50 58 52 48 58 48 50 90 60 47 120 97 112 58 67 114 101 97 116 101 68 97 116 101 62 10 32 32 32 32 32 32 32 32 32 60 120 97 112 58 77 111 100 105 102 121 68 97 116 101 62 50 48 48 55 45 48 53 45 50 56 84 49 50 58 52 51 58 50 54 90 60 47 120 97 112 58 77 111 100 105 102 121 68 97 116 101 62 10 32 32 32 32 32 32 60 47 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 62 10 32 32 32 32 32 32 60 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 32 114 100 102 58 97 98 111 117 116 61 34 34 10 32 32 32 32 32 32 32 32 32 32 32 32 120 109 108 110 115 58 100 99 61 34 104 116 116 112 58 47 47 112 117 114 108 46 111 114 103 47 100 99 47 101 108 101 109 101 110 116 115 47 49 46 49 47 34 62 10 32 32 32 32 32 32 32 32 32 60 100 99 58 102 111 114 109 97 116 62 105 109 97 103 101 47 112 110 103 60 47 100 99 58 102 111 114 109 97 116 62 10 32 32 32 32 32 32 60 47 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 62 10 32 32 32 60 47 114 100 102 58 82 68 70 62 10 60 47 120 58 120 109 112 109 101 116 97 62 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 60 63 120 112 97 99 107 101 116 32 101 110 100 61 34 119 34 63 62 235 251 133 18 0 0 11 59 73 68 65 84 120 156 237 157 91 114 220 186 17 134 127 92 72 202 145 85 62 117 94 83 149 188 101 29 217 66 86 154 37 100 31 89 132 203 23 105 198 36 1 116 30 64 82 51 162 100 128 199 182 116 130 254 63 215 20 169 209 204 152 18 91 125 249 1 52 140 36 17 24 0 56 3 136 200 56 0 3 32 22 34 128 49 64 126 77 2 176 60 39 235 115 23 199 63 128 252 241 183 254 20 126 224 210 1 188 253 245 255 40 207 253 252 246 241 212 35 27 131 91 158 182 79 94 154 182 199 163 145 224 194 88 120 124 253 227 175 193 111 166 110 58 60 222 233 71 131 184 246 18 113 121 56 192 120 72 90 190 255 3 63 156 249 131 239 251 179 28 223 246 250 127 13 70 226 26 62 112 237 75 46 253 162 1 128 25 64 88 158 240 0 186 139 23 252 218 139 36 175 139 127 54 168 60 19 40 5 171 237 88 108 214 179 229 21 79 67 13 249 127 230 121 163 120 66 74 9 73 214 36 84 96 96 158 120 18 1 196 240 248 22 199 124 135 144 255 48 127 206 209 3 227 242 193 171 7 240 249 176 252 127 243 44 136 105 66 140 1 41 229 55 173 113 84 68 96 76 2 76 248 169 23 197 227 145 35 126 250 231 250 235 60 225 242 5 153 57 140 8 33 32 132 25 49 174 222 98 90 202 85 1 76 186 184 56 242 250 252 252 223 189 191 72 12 150 179 108 16 34 64 140 128 136 96 28 71 196 56 99 154 190 33 198 8 235 128 174 243 24 199 17 206 246 63 253 162 200 17 190 159 207 137 124 95 73 49 102 159 63 248 71 93 194 45 113 10 187 68 51 165 132 113 28 241 183 191 255 21 6 1 99 248 130 193 91 0 55 249 35 242 199 227 209 192 218 57 230 95 197 203 223 207 127 68 111 117 125 43 63 154 232 239 194 199 59 108 63 247 83 163 50 57 111 176 54 191 209 32 64 240 128 255 252 247 95 63 120 17 228 207 140 223 25 194 202 98 136 214 218 156 83 196 9 130 0 131 111 175 118 113 228 109 240 207 62 107 144 147 10 36 132 16 48 12 239 96 45 96 32 0 166 215 188 62 242 6 236 131 209 42 91 155 0 32 193 187 30 49 8 62 126 252 132 32 1 194 74 163 121 158 136 87 17 57 217 72 88 227 199 60 207 0 12 222 223 254 6 111 6 92 39 56 164 69 46 60 69 90 52 135 107 188 239 16 163 160 235 58 76 129 161 67 3 246 234 84 30 135 207 141 56 24 177 136 49 160 235 44 198 233 1 222 27 134 15 5 60 147 104 46 118 34 75 29 110 12 100 9 43 6 130 176 41 160 164 85 138 170 135 181 22 41 45 227 28 48 240 47 20 44 164 29 138 70 241 84 38 149 23 133 13 210 10 69 163 72 41 193 90 11 155 101 205 60 108 78 154 230 144 167 16 230 20 42 168 202 41 68 132 57 133 34 138 70 97 140 185 242 22 244 20 237 83 21 62 156 115 0 128 136 72 79 161 128 170 68 51 165 4 99 12 28 28 18 197 171 230 41 26 197 205 205 13 140 49 8 33 64 32 176 156 185 221 60 197 59 124 62 159 17 99 132 247 158 229 168 18 138 70 49 12 3 186 174 131 136 32 72 96 162 169 128 98 214 120 62 159 17 83 132 115 14 222 120 36 184 215 184 46 242 134 28 202 41 0 42 154 26 56 84 125 0 185 44 37 109 83 165 83 120 159 163 76 66 130 99 248 104 158 42 79 17 99 132 49 6 22 150 163 164 10 168 206 41 98 140 203 202 115 230 20 173 83 173 83 56 231 104 16 74 160 78 65 118 20 117 138 211 233 132 36 212 41 52 113 120 236 131 33 164 125 14 85 31 6 134 58 133 2 170 116 138 190 239 243 66 99 9 212 41 20 80 53 29 111 28 115 55 27 111 56 193 70 3 69 163 232 186 14 93 151 219 35 114 130 141 14 138 70 113 127 127 143 148 18 186 174 227 4 27 37 20 239 242 251 247 239 97 140 193 52 77 72 203 63 210 54 197 36 225 225 225 225 202 83 8 189 69 243 28 202 41 34 34 7 196 20 112 168 250 96 57 170 131 162 81 196 24 209 117 29 156 203 211 251 169 104 182 207 225 101 131 28 16 107 31 246 167 32 59 170 198 62 46 97 162 217 62 85 99 31 236 79 161 139 98 44 184 244 20 9 9 1 243 47 189 32 242 246 28 74 52 45 44 186 171 109 162 72 139 84 247 167 88 123 84 176 250 104 159 170 68 211 57 151 171 16 36 86 31 10 56 228 41 168 83 232 128 58 5 217 81 85 146 94 110 41 68 157 162 125 170 117 138 181 239 21 117 138 246 57 164 104 178 143 166 14 152 83 144 29 207 24 69 186 58 223 196 171 101 189 7 61 69 251 188 236 41 150 13 97 182 197 197 198 44 58 5 167 227 181 78 89 167 176 66 157 66 25 204 41 200 14 234 20 100 7 117 10 178 131 58 5 217 193 156 130 236 56 52 201 6 160 78 161 129 170 117 31 107 62 193 249 20 58 40 26 197 150 96 46 125 52 217 201 166 125 170 228 201 24 227 150 112 114 233 96 251 84 87 31 107 78 65 157 162 125 170 60 133 115 110 235 207 77 157 162 125 168 83 144 29 85 19 119 99 140 185 109 34 117 10 21 148 171 15 155 23 255 152 37 193 164 167 104 159 23 140 226 49 100 204 243 12 103 59 174 251 80 196 19 163 216 55 57 235 251 1 34 130 24 179 204 45 76 52 155 167 24 62 166 105 66 140 17 214 90 86 30 74 40 26 197 237 237 45 156 115 220 88 78 17 69 163 248 252 249 51 68 4 195 48 0 96 162 169 129 106 79 49 207 51 19 77 37 28 202 41 216 134 89 7 85 205 85 115 245 145 71 71 217 134 185 125 138 70 49 207 51 188 247 240 222 67 32 244 22 10 40 222 97 239 253 227 238 64 48 28 37 85 192 161 156 130 232 160 106 107 7 234 20 186 160 78 65 118 28 242 20 2 161 78 161 128 162 81 140 227 120 181 5 37 105 159 42 157 98 173 62 0 32 49 124 52 79 209 40 66 8 219 52 255 252 6 86 33 173 83 188 195 125 223 111 11 130 184 177 156 14 138 70 113 58 157 54 195 176 224 156 10 13 20 141 226 195 135 59 140 227 152 149 77 110 23 165 130 162 81 124 252 248 9 125 223 35 132 176 108 65 73 153 187 117 42 60 197 7 140 227 200 166 37 138 176 251 47 151 135 228 227 183 111 103 116 189 195 227 164 94 86 31 173 83 213 222 104 59 231 10 49 21 28 222 130 146 50 119 251 84 239 247 177 66 79 209 62 85 225 99 77 50 35 34 61 133 2 170 86 157 167 148 96 140 129 131 163 162 169 128 162 81 220 220 220 192 24 179 13 157 115 236 163 125 138 119 248 124 62 35 198 8 239 61 53 10 37 20 141 98 24 134 109 154 127 144 192 68 83 1 197 172 241 124 62 35 166 60 74 234 141 71 98 35 180 230 57 148 83 0 148 185 53 112 168 250 0 192 62 154 10 168 210 41 214 206 120 9 137 125 52 21 80 229 41 214 137 187 28 58 215 65 117 78 17 99 132 64 152 83 40 160 90 167 112 206 209 32 148 64 157 130 236 40 234 20 167 211 9 73 168 83 104 226 240 216 7 67 72 251 28 170 62 12 12 117 10 5 84 233 20 125 223 195 90 139 32 129 58 133 2 170 166 227 141 227 136 16 2 188 225 4 27 13 84 45 48 238 186 220 180 157 19 108 116 80 52 138 251 251 123 164 148 208 117 29 39 216 40 161 170 105 137 49 6 211 52 113 129 177 18 138 73 194 195 195 195 149 167 16 122 139 230 57 148 83 68 68 14 136 41 224 80 245 193 114 84 7 85 59 24 119 93 183 53 45 161 162 217 62 135 151 13 114 64 172 125 170 140 34 165 196 181 164 138 56 180 47 41 192 29 140 53 80 53 246 97 173 221 122 115 51 167 104 159 98 44 184 244 20 9 9 1 243 47 189 32 242 246 28 74 52 45 44 58 116 175 113 93 228 13 169 238 79 177 246 168 96 245 209 62 85 137 166 115 142 59 24 43 226 144 167 160 78 161 3 234 20 100 71 85 73 186 174 35 5 168 83 104 160 90 167 96 115 85 61 28 82 52 217 71 83 7 204 41 200 142 103 140 34 93 157 111 226 213 178 222 131 158 162 125 94 246 20 38 27 199 182 184 216 152 69 167 224 116 188 214 41 235 20 86 168 83 40 131 57 5 217 65 157 130 236 160 78 65 118 80 167 32 59 152 83 144 29 135 38 217 0 212 41 52 80 181 238 99 205 39 56 159 66 7 69 163 216 18 204 165 143 38 59 217 180 79 149 60 25 99 220 18 78 46 29 108 159 234 234 99 205 41 168 83 180 79 149 167 112 206 109 253 185 169 83 180 15 117 10 178 163 106 226 110 140 49 183 77 164 78 161 130 114 245 97 243 226 31 179 36 152 244 20 237 243 130 81 60 134 140 121 158 225 108 199 117 31 138 120 98 20 251 38 103 125 63 64 68 16 99 150 185 133 137 102 243 20 195 199 52 77 136 49 194 90 203 202 67 9 69 163 184 189 189 133 115 142 27 203 41 162 104 20 159 63 127 134 136 96 24 6 0 76 52 53 80 237 41 230 121 102 162 169 132 67 57 5 219 48 235 160 170 185 106 174 62 242 232 40 219 48 183 79 209 40 230 121 134 247 30 222 123 8 132 222 66 1 197 59 236 189 127 220 29 8 134 163 164 10 56 148 83 16 29 84 109 237 64 157 66 23 212 41 200 142 67 158 66 32 212 41 20 80 52 138 113 28 175 182 160 36 237 83 165 83 172 213 7 0 36 134 143 230 41 26 69 8 97 155 230 159 223 192 42 164 117 138 119 184 239 251 109 65 16 55 150 211 65 209 40 78 167 211 102 24 22 156 83 161 129 162 81 124 248 112 135 113 28 179 178 201 237 162 84 80 52 138 143 31 63 161 239 123 132 16 150 45 40 41 115 183 78 133 167 248 128 113 28 217 180 68 17 135 114 10 162 131 67 213 7 64 153 91 3 85 203 6 215 70 104 148 185 117 80 213 201 102 158 243 190 97 81 24 66 52 80 165 104 222 222 222 194 24 3 111 232 37 52 80 52 138 97 24 240 229 203 23 136 8 230 196 157 6 53 80 85 125 220 221 221 193 24 131 206 114 167 65 13 20 141 226 230 230 6 95 191 126 205 51 186 217 239 74 5 135 114 10 246 187 210 129 253 238 151 200 213 199 52 77 0 18 4 194 249 20 10 120 193 83 88 64 44 0 139 24 227 214 239 10 72 156 77 161 128 170 62 154 211 52 229 54 71 18 57 32 166 128 234 156 2 0 117 10 37 28 214 41 56 74 218 62 69 163 56 159 207 184 187 187 3 0 116 182 99 248 80 192 33 79 17 17 233 41 20 112 40 167 160 78 161 131 170 234 99 158 103 24 99 22 157 130 179 185 91 167 106 62 133 115 110 107 216 206 117 31 237 83 181 66 236 90 192 34 173 83 85 125 244 125 159 67 8 12 7 197 20 80 181 234 252 116 58 193 57 135 136 200 100 83 1 207 24 197 245 83 227 248 13 253 224 243 154 82 116 16 244 0 203 210 166 169 218 218 225 234 107 26 68 243 84 85 31 214 218 173 36 229 20 255 246 121 166 164 72 249 97 12 0 89 54 169 117 0 44 132 155 192 168 224 194 83 36 192 4 0 17 64 132 152 8 49 9 206 121 204 115 194 208 223 34 4 129 161 78 209 60 246 234 84 246 55 60 132 25 206 25 204 243 140 222 247 175 120 105 228 173 176 16 224 241 225 0 116 203 35 39 148 93 215 1 16 220 63 124 66 144 145 163 164 10 216 187 6 89 158 22 15 192 34 196 9 206 27 252 254 251 111 240 198 51 124 40 224 249 59 44 64 246 20 22 222 123 140 227 25 247 15 95 150 173 162 24 66 90 199 111 178 195 211 168 176 124 157 82 202 13 219 93 15 3 15 193 13 114 93 66 189 162 85 60 112 6 96 1 227 0 152 124 92 13 68 178 62 145 247 171 181 16 120 24 252 5 255 252 199 191 49 120 139 92 190 174 37 106 54 149 214 142 235 102 222 47 125 63 135 211 183 186 190 149 31 13 233 105 249 140 124 52 34 95 37 255 7 30 2 11 131 188 52 80 18 16 35 48 78 103 156 78 39 196 56 99 154 190 229 134 104 14 232 58 159 59 220 88 134 147 183 229 251 6 177 78 121 120 137 167 138 53 0 248 75 139 203 103 139 181 24 192 123 96 154 13 134 97 64 8 230 162 159 102 46 95 253 59 1 204 34 118 145 55 226 231 255 238 255 7 128 87 244 218 203 19 216 123 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:31'!
leftPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 12 0 0 0 2 8 2 0 0 1 148 26 154 86 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 0 53 73 68 65 84 120 156 99 249 249 243 39 27 27 27 203 255 255 255 89 110 220 184 161 174 174 206 192 192 192 242 246 237 91 33 33 33 6 36 192 2 84 247 229 203 151 215 175 95 67 148 96 5 0 44 107 21 6 205 166 58 130 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:31'!
leftTopLinkJpg
^#(255 216 255 224 0 16 74 70 73 70 0 1 1 1 0 72 0 72 0 0 255 225 50 29 104 116 116 112 58 47 47 110 115 46 97 100 111 98 101 46 99 111 109 47 120 97 112 47 49 46 48 47 0 60 63 120 112 97 99 107 101 116 32 98 101 103 105 110 61 34 239 187 191 34 32 105 100 61 34 87 53 77 48 77 112 67 101 104 105 72 122 114 101 83 122 78 84 99 122 107 99 57 100 34 63 62 10 60 120 58 120 109 112 109 101 116 97 32 120 109 108 110 115 58 120 61 34 97 100 111 98 101 58 110 115 58 109 101 116 97 47 34 32 120 58 120 109 112 116 107 61 34 65 100 111 98 101 32 88 77 80 32 67 111 114 101 32 52 46 49 45 99 48 51 52 32 52 54 46 50 55 50 57 55 54 44 32 83 97 116 32 74 97 110 32 50 55 32 50 48 48 55 32 50 50 58 51 55 58 51 55 32 32 32 32 32 32 32 32 34 62 10 32 32 32 60 114 100 102 58 82 68 70 32 120 109 108 110 115 58 114 100 102 61 34 104 116 116 112 58 47 47 119 119 119 46 119 51 46 111 114 103 47 49 57 57 57 47 48 50 47 50 50 45 114 100 102 45 115 121 110 116 97 120 45 110 115 35 34 62 10 32 32 32 32 32 32 60 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 32 114 100 102 58 97 98 111 117 116 61 34 34 10 32 32 32 32 32 32 32 32 32 32 32 32 120 109 108 110 115 58 120 97 112 61 34 104 116 116 112 58 47 47 110 115 46 97 100 111 98 101 46 99 111 109 47 120 97 112 47 49 46 48 47 34 62 10 32 32 32 32 32 32 32 32 32 60 120 97 112 58 67 114 101 97 116 111 114 84 111 111 108 62 65 100 111 98 101 32 70 105 114 101 119 111 114 107 115 32 67 83 51 60 47 120 97 112 58 67 114 101 97 116 111 114 84 111 111 108 62 10 32 32 32 32 32 32 32 32 32 60 120 97 112 58 67 114 101 97 116 101 68 97 116 101 62 50 48 48 55 45 48 53 45 50 57 84 48 55 58 51 53 58 49 48 90 60 47 120 97 112 58 67 114 101 97 116 101 68 97 116 101 62 10 32 32 32 32 32 32 32 32 32 60 120 97 112 58 77 111 100 105 102 121 68 97 116 101 62 50 48 48 55 45 48 53 45 50 57 84 49 48 58 49 50 58 48 51 90 60 47 120 97 112 58 77 111 100 105 102 121 68 97 116 101 62 10 32 32 32 32 32 32 60 47 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 62 10 32 32 32 32 32 32 60 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 32 114 100 102 58 97 98 111 117 116 61 34 34 10 32 32 32 32 32 32 32 32 32 32 32 32 120 109 108 110 115 58 100 99 61 34 104 116 116 112 58 47 47 112 117 114 108 46 111 114 103 47 100 99 47 101 108 101 109 101 110 116 115 47 49 46 49 47 34 62 10 32 32 32 32 32 32 32 32 32 60 100 99 58 102 111 114 109 97 116 62 105 109 97 103 101 47 106 112 101 103 60 47 100 99 58 102 111 114 109 97 116 62 10 32 32 32 32 32 32 60 47 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 62 10 32 32 32 60 47 114 100 102 58 82 68 70 62 10 60 47 120 58 120 109 112 109 101 116 97 62 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 60 63 120 112 97 99 107 101 116 32 101 110 100 61 34 119 34 63 62 255 219 0 67 0 4 3 3 3 3 3 4 3 3 4 5 4 3 4 5 7 5 4 4 5 7 8 6 6 7 6 6 8 10 8 8 8 8 8 8 10 8 10 10 11 10 10 8 13 13 14 14 13 13 18 18 18 18 18 20 20 20 20 20 20 20 20 20 20 255 219 0 67 1 5 5 5 8 7 8 15 10 10 15 18 15 12 15 18 22 21 21 21 21 22 22 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 255 194 0 17 8 0 33 0 133 3 1 17 0 2 17 1 3 17 1 255 196 0 26 0 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 1 6 7 8 2 5 255 196 0 26 1 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 1 4 5 3 2 6 255 218 0 12 3 1 0 2 16 3 16 0 0 1 239 199 47 193 151 31 155 200 0 0 0 0 0 239 221 141 252 131 155 143 19 147 192 0 0 0 0 0 244 159 119 167 199 249 121 48 248 188 104 33 65 10 66 212 138 66 212 143 76 126 135 163 154 207 241 249 222 113 1 72 10 66 144 164 0 173 214 207 79 128 0 0 0 0 0 31 255 196 0 32 16 0 1 4 3 0 2 3 0 0 0 0 0 0 0 0 0 0 4 5 18 22 2 3 19 6 50 1 49 64 255 218 0 8 1 1 0 1 5 2 60 141 213 193 18 219 11 217 97 123 44 47 101 133 236 176 189 150 23 178 194 246 88 94 203 11 217 97 123 44 47 101 133 236 176 189 150 23 178 194 246 101 245 229 24 201 119 51 153 204 230 115 57 156 206 103 51 153 204 230 115 57 156 204 253 124 163 28 190 87 67 97 13 132 54 16 216 67 97 13 132 54 16 216 67 97 13 132 54 16 216 67 97 13 132 54 25 250 172 108 76 187 58 250 2 190 128 175 160 43 232 10 250 2 190 128 175 160 43 232 10 250 2 190 128 175 160 43 232 10 250 2 190 128 175 160 252 95 255 196 0 30 17 0 3 1 1 0 2 3 1 0 0 0 0 0 0 0 0 0 2 19 17 1 3 16 4 33 64 50 255 218 0 8 1 3 1 1 63 1 60 207 222 116 179 22 98 204 89 139 49 102 44 197 152 179 22 98 204 89 139 49 102 44 222 190 71 245 248 124 252 251 48 195 12 48 195 12 48 195 12 48 195 61 117 116 153 50 100 201 147 38 76 153 50 100 201 147 39 207 197 255 196 0 32 17 0 2 3 1 0 2 3 1 1 0 0 0 0 0 0 0 0 17 2 3 20 81 4 16 1 50 129 33 64 255 218 0 8 1 2 1 1 63 1 60 171 165 25 127 13 54 116 211 103 77 54 116 211 103 77 54 116 211 103 77 54 116 211 103 77 54 116 211 103 77 54 116 211 103 77 54 116 211 103 77 54 119 215 153 247 252 16 132 33 8 66 16 132 33 8 94 188 223 191 224 198 49 140 99 24 198 49 140 99 245 111 143 241 103 203 49 196 199 19 28 76 113 49 196 199 19 28 76 113 49 196 197 19 28 76 113 49 196 197 19 28 127 197 255 196 0 44 16 0 0 4 6 1 1 6 7 0 0 0 0 0 0 0 0 0 2 3 162 49 50 52 145 209 210 1 17 16 18 19 64 81 178 4 33 34 36 65 100 113 255 218 0 8 1 1 0 6 63 2 4 75 225 22 240 211 229 30 13 207 29 210 243 243 239 27 215 142 125 5 75 19 212 84 177 61 69 75 19 212 84 177 61 69 75 19 212 84 177 61 69 75 19 212 84 177 61 69 75 19 212 84 177 61 69 75 19 212 84 177 61 69 75 19 212 84 177 61 69 75 19 215 177 46 127 92 190 227 121 20 122 115 207 31 108 95 113 132 198 184 152 215 19 26 226 99 92 76 107 137 141 113 49 174 38 53 196 198 184 152 215 19 26 226 99 92 76 107 137 141 113 49 175 216 85 86 241 120 49 73 220 250 122 126 63 189 125 68 87 185 112 34 187 112 34 187 112 34 189 203 129 21 219 129 21 219 129 21 238 92 8 174 220 8 175 114 224 69 118 224 69 123 151 2 43 183 2 43 183 2 43 183 2 43 220 184 242 95 255 196 0 28 16 0 2 2 3 1 1 0 0 0 0 0 0 0 0 0 0 0 241 1 17 16 49 64 65 33 255 218 0 8 1 1 0 1 63 33 61 60 62 188 23 115 181 14 11 44 178 203 44 178 203 44 178 195 44 179 227 132 48 0 0 220 73 49 30 51 66 192 48 12 3 0 192 48 12 3 0 192 48 12 3 0 192 48 12 6 202 40 52 148 166 42 83 62 48 10 98 108 113 138 98 108 113 138 99 136 83 28 98 152 155 19 99 199 138 255 218 0 12 3 1 0 2 0 3 0 0 0 16 15 109 182 219 109 150 210 182 219 109 182 219 108 28 150 75 44 50 195 66 91 45 150 89 109 152 0 0 0 0 0 1 255 196 0 29 17 1 1 1 1 0 1 5 0 0 0 0 0 0 0 0 0 0 17 1 97 16 64 81 129 161 209 255 218 0 8 1 3 1 1 63 16 79 231 179 171 171 171 171 171 171 171 171 171 171 171 171 171 171 175 140 125 63 83 19 19 19 19 19 19 19 19 19 19 19 19 19 19 197 62 43 90 214 181 173 107 90 214 181 175 198 122 66 16 132 33 8 66 16 132 35 209 159 255 196 0 29 17 0 2 1 4 3 0 0 0 0 0 0 0 0 0 0 0 1 17 64 0 33 49 240 80 81 193 255 218 0 8 1 2 1 1 63 16 160 112 104 47 76 25 36 146 73 36 146 5 154 201 130 0 0 236 214 76 31 251 172 142 151 3 247 127 159 15 255 196 0 33 16 0 2 0 6 2 3 1 0 0 0 0 0 0 0 0 0 0 1 17 49 97 113 193 240 33 177 64 65 145 16 255 218 0 8 1 1 0 1 63 16 110 8 127 238 96 171 143 162 132 35 14 10 208 212 134 164 53 33 169 13 72 107 67 82 26 208 212 134 180 53 33 230 109 76 26 144 214 133 162 179 141 189 103 106 44 69 136 181 22 34 196 90 139 17 106 36 146 45 69 136 181 18 73 22 175 199 9 28 26 114 70 147 55 140 143 120 236 214 50 109 25 30 177 216 183 142 205 227 34 222 59 55 140 156 59 31 77 227 35 222 59 55 140 156 59 31 77 227 34 182 185 134 120 191 182 22 237 197 70 45 239 192 227 209 167 30 157 56 245 226 211 143 78 141 56 185 120 63 255 217 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:32'!
logoJpg
^#(255 216 255 224 0 16 74 70 73 70 0 1 1 1 0 72 0 72 0 0 255 225 50 29 104 116 116 112 58 47 47 110 115 46 97 100 111 98 101 46 99 111 109 47 120 97 112 47 49 46 48 47 0 60 63 120 112 97 99 107 101 116 32 98 101 103 105 110 61 34 239 187 191 34 32 105 100 61 34 87 53 77 48 77 112 67 101 104 105 72 122 114 101 83 122 78 84 99 122 107 99 57 100 34 63 62 10 60 120 58 120 109 112 109 101 116 97 32 120 109 108 110 115 58 120 61 34 97 100 111 98 101 58 110 115 58 109 101 116 97 47 34 32 120 58 120 109 112 116 107 61 34 65 100 111 98 101 32 88 77 80 32 67 111 114 101 32 52 46 49 45 99 48 51 52 32 52 54 46 50 55 50 57 55 54 44 32 83 97 116 32 74 97 110 32 50 55 32 50 48 48 55 32 50 50 58 51 55 58 51 55 32 32 32 32 32 32 32 32 34 62 10 32 32 32 60 114 100 102 58 82 68 70 32 120 109 108 110 115 58 114 100 102 61 34 104 116 116 112 58 47 47 119 119 119 46 119 51 46 111 114 103 47 49 57 57 57 47 48 50 47 50 50 45 114 100 102 45 115 121 110 116 97 120 45 110 115 35 34 62 10 32 32 32 32 32 32 60 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 32 114 100 102 58 97 98 111 117 116 61 34 34 10 32 32 32 32 32 32 32 32 32 32 32 32 120 109 108 110 115 58 120 97 112 61 34 104 116 116 112 58 47 47 110 115 46 97 100 111 98 101 46 99 111 109 47 120 97 112 47 49 46 48 47 34 62 10 32 32 32 32 32 32 32 32 32 60 120 97 112 58 67 114 101 97 116 111 114 84 111 111 108 62 65 100 111 98 101 32 70 105 114 101 119 111 114 107 115 32 67 83 51 60 47 120 97 112 58 67 114 101 97 116 111 114 84 111 111 108 62 10 32 32 32 32 32 32 32 32 32 60 120 97 112 58 67 114 101 97 116 101 68 97 116 101 62 50 48 48 55 45 48 53 45 50 55 84 50 51 58 48 55 58 52 57 90 60 47 120 97 112 58 67 114 101 97 116 101 68 97 116 101 62 10 32 32 32 32 32 32 32 32 32 60 120 97 112 58 77 111 100 105 102 121 68 97 116 101 62 50 48 48 55 45 48 53 45 50 56 84 49 50 58 51 52 58 51 53 90 60 47 120 97 112 58 77 111 100 105 102 121 68 97 116 101 62 10 32 32 32 32 32 32 60 47 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 62 10 32 32 32 32 32 32 60 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 32 114 100 102 58 97 98 111 117 116 61 34 34 10 32 32 32 32 32 32 32 32 32 32 32 32 120 109 108 110 115 58 100 99 61 34 104 116 116 112 58 47 47 112 117 114 108 46 111 114 103 47 100 99 47 101 108 101 109 101 110 116 115 47 49 46 49 47 34 62 10 32 32 32 32 32 32 32 32 32 60 100 99 58 102 111 114 109 97 116 62 105 109 97 103 101 47 106 112 101 103 60 47 100 99 58 102 111 114 109 97 116 62 10 32 32 32 32 32 32 60 47 114 100 102 58 68 101 115 99 114 105 112 116 105 111 110 62 10 32 32 32 60 47 114 100 102 58 82 68 70 62 10 60 47 120 58 120 109 112 109 101 116 97 62 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 10 60 63 120 112 97 99 107 101 116 32 101 110 100 61 34 119 34 63 62 255 219 0 67 0 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 3 2 2 2 2 3 3 3 3 3 3 3 3 4 3 3 3 3 3 3 4 4 4 4 5 4 4 4 5 5 6 6 5 5 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 255 219 0 67 1 2 2 2 3 3 3 5 4 4 5 7 6 5 6 7 8 8 8 8 8 8 8 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 255 194 0 17 8 0 219 2 157 3 1 17 0 2 17 1 3 17 1 255 196 0 28 0 1 0 1 5 1 1 0 0 0 0 0 0 0 0 0 0 0 5 1 3 4 6 7 8 2 255 196 0 28 1 1 0 2 3 1 1 1 0 0 0 0 0 0 0 0 0 0 1 2 3 4 5 6 7 8 255 218 0 12 3 1 0 2 16 3 16 0 0 1 247 164 18 8 0 144 65 32 0 8 132 200 8 38 9 0 16 9 65 32 130 72 132 202 0 4 144 72 132 72 9 32 2 72 130 72 76 160 144 68 38 80 9 32 146 1 36 19 8 144 145 8 144 1 4 220 149 216 138 204 142 31 240 250 0 2 72 0 0 0 0 0 4 36 16 1 32 73 0 34 66 96 18 64 0 0 40 84 0 0 2 72 4 74 96 18 64 0 4 18 36 130 65 0 1 243 102 38 252 244 159 166 91 113 233 79 217 231 95 205 184 192 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 103 232 28 140 127 181 121 146 160 10 216 45 211 46 149 131 165 181 230 208 205 182 8 76 123 51 151 214 74 146 0 154 137 85 31 49 82 112 121 125 12 111 139 122 255 0 76 253 130 242 215 159 58 254 110 198 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 11 63 74 226 99 253 243 199 146 0 161 171 235 116 249 31 63 212 105 90 157 12 172 186 210 150 193 141 72 239 221 127 43 179 237 115 238 226 203 43 171 183 155 143 37 218 101 161 98 248 163 242 235 69 238 234 166 32 185 125 45 59 243 135 185 245 159 210 39 114 244 214 243 183 230 220 96 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 177 244 158 46 63 223 124 133 80 18 86 34 241 237 104 250 253 168 124 59 17 248 182 249 174 167 78 147 79 182 91 115 79 85 244 60 151 75 210 218 231 92 191 69 9 167 209 139 199 181 159 151 91 170 245 252 212 222 109 24 45 238 110 183 210 229 65 114 250 186 127 231 31 111 234 255 0 163 55 95 81 111 59 126 109 160 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 44 253 39 139 143 247 223 32 136 75 19 30 93 3 79 167 191 224 200 234 113 177 153 120 103 51 217 228 198 14 201 208 242 218 70 183 95 153 232 250 79 78 87 143 94 103 86 190 19 232 124 163 106 250 94 205 134 205 209 224 250 147 213 252 230 150 197 164 117 184 186 222 150 230 157 249 199 219 250 191 232 205 215 212 95 206 223 155 113 138 70 90 77 62 149 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 99 253 31 141 99 239 222 55 159 232 250 124 172 186 252 123 155 211 210 57 157 207 82 116 252 134 203 218 225 94 154 71 227 219 189 52 202 203 133 84 39 43 211 204 234 236 203 113 250 95 126 3 233 28 23 127 46 175 189 128 189 20 244 55 175 249 151 71 222 227 70 101 215 230 234 105 159 157 125 167 171 254 141 59 175 167 159 59 126 110 167 213 118 45 227 245 17 91 188 9 92 60 239 168 0 7 213 118 109 78 91 119 212 191 90 4 95 35 15 99 7 99 149 122 48 128 139 223 195 216 197 203 171 91 115 190 128 5 15 188 125 124 124 188 241 127 23 66 198 94 113 55 163 0 2 230 61 236 76 218 151 235 65 247 77 220 124 152 233 25 111 214 49 242 233 95 160 0 0 198 250 71 31 73 251 167 7 206 252 127 109 49 125 125 67 6 125 151 46 143 85 232 243 246 204 252 125 171 103 154 45 86 247 39 22 102 13 189 215 149 215 173 111 193 248 30 171 109 240 159 71 214 45 151 146 118 57 209 217 109 126 173 239 185 229 55 78 175 147 142 199 110 153 155 79 143 124 27 208 250 191 232 211 186 122 123 121 223 243 118 59 54 180 124 250 61 103 165 207 220 121 121 46 85 118 153 237 90 110 70 11 115 181 143 147 75 3 46 122 219 86 95 79 163 141 177 231 164 181 186 90 238 207 98 198 110 92 230 174 36 94 244 241 238 83 102 11 55 168 192 219 243 187 127 39 179 129 177 165 123 30 205 45 165 72 221 183 151 151 139 29 186 100 229 219 101 87 122 251 155 74 244 171 126 38 126 191 91 31 63 23 51 30 166 62 63 99 29 181 204 156 212 164 118 107 221 138 199 102 140 91 236 108 90 57 96 119 180 230 53 240 43 187 113 167 243 27 214 50 115 242 49 116 177 182 56 25 31 65 225 121 131 236 153 117 109 94 157 169 94 179 30 172 138 207 112 236 249 46 159 189 231 241 169 151 202 28 63 109 233 237 174 55 88 197 75 184 179 80 208 188 175 180 219 252 127 173 229 245 235 66 108 107 121 247 169 79 157 141 43 221 15 41 157 151 78 111 99 7 163 247 57 220 195 225 125 127 87 253 26 119 111 79 111 58 254 110 199 123 23 79 87 220 236 228 91 206 125 97 244 116 11 95 173 35 115 214 198 93 75 248 243 253 211 53 187 225 150 212 232 192 116 188 174 221 204 207 171 239 237 103 107 238 200 224 205 170 244 249 27 118 135 50 237 47 169 239 119 111 91 207 253 225 244 121 152 245 228 117 59 122 255 0 67 147 31 179 165 47 165 211 142 218 230 204 234 100 143 203 177 41 173 189 133 159 157 41 169 218 134 223 243 114 90 157 184 157 206 38 217 161 169 175 103 244 153 52 45 162 167 66 151 211 136 217 197 53 167 210 165 181 34 243 222 107 86 34 182 45 122 177 241 109 107 86 180 166 174 252 39 67 206 94 246 156 79 53 125 147 118 11 5 126 5 99 234 211 41 143 55 76 220 243 219 134 247 19 99 166 111 62 114 253 21 251 102 245 254 110 23 209 247 139 36 15 31 187 177 248 63 125 203 105 214 212 183 53 120 191 103 133 180 238 249 205 111 54 28 107 91 58 107 222 118 117 32 126 41 181 234 255 0 163 91 118 244 211 231 95 205 248 241 111 179 1 181 182 156 25 88 239 31 159 20 166 151 126 23 127 135 49 173 142 170 89 181 165 52 189 38 165 214 243 242 122 249 50 99 74 95 90 144 217 250 120 27 26 27 6 150 72 189 156 89 88 173 47 175 142 7 111 44 254 166 8 45 173 172 220 72 141 140 247 171 130 228 99 179 120 189 143 122 198 77 34 110 227 221 199 203 163 245 19 122 185 44 206 111 182 172 238 164 235 251 184 100 245 182 35 243 198 53 243 72 97 197 143 122 216 182 198 110 60 54 109 95 184 139 51 179 37 175 179 123 7 119 84 235 249 121 61 123 223 141 77 39 235 84 212 62 135 173 206 245 226 51 79 110 99 14 109 142 107 45 209 216 220 183 188 86 239 137 41 173 183 153 17 231 120 221 244 134 110 118 102 158 198 173 229 61 135 83 245 94 127 66 241 190 195 65 197 187 169 103 231 194 244 184 48 25 169 15 106 221 180 125 157 47 54 57 15 140 223 213 255 0 70 157 215 211 219 206 223 155 177 128 0 139 216 103 225 93 168 8 141 150 77 25 216 64 0 0 0 0 0 0 0 0 0 0 0 20 42 0 49 114 53 253 212 246 155 51 16 107 127 115 231 72 123 110 103 149 188 231 190 129 165 125 37 236 62 113 179 111 115 249 47 55 217 122 31 55 153 218 180 183 112 214 213 235 146 229 91 162 52 127 59 235 160 252 159 170 152 245 158 74 107 123 159 204 249 155 124 222 45 142 89 87 88 205 92 123 64 221 44 219 62 59 30 175 250 52 238 222 162 222 117 252 219 140 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 28 235 232 59 186 7 210 173 204 58 90 93 63 167 163 213 183 188 135 207 27 179 57 214 228 116 110 103 75 233 33 53 200 174 125 138 155 252 155 207 122 93 59 196 123 216 254 255 0 135 154 217 230 199 35 26 103 71 202 135 203 16 119 174 13 160 124 90 122 79 201 171 236 79 125 109 215 211 79 157 191 55 99 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 4 79 214 249 21 251 87 141 142 197 158 99 38 190 187 229 125 150 7 142 247 187 207 161 243 59 207 119 204 38 213 47 198 93 142 189 11 149 182 183 171 181 199 188 103 169 235 219 156 12 20 232 228 100 185 174 86 151 158 35 109 91 9 184 76 115 155 111 206 105 236 15 160 206 235 233 237 231 111 205 184 192 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 143 245 255 0 59 103 236 254 97 48 48 249 221 136 159 146 253 163 62 147 51 233 252 142 237 233 188 150 82 118 74 239 94 101 161 174 115 182 110 121 205 248 11 52 11 87 224 199 33 141 111 34 50 234 202 144 145 229 198 95 207 49 250 191 232 246 221 125 52 249 219 243 118 48 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 99 253 127 206 216 251 63 152 185 142 101 245 183 182 45 30 142 149 231 189 86 23 136 247 251 22 42 215 232 31 59 233 125 77 28 139 100 22 235 110 91 231 58 90 214 181 183 122 165 106 210 77 104 138 45 43 110 203 5 195 31 157 24 190 2 190 175 250 61 183 95 77 62 118 252 221 140 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 25 255 0 86 229 74 253 99 141 157 76 255 0 88 165 98 26 231 59 171 175 252 255 0 233 121 61 158 63 92 246 158 59 50 73 172 6 28 158 79 243 221 44 92 51 143 145 190 226 110 248 213 132 122 186 184 0 193 208 140 79 1 95 87 253 30 219 175 166 159 59 126 110 198 0 0 0 0 0 0 0 0 0 0 20 42 80 168 5 10 148 5 64 0 1 93 200 220 249 36 49 243 235 0 5 64 38 190 165 207 153 250 175 34 179 32 11 139 232 94 75 222 114 254 253 246 141 158 47 125 199 206 146 203 88 13 92 254 123 243 251 186 74 245 53 220 174 149 172 221 49 176 21 134 0 24 58 17 137 224 43 235 15 163 91 117 244 243 231 95 205 216 192 0 0 0 0 0 0 0 0 0 2 205 125 14 62 199 158 202 215 235 96 108 243 164 112 86 230 62 140 78 204 231 227 231 225 79 114 214 77 9 124 26 214 237 131 238 185 126 109 79 186 218 221 171 247 24 236 211 212 68 236 91 41 198 179 124 95 88 250 178 120 117 98 182 181 101 245 245 126 163 63 197 181 103 126 167 207 153 250 167 30 171 10 69 228 89 102 169 187 161 226 219 243 207 127 161 186 120 76 91 5 181 251 79 115 75 148 113 183 121 31 59 111 6 245 136 62 47 26 229 171 215 180 230 237 65 32 48 116 35 19 192 87 214 31 70 182 235 233 231 206 191 155 177 128 0 0 0 0 0 0 0 0 0 4 123 212 99 236 249 125 131 155 234 53 237 214 38 109 11 248 247 35 243 68 198 174 92 76 147 244 195 39 171 209 143 217 231 125 87 119 233 130 253 31 44 217 25 60 125 154 122 44 123 109 221 105 226 219 102 71 91 161 137 179 196 204 214 237 196 111 121 185 254 111 166 215 250 158 59 127 250 63 27 96 250 151 50 236 219 41 147 62 54 47 50 125 86 57 174 108 190 84 237 117 39 254 125 143 162 105 77 174 158 190 78 142 110 105 54 181 15 163 26 241 166 102 174 85 103 107 192 216 113 167 177 128 143 208 140 111 159 215 214 31 71 182 233 234 39 206 223 155 113 128 0 0 0 0 0 0 0 0 0 6 20 122 92 44 213 144 193 177 173 244 57 217 120 119 113 51 106 102 97 218 139 217 210 206 195 179 127 30 197 155 87 27 54 165 250 71 220 90 254 45 184 252 248 103 240 121 207 172 62 150 63 54 76 60 252 251 22 182 86 29 239 155 232 227 223 44 150 189 177 111 154 206 126 23 94 250 7 59 113 250 166 167 212 72 1 44 36 248 63 167 213 153 240 145 157 89 208 51 87 173 235 219 81 202 144 163 84 201 88 220 149 248 18 31 6 101 27 78 43 73 226 101 115 163 55 231 149 245 127 210 45 186 122 105 243 183 230 236 96 0 0 0 0 0 0 0 0 0 1 139 145 139 145 143 29 207 140 220 60 236 43 212 90 178 245 95 53 232 235 93 12 59 14 158 167 220 62 225 82 220 190 101 155 137 106 204 60 172 220 43 118 99 221 149 141 73 124 150 110 200 198 222 126 153 135 114 250 166 162 100 0 168 121 163 22 231 1 226 231 220 112 223 77 205 93 255 0 5 181 76 140 76 149 195 138 227 89 124 251 128 11 17 97 33 200 142 151 243 90 250 191 232 211 186 250 123 121 219 243 118 48 0 0 0 0 0 0 0 0 0 0 3 230 85 133 64 5 10 75 234 0 0 0 0 0 0 25 94 182 154 223 208 113 218 153 0 10 156 251 46 78 81 191 121 104 189 154 38 235 48 185 81 215 165 189 117 238 43 42 162 64 0 93 214 142 157 226 99 213 127 73 182 235 233 231 206 223 155 113 128 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 3 30 237 127 117 243 32 0 16 157 71 13 250 252 147 110 171 214 154 4 64 249 247 160 190 119 93 163 158 9 0 0 154 212 118 79 178 223 117 244 243 231 111 205 184 192 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 102 239 48 253 178 241 125 88 196 50 164 132 31 46 122 175 129 175 123 241 241 118 160 0 0 14 169 246 171 238 190 158 124 237 249 187 24 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 6 167 215 121 227 235 246 130 178 111 203 70 126 187 163 121 231 85 243 177 151 140 0 0 0 58 167 218 239 186 250 121 226 191 17 173 100 43 33 82 178 172 170 86 65 42 129 33 72 36 133 0 42 138 74 136 64 154 162 169 250 42 148 69 37 82 128 20 144 20 40 4 40 128 41 10 20 16 20 40 129 242 82 100 128 153 162 41 36 128 172 206 183 188 231 253 121 233 252 56 148 192 251 172 128 149 1 244 143 147 230 66 137 29 11 233 22 219 251 72 9 144 0 9 4 0 0 32 19 80 89 0 2 64 32 146 1 4 146 8 32 176 128 9 8 144 64 72 0 18 8 36 17 4 128 130 69 66 101 2 64 72 4 0 1 50 136 10 202 204 137 137 122 191 255 196 0 51 16 0 1 4 2 1 3 2 5 2 5 4 3 0 0 0 0 3 0 1 2 4 5 19 21 6 17 18 16 20 32 33 51 52 80 34 53 23 36 48 49 64 35 50 66 96 22 37 65 255 218 0 8 1 1 0 1 5 2 220 53 184 107 112 214 225 173 195 91 134 183 13 110 26 220 53 184 107 112 214 225 173 163 91 70 182 141 110 26 220 53 184 107 104 214 225 173 176 91 134 183 13 110 26 220 53 180 107 112 214 225 173 195 91 70 182 141 109 26 218 53 180 107 104 214 209 173 163 91 70 182 141 109 130 218 53 180 107 112 214 225 173 163 91 70 182 141 109 26 218 53 180 107 104 214 209 173 163 91 70 183 13 110 26 220 53 184 107 112 214 225 173 163 91 70 183 13 109 26 218 53 180 107 104 214 209 173 163 91 70 183 13 109 26 218 53 184 107 112 214 209 173 163 91 70 183 13 109 130 243 139 252 28 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 181 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 165 200 90 92 133 164 249 27 74 57 179 142 84 114 16 179 6 118 147 126 102 75 187 175 154 249 175 154 249 175 154 249 169 17 161 18 117 13 40 61 123 65 180 62 238 185 58 78 95 154 249 175 154 249 174 238 190 107 230 190 107 230 190 107 230 172 149 197 95 167 178 175 180 46 221 255 0 51 63 237 241 94 202 130 154 189 114 221 183 151 147 42 214 172 84 37 156 189 251 112 198 227 195 112 213 141 143 173 5 8 76 138 52 102 233 169 13 151 179 10 246 97 79 70 14 167 80 177 95 219 211 37 251 119 77 125 44 115 191 182 252 204 254 43 98 49 128 253 59 254 145 58 118 212 68 124 41 132 39 236 187 47 9 118 237 217 116 252 39 111 29 41 10 176 201 212 192 238 78 171 44 94 93 83 121 208 58 176 204 168 228 43 228 4 174 11 188 86 75 246 238 154 250 88 239 181 252 204 255 0 183 169 142 42 195 143 84 227 103 96 219 132 225 148 166 59 70 32 5 124 182 45 203 17 136 247 100 124 125 57 6 230 9 160 198 168 65 79 165 254 88 124 240 167 101 19 26 10 149 243 18 139 191 166 39 32 216 235 105 217 157 136 55 20 242 95 183 116 215 210 199 125 175 193 223 242 164 244 200 220 36 137 139 180 73 19 63 152 108 129 169 145 227 147 108 96 137 8 70 35 138 61 96 90 136 132 48 67 211 45 135 149 170 125 51 19 67 31 110 245 10 249 44 129 153 161 151 135 137 253 122 94 243 219 198 43 0 219 28 155 118 199 244 215 210 199 125 175 163 51 201 229 27 209 104 12 181 155 226 102 121 60 127 92 218 108 242 245 212 69 229 218 95 6 162 34 119 11 127 127 137 226 232 82 218 158 76 211 148 94 40 114 216 158 94 36 248 35 25 73 52 217 229 232 209 121 33 201 138 167 45 109 22 121 180 38 211 111 140 139 51 145 246 21 182 145 86 201 22 164 103 88 115 88 17 86 173 147 181 155 43 44 85 201 92 15 167 148 123 160 137 203 63 78 171 246 230 186 60 121 34 11 85 161 104 83 5 182 45 98 200 145 114 120 147 163 109 48 45 222 235 0 13 98 122 158 245 155 249 198 143 19 211 95 75 29 246 190 147 103 118 5 73 52 203 104 217 27 86 101 26 178 60 226 42 255 0 163 219 66 98 176 42 207 27 3 1 107 89 64 151 184 157 97 70 118 170 51 123 129 150 165 130 12 190 83 132 227 9 216 199 94 123 37 53 130 89 177 54 175 59 68 133 120 197 251 183 252 178 85 47 158 223 180 48 164 241 16 132 34 136 146 20 193 106 53 39 27 76 18 214 41 7 55 153 139 26 100 40 7 226 97 139 249 242 123 19 22 171 180 13 164 132 201 142 200 204 109 213 55 185 60 14 174 92 108 125 123 109 166 239 232 136 152 245 138 17 197 172 142 156 96 245 124 106 89 141 89 249 138 156 125 173 42 210 221 1 150 189 165 93 216 236 2 213 181 56 21 167 102 37 168 83 68 191 234 207 251 101 238 61 219 137 166 207 53 217 188 215 75 183 242 200 209 153 7 98 188 193 151 233 251 2 190 40 198 48 111 65 140 124 180 159 178 119 242 123 86 154 178 132 230 205 50 74 111 0 246 244 197 216 175 87 33 118 232 50 29 63 211 95 75 29 246 190 144 143 155 228 253 237 169 140 13 88 57 97 144 132 201 179 199 26 96 78 198 27 26 38 172 28 75 187 210 192 141 162 177 191 184 227 255 0 116 160 1 154 229 18 202 103 174 255 0 207 180 32 87 122 183 66 92 169 60 5 1 251 176 26 126 247 41 254 199 255 0 150 86 118 185 24 87 63 124 220 124 234 86 166 8 215 198 150 101 143 79 253 60 48 219 125 105 180 115 81 161 57 95 174 17 215 185 216 207 153 223 35 156 112 13 124 153 46 8 215 104 85 97 150 19 240 189 99 247 120 183 148 175 93 198 74 193 100 43 216 240 26 211 86 132 43 223 22 46 126 117 177 111 112 84 197 16 100 33 136 35 72 50 36 51 116 170 73 167 74 153 45 186 163 229 98 166 34 164 194 245 131 3 100 234 145 253 219 126 235 150 62 138 100 140 157 88 44 0 131 23 140 72 120 9 87 242 42 134 183 88 108 165 58 21 27 168 241 74 190 83 14 101 213 214 40 195 49 211 89 65 143 33 27 213 39 55 57 84 45 0 134 191 1 194 218 147 120 189 170 196 191 1 85 57 158 52 64 17 188 163 21 185 187 172 93 121 183 71 116 215 210 199 125 175 161 28 204 222 89 116 241 200 25 75 223 70 46 11 133 29 215 152 177 78 215 238 13 130 80 136 1 37 113 132 6 20 169 87 209 42 142 73 158 92 148 166 0 17 136 72 121 52 231 147 138 29 34 204 179 99 65 170 84 122 236 195 52 140 253 214 130 146 194 176 35 29 221 143 6 255 0 217 236 8 172 2 53 235 206 187 70 172 220 132 150 73 87 172 80 187 86 43 154 115 201 170 213 8 41 123 82 76 242 21 153 148 188 145 20 2 121 154 110 86 96 82 237 57 64 163 65 13 128 68 177 200 89 104 4 192 16 107 154 180 11 28 133 134 24 77 92 64 172 74 209 24 13 4 94 68 205 94 180 32 194 13 144 158 187 204 214 167 201 78 66 9 183 102 172 65 207 90 145 178 79 150 197 241 73 163 112 128 6 47 32 2 202 54 87 177 15 108 110 54 212 105 134 25 202 68 123 152 43 83 126 158 193 78 93 81 211 149 49 54 169 212 161 106 173 134 246 161 159 147 206 144 98 92 111 186 246 1 185 43 81 129 140 89 66 100 136 88 205 116 173 24 118 76 204 222 152 220 173 215 197 116 215 210 199 125 175 198 74 176 36 163 31 22 245 61 88 157 4 17 11 126 24 131 137 27 139 23 113 6 34 111 74 216 209 53 147 219 199 98 135 151 113 223 200 53 152 9 96 235 59 85 181 84 86 198 120 91 171 10 21 154 16 70 174 27 16 142 10 144 84 101 149 172 81 194 69 89 216 198 22 70 241 247 117 114 198 4 207 154 33 149 179 31 97 218 231 120 205 251 29 156 140 72 184 147 78 46 153 221 214 46 216 171 87 233 175 165 142 251 95 204 231 173 158 164 42 99 79 105 168 213 247 19 22 55 205 233 120 213 175 238 73 5 199 10 249 27 179 55 168 198 228 120 197 160 221 76 41 72 155 24 79 178 55 12 207 122 73 229 148 130 150 66 200 227 111 39 88 137 238 9 28 251 189 94 122 215 76 31 244 99 190 215 243 54 43 4 196 32 226 81 214 163 82 155 43 65 243 40 231 43 66 197 23 72 189 70 55 35 198 45 22 89 90 158 246 149 12 107 229 194 49 12 48 176 42 254 40 149 97 53 102 187 138 68 199 246 82 5 136 46 211 90 142 232 116 127 211 233 160 55 142 59 237 127 51 63 128 213 39 114 15 97 166 66 12 181 30 158 81 137 8 18 5 132 32 243 120 197 160 222 151 100 243 117 122 86 26 51 20 221 230 73 137 53 214 69 148 76 228 175 40 252 14 47 26 189 53 244 177 223 107 249 153 250 52 94 72 116 164 234 16 140 27 33 137 222 74 215 164 53 239 106 50 198 252 137 8 52 27 210 79 24 70 206 102 173 72 15 170 224 243 165 115 27 125 17 171 73 93 5 121 59 212 83 132 160 233 227 23 90 68 154 16 138 179 246 189 53 244 177 223 107 249 144 6 5 77 88 44 153 153 190 11 244 90 220 49 76 25 190 78 247 176 152 12 59 33 244 201 194 68 199 158 220 239 153 72 125 229 140 202 251 135 244 179 22 113 252 22 126 211 166 190 150 59 237 127 31 250 123 146 192 225 38 254 133 79 138 48 148 223 45 90 84 110 28 147 177 107 11 151 181 78 85 44 138 224 21 235 208 165 11 56 207 91 176 146 198 228 165 96 17 178 57 43 7 140 155 224 179 246 157 53 244 177 223 107 254 117 137 48 233 2 123 6 228 240 106 179 145 34 207 228 135 217 201 231 55 183 228 202 233 152 54 72 126 231 236 253 188 153 127 245 223 179 252 221 119 236 234 201 129 82 176 73 143 184 58 214 60 162 19 74 68 181 97 224 213 127 90 25 158 118 187 252 223 187 38 118 117 83 225 133 121 73 70 45 22 234 120 141 240 131 118 215 138 241 122 117 172 216 198 16 69 25 135 147 148 155 41 44 129 136 215 8 35 17 136 221 206 221 197 78 215 179 52 39 2 67 225 179 246 157 53 244 177 223 107 254 117 239 218 233 125 1 255 0 108 36 222 98 195 150 68 85 201 41 101 108 223 186 217 28 140 181 159 37 251 142 66 110 217 9 242 50 46 68 186 238 229 237 216 175 59 197 159 177 119 190 80 100 39 49 209 20 188 135 59 83 173 24 67 29 144 88 222 204 74 22 7 27 109 12 160 150 54 82 114 26 30 196 24 249 29 213 86 186 198 1 123 218 165 23 146 210 69 237 200 234 53 84 71 8 122 245 108 101 60 19 89 144 194 43 78 74 56 169 121 82 173 212 176 168 229 17 135 20 241 93 217 209 229 216 72 7 53 103 142 111 179 3 37 78 195 250 218 251 94 154 250 88 239 181 255 0 56 198 166 213 199 107 18 56 211 46 232 225 207 16 14 177 241 148 137 84 176 228 119 227 75 110 197 143 127 114 225 33 98 254 64 154 178 150 231 141 33 238 152 54 172 228 9 11 22 178 36 132 233 218 37 43 96 179 102 161 168 87 250 68 177 94 80 247 248 234 141 140 147 134 85 102 13 225 158 50 132 168 29 247 211 52 135 144 171 100 21 141 92 152 202 118 43 120 202 230 43 227 176 1 217 5 234 102 170 71 120 0 85 110 25 169 212 151 243 7 180 88 91 137 155 201 89 59 185 60 164 237 234 236 206 130 123 21 208 179 36 101 205 84 68 189 80 245 250 107 233 99 190 215 252 233 138 19 94 208 42 205 163 68 52 235 235 23 182 18 96 193 151 182 19 168 134 16 94 77 89 51 30 221 183 172 39 76 1 178 213 14 238 40 58 144 7 37 237 132 201 155 178 144 227 52 213 66 207 225 30 218 160 203 218 139 190 184 246 97 65 150 152 47 108 37 17 198 11 19 253 14 188 171 164 166 36 206 210 253 1 161 13 133 127 154 185 23 137 61 193 187 41 147 193 219 191 199 91 253 221 53 244 177 223 107 248 14 205 241 246 255 0 10 25 40 227 97 46 190 165 23 254 32 81 95 196 10 43 248 129 69 127 16 40 175 226 5 21 252 64 162 186 139 168 69 158 174 167 107 204 84 37 218 70 177 170 118 74 197 154 148 224 52 55 199 174 244 87 122 43 189 21 222 138 239 69 119 162 191 145 81 45 65 174 156 132 162 44 119 218 254 100 131 98 51 226 171 59 241 53 87 19 85 113 53 87 19 85 113 53 87 19 85 100 177 66 106 30 144 255 0 107 187 191 175 234 180 124 102 24 76 30 38 170 226 107 46 38 170 226 106 174 38 170 226 106 174 38 170 226 106 160 215 128 22 59 237 127 232 36 132 73 11 149 103 70 210 242 148 23 118 116 242 104 180 205 35 44 14 41 251 198 62 45 253 28 119 218 255 0 208 178 216 230 200 86 56 203 82 71 51 59 83 195 148 241 108 5 153 78 135 79 68 104 65 128 155 250 88 239 181 92 94 65 113 89 5 197 95 92 85 245 197 95 92 85 245 196 223 92 77 229 196 94 92 69 229 196 94 92 69 229 196 94 92 69 229 196 94 92 69 229 196 94 92 69 229 196 94 92 69 229 196 94 92 69 229 196 94 92 69 229 196 223 92 77 245 196 222 92 69 229 196 94 92 61 213 195 221 92 61 213 195 221 92 61 213 195 221 92 61 213 195 222 92 61 229 196 94 92 69 229 196 94 92 69 229 196 222 92 77 229 196 222 92 77 229 196 222 92 77 245 197 95 92 85 245 197 95 92 85 245 197 95 92 94 65 113 121 5 198 100 23 25 144 92 102 69 113 153 21 198 100 87 25 145 92 110 69 113 185 21 198 228 87 27 145 92 102 69 92 233 187 118 155 255 0 7 187 178 174 6 237 120 113 55 151 23 144 92 94 65 113 121 5 197 228 23 25 145 92 102 69 113 121 5 198 100 87 25 145 92 110 69 113 185 37 198 228 149 33 144 32 90 224 181 193 107 130 215 5 174 11 92 22 184 45 112 90 224 181 193 107 130 215 5 174 11 92 22 184 45 112 90 224 181 193 107 130 215 5 174 11 92 22 184 45 112 90 224 181 193 107 130 215 5 174 11 92 22 184 45 112 90 224 181 193 107 130 215 5 174 11 92 22 184 45 112 90 224 181 193 107 130 215 5 174 11 92 22 184 45 112 90 224 181 193 107 130 215 5 174 11 92 22 184 45 112 90 224 181 193 107 130 215 5 174 11 92 22 184 45 112 90 224 181 193 107 130 215 5 174 11 92 22 184 45 112 90 224 181 193 107 130 215 5 174 11 92 22 184 45 112 90 224 181 193 14 17 105 47 255 196 0 72 17 0 1 3 2 3 3 5 11 9 8 2 1 5 0 0 0 1 0 2 3 4 17 5 18 33 19 49 81 16 20 34 65 145 6 32 50 52 97 113 129 161 177 193 240 21 35 48 66 80 82 209 225 241 36 51 53 64 96 114 130 178 67 83 162 98 146 194 210 226 255 218 0 8 1 3 1 1 63 1 254 148 186 186 186 186 186 186 186 186 186 186 186 186 186 186 186 186 186 186 186 204 85 254 134 253 229 213 249 111 252 173 249 111 244 55 228 186 191 127 116 62 219 198 101 116 112 93 166 218 254 40 215 77 113 211 119 105 92 254 127 190 238 210 185 253 71 223 119 105 92 254 163 239 187 180 174 127 81 247 221 218 87 202 21 71 235 30 213 207 106 126 251 187 74 134 122 185 55 72 238 212 41 113 27 120 103 181 77 61 84 91 220 238 210 155 93 84 255 0 172 238 210 131 235 173 225 59 181 115 218 166 157 94 238 213 207 167 255 0 177 221 165 124 161 55 223 119 105 95 40 205 247 221 218 87 202 19 253 247 118 149 207 106 191 236 61 171 158 213 127 216 123 87 63 159 239 187 180 175 148 102 251 238 237 43 229 25 254 251 187 74 249 70 127 190 238 210 176 202 201 95 56 5 196 250 121 1 191 219 88 247 139 127 151 184 167 111 11 50 204 179 44 193 56 189 82 97 210 204 161 195 35 139 114 1 202 162 141 146 168 176 216 227 85 79 179 116 10 72 100 123 183 39 5 28 55 66 141 10 37 204 216 185 163 19 168 211 169 121 52 88 67 127 104 28 131 237 172 119 197 189 63 138 234 29 229 154 169 228 12 240 147 49 124 190 2 110 62 212 204 76 72 175 159 85 183 99 180 78 179 119 44 153 247 170 138 43 76 161 166 46 117 130 167 192 75 148 93 205 4 59 155 106 151 185 144 171 168 140 6 202 214 10 170 43 114 97 62 48 57 27 246 214 59 226 190 159 197 117 14 88 32 218 33 129 189 202 92 43 32 233 41 155 151 193 84 241 198 226 168 41 160 1 98 88 128 128 88 46 122 251 221 82 227 22 61 37 13 72 152 104 171 25 103 172 12 181 210 106 169 24 44 164 140 130 181 86 43 23 161 18 179 50 127 135 149 72 51 169 227 200 86 17 227 3 145 189 231 55 151 78 137 215 118 155 252 201 205 45 54 58 31 181 113 207 22 244 254 40 156 206 11 10 195 54 190 18 198 40 118 39 162 176 204 63 102 165 153 177 5 87 139 109 116 79 118 171 49 10 42 151 5 44 183 77 78 55 43 11 151 40 85 78 184 88 11 122 74 149 132 169 238 213 153 93 72 51 199 101 139 211 108 164 186 58 42 136 115 5 133 199 106 129 200 222 90 90 103 206 252 141 222 169 233 171 226 143 39 64 219 113 59 194 173 195 166 128 103 146 218 158 254 150 149 245 15 202 205 234 44 62 89 38 116 66 217 154 164 140 177 229 167 120 54 236 229 142 55 61 217 90 46 74 24 5 77 190 175 111 228 170 41 228 133 249 94 44 123 200 227 47 120 104 222 77 187 87 200 21 63 250 123 127 37 87 134 77 78 204 207 181 175 110 255 0 228 105 119 230 25 45 124 221 74 158 157 211 72 24 221 229 79 78 232 101 217 187 120 78 193 229 23 37 195 32 23 205 212 160 133 210 200 24 221 229 84 211 186 9 50 59 127 123 71 135 75 83 114 203 105 197 73 25 99 139 78 241 203 73 65 37 64 113 101 180 84 148 219 119 228 4 3 229 85 24 123 224 140 57 250 19 213 214 169 40 36 168 14 45 182 159 67 141 248 183 167 241 88 101 38 121 84 108 201 185 79 77 180 58 173 197 98 49 146 20 24 31 90 196 233 118 69 102 28 155 34 142 138 8 110 163 110 64 139 174 187 159 102 170 25 3 84 150 122 202 172 183 57 119 77 78 50 221 71 67 153 84 209 229 10 146 59 77 200 222 86 189 205 55 6 203 9 130 87 252 244 175 57 7 151 127 228 177 76 71 156 190 195 192 27 191 21 139 97 241 210 228 200 78 183 223 228 183 145 77 135 198 218 38 205 115 152 246 123 20 56 76 47 165 18 185 228 113 225 191 204 142 23 79 52 69 212 238 36 142 63 160 88 94 27 29 76 110 46 38 227 227 130 167 195 40 230 5 172 144 151 142 207 211 210 176 218 76 213 46 140 184 139 95 193 211 113 84 116 97 245 178 71 157 194 215 212 29 119 141 229 69 66 37 171 124 101 214 0 157 78 253 254 213 22 29 65 49 44 141 238 204 62 56 41 162 49 189 204 61 70 203 5 157 145 212 116 250 197 149 94 29 86 101 50 71 39 174 222 142 11 17 168 146 73 88 217 217 150 220 56 44 83 15 109 51 155 144 221 174 10 187 15 142 158 6 27 157 163 187 60 189 92 148 94 49 31 247 183 218 177 74 90 185 37 6 43 218 220 109 173 202 173 134 166 43 9 137 215 203 116 48 90 109 155 100 115 200 22 23 212 117 250 20 56 116 18 75 39 78 209 179 205 115 241 230 76 194 233 42 24 237 131 205 199 31 209 97 52 17 213 23 103 39 78 31 162 167 195 40 222 227 30 208 153 60 155 189 154 161 135 60 212 236 61 126 78 43 154 210 131 205 182 207 191 11 233 236 178 130 135 45 112 132 187 142 163 126 226 83 176 221 165 115 162 204 108 53 185 212 238 31 138 230 212 142 60 223 108 251 142 162 116 211 209 101 67 14 202 189 172 189 236 125 202 167 14 53 21 175 46 209 130 215 62 132 105 225 125 64 142 35 209 226 126 2 24 126 31 180 217 103 118 127 143 37 149 117 47 55 152 179 127 36 114 10 10 72 239 189 196 95 211 191 176 104 177 250 124 178 137 6 231 123 71 228 168 112 200 223 9 154 87 16 223 34 151 15 165 124 38 72 36 221 212 126 2 194 41 4 204 144 230 112 183 3 110 43 14 195 154 248 204 242 56 134 142 27 244 235 85 20 176 84 70 100 142 98 75 126 241 245 106 187 159 253 220 190 143 122 143 12 141 144 109 106 9 28 0 223 235 11 15 195 133 70 103 19 150 54 252 126 168 97 180 149 13 60 221 231 48 227 250 5 134 225 156 224 184 188 217 173 222 161 195 40 167 146 209 200 116 222 58 252 227 69 204 26 106 204 33 214 104 235 41 152 117 4 143 49 53 238 205 241 228 85 48 24 101 115 15 82 198 91 154 1 253 223 138 195 160 203 110 247 167 162 199 175 182 86 42 48 85 27 27 36 106 175 12 179 238 163 109 147 144 58 172 13 247 77 222 155 185 84 27 32 219 133 81 86 214 5 89 89 156 166 169 69 212 12 179 249 27 203 133 208 243 153 53 240 70 245 138 138 153 62 106 38 28 131 215 249 5 53 20 241 11 189 164 5 143 66 249 68 101 130 251 247 121 108 171 163 116 120 107 90 119 139 38 194 233 112 176 214 239 255 0 244 176 56 95 11 100 123 198 81 229 242 93 96 167 230 38 248 234 93 207 120 203 191 183 222 213 135 127 18 151 252 255 0 217 97 255 0 196 166 244 255 0 176 84 116 81 207 87 57 127 213 113 211 206 74 195 228 38 66 4 27 54 142 191 143 205 98 62 51 39 247 21 133 210 197 81 38 71 146 52 209 115 90 250 121 50 199 154 221 92 63 5 142 248 188 121 255 0 121 249 107 235 178 161 13 172 166 140 59 124 110 30 175 201 98 178 58 162 175 35 122 186 35 207 250 170 138 121 33 118 87 139 21 69 227 17 255 0 123 125 171 28 170 154 57 192 99 136 25 125 229 75 81 36 190 27 137 88 185 253 138 47 71 250 172 38 134 39 194 233 92 220 228 117 121 130 195 36 115 195 190 107 102 58 188 190 197 220 231 133 39 163 222 176 143 30 30 149 3 195 113 89 47 214 45 236 252 21 70 27 52 149 142 109 141 139 175 126 171 21 69 75 205 241 22 50 247 223 254 165 72 217 142 36 253 145 25 180 223 195 40 77 168 116 178 108 166 167 244 239 30 125 219 189 40 83 178 28 77 173 102 239 102 133 73 87 27 231 117 51 134 241 240 59 21 46 27 251 102 198 77 195 214 162 232 84 236 227 167 176 31 91 243 252 214 59 227 94 128 176 218 125 181 67 91 213 188 249 135 197 149 125 109 17 126 73 65 113 111 199 21 81 178 172 162 59 63 169 187 209 249 44 49 213 113 65 157 128 61 156 62 183 161 54 56 107 35 113 124 57 60 187 189 122 110 93 207 254 238 95 71 189 97 78 170 142 18 232 192 115 56 117 250 20 108 138 177 174 218 67 147 203 187 215 162 238 123 70 203 232 247 169 203 113 26 82 230 142 155 58 190 56 143 90 193 93 122 71 180 0 93 115 167 29 21 44 181 23 37 148 237 101 191 199 220 176 199 237 161 157 131 194 113 113 255 0 220 22 11 69 51 39 204 246 150 128 20 116 113 212 87 203 159 112 234 84 110 249 242 198 193 145 163 175 227 243 88 191 141 191 227 168 42 150 102 109 188 168 55 45 187 221 132 142 33 99 24 52 175 155 122 249 26 78 41 152 52 170 142 35 26 171 109 194 203 201 178 179 110 187 159 147 167 170 102 244 55 42 128 170 235 182 122 41 37 115 214 238 66 22 78 151 32 229 166 172 150 11 236 205 175 230 247 175 150 171 62 255 0 169 191 130 168 196 106 39 110 87 186 227 204 63 5 30 43 82 198 100 15 209 73 93 59 227 217 185 215 111 199 94 245 181 44 194 218 90 108 225 255 0 217 84 98 85 19 55 43 157 167 98 130 182 104 90 90 195 96 124 202 158 166 72 29 153 134 199 227 138 142 178 86 72 100 105 233 31 55 90 193 225 58 213 61 222 16 62 221 254 164 234 183 178 161 242 70 109 114 125 101 59 22 170 115 129 207 187 204 166 149 210 184 185 219 202 107 136 55 27 208 198 106 192 182 127 80 83 78 249 93 153 230 229 83 86 77 5 246 110 181 254 58 211 37 115 95 156 111 85 21 18 76 236 207 55 63 28 19 30 88 224 225 188 42 138 153 39 118 105 13 207 199 14 73 171 102 149 129 142 55 3 118 229 77 89 52 30 1 178 249 90 171 62 108 250 250 61 155 148 21 179 66 73 97 177 59 244 10 26 135 196 252 237 54 114 146 119 189 249 201 233 47 150 42 242 219 63 168 40 234 36 100 155 64 122 92 119 251 83 234 101 116 155 66 122 124 119 47 150 106 237 108 254 160 153 81 35 100 218 3 210 227 191 218 164 169 145 242 109 28 122 92 124 202 74 233 222 240 242 238 144 235 208 123 20 152 181 83 247 191 220 170 42 100 157 217 158 110 126 56 42 122 169 96 55 140 219 179 222 158 242 231 18 119 149 79 91 52 23 217 186 215 243 123 213 61 100 208 30 131 172 167 196 234 101 110 87 59 78 197 79 91 52 32 134 27 95 205 239 84 245 114 192 110 199 89 77 138 84 202 220 174 118 157 138 158 178 104 1 12 54 191 153 83 213 75 1 188 102 201 149 82 177 249 218 108 126 61 10 108 86 166 70 229 46 211 177 96 241 177 206 63 56 88 254 175 42 138 25 227 151 107 81 40 45 27 186 191 0 166 172 119 57 116 177 155 106 159 139 85 56 142 158 229 60 239 153 217 159 169 77 3 173 77 87 27 21 52 217 188 20 35 121 222 139 105 219 251 194 132 244 223 241 149 207 92 208 85 86 36 100 37 49 141 226 154 215 245 46 114 229 21 66 50 89 203 15 166 50 204 170 40 91 179 200 159 24 100 160 42 108 70 6 141 233 184 204 74 191 22 190 129 60 245 242 30 80 57 7 219 88 132 165 140 209 83 97 179 85 149 134 225 34 153 189 37 91 59 58 150 41 88 95 189 81 213 24 206 245 14 36 217 152 83 33 204 247 46 143 82 207 39 82 218 130 172 131 51 61 96 20 227 104 185 169 117 202 171 160 58 189 69 79 101 16 106 14 3 66 129 201 169 69 30 91 114 117 125 181 69 75 28 196 237 55 5 89 143 67 73 208 167 181 211 171 179 248 101 73 136 192 206 181 81 158 121 28 230 238 71 0 46 163 18 13 234 152 56 27 43 114 239 76 137 70 50 174 230 221 243 234 91 68 213 95 81 153 221 241 239 10 111 219 88 189 76 145 70 50 117 161 35 131 179 117 169 106 39 147 173 13 60 37 220 206 205 236 123 78 245 135 64 54 121 10 197 112 113 6 161 52 235 203 19 19 91 101 189 202 134 164 195 50 168 197 12 141 87 187 190 132 242 15 182 187 160 240 25 233 247 35 189 89 89 97 83 186 158 80 122 149 25 19 66 30 20 204 99 198 82 177 14 231 156 58 77 82 198 89 45 144 103 77 0 135 36 109 250 55 114 15 182 187 160 240 25 233 247 35 189 108 74 134 146 235 35 89 209 88 54 50 105 250 39 114 137 209 85 234 213 51 36 26 44 102 147 44 232 51 85 110 70 168 155 244 103 144 125 181 138 71 153 161 50 156 43 119 152 94 42 234 103 221 65 81 206 25 180 88 171 52 71 66 138 42 50 163 119 209 158 65 246 133 12 80 200 252 178 27 95 113 242 249 85 109 52 48 181 161 174 204 254 187 120 63 67 94 58 33 91 189 17 103 93 207 86 233 179 43 19 150 229 77 14 136 242 49 169 131 232 207 32 221 252 252 116 241 252 152 95 148 102 227 215 225 114 97 165 129 175 205 9 147 204 47 101 28 18 73 224 52 159 48 186 100 110 121 179 65 39 200 176 184 1 171 12 145 188 116 62 101 93 68 243 82 241 19 13 135 1 228 81 183 231 0 60 86 47 4 81 79 22 86 105 192 117 234 177 50 194 241 150 51 30 155 136 178 101 28 239 23 12 113 30 100 90 65 177 26 174 107 53 237 145 215 62 66 164 133 241 155 61 164 121 212 84 242 201 224 52 149 36 47 140 217 237 35 207 201 13 53 53 45 43 102 145 185 203 173 235 245 38 138 26 200 221 102 136 220 60 195 245 88 110 81 41 205 25 147 77 192 95 172 106 131 136 169 37 140 250 199 163 111 85 149 113 107 98 118 206 7 55 49 212 150 238 243 112 88 44 44 146 162 206 23 22 252 21 84 4 212 189 177 183 172 232 19 162 123 93 148 131 126 29 105 212 85 0 92 198 235 121 185 43 252 17 201 101 149 6 161 18 107 108 169 159 98 138 42 72 209 22 80 142 240 253 1 228 27 191 159 139 248 73 248 250 220 157 207 254 238 95 71 189 119 55 255 0 47 248 255 0 242 88 23 141 122 10 131 248 177 244 251 22 33 138 205 21 73 107 124 22 245 113 243 172 106 54 237 97 144 117 159 194 203 24 241 168 61 31 236 177 207 25 143 75 233 187 142 171 246 231 61 174 113 108 109 225 191 227 181 99 205 27 120 207 198 245 141 214 201 6 76 154 19 125 86 42 237 173 4 111 59 244 245 133 77 207 93 78 214 198 4 109 227 199 218 177 118 222 136 102 33 196 17 175 37 46 37 45 60 65 147 70 114 117 124 29 233 176 208 87 2 35 25 93 217 234 220 176 38 22 85 189 167 169 164 127 228 22 31 60 76 171 156 56 217 197 198 221 165 108 235 89 115 152 74 15 29 22 10 8 172 117 197 180 58 112 213 77 23 54 142 89 99 23 121 215 227 204 176 201 102 231 5 205 110 119 121 125 183 84 109 170 18 29 179 219 175 213 85 237 2 162 64 62 241 85 194 224 32 197 205 208 167 66 37 149 93 65 191 144 173 233 241 171 91 189 61 249 228 27 191 159 162 172 164 230 123 25 93 237 227 230 91 60 39 239 31 252 191 5 130 101 180 249 119 95 79 90 193 171 153 78 247 7 238 119 185 83 77 135 211 77 118 184 155 245 245 15 82 138 178 33 136 25 111 208 215 93 120 118 169 164 195 170 38 206 231 17 236 119 170 254 69 136 226 45 158 118 91 192 111 193 42 190 174 41 234 97 217 155 234 56 241 242 172 90 113 13 100 79 61 67 222 85 84 244 18 72 38 47 36 143 171 240 61 235 23 171 134 119 198 230 58 246 223 189 99 85 145 78 89 179 55 181 248 251 213 85 100 47 161 100 96 244 133 184 240 82 213 209 213 66 209 35 203 72 234 248 5 85 214 210 201 71 179 97 181 183 15 55 171 146 28 66 154 120 4 85 29 93 127 29 106 58 170 10 48 76 87 115 150 21 90 200 234 93 36 166 215 7 180 144 122 148 21 116 187 121 54 173 187 92 227 99 109 218 170 121 168 105 73 115 100 113 242 124 0 168 107 216 43 29 44 154 3 127 141 21 62 40 214 85 188 147 120 222 127 66 168 107 105 169 170 31 99 243 110 220 117 211 201 197 83 207 65 77 54 102 188 184 158 190 30 175 197 87 61 175 157 238 105 184 37 77 185 55 190 9 174 209 13 255 0 78 121 7 216 63 45 145 6 205 140 182 150 191 171 135 125 135 215 10 103 19 147 53 253 94 162 171 107 29 83 38 115 252 148 219 147 123 248 29 162 27 254 156 242 15 182 156 204 200 83 133 176 11 96 22 192 45 128 91 0 182 1 53 150 71 232 110 174 174 174 174 175 200 80 254 131 28 183 87 250 81 253 6 63 145 31 210 131 147 42 202 178 172 171 42 202 178 172 171 42 202 178 172 171 42 202 178 172 171 42 202 178 172 171 42 202 178 171 43 43 43 21 170 178 178 202 172 172 172 172 172 172 172 172 172 172 85 138 177 86 43 85 170 177 90 173 86 171 85 170 213 107 201 170 215 130 215 130 215 130 215 130 215 130 215 130 215 130 215 130 215 130 215 130 213 106 172 85 149 138 177 86 86 86 86 86 239 44 172 155 253 12 239 228 63 255 196 0 82 17 0 0 4 3 2 7 7 14 11 8 2 2 3 0 0 0 0 1 2 3 4 5 17 18 22 19 33 49 83 145 162 210 6 16 34 65 81 82 225 20 21 32 50 53 97 113 130 147 161 177 178 209 240 7 35 52 66 80 84 98 114 129 193 211 36 48 51 68 115 131 179 195 64 96 146 241 37 67 194 255 218 0 8 1 2 1 1 63 1 250 2 159 74 94 201 174 123 85 59 34 246 77 115 218 169 217 23 178 107 158 213 78 200 189 147 92 246 170 118 69 236 154 231 245 83 178 47 100 219 61 170 157 145 123 38 217 253 84 236 139 217 53 207 106 167 100 94 185 174 123 85 59 34 246 77 115 218 169 217 23 178 107 159 243 39 100 94 201 182 127 85 59 34 246 77 115 218 169 217 23 174 107 158 213 78 200 189 115 92 246 170 118 69 236 154 231 181 83 178 47 92 215 61 170 157 145 122 230 185 237 84 236 139 215 53 207 106 167 100 94 201 174 123 85 59 34 246 77 115 218 169 217 23 178 107 158 213 78 200 189 179 108 246 170 54 69 237 155 103 181 81 178 47 100 215 61 170 157 145 123 38 185 237 84 108 139 217 53 207 106 163 100 94 217 182 123 85 27 34 246 205 179 218 168 217 23 182 109 158 213 70 200 189 179 108 246 170 54 69 237 155 103 181 81 178 47 100 215 61 170 141 145 123 38 185 237 84 236 139 217 53 207 106 167 100 94 217 182 123 85 27 34 246 77 115 218 168 217 23 182 109 159 213 78 200 189 179 108 246 170 54 69 236 155 103 181 81 178 47 108 219 61 170 141 145 123 102 217 237 84 108 139 219 54 207 106 163 100 94 217 182 123 85 27 34 246 205 179 218 168 217 23 182 109 158 213 70 200 189 179 108 246 170 54 69 237 155 103 181 81 178 47 108 219 61 170 141 145 123 102 217 237 84 108 139 219 54 207 106 163 100 94 217 182 123 85 27 34 246 205 179 218 168 217 23 178 107 158 213 70 200 189 179 108 246 170 54 69 238 155 103 181 81 178 47 108 219 61 170 141 145 123 38 217 237 84 108 139 217 53 207 106 163 100 94 217 182 123 85 27 34 246 205 179 218 168 217 23 178 109 158 213 70 200 189 147 92 246 170 54 69 237 155 103 181 81 178 47 100 219 61 170 157 145 123 102 217 237 84 108 139 219 54 207 106 163 100 94 217 182 123 85 27 34 246 77 115 218 168 217 23 182 109 158 213 70 200 189 179 108 246 170 54 69 237 155 103 181 81 178 47 108 219 61 170 141 145 123 102 217 237 84 108 139 219 54 207 106 163 100 94 217 182 123 85 27 34 246 205 179 218 168 217 23 182 109 158 213 70 200 189 179 108 246 170 54 69 237 155 103 181 81 178 32 247 91 50 66 234 181 19 137 228 50 34 208 105 34 199 164 187 198 37 147 70 99 155 182 217 228 202 71 148 191 247 197 237 175 211 127 7 16 80 241 83 53 161 230 210 226 112 38 116 81 18 138 182 209 142 134 46 228 174 159 36 103 201 163 216 46 220 175 234 140 249 52 123 5 219 149 253 81 159 38 143 96 187 114 191 170 51 228 209 236 23 110 87 245 70 124 154 61 130 237 74 254 168 207 147 71 176 93 169 95 213 25 242 104 246 3 220 236 164 191 149 103 201 163 100 28 146 82 95 202 49 228 209 178 17 32 148 175 249 70 60 154 61 129 91 156 148 167 249 86 124 154 61 131 172 18 143 170 179 228 209 236 9 220 244 171 234 140 121 52 108 139 187 42 250 163 30 77 30 193 118 229 127 84 103 201 163 216 46 220 175 234 140 249 52 123 5 219 149 125 81 159 38 143 96 187 146 175 170 51 228 209 236 23 114 85 245 70 124 154 61 130 237 202 190 168 199 147 70 200 187 82 175 170 51 228 209 236 23 114 85 245 70 124 154 61 130 238 74 190 168 207 147 71 176 78 183 57 46 76 11 166 136 102 137 86 21 67 38 210 70 71 67 239 8 88 181 74 162 146 231 205 200 162 229 46 140 190 96 211 137 117 4 164 157 72 242 125 53 240 91 221 103 63 162 175 93 176 93 169 246 54 131 177 20 10 116 204 42 161 167 13 33 199 212 160 202 85 80 218 233 188 110 80 28 72 56 161 213 70 58 168 193 69 24 40 145 93 233 194 191 98 115 238 159 160 79 123 97 184 245 154 165 109 215 237 121 148 127 77 124 22 247 89 207 232 171 215 108 39 181 61 250 10 152 90 77 67 168 234 58 148 45 20 20 168 193 239 91 4 254 32 243 180 10 152 165 33 201 201 133 77 140 55 55 49 11 20 78 16 165 147 16 238 140 162 113 242 39 62 233 250 4 247 183 27 143 238 83 94 55 174 127 77 124 22 247 89 207 232 171 215 108 39 181 61 245 170 200 92 201 41 13 76 109 152 65 153 133 40 58 160 203 85 29 78 22 192 91 99 140 77 92 178 144 244 82 132 43 164 161 136 98 16 47 217 80 51 180 144 131 160 97 98 113 242 39 62 233 250 4 247 183 27 142 238 75 126 55 174 125 133 75 233 111 130 254 235 47 250 42 245 219 31 48 56 224 105 209 28 245 179 6 205 161 9 1 96 17 80 25 131 64 36 130 80 200 34 87 140 87 24 156 100 14 158 49 6 98 206 242 49 40 65 187 84 130 198 24 114 201 137 195 149 133 115 238 159 160 79 123 113 184 238 229 53 227 122 231 190 165 18 74 166 45 195 40 234 171 95 135 24 195 54 241 209 190 46 205 75 36 149 76 44 201 41 37 30 67 22 113 17 242 239 153 208 117 74 59 224 138 210 106 89 59 3 29 82 216 105 68 225 209 61 158 20 191 26 210 129 210 176 85 48 146 170 109 16 39 72 241 22 94 64 225 88 202 18 86 147 104 178 118 43 113 41 202 44 226 35 229 223 83 132 156 161 223 139 202 27 225 153 217 226 11 89 36 241 133 32 211 251 143 130 238 235 47 250 42 245 219 15 59 68 24 194 132 184 8 170 33 242 133 197 80 50 229 161 65 81 104 80 60 176 188 102 13 56 196 225 120 131 164 25 118 128 157 4 12 74 157 168 114 48 146 66 30 46 217 136 247 43 14 231 221 63 64 158 246 227 113 221 202 111 194 175 88 247 211 78 49 19 20 155 56 52 32 173 31 123 223 24 98 21 16 172 227 254 34 178 251 4 37 167 201 71 201 79 204 51 85 188 104 6 165 225 176 100 154 133 97 26 89 37 212 210 162 40 205 149 17 7 201 246 104 107 69 8 196 71 197 160 151 74 215 151 190 34 94 52 176 133 217 44 116 240 100 17 75 50 101 10 166 82 47 64 112 162 26 73 45 105 224 152 82 56 36 174 35 15 54 106 70 33 15 48 134 38 137 181 55 211 223 229 12 50 210 90 86 13 86 171 233 16 213 117 39 202 66 20 148 242 149 205 32 162 10 237 79 192 98 93 21 12 219 6 78 17 90 175 37 113 80 130 98 155 89 29 132 150 138 2 117 197 172 210 148 212 195 169 113 9 71 7 132 160 230 21 147 44 42 113 24 140 180 200 116 159 66 73 203 20 72 93 18 217 57 196 97 181 196 37 24 108 18 105 203 199 237 15 187 86 176 180 252 15 194 22 253 33 82 186 101 228 201 148 194 21 18 218 48 216 36 211 151 143 211 81 21 194 106 222 74 144 40 132 162 13 4 88 214 117 244 133 178 182 219 180 178 199 200 44 196 96 240 150 120 33 37 109 178 94 244 60 47 85 60 162 228 35 232 243 136 37 97 24 52 113 163 208 97 74 85 178 66 75 25 139 15 33 194 67 136 203 200 30 89 180 162 42 17 248 68 99 138 39 176 105 42 153 254 96 150 251 39 131 91 68 68 124 222 129 26 154 45 34 41 195 122 34 195 37 81 19 241 70 73 44 106 48 225 60 205 48 169 161 24 137 60 17 211 41 152 120 159 101 63 24 140 161 68 105 100 156 166 80 162 136 67 120 67 79 4 88 170 9 124 163 224 199 186 174 127 69 94 187 97 229 98 61 243 48 64 200 234 32 203 16 170 66 205 33 74 50 48 81 84 32 165 87 122 217 137 195 96 247 152 69 65 21 144 150 173 134 83 96 24 100 232 35 23 88 103 62 233 250 4 247 183 27 142 238 83 94 21 122 199 190 243 150 8 75 78 29 162 194 173 101 111 209 210 23 17 134 94 35 168 148 184 148 165 210 81 208 241 121 170 37 202 37 70 25 151 33 134 158 75 83 11 74 201 208 38 110 225 86 148 150 51 246 137 169 126 208 223 191 24 158 44 204 169 223 252 140 76 62 70 215 129 62 129 31 242 38 127 15 65 136 231 212 220 59 54 120 210 94 130 17 205 145 54 70 111 91 81 241 7 62 76 215 221 32 227 138 108 173 16 40 168 103 17 85 146 79 211 237 18 164 85 199 41 218 116 226 243 84 58 230 1 213 211 34 200 253 244 136 116 117 60 29 79 42 177 251 60 192 142 217 90 32 174 212 252 6 37 77 179 212 170 82 208 70 118 191 34 10 117 178 42 33 36 66 78 116 136 116 251 199 233 17 81 110 97 73 36 171 53 227 19 22 210 131 47 141 194 31 31 190 49 62 237 136 77 215 241 52 240 8 132 86 94 223 123 164 117 122 83 10 138 30 66 34 167 124 132 67 202 118 24 213 239 148 17 183 214 246 237 214 152 242 125 227 6 201 54 222 17 167 255 0 12 134 28 113 110 193 91 94 95 78 48 220 34 209 14 152 164 158 69 100 252 244 136 216 163 83 68 234 120 252 193 196 218 135 182 183 234 124 223 127 96 132 249 17 120 76 58 171 41 51 16 80 113 132 213 182 212 73 37 123 242 6 146 184 104 162 194 124 255 0 207 164 68 52 193 189 97 102 105 62 94 32 181 187 10 180 146 29 182 71 197 209 140 77 17 101 212 137 153 67 173 251 43 170 79 151 139 241 14 41 216 85 21 135 173 247 186 49 137 186 14 218 66 16 169 124 69 131 58 165 101 151 223 144 252 194 53 52 136 73 168 232 92 188 152 196 91 108 21 9 81 10 89 127 229 249 136 226 38 158 101 95 53 36 157 83 19 104 180 184 84 35 168 136 121 77 193 181 103 140 69 32 176 4 165 61 105 71 197 239 208 19 242 38 252 31 153 143 131 67 255 0 229 28 254 138 189 118 193 149 72 194 194 1 152 73 135 34 72 132 52 112 41 131 97 83 6 131 143 212 197 188 65 38 12 97 200 213 65 61 135 51 69 72 43 121 149 89 13 38 216 65 89 25 119 170 31 249 58 254 233 250 4 247 183 27 141 238 91 126 21 122 199 190 148 182 174 220 170 48 48 57 191 58 189 161 37 12 222 52 38 135 248 130 68 50 149 85 39 24 37 176 133 91 74 104 175 127 192 66 165 43 142 225 21 82 117 245 66 74 25 133 85 9 199 164 41 109 184 187 75 42 208 62 180 58 174 17 84 131 171 109 100 73 50 224 151 228 35 159 182 73 101 9 201 236 17 9 73 52 132 44 171 66 47 64 74 33 9 52 177 148 58 226 108 18 83 144 130 21 64 150 96 207 42 60 230 21 18 132 162 195 101 66 5 131 87 110 85 17 49 24 92 92 66 173 165 22 82 91 196 226 16 221 132 21 55 152 83 109 212 200 177 152 248 181 246 229 81 102 18 205 155 24 191 31 72 121 77 59 75 69 82 32 251 169 119 46 64 111 164 144 73 73 98 32 134 161 50 154 61 33 247 144 178 179 78 8 55 80 77 146 8 184 33 12 194 101 179 231 48 251 233 89 88 167 4 97 144 150 236 36 184 33 38 202 81 100 147 136 195 104 132 79 204 6 182 210 139 40 42 23 191 40 36 161 93 176 118 43 130 73 78 66 28 5 255 0 16 171 64 234 218 116 248 101 80 215 83 52 118 146 156 122 67 138 109 213 213 101 80 243 141 188 124 50 168 104 161 90 59 73 78 61 33 197 182 234 234 178 168 125 228 60 124 50 169 5 184 218 138 134 85 32 210 97 91 225 18 113 136 168 133 31 205 180 144 235 172 184 222 13 134 142 209 251 247 204 60 148 161 132 54 188 116 32 132 66 36 187 76 161 110 32 145 97 5 66 31 7 82 245 179 10 184 131 63 227 100 46 75 6 162 199 225 58 254 5 151 30 43 38 160 164 243 130 226 16 222 65 133 90 178 6 152 113 71 140 117 177 53 9 132 36 144 82 194 140 96 200 96 65 21 68 91 248 18 16 209 167 133 48 209 219 104 196 92 181 243 86 34 196 58 212 182 138 161 136 90 2 77 5 123 7 15 226 23 247 79 208 39 189 184 220 119 114 155 240 171 214 63 220 34 32 210 65 71 94 193 168 131 64 117 211 95 208 232 89 164 117 122 131 142 154 247 247 13 58 114 49 24 19 65 37 12 182 217 86 188 35 60 101 201 74 25 39 197 251 86 184 46 197 37 177 17 48 55 177 36 49 4 181 101 18 248 68 52 88 194 219 66 242 12 26 136 194 221 161 11 68 49 11 27 202 85 149 13 208 47 130 20 229 138 24 130 143 200 65 248 235 56 130 220 89 227 24 133 65 2 223 89 252 66 254 233 250 4 247 183 27 142 238 83 126 21 122 199 244 214 225 162 29 65 68 18 56 236 127 251 16 242 167 28 225 56 24 130 160 40 112 135 27 110 132 96 167 36 81 38 216 136 89 80 90 236 76 184 66 126 139 73 20 182 160 194 108 22 249 16 34 22 5 129 65 64 71 65 186 22 74 217 211 32 220 119 114 155 240 171 214 63 166 190 10 217 37 170 42 188 88 63 246 3 178 101 65 131 34 20 27 161 67 136 176 225 8 247 254 62 209 8 9 158 27 17 131 46 198 208 139 102 217 2 133 36 40 47 38 249 24 37 11 98 216 180 42 38 206 41 48 166 100 116 225 55 231 113 36 122 75 16 158 187 142 131 113 253 202 107 198 245 207 233 175 130 142 218 47 251 127 236 7 189 65 50 97 47 52 105 17 8 52 58 104 48 218 236 157 82 32 103 213 224 152 66 137 109 138 240 55 233 81 30 245 146 222 175 99 111 176 154 159 236 167 247 219 255 0 42 4 247 183 27 142 238 83 94 55 174 127 77 124 20 118 209 127 219 255 0 96 49 108 59 17 64 107 83 156 33 51 149 19 220 34 11 135 84 46 37 2 229 33 37 137 182 192 35 197 191 106 202 68 74 240 138 22 5 158 202 216 182 45 137 153 254 203 227 183 254 84 9 239 110 55 29 220 166 188 111 92 254 154 248 49 114 201 197 127 111 253 131 170 12 25 131 223 141 129 195 34 130 41 38 218 236 13 207 91 53 139 28 26 10 10 135 209 105 33 92 21 111 16 82 5 105 217 204 254 75 227 183 254 84 9 239 110 55 29 220 166 188 111 92 254 144 85 170 112 113 134 154 90 170 163 42 39 206 15 247 31 6 159 205 127 111 253 128 143 177 194 80 78 224 75 248 131 115 240 41 75 86 132 108 73 33 219 32 151 81 65 16 245 146 6 171 106 223 202 22 158 206 103 242 95 29 191 242 160 79 123 113 184 238 229 183 227 122 198 15 254 115 85 84 81 35 139 160 60 139 42 160 52 26 140 184 100 159 8 136 36 160 193 226 202 30 169 55 82 4 73 38 16 163 60 102 13 56 132 10 13 198 86 102 121 3 44 252 89 213 68 175 0 53 39 148 80 84 178 212 22 60 128 204 139 40 44 121 55 153 67 175 189 131 65 208 58 136 168 117 17 26 141 105 63 196 69 51 79 157 100 58 209 18 10 167 197 151 243 16 172 90 62 19 137 58 113 17 227 252 68 95 4 177 5 54 73 101 10 62 50 28 85 4 164 159 25 3 33 240 103 150 43 251 127 236 236 178 5 51 132 8 69 150 132 201 95 24 33 98 172 132 47 8 66 60 17 111 160 194 209 81 96 84 90 236 38 127 37 241 219 255 0 42 4 247 183 27 142 238 83 94 55 174 96 255 0 231 66 252 185 30 255 0 52 69 118 226 35 42 4 237 52 82 127 31 200 77 219 36 144 125 4 80 41 62 241 122 68 52 4 57 194 146 151 219 43 205 224 18 226 180 219 133 205 233 18 223 146 187 239 196 37 201 35 133 89 228 199 249 4 245 33 32 202 134 181 114 251 251 4 181 22 161 215 222 63 200 74 97 26 116 150 167 49 210 130 1 162 234 151 16 89 40 126 145 251 42 29 59 117 95 120 75 210 74 137 81 17 80 168 120 131 133 69 2 133 75 199 84 46 138 247 208 22 184 184 90 26 142 210 116 244 137 150 54 201 92 180 244 8 232 117 155 12 154 113 149 146 174 130 6 184 53 208 172 155 103 164 76 136 176 120 177 228 12 175 170 92 105 183 14 136 44 94 254 17 48 75 84 179 91 36 34 78 28 219 44 26 15 194 22 143 137 109 92 169 33 240 105 252 215 246 255 0 217 216 84 84 101 13 226 14 255 0 8 70 149 92 48 72 12 68 88 32 227 214 140 83 22 249 16 32 104 6 208 54 193 239 204 254 77 227 183 254 84 9 239 110 55 29 220 166 188 111 92 255 0 231 161 152 140 53 182 139 39 131 243 48 184 120 245 29 77 5 230 246 136 230 236 27 117 203 255 0 161 55 135 83 170 224 229 33 16 196 100 66 56 68 95 153 249 196 75 74 234 68 183 243 168 9 152 198 217 36 17 17 151 157 62 113 13 13 212 204 40 143 183 87 185 8 54 148 212 51 132 174 63 96 128 111 9 4 226 121 79 242 33 10 136 196 183 130 36 164 136 254 119 185 254 66 5 135 25 105 105 89 101 201 144 75 154 83 77 46 223 29 4 3 74 76 66 150 125 173 12 67 181 18 195 170 54 210 74 175 191 41 8 104 103 219 138 194 47 25 29 106 126 31 56 123 183 9 134 121 43 182 206 129 212 81 111 153 19 156 20 137 146 48 137 74 27 226 167 152 132 75 110 224 219 176 170 41 41 44 95 128 121 17 145 36 73 82 18 93 255 0 115 49 29 15 241 41 105 24 204 168 34 161 237 194 182 154 81 196 23 254 196 84 51 175 54 147 249 229 151 190 31 110 50 33 171 38 148 164 139 139 151 206 29 169 48 132 171 17 145 15 131 111 230 127 183 254 192 125 149 67 209 37 212 193 101 83 49 103 24 90 113 2 222 178 8 133 5 59 11 32 218 6 200 153 183 72 83 251 237 255 0 149 2 123 219 141 199 119 41 175 27 214 63 249 233 112 210 58 165 97 136 68 27 152 69 171 142 180 17 79 90 93 71 84 44 27 166 48 234 10 116 204 41 188 54 43 84 5 131 97 156 26 1 62 162 6 242 140 97 12 19 134 18 234 136 97 212 12 194 86 100 14 33 98 217 140 33 142 168 88 182 96 220 49 133 80 234 133 133 56 102 62 13 242 196 255 0 111 253 128 251 56 149 153 52 41 193 21 198 22 120 129 2 22 127 113 93 233 207 201 15 239 183 254 84 9 239 110 55 29 220 166 188 111 92 254 129 175 103 95 248 91 149 159 28 181 78 17 55 111 9 103 231 89 165 155 95 101 92 224 91 170 136 81 127 1 30 84 255 0 68 94 136 140 202 60 169 254 136 189 17 25 148 121 83 253 17 122 34 51 40 242 167 250 34 244 68 102 81 229 79 244 69 232 136 204 163 202 159 232 139 209 17 153 71 149 63 209 12 207 149 16 233 54 226 18 138 228 162 205 85 60 180 254 26 120 136 206 181 226 167 24 33 64 160 68 8 24 125 204 27 102 174 62 46 42 159 191 120 252 0 231 17 5 255 0 212 143 40 127 164 58 244 254 105 30 80 255 0 72 117 233 236 210 60 161 254 144 235 211 217 164 121 67 253 33 215 167 179 72 242 135 250 67 175 79 230 145 229 15 244 135 94 158 205 35 202 31 233 14 189 61 154 71 148 63 210 17 147 23 162 27 193 154 16 146 180 147 173 179 62 213 100 172 152 50 228 229 19 197 17 168 110 59 185 77 120 222 185 253 52 133 217 5 49 119 148 117 201 222 81 215 39 121 71 92 157 229 29 114 119 148 117 201 222 81 215 39 121 68 146 102 178 143 102 184 234 170 127 229 192 175 225 95 200 30 32 65 68 41 191 51 139 35 240 16 143 154 46 222 33 215 39 121 71 92 93 229 29 114 119 148 117 201 222 81 215 39 121 71 92 157 229 29 114 119 148 117 201 222 80 235 198 188 163 113 221 202 107 198 245 207 254 132 218 205 10 37 36 232 100 37 19 52 199 195 147 152 173 252 226 228 63 97 229 45 21 58 30 253 145 65 27 26 130 73 165 58 125 130 111 49 249 164 20 117 63 221 110 59 185 77 120 222 185 255 0 208 228 211 69 193 68 19 133 143 148 185 75 223 207 65 7 51 134 139 77 91 63 192 242 251 248 4 92 81 54 86 75 182 244 7 39 170 135 224 146 177 7 183 64 78 99 81 136 201 201 171 17 7 28 53 158 63 221 238 59 185 77 120 222 185 239 93 57 158 111 206 46 148 207 55 231 23 70 103 155 243 139 163 51 205 249 197 209 153 243 60 226 232 204 249 158 113 116 38 92 194 210 46 124 203 152 90 69 207 153 115 75 72 185 211 46 105 105 23 58 101 205 45 34 230 204 185 165 164 92 217 151 52 180 244 11 155 50 230 150 158 129 115 102 92 210 211 208 46 108 203 154 90 122 5 205 153 115 75 79 64 185 179 46 66 211 208 46 108 203 154 90 122 5 205 153 115 75 79 64 185 179 46 105 105 232 23 54 101 205 45 61 2 230 204 185 165 164 92 217 151 52 180 139 155 50 230 150 145 115 102 92 210 210 46 108 203 154 90 69 205 153 115 75 72 185 179 46 105 105 23 54 101 205 45 34 231 76 185 165 164 92 233 151 52 180 139 157 50 230 150 145 115 102 92 210 210 46 108 203 154 90 69 205 153 115 75 72 185 179 46 66 211 208 46 108 203 144 180 244 11 153 50 228 45 61 2 230 76 185 19 167 160 92 217 151 33 105 232 23 50 101 200 157 61 2 230 204 185 165 167 160 92 217 151 52 180 139 155 50 230 150 145 115 102 92 210 210 46 116 203 154 90 69 206 153 115 75 72 185 211 46 105 105 23 58 101 205 45 34 231 204 185 133 164 92 249 159 48 180 139 159 50 230 22 145 115 230 92 194 210 46 124 203 152 90 69 207 153 115 11 72 185 211 46 105 105 23 58 101 205 45 34 231 76 185 165 164 92 233 151 52 180 139 155 50 230 150 145 115 102 92 210 211 208 46 100 203 145 58 122 5 204 153 114 39 79 64 185 115 46 68 233 232 23 46 101 200 157 61 2 27 115 51 70 185 186 122 7 90 166 118 105 66 211 208 31 220 156 205 197 87 131 167 160 92 201 159 217 211 208 46 92 199 236 233 232 23 42 99 246 116 244 11 149 49 251 58 122 5 202 152 253 157 39 236 23 42 99 246 52 159 176 92 169 143 216 210 126 193 114 102 95 99 79 64 185 83 31 179 167 160 92 153 143 216 210 126 193 114 102 63 99 73 251 5 201 152 253 141 39 178 46 76 199 236 105 61 145 185 248 23 32 224 80 203 148 180 85 201 223 51 63 250 61 59 50 253 199 255 196 0 84 16 0 1 3 1 5 2 8 7 12 5 10 6 2 3 0 0 1 0 2 3 17 4 18 33 49 50 19 65 16 34 81 97 113 129 147 210 5 20 51 145 146 161 177 32 35 52 66 82 98 114 131 162 178 193 209 67 80 116 130 240 48 53 64 115 148 163 179 180 211 212 36 83 96 194 225 241 21 99 68 84 226 255 218 0 8 1 1 0 6 63 2 212 181 45 75 82 212 181 45 75 82 212 181 45 75 82 212 181 45 75 82 212 181 45 75 82 212 181 45 75 82 212 181 45 75 82 212 181 45 75 82 212 181 45 75 82 212 181 45 75 82 212 181 45 75 82 212 181 45 75 82 212 181 45 75 82 212 181 45 75 82 212 181 45 75 82 212 181 45 75 82 212 181 45 75 82 212 181 45 75 82 212 181 45 75 82 212 181 45 75 82 207 220 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 103 23 117 107 103 103 23 117 107 103 103 23 117 107 103 101 23 117 107 103 103 23 117 107 103 103 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 103 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 103 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 107 103 101 23 117 86 74 72 62 77 208 223 91 104 182 145 146 230 3 71 53 218 155 210 170 63 93 230 86 101 102 86 101 102 86 101 102 83 158 247 220 99 113 46 113 193 83 223 222 223 150 214 225 211 157 86 214 9 132 172 174 109 57 30 66 55 30 98 179 91 31 28 135 105 90 1 127 127 37 114 170 204 172 202 204 172 202 205 102 86 101 102 86 245 153 89 149 60 181 62 245 12 146 122 44 37 9 177 115 93 197 148 124 166 156 243 84 105 171 28 47 176 243 31 250 7 102 6 218 209 79 36 211 151 245 142 248 190 222 101 89 94 46 183 27 141 210 211 209 188 243 159 82 207 157 109 108 242 152 221 145 228 112 228 112 200 173 156 179 81 135 54 196 46 215 166 153 167 137 109 48 217 35 109 29 33 113 109 231 158 70 130 125 104 89 163 182 177 244 196 109 103 14 56 238 169 62 174 10 49 181 85 115 192 246 172 92 226 178 119 157 100 124 235 7 145 211 69 197 227 142 108 252 220 54 239 216 167 255 0 5 220 22 50 73 38 143 110 60 129 248 127 208 14 142 9 188 93 238 160 218 211 16 57 177 9 237 109 176 178 82 48 120 143 10 215 18 106 226 79 157 1 20 241 56 223 5 204 107 119 111 55 158 77 79 82 51 205 45 154 6 140 239 184 229 205 65 157 119 99 85 64 55 145 92 113 30 97 78 10 211 152 112 89 100 144 154 53 166 50 119 155 164 143 98 46 113 17 198 204 73 56 4 91 5 158 89 183 51 117 238 83 74 26 5 116 88 169 253 99 143 178 234 194 43 56 234 63 154 165 162 202 199 243 196 75 125 70 171 105 3 178 160 123 14 109 60 132 126 60 27 65 152 213 209 193 110 253 138 127 240 93 193 99 233 151 239 254 186 235 247 14 154 121 27 28 108 197 206 118 10 59 60 98 208 247 75 32 138 59 177 156 92 77 0 166 121 243 34 199 108 162 125 43 116 155 239 21 31 25 172 192 117 184 38 57 207 137 228 230 232 116 117 98 112 235 69 241 64 251 67 244 181 140 231 222 121 2 247 242 111 178 167 100 224 91 118 188 141 62 223 90 47 154 190 46 205 95 56 252 138 250 202 54 113 4 108 137 212 52 140 93 56 100 106 49 175 58 115 236 122 67 1 216 26 146 231 3 141 9 59 199 173 92 146 50 30 41 197 52 200 138 140 148 35 231 200 62 210 176 88 218 94 25 105 181 22 72 230 130 104 3 73 23 185 6 27 215 252 59 110 92 165 239 157 206 74 133 159 164 23 156 126 137 160 165 122 71 8 149 238 45 129 220 73 254 143 202 61 25 240 80 244 34 211 213 208 173 255 0 177 79 254 19 184 44 125 50 253 255 0 112 48 56 229 206 169 191 245 171 126 151 225 193 226 246 121 118 109 102 51 200 205 85 220 193 200 57 79 87 42 146 205 51 203 232 193 44 46 57 221 173 28 210 119 208 145 143 63 50 54 88 79 252 61 153 228 115 189 249 94 232 27 149 138 120 161 219 63 110 210 216 162 163 111 56 80 10 83 1 76 235 214 169 59 120 132 223 124 69 197 215 221 76 228 113 204 244 1 214 131 24 208 214 176 6 181 173 220 6 64 1 187 130 228 241 135 140 199 49 229 7 48 132 113 52 53 141 200 14 17 105 179 199 91 92 124 106 55 2 248 190 79 61 51 31 249 69 179 70 248 233 104 146 224 144 57 166 238 27 142 57 175 7 69 107 181 195 103 55 39 158 54 204 224 47 186 232 132 1 92 43 199 170 217 141 82 99 212 161 151 26 73 17 139 160 198 73 30 112 239 87 184 100 111 53 150 196 227 100 119 45 214 249 51 232 240 97 173 184 143 201 91 193 192 248 156 248 125 83 184 44 93 50 253 254 26 12 213 34 109 156 231 117 210 19 86 215 145 3 104 115 73 123 169 80 107 143 187 160 78 140 82 243 51 234 78 110 246 155 167 171 220 124 95 58 184 225 117 220 158 227 167 5 147 124 232 23 210 133 215 69 57 125 221 120 183 46 223 218 110 92 93 232 198 117 55 63 106 115 141 208 198 182 253 253 200 93 222 140 103 80 247 38 238 228 230 239 105 161 225 52 166 11 138 70 57 85 52 201 70 151 26 6 239 193 18 41 130 168 254 65 191 75 240 43 138 125 254 106 178 30 110 87 116 10 249 211 248 199 223 41 127 219 154 37 149 115 218 215 182 63 222 20 161 230 7 20 204 41 114 131 168 110 60 171 198 45 18 6 69 12 110 154 22 239 47 52 101 0 235 175 50 171 94 219 59 14 156 4 146 30 115 184 121 186 211 203 200 121 141 247 47 143 140 8 4 26 110 56 240 221 188 43 157 56 41 187 51 195 224 248 93 97 241 227 114 88 95 77 151 19 110 90 90 73 127 52 79 52 4 31 58 101 223 136 192 198 198 107 164 12 49 59 211 161 146 163 120 35 54 56 100 225 206 17 178 182 49 182 96 188 231 154 134 93 220 224 64 57 250 183 163 124 98 41 255 0 165 116 154 11 181 235 86 253 163 155 28 14 177 137 229 145 230 128 24 222 0 169 56 12 36 40 179 193 240 120 201 203 109 41 44 143 168 82 243 189 93 42 27 53 174 59 57 142 210 237 152 48 53 204 44 117 9 4 213 198 163 10 110 252 23 132 141 5 124 70 124 119 249 51 193 98 250 223 191 195 129 32 242 133 227 19 218 101 217 71 198 163 158 235 184 114 227 144 85 101 91 100 132 251 216 249 95 60 168 25 82 118 151 179 249 180 252 211 103 169 196 129 231 66 208 233 54 109 165 231 57 217 1 84 233 108 146 237 46 102 211 252 4 231 3 146 115 33 180 95 145 131 18 6 29 34 185 167 196 92 230 57 151 154 227 25 161 171 77 10 180 195 180 148 92 189 198 14 227 96 224 49 42 120 239 232 149 237 46 121 199 7 81 58 24 103 113 149 130 190 108 57 49 79 140 234 141 197 135 169 2 236 1 194 167 114 125 162 43 99 136 46 188 198 94 45 187 243 70 234 40 133 166 33 14 207 228 102 90 124 225 70 43 86 200 48 39 153 69 82 118 146 157 62 213 84 207 166 223 106 141 246 119 189 176 8 67 93 118 75 188 107 206 221 94 74 38 9 229 145 223 24 7 62 246 73 178 73 54 202 58 11 206 121 0 98 167 247 209 177 132 210 253 70 52 204 133 33 178 202 92 232 243 14 30 108 192 205 19 83 128 25 39 89 219 104 189 59 107 80 220 176 206 134 152 209 62 207 241 216 234 31 205 120 145 182 78 217 14 27 54 184 220 174 116 202 234 241 82 252 178 123 57 46 213 79 0 123 157 112 7 23 60 212 210 232 252 211 172 126 59 104 108 141 253 25 36 50 173 199 34 46 161 13 224 235 143 165 230 228 112 86 153 15 22 8 218 202 191 158 224 192 43 144 186 172 173 54 142 246 175 22 219 157 177 194 131 42 231 74 210 137 208 56 212 138 16 121 143 4 79 165 76 147 52 17 243 107 87 253 148 201 71 147 180 183 237 55 243 20 78 158 87 93 141 130 164 132 233 172 246 128 235 153 181 248 117 99 68 243 126 70 80 126 140 209 27 68 143 115 35 101 77 91 159 23 122 54 152 45 178 72 232 240 187 104 119 93 5 234 21 41 31 35 240 40 79 107 115 163 189 161 159 27 152 99 188 167 72 77 200 219 137 113 79 241 73 239 189 153 181 216 123 64 69 213 186 214 106 69 182 121 239 92 214 220 143 72 168 82 65 122 155 55 80 151 35 103 100 238 219 10 225 209 158 234 41 33 58 163 52 67 165 62 90 251 219 125 238 31 160 14 125 103 31 253 112 57 163 226 83 206 120 43 200 202 121 207 254 22 63 192 86 151 114 206 27 135 32 96 239 112 61 145 202 97 123 129 2 86 128 75 121 232 112 42 120 237 83 186 103 67 51 93 182 35 23 131 136 113 57 130 57 186 147 162 146 75 210 217 168 57 223 25 210 250 250 143 58 163 69 7 12 49 218 163 34 103 139 93 178 55 146 46 200 91 179 142 48 62 132 78 200 128 139 185 42 81 39 125 74 190 237 12 130 89 157 208 208 14 7 151 155 145 86 184 154 56 244 242 250 214 87 159 149 50 243 242 43 207 53 127 168 116 14 172 207 5 158 209 105 115 196 118 115 182 164 109 46 47 118 65 163 118 250 146 127 21 111 181 89 156 93 27 236 54 128 47 96 65 17 184 16 71 87 5 139 166 95 191 195 77 195 52 44 176 193 43 108 173 214 255 0 151 205 208 22 44 45 221 82 172 50 70 199 61 173 47 14 184 9 213 118 153 116 40 193 192 237 35 170 217 198 11 157 64 235 163 125 217 42 66 146 87 131 27 110 2 235 194 154 106 74 181 156 181 83 209 87 183 152 136 251 65 91 191 175 159 252 69 111 233 147 252 64 188 37 180 196 50 212 254 47 45 94 229 35 91 224 239 20 137 160 131 35 133 28 77 114 200 43 111 237 47 246 171 143 222 48 78 16 62 118 10 241 46 154 176 245 100 172 65 244 241 135 63 141 78 75 188 127 181 69 0 118 171 60 205 119 72 27 186 194 112 110 44 179 157 131 122 70 175 90 217 156 29 76 147 62 155 125 170 40 160 180 75 19 13 153 174 45 99 136 21 190 252 80 116 211 62 90 12 47 154 171 19 55 25 217 95 64 167 72 98 219 22 2 68 99 125 5 105 78 117 33 54 33 99 143 13 152 165 11 179 173 112 9 253 13 252 85 253 247 159 248 171 96 63 24 93 29 55 90 127 5 105 18 70 234 73 51 158 215 211 139 117 198 185 166 68 210 11 133 107 79 160 85 175 96 230 7 134 179 7 228 70 205 139 197 173 126 12 189 141 221 176 23 153 244 184 195 1 214 188 94 29 56 58 239 201 171 114 83 120 42 86 16 31 7 148 229 46 26 124 201 246 73 115 103 218 27 143 90 216 67 224 219 141 107 184 214 167 13 220 161 212 252 83 255 0 170 98 3 206 182 22 152 101 158 72 62 72 226 138 208 159 140 19 157 102 15 30 42 106 198 191 87 16 101 191 54 173 188 13 100 236 204 197 83 123 158 237 20 175 158 194 108 142 103 233 28 41 214 29 65 90 115 169 143 205 252 10 50 64 35 154 58 147 177 36 222 174 251 180 82 27 69 128 217 92 223 210 56 83 204 250 3 130 149 217 134 211 241 91 102 48 178 123 44 165 219 44 240 228 253 230 250 212 205 141 140 149 225 174 164 79 201 220 92 1 233 201 61 241 120 42 205 102 123 69 215 85 187 46 170 209 120 70 46 43 101 153 243 56 83 33 181 109 5 57 129 87 159 27 163 115 65 105 190 41 94 142 85 111 15 201 175 173 222 84 232 163 240 119 139 198 194 224 233 220 49 60 148 52 223 210 85 179 233 183 238 4 242 13 28 243 179 20 231 4 159 178 10 196 82 184 245 42 245 14 149 87 106 126 39 242 84 121 166 255 0 193 57 229 165 161 212 45 175 200 220 71 40 175 34 45 14 188 115 52 255 0 194 148 76 101 218 62 208 231 186 48 220 64 160 3 126 240 23 30 115 21 104 61 240 81 94 62 18 179 129 149 43 249 132 203 69 134 211 12 237 158 204 209 43 35 58 92 211 76 79 56 197 89 157 122 232 219 182 55 99 147 100 226 17 208 74 116 112 206 201 158 205 77 128 223 187 204 110 84 3 204 85 89 101 144 231 168 177 190 162 84 144 49 245 150 45 99 29 217 128 114 52 174 52 203 173 88 173 46 194 133 209 23 187 67 6 99 31 138 226 112 7 1 203 187 128 142 66 71 152 171 83 54 180 129 241 152 4 88 98 240 13 93 123 49 137 167 82 44 104 217 108 141 201 30 126 80 205 173 229 62 160 136 169 174 119 185 249 74 196 255 0 28 202 232 21 127 201 252 248 60 41 43 142 15 142 214 246 14 109 153 31 135 5 139 235 126 255 0 9 216 62 227 185 104 15 181 124 43 251 184 187 168 54 121 175 178 181 165 214 15 186 2 185 20 228 55 44 154 72 232 37 54 25 102 46 136 27 215 72 110 127 74 149 67 100 242 201 88 246 93 45 56 249 68 214 207 49 49 252 102 128 27 94 155 168 199 11 238 7 96 236 1 246 133 72 157 117 212 165 104 15 181 73 43 95 73 100 37 206 117 6 110 53 56 100 165 181 205 40 172 129 215 201 160 248 213 36 238 220 173 51 192 226 205 164 242 61 167 153 206 174 69 53 230 208 238 38 154 6 211 147 42 80 245 167 75 43 175 72 252 92 234 1 234 11 144 238 33 81 150 140 55 85 140 39 216 182 246 153 29 35 249 93 248 35 226 242 108 201 207 0 126 240 42 160 241 235 122 188 235 105 43 239 97 77 195 14 165 134 99 16 182 243 62 251 131 110 3 64 48 173 119 116 240 49 175 125 99 140 213 173 160 232 205 123 196 151 13 49 200 143 49 91 79 25 117 234 93 201 180 167 209 165 19 182 79 184 231 226 227 117 190 202 42 177 212 126 56 224 159 44 142 172 143 53 46 203 216 174 178 209 135 45 214 215 207 69 181 190 118 216 155 231 19 83 210 157 104 123 206 213 212 227 140 50 20 25 115 43 173 159 14 91 172 175 177 25 175 147 49 55 175 187 19 94 180 109 18 58 178 225 71 208 13 57 100 132 174 148 223 110 13 112 0 31 80 64 27 65 186 13 120 161 173 203 150 129 9 173 15 218 60 54 237 110 180 97 251 160 47 122 117 215 83 58 3 237 79 146 94 51 222 235 206 39 148 167 120 179 246 119 181 96 211 247 129 94 243 33 97 234 167 152 224 182 114 206 118 103 54 180 6 215 166 138 228 15 185 123 86 0 251 87 188 72 88 124 254 162 182 114 206 118 103 80 96 13 175 77 17 100 15 185 123 86 0 251 87 188 62 227 200 165 104 15 181 23 182 66 217 9 173 230 225 234 201 108 223 57 184 115 186 3 107 214 21 54 206 130 95 138 230 211 241 205 27 77 190 219 27 162 99 92 25 241 70 59 200 160 10 211 105 133 197 151 229 113 97 230 221 130 105 54 131 196 53 109 208 209 231 195 30 180 102 153 215 228 117 42 234 1 151 209 162 55 163 120 241 40 220 54 159 43 106 214 61 193 163 160 14 186 242 99 239 62 93 208 95 39 226 66 75 106 218 158 90 224 60 234 203 99 150 102 203 110 149 187 89 139 43 118 24 134 13 96 229 36 130 73 195 214 182 214 104 11 225 219 54 204 39 168 242 135 155 126 74 207 104 150 228 165 215 196 59 63 127 196 10 26 136 206 4 94 200 245 166 217 228 176 50 23 189 242 196 249 223 103 136 80 93 105 110 67 153 249 126 74 158 250 55 107 42 206 108 254 17 158 27 237 188 34 115 91 35 104 79 23 3 67 151 58 6 211 98 131 194 48 208 123 237 142 65 28 159 189 20 152 30 167 121 214 202 217 5 158 41 191 229 120 74 6 176 245 25 5 15 81 91 79 254 50 199 87 80 213 141 3 162 151 41 78 165 224 251 87 131 172 148 138 87 56 62 59 166 64 38 140 135 199 80 107 80 241 187 155 157 89 237 7 193 176 89 223 52 109 144 198 98 107 94 194 115 25 2 49 76 142 203 72 76 211 178 32 234 86 237 227 139 168 78 38 129 88 33 15 227 205 227 45 154 88 184 184 9 99 218 211 26 130 121 177 1 68 201 1 115 103 179 144 111 234 217 200 13 3 137 198 183 78 42 56 45 141 181 62 209 11 54 68 178 9 164 218 185 162 151 129 99 72 227 210 188 168 77 37 158 107 37 158 105 12 102 211 48 161 14 53 52 108 122 170 105 64 72 66 207 28 111 178 89 131 110 138 212 72 246 229 150 109 30 179 204 154 200 227 188 114 17 179 112 252 2 171 155 113 153 221 7 219 252 117 174 93 235 1 77 252 30 19 240 77 214 75 100 111 130 237 83 7 100 232 133 194 115 25 212 157 248 240 88 190 183 239 255 0 33 120 170 123 142 50 163 127 83 208 170 170 55 134 91 67 156 94 118 207 123 24 69 26 210 238 55 61 78 39 242 87 78 202 1 139 153 4 45 21 60 225 172 246 171 71 132 45 18 108 33 150 235 97 139 11 247 90 0 0 231 230 21 79 134 11 49 142 57 224 146 207 123 34 226 225 129 60 195 62 158 74 40 109 146 189 206 180 90 32 6 76 73 25 52 86 135 121 12 21 40 178 70 138 209 219 55 99 196 113 105 101 65 29 40 50 120 104 247 70 230 177 193 192 223 123 110 138 128 55 18 237 248 166 26 113 88 214 178 62 161 74 240 108 231 138 57 152 113 187 32 7 29 198 135 122 38 197 183 176 19 255 0 234 72 224 59 51 86 250 145 108 236 139 194 22 90 85 147 195 72 231 105 228 146 51 197 119 75 72 232 66 128 140 143 27 15 63 58 176 52 10 236 163 125 162 241 206 187 104 69 225 200 67 65 29 5 8 95 11 152 44 150 217 36 179 205 135 233 51 0 98 110 155 198 167 12 84 17 207 48 138 205 101 152 216 102 124 212 227 236 216 226 101 115 142 88 93 203 127 78 23 124 31 24 185 190 213 104 7 47 254 168 176 36 243 186 131 125 10 46 107 39 182 218 216 41 183 154 188 64 236 196 120 80 101 136 20 231 170 191 60 86 147 92 112 111 253 196 227 234 85 16 61 188 220 95 62 106 129 197 156 196 28 124 203 143 211 189 97 95 49 90 104 23 135 33 147 85 191 193 47 138 39 242 62 54 72 235 191 190 29 234 224 177 125 111 223 253 117 11 96 155 96 201 182 134 103 97 94 45 218 81 199 44 209 158 66 235 61 158 155 71 218 38 174 209 237 24 146 1 196 225 241 143 152 175 25 180 58 129 238 37 187 67 141 51 186 222 82 155 36 86 41 205 50 188 40 58 120 234 203 101 153 204 100 219 62 44 68 138 225 152 28 180 170 100 239 32 217 102 150 88 91 113 134 172 217 102 247 26 226 210 65 174 2 138 23 74 29 75 59 246 152 29 245 6 135 152 145 146 160 232 247 28 219 202 163 85 133 252 109 148 141 158 201 61 223 157 113 237 7 152 220 112 235 91 87 228 207 124 117 121 6 53 37 120 195 172 242 78 241 38 214 43 52 98 243 97 117 3 47 72 227 64 31 134 252 71 38 107 201 217 225 233 115 164 243 128 24 7 156 162 92 235 33 110 120 68 255 0 110 212 251 21 93 100 100 191 213 75 187 150 143 104 246 166 223 179 248 179 206 243 241 185 171 74 21 241 143 82 24 80 54 188 55 239 22 92 227 135 183 54 227 155 80 107 181 80 85 88 190 183 239 254 186 130 73 99 15 116 23 140 85 220 77 49 245 39 196 253 50 53 209 184 115 17 67 237 84 179 192 200 240 13 168 206 131 148 156 120 56 132 54 105 172 207 134 7 59 45 188 100 73 7 70 53 233 86 43 36 78 48 203 183 183 30 56 25 183 56 100 28 135 107 67 188 40 32 171 140 15 105 108 79 144 139 204 150 51 71 65 33 222 112 192 239 203 146 188 60 202 131 130 104 69 54 128 9 96 175 252 200 205 246 212 238 21 20 60 201 179 91 24 248 108 50 53 174 16 29 115 2 49 189 77 44 230 24 158 97 152 138 38 54 56 217 197 107 24 5 0 232 69 210 52 14 113 134 60 156 235 248 201 85 188 82 139 92 42 211 80 127 34 171 103 117 209 255 0 41 217 117 28 199 172 44 97 39 232 80 175 39 47 162 229 197 133 231 167 15 106 180 58 123 146 17 100 180 185 172 165 90 210 219 59 220 211 142 100 28 85 229 98 250 223 191 250 232 123 141 156 68 54 86 210 72 164 63 17 227 34 121 183 30 101 224 235 67 152 34 146 27 68 176 218 27 65 91 210 52 2 73 25 154 178 135 151 53 59 101 129 243 216 229 123 167 107 224 169 44 169 47 169 166 33 192 156 192 197 94 99 197 182 33 133 230 97 51 0 204 72 223 140 71 53 15 50 108 145 184 57 143 1 205 59 168 85 2 160 225 142 202 50 146 175 159 250 177 184 253 51 133 57 56 41 17 44 7 57 27 74 215 175 36 28 109 18 95 27 228 36 250 170 2 197 241 147 201 116 251 65 52 243 21 199 140 141 245 103 24 126 7 212 156 234 96 237 197 85 184 143 113 108 115 179 241 27 77 58 60 93 252 22 47 173 251 255 0 174 135 5 26 9 85 144 221 230 205 81 162 137 214 136 115 121 107 166 131 229 150 144 90 246 159 138 225 76 119 30 100 99 144 18 25 197 35 123 15 37 21 226 240 211 248 156 41 206 85 166 201 241 196 206 180 177 167 15 122 148 212 17 204 13 65 231 84 28 37 206 32 53 160 185 196 238 104 24 147 212 133 170 112 253 173 177 187 72 32 96 227 236 135 147 168 200 96 107 141 49 40 237 108 19 71 30 23 92 217 26 247 83 231 55 10 117 18 140 150 87 66 249 25 172 80 9 27 210 14 33 123 229 207 63 228 155 177 118 56 215 127 39 241 154 192 211 248 245 122 213 28 56 49 104 90 2 193 160 43 103 236 86 159 242 239 224 177 116 203 247 255 0 93 58 245 120 180 245 173 1 80 10 123 138 177 251 27 67 105 114 122 87 44 154 225 189 188 222 106 41 30 248 88 219 83 28 88 250 129 125 164 96 230 222 165 104 8 86 73 162 23 173 59 66 24 202 129 122 42 13 168 38 153 101 215 69 20 241 26 199 51 4 141 60 196 123 121 183 112 218 216 214 25 11 161 35 102 220 222 223 140 208 55 146 48 162 146 214 255 0 211 26 180 99 197 96 168 107 113 222 6 124 252 2 70 57 209 76 205 19 70 104 225 215 201 204 141 150 213 70 90 217 143 52 173 249 77 231 229 28 39 155 31 115 108 253 138 211 254 93 252 22 47 173 251 255 0 172 40 243 118 185 59 243 76 141 175 18 72 69 95 112 130 209 252 140 159 187 248 251 170 53 69 106 139 201 218 198 206 67 242 102 140 84 26 114 61 185 243 183 157 73 60 142 46 52 16 71 200 26 9 173 58 79 241 130 158 205 21 149 214 187 48 227 235 12 217 200 113 45 105 126 14 168 53 35 10 30 156 34 180 195 91 146 138 227 152 53 161 105 27 136 34 132 112 55 139 181 154 90 136 98 202 180 204 147 185 162 184 154 39 203 17 172 142 115 229 144 101 87 56 151 154 114 98 114 94 174 6 76 199 92 123 14 15 27 142 226 132 155 244 75 23 201 120 206 156 199 49 204 179 187 210 174 51 30 95 115 108 253 138 211 254 93 252 22 47 173 251 255 0 211 229 154 156 102 111 253 234 32 228 239 121 124 191 64 86 138 180 39 161 97 143 66 13 60 245 7 161 90 35 108 110 44 141 192 85 160 211 72 43 175 37 103 141 173 214 42 67 119 226 154 209 19 225 23 114 120 165 113 205 105 113 232 10 155 249 21 40 234 156 133 21 14 7 157 96 9 232 84 53 7 159 128 79 60 102 80 226 0 104 21 207 44 240 79 123 34 101 142 86 111 117 214 116 28 48 42 241 137 210 113 116 176 84 169 40 195 131 205 35 35 28 242 33 17 29 150 118 95 119 30 89 25 197 3 230 148 47 99 86 171 68 64 121 57 28 218 14 101 77 252 139 22 186 157 5 96 164 253 223 199 220 213 216 5 64 21 181 207 113 97 137 173 150 7 140 219 40 112 217 145 214 83 8 192 93 29 20 162 100 141 253 51 223 39 218 32 123 19 229 132 25 172 146 29 165 162 199 153 7 124 176 114 30 86 228 119 80 230 201 98 120 124 114 52 61 142 110 53 105 222 173 27 67 80 203 20 18 69 144 186 199 25 67 155 95 164 202 255 0 233 74 199 209 209 202 28 203 180 200 17 235 207 37 122 204 3 48 0 225 241 186 7 181 93 119 21 252 159 145 222 158 57 171 230 91 74 19 27 232 217 128 228 220 224 57 71 179 169 7 177 193 236 117 8 115 114 234 247 86 207 216 173 63 229 223 193 98 250 223 191 253 62 209 252 126 144 38 169 190 143 224 84 181 249 159 138 109 238 66 167 102 230 189 223 117 75 20 5 173 134 10 113 40 56 245 0 155 219 213 145 227 244 198 235 186 169 79 106 176 127 31 29 89 218 5 239 122 211 203 199 56 38 63 107 13 150 17 74 177 212 53 229 221 191 165 64 7 199 143 30 167 43 52 118 98 26 233 111 23 62 128 154 10 80 10 171 36 206 242 133 236 189 78 118 154 166 8 54 118 113 153 149 231 49 209 66 161 123 158 217 36 18 48 25 25 145 220 80 60 202 236 176 25 97 118 95 149 14 5 57 145 70 97 148 11 212 187 118 156 244 28 82 159 31 198 140 57 142 233 107 128 43 194 12 145 193 142 241 151 134 57 220 207 118 9 238 19 67 110 99 135 21 143 27 51 213 76 61 104 135 11 164 94 171 62 78 57 43 109 166 206 221 173 161 239 50 186 187 129 63 246 140 86 214 238 218 71 99 198 62 186 167 27 85 162 18 215 3 118 6 230 61 65 90 163 220 201 222 7 157 73 64 78 156 186 214 146 183 5 198 119 153 96 56 109 119 69 110 152 158 122 4 173 169 78 102 20 196 215 144 111 86 40 227 171 34 101 154 48 70 251 247 69 226 122 211 49 173 28 241 203 241 138 180 248 54 202 211 121 239 13 178 190 130 236 82 57 247 36 39 117 220 111 14 126 148 93 27 229 154 71 2 233 164 157 197 206 121 13 36 52 215 33 158 3 34 189 138 163 3 252 102 186 55 115 167 244 83 172 240 86 9 46 87 18 204 218 122 191 37 239 182 87 215 255 0 160 130 58 104 250 31 106 186 217 131 94 127 71 47 21 221 64 231 213 238 45 127 177 218 127 203 201 193 98 250 223 241 63 167 186 11 83 220 214 191 59 173 127 45 126 40 40 53 150 137 41 244 101 238 43 81 26 70 13 60 216 163 126 183 28 5 72 198 148 68 69 36 174 6 188 98 215 93 103 48 163 106 85 162 211 95 121 115 220 67 168 238 78 76 212 147 185 242 70 225 64 112 117 217 64 24 29 53 230 81 57 141 34 11 62 146 119 227 198 117 58 149 141 209 27 193 148 107 176 35 227 243 171 36 148 189 118 28 71 239 57 54 214 249 103 124 140 2 150 118 131 66 70 89 140 60 234 203 44 15 46 184 210 215 130 215 10 98 14 240 172 166 35 123 102 28 29 129 25 211 149 67 103 105 172 173 145 132 182 135 112 59 242 81 54 211 44 182 115 30 37 172 7 28 41 200 65 77 179 64 94 199 70 246 236 227 123 93 82 26 239 149 74 98 49 205 55 161 108 109 151 155 77 50 0 127 237 78 117 158 252 243 22 208 10 31 89 32 5 37 162 210 105 181 14 115 141 14 110 112 118 65 91 76 241 9 33 154 208 247 178 75 184 128 92 119 102 157 44 86 139 68 132 130 4 2 180 199 154 128 121 212 214 169 253 237 178 94 117 49 52 169 192 96 173 179 23 57 246 91 76 134 245 107 144 210 234 30 108 20 204 4 155 59 177 141 247 93 197 249 164 82 168 200 201 103 153 210 84 58 71 130 67 1 199 146 167 21 105 146 51 122 57 37 47 107 168 70 120 228 84 255 0 185 248 251 185 108 242 138 199 52 110 137 253 4 83 206 173 54 9 188 172 110 116 21 202 248 56 53 227 164 26 172 116 177 160 121 144 107 77 193 39 190 158 81 92 104 175 110 96 130 94 161 53 79 168 41 95 27 233 198 187 230 230 66 58 83 80 243 26 1 230 224 185 25 163 105 71 30 113 144 245 170 23 26 103 238 40 69 66 164 18 185 131 59 135 22 249 142 93 74 147 194 31 243 161 195 236 191 243 92 97 51 63 112 254 21 86 166 71 45 231 186 197 105 45 20 112 255 0 241 228 229 28 22 47 173 251 255 0 211 248 193 105 70 207 103 128 54 243 54 123 74 243 82 180 162 13 114 210 178 90 86 1 23 182 45 161 60 244 252 10 241 137 154 27 70 220 99 70 230 133 82 213 128 85 162 173 22 33 105 224 227 5 91 170 148 193 100 171 117 82 152 44 150 75 74 192 41 255 0 115 241 254 66 193 225 24 104 12 210 27 36 245 229 107 75 227 119 152 16 81 14 57 130 60 249 163 205 25 30 164 73 210 251 32 103 218 255 0 250 85 61 40 60 29 117 52 249 194 152 142 149 75 252 220 1 161 165 239 59 155 201 202 121 22 34 156 213 175 225 238 230 253 138 217 254 82 78 11 23 214 253 255 0 215 82 189 209 58 80 234 105 52 56 85 93 62 14 183 225 201 178 239 47 230 235 127 247 125 245 252 221 111 254 239 190 191 155 173 255 0 221 247 215 243 117 191 251 190 250 254 110 183 255 0 119 223 95 205 182 255 0 238 251 234 207 29 158 199 106 135 196 231 241 137 157 62 206 237 194 12 35 227 26 155 210 52 101 193 179 166 56 52 244 38 252 232 192 76 20 168 56 187 161 11 185 55 14 11 210 18 27 120 3 119 140 238 122 10 138 211 165 26 77 111 113 38 174 119 138 197 254 229 121 79 8 127 101 139 253 194 242 158 16 254 203 23 251 133 229 60 33 253 150 47 247 11 202 120 67 251 44 95 238 23 148 240 135 246 88 191 220 47 41 225 15 236 177 127 184 94 83 194 31 217 98 255 0 112 164 44 54 231 189 214 121 161 99 93 103 141 162 178 194 248 133 93 183 117 53 114 44 66 177 125 111 223 253 117 66 171 116 45 33 105 11 72 90 66 210 22 144 173 102 39 108 139 34 218 146 6 98 34 37 186 114 206 239 227 194 206 129 236 85 56 244 240 134 141 32 209 180 245 158 180 12 141 197 105 11 72 90 66 210 22 144 180 133 164 45 33 81 129 88 190 183 239 255 0 208 79 141 237 14 99 218 90 230 184 84 16 112 32 132 251 51 239 150 142 52 51 56 15 124 103 86 21 7 3 231 165 8 224 196 113 6 24 110 231 166 240 170 49 10 174 52 28 234 228 64 208 230 127 140 144 145 225 0 63 146 177 125 111 223 255 0 160 221 29 237 156 131 141 20 180 6 235 134 70 135 204 121 145 101 169 162 55 52 98 225 140 120 252 151 184 15 90 185 25 189 123 50 223 98 218 54 251 79 205 46 30 197 198 190 126 145 39 218 131 164 10 141 20 254 78 197 245 191 127 131 224 199 211 143 188 190 14 125 56 251 203 224 255 0 110 62 242 248 63 219 143 188 188 135 219 143 188 188 135 219 103 230 188 143 218 103 230 188 144 244 155 249 175 36 61 38 175 38 61 38 175 38 223 73 171 201 183 210 106 242 109 244 130 242 109 244 130 242 109 244 130 242 109 244 130 242 109 244 154 188 152 244 154 188 155 125 38 175 36 61 38 254 107 201 143 73 171 201 143 73 171 201 15 73 171 201 15 73 191 154 242 63 105 159 154 242 63 105 191 154 242 67 210 111 230 188 144 244 155 249 175 38 223 72 45 12 244 130 210 207 73 105 103 164 180 51 210 11 75 61 37 165 158 144 90 25 233 5 161 190 144 90 27 233 5 161 190 144 94 77 190 144 94 77 190 147 87 146 30 147 127 53 228 135 164 223 205 121 33 233 55 243 94 72 122 77 252 215 146 30 147 127 53 228 126 211 127 53 228 126 211 63 53 228 62 219 63 53 240 127 183 31 121 121 15 183 31 121 121 15 182 206 242 248 57 244 227 239 47 131 31 78 62 242 248 49 244 227 239 47 130 187 211 139 190 190 8 239 78 30 250 248 35 253 56 123 235 224 143 244 225 239 175 130 63 211 135 190 190 6 255 0 78 30 250 248 28 158 156 31 234 47 129 73 233 193 254 162 248 27 253 56 63 212 95 3 127 167 7 250 139 224 111 244 225 239 172 108 78 244 225 239 170 248 163 169 95 149 23 125 6 139 25 244 225 239 175 129 159 78 30 250 248 35 189 56 123 235 224 174 244 225 239 175 130 187 211 135 190 190 10 239 78 30 250 248 35 189 56 123 235 224 143 244 225 239 175 130 187 211 135 190 190 8 239 78 30 250 248 27 253 56 63 212 95 2 147 211 131 253 69 240 41 59 72 63 212 95 1 147 180 179 255 0 168 172 108 153 134 57 6 214 172 37 166 156 124 52 146 56 52 133 164 45 33 105 11 72 90 66 210 22 144 180 133 164 45 33 105 11 72 90 66 210 22 144 180 133 164 45 33 105 11 72 90 66 210 22 144 180 133 164 45 33 105 11 72 90 66 210 22 144 180 133 164 45 33 105 11 72 90 66 210 22 144 180 133 164 45 33 105 11 72 90 66 210 22 144 180 133 164 45 33 105 11 72 90 66 210 22 144 180 133 164 45 33 105 11 72 90 66 210 22 144 180 133 164 45 33 105 11 72 90 66 210 22 144 180 133 164 45 33 105 11 72 90 66 210 22 144 180 133 164 44 7 7 255 196 0 43 16 1 0 2 2 1 2 4 6 3 1 1 1 0 0 0 0 1 0 17 33 49 65 81 97 16 113 129 240 32 145 161 177 193 241 80 209 225 48 64 96 255 218 0 8 1 1 0 1 63 33 236 39 97 59 9 216 78 194 118 19 176 157 132 236 39 97 59 9 216 78 210 118 147 180 157 132 236 39 97 59 73 216 78 218 118 19 176 157 132 236 39 105 59 9 216 78 194 118 147 180 157 164 237 39 105 59 73 218 78 210 118 147 180 157 180 237 39 105 59 9 216 78 210 118 147 180 157 164 237 39 105 59 73 218 78 210 118 147 176 157 132 236 39 97 59 9 216 78 210 118 147 176 157 164 237 39 105 59 73 218 78 210 118 147 176 157 164 237 39 97 59 9 218 78 210 118 147 176 157 180 53 140 199 255 0 37 175 94 191 122 245 219 247 239 95 189 122 249 241 227 197 139 31 108 90 181 106 197 171 86 44 88 177 98 199 223 54 172 120 177 98 199 143 22 44 88 181 98 197 139 22 44 88 177 226 199 143 22 44 121 177 98 197 143 22 44 88 241 98 199 159 30 108 81 243 9 112 40 7 86 156 252 180 209 173 93 157 166 3 193 148 252 84 188 60 36 49 252 208 232 218 102 177 228 207 216 50 255 0 212 203 255 0 83 47 253 76 191 245 50 255 0 210 203 255 0 75 43 206 41 192 39 152 197 80 1 99 66 232 0 163 107 230 202 66 202 148 103 216 137 127 234 101 255 0 165 151 254 166 95 250 153 220 252 217 127 234 101 255 0 165 151 254 150 95 87 204 203 255 0 83 47 253 76 161 95 34 249 191 164 57 50 88 107 145 230 94 96 171 15 19 99 152 207 243 95 115 240 252 104 60 9 72 148 58 87 19 28 128 94 27 175 34 112 253 82 69 46 31 15 115 154 64 3 124 99 146 26 140 69 44 46 123 79 72 72 92 90 125 170 87 181 133 238 134 62 141 69 5 186 66 83 130 83 223 198 174 168 166 2 210 188 160 103 128 199 243 75 7 159 247 241 107 90 133 162 172 241 73 99 212 30 156 24 220 245 138 75 211 124 100 32 102 68 209 10 164 134 227 3 210 32 21 72 58 82 165 244 132 253 178 32 85 139 122 32 217 183 72 218 10 37 19 56 180 71 77 62 100 121 10 90 147 221 183 73 252 34 13 31 221 248 170 234 254 110 186 83 228 248 10 196 91 208 137 131 129 146 162 87 178 75 123 8 9 63 93 148 191 155 232 240 139 129 176 70 156 113 54 121 195 46 58 68 148 69 39 149 199 219 33 84 139 115 53 156 69 202 144 137 153 52 213 145 217 42 9 198 234 54 194 89 22 40 175 17 2 94 221 146 58 198 145 177 230 34 43 18 211 80 198 221 124 17 245 124 37 182 33 179 93 63 39 89 75 116 27 29 255 0 43 167 183 111 11 212 203 60 253 144 116 134 204 14 57 56 144 108 7 141 254 214 192 89 156 136 105 1 31 158 166 3 16 111 116 244 120 80 140 147 185 112 252 17 51 169 149 107 242 225 81 251 46 219 120 209 213 178 35 231 73 158 136 214 139 112 233 13 148 123 196 54 120 46 32 136 130 59 30 99 49 76 173 175 233 51 206 17 52 50 47 48 197 208 202 158 175 129 60 98 177 204 199 99 194 34 180 38 93 64 27 109 200 65 178 254 44 97 186 188 198 237 170 183 213 79 222 114 7 249 170 159 16 84 2 215 65 49 216 121 21 127 106 141 250 206 109 201 212 248 2 208 54 129 235 30 63 99 202 86 114 43 151 144 191 136 52 191 136 123 101 164 226 29 96 176 99 149 122 23 21 208 163 171 97 246 48 36 217 77 213 224 239 3 83 162 225 208 234 20 30 165 159 14 20 60 205 78 112 243 232 158 37 62 246 217 88 1 245 29 166 108 9 123 242 124 166 48 66 219 101 185 179 254 42 43 245 99 223 251 161 55 86 115 86 214 67 178 209 127 229 23 162 100 139 1 78 224 44 190 66 223 215 0 133 104 182 201 14 86 30 247 53 236 152 49 227 138 172 148 190 95 2 85 199 44 160 0 10 38 102 41 242 112 72 208 8 188 244 33 83 3 24 246 58 121 9 91 138 175 9 236 0 163 56 118 64 41 146 171 128 169 208 62 93 10 238 12 7 35 190 23 44 225 28 121 144 167 176 169 145 234 159 85 227 170 58 232 167 230 71 249 158 180 182 191 220 65 125 165 106 231 205 224 224 245 135 140 40 199 232 132 35 52 126 43 42 233 47 19 252 16 81 197 93 244 239 19 194 214 210 209 117 72 139 227 136 169 206 20 149 171 206 24 219 247 12 29 56 24 95 70 2 52 210 6 193 231 51 55 117 207 128 51 190 145 246 148 64 42 236 156 90 198 16 4 86 41 89 44 20 122 51 9 49 79 87 81 93 4 241 147 171 150 230 110 80 187 3 44 107 139 230 102 1 111 43 18 219 84 214 37 135 46 214 222 218 236 147 66 192 97 84 22 186 226 194 81 245 67 217 116 193 97 145 175 103 85 229 148 108 68 203 133 225 117 111 88 123 129 49 49 128 83 150 59 6 156 131 87 152 10 220 171 12 14 25 111 33 26 57 133 64 177 175 62 119 52 120 171 139 161 41 234 70 50 42 110 219 250 34 76 186 9 212 203 122 47 235 10 224 182 183 75 96 77 215 120 124 218 106 203 188 94 164 15 23 163 186 1 195 111 126 80 128 188 136 67 33 197 197 130 222 172 25 22 56 198 222 145 51 7 34 83 91 24 49 210 90 51 106 7 7 236 31 89 65 131 234 1 103 130 11 171 246 214 43 176 175 58 150 95 148 154 168 31 107 153 155 42 10 173 27 112 47 210 10 225 38 77 66 232 5 117 174 179 14 126 242 240 185 195 47 242 69 245 50 112 23 29 137 156 237 86 53 194 133 241 178 23 67 32 20 249 224 21 240 14 229 112 110 13 76 94 20 11 222 176 109 132 154 29 137 177 218 198 122 199 62 145 184 164 217 157 87 49 12 212 180 52 104 112 178 225 76 90 104 227 229 30 115 132 194 241 45 149 217 153 4 110 211 158 71 212 134 253 156 137 249 129 36 166 13 127 97 124 145 169 198 178 207 129 102 171 152 42 213 150 181 113 91 106 204 246 109 188 83 17 41 8 254 252 82 145 249 25 64 208 191 105 251 8 36 253 0 240 174 58 203 27 136 15 95 192 129 164 107 7 218 154 67 213 216 56 191 105 13 165 40 87 73 204 13 248 88 116 133 128 197 170 240 193 77 44 189 214 112 21 30 177 3 67 121 70 175 128 60 235 169 253 32 74 129 89 67 56 123 151 210 86 177 56 28 244 143 3 242 134 174 231 6 208 219 166 167 69 86 161 118 19 110 52 79 67 19 49 221 54 37 20 244 143 235 176 187 51 139 87 100 138 106 240 63 123 200 142 185 102 186 84 196 26 161 48 22 201 123 219 231 26 152 109 18 26 21 154 227 48 99 119 164 184 87 127 57 17 19 37 57 237 157 160 123 110 175 244 203 94 145 181 56 120 91 63 172 244 142 45 49 200 113 233 15 101 211 47 162 35 4 74 14 104 34 90 77 100 165 238 174 88 174 96 117 170 254 241 47 238 70 176 28 152 1 137 35 215 29 154 198 107 172 75 126 207 189 7 231 223 243 33 3 235 62 202 148 181 214 251 200 23 134 6 183 46 35 14 141 165 62 146 246 206 218 236 66 27 128 138 223 163 29 195 230 31 84 39 131 146 187 51 190 123 250 206 104 175 226 241 29 51 177 234 61 175 103 27 157 116 79 38 99 81 169 186 110 251 135 25 198 133 195 246 231 76 214 252 134 229 186 77 11 42 64 38 77 94 35 137 68 234 13 205 45 54 161 153 131 27 200 38 5 14 121 167 210 30 23 186 175 74 145 112 230 149 12 142 153 91 85 28 5 129 70 20 205 63 120 58 68 106 59 111 55 44 45 178 167 147 101 228 8 160 198 73 183 157 205 3 173 42 57 96 41 94 224 222 17 3 217 96 100 230 211 77 234 216 192 80 155 234 6 1 55 242 11 30 45 234 198 38 109 95 154 178 163 140 209 26 54 152 218 49 128 187 116 193 27 95 165 27 176 178 246 208 96 64 16 230 184 17 86 186 122 197 54 205 51 226 18 83 18 238 238 163 171 251 128 132 162 113 27 169 200 144 74 136 209 0 218 67 126 28 95 148 52 101 148 71 196 250 20 189 12 116 2 69 254 145 47 148 229 209 249 129 130 66 211 209 241 26 126 55 135 201 53 220 16 135 132 203 248 148 160 4 17 28 137 21 125 185 11 23 161 42 121 65 132 43 250 219 33 6 2 64 5 155 184 13 191 43 171 14 79 79 89 224 71 165 91 246 246 154 167 213 120 237 214 69 126 8 164 173 115 73 112 14 30 106 119 87 50 190 178 165 144 232 5 33 2 254 85 70 237 201 203 203 7 113 177 198 191 172 102 170 144 239 76 95 218 85 88 211 157 138 228 226 122 25 248 111 69 56 156 218 22 123 5 141 158 209 250 137 10 161 44 8 169 254 31 45 54 20 250 146 156 74 82 108 198 207 16 240 101 223 174 209 66 180 135 29 38 12 80 202 169 30 17 155 33 201 243 131 159 172 226 97 244 195 129 160 236 75 32 48 34 221 107 0 138 186 193 203 202 243 222 100 209 133 172 13 96 7 51 5 186 97 59 153 34 47 0 146 40 24 156 169 88 174 210 155 59 89 164 56 11 215 121 123 76 44 17 230 9 2 202 143 155 54 28 52 117 186 190 241 214 213 65 45 181 217 6 248 35 93 140 85 101 222 18 163 126 32 32 172 40 76 67 92 64 100 113 18 215 207 104 226 218 120 24 69 115 187 184 214 76 170 30 13 80 208 38 117 21 138 253 100 90 133 22 105 109 115 185 156 185 86 1 225 168 98 163 114 29 63 27 172 87 235 44 3 25 206 42 197 21 249 106 83 4 14 164 168 119 11 17 183 84 254 128 193 202 171 116 165 172 23 35 92 10 214 177 212 68 246 141 128 151 164 253 19 140 44 116 58 86 41 217 196 101 138 86 203 21 197 141 44 123 160 15 157 132 111 169 41 39 68 69 30 151 48 133 171 109 199 23 172 124 165 196 88 46 244 82 40 195 40 27 54 224 97 218 170 6 104 35 92 232 136 195 21 195 56 95 82 133 121 32 96 81 179 129 130 218 233 114 170 156 251 206 172 186 149 134 85 140 166 18 206 65 79 45 136 234 120 25 189 40 176 13 78 74 25 26 136 240 0 87 52 85 6 45 192 46 191 196 252 129 149 36 79 250 71 18 4 11 111 69 0 13 1 0 101 4 196 18 53 18 159 117 195 80 223 184 86 129 22 165 246 85 199 210 6 182 0 231 21 83 104 123 74 227 48 234 149 219 86 228 224 120 122 158 5 60 11 224 70 92 192 105 137 65 122 137 224 23 253 25 33 55 105 121 190 129 90 139 154 20 115 253 150 48 62 47 131 25 197 80 64 131 108 224 71 148 69 135 66 46 205 1 52 170 91 61 102 36 135 0 240 113 160 215 201 60 38 37 170 125 87 252 50 66 89 8 206 143 135 149 122 191 135 118 118 71 212 165 92 15 28 215 29 4 217 114 139 117 151 128 20 168 54 81 12 108 110 76 176 78 36 70 243 228 125 228 86 132 61 142 58 36 148 152 143 84 53 38 8 163 17 32 228 170 131 167 14 60 9 51 206 197 96 86 96 248 247 190 147 40 129 153 116 188 18 184 64 224 170 14 199 250 113 162 29 231 9 87 1 168 84 63 66 123 41 200 194 184 103 167 17 226 2 98 133 11 217 0 21 11 151 153 138 137 130 238 124 172 225 95 48 230 71 91 74 18 141 70 82 202 130 77 174 227 152 121 211 39 251 150 62 73 170 125 87 243 94 63 49 105 95 158 148 199 223 89 72 214 6 68 46 109 151 42 106 223 127 0 253 91 75 246 21 23 255 0 141 195 59 9 66 104 74 77 87 232 34 122 32 0 0 20 58 124 24 206 4 196 133 4 172 5 97 208 201 20 12 99 122 240 66 61 166 81 48 94 3 1 111 178 27 103 74 145 182 135 168 238 115 237 108 129 221 184 221 112 0 221 122 81 124 26 193 121 124 104 102 145 178 128 140 13 228 56 140 153 88 83 173 102 125 87 243 90 68 228 102 103 233 215 171 136 72 41 201 171 130 111 56 133 111 193 53 103 94 78 129 19 158 54 24 123 69 0 192 179 116 87 156 56 59 7 224 21 243 3 108 10 116 30 8 180 44 58 15 244 32 8 190 246 122 152 96 3 255 0 212 228 219 54 235 25 204 13 55 87 92 92 93 100 186 105 134 181 96 75 35 98 167 54 220 32 5 170 235 141 87 96 156 76 162 93 114 107 37 78 12 122 149 136 224 120 75 1 46 84 79 213 127 52 254 111 129 43 24 13 144 0 71 15 58 31 114 212 1 3 233 166 140 4 64 177 19 177 200 9 2 16 147 218 0 207 205 233 14 158 15 175 135 105 215 101 90 61 192 104 204 26 212 166 227 197 70 37 233 94 99 46 147 26 203 11 154 136 209 220 185 95 33 219 151 162 244 249 38 176 225 240 203 128 183 180 83 244 232 244 135 84 250 143 230 191 203 224 221 147 160 92 161 163 233 201 148 190 59 75 138 40 44 127 68 241 8 181 84 183 137 10 130 243 133 50 194 5 106 158 241 203 20 194 115 187 151 175 139 143 105 232 144 27 71 233 95 236 0 8 107 124 142 238 74 110 3 150 54 37 227 229 102 239 30 78 39 181 205 246 146 250 48 177 97 121 235 225 181 30 103 132 125 182 60 56 234 254 109 52 105 90 1 168 171 166 243 204 22 128 232 31 3 189 28 191 209 199 170 9 135 165 222 227 212 168 199 178 153 8 41 131 241 147 1 211 96 27 249 205 47 50 97 141 246 176 122 119 74 188 62 3 185 28 11 163 232 221 127 147 120 136 233 156 79 197 71 84 250 175 228 49 212 98 172 0 124 210 201 174 187 1 103 49 88 62 22 127 210 42 69 245 122 71 221 40 150 190 97 7 200 65 86 100 47 180 11 177 25 12 250 161 151 80 1 88 85 65 174 249 98 118 30 23 138 64 15 217 1 119 4 176 13 17 195 214 128 69 37 136 137 149 248 26 251 31 243 251 169 72 168 182 31 208 88 54 87 244 155 9 71 127 138 142 169 245 95 251 250 64 224 229 175 229 47 46 72 11 113 241 223 183 52 115 221 23 237 58 207 206 251 68 93 124 23 75 100 133 217 89 32 180 44 243 151 181 121 0 175 57 129 133 117 154 86 131 188 125 151 23 249 48 130 216 7 84 96 139 193 133 111 229 27 3 235 2 223 40 57 189 212 107 239 3 232 9 98 104 120 65 79 191 129 102 10 25 21 9 128 163 108 169 96 87 33 212 3 215 245 11 24 215 121 25 139 128 225 208 249 161 168 180 225 124 137 5 213 189 107 250 66 32 184 230 58 100 16 86 133 90 39 38 206 161 79 202 3 177 242 117 121 204 170 191 134 51 160 110 84 121 55 48 113 130 126 52 197 24 64 143 52 26 186 32 80 241 164 169 74 27 45 70 0 2 46 217 233 33 60 48 131 230 23 175 146 26 86 35 138 24 195 201 196 132 241 218 155 184 5 110 36 75 226 193 219 249 51 243 64 14 132 246 190 61 29 83 234 191 247 255 0 103 15 164 143 211 16 96 225 27 130 184 185 156 25 8 80 47 188 86 171 107 172 63 88 56 173 199 214 207 239 79 166 62 217 159 216 167 83 160 235 5 141 54 203 159 208 98 19 22 207 223 128 158 120 197 168 97 181 204 53 85 90 195 63 93 73 130 86 102 212 117 81 123 212 165 155 252 43 104 87 89 117 243 25 134 99 10 107 109 24 143 204 61 70 184 69 238 162 195 192 211 246 84 88 81 80 10 3 103 7 30 113 91 143 20 87 217 85 152 205 37 23 249 234 12 179 233 168 228 105 94 186 193 209 48 231 112 228 82 219 6 247 208 139 225 106 123 202 70 202 12 115 27 78 87 208 28 162 72 22 254 72 58 159 81 129 191 68 205 103 125 92 248 137 162 16 64 56 230 195 51 37 19 203 238 162 61 197 108 206 18 203 82 206 253 76 64 99 243 129 40 14 95 33 40 225 179 151 89 99 113 213 173 193 100 229 229 113 4 231 23 87 128 157 129 219 39 169 237 95 113 75 13 214 252 159 194 67 26 103 213 73 255 0 220 195 209 170 26 199 158 138 86 0 193 127 154 14 222 228 87 154 85 207 104 159 85 34 108 189 88 103 150 90 85 234 130 230 174 193 175 89 116 190 123 195 83 95 178 42 87 47 20 142 18 182 49 175 88 201 215 138 139 67 208 24 81 250 143 77 58 57 67 88 147 46 197 225 186 65 169 68 44 242 141 219 231 189 155 216 45 190 35 80 243 168 248 206 187 149 77 78 145 245 233 19 0 233 213 214 45 248 51 107 33 131 58 114 75 184 34 205 129 10 64 53 59 138 168 44 80 152 192 216 81 243 42 104 75 49 189 64 64 189 214 99 41 118 73 154 96 94 176 41 65 137 68 50 48 71 204 134 117 0 220 190 189 230 149 101 89 83 0 112 45 199 104 136 6 165 214 2 43 198 138 215 145 31 31 1 179 118 192 194 235 7 72 190 239 202 201 183 64 105 156 142 211 61 197 35 135 26 240 218 248 170 124 70 10 202 85 149 251 71 200 20 208 148 60 147 211 83 195 113 101 80 226 8 103 64 168 216 206 231 45 163 58 252 17 224 149 24 47 159 236 145 176 128 224 184 248 5 165 208 75 32 161 219 73 104 110 29 205 52 195 158 234 196 196 61 156 196 25 244 166 153 245 95 251 253 40 206 206 117 178 66 163 40 171 53 222 87 174 51 51 221 32 116 26 142 192 154 172 151 15 17 141 21 222 14 76 134 173 139 87 206 85 136 208 185 159 18 5 129 113 27 11 154 21 134 128 128 40 212 42 129 148 128 156 74 2 80 105 138 224 64 235 16 10 3 51 126 57 153 238 147 142 191 226 169 86 128 25 125 177 37 76 69 235 12 4 197 83 136 129 179 107 146 74 115 43 102 82 228 9 211 236 30 210 203 82 37 188 7 66 76 193 189 75 163 254 1 78 169 245 95 192 246 112 124 84 59 128 52 87 254 34 47 55 8 60 237 221 198 232 188 176 124 99 142 56 227 166 202 202 51 47 20 38 8 57 94 2 122 212 161 180 98 111 16 35 113 1 170 65 238 197 176 178 163 91 240 162 176 84 153 29 145 67 53 75 234 74 139 146 195 127 15 140 8 16 32 64 129 184 200 88 208 196 224 24 13 218 80 150 41 245 95 205 105 78 198 46 87 189 167 234 231 234 231 234 231 234 231 234 231 234 230 222 153 75 116 135 12 125 184 53 94 55 24 105 51 16 157 170 234 175 198 178 191 69 28 196 50 20 114 79 213 207 215 207 213 207 213 207 213 207 213 207 213 207 213 193 168 7 105 245 95 252 22 4 52 240 251 6 17 48 140 61 18 5 52 219 121 157 199 197 16 121 206 42 229 39 50 1 109 19 72 226 85 37 23 192 152 219 198 155 75 192 28 34 187 191 48 181 160 255 0 151 213 127 240 121 75 4 70 190 249 1 208 34 164 18 238 0 36 91 181 192 234 12 81 30 218 149 254 183 215 69 252 215 31 171 3 74 215 81 4 172 108 239 238 38 54 2 239 48 25 129 255 0 63 170 248 56 100 187 168 59 239 17 199 125 175 133 237 237 111 204 247 247 230 123 235 243 61 197 249 158 227 252 207 115 254 103 184 255 0 51 220 127 153 222 61 221 103 183 191 51 223 95 159 5 222 206 252 207 103 126 103 181 191 62 13 187 73 59 79 29 217 39 184 255 0 62 9 126 187 63 92 240 75 245 223 139 164 142 62 255 0 113 254 103 184 191 63 18 238 221 187 118 237 188 113 189 247 80 119 240 119 255 0 247 76 37 203 229 182 45 82 163 250 100 249 240 58 220 199 152 192 60 194 108 37 167 252 70 4 141 30 52 154 93 163 214 126 26 243 53 49 43 85 109 69 173 103 54 103 15 255 0 41 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 153 152 117 17 240 255 218 0 12 3 1 0 2 0 3 0 0 0 16 210 109 164 210 64 3 144 35 36 134 204 2 12 36 144 66 244 130 72 52 230 19 196 2 8 4 114 23 100 154 12 15 253 247 219 111 160 178 5 12 134 153 32 179 145 32 146 219 67 20 147 37 2 89 32 18 89 100 20 155 109 141 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 246 73 3 16 234 28 182 4 236 234 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 10 11 108 213 63 208 201 75 255 0 25 168 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 224 0 115 178 116 107 160 4 184 137 144 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 6 24 16 139 241 44 202 210 160 136 156 3 224 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 107 66 26 99 22 21 127 234 101 11 96 97 128 0 37 224 114 192 29 236 0 30 175 183 0 6 192 53 208 0 0 11 38 54 4 56 196 229 168 235 40 188 29 136 171 99 51 89 88 253 242 99 184 53 205 62 78 213 235 254 0 65 164 65 129 20 55 46 177 245 192 143 202 160 77 41 6 14 77 224 194 124 33 118 187 126 124 147 21 15 156 12 32 106 130 158 229 29 177 48 129 113 13 158 178 1 40 53 77 70 237 123 198 108 128 117 13 183 51 89 250 149 212 221 158 254 249 50 99 3 69 249 250 245 160 235 224 0 1 64 2 0 0 0 0 0 0 0 0 0 0 0 0 3 0 4 0 12 39 139 132 136 39 91 12 152 2 168 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 99 133 75 5 49 118 205 13 0 218 96 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 8 248 153 32 8 85 0 20 93 34 168 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 160 167 125 158 13 169 130 71 7 202 32 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 10 92 237 20 128 89 153 0 3 110 166 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 61 32 188 101 11 242 64 0 0 234 96 0 0 0 0 0 0 0 0 0 0 24 96 6 27 0 0 1 145 150 192 7 82 3 36 153 207 33 0 0 14 152 0 0 0 0 0 0 0 0 0 0 0 70 11 221 84 212 203 134 81 230 196 21 62 138 28 152 96 145 36 0 233 128 0 0 0 0 0 0 0 0 0 0 29 101 171 239 236 13 75 89 212 169 104 193 4 134 126 154 36 132 72 11 152 0 0 0 0 0 0 0 0 0 0 1 213 86 31 22 26 87 160 187 49 212 156 128 0 248 77 2 128 9 134 40 64 0 0 0 0 0 0 0 0 0 0 18 16 54 88 229 4 0 5 22 136 100 72 1 45 52 24 101 128 72 6 62 0 0 0 0 0 0 0 0 0 0 0 0 192 0 160 0 0 0 0 0 0 255 0 109 178 174 47 10 154 73 160 250 64 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 9 73 36 151 96 1 103 249 36 154 164 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 11 108 150 128 0 0 3 96 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 240 118 32 0 0 0 36 195 72 180 207 86 201 229 107 88 44 25 67 166 247 106 176 42 75 99 91 255 0 221 131 73 98 111 214 211 39 235 201 176 0 1 32 0 9 34 66 73 4 64 33 32 27 4 128 18 72 0 18 65 2 65 65 144 72 64 0 128 0 6 82 255 196 0 43 17 0 2 2 1 2 4 6 3 0 3 1 0 0 0 0 0 1 17 0 33 49 65 81 16 97 113 129 32 145 161 177 209 240 48 80 193 64 225 241 96 255 218 0 8 1 3 1 1 63 16 252 200 248 132 74 16 98 50 248 63 2 139 192 139 224 66 150 120 30 42 24 162 124 52 134 135 18 56 46 2 224 18 248 12 248 53 92 23 4 196 113 113 124 84 51 88 92 191 3 199 143 30 60 120 241 227 199 143 30 60 120 78 60 120 209 184 4 140 113 240 124 28 102 51 26 51 25 224 209 227 49 152 209 240 102 51 28 126 7 224 103 131 62 6 99 199 25 141 193 152 204 113 152 227 49 152 8 71 26 2 140 198 99 142 51 192 36 39 251 177 82 73 108 18 14 54 65 126 149 222 125 235 251 62 213 253 159 106 254 207 181 127 96 249 187 230 17 253 175 88 109 121 135 204 61 135 154 249 153 117 244 222 99 126 199 57 104 124 199 204 43 188 195 230 35 233 122 207 173 127 103 220 191 179 237 95 217 255 0 69 243 63 232 190 98 62 215 172 250 151 246 125 75 251 62 165 253 128 71 60 18 35 222 23 164 75 247 67 78 142 6 72 145 56 0 26 17 97 56 137 9 92 24 32 106 10 177 44 20 100 32 116 96 192 168 110 8 19 155 57 240 28 34 0 224 71 92 67 12 121 126 235 219 251 65 226 81 225 156 114 17 128 230 38 56 188 36 94 33 174 4 20 80 78 16 60 16 93 136 205 240 70 226 81 144 139 34 46 100 207 88 33 154 186 255 0 38 63 115 237 125 160 193 66 33 137 9 122 33 229 1 168 23 115 39 16 194 96 203 49 64 224 164 143 225 145 216 74 164 18 232 72 152 153 246 65 138 225 30 164 67 11 63 116 240 22 13 251 207 98 251 67 192 144 100 26 35 168 63 181 63 71 218 14 16 129 144 21 2 66 120 102 240 67 150 135 2 12 201 132 204 195 152 142 36 217 208 87 12 56 44 32 134 73 103 2 47 92 33 224 60 41 109 9 186 21 231 4 128 92 112 150 237 95 113 136 12 66 218 36 151 102 216 30 58 197 71 116 16 243 222 103 76 54 106 136 5 86 230 102 89 23 82 71 136 49 24 64 66 217 129 217 159 162 122 194 175 236 6 224 234 60 24 150 65 212 144 156 255 0 50 6 174 48 23 104 157 134 222 48 68 28 214 174 216 111 92 68 55 98 116 40 19 236 33 193 69 12 139 22 1 246 49 42 65 204 192 232 41 147 254 183 17 105 104 60 97 195 197 162 197 139 15 194 10 161 27 44 237 70 102 252 32 245 28 105 225 155 43 47 145 218 21 228 10 178 36 105 64 220 36 96 130 7 43 39 101 223 81 41 229 155 43 47 20 118 252 34 251 62 208 185 136 162 172 178 146 20 4 174 235 6 154 5 81 145 136 83 131 44 142 223 0 56 113 150 164 55 130 85 80 228 67 31 2 24 79 56 124 7 36 91 130 143 164 168 27 195 52 153 38 240 245 232 12 54 7 205 29 223 205 135 55 47 135 51 3 176 144 111 2 154 74 99 35 221 235 13 113 54 69 32 4 10 22 107 23 159 41 151 185 141 92 147 1 58 28 127 2 176 68 17 0 99 86 94 144 123 98 206 54 48 8 182 255 0 104 221 177 2 73 152 6 81 168 129 33 192 26 22 11 205 214 101 136 94 64 201 121 50 249 60 162 94 142 213 79 16 67 216 142 85 51 48 111 33 80 28 144 0 128 78 1 36 121 96 142 241 161 237 128 73 57 50 64 99 70 51 47 71 222 162 68 52 75 7 21 152 224 50 5 18 198 112 0 72 133 222 27 122 128 97 41 210 212 72 25 225 247 155 97 211 5 130 187 65 141 8 184 203 25 0 85 93 206 240 67 164 70 0 166 6 139 235 183 56 234 117 151 64 90 164 130 54 218 66 39 149 215 134 8 27 89 7 180 9 128 161 48 25 121 101 180 78 65 188 41 148 240 107 119 202 25 129 176 75 209 170 157 52 222 156 28 161 108 190 82 221 180 243 171 141 1 96 116 138 35 107 99 220 115 139 147 0 17 41 236 178 192 13 135 72 72 35 67 23 182 8 87 62 198 81 134 44 96 135 250 180 53 14 145 27 65 166 129 53 204 157 4 51 54 41 227 76 156 83 97 175 120 117 183 161 166 159 164 249 55 10 229 2 32 238 15 5 178 235 212 250 86 160 48 103 233 213 61 96 147 23 89 33 146 104 235 160 29 225 164 61 173 171 65 130 123 100 28 66 130 187 72 107 42 46 26 10 215 121 100 72 90 32 250 64 208 236 59 158 129 15 76 143 89 245 185 65 235 190 128 22 194 203 59 104 44 242 51 171 3 174 233 165 66 201 118 186 104 173 60 15 153 47 123 90 136 196 59 227 218 232 45 73 242 142 74 57 14 66 28 94 104 231 34 152 119 0 224 104 121 2 97 169 133 244 172 172 74 229 218 24 38 233 239 177 242 133 3 147 218 52 136 120 144 225 32 96 72 97 131 2 44 33 136 211 192 196 16 83 28 195 2 80 18 171 24 125 232 10 28 34 254 176 192 226 52 158 19 184 236 7 93 78 220 224 192 43 106 169 128 62 132 222 130 88 106 76 239 15 116 70 129 59 56 221 24 51 85 145 183 46 208 20 180 107 116 203 227 156 49 100 140 22 161 40 237 28 97 185 247 79 81 151 172 135 160 150 222 70 200 242 202 180 23 172 69 244 112 68 150 16 104 61 253 211 234 55 135 185 65 8 13 100 88 58 95 156 104 208 156 11 49 163 6 156 220 58 82 193 175 79 148 45 97 135 49 163 161 53 204 137 149 44 141 245 102 182 62 66 97 104 105 131 93 137 19 239 54 194 169 66 80 42 242 250 69 84 56 101 168 69 253 218 50 240 32 116 135 26 146 245 125 33 8 136 210 36 114 110 176 171 91 207 71 239 143 179 200 193 198 201 213 31 177 75 161 42 74 17 183 138 235 154 132 34 38 3 45 29 61 126 97 168 80 28 147 100 115 237 122 233 45 104 181 95 52 105 205 238 156 223 11 113 171 251 222 41 54 174 228 134 158 160 119 7 147 9 61 68 236 163 11 171 101 94 117 135 84 58 204 104 178 19 202 199 148 207 38 214 7 212 61 125 211 149 88 96 52 244 114 125 33 37 157 131 144 71 25 57 52 33 203 16 155 155 52 104 59 162 234 198 108 192 140 106 13 140 132 35 75 97 79 173 202 25 51 28 205 194 116 25 34 209 108 96 51 110 224 134 160 214 236 33 21 171 11 120 67 84 1 246 66 193 16 157 71 167 177 200 160 197 136 143 134 4 249 20 68 48 174 161 36 91 147 71 134 116 199 41 79 5 83 2 128 185 2 51 9 204 0 176 153 58 13 247 97 138 130 43 219 161 224 105 104 114 229 10 4 216 107 101 98 208 207 90 92 245 34 11 185 31 216 58 166 120 56 46 20 200 8 96 183 15 72 33 204 169 50 210 20 48 7 9 210 137 74 66 97 195 101 194 138 24 74 204 0 101 12 108 0 7 172 51 30 36 203 116 162 199 65 224 110 64 198 145 125 128 204 29 40 80 36 13 129 33 252 105 2 185 1 14 186 27 206 26 171 9 27 139 115 216 0 110 168 7 237 53 4 154 39 75 80 116 218 22 52 4 208 52 193 208 70 130 43 171 217 68 217 154 33 89 229 3 34 234 29 55 146 192 29 154 195 151 103 212 16 216 52 123 136 120 219 4 19 10 194 71 187 229 11 91 204 80 28 176 0 16 8 132 6 8 200 158 252 8 125 189 227 33 222 63 104 114 19 101 3 160 90 198 7 120 12 74 150 14 111 123 152 90 19 64 87 96 16 208 162 2 14 196 22 51 7 16 2 192 20 201 208 55 60 40 22 209 2 65 11 0 28 111 10 147 123 34 136 61 139 243 204 213 116 88 69 217 230 79 156 230 221 33 246 32 129 157 20 212 168 180 14 115 68 17 233 13 239 11 120 177 140 37 218 1 122 171 60 215 174 102 234 123 90 193 7 83 96 156 198 187 133 45 65 12 33 128 161 247 197 158 208 8 155 182 179 61 207 188 191 26 232 28 152 0 82 155 47 144 4 30 64 251 192 192 96 32 208 22 27 161 125 49 202 14 36 4 208 21 216 55 135 108 136 70 138 187 12 38 108 140 157 201 204 18 27 133 22 58 14 240 160 220 200 164 123 16 166 84 90 0 27 170 1 244 196 66 182 116 79 200 119 132 133 54 117 7 168 44 67 203 44 128 3 117 65 246 196 72 182 116 79 204 29 244 135 46 136 70 129 174 132 17 12 205 46 72 65 190 90 57 37 50 102 202 1 186 144 1 237 137 71 30 130 0 13 66 34 237 86 187 84 91 34 118 47 82 16 10 217 158 113 180 17 36 121 104 193 220 104 97 240 163 48 128 1 243 10 250 22 33 105 155 170 3 29 0 17 24 173 112 131 170 24 164 211 233 184 163 42 210 0 222 179 78 80 65 105 109 192 171 142 120 73 42 64 22 148 25 128 10 6 19 103 10 214 204 62 5 144 204 127 116 44 89 152 225 113 153 101 237 176 249 4 68 104 16 43 213 64 145 206 4 128 30 4 199 16 208 10 79 41 240 18 22 96 70 224 157 72 230 135 136 24 63 116 61 16 11 245 127 19 36 176 132 6 135 206 61 202 39 16 16 33 161 42 56 62 3 70 96 178 141 76 1 192 240 159 1 112 97 251 161 65 43 190 203 230 33 11 128 136 65 156 241 91 138 26 174 53 12 41 32 120 219 1 0 65 200 69 24 198 39 194 97 226 21 12 195 247 94 178 25 34 13 98 152 54 198 92 212 88 51 10 28 113 193 13 110 57 112 65 168 252 46 56 227 133 80 254 239 235 96 5 227 97 224 89 230 36 152 0 56 124 19 138 16 64 56 174 101 113 199 248 112 134 99 251 161 191 163 254 64 34 113 112 76 137 136 66 10 64 159 137 100 163 241 225 12 199 246 12 253 84 16 0 234 12 30 163 109 105 137 80 204 129 14 66 155 234 116 197 215 224 119 115 249 24 120 95 233 58 10 85 192 222 4 63 150 48 135 244 1 101 228 225 200 207 78 13 130 164 116 115 157 159 242 56 163 203 17 225 161 202 114 179 64 73 242 19 77 67 3 80 89 6 17 72 195 50 20 42 161 64 253 32 131 215 4 65 0 155 32 216 165 169 56 137 60 230 27 54 189 39 54 240 9 30 155 246 143 240 6 132 35 229 8 196 197 128 92 119 65 105 29 131 200 166 138 251 2 99 160 121 8 247 225 112 52 195 1 88 8 208 44 147 175 97 55 88 13 83 131 72 6 255 0 240 194 89 143 160 157 6 218 62 115 96 128 109 76 221 14 5 45 35 161 88 9 6 199 118 61 50 87 35 33 221 40 216 110 13 48 165 0 148 1 216 104 34 1 111 2 60 153 137 113 212 245 170 239 192 253 79 228 184 145 33 12 100 90 96 161 190 26 161 74 38 19 224 31 30 16 254 128 247 252 63 75 151 0 99 151 173 153 169 0 131 1 160 19 187 85 94 241 32 69 71 154 37 238 51 215 123 17 235 122 121 154 239 136 20 86 155 134 223 77 113 130 235 4 16 89 23 218 145 16 242 8 9 65 80 111 188 65 238 77 245 31 99 180 85 59 89 176 221 36 102 218 15 74 134 46 162 24 38 193 35 175 3 35 96 68 42 217 2 13 177 22 1 14 131 14 105 112 242 188 196 202 113 174 160 48 90 152 134 183 195 61 171 94 208 35 12 80 0 125 144 70 169 18 4 17 51 232 138 237 137 119 72 145 58 2 116 228 22 181 201 218 24 207 130 201 36 205 236 29 59 161 13 133 0 40 50 47 160 161 141 124 224 24 67 250 79 53 127 37 48 20 44 16 128 4 21 129 6 101 192 23 4 129 226 127 3 8 127 65 40 77 182 23 56 88 33 193 209 15 245 38 62 146 156 21 121 69 147 86 172 195 213 129 176 41 202 9 139 198 168 107 185 189 113 193 169 5 47 100 41 56 11 66 144 2 58 27 133 26 173 225 18 165 163 189 138 59 4 51 92 212 51 38 0 104 45 118 8 2 152 184 114 71 158 208 49 56 32 118 70 51 78 116 7 61 118 97 4 10 193 212 7 174 33 186 145 168 50 182 8 243 247 66 162 39 100 47 89 151 210 128 57 75 96 141 181 28 163 206 35 12 5 144 91 163 97 121 229 193 248 130 170 214 168 22 25 9 150 23 154 134 152 176 181 242 100 0 3 202 185 169 37 17 59 16 15 57 83 15 109 16 37 166 81 7 168 58 109 86 118 174 175 146 31 82 33 166 87 65 41 144 133 14 131 49 143 17 155 43 145 156 82 88 210 132 2 189 37 24 29 129 20 180 209 192 150 193 44 153 2 111 97 36 144 3 244 91 186 146 2 136 205 224 128 107 28 48 241 21 199 201 248 68 194 127 14 16 204 127 67 69 79 88 244 178 75 92 207 136 235 2 69 106 142 10 170 164 0 208 15 123 39 252 172 248 63 8 152 79 225 194 31 221 196 8 192 188 117 84 63 0 227 252 96 19 193 143 238 223 141 203 120 199 248 31 12 127 240 100 252 36 194 127 30 31 248 48 87 23 28 39 242 97 193 163 70 141 26 52 104 209 163 199 143 26 60 104 241 227 70 141 30 60 104 209 163 78 68 91 35 71 141 30 60 120 241 227 199 143 27 133 200 156 137 200 139 100 91 39 35 219 230 45 158 223 49 108 246 249 139 103 183 204 91 61 190 98 217 237 243 22 207 111 153 209 237 243 47 111 111 153 209 237 243 58 158 159 51 169 233 243 62 149 243 62 149 243 58 158 159 51 169 233 243 58 158 159 51 169 233 243 58 158 159 51 233 95 51 160 250 124 206 137 200 141 180 228 78 68 120 241 163 70 218 35 17 218 35 180 109 163 109 3 252 39 31 141 199 196 248 156 112 152 12 102 19 196 136 7 229 102 62 3 139 142 19 1 135 240 28 124 0 226 162 227 255 196 0 43 17 0 2 1 2 4 6 2 2 3 1 1 0 0 0 0 0 1 17 0 33 49 16 65 81 97 32 113 129 161 209 240 145 177 80 193 48 225 241 64 96 255 218 0 8 1 2 1 1 63 16 224 0 193 139 193 46 35 4 180 164 113 193 31 11 224 99 46 112 164 206 85 172 69 101 34 143 7 45 89 124 28 88 62 11 96 97 16 109 130 166 57 194 117 132 40 196 74 40 105 14 184 25 188 116 138 83 11 112 110 226 221 197 187 139 119 22 234 45 236 91 200 183 113 111 226 58 184 183 83 111 34 221 197 191 139 127 22 238 45 252 91 248 183 241 110 226 221 197 191 139 115 6 230 13 220 91 248 55 240 110 96 220 193 185 131 115 6 230 13 252 27 184 183 113 110 96 223 193 184 139 115 1 54 120 55 60 32 155 109 167 115 0 224 36 154 72 220 193 185 131 115 6 255 0 0 78 231 18 77 59 184 55 48 110 96 2 207 6 254 13 204 27 216 183 48 111 97 220 193 191 131 115 6 230 13 204 27 152 55 56 138 105 56 219 105 160 134 72 62 126 103 225 82 16 140 2 80 217 80 132 133 84 146 7 201 128 77 46 32 72 60 128 63 52 33 80 32 192 4 34 67 68 135 116 78 176 63 194 213 171 86 173 111 3 118 199 40 150 104 91 148 233 53 191 145 170 149 86 237 27 183 106 65 179 102 228 89 192 66 32 129 32 131 98 32 200 205 129 85 200 60 200 0 179 85 0 76 28 64 129 33 80 65 12 16 69 199 230 245 28 28 113 198 96 92 119 9 156 63 60 151 163 3 116 32 40 44 1 56 32 60 192 38 19 61 143 54 25 44 201 68 3 173 0 3 144 1 1 144 167 231 132 162 24 160 66 232 195 4 240 14 34 32 48 101 164 189 38 102 49 138 98 178 4 188 149 161 166 122 189 88 79 67 95 206 81 73 225 155 192 70 136 5 137 74 86 142 68 80 49 38 84 44 61 40 33 188 17 92 12 33 130 34 83 12 113 162 122 29 88 69 242 240 80 64 75 20 189 109 206 42 60 191 42 49 4 142 6 8 75 68 45 49 182 98 17 145 177 49 16 160 130 138 240 47 128 129 136 146 96 6 32 16 133 144 61 29 120 69 242 99 114 193 3 80 90 96 2 77 107 9 0 16 3 69 75 66 23 22 82 35 74 200 235 88 118 64 99 174 32 6 104 37 95 15 238 9 155 224 18 4 233 89 204 248 254 225 243 24 15 165 60 194 23 8 130 218 61 97 205 137 136 236 31 213 34 176 18 101 106 247 220 162 164 228 138 110 56 82 234 132 180 160 198 38 128 106 192 32 192 149 8 13 80 10 156 171 5 131 86 24 71 248 140 40 145 133 2 16 25 134 47 129 179 31 162 46 184 22 48 72 133 221 43 66 72 140 170 83 89 96 113 208 19 213 213 132 83 115 129 53 12 66 50 42 192 48 242 20 187 183 196 74 174 227 77 156 179 212 236 161 17 1 181 191 49 210 83 208 131 231 72 209 141 128 23 37 115 182 187 74 27 51 15 244 130 179 206 38 162 191 51 64 69 175 35 90 30 146 136 0 66 17 132 14 56 206 65 25 136 208 57 116 50 144 20 13 186 1 2 71 201 93 105 212 69 133 96 17 214 19 10 200 170 212 64 147 5 34 80 60 204 135 229 101 8 220 230 192 26 165 8 189 99 169 8 181 27 31 240 196 0 26 191 95 184 130 167 188 210 23 96 204 135 214 8 230 12 36 8 168 72 204 232 37 82 153 64 2 77 58 233 46 0 86 69 7 96 119 217 197 240 13 14 246 38 206 208 10 2 21 119 135 99 41 59 214 204 54 31 41 164 135 125 62 98 4 131 152 10 44 219 249 126 161 20 242 16 108 246 134 38 33 136 2 134 254 135 206 10 50 122 1 83 75 139 150 251 140 116 57 3 113 89 89 32 131 170 167 62 90 206 133 23 97 157 117 210 38 159 238 173 102 251 111 104 181 8 31 177 129 133 26 45 204 5 242 171 147 142 75 222 156 139 237 11 118 214 255 0 68 166 22 170 175 157 24 90 217 74 238 218 30 150 131 0 234 173 179 46 244 129 57 229 14 140 212 10 206 199 180 79 58 159 177 14 4 75 156 183 44 27 13 115 52 27 184 70 64 118 238 108 33 93 218 25 124 19 241 158 80 224 1 144 247 124 160 217 0 200 110 57 20 104 97 204 196 108 32 224 108 187 218 141 137 67 88 28 53 129 13 105 126 10 5 129 4 48 153 48 96 132 129 24 0 201 194 97 142 28 33 33 25 92 12 7 209 213 132 88 143 81 23 54 132 100 14 195 239 205 216 115 48 20 8 14 66 4 101 106 95 152 150 140 139 72 152 135 163 80 126 99 54 28 82 174 64 7 88 48 30 95 72 51 145 96 56 30 107 58 53 90 47 52 163 230 80 88 1 84 220 173 50 229 3 216 202 34 163 120 8 80 170 194 13 107 250 24 218 232 40 245 205 222 4 58 194 228 78 125 62 208 66 161 83 200 255 0 65 235 5 112 19 218 105 11 96 208 200 101 89 251 132 140 58 5 53 228 125 88 122 161 30 134 83 121 1 2 96 170 179 96 89 42 221 90 61 32 142 169 253 67 108 175 214 16 177 153 158 76 127 112 148 141 156 106 161 90 245 191 42 195 172 16 10 191 36 12 2 70 119 1 187 40 253 71 95 66 134 167 101 218 19 109 30 192 175 235 164 118 3 208 0 127 160 141 8 221 89 71 185 152 60 160 106 192 80 78 110 196 63 154 32 9 90 161 151 57 190 181 154 148 192 205 186 130 226 5 42 69 139 222 193 32 4 237 58 72 23 35 127 116 222 104 199 130 250 22 0 221 17 113 79 115 246 32 9 17 22 138 50 171 44 159 201 80 106 32 57 11 232 88 3 201 23 2 30 101 254 165 22 144 100 174 187 88 230 112 167 131 42 47 152 70 225 184 208 165 66 63 36 202 232 103 43 28 128 173 205 66 247 34 34 241 36 26 85 13 244 229 120 95 184 19 210 242 200 177 9 96 59 176 205 185 85 0 55 116 17 128 18 162 85 132 12 8 32 77 12 210 65 192 137 134 67 82 142 195 0 17 47 24 139 225 156 40 20 194 126 189 88 84 77 214 5 5 0 220 143 162 37 8 132 173 197 114 251 38 86 31 152 128 121 128 87 181 135 161 16 38 207 43 63 164 34 166 25 218 10 213 202 73 41 201 146 190 224 131 43 42 70 249 25 154 161 220 143 165 30 240 0 0 205 129 10 183 109 224 76 21 133 158 128 6 102 240 123 145 145 184 0 106 63 70 17 8 106 100 187 186 22 199 69 19 53 104 50 86 119 36 152 197 67 18 161 91 144 62 254 162 145 218 253 234 119 48 14 41 198 228 125 17 1 80 88 150 210 214 133 243 53 235 0 6 134 199 234 26 160 37 154 147 84 3 169 58 9 156 163 152 145 44 215 59 18 175 17 173 190 15 200 134 221 169 221 159 83 232 214 208 160 101 138 159 182 242 206 26 66 244 215 46 81 73 50 175 246 225 70 227 154 124 63 234 82 93 148 20 8 90 210 140 83 165 238 89 187 204 203 183 212 251 148 249 149 10 5 209 64 33 74 169 83 123 221 195 173 100 25 35 185 61 163 146 1 144 170 73 190 140 211 157 247 133 239 5 166 77 109 152 233 11 82 199 50 62 166 72 1 1 160 17 156 108 85 35 232 137 104 46 175 228 86 10 207 73 36 167 39 110 119 129 20 235 84 143 162 37 186 190 15 200 172 88 43 98 73 78 78 157 96 133 58 213 35 232 136 32 43 15 49 244 160 130 174 70 182 222 239 118 225 110 76 201 43 228 145 42 11 152 11 167 197 185 192 145 4 15 51 76 129 100 15 84 32 98 8 2 55 205 117 204 66 16 2 132 89 38 155 23 78 136 193 84 28 153 55 189 201 48 12 72 32 139 131 178 212 148 0 128 11 18 16 228 25 209 41 5 21 166 92 17 10 133 224 157 136 82 109 25 8 203 8 50 184 228 239 21 142 98 38 220 67 174 161 138 184 65 0 99 237 185 191 138 110 172 35 79 130 223 10 51 248 115 172 74 106 24 103 18 26 219 169 69 233 106 132 217 11 140 89 84 67 118 33 7 44 2 229 178 165 244 194 73 146 168 76 68 94 0 81 3 152 66 64 181 88 121 173 24 96 62 15 162 230 194 15 205 110 1 107 244 255 0 121 121 227 54 148 224 146 107 88 117 217 203 81 129 142 36 199 41 140 45 24 9 81 108 37 199 136 137 24 129 131 104 8 1 43 114 185 58 105 148 8 184 252 201 56 34 23 161 65 32 8 64 218 86 21 73 88 117 36 72 99 18 97 48 49 129 59 157 115 196 32 129 141 32 34 85 10 160 101 50 33 33 152 36 26 24 120 119 159 155 179 190 39 20 67 18 234 90 102 13 149 162 19 113 16 38 32 199 6 136 203 131 132 32 87 3 25 132 215 117 40 252 212 206 176 8 193 210 118 130 13 248 108 128 198 114 184 26 199 98 44 24 98 195 7 134 2 131 23 81 130 93 212 163 243 38 90 4 134 22 85 136 140 144 169 109 25 28 163 2 138 6 56 23 65 124 80 226 12 185 130 93 212 175 241 235 96 162 227 53 180 2 166 179 4 23 190 218 5 112 81 112 221 0 156 112 12 69 216 49 81 83 6 72 86 15 96 131 8 188 138 3 130 183 17 151 48 75 191 255 0 191 133 138 172 186 140 97 11 14 96 156 89 96 115 42 114 220 225 16 109 42 57 193 3 1 55 53 53 34 18 173 165 80 153 142 84 138 33 71 114 106 150 129 20 64 117 142 158 80 28 80 67 113 5 89 49 180 186 0 231 1 25 2 54 174 6 84 119 86 173 123 87 160 154 87 17 217 113 90 131 166 93 225 73 0 170 228 173 96 107 36 192 210 214 30 240 1 41 20 168 29 148 107 174 93 76 148 146 172 33 86 11 59 136 42 131 77 114 134 16 119 49 10 47 132 29 99 142 56 240 177 62 26 29 227 85 128 100 54 98 135 14 9 136 87 128 101 204 18 238 191 239 175 103 52 191 7 230 253 136 21 160 98 181 19 60 198 1 60 75 42 205 132 128 150 201 215 61 161 79 228 99 173 95 83 215 207 10 35 118 186 81 88 65 103 90 6 22 153 229 13 135 253 32 70 37 32 200 12 182 105 202 147 35 20 116 5 241 17 23 52 100 121 177 107 66 86 61 69 192 184 17 224 139 176 119 243 74 247 66 236 194 175 86 204 208 85 159 120 249 17 144 228 72 136 6 141 8 62 69 243 202 55 131 92 130 189 89 98 186 7 250 12 73 33 187 90 95 172 10 82 0 220 129 246 84 121 91 120 19 81 208 105 96 160 224 82 13 78 199 185 191 73 191 62 132 58 192 235 194 25 39 16 209 114 192 132 6 21 80 243 56 139 135 29 134 152 4 96 101 220 16 238 191 239 165 0 164 216 206 133 96 74 192 57 204 161 243 57 221 0 53 20 179 127 228 17 35 144 168 8 209 101 160 237 202 22 157 32 195 26 234 215 120 99 73 170 134 18 88 200 117 6 183 174 145 137 6 232 25 80 163 234 89 223 103 1 106 44 69 65 205 161 50 184 47 41 14 202 34 201 48 13 237 86 213 31 210 20 48 36 201 208 140 137 89 94 3 193 50 84 27 61 9 131 250 160 54 51 35 38 251 65 162 232 100 141 94 160 223 35 9 2 0 64 132 8 104 197 6 150 133 84 164 2 239 71 81 84 8 210 174 20 64 101 220 118 0 146 74 179 166 241 107 110 2 160 80 130 229 109 23 204 0 90 132 128 206 212 61 12 233 183 110 155 177 114 19 73 107 32 208 44 212 235 4 12 0 36 174 108 118 189 91 191 51 14 240 156 140 117 2 10 201 229 115 7 82 20 4 2 98 154 128 0 51 151 128 14 81 136 96 218 151 4 138 222 29 100 46 23 5 81 160 218 24 172 64 16 8 56 128 97 108 19 71 236 24 239 39 63 247 91 166 236 30 44 171 13 218 110 207 105 68 77 200 73 169 128 121 203 153 132 0 105 249 253 136 12 142 172 147 153 63 224 18 132 12 185 24 86 156 4 32 101 148 194 92 227 37 134 20 19 149 28 39 206 4 39 9 203 132 205 233 66 242 231 133 11 133 199 8 4 23 49 0 64 137 20 66 33 192 176 82 30 3 93 212 171 240 13 196 12 111 248 135 36 179 62 193 62 68 179 113 160 113 168 209 163 70 141 26 175 0 135 16 131 73 0 86 155 6 24 4 6 8 97 161 0 128 160 157 128 104 161 169 4 1 150 74 94 63 131 231 223 191 126 125 251 250 132 132 6 128 109 85 147 77 151 148 76 238 191 53 72 76 64 2 105 184 155 137 184 155 137 184 155 136 179 168 85 216 190 65 213 223 48 111 0 56 0 133 10 207 3 115 153 228 104 171 96 243 132 16 57 184 155 201 184 155 137 184 155 137 184 155 136 93 199 117 255 0 130 165 160 80 32 130 136 34 160 130 42 8 53 4 64 241 2 131 220 173 106 145 152 170 196 26 141 224 34 3 52 16 225 115 201 228 236 77 149 155 112 0 18 194 184 255 0 23 113 255 0 131 160 85 5 24 64 53 193 95 33 176 0 145 74 13 86 157 33 159 38 93 43 81 165 209 132 247 3 253 115 211 231 70 126 155 144 31 176 96 128 201 29 62 148 126 88 216 95 199 221 97 125 161 61 225 61 1 54 159 9 177 248 65 165 248 97 143 81 226 123 79 19 214 120 158 251 196 245 94 63 140 65 36 146 75 36 16 65 61 87 137 238 188 79 117 226 122 175 19 213 120 158 171 196 245 94 39 170 241 61 215 137 239 188 79 125 226 122 175 19 213 120 158 171 199 9 150 24 71 251 108 68 73 238 188 79 85 226 122 175 19 223 120 158 251 196 247 222 39 172 241 61 199 140 1 234 60 79 113 226 123 143 19 220 120 158 179 196 247 222 39 190 241 61 247 137 234 188 97 167 251 72 255 0 105 31 232 35 253 4 19 160 249 33 247 62 104 118 119 100 218 251 227 103 230 157 159 154 118 126 105 217 198 113 199 106 33 254 247 15 92 96 130 190 197 238 20 75 129 145 210 46 53 23 10 136 69 20 81 96 162 138 44 4 80 28 1 132 199 6 38 62 53 20 48 112 136 162 193 66 56 12 88 56 76 92 2 166 36 81 65 8 198 168 109 20 34 60 68 255 196 0 43 16 1 0 2 1 2 4 6 2 3 1 1 1 0 0 0 0 1 0 17 33 49 65 81 97 113 240 16 32 129 161 193 241 48 80 64 145 177 209 96 225 255 218 0 8 1 1 0 1 63 16 252 162 34 34 34 34 34 55 198 240 247 236 124 68 126 246 125 143 224 241 17 27 31 177 240 31 189 243 93 204 207 222 254 25 157 249 177 30 179 229 153 185 251 223 11 251 159 199 114 34 34 35 183 246 62 107 153 187 233 51 123 193 31 185 242 200 140 253 207 146 71 241 73 131 75 28 127 228 154 52 104 129 163 64 8 16 48 64 209 130 133 74 149 106 212 131 86 237 219 181 102 205 171 86 173 90 181 32 81 187 82 173 90 181 42 85 171 86 172 218 181 106 213 171 86 173 74 181 42 85 171 82 141 90 181 106 85 171 86 165 90 181 40 84 163 87 133 15 180 121 250 96 32 0 32 52 32 253 42 67 53 175 19 27 87 66 78 188 41 202 169 209 164 65 192 17 47 247 76 40 134 171 86 165 235 199 175 152 217 36 146 78 225 248 157 195 241 3 132 44 174 191 70 86 54 141 101 104 164 50 200 211 195 64 150 105 135 184 126 60 100 147 176 126 60 36 238 31 137 220 63 30 79 100 144 250 44 53 129 173 120 105 128 48 118 40 61 64 161 201 105 74 11 43 69 22 5 180 26 245 223 93 255 0 117 161 234 254 155 227 204 248 196 38 238 192 37 59 43 19 240 131 0 207 185 0 107 198 17 243 89 148 2 55 232 33 112 61 0 98 32 88 241 20 12 224 26 163 196 19 62 203 226 90 137 60 89 111 162 152 216 48 98 31 215 238 104 218 196 187 173 64 158 166 33 248 128 12 208 224 16 133 236 131 4 15 55 33 49 218 160 129 157 157 143 0 192 2 231 149 0 28 115 56 215 92 67 10 103 184 2 191 232 65 56 30 1 255 0 33 0 122 171 201 17 35 62 203 227 193 13 116 253 203 22 225 65 215 75 249 224 13 3 132 108 204 93 204 12 147 253 16 44 91 8 112 224 33 83 253 33 155 171 140 73 211 160 0 5 231 171 16 197 228 2 132 69 249 131 4 242 49 0 1 236 33 68 157 24 155 128 7 144 12 0 108 98 195 253 151 199 135 23 94 52 89 227 151 70 189 147 36 180 208 168 60 17 201 251 95 98 139 252 20 5 6 245 161 24 29 89 84 96 198 65 146 28 125 22 160 5 63 60 64 34 32 99 200 7 222 104 49 58 16 212 37 163 26 118 76 97 226 124 24 232 193 106 213 12 44 210 122 188 209 9 240 130 6 174 130 5 250 198 239 236 190 60 32 52 240 1 148 16 116 99 156 176 42 238 140 130 197 239 211 72 159 44 84 40 181 220 82 194 33 163 230 223 82 207 69 18 245 153 102 24 115 87 17 15 201 78 13 211 61 227 197 19 62 181 11 29 96 247 1 123 165 153 16 193 201 163 152 70 182 242 37 123 36 105 106 137 103 36 230 226 245 158 45 81 76 134 206 24 29 18 252 171 69 203 54 65 160 100 145 85 141 103 72 179 133 107 129 179 252 19 77 246 109 10 199 250 32 35 106 107 104 37 91 134 157 56 146 180 146 234 234 52 184 144 27 68 99 252 135 202 45 142 3 188 120 97 150 108 56 239 116 248 219 16 35 141 168 188 30 17 49 65 200 210 51 145 121 136 209 127 245 21 77 86 204 222 228 178 176 176 216 69 224 240 149 229 200 252 26 58 190 108 33 16 9 16 110 198 44 109 156 1 26 232 9 133 147 179 30 64 77 163 88 133 27 35 4 32 61 187 132 8 217 249 4 10 99 132 219 94 134 96 141 220 60 61 243 64 69 69 16 200 33 63 0 41 160 11 55 48 191 24 118 143 81 28 107 39 115 91 3 57 251 47 137 216 184 166 199 133 20 18 10 43 224 131 5 83 59 71 61 244 197 104 227 67 139 27 11 128 134 195 207 105 247 134 218 209 236 180 224 65 177 230 194 234 220 205 184 204 114 181 226 235 80 177 104 7 40 11 210 90 75 44 29 169 244 76 170 218 113 164 233 208 96 187 86 120 58 75 180 58 67 56 163 78 23 193 206 73 130 137 105 70 24 226 156 33 14 178 73 185 5 150 79 20 47 239 113 141 76 182 95 86 90 116 19 184 21 48 135 246 24 132 221 236 249 107 78 77 92 209 198 176 72 178 216 106 175 158 98 117 115 219 81 136 221 212 26 28 217 209 86 86 56 198 34 218 13 54 113 151 77 219 44 229 108 6 172 154 174 211 104 21 42 212 212 111 57 37 191 11 148 239 159 85 29 23 182 78 91 75 67 87 87 240 17 9 133 243 151 72 44 72 171 96 84 55 224 102 93 241 80 194 40 49 104 208 108 170 184 161 252 1 5 26 2 144 29 152 55 75 20 53 186 211 130 83 134 218 4 107 45 109 107 44 213 83 64 16 7 165 74 87 107 99 87 189 70 227 126 164 51 151 155 90 241 112 217 177 87 190 31 82 138 27 54 115 143 160 223 218 175 6 160 49 131 164 207 220 54 209 128 88 104 23 86 162 124 209 180 40 20 50 57 47 13 147 85 134 255 0 181 31 67 46 131 93 73 138 94 161 157 210 173 90 250 199 84 176 139 104 151 222 182 221 142 9 78 34 66 179 74 108 236 244 139 69 237 172 4 212 249 116 67 9 151 14 123 74 181 75 98 238 144 152 164 71 20 82 200 56 65 107 134 181 194 22 191 12 88 91 86 87 130 104 233 69 246 7 201 174 217 164 204 32 150 2 186 87 5 140 205 102 175 106 58 118 173 66 142 48 221 105 216 83 214 111 137 4 112 117 186 52 97 204 110 171 89 98 13 67 104 55 104 122 71 133 85 50 234 28 1 98 150 13 88 92 189 208 107 214 115 56 80 106 210 11 30 192 139 80 246 56 208 234 100 178 194 96 18 90 35 200 173 205 2 73 61 162 111 37 144 20 198 102 111 143 9 160 165 181 131 58 100 227 143 128 163 7 55 121 241 64 45 60 68 13 0 225 130 68 159 128 49 179 183 64 5 21 144 38 17 25 151 36 1 227 87 195 59 30 181 79 193 16 16 152 64 29 152 128 20 245 16 183 144 126 204 67 239 224 202 85 0 211 1 182 246 95 30 76 115 229 1 199 75 192 235 28 17 248 249 38 65 183 253 52 20 229 165 106 217 139 122 68 23 185 203 132 10 106 90 109 188 111 193 87 84 69 204 185 160 72 184 25 27 183 35 116 13 88 72 234 12 88 26 218 43 154 227 48 20 46 242 21 41 56 69 231 155 57 43 241 16 133 162 163 213 19 175 38 132 225 247 5 171 3 95 234 67 203 163 27 1 106 173 80 148 112 93 158 181 132 120 42 189 163 87 201 12 95 86 7 108 197 152 34 107 59 98 43 185 138 15 123 244 32 194 234 93 88 255 0 210 206 164 101 111 146 113 97 242 71 3 91 156 66 239 110 224 136 27 65 86 67 3 147 195 194 229 180 184 132 136 169 64 190 1 9 84 140 135 33 204 162 91 135 52 131 91 124 90 92 199 230 105 96 38 216 27 228 44 78 25 209 26 2 219 61 85 203 13 29 182 137 29 97 113 218 229 229 212 102 0 128 61 150 44 57 176 32 186 16 211 183 62 13 197 107 136 61 48 246 211 68 232 224 250 203 53 94 3 200 197 139 166 138 119 52 136 156 88 92 194 14 48 89 120 108 134 134 36 46 251 217 58 40 219 160 149 196 168 137 77 240 200 37 57 5 67 39 204 185 87 167 123 74 97 166 77 73 84 172 87 156 231 81 217 144 240 27 11 240 136 156 243 49 62 239 21 253 180 245 143 87 43 252 112 62 2 216 37 70 195 124 66 240 19 146 22 243 33 124 148 150 80 66 133 52 14 50 45 206 160 224 243 74 99 202 117 70 182 208 45 64 49 32 135 50 148 176 48 153 22 104 91 43 134 203 240 167 215 44 98 224 2 194 148 106 59 7 136 70 86 240 42 150 168 12 96 154 179 129 91 229 201 97 121 129 104 174 82 75 70 68 90 18 84 46 176 60 35 5 103 54 228 199 80 180 155 84 181 118 56 242 66 152 202 216 107 21 229 132 171 117 106 50 51 92 166 23 46 239 213 164 194 198 66 238 179 6 149 11 14 167 138 56 5 40 218 121 196 249 64 8 193 1 208 7 196 22 136 127 8 46 45 239 24 165 128 10 60 69 4 36 184 67 221 160 136 23 182 232 32 199 146 105 7 139 212 196 57 246 16 171 176 114 4 36 192 167 138 32 225 9 195 16 108 215 184 48 255 0 182 35 23 85 196 124 5 125 78 94 17 130 73 159 101 241 59 23 31 142 64 46 217 116 248 116 220 85 129 248 68 33 0 19 129 32 236 5 139 149 231 234 244 16 85 131 56 218 160 205 84 170 89 206 86 45 183 188 70 112 88 80 186 166 209 176 142 18 199 12 183 172 197 133 35 131 217 116 234 205 75 239 184 169 91 242 211 42 196 182 11 195 88 37 55 16 218 35 123 59 92 214 35 87 24 109 45 61 113 136 0 168 234 160 53 139 180 32 13 69 96 17 26 244 33 8 189 156 186 59 54 86 5 226 160 28 4 21 164 180 246 134 153 106 230 100 159 94 135 200 25 17 150 53 252 8 13 179 48 227 105 131 166 4 189 65 164 74 180 6 177 195 88 250 207 0 45 218 102 81 96 203 14 92 173 230 137 220 99 10 34 142 40 109 115 23 2 12 105 236 166 198 147 120 154 130 118 43 104 218 69 165 231 88 219 210 69 193 149 125 161 166 242 76 193 186 220 24 174 154 17 198 174 49 78 39 50 213 108 88 195 159 24 244 191 164 193 202 107 0 75 13 78 6 11 107 202 239 132 165 153 148 216 64 192 26 32 169 181 168 174 163 110 110 121 195 51 155 90 82 50 192 181 184 218 108 88 181 128 42 194 25 222 243 56 72 10 41 237 181 190 190 177 28 230 58 218 47 41 189 224 145 153 35 0 64 36 141 166 65 171 230 208 154 172 187 181 120 176 90 76 112 25 192 54 13 178 220 153 59 51 177 171 81 173 81 121 155 168 83 85 208 147 14 91 98 216 234 226 187 77 134 183 144 121 0 106 208 154 169 55 93 46 144 172 54 142 220 230 45 72 197 186 116 69 53 86 170 225 112 60 46 144 224 107 26 47 74 235 93 33 8 99 170 250 158 70 21 71 114 99 44 41 166 7 129 225 105 42 88 148 84 40 178 201 173 92 76 216 197 45 66 75 156 255 0 66 107 3 77 155 10 176 119 46 157 201 74 34 1 139 170 172 148 219 94 24 141 148 193 164 0 58 251 46 46 63 44 149 73 224 212 105 72 61 32 32 103 108 120 119 64 228 67 29 124 172 149 226 160 11 171 196 215 127 242 107 89 88 116 35 71 248 194 0 27 129 31 8 24 153 113 113 0 60 93 132 3 141 182 225 196 48 3 21 116 5 92 65 15 46 90 48 102 155 96 15 198 216 0 203 202 24 165 233 145 59 0 130 122 63 100 112 63 212 164 14 62 140 35 16 137 118 112 115 36 178 217 189 49 141 145 147 198 161 12 88 16 128 59 4 65 75 192 45 196 240 9 137 26 172 114 64 199 103 178 248 157 139 142 110 249 219 9 172 184 16 208 209 228 92 64 131 163 0 140 28 63 78 44 144 168 156 5 101 80 81 67 135 141 113 181 122 140 47 5 176 120 164 17 67 139 168 16 56 181 158 132 252 177 192 239 18 6 142 119 99 192 69 215 34 128 141 135 154 138 136 35 216 135 87 220 136 44 236 24 88 91 176 192 121 24 0 177 86 65 48 67 138 255 0 134 0 29 6 235 42 25 229 11 62 32 48 105 161 159 149 130 49 253 130 45 112 137 228 66 3 104 62 37 114 29 125 37 65 111 236 190 39 96 227 253 209 241 72 186 206 219 91 187 250 126 112 66 147 18 155 76 112 248 128 9 190 0 61 127 72 67 32 239 203 28 195 59 76 160 0 110 3 232 132 16 34 125 195 128 51 107 190 80 17 94 70 132 193 63 49 68 71 191 160 133 246 193 25 223 129 228 28 99 182 122 189 159 125 92 34 176 43 224 195 88 201 160 42 169 151 3 206 119 46 57 127 185 6 131 110 157 29 45 124 158 112 0 2 218 128 102 115 2 20 34 199 6 52 78 241 198 33 5 243 92 198 122 195 240 132 124 0 134 170 39 111 90 16 225 213 233 16 96 32 226 35 123 119 36 96 60 129 248 194 70 225 39 122 68 121 176 1 204 64 251 20 217 96 182 12 184 7 147 20 3 110 113 255 0 103 115 226 155 191 185 236 122 126 72 132 206 207 100 133 202 168 85 102 0 0 29 155 38 0 138 51 242 66 17 26 132 56 1 235 229 2 75 254 16 144 174 120 60 71 118 35 196 127 243 1 29 45 1 102 32 62 229 224 5 144 58 138 50 115 236 190 60 52 55 253 207 251 127 136 248 84 109 99 196 13 144 44 129 11 51 226 0 254 40 235 230 35 126 212 152 40 68 207 98 58 99 99 103 224 136 63 127 102 24 240 23 192 197 62 228 143 49 184 0 190 160 44 32 28 138 124 70 12 167 217 124 79 112 123 161 251 154 66 104 214 237 28 250 68 63 128 44 33 104 151 18 100 131 20 31 85 138 8 32 116 240 2 48 88 164 88 204 161 192 65 58 67 66 32 139 64 71 219 150 215 240 6 7 39 217 124 78 197 197 18 255 0 94 112 148 145 66 217 131 71 169 40 99 1 58 208 66 222 167 76 214 73 177 150 92 178 115 9 103 155 119 244 252 160 15 145 123 182 235 32 69 206 27 84 129 26 42 43 1 33 119 101 34 232 1 198 62 26 232 56 233 154 172 120 0 182 60 33 245 28 35 186 199 21 126 112 226 47 39 217 124 78 205 199 252 252 189 195 14 237 127 72 125 96 76 110 93 53 108 114 224 141 219 41 163 111 75 163 173 48 55 77 104 171 28 162 214 230 148 72 166 68 76 211 176 140 34 175 85 66 160 52 108 71 4 71 50 230 44 208 139 65 170 48 68 225 110 211 52 29 205 175 148 9 94 93 25 233 70 101 144 69 160 64 240 86 73 95 34 84 172 53 69 102 174 80 102 148 99 78 38 23 46 49 218 227 58 212 52 229 149 215 153 66 229 226 225 36 231 139 89 5 130 247 49 184 75 160 90 164 165 169 97 74 177 103 43 83 9 26 115 160 40 27 109 124 230 72 159 122 75 123 65 145 179 3 166 88 14 2 207 251 195 91 64 12 151 50 44 175 251 16 206 204 56 184 193 40 32 61 11 178 233 150 115 26 18 22 180 14 42 177 235 8 162 57 79 247 142 254 158 58 191 138 8 31 0 16 11 5 79 99 14 86 48 4 65 6 58 66 42 49 192 19 4 48 43 41 192 11 150 34 3 207 160 1 222 41 177 31 27 73 80 137 145 82 125 151 196 236 220 127 207 247 17 251 28 115 159 136 222 134 30 239 254 99 232 38 235 228 68 4 156 201 243 94 204 117 245 101 93 20 2 106 101 166 109 243 42 33 139 0 125 56 69 77 38 207 60 251 164 148 203 69 68 241 252 209 46 27 93 1 173 179 153 175 121 163 169 61 25 134 162 90 168 80 0 151 13 140 235 102 136 61 184 22 108 1 174 156 37 2 227 7 16 177 177 202 151 138 106 5 73 9 15 71 131 112 98 246 52 137 169 62 41 126 121 132 228 128 115 129 74 226 237 145 132 220 208 22 19 48 178 243 169 99 14 38 130 25 64 122 24 167 242 243 9 103 22 225 126 201 159 68 9 75 180 47 68 26 18 178 58 236 177 240 37 22 115 221 43 248 23 41 42 190 124 106 21 138 32 112 91 66 174 217 97 58 95 245 45 208 214 82 227 1 105 102 246 91 112 108 83 46 148 32 224 17 134 219 176 46 68 120 120 29 98 12 7 225 15 8 120 215 1 193 8 129 145 248 4 36 12 112 125 180 200 24 177 125 1 80 217 248 193 136 119 17 180 131 80 0 85 152 7 117 128 10 255 0 58 62 37 239 78 239 73 217 184 191 158 195 76 232 227 77 110 68 16 179 1 172 22 171 32 6 219 1 205 28 151 53 203 121 65 96 90 86 144 244 154 37 44 80 164 25 104 54 163 91 106 0 20 89 54 246 142 222 50 248 120 213 22 50 113 153 100 194 227 155 227 238 127 8 10 102 139 105 104 132 23 26 107 234 118 112 185 102 2 87 219 53 49 96 217 206 99 182 131 18 40 130 246 17 212 187 99 62 65 50 83 187 109 150 238 156 226 98 69 111 220 234 236 90 92 20 82 243 23 183 56 34 23 191 177 83 64 44 111 254 139 69 27 93 83 58 181 40 229 171 24 132 34 128 41 233 14 46 97 36 211 169 246 57 27 185 162 192 170 101 51 101 236 141 14 26 56 134 124 13 126 177 21 109 180 143 30 87 167 152 129 156 22 109 122 50 56 151 84 106 56 208 8 215 93 43 113 231 95 152 180 13 46 167 51 92 49 38 138 81 50 120 18 154 165 110 89 20 50 25 45 178 104 183 107 58 10 99 224 117 196 5 226 153 187 153 127 106 43 81 247 57 108 159 231 254 231 87 241 192 30 44 140 1 165 150 16 3 24 107 70 1 78 216 0 167 192 133 46 252 2 189 88 10 180 33 236 33 244 228 2 140 130 221 96 157 53 79 226 208 59 206 207 73 220 184 225 252 239 121 210 36 53 109 99 4 77 254 71 39 64 89 182 175 57 167 72 54 157 212 51 51 45 215 4 98 98 215 17 229 201 228 66 18 227 148 68 222 6 26 179 102 39 245 45 223 137 36 27 50 215 30 46 129 68 36 143 21 17 27 39 66 81 60 237 67 43 13 26 149 248 238 68 170 64 154 96 128 142 132 169 21 226 76 168 238 244 34 187 66 170 163 77 224 42 90 22 221 232 74 53 82 170 161 3 26 138 148 250 134 9 153 150 235 137 163 93 36 213 209 254 255 0 36 35 162 104 167 172 19 228 196 243 8 136 71 129 176 208 157 107 0 45 52 240 132 70 112 63 0 146 63 217 252 78 197 197 15 208 44 181 158 41 0 52 199 149 192 2 115 154 25 209 252 37 128 208 193 117 5 14 164 210 43 140 166 207 161 68 250 184 125 92 62 174 31 87 15 171 129 69 43 233 1 200 85 104 195 252 146 229 187 179 55 131 121 64 115 64 12 137 105 66 59 64 171 132 72 102 127 72 103 74 220 155 249 58 221 43 122 63 3 183 239 223 191 127 141 185 114 10 178 84 230 93 20 25 128 12 235 126 147 177 113 67 247 33 216 84 140 96 36 183 25 245 89 245 89 245 89 245 89 245 89 245 89 96 17 213 200 117 83 186 183 75 5 26 71 79 44 1 188 74 144 151 126 22 108 254 194 207 245 1 252 215 156 79 162 207 166 207 170 207 170 207 170 207 170 207 170 207 162 206 149 57 83 185 113 67 255 0 2 152 50 226 17 245 114 5 38 24 118 240 248 194 102 193 53 53 41 211 15 128 1 215 68 63 89 155 132 186 23 0 6 176 11 167 66 153 205 79 136 57 116 107 36 165 189 33 172 10 42 145 248 187 23 28 227 255 0 130 90 204 225 145 40 11 112 61 208 161 113 97 160 151 6 250 197 165 20 216 121 82 160 247 146 149 41 225 214 48 101 30 24 21 71 107 171 170 205 4 195 19 119 131 24 26 7 34 22 193 84 16 138 133 96 252 125 139 142 31 30 81 196 125 160 230 127 0 140 39 110 231 64 52 207 167 207 200 177 110 220 143 182 62 97 59 191 252 242 203 180 254 253 45 222 159 63 150 176 232 246 252 159 175 195 2 237 117 136 225 157 126 9 200 122 199 177 254 60 184 58 231 29 139 243 59 35 230 119 199 207 143 167 156 195 167 154 108 217 209 178 244 118 189 62 127 4 127 242 32 193 165 201 240 4 156 57 61 153 255 0 149 18 123 221 62 127 58 99 130 0 173 51 210 255 0 145 226 31 123 66 126 195 49 224 181 231 41 176 117 96 126 40 55 76 12 181 183 67 198 80 195 15 47 33 82 197 202 27 253 34 27 99 175 193 224 202 15 243 124 162 30 7 242 188 71 167 130 130 181 32 215 30 156 180 201 215 52 216 78 201 157 147 59 38 118 76 236 153 217 51 178 103 100 206 201 157 147 59 38 118 76 236 153 217 51 178 103 100 206 201 157 147 59 38 118 76 236 153 217 51 178 103 100 206 201 157 147 59 38 118 76 236 153 217 51 178 103 100 206 201 157 147 59 38 118 76 236 153 217 51 178 103 100 206 201 157 147 59 38 118 76 236 153 217 51 178 103 100 206 201 157 147 59 38 118 76 236 153 217 51 178 103 100 206 201 157 147 59 38 118 76 236 153 217 51 178 103 100 206 201 157 147 59 38 118 76 236 153 217 51 178 103 100 206 201 157 147 59 38 118 76 236 153 217 51 178 103 100 206 201 157 147 56 208 97 126 31 255 217 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:32'!
menuBgPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 183 0 0 0 60 8 3 0 0 1 100 192 235 32 0 0 0 90 80 76 84 69 255 0 0 204 204 204 236 236 236 237 237 237 238 238 238 239 239 239 240 248 225 244 249 233 246 251 238 240 240 240 241 241 241 242 242 242 243 243 243 244 244 244 245 245 245 246 246 246 247 247 247 248 251 241 250 252 244 248 248 248 249 249 249 250 250 250 251 251 251 251 253 248 252 253 249 252 252 252 253 253 253 254 254 252 254 254 254 255 255 255 245 142 141 1 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 4 31 73 68 65 84 120 156 213 90 123 115 219 32 12 23 143 44 175 110 187 181 223 255 3 102 255 236 174 235 123 115 60 144 16 8 140 219 114 13 187 162 139 9 8 89 254 69 128 144 112 172 134 61 84 200 30 86 248 59 216 85 249 85 105 199 175 179 79 43 252 23 123 166 138 62 103 252 107 171 67 77 103 252 163 213 245 7 219 249 80 231 223 84 225 95 10 191 178 103 68 78 240 211 143 112 56 9 57 193 215 137 191 129 186 161 87 224 28 110 191 125 109 16 223 220 192 75 131 184 126 91 70 138 111 27 132 127 174 153 178 78 7 176 207 13 226 198 99 111 176 35 88 227 238 105 16 223 66 203 143 117 218 155 196 55 160 90 196 1 90 108 179 54 181 215 196 91 116 183 77 130 211 234 122 170 146 106 19 135 54 113 231 31 112 177 106 56 47 220 77 104 250 94 94 200 30 204 159 247 107 215 141 203 3 231 140 35 231 157 212 172 230 192 115 21 95 199 182 175 195 204 245 192 142 146 220 34 190 23 20 124 198 178 241 133 129 201 151 19 62 110 226 54 119 24 110 76 92 227 166 235 37 86 188 129 235 30 249 221 213 158 96 69 188 9 121 68 145 19 179 8 108 40 231 216 229 12 238 127 142 253 226 188 127 235 60 120 47 145 195 195 159 35 103 64 233 189 125 3 242 25 194 130 105 166 36 70 152 54 110 186 56 131 157 217 88 201 128 209 132 194 196 102 202 218 113 44 140 24 33 28 19 26 137 94 6 33 178 13 91 76 187 242 78 122 79 61 149 43 120 222 118 84 238 62 189 148 31 208 209 53 120 174 102 178 79 56 69 105 110 134 153 90 155 190 176 152 229 165 67 241 42 162 176 57 251 118 223 121 142 203 191 109 123 124 183 110 220 209 131 83 134 217 183 200 45 75 23 55 87 28 99 114 132 120 51 21 192 94 31 249 86 132 146 23 199 141 155 196 134 28 13 208 128 102 3 148 170 113 252 38 146 45 29 24 13 62 120 45 84 90 125 127 116 184 53 97 215 69 48 172 179 106 10 149 53 64 188 86 68 93 105 213 213 221 21 108 102 185 79 122 187 243 222 34 204 158 219 188 24 0 222 122 104 172 104 31 181 250 246 91 55 123 219 107 68 216 69 119 31 181 66 183 140 17 202 168 73 135 232 42 219 34 107 155 106 206 243 59 39 90 253 111 31 220 115 30 215 188 190 205 230 148 71 72 139 126 99 235 57 231 69 200 62 116 196 109 76 46 158 46 206 146 48 93 50 80 75 154 12 8 246 162 191 49 10 110 162 78 155 67 215 72 162 163 110 213 81 183 163 231 109 55 221 253 130 43 5 187 167 93 39 221 123 255 233 58 150 101 102 26 124 40 84 120 236 100 125 167 142 135 80 69 122 34 200 78 133 247 158 68 89 229 77 88 161 18 38 193 94 234 174 229 74 31 39 220 128 237 180 112 95 34 58 169 120 183 215 99 211 204 169 254 63 255 29 129 96 229 195 184 83 104 210 39 152 237 68 2 119 159 224 170 19 81 140 140 52 22 110 135 246 1 112 146 111 222 146 253 76 228 99 205 35 220 195 145 237 29 178 139 16 241 136 60 82 100 144 188 112 56 103 9 145 17 231 45 89 56 68 178 134 211 157 41 46 54 177 0 77 82 75 143 20 169 106 72 127 132 34 148 247 246 254 13 120 152 66 219 132 118 126 93 251 15 22 128 23 114 125 59 36 1 216 137 223 212 237 218 192 162 204 11 170 130 44 221 79 189 65 148 31 2 252 12 108 210 35 117 204 54 124 29 84 122 40 223 237 113 127 103 243 196 111 147 162 192 44 42 204 27 41 74 148 162 50 114 52 73 54 126 197 203 100 122 88 198 84 85 44 52 216 174 167 5 253 72 228 105 197 185 173 36 153 193 114 34 155 228 229 185 43 166 192 176 150 228 86 21 139 99 101 88 102 201 249 161 110 124 172 48 119 37 10 248 188 36 226 147 158 7 99 23 39 113 140 87 63 54 82 80 178 23 130 106 46 70 55 187 167 58 79 84 118 112 149 247 172 43 23 51 101 208 101 9 254 221 195 136 212 248 198 239 211 208 88 243 228 20 107 99 225 78 52 24 238 248 194 112 44 220 42 198 91 99 225 70 122 246 127 100 24 11 183 179 246 19 253 73 105 44 220 251 199 253 30 126 193 15 120 28 11 183 242 9 165 127 137 113 24 11 119 162 78 231 85 221 201 246 121 69 210 157 236 80 217 66 74 206 236 11 20 239 103 203 63 225 208 13 203 240 88 190 141 204 123 138 58 100 18 175 41 207 239 89 70 254 66 112 216 117 57 104 252 253 15 41 114 144 154 158 226 131 31 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'elements-general' stamp: 'np 4/3/2008 05:22'!
menuElement
	"just for example, override in your own style subclass!! "
 	| menu |
	menu := JoomlaMenu new
		header: 'Main menu';
		addFirstPageWithDescription: 'Introduction';
		addPage: 'Documentation' description: 'Documentation';
		addPage: 'Download' description: 'Download'.
	self app canBeEdited ifTrue:
		[menu addPage: 'Authors' description: 'Authors only'].
	^menu! !

!JoomlaStyle methodsFor: 'frame printing' stamp: ' 10/1/08 16:19'!
pageFrameWideWith: aWebElement title: aTitleString
	"wide=without right modules. template name: mitra"
	| e |
	self app title: aTitleString. self app htmlHeaderElements.
	e := WebElement new.
	e table width: 1 "100%"; cellPadding: 0; cellSpacing: 0.
	e cell add: self joomlaHeaderElement. e newRow.
	e cell add: self joomlaToolbarElement.  e newRow.
	e cell table width: 1; cellPadding: 0; cellSpacing: 0.
	e cell cell width: 4. 
	e cell newCell valign: #top; width: 201; add: self joomlaLeftElement.
	e cell newCell valign: #top; add: (self joomlaCenterWith: aWebElement).
	e cell add: (WebElement newDiv align: #center; 
		add: self joomlaFooterElement; yourself).
	^self app add: e; yourself! !

!JoomlaStyle methodsFor: 'frame printing' stamp: ' 10/1/08 16:19'!
pageFrameWith: aWebElement title: aTitleString
	"template name: mitra"
	| e |
	self app title: aTitleString. self app htmlHeaderElements.
	e := WebElement new.
	e table width: 1 "100%"; cellPadding: 0; cellSpacing: 0.
	e cell add: self joomlaHeaderElement. e newRow.
	e cell add: self joomlaToolbarElement.  e newRow.
	e cell table width: 1; cellPadding: 0; cellSpacing: 0.
	e cell cell width: 4. 
	e cell newCell valign: #top; width: 201; add: self joomlaLeftElement.
	e cell newCell valign: #top; add: (self joomlaCenterWith: aWebElement).
	e cell newCell valign: #top; width: 201 "5"; add: self joomlaRightElement.
	e cell add: (WebElement newDiv align: #center; 
		add: self joomlaFooterElement; yourself).
	^self app add: e; yourself! !

!JoomlaStyle methodsFor: 'frame printing' stamp: ' 10/1/08 16:19'!
pageFrameWith: aWebElement wide: aWideElement title: aTitleString
	aWebElement add: aWideElement.
	^self pageFrameWith: aWebElement title: aTitleString! !

!JoomlaStyle methodsFor: 'elements-general' stamp: ' 10/1/08 16:19'!
rightBottomElement
	"just for example, override in your own style subclass!! "
 	| e |
	e := JoomlaModule new.
	^e! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:33'!
rightFooterPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 12 0 0 0 26 8 2 0 0 1 123 159 26 160 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 1 109 73 68 65 84 120 156 189 146 75 143 130 80 12 133 121 84 48 18 23 138 198 13 113 227 194 255 255 131 88 248 74 148 248 0 3 24 21 100 190 203 245 21 113 51 139 153 110 40 189 237 233 233 105 229 114 185 180 90 45 201 243 124 187 221 138 109 219 147 201 68 48 195 48 228 118 187 213 223 44 203 84 72 200 157 207 231 65 16 168 244 193 96 224 186 174 80 108 212 38 215 235 245 94 119 62 159 119 187 221 104 52 146 162 40 248 156 78 39 209 73 202 163 199 189 194 178 172 134 183 88 44 64 105 183 219 158 231 105 108 233 245 122 252 224 45 151 75 90 246 251 125 41 203 242 120 60 18 130 12 236 210 52 21 211 52 141 135 145 190 223 239 95 168 79 251 22 130 183 85 219 43 180 217 108 28 199 1 5 30 250 65 134 195 225 225 112 88 175 215 190 239 119 187 93 162 170 35 195 85 85 21 134 33 84 59 157 142 210 133 23 222 199 227 49 12 40 87 89 164 32 29 136 240 164 92 248 7 18 149 8 49 9 191 223 120 189 15 244 7 33 166 209 99 219 15 195 255 200 83 66 152 181 241 12 89 212 103 36 28 198 125 173 108 58 157 62 43 152 51 138 162 36 73 208 0 65 148 0 162 186 75 28 199 26 6 85 1 96 111 140 189 90 173 184 58 246 75 170 186 66 221 94 47 151 58 82 137 144 10 13 32 137 40 217 223 9 114 95 172 95 95 7 48 179 217 140 109 170 179 252 24 132 75 214 14 173 241 171 218 190 168 210 180 255 78 106 30 194 47 144 244 97 105 251 1 223 47 176 91 251 203 33 187 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:33'!
rightPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 14 0 0 0 2 8 2 0 0 1 144 239 74 107 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 0 46 73 68 65 84 120 156 99 249 255 255 63 3 3 3 11 144 122 247 238 29 11 132 7 21 96 64 2 55 111 222 84 84 84 68 23 197 4 64 117 162 162 162 60 60 60 0 130 56 23 172 238 202 217 139 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'elements-general' stamp: ' 10/1/08 16:19'!
rightTopElement
	"just for example, override in your own style subclass!! "
 	| e |
	e := JoomlaModule new.
	^e! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:33'!
rightTopLinkPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 11 0 0 0 33 8 2 0 0 1 247 254 246 183 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 0 254 73 68 65 84 120 156 189 83 75 14 132 32 12 21 196 168 113 167 27 239 127 52 151 122 2 127 56 15 218 128 48 213 100 54 211 132 216 239 43 175 84 179 239 187 33 41 138 194 157 219 231 56 14 23 176 214 154 243 60 205 182 109 94 239 186 206 92 215 5 85 107 237 114 129 81 215 117 170 69 136 178 44 157 134 10 246 173 235 202 237 128 8 48 246 197 138 228 22 78 83 74 177 54 142 35 107 1 47 166 225 16 54 181 140 46 212 231 174 170 170 242 194 182 109 19 151 0 159 187 150 101 65 21 177 97 23 70 160 189 176 125 15 50 10 77 46 218 24 174 242 18 237 80 197 249 96 155 196 115 188 56 93 33 14 59 137 191 177 250 217 158 231 185 105 154 200 167 239 251 132 255 247 59 201 136 32 65 12 195 99 231 25 216 12 66 126 204 8 196 195 56 5 12 177 69 130 129 22 247 145 203 247 8 11 37 99 208 204 229 37 165 12 235 229 17 131 194 111 24 47 242 167 140 105 154 232 215 162 71 17 222 101 24 6 154 35 146 176 211 97 45 130 124 0 36 26 171 206 103 149 239 16 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:34'!
spacerPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 1 0 0 0 1 1 3 0 0 0 37 219 86 202 0 0 0 3 115 66 73 84 8 8 8 219 225 79 224 0 0 0 6 80 76 84 69 255 255 255 0 0 0 85 194 211 126 0 0 0 2 116 82 78 83 0 0 118 147 205 56 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 0 22 116 69 88 116 67 114 101 97 116 105 111 110 32 84 105 109 101 0 48 56 47 48 56 47 48 52 239 235 69 241 0 0 0 37 116 69 88 116 83 111 102 116 119 97 114 101 0 77 97 99 114 111 109 101 100 105 97 32 70 105 114 101 119 111 114 107 115 32 77 88 32 50 48 48 52 135 118 172 207 0 0 0 10 73 68 65 84 120 156 99 96 0 0 0 2 0 1 72 175 164 113 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:34'!
subheadBgPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 9 0 0 0 21 8 2 0 0 1 108 224 99 49 0 0 0 9 112 72 89 115 0 0 10 235 0 0 10 235 1 130 139 13 90 0 0 0 162 73 68 65 84 120 156 157 142 57 14 194 64 12 69 93 184 65 74 77 133 132 168 83 112 121 206 65 71 77 65 68 65 88 10 80 134 204 98 147 89 164 56 145 156 2 55 246 243 211 31 15 182 199 3 0 224 106 189 3 102 116 239 123 164 97 25 55 17 209 220 206 144 10 109 146 113 250 52 167 50 149 120 181 169 99 11 182 75 173 79 141 50 117 233 1 52 143 11 136 26 172 153 112 14 9 182 83 166 153 207 151 11 84 219 253 8 16 188 136 5 55 2 75 32 175 26 9 160 102 190 207 70 192 235 10 74 33 139 212 220 17 121 213 177 235 117 183 148 243 246 191 123 11 255 228 160 231 72 58 38 233 126 215 146 109 117 12 175 199 161 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:35'!
topLeftPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 12 0 0 0 33 8 2 0 0 1 21 34 237 206 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 1 115 73 68 65 84 120 156 221 146 203 142 130 64 16 69 65 90 19 195 202 141 232 70 141 11 254 255 147 72 140 137 160 27 5 124 242 240 148 69 64 122 88 204 108 167 19 165 250 214 189 245 234 50 117 93 87 85 101 248 165 105 106 138 162 240 60 207 56 142 51 30 143 229 35 55 40 98 10 199 140 70 163 143 13 22 69 209 102 179 49 175 215 107 189 94 95 46 23 193 92 215 53 28 231 115 36 4 24 22 28 81 227 21 12 97 16 4 164 18 109 89 150 34 130 68 104 137 220 179 72 255 3 131 222 84 192 135 168 73 146 0 205 102 51 113 158 207 231 233 116 186 88 44 178 44 187 94 175 210 80 28 199 97 24 226 80 173 20 162 77 107 200 38 170 166 176 161 182 132 30 52 192 210 162 186 186 56 24 52 215 12 79 135 176 219 237 232 220 247 125 129 30 143 199 241 120 220 110 183 141 16 242 126 191 231 142 132 113 138 240 249 124 194 199 201 189 169 158 40 52 152 231 121 155 90 132 140 245 126 191 247 138 208 58 58 104 160 84 171 155 14 106 41 127 101 253 14 250 62 70 39 213 99 181 35 212 163 175 69 127 108 40 255 180 43 187 211 186 241 49 148 195 225 192 75 205 231 115 222 222 14 14 253 116 58 161 91 173 86 109 34 52 140 87 118 26 6 23 222 130 205 93 46 151 170 97 127 240 117 145 80 67 191 221 110 196 224 142 77 41 118 123 154 11 234 100 50 113 62 47 108 181 210 144 52 178 250 248 255 158 237 192 160 6 25 54 9 198 119 41 195 36 171 222 255 68 210 29 26 36 189 1 144 157 23 59 14 199 48 93 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:35'!
topLinkBacPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 3 0 0 0 33 8 3 0 0 1 92 149 209 38 0 0 0 105 80 76 84 69 40 112 4 44 114 6 45 114 6 44 114 8 47 116 12 51 118 11 52 118 11 50 118 15 53 120 19 59 122 16 56 122 23 59 124 26 62 126 30 65 126 21 66 126 21 73 130 26 80 134 31 66 129 34 68 130 37 72 133 41 75 135 45 87 138 36 94 142 41 79 137 49 81 139 52 85 141 56 101 147 46 109 151 51 225 225 225 226 226 226 234 234 234 243 243 243 244 244 244 251 251 251 255 255 255 182 49 130 82 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 0 111 73 68 65 84 120 156 45 141 75 14 194 64 12 67 61 51 175 31 40 130 77 47 80 238 127 167 170 98 87 33 117 213 77 11 42 14 16 201 113 18 217 14 3 51 87 222 116 192 141 157 158 129 7 13 45 53 171 251 133 3 137 86 156 68 45 22 113 22 157 40 178 114 36 170 88 155 205 153 167 29 177 85 78 120 121 186 219 58 25 17 81 140 172 8 249 113 220 230 35 209 252 111 129 234 251 74 108 74 214 167 15 203 207 15 177 87 116 3 173 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:35'!
topLinkPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 7 0 0 0 25 8 3 0 0 1 189 87 244 156 0 0 0 150 80 76 84 69 45 114 6 47 116 10 50 117 11 51 119 14 57 121 15 54 120 18 57 122 22 59 124 25 61 126 28 64 126 21 70 129 25 64 128 31 77 133 31 66 129 34 69 131 37 72 131 35 77 135 38 72 133 41 77 136 47 83 136 35 83 138 42 90 140 40 89 141 46 80 138 51 84 141 54 86 142 57 90 144 57 88 144 60 91 145 63 97 144 45 96 145 51 101 148 55 104 149 50 98 148 62 107 151 58 112 153 55 114 156 63 93 147 66 97 149 70 106 153 67 111 155 70 99 151 72 102 153 76 108 157 83 112 159 88 121 160 68 119 160 76 127 165 81 116 162 92 133 168 84 98 93 193 166 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 0 167 73 68 65 84 120 156 29 140 81 79 131 64 16 132 111 111 191 59 184 37 160 54 152 80 19 255 255 255 226 197 164 188 84 5 219 34 98 87 147 205 76 102 50 251 161 116 92 121 97 224 192 133 71 234 87 158 26 182 157 97 162 61 147 10 185 230 253 76 59 163 29 249 57 76 84 93 248 192 98 48 38 241 104 15 97 166 175 67 79 251 217 130 248 137 144 37 163 81 49 177 255 110 188 141 164 152 200 49 83 66 33 165 68 17 247 210 188 29 151 11 17 21 117 37 89 180 111 252 215 131 111 189 14 162 206 51 167 218 134 254 209 118 229 52 55 183 211 178 56 167 138 213 234 76 115 255 249 197 200 41 175 174 230 188 230 235 14 65 160 37 159 65 226 146 26 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:36'!
topModPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 201 0 0 0 34 8 3 0 0 1 181 156 61 224 0 0 0 219 80 76 84 69 126 177 38 127 178 38 128 180 39 130 181 40 131 182 41 133 184 42 135 185 42 136 187 43 138 189 44 139 190 45 141 191 46 142 193 47 144 195 47 145 196 48 147 197 49 148 199 50 150 201 51 151 202 52 153 203 53 153 204 53 155 205 53 156 206 54 158 208 55 157 208 56 156 203 65 160 206 72 168 213 76 169 212 82 172 215 83 175 215 99 177 216 99 181 219 104 185 221 110 214 235 173 220 238 181 216 216 216 217 217 217 218 218 218 219 219 219 220 220 220 221 221 221 223 223 223 224 224 224 227 227 227 228 228 228 229 229 229 230 230 230 231 231 231 232 232 232 233 233 233 235 235 235 236 236 236 237 237 237 238 238 238 239 239 239 247 251 238 240 240 240 241 241 241 242 242 242 243 243 243 244 244 244 245 245 245 246 246 246 247 247 247 248 248 248 249 249 249 250 250 250 251 251 251 252 254 249 252 252 252 253 253 253 254 254 254 255 255 255 249 11 79 202 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 3 177 73 68 65 84 120 156 221 89 223 111 27 69 16 158 89 207 173 27 90 84 42 37 45 141 106 18 209 240 192 99 121 9 15 188 55 255 122 197 11 72 128 84 26 126 132 164 85 105 42 165 144 144 248 110 239 102 58 251 235 236 138 179 170 75 125 9 230 147 229 243 249 188 247 205 236 236 204 55 123 166 61 118 0 8 2 250 46 232 15 17 133 33 118 63 64 23 30 49 185 111 59 175 128 41 9 166 221 151 0 8 190 214 119 241 52 74 136 8 156 120 79 45 237 149 209 0 253 30 96 102 5 90 130 159 184 235 102 31 125 73 252 79 39 205 133 80 189 192 6 33 222 5 239 36 34 171 211 16 205 1 195 40 180 119 176 181 200 240 110 28 108 81 245 217 143 167 125 134 236 110 85 36 124 220 139 6 89 8 22 57 180 24 180 7 85 158 71 117 49 190 242 89 7 76 1 196 104 123 113 136 208 31 219 39 191 246 24 241 21 30 144 155 62 237 67 242 100 215 169 255 117 47 195 88 215 90 213 111 72 163 67 92 191 33 2 151 50 140 191 9 41 224 147 46 166 66 190 157 207 134 188 40 53 236 33 45 253 119 26 74 254 19 239 140 123 49 245 64 121 34 247 140 26 246 102 237 62 84 108 6 225 144 186 248 20 202 139 79 200 189 222 105 228 251 65 56 60 110 238 216 241 225 77 98 225 242 103 55 24 203 27 171 21 131 73 221 114 127 15 70 162 104 200 87 77 224 158 107 185 39 68 34 75 207 181 220 19 190 138 251 25 171 31 134 85 221 38 64 88 244 200 24 202 82 144 3 175 13 210 254 64 191 49 172 21 58 88 26 127 157 202 153 14 243 166 199 100 138 2 182 191 41 161 244 41 141 204 151 185 120 51 152 171 129 89 242 0 224 221 75 239 5 34 97 112 165 122 177 126 107 57 115 243 111 84 229 243 77 171 36 211 227 237 161 40 0 172 189 117 184 113 131 154 151 219 192 13 225 251 7 244 135 72 109 97 242 251 132 202 117 149 251 253 243 33 56 116 129 108 203 24 214 75 114 107 80 239 119 119 5 31 14 222 127 212 140 236 5 213 86 138 1 179 254 233 67 99 79 73 68 76 103 223 179 28 56 214 164 241 217 8 3 146 156 113 42 44 163 1 73 84 184 99 93 25 210 147 76 2 195 146 240 72 73 100 88 79 82 76 254 15 36 0 97 186 126 217 65 241 29 124 108 236 131 72 4 196 22 127 214 85 201 172 234 251 51 35 144 202 126 122 5 217 129 60 34 9 67 242 4 238 133 187 38 241 1 200 250 149 148 196 72 75 144 36 205 127 244 34 22 36 42 216 7 121 115 131 48 215 243 5 146 19 136 162 165 231 124 113 90 241 66 29 198 217 104 73 138 247 142 148 97 54 63 201 25 102 146 172 164 209 222 249 29 214 114 48 66 99 63 94 51 129 207 79 23 87 175 240 238 253 37 147 92 25 202 23 114 215 154 176 186 220 235 102 18 188 226 70 186 39 237 63 25 19 93 119 80 232 113 60 145 163 209 122 129 36 213 49 109 122 55 92 193 191 113 93 45 145 107 80 144 33 243 185 26 173 209 192 201 203 87 27 150 234 170 218 208 201 170 138 243 163 242 186 173 235 3 63 229 223 141 31 220 46 173 6 251 206 243 202 144 184 70 247 116 174 120 246 215 178 211 241 10 112 254 236 246 23 206 234 10 107 220 152 132 117 245 10 227 180 185 110 179 46 133 41 178 106 133 47 246 65 227 5 26 60 187 110 155 46 135 51 204 17 8 158 64 211 12 42 193 131 162 105 70 225 152 98 194 171 235 9 167 166 33 118 144 82 175 174 39 181 112 8 138 247 68 183 40 171 28 147 84 115 99 158 172 182 39 177 165 136 121 178 218 158 196 160 80 58 133 199 215 104 204 7 33 135 32 229 137 28 165 103 94 216 238 46 242 35 48 104 55 28 56 251 20 255 4 153 107 43 219 223 229 7 102 249 114 187 217 72 151 195 118 35 111 48 98 147 217 238 137 210 39 73 29 105 122 204 54 179 160 181 2 0 242 159 52 226 205 207 158 120 60 184 162 25 92 62 14 211 241 45 143 186 3 247 166 26 180 178 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:36'!
topPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 3 0 0 0 33 8 2 0 0 1 228 41 182 67 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 0 116 73 68 65 84 120 156 173 78 201 13 192 32 12 227 48 172 194 254 219 48 6 240 65 188 104 2 137 104 85 245 215 72 150 157 196 57 48 198 64 239 29 222 123 132 16 22 207 57 209 90 3 7 215 216 19 99 68 173 21 214 90 56 231 22 216 203 204 53 154 49 198 160 148 194 68 21 38 106 222 136 236 183 76 44 186 91 134 215 46 105 191 160 183 20 234 203 57 155 21 72 41 137 218 143 156 67 79 181 191 61 223 124 249 126 81 23 22 6 39 141 45 150 224 96 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'imgs' stamp: 'mivsek 1/10/2008 16:36'!
topRightPng
^#(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 14 0 0 0 33 8 2 0 0 1 17 215 61 243 0 0 0 9 112 72 89 115 0 0 11 18 0 0 11 18 1 210 221 126 252 0 0 1 151 73 68 65 84 120 156 237 147 201 138 194 64 16 134 179 148 34 184 33 184 132 224 2 94 124 255 183 241 36 24 34 162 32 26 131 136 251 124 157 238 49 216 198 131 135 185 77 29 122 169 174 229 175 250 171 229 116 58 149 203 101 57 30 143 106 243 125 223 113 28 41 149 74 106 51 183 203 229 34 200 225 112 48 38 24 43 147 243 249 252 107 242 120 60 216 226 56 238 247 251 178 223 239 185 112 218 110 183 202 209 201 4 19 19 87 139 40 119 17 215 117 141 26 36 146 36 137 118 80 250 219 237 214 104 52 196 243 60 227 145 159 116 218 87 29 30 230 68 42 48 19 127 181 90 13 6 3 83 205 110 183 83 155 200 120 60 230 48 155 205 70 163 145 241 34 21 169 155 205 38 230 212 153 71 213 66 132 52 77 109 173 141 229 69 91 108 251 196 248 162 5 41 15 94 38 185 86 119 120 179 217 80 78 183 219 213 111 170 55 176 208 235 245 208 206 231 243 225 112 200 131 170 141 8 116 139 53 12 67 192 82 139 201 6 29 92 112 130 158 122 189 46 207 12 20 90 173 86 175 215 235 253 126 207 145 113 113 50 182 20 97 239 120 139 181 159 107 43 214 126 211 157 111 180 239 170 207 182 211 233 148 13 246 120 102 173 84 42 116 135 21 108 150 131 76 38 19 203 27 102 150 203 37 99 19 4 1 68 229 83 72 183 53 151 52 92 15 30 7 254 2 125 91 44 22 228 105 183 219 102 236 117 143 233 55 20 96 135 79 173 86 115 51 193 129 217 93 175 215 157 78 135 40 118 5 184 65 49 88 137 205 181 213 106 69 81 4 36 53 10 250 43 90 162 127 185 6 3 104 18 42 30 11 89 176 4 120 31 41 47 148 127 211 191 49 253 1 124 109 201 2 220 120 109 78 0 0 0 0 73 69 78 68 174 66 96 130 )! !

!JoomlaStyle methodsFor: 'frame printing' stamp: ' 10/1/08 16:19'!
wikiPageFrameWith: aWebElement title: aTitleString
	^self pageFrameWith: aWebElement title: aTitleString! !

!WebStyle class methodsFor: 'accessing' stamp: 'np 4/2/2008 23:34'!
default
	^AIDASite default style! !

!WebStyle class methodsFor: 'instance creation' stamp: 'np 4/2/2008 23:34'!
newOnSite: anAIDASite

	^super new site: anAIDASite! !

!WebStyle methodsFor: 'private-obsolete-printing-elements' stamp: 'np 4/2/2008 23:32'!
addBreaksTo: aString
	"changes all cr with <br>"

	^aString copyReplaceAll: (String with: Character cr) with: '<br>'

"WebFrameApp new addBreaksTo: ('aaa', (String with: Character cr), 'bbb')"! !

!WebStyle methodsFor: 'texts' stamp: 'np 4/2/2008 23:32'!
addHelpText
	^'Add help page'! !

!WebStyle methodsFor: 'private-obsolete-printing-elements' stamp: 'np 4/2/2008 23:32'!
addParagraphsTo: aString
	"changes all cr with <p>"

	^aString copyReplaceAll: (String with: Character cr) with: '<p>'

"WebFrameApp new addParagraphsTo: ('aaa', (String with: Character cr), 'bbb')"! !

!WebStyle methodsFor: 'styles-print' stamp: 'np 4/2/2008 23:30'!
addPrintSuperStyles
	"if false, don't add cssPrint methods from superclasses"
	^true! !

!WebStyle methodsFor: 'styles-screen' stamp: 'np 4/2/2008 23:30'!
addSuperStyles
	"if false, don't add css methods from superclasses"
	^true! !

!WebStyle methodsFor: 'maintenance' stamp: 'np 4/2/2008 23:19'!
adjustCSSLink: aString
	" '../images/arrow_left.gif' to '/img/arrowLeftGif.gif' "
	| in name extension |
	in := aString readStream.
	[in atEnd] whileFalse: [name := in upTo: $/]. "to find filename only"
	extension := name readStream upTo: $. ; upToEnd.
	name := self adjustImageName: name.
	^'/img/', name, '.', extension.! !

!WebStyle methodsFor: 'maintenance' stamp: 'np 4/2/2008 23:19'!
adjustImageName: aFilenameString
	"left_arrow.gif = leftArrowGif"
	| in out |
	in := aFilenameString readStream. out := WriteStream on: String new.
	in peek isDigit ifTrue: [out nextPut: $a]. "method cannot start with digit!!"
	[in atEnd] whileFalse: 
		[(#($- $_ $. ) includes:  in peek) 
			ifTrue: [in next "skip". out nextPut: in next asUppercase] 
			ifFalse: [out nextPut: in next] ].
	^out contents! !

!WebStyle methodsFor: 'maintenance' stamp: 'np 4/2/2008 23:19'!
adjustLinksInCSSMethod: aMethodSymbol
	"external image links in CSS are changed to format, eg. for arrow.gif  '/img/arrowGif.gif' "
	"this method is then recompiled back in 'styles-screen' !! "
	"WebStyle new adjustLinksInCSSMethod: #css1Body"
	| method in out chunk link dd |
	method := self class compiledMethodAt: aMethodSymbol. method := method getSource asString.
	in := method readStream. out := WriteStream on: String new.
	[in atEnd] whileFalse:
		[chunk := in upToAll: 'url('. in atEnd not ifTrue: [in skip: 4]. link := in upTo: $). 
		dd := (link includes: $") ifTrue: [link := link copyWithout: $". '"'] ifFalse: [''].
		out nextPutAll: chunk. 
		link notEmpty ifTrue: 
			[link := self adjustCSSLink: link.  out nextPutAll: 'url(', dd, link, dd, ')'] ].
	out contents = method ifFalse:
		[self class compile: out contents classified: 'styles-screen'].
	^out contents! !

!WebStyle methodsFor: 'private-obsolete-colors' stamp: 'np 4/2/2008 23:32'!
adminColor
	^#green! !

!WebStyle methodsFor: 'private-css' stamp: 'np 4/3/2008 00:12'!
allCssPrintMethods
	"WebStyle new allCssPrintMethods"
	| methods selectors |
	self addSuperStyles 
		ifTrue: [selectors := self class allSelectors]
		ifFalse: [selectors := self class selectors].
	methods := selectors select: [:each | 'cssPrint*' match:each asString].
	^SortedCollection withAll: methods sortBlock: [:a :b | a < b]! !

!WebStyle methodsFor: 'private-css' stamp: 'np 4/3/2008 00:14'!
allCssScreenMethods
	"for screen styles, all css* except cssPrint* , sorted alphabeticaly"
	"WebStyle new allCssScreenMethods"
	| selectors methods print |
	self addSuperStyles 
		ifTrue: [selectors := self class allSelectors] "From superclasses too"
		ifFalse: [selectors := self class selectors].
	methods := selectors select: [:each | 'css*' match:each asString].
	print := self allCssPrintMethods asSet.
	^SortedCollection 
		withAll: (methods reject: [:each | print includes: each])
		sortBlock: [:a :b | a < b]! !

!WebStyle methodsFor: 'private-javascript' stamp: 'np 4/2/2008 23:31'!
allJavascriptMethods
	"for javascript, all js* sorted alphabeticaly (sort is not realy needed, just for convenience)"
	"WebStyle new allJavascriptMethods"
	| methods |
	methods := self class allSelectors select: [:each | 'js*' match:each asString].
	^SortedCollection withAll: methods sortBlock: [:a :b | a < b]! !

!WebStyle methodsFor: 'accessing' stamp: 'np 4/2/2008 23:19'!
app
	"try to find a first sender up in calling stack, who is  WebApplication"
	^self firstAppFromStack! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 4/2/2008 23:30'!
calendarCSS
	"/jscalendar/calendar.css"
	^'
/*calendar-brown.css*/
/* The main calendar widget.  DIV containing a table. */

div.calendar { position: relative; }

.calendar, .calendar table {
  border: 1px solid #655;
  font-size: 11px;
  color: #000;
  cursor: default;
  background: #ffd;
  font-family: tahoma,verdana,sans-serif;
}

/* Header part -- contains navigation buttons and day names. */

.calendar .button { /* "<<", "<", ">", ">>" buttons have this class */
  text-align: center;    /* They are the navigation buttons */
  padding: 2px;          /* Make the buttons seem like they are pressing */
}

.calendar .nav {
/*  background: #edc url(menuarrow.gif) no-repeat 100% 100%; */
  background: #edc no-repeat 100% 100%;
}

.calendar thead .title { /* This holds the current "month, year" */
  font-weight: bold;      /* Pressing it will take you to the current date */
  text-align: center;
  background: #654;
  color: #fed;
  padding: 2px;
}

.calendar thead .headrow { /* Row <TR> containing navigation buttons */
  background: #edc;
  color: #000;
}

.calendar thead .name { /* Cells <TD> containing the day names */
  border-bottom: 1px solid #655;
  padding: 2px;
  text-align: center;
  color: #000;
}

.calendar thead .weekend { /* How a weekend day name shows in header */
  color: #f00;
}

.calendar thead .hilite { /* How do the buttons in header appear when hover */
  background-color: #faa;
  color: #000;
  border: 1px solid #f40;
  padding: 1px;
}

.calendar thead .active { /* Active (pressed) buttons in header */
  background-color: #c77;
  padding: 2px 0px 0px 2px;
}

.calendar thead .daynames { /* Row <TR> containing the day names */
  background: #fed;
}

/* The body part -- contains all the days in month. */

.calendar tbody .day { /* Cells <TD> containing month days dates */
  width: 2em;
  text-align: right;
  padding: 2px 4px 2px 2px;
}
.calendar tbody .day.othermonth {
  font-size: 80%;
  color: #bbb;
}
.calendar tbody .day.othermonth.oweekend {
  color: #fbb;
}

.calendar table .wn {
  padding: 2px 3px 2px 2px;
  border-right: 1px solid #000;
  background: #fed;
}

.calendar tbody .rowhilite td {
  background: #ddf;
}

.calendar tbody .rowhilite td.wn {
  background: #efe;
}

.calendar tbody td.hilite { /* Hovered cells <TD> */
  background: #ffe;
  padding: 1px 3px 1px 1px;
  border: 1px solid #bbb;
}

.calendar tbody td.active { /* Active (pressed) cells <TD> */
  background: #ddc;
  padding: 2px 2px 0px 2px;
}

.calendar tbody td.selected { /* Cell showing today date */
  font-weight: bold;
  border: 1px solid #000;
  padding: 1px 3px 1px 1px;
  background: #fea;
}

.calendar tbody td.weekend { /* Cells showing weekend days */
  color: #f00;
}

.calendar tbody td.today { font-weight: bold; }

.calendar tbody .disabled { color: #999; }

.calendar tbody .emptycell { /* Empty cells (the best is to hide them) */
  visibility: hidden;
}

.calendar tbody .emptyrow { /* Empty row (some months need less than 6 rows) */
  display: none;
}

/* The footer part -- status bar and "Close" button */

.calendar tfoot .footrow { /* The <TR> in footer (only one right now) */
  text-align: center;
  background: #988;
  color: #000;
}

.calendar tfoot .ttip { /* Tooltip (status bar) cell <TD> */
  border-top: 1px solid #655;
  background: #dcb;
  color: #840;
}

.calendar tfoot .hilite { /* Hover style for buttons in footer */
  background: #faa;
  border: 1px solid #f40;
  padding: 1px;
}

.calendar tfoot .active { /* Active (pressed) style for buttons in footer */
  background: #c77;
  padding: 2px 0px 0px 2px;
}

/* Combo boxes (menus that display months/years for direct selection) */

.calendar .combo {
  position: absolute;
  display: none;
  top: 0px;
  left: 0px;
  width: 4em;
  cursor: default;
  border: 1px solid #655;
  background: #ffe;
  color: #000;
  font-size: 90%;
  z-index: 100;
}

.calendar .combo .label,
.calendar .combo .label-IEfix {
  text-align: center;
  padding: 1px;
}

.calendar .combo .label-IEfix {
  width: 4em;
}

.calendar .combo .hilite {
  background: #fc8;
}

.calendar .combo .active {
  border-top: 1px solid #a64;
  border-bottom: 1px solid #a64;
  background: #fee;
  font-weight: bold;
}

.calendar td.time {
  border-top: 1px solid #a88;
  padding: 1px 0px;
  text-align: center;
  background-color: #fed;
}

.calendar td.time .hour,
.calendar td.time .minute,
.calendar td.time .ampm {
  padding: 0px 3px 0px 4px;
  border: 1px solid #988;
  font-weight: bold;
  background-color: #fff;
}

.calendar td.time .ampm {
  text-align: center;
}

.calendar td.time .colon {
  padding: 0px 2px 0px 3px;
  font-weight: bold;
}

.calendar td.time span.hilite {
  border-color: #000;
  background-color: #866;
  color: #fff;
}

.calendar td.time span.active {
  border-color: #f00;
  background-color: #000;
  color: #0f0;
}
'! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 4/2/2008 23:30'!
calendarCSSResource
	"For Javascript calendar input fields , see WebDateInputField"
	"WebStyle new calendarCSSResource"
	^self resources at: #jsCalendarCSS ifAbsentPut:
		[WebMethodResource 
			fromMethod: #calendarCSS on: self 
			contentType: 'text/css' preferedUrl: '/jscalendar/calendar.css' site: self site].! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 4/2/2008 23:30'!
calendarJavascript
	"For Javascript calendar input fields , see WebDateInputField"
	"WebStyle new calendarJavascript"
	^'
/*  Copyright Mihai Bazon, 2002-2005  |  www.bazon.net/mishoo
 * -----------------------------------------------------------
 *
 * The DHTML Calendar, version 1.0 "It is happening again"
 *
 * Details and latest version at:
 * www.dynarch.com/projects/calendar
 *
 * This script is developed by Dynarch.com.  Visit us at www.dynarch.com.
 *
 * This script is distributed under the GNU Lesser General Public License.
 * Read the entire license text here: http://www.gnu.org/licenses/lgpl.html
 */

// $Id: calendar.js,v 1.51 2005/03/07 16:44:31 mishoo Exp $

/** The Calendar object constructor. */
Calendar = function (firstDayOfWeek, dateStr, onSelected, onClose) {
	// member variables
	this.activeDiv = null;
	this.currentDateEl = null;
	this.getDateStatus = null;
	this.getDateToolTip = null;
	this.getDateText = null;
	this.timeout = null;
	this.onSelected = onSelected || null;
	this.onClose = onClose || null;
	this.dragging = false;
	this.hidden = false;
	this.minYear = 1970;
	this.maxYear = 2050;
	this.dateFormat = Calendar._TT["DEF_DATE_FORMAT"];
	this.ttDateFormat = Calendar._TT["TT_DATE_FORMAT"];
	this.isPopup = true;
	this.weekNumbers = true;
	this.firstDayOfWeek = typeof firstDayOfWeek == "number" ? firstDayOfWeek : Calendar._FD; // 0 for Sunday, 1 for Monday, etc.
	this.showsOtherMonths = false;
	this.dateStr = dateStr;
	this.ar_days = null;
	this.showsTime = false;
	this.time24 = true;
	this.yearStep = 2;
	this.hiliteToday = true;
	this.multiple = null;
	// HTML elements
	this.table = null;
	this.element = null;
	this.tbody = null;
	this.firstdayname = null;
	// Combo boxes
	this.monthsCombo = null;
	this.yearsCombo = null;
	this.hilitedMonth = null;
	this.activeMonth = null;
	this.hilitedYear = null;
	this.activeYear = null;
	// Information
	this.dateClicked = false;

	// one-time initializations
	if (typeof Calendar._SDN == "undefined") {
		// table of short day names
		if (typeof Calendar._SDN_len == "undefined")
			Calendar._SDN_len = 3;
		var ar = new Array();
		for (var i = 8; i > 0;) {
			ar[--i] = Calendar._DN[i].substr(0, Calendar._SDN_len);
		}
		Calendar._SDN = ar;
		// table of short month names
		if (typeof Calendar._SMN_len == "undefined")
			Calendar._SMN_len = 3;
		ar = new Array();
		for (var i = 12; i > 0;) {
			ar[--i] = Calendar._MN[i].substr(0, Calendar._SMN_len);
		}
		Calendar._SMN = ar;
	}
};

// ** constants

/// "static", needed for event handlers.
Calendar._C = null;

/// detect a special case of "web browser"
Calendar.is_ie = ( /msie/i.test(navigator.userAgent) &&
		   !!/opera/i.test(navigator.userAgent) );

Calendar.is_ie5 = ( Calendar.is_ie && /msie 5\.0/i.test(navigator.userAgent) );

/// detect Opera browser
Calendar.is_opera = /opera/i.test(navigator.userAgent);

/// detect KHTML-based browsers
Calendar.is_khtml = /Konqueror|Safari|KHTML/i.test(navigator.userAgent);

// BEGIN: UTILITY FUNCTIONS; beware that these might be moved into a separate
//        library, at some point.

Calendar.getAbsolutePos = function(el) {
	var SL = 0, ST = 0;
	var is_div = /^div$/i.test(el.tagName);
	if (is_div && el.scrollLeft)
		SL = el.scrollLeft;
	if (is_div && el.scrollTop)
		ST = el.scrollTop;
	var r = { x: el.offsetLeft - SL, y: el.offsetTop - ST };
	if (el.offsetParent) {
		var tmp = this.getAbsolutePos(el.offsetParent);
		r.x += tmp.x;
		r.y += tmp.y;
	}
	return r;
};

Calendar.isRelated = function (el, evt) {
	var related = evt.relatedTarget;
	if (!!related) {
		var type = evt.type;
		if (type == "mouseover") {
			related = evt.fromElement;
		} else if (type == "mouseout") {
			related = evt.toElement;
		}
	}
	while (related) {
		if (related == el) {
			return true;
		}
		related = related.parentNode;
	}
	return false;
};

Calendar.removeClass = function(el, className) {
	if (!!(el && el.className)) {
		return;
	}
	var cls = el.className.split(" ");
	var ar = new Array();
	for (var i = cls.length; i > 0;) {
		if (cls[--i] !!= className) {
			ar[ar.length] = cls[i];
		}
	}
	el.className = ar.join(" ");
};

Calendar.addClass = function(el, className) {
	Calendar.removeClass(el, className);
	el.className += " " + className;
};

// FIXME: the following 2 functions totally suck, are useless and should be replaced immediately.
Calendar.getElement = function(ev) {
	var f = Calendar.is_ie ? window.event.srcElement : ev.currentTarget;
	while (f.nodeType !!= 1 || /^div$/i.test(f.tagName))
		f = f.parentNode;
	return f;
};

Calendar.getTargetElement = function(ev) {
	var f = Calendar.is_ie ? window.event.srcElement : ev.target;
	while (f.nodeType !!= 1)
		f = f.parentNode;
	return f;
};

Calendar.stopEvent = function(ev) {
	ev || (ev = window.event);
	if (Calendar.is_ie) {
		ev.cancelBubble = true;
		ev.returnValue = false;
	} else {
		ev.preventDefault();
		ev.stopPropagation();
	}
	return false;
};

Calendar.addEvent = function(el, evname, func) {
	if (el.attachEvent) { // IE
		el.attachEvent("on" + evname, func);
	} else if (el.addEventListener) { // Gecko / W3C
		el.addEventListener(evname, func, true);
	} else {
		el["on" + evname] = func;
	}
};

Calendar.removeEvent = function(el, evname, func) {
	if (el.detachEvent) { // IE
		el.detachEvent("on" + evname, func);
	} else if (el.removeEventListener) { // Gecko / W3C
		el.removeEventListener(evname, func, true);
	} else {
		el["on" + evname] = null;
	}
};

Calendar.createElement = function(type, parent) {
	var el = null;
	if (document.createElementNS) {
		// use the XHTML namespace; IE won''t normally get here unless
		// _they_ "fix" the DOM2 implementation.
		el = document.createElementNS("http://www.w3.org/1999/xhtml", type);
	} else {
		el = document.createElement(type);
	}
	if (typeof parent !!= "undefined") {
		parent.appendChild(el);
	}
	return el;
};

// END: UTILITY FUNCTIONS

// BEGIN: CALENDAR STATIC FUNCTIONS

/** Internal -- adds a set of events to make some element behave like a button. */
Calendar._add_evs = function(el) {
	with (Calendar) {
		addEvent(el, "mouseover", dayMouseOver);
		addEvent(el, "mousedown", dayMouseDown);
		addEvent(el, "mouseout", dayMouseOut);
		if (is_ie) {
			addEvent(el, "dblclick", dayMouseDblClick);
			el.setAttribute("unselectable", true);
		}
	}
};

Calendar.findMonth = function(el) {
	if (typeof el.month !!= "undefined") {
		return el;
	} else if (typeof el.parentNode.month !!= "undefined") {
		return el.parentNode;
	}
	return null;
};

Calendar.findYear = function(el) {
	if (typeof el.year !!= "undefined") {
		return el;
	} else if (typeof el.parentNode.year !!= "undefined") {
		return el.parentNode;
	}
	return null;
};

Calendar.showMonthsCombo = function () {
	var cal = Calendar._C;
	if (!!cal) {
		return false;
	}
	var cal = cal;
	var cd = cal.activeDiv;
	var mc = cal.monthsCombo;
	if (cal.hilitedMonth) {
		Calendar.removeClass(cal.hilitedMonth, "hilite");
	}
	if (cal.activeMonth) {
		Calendar.removeClass(cal.activeMonth, "active");
	}
	var mon = cal.monthsCombo.getElementsByTagName("div")[cal.date.getMonth()];
	Calendar.addClass(mon, "active");
	cal.activeMonth = mon;
	var s = mc.style;
	s.display = "block";
	if (cd.navtype < 0)
		s.left = cd.offsetLeft + "px";
	else {
		var mcw = mc.offsetWidth;
		if (typeof mcw == "undefined")
			// Konqueror brain-dead techniques
			mcw = 50;
		s.left = (cd.offsetLeft + cd.offsetWidth - mcw) + "px";
	}
	s.top = (cd.offsetTop + cd.offsetHeight) + "px";
};

Calendar.showYearsCombo = function (fwd) {
	var cal = Calendar._C;
	if (!!cal) {
		return false;
	}
	var cal = cal;
	var cd = cal.activeDiv;
	var yc = cal.yearsCombo;
	if (cal.hilitedYear) {
		Calendar.removeClass(cal.hilitedYear, "hilite");
	}
	if (cal.activeYear) {
		Calendar.removeClass(cal.activeYear, "active");
	}
	cal.activeYear = null;
	var Y = cal.date.getFullYear() + (fwd ? 1 : -1);
	var yr = yc.firstChild;
	var show = false;
	for (var i = 12; i > 0; --i) {
		if (Y >= cal.minYear && Y <= cal.maxYear) {
			yr.innerHTML = Y;
			yr.year = Y;
			yr.style.display = "block";
			show = true;
		} else {
			yr.style.display = "none";
		}
		yr = yr.nextSibling;
		Y += fwd ? cal.yearStep : -cal.yearStep;
	}
	if (show) {
		var s = yc.style;
		s.display = "block";
		if (cd.navtype < 0)
			s.left = cd.offsetLeft + "px";
		else {
			var ycw = yc.offsetWidth;
			if (typeof ycw == "undefined")
				// Konqueror brain-dead techniques
				ycw = 50;
			s.left = (cd.offsetLeft + cd.offsetWidth - ycw) + "px";
		}
		s.top = (cd.offsetTop + cd.offsetHeight) + "px";
	}
};

// event handlers

Calendar.tableMouseUp = function(ev) {
	var cal = Calendar._C;
	if (!!cal) {
		return false;
	}
	if (cal.timeout) {
		clearTimeout(cal.timeout);
	}
	var el = cal.activeDiv;
	if (!!el) {
		return false;
	}
	var target = Calendar.getTargetElement(ev);
	ev || (ev = window.event);
	Calendar.removeClass(el, "active");
	if (target == el || target.parentNode == el) {
		Calendar.cellClick(el, ev);
	}
	var mon = Calendar.findMonth(target);
	var date = null;
	if (mon) {
		date = new Date(cal.date);
		if (mon.month !!= date.getMonth()) {
			date.setMonth(mon.month);
			cal.setDate(date);
			cal.dateClicked = false;
			cal.callHandler();
		}
	} else {
		var year = Calendar.findYear(target);
		if (year) {
			date = new Date(cal.date);
			if (year.year !!= date.getFullYear()) {
				date.setFullYear(year.year);
				cal.setDate(date);
				cal.dateClicked = false;
				cal.callHandler();
			}
		}
	}
	with (Calendar) {
		removeEvent(document, "mouseup", tableMouseUp);
		removeEvent(document, "mouseover", tableMouseOver);
		removeEvent(document, "mousemove", tableMouseOver);
		cal._hideCombos();
		_C = null;
		return stopEvent(ev);
	}
};

Calendar.tableMouseOver = function (ev) {
	var cal = Calendar._C;
	if (!!cal) {
		return;
	}
	var el = cal.activeDiv;
	var target = Calendar.getTargetElement(ev);
	if (target == el || target.parentNode == el) {
		Calendar.addClass(el, "hilite active");
		Calendar.addClass(el.parentNode, "rowhilite");
	} else {
		if (typeof el.navtype == "undefined" || (el.navtype !!= 50 && (el.navtype == 0 || Math.abs(el.navtype) > 2)))
			Calendar.removeClass(el, "active");
		Calendar.removeClass(el, "hilite");
		Calendar.removeClass(el.parentNode, "rowhilite");
	}
	ev || (ev = window.event);
	if (el.navtype == 50 && target !!= el) {
		var pos = Calendar.getAbsolutePos(el);
		var w = el.offsetWidth;
		var x = ev.clientX;
		var dx;
		var decrease = true;
		if (x > pos.x + w) {
			dx = x - pos.x - w;
			decrease = false;
		} else
			dx = pos.x - x;

		if (dx < 0) dx = 0;
		var range = el._range;
		var current = el._current;
		var count = Math.floor(dx / 10) % range.length;
		for (var i = range.length; --i >= 0;)
			if (range[i] == current)
				break;
		while (count-- > 0)
			if (decrease) {
				if (--i < 0)
					i = range.length - 1;
			} else if ( ++i >= range.length )
				i = 0;
		var newval = range[i];
		el.innerHTML = newval;

		cal.onUpdateTime();
	}
	var mon = Calendar.findMonth(target);
	if (mon) {
		if (mon.month !!= cal.date.getMonth()) {
			if (cal.hilitedMonth) {
				Calendar.removeClass(cal.hilitedMonth, "hilite");
			}
			Calendar.addClass(mon, "hilite");
			cal.hilitedMonth = mon;
		} else if (cal.hilitedMonth) {
			Calendar.removeClass(cal.hilitedMonth, "hilite");
		}
	} else {
		if (cal.hilitedMonth) {
			Calendar.removeClass(cal.hilitedMonth, "hilite");
		}
		var year = Calendar.findYear(target);
		if (year) {
			if (year.year !!= cal.date.getFullYear()) {
				if (cal.hilitedYear) {
					Calendar.removeClass(cal.hilitedYear, "hilite");
				}
				Calendar.addClass(year, "hilite");
				cal.hilitedYear = year;
			} else if (cal.hilitedYear) {
				Calendar.removeClass(cal.hilitedYear, "hilite");
			}
		} else if (cal.hilitedYear) {
			Calendar.removeClass(cal.hilitedYear, "hilite");
		}
	}
	return Calendar.stopEvent(ev);
};

Calendar.tableMouseDown = function (ev) {
	if (Calendar.getTargetElement(ev) == Calendar.getElement(ev)) {
		return Calendar.stopEvent(ev);
	}
};

Calendar.calDragIt = function (ev) {
	var cal = Calendar._C;
	if (!!(cal && cal.dragging)) {
		return false;
	}
	var posX;
	var posY;
	if (Calendar.is_ie) {
		posY = window.event.clientY + document.body.scrollTop;
		posX = window.event.clientX + document.body.scrollLeft;
	} else {
		posX = ev.pageX;
		posY = ev.pageY;
	}
	cal.hideShowCovered();
	var st = cal.element.style;
	st.left = (posX - cal.xOffs) + "px";
	st.top = (posY - cal.yOffs) + "px";
	return Calendar.stopEvent(ev);
};

Calendar.calDragEnd = function (ev) {
	var cal = Calendar._C;
	if (!!cal) {
		return false;
	}
	cal.dragging = false;
	with (Calendar) {
		removeEvent(document, "mousemove", calDragIt);
		removeEvent(document, "mouseup", calDragEnd);
		tableMouseUp(ev);
	}
	cal.hideShowCovered();
};

Calendar.dayMouseDown = function(ev) {
	var el = Calendar.getElement(ev);
	if (el.disabled) {
		return false;
	}
	var cal = el.calendar;
	cal.activeDiv = el;
	Calendar._C = cal;
	if (el.navtype !!= 300) with (Calendar) {
		if (el.navtype == 50) {
			el._current = el.innerHTML;
			addEvent(document, "mousemove", tableMouseOver);
		} else
			addEvent(document, Calendar.is_ie5 ? "mousemove" : "mouseover", tableMouseOver);
		addClass(el, "hilite active");
		addEvent(document, "mouseup", tableMouseUp);
	} else if (cal.isPopup) {
		cal._dragStart(ev);
	}
	if (el.navtype == -1 || el.navtype == 1) {
		if (cal.timeout) clearTimeout(cal.timeout);
		cal.timeout = setTimeout("Calendar.showMonthsCombo()", 250);
	} else if (el.navtype == -2 || el.navtype == 2) {
		if (cal.timeout) clearTimeout(cal.timeout);
		cal.timeout = setTimeout((el.navtype > 0) ? "Calendar.showYearsCombo(true)" : "Calendar.showYearsCombo(false)", 250);
	} else {
		cal.timeout = null;
	}
	return Calendar.stopEvent(ev);
};

Calendar.dayMouseDblClick = function(ev) {
	Calendar.cellClick(Calendar.getElement(ev), ev || window.event);
	if (Calendar.is_ie) {
		document.selection.empty();
	}
};

Calendar.dayMouseOver = function(ev) {
	var el = Calendar.getElement(ev);
	if (Calendar.isRelated(el, ev) || Calendar._C || el.disabled) {
		return false;
	}
	if (el.ttip) {
		if (el.ttip.substr(0, 1) == "_") {
			el.ttip = el.caldate.print(el.calendar.ttDateFormat) + el.ttip.substr(1);
		}
		el.calendar.tooltips.innerHTML = el.ttip;
	}
	if (el.navtype !!= 300) {
		Calendar.addClass(el, "hilite");
		if (el.caldate) {
			Calendar.addClass(el.parentNode, "rowhilite");
		}
	}
	return Calendar.stopEvent(ev);
};

Calendar.dayMouseOut = function(ev) {
	with (Calendar) {
		var el = getElement(ev);
		if (isRelated(el, ev) || _C || el.disabled)
			return false;
		removeClass(el, "hilite");
		if (el.caldate)
			removeClass(el.parentNode, "rowhilite");
		if (el.calendar)
			el.calendar.tooltips.innerHTML = _TT["SEL_DATE"];
		return stopEvent(ev);
	}
};

/**
 *  A generic "click" handler :) handles all types of buttons defined in this
 *  calendar.
 */
Calendar.cellClick = function(el, ev) {
	var cal = el.calendar;
	var closing = false;
	var newdate = false;
	var date = null;
	if (typeof el.navtype == "undefined") {
		if (cal.currentDateEl) {
			Calendar.removeClass(cal.currentDateEl, "selected");
			Calendar.addClass(el, "selected");
			closing = (cal.currentDateEl == el);
			if (!!closing) {
				cal.currentDateEl = el;
			}
		}
		cal.date.setDateOnly(el.caldate);
		date = cal.date;
		var other_month = !!(cal.dateClicked = !!el.otherMonth);
		if (!!other_month && !!cal.currentDateEl)
			cal._toggleMultipleDate(new Date(date));
		else
			newdate = !!el.disabled;
		// a date was clicked
		if (other_month)
			cal._init(cal.firstDayOfWeek, date);
	} else {
		if (el.navtype == 200) {
			Calendar.removeClass(el, "hilite");
			cal.callCloseHandler();
			return;
		}
		date = new Date(cal.date);
		if (el.navtype == 0)
			date.setDateOnly(new Date()); // TODAY
		// unless "today" was clicked, we assume no date was clicked so
		// the selected handler will know not to close the calenar when
		// in single-click mode.
		// cal.dateClicked = (el.navtype == 0);
		cal.dateClicked = false;
		var year = date.getFullYear();
		var mon = date.getMonth();
		function setMonth(m) {
			var day = date.getDate();
			var max = date.getMonthDays(m);
			if (day > max) {
				date.setDate(max);
			}
			date.setMonth(m);
		};
		switch (el.navtype) {
		    case 400:
			Calendar.removeClass(el, "hilite");
			var text = Calendar._TT["ABOUT"];
			if (typeof text !!= "undefined") {
				text += cal.showsTime ? Calendar._TT["ABOUT_TIME"] : "";
			} else {
				// FIXME: this should be removed as soon as lang files get updated!!
				text = "Help and about box text is not translated into this language.\n" +
					"If you know this language and you feel generous please update\n" +
					"the corresponding file in \"lang\" subdir to match calendar-en.js\n" +
					"and send it back to <mihai_bazon@yahoo.com> to get it into the distribution  ;-)\n\n" +
					"Thank you!!\n" +
					"http://dynarch.com/mishoo/calendar.epl\n";
			}
			alert(text);
			return;
		    case -2:
			if (year > cal.minYear) {
				date.setFullYear(year - 1);
			}
			break;
		    case -1:
			if (mon > 0) {
				setMonth(mon - 1);
			} else if (year-- > cal.minYear) {
				date.setFullYear(year);
				setMonth(11);
			}
			break;
		    case 1:
			if (mon < 11) {
				setMonth(mon + 1);
			} else if (year < cal.maxYear) {
				date.setFullYear(year + 1);
				setMonth(0);
			}
			break;
		    case 2:
			if (year < cal.maxYear) {
				date.setFullYear(year + 1);
			}
			break;
		    case 100:
			cal.setFirstDayOfWeek(el.fdow);
			return;
		    case 50:
			var range = el._range;
			var current = el.innerHTML;
			for (var i = range.length; --i >= 0;)
				if (range[i] == current)
					break;
			if (ev && ev.shiftKey) {
				if (--i < 0)
					i = range.length - 1;
			} else if ( ++i >= range.length )
				i = 0;
			var newval = range[i];
			el.innerHTML = newval;
			cal.onUpdateTime();
			return;
		    case 0:
			// TODAY will bring us here
			if ((typeof cal.getDateStatus == "function") &&
			    cal.getDateStatus(date, date.getFullYear(), date.getMonth(), date.getDate())) {
				return false;
			}
			break;
		}
		if (!!date.equalsTo(cal.date)) {
			cal.setDate(date);
			newdate = true;
		} else if (el.navtype == 0)
			newdate = closing = true;
	}
	if (newdate) {
		ev && cal.callHandler();
	}
	if (closing) {
		Calendar.removeClass(el, "hilite");
		ev && cal.callCloseHandler();
	}
};

// END: CALENDAR STATIC FUNCTIONS

// BEGIN: CALENDAR OBJECT FUNCTIONS

/**
 *  This function creates the calendar inside the given parent.  If _par is
 *  null than it creates a popup calendar inside the BODY element.  If _par is
 *  an element, be it BODY, then it creates a non-popup calendar (still
 *  hidden).  Some properties need to be set before calling this function.
 */
Calendar.prototype.create = function (_par) {
	var parent = null;
	if (!! _par) {
		// default parent is the document body, in which case we create
		// a popup calendar.
		parent = document.getElementsByTagName("body")[0];
		this.isPopup = true;
	} else {
		parent = _par;
		this.isPopup = false;
	}
	this.date = this.dateStr ? new Date(this.dateStr) : new Date();

	var table = Calendar.createElement("table");
	this.table = table;
	table.cellSpacing = 0;
	table.cellPadding = 0;
	table.calendar = this;
	Calendar.addEvent(table, "mousedown", Calendar.tableMouseDown);

	var div = Calendar.createElement("div");
	this.element = div;
	div.className = "calendar";
	if (this.isPopup) {
		div.style.position = "absolute";
		div.style.display = "none";
	}
	div.appendChild(table);

	var thead = Calendar.createElement("thead", table);
	var cell = null;
	var row = null;

	var cal = this;
	var hh = function (text, cs, navtype) {
		cell = Calendar.createElement("td", row);
		cell.colSpan = cs;
		cell.className = "button";
		if (navtype !!= 0 && Math.abs(navtype) <= 2)
			cell.className += " nav";
		Calendar._add_evs(cell);
		cell.calendar = cal;
		cell.navtype = navtype;
		cell.innerHTML = "<div unselectable=''on''>" + text + "</div>";
		return cell;
	};

	row = Calendar.createElement("tr", thead);
	var title_length = 6;
	(this.isPopup) && --title_length;
	(this.weekNumbers) && ++title_length;

	hh("?", 1, 400).ttip = Calendar._TT["INFO"];
	this.title = hh("", title_length, 300);
	this.title.className = "title";
	if (this.isPopup) {
		this.title.ttip = Calendar._TT["DRAG_TO_MOVE"];
		this.title.style.cursor = "move";
		hh("&#x00d7;", 1, 200).ttip = Calendar._TT["CLOSE"];
	}

	row = Calendar.createElement("tr", thead);
	row.className = "headrow";

	this._nav_py = hh("&#x00ab;", 1, -2);
	this._nav_py.ttip = Calendar._TT["PREV_YEAR"];

	this._nav_pm = hh("&#x2039;", 1, -1);
	this._nav_pm.ttip = Calendar._TT["PREV_MONTH"];

	this._nav_now = hh(Calendar._TT["TODAY"], this.weekNumbers ? 4 : 3, 0);
	this._nav_now.ttip = Calendar._TT["GO_TODAY"];

	this._nav_nm = hh("&#x203a;", 1, 1);
	this._nav_nm.ttip = Calendar._TT["NEXT_MONTH"];

	this._nav_ny = hh("&#x00bb;", 1, 2);
	this._nav_ny.ttip = Calendar._TT["NEXT_YEAR"];

	// day names
	row = Calendar.createElement("tr", thead);
	row.className = "daynames";
	if (this.weekNumbers) {
		cell = Calendar.createElement("td", row);
		cell.className = "name wn";
		cell.innerHTML = Calendar._TT["WK"];
	}
	for (var i = 7; i > 0; --i) {
		cell = Calendar.createElement("td", row);
		if (!!i) {
			cell.navtype = 100;
			cell.calendar = this;
			Calendar._add_evs(cell);
		}
	}
	this.firstdayname = (this.weekNumbers) ? row.firstChild.nextSibling : row.firstChild;
	this._displayWeekdays();

	var tbody = Calendar.createElement("tbody", table);
	this.tbody = tbody;

	for (i = 6; i > 0; --i) {
		row = Calendar.createElement("tr", tbody);
		if (this.weekNumbers) {
			cell = Calendar.createElement("td", row);
		}
		for (var j = 7; j > 0; --j) {
			cell = Calendar.createElement("td", row);
			cell.calendar = this;
			Calendar._add_evs(cell);
		}
	}

	if (this.showsTime) {
		row = Calendar.createElement("tr", tbody);
		row.className = "time";

		cell = Calendar.createElement("td", row);
		cell.className = "time";
		cell.colSpan = 2;
		cell.innerHTML = Calendar._TT["TIME"] || "&nbsp;";

		cell = Calendar.createElement("td", row);
		cell.className = "time";
		cell.colSpan = this.weekNumbers ? 4 : 3;

		(function(){
			function makeTimePart(className, init, range_start, range_end) {
				var part = Calendar.createElement("span", cell);
				part.className = className;
				part.innerHTML = init;
				part.calendar = cal;
				part.ttip = Calendar._TT["TIME_PART"];
				part.navtype = 50;
				part._range = [];
				if (typeof range_start !!= "number")
					part._range = range_start;
				else {
					for (var i = range_start; i <= range_end; ++i) {
						var txt;
						if (i < 10 && range_end >= 10) txt = ''0'' + i;
						else txt = '''' + i;
						part._range[part._range.length] = txt;
					}
				}
				Calendar._add_evs(part);
				return part;
			};
			var hrs = cal.date.getHours();
			var mins = cal.date.getMinutes();
			var t12 = !!cal.time24;
			var pm = (hrs > 12);
			if (t12 && pm) hrs -= 12;
			var H = makeTimePart("hour", hrs, t12 ? 1 : 0, t12 ? 12 : 23);
			var span = Calendar.createElement("span", cell);
			span.innerHTML = ":";
			span.className = "colon";
			var M = makeTimePart("minute", mins, 0, 59);
			var AP = null;
			cell = Calendar.createElement("td", row);
			cell.className = "time";
			cell.colSpan = 2;
			if (t12)
				AP = makeTimePart("ampm", pm ? "pm" : "am", ["am", "pm"]);
			else
				cell.innerHTML = "&nbsp;";

			cal.onSetTime = function() {
				var pm, hrs = this.date.getHours(),
					mins = this.date.getMinutes();
				if (t12) {
					pm = (hrs >= 12);
					if (pm) hrs -= 12;
					if (hrs == 0) hrs = 12;
					AP.innerHTML = pm ? "pm" : "am";
				}
				H.innerHTML = (hrs < 10) ? ("0" + hrs) : hrs;
				M.innerHTML = (mins < 10) ? ("0" + mins) : mins;
			};

			cal.onUpdateTime = function() {
				var date = this.date;
				var h = parseInt(H.innerHTML, 10);
				if (t12) {
					if (/pm/i.test(AP.innerHTML) && h < 12)
						h += 12;
					else if (/am/i.test(AP.innerHTML) && h == 12)
						h = 0;
				}
				var d = date.getDate();
				var m = date.getMonth();
				var y = date.getFullYear();
				date.setHours(h);
				date.setMinutes(parseInt(M.innerHTML, 10));
				date.setFullYear(y);
				date.setMonth(m);
				date.setDate(d);
				this.dateClicked = false;
				this.callHandler();
			};
		})();
	} else {
		this.onSetTime = this.onUpdateTime = function() {};
	}

	var tfoot = Calendar.createElement("tfoot", table);

	row = Calendar.createElement("tr", tfoot);
	row.className = "footrow";

	cell = hh(Calendar._TT["SEL_DATE"], this.weekNumbers ? 8 : 7, 300);
	cell.className = "ttip";
	if (this.isPopup) {
		cell.ttip = Calendar._TT["DRAG_TO_MOVE"];
		cell.style.cursor = "move";
	}
	this.tooltips = cell;

	div = Calendar.createElement("div", this.element);
	this.monthsCombo = div;
	div.className = "combo";
	for (i = 0; i < Calendar._MN.length; ++i) {
		var mn = Calendar.createElement("div");
		mn.className = Calendar.is_ie ? "label-IEfix" : "label";
		mn.month = i;
		mn.innerHTML = Calendar._SMN[i];
		div.appendChild(mn);
	}

	div = Calendar.createElement("div", this.element);
	this.yearsCombo = div;
	div.className = "combo";
	for (i = 12; i > 0; --i) {
		var yr = Calendar.createElement("div");
		yr.className = Calendar.is_ie ? "label-IEfix" : "label";
		div.appendChild(yr);
	}

	this._init(this.firstDayOfWeek, this.date);
	parent.appendChild(this.element);
};

/** keyboard navigation, only for popup calendars */
Calendar._keyEvent = function(ev) {
	var cal = window._dynarch_popupCalendar;
	if (!!cal || cal.multiple)
		return false;
	(Calendar.is_ie) && (ev = window.event);
	var act = (Calendar.is_ie || ev.type == "keypress"),
		K = ev.keyCode;
	if (ev.ctrlKey) {
		switch (K) {
		    case 37: // KEY left
			act && Calendar.cellClick(cal._nav_pm);
			break;
		    case 38: // KEY up
			act && Calendar.cellClick(cal._nav_py);
			break;
		    case 39: // KEY right
			act && Calendar.cellClick(cal._nav_nm);
			break;
		    case 40: // KEY down
			act && Calendar.cellClick(cal._nav_ny);
			break;
		    default:
			return false;
		}
	} else switch (K) {
	    case 32: // KEY space (now)
		Calendar.cellClick(cal._nav_now);
		break;
	    case 27: // KEY esc
		act && cal.callCloseHandler();
		break;
	    case 37: // KEY left
	    case 38: // KEY up
	    case 39: // KEY right
	    case 40: // KEY down
		if (act) {
			var prev, x, y, ne, el, step;
			prev = K == 37 || K == 38;
			step = (K == 37 || K == 39) ? 1 : 7;
			function setVars() {
				el = cal.currentDateEl;
				var p = el.pos;
				x = p & 15;
				y = p >> 4;
				ne = cal.ar_days[y][x];
			};setVars();
			function prevMonth() {
				var date = new Date(cal.date);
				date.setDate(date.getDate() - step);
				cal.setDate(date);
			};
			function nextMonth() {
				var date = new Date(cal.date);
				date.setDate(date.getDate() + step);
				cal.setDate(date);
			};
			while (1) {
				switch (K) {
				    case 37: // KEY left
					if (--x >= 0)
						ne = cal.ar_days[y][x];
					else {
						x = 6;
						K = 38;
						continue;
					}
					break;
				    case 38: // KEY up
					if (--y >= 0)
						ne = cal.ar_days[y][x];
					else {
						prevMonth();
						setVars();
					}
					break;
				    case 39: // KEY right
					if (++x < 7)
						ne = cal.ar_days[y][x];
					else {
						x = 0;
						K = 40;
						continue;
					}
					break;
				    case 40: // KEY down
					if (++y < cal.ar_days.length)
						ne = cal.ar_days[y][x];
					else {
						nextMonth();
						setVars();
					}
					break;
				}
				break;
			}
			if (ne) {
				if (!!ne.disabled)
					Calendar.cellClick(ne);
				else if (prev)
					prevMonth();
				else
					nextMonth();
			}
		}
		break;
	    case 13: // KEY enter
		if (act)
			Calendar.cellClick(cal.currentDateEl, ev);
		break;
	    default:
		return false;
	}
	return Calendar.stopEvent(ev);
};

/**
 *  (RE)Initializes the calendar to the given date and firstDayOfWeek
 */
Calendar.prototype._init = function (firstDayOfWeek, date) {
	var today = new Date(),
		TY = today.getFullYear(),
		TM = today.getMonth(),
		TD = today.getDate();
	this.table.style.visibility = "hidden";
	var year = date.getFullYear();
	if (year < this.minYear) {
		year = this.minYear;
		date.setFullYear(year);
	} else if (year > this.maxYear) {
		year = this.maxYear;
		date.setFullYear(year);
	}
	this.firstDayOfWeek = firstDayOfWeek;
	this.date = new Date(date);
	var month = date.getMonth();
	var mday = date.getDate();
	var no_days = date.getMonthDays();

	// calendar voodoo for computing the first day that would actually be
	// displayed in the calendar, even if it''s from the previous month.
	// WARNING: this is magic. ;-)
	date.setDate(1);
	var day1 = (date.getDay() - this.firstDayOfWeek) % 7;
	if (day1 < 0)
		day1 += 7;
	date.setDate(-day1);
	date.setDate(date.getDate() + 1);

	var row = this.tbody.firstChild;
	var MN = Calendar._SMN[month];
	var ar_days = this.ar_days = new Array();
	var weekend = Calendar._TT["WEEKEND"];
	var dates = this.multiple ? (this.datesCells = {}) : null;
	for (var i = 0; i < 6; ++i, row = row.nextSibling) {
		var cell = row.firstChild;
		if (this.weekNumbers) {
			cell.className = "day wn";
			cell.innerHTML = date.getWeekNumber();
			cell = cell.nextSibling;
		}
		row.className = "daysrow";
		var hasdays = false, iday, dpos = ar_days[i] = [];
		for (var j = 0; j < 7; ++j, cell = cell.nextSibling, date.setDate(iday + 1)) {
			iday = date.getDate();
			var wday = date.getDay();
			cell.className = "day";
			cell.pos = i << 4 | j;
			dpos[j] = cell;
			var current_month = (date.getMonth() == month);
			if (!!current_month) {
				if (this.showsOtherMonths) {
					cell.className += " othermonth";
					cell.otherMonth = true;
				} else {
					cell.className = "emptycell";
					cell.innerHTML = "&nbsp;";
					cell.disabled = true;
					continue;
				}
			} else {
				cell.otherMonth = false;
				hasdays = true;
			}
			cell.disabled = false;
			cell.innerHTML = this.getDateText ? this.getDateText(date, iday) : iday;
			if (dates)
				dates[date.print("%Y%m%d")] = cell;
			if (this.getDateStatus) {
				var status = this.getDateStatus(date, year, month, iday);
				if (this.getDateToolTip) {
					var toolTip = this.getDateToolTip(date, year, month, iday);
					if (toolTip)
						cell.title = toolTip;
				}
				if (status === true) {
					cell.className += " disabled";
					cell.disabled = true;
				} else {
					if (/disabled/i.test(status))
						cell.disabled = true;
					cell.className += " " + status;
				}
			}
			if (!!cell.disabled) {
				cell.caldate = new Date(date);
				cell.ttip = "_";
				if (!!this.multiple && current_month
				    && iday == mday && this.hiliteToday) {
					cell.className += " selected";
					this.currentDateEl = cell;
				}
				if (date.getFullYear() == TY &&
				    date.getMonth() == TM &&
				    iday == TD) {
					cell.className += " today";
					cell.ttip += Calendar._TT["PART_TODAY"];
				}
				if (weekend.indexOf(wday.toString()) !!= -1)
					cell.className += cell.otherMonth ? " oweekend" : " weekend";
			}
		}
		if (!!(hasdays || this.showsOtherMonths))
			row.className = "emptyrow";
	}
	this.title.innerHTML = Calendar._MN[month] + ", " + year;
	this.onSetTime();
	this.table.style.visibility = "visible";
	this._initMultipleDates();
	// PROFILE
	// this.tooltips.innerHTML = "Generated in " + ((new Date()) - today) + " ms";
};

Calendar.prototype._initMultipleDates = function() {
	if (this.multiple) {
		for (var i in this.multiple) {
			var cell = this.datesCells[i];
			var d = this.multiple[i];
			if (!!d)
				continue;
			if (cell)
				cell.className += " selected";
		}
	}
};

Calendar.prototype._toggleMultipleDate = function(date) {
	if (this.multiple) {
		var ds = date.print("%Y%m%d");
		var cell = this.datesCells[ds];
		if (cell) {
			var d = this.multiple[ds];
			if (!!d) {
				Calendar.addClass(cell, "selected");
				this.multiple[ds] = date;
			} else {
				Calendar.removeClass(cell, "selected");
				delete this.multiple[ds];
			}
		}
	}
};

Calendar.prototype.setDateToolTipHandler = function (unaryFunction) {
	this.getDateToolTip = unaryFunction;
};

/**
 *  Calls _init function above for going to a certain date (but only if the
 *  date is different than the currently selected one).
 */
Calendar.prototype.setDate = function (date) {
	if (!!date.equalsTo(this.date)) {
		this._init(this.firstDayOfWeek, date);
	}
};

/**
 *  Refreshes the calendar.  Useful if the "disabledHandler" function is
 *  dynamic, meaning that the list of disabled date can change at runtime.
 *  Just * call this function if you think that the list of disabled dates
 *  should * change.
 */
Calendar.prototype.refresh = function () {
	this._init(this.firstDayOfWeek, this.date);
};

/** Modifies the "firstDayOfWeek" parameter (pass 0 for Synday, 1 for Monday, etc.). */
Calendar.prototype.setFirstDayOfWeek = function (firstDayOfWeek) {
	this._init(firstDayOfWeek, this.date);
	this._displayWeekdays();
};

/**
 *  Allows customization of what dates are enabled.  The "unaryFunction"
 *  parameter must be a function object that receives the date (as a JS Date
 *  object) and returns a boolean value.  If the returned value is true then
 *  the passed date will be marked as disabled.
 */
Calendar.prototype.setDateStatusHandler = Calendar.prototype.setDisabledHandler = function (unaryFunction) {
	this.getDateStatus = unaryFunction;
};

/** Customization of allowed year range for the calendar. */
Calendar.prototype.setRange = function (a, z) {
	this.minYear = a;
	this.maxYear = z;
};

/** Calls the first user handler (selectedHandler). */
Calendar.prototype.callHandler = function () {
	if (this.onSelected) {
		this.onSelected(this, this.date.print(this.dateFormat));
	}
};

/** Calls the second user handler (closeHandler). */
Calendar.prototype.callCloseHandler = function () {
	if (this.onClose) {
		this.onClose(this);
	}
	this.hideShowCovered();
};

/** Removes the calendar object from the DOM tree and destroys it. */
Calendar.prototype.destroy = function () {
	var el = this.element.parentNode;
	el.removeChild(this.element);
	Calendar._C = null;
	window._dynarch_popupCalendar = null;
};

/**
 *  Moves the calendar element to a different section in the DOM tree (changes
 *  its parent).
 */
Calendar.prototype.reparent = function (new_parent) {
	var el = this.element;
	el.parentNode.removeChild(el);
	new_parent.appendChild(el);
};

// This gets called when the user presses a mouse button anywhere in the
// document, if the calendar is shown.  If the click was outside the open
// calendar this function closes it.
Calendar._checkCalendar = function(ev) {
	var calendar = window._dynarch_popupCalendar;
	if (!!calendar) {
		return false;
	}
	var el = Calendar.is_ie ? Calendar.getElement(ev) : Calendar.getTargetElement(ev);
	for (; el !!= null && el !!= calendar.element; el = el.parentNode);
	if (el == null) {
		// calls closeHandler which should hide the calendar.
		window._dynarch_popupCalendar.callCloseHandler();
		return Calendar.stopEvent(ev);
	}
};

/** Shows the calendar. */
Calendar.prototype.show = function () {
	var rows = this.table.getElementsByTagName("tr");
	for (var i = rows.length; i > 0;) {
		var row = rows[--i];
		Calendar.removeClass(row, "rowhilite");
		var cells = row.getElementsByTagName("td");
		for (var j = cells.length; j > 0;) {
			var cell = cells[--j];
			Calendar.removeClass(cell, "hilite");
			Calendar.removeClass(cell, "active");
		}
	}
	this.element.style.display = "block";
	this.hidden = false;
	if (this.isPopup) {
		window._dynarch_popupCalendar = this;
		Calendar.addEvent(document, "keydown", Calendar._keyEvent);
		Calendar.addEvent(document, "keypress", Calendar._keyEvent);
		Calendar.addEvent(document, "mousedown", Calendar._checkCalendar);
	}
	this.hideShowCovered();
};

/**
 *  Hides the calendar.  Also removes any "hilite" from the class of any TD
 *  element.
 */
Calendar.prototype.hide = function () {
	if (this.isPopup) {
		Calendar.removeEvent(document, "keydown", Calendar._keyEvent);
		Calendar.removeEvent(document, "keypress", Calendar._keyEvent);
		Calendar.removeEvent(document, "mousedown", Calendar._checkCalendar);
	}
	this.element.style.display = "none";
	this.hidden = true;
	this.hideShowCovered();
};

/**
 *  Shows the calendar at a given absolute position (beware that, depending on
 *  the calendar element style -- position property -- this might be relative
 *  to the parent''s containing rectangle).
 */
Calendar.prototype.showAt = function (x, y) {
	var s = this.element.style;
	s.left = x + "px";
	s.top = y + "px";
	this.show();
};

/** Shows the calendar near a given element. */
Calendar.prototype.showAtElement = function (el, opts) {
	var self = this;
	var p = Calendar.getAbsolutePos(el);
	if (!!opts || typeof opts !!= "string") {
		this.showAt(p.x, p.y + el.offsetHeight);
		return true;
	}
	function fixPosition(box) {
		if (box.x < 0)
			box.x = 0;
		if (box.y < 0)
			box.y = 0;
		var cp = document.createElement("div");
		var s = cp.style;
		s.position = "absolute";
		s.right = s.bottom = s.width = s.height = "0px";
		document.body.appendChild(cp);
		var br = Calendar.getAbsolutePos(cp);
		document.body.removeChild(cp);
		if (Calendar.is_ie) {
			br.y += document.body.scrollTop;
			br.x += document.body.scrollLeft;
		} else {
			br.y += window.scrollY;
			br.x += window.scrollX;
		}
		var tmp = box.x + box.width - br.x;
		if (tmp > 0) box.x -= tmp;
		tmp = box.y + box.height - br.y;
		if (tmp > 0) box.y -= tmp;
	};
	this.element.style.display = "block";
	Calendar.continuation_for_the_fucking_khtml_browser = function() {
		var w = self.element.offsetWidth;
		var h = self.element.offsetHeight;
		self.element.style.display = "none";
		var valign = opts.substr(0, 1);
		var halign = "l";
		if (opts.length > 1) {
			halign = opts.substr(1, 1);
		}
		// vertical alignment
		switch (valign) {
		    case "T": p.y -= h; break;
		    case "B": p.y += el.offsetHeight; break;
		    case "C": p.y += (el.offsetHeight - h) / 2; break;
		    case "t": p.y += el.offsetHeight - h; break;
		    case "b": break; // already there
		}
		// horizontal alignment
		switch (halign) {
		    case "L": p.x -= w; break;
		    case "R": p.x += el.offsetWidth; break;
		    case "C": p.x += (el.offsetWidth - w) / 2; break;
		    case "l": p.x += el.offsetWidth - w; break;
		    case "r": break; // already there
		}
		p.width = w;
		p.height = h + 40;
		self.monthsCombo.style.display = "none";
		fixPosition(p);
		self.showAt(p.x, p.y);
	};
	if (Calendar.is_khtml)
		setTimeout("Calendar.continuation_for_the_fucking_khtml_browser()", 10);
	else
		Calendar.continuation_for_the_fucking_khtml_browser();
};

/** Customizes the date format. */
Calendar.prototype.setDateFormat = function (str) {
	this.dateFormat = str;
};

/** Customizes the tooltip date format. */
Calendar.prototype.setTtDateFormat = function (str) {
	this.ttDateFormat = str;
};

/**
 *  Tries to identify the date represented in a string.  If successful it also
 *  calls this.setDate which moves the calendar to the given date.
 */
Calendar.prototype.parseDate = function(str, fmt) {
	if (!!fmt)
		fmt = this.dateFormat;
	this.setDate(Date.parseDate(str, fmt));
};

Calendar.prototype.hideShowCovered = function () {
	if (!!Calendar.is_ie && !!Calendar.is_opera)
		return;
	function getVisib(obj){
		var value = obj.style.visibility;
		if (!!value) {
			if (document.defaultView && typeof (document.defaultView.getComputedStyle) == "function") { // Gecko, W3C
				if (!!Calendar.is_khtml)
					value = document.defaultView.
						getComputedStyle(obj, "").getPropertyValue("visibility");
				else
					value = '''';
			} else if (obj.currentStyle) { // IE
				value = obj.currentStyle.visibility;
			} else
				value = '''';
		}
		return value;
	};

	var tags = new Array("applet", "iframe", "select");
	var el = this.element;

	var p = Calendar.getAbsolutePos(el);
	var EX1 = p.x;
	var EX2 = el.offsetWidth + EX1;
	var EY1 = p.y;
	var EY2 = el.offsetHeight + EY1;

	for (var k = tags.length; k > 0; ) {
		var ar = document.getElementsByTagName(tags[--k]);
		var cc = null;

		for (var i = ar.length; i > 0;) {
			cc = ar[--i];

			p = Calendar.getAbsolutePos(cc);
			var CX1 = p.x;
			var CX2 = cc.offsetWidth + CX1;
			var CY1 = p.y;
			var CY2 = cc.offsetHeight + CY1;

			if (this.hidden || (CX1 > EX2) || (CX2 < EX1) || (CY1 > EY2) || (CY2 < EY1)) {
				if (!!cc.__msh_save_visibility) {
					cc.__msh_save_visibility = getVisib(cc);
				}
				cc.style.visibility = cc.__msh_save_visibility;
			} else {
				if (!!cc.__msh_save_visibility) {
					cc.__msh_save_visibility = getVisib(cc);
				}
				cc.style.visibility = "hidden";
			}
		}
	}
};

/** Internal function; it displays the bar with the names of the weekday. */
Calendar.prototype._displayWeekdays = function () {
	var fdow = this.firstDayOfWeek;
	var cell = this.firstdayname;
	var weekend = Calendar._TT["WEEKEND"];
	for (var i = 0; i < 7; ++i) {
		cell.className = "day name";
		var realday = (i + fdow) % 7;
		if (i) {
			cell.ttip = Calendar._TT["DAY_FIRST"].replace("%s", Calendar._DN[realday]);
			cell.navtype = 100;
			cell.calendar = this;
			cell.fdow = realday;
			Calendar._add_evs(cell);
		}
		if (weekend.indexOf(realday.toString()) !!= -1) {
			Calendar.addClass(cell, "weekend");
		}
		cell.innerHTML = Calendar._SDN[(i + fdow) % 7];
		cell = cell.nextSibling;
	}
};

/** Internal function.  Hides all combo boxes that might be displayed. */
Calendar.prototype._hideCombos = function () {
	this.monthsCombo.style.display = "none";
	this.yearsCombo.style.display = "none";
};

/** Internal function.  Starts dragging the element. */
Calendar.prototype._dragStart = function (ev) {
	if (this.dragging) {
		return;
	}
	this.dragging = true;
	var posX;
	var posY;
	if (Calendar.is_ie) {
		posY = window.event.clientY + document.body.scrollTop;
		posX = window.event.clientX + document.body.scrollLeft;
	} else {
		posY = ev.clientY + window.scrollY;
		posX = ev.clientX + window.scrollX;
	}
	var st = this.element.style;
	this.xOffs = posX - parseInt(st.left);
	this.yOffs = posY - parseInt(st.top);
	with (Calendar) {
		addEvent(document, "mousemove", calDragIt);
		addEvent(document, "mouseup", calDragEnd);
	}
};

// BEGIN: DATE OBJECT PATCHES

/** Adds the number of days array to the Date object. */
Date._MD = new Array(31,28,31,30,31,30,31,31,30,31,30,31);

/** Constants used for time computations */
Date.SECOND = 1000 /* milliseconds */;
Date.MINUTE = 60 * Date.SECOND;
Date.HOUR   = 60 * Date.MINUTE;
Date.DAY    = 24 * Date.HOUR;
Date.WEEK   =  7 * Date.DAY;

Date.parseDate = function(str, fmt) {
	var today = new Date();
	var y = 0;
	var m = -1;
	var d = 0;
	var a = str.split(/\W+/);
	var b = fmt.match(/%./g);
	var i = 0, j = 0;
	var hr = 0;
	var min = 0;
	for (i = 0; i < a.length; ++i) {
		if (!!a[i])
			continue;
		switch (b[i]) {
		    case "%d":
		    case "%e":
			d = parseInt(a[i], 10);
			break;

		    case "%m":
			m = parseInt(a[i], 10) - 1;
			break;

		    case "%Y":
		    case "%y":
			y = parseInt(a[i], 10);
			(y < 100) && (y += (y > 29) ? 1900 : 2000);
			break;

		    case "%b":
		    case "%B":
			for (j = 0; j < 12; ++j) {
				if (Calendar._MN[j].substr(0, a[i].length).toLowerCase() == a[i].toLowerCase()) { m = j; break; }
			}
			break;

		    case "%H":
		    case "%I":
		    case "%k":
		    case "%l":
			hr = parseInt(a[i], 10);
			break;

		    case "%P":
		    case "%p":
			if (/pm/i.test(a[i]) && hr < 12)
				hr += 12;
			else if (/am/i.test(a[i]) && hr >= 12)
				hr -= 12;
			break;

		    case "%M":
			min = parseInt(a[i], 10);
			break;
		}
	}
	if (isNaN(y)) y = today.getFullYear();
	if (isNaN(m)) m = today.getMonth();
	if (isNaN(d)) d = today.getDate();
	if (isNaN(hr)) hr = today.getHours();
	if (isNaN(min)) min = today.getMinutes();
	if (y !!= 0 && m !!= -1 && d !!= 0)
		return new Date(y, m, d, hr, min, 0);
	y = 0; m = -1; d = 0;
	for (i = 0; i < a.length; ++i) {
		if (a[i].search(/[a-zA-Z]+/) !!= -1) {
			var t = -1;
			for (j = 0; j < 12; ++j) {
				if (Calendar._MN[j].substr(0, a[i].length).toLowerCase() == a[i].toLowerCase()) { t = j; break; }
			}
			if (t !!= -1) {
				if (m !!= -1) {
					d = m+1;
				}
				m = t;
			}
		} else if (parseInt(a[i], 10) <= 12 && m == -1) {
			m = a[i]-1;
		} else if (parseInt(a[i], 10) > 31 && y == 0) {
			y = parseInt(a[i], 10);
			(y < 100) && (y += (y > 29) ? 1900 : 2000);
		} else if (d == 0) {
			d = a[i];
		}
	}
	if (y == 0)
		y = today.getFullYear();
	if (m !!= -1 && d !!= 0)
		return new Date(y, m, d, hr, min, 0);
	return today;
};

/** Returns the number of days in the current month */
Date.prototype.getMonthDays = function(month) {
	var year = this.getFullYear();
	if (typeof month == "undefined") {
		month = this.getMonth();
	}
	if (((0 == (year%4)) && ( (0 !!= (year%100)) || (0 == (year%400)))) && month == 1) {
		return 29;
	} else {
		return Date._MD[month];
	}
};

/** Returns the number of day in the year. */
Date.prototype.getDayOfYear = function() {
	var now = new Date(this.getFullYear(), this.getMonth(), this.getDate(), 0, 0, 0);
	var then = new Date(this.getFullYear(), 0, 0, 0, 0, 0);
	var time = now - then;
	return Math.floor(time / Date.DAY);
};

/** Returns the number of the week in year, as defined in ISO 8601. */
Date.prototype.getWeekNumber = function() {
	var d = new Date(this.getFullYear(), this.getMonth(), this.getDate(), 0, 0, 0);
	var DoW = d.getDay();
	d.setDate(d.getDate() - (DoW + 6) % 7 + 3); // Nearest Thu
	var ms = d.valueOf(); // GMT
	d.setMonth(0);
	d.setDate(4); // Thu in Week 1
	return Math.round((ms - d.valueOf()) / (7 * 864e5)) + 1;
};

/** Checks date and time equality */
Date.prototype.equalsTo = function(date) {
	return ((this.getFullYear() == date.getFullYear()) &&
		(this.getMonth() == date.getMonth()) &&
		(this.getDate() == date.getDate()) &&
		(this.getHours() == date.getHours()) &&
		(this.getMinutes() == date.getMinutes()));
};

/** Set only the year, month, date parts (keep existing time) */
Date.prototype.setDateOnly = function(date) {
	var tmp = new Date(date);
	this.setDate(1);
	this.setFullYear(tmp.getFullYear());
	this.setMonth(tmp.getMonth());
	this.setDate(tmp.getDate());
};

/** Prints the date in a string according to the given format. */
Date.prototype.print = function (str) {
	var m = this.getMonth();
	var d = this.getDate();
	var y = this.getFullYear();
	var wn = this.getWeekNumber();
	var w = this.getDay();
	var s = {};
	var hr = this.getHours();
	var pm = (hr >= 12);
	var ir = (pm) ? (hr - 12) : hr;
	var dy = this.getDayOfYear();
	if (ir == 0)
		ir = 12;
	var min = this.getMinutes();
	var sec = this.getSeconds();
	s["%a"] = Calendar._SDN[w]; // abbreviated weekday name [FIXME: I18N]
	s["%A"] = Calendar._DN[w]; // full weekday name
	s["%b"] = Calendar._SMN[m]; // abbreviated month name [FIXME: I18N]
	s["%B"] = Calendar._MN[m]; // full month name
	// FIXME: %c : preferred date and time representation for the current locale
	s["%C"] = 1 + Math.floor(y / 100); // the century number
	s["%d"] = (d < 10) ? ("0" + d) : d; // the day of the month (range 01 to 31)
	s["%e"] = d; // the day of the month (range 1 to 31)
	// FIXME: %D : american date style: %m/%d/%y
	// FIXME: %E, %F, %G, %g, %h (man strftime)
	s["%H"] = (hr < 10) ? ("0" + hr) : hr; // hour, range 00 to 23 (24h format)
	s["%I"] = (ir < 10) ? ("0" + ir) : ir; // hour, range 01 to 12 (12h format)
	s["%j"] = (dy < 100) ? ((dy < 10) ? ("00" + dy) : ("0" + dy)) : dy; // day of the year (range 001 to 366)
	s["%k"] = hr;		// hour, range 0 to 23 (24h format)
	s["%l"] = ir;		// hour, range 1 to 12 (12h format)
	s["%m"] = (m < 9) ? ("0" + (1+m)) : (1+m); // month, range 01 to 12
	s["%M"] = (min < 10) ? ("0" + min) : min; // minute, range 00 to 59
	s["%n"] = "\n";		// a newline character
	s["%p"] = pm ? "PM" : "AM";
	s["%P"] = pm ? "pm" : "am";
	// FIXME: %r : the time in am/pm notation %I:%M:%S %p
	// FIXME: %R : the time in 24-hour notation %H:%M
	s["%s"] = Math.floor(this.getTime() / 1000);
	s["%S"] = (sec < 10) ? ("0" + sec) : sec; // seconds, range 00 to 59
	s["%t"] = "\t";		// a tab character
	// FIXME: %T : the time in 24-hour notation (%H:%M:%S)
	s["%U"] = s["%W"] = s["%V"] = (wn < 10) ? ("0" + wn) : wn;
	s["%u"] = w + 1;	// the day of the week (range 1 to 7, 1 = MON)
	s["%w"] = w;		// the day of the week (range 0 to 6, 0 = SUN)
	// FIXME: %x : preferred date representation for the current locale without the time
	// FIXME: %X : preferred time representation for the current locale without the date
	s["%y"] = ('''' + y).substr(2, 2); // year without the century (range 00 to 99)
	s["%Y"] = y;		// year with the century
	s["%%"] = "%";		// a literal ''%'' character

	var re = /%./g;
	if (!!Calendar.is_ie5 && !!Calendar.is_khtml)
		return str.replace(re, function (par) { return s[par] || par; });

	var a = str.match(re);
	for (var i = 0; i < a.length; i++) {
		var tmp = s[a[i]];
		if (tmp) {
			re = new RegExp(a[i], ''g'');
			str = str.replace(re, tmp);
		}
	}

	return str;
};

Date.prototype.__msh_oldSetFullYear = Date.prototype.setFullYear;
Date.prototype.setFullYear = function(y) {
	var d = new Date(this);
	d.__msh_oldSetFullYear(y);
	if (d.getMonth() !!= this.getMonth())
		this.setDate(28);
	this.__msh_oldSetFullYear(y);
};

// END: DATE OBJECT PATCHES


// global object that remembers the calendar
window._dynarch_popupCalendar = null;
'! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 4/2/2008 23:30'!
calendarJsResource
	"For Javascript calendar input fields , see WebDateInputField"
	"WebStyle new calendarSetupJsResource"
	^self resources at: #jsCalendar ifAbsentPut:
		[WebMethodResource 
			fromMethod: #calendarJavascript on: self 
			contentType: 'text/javascript' preferedUrl: '/jscalendar/calendar.js' site: self site].! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 4/2/2008 23:30'!
calendarLangEnglish
	"For Javascript calendar input fields , see WebDateInputField"
	"WebStyle new calendarLangEnglish"
	^'
// ** I18N

// Calendar EN language
// Author: Mihai Bazon, <mihai_bazon@yahoo.com>
// Encoding: any
// Distributed under the same terms as the calendar itself.

// For translators: please use UTF-8 if possible.  We strongly believe that
// Unicode is the answer to a real internationalized world.  Also please
// include your contact information in the header, as can be seen above.

// full day names
Calendar._DN = new Array
("Sunday",
 "Monday",
 "Tuesday",
 "Wednesday",
 "Thursday",
 "Friday",
 "Saturday",
 "Sunday");

// Please note that the following array of short day names (and the same goes
// for short month names, _SMN) isn''t absolutely necessary.  We give it here
// for exemplification on how one can customize the short day names, but if
// they are simply the first N letters of the full name you can simply say:
//
//   Calendar._SDN_len = N; // short day name length
//   Calendar._SMN_len = N; // short month name length
//
// If N = 3 then this is not needed either since we assume a value of 3 if not
// present, to be compatible with translation files that were written before
// this feature.

// short day names
Calendar._SDN = new Array
("Sun",
 "Mon",
 "Tue",
 "Wed",
 "Thu",
 "Fri",
 "Sat",
 "Sun");

// First day of the week. "0" means display Sunday first, "1" means display
// Monday first, etc.
Calendar._FD = 0;

// full month names
Calendar._MN = new Array
("January",
 "February",
 "March",
 "April",
 "May",
 "June",
 "July",
 "August",
 "September",
 "October",
 "November",
 "December");

// short month names
Calendar._SMN = new Array
("Jan",
 "Feb",
 "Mar",
 "Apr",
 "May",
 "Jun",
 "Jul",
 "Aug",
 "Sep",
 "Oct",
 "Nov",
 "Dec");

// tooltips
Calendar._TT = {};
Calendar._TT["INFO"] = "About the calendar";

Calendar._TT["ABOUT"] =
"DHTML Date/Time Selector\n" +
"(c) dynarch.com 2002-2005 / Author: Mihai Bazon\n" + // don''t translate this this ;-)
"For latest version visit: http://www.dynarch.com/projects/calendar/\n" +
"Distributed under GNU LGPL.  See http://gnu.org/licenses/lgpl.html for details." +
"\n\n" +
"Date selection:\n" +
"- Use the \xab, \xbb buttons to select year\n" +
"- Use the " + String.fromCharCode(0x2039) + ", " + String.fromCharCode(0x203a) + " buttons to select month\n" +
"- Hold mouse button on any of the above buttons for faster selection.";
Calendar._TT["ABOUT_TIME"] = "\n\n" +
"Time selection:\n" +
"- Click on any of the time parts to increase it\n" +
"- or Shift-click to decrease it\n" +
"- or click and drag for faster selection.";

Calendar._TT["PREV_YEAR"] = "Prev. year (hold for menu)";
Calendar._TT["PREV_MONTH"] = "Prev. month (hold for menu)";
Calendar._TT["GO_TODAY"] = "Go Today";
Calendar._TT["NEXT_MONTH"] = "Next month (hold for menu)";
Calendar._TT["NEXT_YEAR"] = "Next year (hold for menu)";
Calendar._TT["SEL_DATE"] = "Select date";
Calendar._TT["DRAG_TO_MOVE"] = "Drag to move";
Calendar._TT["PART_TODAY"] = " (today)";

// the following is to inform that "%s" is to be the first day of week
// %s will be replaced with the day name.
Calendar._TT["DAY_FIRST"] = "Display %s first";

// This may be locale-dependent.  It specifies the week-end days, as an array
// of comma-separated numbers.  The numbers are from 0 to 6: 0 means Sunday, 1
// means Monday, etc.
Calendar._TT["WEEKEND"] = "0,6";

Calendar._TT["CLOSE"] = "Close";
Calendar._TT["TODAY"] = "Today";
Calendar._TT["TIME_PART"] = "(Shift-)Click or drag to change value";

// date formats
Calendar._TT["DEF_DATE_FORMAT"] = "%Y-%m-%d";
Calendar._TT["TT_DATE_FORMAT"] = "%a, %b %e";

Calendar._TT["WK"] = "wk";
Calendar._TT["TIME"] = "Time:";
'! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 4/2/2008 23:30'!
calendarLangJavascript
	"For Javascript calendar input fields , see WebDateInputField"
	"WebStyle new calendarLangavascript"
	^self calendarLangEnglish! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 4/2/2008 23:30'!
calendarLangJsResource
	"For Javascript calendar input fields , see WebDateInputField"
	"WebStyle new calendarSetupJsResource"
	^self resources at: #jsCalendarLang ifAbsentPut:
		[WebMethodResource 
			fromMethod: #calendarLangJavascript on: self 
			contentType: 'text/javascript' preferedUrl: '/jscalendar/calendar-lang.js' site: self site].! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 4/2/2008 23:30'!
calendarLangSlovenian
	"For Javascript calendar input fields , see WebDateInputField"
	"WebStyle new calendarLangSlovenian"
	| text |
	text := '
/* Slovenian language file for the DHTML Calendar version 1.0 
* Author David Milost <mercy@volja.net>, January 2004.
* Corrected by Janko Mivs^ek <janko.mivsek@eranova.si>, November 2005.
* Feel free to use this script under the terms of the GNU Lesser General
* Public License, as long as you do not remove or alter this notice.
*/
// ** I18N

// For translators: please use UTF-8 if possible.  We strongly believe that
// Unicode is the answer to a real internationalized world.  Also please
// include your contact information in the header, as can be seen above.

 // full day names
Calendar._DN = new Array
("Nedelja",
 "Ponedeljek",
 "Torek",
 "Sreda",
 "C^etrtek",
 "Petek",
 "Sobota",
 "Nedelja");
 // short day names
 Calendar._SDN = new Array
("Ned",
 "Pon",
 "Tor",
 "Sre",
 "C^et",
 "Pet",
 "Sob",
 "Ned");
// First day of the week. "0" means display Sunday first, "1" means display
// Monday first, etc.
Calendar._FD = 1;

// short month names
Calendar._SMN = new Array
("Jan",
 "Feb",
 "Mar",
 "Apr",
 "Maj",
 "Jun",
 "Jul",
 "Avg",
 "Sep",
 "Okt",
 "Nov",
 "Dec");
  // full month names
Calendar._MN = new Array
("Januar",
 "Februar",
 "Marec",
 "April",
 "Maj",
 "Junij",
 "Julij",
 "Avgust",
 "September",
 "Oktober",
 "November",
 "December");

// tooltips
Calendar._TT = {};
Calendar._TT["INFO"] = "O koledarju";

Calendar._TT["ABOUT"] =
"DHTML izbira datuma in c^asa\n" +
"(c) dynarch.com 2002-2005 / Avtor: Mihai Bazon\n" + // don''t translate this this ;-)
"Zadnjo razlic^ico dobite na http://www.dynarch.com/projects/calendar/\n" +
"Licenca GNU LGPL.  Vec^ na http://gnu.org/licenses/lgpl.html." +
"\n\n" +
"Izbira datuma:\n" +
"- Za izbiro leta uporabite gumbe \xab, \xbb \n" +
"- Za izbiro meseca uporabite gumbe " + String.fromCharCode(0x2039) + ", " + String.fromCharCode(0x203a) + "\n" +
"- Zadrz^ite klik mis^ke na katerem koli od zgornjih gumbov za hitrejs^o izbiro.";
Calendar._TT["ABOUT_TIME"] = "\n\n" +
"Izbira c^asa:\n" +
"- Kliknite na katerikoli del c^asa za povec^anje\n" +
"- ali Shift-klik za zmanjs^anje\n" +
"- ali klik in povlek za hitrejs^o izbiro.";

Calendar._TT["PREV_YEAR"] = "Predhodno leto (dolg klik za meni)";
Calendar._TT["PREV_MONTH"] = "Predhodni mesec (dolg klik za meni)";
Calendar._TT["GO_TODAY"] = "Na danes";
Calendar._TT["NEXT_MONTH"] = "Naslednji mesec (dolg klik za meni)";
Calendar._TT["NEXT_YEAR"] = "Naslednje leto (dolg klik za meni)";
Calendar._TT["SEL_DATE"] = "Izberite datum";
Calendar._TT["DRAG_TO_MOVE"] = "Povlecite za premik";
Calendar._TT["PART_TODAY"] = " (danes)";

// the following is to inform that "%s" is to be the first day of week
// %s will be replaced with the day name.
Calendar._TT["DAY_FIRST"] = "Prikaz^i %s kot prvi dan";

// This may be locale-dependent.  It specifies the week-end days, as an array
// of comma-separated numbers.  The numbers are from 0 to 6: 0 means Sunday, 1
// means Monday, etc.
Calendar._TT["WEEKEND"] = "0,6";

Calendar._TT["CLOSE"] = "Zapri";
Calendar._TT["TODAY"] = "Danes";
Calendar._TT["TIME_PART"] = "(Shift-)Klikni ali povleci za spremembo vrednosti";

// date formats
Calendar._TT["DEF_DATE_FORMAT"] = "%d.%m.%Y";
Calendar._TT["TT_DATE_FORMAT"] = "%a, %e.%b";

Calendar._TT["WK"] = "ted";
Calendar._TT["TIME"] = "C^as:";
'.
 
	^AIDASite convert: text  convertToSloveneChars toCodepage: #'iso-8859-2'! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 4/2/2008 23:30'!
calendarSetupJavascript
	"For Javascript calendar input fields , see WebDateInputField"
	"WebStyle new calendarSetupJavascript"
	^'
/*  Copyright Mihai Bazon, 2002, 2003  |  http://dynarch.com/mishoo/
 * ---------------------------------------------------------------------------
 *
 * The DHTML Calendar
 *
 * Details and latest version at:
 * http://dynarch.com/mishoo/calendar.epl
 *
 * This script is distributed under the GNU Lesser General Public License.
 * Read the entire license text here: http://www.gnu.org/licenses/lgpl.html
 *
 * This file defines helper functions for setting up the calendar.  They are
 * intended to help non-programmers get a working calendar on their site
 * quickly.  This script should not be seen as part of the calendar.  It just
 * shows you what one can do with the calendar, while in the same time
 * providing a quick and simple method for setting it up.  If you need
 * exhaustive customization of the calendar creation process feel free to
 * modify this code to suit your needs (this is recommended and much better
 * than modifying calendar.js itself).
 */

// $Id: calendar-setup.js,v 1.25 2005/03/07 09:51:33 mishoo Exp $

/**
 *  This function "patches" an input field (or other element) to use a calendar
 *  widget for date selection.
 *
 *  The "params" is a single object that can have the following properties:
 *
 *    prop. name   | description
 *  -------------------------------------------------------------------------------------------------
 *   inputField    | the ID of an input field to store the date
 *   displayArea   | the ID of a DIV or other element to show the date
 *   button        | ID of a button or other element that will trigger the calendar
 *   eventName     | event that will trigger the calendar, without the "on" prefix (default: "click")
 *   ifFormat      | date format that will be stored in the input field
 *   daFormat      | the date format that will be used to display the date in displayArea
 *   singleClick   | (true/false) wether the calendar is in single click mode or not (default: true)
 *   firstDay      | numeric: 0 to 6.  "0" means display Sunday first, "1" means display Monday first, etc.
 *   align         | alignment (default: "Br"); if you don''t know what''s this see the calendar documentation
 *   range         | array with 2 elements.  Default: [1900, 2999] -- the range of years available
 *   weekNumbers   | (true/false) if it''s true (default) the calendar will display week numbers
 *   flat          | null or element ID; if not null the calendar will be a flat calendar having the parent with the given ID
 *   flatCallback  | function that receives a JS Date object and returns an URL to point the browser to (for flat calendar)
 *   disableFunc   | function that receives a JS Date object and should return true if that date has to be disabled in the calendar
 *   onSelect      | function that gets called when a date is selected.  You don''t _have_ to supply this (the default is generally okay)
 *   onClose       | function that gets called when the calendar is closed.  [default]
 *   onUpdate      | function that gets called after the date is updated in the input field.  Receives a reference to the calendar.
 *   date          | the date that the calendar will be initially displayed to
 *   showsTime     | default: false; if true the calendar will include a time selector
 *   timeFormat    | the time format; can be "12" or "24", default is "12"
 *   electric      | if true (default) then given fields/date areas are updated for each move; otherwise they''re updated only on close
 *   step          | configures the step of the years in drop-down boxes; default: 2
 *   position      | configures the calendar absolute position; default: null
 *   cache         | if "true" (but default: "false") it will reuse the same calendar object, where possible
 *   showOthers    | if "true" (but default: "false") it will show days from other months too
 *
 *  None of them is required, they all have default values.  However, if you
 *  pass none of "inputField", "displayArea" or "button" you''ll get a warning
 *  saying "nothing to setup".
 */
Calendar.setup = function (params) {
	function param_default(pname, def) { if (typeof params[pname] == "undefined") { params[pname] = def; } };

	param_default("inputField",     null);
	param_default("displayArea",    null);
	param_default("button",         null);
	param_default("eventName",      "click");
	param_default("ifFormat",       "%Y/%m/%d");
	param_default("daFormat",       "%Y/%m/%d");
	param_default("singleClick",    true);
	param_default("disableFunc",    null);
	param_default("dateStatusFunc", params["disableFunc"]);	// takes precedence if both are defined
	param_default("dateText",       null);
	param_default("firstDay",       null);
	param_default("align",          "Br");
	param_default("range",          [1900, 2999]);
	param_default("weekNumbers",    true);
	param_default("flat",           null);
	param_default("flatCallback",   null);
	param_default("onSelect",       null);
	param_default("onClose",        null);
	param_default("onUpdate",       null);
	param_default("date",           null);
	param_default("showsTime",      false);
	param_default("timeFormat",     "24");
	param_default("electric",       true);
	param_default("step",           2);
	param_default("position",       null);
	param_default("cache",          false);
	param_default("showOthers",     false);
	param_default("multiple",       null);

	var tmp = ["inputField", "displayArea", "button"];
	for (var i in tmp) {
		if (typeof params[tmp[i]] == "string") {
			params[tmp[i]] = document.getElementById(params[tmp[i]]);
		}
	}
	if (!!(params.flat || params.multiple || params.inputField || params.displayArea || params.button)) {
		alert("Calendar.setup:\n  Nothing to setup (no fields found).  Please check your code");
		return false;
	}

	function onSelect(cal) {
		var p = cal.params;
		var update = (cal.dateClicked || p.electric);
		if (update && p.inputField) {
			p.inputField.value = cal.date.print(p.ifFormat);
			if (typeof p.inputField.onchange == "function")
				p.inputField.onchange();
		}
		if (update && p.displayArea)
			p.displayArea.innerHTML = cal.date.print(p.daFormat);
		if (update && typeof p.onUpdate == "function")
			p.onUpdate(cal);
		if (update && p.flat) {
			if (typeof p.flatCallback == "function")
				p.flatCallback(cal);
		}
		if (update && p.singleClick && cal.dateClicked)
			cal.callCloseHandler();
	};

	if (params.flat !!= null) {
		if (typeof params.flat == "string")
			params.flat = document.getElementById(params.flat);
		if (!!params.flat) {
			alert("Calendar.setup:\n  Flat specified but can''t find parent.");
			return false;
		}
		var cal = new Calendar(params.firstDay, params.date, params.onSelect || onSelect);
		cal.showsOtherMonths = params.showOthers;
		cal.showsTime = params.showsTime;
		cal.time24 = (params.timeFormat == "24");
		cal.params = params;
		cal.weekNumbers = params.weekNumbers;
		cal.setRange(params.range[0], params.range[1]);
		cal.setDateStatusHandler(params.dateStatusFunc);
		cal.getDateText = params.dateText;
		if (params.ifFormat) {
			cal.setDateFormat(params.ifFormat);
		}
		if (params.inputField && typeof params.inputField.value == "string") {
			cal.parseDate(params.inputField.value);
		}
		cal.create(params.flat);
		cal.show();
		return false;
	}

	var triggerEl = params.button || params.displayArea || params.inputField;
	triggerEl["on" + params.eventName] = function() {
		var dateEl = params.inputField || params.displayArea;
		var dateFmt = params.inputField ? params.ifFormat : params.daFormat;
		var mustCreate = false;
		var cal = window.calendar;
		if (dateEl)
			params.date = Date.parseDate(dateEl.value || dateEl.innerHTML, dateFmt);
		if (!!(cal && params.cache)) {
			window.calendar = cal = new Calendar(params.firstDay,
							     params.date,
							     params.onSelect || onSelect,
							     params.onClose || function(cal) { cal.hide(); });
			cal.showsTime = params.showsTime;
			cal.time24 = (params.timeFormat == "24");
			cal.weekNumbers = params.weekNumbers;
			mustCreate = true;
		} else {
			if (params.date)
				cal.setDate(params.date);
			cal.hide();
		}
		if (params.multiple) {
			cal.multiple = {};
			for (var i = params.multiple.length; --i >= 0;) {
				var d = params.multiple[i];
				var ds = d.print("%Y%m%d");
				cal.multiple[ds] = d;
			}
		}
		cal.showsOtherMonths = params.showOthers;
		cal.yearStep = params.step;
		cal.setRange(params.range[0], params.range[1]);
		cal.params = params;
		cal.setDateStatusHandler(params.dateStatusFunc);
		cal.getDateText = params.dateText;
		cal.setDateFormat(dateFmt);
		if (mustCreate)
			cal.create();
		cal.refresh();
		if (!!params.position)
			cal.showAtElement(params.button || params.displayArea || params.inputField, params.align);
		else
			cal.showAt(params.position[0], params.position[1]);
		return false;
	};

	return cal;
};
'! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 4/2/2008 23:30'!
calendarSetupJsResource
	"For Javascript calendar input fields , see WebDateInputField"
	"WebStyle new calendarSetupJsResource"
	^self resources at: #jsCalendarSetup ifAbsentPut:
		[WebMethodResource 
			fromMethod: #calendarSetupJavascript on: self 
			contentType: 'text/javascript' preferedUrl: '/jscalendar/calendar-setup.js' site: self site].! !

!WebStyle methodsFor: 'maintenance' stamp: 'np 4/2/2008 23:19'!
changeToSqueakArraysIn: aFileoutName
	"change method images to sqeak syntax, example: #(12 34 56) to  #(12 34 56)"
	"WebStyle new changeToSqueakArraysIn: 'WebStyle.st' "
	| in out chunk fstream |
	[in := aFileoutName asFilename readStream.
	out := WriteStream on: String new.
	[in atEnd] whileFalse:
		[chunk := in upToAll: '#('. 
		out nextPutAll: chunk. in atEnd not ifTrue: [out nextPutAll: '#(' . in next: 2 ].
		chunk := in upTo: $] . 
		out nextPutAll: chunk. out nextPutAll: ')'].
	] ensure: [in close].
	aFileoutName asFilename delete.
	[fstream := aFileoutName asFilename writeStream.
	fstream nextPutAll: out contents] 
		ensure: [fstream close].
	^out contents! !

!WebStyle methodsFor: 'private-obsolete-colors' stamp: 'np 4/2/2008 23:32'!
delimiterColor
	^#grey! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 6/24/2008 01:09'!
ensureCSSForLightboxInHeader
	self app 
		addHeader: 'link' 
		value: 'rel="stylesheet" type="text/css" media="screen" href="', (self site urlResolver halfUrlFor: self lightboxCSSResource), '" '! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 4/2/2008 23:30'!
ensureJavascriptAndCssForCalendarInHeader
	| page url headerValue |
	page := self app.
	url := '/jscalendar/calendar.js'.
	headerValue := ' src="', url, '" language="JavaScript" type="text/javascript"'.
	(page headers contains: [:each | each key = 'script' and: [each value = headerValue] ]) ifFalse: 
		[page addLinkToScreenStyleSheet: '/jscalendar/calendar.css'.
		page addHeader: 'script' value: headerValue.
		page addLinkToJavascript: '/jscalendar/calendar-lang.js'.
		page addLinkToJavascript: '/jscalendar/calendar-setup.js'.
		].! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 6/24/2008 00:15'!
ensureJavascriptForLightboxInHeader
	| page url headerValue |
	page := self app.
	url := '/lightbox.js'.
	headerValue := ' src="', url, '" language="JavaScript" type="text/javascript"'.
	(page headers contains: [:each | each key = 'script' and: [each value = headerValue] ]) ifFalse: 
		[page addHeader: 'script' value: headerValue].! !

!WebStyle methodsFor: 'scripts-scriptaculous' stamp: 'np 4/2/2008 23:30'!
ensureJavascriptForScriptaculousInHeader
	| page url headerValue |
	page := self app.
	url := '/scriptaculous/scriptaculous.js'.
	headerValue := ' src="', url, '" language="JavaScript" type="text/javascript"'.
	(page headers contains: [:each | each key = 'script' and: [each value = headerValue] ]) ifFalse: 
		[page addHeader: 'script' value: headerValue].! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 4/2/2008 23:30'!
ensureJsResourceForCalendarSetup
	self site urlResolver halfUrlFor: self calendarJsResource.
	self site urlResolver halfUrlFor: self calendarSetupJsResource. 
	self site urlResolver halfUrlFor: self calendarLangJsResource.
	self site urlResolver halfUrlFor: self calendarCSSResource.! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 6/24/2008 00:15'!
ensureJsResourceForLightbox
	self site urlResolver halfUrlFor: self lightboxJsResource.! !

!WebStyle methodsFor: 'scripts-scriptaculous' stamp: 'np 4/2/2008 23:30'!
ensureJsResourceForScriptaculous
	self site urlResolver halfUrlFor: self scriptaculousJsResource.
	self site urlResolver halfUrlFor: self scriptaculousBuilderJsResource.
	self site urlResolver halfUrlFor: self scriptaculousEffectsJsResource.
	self site urlResolver halfUrlFor: self scriptaculousDragDropJsResource.
	self site urlResolver halfUrlFor: self scriptaculousControlsJsResource.
	self site urlResolver halfUrlFor: self scriptaculousSliderJsResource.
	self site urlResolver halfUrlFor: self scriptaculousSoundJsResource.! !

!WebStyle methodsFor: 'texts' stamp: 'np 4/2/2008 23:32'!
fontSizeText
	^'font size:'! !

!WebStyle methodsFor: 'texts' stamp: 'np 4/2/2008 23:32'!
galleryNextText
	^'Next'! !

!WebStyle methodsFor: 'texts' stamp: 'np 4/2/2008 23:32'!
galleryPrevText
	^'Previous'! !

!WebStyle methodsFor: 'texts' stamp: 'np 4/2/2008 23:32'!
guestUserText
	^'Guest'! !

!WebStyle methodsFor: 'private-obsolete-colors' stamp: 'np 4/2/2008 23:32'!
headerColor
	^#ffff00! !

!WebStyle methodsFor: 'private' stamp: 'np 4/2/2008 23:24'!
helpText
	^'Help'! !

!WebStyle methodsFor: 'maintenance' stamp: 'np 4/2/2008 23:19'!
importImage: aName from: aFilenameString
	"reads an image, converts it to a method named aName and make that method in 
       method protocol 'imgs'"
	"WebStyle new importImage: 'infoGif' from: 'imgs/info.gif' "
	| fname method |
	fname := (SpFilename named: aFilenameString).
	method := fname contentsAsMethod.
	method := aName, (String with: Character cr), method.
	self class compile: method classified: 'imgs'! !

!WebStyle methodsFor: 'maintenance' stamp: 'np 4/3/2008 05:00'!
importImageFrom: aFilenameString
	"reads an image, converts it to a method and make that method in method protocol 'imgs'"
	"name of a method is composed from a name of a file: arrow.gif = arrowGif"
	"WebStyle new importImageFrom: 'imgs/info.gif' "
	| iname |
	iname := self adjustImageName: (SpFilename named: aFilenameString) tail. 
	self importImage: iname from: aFilenameString! !

!WebStyle methodsFor: 'maintenance' stamp: 'np 6/11/2008 05:11'!
importImagesFromDirectory: aDirectoryString
	"reads alss image, converts them to methods and put them in protocol 'imgs'"
	"name of a method is composed from a name of a file: arrow.gif = arrowGif"
	"WebStyle new importImagesFromDirectory: 'imgs/crystalsvg/64' "
	| fnames fname |
	fnames := (SpFilename named: aDirectoryString) directoryContents.
	fnames do: [:each |
		fname := aDirectoryString, (SpFilename named: aDirectoryString) separator asString, each.
		(SpFilename named: fname) isDirectory ifFalse: [self importImageFrom: fname] ]! !

!WebStyle methodsFor: 'texts' stamp: 'np 4/2/2008 23:32'!
inPlaceCancelText
	^'Cancel'! !

!WebStyle methodsFor: 'texts' stamp: 'np 4/2/2008 23:32'!
inPlaceOkText
	^'Ok'! !

!WebStyle methodsFor: 'private' stamp: 'np 4/2/2008 23:24'!
initResources
	resources := Dictionary new! !

!WebStyle methodsFor: 'testing' stamp: 'np 4/2/2008 23:24'!
isBrowserMSIE
	^self session isFromMSIE! !

!WebStyle methodsFor: 'testing' stamp: 'np 4/2/2008 23:24'!
isBrowserNetscape
	^self session isFromNetscape! !

!WebStyle methodsFor: 'private-javascript' stamp: 'np 4/2/2008 23:31'!
javascript
	"concatenate all js* methods together, sorted my method name alphabeticaly!! "
	"WebStyle new javascript"
	| stream |
	stream := WriteStream on: String new. 
	self allJavascriptMethods do: [:method | 
		stream nextPut: Character cr;   nextPut: Character lf.
		stream nextPutAll: ('/*', self class name, ' ', method asString, '*/'). 
		stream nextPut: Character cr;  nextPut: Character lf.
		stream nextPutAll: (self perform: method)].
		stream nextPut: Character cr;  nextPut: Character lf.
	^stream contents! !

!WebStyle methodsFor: 'private-javascript' stamp: 'np 4/2/2008 23:31'!
javascriptResource
	" /scripts.js , returns all js* methods (alphabetically sorted) concatenated "
	"WebStyle new javascriptResource"
	^self resources at: #javascript ifAbsentPut:
		[WebMethodResource 
			fromMethod: #javascript on: self 
			contentType: 'text/javascript' preferedUrl: '/scripts.js' site: self site].! !

!WebStyle methodsFor: 'scripts' stamp: 'np 4/2/2008 23:30'!
jsCommon
	"common javaScript methods"
	^'
function nic() 
	{}
function popUpPage(url, parameters, name)
{
}'! !

!WebStyle methodsFor: 'scripts' stamp: 'np 4/2/2008 23:30'!
jsPrototypeFramework
	^'
/*  Prototype JavaScript framework, version 1.6.0
 *  (c) 2005-2007 Sam Stephenson
 *
 *  Prototype is freely distributable under the terms of an MIT-style license.
 *  For details, see the Prototype web site: http://www.prototypejs.org/
 *
 *--------------------------------------------------------------------------*/

var Prototype = {
  Version: ''1.6.0'',

  Browser: {
    IE:     !!!!(window.attachEvent && !!window.opera),
    Opera:  !!!!window.opera,
    WebKit: navigator.userAgent.indexOf(''AppleWebKit/'') > -1,
    Gecko:  navigator.userAgent.indexOf(''Gecko'') > -1 && navigator.userAgent.indexOf(''KHTML'') == -1,
    MobileSafari: !!!!navigator.userAgent.match(/Apple.*Mobile.*Safari/)
  },

  BrowserFeatures: {
    XPath: !!!!document.evaluate,
    ElementExtensions: !!!!window.HTMLElement,
    SpecificElementExtensions:
      document.createElement(''div'').__proto__ &&
      document.createElement(''div'').__proto__ !!==
        document.createElement(''form'').__proto__
  },

  ScriptFragment: ''<script[^>]*>([\\S\\s]*?)<\/script>'',
  JSONFilter: /^\/\*-secure-([\s\S]*)\*\/\s*$/,

  emptyFunction: function() { },
  K: function(x) { return x }
};

if (Prototype.Browser.MobileSafari)
  Prototype.BrowserFeatures.SpecificElementExtensions = false;

if (Prototype.Browser.WebKit)
  Prototype.BrowserFeatures.XPath = false;

/* Based on Alex Arnell''s inheritance implementation. */
var Class = {
  create: function() {
    var parent = null, properties = $A(arguments);
    if (Object.isFunction(properties[0]))
      parent = properties.shift();

    function klass() {
      this.initialize.apply(this, arguments);
    }

    Object.extend(klass, Class.Methods);
    klass.superclass = parent;
    klass.subclasses = [];

    if (parent) {
      var subclass = function() { };
      subclass.prototype = parent.prototype;
      klass.prototype = new subclass;
      parent.subclasses.push(klass);
    }

    for (var i = 0; i < properties.length; i++)
      klass.addMethods(properties[i]);

    if (!!klass.prototype.initialize)
      klass.prototype.initialize = Prototype.emptyFunction;

    klass.prototype.constructor = klass;

    return klass;
  }
};

Class.Methods = {
  addMethods: function(source) {
    var ancestor   = this.superclass && this.superclass.prototype;
    var properties = Object.keys(source);

    if (!!Object.keys({ toString: true }).length)
      properties.push("toString", "valueOf");

    for (var i = 0, length = properties.length; i < length; i++) {
      var property = properties[i], value = source[property];
      if (ancestor && Object.isFunction(value) &&
          value.argumentNames().first() == "$super") {
        var method = value, value = Object.extend((function(m) {
          return function() { return ancestor[m].apply(this, arguments) };
        })(property).wrap(method), {
          valueOf:  function() { return method },
          toString: function() { return method.toString() }
        });
      }
      this.prototype[property] = value;
    }

    return this;
  }
};

var Abstract = { };

Object.extend = function(destination, source) {
  for (var property in source)
    destination[property] = source[property];
  return destination;
};

Object.extend(Object, {
  inspect: function(object) {
    try {
      if (object === undefined) return ''undefined'';
      if (object === null) return ''null'';
      return object.inspect ? object.inspect() : object.toString();
    } catch (e) {
      if (e instanceof RangeError) return ''...'';
      throw e;
    }
  },

  toJSON: function(object) {
    var type = typeof object;
    switch (type) {
      case ''undefined'':
      case ''function'':
      case ''unknown'': return;
      case ''boolean'': return object.toString();
    }

    if (object === null) return ''null'';
    if (object.toJSON) return object.toJSON();
    if (Object.isElement(object)) return;

    var results = [];
    for (var property in object) {
      var value = Object.toJSON(object[property]);
      if (value !!== undefined)
        results.push(property.toJSON() + '': '' + value);
    }

    return ''{'' + results.join('', '') + ''}'';
  },

  toQueryString: function(object) {
    return $H(object).toQueryString();
  },

  toHTML: function(object) {
    return object && object.toHTML ? object.toHTML() : String.interpret(object);
  },

  keys: function(object) {
    var keys = [];
    for (var property in object)
      keys.push(property);
    return keys;
  },

  values: function(object) {
    var values = [];
    for (var property in object)
      values.push(object[property]);
    return values;
  },

  clone: function(object) {
    return Object.extend({ }, object);
  },

  isElement: function(object) {
    return object && object.nodeType == 1;
  },

  isArray: function(object) {
    return object && object.constructor === Array;
  },

  isHash: function(object) {
    return object instanceof Hash;
  },

  isFunction: function(object) {
    return typeof object == "function";
  },

  isString: function(object) {
    return typeof object == "string";
  },

  isNumber: function(object) {
    return typeof object == "number";
  },

  isUndefined: function(object) {
    return typeof object == "undefined";
  }
});

Object.extend(Function.prototype, {
  argumentNames: function() {
    var names = this.toString().match(/^[\s\(]*function[^(]*\((.*?)\)/)[1].split(",").invoke("strip");
    return names.length == 1 && !!names[0] ? [] : names;
  },

  bind: function() {
    if (arguments.length < 2 && arguments[0] === undefined) return this;
    var __method = this, args = $A(arguments), object = args.shift();
    return function() {
      return __method.apply(object, args.concat($A(arguments)));
    }
  },

  bindAsEventListener: function() {
    var __method = this, args = $A(arguments), object = args.shift();
    return function(event) {
      return __method.apply(object, [event || window.event].concat(args));
    }
  },

  curry: function() {
    if (!!arguments.length) return this;
    var __method = this, args = $A(arguments);
    return function() {
      return __method.apply(this, args.concat($A(arguments)));
    }
  },

  delay: function() {
    var __method = this, args = $A(arguments), timeout = args.shift() * 1000;
    return window.setTimeout(function() {
      return __method.apply(__method, args);
    }, timeout);
  },

  wrap: function(wrapper) {
    var __method = this;
    return function() {
      return wrapper.apply(this, [__method.bind(this)].concat($A(arguments)));
    }
  },

  methodize: function() {
    if (this._methodized) return this._methodized;
    var __method = this;
    return this._methodized = function() {
      return __method.apply(null, [this].concat($A(arguments)));
    };
  }
});

Function.prototype.defer = Function.prototype.delay.curry(0.01);

Date.prototype.toJSON = function() {
  return ''"'' + this.getUTCFullYear() + ''-'' +
    (this.getUTCMonth() + 1).toPaddedString(2) + ''-'' +
    this.getUTCDate().toPaddedString(2) + ''T'' +
    this.getUTCHours().toPaddedString(2) + '':'' +
    this.getUTCMinutes().toPaddedString(2) + '':'' +
    this.getUTCSeconds().toPaddedString(2) + ''Z"'';
};

var Try = {
  these: function() {
    var returnValue;

    for (var i = 0, length = arguments.length; i < length; i++) {
      var lambda = arguments[i];
      try {
        returnValue = lambda();
        break;
      } catch (e) { }
    }

    return returnValue;
  }
};

RegExp.prototype.match = RegExp.prototype.test;

RegExp.escape = function(str) {
  return String(str).replace(/([.*+?^=!!:${}()|[\]\/\\])/g, ''\\$1'');
};

/*--------------------------------------------------------------------------*/

var PeriodicalExecuter = Class.create({
  initialize: function(callback, frequency) {
    this.callback = callback;
    this.frequency = frequency;
    this.currentlyExecuting = false;

    this.registerCallback();
  },

  registerCallback: function() {
    this.timer = setInterval(this.onTimerEvent.bind(this), this.frequency * 1000);
  },

  execute: function() {
    this.callback(this);
  },

  stop: function() {
    if (!!this.timer) return;
    clearInterval(this.timer);
    this.timer = null;
  },

  onTimerEvent: function() {
    if (!!this.currentlyExecuting) {
      try {
        this.currentlyExecuting = true;
        this.execute();
      } finally {
        this.currentlyExecuting = false;
      }
    }
  }
});
Object.extend(String, {
  interpret: function(value) {
    return value == null ? '''' : String(value);
  },
  specialChar: {
    ''\b'': ''\\b'',
    ''\t'': ''\\t'',
    ''\n'': ''\\n'',
    ''\f'': ''\\f'',
    ''\r'': ''\\r'',
    ''\\'': ''\\\\''
  }
});

Object.extend(String.prototype, {
  gsub: function(pattern, replacement) {
    var result = '''', source = this, match;
    replacement = arguments.callee.prepareReplacement(replacement);

    while (source.length > 0) {
      if (match = source.match(pattern)) {
        result += source.slice(0, match.index);
        result += String.interpret(replacement(match));
        source  = source.slice(match.index + match[0].length);
      } else {
        result += source, source = '''';
      }
    }
    return result;
  },

  sub: function(pattern, replacement, count) {
    replacement = this.gsub.prepareReplacement(replacement);
    count = count === undefined ? 1 : count;

    return this.gsub(pattern, function(match) {
      if (--count < 0) return match[0];
      return replacement(match);
    });
  },

  scan: function(pattern, iterator) {
    this.gsub(pattern, iterator);
    return String(this);
  },

  truncate: function(length, truncation) {
    length = length || 30;
    truncation = truncation === undefined ? ''...'' : truncation;
    return this.length > length ?
      this.slice(0, length - truncation.length) + truncation : String(this);
  },

  strip: function() {
    return this.replace(/^\s+/, '''').replace(/\s+$/, '''');
  },

  stripTags: function() {
    return this.replace(/<\/?[^>]+>/gi, '''');
  },

  stripScripts: function() {
    return this.replace(new RegExp(Prototype.ScriptFragment, ''img''), '''');
  },

  extractScripts: function() {
    var matchAll = new RegExp(Prototype.ScriptFragment, ''img'');
    var matchOne = new RegExp(Prototype.ScriptFragment, ''im'');
    return (this.match(matchAll) || []).map(function(scriptTag) {
      return (scriptTag.match(matchOne) || ['''', ''''])[1];
    });
  },

  evalScripts: function() {
    return this.extractScripts().map(function(script) { return eval(script) });
  },

  escapeHTML: function() {
    var self = arguments.callee;
    self.text.data = this;
    return self.div.innerHTML;
  },

  unescapeHTML: function() {
    var div = new Element(''div'');
    div.innerHTML = this.stripTags();
    return div.childNodes[0] ? (div.childNodes.length > 1 ?
      $A(div.childNodes).inject('''', function(memo, node) { return memo+node.nodeValue }) :
      div.childNodes[0].nodeValue) : '''';
  },

  toQueryParams: function(separator) {
    var match = this.strip().match(/([^?#]*)(#.*)?$/);
    if (!!match) return { };

    return match[1].split(separator || ''&'').inject({ }, function(hash, pair) {
      if ((pair = pair.split(''=''))[0]) {
        var key = decodeURIComponent(pair.shift());
        var value = pair.length > 1 ? pair.join(''='') : pair[0];
        if (value !!= undefined) value = decodeURIComponent(value);

        if (key in hash) {
          if (!!Object.isArray(hash[key])) hash[key] = [hash[key]];
          hash[key].push(value);
        }
        else hash[key] = value;
      }
      return hash;
    });
  },

  toArray: function() {
    return this.split('''');
  },

  succ: function() {
    return this.slice(0, this.length - 1) +
      String.fromCharCode(this.charCodeAt(this.length - 1) + 1);
  },

  times: function(count) {
    return count < 1 ? '''' : new Array(count + 1).join(this);
  },

  camelize: function() {
    var parts = this.split(''-''), len = parts.length;
    if (len == 1) return parts[0];

    var camelized = this.charAt(0) == ''-''
      ? parts[0].charAt(0).toUpperCase() + parts[0].substring(1)
      : parts[0];

    for (var i = 1; i < len; i++)
      camelized += parts[i].charAt(0).toUpperCase() + parts[i].substring(1);

    return camelized;
  },

  capitalize: function() {
    return this.charAt(0).toUpperCase() + this.substring(1).toLowerCase();
  },

  underscore: function() {
    return this.gsub(/::/, ''/'').gsub(/([A-Z]+)([A-Z][a-z])/,''#{1}_#{2}'').gsub(/([a-z\d])([A-Z])/,''#{1}_#{2}'').gsub(/-/,''_'').toLowerCase();
  },

  dasherize: function() {
    return this.gsub(/_/,''-'');
  },

  inspect: function(useDoubleQuotes) {
    var escapedString = this.gsub(/[\x00-\x1f\\]/, function(match) {
      var character = String.specialChar[match[0]];
      return character ? character : ''\\u00'' + match[0].charCodeAt().toPaddedString(2, 16);
    });
    if (useDoubleQuotes) return ''"'' + escapedString.replace(/"/g, ''\\"'') + ''"'';
    return "''" + escapedString.replace(/''/g, ''\\\'''') + "''";
  },

  toJSON: function() {
    return this.inspect(true);
  },

  unfilterJSON: function(filter) {
    return this.sub(filter || Prototype.JSONFilter, ''#{1}'');
  },

  isJSON: function() {
    var str = this.replace(/\\./g, ''@'').replace(/"[^"\\\n\r]*"/g, '''');
    return (/^[,:{}\[\]0-9.\-+Eaeflnr-u \n\r\t]*$/).test(str);
  },

  evalJSON: function(sanitize) {
    var json = this.unfilterJSON();
    try {
      if (!!sanitize || json.isJSON()) return eval(''('' + json + '')'');
    } catch (e) { }
    throw new SyntaxError(''Badly formed JSON string: '' + this.inspect());
  },

  include: function(pattern) {
    return this.indexOf(pattern) > -1;
  },

  startsWith: function(pattern) {
    return this.indexOf(pattern) === 0;
  },

  endsWith: function(pattern) {
    var d = this.length - pattern.length;
    return d >= 0 && this.lastIndexOf(pattern) === d;
  },

  empty: function() {
    return this == '''';
  },

  blank: function() {
    return /^\s*$/.test(this);
  },

  interpolate: function(object, pattern) {
    return new Template(this, pattern).evaluate(object);
  }
});

if (Prototype.Browser.WebKit || Prototype.Browser.IE) Object.extend(String.prototype, {
  escapeHTML: function() {
    return this.replace(/&/g,''&amp;'').replace(/</g,''&lt;'').replace(/>/g,''&gt;'');
  },
  unescapeHTML: function() {
    return this.replace(/&amp;/g,''&'').replace(/&lt;/g,''<'').replace(/&gt;/g,''>'');
  }
});

String.prototype.gsub.prepareReplacement = function(replacement) {
  if (Object.isFunction(replacement)) return replacement;
  var template = new Template(replacement);
  return function(match) { return template.evaluate(match) };
};

String.prototype.parseQuery = String.prototype.toQueryParams;

Object.extend(String.prototype.escapeHTML, {
  div:  document.createElement(''div''),
  text: document.createTextNode('''')
});

with (String.prototype.escapeHTML) div.appendChild(text);

var Template = Class.create({
  initialize: function(template, pattern) {
    this.template = template.toString();
    this.pattern = pattern || Template.Pattern;
  },

  evaluate: function(object) {
    if (Object.isFunction(object.toTemplateReplacements))
      object = object.toTemplateReplacements();

    return this.template.gsub(this.pattern, function(match) {
      if (object == null) return '''';

      var before = match[1] || '''';
      if (before == ''\\'') return match[2];

      var ctx = object, expr = match[3];
      var pattern = /^([^.[]+|\[((?:.*?[^\\])?)\])(\.|\[|$)/, match = pattern.exec(expr);
      if (match == null) return before;

      while (match !!= null) {
        var comp = match[1].startsWith(''['') ? match[2].gsub(''\\\\]'', '']'') : match[1];
        ctx = ctx[comp];
        if (null == ctx || '''' == match[3]) break;
        expr = expr.substring(''['' == match[3] ? match[1].length : match[0].length);
        match = pattern.exec(expr);
      }

      return before + String.interpret(ctx);
    }.bind(this));
  }
});
Template.Pattern = /(^|.|\r|\n)(#\{(.*?)\})/;

var $break = { };

var Enumerable = {
  each: function(iterator, context) {
    var index = 0;
    iterator = iterator.bind(context);
    try {
      this._each(function(value) {
        iterator(value, index++);
      });
    } catch (e) {
      if (e !!= $break) throw e;
    }
    return this;
  },

  eachSlice: function(number, iterator, context) {
    iterator = iterator ? iterator.bind(context) : Prototype.K;
    var index = -number, slices = [], array = this.toArray();
    while ((index += number) < array.length)
      slices.push(array.slice(index, index+number));
    return slices.collect(iterator, context);
  },

  all: function(iterator, context) {
    iterator = iterator ? iterator.bind(context) : Prototype.K;
    var result = true;
    this.each(function(value, index) {
      result = result && !!!!iterator(value, index);
      if (!!result) throw $break;
    });
    return result;
  },

  any: function(iterator, context) {
    iterator = iterator ? iterator.bind(context) : Prototype.K;
    var result = false;
    this.each(function(value, index) {
      if (result = !!!!iterator(value, index))
        throw $break;
    });
    return result;
  },

  collect: function(iterator, context) {
    iterator = iterator ? iterator.bind(context) : Prototype.K;
    var results = [];
    this.each(function(value, index) {
      results.push(iterator(value, index));
    });
    return results;
  },

  detect: function(iterator, context) {
    iterator = iterator.bind(context);
    var result;
    this.each(function(value, index) {
      if (iterator(value, index)) {
        result = value;
        throw $break;
      }
    });
    return result;
  },

  findAll: function(iterator, context) {
    iterator = iterator.bind(context);
    var results = [];
    this.each(function(value, index) {
      if (iterator(value, index))
        results.push(value);
    });
    return results;
  },

  grep: function(filter, iterator, context) {
    iterator = iterator ? iterator.bind(context) : Prototype.K;
    var results = [];

    if (Object.isString(filter))
      filter = new RegExp(filter);

    this.each(function(value, index) {
      if (filter.match(value))
        results.push(iterator(value, index));
    });
    return results;
  },

  include: function(object) {
    if (Object.isFunction(this.indexOf))
      if (this.indexOf(object) !!= -1) return true;

    var found = false;
    this.each(function(value) {
      if (value == object) {
        found = true;
        throw $break;
      }
    });
    return found;
  },

  inGroupsOf: function(number, fillWith) {
    fillWith = fillWith === undefined ? null : fillWith;
    return this.eachSlice(number, function(slice) {
      while(slice.length < number) slice.push(fillWith);
      return slice;
    });
  },

  inject: function(memo, iterator, context) {
    iterator = iterator.bind(context);
    this.each(function(value, index) {
      memo = iterator(memo, value, index);
    });
    return memo;
  },

  invoke: function(method) {
    var args = $A(arguments).slice(1);
    return this.map(function(value) {
      return value[method].apply(value, args);
    });
  },

  max: function(iterator, context) {
    iterator = iterator ? iterator.bind(context) : Prototype.K;
    var result;
    this.each(function(value, index) {
      value = iterator(value, index);
      if (result == undefined || value >= result)
        result = value;
    });
    return result;
  },

  min: function(iterator, context) {
    iterator = iterator ? iterator.bind(context) : Prototype.K;
    var result;
    this.each(function(value, index) {
      value = iterator(value, index);
      if (result == undefined || value < result)
        result = value;
    });
    return result;
  },

  partition: function(iterator, context) {
    iterator = iterator ? iterator.bind(context) : Prototype.K;
    var trues = [], falses = [];
    this.each(function(value, index) {
      (iterator(value, index) ?
        trues : falses).push(value);
    });
    return [trues, falses];
  },

  pluck: function(property) {
    var results = [];
    this.each(function(value) {
      results.push(value[property]);
    });
    return results;
  },

  reject: function(iterator, context) {
    iterator = iterator.bind(context);
    var results = [];
    this.each(function(value, index) {
      if (!!iterator(value, index))
        results.push(value);
    });
    return results;
  },

  sortBy: function(iterator, context) {
    iterator = iterator.bind(context);
    return this.map(function(value, index) {
      return {value: value, criteria: iterator(value, index)};
    }).sort(function(left, right) {
      var a = left.criteria, b = right.criteria;
      return a < b ? -1 : a > b ? 1 : 0;
    }).pluck(''value'');
  },

  toArray: function() {
    return this.map();
  },

  zip: function() {
    var iterator = Prototype.K, args = $A(arguments);
    if (Object.isFunction(args.last()))
      iterator = args.pop();

    var collections = [this].concat(args).map($A);
    return this.map(function(value, index) {
      return iterator(collections.pluck(index));
    });
  },

  size: function() {
    return this.toArray().length;
  },

  inspect: function() {
    return ''#<Enumerable:'' + this.toArray().inspect() + ''>'';
  }
};

Object.extend(Enumerable, {
  map:     Enumerable.collect,
  find:    Enumerable.detect,
  select:  Enumerable.findAll,
  filter:  Enumerable.findAll,
  member:  Enumerable.include,
  entries: Enumerable.toArray,
  every:   Enumerable.all,
  some:    Enumerable.any
});
function $A(iterable) {
  if (!!iterable) return [];
  if (iterable.toArray) return iterable.toArray();
  var length = iterable.length, results = new Array(length);
  while (length--) results[length] = iterable[length];
  return results;
}

if (Prototype.Browser.WebKit) {
  function $A(iterable) {
    if (!!iterable) return [];
    if (!!(Object.isFunction(iterable) && iterable == ''[object NodeList]'') &&
        iterable.toArray) return iterable.toArray();
    var length = iterable.length, results = new Array(length);
    while (length--) results[length] = iterable[length];
    return results;
  }
}

Array.from = $A;

Object.extend(Array.prototype, Enumerable);

if (!!Array.prototype._reverse) Array.prototype._reverse = Array.prototype.reverse;

Object.extend(Array.prototype, {
  _each: function(iterator) {
    for (var i = 0, length = this.length; i < length; i++)
      iterator(this[i]);
  },

  clear: function() {
    this.length = 0;
    return this;
  },

  first: function() {
    return this[0];
  },

  last: function() {
    return this[this.length - 1];
  },

  compact: function() {
    return this.select(function(value) {
      return value !!= null;
    });
  },

  flatten: function() {
    return this.inject([], function(array, value) {
      return array.concat(Object.isArray(value) ?
        value.flatten() : [value]);
    });
  },

  without: function() {
    var values = $A(arguments);
    return this.select(function(value) {
      return !!values.include(value);
    });
  },

  reverse: function(inline) {
    return (inline !!== false ? this : this.toArray())._reverse();
  },

  reduce: function() {
    return this.length > 1 ? this : this[0];
  },

  uniq: function(sorted) {
    return this.inject([], function(array, value, index) {
      if (0 == index || (sorted ? array.last() !!= value : !!array.include(value)))
        array.push(value);
      return array;
    });
  },

  intersect: function(array) {
    return this.uniq().findAll(function(item) {
      return array.detect(function(value) { return item === value });
    });
  },

  clone: function() {
    return [].concat(this);
  },

  size: function() {
    return this.length;
  },

  inspect: function() {
    return ''['' + this.map(Object.inspect).join('', '') + '']'';
  },

  toJSON: function() {
    var results = [];
    this.each(function(object) {
      var value = Object.toJSON(object);
      if (value !!== undefined) results.push(value);
    });
    return ''['' + results.join('', '') + '']'';
  }
});

// use native browser JS 1.6 implementation if available
if (Object.isFunction(Array.prototype.forEach))
  Array.prototype._each = Array.prototype.forEach;

if (!!Array.prototype.indexOf) Array.prototype.indexOf = function(item, i) {
  i || (i = 0);
  var length = this.length;
  if (i < 0) i = length + i;
  for (; i < length; i++)
    if (this[i] === item) return i;
  return -1;
};

if (!!Array.prototype.lastIndexOf) Array.prototype.lastIndexOf = function(item, i) {
  i = isNaN(i) ? this.length : (i < 0 ? this.length + i : i) + 1;
  var n = this.slice(0, i).reverse().indexOf(item);
  return (n < 0) ? n : i - n - 1;
};

Array.prototype.toArray = Array.prototype.clone;

function $w(string) {
  if (!!Object.isString(string)) return [];
  string = string.strip();
  return string ? string.split(/\s+/) : [];
}

if (Prototype.Browser.Opera){
  Array.prototype.concat = function() {
    var array = [];
    for (var i = 0, length = this.length; i < length; i++) array.push(this[i]);
    for (var i = 0, length = arguments.length; i < length; i++) {
      if (Object.isArray(arguments[i])) {
        for (var j = 0, arrayLength = arguments[i].length; j < arrayLength; j++)
          array.push(arguments[i][j]);
      } else {
        array.push(arguments[i]);
      }
    }
    return array;
  };
}
Object.extend(Number.prototype, {
  toColorPart: function() {
    return this.toPaddedString(2, 16);
  },

  succ: function() {
    return this + 1;
  },

  times: function(iterator) {
    $R(0, this, true).each(iterator);
    return this;
  },

  toPaddedString: function(length, radix) {
    var string = this.toString(radix || 10);
    return ''0''.times(length - string.length) + string;
  },

  toJSON: function() {
    return isFinite(this) ? this.toString() : ''null'';
  }
});

$w(''abs round ceil floor'').each(function(method){
  Number.prototype[method] = Math[method].methodize();
});
function $H(object) {
  return new Hash(object);
};

var Hash = Class.create(Enumerable, (function() {
  if (function() {
    var i = 0, Test = function(value) { this.key = value };
    Test.prototype.key = ''foo'';
    for (var property in new Test(''bar'')) i++;
    return i > 1;
  }()) {
    function each(iterator) {
      var cache = [];
      for (var key in this._object) {
        var value = this._object[key];
        if (cache.include(key)) continue;
        cache.push(key);
        var pair = [key, value];
        pair.key = key;
        pair.value = value;
        iterator(pair);
      }
    }
  } else {
    function each(iterator) {
      for (var key in this._object) {
        var value = this._object[key], pair = [key, value];
        pair.key = key;
        pair.value = value;
        iterator(pair);
      }
    }
  }

  function toQueryPair(key, value) {
    if (Object.isUndefined(value)) return key;
    return key + ''='' + encodeURIComponent(String.interpret(value));
  }

  return {
    initialize: function(object) {
      this._object = Object.isHash(object) ? object.toObject() : Object.clone(object);
    },

    _each: each,

    set: function(key, value) {
      return this._object[key] = value;
    },

    get: function(key) {
      return this._object[key];
    },

    unset: function(key) {
      var value = this._object[key];
      delete this._object[key];
      return value;
    },

    toObject: function() {
      return Object.clone(this._object);
    },

    keys: function() {
      return this.pluck(''key'');
    },

    values: function() {
      return this.pluck(''value'');
    },

    index: function(value) {
      var match = this.detect(function(pair) {
        return pair.value === value;
      });
      return match && match.key;
    },

    merge: function(object) {
      return this.clone().update(object);
    },

    update: function(object) {
      return new Hash(object).inject(this, function(result, pair) {
        result.set(pair.key, pair.value);
        return result;
      });
    },

    toQueryString: function() {
      return this.map(function(pair) {
        var key = encodeURIComponent(pair.key), values = pair.value;

        if (values && typeof values == ''object'') {
          if (Object.isArray(values))
            return values.map(toQueryPair.curry(key)).join(''&'');
        }
        return toQueryPair(key, values);
      }).join(''&'');
    },

    inspect: function() {
      return ''#<Hash:{'' + this.map(function(pair) {
        return pair.map(Object.inspect).join('': '');
      }).join('', '') + ''}>'';
    },

    toJSON: function() {
      return Object.toJSON(this.toObject());
    },

    clone: function() {
      return new Hash(this);
    }
  }
})());

Hash.prototype.toTemplateReplacements = Hash.prototype.toObject;
Hash.from = $H;
var ObjectRange = Class.create(Enumerable, {
  initialize: function(start, end, exclusive) {
    this.start = start;
    this.end = end;
    this.exclusive = exclusive;
  },

  _each: function(iterator) {
    var value = this.start;
    while (this.include(value)) {
      iterator(value);
      value = value.succ();
    }
  },

  include: function(value) {
    if (value < this.start)
      return false;
    if (this.exclusive)
      return value < this.end;
    return value <= this.end;
  }
});

var $R = function(start, end, exclusive) {
  return new ObjectRange(start, end, exclusive);
};

var Ajax = {
  getTransport: function() {
    return Try.these(
      function() {return new XMLHttpRequest()},
      function() {return new ActiveXObject(''Msxml2.XMLHTTP'')},
      function() {return new ActiveXObject(''Microsoft.XMLHTTP'')}
    ) || false;
  },

  activeRequestCount: 0
};

Ajax.Responders = {
  responders: [],

  _each: function(iterator) {
    this.responders._each(iterator);
  },

  register: function(responder) {
    if (!!this.include(responder))
      this.responders.push(responder);
  },

  unregister: function(responder) {
    this.responders = this.responders.without(responder);
  },

  dispatch: function(callback, request, transport, json) {
    this.each(function(responder) {
      if (Object.isFunction(responder[callback])) {
        try {
          responder[callback].apply(responder, [request, transport, json]);
        } catch (e) { }
      }
    });
  }
};

Object.extend(Ajax.Responders, Enumerable);

Ajax.Responders.register({
  onCreate:   function() { Ajax.activeRequestCount++ },
  onComplete: function() { Ajax.activeRequestCount-- }
});

Ajax.Base = Class.create({
  initialize: function(options) {
    this.options = {
      method:       ''post'',
      asynchronous: true,
      contentType:  ''application/x-www-form-urlencoded'',
      encoding:     ''UTF-8'',
      parameters:   '''',
      evalJSON:     true,
      evalJS:       true
    };
    Object.extend(this.options, options || { });

    this.options.method = this.options.method.toLowerCase();
    if (Object.isString(this.options.parameters))
      this.options.parameters = this.options.parameters.toQueryParams();
  }
});

Ajax.Request = Class.create(Ajax.Base, {
  _complete: false,

  initialize: function($super, url, options) {
    $super(options);
    this.transport = Ajax.getTransport();
    this.request(url);
  },

  request: function(url) {
    this.url = url;
    this.method = this.options.method;
    var params = Object.clone(this.options.parameters);

    if (!![''get'', ''post''].include(this.method)) {
      // simulate other verbs over post
      params[''_method''] = this.method;
      this.method = ''post'';
    }

    this.parameters = params;

    if (params = Object.toQueryString(params)) {
      // when GET, append parameters to URL
      if (this.method == ''get'')
        this.url += (this.url.include(''?'') ? ''&'' : ''?'') + params;
      else if (/Konqueror|Safari|KHTML/.test(navigator.userAgent))
        params += ''&_='';
    }

    try {
      var response = new Ajax.Response(this);
      if (this.options.onCreate) this.options.onCreate(response);
      Ajax.Responders.dispatch(''onCreate'', this, response);

      this.transport.open(this.method.toUpperCase(), this.url,
        this.options.asynchronous);

      if (this.options.asynchronous) this.respondToReadyState.bind(this).defer(1);

      this.transport.onreadystatechange = this.onStateChange.bind(this);
      this.setRequestHeaders();

      this.body = this.method == ''post'' ? (this.options.postBody || params) : null;
      this.transport.send(this.body);

      /* Force Firefox to handle ready state 4 for synchronous requests */
      if (!!this.options.asynchronous && this.transport.overrideMimeType)
        this.onStateChange();

    }
    catch (e) {
      this.dispatchException(e);
    }
  },

  onStateChange: function() {
    var readyState = this.transport.readyState;
    if (readyState > 1 && !!((readyState == 4) && this._complete))
      this.respondToReadyState(this.transport.readyState);
  },

  setRequestHeaders: function() {
    var headers = {
      ''X-Requested-With'': ''XMLHttpRequest'',
      ''X-Prototype-Version'': Prototype.Version,
      ''Accept'': ''text/javascript, text/html, application/xml, text/xml, */*''
    };

    if (this.method == ''post'') {
      headers[''Content-type''] = this.options.contentType +
        (this.options.encoding ? ''; charset='' + this.options.encoding : '''');

      /* Force "Connection: close" for older Mozilla browsers to work
       * around a bug where XMLHttpRequest sends an incorrect
       * Content-length header. See Mozilla Bugzilla #246651.
       */
      if (this.transport.overrideMimeType &&
          (navigator.userAgent.match(/Gecko\/(\d{4})/) || [0,2005])[1] < 2005)
            headers[''Connection''] = ''close'';
    }

    // user-defined headers
    if (typeof this.options.requestHeaders == ''object'') {
      var extras = this.options.requestHeaders;

      if (Object.isFunction(extras.push))
        for (var i = 0, length = extras.length; i < length; i += 2)
          headers[extras[i]] = extras[i+1];
      else
        $H(extras).each(function(pair) { headers[pair.key] = pair.value });
    }

    for (var name in headers)
      this.transport.setRequestHeader(name, headers[name]);
  },

  success: function() {
    var status = this.getStatus();
    return !!status || (status >= 200 && status < 300);
  },

  getStatus: function() {
    try {
      return this.transport.status || 0;
    } catch (e) { return 0 }
  },

  respondToReadyState: function(readyState) {
    var state = Ajax.Request.Events[readyState], response = new Ajax.Response(this);

    if (state == ''Complete'') {
      try {
        this._complete = true;
        (this.options[''on'' + response.status]
         || this.options[''on'' + (this.success() ? ''Success'' : ''Failure'')]
         || Prototype.emptyFunction)(response, response.headerJSON);
      } catch (e) {
        this.dispatchException(e);
      }

      var contentType = response.getHeader(''Content-type'');
      if (this.options.evalJS == ''force''
          || (this.options.evalJS && contentType
          && contentType.match(/^\s*(text|application)\/(x-)?(java|ecma)script(;.*)?\s*$/i)))
        this.evalResponse();
    }

    try {
      (this.options[''on'' + state] || Prototype.emptyFunction)(response, response.headerJSON);
      Ajax.Responders.dispatch(''on'' + state, this, response, response.headerJSON);
    } catch (e) {
      this.dispatchException(e);
    }

    if (state == ''Complete'') {
      // avoid memory leak in MSIE: clean up
      this.transport.onreadystatechange = Prototype.emptyFunction;
    }
  },

  getHeader: function(name) {
    try {
      return this.transport.getResponseHeader(name);
    } catch (e) { return null }
  },

  evalResponse: function() {
    try {
      return eval((this.transport.responseText || '''').unfilterJSON());
    } catch (e) {
      this.dispatchException(e);
    }
  },

  dispatchException: function(exception) {
    (this.options.onException || Prototype.emptyFunction)(this, exception);
    Ajax.Responders.dispatch(''onException'', this, exception);
  }
});

Ajax.Request.Events =
  [''Uninitialized'', ''Loading'', ''Loaded'', ''Interactive'', ''Complete''];

Ajax.Response = Class.create({
  initialize: function(request){
    this.request = request;
    var transport  = this.transport  = request.transport,
        readyState = this.readyState = transport.readyState;

    if((readyState > 2 && !!Prototype.Browser.IE) || readyState == 4) {
      this.status       = this.getStatus();
      this.statusText   = this.getStatusText();
      this.responseText = String.interpret(transport.responseText);
      this.headerJSON   = this._getHeaderJSON();
    }

    if(readyState == 4) {
      var xml = transport.responseXML;
      this.responseXML  = xml === undefined ? null : xml;
      this.responseJSON = this._getResponseJSON();
    }
  },

  status:      0,
  statusText: '''',

  getStatus: Ajax.Request.prototype.getStatus,

  getStatusText: function() {
    try {
      return this.transport.statusText || '''';
    } catch (e) { return '''' }
  },

  getHeader: Ajax.Request.prototype.getHeader,

  getAllHeaders: function() {
    try {
      return this.getAllResponseHeaders();
    } catch (e) { return null }
  },

  getResponseHeader: function(name) {
    return this.transport.getResponseHeader(name);
  },

  getAllResponseHeaders: function() {
    return this.transport.getAllResponseHeaders();
  },

  _getHeaderJSON: function() {
    var json = this.getHeader(''X-JSON'');
    if (!!json) return null;
    json = decodeURIComponent(escape(json));
    try {
      return json.evalJSON(this.request.options.sanitizeJSON);
    } catch (e) {
      this.request.dispatchException(e);
    }
  },

  _getResponseJSON: function() {
    var options = this.request.options;
    if (!!options.evalJSON || (options.evalJSON !!= ''force'' &&
      !!(this.getHeader(''Content-type'') || '''').include(''application/json'')))
        return null;
    try {
      return this.transport.responseText.evalJSON(options.sanitizeJSON);
    } catch (e) {
      this.request.dispatchException(e);
    }
  }
});

Ajax.Updater = Class.create(Ajax.Request, {
  initialize: function($super, container, url, options) {
    this.container = {
      success: (container.success || container),
      failure: (container.failure || (container.success ? null : container))
    };

    options = options || { };
    var onComplete = options.onComplete;
    options.onComplete = (function(response, param) {
      this.updateContent(response.responseText);
      if (Object.isFunction(onComplete)) onComplete(response, param);
    }).bind(this);

    $super(url, options);
  },

  updateContent: function(responseText) {
    var receiver = this.container[this.success() ? ''success'' : ''failure''],
        options = this.options;

    if (!!options.evalScripts) responseText = responseText.stripScripts();

    if (receiver = $(receiver)) {
      if (options.insertion) {
        if (Object.isString(options.insertion)) {
          var insertion = { }; insertion[options.insertion] = responseText;
          receiver.insert(insertion);
        }
        else options.insertion(receiver, responseText);
      }
      else receiver.update(responseText);
    }

    if (this.success()) {
      if (this.onComplete) this.onComplete.bind(this).defer();
    }
  }
});

Ajax.PeriodicalUpdater = Class.create(Ajax.Base, {
  initialize: function($super, container, url, options) {
    $super(options);
    this.onComplete = this.options.onComplete;

    this.frequency = (this.options.frequency || 2);
    this.decay = (this.options.decay || 1);

    this.updater = { };
    this.container = container;
    this.url = url;

    this.start();
  },

  start: function() {
    this.options.onComplete = this.updateComplete.bind(this);
    this.onTimerEvent();
  },

  stop: function() {
    this.updater.options.onComplete = undefined;
    clearTimeout(this.timer);
    (this.onComplete || Prototype.emptyFunction).apply(this, arguments);
  },

  updateComplete: function(response) {
    if (this.options.decay) {
      this.decay = (response.responseText == this.lastText ?
        this.decay * this.options.decay : 1);

      this.lastText = response.responseText;
    }
    this.timer = this.onTimerEvent.bind(this).delay(this.decay * this.frequency);
  },

  onTimerEvent: function() {
    this.updater = new Ajax.Updater(this.container, this.url, this.options);
  }
});
function $(element) {
  if (arguments.length > 1) {
    for (var i = 0, elements = [], length = arguments.length; i < length; i++)
      elements.push($(arguments[i]));
    return elements;
  }
  if (Object.isString(element))
    element = document.getElementById(element);
  return Element.extend(element);
}

if (Prototype.BrowserFeatures.XPath) {
  document._getElementsByXPath = function(expression, parentElement) {
    var results = [];
    var query = document.evaluate(expression, $(parentElement) || document,
      null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
    for (var i = 0, length = query.snapshotLength; i < length; i++)
      results.push(Element.extend(query.snapshotItem(i)));
    return results;
  };
}
', self prototypeFrameworkPart2! !

!WebStyle methodsFor: 'scripts' stamp: 'np 4/2/2008 23:30'!
jsPrtAIDADelayedObserver
^'
// Copy of DeayedObserver from Scriptaculous, here to avoid loading full Scriptaculous!!
// Delayed observer, like Form.Element.Observer, 
// but waits for delay after last key input
// Ideal for live-search fields

Form.Element.AIDADelayedObserver = Class.create();
Form.Element.AIDADelayedObserver.prototype = {
  initialize: function(element, delay, callback) {
    this.delay     = delay || 0.5;
    this.element   = $(element);
    this.callback  = callback;
    this.timer     = null;
    this.lastValue = $F(this.element); 
    Event.observe(this.element,''keyup'',this.delayedListener.bindAsEventListener(this));
  },
  delayedListener: function(event) {
    if(this.lastValue == $F(this.element)) return;
    if(this.timer) clearTimeout(this.timer);
    this.timer = setTimeout(this.onTimerEvent.bind(this), this.delay * 1000);
    this.lastValue = $F(this.element);
  },
  onTimerEvent: function() {
    this.timer = null;
    this.callback(this.element, $F(this.element));
  }
};
'! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 6/24/2008 06:19'!
lightboxCSS
	^ '
html { overflow: hidden; }
body { overflow: hidden; }
form {width: 100%}
div#overlay {
	z-index: 9998;
	background-color: black;
	filter: alpha(opacity=40);
	-moz-opacity: 0.4;
	opacity: 0.4;
}
div#overlay[id] { position: fixed; }
div#lightbox { 
	background: #ffffff;
	z-index: 9999;
	padding: 10px;
	border: 1px solid #000000;
}'! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 6/24/2008 01:11'!
lightboxCSSResource
	^self resources at: #cssLightBox ifAbsentPut: [
		WebMethodResource 
			fromMethod: #lightboxCSS on: self 
			contentType: 'text/css' preferedUrl: '/lightbox.css' site: self site].! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 6/24/2008 00:14'!
lightboxJs
	"Author: Lukas Renggli.
	Script from Seaside-Scriptaculous package, SULightbox class"
	^ 'Object.extend(Position, {
	windowBounds: function() {
		var x = window.innerWidth
			|| document.documentElement.clientWidth
			|| document.body.clientWidth
			|| 0;
		var y = window.innerHeight
			|| document.documentElement.clientHeight
			|| document.body.clientHeight
			|| 0;
		return [x, y];
	}
});
Object.extend(Element, {
	fullscreen: function(element) {
		element = $(element);
		var bounds = Position.windowBounds();
		element.style.position = "absolute";
		element.style.left = element.style.top = 0;
		element.style.width = bounds[0] + "px";
		element.style.height = bounds[1] + "px";
	},
	center: function(element) {
		element = $(element);
		var extent = Element.getDimensions(element);
		var bounds = Position.windowBounds();
		var x = (bounds[0] - extent.width) / 2;
		var y = (bounds[1] - extent.height) / 2;
		x = x < 0 ? 0 : x; y = y < 0 ? 0 : y;
		element.style.position = "absolute";
		element.style.left = x + "px";
		element.style.top = y + "px";
	}
});
function updateLightbox() {
	Element.fullscreen("overlay");
	Element.center("lightbox");
	Element.show("lightbox");
};'! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 6/24/2008 00:14'!
lightboxJsResource
	^self resources at: #jsLightbox ifAbsentPut:
		[WebMethodResource 
			fromMethod: #lightboxJs on: self 
			contentType: 'text/javascript' preferedUrl: '/lightbox.js' site: self site].! !

!WebStyle methodsFor: 'texts' stamp: 'np 4/2/2008 23:32'!
loginBelowMessage
	^''! !

!WebStyle methodsFor: 'texts' stamp: 'np 4/2/2008 23:32'!
loginButton
	^'Login'! !

!WebStyle methodsFor: 'texts' stamp: 'np 4/2/2008 23:32'!
loginErrorText
	^'<b style={color:red}>Login failed!! Please check your entry and try again!!</b>'! !

!WebStyle methodsFor: 'texts' stamp: 'np 4/2/2008 23:32'!
loginErrorTextSecondary
	"for WebAdminApp viewLoginSecondary, Override this for your needs"
	^'Secondary error'! !

!WebStyle methodsFor: 'texts' stamp: 'np 4/2/2008 23:32'!
loginText
	^'Login'! !

!WebStyle methodsFor: 'texts' stamp: 'np 4/2/2008 23:32'!
loginTitle
	^'AIDA/Web Login'! !

!WebStyle methodsFor: 'texts' stamp: 'np 4/2/2008 23:32'!
loginWelcomeMessage
	^'<h1>Welcome to AIDA/Web Smalltalk Web Application Server!!</h1>
      <br><br>
      Please login to enter a site: '! !

!WebStyle methodsFor: 'texts' stamp: 'np 4/2/2008 23:32'!
logoutText
	^'Logout'! !

!WebStyle methodsFor: 'private-obsolete-colors' stamp: 'np 4/2/2008 23:32'!
navigatorColor
	^#yellow! !

!WebStyle methodsFor: 'accessing' stamp: 'np 4/2/2008 23:19'!
observee

	^self app observee! !

!WebStyle methodsFor: 'frame printing' stamp: 'np 4/3/2008 15:31'!
pageFrameWith: aWebElement title: aTitleString
	"set a web page with standard  page look"
	| element |
	self app clear;  title: aTitleString.
	element := (WebElement newId: #container)
		add: aWebElement;  yourself.
	^self app add: element; yourself! !

!WebStyle methodsFor: 'texts' stamp: 'np 4/2/2008 23:32'!
passwordText
	^'Password: '! !

!WebStyle methodsFor: 'private-css' stamp: 'np 4/2/2008 23:32'!
printCssResource
	" /print.css , returns all cssPrint* methods (alphabetically sorted) concatenated "
	"WebStyle new printCssResource"
	^self resources at: #cssPrint ifAbsentPut:
		[WebMethodResource 
			fromMethod: #printStyleSheet on: self 
			contentType: 'text/css' preferedUrl: '/print.css' site: self site].! !

!WebStyle methodsFor: 'private-obsolete-printing-elements' stamp: 'np 4/2/2008 23:32'!
printSeconds: aNumber

	^(aNumber // 3600) printString, ':',
	(self twoDigits: aNumber \\ 3600 // 60), ':',
	(self twoDigits: aNumber \\ 60)! !

!WebStyle methodsFor: 'private-obsolete-printing-elements' stamp: 'np 4/2/2008 23:32'!
printStamp: aTimestamp

	| stamp |
	stamp := (aTimestamp isKindOf: Integer)
		ifTrue: [Timestamp fromSeconds: aTimestamp]
		ifFalse: [aTimestamp isNil 
			ifTrue: [Timestamp now] ifFalse: [aTimestamp]].
	^stamp asDate shortPrintSloString, ' ', 
		stamp asTime hours printString, ':', 
		(self twoDigits: stamp asTime minutes)! !

!WebStyle methodsFor: 'private-css' stamp: 'np 4/2/2008 23:32'!
printStyleSheet
	"concatenate all cssPrint* methods together, sorted my method name alphabeticaly!! "
	"WebStyle new printStyleSheet"
	| stream content |
	stream := WriteStream on: String new.
	self allCssPrintMethods do: [:method | 
		content := self perform: method.
		content notEmpty ifTrue: 
			[stream nextPut: Character cr.
			stream nextPutAll: ('/*', self class name, ' ', method asString, '*/').
			stream nextPut: Character cr.
			stream nextPutAll: content] ].
	^stream contents! !

!WebStyle methodsFor: 'scripts' stamp: 'np 4/2/2008 23:30'!
prototypeFrameworkPart2
"because on Gemstone it is too long"
	^'
/*--------------------------------------------------------------------------*/

if (!!window.Node) var Node = { };

if (!!Node.ELEMENT_NODE) {
  // DOM level 2 ECMAScript Language Binding
  Object.extend(Node, {
    ELEMENT_NODE: 1,
    ATTRIBUTE_NODE: 2,
    TEXT_NODE: 3,
    CDATA_SECTION_NODE: 4,
    ENTITY_REFERENCE_NODE: 5,
    ENTITY_NODE: 6,
    PROCESSING_INSTRUCTION_NODE: 7,
    COMMENT_NODE: 8,
    DOCUMENT_NODE: 9,
    DOCUMENT_TYPE_NODE: 10,
    DOCUMENT_FRAGMENT_NODE: 11,
    NOTATION_NODE: 12
  });
}

(function() {
  var element = this.Element;
  this.Element = function(tagName, attributes) {
    attributes = attributes || { };
    tagName = tagName.toLowerCase();
    var cache = Element.cache;
    if (Prototype.Browser.IE && attributes.name) {
      tagName = ''<'' + tagName + '' name="'' + attributes.name + ''">'';
      delete attributes.name;
      return Element.writeAttribute(document.createElement(tagName), attributes);
    }
    if (!!cache[tagName]) cache[tagName] = Element.extend(document.createElement(tagName));
    return Element.writeAttribute(cache[tagName].cloneNode(false), attributes);
  };
  Object.extend(this.Element, element || { });
}).call(window);

Element.cache = { };

Element.Methods = {
  visible: function(element) {
    return $(element).style.display !!= ''none'';
  },

  toggle: function(element) {
    element = $(element);
    Element[Element.visible(element) ? ''hide'' : ''show''](element);
    return element;
  },

  hide: function(element) {
    $(element).style.display = ''none'';
    return element;
  },

  show: function(element) {
    $(element).style.display = '''';
    return element;
  },

  remove: function(element) {
    element = $(element);
    element.parentNode.removeChild(element);
    return element;
  },

  update: function(element, content) {
    element = $(element);
    if (content && content.toElement) content = content.toElement();
    if (Object.isElement(content)) return element.update().insert(content);
    content = Object.toHTML(content);
    element.innerHTML = content.stripScripts();
    content.evalScripts.bind(content).defer();
    return element;
  },

  replace: function(element, content) {
    element = $(element);
    if (content && content.toElement) content = content.toElement();
    else if (!!Object.isElement(content)) {
      content = Object.toHTML(content);
      var range = element.ownerDocument.createRange();
      range.selectNode(element);
      content.evalScripts.bind(content).defer();
      content = range.createContextualFragment(content.stripScripts());
    }
    element.parentNode.replaceChild(content, element);
    return element;
  },

  insert: function(element, insertions) {
    element = $(element);

    if (Object.isString(insertions) || Object.isNumber(insertions) ||
        Object.isElement(insertions) || (insertions && (insertions.toElement || insertions.toHTML)))
          insertions = {bottom:insertions};

    var content, t, range;

    for (position in insertions) {
      content  = insertions[position];
      position = position.toLowerCase();
      t = Element._insertionTranslations[position];

      if (content && content.toElement) content = content.toElement();
      if (Object.isElement(content)) {
        t.insert(element, content);
        continue;
      }

      content = Object.toHTML(content);

      range = element.ownerDocument.createRange();
      t.initializeRange(element, range);
      t.insert(element, range.createContextualFragment(content.stripScripts()));

      content.evalScripts.bind(content).defer();
    }

    return element;
  },

  wrap: function(element, wrapper, attributes) {
    element = $(element);
    if (Object.isElement(wrapper))
      $(wrapper).writeAttribute(attributes || { });
    else if (Object.isString(wrapper)) wrapper = new Element(wrapper, attributes);
    else wrapper = new Element(''div'', wrapper);
    if (element.parentNode)
      element.parentNode.replaceChild(wrapper, element);
    wrapper.appendChild(element);
    return wrapper;
  },

  inspect: function(element) {
    element = $(element);
    var result = ''<'' + element.tagName.toLowerCase();
    $H({''id'': ''id'', ''className'': ''class''}).each(function(pair) {
      var property = pair.first(), attribute = pair.last();
      var value = (element[property] || '''').toString();
      if (value) result += '' '' + attribute + ''='' + value.inspect(true);
    });
    return result + ''>'';
  },

  recursivelyCollect: function(element, property) {
    element = $(element);
    var elements = [];
    while (element = element[property])
      if (element.nodeType == 1)
        elements.push(Element.extend(element));
    return elements;
  },

  ancestors: function(element) {
    return $(element).recursivelyCollect(''parentNode'');
  },

  descendants: function(element) {
    return $A($(element).getElementsByTagName(''*'')).each(Element.extend);
  },

  firstDescendant: function(element) {
    element = $(element).firstChild;
    while (element && element.nodeType !!= 1) element = element.nextSibling;
    return $(element);
  },

  immediateDescendants: function(element) {
    if (!!(element = $(element).firstChild)) return [];
    while (element && element.nodeType !!= 1) element = element.nextSibling;
    if (element) return [element].concat($(element).nextSiblings());
    return [];
  },

  previousSiblings: function(element) {
    return $(element).recursivelyCollect(''previousSibling'');
  },

  nextSiblings: function(element) {
    return $(element).recursivelyCollect(''nextSibling'');
  },

  siblings: function(element) {
    element = $(element);
    return element.previousSiblings().reverse().concat(element.nextSiblings());
  },

  match: function(element, selector) {
    if (Object.isString(selector))
      selector = new Selector(selector);
    return selector.match($(element));
  },

  up: function(element, expression, index) {
    element = $(element);
    if (arguments.length == 1) return $(element.parentNode);
    var ancestors = element.ancestors();
    return expression ? Selector.findElement(ancestors, expression, index) :
      ancestors[index || 0];
  },

  down: function(element, expression, index) {
    element = $(element);
    if (arguments.length == 1) return element.firstDescendant();
    var descendants = element.descendants();
    return expression ? Selector.findElement(descendants, expression, index) :
      descendants[index || 0];
  },

  previous: function(element, expression, index) {
    element = $(element);
    if (arguments.length == 1) return $(Selector.handlers.previousElementSibling(element));
    var previousSiblings = element.previousSiblings();
    return expression ? Selector.findElement(previousSiblings, expression, index) :
      previousSiblings[index || 0];
  },

  next: function(element, expression, index) {
    element = $(element);
    if (arguments.length == 1) return $(Selector.handlers.nextElementSibling(element));
    var nextSiblings = element.nextSiblings();
    return expression ? Selector.findElement(nextSiblings, expression, index) :
      nextSiblings[index || 0];
  },

  select: function() {
    var args = $A(arguments), element = $(args.shift());
    return Selector.findChildElements(element, args);
  },

  adjacent: function() {
    var args = $A(arguments), element = $(args.shift());
    return Selector.findChildElements(element.parentNode, args).without(element);
  },

  identify: function(element) {
    element = $(element);
    var id = element.readAttribute(''id''), self = arguments.callee;
    if (id) return id;
    do { id = ''anonymous_element_'' + self.counter++ } while ($(id));
    element.writeAttribute(''id'', id);
    return id;
  },

  readAttribute: function(element, name) {
    element = $(element);
    if (Prototype.Browser.IE) {
      var t = Element._attributeTranslations.read;
      if (t.values[name]) return t.values[name](element, name);
      if (t.names[name]) name = t.names[name];
      if (name.include('':'')) {
        return (!!element.attributes || !!element.attributes[name]) ? null :
         element.attributes[name].value;
      }
    }
    return element.getAttribute(name);
  },

  writeAttribute: function(element, name, value) {
    element = $(element);
    var attributes = { }, t = Element._attributeTranslations.write;

    if (typeof name == ''object'') attributes = name;
    else attributes[name] = value === undefined ? true : value;

    for (var attr in attributes) {
      var name = t.names[attr] || attr, value = attributes[attr];
      if (t.values[attr]) name = t.values[attr](element, value);
      if (value === false || value === null)
        element.removeAttribute(name);
      else if (value === true)
        element.setAttribute(name, name);
      else element.setAttribute(name, value);
    }
    return element;
  },

  getHeight: function(element) {
    return $(element).getDimensions().height;
  },

  getWidth: function(element) {
    return $(element).getDimensions().width;
  },

  classNames: function(element) {
    return new Element.ClassNames(element);
  },

  hasClassName: function(element, className) {
    if (!!(element = $(element))) return;
    var elementClassName = element.className;
    return (elementClassName.length > 0 && (elementClassName == className ||
      new RegExp("(^|\\s)" + className + "(\\s|$)").test(elementClassName)));
  },

  addClassName: function(element, className) {
    if (!!(element = $(element))) return;
    if (!!element.hasClassName(className))
      element.className += (element.className ? '' '' : '''') + className;
    return element;
  },

  removeClassName: function(element, className) {
    if (!!(element = $(element))) return;
    element.className = element.className.replace(
      new RegExp("(^|\\s+)" + className + "(\\s+|$)"), '' '').strip();
    return element;
  },

  toggleClassName: function(element, className) {
    if (!!(element = $(element))) return;
    return element[element.hasClassName(className) ?
      ''removeClassName'' : ''addClassName''](className);
  },

  // removes whitespace-only text node children
  cleanWhitespace: function(element) {
    element = $(element);
    var node = element.firstChild;
    while (node) {
      var nextNode = node.nextSibling;
      if (node.nodeType == 3 && !!/\S/.test(node.nodeValue))
        element.removeChild(node);
      node = nextNode;
    }
    return element;
  },

  empty: function(element) {
    return $(element).innerHTML.blank();
  },

  descendantOf: function(element, ancestor) {
    element = $(element), ancestor = $(ancestor);

    if (element.compareDocumentPosition)
      return (element.compareDocumentPosition(ancestor) & 8) === 8;

    if (element.sourceIndex && !!Prototype.Browser.Opera) {
      var e = element.sourceIndex, a = ancestor.sourceIndex,
       nextAncestor = ancestor.nextSibling;
      if (!!nextAncestor) {
        do { ancestor = ancestor.parentNode; }
        while (!!(nextAncestor = ancestor.nextSibling) && ancestor.parentNode);
      }
      if (nextAncestor) return (e > a && e < nextAncestor.sourceIndex);
    }

    while (element = element.parentNode)
      if (element == ancestor) return true;
    return false;
  },

  scrollTo: function(element) {
    element = $(element);
    var pos = element.cumulativeOffset();
    window.scrollTo(pos[0], pos[1]);
    return element;
  },

  getStyle: function(element, style) {
    element = $(element);
    style = style == ''float'' ? ''cssFloat'' : style.camelize();
    var value = element.style[style];
    if (!!value) {
      var css = document.defaultView.getComputedStyle(element, null);
      value = css ? css[style] : null;
    }
    if (style == ''opacity'') return value ? parseFloat(value) : 1.0;
    return value == ''auto'' ? null : value;
  },

  getOpacity: function(element) {
    return $(element).getStyle(''opacity'');
  },

  setStyle: function(element, styles) {
    element = $(element);
    var elementStyle = element.style, match;
    if (Object.isString(styles)) {
      element.style.cssText += '';'' + styles;
      return styles.include(''opacity'') ?
        element.setOpacity(styles.match(/opacity:\s*(\d?\.?\d*)/)[1]) : element;
    }
    for (var property in styles)
      if (property == ''opacity'') element.setOpacity(styles[property]);
      else
        elementStyle[(property == ''float'' || property == ''cssFloat'') ?
          (elementStyle.styleFloat === undefined ? ''cssFloat'' : ''styleFloat'') :
            property] = styles[property];

    return element;
  },

  setOpacity: function(element, value) {
    element = $(element);
    element.style.opacity = (value == 1 || value === '''') ? '''' :
      (value < 0.00001) ? 0 : value;
    return element;
  },

  getDimensions: function(element) {
    element = $(element);
    var display = $(element).getStyle(''display'');
    if (display !!= ''none'' && display !!= null) // Safari bug
      return {width: element.offsetWidth, height: element.offsetHeight};

    // All *Width and *Height properties give 0 on elements with display none,
    // so enable the element temporarily
    var els = element.style;
    var originalVisibility = els.visibility;
    var originalPosition = els.position;
    var originalDisplay = els.display;
    els.visibility = ''hidden'';
    els.position = ''absolute'';
    els.display = ''block'';
    var originalWidth = element.clientWidth;
    var originalHeight = element.clientHeight;
    els.display = originalDisplay;
    els.position = originalPosition;
    els.visibility = originalVisibility;
    return {width: originalWidth, height: originalHeight};
  },

  makePositioned: function(element) {
    element = $(element);
    var pos = Element.getStyle(element, ''position'');
    if (pos == ''static'' || !!pos) {
      element._madePositioned = true;
      element.style.position = ''relative'';
      // Opera returns the offset relative to the positioning context, when an
      // element is position relative but top and left have not been defined
      if (window.opera) {
        element.style.top = 0;
        element.style.left = 0;
      }
    }
    return element;
  },

  undoPositioned: function(element) {
    element = $(element);
    if (element._madePositioned) {
      element._madePositioned = undefined;
      element.style.position =
        element.style.top =
        element.style.left =
        element.style.bottom =
        element.style.right = '''';
    }
    return element;
  },

  makeClipping: function(element) {
    element = $(element);
    if (element._overflow) return element;
    element._overflow = Element.getStyle(element, ''overflow'') || ''auto'';
    if (element._overflow !!== ''hidden'')
      element.style.overflow = ''hidden'';
    return element;
  },

  undoClipping: function(element) {
    element = $(element);
    if (!!element._overflow) return element;
    element.style.overflow = element._overflow == ''auto'' ? '''' : element._overflow;
    element._overflow = null;
    return element;
  },

  cumulativeOffset: function(element) {
    var valueT = 0, valueL = 0;
    do {
      valueT += element.offsetTop  || 0;
      valueL += element.offsetLeft || 0;
      element = element.offsetParent;
    } while (element);
    return Element._returnOffset(valueL, valueT);
  },

  positionedOffset: function(element) {
    var valueT = 0, valueL = 0;
    do {
      valueT += element.offsetTop  || 0;
      valueL += element.offsetLeft || 0;
      element = element.offsetParent;
      if (element) {
        if (element.tagName == ''BODY'') break;
        var p = Element.getStyle(element, ''position'');
        if (p == ''relative'' || p == ''absolute'') break;
      }
    } while (element);
    return Element._returnOffset(valueL, valueT);
  },

  absolutize: function(element) {
    element = $(element);
    if (element.getStyle(''position'') == ''absolute'') return;
    // Position.prepare(); // To be done manually by Scripty when it needs it.

    var offsets = element.positionedOffset();
    var top     = offsets[1];
    var left    = offsets[0];
    var width   = element.clientWidth;
    var height  = element.clientHeight;

    element._originalLeft   = left - parseFloat(element.style.left  || 0);
    element._originalTop    = top  - parseFloat(element.style.top || 0);
    element._originalWidth  = element.style.width;
    element._originalHeight = element.style.height;

    element.style.position = ''absolute'';
    element.style.top    = top + ''px'';
    element.style.left   = left + ''px'';
    element.style.width  = width + ''px'';
    element.style.height = height + ''px'';
    return element;
  },

  relativize: function(element) {
    element = $(element);
    if (element.getStyle(''position'') == ''relative'') return;
    // Position.prepare(); // To be done manually by Scripty when it needs it.

    element.style.position = ''relative'';
    var top  = parseFloat(element.style.top  || 0) - (element._originalTop || 0);
    var left = parseFloat(element.style.left || 0) - (element._originalLeft || 0);

    element.style.top    = top + ''px'';
    element.style.left   = left + ''px'';
    element.style.height = element._originalHeight;
    element.style.width  = element._originalWidth;
    return element;
  },

  cumulativeScrollOffset: function(element) {
    var valueT = 0, valueL = 0;
    do {
      valueT += element.scrollTop  || 0;
      valueL += element.scrollLeft || 0;
      element = element.parentNode;
    } while (element);
    return Element._returnOffset(valueL, valueT);
  },

  getOffsetParent: function(element) {
    if (element.offsetParent) return $(element.offsetParent);
    if (element == document.body) return $(element);

    while ((element = element.parentNode) && element !!= document.body)
      if (Element.getStyle(element, ''position'') !!= ''static'')
        return $(element);

    return $(document.body);
  },

  viewportOffset: function(forElement) {
    var valueT = 0, valueL = 0;

    var element = forElement;
    do {
      valueT += element.offsetTop  || 0;
      valueL += element.offsetLeft || 0;

      // Safari fix
      if (element.offsetParent == document.body &&
        Element.getStyle(element, ''position'') == ''absolute'') break;

    } while (element = element.offsetParent);

    element = forElement;
    do {
      if (!!Prototype.Browser.Opera || element.tagName == ''BODY'') {
        valueT -= element.scrollTop  || 0;
        valueL -= element.scrollLeft || 0;
      }
    } while (element = element.parentNode);

    return Element._returnOffset(valueL, valueT);
  },

  clonePosition: function(element, source) {
    var options = Object.extend({
      setLeft:    true,
      setTop:     true,
      setWidth:   true,
      setHeight:  true,
      offsetTop:  0,
      offsetLeft: 0
    }, arguments[2] || { });

    // find page position of source
    source = $(source);
    var p = source.viewportOffset();

    // find coordinate system to use
    element = $(element);
    var delta = [0, 0];
    var parent = null;
    // delta [0,0] will do fine with position: fixed elements,
    // position:absolute needs offsetParent deltas
    if (Element.getStyle(element, ''position'') == ''absolute'') {
      parent = element.getOffsetParent();
      delta = parent.viewportOffset();
    }

    // correct by body offsets (fixes Safari)
    if (parent == document.body) {
      delta[0] -= document.body.offsetLeft;
      delta[1] -= document.body.offsetTop;
    }

    // set position
    if (options.setLeft)   element.style.left  = (p[0] - delta[0] + options.offsetLeft) + ''px'';
    if (options.setTop)    element.style.top   = (p[1] - delta[1] + options.offsetTop) + ''px'';
    if (options.setWidth)  element.style.width = source.offsetWidth + ''px'';
    if (options.setHeight) element.style.height = source.offsetHeight + ''px'';
    return element;
  }
};

Element.Methods.identify.counter = 1;

Object.extend(Element.Methods, {
  getElementsBySelector: Element.Methods.select,
  childElements: Element.Methods.immediateDescendants
});

Element._attributeTranslations = {
  write: {
    names: {
      className: ''class'',
      htmlFor:   ''for''
    },
    values: { }
  }
};


if (!!document.createRange || Prototype.Browser.Opera) {
  Element.Methods.insert = function(element, insertions) {
    element = $(element);

    if (Object.isString(insertions) || Object.isNumber(insertions) ||
        Object.isElement(insertions) || (insertions && (insertions.toElement || insertions.toHTML)))
          insertions = { bottom: insertions };

    var t = Element._insertionTranslations, content, position, pos, tagName;

    for (position in insertions) {
      content  = insertions[position];
      position = position.toLowerCase();
      pos      = t[position];

      if (content && content.toElement) content = content.toElement();
      if (Object.isElement(content)) {
        pos.insert(element, content);
        continue;
      }

      content = Object.toHTML(content);
      tagName = ((position == ''before'' || position == ''after'')
        ? element.parentNode : element).tagName.toUpperCase();

      if (t.tags[tagName]) {
        var fragments = Element._getContentFromAnonymousElement(tagName, content.stripScripts());
        if (position == ''top'' || position == ''after'') fragments.reverse();
        fragments.each(pos.insert.curry(element));
      }
      else element.insertAdjacentHTML(pos.adjacency, content.stripScripts());

      content.evalScripts.bind(content).defer();
    }

    return element;
  };
}

if (Prototype.Browser.Opera) {
  Element.Methods._getStyle = Element.Methods.getStyle;
  Element.Methods.getStyle = function(element, style) {
    switch(style) {
      case ''left'':
      case ''top'':
      case ''right'':
      case ''bottom'':
        if (Element._getStyle(element, ''position'') == ''static'') return null;
      default: return Element._getStyle(element, style);
    }
  };
  Element.Methods._readAttribute = Element.Methods.readAttribute;
  Element.Methods.readAttribute = function(element, attribute) {
    if (attribute == ''title'') return element.title;
    return Element._readAttribute(element, attribute);
  };
}

else if (Prototype.Browser.IE) {
  $w(''positionedOffset getOffsetParent viewportOffset'').each(function(method) {
    Element.Methods[method] = Element.Methods[method].wrap(
      function(proceed, element) {
        element = $(element);
        var position = element.getStyle(''position'');
        if (position !!= ''static'') return proceed(element);
        element.setStyle({ position: ''relative'' });
        var value = proceed(element);
        element.setStyle({ position: position });
        return value;
      }
    );
  });

  Element.Methods.getStyle = function(element, style) {
    element = $(element);
    style = (style == ''float'' || style == ''cssFloat'') ? ''styleFloat'' : style.camelize();
    var value = element.style[style];
    if (!!value && element.currentStyle) value = element.currentStyle[style];

    if (style == ''opacity'') {
      if (value = (element.getStyle(''filter'') || '''').match(/alpha\(opacity=(.*)\)/))
        if (value[1]) return parseFloat(value[1]) / 100;
      return 1.0;
    }

    if (value == ''auto'') {
      if ((style == ''width'' || style == ''height'') && (element.getStyle(''display'') !!= ''none''))
        return element[''offset'' + style.capitalize()] + ''px'';
      return null;
    }
    return value;
  };

  Element.Methods.setOpacity = function(element, value) {
    function stripAlpha(filter){
      return filter.replace(/alpha\([^\)]*\)/gi,'''');
    }
    element = $(element);
    var currentStyle = element.currentStyle;
    if ((currentStyle && !!currentStyle.hasLayout) ||
      (!!currentStyle && element.style.zoom == ''normal''))
        element.style.zoom = 1;

    var filter = element.getStyle(''filter''), style = element.style;
    if (value == 1 || value === '''') {
      (filter = stripAlpha(filter)) ?
        style.filter = filter : style.removeAttribute(''filter'');
      return element;
    } else if (value < 0.00001) value = 0;
    style.filter = stripAlpha(filter) +
      ''alpha(opacity='' + (value * 100) + '')'';
    return element;
  };

  Element._attributeTranslations = {
    read: {
      names: {
        ''class'': ''className'',
        ''for'':   ''htmlFor''
      },
      values: {
        _getAttr: function(element, attribute) {
          return element.getAttribute(attribute, 2);
        },
        _getAttrNode: function(element, attribute) {
          var node = element.getAttributeNode(attribute);
          return node ? node.value : "";
        },
        _getEv: function(element, attribute) {
          var attribute = element.getAttribute(attribute);
          return attribute ? attribute.toString().slice(23, -2) : null;
        },
        _flag: function(element, attribute) {
          return $(element).hasAttribute(attribute) ? attribute : null;
        },
        style: function(element) {
          return element.style.cssText.toLowerCase();
        },
        title: function(element) {
          return element.title;
        }
      }
    }
  };

  Element._attributeTranslations.write = {
    names: Object.clone(Element._attributeTranslations.read.names),
    values: {
      checked: function(element, value) {
        element.checked = !!!!value;
      },

      style: function(element, value) {
        element.style.cssText = value ? value : '''';
      }
    }
  };

  Element._attributeTranslations.has = {};

  $w(''colSpan rowSpan vAlign dateTime accessKey tabIndex '' +
      ''encType maxLength readOnly longDesc'').each(function(attr) {
    Element._attributeTranslations.write.names[attr.toLowerCase()] = attr;
    Element._attributeTranslations.has[attr.toLowerCase()] = attr;
  });

  (function(v) {
    Object.extend(v, {
      href:        v._getAttr,
      src:         v._getAttr,
      type:        v._getAttr,
      action:      v._getAttrNode,
      disabled:    v._flag,
      checked:     v._flag,
      readonly:    v._flag,
      multiple:    v._flag,
      onload:      v._getEv,
      onunload:    v._getEv,
      onclick:     v._getEv,
      ondblclick:  v._getEv,
      onmousedown: v._getEv,
      onmouseup:   v._getEv,
      onmouseover: v._getEv,
      onmousemove: v._getEv,
      onmouseout:  v._getEv,
      onfocus:     v._getEv,
      onblur:      v._getEv,
      onkeypress:  v._getEv,
      onkeydown:   v._getEv,
      onkeyup:     v._getEv,
      onsubmit:    v._getEv,
      onreset:     v._getEv,
      onselect:    v._getEv,
      onchange:    v._getEv
    });
  })(Element._attributeTranslations.read.values);
}

else if (Prototype.Browser.Gecko && /rv:1\.8\.0/.test(navigator.userAgent)) {
  Element.Methods.setOpacity = function(element, value) {
    element = $(element);
    element.style.opacity = (value == 1) ? 0.999999 :
      (value === '''') ? '''' : (value < 0.00001) ? 0 : value;
    return element;
  };
}

else if (Prototype.Browser.WebKit) {
  Element.Methods.setOpacity = function(element, value) {
    element = $(element);
    element.style.opacity = (value == 1 || value === '''') ? '''' :
      (value < 0.00001) ? 0 : value;

    if (value == 1)
      if(element.tagName == ''IMG'' && element.width) {
        element.width++; element.width--;
      } else try {
        var n = document.createTextNode('' '');
        element.appendChild(n);
        element.removeChild(n);
      } catch (e) { }

    return element;
  };

  // Safari returns margins on body which is incorrect if the child is absolutely
  // positioned.  For performance reasons, redefine Position.cumulativeOffset for
  // KHTML/WebKit only.
  Element.Methods.cumulativeOffset = function(element) {
    var valueT = 0, valueL = 0;
    do {
      valueT += element.offsetTop  || 0;
      valueL += element.offsetLeft || 0;
      if (element.offsetParent == document.body)
        if (Element.getStyle(element, ''position'') == ''absolute'') break;

      element = element.offsetParent;
    } while (element);

    return Element._returnOffset(valueL, valueT);
  };
}

if (Prototype.Browser.IE || Prototype.Browser.Opera) {
  // IE and Opera are missing .innerHTML support for TABLE-related and SELECT elements
  Element.Methods.update = function(element, content) {
    element = $(element);

    if (content && content.toElement) content = content.toElement();
    if (Object.isElement(content)) return element.update().insert(content);

    content = Object.toHTML(content);
    var tagName = element.tagName.toUpperCase();

    if (tagName in Element._insertionTranslations.tags) {
      $A(element.childNodes).each(function(node) { element.removeChild(node) });
      Element._getContentFromAnonymousElement(tagName, content.stripScripts())
        .each(function(node) { element.appendChild(node) });
    }
    else element.innerHTML = content.stripScripts();

    content.evalScripts.bind(content).defer();
    return element;
  };
}

if (document.createElement(''div'').outerHTML) {
  Element.Methods.replace = function(element, content) {
    element = $(element);

    if (content && content.toElement) content = content.toElement();
    if (Object.isElement(content)) {
      element.parentNode.replaceChild(content, element);
      return element;
    }

    content = Object.toHTML(content);
    var parent = element.parentNode, tagName = parent.tagName.toUpperCase();

    if (Element._insertionTranslations.tags[tagName]) {
      var nextSibling = element.next();
      var fragments = Element._getContentFromAnonymousElement(tagName, content.stripScripts());
      parent.removeChild(element);
      if (nextSibling)
        fragments.each(function(node) { parent.insertBefore(node, nextSibling) });
      else
        fragments.each(function(node) { parent.appendChild(node) });
    }
    else element.outerHTML = content.stripScripts();

    content.evalScripts.bind(content).defer();
    return element;
  };
}

Element._returnOffset = function(l, t) {
  var result = [l, t];
  result.left = l;
  result.top = t;
  return result;
};

Element._getContentFromAnonymousElement = function(tagName, html) {
  var div = new Element(''div''), t = Element._insertionTranslations.tags[tagName];
  div.innerHTML = t[0] + html + t[1];
  t[2].times(function() { div = div.firstChild });
  return $A(div.childNodes);
};

Element._insertionTranslations = {
  before: {
    adjacency: ''beforeBegin'',
    insert: function(element, node) {
      element.parentNode.insertBefore(node, element);
    },
    initializeRange: function(element, range) {
      range.setStartBefore(element);
    }
  },
  top: {
    adjacency: ''afterBegin'',
    insert: function(element, node) {
      element.insertBefore(node, element.firstChild);
    },
    initializeRange: function(element, range) {
      range.selectNodeContents(element);
      range.collapse(true);
    }
  },
  bottom: {
    adjacency: ''beforeEnd'',
    insert: function(element, node) {
      element.appendChild(node);
    }
  },
  after: {
    adjacency: ''afterEnd'',
    insert: function(element, node) {
      element.parentNode.insertBefore(node, element.nextSibling);
    },
    initializeRange: function(element, range) {
      range.setStartAfter(element);
    }
  },
  tags: {
    TABLE:  [''<table>'',                ''</table>'',                   1],
    TBODY:  [''<table><tbody>'',         ''</tbody></table>'',           2],
    TR:     [''<table><tbody><tr>'',     ''</tr></tbody></table>'',      3],
    TD:     [''<table><tbody><tr><td>'', ''</td></tr></tbody></table>'', 4],
    SELECT: [''<select>'',               ''</select>'',                  1]
  }
};

(function() {
  this.bottom.initializeRange = this.top.initializeRange;
  Object.extend(this.tags, {
    THEAD: this.tags.TBODY,
    TFOOT: this.tags.TBODY,
    TH:    this.tags.TD
  });
}).call(Element._insertionTranslations);

Element.Methods.Simulated = {
  hasAttribute: function(element, attribute) {
    attribute = Element._attributeTranslations.has[attribute] || attribute;
    var node = $(element).getAttributeNode(attribute);
    return node && node.specified;
  }
};

Element.Methods.ByTag = { };

Object.extend(Element, Element.Methods);

if (!!Prototype.BrowserFeatures.ElementExtensions &&
    document.createElement(''div'').__proto__) {
  window.HTMLElement = { };
  window.HTMLElement.prototype = document.createElement(''div'').__proto__;
  Prototype.BrowserFeatures.ElementExtensions = true;
}

Element.extend = (function() {
  if (Prototype.BrowserFeatures.SpecificElementExtensions)
    return Prototype.K;

  var Methods = { }, ByTag = Element.Methods.ByTag;

  var extend = Object.extend(function(element) {
    if (!!element || element._extendedByPrototype ||
        element.nodeType !!= 1 || element == window) return element;

    var methods = Object.clone(Methods),
      tagName = element.tagName, property, value;

    // extend methods for specific tags
    if (ByTag[tagName]) Object.extend(methods, ByTag[tagName]);

    for (property in methods) {
      value = methods[property];
      if (Object.isFunction(value) && !!(property in element))
        element[property] = value.methodize();
    }

    element._extendedByPrototype = Prototype.emptyFunction;
    return element;

  }, {
    refresh: function() {
      // extend methods for all tags (Safari doesn''t need this)
      if (!!Prototype.BrowserFeatures.ElementExtensions) {
        Object.extend(Methods, Element.Methods);
        Object.extend(Methods, Element.Methods.Simulated);
      }
    }
  });

  extend.refresh();
  return extend;
})();

Element.hasAttribute = function(element, attribute) {
  if (element.hasAttribute) return element.hasAttribute(attribute);
  return Element.Methods.Simulated.hasAttribute(element, attribute);
};

Element.addMethods = function(methods) {
  var F = Prototype.BrowserFeatures, T = Element.Methods.ByTag;

  if (!!methods) {
    Object.extend(Form, Form.Methods);
    Object.extend(Form.Element, Form.Element.Methods);
    Object.extend(Element.Methods.ByTag, {
      "FORM":     Object.clone(Form.Methods),
      "INPUT":    Object.clone(Form.Element.Methods),
      "SELECT":   Object.clone(Form.Element.Methods),
      "TEXTAREA": Object.clone(Form.Element.Methods)
    });
  }

  if (arguments.length == 2) {
    var tagName = methods;
    methods = arguments[1];
  }

  if (!!tagName) Object.extend(Element.Methods, methods || { });
  else {
    if (Object.isArray(tagName)) tagName.each(extend);
    else extend(tagName);
  }

  function extend(tagName) {
    tagName = tagName.toUpperCase();
    if (!!Element.Methods.ByTag[tagName])
      Element.Methods.ByTag[tagName] = { };
    Object.extend(Element.Methods.ByTag[tagName], methods);
  }

  function copy(methods, destination, onlyIfAbsent) {
    onlyIfAbsent = onlyIfAbsent || false;
    for (var property in methods) {
      var value = methods[property];
      if (!!Object.isFunction(value)) continue;
      if (!!onlyIfAbsent || !!(property in destination))
        destination[property] = value.methodize();
    }
  }

  function findDOMClass(tagName) {
    var klass;
    var trans = {
      "OPTGROUP": "OptGroup", "TEXTAREA": "TextArea", "P": "Paragraph",
      "FIELDSET": "FieldSet", "UL": "UList", "OL": "OList", "DL": "DList",
      "DIR": "Directory", "H1": "Heading", "H2": "Heading", "H3": "Heading",
      "H4": "Heading", "H5": "Heading", "H6": "Heading", "Q": "Quote",
      "INS": "Mod", "DEL": "Mod", "A": "Anchor", "IMG": "Image", "CAPTION":
      "TableCaption", "COL": "TableCol", "COLGROUP": "TableCol", "THEAD":
      "TableSection", "TFOOT": "TableSection", "TBODY": "TableSection", "TR":
      "TableRow", "TH": "TableCell", "TD": "TableCell", "FRAMESET":
      "FrameSet", "IFRAME": "IFrame"
    };
    if (trans[tagName]) klass = ''HTML'' + trans[tagName] + ''Element'';
    if (window[klass]) return window[klass];
    klass = ''HTML'' + tagName + ''Element'';
    if (window[klass]) return window[klass];
    klass = ''HTML'' + tagName.capitalize() + ''Element'';
    if (window[klass]) return window[klass];

    window[klass] = { };
    window[klass].prototype = document.createElement(tagName).__proto__;
    return window[klass];
  }

  if (F.ElementExtensions) {
    copy(Element.Methods, HTMLElement.prototype);
    copy(Element.Methods.Simulated, HTMLElement.prototype, true);
  }

  if (F.SpecificElementExtensions) {
    for (var tag in Element.Methods.ByTag) {
      var klass = findDOMClass(tag);
      if (Object.isUndefined(klass)) continue;
      copy(T[tag], klass.prototype);
    }
  }

  Object.extend(Element, Element.Methods);
  delete Element.ByTag;

  if (Element.extend.refresh) Element.extend.refresh();
  Element.cache = { };
};

document.viewport = {
  getDimensions: function() {
    var dimensions = { };
    $w(''width height'').each(function(d) {
      var D = d.capitalize();
      dimensions[d] = self[''inner'' + D] ||
       (document.documentElement[''client'' + D] || document.body[''client'' + D]);
    });
    return dimensions;
  },

  getWidth: function() {
    return this.getDimensions().width;
  },

  getHeight: function() {
    return this.getDimensions().height;
  },

  getScrollOffsets: function() {
    return Element._returnOffset(
      window.pageXOffset || document.documentElement.scrollLeft || document.body.scrollLeft,
      window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop);
  }
};
/* Portions of the Selector class are derived from Jack SlocumǨѢs DomQuery,
 * part of YUI-Ext version 0.40, distributed under the terms of an MIT-style
 * license.  Please see http://www.yui-ext.com/ for more information. */

var Selector = Class.create({
  initialize: function(expression) {
    this.expression = expression.strip();
    this.compileMatcher();
  },

  compileMatcher: function() {
    // Selectors with namespaced attributes can''t use the XPath version
    if (Prototype.BrowserFeatures.XPath && !!(/(\[[\w-]*?:|:checked)/).test(this.expression))
      return this.compileXPathMatcher();

    var e = this.expression, ps = Selector.patterns, h = Selector.handlers,
        c = Selector.criteria, le, p, m;

    if (Selector._cache[e]) {
      this.matcher = Selector._cache[e];
      return;
    }

    this.matcher = ["this.matcher = function(root) {",
                    "var r = root, h = Selector.handlers, c = false, n;"];

    while (e && le !!= e && (/\S/).test(e)) {
      le = e;
      for (var i in ps) {
        p = ps[i];
        if (m = e.match(p)) {
          this.matcher.push(Object.isFunction(c[i]) ? c[i](m) :
    	      new Template(c[i]).evaluate(m));
          e = e.replace(m[0], '''');
          break;
        }
      }
    }

    this.matcher.push("return h.unique(n);\n}");
    eval(this.matcher.join(''\n''));
    Selector._cache[this.expression] = this.matcher;
  },

  compileXPathMatcher: function() {
    var e = this.expression, ps = Selector.patterns,
        x = Selector.xpath, le, m;

    if (Selector._cache[e]) {
      this.xpath = Selector._cache[e]; return;
    }

    this.matcher = [''.//*''];
    while (e && le !!= e && (/\S/).test(e)) {
      le = e;
      for (var i in ps) {
        if (m = e.match(ps[i])) {
          this.matcher.push(Object.isFunction(x[i]) ? x[i](m) :
            new Template(x[i]).evaluate(m));
          e = e.replace(m[0], '''');
          break;
        }
      }
    }

    this.xpath = this.matcher.join('''');
    Selector._cache[this.expression] = this.xpath;
  },

  findElements: function(root) {
    root = root || document;
    if (this.xpath) return document._getElementsByXPath(this.xpath, root);
    return this.matcher(root);
  },

  match: function(element) {
    this.tokens = [];

    var e = this.expression, ps = Selector.patterns, as = Selector.assertions;
    var le, p, m;

    while (e && le !!== e && (/\S/).test(e)) {
      le = e;
      for (var i in ps) {
        p = ps[i];
        if (m = e.match(p)) {
          // use the Selector.assertions methods unless the selector
          // is too complex.
          if (as[i]) {
            this.tokens.push([i, Object.clone(m)]);
            e = e.replace(m[0], '''');
          } else {
            // reluctantly do a document-wide search
            // and look for a match in the array
            return this.findElements(document).include(element);
          }
        }
      }
    }

    var match = true, name, matches;
    for (var i = 0, token; token = this.tokens[i]; i++) {
      name = token[0], matches = token[1];
      if (!!Selector.assertions[name](element, matches)) {
        match = false; break;
      }
    }

    return match;
  },

  toString: function() {
    return this.expression;
  },

  inspect: function() {
    return "#<Selector:" + this.expression.inspect() + ">";
  }
});

Object.extend(Selector, {
  _cache: { },

  xpath: {
    descendant:   "//*",
    child:        "/*",
    adjacent:     "/following-sibling::*[1]",
    laterSibling: ''/following-sibling::*'',
    tagName:      function(m) {
      if (m[1] == ''*'') return '''';
      return "[local-name()=''" + m[1].toLowerCase() +
             "'' or local-name()=''" + m[1].toUpperCase() + "'']";
    },
    className:    "[contains(concat('' '', @class, '' ''), '' #{1} '')]",
    id:           "[@id=''#{1}'']",
    attrPresence: "[@#{1}]",
    attr: function(m) {
      m[3] = m[5] || m[6];
      return new Template(Selector.xpath.operators[m[2]]).evaluate(m);
    },
    pseudo: function(m) {
      var h = Selector.xpath.pseudos[m[1]];
      if (!!h) return '''';
      if (Object.isFunction(h)) return h(m);
      return new Template(Selector.xpath.pseudos[m[1]]).evaluate(m);
    },
    operators: {
      ''='':  "[@#{1}=''#{3}'']",
      ''!!='': "[@#{1}!!=''#{3}'']",
      ''^='': "[starts-with(@#{1}, ''#{3}'')]",
      ''$='': "[substring(@#{1}, (string-length(@#{1}) - string-length(''#{3}'') + 1))=''#{3}'']",
      ''*='': "[contains(@#{1}, ''#{3}'')]",
      ''~='': "[contains(concat('' '', @#{1}, '' ''), '' #{3} '')]",
      ''|='': "[contains(concat(''-'', @#{1}, ''-''), ''-#{3}-'')]"
    },
    pseudos: {
      ''first-child'': ''[not(preceding-sibling::*)]'',
      ''last-child'':  ''[not(following-sibling::*)]'',
      ''only-child'':  ''[not(preceding-sibling::* or following-sibling::*)]'',
      ''empty'':       "[count(*) = 0 and (count(text()) = 0 or translate(text(), '' \t\r\n'', '''') = '''')]",
      ''checked'':     "[@checked]",
      ''disabled'':    "[@disabled]",
      ''enabled'':     "[not(@disabled)]",
      ''not'': function(m) {
        var e = m[6], p = Selector.patterns,
            x = Selector.xpath, le, m, v;

        var exclusion = [];
        while (e && le !!= e && (/\S/).test(e)) {
          le = e;
          for (var i in p) {
            if (m = e.match(p[i])) {
              v = Object.isFunction(x[i]) ? x[i](m) : new Template(x[i]).evaluate(m);
              exclusion.push("(" + v.substring(1, v.length - 1) + ")");
              e = e.replace(m[0], '''');
              break;
            }
          }
        }
        return "[not(" + exclusion.join(" and ") + ")]";
      },
      ''nth-child'':      function(m) {
        return Selector.xpath.pseudos.nth("(count(./preceding-sibling::*) + 1) ", m);
      },
      ''nth-last-child'': function(m) {
        return Selector.xpath.pseudos.nth("(count(./following-sibling::*) + 1) ", m);
      },
      ''nth-of-type'':    function(m) {
        return Selector.xpath.pseudos.nth("position() ", m);
      },
      ''nth-last-of-type'': function(m) {
        return Selector.xpath.pseudos.nth("(last() + 1 - position()) ", m);
      },
      ''first-of-type'':  function(m) {
        m[6] = "1"; return Selector.xpath.pseudos[''nth-of-type''](m);
      },
      ''last-of-type'':   function(m) {
        m[6] = "1"; return Selector.xpath.pseudos[''nth-last-of-type''](m);
      },
      ''only-of-type'':   function(m) {
        var p = Selector.xpath.pseudos; return p[''first-of-type''](m) + p[''last-of-type''](m);
      },
      nth: function(fragment, m) {
        var mm, formula = m[6], predicate;
        if (formula == ''even'') formula = ''2n+0'';
        if (formula == ''odd'')  formula = ''2n+1'';
        if (mm = formula.match(/^(\d+)$/)) // digit only
          return ''['' + fragment + "= " + mm[1] + '']'';
        if (mm = formula.match(/^(-?\d*)?n(([+-])(\d+))?/)) { // an+b
          if (mm[1] == "-") mm[1] = -1;
          var a = mm[1] ? Number(mm[1]) : 1;
          var b = mm[2] ? Number(mm[2]) : 0;
          predicate = "[((#{fragment} - #{b}) mod #{a} = 0) and " +
          "((#{fragment} - #{b}) div #{a} >= 0)]";
          return new Template(predicate).evaluate({
            fragment: fragment, a: a, b: b });
        }
      }
    }
  },

  criteria: {
    tagName:      ''n = h.tagName(n, r, "#{1}", c);   c = false;'',
    className:    ''n = h.className(n, r, "#{1}", c); c = false;'',
    id:           ''n = h.id(n, r, "#{1}", c);        c = false;'',
    attrPresence: ''n = h.attrPresence(n, r, "#{1}"); c = false;'',
    attr: function(m) {
      m[3] = (m[5] || m[6]);
      return new Template(''n = h.attr(n, r, "#{1}", "#{3}", "#{2}"); c = false;'').evaluate(m);
    },
    pseudo: function(m) {
      if (m[6]) m[6] = m[6].replace(/"/g, ''\\"'');
      return new Template(''n = h.pseudo(n, "#{1}", "#{6}", r, c); c = false;'').evaluate(m);
    },
    descendant:   ''c = "descendant";'',
    child:        ''c = "child";'',
    adjacent:     ''c = "adjacent";'',
    laterSibling: ''c = "laterSibling";''
  },

  patterns: {
    // combinators must be listed first
    // (and descendant needs to be last combinator)
    laterSibling: /^\s*~\s*/,
    child:        /^\s*>\s*/,
    adjacent:     /^\s*\+\s*/,
    descendant:   /^\s/,

    // selectors follow
    tagName:      /^\s*(\*|[\w\-]+)(\b|$)?/,
    id:           /^#([\w\-\*]+)(\b|$)/,
    className:    /^\.([\w\-\*]+)(\b|$)/,
    pseudo:       /^:((first|last|nth|nth-last|only)(-child|-of-type)|empty|checked|(en|dis)abled|not)(\((.*?)\))?(\b|$|(?=\s)|(?=:))/,
    attrPresence: /^\[([\w]+)\]/,
    attr:         /\[((?:[\w-]*:)?[\w-]+)\s*(?:([!!^$*~|]?=)\s*(([''"])([^\4]*?)\4|([^''"][^\]]*?)))?\]/
  },

  // for Selector.match and Element#match
  assertions: {
    tagName: function(element, matches) {
      return matches[1].toUpperCase() == element.tagName.toUpperCase();
    },

    className: function(element, matches) {
      return Element.hasClassName(element, matches[1]);
    },

    id: function(element, matches) {
      return element.id === matches[1];
    },

    attrPresence: function(element, matches) {
      return Element.hasAttribute(element, matches[1]);
    },

    attr: function(element, matches) {
      var nodeValue = Element.readAttribute(element, matches[1]);
      return Selector.operators[matches[2]](nodeValue, matches[3]);
    }
  },

  handlers: {
    // UTILITY FUNCTIONS
    // joins two collections
    concat: function(a, b) {
      for (var i = 0, node; node = b[i]; i++)
        a.push(node);
      return a;
    },

    // marks an array of nodes for counting
    mark: function(nodes) {
      for (var i = 0, node; node = nodes[i]; i++)
        node._counted = true;
      return nodes;
    },

    unmark: function(nodes) {
      for (var i = 0, node; node = nodes[i]; i++)
        node._counted = undefined;
      return nodes;
    },

    // mark each child node with its position (for nth calls)
    // "ofType" flag indicates whether we''re indexing for nth-of-type
    // rather than nth-child
    index: function(parentNode, reverse, ofType) {
      parentNode._counted = true;
      if (reverse) {
        for (var nodes = parentNode.childNodes, i = nodes.length - 1, j = 1; i >= 0; i--) {
          var node = nodes[i];
          if (node.nodeType == 1 && (!!ofType || node._counted)) node.nodeIndex = j++;
        }
      } else {
        for (var i = 0, j = 1, nodes = parentNode.childNodes; node = nodes[i]; i++)
          if (node.nodeType == 1 && (!!ofType || node._counted)) node.nodeIndex = j++;
      }
    },

    // filters out duplicates and extends all nodes
    unique: function(nodes) {
      if (nodes.length == 0) return nodes;
      var results = [], n;
      for (var i = 0, l = nodes.length; i < l; i++)
        if (!!(n = nodes[i])._counted) {
          n._counted = true;
          results.push(Element.extend(n));
        }
      return Selector.handlers.unmark(results);
    },

    // COMBINATOR FUNCTIONS
    descendant: function(nodes) {
      var h = Selector.handlers;
      for (var i = 0, results = [], node; node = nodes[i]; i++)
        h.concat(results, node.getElementsByTagName(''*''));
      return results;
    },

    child: function(nodes) {
      var h = Selector.handlers;
      for (var i = 0, results = [], node; node = nodes[i]; i++) {
        for (var j = 0, children = [], child; child = node.childNodes[j]; j++)
          if (child.nodeType == 1 && child.tagName !!= ''!!'') results.push(child);
      }
      return results;
    },

    adjacent: function(nodes) {
      for (var i = 0, results = [], node; node = nodes[i]; i++) {
        var next = this.nextElementSibling(node);
        if (next) results.push(next);
      }
      return results;
    },

    laterSibling: function(nodes) {
      var h = Selector.handlers;
      for (var i = 0, results = [], node; node = nodes[i]; i++)
        h.concat(results, Element.nextSiblings(node));
      return results;
    },

    nextElementSibling: function(node) {
      while (node = node.nextSibling)
	      if (node.nodeType == 1) return node;
      return null;
    },

    previousElementSibling: function(node) {
      while (node = node.previousSibling)
        if (node.nodeType == 1) return node;
      return null;
    },

    // TOKEN FUNCTIONS
    tagName: function(nodes, root, tagName, combinator) {
      tagName = tagName.toUpperCase();
      var results = [], h = Selector.handlers;
      if (nodes) {
        if (combinator) {
          // fastlane for ordinary descendant combinators
          if (combinator == "descendant") {
            for (var i = 0, node; node = nodes[i]; i++)
              h.concat(results, node.getElementsByTagName(tagName));
            return results;
          } else nodes = this[combinator](nodes);
          if (tagName == "*") return nodes;
        }
        for (var i = 0, node; node = nodes[i]; i++)
          if (node.tagName.toUpperCase() == tagName) results.push(node);
        return results;
      } else return root.getElementsByTagName(tagName);
    },

    id: function(nodes, root, id, combinator) {
      var targetNode = $(id), h = Selector.handlers;
      if (!!targetNode) return [];
      if (!!nodes && root == document) return [targetNode];
      if (nodes) {
        if (combinator) {
          if (combinator == ''child'') {
            for (var i = 0, node; node = nodes[i]; i++)
              if (targetNode.parentNode == node) return [targetNode];
          } else if (combinator == ''descendant'') {
            for (var i = 0, node; node = nodes[i]; i++)
              if (Element.descendantOf(targetNode, node)) return [targetNode];
          } else if (combinator == ''adjacent'') {
            for (var i = 0, node; node = nodes[i]; i++)
              if (Selector.handlers.previousElementSibling(targetNode) == node)
                return [targetNode];
          } else nodes = h[combinator](nodes);
        }
        for (var i = 0, node; node = nodes[i]; i++)
          if (node == targetNode) return [targetNode];
        return [];
      }
      return (targetNode && Element.descendantOf(targetNode, root)) ? [targetNode] : [];
    },

    className: function(nodes, root, className, combinator) {
      if (nodes && combinator) nodes = this[combinator](nodes);
      return Selector.handlers.byClassName(nodes, root, className);
    },

    byClassName: function(nodes, root, className) {
      if (!!nodes) nodes = Selector.handlers.descendant([root]);
      var needle = '' '' + className + '' '';
      for (var i = 0, results = [], node, nodeClassName; node = nodes[i]; i++) {
        nodeClassName = node.className;
        if (nodeClassName.length == 0) continue;
        if (nodeClassName == className || ('' '' + nodeClassName + '' '').include(needle))
          results.push(node);
      }
      return results;
    },

    attrPresence: function(nodes, root, attr) {
      if (!!nodes) nodes = root.getElementsByTagName("*");
      var results = [];
      for (var i = 0, node; node = nodes[i]; i++)
        if (Element.hasAttribute(node, attr)) results.push(node);
      return results;
    },

    attr: function(nodes, root, attr, value, operator) {
      if (!!nodes) nodes = root.getElementsByTagName("*");
      var handler = Selector.operators[operator], results = [];
      for (var i = 0, node; node = nodes[i]; i++) {
        var nodeValue = Element.readAttribute(node, attr);
        if (nodeValue === null) continue;
        if (handler(nodeValue, value)) results.push(node);
      }
      return results;
    },

    pseudo: function(nodes, name, value, root, combinator) {
      if (nodes && combinator) nodes = this[combinator](nodes);
      if (!!nodes) nodes = root.getElementsByTagName("*");
      return Selector.pseudos[name](nodes, value, root);
    }
  },

  pseudos: {
    ''first-child'': function(nodes, value, root) {
      for (var i = 0, results = [], node; node = nodes[i]; i++) {
        if (Selector.handlers.previousElementSibling(node)) continue;
          results.push(node);
      }
      return results;
    },
    ''last-child'': function(nodes, value, root) {
      for (var i = 0, results = [], node; node = nodes[i]; i++) {
        if (Selector.handlers.nextElementSibling(node)) continue;
          results.push(node);
      }
      return results;
    },
    ''only-child'': function(nodes, value, root) {
      var h = Selector.handlers;
      for (var i = 0, results = [], node; node = nodes[i]; i++)
        if (!!h.previousElementSibling(node) && !!h.nextElementSibling(node))
          results.push(node);
      return results;
    },
    ''nth-child'':        function(nodes, formula, root) {
      return Selector.pseudos.nth(nodes, formula, root);
    },
    ''nth-last-child'':   function(nodes, formula, root) {
      return Selector.pseudos.nth(nodes, formula, root, true);
    },
    ''nth-of-type'':      function(nodes, formula, root) {
      return Selector.pseudos.nth(nodes, formula, root, false, true);
    },
    ''nth-last-of-type'': function(nodes, formula, root) {
      return Selector.pseudos.nth(nodes, formula, root, true, true);
    },
    ''first-of-type'':    function(nodes, formula, root) {
      return Selector.pseudos.nth(nodes, "1", root, false, true);
    },
    ''last-of-type'':     function(nodes, formula, root) {
      return Selector.pseudos.nth(nodes, "1", root, true, true);
    },
    ''only-of-type'':     function(nodes, formula, root) {
      var p = Selector.pseudos;
      return p[''last-of-type''](p[''first-of-type''](nodes, formula, root), formula, root);
    },

    // handles the an+b logic
    getIndices: function(a, b, total) {
      if (a == 0) return b > 0 ? [b] : [];
      return $R(1, total).inject([], function(memo, i) {
        if (0 == (i - b) % a && (i - b) / a >= 0) memo.push(i);
        return memo;
      });
    },

    // handles nth(-last)-child, nth(-last)-of-type, and (first|last)-of-type
    nth: function(nodes, formula, root, reverse, ofType) {
      if (nodes.length == 0) return [];
      if (formula == ''even'') formula = ''2n+0'';
      if (formula == ''odd'')  formula = ''2n+1'';
      var h = Selector.handlers, results = [], indexed = [], m;
      h.mark(nodes);
      for (var i = 0, node; node = nodes[i]; i++) {
        if (!!node.parentNode._counted) {
          h.index(node.parentNode, reverse, ofType);
          indexed.push(node.parentNode);
        }
      }
      if (formula.match(/^\d+$/)) { // just a number
        formula = Number(formula);
        for (var i = 0, node; node = nodes[i]; i++)
          if (node.nodeIndex == formula) results.push(node);
      } else if (m = formula.match(/^(-?\d*)?n(([+-])(\d+))?/)) { // an+b
        if (m[1] == "-") m[1] = -1;
        var a = m[1] ? Number(m[1]) : 1;
        var b = m[2] ? Number(m[2]) : 0;
        var indices = Selector.pseudos.getIndices(a, b, nodes.length);
        for (var i = 0, node, l = indices.length; node = nodes[i]; i++) {
          for (var j = 0; j < l; j++)
            if (node.nodeIndex == indices[j]) results.push(node);
        }
      }
      h.unmark(nodes);
      h.unmark(indexed);
      return results;
    },

    ''empty'': function(nodes, value, root) {
      for (var i = 0, results = [], node; node = nodes[i]; i++) {
        // IE treats comments as element nodes
        if (node.tagName == ''!!'' || (node.firstChild && !!node.innerHTML.match(/^\s*$/))) continue;
        results.push(node);
      }
      return results;
    },

    ''not'': function(nodes, selector, root) {
      var h = Selector.handlers, selectorType, m;
      var exclusions = new Selector(selector).findElements(root);
      h.mark(exclusions);
      for (var i = 0, results = [], node; node = nodes[i]; i++)
        if (!!node._counted) results.push(node);
      h.unmark(exclusions);
      return results;
    },

    ''enabled'': function(nodes, value, root) {
      for (var i = 0, results = [], node; node = nodes[i]; i++)
        if (!!node.disabled) results.push(node);
      return results;
    },

    ''disabled'': function(nodes, value, root) {
      for (var i = 0, results = [], node; node = nodes[i]; i++)
        if (node.disabled) results.push(node);
      return results;
    },

    ''checked'': function(nodes, value, root) {
      for (var i = 0, results = [], node; node = nodes[i]; i++)
        if (node.checked) results.push(node);
      return results;
    }
  },

  operators: {
    ''='':  function(nv, v) { return nv == v; },
    ''!!='': function(nv, v) { return nv !!= v; },
    ''^='': function(nv, v) { return nv.startsWith(v); },
    ''$='': function(nv, v) { return nv.endsWith(v); },
    ''*='': function(nv, v) { return nv.include(v); },
    ''~='': function(nv, v) { return ('' '' + nv + '' '').include('' '' + v + '' ''); },
    ''|='': function(nv, v) { return (''-'' + nv.toUpperCase() + ''-'').include(''-'' + v.toUpperCase() + ''-''); }
  },

  matchElements: function(elements, expression) {
    var matches = new Selector(expression).findElements(), h = Selector.handlers;
    h.mark(matches);
    for (var i = 0, results = [], element; element = elements[i]; i++)
      if (element._counted) results.push(element);
    h.unmark(matches);
    return results;
  },

  findElement: function(elements, expression, index) {
    if (Object.isNumber(expression)) {
      index = expression; expression = false;
    }
    return Selector.matchElements(elements, expression || ''*'')[index || 0];
  },

  findChildElements: function(element, expressions) {
    var exprs = expressions.join('',''), expressions = [];
    exprs.scan(/(([\w#:.~>+()\s-]+|\*|\[.*?\])+)\s*(,|$)/, function(m) {
      expressions.push(m[1].strip());
    });
    var results = [], h = Selector.handlers;
    for (var i = 0, l = expressions.length, selector; i < l; i++) {
      selector = new Selector(expressions[i].strip());
      h.concat(results, selector.findElements(element));
    }
    return (l > 1) ? h.unique(results) : results;
  }
});

function $$() {
  return Selector.findChildElements(document, $A(arguments));
}
var Form = {
  reset: function(form) {
    $(form).reset();
    return form;
  },

  serializeElements: function(elements, options) {
    if (typeof options !!= ''object'') options = { hash: !!!!options };
    else if (options.hash === undefined) options.hash = true;
    var key, value, submitted = false, submit = options.submit;

    var data = elements.inject({ }, function(result, element) {
      if (!!element.disabled && element.name) {
        key = element.name; value = $(element).getValue();
        if (value !!= null && (element.type !!= ''submit'' || (!!submitted &&
            submit !!== false && (!!submit || key == submit) && (submitted = true)))) {
          if (key in result) {
            // a key is already present; construct an array of values
            if (!!Object.isArray(result[key])) result[key] = [result[key]];
            result[key].push(value);
          }
          else result[key] = value;
        }
      }
      return result;
    });

    return options.hash ? data : Object.toQueryString(data);
  }
};

Form.Methods = {
  serialize: function(form, options) {
    return Form.serializeElements(Form.getElements(form), options);
  },

  getElements: function(form) {
    return $A($(form).getElementsByTagName(''*'')).inject([],
      function(elements, child) {
        if (Form.Element.Serializers[child.tagName.toLowerCase()])
          elements.push(Element.extend(child));
        return elements;
      }
    );
  },

  getInputs: function(form, typeName, name) {
    form = $(form);
    var inputs = form.getElementsByTagName(''input'');

    if (!!typeName && !!name) return $A(inputs).map(Element.extend);

    for (var i = 0, matchingInputs = [], length = inputs.length; i < length; i++) {
      var input = inputs[i];
      if ((typeName && input.type !!= typeName) || (name && input.name !!= name))
        continue;
      matchingInputs.push(Element.extend(input));
    }

    return matchingInputs;
  },

  disable: function(form) {
    form = $(form);
    Form.getElements(form).invoke(''disable'');
    return form;
  },

  enable: function(form) {
    form = $(form);
    Form.getElements(form).invoke(''enable'');
    return form;
  },

  findFirstElement: function(form) {
    var elements = $(form).getElements().findAll(function(element) {
      return ''hidden'' !!= element.type && !!element.disabled;
    });
    var firstByIndex = elements.findAll(function(element) {
      return element.hasAttribute(''tabIndex'') && element.tabIndex >= 0;
    }).sortBy(function(element) { return element.tabIndex }).first();

    return firstByIndex ? firstByIndex : elements.find(function(element) {
      return [''input'', ''select'', ''textarea''].include(element.tagName.toLowerCase());
    });
  },

  focusFirstElement: function(form) {
    form = $(form);
    form.findFirstElement().activate();
    return form;
  },

  request: function(form, options) {
    form = $(form), options = Object.clone(options || { });

    var params = options.parameters, action = form.readAttribute(''action'') || '''';
    if (action.blank()) action = window.location.href;
    options.parameters = form.serialize(true);

    if (params) {
      if (Object.isString(params)) params = params.toQueryParams();
      Object.extend(options.parameters, params);
    }

    if (form.hasAttribute(''method'') && !!options.method)
      options.method = form.method;

    return new Ajax.Request(action, options);
  }
};

/*--------------------------------------------------------------------------*/

Form.Element = {
  focus: function(element) {
    $(element).focus();
    return element;
  },

  select: function(element) {
    $(element).select();
    return element;
  }
};

Form.Element.Methods = {
  serialize: function(element) {
    element = $(element);
    if (!!element.disabled && element.name) {
      var value = element.getValue();
      if (value !!= undefined) {
        var pair = { };
        pair[element.name] = value;
        return Object.toQueryString(pair);
      }
    }
    return '''';
  },

  getValue: function(element) {
    element = $(element);
    var method = element.tagName.toLowerCase();
    return Form.Element.Serializers[method](element);
  },

  setValue: function(element, value) {
    element = $(element);
    var method = element.tagName.toLowerCase();
    Form.Element.Serializers[method](element, value);
    return element;
  },

  clear: function(element) {
    $(element).value = '''';
    return element;
  },

  present: function(element) {
    return $(element).value !!= '''';
  },

  activate: function(element) {
    element = $(element);
    try {
      element.focus();
      if (element.select && (element.tagName.toLowerCase() !!= ''input'' ||
          !![''button'', ''reset'', ''submit''].include(element.type)))
        element.select();
    } catch (e) { }
    return element;
  },

  disable: function(element) {
    element = $(element);
    element.blur();
    element.disabled = true;
    return element;
  },

  enable: function(element) {
    element = $(element);
    element.disabled = false;
    return element;
  }
};

/*--------------------------------------------------------------------------*/

var Field = Form.Element;
var $F = Form.Element.Methods.getValue;

/*--------------------------------------------------------------------------*/

Form.Element.Serializers = {
  input: function(element, value) {
    switch (element.type.toLowerCase()) {
      case ''checkbox'':
      case ''radio'':
        return Form.Element.Serializers.inputSelector(element, value);
      default:
        return Form.Element.Serializers.textarea(element, value);
    }
  },

  inputSelector: function(element, value) {
    if (value === undefined) return element.checked ? element.value : null;
    else element.checked = !!!!value;
  },

  textarea: function(element, value) {
    if (value === undefined) return element.value;
    else element.value = value;
  },

  select: function(element, index) {
    if (index === undefined)
      return this[element.type == ''select-one'' ?
        ''selectOne'' : ''selectMany''](element);
    else {
      var opt, value, single = !!Object.isArray(index);
      for (var i = 0, length = element.length; i < length; i++) {
        opt = element.options[i];
        value = this.optionValue(opt);
        if (single) {
          if (value == index) {
            opt.selected = true;
            return;
          }
        }
        else opt.selected = index.include(value);
      }
    }
  },

  selectOne: function(element) {
    var index = element.selectedIndex;
    return index >= 0 ? this.optionValue(element.options[index]) : null;
  },

  selectMany: function(element) {
    var values, length = element.length;
    if (!!length) return null;

    for (var i = 0, values = []; i < length; i++) {
      var opt = element.options[i];
      if (opt.selected) values.push(this.optionValue(opt));
    }
    return values;
  },

  optionValue: function(opt) {
    // extend element because hasAttribute may not be native
    return Element.extend(opt).hasAttribute(''value'') ? opt.value : opt.text;
  }
};

/*--------------------------------------------------------------------------*/

Abstract.TimedObserver = Class.create(PeriodicalExecuter, {
  initialize: function($super, element, frequency, callback) {
    $super(callback, frequency);
    this.element   = $(element);
    this.lastValue = this.getValue();
  },

  execute: function() {
    var value = this.getValue();
    if (Object.isString(this.lastValue) && Object.isString(value) ?
        this.lastValue !!= value : String(this.lastValue) !!= String(value)) {
      this.callback(this.element, value);
      this.lastValue = value;
    }
  }
});

Form.Element.Observer = Class.create(Abstract.TimedObserver, {
  getValue: function() {
    return Form.Element.getValue(this.element);
  }
});

Form.Observer = Class.create(Abstract.TimedObserver, {
  getValue: function() {
    return Form.serialize(this.element);
  }
});

/*--------------------------------------------------------------------------*/

Abstract.EventObserver = Class.create({
  initialize: function(element, callback) {
    this.element  = $(element);
    this.callback = callback;

    this.lastValue = this.getValue();
    if (this.element.tagName.toLowerCase() == ''form'')
      this.registerFormCallbacks();
    else
      this.registerCallback(this.element);
  },

  onElementEvent: function() {
    var value = this.getValue();
    if (this.lastValue !!= value) {
      this.callback(this.element, value);
      this.lastValue = value;
    }
  },

  registerFormCallbacks: function() {
    Form.getElements(this.element).each(this.registerCallback, this);
  },

  registerCallback: function(element) {
    if (element.type) {
      switch (element.type.toLowerCase()) {
        case ''checkbox'':
        case ''radio'':
          Event.observe(element, ''click'', this.onElementEvent.bind(this));
          break;
        default:
          Event.observe(element, ''change'', this.onElementEvent.bind(this));
          break;
      }
    }
  }
});

Form.Element.EventObserver = Class.create(Abstract.EventObserver, {
  getValue: function() {
    return Form.Element.getValue(this.element);
  }
});

Form.EventObserver = Class.create(Abstract.EventObserver, {
  getValue: function() {
    return Form.serialize(this.element);
  }
});
if (!!window.Event) var Event = { };

Object.extend(Event, {
  KEY_BACKSPACE: 8,
  KEY_TAB:       9,
  KEY_RETURN:   13,
  KEY_ESC:      27,
  KEY_LEFT:     37,
  KEY_UP:       38,
  KEY_RIGHT:    39,
  KEY_DOWN:     40,
  KEY_DELETE:   46,
  KEY_HOME:     36,
  KEY_END:      35,
  KEY_PAGEUP:   33,
  KEY_PAGEDOWN: 34,
  KEY_INSERT:   45,

  cache: { },

  relatedTarget: function(event) {
    var element;
    switch(event.type) {
      case ''mouseover'': element = event.fromElement; break;
      case ''mouseout'':  element = event.toElement;   break;
      default: return null;
    }
    return Element.extend(element);
  }
});

Event.Methods = (function() {
  var isButton;

  if (Prototype.Browser.IE) {
    var buttonMap = { 0: 1, 1: 4, 2: 2 };
    isButton = function(event, code) {
      return event.button == buttonMap[code];
    };

  } else if (Prototype.Browser.WebKit) {
    isButton = function(event, code) {
      switch (code) {
        case 0: return event.which == 1 && !!event.metaKey;
        case 1: return event.which == 1 && event.metaKey;
        default: return false;
      }
    };

  } else {
    isButton = function(event, code) {
      return event.which ? (event.which === code + 1) : (event.button === code);
    };
  }

  return {
    isLeftClick:   function(event) { return isButton(event, 0) },
    isMiddleClick: function(event) { return isButton(event, 1) },
    isRightClick:  function(event) { return isButton(event, 2) },

    element: function(event) {
      var node = Event.extend(event).target;
      return Element.extend(node.nodeType == Node.TEXT_NODE ? node.parentNode : node);
    },

    findElement: function(event, expression) {
      var element = Event.element(event);
      return element.match(expression) ? element : element.up(expression);
    },

    pointer: function(event) {
      return {
        x: event.pageX || (event.clientX +
          (document.documentElement.scrollLeft || document.body.scrollLeft)),
        y: event.pageY || (event.clientY +
          (document.documentElement.scrollTop || document.body.scrollTop))
      };
    },

    pointerX: function(event) { return Event.pointer(event).x },
    pointerY: function(event) { return Event.pointer(event).y },

    stop: function(event) {
      Event.extend(event);
      event.preventDefault();
      event.stopPropagation();
      event.stopped = true;
    }
  };
})();

Event.extend = (function() {
  var methods = Object.keys(Event.Methods).inject({ }, function(m, name) {
    m[name] = Event.Methods[name].methodize();
    return m;
  });

  if (Prototype.Browser.IE) {
    Object.extend(methods, {
      stopPropagation: function() { this.cancelBubble = true },
      preventDefault:  function() { this.returnValue = false },
      inspect: function() { return "[object Event]" }
    });

    return function(event) {
      if (!!event) return false;
      if (event._extendedByPrototype) return event;

      event._extendedByPrototype = Prototype.emptyFunction;
      var pointer = Event.pointer(event);
      Object.extend(event, {
        target: event.srcElement,
        relatedTarget: Event.relatedTarget(event),
        pageX:  pointer.x,
        pageY:  pointer.y
      });
      return Object.extend(event, methods);
    };

  } else {
    Event.prototype = Event.prototype || document.createEvent("HTMLEvents").__proto__;
    Object.extend(Event.prototype, methods);
    return Prototype.K;
  }
})();

Object.extend(Event, (function() {
  var cache = Event.cache;

  function getEventID(element) {
    if (element._eventID) return element._eventID;
    arguments.callee.id = arguments.callee.id || 1;
    return element._eventID = ++arguments.callee.id;
  }

  function getDOMEventName(eventName) {
    if (eventName && eventName.include('':'')) return "dataavailable";
    return eventName;
  }

  function getCacheForID(id) {
    return cache[id] = cache[id] || { };
  }

  function getWrappersForEventName(id, eventName) {
    var c = getCacheForID(id);
    return c[eventName] = c[eventName] || [];
  }

  function createWrapper(element, eventName, handler) {
    var id = getEventID(element);
    var c = getWrappersForEventName(id, eventName);
    if (c.pluck("handler").include(handler)) return false;

    var wrapper = function(event) {
      if (!!Event || !!Event.extend ||
        (event.eventName && event.eventName !!= eventName))
          return false;

      Event.extend(event);
      handler.call(element, event)
    };

    wrapper.handler = handler;
    c.push(wrapper);
    return wrapper;
  }

  function findWrapper(id, eventName, handler) {
    var c = getWrappersForEventName(id, eventName);
    return c.find(function(wrapper) { return wrapper.handler == handler });
  }

  function destroyWrapper(id, eventName, handler) {
    var c = getCacheForID(id);
    if (!!c[eventName]) return false;
    c[eventName] = c[eventName].without(findWrapper(id, eventName, handler));
  }

  function destroyCache() {
    for (var id in cache)
      for (var eventName in cache[id])
        cache[id][eventName] = null;
  }

  if (window.attachEvent) {
    window.attachEvent("onunload", destroyCache);
  }

  return {
    observe: function(element, eventName, handler) {
      element = $(element);
      var name = getDOMEventName(eventName);

      var wrapper = createWrapper(element, eventName, handler);
      if (!!wrapper) return element;

      if (element.addEventListener) {
        element.addEventListener(name, wrapper, false);
      } else {
        element.attachEvent("on" + name, wrapper);
      }

      return element;
    },

    stopObserving: function(element, eventName, handler) {
      element = $(element);
      var id = getEventID(element), name = getDOMEventName(eventName);

      if (!!handler && eventName) {
        getWrappersForEventName(id, eventName).each(function(wrapper) {
          element.stopObserving(eventName, wrapper.handler);
        });
        return element;

      } else if (!!eventName) {
        Object.keys(getCacheForID(id)).each(function(eventName) {
          element.stopObserving(eventName);
        });
        return element;
      }

      var wrapper = findWrapper(id, eventName, handler);
      if (!!wrapper) return element;

      if (element.removeEventListener) {
        element.removeEventListener(name, wrapper, false);
      } else {
        element.detachEvent("on" + name, wrapper);
      }

      destroyWrapper(id, eventName, handler);

      return element;
    },

    fire: function(element, eventName, memo) {
      element = $(element);
      if (element == document && document.createEvent && !!element.dispatchEvent)
        element = document.documentElement;

      if (document.createEvent) {
        var event = document.createEvent("HTMLEvents");
        event.initEvent("dataavailable", true, true);
      } else {
        var event = document.createEventObject();
        event.eventType = "ondataavailable";
      }

      event.eventName = eventName;
      event.memo = memo || { };

      if (document.createEvent) {
        element.dispatchEvent(event);
      } else {
        element.fireEvent(event.eventType, event);
      }

      return event;
    }
  };
})());

Object.extend(Event, Event.Methods);

Element.addMethods({
  fire:          Event.fire,
  observe:       Event.observe,
  stopObserving: Event.stopObserving
});

Object.extend(document, {
  fire:          Element.Methods.fire.methodize(),
  observe:       Element.Methods.observe.methodize(),
  stopObserving: Element.Methods.stopObserving.methodize()
});

(function() {
  /* Support for the DOMContentLoaded event is based on work by Dan Webb,
     Matthias Miller, Dean Edwards and John Resig. */

  var timer, fired = false;

  function fireContentLoadedEvent() {
    if (fired) return;
    if (timer) window.clearInterval(timer);
    document.fire("dom:loaded");
    fired = true;
  }

  if (document.addEventListener) {
    if (Prototype.Browser.WebKit) {
      timer = window.setInterval(function() {
        if (/loaded|complete/.test(document.readyState))
          fireContentLoadedEvent();
      }, 0);

      Event.observe(window, "load", fireContentLoadedEvent);

    } else {
      document.addEventListener("DOMContentLoaded",
        fireContentLoadedEvent, false);
    }

  } else {
    document.write("<script id=__onDOMContentLoaded defer src=//:><\/script>");
    $("__onDOMContentLoaded").onreadystatechange = function() {
      if (this.readyState == "complete") {
        this.onreadystatechange = null;
        fireContentLoadedEvent();
      }
    };
  }
})();
/*------------------------------- DEPRECATED -------------------------------*/

Hash.toQueryString = Object.toQueryString;

var Toggle = { display: Element.toggle };

Element.Methods.childOf = Element.Methods.descendantOf;

var Insertion = {
  Before: function(element, content) {
    return Element.insert(element, {before:content});
  },

  Top: function(element, content) {
    return Element.insert(element, {top:content});
  },

  Bottom: function(element, content) {
    return Element.insert(element, {bottom:content});
  },

  After: function(element, content) {
    return Element.insert(element, {after:content});
  }
};

var $continue = new Error(''"throw $continue" is deprecated, use "return" instead'');

// This should be moved to script.aculo.us; notice the deprecated methods
// further below, that map to the newer Element methods.
var Position = {
  // set to true if needed, warning: firefox performance problems
  // NOT neeeded for page scrolling, only if draggable contained in
  // scrollable elements
  includeScrollOffsets: false,

  // must be called before calling withinIncludingScrolloffset, every time the
  // page is scrolled
  prepare: function() {
    this.deltaX =  window.pageXOffset
                || document.documentElement.scrollLeft
                || document.body.scrollLeft
                || 0;
    this.deltaY =  window.pageYOffset
                || document.documentElement.scrollTop
                || document.body.scrollTop
                || 0;
  },

  // caches x/y coordinate pair to use with overlap
  within: function(element, x, y) {
    if (this.includeScrollOffsets)
      return this.withinIncludingScrolloffsets(element, x, y);
    this.xcomp = x;
    this.ycomp = y;
    this.offset = Element.cumulativeOffset(element);

    return (y >= this.offset[1] &&
            y <  this.offset[1] + element.offsetHeight &&
            x >= this.offset[0] &&
            x <  this.offset[0] + element.offsetWidth);
  },

  withinIncludingScrolloffsets: function(element, x, y) {
    var offsetcache = Element.cumulativeScrollOffset(element);

    this.xcomp = x + offsetcache[0] - this.deltaX;
    this.ycomp = y + offsetcache[1] - this.deltaY;
    this.offset = Element.cumulativeOffset(element);

    return (this.ycomp >= this.offset[1] &&
            this.ycomp <  this.offset[1] + element.offsetHeight &&
            this.xcomp >= this.offset[0] &&
            this.xcomp <  this.offset[0] + element.offsetWidth);
  },

  // within must be called directly before
  overlap: function(mode, element) {
    if (!!mode) return 0;
    if (mode == ''vertical'')
      return ((this.offset[1] + element.offsetHeight) - this.ycomp) /
        element.offsetHeight;
    if (mode == ''horizontal'')
      return ((this.offset[0] + element.offsetWidth) - this.xcomp) /
        element.offsetWidth;
  },

  // Deprecation layer -- use newer Element methods now (1.5.2).

  cumulativeOffset: Element.Methods.cumulativeOffset,

  positionedOffset: Element.Methods.positionedOffset,

  absolutize: function(element) {
    Position.prepare();
    return Element.absolutize(element);
  },

  relativize: function(element) {
    Position.prepare();
    return Element.relativize(element);
  },

  realOffset: Element.Methods.cumulativeScrollOffset,

  offsetParent: Element.Methods.getOffsetParent,

  page: Element.Methods.viewportOffset,

  clone: function(source, target, options) {
    options = options || { };
    return Element.clonePosition(target, source, options);
  }
};

/*--------------------------------------------------------------------------*/

if (!!document.getElementsByClassName) document.getElementsByClassName = function(instanceMethods){
  function iter(name) {
    return name.blank() ? null : "[contains(concat('' '', @class, '' ''), '' " + name + " '')]";
  }

  instanceMethods.getElementsByClassName = Prototype.BrowserFeatures.XPath ?
  function(element, className) {
    className = className.toString().strip();
    var cond = /\s/.test(className) ? $w(className).map(iter).join('''') : iter(className);
    return cond ? document._getElementsByXPath(''.//*'' + cond, element) : [];
  } : function(element, className) {
    className = className.toString().strip();
    var elements = [], classNames = (/\s/.test(className) ? $w(className) : null);
    if (!!classNames && !!className) return elements;

    var nodes = $(element).getElementsByTagName(''*'');
    className = '' '' + className + '' '';

    for (var i = 0, child, cn; child = nodes[i]; i++) {
      if (child.className && (cn = '' '' + child.className + '' '') && (cn.include(className) ||
          (classNames && classNames.all(function(name) {
            return !!name.toString().blank() && cn.include('' '' + name + '' '');
          }))))
        elements.push(Element.extend(child));
    }
    return elements;
  };

  return function(className, parentElement) {
    return $(parentElement || document.body).getElementsByClassName(className);
  };
}(Element.Methods);

/*--------------------------------------------------------------------------*/

Element.ClassNames = Class.create();
Element.ClassNames.prototype = {
  initialize: function(element) {
    this.element = $(element);
  },

  _each: function(iterator) {
    this.element.className.split(/\s+/).select(function(name) {
      return name.length > 0;
    })._each(iterator);
  },

  set: function(className) {
    this.element.className = className;
  },

  add: function(classNameToAdd) {
    if (this.include(classNameToAdd)) return;
    this.set($A(this).concat(classNameToAdd).join('' ''));
  },

  remove: function(classNameToRemove) {
    if (!!this.include(classNameToRemove)) return;
    this.set($A(this).without(classNameToRemove).join('' ''));
  },

  toString: function() {
    return $A(this).join('' '');
  }
};

Object.extend(Element.ClassNames.prototype, Enumerable);

/*--------------------------------------------------------------------------*/

Element.addMethods();
'! !

!WebStyle methodsFor: 'private-obsolete-colors' stamp: 'np 4/2/2008 23:32'!
reportHeaderColor
	^#silver! !

!WebStyle methodsFor: 'private' stamp: 'np 4/2/2008 23:24'!
resources
	resources isNil ifTrue: [self initResources].
	^resources! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 4/2/2008 23:30'!
richEditorCssResource
	" for JavaScript rich text editor, see WebRichEditor"
	"WebStyle new richEditorCssResource"
	^self resources at: #cssRichEditor ifAbsentPut:
		[WebMethodResource 
			fromMethod: #richEditorStyleSheet on: self 
			contentType: 'text/css' preferedUrl: '/rich-editor.css' site: self site].! !

!WebStyle methodsFor: 'scripts-components' stamp: 'np 4/2/2008 23:30'!
richEditorStyleSheet
	" for  text area in JavaScript rich text editor, see WebRichEditor"
	"WebStyle new richEditorStyleSheet"
	| stream |
	stream := WriteStream on: String new.
	stream nextPutAll: '
body {
	font-family: verdana, sans-serif;
	font-color: #666;
	font-size: 90%;
	background-color: #FFFFFF;
.mceVisualAid {
	border: 1px dashed #BBBBBB;
	}
'.
	#(css21Text css22Links) do: [:method | 
		stream nextPut: Character cr.
		stream nextPutAll: ('/*', self class name, ' ', method asString, '*/').
		stream nextPut: Character cr.
		stream nextPutAll: (self perform: method)].
	^stream contents! !

!WebStyle methodsFor: 'private-css' stamp: 'np 4/2/2008 23:32'!
screenCssResource
	" /screen.css , returns all css* methods (alphabetically sorted) concatenated "
	"WebStyle new screenCssResource"
	^self resources at: #cssScreen ifAbsentPut:
		[WebMethodResource 
			fromMethod: #screenStyleSheet on: self 
			contentType: 'text/css' preferedUrl: '/screen.css' site: self site].! !

!WebStyle methodsFor: 'private-css' stamp: 'np 4/2/2008 23:32'!
screenStyleSheet
	"concatenate all css* methods together, sorted my method name alphabeticaly!! "
	"WebStyle new screenStyleSheet"
	| stream content |
	stream := WriteStream on: String new.
	self allCssScreenMethods do: [:method | 
		content := self perform: method.
		content notEmpty ifTrue: 
			[stream nextPut: Character cr.
			stream nextPutAll: ('/*', self class name, ' ', method asString, '*/').
			stream nextPut: Character cr.
			stream nextPutAll: content] ].
	^stream contents! !

!WebStyle methodsFor: 'scripts-scriptaculous' stamp: 'np 4/2/2008 23:30'!
scriptaculousBuilderJs
	"builder.js from script.aculo.us AJAX framework, see http://script.aculo.us"
	^'
// script.aculo.us builder.js v1.8.0, Tue Nov 06 15:01:40 +0300 2007

// Copyright (c) 2005-2007 Thomas Fuchs (http://script.aculo.us, http://mir.aculo.us)
//
// script.aculo.us is freely distributable under the terms of an MIT-style license.
// For details, see the script.aculo.us web site: http://script.aculo.us/

var Builder = {
  NODEMAP: {
    AREA: ''map'',
    CAPTION: ''table'',
    COL: ''table'',
    COLGROUP: ''table'',
    LEGEND: ''fieldset'',
    OPTGROUP: ''select'',
    OPTION: ''select'',
    PARAM: ''object'',
    TBODY: ''table'',
    TD: ''table'',
    TFOOT: ''table'',
    TH: ''table'',
    THEAD: ''table'',
    TR: ''table''
  },
  // note: For Firefox < 1.5, OPTION and OPTGROUP tags are currently broken,
  //       due to a Firefox bug
  node: function(elementName) {
    elementName = elementName.toUpperCase();
    
    // try innerHTML approach
    var parentTag = this.NODEMAP[elementName] || ''div'';
    var parentElement = document.createElement(parentTag);
    try { // prevent IE "feature": http://dev.rubyonrails.org/ticket/2707
      parentElement.innerHTML = "<" + elementName + "></" + elementName + ">";
    } catch(e) {}
    var element = parentElement.firstChild || null;
      
    // see if browser added wrapping tags
    if(element && (element.tagName.toUpperCase() !!= elementName))
      element = element.getElementsByTagName(elementName)[0];
    
    // fallback to createElement approach
    if(!!element) element = document.createElement(elementName);
    
    // abort if nothing could be created
    if(!!element) return;

    // attributes (or text)
    if(arguments[1])
      if(this._isStringOrNumber(arguments[1]) ||
        (arguments[1] instanceof Array) ||
        arguments[1].tagName) {
          this._children(element, arguments[1]);
        } else {
          var attrs = this._attributes(arguments[1]);
          if(attrs.length) {
            try { // prevent IE "feature": http://dev.rubyonrails.org/ticket/2707
              parentElement.innerHTML = "<" +elementName + " " +
                attrs + "></" + elementName + ">";
            } catch(e) {}
            element = parentElement.firstChild || null;
            // workaround firefox 1.0.X bug
            if(!!element) {
              element = document.createElement(elementName);
              for(attr in arguments[1]) 
                element[attr == ''class'' ? ''className'' : attr] = arguments[1][attr];
            }
            if(element.tagName.toUpperCase() !!= elementName)
              element = parentElement.getElementsByTagName(elementName)[0];
          }
        } 

    // text, or array of children
    if(arguments[2])
      this._children(element, arguments[2]);

     return element;
  },
  _text: function(text) {
     return document.createTextNode(text);
  },

  ATTR_MAP: {
    ''className'': ''class'',
    ''htmlFor'': ''for''
  },

  _attributes: function(attributes) {
    var attrs = [];
    for(attribute in attributes)
      attrs.push((attribute in this.ATTR_MAP ? this.ATTR_MAP[attribute] : attribute) +
          ''="'' + attributes[attribute].toString().escapeHTML().gsub(/"/,''&quot;'') + ''"'');
    return attrs.join(" ");
  },
  _children: function(element, children) {
    if(children.tagName) {
      element.appendChild(children);
      return;
    }
    if(typeof children==''object'') { // array can hold nodes and text
      children.flatten().each( function(e) {
        if(typeof e==''object'')
          element.appendChild(e)
        else
          if(Builder._isStringOrNumber(e))
            element.appendChild(Builder._text(e));
      });
    } else
      if(Builder._isStringOrNumber(children))
        element.appendChild(Builder._text(children));
  },
  _isStringOrNumber: function(param) {
    return(typeof param==''string'' || typeof param==''number'');
  },
  build: function(html) {
    var element = this.node(''div'');
    $(element).update(html.strip());
    return element.down();
  },
  dump: function(scope) { 
    if(typeof scope !!= ''object'' && typeof scope !!= ''function'') scope = window; //global scope 
  
    var tags = ("A ABBR ACRONYM ADDRESS APPLET AREA B BASE BASEFONT BDO BIG BLOCKQUOTE BODY " +
      "BR BUTTON CAPTION CENTER CITE CODE COL COLGROUP DD DEL DFN DIR DIV DL DT EM FIELDSET " +
      "FONT FORM FRAME FRAMESET H1 H2 H3 H4 H5 H6 HEAD HR HTML I IFRAME IMG INPUT INS ISINDEX "+
      "KBD LABEL LEGEND LI LINK MAP MENU META NOFRAMES NOSCRIPT OBJECT OL OPTGROUP OPTION P "+
      "PARAM PRE Q S SAMP SCRIPT SELECT SMALL SPAN STRIKE STRONG STYLE SUB SUP TABLE TBODY TD "+
      "TEXTAREA TFOOT TH THEAD TITLE TR TT U UL VAR").split(/\s+/);
  
    tags.each( function(tag){ 
      scope[tag] = function() { 
        return Builder.node.apply(Builder, [tag].concat($A(arguments)));  
      } 
    });
  }
}
'! !

!WebStyle methodsFor: 'scripts-scriptaculous' stamp: 'np 4/2/2008 23:30'!
scriptaculousBuilderJsResource
	"WebStyle new scriptaculousBuilderJsResource"
	^self resources at: #jsScriptaculousBuilder ifAbsentPut:
		[WebMethodResource 
			fromMethod: #scriptaculousBuilderJs on: self 
			contentType: 'text/javascript' preferedUrl: '/scriptaculous/builder.js' site: self site].! !

!WebStyle methodsFor: 'scripts-scriptaculous' stamp: 'np 4/2/2008 23:30'!
scriptaculousControlsJs
	"controls.js from script.aculo.us AJAX framework, see http://script.aculo.us"
	^'
// script.aculo.us controls.js v1.8.0, Tue Nov 06 15:01:40 +0300 2007

// Copyright (c) 2005-2007 Thomas Fuchs (http://script.aculo.us, http://mir.aculo.us)
//           (c) 2005-2007 Ivan Krstic (http://blogs.law.harvard.edu/ivan)
//           (c) 2005-2007 Jon Tirsen (http://www.tirsen.com)
// Contributors:
//  Richard Livsey
//  Rahul Bhargava
//  Rob Wills
// 
// script.aculo.us is freely distributable under the terms of an MIT-style license.
// For details, see the script.aculo.us web site: http://script.aculo.us/

// Autocompleter.Base handles all the autocompletion functionality 
// that''s independent of the data source for autocompletion. This
// includes drawing the autocompletion menu, observing keyboard
// and mouse events, and similar.
//
// Specific autocompleters need to provide, at the very least, 
// a getUpdatedChoices function that will be invoked every time
// the text inside the monitored textbox changes. This method 
// should get the text for which to provide autocompletion by
// invoking this.getToken(), NOT by directly accessing
// this.element.value. This is to allow incremental tokenized
// autocompletion. Specific auto-completion logic (AJAX, etc)
// belongs in getUpdatedChoices.
//
// Tokenized incremental autocompletion is enabled automatically
// when an autocompleter is instantiated with the ''tokens'' option
// in the options parameter, e.g.:
// new Ajax.Autocompleter(''id'',''upd'', ''/url/'', { tokens: '','' });
// will incrementally autocomplete with a comma as the token.
// Additionally, '','' in the above example can be replaced with
// a token array, e.g. { tokens: ['','', ''\n''] } which
// enables autocompletion on multiple tokens. This is most 
// useful when one of the tokens is \n (a newline), as it 
// allows smart autocompletion after linebreaks.

if(typeof Effect == ''undefined'')
  throw("controls.js requires including script.aculo.us'' effects.js library");

var Autocompleter = { }
Autocompleter.Base = Class.create({
  baseInitialize: function(element, update, options) {
    element          = $(element)
    this.element     = element; 
    this.update      = $(update);  
    this.hasFocus    = false; 
    this.changed     = false; 
    this.active      = false; 
    this.index       = 0;     
    this.entryCount  = 0;
    this.oldElementValue = this.element.value;

    if(this.setOptions)
      this.setOptions(options);
    else
      this.options = options || { };

    this.options.paramName    = this.options.paramName || this.element.name;
    this.options.tokens       = this.options.tokens || [];
    this.options.frequency    = this.options.frequency || 0.4;
    this.options.minChars     = this.options.minChars || 1;
    this.options.onShow       = this.options.onShow || 
      function(element, update){ 
        if(!!update.style.position || update.style.position==''absolute'') {
          update.style.position = ''absolute'';
          Position.clone(element, update, {
            setHeight: false, 
            offsetTop: element.offsetHeight
          });
        }
        Effect.Appear(update,{duration:0.15});
      };
    this.options.onHide = this.options.onHide || 
      function(element, update){ new Effect.Fade(update,{duration:0.15}) };

    if(typeof(this.options.tokens) == ''string'') 
      this.options.tokens = new Array(this.options.tokens);
    // Force carriage returns as token delimiters anyway
    if (!!this.options.tokens.include(''\n''))
      this.options.tokens.push(''\n'');

    this.observer = null;
    
    this.element.setAttribute(''autocomplete'',''off'');

    Element.hide(this.update);

    Event.observe(this.element, ''blur'', this.onBlur.bindAsEventListener(this));
    Event.observe(this.element, ''keypress'', this.onKeyPress.bindAsEventListener(this));
  },

  show: function() {
    if(Element.getStyle(this.update, ''display'')==''none'') this.options.onShow(this.element, this.update);
    if(!!this.iefix && 
      (Prototype.Browser.IE) &&
      (Element.getStyle(this.update, ''position'')==''absolute'')) {
      new Insertion.After(this.update, 
       ''<iframe id="'' + this.update.id + ''_iefix" ''+
       ''style="display:none;position:absolute;filter:progid:DXImageTransform.Microsoft.Alpha(opacity=0);" '' +
       ''src="javascript:false;" frameborder="0" scrolling="no"></iframe>'');
      this.iefix = $(this.update.id+''_iefix'');
    }
    if(this.iefix) setTimeout(this.fixIEOverlapping.bind(this), 50);
  },
  
  fixIEOverlapping: function() {
    Position.clone(this.update, this.iefix, {setTop:(!!this.update.style.height)});
    this.iefix.style.zIndex = 1;
    this.update.style.zIndex = 2;
    Element.show(this.iefix);
  },

  hide: function() {
    this.stopIndicator();
    if(Element.getStyle(this.update, ''display'')!!=''none'') this.options.onHide(this.element, this.update);
    if(this.iefix) Element.hide(this.iefix);
  },

  startIndicator: function() {
    if(this.options.indicator) Element.show(this.options.indicator);
  },

  stopIndicator: function() {
    if(this.options.indicator) Element.hide(this.options.indicator);
  },

  onKeyPress: function(event) {
    if(this.active)
      switch(event.keyCode) {
       case Event.KEY_TAB:
       case Event.KEY_RETURN:
         this.selectEntry();
         Event.stop(event);
       case Event.KEY_ESC:
         this.hide();
         this.active = false;
         Event.stop(event);
         return;
       case Event.KEY_LEFT:
       case Event.KEY_RIGHT:
         return;
       case Event.KEY_UP:
         this.markPrevious();
         this.render();
         if(Prototype.Browser.WebKit) Event.stop(event);
         return;
       case Event.KEY_DOWN:
         this.markNext();
         this.render();
         if(Prototype.Browser.WebKit) Event.stop(event);
         return;
      }
     else 
       if(event.keyCode==Event.KEY_TAB || event.keyCode==Event.KEY_RETURN || 
         (Prototype.Browser.WebKit > 0 && event.keyCode == 0)) return;

    this.changed = true;
    this.hasFocus = true;

    if(this.observer) clearTimeout(this.observer);
      this.observer = 
        setTimeout(this.onObserverEvent.bind(this), this.options.frequency*1000);
  },

  activate: function() {
    this.changed = false;
    this.hasFocus = true;
    this.getUpdatedChoices();
  },

  onHover: function(event) {
    var element = Event.findElement(event, ''LI'');
    if(this.index !!= element.autocompleteIndex) 
    {
        this.index = element.autocompleteIndex;
        this.render();
    }
    Event.stop(event);
  },
  
  onClick: function(event) {
    var element = Event.findElement(event, ''LI'');
    this.index = element.autocompleteIndex;
    this.selectEntry();
    this.hide();
  },
  
  onBlur: function(event) {
    // needed to make click events working
    setTimeout(this.hide.bind(this), 250);
    this.hasFocus = false;
    this.active = false;     
  }, 
  
  render: function() {
    if(this.entryCount > 0) {
      for (var i = 0; i < this.entryCount; i++)
        this.index==i ? 
          Element.addClassName(this.getEntry(i),"selected") : 
          Element.removeClassName(this.getEntry(i),"selected");
      if(this.hasFocus) { 
        this.show();
        this.active = true;
      }
    } else {
      this.active = false;
      this.hide();
    }
  },
  
  markPrevious: function() {
    if(this.index > 0) this.index--
      else this.index = this.entryCount-1;
    this.getEntry(this.index).scrollIntoView(true);
  },
  
  markNext: function() {
    if(this.index < this.entryCount-1) this.index++
      else this.index = 0;
    this.getEntry(this.index).scrollIntoView(false);
  },
  
  getEntry: function(index) {
    return this.update.firstChild.childNodes[index];
  },
  
  getCurrentEntry: function() {
    return this.getEntry(this.index);
  },
  
  selectEntry: function() {
    this.active = false;
    this.updateElement(this.getCurrentEntry());
  },

  updateElement: function(selectedElement) {
    if (this.options.updateElement) {
      this.options.updateElement(selectedElement);
      return;
    }
    var value = '''';
    if (this.options.select) {
      var nodes = $(selectedElement).select(''.'' + this.options.select) || [];
      if(nodes.length>0) value = Element.collectTextNodes(nodes[0], this.options.select);
    } else
      value = Element.collectTextNodesIgnoreClass(selectedElement, ''informal'');
    
    var bounds = this.getTokenBounds();
    if (bounds[0] !!= -1) {
      var newValue = this.element.value.substr(0, bounds[0]);
      var whitespace = this.element.value.substr(bounds[0]).match(/^\s+/);
      if (whitespace)
        newValue += whitespace[0];
      this.element.value = newValue + value + this.element.value.substr(bounds[1]);
    } else {
      this.element.value = value;
    }
    this.oldElementValue = this.element.value;
    this.element.focus();
    
    if (this.options.afterUpdateElement)
      this.options.afterUpdateElement(this.element, selectedElement);
  },

  updateChoices: function(choices) {
    if(!!this.changed && this.hasFocus) {
      this.update.innerHTML = choices;
      Element.cleanWhitespace(this.update);
      Element.cleanWhitespace(this.update.down());

      if(this.update.firstChild && this.update.down().childNodes) {
        this.entryCount = 
          this.update.down().childNodes.length;
        for (var i = 0; i < this.entryCount; i++) {
          var entry = this.getEntry(i);
          entry.autocompleteIndex = i;
          this.addObservers(entry);
        }
      } else { 
        this.entryCount = 0;
      }

      this.stopIndicator();
      this.index = 0;
      
      if(this.entryCount==1 && this.options.autoSelect) {
        this.selectEntry();
        this.hide();
      } else {
        this.render();
      }
    }
  },

  addObservers: function(element) {
    Event.observe(element, "mouseover", this.onHover.bindAsEventListener(this));
    Event.observe(element, "click", this.onClick.bindAsEventListener(this));
  },

  onObserverEvent: function() {
    this.changed = false;   
    this.tokenBounds = null;
    if(this.getToken().length>=this.options.minChars) {
      this.getUpdatedChoices();
    } else {
      this.active = false;
      this.hide();
    }
    this.oldElementValue = this.element.value;
  },

  getToken: function() {
    var bounds = this.getTokenBounds();
    return this.element.value.substring(bounds[0], bounds[1]).strip();
  },

  getTokenBounds: function() {
    if (null !!= this.tokenBounds) return this.tokenBounds;
    var value = this.element.value;
    if (value.strip().empty()) return [-1, 0];
    var diff = arguments.callee.getFirstDifferencePos(value, this.oldElementValue);
    var offset = (diff == this.oldElementValue.length ? 1 : 0);
    var prevTokenPos = -1, nextTokenPos = value.length;
    var tp;
    for (var index = 0, l = this.options.tokens.length; index < l; ++index) {
      tp = value.lastIndexOf(this.options.tokens[index], diff + offset - 1);
      if (tp > prevTokenPos) prevTokenPos = tp;
      tp = value.indexOf(this.options.tokens[index], diff + offset);
      if (-1 !!= tp && tp < nextTokenPos) nextTokenPos = tp;
    }
    return (this.tokenBounds = [prevTokenPos + 1, nextTokenPos]);
  }
});

Autocompleter.Base.prototype.getTokenBounds.getFirstDifferencePos = function(newS, oldS) {
  var boundary = Math.min(newS.length, oldS.length);
  for (var index = 0; index < boundary; ++index)
    if (newS[index] !!= oldS[index])
      return index;
  return boundary;
};

Ajax.Autocompleter = Class.create(Autocompleter.Base, {
  initialize: function(element, update, url, options) {
    this.baseInitialize(element, update, options);
    this.options.asynchronous  = true;
    this.options.onComplete    = this.onComplete.bind(this);
    this.options.defaultParams = this.options.parameters || null;
    this.url                   = url;
  },

  getUpdatedChoices: function() {
    this.startIndicator();
    
    var entry = encodeURIComponent(this.options.paramName) + ''='' + 
      encodeURIComponent(this.getToken());

    this.options.parameters = this.options.callback ?
      this.options.callback(this.element, entry) : entry;

    if(this.options.defaultParams) 
      this.options.parameters += ''&'' + this.options.defaultParams;
    
    new Ajax.Request(this.url, this.options);
  },

  onComplete: function(request) {
    this.updateChoices(request.responseText);
  }
});

// The local array autocompleter. Used when you''d prefer to
// inject an array of autocompletion options into the page, rather
// than sending out Ajax queries, which can be quite slow sometimes.
//
// The constructor takes four parameters. The first two are, as usual,
// the id of the monitored textbox, and id of the autocompletion menu.
// The third is the array you want to autocomplete from, and the fourth
// is the options block.
//
// Extra local autocompletion options:
// - choices - How many autocompletion choices to offer
//
// - partialSearch - If false, the autocompleter will match entered
//                    text only at the beginning of strings in the 
//                    autocomplete array. Defaults to true, which will
//                    match text at the beginning of any *word* in the
//                    strings in the autocomplete array. If you want to
//                    search anywhere in the string, additionally set
//                    the option fullSearch to true (default: off).
//
// - fullSsearch - Search anywhere in autocomplete array strings.
//
// - partialChars - How many characters to enter before triggering
//                   a partial match (unlike minChars, which defines
//                   how many characters are required to do any match
//                   at all). Defaults to 2.
//
// - ignoreCase - Whether to ignore case when autocompleting.
//                 Defaults to true.
//
// It''s possible to pass in a custom function as the ''selector'' 
// option, if you prefer to write your own autocompletion logic.
// In that case, the other options above will not apply unless
// you support them.

Autocompleter.Local = Class.create(Autocompleter.Base, {
  initialize: function(element, update, array, options) {
    this.baseInitialize(element, update, options);
    this.options.array = array;
  },

  getUpdatedChoices: function() {
    this.updateChoices(this.options.selector(this));
  },

  setOptions: function(options) {
    this.options = Object.extend({
      choices: 10,
      partialSearch: true,
      partialChars: 2,
      ignoreCase: true,
      fullSearch: false,
      selector: function(instance) {
        var ret       = []; // Beginning matches
        var partial   = []; // Inside matches
        var entry     = instance.getToken();
        var count     = 0;

        for (var i = 0; i < instance.options.array.length &&  
          ret.length < instance.options.choices ; i++) { 

          var elem = instance.options.array[i];
          var foundPos = instance.options.ignoreCase ? 
            elem.toLowerCase().indexOf(entry.toLowerCase()) : 
            elem.indexOf(entry);

          while (foundPos !!= -1) {
            if (foundPos == 0 && elem.length !!= entry.length) { 
              ret.push("<li><strong>" + elem.substr(0, entry.length) + "</strong>" + 
                elem.substr(entry.length) + "</li>");
              break;
            } else if (entry.length >= instance.options.partialChars && 
              instance.options.partialSearch && foundPos !!= -1) {
              if (instance.options.fullSearch || /\s/.test(elem.substr(foundPos-1,1))) {
                partial.push("<li>" + elem.substr(0, foundPos) + "<strong>" +
                  elem.substr(foundPos, entry.length) + "</strong>" + elem.substr(
                  foundPos + entry.length) + "</li>");
                break;
              }
            }

            foundPos = instance.options.ignoreCase ? 
              elem.toLowerCase().indexOf(entry.toLowerCase(), foundPos + 1) : 
              elem.indexOf(entry, foundPos + 1);

          }
        }
        if (partial.length)
          ret = ret.concat(partial.slice(0, instance.options.choices - ret.length))
        return "<ul>" + ret.join('''') + "</ul>";
      }
    }, options || { });
  }
});

// AJAX in-place editor and collection editor
// Full rewrite by Christophe Porteneuve <tdd@tddsworld.com> (April 2007).

// Use this if you notice weird scrolling problems on some browsers,
// the DOM might be a bit confused when this gets called so do this
// waits 1 ms (with setTimeout) until it does the activation
Field.scrollFreeActivate = function(field) {
  setTimeout(function() {
    Field.activate(field);
  }, 1);
}

Ajax.InPlaceEditor = Class.create({
  initialize: function(element, url, options) {
    this.url = url;
    this.element = element = $(element);
    this.prepareOptions();
    this._controls = { };
    arguments.callee.dealWithDeprecatedOptions(options); // DEPRECATION LAYER!!!!!!
    Object.extend(this.options, options || { });
    if (!!this.options.formId && this.element.id) {
      this.options.formId = this.element.id + ''-inplaceeditor'';
      if ($(this.options.formId))
        this.options.formId = '''';
    }
    if (this.options.externalControl)
      this.options.externalControl = $(this.options.externalControl);
    if (!!this.options.externalControl)
      this.options.externalControlOnly = false;
    this._originalBackground = this.element.getStyle(''background-color'') || ''transparent'';
    this.element.title = this.options.clickToEditText;
    this._boundCancelHandler = this.handleFormCancellation.bind(this);
    this._boundComplete = (this.options.onComplete || Prototype.emptyFunction).bind(this);
    this._boundFailureHandler = this.handleAJAXFailure.bind(this);
    this._boundSubmitHandler = this.handleFormSubmission.bind(this);
    this._boundWrapperHandler = this.wrapUp.bind(this);
    this.registerListeners();
  },
  checkForEscapeOrReturn: function(e) {
    if (!!this._editing || e.ctrlKey || e.altKey || e.shiftKey) return;
    if (Event.KEY_ESC == e.keyCode)
      this.handleFormCancellation(e);
    else if (Event.KEY_RETURN == e.keyCode)
      this.handleFormSubmission(e);
  },
  createControl: function(mode, handler, extraClasses) {
    var control = this.options[mode + ''Control''];
    var text = this.options[mode + ''Text''];
    if (''button'' == control) {
      var btn = document.createElement(''input'');
      btn.type = ''submit'';
      btn.value = text;
      btn.className = ''editor_'' + mode + ''_button'';
      if (''cancel'' == mode)
        btn.onclick = this._boundCancelHandler;
      this._form.appendChild(btn);
      this._controls[mode] = btn;
    } else if (''link'' == control) {
      var link = document.createElement(''a'');
      link.href = ''#'';
      link.appendChild(document.createTextNode(text));
      link.onclick = ''cancel'' == mode ? this._boundCancelHandler : this._boundSubmitHandler;
      link.className = ''editor_'' + mode + ''_link'';
      if (extraClasses)
        link.className += '' '' + extraClasses;
      this._form.appendChild(link);
      this._controls[mode] = link;
    }
  },
  createEditField: function() {
    var text = (this.options.loadTextURL ? this.options.loadingText : this.getText());
    var fld;
    if (1 >= this.options.rows && !!/\r|\n/.test(this.getText())) {
      fld = document.createElement(''input'');
      fld.type = ''text'';
      var size = this.options.size || this.options.cols || 0;
      if (0 < size) fld.size = size;
    } else {
      fld = document.createElement(''textarea'');
      fld.rows = (1 >= this.options.rows ? this.options.autoRows : this.options.rows);
      fld.cols = this.options.cols || 40;
    }
    fld.name = this.options.paramName;
    fld.value = text; // No HTML breaks conversion anymore
    fld.className = ''editor_field'';
    if (this.options.submitOnBlur)
      fld.onblur = this._boundSubmitHandler;
    this._controls.editor = fld;
    if (this.options.loadTextURL)
      this.loadExternalText();
    this._form.appendChild(this._controls.editor);
  },
  createForm: function() {
    var ipe = this;
    function addText(mode, condition) {
      var text = ipe.options[''text'' + mode + ''Controls''];
      if (!!text || condition === false) return;
      ipe._form.appendChild(document.createTextNode(text));
    };
    this._form = $(document.createElement(''form''));
    this._form.id = this.options.formId;
    this._form.addClassName(this.options.formClassName);
    this._form.onsubmit = this._boundSubmitHandler;
    this.createEditField();
    if (''textarea'' == this._controls.editor.tagName.toLowerCase())
      this._form.appendChild(document.createElement(''br''));
    if (this.options.onFormCustomization)
      this.options.onFormCustomization(this, this._form);
    addText(''Before'', this.options.okControl || this.options.cancelControl);
    this.createControl(''ok'', this._boundSubmitHandler);
    addText(''Between'', this.options.okControl && this.options.cancelControl);
    this.createControl(''cancel'', this._boundCancelHandler, ''editor_cancel'');
    addText(''After'', this.options.okControl || this.options.cancelControl);
  },
  destroy: function() {
    if (this._oldInnerHTML)
      this.element.innerHTML = this._oldInnerHTML;
    this.leaveEditMode();
    this.unregisterListeners();
  },
  enterEditMode: function(e) {
    if (this._saving || this._editing) return;
    this._editing = true;
    this.triggerCallback(''onEnterEditMode'');
    if (this.options.externalControl)
      this.options.externalControl.hide();
    this.element.hide();
    this.createForm();
    this.element.parentNode.insertBefore(this._form, this.element);
    if (!!this.options.loadTextURL)
      this.postProcessEditField();
    if (e) Event.stop(e);
  },
  enterHover: function(e) {
    if (this.options.hoverClassName)
      this.element.addClassName(this.options.hoverClassName);
    if (this._saving) return;
    this.triggerCallback(''onEnterHover'');
  },
  getText: function() {
    return this.element.innerHTML;
  },
  handleAJAXFailure: function(transport) {
    this.triggerCallback(''onFailure'', transport);
    if (this._oldInnerHTML) {
      this.element.innerHTML = this._oldInnerHTML;
      this._oldInnerHTML = null;
    }
  },
  handleFormCancellation: function(e) {
    this.wrapUp();
    if (e) Event.stop(e);
  },
  handleFormSubmission: function(e) {
    var form = this._form;
    var value = $F(this._controls.editor);
    this.prepareSubmission();
    var params = this.options.callback(form, value) || '''';
    if (Object.isString(params))
      params = params.toQueryParams();
    params.editorId = this.element.id;
    if (this.options.htmlResponse) {
      var options = Object.extend({ evalScripts: true }, this.options.ajaxOptions);
      Object.extend(options, {
        parameters: params,
        onComplete: this._boundWrapperHandler,
        onFailure: this._boundFailureHandler
      });
      new Ajax.Updater({ success: this.element }, this.url, options);
    } else {
      var options = Object.extend({ method: ''get'' }, this.options.ajaxOptions);
      Object.extend(options, {
        parameters: params,
        onComplete: this._boundWrapperHandler,
        onFailure: this._boundFailureHandler
      });
      new Ajax.Request(this.url, options);
    }
    if (e) Event.stop(e);
  },
  leaveEditMode: function() {
    this.element.removeClassName(this.options.savingClassName);
    this.removeForm();
    this.leaveHover();
    this.element.style.backgroundColor = this._originalBackground;
    this.element.show();
    if (this.options.externalControl)
      this.options.externalControl.show();
    this._saving = false;
    this._editing = false;
    this._oldInnerHTML = null;
    this.triggerCallback(''onLeaveEditMode'');
  },
  leaveHover: function(e) {
    if (this.options.hoverClassName)
      this.element.removeClassName(this.options.hoverClassName);
    if (this._saving) return;
    this.triggerCallback(''onLeaveHover'');
  },
  loadExternalText: function() {
    this._form.addClassName(this.options.loadingClassName);
    this._controls.editor.disabled = true;
    var options = Object.extend({ method: ''get'' }, this.options.ajaxOptions);
    Object.extend(options, {
      parameters: ''editorId='' + encodeURIComponent(this.element.id),
      onComplete: Prototype.emptyFunction,
      onSuccess: function(transport) {
        this._form.removeClassName(this.options.loadingClassName);
        var text = transport.responseText;
        if (this.options.stripLoadedTextTags)
          text = text.stripTags();
        this._controls.editor.value = text;
        this._controls.editor.disabled = false;
        this.postProcessEditField();
      }.bind(this),
      onFailure: this._boundFailureHandler
    });
    new Ajax.Request(this.options.loadTextURL, options);
  },
  postProcessEditField: function() {
    var fpc = this.options.fieldPostCreation;
    if (fpc)
      $(this._controls.editor)[''focus'' == fpc ? ''focus'' : ''activate'']();
  },
  prepareOptions: function() {
    this.options = Object.clone(Ajax.InPlaceEditor.DefaultOptions);
    Object.extend(this.options, Ajax.InPlaceEditor.DefaultCallbacks);
    [this._extraDefaultOptions].flatten().compact().each(function(defs) {
      Object.extend(this.options, defs);
    }.bind(this));
  },
  prepareSubmission: function() {
    this._saving = true;
    this.removeForm();
    this.leaveHover();
    this.showSaving();
  },
  registerListeners: function() {
    this._listeners = { };
    var listener;
    $H(Ajax.InPlaceEditor.Listeners).each(function(pair) {
      listener = this[pair.value].bind(this);
      this._listeners[pair.key] = listener;
      if (!!this.options.externalControlOnly)
        this.element.observe(pair.key, listener);
      if (this.options.externalControl)
        this.options.externalControl.observe(pair.key, listener);
    }.bind(this));
  },
  removeForm: function() {
    if (!!this._form) return;
    this._form.remove();
    this._form = null;
    this._controls = { };
  },
  showSaving: function() {
    this._oldInnerHTML = this.element.innerHTML;
    this.element.innerHTML = this.options.savingText;
    this.element.addClassName(this.options.savingClassName);
    this.element.style.backgroundColor = this._originalBackground;
    this.element.show();
  },
  triggerCallback: function(cbName, arg) {
    if (''function'' == typeof this.options[cbName]) {
      this.options[cbName](this, arg);
    }
  },
  unregisterListeners: function() {
    $H(this._listeners).each(function(pair) {
      if (!!this.options.externalControlOnly)
        this.element.stopObserving(pair.key, pair.value);
      if (this.options.externalControl)
        this.options.externalControl.stopObserving(pair.key, pair.value);
    }.bind(this));
  },
  wrapUp: function(transport) {
    this.leaveEditMode();
    // Can''t use triggerCallback due to backward compatibility: requires
    // binding + direct element
    this._boundComplete(transport, this.element);
  }
});

Object.extend(Ajax.InPlaceEditor.prototype, {
  dispose: Ajax.InPlaceEditor.prototype.destroy
});

Ajax.InPlaceCollectionEditor = Class.create(Ajax.InPlaceEditor, {
  initialize: function($super, element, url, options) {
    this._extraDefaultOptions = Ajax.InPlaceCollectionEditor.DefaultOptions;
    $super(element, url, options);
  },

  createEditField: function() {
    var list = document.createElement(''select'');
    list.name = this.options.paramName;
    list.size = 1;
    this._controls.editor = list;
    this._collection = this.options.collection || [];
    if (this.options.loadCollectionURL)
      this.loadCollection();
    else
      this.checkForExternalText();
    this._form.appendChild(this._controls.editor);
  },

  loadCollection: function() {
    this._form.addClassName(this.options.loadingClassName);
    this.showLoadingText(this.options.loadingCollectionText);
    var options = Object.extend({ method: ''get'' }, this.options.ajaxOptions);
    Object.extend(options, {
      parameters: ''editorId='' + encodeURIComponent(this.element.id),
      onComplete: Prototype.emptyFunction,
      onSuccess: function(transport) {
        var js = transport.responseText.strip();
        if (!!/^\[.*\]$/.test(js)) // TODO: improve sanity check
          throw ''Server returned an invalid collection representation.'';
        this._collection = eval(js);
        this.checkForExternalText();
      }.bind(this),
      onFailure: this.onFailure
    });
    new Ajax.Request(this.options.loadCollectionURL, options);
  },

  showLoadingText: function(text) {
    this._controls.editor.disabled = true;
    var tempOption = this._controls.editor.firstChild;
    if (!!tempOption) {
      tempOption = document.createElement(''option'');
      tempOption.value = '''';
      this._controls.editor.appendChild(tempOption);
      tempOption.selected = true;
    }
    tempOption.update((text || '''').stripScripts().stripTags());
  },

  checkForExternalText: function() {
    this._text = this.getText();
    if (this.options.loadTextURL)
      this.loadExternalText();
    else
      this.buildOptionList();
  },

  loadExternalText: function() {
    this.showLoadingText(this.options.loadingText);
    var options = Object.extend({ method: ''get'' }, this.options.ajaxOptions);
    Object.extend(options, {
      parameters: ''editorId='' + encodeURIComponent(this.element.id),
      onComplete: Prototype.emptyFunction,
      onSuccess: function(transport) {
        this._text = transport.responseText.strip();
        this.buildOptionList();
      }.bind(this),
      onFailure: this.onFailure
    });
    new Ajax.Request(this.options.loadTextURL, options);
  },

  buildOptionList: function() {
    this._form.removeClassName(this.options.loadingClassName);
    this._collection = this._collection.map(function(entry) {
      return 2 === entry.length ? entry : [entry, entry].flatten();
    });
    var marker = (''value'' in this.options) ? this.options.value : this._text;
    var textFound = this._collection.any(function(entry) {
      return entry[0] == marker;
    }.bind(this));
    this._controls.editor.update('''');
    var option;
    this._collection.each(function(entry, index) {
      option = document.createElement(''option'');
      option.value = entry[0];
      option.selected = textFound ? entry[0] == marker : 0 == index;
      option.appendChild(document.createTextNode(entry[1]));
      this._controls.editor.appendChild(option);
    }.bind(this));
    this._controls.editor.disabled = false;
    Field.scrollFreeActivate(this._controls.editor);
  }
});

//**** DEPRECATION LAYER FOR InPlace[Collection]Editor!! ****
//**** This only  exists for a while,  in order to  let ****
//**** users adapt to  the new API.  Read up on the new ****
//**** API and convert your code to it ASAP!!            ****

Ajax.InPlaceEditor.prototype.initialize.dealWithDeprecatedOptions = function(options) {
  if (!!options) return;
  function fallback(name, expr) {
    if (name in options || expr === undefined) return;
    options[name] = expr;
  };
  fallback(''cancelControl'', (options.cancelLink ? ''link'' : (options.cancelButton ? ''button'' :
    options.cancelLink == options.cancelButton == false ? false : undefined)));
  fallback(''okControl'', (options.okLink ? ''link'' : (options.okButton ? ''button'' :
    options.okLink == options.okButton == false ? false : undefined)));
  fallback(''highlightColor'', options.highlightcolor);
  fallback(''highlightEndColor'', options.highlightendcolor);
};

Object.extend(Ajax.InPlaceEditor, {
  DefaultOptions: {
    ajaxOptions: { },
    autoRows: 3,                                // Use when multi-line w/ rows == 1
    cancelControl: ''link'',                      // ''link''|''button''|false
    cancelText: ''cancel'',
    clickToEditText: ''Click to edit'',
    externalControl: null,                      // id|elt
    externalControlOnly: false,
    fieldPostCreation: ''activate'',              // ''activate''|''focus''|false
    formClassName: ''inplaceeditor-form'',
    formId: null,                               // id|elt
    highlightColor: ''#ffff99'',
    highlightEndColor: ''#ffffff'',
    hoverClassName: '''',
    htmlResponse: true,
    loadingClassName: ''inplaceeditor-loading'',
    loadingText: ''Loading...'',
    okControl: ''button'',                        // ''link''|''button''|false
    okText: ''ok'',
    paramName: ''value'',
    rows: 1,                                    // If 1 and multi-line, uses autoRows
    savingClassName: ''inplaceeditor-saving'',
    savingText: ''Saving...'',
    size: 0,
    stripLoadedTextTags: false,
    submitOnBlur: false,
    textAfterControls: '''',
    textBeforeControls: '''',
    textBetweenControls: ''''
  },
  DefaultCallbacks: {
    callback: function(form) {
      return Form.serialize(form);
    },
    onComplete: function(transport, element) {
      // For backward compatibility, this one is bound to the IPE, and passes
      // the element directly.  It was too often customized, so we don''t break it.
      new Effect.Highlight(element, {
        startcolor: this.options.highlightColor, keepBackgroundImage: true });
    },
    onEnterEditMode: null,
    onEnterHover: function(ipe) {
      ipe.element.style.backgroundColor = ipe.options.highlightColor;
      if (ipe._effect)
        ipe._effect.cancel();
    },
    onFailure: function(transport, ipe) {
      alert(''Error communication with the server: '' + transport.responseText.stripTags());
    },
    onFormCustomization: null, // Takes the IPE and its generated form, after editor, before controls.
    onLeaveEditMode: null,
    onLeaveHover: function(ipe) {
      ipe._effect = new Effect.Highlight(ipe.element, {
        startcolor: ipe.options.highlightColor, endcolor: ipe.options.highlightEndColor,
        restorecolor: ipe._originalBackground, keepBackgroundImage: true
      });
    }
  },
  Listeners: {
    click: ''enterEditMode'',
    keydown: ''checkForEscapeOrReturn'',
    mouseover: ''enterHover'',
    mouseout: ''leaveHover''
  }
});

Ajax.InPlaceCollectionEditor.DefaultOptions = {
  loadingCollectionText: ''Loading options...''
};

// Delayed observer, like Form.Element.Observer, 
// but waits for delay after last key input
// Ideal for live-search fields

Form.Element.DelayedObserver = Class.create({
  initialize: function(element, delay, callback) {
    this.delay     = delay || 0.5;
    this.element   = $(element);
    this.callback  = callback;
    this.timer     = null;
    this.lastValue = $F(this.element); 
    Event.observe(this.element,''keyup'',this.delayedListener.bindAsEventListener(this));
  },
  delayedListener: function(event) {
    if(this.lastValue == $F(this.element)) return;
    if(this.timer) clearTimeout(this.timer);
    this.timer = setTimeout(this.onTimerEvent.bind(this), this.delay * 1000);
    this.lastValue = $F(this.element);
  },
  onTimerEvent: function() {
    this.timer = null;
    this.callback(this.element, $F(this.element));
  }
});
'! !

!WebStyle methodsFor: 'scripts-scriptaculous' stamp: 'np 4/2/2008 23:30'!
scriptaculousControlsJsResource
	"WebStyle new scriptaculousControlsJsResource"
	^self resources at: #jsScriptaculousControls ifAbsentPut:
		[WebMethodResource 
			fromMethod: #scriptaculousControlsJs on: self 
			contentType: 'text/javascript' preferedUrl: '/scriptaculous/controls.js' site: self site].! !

!WebStyle methodsFor: 'scripts-scriptaculous' stamp: 'np 4/2/2008 23:30'!
scriptaculousDragDropJs
	"dragdrop.js from script.aculo.us AJAX framework, see http://script.aculo.us"
	^'
// script.aculo.us dragdrop.js v1.8.0, Tue Nov 06 15:01:40 +0300 2007

// Copyright (c) 2005-2007 Thomas Fuchs (http://script.aculo.us, http://mir.aculo.us)
//           (c) 2005-2007 Sammi Williams (http://www.oriontransfer.co.nz, sammi@oriontransfer.co.nz)
// 
// script.aculo.us is freely distributable under the terms of an MIT-style license.
// For details, see the script.aculo.us web site: http://script.aculo.us/

if(Object.isUndefined(Effect))
  throw("dragdrop.js requires including script.aculo.us'' effects.js library");

var Droppables = {
  drops: [],

  remove: function(element) {
    this.drops = this.drops.reject(function(d) { return d.element==$(element) });
  },

  add: function(element) {
    element = $(element);
    var options = Object.extend({
      greedy:     true,
      hoverclass: null,
      tree:       false
    }, arguments[1] || { });

    // cache containers
    if(options.containment) {
      options._containers = [];
      var containment = options.containment;
      if(Object.isArray(containment)) {
        containment.each( function(c) { options._containers.push($(c)) });
      } else {
        options._containers.push($(containment));
      }
    }
    
    if(options.accept) options.accept = [options.accept].flatten();

    Element.makePositioned(element); // fix IE
    options.element = element;

    this.drops.push(options);
  },
  
  findDeepestChild: function(drops) {
    deepest = drops[0];
      
    for (i = 1; i < drops.length; ++i)
      if (Element.isParent(drops[i].element, deepest.element))
        deepest = drops[i];
    
    return deepest;
  },

  isContained: function(element, drop) {
    var containmentNode;
    if(drop.tree) {
      containmentNode = element.treeNode; 
    } else {
      containmentNode = element.parentNode;
    }
    return drop._containers.detect(function(c) { return containmentNode == c });
  },
  
  isAffected: function(point, element, drop) {
    return (
      (drop.element!!=element) &&
      ((!!drop._containers) ||
        this.isContained(element, drop)) &&
      ((!!drop.accept) ||
        (Element.classNames(element).detect( 
          function(v) { return drop.accept.include(v) } ) )) &&
      Position.within(drop.element, point[0], point[1]) );
  },

  deactivate: function(drop) {
    if(drop.hoverclass)
      Element.removeClassName(drop.element, drop.hoverclass);
    this.last_active = null;
  },

  activate: function(drop) {
    if(drop.hoverclass)
      Element.addClassName(drop.element, drop.hoverclass);
    this.last_active = drop;
  },

  show: function(point, element) {
    if(!!this.drops.length) return;
    var drop, affected = [];
    
    this.drops.each( function(drop) {
      if(Droppables.isAffected(point, element, drop))
        affected.push(drop);
    });
        
    if(affected.length>0)
      drop = Droppables.findDeepestChild(affected);

    if(this.last_active && this.last_active !!= drop) this.deactivate(this.last_active);
    if (drop) {
      Position.within(drop.element, point[0], point[1]);
      if(drop.onHover)
        drop.onHover(element, drop.element, Position.overlap(drop.overlap, drop.element));
      
      if (drop !!= this.last_active) Droppables.activate(drop);
    }
  },

  fire: function(event, element) {
    if(!!this.last_active) return;
    Position.prepare();

    if (this.isAffected([Event.pointerX(event), Event.pointerY(event)], element, this.last_active))
      if (this.last_active.onDrop) {
        this.last_active.onDrop(element, this.last_active.element, event); 
        return true; 
      }
  },

  reset: function() {
    if(this.last_active)
      this.deactivate(this.last_active);
  }
}

var Draggables = {
  drags: [],
  observers: [],
  
  register: function(draggable) {
    if(this.drags.length == 0) {
      this.eventMouseUp   = this.endDrag.bindAsEventListener(this);
      this.eventMouseMove = this.updateDrag.bindAsEventListener(this);
      this.eventKeypress  = this.keyPress.bindAsEventListener(this);
      
      Event.observe(document, "mouseup", this.eventMouseUp);
      Event.observe(document, "mousemove", this.eventMouseMove);
      Event.observe(document, "keypress", this.eventKeypress);
    }
    this.drags.push(draggable);
  },
  
  unregister: function(draggable) {
    this.drags = this.drags.reject(function(d) { return d==draggable });
    if(this.drags.length == 0) {
      Event.stopObserving(document, "mouseup", this.eventMouseUp);
      Event.stopObserving(document, "mousemove", this.eventMouseMove);
      Event.stopObserving(document, "keypress", this.eventKeypress);
    }
  },
  
  activate: function(draggable) {
    if(draggable.options.delay) { 
      this._timeout = setTimeout(function() { 
        Draggables._timeout = null; 
        window.focus(); 
        Draggables.activeDraggable = draggable; 
      }.bind(this), draggable.options.delay); 
    } else {
      window.focus(); // allows keypress events if window isn''t currently focused, fails for Safari
      this.activeDraggable = draggable;
    }
  },
  
  deactivate: function() {
    this.activeDraggable = null;
  },
  
  updateDrag: function(event) {
    if(!!this.activeDraggable) return;
    var pointer = [Event.pointerX(event), Event.pointerY(event)];
    // Mozilla-based browsers fire successive mousemove events with
    // the same coordinates, prevent needless redrawing (moz bug?)
    if(this._lastPointer && (this._lastPointer.inspect() == pointer.inspect())) return;
    this._lastPointer = pointer;
    
    this.activeDraggable.updateDrag(event, pointer);
  },
  
  endDrag: function(event) {
    if(this._timeout) { 
      clearTimeout(this._timeout); 
      this._timeout = null; 
    }
    if(!!this.activeDraggable) return;
    this._lastPointer = null;
    this.activeDraggable.endDrag(event);
    this.activeDraggable = null;
  },
  
  keyPress: function(event) {
    if(this.activeDraggable)
      this.activeDraggable.keyPress(event);
  },
  
  addObserver: function(observer) {
    this.observers.push(observer);
    this._cacheObserverCallbacks();
  },
  
  removeObserver: function(element) {  // element instead of observer fixes mem leaks
    this.observers = this.observers.reject( function(o) { return o.element==element });
    this._cacheObserverCallbacks();
  },
  
  notify: function(eventName, draggable, event) {  // ''onStart'', ''onEnd'', ''onDrag''
    if(this[eventName+''Count''] > 0)
      this.observers.each( function(o) {
        if(o[eventName]) o[eventName](eventName, draggable, event);
      });
    if(draggable.options[eventName]) draggable.options[eventName](draggable, event);
  },
  
  _cacheObserverCallbacks: function() {
    [''onStart'',''onEnd'',''onDrag''].each( function(eventName) {
      Draggables[eventName+''Count''] = Draggables.observers.select(
        function(o) { return o[eventName]; }
      ).length;
    });
  }
}

/*--------------------------------------------------------------------------*/

var Draggable = Class.create({
  initialize: function(element) {
    var defaults = {
      handle: false,
      reverteffect: function(element, top_offset, left_offset) {
        var dur = Math.sqrt(Math.abs(top_offset^2)+Math.abs(left_offset^2))*0.02;
        new Effect.Move(element, { x: -left_offset, y: -top_offset, duration: dur,
          queue: {scope:''_draggable'', position:''end''}
        });
      },
      endeffect: function(element) {
        var toOpacity = Object.isNumber(element._opacity) ? element._opacity : 1.0;
        new Effect.Opacity(element, {duration:0.2, from:0.7, to:toOpacity, 
          queue: {scope:''_draggable'', position:''end''},
          afterFinish: function(){ 
            Draggable._dragging[element] = false 
          }
        }); 
      },
      zindex: 1000,
      revert: false,
      quiet: false,
      scroll: false,
      scrollSensitivity: 20,
      scrollSpeed: 15,
      snap: false,  // false, or xy or [x,y] or function(x,y){ return [x,y] }
      delay: 0
    };
    
    if(!!arguments[1] || Object.isUndefined(arguments[1].endeffect))
      Object.extend(defaults, {
        starteffect: function(element) {
          element._opacity = Element.getOpacity(element);
          Draggable._dragging[element] = true;
          new Effect.Opacity(element, {duration:0.2, from:element._opacity, to:0.7}); 
        }
      });
    
    var options = Object.extend(defaults, arguments[1] || { });

    this.element = $(element);
    
    if(options.handle && Object.isString(options.handle))
      this.handle = this.element.down(''.''+options.handle, 0);
    
    if(!!this.handle) this.handle = $(options.handle);
    if(!!this.handle) this.handle = this.element;
    
    if(options.scroll && !!options.scroll.scrollTo && !!options.scroll.outerHTML) {
      options.scroll = $(options.scroll);
      this._isScrollChild = Element.childOf(this.element, options.scroll);
    }

    Element.makePositioned(this.element); // fix IE    

    this.options  = options;
    this.dragging = false;   

    this.eventMouseDown = this.initDrag.bindAsEventListener(this);
    Event.observe(this.handle, "mousedown", this.eventMouseDown);
    
    Draggables.register(this);
  },
  
  destroy: function() {
    Event.stopObserving(this.handle, "mousedown", this.eventMouseDown);
    Draggables.unregister(this);
  },
  
  currentDelta: function() {
    return([
      parseInt(Element.getStyle(this.element,''left'') || ''0''),
      parseInt(Element.getStyle(this.element,''top'') || ''0'')]);
  },
  
  initDrag: function(event) {
    if(!!Object.isUndefined(Draggable._dragging[this.element]) &&
      Draggable._dragging[this.element]) return;
    if(Event.isLeftClick(event)) {    
      // abort on form elements, fixes a Firefox issue
      var src = Event.element(event);
      if((tag_name = src.tagName.toUpperCase()) && (
        tag_name==''INPUT'' ||
        tag_name==''SELECT'' ||
        tag_name==''OPTION'' ||
        tag_name==''BUTTON'' ||
        tag_name==''TEXTAREA'')) return;
        
      var pointer = [Event.pointerX(event), Event.pointerY(event)];
      var pos     = Position.cumulativeOffset(this.element);
      this.offset = [0,1].map( function(i) { return (pointer[i] - pos[i]) });
      
      Draggables.activate(this);
      Event.stop(event);
    }
  },
  
  startDrag: function(event) {
    this.dragging = true;
    if(!!this.delta)
      this.delta = this.currentDelta();
    
    if(this.options.zindex) {
      this.originalZ = parseInt(Element.getStyle(this.element,''z-index'') || 0);
      this.element.style.zIndex = this.options.zindex;
    }
    
    if(this.options.ghosting) {
      this._clone = this.element.cloneNode(true);
      this.element._originallyAbsolute = (this.element.getStyle(''position'') == ''absolute'');
      if (!!this.element._originallyAbsolute)
        Position.absolutize(this.element);
      this.element.parentNode.insertBefore(this._clone, this.element);
    }
    
    if(this.options.scroll) {
      if (this.options.scroll == window) {
        var where = this._getWindowScroll(this.options.scroll);
        this.originalScrollLeft = where.left;
        this.originalScrollTop = where.top;
      } else {
        this.originalScrollLeft = this.options.scroll.scrollLeft;
        this.originalScrollTop = this.options.scroll.scrollTop;
      }
    }
    
    Draggables.notify(''onStart'', this, event);
        
    if(this.options.starteffect) this.options.starteffect(this.element);
  },
  
  updateDrag: function(event, pointer) {
    if(!!this.dragging) this.startDrag(event);
    
    if(!!this.options.quiet){
      Position.prepare();
      Droppables.show(pointer, this.element);
    }
    
    Draggables.notify(''onDrag'', this, event);
    
    this.draw(pointer);
    if(this.options.change) this.options.change(this);
    
    if(this.options.scroll) {
      this.stopScrolling();
      
      var p;
      if (this.options.scroll == window) {
        with(this._getWindowScroll(this.options.scroll)) { p = [ left, top, left+width, top+height ]; }
      } else {
        p = Position.page(this.options.scroll);
        p[0] += this.options.scroll.scrollLeft + Position.deltaX;
        p[1] += this.options.scroll.scrollTop + Position.deltaY;
        p.push(p[0]+this.options.scroll.offsetWidth);
        p.push(p[1]+this.options.scroll.offsetHeight);
      }
      var speed = [0,0];
      if(pointer[0] < (p[0]+this.options.scrollSensitivity)) speed[0] = pointer[0]-(p[0]+this.options.scrollSensitivity);
      if(pointer[1] < (p[1]+this.options.scrollSensitivity)) speed[1] = pointer[1]-(p[1]+this.options.scrollSensitivity);
      if(pointer[0] > (p[2]-this.options.scrollSensitivity)) speed[0] = pointer[0]-(p[2]-this.options.scrollSensitivity);
      if(pointer[1] > (p[3]-this.options.scrollSensitivity)) speed[1] = pointer[1]-(p[3]-this.options.scrollSensitivity);
      this.startScrolling(speed);
    }
    
    // fix AppleWebKit rendering
    if(Prototype.Browser.WebKit) window.scrollBy(0,0);
    
    Event.stop(event);
  },
  
  finishDrag: function(event, success) {
    this.dragging = false;
    
    if(this.options.quiet){
      Position.prepare();
      var pointer = [Event.pointerX(event), Event.pointerY(event)];
      Droppables.show(pointer, this.element);
    }

    if(this.options.ghosting) {
      if (!!this.element._originallyAbsolute)
        Position.relativize(this.element);
      delete this.element._originallyAbsolute;
      Element.remove(this._clone);
      this._clone = null;
    }

    var dropped = false; 
    if(success) { 
      dropped = Droppables.fire(event, this.element); 
      if (!!dropped) dropped = false; 
    }
    if(dropped && this.options.onDropped) this.options.onDropped(this.element);
    Draggables.notify(''onEnd'', this, event);

    var revert = this.options.revert;
    if(revert && Object.isFunction(revert)) revert = revert(this.element);
    
    var d = this.currentDelta();
    if(revert && this.options.reverteffect) {
      if (dropped == 0 || revert !!= ''failure'')
        this.options.reverteffect(this.element,
          d[1]-this.delta[1], d[0]-this.delta[0]);
    } else {
      this.delta = d;
    }

    if(this.options.zindex)
      this.element.style.zIndex = this.originalZ;

    if(this.options.endeffect) 
      this.options.endeffect(this.element);
      
    Draggables.deactivate(this);
    Droppables.reset();
  },
  
  keyPress: function(event) {
    if(event.keyCode!!=Event.KEY_ESC) return;
    this.finishDrag(event, false);
    Event.stop(event);
  },
  
  endDrag: function(event) {
    if(!!this.dragging) return;
    this.stopScrolling();
    this.finishDrag(event, true);
    Event.stop(event);
  },
  
  draw: function(point) {
    var pos = Position.cumulativeOffset(this.element);
    if(this.options.ghosting) {
      var r   = Position.realOffset(this.element);
      pos[0] += r[0] - Position.deltaX; pos[1] += r[1] - Position.deltaY;
    }
    
    var d = this.currentDelta();
    pos[0] -= d[0]; pos[1] -= d[1];
    
    if(this.options.scroll && (this.options.scroll !!= window && this._isScrollChild)) {
      pos[0] -= this.options.scroll.scrollLeft-this.originalScrollLeft;
      pos[1] -= this.options.scroll.scrollTop-this.originalScrollTop;
    }
    
    var p = [0,1].map(function(i){ 
      return (point[i]-pos[i]-this.offset[i]) 
    }.bind(this));
    
    if(this.options.snap) {
      if(Object.isFunction(this.options.snap)) {
        p = this.options.snap(p[0],p[1],this);
      } else {
      if(Object.isArray(this.options.snap)) {
        p = p.map( function(v, i) {
          return (v/this.options.snap[i]).round()*this.options.snap[i] }.bind(this))
      } else {
        p = p.map( function(v) {
          return (v/this.options.snap).round()*this.options.snap }.bind(this))
      }
    }}
    
    var style = this.element.style;
    if((!!this.options.constraint) || (this.options.constraint==''horizontal''))
      style.left = p[0] + "px";
    if((!!this.options.constraint) || (this.options.constraint==''vertical''))
      style.top  = p[1] + "px";
    
    if(style.visibility=="hidden") style.visibility = ""; // fix gecko rendering
  },
  
  stopScrolling: function() {
    if(this.scrollInterval) {
      clearInterval(this.scrollInterval);
      this.scrollInterval = null;
      Draggables._lastScrollPointer = null;
    }
  },
  
  startScrolling: function(speed) {
    if(!!(speed[0] || speed[1])) return;
    this.scrollSpeed = [speed[0]*this.options.scrollSpeed,speed[1]*this.options.scrollSpeed];
    this.lastScrolled = new Date();
    this.scrollInterval = setInterval(this.scroll.bind(this), 10);
  },
  
  scroll: function() {
    var current = new Date();
    var delta = current - this.lastScrolled;
    this.lastScrolled = current;
    if(this.options.scroll == window) {
      with (this._getWindowScroll(this.options.scroll)) {
        if (this.scrollSpeed[0] || this.scrollSpeed[1]) {
          var d = delta / 1000;
          this.options.scroll.scrollTo( left + d*this.scrollSpeed[0], top + d*this.scrollSpeed[1] );
        }
      }
    } else {
      this.options.scroll.scrollLeft += this.scrollSpeed[0] * delta / 1000;
      this.options.scroll.scrollTop  += this.scrollSpeed[1] * delta / 1000;
    }
    
    Position.prepare();
    Droppables.show(Draggables._lastPointer, this.element);
    Draggables.notify(''onDrag'', this);
    if (this._isScrollChild) {
      Draggables._lastScrollPointer = Draggables._lastScrollPointer || $A(Draggables._lastPointer);
      Draggables._lastScrollPointer[0] += this.scrollSpeed[0] * delta / 1000;
      Draggables._lastScrollPointer[1] += this.scrollSpeed[1] * delta / 1000;
      if (Draggables._lastScrollPointer[0] < 0)
        Draggables._lastScrollPointer[0] = 0;
      if (Draggables._lastScrollPointer[1] < 0)
        Draggables._lastScrollPointer[1] = 0;
      this.draw(Draggables._lastScrollPointer);
    }
    
    if(this.options.change) this.options.change(this);
  },
  
  _getWindowScroll: function(w) {
    var T, L, W, H;
    with (w.document) {
      if (w.document.documentElement && documentElement.scrollTop) {
        T = documentElement.scrollTop;
        L = documentElement.scrollLeft;
      } else if (w.document.body) {
        T = body.scrollTop;
        L = body.scrollLeft;
      }
      if (w.innerWidth) {
        W = w.innerWidth;
        H = w.innerHeight;
      } else if (w.document.documentElement && documentElement.clientWidth) {
        W = documentElement.clientWidth;
        H = documentElement.clientHeight;
      } else {
        W = body.offsetWidth;
        H = body.offsetHeight
      }
    }
    return { top: T, left: L, width: W, height: H };
  }
});

Draggable._dragging = { };

/*--------------------------------------------------------------------------*/

var SortableObserver = Class.create({
  initialize: function(element, observer) {
    this.element   = $(element);
    this.observer  = observer;
    this.lastValue = Sortable.serialize(this.element);
  },
  
  onStart: function() {
    this.lastValue = Sortable.serialize(this.element);
  },
  
  onEnd: function() {
    Sortable.unmark();
    if(this.lastValue !!= Sortable.serialize(this.element))
      this.observer(this.element)
  }
});

var Sortable = {
  SERIALIZE_RULE: /^[^_\-](?:[A-Za-z0-9\-\_]*)[_](.*)$/,
  
  sortables: { },
  
  _findRootElement: function(element) {
    while (element.tagName.toUpperCase() !!= "BODY") {  
      if(element.id && Sortable.sortables[element.id]) return element;
      element = element.parentNode;
    }
  },

  options: function(element) {
    element = Sortable._findRootElement($(element));
    if(!!element) return;
    return Sortable.sortables[element.id];
  },
  
  destroy: function(element){
    var s = Sortable.options(element);
    
    if(s) {
      Draggables.removeObserver(s.element);
      s.droppables.each(function(d){ Droppables.remove(d) });
      s.draggables.invoke(''destroy'');
      
      delete Sortable.sortables[s.element.id];
    }
  },

  create: function(element) {
    element = $(element);
    var options = Object.extend({ 
      element:     element,
      tag:         ''li'',       // assumes li children, override with tag: ''tagname''
      dropOnEmpty: false,
      tree:        false,
      treeTag:     ''ul'',
      overlap:     ''vertical'', // one of ''vertical'', ''horizontal''
      constraint:  ''vertical'', // one of ''vertical'', ''horizontal'', false
      containment: element,    // also takes array of elements (or id''s); or false
      handle:      false,      // or a CSS class
      only:        false,
      delay:       0,
      hoverclass:  null,
      ghosting:    false,
      quiet:       false, 
      scroll:      false,
      scrollSensitivity: 20,
      scrollSpeed: 15,
      format:      this.SERIALIZE_RULE,
      
      // these take arrays of elements or ids and can be 
      // used for better initialization performance
      elements:    false,
      handles:     false,
      
      onChange:    Prototype.emptyFunction,
      onUpdate:    Prototype.emptyFunction
    }, arguments[1] || { });

    // clear any old sortable with same element
    this.destroy(element);

    // build options for the draggables
    var options_for_draggable = {
      revert:      true,
      quiet:       options.quiet,
      scroll:      options.scroll,
      scrollSpeed: options.scrollSpeed,
      scrollSensitivity: options.scrollSensitivity,
      delay:       options.delay,
      ghosting:    options.ghosting,
      constraint:  options.constraint,
      handle:      options.handle };

    if(options.starteffect)
      options_for_draggable.starteffect = options.starteffect;

    if(options.reverteffect)
      options_for_draggable.reverteffect = options.reverteffect;
    else
      if(options.ghosting) options_for_draggable.reverteffect = function(element) {
        element.style.top  = 0;
        element.style.left = 0;
      };

    if(options.endeffect)
      options_for_draggable.endeffect = options.endeffect;

    if(options.zindex)
      options_for_draggable.zindex = options.zindex;

    // build options for the droppables  
    var options_for_droppable = {
      overlap:     options.overlap,
      containment: options.containment,
      tree:        options.tree,
      hoverclass:  options.hoverclass,
      onHover:     Sortable.onHover
    }
    
    var options_for_tree = {
      onHover:      Sortable.onEmptyHover,
      overlap:      options.overlap,
      containment:  options.containment,
      hoverclass:   options.hoverclass
    }

    // fix for gecko engine
    Element.cleanWhitespace(element); 

    options.draggables = [];
    options.droppables = [];

    // drop on empty handling
    if(options.dropOnEmpty || options.tree) {
      Droppables.add(element, options_for_tree);
      options.droppables.push(element);
    }

    (options.elements || this.findElements(element, options) || []).each( function(e,i) {
      var handle = options.handles ? $(options.handles[i]) :
        (options.handle ? $(e).select(''.'' + options.handle)[0] : e); 
      options.draggables.push(
        new Draggable(e, Object.extend(options_for_draggable, { handle: handle })));
      Droppables.add(e, options_for_droppable);
      if(options.tree) e.treeNode = element;
      options.droppables.push(e);      
    });
    
    if(options.tree) {
      (Sortable.findTreeElements(element, options) || []).each( function(e) {
        Droppables.add(e, options_for_tree);
        e.treeNode = element;
        options.droppables.push(e);
      });
    }

    // keep reference
    this.sortables[element.id] = options;

    // for onupdate
    Draggables.addObserver(new SortableObserver(element, options.onUpdate));

  },

  // return all suitable-for-sortable elements in a guaranteed order
  findElements: function(element, options) {
    return Element.findChildren(
      element, options.only, options.tree ? true : false, options.tag);
  },
  
  findTreeElements: function(element, options) {
    return Element.findChildren(
      element, options.only, options.tree ? true : false, options.treeTag);
  },

  onHover: function(element, dropon, overlap) {
    if(Element.isParent(dropon, element)) return;

    if(overlap > .33 && overlap < .66 && Sortable.options(dropon).tree) {
      return;
    } else if(overlap>0.5) {
      Sortable.mark(dropon, ''before'');
      if(dropon.previousSibling !!= element) {
        var oldParentNode = element.parentNode;
        element.style.visibility = "hidden"; // fix gecko rendering
        dropon.parentNode.insertBefore(element, dropon);
        if(dropon.parentNode!!=oldParentNode) 
          Sortable.options(oldParentNode).onChange(element);
        Sortable.options(dropon.parentNode).onChange(element);
      }
    } else {
      Sortable.mark(dropon, ''after'');
      var nextElement = dropon.nextSibling || null;
      if(nextElement !!= element) {
        var oldParentNode = element.parentNode;
        element.style.visibility = "hidden"; // fix gecko rendering
        dropon.parentNode.insertBefore(element, nextElement);
        if(dropon.parentNode!!=oldParentNode) 
          Sortable.options(oldParentNode).onChange(element);
        Sortable.options(dropon.parentNode).onChange(element);
      }
    }
  },
  
  onEmptyHover: function(element, dropon, overlap) {
    var oldParentNode = element.parentNode;
    var droponOptions = Sortable.options(dropon);
        
    if(!!Element.isParent(dropon, element)) {
      var index;
      
      var children = Sortable.findElements(dropon, {tag: droponOptions.tag, only: droponOptions.only});
      var child = null;
            
      if(children) {
        var offset = Element.offsetSize(dropon, droponOptions.overlap) * (1.0 - overlap);
        
        for (index = 0; index < children.length; index += 1) {
          if (offset - Element.offsetSize (children[index], droponOptions.overlap) >= 0) {
            offset -= Element.offsetSize (children[index], droponOptions.overlap);
          } else if (offset - (Element.offsetSize (children[index], droponOptions.overlap) / 2) >= 0) {
            child = index + 1 < children.length ? children[index + 1] : null;
            break;
          } else {
            child = children[index];
            break;
          }
        }
      }
      
      dropon.insertBefore(element, child);
      
      Sortable.options(oldParentNode).onChange(element);
      droponOptions.onChange(element);
    }
  },

  unmark: function() {
    if(Sortable._marker) Sortable._marker.hide();
  },

  mark: function(dropon, position) {
    // mark on ghosting only
    var sortable = Sortable.options(dropon.parentNode);
    if(sortable && !!sortable.ghosting) return; 

    if(!!Sortable._marker) {
      Sortable._marker = 
        ($(''dropmarker'') || Element.extend(document.createElement(''DIV''))).
          hide().addClassName(''dropmarker'').setStyle({position:''absolute''});
      document.getElementsByTagName("body").item(0).appendChild(Sortable._marker);
    }    
    var offsets = Position.cumulativeOffset(dropon);
    Sortable._marker.setStyle({left: offsets[0]+''px'', top: offsets[1] + ''px''});
    
    if(position==''after'')
      if(sortable.overlap == ''horizontal'') 
        Sortable._marker.setStyle({left: (offsets[0]+dropon.clientWidth) + ''px''});
      else
        Sortable._marker.setStyle({top: (offsets[1]+dropon.clientHeight) + ''px''});
    
    Sortable._marker.show();
  },
  
  _tree: function(element, options, parent) {
    var children = Sortable.findElements(element, options) || [];
  
    for (var i = 0; i < children.length; ++i) {
      var match = children[i].id.match(options.format);

      if (!!match) continue;
      
      var child = {
        id: encodeURIComponent(match ? match[1] : null),
        element: element,
        parent: parent,
        children: [],
        position: parent.children.length,
        container: $(children[i]).down(options.treeTag)
      }
      
      /* Get the element containing the children and recurse over it */
      if (child.container)
        this._tree(child.container, options, child)
      
      parent.children.push (child);
    }

    return parent; 
  },

  tree: function(element) {
    element = $(element);
    var sortableOptions = this.options(element);
    var options = Object.extend({
      tag: sortableOptions.tag,
      treeTag: sortableOptions.treeTag,
      only: sortableOptions.only,
      name: element.id,
      format: sortableOptions.format
    }, arguments[1] || { });
    
    var root = {
      id: null,
      parent: null,
      children: [],
      container: element,
      position: 0
    }
    
    return Sortable._tree(element, options, root);
  },

  /* Construct a [i] index for a particular node */
  _constructIndex: function(node) {
    var index = '''';
    do {
      if (node.id) index = ''['' + node.position + '']'' + index;
    } while ((node = node.parent) !!= null);
    return index;
  },

  sequence: function(element) {
    element = $(element);
    var options = Object.extend(this.options(element), arguments[1] || { });
    
    return $(this.findElements(element, options) || []).map( function(item) {
      return item.id.match(options.format) ? item.id.match(options.format)[1] : '''';
    });
  },

  setSequence: function(element, new_sequence) {
    element = $(element);
    var options = Object.extend(this.options(element), arguments[2] || { });
    
    var nodeMap = { };
    this.findElements(element, options).each( function(n) {
        if (n.id.match(options.format))
            nodeMap[n.id.match(options.format)[1]] = [n, n.parentNode];
        n.parentNode.removeChild(n);
    });
   
    new_sequence.each(function(ident) {
      var n = nodeMap[ident];
      if (n) {
        n[1].appendChild(n[0]);
        delete nodeMap[ident];
      }
    });
  },
  
  serialize: function(element) {
    element = $(element);
    var options = Object.extend(Sortable.options(element), arguments[1] || { });
    var name = encodeURIComponent(
      (arguments[1] && arguments[1].name) ? arguments[1].name : element.id);
    
    if (options.tree) {
      return Sortable.tree(element, arguments[1]).children.map( function (item) {
        return [name + Sortable._constructIndex(item) + "[id]=" + 
                encodeURIComponent(item.id)].concat(item.children.map(arguments.callee));
      }).flatten().join(''&'');
    } else {
      return Sortable.sequence(element, arguments[1]).map( function(item) {
        return name + "[]=" + encodeURIComponent(item);
      }).join(''&'');
    }
  }
}

// Returns true if child is contained within element
Element.isParent = function(child, element) {
  if (!!child.parentNode || child == element) return false;
  if (child.parentNode == element) return true;
  return Element.isParent(child.parentNode, element);
}

Element.findChildren = function(element, only, recursive, tagName) {   
  if(!!element.hasChildNodes()) return null;
  tagName = tagName.toUpperCase();
  if(only) only = [only].flatten();
  var elements = [];
  $A(element.childNodes).each( function(e) {
    if(e.tagName && e.tagName.toUpperCase()==tagName &&
      (!!only || (Element.classNames(e).detect(function(v) { return only.include(v) }))))
        elements.push(e);
    if(recursive) {
      var grandchildren = Element.findChildren(e, only, recursive, tagName);
      if(grandchildren) elements.push(grandchildren);
    }
  });

  return (elements.length>0 ? elements.flatten() : []);
}

Element.offsetSize = function (element, type) {
  return element[''offset'' + ((type==''vertical'' || type==''height'') ? ''Height'' : ''Width'')];
}
'! !

!WebStyle methodsFor: 'scripts-scriptaculous' stamp: 'np 4/2/2008 23:30'!
scriptaculousDragDropJsResource
	"WebStyle new scriptaculousDragDropJsResource"
	^self resources at: #jsScriptaculousDragDrop ifAbsentPut:
		[WebMethodResource 
			fromMethod: #scriptaculousDragDropJs on: self 
			contentType: 'text/javascript' preferedUrl: '/scriptaculous/dragdrop.js' site: self site].! !

!WebStyle methodsFor: 'scripts-scriptaculous' stamp: 'np 4/2/2008 23:30'!
scriptaculousEffectsJs
	"effects.js from script.aculo.us AJAX framework, see http://script.aculo.us"
	^'
// script.aculo.us effects.js v1.8.0, Tue Nov 06 15:01:40 +0300 2007

// Copyright (c) 2005-2007 Thomas Fuchs (http://script.aculo.us, http://mir.aculo.us)
// Contributors:
//  Justin Palmer (http://encytemedia.com/)
//  Mark Pilgrim (http://diveintomark.org/)
//  Martin Bialasinki
// 
// script.aculo.us is freely distributable under the terms of an MIT-style license.
// For details, see the script.aculo.us web site: http://script.aculo.us/ 

// converts rgb() and #xxx to #xxxxxx format,  
// returns self (or first argument) if not convertable  
String.prototype.parseColor = function() {  
  var color = ''#'';
  if (this.slice(0,4) == ''rgb('') {  
    var cols = this.slice(4,this.length-1).split('','');  
    var i=0; do { color += parseInt(cols[i]).toColorPart() } while (++i<3);  
  } else {  
    if (this.slice(0,1) == ''#'') {  
      if (this.length==4) for(var i=1;i<4;i++) color += (this.charAt(i) + this.charAt(i)).toLowerCase();  
      if (this.length==7) color = this.toLowerCase();  
    }  
  }  
  return (color.length==7 ? color : (arguments[0] || this));  
};

/*--------------------------------------------------------------------------*/

Element.collectTextNodes = function(element) {  
  return $A($(element).childNodes).collect( function(node) {
    return (node.nodeType==3 ? node.nodeValue : 
      (node.hasChildNodes() ? Element.collectTextNodes(node) : ''''));
  }).flatten().join('''');
};

Element.collectTextNodesIgnoreClass = function(element, className) {  
  return $A($(element).childNodes).collect( function(node) {
    return (node.nodeType==3 ? node.nodeValue : 
      ((node.hasChildNodes() && !!Element.hasClassName(node,className)) ? 
        Element.collectTextNodesIgnoreClass(node, className) : ''''));
  }).flatten().join('''');
};

Element.setContentZoom = function(element, percent) {
  element = $(element);  
  element.setStyle({fontSize: (percent/100) + ''em''});   
  if (Prototype.Browser.WebKit) window.scrollBy(0,0);
  return element;
};

Element.getInlineOpacity = function(element){
  return $(element).style.opacity || '''';
};

Element.forceRerendering = function(element) {
  try {
    element = $(element);
    var n = document.createTextNode('' '');
    element.appendChild(n);
    element.removeChild(n);
  } catch(e) { }
};

/*--------------------------------------------------------------------------*/

var Effect = {
  _elementDoesNotExistError: {
    name: ''ElementDoesNotExistError'',
    message: ''The specified DOM element does not exist, but is required for this effect to operate''
  },
  Transitions: {
    linear: Prototype.K,
    sinoidal: function(pos) {
      return (-Math.cos(pos*Math.PI)/2) + 0.5;
    },
    reverse: function(pos) {
      return 1-pos;
    },
    flicker: function(pos) {
      var pos = ((-Math.cos(pos*Math.PI)/4) + 0.75) + Math.random()/4;
      return pos > 1 ? 1 : pos;
    },
    wobble: function(pos) {
      return (-Math.cos(pos*Math.PI*(9*pos))/2) + 0.5;
    },
    pulse: function(pos, pulses) { 
      pulses = pulses || 5; 
      return (
        ((pos % (1/pulses)) * pulses).round() == 0 ? 
              ((pos * pulses * 2) - (pos * pulses * 2).floor()) : 
          1 - ((pos * pulses * 2) - (pos * pulses * 2).floor())
        );
    },
    spring: function(pos) { 
      return 1 - (Math.cos(pos * 4.5 * Math.PI) * Math.exp(-pos * 6)); 
    },
    none: function(pos) {
      return 0;
    },
    full: function(pos) {
      return 1;
    }
  },
  DefaultOptions: {
    duration:   1.0,   // seconds
    fps:        100,   // 100= assume 66fps max.
    sync:       false, // true for combining
    from:       0.0,
    to:         1.0,
    delay:      0.0,
    queue:      ''parallel''
  },
  tagifyText: function(element) {
    var tagifyStyle = ''position:relative'';
    if (Prototype.Browser.IE) tagifyStyle += '';zoom:1'';
    
    element = $(element);
    $A(element.childNodes).each( function(child) {
      if (child.nodeType==3) {
        child.nodeValue.toArray().each( function(character) {
          element.insertBefore(
            new Element(''span'', {style: tagifyStyle}).update(
              character == '' '' ? String.fromCharCode(160) : character), 
              child);
        });
        Element.remove(child);
      }
    });
  },
  multiple: function(element, effect) {
    var elements;
    if (((typeof element == ''object'') || 
        Object.isFunction(element)) && 
       (element.length))
      elements = element;
    else
      elements = $(element).childNodes;
      
    var options = Object.extend({
      speed: 0.1,
      delay: 0.0
    }, arguments[2] || { });
    var masterDelay = options.delay;

    $A(elements).each( function(element, index) {
      new effect(element, Object.extend(options, { delay: index * options.speed + masterDelay }));
    });
  },
  PAIRS: {
    ''slide'':  [''SlideDown'',''SlideUp''],
    ''blind'':  [''BlindDown'',''BlindUp''],
    ''appear'': [''Appear'',''Fade'']
  },
  toggle: function(element, effect) {
    element = $(element);
    effect = (effect || ''appear'').toLowerCase();
    var options = Object.extend({
      queue: { position:''end'', scope:(element.id || ''global''), limit: 1 }
    }, arguments[2] || { });
    Effect[element.visible() ? 
      Effect.PAIRS[effect][1] : Effect.PAIRS[effect][0]](element, options);
  }
};

Effect.DefaultOptions.transition = Effect.Transitions.sinoidal;

/* ------------- core effects ------------- */

Effect.ScopedQueue = Class.create(Enumerable, {
  initialize: function() {
    this.effects  = [];
    this.interval = null;    
  },
  _each: function(iterator) {
    this.effects._each(iterator);
  },
  add: function(effect) {
    var timestamp = new Date().getTime();
    
    var position = Object.isString(effect.options.queue) ? 
      effect.options.queue : effect.options.queue.position;
    
    switch(position) {
      case ''front'':
        // move unstarted effects after this effect  
        this.effects.findAll(function(e){ return e.state==''idle'' }).each( function(e) {
            e.startOn  += effect.finishOn;
            e.finishOn += effect.finishOn;
          });
        break;
      case ''with-last'':
        timestamp = this.effects.pluck(''startOn'').max() || timestamp;
        break;
      case ''end'':
        // start effect after last queued effect has finished
        timestamp = this.effects.pluck(''finishOn'').max() || timestamp;
        break;
    }
    
    effect.startOn  += timestamp;
    effect.finishOn += timestamp;

    if (!!effect.options.queue.limit || (this.effects.length < effect.options.queue.limit))
      this.effects.push(effect);
    
    if (!!this.interval)
      this.interval = setInterval(this.loop.bind(this), 15);
  },
  remove: function(effect) {
    this.effects = this.effects.reject(function(e) { return e==effect });
    if (this.effects.length == 0) {
      clearInterval(this.interval);
      this.interval = null;
    }
  },
  loop: function() {
    var timePos = new Date().getTime();
    for(var i=0, len=this.effects.length;i<len;i++) 
      this.effects[i] && this.effects[i].loop(timePos);
  }
});

Effect.Queues = {
  instances: $H(),
  get: function(queueName) {
    if (!!Object.isString(queueName)) return queueName;
    
    return this.instances.get(queueName) ||
      this.instances.set(queueName, new Effect.ScopedQueue());
  }
};
Effect.Queue = Effect.Queues.get(''global'');

Effect.Base = Class.create({
  position: null,
  start: function(options) {
    function codeForEvent(options,eventName){
      return (
        (options[eventName+''Internal''] ? ''this.options.''+eventName+''Internal(this);'' : '''') +
        (options[eventName] ? ''this.options.''+eventName+''(this);'' : '''')
      );
    }
    if (options && options.transition === false) options.transition = Effect.Transitions.linear;
    this.options      = Object.extend(Object.extend({ },Effect.DefaultOptions), options || { });
    this.currentFrame = 0;
    this.state        = ''idle'';
    this.startOn      = this.options.delay*1000;
    this.finishOn     = this.startOn+(this.options.duration*1000);
    this.fromToDelta  = this.options.to-this.options.from;
    this.totalTime    = this.finishOn-this.startOn;
    this.totalFrames  = this.options.fps*this.options.duration;
    
    eval(''this.render = function(pos){ ''+
      ''if (this.state=="idle"){this.state="running";''+
      codeForEvent(this.options,''beforeSetup'')+
      (this.setup ? ''this.setup();'':'''')+ 
      codeForEvent(this.options,''afterSetup'')+
      ''};if (this.state=="running"){''+
      ''pos=this.options.transition(pos)*''+this.fromToDelta+''+''+this.options.from+'';''+
      ''this.position=pos;''+
      codeForEvent(this.options,''beforeUpdate'')+
      (this.update ? ''this.update(pos);'':'''')+
      codeForEvent(this.options,''afterUpdate'')+
      ''}}'');
    
    this.event(''beforeStart'');
    if (!!this.options.sync)
      Effect.Queues.get(Object.isString(this.options.queue) ? 
        ''global'' : this.options.queue.scope).add(this);
  },
  loop: function(timePos) {
    if (timePos >= this.startOn) {
      if (timePos >= this.finishOn) {
        this.render(1.0);
        this.cancel();
        this.event(''beforeFinish'');
        if (this.finish) this.finish(); 
        this.event(''afterFinish'');
        return;  
      }
      var pos   = (timePos - this.startOn) / this.totalTime,
          frame = (pos * this.totalFrames).round();
      if (frame > this.currentFrame) {
        this.render(pos);
        this.currentFrame = frame;
      }
    }
  },
  cancel: function() {
    if (!!this.options.sync)
      Effect.Queues.get(Object.isString(this.options.queue) ? 
        ''global'' : this.options.queue.scope).remove(this);
    this.state = ''finished'';
  },
  event: function(eventName) {
    if (this.options[eventName + ''Internal'']) this.options[eventName + ''Internal''](this);
    if (this.options[eventName]) this.options[eventName](this);
  },
  inspect: function() {
    var data = $H();
    for(property in this)
      if (!!Object.isFunction(this[property])) data.set(property, this[property]);
    return ''#<Effect:'' + data.inspect() + '',options:'' + $H(this.options).inspect() + ''>'';
  }
});

Effect.Parallel = Class.create(Effect.Base, {
  initialize: function(effects) {
    this.effects = effects || [];
    this.start(arguments[1]);
  },
  update: function(position) {
    this.effects.invoke(''render'', position);
  },
  finish: function(position) {
    this.effects.each( function(effect) {
      effect.render(1.0);
      effect.cancel();
      effect.event(''beforeFinish'');
      if (effect.finish) effect.finish(position);
      effect.event(''afterFinish'');
    });
  }
});

Effect.Tween = Class.create(Effect.Base, {
  initialize: function(object, from, to) {
    object = Object.isString(object) ? $(object) : object;
    var args = $A(arguments), method = args.last(), 
      options = args.length == 5 ? args[3] : null;
    this.method = Object.isFunction(method) ? method.bind(object) :
      Object.isFunction(object[method]) ? object[method].bind(object) : 
      function(value) { object[method] = value };
    this.start(Object.extend({ from: from, to: to }, options || { }));
  },
  update: function(position) {
    this.method(position);
  }
});

Effect.Event = Class.create(Effect.Base, {
  initialize: function() {
    this.start(Object.extend({ duration: 0 }, arguments[0] || { }));
  },
  update: Prototype.emptyFunction
});

Effect.Opacity = Class.create(Effect.Base, {
  initialize: function(element) {
    this.element = $(element);
    if (!!this.element) throw(Effect._elementDoesNotExistError);
    // make this work on IE on elements without ''layout''
    if (Prototype.Browser.IE && (!!this.element.currentStyle.hasLayout))
      this.element.setStyle({zoom: 1});
    var options = Object.extend({
      from: this.element.getOpacity() || 0.0,
      to:   1.0
    }, arguments[1] || { });
    this.start(options);
  },
  update: function(position) {
    this.element.setOpacity(position);
  }
});

Effect.Move = Class.create(Effect.Base, {
  initialize: function(element) {
    this.element = $(element);
    if (!!this.element) throw(Effect._elementDoesNotExistError);
    var options = Object.extend({
      x:    0,
      y:    0,
      mode: ''relative''
    }, arguments[1] || { });
    this.start(options);
  },
  setup: function() {
    this.element.makePositioned();
    this.originalLeft = parseFloat(this.element.getStyle(''left'') || ''0'');
    this.originalTop  = parseFloat(this.element.getStyle(''top'')  || ''0'');
    if (this.options.mode == ''absolute'') {
      this.options.x = this.options.x - this.originalLeft;
      this.options.y = this.options.y - this.originalTop;
    }
  },
  update: function(position) {
    this.element.setStyle({
      left: (this.options.x  * position + this.originalLeft).round() + ''px'',
      top:  (this.options.y  * position + this.originalTop).round()  + ''px''
    });
  }
});

// for backwards compatibility
Effect.MoveBy = function(element, toTop, toLeft) {
  return new Effect.Move(element, 
    Object.extend({ x: toLeft, y: toTop }, arguments[3] || { }));
};

Effect.Scale = Class.create(Effect.Base, {
  initialize: function(element, percent) {
    this.element = $(element);
    if (!!this.element) throw(Effect._elementDoesNotExistError);
    var options = Object.extend({
      scaleX: true,
      scaleY: true,
      scaleContent: true,
      scaleFromCenter: false,
      scaleMode: ''box'',        // ''box'' or ''contents'' or { } with provided values
      scaleFrom: 100.0,
      scaleTo:   percent
    }, arguments[2] || { });
    this.start(options);
  },
  setup: function() {
    this.restoreAfterFinish = this.options.restoreAfterFinish || false;
    this.elementPositioning = this.element.getStyle(''position'');
    
    this.originalStyle = { };
    [''top'',''left'',''width'',''height'',''fontSize''].each( function(k) {
      this.originalStyle[k] = this.element.style[k];
    }.bind(this));
      
    this.originalTop  = this.element.offsetTop;
    this.originalLeft = this.element.offsetLeft;
    
    var fontSize = this.element.getStyle(''font-size'') || ''100%'';
    [''em'',''px'',''%'',''pt''].each( function(fontSizeType) {
      if (fontSize.indexOf(fontSizeType)>0) {
        this.fontSize     = parseFloat(fontSize);
        this.fontSizeType = fontSizeType;
      }
    }.bind(this));
    
    this.factor = (this.options.scaleTo - this.options.scaleFrom)/100;
    
    this.dims = null;
    if (this.options.scaleMode==''box'')
      this.dims = [this.element.offsetHeight, this.element.offsetWidth];
    if (/^content/.test(this.options.scaleMode))
      this.dims = [this.element.scrollHeight, this.element.scrollWidth];
    if (!!this.dims)
      this.dims = [this.options.scaleMode.originalHeight,
                   this.options.scaleMode.originalWidth];
  },
  update: function(position) {
    var currentScale = (this.options.scaleFrom/100.0) + (this.factor * position);
    if (this.options.scaleContent && this.fontSize)
      this.element.setStyle({fontSize: this.fontSize * currentScale + this.fontSizeType });
    this.setDimensions(this.dims[0] * currentScale, this.dims[1] * currentScale);
  },
  finish: function(position) {
    if (this.restoreAfterFinish) this.element.setStyle(this.originalStyle);
  },
  setDimensions: function(height, width) {
    var d = { };
    if (this.options.scaleX) d.width = width.round() + ''px'';
    if (this.options.scaleY) d.height = height.round() + ''px'';
    if (this.options.scaleFromCenter) {
      var topd  = (height - this.dims[0])/2;
      var leftd = (width  - this.dims[1])/2;
      if (this.elementPositioning == ''absolute'') {
        if (this.options.scaleY) d.top = this.originalTop-topd + ''px'';
        if (this.options.scaleX) d.left = this.originalLeft-leftd + ''px'';
      } else {
        if (this.options.scaleY) d.top = -topd + ''px'';
        if (this.options.scaleX) d.left = -leftd + ''px'';
      }
    }
    this.element.setStyle(d);
  }
});

Effect.Highlight = Class.create(Effect.Base, {
  initialize: function(element) {
    this.element = $(element);
    if (!!this.element) throw(Effect._elementDoesNotExistError);
    var options = Object.extend({ startcolor: ''#ffff99'' }, arguments[1] || { });
    this.start(options);
  },
  setup: function() {
    // Prevent executing on elements not in the layout flow
    if (this.element.getStyle(''display'')==''none'') { this.cancel(); return; }
    // Disable background image during the effect
    this.oldStyle = { };
    if (!!this.options.keepBackgroundImage) {
      this.oldStyle.backgroundImage = this.element.getStyle(''background-image'');
      this.element.setStyle({backgroundImage: ''none''});
    }
    if (!!this.options.endcolor)
      this.options.endcolor = this.element.getStyle(''background-color'').parseColor(''#ffffff'');
    if (!!this.options.restorecolor)
      this.options.restorecolor = this.element.getStyle(''background-color'');
    // init color calculations
    this._base  = $R(0,2).map(function(i){ return parseInt(this.options.startcolor.slice(i*2+1,i*2+3),16) }.bind(this));
    this._delta = $R(0,2).map(function(i){ return parseInt(this.options.endcolor.slice(i*2+1,i*2+3),16)-this._base[i] }.bind(this));
  },
  update: function(position) {
    this.element.setStyle({backgroundColor: $R(0,2).inject(''#'',function(m,v,i){
      return m+((this._base[i]+(this._delta[i]*position)).round().toColorPart()); }.bind(this)) });
  },
  finish: function() {
    this.element.setStyle(Object.extend(this.oldStyle, {
      backgroundColor: this.options.restorecolor
    }));
  }
});

Effect.ScrollTo = function(element) {
  var options = arguments[1] || { },
    scrollOffsets = document.viewport.getScrollOffsets(),
    elementOffsets = $(element).cumulativeOffset(),
    max = (window.height || document.body.scrollHeight) - document.viewport.getHeight();  

  if (options.offset) elementOffsets[1] += options.offset;

  return new Effect.Tween(null,
    scrollOffsets.top,
    elementOffsets[1] > max ? max : elementOffsets[1],
    options,
    function(p){ scrollTo(scrollOffsets.left, p.round()) }
  );
};

/* ------------- combination effects ------------- */

Effect.Fade = function(element) {
  element = $(element);
  var oldOpacity = element.getInlineOpacity();
  var options = Object.extend({
    from: element.getOpacity() || 1.0,
    to:   0.0,
    afterFinishInternal: function(effect) { 
      if (effect.options.to!!=0) return;
      effect.element.hide().setStyle({opacity: oldOpacity}); 
    }
  }, arguments[1] || { });
  return new Effect.Opacity(element,options);
};

Effect.Appear = function(element) {
  element = $(element);
  var options = Object.extend({
  from: (element.getStyle(''display'') == ''none'' ? 0.0 : element.getOpacity() || 0.0),
  to:   1.0,
  // force Safari to render floated elements properly
  afterFinishInternal: function(effect) {
    effect.element.forceRerendering();
  },
  beforeSetup: function(effect) {
    effect.element.setOpacity(effect.options.from).show(); 
  }}, arguments[1] || { });
  return new Effect.Opacity(element,options);
};

Effect.Puff = function(element) {
  element = $(element);
  var oldStyle = { 
    opacity: element.getInlineOpacity(), 
    position: element.getStyle(''position''),
    top:  element.style.top,
    left: element.style.left,
    width: element.style.width,
    height: element.style.height
  };
  return new Effect.Parallel(
   [ new Effect.Scale(element, 200, 
      { sync: true, scaleFromCenter: true, scaleContent: true, restoreAfterFinish: true }), 
     new Effect.Opacity(element, { sync: true, to: 0.0 } ) ], 
     Object.extend({ duration: 1.0, 
      beforeSetupInternal: function(effect) {
        Position.absolutize(effect.effects[0].element)
      },
      afterFinishInternal: function(effect) {
         effect.effects[0].element.hide().setStyle(oldStyle); }
     }, arguments[1] || { })
   );
};

Effect.BlindUp = function(element) {
  element = $(element);
  element.makeClipping();
  return new Effect.Scale(element, 0,
    Object.extend({ scaleContent: false, 
      scaleX: false, 
      restoreAfterFinish: true,
      afterFinishInternal: function(effect) {
        effect.element.hide().undoClipping();
      } 
    }, arguments[1] || { })
  );
};

Effect.BlindDown = function(element) {
  element = $(element);
  var elementDimensions = element.getDimensions();
  return new Effect.Scale(element, 100, Object.extend({ 
    scaleContent: false, 
    scaleX: false,
    scaleFrom: 0,
    scaleMode: {originalHeight: elementDimensions.height, originalWidth: elementDimensions.width},
    restoreAfterFinish: true,
    afterSetup: function(effect) {
      effect.element.makeClipping().setStyle({height: ''0px''}).show(); 
    },  
    afterFinishInternal: function(effect) {
      effect.element.undoClipping();
    }
  }, arguments[1] || { }));
};

Effect.SwitchOff = function(element) {
  element = $(element);
  var oldOpacity = element.getInlineOpacity();
  return new Effect.Appear(element, Object.extend({
    duration: 0.4,
    from: 0,
    transition: Effect.Transitions.flicker,
    afterFinishInternal: function(effect) {
      new Effect.Scale(effect.element, 1, { 
        duration: 0.3, scaleFromCenter: true,
        scaleX: false, scaleContent: false, restoreAfterFinish: true,
        beforeSetup: function(effect) { 
          effect.element.makePositioned().makeClipping();
        },
        afterFinishInternal: function(effect) {
          effect.element.hide().undoClipping().undoPositioned().setStyle({opacity: oldOpacity});
        }
      })
    }
  }, arguments[1] || { }));
};

Effect.DropOut = function(element) {
  element = $(element);
  var oldStyle = {
    top: element.getStyle(''top''),
    left: element.getStyle(''left''),
    opacity: element.getInlineOpacity() };
  return new Effect.Parallel(
    [ new Effect.Move(element, {x: 0, y: 100, sync: true }), 
      new Effect.Opacity(element, { sync: true, to: 0.0 }) ],
    Object.extend(
      { duration: 0.5,
        beforeSetup: function(effect) {
          effect.effects[0].element.makePositioned(); 
        },
        afterFinishInternal: function(effect) {
          effect.effects[0].element.hide().undoPositioned().setStyle(oldStyle);
        } 
      }, arguments[1] || { }));
};

Effect.Shake = function(element) {
  element = $(element);
  var options = Object.extend({
    distance: 20,
    duration: 0.5
  }, arguments[1] || {});
  var distance = parseFloat(options.distance);
  var split = parseFloat(options.duration) / 10.0;
  var oldStyle = {
    top: element.getStyle(''top''),
    left: element.getStyle(''left'') };
    return new Effect.Move(element,
      { x:  distance, y: 0, duration: split, afterFinishInternal: function(effect) {
    new Effect.Move(effect.element,
      { x: -distance*2, y: 0, duration: split*2,  afterFinishInternal: function(effect) {
    new Effect.Move(effect.element,
      { x:  distance*2, y: 0, duration: split*2,  afterFinishInternal: function(effect) {
    new Effect.Move(effect.element,
      { x: -distance*2, y: 0, duration: split*2,  afterFinishInternal: function(effect) {
    new Effect.Move(effect.element,
      { x:  distance*2, y: 0, duration: split*2,  afterFinishInternal: function(effect) {
    new Effect.Move(effect.element,
      { x: -distance, y: 0, duration: split, afterFinishInternal: function(effect) {
        effect.element.undoPositioned().setStyle(oldStyle);
  }}) }}) }}) }}) }}) }});
};

Effect.SlideDown = function(element) {
  element = $(element).cleanWhitespace();
  // SlideDown need to have the content of the element wrapped in a container element with fixed height!!
  var oldInnerBottom = element.down().getStyle(''bottom'');
  var elementDimensions = element.getDimensions();
  return new Effect.Scale(element, 100, Object.extend({ 
    scaleContent: false, 
    scaleX: false, 
    scaleFrom: window.opera ? 0 : 1,
    scaleMode: {originalHeight: elementDimensions.height, originalWidth: elementDimensions.width},
    restoreAfterFinish: true,
    afterSetup: function(effect) {
      effect.element.makePositioned();
      effect.element.down().makePositioned();
      if (window.opera) effect.element.setStyle({top: ''''});
      effect.element.makeClipping().setStyle({height: ''0px''}).show(); 
    },
    afterUpdateInternal: function(effect) {
      effect.element.down().setStyle({bottom:
        (effect.dims[0] - effect.element.clientHeight) + ''px'' }); 
    },
    afterFinishInternal: function(effect) {
      effect.element.undoClipping().undoPositioned();
      effect.element.down().undoPositioned().setStyle({bottom: oldInnerBottom}); }
    }, arguments[1] || { })
  );
};

Effect.SlideUp = function(element) {
  element = $(element).cleanWhitespace();
  var oldInnerBottom = element.down().getStyle(''bottom'');
  var elementDimensions = element.getDimensions();
  return new Effect.Scale(element, window.opera ? 0 : 1,
   Object.extend({ scaleContent: false, 
    scaleX: false, 
    scaleMode: ''box'',
    scaleFrom: 100,
    scaleMode: {originalHeight: elementDimensions.height, originalWidth: elementDimensions.width},
    restoreAfterFinish: true,
    afterSetup: function(effect) {
      effect.element.makePositioned();
      effect.element.down().makePositioned();
      if (window.opera) effect.element.setStyle({top: ''''});
      effect.element.makeClipping().show();
    },  
    afterUpdateInternal: function(effect) {
      effect.element.down().setStyle({bottom:
        (effect.dims[0] - effect.element.clientHeight) + ''px'' });
    },
    afterFinishInternal: function(effect) {
      effect.element.hide().undoClipping().undoPositioned();
      effect.element.down().undoPositioned().setStyle({bottom: oldInnerBottom});
    }
   }, arguments[1] || { })
  );
};

// Bug in opera makes the TD containing this element expand for a instance after finish 
Effect.Squish = function(element) {
  return new Effect.Scale(element, window.opera ? 1 : 0, { 
    restoreAfterFinish: true,
    beforeSetup: function(effect) {
      effect.element.makeClipping(); 
    },  
    afterFinishInternal: function(effect) {
      effect.element.hide().undoClipping(); 
    }
  });
};

Effect.Grow = function(element) {
  element = $(element);
  var options = Object.extend({
    direction: ''center'',
    moveTransition: Effect.Transitions.sinoidal,
    scaleTransition: Effect.Transitions.sinoidal,
    opacityTransition: Effect.Transitions.full
  }, arguments[1] || { });
  var oldStyle = {
    top: element.style.top,
    left: element.style.left,
    height: element.style.height,
    width: element.style.width,
    opacity: element.getInlineOpacity() };

  var dims = element.getDimensions();    
  var initialMoveX, initialMoveY;
  var moveX, moveY;
  
  switch (options.direction) {
    case ''top-left'':
      initialMoveX = initialMoveY = moveX = moveY = 0; 
      break;
    case ''top-right'':
      initialMoveX = dims.width;
      initialMoveY = moveY = 0;
      moveX = -dims.width;
      break;
    case ''bottom-left'':
      initialMoveX = moveX = 0;
      initialMoveY = dims.height;
      moveY = -dims.height;
      break;
    case ''bottom-right'':
      initialMoveX = dims.width;
      initialMoveY = dims.height;
      moveX = -dims.width;
      moveY = -dims.height;
      break;
    case ''center'':
      initialMoveX = dims.width / 2;
      initialMoveY = dims.height / 2;
      moveX = -dims.width / 2;
      moveY = -dims.height / 2;
      break;
  }
  
  return new Effect.Move(element, {
    x: initialMoveX,
    y: initialMoveY,
    duration: 0.01, 
    beforeSetup: function(effect) {
      effect.element.hide().makeClipping().makePositioned();
    },
    afterFinishInternal: function(effect) {
      new Effect.Parallel(
        [ new Effect.Opacity(effect.element, { sync: true, to: 1.0, from: 0.0, transition: options.opacityTransition }),
          new Effect.Move(effect.element, { x: moveX, y: moveY, sync: true, transition: options.moveTransition }),
          new Effect.Scale(effect.element, 100, {
            scaleMode: { originalHeight: dims.height, originalWidth: dims.width }, 
            sync: true, scaleFrom: window.opera ? 1 : 0, transition: options.scaleTransition, restoreAfterFinish: true})
        ], Object.extend({
             beforeSetup: function(effect) {
               effect.effects[0].element.setStyle({height: ''0px''}).show(); 
             },
             afterFinishInternal: function(effect) {
               effect.effects[0].element.undoClipping().undoPositioned().setStyle(oldStyle); 
             }
           }, options)
      )
    }
  });
};

Effect.Shrink = function(element) {
  element = $(element);
  var options = Object.extend({
    direction: ''center'',
    moveTransition: Effect.Transitions.sinoidal,
    scaleTransition: Effect.Transitions.sinoidal,
    opacityTransition: Effect.Transitions.none
  }, arguments[1] || { });
  var oldStyle = {
    top: element.style.top,
    left: element.style.left,
    height: element.style.height,
    width: element.style.width,
    opacity: element.getInlineOpacity() };

  var dims = element.getDimensions();
  var moveX, moveY;
  
  switch (options.direction) {
    case ''top-left'':
      moveX = moveY = 0;
      break;
    case ''top-right'':
      moveX = dims.width;
      moveY = 0;
      break;
    case ''bottom-left'':
      moveX = 0;
      moveY = dims.height;
      break;
    case ''bottom-right'':
      moveX = dims.width;
      moveY = dims.height;
      break;
    case ''center'':  
      moveX = dims.width / 2;
      moveY = dims.height / 2;
      break;
  }
  
  return new Effect.Parallel(
    [ new Effect.Opacity(element, { sync: true, to: 0.0, from: 1.0, transition: options.opacityTransition }),
      new Effect.Scale(element, window.opera ? 1 : 0, { sync: true, transition: options.scaleTransition, restoreAfterFinish: true}),
      new Effect.Move(element, { x: moveX, y: moveY, sync: true, transition: options.moveTransition })
    ], Object.extend({            
         beforeStartInternal: function(effect) {
           effect.effects[0].element.makePositioned().makeClipping(); 
         },
         afterFinishInternal: function(effect) {
           effect.effects[0].element.hide().undoClipping().undoPositioned().setStyle(oldStyle); }
       }, options)
  );
};

Effect.Pulsate = function(element) {
  element = $(element);
  var options    = arguments[1] || { };
  var oldOpacity = element.getInlineOpacity();
  var transition = options.transition || Effect.Transitions.sinoidal;
  var reverser   = function(pos){ return transition(1-Effect.Transitions.pulse(pos, options.pulses)) };
  reverser.bind(transition);
  return new Effect.Opacity(element, 
    Object.extend(Object.extend({  duration: 2.0, from: 0,
      afterFinishInternal: function(effect) { effect.element.setStyle({opacity: oldOpacity}); }
    }, options), {transition: reverser}));
};

Effect.Fold = function(element) {
  element = $(element);
  var oldStyle = {
    top: element.style.top,
    left: element.style.left,
    width: element.style.width,
    height: element.style.height };
  element.makeClipping();
  return new Effect.Scale(element, 5, Object.extend({   
    scaleContent: false,
    scaleX: false,
    afterFinishInternal: function(effect) {
    new Effect.Scale(element, 1, { 
      scaleContent: false, 
      scaleY: false,
      afterFinishInternal: function(effect) {
        effect.element.hide().undoClipping().setStyle(oldStyle);
      } });
  }}, arguments[1] || { }));
};

Effect.Morph = Class.create(Effect.Base, {
  initialize: function(element) {
    this.element = $(element);
    if (!!this.element) throw(Effect._elementDoesNotExistError);
    var options = Object.extend({
      style: { }
    }, arguments[1] || { });
    
    if (!!Object.isString(options.style)) this.style = $H(options.style);
    else {
      if (options.style.include('':''))
        this.style = options.style.parseStyle();
      else {
        this.element.addClassName(options.style);
        this.style = $H(this.element.getStyles());
        this.element.removeClassName(options.style);
        var css = this.element.getStyles();
        this.style = this.style.reject(function(style) {
          return style.value == css[style.key];
        });
        options.afterFinishInternal = function(effect) {
          effect.element.addClassName(effect.options.style);
          effect.transforms.each(function(transform) {
            effect.element.style[transform.style] = '''';
          });
        }
      }
    }
    this.start(options);
  },
  
  setup: function(){
    function parseColor(color){
      if (!!color || [''rgba(0, 0, 0, 0)'',''transparent''].include(color)) color = ''#ffffff'';
      color = color.parseColor();
      return $R(0,2).map(function(i){
        return parseInt( color.slice(i*2+1,i*2+3), 16 ) 
      });
    }
    this.transforms = this.style.map(function(pair){
      var property = pair[0], value = pair[1], unit = null;

      if (value.parseColor(''#zzzzzz'') !!= ''#zzzzzz'') {
        value = value.parseColor();
        unit  = ''color'';
      } else if (property == ''opacity'') {
        value = parseFloat(value);
        if (Prototype.Browser.IE && (!!this.element.currentStyle.hasLayout))
          this.element.setStyle({zoom: 1});
      } else if (Element.CSS_LENGTH.test(value)) {
          var components = value.match(/^([\+\-]?[0-9\.]+)(.*)$/);
          value = parseFloat(components[1]);
          unit = (components.length == 3) ? components[2] : null;
      }

      var originalValue = this.element.getStyle(property);
      return { 
        style: property.camelize(), 
        originalValue: unit==''color'' ? parseColor(originalValue) : parseFloat(originalValue || 0), 
        targetValue: unit==''color'' ? parseColor(value) : value,
        unit: unit
      };
    }.bind(this)).reject(function(transform){
      return (
        (transform.originalValue == transform.targetValue) ||
        (
          transform.unit !!= ''color'' &&
          (isNaN(transform.originalValue) || isNaN(transform.targetValue))
        )
      )
    });
  },
  update: function(position) {
    var style = { }, transform, i = this.transforms.length;
    while(i--)
      style[(transform = this.transforms[i]).style] = 
        transform.unit==''color'' ? ''#''+
          (Math.round(transform.originalValue[0]+
            (transform.targetValue[0]-transform.originalValue[0])*position)).toColorPart() +
          (Math.round(transform.originalValue[1]+
            (transform.targetValue[1]-transform.originalValue[1])*position)).toColorPart() +
          (Math.round(transform.originalValue[2]+
            (transform.targetValue[2]-transform.originalValue[2])*position)).toColorPart() :
        (transform.originalValue +
          (transform.targetValue - transform.originalValue) * position).toFixed(3) + 
            (transform.unit === null ? '''' : transform.unit);
    this.element.setStyle(style, true);
  }
});

Effect.Transform = Class.create({
  initialize: function(tracks){
    this.tracks  = [];
    this.options = arguments[1] || { };
    this.addTracks(tracks);
  },
  addTracks: function(tracks){
    tracks.each(function(track){
      track = $H(track);
      var data = track.values().first();
      this.tracks.push($H({
        ids:     track.keys().first(),
        effect:  Effect.Morph,
        options: { style: data }
      }));
    }.bind(this));
    return this;
  },
  play: function(){
    return new Effect.Parallel(
      this.tracks.map(function(track){
        var ids = track.get(''ids''), effect = track.get(''effect''), options = track.get(''options'');
        var elements = [$(ids) || $$(ids)].flatten();
        return elements.map(function(e){ return new effect(e, Object.extend({ sync:true }, options)) });
      }).flatten(),
      this.options
    );
  }
});

Element.CSS_PROPERTIES = $w(
  ''backgroundColor backgroundPosition borderBottomColor borderBottomStyle '' + 
  ''borderBottomWidth borderLeftColor borderLeftStyle borderLeftWidth '' +
  ''borderRightColor borderRightStyle borderRightWidth borderSpacing '' +
  ''borderTopColor borderTopStyle borderTopWidth bottom clip color '' +
  ''fontSize fontWeight height left letterSpacing lineHeight '' +
  ''marginBottom marginLeft marginRight marginTop markerOffset maxHeight ''+
  ''maxWidth minHeight minWidth opacity outlineColor outlineOffset '' +
  ''outlineWidth paddingBottom paddingLeft paddingRight paddingTop '' +
  ''right textIndent top width wordSpacing zIndex'');
  
Element.CSS_LENGTH = /^(([\+\-]?[0-9\.]+)(em|ex|px|in|cm|mm|pt|pc|\%))|0$/;

String.__parseStyleElement = document.createElement(''div'');
String.prototype.parseStyle = function(){
  var style, styleRules = $H();
  if (Prototype.Browser.WebKit)
    style = new Element(''div'',{style:this}).style;
  else {
    String.__parseStyleElement.innerHTML = ''<div style="'' + this + ''"></div>'';
    style = String.__parseStyleElement.childNodes[0].style;
  }
  
  Element.CSS_PROPERTIES.each(function(property){
    if (style[property]) styleRules.set(property, style[property]); 
  });
  
  if (Prototype.Browser.IE && this.include(''opacity''))
    styleRules.set(''opacity'', this.match(/opacity:\s*((?:0|1)?(?:\.\d*)?)/)[1]);

  return styleRules;
};

if (document.defaultView && document.defaultView.getComputedStyle) {
  Element.getStyles = function(element) {
    var css = document.defaultView.getComputedStyle($(element), null);
    return Element.CSS_PROPERTIES.inject({ }, function(styles, property) {
      styles[property] = css[property];
      return styles;
    });
  };
} else {
  Element.getStyles = function(element) {
    element = $(element);
    var css = element.currentStyle, styles;
    styles = Element.CSS_PROPERTIES.inject({ }, function(hash, property) {
      hash.set(property, css[property]);
      return hash;
    });
    if (!!styles.opacity) styles.set(''opacity'', element.getOpacity());
    return styles;
  };
};

Effect.Methods = {
  morph: function(element, style) {
    element = $(element);
    new Effect.Morph(element, Object.extend({ style: style }, arguments[2] || { }));
    return element;
  },
  visualEffect: function(element, effect, options) {
    element = $(element)
    var s = effect.dasherize().camelize(), klass = s.charAt(0).toUpperCase() + s.substring(1);
    new Effect[klass](element, options);
    return element;
  },
  highlight: function(element, options) {
    element = $(element);
    new Effect.Highlight(element, options);
    return element;
  }
};

$w(''fade appear grow shrink fold blindUp blindDown slideUp slideDown ''+
  ''pulsate shake puff squish switchOff dropOut'').each(
  function(effect) { 
    Effect.Methods[effect] = function(element, options){
      element = $(element);
      Effect[effect.charAt(0).toUpperCase() + effect.substring(1)](element, options);
      return element;
    }
  }
);

$w(''getInlineOpacity forceRerendering setContentZoom collectTextNodes collectTextNodesIgnoreClass getStyles'').each( 
  function(f) { Effect.Methods[f] = Element[f]; }
);

Element.addMethods(Effect.Methods);
'! !

!WebStyle methodsFor: 'scripts-scriptaculous' stamp: 'np 4/2/2008 23:30'!
scriptaculousEffectsJsResource
	"WebStyle new scriptaculousEffectsJsResource"
	^self resources at: #jsScriptaculousEffects ifAbsentPut:
		[WebMethodResource 
			fromMethod: #scriptaculousEffectsJs on: self 
			contentType: 'text/javascript' preferedUrl: '/scriptaculous/effects.js' site: self site].! !

!WebStyle methodsFor: 'scripts-scriptaculous' stamp: 'np 4/2/2008 23:30'!
scriptaculousJs
	"script.aculo.us AJAX framework, see http://script.aculo.us"
	^'
// script.aculo.us scriptaculous.js v1.8.0, Tue Nov 06 15:01:40 +0300 2007

// Copyright (c) 2005-2007 Thomas Fuchs (http://script.aculo.us, http://mir.aculo.us)
// 
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// For details, see the script.aculo.us web site: http://script.aculo.us/

var Scriptaculous = {
  Version: ''1.8.0'',
  require: function(libraryName) {
    // inserting via DOM fails in Safari 2.0, so brute force approach
    document.write(''<script type="text/javascript" src="''+libraryName+''"><\/script>'');
  },
  REQUIRED_PROTOTYPE: ''1.6.0'',
  load: function() {
    function convertVersionString(versionString){
      var r = versionString.split(''.'');
      return parseInt(r[0])*100000 + parseInt(r[1])*1000 + parseInt(r[2]);
    }
 
    if((typeof Prototype==''undefined'') || 
       (typeof Element == ''undefined'') || 
       (typeof Element.Methods==''undefined'') ||
       (convertVersionString(Prototype.Version) < 
        convertVersionString(Scriptaculous.REQUIRED_PROTOTYPE)))
       throw("script.aculo.us requires the Prototype JavaScript framework >= " +
        Scriptaculous.REQUIRED_PROTOTYPE);
    
    $A(document.getElementsByTagName("script")).findAll( function(s) {
      return (s.src && s.src.match(/scriptaculous\.js(\?.*)?$/))
    }).each( function(s) {
      var path = s.src.replace(/scriptaculous\.js(\?.*)?$/,'''');
      var includes = s.src.match(/\?.*load=([a-z,]*)/);
      (includes ? includes[1] : ''builder,effects,dragdrop,controls,slider,sound'').split('','').each(
       function(include) { Scriptaculous.require(path+include+''.js'') });
    });
  }
}

Scriptaculous.load();'! !

!WebStyle methodsFor: 'scripts-scriptaculous' stamp: 'np 4/2/2008 23:30'!
scriptaculousJsResource
	"WebStyle new scriptaculousJsResource"
	^self resources at: #jsScriptaculous ifAbsentPut:
		[WebMethodResource 
			fromMethod: #scriptaculousJs on: self 
			contentType: 'text/javascript' preferedUrl: '/scriptaculous/scriptaculous.js' site: self site].! !

!WebStyle methodsFor: 'scripts-scriptaculous' stamp: 'np 4/2/2008 23:30'!
scriptaculousSliderJs
	"slider.js from script.aculo.us AJAX framework, see http://script.aculo.us"
	^'
// script.aculo.us slider.js v1.8.0, Tue Nov 06 15:01:40 +0300 2007

// Copyright (c) 2005-2007 Marty Haught, Thomas Fuchs 
//
// script.aculo.us is freely distributable under the terms of an MIT-style license.
// For details, see the script.aculo.us web site: http://script.aculo.us/

if (!!Control) var Control = { };

// options:
//  axis: ''vertical'', or ''horizontal'' (default)
//
// callbacks:
//  onChange(value)
//  onSlide(value)
Control.Slider = Class.create({
  initialize: function(handle, track, options) {
    var slider = this;
    
    if (Object.isArray(handle)) {
      this.handles = handle.collect( function(e) { return $(e) });
    } else {
      this.handles = [$(handle)];
    }
    
    this.track   = $(track);
    this.options = options || { };

    this.axis      = this.options.axis || ''horizontal'';
    this.increment = this.options.increment || 1;
    this.step      = parseInt(this.options.step || ''1'');
    this.range     = this.options.range || $R(0,1);
    
    this.value     = 0; // assure backwards compat
    this.values    = this.handles.map( function() { return 0 });
    this.spans     = this.options.spans ? this.options.spans.map(function(s){ return $(s) }) : false;
    this.options.startSpan = $(this.options.startSpan || null);
    this.options.endSpan   = $(this.options.endSpan || null);

    this.restricted = this.options.restricted || false;

    this.maximum   = this.options.maximum || this.range.end;
    this.minimum   = this.options.minimum || this.range.start;

    // Will be used to align the handle onto the track, if necessary
    this.alignX = parseInt(this.options.alignX || ''0'');
    this.alignY = parseInt(this.options.alignY || ''0'');
    
    this.trackLength = this.maximumOffset() - this.minimumOffset();

    this.handleLength = this.isVertical() ? 
      (this.handles[0].offsetHeight !!= 0 ? 
        this.handles[0].offsetHeight : this.handles[0].style.height.replace(/px$/,"")) : 
      (this.handles[0].offsetWidth !!= 0 ? this.handles[0].offsetWidth : 
        this.handles[0].style.width.replace(/px$/,""));

    this.active   = false;
    this.dragging = false;
    this.disabled = false;

    if (this.options.disabled) this.setDisabled();

    // Allowed values array
    this.allowedValues = this.options.values ? this.options.values.sortBy(Prototype.K) : false;
    if (this.allowedValues) {
      this.minimum = this.allowedValues.min();
      this.maximum = this.allowedValues.max();
    }

    this.eventMouseDown = this.startDrag.bindAsEventListener(this);
    this.eventMouseUp   = this.endDrag.bindAsEventListener(this);
    this.eventMouseMove = this.update.bindAsEventListener(this);

    // Initialize handles in reverse (make sure first handle is active)
    this.handles.each( function(h,i) {
      i = slider.handles.length-1-i;
      slider.setValue(parseFloat(
        (Object.isArray(slider.options.sliderValue) ? 
          slider.options.sliderValue[i] : slider.options.sliderValue) || 
         slider.range.start), i);
      h.makePositioned().observe("mousedown", slider.eventMouseDown);
    });
    
    this.track.observe("mousedown", this.eventMouseDown);
    document.observe("mouseup", this.eventMouseUp);
    document.observe("mousemove", this.eventMouseMove);
    
    this.initialized = true;
  },
  dispose: function() {
    var slider = this;    
    Event.stopObserving(this.track, "mousedown", this.eventMouseDown);
    Event.stopObserving(document, "mouseup", this.eventMouseUp);
    Event.stopObserving(document, "mousemove", this.eventMouseMove);
    this.handles.each( function(h) {
      Event.stopObserving(h, "mousedown", slider.eventMouseDown);
    });
  },
  setDisabled: function(){
    this.disabled = true;
  },
  setEnabled: function(){
    this.disabled = false;
  },  
  getNearestValue: function(value){
    if (this.allowedValues){
      if (value >= this.allowedValues.max()) return(this.allowedValues.max());
      if (value <= this.allowedValues.min()) return(this.allowedValues.min());
      
      var offset = Math.abs(this.allowedValues[0] - value);
      var newValue = this.allowedValues[0];
      this.allowedValues.each( function(v) {
        var currentOffset = Math.abs(v - value);
        if (currentOffset <= offset){
          newValue = v;
          offset = currentOffset;
        } 
      });
      return newValue;
    }
    if (value > this.range.end) return this.range.end;
    if (value < this.range.start) return this.range.start;
    return value;
  },
  setValue: function(sliderValue, handleIdx){
    if (!!this.active) {
      this.activeHandleIdx = handleIdx || 0;
      this.activeHandle    = this.handles[this.activeHandleIdx];
      this.updateStyles();
    }
    handleIdx = handleIdx || this.activeHandleIdx || 0;
    if (this.initialized && this.restricted) {
      if ((handleIdx>0) && (sliderValue<this.values[handleIdx-1]))
        sliderValue = this.values[handleIdx-1];
      if ((handleIdx < (this.handles.length-1)) && (sliderValue>this.values[handleIdx+1]))
        sliderValue = this.values[handleIdx+1];
    }
    sliderValue = this.getNearestValue(sliderValue);
    this.values[handleIdx] = sliderValue;
    this.value = this.values[0]; // assure backwards compat
    
    this.handles[handleIdx].style[this.isVertical() ? ''top'' : ''left''] = 
      this.translateToPx(sliderValue);
    
    this.drawSpans();
    if (!!this.dragging || !!this.event) this.updateFinished();
  },
  setValueBy: function(delta, handleIdx) {
    this.setValue(this.values[handleIdx || this.activeHandleIdx || 0] + delta, 
      handleIdx || this.activeHandleIdx || 0);
  },
  translateToPx: function(value) {
    return Math.round(
      ((this.trackLength-this.handleLength)/(this.range.end-this.range.start)) * 
      (value - this.range.start)) + "px";
  },
  translateToValue: function(offset) {
    return ((offset/(this.trackLength-this.handleLength) * 
      (this.range.end-this.range.start)) + this.range.start);
  },
  getRange: function(range) {
    var v = this.values.sortBy(Prototype.K); 
    range = range || 0;
    return $R(v[range],v[range+1]);
  },
  minimumOffset: function(){
    return(this.isVertical() ? this.alignY : this.alignX);
  },
  maximumOffset: function(){
    return(this.isVertical() ? 
      (this.track.offsetHeight !!= 0 ? this.track.offsetHeight :
        this.track.style.height.replace(/px$/,"")) - this.alignY : 
      (this.track.offsetWidth !!= 0 ? this.track.offsetWidth : 
        this.track.style.width.replace(/px$/,"")) - this.alignX);
  },  
  isVertical:  function(){
    return (this.axis == ''vertical'');
  },
  drawSpans: function() {
    var slider = this;
    if (this.spans)
      $R(0, this.spans.length-1).each(function(r) { slider.setSpan(slider.spans[r], slider.getRange(r)) });
    if (this.options.startSpan)
      this.setSpan(this.options.startSpan,
        $R(0, this.values.length>1 ? this.getRange(0).min() : this.value ));
    if (this.options.endSpan)
      this.setSpan(this.options.endSpan, 
        $R(this.values.length>1 ? this.getRange(this.spans.length-1).max() : this.value, this.maximum));
  },
  setSpan: function(span, range) {
    if (this.isVertical()) {
      span.style.top = this.translateToPx(range.start);
      span.style.height = this.translateToPx(range.end - range.start + this.range.start);
    } else {
      span.style.left = this.translateToPx(range.start);
      span.style.width = this.translateToPx(range.end - range.start + this.range.start);
    }
  },
  updateStyles: function() {
    this.handles.each( function(h){ Element.removeClassName(h, ''selected'') });
    Element.addClassName(this.activeHandle, ''selected'');
  },
  startDrag: function(event) {
    if (Event.isLeftClick(event)) {
      if (!!this.disabled){
        this.active = true;
        
        var handle = Event.element(event);
        var pointer  = [Event.pointerX(event), Event.pointerY(event)];
        var track = handle;
        if (track==this.track) {
          var offsets  = Position.cumulativeOffset(this.track); 
          this.event = event;
          this.setValue(this.translateToValue( 
           (this.isVertical() ? pointer[1]-offsets[1] : pointer[0]-offsets[0])-(this.handleLength/2)
          ));
          var offsets  = Position.cumulativeOffset(this.activeHandle);
          this.offsetX = (pointer[0] - offsets[0]);
          this.offsetY = (pointer[1] - offsets[1]);
        } else {
          // find the handle (prevents issues with Safari)
          while((this.handles.indexOf(handle) == -1) && handle.parentNode) 
            handle = handle.parentNode;
            
          if (this.handles.indexOf(handle)!!=-1) {
            this.activeHandle    = handle;
            this.activeHandleIdx = this.handles.indexOf(this.activeHandle);
            this.updateStyles();
            
            var offsets  = Position.cumulativeOffset(this.activeHandle);
            this.offsetX = (pointer[0] - offsets[0]);
            this.offsetY = (pointer[1] - offsets[1]);
          }
        }
      }
      Event.stop(event);
    }
  },
  update: function(event) {
   if (this.active) {
      if (!!this.dragging) this.dragging = true;
      this.draw(event);
      if (Prototype.Browser.WebKit) window.scrollBy(0,0);
      Event.stop(event);
   }
  },
  draw: function(event) {
    var pointer = [Event.pointerX(event), Event.pointerY(event)];
    var offsets = Position.cumulativeOffset(this.track);
    pointer[0] -= this.offsetX + offsets[0];
    pointer[1] -= this.offsetY + offsets[1];
    this.event = event;
    this.setValue(this.translateToValue( this.isVertical() ? pointer[1] : pointer[0] ));
    if (this.initialized && this.options.onSlide)
      this.options.onSlide(this.values.length>1 ? this.values : this.value, this);
  },
  endDrag: function(event) {
    if (this.active && this.dragging) {
      this.finishDrag(event, true);
      Event.stop(event);
    }
    this.active = false;
    this.dragging = false;
  },  
  finishDrag: function(event, success) {
    this.active = false;
    this.dragging = false;
    this.updateFinished();
  },
  updateFinished: function() {
    if (this.initialized && this.options.onChange) 
      this.options.onChange(this.values.length>1 ? this.values : this.value, this);
    this.event = null;
  }
});
'! !

!WebStyle methodsFor: 'scripts-scriptaculous' stamp: 'np 4/2/2008 23:30'!
scriptaculousSliderJsResource
	"WebStyle new scriptaculousSliderJsResource"
	^self resources at: #jsScriptaculousSlider ifAbsentPut:
		[WebMethodResource 
			fromMethod: #scriptaculousSliderJs on: self 
			contentType: 'text/javascript' preferedUrl: '/scriptaculous/slider.js' site: self site].! !

!WebStyle methodsFor: 'scripts-scriptaculous' stamp: 'np 4/2/2008 23:30'!
scriptaculousSoundJs
	"sound.js from script.aculo.us AJAX framework, see http://script.aculo.us"
	^'
// script.aculo.us sound.js v1.8.0, Tue Nov 06 15:01:40 +0300 2007

// Copyright (c) 2005-2007 Thomas Fuchs (http://script.aculo.us, http://mir.aculo.us)
//
// Based on code created by Jules Gravinese (http://www.webveteran.com/)
//
// script.aculo.us is freely distributable under the terms of an MIT-style license.
// For details, see the script.aculo.us web site: http://script.aculo.us/

Sound = {
  tracks: {},
  _enabled: true,
  template:
    new Template(''<embed style="height:0" id="sound_#{track}_#{id}" src="#{url}" loop="false" autostart="true" hidden="true"/>''),
  enable: function(){
    Sound._enabled = true;
  },
  disable: function(){
    Sound._enabled = false;
  },
  play: function(url){
    if(!!Sound._enabled) return;
    var options = Object.extend({
      track: ''global'', url: url, replace: false
    }, arguments[1] || {});
    
    if(options.replace && this.tracks[options.track]) {
      $R(0, this.tracks[options.track].id).each(function(id){
        var sound = $(''sound_''+options.track+''_''+id);
        sound.Stop && sound.Stop();
        sound.remove();
      })
      this.tracks[options.track] = null;
    }
      
    if(!!this.tracks[options.track])
      this.tracks[options.track] = { id: 0 }
    else
      this.tracks[options.track].id++;
      
    options.id = this.tracks[options.track].id;
    $$(''body'')[0].insert( 
      Prototype.Browser.IE ? new Element(''bgsound'',{
        id: ''sound_''+options.track+''_''+options.id,
        src: options.url, loop: 1, autostart: true
      }) : Sound.template.evaluate(options));
  }
};

if(Prototype.Browser.Gecko && navigator.userAgent.indexOf("Win") > 0){
  if(navigator.plugins && $A(navigator.plugins).detect(function(p){ return p.name.indexOf(''QuickTime'') !!= -1 }))
    Sound.template = new Template(''<object id="sound_#{track}_#{id}" width="0" height="0" type="audio/mpeg" data="#{url}"/>'')
  else
    Sound.play = function(){}
}
'! !

!WebStyle methodsFor: 'scripts-scriptaculous' stamp: 'np 4/2/2008 23:30'!
scriptaculousSoundJsResource
	"WebStyle new scriptaculousSoundJsResource"
	^self resources at: #jsScriptaculousSound ifAbsentPut:
		[WebMethodResource 
			fromMethod: #scriptaculousSoundJs on: self 
			contentType: 'text/javascript' preferedUrl: '/scriptaculous/sound.js' site: self site].! !

!WebStyle methodsFor: 'frame printing' stamp: 'np 4/3/2008 15:31'!
searchButton
	searchButton isNil ifTrue:
		[searchButton := WebButton new
			title: 'Search'; text: 'S'; action: #search; style: '{font-size: 90%}';
			tabIndex: 99;
			id: #search234;
			yourself].
	^searchButton! !

!WebStyle methodsFor: 'accessing' stamp: 'np 4/2/2008 23:19'!
session
	"try to find a first sender up in calling stack, who is AIDASite and find a session in 
	its local variable stack"

	^self firstSessionFromStack
! !

!WebStyle methodsFor: 'texts' stamp: 'np 4/2/2008 23:32'!
sessionTimeoutText
	^'Session timed-out after 15min of inactivity!! '! !

!WebStyle methodsFor: 'accessing' stamp: 'np 4/2/2008 23:19'!
site
	^site! !

!WebStyle methodsFor: 'private' stamp: 'np 4/2/2008 23:24'!
site: anAIDASite
	site := anAIDASite! !

!WebStyle methodsFor: 'private-obsolete-printing-elements' stamp: 'np 4/2/2008 23:32'!
smallNumber: aNumber

	^self smallText: aNumber printDotString! !

!WebStyle methodsFor: 'private-obsolete-printing-elements' stamp: 'np 4/2/2008 23:32'!
smallNumber: aNumber attributes: aSymbol

	^self smallText: aNumber printDotString attributes: aSymbol! !

!WebStyle methodsFor: 'private-obsolete-printing-elements' stamp: 'np 4/2/2008 23:32'!
smallText: aText

	| text |
	text := aText isEmpty ifTrue: ['&nbsp;'] ifFalse: [aText].
	^(WebText text: text) size: -1; font: #helvetica! !

!WebStyle methodsFor: 'private-obsolete-printing-elements' stamp: 'np 4/2/2008 23:32'!
smallText: aText attributes: aSymbol

	| text |
	text := aText isEmpty ifTrue: ['&nbsp;'] ifFalse: [aText].
	^(WebText text: text attributes: aSymbol) size: -1; font: #helvetica! !

!WebStyle methodsFor: 'private-obsolete-colors' stamp: 'np 4/2/2008 23:32'!
tableHeaderColor
	^#thistle! !

!WebStyle methodsFor: 'private-obsolete-colors' stamp: 'np 4/2/2008 23:32'!
tableRowColor
	^#fffbf0! !

!WebStyle methodsFor: 'private-obsolete-colors' stamp: 'np 4/2/2008 23:32'!
tableSumaColor
	^#mediumgoldenrod! !

!WebStyle methodsFor: 'private-obsolete-printing-elements' stamp: 'np 4/2/2008 23:32'!
twoDigits: aNumber
	
	aNumber < 10
		ifTrue: [^'0',aNumber printString]
		ifFalse: [^aNumber printString]! !

!WebStyle methodsFor: 'texts' stamp: 'np 4/2/2008 23:32'!
usernameText
	^'Username: '! !

Object subclass: #WebTransactionMonitor
	instanceVariableNames: ''
	classVariableNames: 'LocalServers Lock NotificationSent Transactions BusySessions LastCommit'
	poolDictionaries: ''
	category: 'Aida-Core'!

WebTransactionMonitor class
	instanceVariableNames: 'Lock'!

WebTransactionMonitor class
	instanceVariableNames: 'Lock'!

!WebTransactionMonitor class methodsFor: 'local servers' stamp: ' 21/4/07 22:07'!
addServer: aWebServer

	self localServers add: aWebServer! !

!WebTransactionMonitor class methodsFor: 'transactions' stamp: ' 21/4/07 22:07'!
beginTransaction

	"Mark the beginning of transaction on current process. 
	Mark the end with commit or cancelTransaction"

	self beginTransactionOn: Processor activeProcess.! !

!WebTransactionMonitor class methodsFor: 'transactions' stamp: ' 21/4/07 22:07'!
beginTransactionOn: aProcess

	"Mark the beginning of transaction. Mark the end with commit or cancelTransaction"

	self critical: 
		[self transactions add: aProcess]! !

!WebTransactionMonitor class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
busySessions

	"a collection of database sessions, which are busy on that moment"

	BusySessions isNil ifTrue: [self initBusySessions].
	^BusySessions! !

!WebTransactionMonitor class methodsFor: 'transactions' stamp: ' 21/4/07 22:07'!
cancelTransaction

	"cancel the transaction without commiting on current process. 
	Used in exception handling routines"

	self cancelTransactionOn: Processor activeProcess! !

!WebTransactionMonitor class methodsFor: 'transactions' stamp: ' 21/4/07 22:07'!
cancelTransactionOn: aProcess

	"cancel the transaction without commiting. Used in exception handling routines"

	self critical: 
		[self transactions remove: aProcess ifAbsent: []].! !

!WebTransactionMonitor class methodsFor: 'transactions' stamp: ' 21/4/07 22:07'!
clearHangedProcesses

	"remove all processes with hanged  suspended context from set of transactions. "
	"WebTransactionMonitor clearHangedProcesses"
" TEMORARY, vw5i4 sender unknown !! "
	self transactions copy do: 
		[:each | (each suspendedContext notNil and:
""
			[(each suspendedContext sender selector = #readWait) | 
			(each suspendedContext sender selector = #writeWait)] )
""
				ifTrue:
					[each terminate.
					self transactions remove: each] ].
""! !

!WebTransactionMonitor class methodsFor: 'transactions' stamp: ' 21/4/07 22:07'!
clearNilProcesses

	"remove all processes with nil  suspended context from set of transactions. Such processes
	died somewhere and were not removed with method cancelTransaction"

	self transactions copy do: 
		[:each | each suspendedContext isNil 
			ifTrue:
				[self transactions remove: each] ].! !

!WebTransactionMonitor class methodsFor: 'transactions' stamp: ' 21/4/07 22:07'!
commit

	"commit the transaction if there is no more pending transactions, otherwise defer commit to
	the next call of this method."
	"WebTransactionMonitor commit"
	self critical: 
		[self clearNilProcesses.
		"self transactions isEmpty" true
			ifTrue: 
				["WebServer default diagnostics ifTrue: [Transcript show: ' comm.']."
"				Swazoo.SwazooServer isPersistent ifTrue: [self gemstoneCommit].  "
				"WebServer default diagnostics ifTrue: [Transcript show: '.ited ']."
				self setLastCommit. ]
			ifFalse: 
				[self clearHangedProcesses.
				self transactions notEmpty ifTrue: 
					[self noCommitFor10min ifTrue: 
						[self notifyNoCommiting.
"						Swazoo.SwazooServer isPersistent ifTrue: [self gemstoneCommit] "]
					]
				]
		].

"WebTransactionMonitor commit"! !

!WebTransactionMonitor class methodsFor: 'transactions' stamp: ' 21/4/07 22:07'!
commitTransaction

	"commit the transaction on current process if there is no more pending transactions, 
	otherwise defer commit to the next call of this method."

	self commitTransactionOn: Processor activeProcess! !

!WebTransactionMonitor class methodsFor: 'transactions' stamp: ' 21/4/07 22:07'!
commitTransactionOn: aProcess

	"commit the transaction if there is no more pending transactions, otherwise defer commit to
	the next call of this method."

	self critical: 
		[self cancelTransactionOn: aProcess.
		self commit].! !

!WebTransactionMonitor class methodsFor: 'critical sections' stamp: 'JM 4/25/2007 20:45'!
critical: aBlock
	"For protecting critical sections in parallel execution of web requests. Use it always
	when you do things, which cannot be disturbed by another request. Example:
		WebTransactionMonitor critical: [<a block with critical section>]. "

"	^self lock critical: aBlock. "
	^aBlock value "Semaphore critical: somehow don't work"! !

!WebTransactionMonitor class methodsFor: 'odb specific' stamp: ' 21/4/07 22:07'!
freeSession

	"find a session in a pool of sessions which is not current and not busy"

	| allSessions pick |
	allSessions := GBSM loggedInSessions asOrderedCollection.
	allSessions 
		remove: GBSM currentSession;
		removeAll: self busySessions.
	allSessions notEmpty
		ifTrue:
			[pick := (Random new next * allSessions size) truncated + 1.
			^allSessions at: pick]
		ifFalse: [^GBSM currentSession]


"WebTransactionMonitor freeSession"! !

!WebTransactionMonitor class methodsFor: 'odb specific' stamp: ' 21/4/07 22:07'!
gemstoneCommit

	| result |
	AIDARoot critical:
		[result := GBSM currentSession commitTransaction.
		result == true ifFalse: 
			[result := GBSM currentSession commitTransaction.
			result == true ifFalse: 
				[Transcript cr; show: '****** commit troubles !!!!!! ******'] ].
		].

"WebTransactionMonitor commit"! !

!WebTransactionMonitor class methodsFor: 'initialize' stamp: ' 21/4/07 22:07'!
initBusySessions

	BusySessions := Set new.! !

!WebTransactionMonitor class methodsFor: 'initialize' stamp: ' 21/4/07 22:07'!
initTransactions

	Transactions := Set new.! !

!WebTransactionMonitor class methodsFor: 'initialize' stamp: ' 21/4/07 22:07'!
initialize
	self initTransactions.
	self initBusySessions.

"WebTransactionMonitor initialize"! !

!WebTransactionMonitor class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
isNotificationSent

	NotificationSent isNil ifTrue: [self resetNotificationSent].
	^NotificationSent! !

!WebTransactionMonitor class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
lastCommit

	"return the time of last real odb commit"

	^AIDASite default lastCommitTimestamp.! !

!WebTransactionMonitor class methodsFor: 'private' stamp: ' 21/4/07 22:07'!
localServers
	"all active web servers, whose are running on local image"
	LocalServers isNil ifTrue: [LocalServers := Set new].
	self removeNonactiveServers.
	^LocalServers! !

!WebTransactionMonitor class methodsFor: 'private' stamp: 'JM 4/25/2007 20:33'!
lock
	""
	Lock isNil ifTrue: [Lock := Semaphore new].
	^Lock.! !

!WebTransactionMonitor class methodsFor: 'error notifying' stamp: ' 21/4/07 22:07'!
noCommitFor10min

	"check if there was no real odb commit more than 10min"

	^(Timestamp now asSeconds - self lastCommit asSeconds) > (10 * 60)! !

!WebTransactionMonitor class methodsFor: 'error notifying' stamp: ' 21/4/07 22:07'!
notifyNoCommiting

	"if there is no odb commit for more than 10min then an e-mail message is sent to a 
	specified address (from settings in WebServer)"
	"WebTransactionMonitor notifyNoCommiting"



	| subject body |

	self isNotificationSent ifTrue: [^self].
"
	subject := 'AIDAWeb Urgent: No odb commit!!'.
	body := 'No commit for more than 10 minutes!!'.
	WebServer default urgentEMailSubject: subject body: body.
"
	self setNotificationSent.! !

!WebTransactionMonitor class methodsFor: 'odb specific' stamp: ' 21/4/07 22:07'!
onFreeSessionFor: anObject remotePerform: aSelector

	^self onFreeSessionFor: anObject remotePerform: aSelector withArgs: #()


"WebTransactionMonitor onFreeSessionFor: AIDARoot remotePerform: #copy"! !

!WebTransactionMonitor class methodsFor: 'odb specific' stamp: ' 21/4/07 22:07'!
onFreeSessionFor: anObject remotePerform: aSelector with: anArgument

	^self onFreeSessionFor: anObject remotePerform: aSelector withArgs: (Array with: anArgument)

"WebTransactionMonitor onFreeSessionFor: AIDARoot remotePerform: #at: with: 'planid' "! !

!WebTransactionMonitor class methodsFor: 'odb specific' stamp: ' 21/4/07 22:07'!
onFreeSessionFor: anObject remotePerform: aSelector withArgs: argsArray

	| session gsObject result |
	session := self freeSession.
	[	
		self busySessions add: session.
		gsObject := anObject asGSObjectInSession: session .
		result := gsObject remotePerform: aSelector withArgs: argsArray.
		result := gsObject remotePerform: aSelector withArgs: argsArray.
		result := result asLocalObject.
		session commitTransaction
	] valueNowOrOnUnwindDo: 
		[self busySessions remove: session ifAbsent: [].
		 session disconnectST: anObject fromGS: gsObject].
	^result

"WebTransactionMonitor onFreeSessionFor: WebServer default remotePerform: #settings withArgs: #()"! !

!WebTransactionMonitor class methodsFor: 'private' stamp: ' 21/4/07 22:07'!
removeNonactiveServers
	LocalServers copy do: [:srv | 
		srv isServing ifFalse: [LocalServers remove: srv ifAbsent: []] ]! !

!WebTransactionMonitor class methodsFor: 'local servers' stamp: ' 21/4/07 22:07'!
removeServer: aWebServer

	self localServers remove: aWebServer ifAbsent: []! !

!WebTransactionMonitor class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
resetNotificationSent

	NotificationSent := false.! !

!WebTransactionMonitor class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
setLastCommit

	"set the time of last real odb commit to time now"
	
	self localServers do: [:each | each setLastCommitTimestamp].
	self resetNotificationSent! !

!WebTransactionMonitor class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
setNotificationSent

	NotificationSent := true.! !

!WebTransactionMonitor class methodsFor: 'private' stamp: ' 21/4/07 22:07'!
transactions

	"a collection of all processes with open transactions . Such a process is 
	addded with method beginTransaction and removed with commit or cancelTransaction"
	"WebTransactionMonitor transactions"
	Transactions isNil ifTrue: [self initTransactions].
	^Transactions! !

!WebTransactionMonitor class methodsFor: 'odb specific' stamp: ' 21/4/07 22:07'!
versantCommit

"	ODBInterface checkpointCommit "! !

Object subclass: #WebUser
	instanceVariableNames: 'parent id username password name surname company address city zip country email phone fax groups otherValues'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Core'!

!WebUser class methodsFor: 'defaults' stamp: ' 21/4/07 22:07'!
adminEMail
	^'admin'! !

!WebUser class methodsFor: 'defaults' stamp: ' 21/4/07 22:07'!
adminName
	^'Admin'! !

!WebUser class methodsFor: 'defaults' stamp: ' 21/4/07 22:07'!
adminPassword
	^'password'! !

!WebUser class methodsFor: 'defaults' stamp: ' 21/4/07 22:07'!
adminSurname
	^''! !

!WebUser class methodsFor: 'defaults' stamp: ' 21/4/07 22:07'!
adminUsername
	^'admin'! !

!WebUser class methodsFor: 'defaults' stamp: 'mivsek 1/10/2008 19:04'!
extranetEMail
	^'extranet'! !

!WebUser class methodsFor: 'defaults' stamp: 'mivsek 1/10/2008 19:04'!
extranetName
	^'Extranet'! !

!WebUser class methodsFor: 'defaults' stamp: 'mivsek 1/10/2008 19:04'!
extranetPassword
	^'password'! !

!WebUser class methodsFor: 'defaults' stamp: 'mivsek 1/10/2008 19:04'!
extranetSurname
	^''! !

!WebUser class methodsFor: 'defaults' stamp: 'mivsek 1/10/2008 19:05'!
extranetUsername
	^'extranet'! !

!WebUser class methodsFor: 'defaults' stamp: ' 21/4/07 22:07'!
guestEMail
	^'guest'! !

!WebUser class methodsFor: 'defaults' stamp: ' 21/4/07 22:07'!
guestName
	^'Guest'! !

!WebUser class methodsFor: 'defaults' stamp: ' 21/4/07 22:07'!
guestPassword
	^'guest'! !

!WebUser class methodsFor: 'defaults' stamp: ' 21/4/07 22:07'!
guestSurname
	^''! !

!WebUser class methodsFor: 'defaults' stamp: ' 21/4/07 22:07'!
guestUsername
	^'guest'! !

!WebUser class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
new
	^super new initialize! !

!WebUser class methodsFor: 'instance creation' stamp: 'janko 3/16/2008 13:30'!
newAdmin
	^self new
		name: self adminName;
		surname: self adminSurname;
		email: self adminEMail;
		username: self adminUsername;
		password: self adminPassword! !

!WebUser class methodsFor: 'instance creation' stamp: 'janko 3/16/2008 13:30'!
newExtranet
	"a common extranet user, for easier setup of access rights"
	^self new
		name: self extranetName;
		surname: self extranetSurname;
		email: self extranetEMail;
		username: self extranetUsername;
		password: self extranetPassword! !

!WebUser class methodsFor: 'instance creation' stamp: 'np 3/17/2008 21:43'!
newGuest
	^self new
		name: self guestName;
		surname: self guestSurname;
		email: self guestEMail;
		username: self guestUsername;
		password: self guestPassword! !

!WebUser methodsFor: 'groups' stamp: ' 21/4/07 22:07'!
addToActivatingGroup
	self parent addActivatingUser: self! !

!WebUser methodsFor: 'groups' stamp: ' 21/4/07 22:07'!
addToGroup: aWebUserGroup
 	self groups add: aWebUserGroup! !

!WebUser methodsFor: 'groups' stamp: ' 21/4/07 22:07'!
addToRegisteredGroup
	self parent addRegisteredUser: self! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
address
	address isNil ifTrue: [^'']. 
	^address! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
address: aString
	address := aString.! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allGroups 
	^groups copy! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
asPerson
	"parallel Person object (from Party framework if present, otherwise nil)"
	^self otherValuesAt: #Person ifAbsent: [^self initPerson]! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
asWebUser
	^self! !

!WebUser methodsFor: 'accessing-other' stamp: ' 21/4/07 22:07'!
autoLogout
	"logout after 15min of inactivity. default is NO!! "
	^self otherValuesAt: #AutoLogout ifAbsent: [false]! !

!WebUser methodsFor: 'accessing-other' stamp: ' 21/4/07 22:07'!
autoLogout: aBoolean
	"logout after 15min of inactivity"
	^self otherValuesAt: #AutoLogout put: aBoolean! !

!WebUser methodsFor: 'groups' stamp: ' 21/4/07 22:07'!
becomeAdmin
	self parent adminGroup addUser: self! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
city
	city isNil ifTrue: [^'']. 
	^city! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
city: aString
	city := aString.! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
company
	company isNil ifTrue: [^'']. 
	^company! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
company: aString 
	company := aString.! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
country
	country isNil ifTrue: [^'']. 
	^country! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
country: aString
	country := aString.! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
email 
	email isNil ifTrue: [^''].
	^email! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
email: aString
	email := aString.! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
fax
	fax isNil ifTrue: [^''].
	^fax! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
fax: aString 
	fax := aString.! !

!WebUser methodsFor: 'private' stamp: ' 21/4/07 22:07'!
groups 
	groups isNil ifTrue: [self initGroups].
	^groups! !

!WebUser methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
hasEMail 

	"at least email must be entered to accept a new user"
	
	^self email ~= ''! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
id
	"unique id of that user"
	^id! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
id: aNumber
	id := aNumber.! !

!WebUser methodsFor: 'private' stamp: ' 21/4/07 22:07'!
initGroups
	groups := Set new.! !

!WebUser methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initId
	self id: (Random new next * 1000000000) asInteger.! !

!WebUser methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initOtherValues
	otherValues := Dictionary new.! !

!WebUser methodsFor: 'initialize-release' stamp: 'np 1/23/2008 11:12'!
initPerson
	^self otherValuesAt: #Person put: (Person new webUser: self)! !

!WebUser methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	self initId! !

!WebUser methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isActivating
	"user needs to confirm registration"
	^self groups contains: [:each | each isActivatingGroup]! !

!WebUser methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isAdmin
	"all from Administrators group are admins, also default Admin user"
	^self groups contains: [:each | each isAdminGroup]! !

!WebUser methodsFor: 'testing' stamp: 'mivsek 1/10/2008 19:35'!
isExtranet
	"a general user for extranets"
	^self name = 'Extranet' or: [(self username = 'extranet') & (self password = 'password')]! !

!WebUser methodsFor: 'testing' stamp: 'mivsek 3/7/2008 22:57'!
isGuest
	"all non registered visitors have the same user: a Guest"
	^self name = 'Guest' and: 
		[(self username = 'guest') 
			"& (self password = (WebSecurityManager hashPassword: 'guest'))"]! !

!WebUser methodsFor: 'testing' stamp: 'np 1/18/2008 11:23'!
isPerson
	^false! !

!WebUser methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isRegistered
	^self groups contains: [:each | each isRegisteredGroup]! !

!WebUser methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isWebUser
	^true! !

!WebUser methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isWebUserGroup
	^false! !

!WebUser methodsFor: 'accessing-other' stamp: ' 21/4/07 22:07'!
lastAppUrl
	"an url which last WebApplication was called. "
	^self otherValuesAt: #LastAppUrl ifAbsent: [nil].! !

!WebUser methodsFor: 'accessing-other' stamp: ' 21/4/07 22:07'!
lastAppUrl: aString
	"an url which last WebApplication was called. It is set AFTER the app view is generated!!"
	^self otherValuesAt: #LastAppUrl put: aString! !

!WebUser methodsFor: 'accessing-other' stamp: ' 21/4/07 22:07'!
logoutFromUrl
	"From which page user logout. To be redirected back after login"
	^self otherValuesAt: #LogoutFromUrl ifAbsent: [nil].! !

!WebUser methodsFor: 'accessing-other' stamp: ' 21/4/07 22:07'!
logoutFromUrl: aString
	"From which page user logout. To be redirected back after login"
	^self otherValuesAt: #LogoutFromUrl put: aString! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
menuName
	"in dropdown menus"
	^self surnameName! !

!WebUser methodsFor: 'private' stamp: ' 21/4/07 22:07'!
migrateToUnicode
	"from iso8859-2"
	"WebUser allInstances do: [:each | each migrateToUnicode]"
	username notNil ifTrue: [username := username ensureUnicodeSloveneChars].
	password notNil ifTrue: [password := password ensureUnicodeSloveneChars].
	name notNil ifTrue: [name := name ensureUnicodeSloveneChars].
	surname notNil ifTrue: [surname := surname ensureUnicodeSloveneChars].
	company notNil ifTrue: [company := company ensureUnicodeSloveneChars].
	city notNil ifTrue: [city := city ensureUnicodeSloveneChars].
	country notNil ifTrue: [country := country ensureUnicodeSloveneChars].! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
name
	name isNil ifTrue: [self asPerson notNil ifTrue: [^self asPerson name] ifFalse: [^''] ].
	^name! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
name: aString 
	self asPerson notNil ifTrue: [^self asPerson name: aString].
	name := aString trimBlanks.! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
nameSurname 
	^self name, ' ', self surname! !

!WebUser methodsFor: 'private' stamp: ' 21/4/07 22:07'!
otherValues
	^otherValues! !

!WebUser methodsFor: 'private' stamp: ' 21/4/07 22:07'!
otherValuesAt: aString
	^self otherValuesAt: aString ifAbsent: [nil]! !

!WebUser methodsFor: 'private' stamp: ' 21/4/07 22:07'!
otherValuesAt: aString ifAbsent: aBlock
	self otherValues isNil ifTrue: [^aBlock value].
	^self otherValues at: aString ifAbsent: [aBlock value]! !

!WebUser methodsFor: 'private' stamp: ' 21/4/07 22:07'!
otherValuesAt: aString put: anObject
	self otherValues isNil ifTrue: [self initOtherValues].
	^self otherValues at: aString put: anObject! !

!WebUser methodsFor: 'private' stamp: ' 21/4/07 22:07'!
parent
	^parent! !

!WebUser methodsFor: 'private' stamp: ' 21/4/07 22:07'!
parent: aWebSecurityManager
	parent := aWebSecurityManager! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
password		 
	password isNil ifTrue: [^''].
	^password! !

!WebUser methodsFor: 'accessing' stamp: 'np 4/6/2008 15:53'!
password: aString	
	"Store hashed string"
	aString notEmpty ifTrue: [
		password := WebSecurityManager hashPassword: aString]! !

!WebUser methodsFor: 'private' stamp: ' 21/4/07 22:07'!
person: aPerson
	"connect to parrallel Person from Party framework"
	self otherValuesAt: #Person put: aPerson.
	aPerson asWebUser ~= self ifTrue: [aPerson webUser: self].! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
phone 
	phone isNil ifTrue: [^''].
	^phone! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
phone: aString 
	phone := aString.! !

!WebUser methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
printString 
	^'aWebUser named ', self surname, ', ', self name, ' (', self username, ') '! !

!WebUser methodsFor: 'groups' stamp: ' 21/4/07 22:07'!
removeFromGroup: aWebUserGroup 
	self groups remove: aWebUserGroup ifAbsent: []! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
someId
	"just return some text, possibly surnameName"
	self surnameName trimBlanks notEmpty ifTrue: [^self surnameName].
	self email notEmpty ifTrue: [^self email].
	^'----'! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
surname
	surname isNil ifTrue: [self asPerson notNil ifTrue: [^self asPerson surname] ifFalse: [^''] ].
	^surname! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
surname: aString 
	self asPerson notNil ifTrue: [^self asPerson surname: aString].
	surname := aString trimBlanks.! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
surnameName
	^self surname, ' ', self name! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
username
 
	username isNil ifTrue: [^''].
	^username.! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
username: aString 
	username := aString asLowercase trimBlanks.! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
uuid
	"some unique identifier. Hash for now, probably unique enough!! "
	^self hash printString! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
zip
	zip isNil ifTrue: [^'']. 
	^zip! !

!WebUser methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
zip: aString
	zip := aString.! !

Object subclass: #WebUserGroup
	instanceVariableNames: 'name users'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Aida-Core'!

!WebUserGroup class methodsFor: 'defaults' stamp: ' 21/4/07 22:07'!
activatingGroupName
	"group of users waiting to confirm registration"
	^'Users waiting activation'! !

!WebUserGroup class methodsFor: 'defaults' stamp: ' 21/4/07 22:07'!
adminGroupName
	"return a name of group for administrators" 
	^'Administrators'! !

!WebUserGroup class methodsFor: 'defaults' stamp: ' 21/4/07 22:07'!
allUsersGroupName
	"return a name of group, where all users are there by default" 
	^'AllUsers'! !

!WebUserGroup class methodsFor: 'defaults' stamp: ' 21/4/07 22:07'!
guestGroupName
	"return a name of group for guests" 
	^self allUsersGroupName! !

!WebUserGroup class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newActivating
	^super new name: self activatingGroupName! !

!WebUserGroup class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newAdmin
	^super new name: self adminGroupName! !

!WebUserGroup class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newAllUsers
	^super new name: self allUsersGroupName! !

!WebUserGroup class methodsFor: 'instance creation' stamp: ' 21/4/07 22:07'!
newRegistered
	^super new name: self registeredGroupName! !

!WebUserGroup class methodsFor: 'defaults' stamp: ' 21/4/07 22:07'!
registeredGroupName
	"return a name of group of registered users" 
	^'Registered Users'! !

!WebUserGroup methodsFor: 'adding-removing' stamp: ' 21/4/07 22:07'!
addUser: aWebUser
 	self users add: aWebUser.
	aWebUser addToGroup: self.! !

!WebUserGroup methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
allUsers
	^self users copy! !

!WebUserGroup methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
includes: aWebUser
	^self users includes: aWebUser! !

!WebUserGroup methodsFor: 'private' stamp: ' 21/4/07 22:07'!
initUsers 
	users  := Set new.! !

!WebUserGroup methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isActivatingGroup
	"group of users waiting to confirm registration"
	^self name = self class activatingGroupName! !

!WebUserGroup methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isAdminGroup
	^self name = self class adminGroupName! !

!WebUserGroup methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isAllUsersGroup
	^self name = self class allUsersGroupName! !

!WebUserGroup methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isGuestGroup
	^self name = self class guestGroupName! !

!WebUserGroup methodsFor: 'testing' stamp: 'np 1/18/2008 11:23'!
isPerson
	^false! !

!WebUserGroup methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isRegisteredGroup
	^self name = self class registeredGroupName! !

!WebUserGroup methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isWebUser
	^false! !

!WebUserGroup methodsFor: 'testing' stamp: ' 21/4/07 22:07'!
isWebUserGroup
	^true! !

!WebUserGroup methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
menuName
	"in dropdown menus"
	^name! !

!WebUserGroup methodsFor: 'private' stamp: ' 21/4/07 22:07'!
migrateToUnicode
	"from iso8859-2"
	"WebUserGroup allInstances do: [:each | each migrateToUnicode]"
	name notNil ifTrue: [name := name ensureUnicodeSloveneChars].! !

!WebUserGroup methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
name
	name isNil ifTrue: [self name: '']. 
	^name! !

!WebUserGroup methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
name: aString
	name := aString.! !

!WebUserGroup methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
printString
	^'aWebUserGroup: ', self name! !

!WebUserGroup methodsFor: 'adding-removing' stamp: ' 21/4/07 22:07'!
removeUser: aWebUser
 	self users remove: aWebUser ifAbsent: [].
	aWebUser removeFromGroup: self.! !

!WebUserGroup methodsFor: 'private' stamp: ' 21/4/07 22:07'!
users
	users isNil ifTrue: [self initUsers]. 
	^users! !

!WebUserGroup methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
uuid
	"some unique identifier. Hash for now, probably unique enough!! "
	^self hash printString! !

Site subclass: #AIDASite
	instanceVariableNames: 'style settings systemServices userServices timestamps counters'
	classVariableNames: 'SloveneCharacters Default RandomGen Instance'
	poolDictionaries: ''
	category: 'Aida-Core'!

!AIDASite class methodsFor: 'slovene characters' stamp: 'JM 4/26/2007 20:02'!
charC
	"return internal  unicode code for slovene character: uppercase c"
	^16r010C asCharacter! !

!AIDASite class methodsFor: 'slovene characters' stamp: ' 21/4/07 22:07'!
charCszSet
	^Set new
		add: self charc; add: self charC;
		add: self chars; add: self charS;
		add: self charz; add: self charZ;
		yourself! !

!AIDASite class methodsFor: 'slovene characters' stamp: 'JM 4/26/2007 20:06'!
charS
	"return internal  unicode code for slovene character: uppercase s"
	^16r0160 asCharacter! !

!AIDASite class methodsFor: 'slovene characters' stamp: ' 21/4/07 22:07'!
charZ
	"return internal unicode code for slovene character: uppercase z"
	^16r017D asCharacter! !

!AIDASite class methodsFor: 'slovene characters' stamp: ' 21/4/07 22:07'!
charc
	"return internal  unicode code for slovene character: lowercase c"
	^16r010D asCharacter! !

!AIDASite class methodsFor: 'slovene characters' stamp: ' 21/4/07 22:07'!
chars
	"return internal  unicode code for slovene character: lowercase s"
	^16r0161 asCharacter! !

!AIDASite class methodsFor: 'slovene characters' stamp: 'JM 4/26/2007 20:07'!
charz
	"return internal unicode code for slovene character: lowercase z"
	^16r017E asCharacter! !

!AIDASite class methodsFor: 'codepage converting' stamp: 'mu 3/7/2008 20:21'!
convert: aString fromCodepage: aSymbol
	"convert aString to internal unicode"
	| encoding |
	(#(#'win-1250' #'win1250' #'Windows-1250' cp1250) includes: aSymbol) 
		ifTrue: [encoding := #'cp-1250'].
	(#(#'iso-8859-2' #'iso8859-2' #'ISO-8859-2' iso2) includes: aSymbol) 
		ifTrue: [encoding := #'iso-8859-2'].
	(#(#utf8 #'utf-8' #'utf_8' #UTF8 #'UTF-8' #'UTF_8') includes: aSymbol) 
		ifTrue: [encoding := #utf8].
	encoding isNil ifTrue: [^aString].

	^[aString convertFromEncoding: encoding] on: Error do: [:ex | aString ].

	
"AIDASite convert: (String with: 16rC4 asCharacter with: 16r8D asCharacter) fromCodepage: #utf8 "! !

!AIDASite class methodsFor: 'codepage converting' stamp: 'mu 3/8/2008 16:34'!
convert: aString toCodepage: aSymbol
	"convert internal unicode aString to codepage. "
	| encoding converter in out |
	(#(#'win-1250' #'win1250' #'Windows-1250' cp1250) includes: aSymbol) 
		ifTrue: [encoding := #'cp-1250'].
	(#(#'iso-8859-2' #'iso8859-2' #'ISO-8859-2' iso2) includes: aSymbol) 
		ifTrue: [encoding := #'iso-8859-2'].
	(#(#utf8 #'utf-8' #'utf_8' #UTF8 #'UTF-8' #'UTF_8') includes: aSymbol) 
		ifTrue: [encoding := #'utf8'].
	encoding isNil ifTrue: [^aString].

	converter := (TextConverter newForEncoding: encoding) ifNil: [UTF8TextConverter new].
	in := aString readStream.
	out := WriteStream on: ''.
	[in atEnd] whileFalse: [converter nextPut: in next toStream: out].
	^out contents

"AIDASite convert: (String with: 16r010D asCharacter) toCodepage: #utf8 "! !

!AIDASite class methodsFor: 'codepage converting' stamp: ' 21/4/07 22:07'!
convertFromWeb: aString on: aSession
	"convert aString, which is in code page, defined in aSession to an internal unicode "
	^self convert: aString fromCodepage: aSession codePage! !

!AIDASite class methodsFor: 'web converting' stamp: 'JM 4/26/2007 11:06'!
convertFromWebString: aString
	"converts special chars in http stream: 
		'+' as ' '
		%xx as appropriate ASCII char
	"
	| newString |
	newString := HTTPString decodedHTTPFrom: aString.
"	newString := newString copyReplaceAll: '+' with: ' '. "
	newString := AIDASite convert:newString fromCodepage: #'utf-8'.
	^newString

"AIDASite convertFromWebString: (AIDASite convertToWebString: '+-% test one') "! !

!AIDASite class methodsFor: 'codepage converting' stamp: ' 21/4/07 22:07'!
convertToWeb: aString on: aSession
	"rconver internal unicode aString to a proper code page, defined in aSession"
	^self convert: aString toCodepage: aSession codePage! !

!AIDASite class methodsFor: 'web converting' stamp: 'JM 4/26/2007 11:06'!
convertToWebString: aString
	"converts some special chars in http stream: 
		' ' as '+'
		%xx as appropriate ASCII char"
		
	^HTTPString encodedHTTPFrom: aString

"AIDASite convertToWebString: '+-% test one' "! !

!AIDASite class methodsFor: 'accessing' stamp: 'mu 2/3/2008 16:44'!
default
	"just return default instance set by default:"
	^Default! !

!AIDASite class methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
default: aString
	"make a site with that name a default one"
	| site |
	site := SwazooServer singleton siteNamed: aString.
	site isNil ifTrue: [^self error: 'site with that name does not exist!!'].
	Default := site.
	^site! !

!AIDASite class methodsFor: 'web converting' stamp: ' 21/4/07 22:07'!
fromHexValue: aCharacter
	aCharacter isDigit 
		ifTrue: [^aCharacter asInteger - $0 asInteger]
		ifFalse: [
			(aCharacter asLowercase between: $a and: $f)
				ifTrue: [^aCharacter asLowercase asInteger - $a asInteger+10]
				ifFalse: [^nil]].! !

!AIDASite class methodsFor: 'web converting' stamp: ' 21/4/07 22:07'!
hexCharFrom: aNumber
	^aNumber < 10 
		ifTrue: [($0 asInteger + aNumber) asCharacter]
		ifFalse: [($A asInteger + aNumber - 10) asCharacter]! !

!AIDASite class methodsFor: 'slovene characters' stamp: 'JM 4/26/2007 20:07'!
initSloveneCharacters
	"AIDASite initSloveneCharacters"
	SloveneCharacters := Set new: 6.
	^SloveneCharacters 
		add: self charC;
		add: self charc;
		add: self charS;
		add: self chars;
		add: self charZ;
		add: self charz;
		yourself! !

!AIDASite class methodsFor: 'slovene characters' stamp: 'JM 4/26/2007 20:07'!
isSloveneCharacter: aCharacter
	"return true if character is slovene"
	SloveneCharacters isNil ifTrue: [self initSloveneCharacters].
	^SloveneCharacters includes: aCharacter

"AIDASite isSloveneCharacter: AIDASite charC"! !

!AIDASite class methodsFor: 'utilities' stamp: 'janko 7/18/2008 17:57'!
random
	"a random generator, always seeded and therefore ready to geenrate numbers which are really 
       random"
	RandomGen isNil ifTrue: [RandomGen := Random new].
	^RandomGen! !

!AIDASite class methodsFor: 'utilities' stamp: ' 21/4/07 22:07'!
slash

	"return '/'  on Unix, '\' on PC platforms, for making full filenames"

	(IOAccessor defaultClass = PCIOAccessor)
		ifTrue: [^'\'] ifFalse: [^'/']! !

!AIDASite class methodsFor: 'utilities' stamp: ' 21/4/07 22:07'!
slashCh

	"return character:  '/'  on Unix, '\' on PC platforms, for making full filenames"

	(IOAccessor defaultClass = PCIOAccessor)
		ifTrue: [^$\] ifFalse: [^$/]! !

!AIDASite class methodsFor: 'slovene characters' stamp: 'JM 4/26/2007 20:07'!
sloveneLowercase: aString
	^aString collect: [:chr | 
		(AIDASite isSloveneCharacter: chr)
			ifTrue: 	[(chr = AIDASite charC) ifTrue: [AIDASite charc]
				ifFalse: [(chr = AIDASite charS) ifTrue: [AIDASite chars]
					ifFalse: [(chr = AIDASite charZ) ifTrue: [AIDASite charz]
						ifFalse: [chr] ]]]
			ifFalse: [chr asLowercase]
		].! !

!AIDASite class methodsFor: 'slovene characters' stamp: 'JM 4/26/2007 20:08'!
sloveneUppercase: aString
	^aString collect: [:chr | 
		(AIDASite isSloveneCharacter: chr)
			ifTrue: 	[(chr = AIDASite charc) ifTrue: [AIDASite charC]
				ifFalse: [(chr = AIDASite chars) ifTrue: [AIDASite charS]
					ifFalse: [(chr = AIDASite charz) ifTrue: [AIDASite charZ]
						ifFalse: [chr] ]]]
			ifFalse: [chr asUppercase]
		].! !

!AIDASite methodsFor: 'private-serving' stamp: ' 21/4/07 22:07'!
addAllowHeaderTo: aHTTPResponse
	| methods |
	methods := 'OPTIONS,GET,HEAD,POST,DELETE,TRACE,'.
	methods := methods, 'PROPFIND,PROPPATCH,MKCOL,PUT,COPY,MOVE,LOCK,UNLOCK'.
	aHTTPResponse addHeaderName: 'Allow' value: methods! !

!AIDASite methodsFor: 'private-serving' stamp: ' 21/8/07 09:38'!
addDontCacheHeaderTo: aHTTPResponse forPage: aWebPage
	| object value |
	object := aWebPage isWebApplication ifTrue: [aWebPage observee] ifFalse: [aWebPage].
	value := object aidaDontCache.
	value ifFalse: [value := aWebPage aidaDontCache]. "WebApp subclass can set it too!! "
	value ifTrue: [aHTTPResponse cacheControl: 'no-store, no-cache, must-revalidate'].! !

!AIDASite methodsFor: 'private-serving' stamp: ' 21/8/07 09:38'!
addExpiresHeaderTo: aHTTPResponse forPage: aWebPage
	| object value |
	object := aWebPage isWebApplication ifTrue: [aWebPage observee] ifFalse: [aWebPage].
	value := object expiresTimestamp.
	value isNil ifTrue: [value := aWebPage expiresTimestamp]. "WebApp subclass can set it too!! "
	value notNil ifTrue: [aHTTPResponse expires: value asSpTimestamp].! !

!AIDASite methodsFor: 'private-serving' stamp: ' 21/8/07 09:38'!
addModifiedHeaderTo: aHTTPResponse forPage: aWebPage
	| object value |
	object := aWebPage isWebApplication ifTrue: [aWebPage observee] ifFalse: [aWebPage].
	value := object modifiedTimestamp.
	value isNil ifTrue: [value := aWebPage modifiedTimestamp]. "WebApp subclass can set it too!! "
	value notNil ifTrue: [aHTTPResponse lastModified: value asSpTimestamp].! !

!AIDASite methodsFor: 'private-serving' stamp: ' 21/8/07 09:38'!
addResponseHeadersTo: aHTTPResponse forPage: aWebPage on: aWebSession
	"use observee in app to find timestamps, because observee points 
	to right version of an object!! "
	| object |
	object := aWebPage isWebApplication ifTrue: [aWebPage observee] ifFalse: [aWebPage].
	aHTTPResponse contentType: object contentType.
	self addModifiedHeaderTo: aHTTPResponse forPage: aWebPage.
	self addExpiresHeaderTo: aHTTPResponse forPage: aWebPage.
	self addDontCacheHeaderTo: aHTTPResponse forPage: aWebPage.
	aWebSession cookie ifFalse: [aHTTPResponse cookie: (self cookieHeaderFor: aWebSession)].! !

!AIDASite methodsFor: 'system services' stamp: ' 21/4/07 22:07'!
addSystemService: aServiceObject named: aSymbol 
	"add a new system service (e.g URLResolver) to the dictionary of services"
	self systemServices
		at: aSymbol asSymbol put: aServiceObject! !

!AIDASite methodsFor: 'private' stamp: ' 21/4/07 22:07'!
addToRunningHistoryCrashed: aBoolean
	"Call it at server startup (in setStartedTimestamp). It reads old timestamps (started and
	last alive) and add it to the history" 
	| array |
	array := Array new: 3.
	array
		at: 1 put: self startedTimestamp asSeconds;
		at: 2 put: self lastTimeAliveTimestamp asSeconds;
		at: 3 put: aBoolean.  "true if server crashed"
	self runningHistoryCollection add: array.! !

!AIDASite methodsFor: 'user services' stamp: ' 21/4/07 22:07'!
addUserService: aServiceObject named: aSymbol 
	"add a new user service (e.g WebIndex) to the dictionary of services"
	self userServices
		at: aSymbol asSymbol put: aServiceObject

"
WebServer default 
	addUserService: (URLResolver default ooRefFromURL: '/koledar.html')
	named: #Koledar
"! !

!AIDASite methodsFor: 'private-serving' stamp: ' 21/4/07 22:07'!
addressTextFor: anIPAddress

	^((anIPAddress hostAddress at: 1) printString, '.',
	(anIPAddress hostAddress at: 2) printString, '.',
	(anIPAddress hostAddress at: 3) printString, '.',
	(anIPAddress hostAddress at: 4) printString)! !

!AIDASite methodsFor: 'system services' stamp: ' 21/4/07 22:07'!
admin
	^self systemServices 
		at: #Admin
		ifAbsent: 
			[self addSystemService: (WebAdmin new) named: #Admin.
			self urlResolver defaultURL: '/admin.html' forObject: self admin.
			^self admin].! !

!AIDASite methodsFor: 'settings' stamp: 'mivsek 6/13/2007 15:54'!
afterLogin
	"url of a page to jump after login or #lastPage to back to last page before logout"

	^self settings at: #afterLogin ifAbsentPut: [#lastPage]. "back to last page before logout"! !

!AIDASite methodsFor: 'settings' stamp: 'mivsek 6/13/2007 15:55'!
afterLogin: aSymbol
	"1. url of page to jump"
	"2. #lastPage - back to a page before logout"
	self settings  at: #afterLogin put: aSymbol! !

!AIDASite methodsFor: 'serving' stamp: 'mivsek 8/22/2007 11:08'!
answer: anObject to: aRequest on: aWebSession
	aRequest isGet | aRequest isPost | aRequest isHead
		ifTrue: [^self answer: anObject toGetOrPost: aRequest on: aWebSession].
	aRequest isOptions ifTrue: [^self answer: anObject toOptions: aRequest on: aWebSession].
	aRequest isPropFind ifTrue: [^self answer: anObject toPropFind: aRequest on: aWebSession].
	aRequest isLock ifTrue: [^self answer: anObject toLock: aRequest on: aWebSession].
	aRequest isUnlock ifTrue: [^self answer: anObject toUnlock: aRequest on: aWebSession].
	aRequest isPut ifTrue: [^self answer: anObject toPut: aRequest on: aWebSession].
	^HTTPException notImplemented! !

!AIDASite methodsFor: 'serving' stamp: 'janko 7/18/2008 20:46'!
answer: anObject toGetOrPost: aRequest on: aWebSession 
	| page stream response |
	page := anObject 
		printWebPageFor: aRequest
		on: aWebSession.
	page isNil ifTrue: [ ^ HTTPException notFound ].
	aWebSession isHttpAuthenticationNeeded ifTrue: 
		[ "because of logout"
		^ self unauthorizedResponse ].
	aWebSession redirectLink notNil ifTrue: [ ^ self redirectOn: aWebSession ].
	self log: 'h'.
	page isRespondingStreamed 
		ifTrue: 
			[ page 
				printHTMLPageOn: nil for: aRequest
				on: aWebSession.	"not needed"
			self log: ' done'.
			^ aRequest streamedResponse ]
		ifFalse: 
			[ response := HTTPResponse ok.
			self 
				addResponseHeadersTo: response
				forPage: page
				on: aWebSession.
			stream := WriteStream on: (String new: 10000).
			page 
				printHTMLPageOn: stream for: aRequest on: aWebSession.
			response entity: (self properString: stream contents).
			self log: ' done'.
			^ response ]! !

!AIDASite methodsFor: 'serving' stamp: ' 21/4/07 22:07'!
answer: anObject toLock: aRequest on: aWebSession
	"WebDAV lock request for that object"
	|  rsp |
	"Temporary!!!! just return a fake lock!!"
	rsp := Swazoo.HTTPLockResponse new.
	rsp start.
	rsp addTagName: 'locktype' value: #write.
	rsp addTagName: 'lockscope' value: #exclusive.
	rsp addTagName: 'depth' value: '0'.
	rsp addLockOwner: 'Administrator'.
	rsp addTagName: 'timeout' value: 'Second-604800'.  "one week, temporary!!"
	rsp addLockToken: 'opaquelocktoken:89001c0a-23f2-0310-b37d-c58bc335a1ff'.
	^rsp! !

!AIDASite methodsFor: 'serving' stamp: ' 21/4/07 22:07'!
answer: anObject toOptions: aRequest on: aWebSession
	| response |
	response := Swazoo.HTTPResponse ok.
	self isWebDAVEnabled ifTrue: 
	 	[response addHeaderName: 'DAV' value: '1,2'.
		"without following header Windows refuse to add new Web folder!! "
		response addHeaderName: 'MS-Author-Via' value: 'DAV'].
	self addAllowHeaderTo: response.
	^response! !

!AIDASite methodsFor: 'serving' stamp: ' 21/4/07 22:07'!
answer: anObject toPropFind: aRequest on: aWebSession
	"WebDAV properties request from object"
	|  multiResponse |
	"Temporary!!!! Now it just return empty collection, which is enough for MS web folders to add new!! "
	multiResponse := Swazoo.HTTPPropFindResponse new.
	multiResponse multiStart.
	multiResponse startResponseFor: aRequest uri value.
	multiResponse addPropertyName: 'getlastmodified' value: 'Mon, 21 Feb 2005 14:11:01 GMT'.
	aRequest uri value last = $/ ifTrue: "directory"
		[multiResponse addPropertyName: 'resourcetype' value: #collection].
	multiResponse endResponseWithStatusCode: 200.
	multiResponse multiEnd.
	^multiResponse! !

!AIDASite methodsFor: 'serving' stamp: ' 21/4/07 22:07'!
answer: anObject toPut: aRequest on: aWebSession
	"WebDAV: try to PUT content into that object"
	(anObject isKindOf: FileProxy) ifFalse: [^HTTPException forbidden].
	anObject put: aRequest putData.
	^HTTPResponse ok! !

!AIDASite methodsFor: 'serving' stamp: 'JM 4/21/2007 22:20'!
answer: anObject toUnlock: aRequest on: aWebSession
	"WebDAV unlock request for that object"
	^HTTPResponse noContent. "204"! !

!AIDASite methodsFor: 'serving' stamp: 'janko 7/13/2008 11:27'!
answerTo: aRequest 
	| session object |
	"["self	log: self cr , (self logStringFor: aRequest) , self cr , '              s'.
	self shouldRedirect ifTrue: [^self redirectToOtherHost: aRequest].
	aRequest isPing ifTrue: [^HTTPResponse ok].  "/ping.html for monitoring the site"
	session := self sessionManager findOrMakeSessionFor: aRequest.
	session isHttpAuthenticationNeeded ifTrue: [session authenticateFrom: aRequest].
	session isHttpAuthenticationNeeded "still" ifTrue: [^self unauthorizedResponse].
	self log: 'p'.
	object := self objectTo: aRequest forSession: session.
	object isNil ifTrue: [^HTTPResponse notFound].
	^self answer: object to: aRequest on: session"]"
"		on: Error, Exception
		do: [:exception |
			exception class == Swazoo.HTTPException ifTrue: [^exception response].
			exception class == SpSocketError 
				ifTrue: [self reportSocketError: exception request: aRequest] ""temporary""
				ifFalse:
					[self reportException: exception.
					exception defaultAction]. 
			errorResponse := self httpResponseOnException: exception.
			HTTPException raiseResponse: errorResponse.
			^errorResponse].
"! !

!AIDASite methodsFor: 'private' stamp: ' 21/4/07 22:07'!
authenticationScheme
	"#Basic or #Digest, see rfc2617. Digest is recomended because password
	goes encrypted to server"
	self securityManager hasHttpBasicAuthenticationScheme ifTrue: [^#Basic].
	self securityManager hasHttpDigestAuthenticationScheme ifTrue: [^#Digest].
	^#None! !

!AIDASite methodsFor: 'settings' stamp: 'mivsek 1/10/2008 18:48'!
autoLogout
	"logout after 15min of inactivity. default is NO!! "
	^self settings at: #AutoLogout ifAbsent: [false]! !

!AIDASite methodsFor: 'settings' stamp: 'mivsek 1/10/2008 18:48'!
autoLogout: aBoolean
	"logout after 15min of inactivity"
	^self settings at: #AutoLogout put: aBoolean! !

!AIDASite methodsFor: 'statistics' stamp: 'JM 4/26/2007 19:58'!
availability
	"return server availability from first run in %"
	"AIDASite default availability"
	| uptime downtime |
	uptime := self totalUptime.
	downtime := self totalDowntime.
	^((uptime / ((uptime + downtime) max: 1)) * 100) asScaledDecimal: 2! !

!AIDASite methodsFor: 'private-backup/restore' stamp: 'np 1/15/2008 20:48'!
backupToFile
	"file out all object tree to file aidasite-<sitename>.obj"
	"(AIDASite named: 'biart') backupToFile"
	"(AIDASite named: 'biart') deepSearchOfClass: 'ObsoleteIntranet'"
	"(AIDASite named: 'biart') deepSearchOfObsoleteClasses"
	| filename fileDir |
	filename := 'aidasite-', self name, '.obj'.
	fileDir := FileDirectory forFileName: filename.
	(fileDir fileExists: filename) ifTrue: 
		[fileDir copyFileWithoutOverwriteConfirmationNamed: filename toFileNamed: ('aidasite-', self name, '-old.obj')].
	(FileStream fileNamed: filename)
		fileOutClass: nil andObject: self! !

!AIDASite methodsFor: 'private-serving' stamp: ' 21/4/07 22:07'!
cookieHeaderFor: aSession 
	"set a permanent cookie on client (up to year 2020, enough?) "
	"for secure sessions different id as for usual !! "
	| id |
	id := aSession lastRequest isEncrypted 
		ifTrue: [aSession secureId] ifFalse: [aSession id].
	^(WriteStream on: String new)
		nextPutAll: self cookieName;
		nextPutAll: '=';
		nextPutAll: id printString;
		nextPutAll: '; path=/; expires=Friday, 01-Jan-2020 01:00:00 GMT';
		contents! !

!AIDASite methodsFor: 'private-serving' stamp: ' 21/4/07 22:07'!
cookieName
	"this is the name for our cookie. It shold be unique, so it is a bit strange"
	^'aida9357'! !

!AIDASite methodsFor: 'private-counters' stamp: ' 21/4/07 22:07'!
counters

	counters isNil ifTrue: [self initCounters].
	^counters

"WebServer default counters"! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
countingPolicy
	^self settings at: #countingPolicy ifAbsentPut: [#all].! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
countingPolicy: aSymbol
	"from who to count requests with WebCounters: #all, #excludeAdmins, #onlyGuests, #none"
	self settings  at: #countingPolicy put: aSymbol! !

!AIDASite methodsFor: 'private' stamp: ' 21/4/07 22:07'!
cr
	^String with: Character cr! !

!AIDASite methodsFor: 'statistics' stamp: 'JM 4/26/2007 19:44'!
createdTimestamp
	"time and date of  server creation"
	^SpTimestamp fromSeconds:
		(self timestamps at: #Created ifAbsentPut: [SpTimestamp now asSeconds])! !

!AIDASite methodsFor: 'accessing' stamp: ' 21/4/07 22:07'!
critical: aBlock

	"For protecting critical sections in parallel execution of web requests. Use it always
	when you do things, which cannot be disturbed by another request. Exmaple:
		aServer critical: [<a block with critical section>]. "

	^WebTransactionMonitor critical: aBlock! !

!AIDASite methodsFor: 'settings' stamp: 'mivsek 1/10/2008 18:50'!
defaultLanguage
	"ISO 639 2-letter language code symbol for default language of that site"
	^self settings at: #defaultLanguage ifAbsentPut: [#en].! !

!AIDASite methodsFor: 'settings' stamp: 'mivsek 1/10/2008 18:50'!
defaultLanguage: aLanguageCodeSymbol
	"ISO 639 2-letter language code symbol for default language of that site"
	^self settings at: #defaultLanguage put: aLanguageCodeSymbol! !

!AIDASite methodsFor: 'user services' stamp: ' 21/4/07 22:07'!
demo
	^self userServices at: #WebDemo
		ifAbsent: 
			[self addUserService: WebDemo new named: #WebDemo.
			self urlResolver defaultURL: '/demo.html' forObject: self demo.
			^self demo]! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
diagnostics

	"if true, diagnostic messages are written on transcript for every web request"
	^self settings  at: #diagnostics ifAbsentPut: [true].! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
diagnostics: aBoolean
	"if true, diagnostic messages are written on transcript for every web request"
	self settings  at: #diagnostics put: aBoolean.

"WebServer default diagnostics: true"! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
disableWebDAVSupport
	"Web Distributed Authoring and Versioning (WebDAV, www.webdav.org) is disabled on this site"
	"AIDASite default disableWebDAVSupport"
	self settings at: #WebDAVEnabled put: false! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
enableWebDAVSupport
	"Web Distributed Authoring and Versioning (WebDAV, www.webdav.org) is enabled on this site"
	"AIDASite default enableWebDAVSupport"
	self settings at: #WebDAVEnabled put: true! !

!AIDASite methodsFor: 'system services' stamp: ' 21/4/07 22:07'!
help
	^self systemServices 
		at: #Help
		ifAbsent: 
			[self addSystemService: (WebHelp new) named: #Help.
			self urlResolver defaultURL: '/help.html' forObject: self help.
			^self help].! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
helpDirectory

	^self settings  at: #helpDirectory ifAbsent: [^'']! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
helpDirectory: aString
	" a directory, where help html dokuments are stored. See WebApplication help pages category for 
 	details"
	self settings  at: #helpDirectory put: aString.! !

!AIDASite methodsFor: 'private-serving' stamp: ' 21/4/07 22:07'!
helpResolve: aResolution 
	(self canAnswer and: [aResolution siteMatch: self]) ifFalse: [^nil].
	^self answerTo: aResolution request! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
helpURL

	^self settings  at: #helpURL ifAbsent: [^'']! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
helpURL: aString
	self settings at: #helpURL put: aString.! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
homeDirectory

	^self settings  at: #homeDirectory ifAbsent: [^'']! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
homeDirectory: aString
	" a directory, which is the root for static html dokuments. Used for anAIDASite, when it searches 
	for usual html files. "
	self settings  at: #homeDirectory put: aString.! !

!AIDASite methodsFor: 'statistics-counters' stamp: ' 21/4/07 22:07'!
hourlyStatsProcess

	"hourly report to transcript"
	"WebServer default hourlyStatsProcess"
	| hits pages new returning afterHits afterPages afterNew afterReturning |
	[
		hits := self totalCounter total.
		pages := self pagesCounter total.
		new :=	self newVisitorsCounter total.
		returning := self returningVisitorsCounter total.
		(Delay forSeconds: self secondsToHour) wait.
		[self isServing] whileTrue: 
			[afterHits := self totalCounter total.
			afterPages := self pagesCounter total.
			afterNew := self newVisitorsCounter total.
			afterReturning := self returningVisitorsCounter total.
			Transcript cr; show: '***'; cr; show:
				'***  ',  self name, ' ', 
				Time now hours printString, ':', Time now minutes printString, ' ',
				'    last hour hits: ', (afterHits - hits) printDotString, 
				'    pages: ', (afterPages - pages) printDotString, 
				'    visitors new: ', (afterNew - new) printDotString, 
				'    returning: ', (afterReturning - returning) printDotString;
				cr; show: '***'.
			self registerTopHourHits: (afterHits - hits) pages: (afterPages - pages).
			hits := afterHits. pages := afterPages.
			new := afterNew. returning := afterReturning.
			(Delay forSeconds: self secondsToHour) wait].
	] fork.! !

!AIDASite methodsFor: 'private-serving' stamp: 'JM 4/26/2007 14:30'!
httpResponseOnException: anException
	| response |
	response := HTTPResponse internalServerError.
	response entity: '
<HTML>
<HEAD><TITLE>', response codeText, '</TITLE></HEAD>
  <BODY>
   <H2>', response code printString, ' ', response codeText, '</H2>
   <P>The server experienced an error while processing this request. <BR>
   If this problem persists, please contact the webmaster.</P>
  <P>Error description:</P>
  <P>', anException description, '</P>
  </BODY>
</HTML>'.
	^response! !

!AIDASite methodsFor: 'private-counters' stamp: ' 21/4/07 22:07'!
incNotFoundCounter

	"call this after each request is processed"


	self critical:
		[self notFoundCounter incCounter].! !

!AIDASite methodsFor: 'private-counters' stamp: ' 21/4/07 22:07'!
incRequestCounterFor: anObject

	"call this after each request is processed"


	self critical:
		[self totalCounter incCounter.
		(anObject contentType = 'text/html') 
			ifTrue: [self pagesCounter incCounter] ].! !

!AIDASite methodsFor: 'user services' stamp: ' 21/4/07 22:07'!
index
	^self userServices 
		at: #WebIndex
		ifAbsent: 
			[self addUserService: WebIndex new named: #WebIndex.
			self urlResolver defaultURL: '/search.html' forObject: self index.
			^self index].! !

!AIDASite methodsFor: 'private-counters' stamp: ' 21/4/07 22:07'!
initCounters
	"counters of web requests: total and by year/day of year"
	self critical: 
		[counters := IdentityDictionary new.
		self pagesCounter.
		self totalCounter.
		self newVisitorsCounter.
		self returningVisitorsCounter.
		self notFoundCounter].

"WebServer default initCounters"! !

!AIDASite methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initFavicon
	"/favicon.ico, a small icon shown in browser near url address"
	"(AIDASite named: 'biart') initFavicon."
	| icon |
	icon := WebMethodImage 
		fromMethod: #favicon on: self style contentType: 'image/x-icon' site: self.
	self urlResolver defaultURL: '/favicon.ico' forObject: icon.! !

!AIDASite methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initTimestamps
	timestamps := IdentityDictionary new.
	self setLastTimeAliveTimestamp.
	self setCreatedTimestamp.

"WebServer default initTimestamps"! !

!AIDASite methodsFor: 'initialize-release' stamp: 'mivsek 1/10/2008 18:53'!
initUrlsForSystemServices
	"register urls for access to system services if not already done"
	"AIDASite default initUrlsForSystemServices"
	self admin.
	self securityManager.
	self webMsgs.
	self statistics
! !

!AIDASite methodsFor: 'initialize-release' stamp: ' 21/4/07 22:07'!
initialize
	super initialize.
	WebTransactionMonitor addServer: self.
	self initializeDefaultSettings.
	self initUrlsForSystemServices.
	self initFavicon.
	WebTransactionMonitor commit.! !

!AIDASite methodsFor: 'initialize-release' stamp: 'np 4/2/2008 23:44'!
initializeDefaultSettings

	self host: 'newsite' ip: '127.0.0.1'  port: 8888.  "this is usable for tests only"
	self homeDirectory: (SpEnvironment onWindows ifTrue: ['.\'] ifFalse: ['./']).
	self helpDirectory: '.', (SpEnvironment onWindows ifTrue: ['\'] ifFalse: ['/']), 'help'.
	self helpURL: '/help'.
	self styleClass: 'DefaultWebStyle'.
	self diagnostics: false.
	self loggingPolicy: #none.
	self countingPolicy: #all.
	self smtpServer: self ip.
	self urgentNotificationEMail: 'info@eranova.si'.! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
isWebDAVEnabled
	"Is Web Distributed Authoring and Versioning (WebDAV, www.webdav.org) ienabled on this site"
		^(self settings includesKey: #WebDAVEnabled)
			 and: [self settings includesKey: #WebDAVEnabled]! !

!AIDASite methodsFor: 'statistics' stamp: ' 21/4/07 22:07'!
lastCommitTimestamp
	"time and date of last commit to the database. If noone yet, return nil"
	^Timestamp fromSeconds: (self timestamps at: #LastCommit ifAbsent: [^nil])! !

!AIDASite methodsFor: 'statistics' stamp: ' 21/4/07 22:07'!
lastRequestTimestamp
	"time and date of last web request. If noone yet, return nil"
	^Timestamp fromSeconds: (self timestamps at: #LastRequest ifAbsent: [^nil])! !

!AIDASite methodsFor: 'statistics' stamp: 'JM 4/25/2007 21:09'!
lastTimeAliveTimestamp
	"lastTimeAliveThread writes to timestamps every minute and commit. 
	Used in case of crash to determine, when a server was last time alive 
	and worked properly"
	^SpTimestamp fromSeconds: 
		(self timestamps at: #LastTimeAlive ifAbsentPut: [SpTimestamp now asSeconds])! !

!AIDASite methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
log: aString
	self diagnostics ifTrue: [Transcript show: aString].! !

!AIDASite methodsFor: 'private-serving' stamp: ' 21/4/07 22:07'!
logStringFor: aRequest
	^self name, ' ',
 		Date today dayOfMonth printString, '.',
		Date today monthIndex printString, ' ',
		Time now hours printString, ':', 
		Time now minutes printString,
		' from ', (aRequest peer notNil ifTrue: [aRequest peer] ifFalse: ['']), ' url: ',
		aRequest urlString! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
logging

	"if true, all requests are logged into a database"
	^self settings  at: #logging ifAbsentPut: [false].! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
loggingPolicy
	^self settings at: #loggingPolicy ifAbsentPut: [#none].! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
loggingPolicy: aSymbol
	"from who to log requests with WebCounters: #all, #excludeAdmins, #onlyGuests, #none"
	self settings  at: #loggingPolicy put: aSymbol! !

!AIDASite methodsFor: 'system services' stamp: ' 21/4/07 22:07'!
mimeMap
	^self systemServices 
		at: #MimeMap
		ifAbsent: 
			[self addSystemService: MIMEMap new named: #MimeMap.
			^self mimeMap].! !

!AIDASite methodsFor: 'statistics-counters' stamp: ' 21/4/07 22:07'!
minuteStatsProcess

	"every minute report to transcript"
	"WebServer default minuteStatsProcess"
	| hits pages new returning afterHits afterPages afterNew afterReturning |
	[
		hits := self totalCounter total.
		pages := self pagesCounter total.
		new :=	self newVisitorsCounter total.
		returning := self returningVisitorsCounter total.
		(Delay forSeconds: self secondsToMinute) wait.
		[self isServing] whileTrue: 
			[afterHits := self totalCounter total.
			afterPages := self pagesCounter total.
			afterNew := self newVisitorsCounter total.
			afterReturning := self returningVisitorsCounter total.
			Transcript cr; show: 
				self name, ' ', 
				Time now hours printString, ':', Time now minutes printString, ' ',
				'    last minute hits: ', (afterHits - hits) printDotString, 
				'    pages: ', (afterPages - pages) printDotString, 
				'    visitors new: ', (afterNew - new) printDotString, 
				'    returning: ', (afterReturning - returning) printDotString.
			self registerTopMinuteHits: (afterHits - hits) pages: (afterPages - pages).
			hits := afterHits. pages := afterPages.
			new := afterNew. returning := afterReturning.
			(Delay forSeconds: self secondsToMinute) wait].
	] fork.! !

!AIDASite methodsFor: 'private-counters' stamp: ' 21/4/07 22:07'!
newVisitorsCounter
	"counter of new sessions"
	(self counters includesKey: #NewVisitors) ifFalse: 
 		["self error: 'newVisitorsCounter ??'."
		self counters at: #NewVisitors put: WebCounter new].
	^self counters at: #NewVisitors

"WebServer default newVisitorsCounter"! !

!AIDASite methodsFor: 'private-counters' stamp: ' 21/4/07 22:07'!
notFoundCounter
	"counter of not found errors"
	(self counters includesKey: #NotFound) ifFalse: 
 		[self counters at: #NotFound put: WebCounter new].
	^self counters at: #NotFound

"WebServer default notFoundCounter"! !

!AIDASite methodsFor: 'private-serving' stamp: 'mivsek 3/7/2008 19:42'!
objectTo: aRequest  forSession: aSession
	| object  | 
	object := self urlResolver  ooRefFromURL: 
		(AIDASite convertFromWeb: aRequest uriString on: aSession). "for international Urls"
	aSession shouldCountRequests ifFalse: [^object].    "to skip admin requests etc."
	object notNil 
		ifTrue: 
			[self urlResolver incCounterFor: object.
			self incRequestCounterFor: object]
		ifFalse: 
			[self incNotFoundCounter.
			self log: ' not found: ', aRequest uriString, ' '.
			^nil ].
	self statistics collectStatsFrom: aRequest.
	^object! !

!AIDASite methodsFor: 'private-counters' stamp: ' 21/4/07 22:07'!
pagesCounter
	"counter of only text/html pages"
	(self counters includesKey: #Pages) ifFalse: 
 		["self error: 'pagesCounter ??'."
		self counters 	at: #Pages put: WebCounter new].
	^self counters at: #Pages

"WebServer default pagesCounter"! !

!AIDASite methodsFor: 'printing' stamp: ' 21/4/07 22:07'!
printString

	^'anAIDASite named: ', self name! !

!AIDASite methodsFor: 'private-serving' stamp: ' 21/8/07 09:38'!
properString: aString 
	"if two byte string, convert it to one byte, cut twobyte characters, make them $? "
	| stream |
	aString class == ByteString ifTrue: [^aString].
	stream := WriteStream on: String new.
	aString 
		do: [:char | stream nextPut: (char asInteger < 256 ifTrue: [char] ifFalse: [$?])].
	^stream contents! !

!AIDASite methodsFor: 'private-serving' stamp: 'mu 4/9/2008 21:13'!
redirectOn: aSession
	| loc response |
	loc := aSession redirectLink composeURLOn: aSession.
	loc := AIDASite convert: loc toCodepage: aSession codePage.
	response := HTTPResponse redirectLink 
		location: loc.
	aSession cookie ifFalse: [response cookie: (self cookieHeaderFor: aSession)].
	aSession redirectLink: nil.
	^response.! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
redirectToHost
	"if set, all requests are redirected to that hostname!!"
	| host |
	host := self settings  at: #redirectToHost ifAbsent: [^nil].
	(host isNil or: [host isEmpty]) ifTrue: [^nil].
	^host! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
redirectToHost: aHostnameString
	"if set, all requests are redirected to that hostname!!"
	self settings  at: #redirectToHost put: aHostnameString! !

!AIDASite methodsFor: 'private-serving' stamp: ' 21/4/07 22:07'!
redirectToOtherHost: aRequest
	| response |
	response := Swazoo.HTTPResponse redirectLink 
		location: (aRequest urlString copyReplaceAll: self host with: self redirectToHost).
	^response.! !

!AIDASite methodsFor: 'statistics-counters' stamp: ' 21/4/07 22:07'!
registerTopHourHits: aHitNumber pages: aPageNumber
	(self topHour at: 2) < aHitNumber ifTrue: 
		[self topHour
			at: 1 put: Timestamp now;
			at: 2 put: aHitNumber;
			at: 3 put: aPageNumber].! !

!AIDASite methodsFor: 'statistics-counters' stamp: ' 21/4/07 22:07'!
registerTopMinuteHits: aHitNumber pages: aPageNumber
	(self topMinute at: 2) < aHitNumber ifTrue: 
		[self topMinute
			at: 1 put: Timestamp now;
			at: 2 put: aHitNumber;
			at: 3 put: aPageNumber].! !

!AIDASite methodsFor: 'serving' stamp: 'mivsek 1/10/2008 18:46'!
reportException: anException
	 | report |
	Transcript cr; show: '*** error in web app code: ', anException errorString.
"	report := DebuggerService shortStackFor: anException initialContext ofSize: 10.
	(report  tokensBasedOn: Character cr) do: [:each |
		Transcript cr; show: '     ', each].
"! !

!AIDASite methodsFor: 'serving' stamp: 'mivsek 1/10/2008 18:47'!
reportSocketError: anException request: aRequest
	"some hard to find error while streaming, i'm trying to debug it with that report"
	Transcript cr; show: '**SpSocketError on ', self name, ': ', 
		anException parameter messageText; cr.
	Transcript show:  aRequest printString; cr.! !

!AIDASite methodsFor: 'statistics-counters' stamp: ' 21/4/07 22:07'!
requestsInYear: aYearNumber
	"return a number of requests on specified year"

	| count |
	count := 0.
	((self counters at: #Daily) at: aYearNumber ifAbsent: [^0]) do: [:dayCount |
		count := count + dayCount].
	^count! !

!AIDASite methodsFor: 'statistics-counters' stamp: ' 21/4/07 22:07'!
requestsOnDate: aDate
	"return a number of requests on specified date"

	^self totalCounter countsOnDate: aDate! !

!AIDASite methodsFor: 'start/stop' stamp: ' 21/4/07 22:07'!
restart
	self stop.
	self start.! !

!AIDASite methodsFor: 'private-backup/restore' stamp: 'np 1/15/2008 20:49'!
restoreSiteNamed: aString
	"boss in from file aidasite-<sitename>.obj"
	"AIDASite new restoreSiteNamed: 'test' "
	| filename site |
	filename := 'aidasite-', aString, '.obj'.
	site := (FileStream fileNamed: filename) fileInObjectAndCode.
	^site! !

!AIDASite methodsFor: 'private-counters' stamp: ' 21/4/07 22:07'!
returningVisitorsCounter
	"counter of returning sessions"
	(self counters includesKey: #ReturningVisitors) ifFalse: 
 		["self error: 'returningVisitorsCounter ??'."
		self counters
			at: #ReturningVisitors put: WebCounter new].
	^self counters at: #ReturningVisitors

"WebServer default returningVisitorsCounter"! !

!AIDASite methodsFor: 'statistics' stamp: 'JM 4/26/2007 19:57'!
runningHistory
	"return a collection of server runs as array of:
		started timestamp,
		last alive = stopped (more or less) timestamp,
		uptime in seconds,
		downtime from previous run in seconds,
		crashed or not.
	Used to determine history 
	of server runnings and to calculate server avaiability. Last entry in collection is current run
	with last alive no more than minute before current time"
	"AIDASite default runningHistory"
	| collection newArray  arry |
	collection := OrderedCollection new.
	self runningHistoryCollection do: [:array |
		newArray := Array new: 5.
		newArray 
			at: 1 put: (SpTimestamp fromSeconds: (array at: 1));
			at: 2 put: (SpTimestamp fromSeconds: (array at: 2));
			at: 3 put: (((array at: 2) - (array at: 1)) max: 0);
			at: 4 put: 0;  "we will calculate later"
			at: 5 put: ((array size < 3) ifTrue: [true] ifFalse: [array at:3]).
		collection add: newArray copy].
	newArray := Array new: 5.
	newArray
		at: 1 put: self startedTimestamp;
		at: 2 put: self lastTimeAliveTimestamp;
		at: 3 put: (self lastTimeAliveTimestamp asSeconds - 
			self startedTimestamp asSeconds);
		at: 4 put: 0;  "we will calculate later"
		at: 5 put: false. "current run not crashed"
	collection add: newArray copy.
	2 to: collection size do: [:inx |
		arry := collection at: inx.
		arry at: 4 put:   "this started - previous last alived"
			((arry at: 1) asSeconds - ((collection at: inx-1) at: 2) asSeconds)
		].
	^collection! !

!AIDASite methodsFor: 'private' stamp: ' 21/4/07 22:07'!
runningHistoryCollection
	^self timestamps at: #RunningHistory ifAbsentPut: [OrderedCollection new].! !

!AIDASite methodsFor: 'system services' stamp: 'JM 4/21/2007 22:19'!
scheduler
	^self systemServices 
		at: #Scheduler
		ifAbsent: 
			[self 
				addSystemService: (Scheduler newOn: self) 
				named: #Scheduler.
			^self scheduler].! !

!AIDASite methodsFor: 'statistics-counters' stamp: ' 21/4/07 22:07'!
secondsToHour

	| now | 
	now := Time now asSeconds.
	^(3600 - (now - (now // 3600 * 3600))) max: 0

"WebServer default secondsToHour"! !

!AIDASite methodsFor: 'statistics-counters' stamp: ' 21/4/07 22:07'!
secondsToMinute

	| now | 
	now := Time now asSeconds.
	^(60 - (now - (now // 60 * 60))) max: 0

"WebServer default secondsToMinute"! !

!AIDASite methodsFor: 'system services' stamp: 'mivsek 1/10/2008 18:55'!
securityManager
	^self systemServices 
		at: #SecurityManager
		ifAbsent: 
			[self 
				addSystemService: (WebSecurityManager newOn: self)
				named: #SecurityManager.
			self urlResolver defaultURL: '/security.html' forObject: self securityManager.
			self securityManager initialize.
			^self securityManager].! !

!AIDASite methodsFor: 'system services' stamp: ' 21/4/07 22:07'!
sessionManager
	^self systemServices 
		at: #SessionManager
		ifAbsent: 
			[self 
				addSystemService: (WebSessionManager newOn: self)
				named: #SessionManager.
			^self sessionManager].! !

!AIDASite methodsFor: 'private' stamp: 'JM 4/25/2007 20:47'!
setCreatedTimestamp
	"set the server creation timestamp to current date and time."
	self timestamps 
		at: #Created put: SpTimestamp now asSeconds.! !

!AIDASite methodsFor: 'private' stamp: ' 21/4/07 22:07'!
setLastCommitTimestamp
	"set the last commit timestamp to current date and time"
	self critical: [
		self timestamps 
			at: #LastCommit 
			put: Timestamp now asSeconds].! !

!AIDASite methodsFor: 'private' stamp: ' 21/4/07 22:07'!
setLastRequestTimestamp
	"set the last web request timestamp to current date and time"
	self critical: [
		self timestamps at: #LastRequest put: Timestamp now asSeconds].! !

!AIDASite methodsFor: 'private' stamp: 'JM 4/25/2007 20:48'!
setLastTimeAliveTimestamp
	"A lastTimeAliveThread sets this timestamp every minute"
	self critical: [
		self timestamps at: #LastTimeAlive put: SpTimestamp now asSeconds].! !

!AIDASite methodsFor: 'private' stamp: 'JM 4/25/2007 21:09'!
setStartedTimestampCrashed: aBoolean
	"set the last server startup timestamp to current date and time.  
	Before that records old created and last alive timestamp to a running history.
	Argument shows if server was running, therefore crashed, or was stoped normaly"
	self addToRunningHistoryCrashed: aBoolean.
	self timestamps 
		at: #Started put: SpTimestamp now asSeconds.! !

!AIDASite methodsFor: 'private' stamp: ' 21/4/07 22:07'!
settings
	settings isNil ifTrue: 
		[settings := Dictionary new. 
		self initializeDefaultSettings].
	^settings

"WebServer default settings"! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
settingsReport
	"a brief report of current settings to transcript, used at site startup"
	Transcript cr; show: 'server diagnostics: '.
	self diagnostics
		ifTrue: [Transcript show: 'ON']
		ifFalse: [Transcript show: 'OFF'].
	Transcript cr.! !

!AIDASite methodsFor: 'private-serving' stamp: ' 21/4/07 22:07'!
shouldRedirect
	^self redirectToHost notNil! !

!AIDASite methodsFor: 'settings' stamp: 'mivsek 1/10/2008 18:55'!
smtpServer
	"an address of email server for outgoing mails (notifications, mailing lists etc.)"
	^self settings  at: #smtpServer ifAbsentPut: [self host].
! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
smtpServer: anAddressString
	"an address of email server for outgoing mails (notifications, mailing lists etc.)"
	self settings  at: #smtpServer put: anAddressString.! !

!AIDASite methodsFor: 'start/stop' stamp: ' 21/4/07 22:07'!
start
	self setStartedTimestampCrashed: self isServing.
	self setLastTimeAliveTimestamp.
	super start.
	self startServices.! !

!AIDASite methodsFor: 'private' stamp: ' 21/4/07 22:07'!
startServices
	self systemServices values do: [:service | (service class canUnderstand: #start) ifTrue: [service start] ].
	self userServices values do: [:service | (service class canUnderstand: #start) ifTrue: [service start] ].! !

!AIDASite methodsFor: 'statistics' stamp: 'JM 4/25/2007 21:08'!
startedTimestamp
	"time and date of last server startup"
	^SpTimestamp fromSeconds:
		(self timestamps at: #Started ifAbsentPut: [SpTimestamp now asSeconds])! !

!AIDASite methodsFor: 'system services' stamp: ' 21/4/07 22:07'!
statistics
	^self systemServices 
		at: #Statistics
		ifAbsent: 
			[self 
				addSystemService: (WebStatistics newOn: self)
				named: #Statistics.
			self urlResolver defaultURL: '/stat.html' forObject: self statistics.
			^self statistics].! !

!AIDASite methodsFor: 'start/stop' stamp: ' 21/4/07 22:07'!
stop
	super stop.
	self stopServices.
	self setLastTimeAliveTimestamp.! !

!AIDASite methodsFor: 'private' stamp: ' 21/4/07 22:07'!
stopServices
	self systemServices values do: [:service | (service class canUnderstand: #stop) ifTrue: [service stop] ].
	self userServices values do: [:service | (service class canUnderstand: #stop) ifTrue: [service stop] ].! !

!AIDASite methodsFor: 'private' stamp: ' 21/4/07 22:07'!
stopWatchdog
	self watchdog notNil ifTrue: 
		[self watchdog terminate.
		self watchdog: nil].! !

!AIDASite methodsFor: 'accessing' stamp: 'JM 4/25/2007 21:07'!
style
	style isNil ifTrue: 
		[self style: ((Smalltalk at: self styleClass asSymbol) value newOnSite: self)].
	^style! !

!AIDASite methodsFor: 'private' stamp: ' 21/4/07 22:07'!
style: aWebStyle
	style := aWebStyle.! !

!AIDASite methodsFor: 'settings' stamp: 'np 4/2/2008 23:36'!
styleClass
	(self settings includesKey: #styleClass) ifFalse: 
		[self styleClass: 'DefaultWebStyle' ].
	^(self settings  at: #styleClass) asString! !

!AIDASite methodsFor: 'settings' stamp: 'mivsek 1/10/2008 18:56'!
styleClass: aString
	"name of a class to provide a style support for this site. Default is WebStyle"
	self settings  at: #styleClass put: aString asString.
	self style: nil! !

!AIDASite methodsFor: 'settings' stamp: 'mivsek 1/10/2008 18:56'!
supportedLanguages
	"Which languages are supported on this site. Content may be in all those languages. 
	Default language is always among supported."
	^(self settings at: #supportedLanguages ifAbsent: [Set new])
		add: self defaultLanguage; yourself! !

!AIDASite methodsFor: 'settings' stamp: 'mivsek 1/10/2008 18:57'!
supportedLanguages: aSetOfLangCodes
	"Which languages are supported on this site. Content may be in all those languages. 
	Default language is always among supported."
	^self settings at: #supportedLanguages put: aSetOfLangCodes asSet! !

!AIDASite methodsFor: 'system services' stamp: ' 21/4/07 22:07'!
systemServiceNamed: aSymbol

	"return a system service object with a specified name"

	^self systemServices at: aSymbol asSymbol ifAbsent: [^nil]! !

!AIDASite methodsFor: 'private' stamp: ' 21/4/07 22:07'!
systemServices
	"return a dictionary of system services (e.g. URLResolver, ImageManager etc.)"
	systemServices isNil 
		ifTrue: [systemServices := Dictionary new].
	^systemServices! !

!AIDASite methodsFor: 'private' stamp: ' 21/4/07 22:07'!
timestamps
	timestamps isNil ifTrue: [self initTimestamps].
	^timestamps! !

!AIDASite methodsFor: 'private-counters' stamp: ' 21/4/07 22:07'!
topHour
	"a hour of most hits, an array with timestamp, hits and pages"
	(self counters includesKey: #TopHour) ifFalse: 
 		[self counters at: #TopHour put: (Array with: Timestamp now with: 0 with: 0)].
	^self counters at: #TopHour

"WebServer default topHour"! !

!AIDASite methodsFor: 'private-counters' stamp: ' 21/4/07 22:07'!
topMinute
	"a minute of most hits, an array with timestamp, hits and pages"
	(self counters includesKey: #TopMinute) ifFalse: 
 		[self counters at: #TopMinute put:  (Array with: Timestamp now with: 0 with: 0)].
	^self counters at: #TopMinute

"WebServer default topMinute"! !

!AIDASite methodsFor: 'private-counters' stamp: ' 21/4/07 22:07'!
totalCounter
	"counter of all web requests"
	(self counters includesKey: #Total) ifFalse: 
 		["self error: 'totalCounter ??'."
		self counters at: #Total put: WebCounter new].
	^self counters at: #Total

"WebServer default totalCounter"! !

!AIDASite methodsFor: 'statistics' stamp: ' 21/4/07 22:07'!
totalDowntime
	"return downtime between all server runs in seconds"
	"AIDASite default totalDowntime"
	| history downtime |
	history := self runningHistory.
	downtime := 0.
	history do: [:array | downtime := downtime + (array at: 4)].
	^downtime! !

!AIDASite methodsFor: 'statistics-counters' stamp: ' 21/4/07 22:07'!
totalRequests
	"return a number of requests from server creation"

	^self totalCounter total.! !

!AIDASite methodsFor: 'statistics' stamp: ' 21/4/07 22:07'!
totalUptime
	"return uptime of all server runs in seconds"
	"AIDASite default totalUptime"
	| history uptime |
	history := self runningHistory.
	uptime := 0.
	history do: [:array | uptime := uptime + (array at: 3)].
	^uptime! !

!AIDASite methodsFor: 'statistics' stamp: ' 21/4/07 22:07'!
uptime
	"return uptime of current server run in seconds (with 60s precision)"
	"AIDASite default uptime"
	^self lastTimeAliveTimestamp asSeconds - self startedTimestamp asSeconds! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
urgentNotificationEMail

	"an email address or addresses (separated by comma) of all recipients of urgent server notifications "

	^self settings  at: #urgentNotificationEMail ifAbsentPut: ['janko.mivsek@eranova.si'].! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
urgentNotificationEMail: anEMailAddressString
	"an email address or addresses (separated by comma) of all recipients of urgent server notifications "
	self settings  at: #urgentNotificationEMail put: anEMailAddressString.! !

!AIDASite methodsFor: 'system services' stamp: ' 21/4/07 22:07'!
urlResolver
	^self systemServices 
		at: #URLResolver
		ifAbsent: 
			[self 
				addSystemService: (URLResolver newOn: self) 
				named: #URLResolver.
			^self urlResolver].! !

!AIDASite methodsFor: 'settings' stamp: 'np 5/6/2008 17:18'!
urlsWithSessionId
	"is session id added to urls when browser cookie support is disabled?"
	^self settings at: #urlsWithSessionId ifAbsentPut: false! !

!AIDASite methodsFor: 'settings' stamp: 'np 5/6/2008 17:18'!
urlsWithSessionId: aBoolean
	"is session id added to urls when browser cookie support is disabled?"
	^self settings at: #urlsWithSessionId put: aBoolean! !

!AIDASite methodsFor: 'user services' stamp: ' 21/4/07 22:07'!
userServiceNamed: aSymbol

	"return a user service object with a specified name"

	^self userServices at: aSymbol asSymbol ifAbsent: [^nil]! !

!AIDASite methodsFor: 'private' stamp: ' 21/4/07 22:07'!
userServices
	"return a dictionary of user services (e.g. WebIndex, WebNews etc.)"
	userServices isNil ifTrue: [userServices := Dictionary new. ].
	^userServices! !

!AIDASite methodsFor: 'private' stamp: 'janko 8/2/2008 15:02'!
watchdogAction
	self setLastTimeAliveTimestamp! !

!AIDASite methodsFor: 'system services' stamp: ' 21/4/07 22:07'!
webMsgs
	^self systemServices 
		at: #WebMsgs
		ifAbsent: 
			[self addSystemService: (WebMsgs new)	named: #WebMsgs.
			^self webMsgs].! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
webServerEMail

	"an email address of this web server "

	^self settings  at: #webServerEMail ifAbsentPut: ['aida-web@eranova.si'].! !

!AIDASite methodsFor: 'settings' stamp: ' 21/4/07 22:07'!
webServerEMail: anEMailString
	"an email address of this web server "
	self settings  at: #webServerEMail put: anEMailString.! !

!SortedCollection class methodsFor: '*Aida' stamp: 'JM 4/26/2007 14:09'!
withAll: aCollection sortBlock: aBlock
	" Answer a new SortedCollection with
	the given elements and sortBlock. "

	| newCollection |
	newCollection := self new: aCollection size.
	newCollection sortBlock: aBlock.
	newCollection addAll: aCollection.
	^newCollection! !

!Number methodsFor: '*Aida' stamp: 'JM 4/22/2007 18:59'!
printDotString
	"format and return the number with dots between thousandths:  '3.120.123' "
	| mrds millions thousanths rest text number |
	number := self abs.
	mrds := number // 1000000000.
	millions := (number \\ 1000000000)  // 1000000.
	thousanths := (number \\ 1000000) // 1000.
	rest := (number \\ 1000).
	text := ''.
	mrds > 0 ifTrue: 
		[text := mrds printString, '.', (millions withZeros), '.', 
		(thousanths withZeros), '.', (rest withZeros).
		^text].
	millions > 0 	ifTrue: 
		[text := millions printString, '.', 
		(thousanths withZeros), '.', (rest withZeros).
		^text].
	thousanths > 0 
		ifTrue: 
			[text := thousanths printString, '.', 
			(rest withZeros)]
		ifFalse: [text := rest printString].
	^(self < 0 ifTrue: ['- '] ifFalse: ['']), text

"
-123 printDotString  '- 123'
"! !

!Number methodsFor: '*Aida' stamp: 'JM 4/22/2007 18:59'!
withZeros
	"return three digit number with leading zeros"
	^(self < 100 ifTrue: ['0'] ifFalse: ['']), 
	(self < 10 ifTrue: ['0'] ifFalse: ['']), 
	self printString.! !

!SpDate methodsFor: '*Aida' stamp: 'JM 4/22/2007 19:01'!
+ aNumber 
	^self addDays: aNumber! !

!SpDate methodsFor: '*Aida' stamp: 'JM 4/22/2007 19:01'!
- aNumber 
	"subtract number of days from a date."
	^self subtractDays: aNumber! !

!SpDate methodsFor: '*Aida' stamp: 'np 1/22/2008 12:20'!
monthAndDayString
	"with leading zeros, example: 05-16 for 16may"
	^(self monthIndex < 10 ifTrue: ['0'] ifFalse: ['']), self monthIndex printString, '-',
	 	(self underlyingDate dayOfMonth < 10 ifTrue: ['0'] ifFalse: ['']), self underlyingDate dayOfMonth printString

"SpDate today monthAndDayString"! !

!SpDate methodsFor: '*Aida' stamp: 'JM 4/22/2007 19:02'!
printISOString
	^self year printString, 
		(self monthIndex < 10 ifTrue: ['0'] ifFalse: ['']), self monthIndex printString,
		(self dayOfMonth < 10 ifTrue: ['0'] ifFalse: ['']), self dayOfMonth printString

"Date today printISOString   "! !

!SpDate methodsFor: '*Aida' stamp: 'JM 4/26/2007 19:56'!
shortPrintSloString
	^self underlyingDate dayOfMonth printString, '.', 
		self underlyingDate monthIndex printString, '.', self year printString.

"SpDate today shortPrintSloString"! !

!SpDate methodsFor: '*Aida' stamp: 'JM 4/22/2007 19:02'!
shorterPrintSloString
	"year in two digits only"
	| yeart |
	yeart := (self year \\ 100) printString. yeart size = 1 ifTrue: [yeart := '0', yeart].
	^self dayOfMonth printString, '.', self monthIndex printString, '.', yeart.

"Date today shorterPrintSloString"! !

!SpTimestamp methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:32'!
printSloString
	^self asDate shortPrintSloString, ' ', self asTime printSloString

"SpTimestamp now printSloString"! !

!SpTimestamp methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:33'!
sendOver: aStream 
	"from Wiki rendering"
	self printOn: aStream
"
	TimestampPrintPolicy 
		print: (TimeZone default convertToGMT: self asSeconds
				do: [:date :seconds | Timestamp fromDate: date andTime: (Time fromSeconds: seconds)])
		on: aStream
		using: 'ddd, dd mmm yyyy hh:mm:ss GMT'
"! !

!PositionableStream methodsFor: '*Aida' stamp: 'JM 4/26/2007 14:28'!
upToSeparator
	"Answer a subcollection from position to the occurrence (if any, exclusive) of a separator.
	The stream is left positioned after the separator.
	If no separator is found answer everything."

	| newStream element |
	newStream := collection class new writeStream.
	[self atEnd]
		whileFalse:
			[element := self next.
			element isSeparator
				ifTrue: [^newStream contents].
			newStream nextPut: element.].
	^newStream contents
	
" 'abc def' readStream upToSeparator"
! !

!HTTPException class methodsFor: '*Aida' stamp: 'mivsek 8/22/2007 11:24'!
raiseResponse: aHTTPResponse
	"Raise an exception to immediatelly return that response."
	"put that on Swazoo next version!!!!!!"
	^self new 
		response: aHTTPResponse;
		signal.! !

!Collection methodsFor: '*Aida-Daily collections' stamp: 'JM 4/22/2007 19:04'!
allDailyCollectionDated: aDate
	"all on that date in any of daily collections"
	^self inject: OrderedCollection new into: [:col :each | col addAll: (each allDated: aDate). col]

"DailyCollection allInstances allDailyCollectionDated: 
		DailyCollection allInstances allDailyCollectionDates asOrderedCollection last."! !

!Collection methodsFor: '*Aida-Daily collections' stamp: 'JM 4/22/2007 19:05'!
allDailyCollectionDates
	"all dates which occur of any of daily collections"
	^self allDailyCollectionDays collect: [:each | Date fromDays: each].

"DailyCollection allInstances allDailyCollectionDates"! !

!Collection methodsFor: '*Aida-Daily collections' stamp: 'JM 4/22/2007 19:04'!
allDailyCollectionDays
	"all day numbers which occur of any of daily collections"
	^self inject: Set new into: [:set :each | set addAll: each allDays. set].

"DailyCollection allInstances allDailyCollectionDays"! !

!Collection methodsFor: '*Aida-Daily collections' stamp: 'JM 4/22/2007 19:05'!
allDailyCollectionEntries
	"sorted by date"
	| days |
	days := self allDailyCollectionDays.
	days := days asSortedCollection. 
	^days inject: OrderedCollection new into: [:col :day | 
		self do: [:daily | (daily days includesKey: day) ifTrue: [col addAll: (daily days at: day)] ].
		col].

"DailyCollection allInstances allDailyCollectionEntries"! !

!Collection methodsFor: '*Aida-Daily collections' stamp: 'JM 4/22/2007 19:05'!
allDailyCollectionFromDate: aStartDate to: anEndDate
	"sorted by date"
	| startDay endDay daysInRange |
	startDay := aStartDate asDays. endDay := anEndDate asDays.
	daysInRange := self allDailyCollectionDays select: [:each | each >= startDay and: [each <= endDay] ].
	daysInRange := daysInRange asSortedCollection. 
	^daysInRange inject: OrderedCollection new into: [:col :day | 
		self do: [:daily | (daily days includesKey: day) ifTrue: [col addAll: (daily days at: day)] ].
		col].

"DailyCollection allInstances allDailyCollectionFromDate: (Date readSloFrom: '1.1.2004' readStream) 
  	to: (Date readSloFrom: '31.12.2006' readStream)"! !

!Collection methodsFor: '*Aida-Daily collections' stamp: 'JM 4/26/2007 21:29'!
allDailyCollectionYear: aNumber
	^self 
		allDailyCollectionFromDate: (SpDate newDay: 1 month: 1 year: aNumber)
		to: (SpDate newDay: 31 month: 12 year: aNumber)

"DailyCollection allInstances allDailyCollectionYear: 2004 "! !

!Collection methodsFor: '*Aida-Daily collections' stamp: 'JM 4/22/2007 19:05'!
allDailyCollectionYears
	"all years on which occur of anything of daily collections. Sorted!!"
	^SortedCollection 
		withAll: (self allDailyCollectionDates inject: Set new into: [:set :each | set add: each year. set])
		sortBlock: [:a :b | a < b].

"DailyCollection allInstances allDailyCollectionYears"! !

!Collection methodsFor: '*Aida-Daily collections' stamp: 'JM 4/22/2007 19:06'!
allDailyCollectionYearsForKindOf: aClass
	"all years on which occur of anything of that class. Sorted!!"
	"optimize that!!"
	^(self inject: Set new into: [:set :each | set addAll: (each allYearsForKindOf: aClass). set])
		asSortedCollection

"DailyCollection allInstances allDailyCollectionYearsForKindOf: AIDA.Invoice"! !

!Collection methodsFor: '*Aida' stamp: ' 21/8/07 09:38'!
isRespondingStreamed
	^false! !

!Collection methodsFor: '*Aida' stamp: 'JM 4/22/2007 19:06'!
isWebElement
	^true "more or less"! !

!Collection methodsFor: '*Aida' stamp: 'JM 4/22/2007 19:06'!
isWebPage
	^false! !

!Collection methodsFor: '*Aida' stamp: 'janko 7/18/2008 20:15'!
printHTMLPageOn: aStream for: aRequest on: aSession 
	"only inner html, for ajax element updates"
	self do: 
		[ : element | 
		element notNil ifTrue: 
			[ element 
				printHTMLPageOn: aStream for: aRequest on: aSession ] ]! !

!HTTPRequest methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:20'!
ajaxElementId
	"Asynchronous JavaScript (AJAX) request, it has additional parameter ajaxGetElementId"
	| id |
	id := self queryAt: 'ajaxGetElementId' ifAbsent: [nil].
	id notNil ifTrue: [^id asSymbol].
	^self isPost 
		ifTrue: [(self postDataAt: 'ajaxGetElementId' ifAbsent: [^nil]) value asSymbol]
		ifFalse: [nil]! !

!HTTPRequest methodsFor: '*Aida' stamp: 'mivsek 1/10/2008 18:06'!
idFromCookie
	"return session id from our cookie. nil if not aida field present in cookie"
	| stream part |
	stream := self cookie readStream.
	[stream atEnd] whileFalse:
		[part := stream upTo: $; .
		('*aida9357*' match: part) ifTrue:
			[^part readStream upTo: $=; upToEnd] ].
	^nil! !

!HTTPRequest methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:18'!
isAjaxAutocompleteRequest
	"AJAX autocomplete input field request, see WebAutocompleteField"
	self isAjaxRequest ifFalse: [^false].
	^self includesQuery: 'ajaxAutocompleteField'! !

!HTTPRequest methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:18'!
isAjaxInPlaceEditingRequest
	"AJAX in place editor request, see WebInPlaceEditingText"
	self isAjaxRequest ifFalse: [^false].
	^self includesQuery: 'ajaxInPlaceEditing'! !

!HTTPRequest methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:18'!
isAjaxPostRequest
	"Asynchronous JavaScript (AJAX) request, with posting an input field in a query"
	self isAjaxRequest ifFalse: [^false].
	self isPost ifTrue: [^true].
	^false! !

!HTTPRequest methodsFor: '*Aida' stamp: 'mivsek 8/23/2007 00:06'!
isAjaxPostWithInput
	"Asynchronous JavaScript (AJAX) request, with posting an input field in a post data"
	self isAjaxRequest ifFalse: [^false].
	self isPost ifFalse: [^false].
	^self postDataKeys contains: [:each | 'field*' match: each].! !

!HTTPRequest methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:19'!
isAjaxRequest
	"Asynchronous JavaScript (AJAX) request, it has additional parameter ajaxRequest"
	^(self headerAt: 'X-Requested-With' ifAbsent: [^false]) value = 'XMLHttpRequest'! !

!HTTPRequest methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:19'!
isAjaxWikiFormatedRequest
	"AJAX in place editor request, see WebInPlaceEditingText"
	self isAjaxRequest ifFalse: [^false].
	^self includesQuery: 'wikiFormated'! !

!HTTPRequest methodsFor: '*Aida' stamp: 'janko 7/13/2008 11:28'!
isPing
	"/ping.html, for monitoring the site etc."
	^self requestLine requestURI identifier = '/ping.html'! !

!HTTPRequest methodsFor: '*Aida' stamp: 'JM 4/22/2007 15:20'!
view
	| view |
	view := self queryAt: 'view' ifAbsent: [nil].
	view notNil ifTrue: [^view asSymbol].
	^self isPost 
		ifTrue: [(self postDataAt: 'view' ifAbsent: [^nil]) value asSymbol]
		ifFalse: [nil]! !

!SwazooServer class methodsFor: '*Aida' stamp: 'mivsek 1/10/2008 18:10'!
aidaStart
	"SwazooServer aidaStart"
	"will start an Aida demo site on http://localhost:8888 "
	self singleton aidaStart! !

!SwazooServer class methodsFor: '*Aida' stamp: 'mivsek 1/10/2008 18:10'!
aidaStartOn: aPortNumber
	"will start an Aida site on all ip interfaces, ignoring host"
	"if there is no Aida site yet, it created one named 'aidademo''"
	"be sure that it is only site!!"
	"SwazooServer aidaStartOn: 8765"
	self singleton aidaStartOn: aPortNumber! !

!SwazooServer class methodsFor: '*Aida' stamp: 'mivsek 9/6/2007 14:03'!
aidaStop
	"SwazooServer aidaStop"
	self singleton aidaStop! !

!SwazooServer methodsFor: '*Aida' stamp: 'mu 2/3/2008 16:50'!
aidaStart
	| siteName |
	siteName := 'aidademo'.
	(self siteNamed: siteName) isNil ifTrue:
		[AIDASite newNamed: siteName.
		AIDASite default: siteName.
		(AIDASite named: siteName) host: '*' ip: '*'  port: 8888].
	self startSite: siteName! !

!SwazooServer methodsFor: '*Aida' stamp: 'mivsek 1/10/2008 22:06'!
aidaStartOn: aPortNumber
	| site |
	site := self siteAnyHostAllInterfacesOnPort: aPortNumber.
	site isNil ifTrue: [site := self prepareAidaDemoSiteOnPort: aPortNumber].
	site start.
	^site! !

!SwazooServer methodsFor: '*Aida' stamp: 'mivsek 9/6/2007 14:05'!
aidaStop
	self stopSite: 'aidademo'! !

!SwazooServer methodsFor: '*Aida' stamp: 'mivsek 1/10/2008 22:07'!
prepareAidaDemoSiteOnPort: aNumber
	"this site will run on all IP interfaces on that port"
	| name site |
	name := 'aidaOn', aNumber printString.
	site := AIDASite newNamed: name. "which is now also added to SwazoServer"
	site host: '*' ip: '*' port: aNumber.
	^site! !

!OrderedCollection methodsFor: '*Aida' stamp: 'np 8/22/2008 18:40'!
moveDown: anElement
	"move that element down (after current position) in a collection"
	| inx |
	(self notEmpty and: [self last = anElement]) ifTrue: [^self].
	inx := self indexOf: anElement.
	self add: anElement beforeIndex: inx+2.
	self removeAt: inx
	"#(1 2 3) asOrderedCollection moveDown: 1"! !

!OrderedCollection methodsFor: '*Aida' stamp: 'np 8/22/2008 18:40'!
moveUp: anElement
	"move that element up (before current position) in a collection"
	| inx |
	(self notEmpty and: [self first = anElement]) ifTrue: [^self].
	inx := self indexOf: anElement.
	self add: anElement beforeIndex: inx-1.
	self removeAt: inx+1
	"#(1 2 3) asOrderedCollection moveUp: 3"! !

!OrderedCollection methodsFor: '*Aida' stamp: 'mivsek 2/20/2008 10:21'!
removeAll
	"remove all elements quickly"
	self removeAllSuchThat: [:each | true]. "Squeak specific. #become: is VERY slow!!"
"	self become: OrderedCollection new " "VW specific"! !
GifImageStream initialize!
ImageStream initialize!
WebText initialize!
WebTransactionMonitor initialize!
